#ifndef BatchLab_MemoryTuple_h
#define BatchLab_MemoryTuple_h

// Inheritance :
#include <BatchLab/Core/BaseTuple.h>
#include <Slash/Store/IStorable.h>
#include <Slash/Meta/IObject.h>

namespace BatchLab {

class MemoryTuple 
:public BaseTuple 
,public virtual Slash::Store::IStorable
,public virtual Slash::Meta::IObject
{
public: //AIDA/ITuple
  //fill :
  virtual bool fill(int,double);
  virtual bool fill(int,float);
  virtual bool fill(int,int);
  virtual bool fill(int,short);
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual bool fill(int,long);
#else
  virtual bool fill(int,AIDA::int64);
  virtual bool fill(int,unsigned char);
#endif
  virtual bool fill(int,char);
  virtual bool fill(int,bool);
  virtual bool fill(int,const std::string&);
  virtual bool fill(int,const AIDA::ITupleEntry&);
  virtual bool fill(const std::vector<double>&);
  virtual bool fill(const std::vector<float>&);
  //get :
  virtual double getDouble(int) const;
  virtual float getFloat(int) const;
  virtual int getInt(int) const;
  virtual short getShort(int) const;
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual long getLong(int) const;
#else
  virtual AIDA::int64 getLong(int) const;
#endif
  virtual char getChar(int) const;
  virtual unsigned char getByte(int) const; //AIDA-v3r3p0
  virtual bool getBoolean(int) const;
  virtual std::string getString(int) const;
  virtual const AIDA::ITupleEntry* getObject(int) const;
  //else :
  virtual int rows() const;
  virtual bool addRow();
  virtual void* cast(const std::string&) const;
  virtual double columnMin(int column) const;
  virtual double columnMax(int column) const;
  virtual double columnMean(int column) const;
  virtual double columnRms(int column) const;
  virtual const AIDA::ITuple* getTuple(int) const; //deprecated in AIDA-v3r3p0
  virtual AIDA::ITuple* getTuple(int); //deprecated in AIDA-v3r3p0
  virtual const AIDA::ITuple* findTuple(int) const; //AIDA-v3r3p0
  virtual AIDA::ITuple* findTuple(int); //AIDA-v3r3p0
public: //Slash::Store::IObject
  virtual Slash::Store::IObject* copy() const;
  virtual std::string storeClassName() const;
public: //IVisited
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public:
  MemoryTuple(Slash::Core::IWriter&,const std::string& = "");
  MemoryTuple(Slash::Core::IWriter&,
              const std::string&,
              const std::string&,
              const std::string&);
  MemoryTuple(Slash::Core::IWriter&,
              const std::string&,
              const std::string&,
              const std::list<Lib::Value>&);
  MemoryTuple(const MemoryTuple&);
  virtual ~MemoryTuple();
private:
  bool getBooking(std::string&) const;
  SLASH_STORE_HEADER
protected:
  bool initialize(const std::list<Lib::Value>&);
};

}

#endif

