// this :
#include <BatchLab/Core/ArrayFactory.h>

#include <Lib/smanip.h>
#include <Lib/Debug.h>

#include <BatchLab/Core/Array.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::ArrayFactory::ArrayFactory(
 Slash::Core::IWriter& aPrinter
,Slash::Store::ITree& aTree
)
:BaseFactory(aPrinter,aTree)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::ArrayFactory");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::ArrayFactory::~ArrayFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("BatchLab::ArrayFactory");
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::create(
 const std::string& aPath
,const std::string& aTitle
,const std::vector<unsigned int>& aOrders
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,aTitle,
                                        aOrders);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
/*
bool BatchLab::ArrayFactory::destroy(
 AIDA::Slash::Data::IArray* 
)
{ 
  return 0;
}
*/
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::add(
 const std::string& aPath
,const Slash::Data::IArray& aV1
,const Slash::Data::IArray& aV2
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aV1.orders()!=aV2.orders()) return 0;
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aV1.orders());
  object->copy(aV1);
  object->add(aV2);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::subtract(
 const std::string& aPath
,const Slash::Data::IArray& aV1
,const Slash::Data::IArray& aV2
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aV1.orders()!=aV2.orders()) return 0;
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aV1.orders());
  object->copy(aV1);
  object->subtract(aV2);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::multiply(
 const std::string& aPath
,const Slash::Data::IArray& aV1
,const Slash::Data::IArray& aV2
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aV1.orders()!=aV2.orders()) return 0;
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aV1.orders());
  object->copy(aV1);
  object->multiply(aV2);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::divide(
 const std::string& aPath
,const Slash::Data::IArray& aV1
,const Slash::Data::IArray& aV2
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aV1.orders()!=aV2.orders()) return 0;
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aV1.orders());
  object->copy(aV1);
  object->divide(aV2);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::add(
 const std::string& aPath
,const Slash::Data::IArray& aArray
,double aValue
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aArray.orders());
  object->copy(aArray);
  object->add(aValue);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IArray* BatchLab::ArrayFactory::multiply(
 const std::string& aPath
,const Slash::Data::IArray& aArray
,double aValue
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::string path,name,suffix;
  if(!Lib::smanip::pathNameSuffix(aPath,path,name,suffix)) return 0;
  BatchLab::Array* object = new BatchLab::Array(fPrinter,
                                        name,"",
                                        aArray.orders());
  object->copy(aArray);
  object->multiply(aValue);
  if(!manage(aOptions)) return object;
  if(manageObject(object,path)) return object;
  delete object;
  return 0;
}
