// this :
#include <BatchLab/Core/AsciiTree.h>

#include <AIDA/ITuple.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/FileWriter.h>
#include <Lib/Cast.h>

#include <Slash/Store/IObject.h>
#include <Slash/Store/IFolderIterator.h>
#include <Slash/Store/IFolderVisitor.h>
#include <BatchLab/Core/amanip.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::AsciiTree::AsciiTree(
 Slash::Core::ISession& aSession
,const std::string& aName
,const std::string& aStoreName
,bool aReadOnly
,bool aCreateNew
,const std::string& //aOptions
)
:MemoryTree(aSession,aName)
,fStoreName(aStoreName)
,fReadOnly(aReadOnly)
,fCreateNew(aCreateNew)
,fHippoStyle(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::AsciiTree::~AsciiTree(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::AsciiTree::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::AsciiTree)
  else return MemoryTree::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::AsciiTree::storeName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStoreName;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::AsciiTree::storeType(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::AsciiTree";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::AsciiTree::isReadOnly(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fReadOnly;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::AsciiTree::createNew(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCreateNew;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::AsciiTree::setHippoStyle(
  bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHippoStyle = aValue;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::AsciiTree::hippoStyle(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHippoStyle;
}

namespace BatchLab {
  class AsciiStoreVisitor : public virtual Slash::Store::IFolderVisitor {
  public:
    AsciiStoreVisitor(AsciiTree& aTree,
                      Lib::Out& aOut,
                      Slash::Core::IWriter& aPrinter)
    :fTree(aTree),fOut(aOut),fPrinter(aPrinter) {}
  public:
    virtual bool visit(const Slash::Store::IFolder& aFolder,
                       const Slash::Store::IObject& aObject){
      if(fTree.verboseLevel()) {
        Lib::Out out(fPrinter);
        out << "BatchLab::AsciiStoreVisitor::visit :" 
            << " path : " << Lib::sout(aFolder.path())
            << " name : " << Lib::sout(aObject.name())
            << Lib::endl;
      }

      if(const AIDA::ITuple* object = Lib_ConstCast(aObject,AIDA::ITuple)) {

        if(fTree.hippoStyle()) {
          if(!amanip::write(*(const_cast<AIDA::ITuple*>(object)),
                            fOut,true,true,
                            fPrinter)) return false;
        } else {
          if(!amanip::write(*(const_cast<AIDA::ITuple*>(object)),
                            fOut,false,false,
                            fPrinter)) return false;
        }

      } else {

        Lib::Out out(fPrinter);
        out << "BatchLab::AsciiStoreVisitor::visit :" 
            << " path : " << Lib::sout(aFolder.path())
            << " name : " << Lib::sout(aObject.name())
            << " object not written (only tuples are supported)."
            << Lib::endl;

      }
        
      return true;
    }
    virtual bool beginVisit(const Slash::Store::IFolder&,const Slash::Store::IFolder&){ return true;}
    virtual bool endVisit(const Slash::Store::IFolder&,const Slash::Store::IFolder&){ return true;}
    virtual bool isRecursive() const { return true;}
  private:
    AsciiTree& fTree;
    Lib::Out& fOut;
    Slash::Core::IWriter& fPrinter;
};
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::AsciiTree::commit(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fReadOnly) return false;

  std::string fileName = name(); //!!!
  if(fCreateNew) {
    Lib::fmanip::remove(fileName);
  } else {
    //Can't update a file !
    return false;
  }

  // The name may contain env variables.
  std::string fullName = Lib::fmanip::name(fileName);
  Slash::Core::IWriter* writer = new Lib::FileWriter(fullName,printer());
  if(!writer->enabled()) {
    delete writer;
    return false;
  }
  Lib::Out out(*writer);
  
  AsciiStoreVisitor visitor(*this,out,printer());
  rootFolder().visit(visitor);

  delete writer;

  return true;
}
