
// this :
#include <BatchLab/Core/BaseFolder.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>
#include <Lib/Debug.h>

#include <Slash/Store/IObject.h>
#include <Slash/Store/ITree.h>
#include <Slash/Store/IObjectIterator.h>
#include <Slash/Store/IFolderVisitor.h>
#include <Slash/Store/IFolderIterator.h>

#include <BatchLab/Core/amanip.h> //FIXME

#include <AIDA/IBaseHistogram.h>
#include <AIDA/IManagedObject.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseFolder::BaseFolder(
 Slash::Core::IWriter& aPrinter
,Slash::Store::IFolder* aParent
,const std::string& aName
,Slash::Store::ITree& aTree
)
:fPrinter(aPrinter)
,fParent(aParent)
,fName(aName)
,fTree(aTree)
,fLinkingFolder(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseFolder");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseFolder::~BaseFolder(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("BatchLab::BaseFolder");
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IFolder* BatchLab::BaseFolder::parent(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParent;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseFolder::setParent(
 Slash::Store::IFolder* aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParent = aParent;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFolder::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseFolder::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fName = aName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::ITree& BatchLab::BaseFolder::tree(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTree;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseFolder::setLinkingFolder(
  Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLinkingFolder = aFolder;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IFolder* BatchLab::BaseFolder::linkingFolder(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLinkingFolder;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFolder::path(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path = "";
  const Slash::Store::IFolder* folder = static_cast<const Slash::Store::IFolder*>(this);
  while(folder) {
    std::string tmp = path;
    path = "/";
    path += folder->name();
    path += tmp;
    folder = folder->parent();
  }
  Lib::smanip::replace(path,"//","/");
  return path;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFolder::linkingPath(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(BatchLab::BaseFolder);
  // Take care of links :
  std::string path = "";
  Slash::Store::IFolder* folder = &self;
  while(folder) {
    std::string tmp = path;
    path = "/";
    path += folder->name();
    path += tmp;
    if(Slash::Store::IFolder* p = folder->linkingFolder()) {
      if(&(p->tree())!=&(folder->tree())) { //Mouting point :
        folder = p;
      } else { //Symlink :
        folder = folder->parent();
      }
    } else {
      folder = folder->parent();
    }
  }
  Lib::smanip::replace(path,"//","/");
  return path;
}
//////////////////////////////////////////////////////////////////////////////
unsigned int BatchLab::BaseFolder::numberOfObjects(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IObjectIterator* it = objects();
  if(!it) return 0;
  unsigned int number = 0;
  for(;it->object();it->next()) {
    number++;
  }
  return number;
}
//////////////////////////////////////////////////////////////////////////////
unsigned int BatchLab::BaseFolder::numberOfFolders(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IFolderIterator* it = folders();
  if(!it) return 0;
  unsigned int number = 0;
  for(;it->folder();it->next()) {
    number++;
  }
  delete it;
  return number;
}

std::string object_ls(const Slash::Store::IObject&);

//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseFolder::ls(
 bool aRecursive
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(fPrinter);

  out << " ************** Directory ===> "
      << fName
      << " <===" 
      << Lib::endl;

  if(numberOfFolders()) {
    out << " ===> List of subdirectories" << Lib::endl;
    Slash::Store::IFolderIterator* itf = folders();
    if(itf) {
      for(;itf->folder();itf->next()) {
        out << "  " << itf->folder()->name() << Lib::endl;
      }
      delete itf;
    }
  }

  out << " ===> List of objects" << Lib::endl;
  Slash::Store::IObjectIterator* it = objects();
  if(it) {
    for(;it->object();it->next()) {
      out << "  " << object_ls(*(it->object())) << Lib::endl;
    }
    delete it;
  }

  if(aRecursive) {
    Slash::Store::IFolderIterator* itf = folders();
    if(itf) {
      for(;itf->folder();itf->next()) {
        itf->folder()->ls(aRecursive);
      }
      delete itf;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::BaseFolder::findInTree(
 const std::string& aName
,Slash::Store::IFolder*& aFolder
) const
//////////////////////////////////////////////////////////////////////////////
// Look in folder and descent in sub folders.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IObject* object = findObject(aName);
  if(object) {
    aFolder = (Slash::Store::IFolder*)static_cast<const Slash::Store::IFolder*>(this);
    return object;
  }

  // children :
  Slash::Store::IFolderIterator* itf = folders();
  if(itf) {
    for(;itf->folder();itf->next()) {
      object = itf->folder()->findInTree(aName,aFolder);
      if(object) {
        delete itf;
        return object;
      }
    }
    delete itf;
  }
  aFolder = 0;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFolder::isValid(
 const Slash::Store::IObject* aObject
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IObjectIterator* it = objects();
  if(it) {
    for(;it->object();it->next()) {
      if(it->object()==aObject) {
        delete it;
        return true;
      }
    }
    delete it;
  }

  Slash::Store::IFolderIterator* itf = folders();
  if(itf) {
    for(;itf->folder();itf->next()) {
      if(itf->folder()->isValid(aObject)) {
        delete itf;
        return true;
      }
    }
    delete itf;
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IFolder* BatchLab::BaseFolder::findFolder(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IFolderIterator* itf = folders();
  if(itf) {
    for(;itf->folder();itf->next()) {
      Slash::Store::IFolder* folder = itf->folder();
      if(folder->name()==aName) {
        delete itf;
        return folder;
      }
    }
    delete itf;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::BaseFolder::findObject(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IObjectIterator* it = objects();
  if(it) {
    for(;it->object();it->next()) {
      Slash::Store::IObject* object = it->object();
      if(aName==object->name()) {
        delete it;
        return object;
      }
    }
    delete it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFolder::visit(
 Slash::Store::IFolderVisitor& aVisitor
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Store::IObjectIterator* it = objects();
  if(it) {
    for(;it->object();it->next()) {
      if(!aVisitor.visit(*this,*(it->object()))) {
        delete it;
        return false;
      }
    }
    delete it;
  }
  // Sub folders :
  bool status = true;
  Slash::Store::IFolderIterator* itf = folders();
  if(itf) {
    for(;itf->folder();itf->next()) {
      Slash::Store::IFolder* folder = itf->folder();
      if(!aVisitor.beginVisit(*this,*folder)) {
        delete itf;
        return false;
      }
      if(aVisitor.isRecursive()) status = folder->visit(aVisitor);
      if(!aVisitor.endVisit(*this,*folder)) {
        delete itf;
        return false;
      }
      if(!status) break;
    }
    delete itf;
  }
  return status;
}
namespace BatchLab {
  class HierarchyVisitor : public virtual Slash::Store::IFolderVisitor {
  public:
    HierarchyVisitor(const std::string& aFormat,std::string& aOut)
    :fFormat(aFormat),fOut(aOut) {
    }
  public:
    virtual bool visit(const Slash::Store::IFolder&,const Slash::Store::IObject& aObject){
      const AIDA::IManagedObject* aobject = 
        Lib_ConstCast(aObject,AIDA::IManagedObject);
      if(!aobject) return true; //FIXME : warning.
      fOut += "<treeItem><label>";
      std::string name = Lib::smanip::toxml(aObject.name());
      if(fFormat=="") {	
        fOut += name;
      } else {
        fOut += name;
        fOut += " class=";
        fOut += amanip::managed_object_type(*aobject); //FIXME
        fOut += " title=";
        std::string title = 
          Lib::smanip::toxml(amanip::managed_object_title(*aobject));
        fOut += "\""+title+"\"";
      }
      fOut += "</label></treeItem>";
      return true;
    }
    virtual bool beginVisit(const Slash::Store::IFolder&,const Slash::Store::IFolder& aFolder){
      std::string name = Lib::smanip::toxml(aFolder.name());
      fOut += "<treeItem><label>" + name + "</label>";
      return true;
    }
    virtual bool endVisit(const Slash::Store::IFolder&,const Slash::Store::IFolder&){
      fOut += "</treeItem>";
      return true;
    }
    virtual bool isRecursive() const { return true;}
  private:
    std::string fFormat;
    std::string& fOut;
  };
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFolder::hierarchy(
 const std::string& aFormat
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sout;
  HierarchyVisitor visitor(aFormat,sout);
  visit(visitor);
  return sout;
}
//////////////////////////////////////////////////////////////////////////////
std::string object_ls(
 const Slash::Store::IObject& aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 std::string s = aObject.name();
 if(const AIDA::IBaseHistogram* aobject = 
      Lib_ConstCast(aObject,AIDA::IBaseHistogram)) {
   s += " ";
   s += "(" + Lib::smanip::tostring(aobject->dimension())+")";
   s += " ";
   s += aobject->title();
 }
 return s;
}
