// this :
#include <BatchLab/Core/BaseObject.h>

#include <Slash/Core/IRelationManager.h>
#include <Slash/Core/IRelationVisitor.h>
#include <Slash/Core/IRelation.h>
#include <Lib/Debug.h>
#include <Lib/Cast.h>

#include <Slash/Store/IFolder.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::BaseObject(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:fPrinter(aPrinter)
,fName(aName)
,fFolder(0)
,fRelationManager(0)
,fVerboseLevel(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::BaseObject(
 const BaseObject& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,fPrinter(aFrom.fPrinter)
,fName(aFrom.fName)
,fFolder(0)
,fRelationManager(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseObject::~BaseObject(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fFolder) fFolder->remove(this);
  //FIXME : do we want to notifiy the plotter ?
  if(fRelationManager) fRelationManager->removeWith(this);
  Lib::Debug::decrement("BatchLab::BaseObject");
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseObject::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IManagedObject)
  else if_Lib_SCast(Slash::Store::IObject) 
  else if_Lib_SCast(BatchLab::BaseObject)
  else return 0;
}

namespace BatchLab {
    class BaseObjectRelationVisitor : public virtual Slash::Core::IRelationVisitor {
    public: //IRelationVisitor
      virtual bool visit(Slash::Core::IRelation& aRelation) {
        if(fObject!=aRelation.first()) return true;
        aRelation.schedule();
        return true;
      }
    public:
      BaseObjectRelationVisitor(BaseObject* aObject):fObject(aObject){}
      virtual ~BaseObjectRelationVisitor(){}
    private:
      BaseObject* fObject;
    };
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::emitUpdate(
)
//////////////////////////////////////////////////////////////////////////////
// To update plotters.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fRelationManager) {
    BaseObjectRelationVisitor visitor(this);
    fRelationManager->visit(visitor);
  }
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setFolder(
 Slash::Store::IFolder* aFolder
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFolder = aFolder;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setRelationManager(
 Slash::Core::IRelationManager* aRelationManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fRelationManager = aRelationManager;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseObject::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fName = aName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& BatchLab::BaseObject::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseObject::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::BaseObject::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
