// this :
#include <BatchLab/Core/BaseTree.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IRelationManager.h>
#include <Slash/Core/IRelationVisitor.h>
#include <Slash/Core/IRelation.h>
#include <Lib/Out.h>
#include <Lib/fmanip.h>
#include <Lib/smanip.h>
#include <Lib/Manager.h>
#include <Lib/Cast.h>
#include <Lib/Debug.h>

#include <Slash/Store/IObject.h>

namespace BatchLab {
  class BaseTreeRelationVisitor : public virtual Slash::Core::IRelationVisitor {
  public: //IRelationVisitor
    virtual bool visit(Slash::Core::IRelation& aRelation) {
      if(fTree!=aRelation.first()) return true;
      aRelation.schedule();
      return true;
    }
  public:
    BaseTreeRelationVisitor(AIDA::ITree* aTree):fTree(aTree){}
    virtual ~BaseTreeRelationVisitor(){}
  private:
    AIDA::ITree* fTree;
  };
}

//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseTree::BaseTree(
 Slash::Core::ISession& aSession
,const std::string& aName
)
:fSession(aSession) //For Validers
,fName(aName)
,fVerboseLevel(0)
,fRelationManager(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::BaseTree");
  //printf("debug : %lu %lu :%lu\n",this,(AIDA::ITree*)this,&fKey);

  fRelationManager = 
    Lib_findManager(fSession,"Lib::RelationManager",Slash::Core::IRelationManager);

  /* 
  //Set relation to UI :
  //Done in TreeFactory to avoid a dependency to IUI.  
  if(fRelationManager) {
    IUI* ui = Lib_findManager(fSession,"UI_Manager",IUI);
    if(ui) {
      fRelationManager->add(new AIDA_ITree_IUI_Relation(*this,*ui));
    }
  }
  // Can't emitUpdate here since the construction is not finished.
  */
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseTree::~BaseTree(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "BatchLab::BaseTree::~Tree :"
        << " called for " << this << " " << fName 
        << Lib::endl;
  }

  // Notify UI.
  //emitUpdate(); //virtual !
  if(fRelationManager) {
    BaseTreeRelationVisitor visitor(this);
    fRelationManager->visit(visitor);
  }

  // Remove relations concerning this tree :
  if(fRelationManager) {    
    fRelationManager->removeWith(static_cast<AIDA::ITree*>(this));
  }

  fSession.removeManager(this);  //To support the AIDA delete tree.
  Lib::Debug::decrement("BatchLab::BaseTree");
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseTree::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseTree::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::ITree)
  else if_Lib_SCast(Slash::Store::ITree) 
  else if_Lib_SCast(BatchLab::BaseTree) 
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ISession& BatchLab::BaseTree::session(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSession;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& BatchLab::BaseTree::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSession.printer();
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseTree::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::BaseTree::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseTree::emitUpdate(
)
//////////////////////////////////////////////////////////////////////////////
// To update UI.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{  
  if(!fRelationManager) return;
  BaseTreeRelationVisitor visitor(this);
  fRelationManager->visit(visitor);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseTree::manageObject(
 Slash::Store::IObject* aObject
,const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return addObject(aObject,aPath);
}


