// this :
#include <BatchLab/Core/Calculator.h>

#include <AIDA/ITuple.h>

#include <Lib/Out.h>
#include <Lib/Processor.h>
#include <Lib/Debug.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <BatchLab/Core/amanip.h>
#include <BatchLab/Core/BaseTuple.h>
 
#define NotFound (-1)

//////////////////////////////////////////////////////////////////////////////
BatchLab::Calculator::Calculator(
 Slash::Core::IWriter& aPrinter
,const std::string& aScript
)
:fPrinter(aPrinter)
,fTuple(0)
,fScript(aScript)
,fBaseTuple(0)
,fProcessor(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::Calculator");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Calculator::~Calculator(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fProcessor;
  Lib::Debug::decrement("BatchLab::Calculator");
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Calculator::initialize(
 AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTuple = 0;
  delete fProcessor;
  fProcessor = 0;
  fNames.clear();
  fIndices.clear();
  fBaseTuple = 0;
  fBaseCols.clear();
  fVals.clear();

  // Get cut variables :
  fProcessor = new Lib::Processor(fScript,fPrinter);
  if(!fProcessor->findVariables(fNames)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Calculator::initialize : "
        << " findVariables failed." 
        << Lib::endl;
    fNames.clear();
    delete fProcessor;
    fProcessor = 0;
    return false;    
  }
  if(!fProcessor->compile(fNames)) {
    Lib::Out out(fPrinter);
    out << "BatchLab::Calculator::initialize : "
        << " compilation failed." 
        << Lib::endl;
    fNames.clear();
    delete fProcessor;
    fProcessor = 0;
    return false;    
  }

  unsigned int number = fNames.size(); //Could be zero.
  fIndices.resize(number);
  for(unsigned int count=0;count<number;count++) {
    //printf("debug : variable \"%s\" %lu\n",fNames[count].c_str(),&aTuple);
    int index = aTuple.findColumn(fNames[count]);
    if(index==NotFound) {
      Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize : "
          << Lib::sout(fNames[count]) << " not a column of tuple with title " 
          << Lib::sout(aTuple.title()) 
          << Lib::endl;
      delete fProcessor;
      fProcessor = 0;
      fNames.clear();
      fIndices.clear();
      fBaseTuple = 0;
      fBaseCols.clear();
      fVals.clear();
      // Dump column names :
     {Lib::Out out(fPrinter);
      out << "BatchLab::Calculator::initialize :"
          << " columns are :"
          << Lib::endl;
      for(unsigned int count=0;count<number;count++) {
        out << Lib::sout(aTuple.columnName(count)) << Lib::endl;
      }}
      return false;
    } 
    fIndices[count] = index;
  }
  fTuple = &aTuple;

  fVals.resize(number);

  // To optimize the evaluateDouble :
  fBaseTuple = Lib_ConstCast(*fTuple,BatchLab::BaseTuple);
  if(fBaseTuple) {
    fBaseCols.resize(number);
    for(unsigned int count=0;count<number;count++) {
      fBaseCols[count] = fBaseTuple->findColumn1(fIndices[count]);
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Calculator::getVariables(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fProcessor) return false;

  unsigned int number = fNames.size(); //Could be zero.

  if(fBaseTuple) {  
    // Faster :

    int irow = fBaseTuple->rowIndex();

    for(unsigned int count=0;count<number;count++) {
      BaseColumn* column = fBaseCols[count];
      if(!column->value(irow,fVals[count])) {
        Lib::Out out(fPrinter);
        out << "BatchLab::Calculator::evaluateDouble : "
            << " can't get Variable value for column " 
            << Lib::sout(column->name()) << "."
            << Lib::endl;
        return false;
      }
    }

  } else {

    // Through AIDA::ITuple only but more slower :

    for(unsigned int count=0;count<number;count++) {
      int icol = fIndices[count];
      std::string type = fTuple->columnType(icol);
      if(type=="float") {
        fVals[count].set(fTuple->getFloat(icol));
      } else if(type=="double") {
        fVals[count].set(fTuple->getDouble(icol));
      } else if(type=="char") {
        fVals[count].set(fTuple->getChar(icol));
      } else if(type=="short") {
        fVals[count].set(fTuple->getShort(icol));
      } else if(type=="int") {
        fVals[count].set(fTuple->getInt(icol));
      } else if(type=="boolean") {
        fVals[count].set(fTuple->getBoolean(icol));
      } else {
        Lib::Out out(fPrinter);
        out << "BatchLab::Calculator::evaluateDouble : "
            << " make no sense on column type " << type << "."
            << Lib::endl;
      }
    }
  }

  return true;
}
