// this :
#include <BatchLab/Core/Cloud.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Cast.h>

#include <BatchLab/Core/Histogram.h> // For autoconvertion.

#include <math.h>

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

#define BATCHLAB_UNLIMITED (-1)
#define BINS 100
#define BATCHLAB_CHUNCK 512

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseCloud::BaseCloud(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,int aLimit
)
:BaseObject(aPrinter,aName)
,fLimit(aLimit)
,fSw(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseCloud::BaseCloud(
 const BaseCloud& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
,fTitle(aFrom.fTitle)
,fLimit(aFrom.fLimit)
,fSw(aFrom.fSw)
,fWs(aFrom.fWs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseCloud::~BaseCloud(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseCloud::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::BaseCloud)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseCloud::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return true;
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Annotation& BatchLab::BaseCloud::getAnnotation(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const BatchLab::Annotation& BatchLab::BaseCloud::getAnnotation(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseCloud::setAnnotation(
 const BatchLab::Annotation& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::BaseCloud::ws(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fWs;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud1D::Cloud1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseCloud(aPrinter,aName,-1)
,fLowerX(0)
,fUpperX(0)
,fSxw(0)
,fSx2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud1D::Cloud1D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aLimit
)
:BaseCloud(aPrinter,aName,aLimit)
,fLowerX(0)
,fUpperX(0)
,fSxw(0)
,fSx2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setTitle(aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud1D::Cloud1D(
 const Cloud1D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::ICloud(aFrom)
,AIDA::ICloud1D(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseCloud(aFrom)
,fXs(aFrom.fXs)
,fLowerX(aFrom.fLowerX)
,fUpperX(aFrom.fUpperX)
,fSxw(aFrom.fSxw)
,fSx2w(aFrom.fSx2w)
,fCnvXnumber(aFrom.fCnvXnumber)
,fCnvXmin(aFrom.fCnvXmin)
,fCnvXmax(aFrom.fCnvXmax)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFrom.fHistogram)
    fHistogram = new BatchLab::Histogram1D(*aFrom.fHistogram);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud1D::~Cloud1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Cloud1D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Cloud1D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud1D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Cloud1D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud1D::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BaseObject::setName(aName);
  if(fHistogram) fHistogram->setName(aName);
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Cloud1D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::ICloud1D)
  else if_Lib_SCast(AIDA::ICloud)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Cloud1D)
  else return BaseCloud::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud1D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::ICloud1D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud1D::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLowerX = 0;
  fUpperX = 0;
  fSw = 0;
  fSxw = 0;
  fSx2w = 0;
  fXs.clear();
  fWs.clear();
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud1D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fTitle;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::setTitle(
 const std::string& aTitle
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  if(isConverted()) fHistogram->setTitle(aTitle);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::Cloud1D::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::Cloud1D::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud1D::entries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return isConverted() ? fHistogram->allEntries() : fWs.size();
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud1D::nanEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud1D::maxEntries(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLimit;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  clear();
  delete fHistogram;
  fHistogram = 0;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud1D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::sumOfWeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return (isConverted() ? fHistogram->sumBinHeights() : fSw);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::lowerEdge(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return isConverted() ? fHistogram->axis().lowerEdge() : fLowerX;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::upperEdge(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return isConverted() ? fHistogram->axis().upperEdge() : fUpperX;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::value(
 int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? 0 : fXs[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::weight(
 int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? 0 : fWs[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::mean(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->mean() : (fSw?fSxw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud1D::rms(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rms();
  if(fSw==0) return 0;
  double mean = fSxw / fSw;
  double rms = ::sqrt(::fabs( (fSx2w / fSw) - mean * mean));
  return rms;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::scale(
 double aScale
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) {
    return fHistogram->scale(aScale);
  } else {
    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) fWs[index] *= aScale;
    fSw *= aScale;
    fSxw *= aScale;
    fSx2w *= aScale;
    return true;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::fill(
 double aX
,double aW
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( !fHistogram && (fLimit!=BATCHLAB_UNLIMITED) && ((int)fXs.size()>=fLimit) )
    convertToHistogram();

  if(fHistogram) {
    return fHistogram->fill(aX,aW);
  } else {
    if(fXs.size()) {
      fLowerX = MINIMUM(aX,fLowerX);
      fUpperX = MAXIMUM(aX,fUpperX);
    } else {
      fLowerX = aX;
      fUpperX = aX;
    }
    fXs.push_back(aX);
    fWs.push_back(aW);
    fSw += aW;
    double xw = aX * aW;
    fSxw += xw;
    fSx2w += aX * xw;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::isConverted(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::setConversionParameters(
 int aCnvXnumber
,double aCnvXmin
,double aCnvXmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCnvXnumber = aCnvXnumber;
  fCnvXmin = aCnvXmin;
  fCnvXmax = aCnvXmax;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::convertToHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( (fCnvXnumber<=0) || (fCnvXmax<=fCnvXmin) ) {
    // Cloud min, max should be included in the histo.
    double dx = 0.01 * (upperEdge() - lowerEdge())/BINS;
    return convert(BINS,lowerEdge(),upperEdge() + dx);
  } else {
    return convert(fCnvXnumber,fCnvXmin,fCnvXmax);
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::convert(
 int aBins
,double aLowerEdge
,double aUpperEdge
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram1D(printer(),
                                    this->name(),this->title(),
                                    aBins,aLowerEdge,aUpperEdge);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::convert(
 const std::vector<double>& aEdges
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram1D(printer(),
                                    this->name(),this->title(),
                                    aEdges);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::fillHistogram(
 AIDA::IHistogram1D& aHistogram
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fXs.size();
  for(unsigned int index=0;index<number;index++) {
    aHistogram.fill(fXs[index],fWs[index]);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IHistogram1D& BatchLab::Cloud1D::histogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(BatchLab::Cloud1D);
  if(!fHistogram) self.convertToHistogram();
  return *fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::setHistogram(
 BatchLab::Histogram1D* aHistogram
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  fHistogram = aHistogram;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud1D::xs(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud1D::update(
)
//////////////////////////////////////////////////////////////////////////////
// Used after IO, to update fields.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fHistogram) {
    fSw = 0;

    fLowerX = 0;
    fUpperX = 0;
    fSxw = 0;
    fSx2w = 0;
  } else {
    fSw = 0;

    if(fXs.size()) {
      fLowerX = fXs[0];
      fUpperX = fXs[0];
    } else {
      fLowerX = 0;
      fUpperX = 0;
    }
    fSxw = 0;
    fSx2w = 0;

    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) {
      double w = fWs[index];
      fSw += w;

      double x = fXs[index];
      fLowerX = MINIMUM(x,fLowerX);
      fUpperX = MAXIMUM(x,fUpperX);
      double xw = x * w;
      fSxw += xw;
      fSx2w += x * xw;
    }

  }
}
SLASH_STORE_SOURCE(BatchLab::Cloud1D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Cloud1D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  // BaseCloud :
  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fLimit",fLimit)) return false;
  if(!aVisitor.visit("fSw",fSw)) return false;
  if(!aVisitor.visit("fWs",fWs)) return false;

  // Cloud1D :
  if(!aVisitor.visit("fXs",fXs)) return false;
  if(!aVisitor.visit("fLowerX",fLowerX)) return false;
  if(!aVisitor.visit("fUpperX",fUpperX)) return false;
  if(!aVisitor.visit("fSxw",fSxw)) return false;
  if(!aVisitor.visit("fSx2w",fSx2w)) return false;
  if(!aVisitor.visit("fCnvXnumber",fCnvXnumber)) return false;
  if(!aVisitor.visit("fCnvXmin",fCnvXmin)) return false;
  if(!aVisitor.visit("fCnvXmax",fCnvXmax)) return false;

  bool isConverted = fHistogram ? true : false;
  if(!aVisitor.visit("fIsConverted",isConverted)) return false;
  if(isConverted) {
    if(!aVisitor.visit("fHistogram",*fHistogram)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud1D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  // BaseCloud :
  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fLimit)) return false;
  if(!aVisitor.visit(fSw)) return false;
  if(!aVisitor.visit(fWs)) return false;

  // Cloud1D :
  if(!aVisitor.visit(fXs)) return false;
  if(!aVisitor.visit(fLowerX)) return false;
  if(!aVisitor.visit(fUpperX)) return false;
  if(!aVisitor.visit(fSxw)) return false;
  if(!aVisitor.visit(fSx2w)) return false;
  if(!aVisitor.visit(fCnvXnumber)) return false;
  if(!aVisitor.visit(fCnvXmin)) return false;
  if(!aVisitor.visit(fCnvXmax)) return false;
  bool isConverted;
  if(!aVisitor.visit(isConverted)) return false;
  if(isConverted) {
    fHistogram = new BatchLab::Histogram1D(printer(),this->name());
    if(!fHistogram->read(aVisitor)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud2D::Cloud2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseCloud(aPrinter,aName,-1)
,fLowerX(0)
,fUpperX(0)
,fLowerY(0)
,fUpperY(0)
,fSxw(0)
,fSx2w(0)
,fSyw(0)
,fSy2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fCnvYnumber(0)
,fCnvYmin(0)
,fCnvYmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud2D::Cloud2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aLimit
)
:BaseCloud(aPrinter,aName,aLimit)
,fLowerX(0)
,fUpperX(0)
,fLowerY(0)
,fUpperY(0)
,fSxw(0)
,fSx2w(0)
,fSyw(0)
,fSy2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fCnvYnumber(0)
,fCnvYmin(0)
,fCnvYmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setTitle(aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud2D::Cloud2D(
 const Cloud2D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::ICloud(aFrom)
,AIDA::ICloud2D(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseCloud(aFrom)
,fXs(aFrom.fXs)
,fYs(aFrom.fYs)
,fLowerX(aFrom.fLowerX)
,fUpperX(aFrom.fUpperX)
,fLowerY(aFrom.fLowerY)
,fUpperY(aFrom.fUpperY)
,fSxw(aFrom.fSxw)
,fSx2w(aFrom.fSx2w)
,fSyw(aFrom.fSyw)
,fSy2w(aFrom.fSy2w)
,fCnvXnumber(aFrom.fCnvXnumber)
,fCnvXmin(aFrom.fCnvXmin)
,fCnvXmax(aFrom.fCnvXmax)
,fCnvYnumber(aFrom.fCnvYnumber)
,fCnvYmin(aFrom.fCnvYmin)
,fCnvYmax(aFrom.fCnvYmax)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFrom.fHistogram)
    fHistogram = new BatchLab::Histogram2D(*aFrom.fHistogram);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud2D::~Cloud2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Cloud2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Cloud2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud2D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Cloud2D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud2D::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BaseObject::setName(aName);
  if(fHistogram) fHistogram->setName(aName);
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Cloud2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::ICloud2D)
  else if_Lib_SCast(AIDA::ICloud)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Cloud2D)
  else return BaseCloud::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud2D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::ICloud2D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud2D::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLowerX = 0;
  fUpperX = 0;
  fLowerY = 0;
  fUpperY = 0;
  fSw = 0;
  fSxw = 0;
  fSx2w = 0;
  fSyw = 0;
  fSy2w = 0;
  fXs.clear();
  fYs.clear();
  fWs.clear();
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::sumOfWeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return (isConverted() ? fHistogram->sumBinHeights() : fSw);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud2D::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fTitle;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::setTitle(
 const std::string& aTitle
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  if(isConverted()) fHistogram->setTitle(aTitle);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::Cloud2D::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::Cloud2D::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud2D::entries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return isConverted() ? fHistogram->allEntries() : fWs.size();
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud2D::nanEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud2D::maxEntries(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLimit;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  clear();
  delete fHistogram;
  fHistogram = 0;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud2D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 2;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::scale(
 double aScale
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) {
    return fHistogram->scale(aScale);
  } else {
    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) fWs[index] *= aScale;
    fSw *= aScale;
    fSxw *= aScale;
    fSx2w *= aScale;
    fSyw *= aScale;
    fSy2w *= aScale;
    return true;
  }
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::lowerEdgeX(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->xAxis().lowerEdge() : fLowerX;
}
double BatchLab::Cloud2D::lowerEdgeY() const { 
  return isConverted() ? fHistogram->yAxis().lowerEdge() : fLowerY;
}
double BatchLab::Cloud2D::upperEdgeX() const { 
  return isConverted() ? fHistogram->xAxis().upperEdge() : fUpperX;
}
double BatchLab::Cloud2D::upperEdgeY() const { 
  return isConverted() ? fHistogram->yAxis().upperEdge() : fUpperY;
}
double BatchLab::Cloud2D::valueX(int aIndex) const { 
  return isConverted() ? 0 : fXs[aIndex];
}
double BatchLab::Cloud2D::valueY(int aIndex) const { 
  return isConverted() ? 0 : fYs[aIndex];
}
double BatchLab::Cloud2D::weight(int aIndex) const { 
  return isConverted() ? 0 : fWs[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::meanX(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->meanX() : (fSw?fSxw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::meanY(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->meanY() : (fSw?fSyw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::rmsX(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rmsX();
  if(fSw==0) return 0;
  double mean = fSxw / fSw;
  return ::sqrt(::fabs( (fSx2w / fSw) - mean * mean));
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud2D::rmsY(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rmsY();
  if(fSw==0) return 0;
  double mean = fSyw / fSw;
  return ::sqrt(::fabs( (fSy2w / fSw) - mean * mean));
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::convert( 
 int aBinsX
,double aLowerEdgeX
,double aUpperEdgeX
,int aBinsY
,double aLowerEdgeY
,double aUpperEdgeY 
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram2D(printer(),
                                    this->name(),this->title(),
                                    aBinsX,aLowerEdgeX,aUpperEdgeX,
                                    aBinsY,aLowerEdgeY,aUpperEdgeY);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::convert( 
 const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram2D(printer(),
                                    this->name(),this->title(),
                                    aEdgesX,aEdgesY);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::fillHistogram( 
 AIDA::IHistogram2D& aHistogram
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int number = fXs.size();
  for(unsigned int index=0;index<number;index++) {
    aHistogram.fill(fXs[index],fYs[index],fWs[index]);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::isConverted(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::setConversionParameters(
 int aCnvXnumber
,double aCnvXmin
,double aCnvXmax
,int aCnvYnumber
,double aCnvYmin
,double aCnvYmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCnvXnumber = aCnvXnumber;
  fCnvXmin = aCnvXmin;
  fCnvXmax = aCnvXmax;
  fCnvYnumber = aCnvYnumber;
  fCnvYmin = aCnvYmin;
  fCnvYmax = aCnvYmax;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::convertToHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( (fCnvXnumber<=0) || (fCnvXmax<=fCnvXmin) ||
      (fCnvYnumber<=0) || (fCnvYmax<=fCnvYmin) ) {
    double dx = 0.01 * (upperEdgeX() - lowerEdgeX())/BINS;
    double dy = 0.01 * (upperEdgeY() - lowerEdgeY())/BINS;
    return convert(BINS,lowerEdgeX(),upperEdgeX()+dx,
                   BINS,lowerEdgeY(),upperEdgeY()+dy);
  } else {
    return convert(fCnvXnumber,fCnvXmin,fCnvXmax,
                   fCnvYnumber,fCnvYmin,fCnvYmax);
  }
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IHistogram2D& BatchLab::Cloud2D::histogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Lib_self(BatchLab::Cloud2D);
  if(!fHistogram) self.convertToHistogram();
  return *fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::setHistogram(
 BatchLab::Histogram2D* aHistogram
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  fHistogram = aHistogram;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::fill(
 double aX
,double aY
,double aW
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( !fHistogram && (fLimit!=BATCHLAB_UNLIMITED) && ((int)fXs.size()>=fLimit) )
    convertToHistogram();

  if(fHistogram) {
    return fHistogram->fill(aX,aY,aW);
  } else {
    if(fXs.size()) {
      fLowerX = MINIMUM(aX,fLowerX);
      fUpperX = MAXIMUM(aX,fUpperX);
    } else {
      fLowerX = aX;
      fUpperX = aX;
    }
    if(fYs.size()) { 
      fLowerY = MINIMUM(aY,fLowerY);
      fUpperY = MAXIMUM(aY,fUpperY);
    } else {
      fLowerY = aY;
      fUpperY = aY;
    }
    fXs.push_back(aX);
    fYs.push_back(aY);
    fWs.push_back(aW);
    fSw += aW;
    double xw = aX * aW;
    fSxw += xw;
    fSx2w += aX * xw;
    double yw = aY * aW; 
    fSyw += yw;
    fSy2w += aY * yw;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud2D::xs(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud2D::ys(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fYs;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud2D::update(
)
//////////////////////////////////////////////////////////////////////////////
// Used after IO, to update fields.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fHistogram) {
    fSw = 0;

    fLowerX = 0;
    fUpperX = 0;
    fSxw = 0;
    fSx2w = 0;

    fLowerY = 0;
    fUpperY = 0;
    fSyw = 0;
    fSy2w = 0;
  } else {
    fSw = 0;

    if(fXs.size()) {
      fLowerX = fXs[0];
      fUpperX = fXs[0];
    } else {
      fLowerX = 0;
      fUpperX = 0;
    }
    fSxw = 0;
    fSx2w = 0;

    if(fYs.size()) {
      fLowerY = fYs[0];
      fUpperY = fYs[0];
    } else {
      fLowerY = 0;
      fUpperY = 0;
    }
    fSyw = 0;
    fSy2w = 0;

    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) {
      double w = fWs[index];
      fSw += w;

      double x = fXs[index];
      fLowerX = MINIMUM(x,fLowerX);
      fUpperX = MAXIMUM(x,fUpperX);
      double xw = x * w;
      fSxw += xw;
      fSx2w += x * xw;

      double y = fYs[index];
      fLowerY = MINIMUM(y,fLowerY);
      fUpperY = MAXIMUM(y,fUpperY);
      double yw = y * w;
      fSyw += yw;
      fSy2w += y * yw;
    }
  }
}
SLASH_STORE_SOURCE(BatchLab::Cloud2D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Cloud2D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  // BaseCloud :
  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fLimit",fLimit)) return false;
  if(!aVisitor.visit("fSw",fSw)) return false;
  if(!aVisitor.visit("fWs",fWs)) return false;

  // Cloud2D :
  if(!aVisitor.visit("fXs",fXs)) return false;
  if(!aVisitor.visit("fYs",fYs)) return false;
  if(!aVisitor.visit("fLowerX",fLowerX)) return false;
  if(!aVisitor.visit("fUpperX",fUpperX)) return false;
  if(!aVisitor.visit("fLowerY",fLowerY)) return false;
  if(!aVisitor.visit("fUpperY",fUpperY)) return false;
  if(!aVisitor.visit("fSxw",fSxw)) return false;
  if(!aVisitor.visit("fSx2w",fSx2w)) return false;
  if(!aVisitor.visit("fSyw",fSyw)) return false;
  if(!aVisitor.visit("fSy2w",fSy2w)) return false;
  if(!aVisitor.visit("fCnvXnumber",fCnvXnumber)) return false;
  if(!aVisitor.visit("fCnvXmin",fCnvXmin)) return false;
  if(!aVisitor.visit("fCnvXmax",fCnvXmax)) return false;
  if(!aVisitor.visit("fCnvYnumber",fCnvYnumber)) return false;
  if(!aVisitor.visit("fCnvYmin",fCnvYmin)) return false;
  if(!aVisitor.visit("fCnvYmax",fCnvYmax)) return false;

  bool isConverted = fHistogram ? true : false;
  if(!aVisitor.visit("fIsConverted",isConverted)) return false;
  if(isConverted) {
    if(!aVisitor.visit("fHistogram",*fHistogram)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud2D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  // BaseCloud :
  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fLimit)) return false;
  if(!aVisitor.visit(fSw)) return false;
  if(!aVisitor.visit(fWs)) return false;

  // Cloud2D :
  if(!aVisitor.visit(fXs)) return false;
  if(!aVisitor.visit(fYs)) return false;
  if(!aVisitor.visit(fLowerX)) return false;
  if(!aVisitor.visit(fUpperX)) return false;
  if(!aVisitor.visit(fLowerY)) return false;
  if(!aVisitor.visit(fUpperY)) return false;
  if(!aVisitor.visit(fSxw)) return false;
  if(!aVisitor.visit(fSx2w)) return false;
  if(!aVisitor.visit(fSyw)) return false;
  if(!aVisitor.visit(fSy2w)) return false;
  if(!aVisitor.visit(fCnvXnumber)) return false;
  if(!aVisitor.visit(fCnvXmin)) return false;
  if(!aVisitor.visit(fCnvXmax)) return false;
  if(!aVisitor.visit(fCnvYnumber)) return false;
  if(!aVisitor.visit(fCnvYmin)) return false;
  if(!aVisitor.visit(fCnvYmax)) return false;
  bool isConverted;
  if(!aVisitor.visit(isConverted)) return false;
  if(isConverted) {
    fHistogram = new BatchLab::Histogram2D(printer(),this->name());
    if(!fHistogram->read(aVisitor)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud3D::Cloud3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseCloud(aPrinter,aName,-1)
,fLowerX(0)
,fUpperX(0)
,fLowerY(0)
,fUpperY(0)
,fLowerZ(0)
,fUpperZ(0)
,fSxw(0)
,fSx2w(0)
,fSyw(0)
,fSy2w(0)
,fSzw(0)
,fSz2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fCnvYnumber(0)
,fCnvYmin(0)
,fCnvYmax(0)
,fCnvZnumber(0)
,fCnvZmin(0)
,fCnvZmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud3D::Cloud3D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,int aLimit
)
:BaseCloud(aPrinter,aName,aLimit)
,fLowerX(0)
,fUpperX(0)
,fLowerY(0)
,fUpperY(0)
,fLowerZ(0)
,fUpperZ(0)
,fSxw(0)
,fSx2w(0)
,fSyw(0)
,fSy2w(0)
,fSzw(0)
,fSz2w(0)
,fCnvXnumber(0)
,fCnvXmin(0)
,fCnvXmax(0)
,fCnvYnumber(0)
,fCnvYmin(0)
,fCnvYmax(0)
,fCnvZnumber(0)
,fCnvZmin(0)
,fCnvZmax(0)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setTitle(aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud3D::Cloud3D(
 const Cloud3D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IBaseHistogram(aFrom)
,AIDA::ICloud(aFrom)
,AIDA::ICloud3D(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseCloud(aFrom)
,fXs(aFrom.fXs)
,fYs(aFrom.fYs)
,fZs(aFrom.fZs)
,fLowerX(aFrom.fLowerX)
,fUpperX(aFrom.fUpperX)
,fLowerY(aFrom.fLowerY)
,fUpperY(aFrom.fUpperY)
,fLowerZ(aFrom.fLowerZ)
,fUpperZ(aFrom.fUpperZ)
,fSxw(aFrom.fSxw)
,fSx2w(aFrom.fSx2w)
,fSyw(aFrom.fSyw)
,fSy2w(aFrom.fSy2w)
,fSzw(aFrom.fSzw)
,fSz2w(aFrom.fSz2w)
,fCnvXnumber(aFrom.fCnvXnumber)
,fCnvXmin(aFrom.fCnvXmin)
,fCnvXmax(aFrom.fCnvXmax)
,fCnvYnumber(aFrom.fCnvYnumber)
,fCnvYmin(aFrom.fCnvYmin)
,fCnvYmax(aFrom.fCnvYmax)
,fCnvZnumber(aFrom.fCnvZnumber)
,fCnvZmin(aFrom.fCnvZmin)
,fCnvZmax(aFrom.fCnvZmax)
,fHistogram(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFrom.fHistogram)
    fHistogram = new BatchLab::Histogram3D(*aFrom.fHistogram);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Cloud3D::~Cloud3D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Cloud3D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Cloud3D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud3D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Cloud3D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud3D::setName(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BaseObject::setName(aName);
  if(fHistogram) fHistogram->setName(aName);
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Cloud3D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::ICloud3D)
  else if_Lib_SCast(AIDA::ICloud)
  else if_Lib_SCast(AIDA::IBaseHistogram)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(Slash::Meta::IObject)
  else if_Lib_SCast(BatchLab::Cloud3D)
  else return BaseCloud::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Cloud3D::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::ICloud3D";
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud3D::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLowerX = 0;
  fUpperX = 0;
  fLowerY = 0;
  fUpperY = 0;
  fLowerZ = 0;
  fUpperZ = 0;
  fSw = 0;
  fSxw = 0;
  fSx2w = 0;
  fSyw = 0;
  fSy2w = 0;
  fSzw = 0;
  fSz2w = 0;
  fXs.clear();
  fYs.clear();
  fZs.clear();
  fWs.clear();
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::sumOfWeights(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return (isConverted() ? fHistogram->sumBinHeights() : fSw);
}
std::string BatchLab::Cloud3D::title() const { return fTitle;}
bool BatchLab::Cloud3D::setTitle(const std::string& aTitle){
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  if(isConverted()) fHistogram->setTitle(aTitle);
  return true;
}
AIDA::IAnnotation& BatchLab::Cloud3D::annotation() {
  return fAnnotation;}
const AIDA::IAnnotation& BatchLab::Cloud3D::annotation() const {
  return fAnnotation;}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud3D::entries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return isConverted() ? fHistogram->allEntries() : fWs.size();
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud3D::nanEntries(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Cloud3D::maxEntries(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLimit;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  clear();
  delete fHistogram;
  fHistogram = 0;
  return true;
}
int BatchLab::Cloud3D::dimension() const { return 3;}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::scale(
 double aScale
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) {
    return fHistogram->scale(aScale);
  } else {
    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) fWs[index] *= aScale;
    fSw *= aScale;
    fSxw *= aScale;
    fSx2w *= aScale;
    fSyw *= aScale;
    fSy2w *= aScale;
    fSzw *= aScale;
    fSz2w *= aScale;
    return true;
  }
}
double BatchLab::Cloud3D::lowerEdgeX() const { 
  return isConverted() ? fHistogram->xAxis().lowerEdge() : fLowerX;
}
double BatchLab::Cloud3D::lowerEdgeY() const { 
  return isConverted() ? fHistogram->yAxis().lowerEdge() : fLowerY;
}
double BatchLab::Cloud3D::lowerEdgeZ() const { 
  return isConverted() ? fHistogram->zAxis().lowerEdge() : fLowerZ;
}
double BatchLab::Cloud3D::upperEdgeX() const { 
  return isConverted() ? fHistogram->xAxis().upperEdge() : fUpperX;
}
double BatchLab::Cloud3D::upperEdgeY() const { 
  return isConverted() ? fHistogram->yAxis().upperEdge() : fUpperY;
}
double BatchLab::Cloud3D::upperEdgeZ() const { 
  return isConverted() ? fHistogram->zAxis().upperEdge() : fUpperZ;
}
double BatchLab::Cloud3D::valueX(int aIndex) const { 
  return isConverted() ? 0 : fXs[aIndex];
}
double BatchLab::Cloud3D::valueY(int aIndex) const { 
  return isConverted() ? 0 : fYs[aIndex];
}
double BatchLab::Cloud3D::valueZ(int aIndex) const { 
  return isConverted() ? 0 : fZs[aIndex];
}
double BatchLab::Cloud3D::weight(int aIndex) const { 
  return isConverted() ? 0 : fWs[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::meanX(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->meanX() : (fSw?fSxw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::meanY(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->meanY() : (fSw?fSyw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::meanZ(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return isConverted() ? fHistogram->meanZ() : (fSw?fSzw/fSw:0);
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::rmsX(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rmsX();
  if(fSw==0) return 0;
  double mean = fSxw / fSw;
  return ::sqrt(::fabs( (fSx2w / fSw) - mean * mean));
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::rmsY(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rmsY();
  if(fSw==0) return 0;
  double mean = fSyw / fSw;
  return ::sqrt(::fabs( (fSy2w / fSw) - mean * mean));
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Cloud3D::rmsZ(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(isConverted()) return fHistogram->rmsZ();
  if(fSw==0) return 0;
  double mean = fSzw / fSw;
  return ::sqrt(::fabs( (fSz2w / fSw) - mean * mean));
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::isConverted(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHistogram ? true : false;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::setConversionParameters(
 int aCnvXnumber
,double aCnvXmin
,double aCnvXmax
,int aCnvYnumber
,double aCnvYmin
,double aCnvYmax
,int aCnvZnumber
,double aCnvZmin
,double aCnvZmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCnvXnumber = aCnvXnumber;
  fCnvXmin = aCnvXmin;
  fCnvXmax = aCnvXmax;
  fCnvYnumber = aCnvYnumber;
  fCnvYmin = aCnvYmin;
  fCnvYmax = aCnvYmax;
  fCnvZnumber = aCnvZnumber;
  fCnvZmin = aCnvZmin;
  fCnvZmax = aCnvZmax;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::convert( 
 int aBinsX
,double aLowerEdgeX
,double aUpperEdgeX
,int aBinsY
,double aLowerEdgeY
,double aUpperEdgeY 
,int aBinsZ
,double aLowerEdgeZ
,double aUpperEdgeZ 
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram3D(printer(),
                                    this->name(),this->title(),
                                    aBinsX,aLowerEdgeX,aUpperEdgeX,
                                    aBinsY,aLowerEdgeY,aUpperEdgeY,
                                    aBinsZ,aLowerEdgeZ,aUpperEdgeZ);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::convert( 
 const std::vector<double>& aEdgesX
,const std::vector<double>& aEdgesY
,const std::vector<double>& aEdgesZ
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fHistogram) return true; // Done.
  fHistogram = new BatchLab::Histogram3D(printer(),
                                    this->name(),this->title(),
                                    aEdgesX,aEdgesY,aEdgesZ);
  if(!fHistogram) return false;
  fillHistogram(*fHistogram);
  clear(); // Free memory:
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::fillHistogram( 
 AIDA::IHistogram3D& aHistogram
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  unsigned int number = fXs.size();
  for(unsigned int index=0;index<number;index++) {
    aHistogram.fill(fXs[index],fYs[index],fZs[index],fWs[index]);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::convertToHistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( (fCnvXnumber<=0) || (fCnvXmax<=fCnvXmin) ||
      (fCnvYnumber<=0) || (fCnvYmax<=fCnvYmin) ||
      (fCnvZnumber<=0) || (fCnvZmax<=fCnvZmin) ) {
    double dx = 0.01 * (upperEdgeX() - lowerEdgeX())/BINS;
    double dy = 0.01 * (upperEdgeY() - lowerEdgeY())/BINS;
    double dz = 0.01 * (upperEdgeZ() - lowerEdgeZ())/BINS;
    return convert(BINS,lowerEdgeX(),upperEdgeX()+dx,
                   BINS,lowerEdgeY(),upperEdgeY()+dy,
                   BINS,lowerEdgeZ(),upperEdgeZ()+dz);
  } else {
    return convert(fCnvXnumber,fCnvXmin,fCnvXmax,
                   fCnvYnumber,fCnvYmin,fCnvYmax,
                   fCnvZnumber,fCnvZmin,fCnvZmax);
  }
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IHistogram3D& BatchLab::Cloud3D::histogram(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Lib_self(BatchLab::Cloud3D);
  if(!fHistogram) self.convertToHistogram();
  return *fHistogram;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::setHistogram(
 BatchLab::Histogram3D* aHistogram
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  fHistogram = aHistogram;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::fill(
 double aX
,double aY
,double aZ
,double aW
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( !fHistogram && (fLimit!=BATCHLAB_UNLIMITED) && ((int)fXs.size()>=fLimit) )
    convertToHistogram();

  if(fHistogram) {
    return fHistogram->fill(aX,aY,aZ,aW);
  } else {
    if(fXs.size()) {
      fLowerX = MINIMUM(aX,fLowerX);
      fUpperX = MAXIMUM(aX,fUpperX);
    } else {
      fLowerX = aX;
      fUpperX = aX;
    }
    if(fYs.size()) { 
      fLowerY = MINIMUM(aY,fLowerY);
      fUpperY = MAXIMUM(aY,fUpperY);
    } else {
      fLowerY = aY;
      fUpperY = aY;
    }
    if(fZs.size()) { 
      fLowerZ = MINIMUM(aZ,fLowerZ);
      fUpperZ = MAXIMUM(aZ,fUpperZ);
    } else {
      fLowerZ = aZ;
      fUpperZ = aZ;
    }
    fXs.push_back(aX);
    fYs.push_back(aY);
    fZs.push_back(aZ);
    fWs.push_back(aW);
    fSw += aW;
    double xw = aX * aW;
    fSxw += xw;
    fSx2w += aX * xw;
    double yw = aY * aW; 
    fSyw += yw;
    fSy2w += aY * yw;
    double zw = aZ * aW;
    fSzw += zw;
    fSz2w += aZ * zw;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud3D::xs(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud3D::ys(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fYs;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double>& BatchLab::Cloud3D::zs(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fZs;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Cloud3D::update(
)
//////////////////////////////////////////////////////////////////////////////
// Used after IO, to update fields.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fHistogram) {
    fSw = 0;

    fLowerX = 0;
    fUpperX = 0;
    fSxw = 0;
    fSx2w = 0;

    fLowerY = 0;
    fUpperY = 0;
    fSyw = 0;
    fSy2w = 0;

    fLowerZ = 0;
    fUpperZ = 0;
    fSzw = 0;
    fSz2w = 0;
  } else {
    fSw = 0;

    if(fXs.size()) {
      fLowerX = fXs[0];
      fUpperX = fXs[0];
    } else {
      fLowerX = 0;
      fUpperX = 0;
    }
    fSxw = 0;
    fSx2w = 0;

    if(fYs.size()) {
      fLowerY = fYs[0];
      fUpperY = fYs[0];
    } else {
      fLowerY = 0;
      fUpperY = 0;
    }
    fSyw = 0;
    fSy2w = 0;

    if(fZs.size()) {
      fLowerZ = fZs[0];
      fUpperZ = fZs[0];
    } else {
      fLowerZ = 0;
      fUpperZ = 0;
    }
    fSzw = 0;
    fSz2w = 0;

    unsigned int number = fWs.size();
    for(unsigned int index=0;index<number;index++) {
      double w = fWs[index];
      fSw += w;

      double x = fXs[index];
      fLowerX = MINIMUM(x,fLowerX);
      fUpperX = MAXIMUM(x,fUpperX);
      double xw = x * w;
      fSxw += xw;
      fSx2w += x * xw;

      double y = fYs[index];
      fLowerY = MINIMUM(y,fLowerY);
      fUpperY = MAXIMUM(y,fUpperY);
      double yw = y * w;
      fSyw += yw;
      fSy2w += y * yw;

      double z = fZs[index];
      fLowerZ = MINIMUM(z,fLowerZ);
      fUpperZ = MAXIMUM(z,fUpperZ);
      double zw = z * w;
      fSzw += zw;
      fSz2w += z * zw;
    }
  }
}
SLASH_STORE_SOURCE(BatchLab::Cloud3D)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Cloud3D)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  // BaseCloud :
  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fLimit",fLimit)) return false;
  if(!aVisitor.visit("fSw",fSw)) return false;
  if(!aVisitor.visit("fWs",fWs)) return false;

  // Cloud3D :
  if(!aVisitor.visit("fXs",fXs)) return false;
  if(!aVisitor.visit("fYs",fYs)) return false;
  if(!aVisitor.visit("fZs",fZs)) return false;
  if(!aVisitor.visit("fLowerX",fLowerX)) return false;
  if(!aVisitor.visit("fUpperX",fUpperX)) return false;
  if(!aVisitor.visit("fLowerY",fLowerY)) return false;
  if(!aVisitor.visit("fUpperY",fUpperY)) return false;
  if(!aVisitor.visit("fLowerZ",fLowerZ)) return false;
  if(!aVisitor.visit("fUpperZ",fUpperZ)) return false;
  if(!aVisitor.visit("fSxw",fSxw)) return false;
  if(!aVisitor.visit("fSx2w",fSx2w)) return false;
  if(!aVisitor.visit("fSyw",fSyw)) return false;
  if(!aVisitor.visit("fSy2w",fSy2w)) return false;
  if(!aVisitor.visit("fSzw",fSzw)) return false;
  if(!aVisitor.visit("fSz2w",fSz2w)) return false;
  if(!aVisitor.visit("fCnvXnumber",fCnvXnumber)) return false;
  if(!aVisitor.visit("fCnvXmin",fCnvXmin)) return false;
  if(!aVisitor.visit("fCnvXmax",fCnvXmax)) return false;
  if(!aVisitor.visit("fCnvYnumber",fCnvYnumber)) return false;
  if(!aVisitor.visit("fCnvYmin",fCnvYmin)) return false;
  if(!aVisitor.visit("fCnvYmax",fCnvYmax)) return false;
  if(!aVisitor.visit("fCnvZnumber",fCnvZnumber)) return false;
  if(!aVisitor.visit("fCnvZmin",fCnvZmin)) return false;
  if(!aVisitor.visit("fCnvZmax",fCnvZmax)) return false;

  bool isConverted = fHistogram ? true : false;
  if(!aVisitor.visit("fIsConverted",isConverted)) return false;
  if(isConverted) {
    if(!aVisitor.visit("fHistogram",*fHistogram)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Cloud3D::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  // BaseCloud :
  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fLimit)) return false;
  if(!aVisitor.visit(fSw)) return false;
  if(!aVisitor.visit(fWs)) return false;

  // Cloud3D :
  if(!aVisitor.visit(fXs)) return false;
  if(!aVisitor.visit(fYs)) return false;
  if(!aVisitor.visit(fZs)) return false;
  if(!aVisitor.visit(fLowerX)) return false;
  if(!aVisitor.visit(fUpperX)) return false;
  if(!aVisitor.visit(fLowerY)) return false;
  if(!aVisitor.visit(fUpperY)) return false;
  if(!aVisitor.visit(fLowerZ)) return false;
  if(!aVisitor.visit(fUpperZ)) return false;
  if(!aVisitor.visit(fSxw)) return false;
  if(!aVisitor.visit(fSx2w)) return false;
  if(!aVisitor.visit(fSyw)) return false;
  if(!aVisitor.visit(fSy2w)) return false;
  if(!aVisitor.visit(fSzw)) return false;
  if(!aVisitor.visit(fSz2w)) return false;
  if(!aVisitor.visit(fCnvXnumber)) return false;
  if(!aVisitor.visit(fCnvXmin)) return false;
  if(!aVisitor.visit(fCnvXmax)) return false;
  if(!aVisitor.visit(fCnvYnumber)) return false;
  if(!aVisitor.visit(fCnvYmin)) return false;
  if(!aVisitor.visit(fCnvYmax)) return false;
  if(!aVisitor.visit(fCnvZnumber)) return false;
  if(!aVisitor.visit(fCnvZmin)) return false;
  if(!aVisitor.visit(fCnvZmax)) return false;
  bool isConverted;
  if(!aVisitor.visit(isConverted)) return false;
  if(isConverted) {
    fHistogram = new BatchLab::Histogram3D(printer(),this->name());
    if(!fHistogram->read(aVisitor)) return false;
  }

  if(!aVisitor.end(*this)) return false;
  return true;
}
