// this :
#include <BatchLab/Core/FitFactory.h>

#include <Lib/Debug.h>
#include <Lib/smanip.h>

#include <BatchLab/Core/MinuitFitter.h>
#include <BatchLab/Core/MidnightFitter.h>
#include <BatchLab/Core/FitData.h>

#include <AIDA/IFunctionFactory.h>

//////////////////////////////////////////////////////////////////////////////
BatchLab::FitFactory::FitFactory(
 Slash::Core::IWriter& aPrinter
,AIDA::IFunctionFactory* aFunctionFactory
)
:fPrinter(aPrinter)
,fFunctionFactory(aFunctionFactory)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::FitFactory");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::FitFactory::~FitFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("BatchLab::FitFactory");
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitData* BatchLab::FitFactory::createFitData(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::FitData();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitter* BatchLab::FitFactory::createFitter(
 const std::string& //aFitterType
,const std::string& aEngineType
,const std::string& //aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : dynamic load a fitter.
  // Not managed.
  std::string engine = aEngineType;
  Lib::smanip::tolowercase(engine);
  if(engine=="minuitc++") {
    return new BatchLab::MinuitFitter(fPrinter,*fFunctionFactory);
  } else {
    //Default.
    return new BatchLab::MidnightFitter(fPrinter,*fFunctionFactory);
  }
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::FitFactory::availableFitMethods(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vs;
  vs.push_back("chi2");
  return vs;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::FitFactory::availableFitEngines(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vs;
  vs.push_back("minuitc++");
  vs.push_back("midnight");
  return vs;
}
