// this :
#include <BatchLab/Core/Function.h>

#include <Slash/Data/IFunction.h>
#include <Slash/Data/IFunctionManager.h>
#include <Slash/Data/IArray.h> //For Compiled.

#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>
#include <Lib/Manager.h>
#include <Lib/valuemanip.h>

#include <AIDA/IConstants.h>

#include <BatchLab/Core/FunctionCatalog.h>

#include <BatchLab/Core/Array.h> //For Compiled.

#include <math.h>

#define NotFound (-1)


#define VISIT_SOURCE(aClass) \
SLASH_STORE_SOURCE(aClass)\
bool aClass::visit(Slash::Store::IConstVisitor& aVisitor) const {\
  SLASH_STORE_BEGIN(aClass)\
  int version = 1;\
  if(!aVisitor.visit("fVersion",version)) return false;\
  if(!BaseFunction::visit(aVisitor)) return false;\
  if(!aVisitor.end(*this)) return false;\
  return true;\
}\
bool aClass::read(Slash::Store::IVisitor& aVisitor){\
  if(!aVisitor.begin(*this)) return false;\
  int version;\
  if(!aVisitor.visit(version)) return false;\
  if(!BaseFunction::read(aVisitor)) return false;\
  if(!aVisitor.end(*this)) return false;\
  if(!fFunction.setParameters(fParams)) return false;\
  return true;\
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseFunction::BaseFunction(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
)
:BaseObject(aPrinter,aName)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setTitle(aTitle);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseFunction::BaseFunction(
 const BaseFunction& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,BaseObject(aFrom)
,fAnnotation(aFrom.fAnnotation)
,fTitle(aFrom.fTitle)
,fCodelet(aFrom.fCodelet)
,fParams(aFrom.fParams)
,fParamNames(aFrom.fParamNames)
,fVariableNames(aFrom.fVariableNames)
,fGradient(aFrom.fGradient)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseFunction::~BaseFunction(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::BaseFunction::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(AIDA::IFunction)
  else if_Lib_SCast(Slash::Store::IStorable)
  else if_Lib_SCast(BatchLab::BaseFunction)
  else return BaseObject::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Annotation& BatchLab::BaseFunction::getAnnotation(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::BaseFunction::setAnnotation(
 const BatchLab::Annotation& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAnnotation = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFunction::type(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return "AIDA::IFunction";
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFunction::normalizationParameter(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return ""; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFunction::title(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fTitle;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::setTitle(
 const std::string& aTitle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fTitle = aTitle;
  fAnnotation.setValue("Title",aTitle);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IAnnotation& BatchLab::BaseFunction::annotation(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
const AIDA::IAnnotation& BatchLab::BaseFunction::annotation(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAnnotation;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::BaseFunction::parameterNames(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParamNames;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fParamNames.size()!=fParams.size()) return false;
  unsigned int number = fParamNames.size();
  for(unsigned int i=0;i<number;i++) {
    if(fParamNames[i]==aName) {
      fParams[i] = aValue;
      return true;
    }
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& BatchLab::BaseFunction::parameters(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParams;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::isEqual(
 const IFunction&
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME
  return false;  
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& BatchLab::BaseFunction::gradient(
 const std::vector<double>&
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fGradient;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::providesGradient(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFunction::variableName(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aIndex<0)||(aIndex>=(int)fVariableNames.size())) return "";
  return fVariableNames[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> BatchLab::BaseFunction::variableNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVariableNames;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::BaseFunction::numberOfParameters(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParams.size();
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::BaseFunction::indexOfParameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fParamNames.size();
  for(unsigned int i=0;i<number;i++) {
    if(fParamNames[i]==aName) return i;
  }
  return NotFound;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::BaseFunction::codeletString(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCodelet;
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::BaseFunction::value(
 const std::vector<double>& aPoint
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  double d;
  if(!value(aPoint,d)) {} //FIXME : throw
  return d;
}
SLASH_STORE_SOURCE(BatchLab::BaseFunction)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::BaseFunction)

  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;

  if(!aVisitor.visit("fAnnotation",fAnnotation)) return false;
  if(!aVisitor.visit("fTitle",fTitle)) return false;
  if(!aVisitor.visit("fCodelet",fCodelet)) return false;
  if(!aVisitor.visit("fParams",fParams)) return false;
  if(!aVisitor.visit("fParamNames",fParamNames)) return false;
  if(!aVisitor.visit("fVariableNames",fVariableNames)) return false;
  if(!aVisitor.visit("fGradient",fGradient)) return false;

  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::BaseFunction::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;

  int version;
  if(!aVisitor.visit(version)) return false;

  if(!fAnnotation.read(aVisitor)) return false;
  if(!aVisitor.visit(fTitle)) return false;
  if(!aVisitor.visit(fCodelet)) return false;
  if(!aVisitor.visit(fParams)) return false;
  if(!aVisitor.visit(fParamNames)) return false;
  if(!aVisitor.visit(fVariableNames)) return false;
  if(!aVisitor.visit(fGradient)) return false;

  if(!aVisitor.end(*this)) return false;

  //FIXME : have to update the Lib functions !

  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Gaussian::Gaussian(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BaseFunction(aPrinter,aName,"")
//,fFunction(aScale,aMean,aSigma)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Gaussian::Gaussian(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,double aScale
,double aMean
,double aSigma
)
:BaseFunction(aPrinter,aName,aTitle)
,fFunction(aScale,aMean,aSigma)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(3);
  fParamNames[0] = "amplitude"; //AIDA convention.
  fParamNames[1] = "mean";
  fParamNames[2] = "sigma";
  fParams.resize(3);
  fParams[0] = aScale;
  fParams[1] = aMean;
  fParams[2] = aSigma;
  fVariableNames.push_back("x[0]");
  fCodelet = "class:Gaussian";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Gaussian::Gaussian(
 const Gaussian& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Gaussian::~Gaussian(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Gaussian::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Gaussian)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Gaussian)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Gaussian::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Gaussian(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Gaussian::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Gaussian";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Gaussian::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="amplitude") return fParams[0];
  else if(aName=="mean") return fParams[1];
  else if(aName=="sigma") return fParams[2];
  else {
    Lib::Out out(printer());
    out << "BatchLab::Function::Gaussian::parameter :"
        << " unknown parameter " << aName << "."
        << Lib::endl;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Gaussian::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Gaussian::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Gaussian::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Gaussian::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  if(index==0) {
    fFunction.setScale(aValue);
  } else if(index==1) {
    fFunction.setMean(aValue);
  } else if(index==2) {
    fFunction.setSigma(aValue);
  }
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Gaussian::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Gaussian::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::BreitWigner::BreitWigner(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::BreitWigner::BreitWigner(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,double aHeight
,double aCenter
,double aWidth
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aHeight,aCenter,aWidth)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(3);
  fParamNames[0] = "amplitude"; //AIDA convention.
  fParamNames[1] = "origin";
  fParamNames[2] = "width";
  fParams.resize(3);
  fParams[0] = aHeight;
  fParams[1] = aCenter;
  fParams[2] = aWidth;
  fVariableNames.push_back("x[0]");
  fCodelet = "class:BreitWigner";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::BreitWigner::BreitWigner(
 const BreitWigner& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::BreitWigner::~BreitWigner(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::BreitWigner::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::BreitWigner)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::BreitWigner)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::BreitWigner::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::BreitWigner(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::BreitWigner::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::BreitWigner";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::BreitWigner::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="amplitude") return fParams[0];
  else if(aName=="origin") return fParams[1];
  else if(aName=="width") return fParams[2];
  else {
    Lib::Out out(printer());
    out << "BatchLab::Function::BreitWigner::parameter :"
        << " unknown parameter " << aName << "."
        << Lib::endl;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::BreitWigner::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) {
    Lib::Out out(printer());
    out << "BatchLab::Function::BreitWigner::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::BreitWigner::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::BreitWigner::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  if(index==0) {
    fFunction.setHeight(aValue);
  } else if(index==1) {
    fFunction.setCenter(aValue);
  } else if(index==2) {
    fFunction.setWidth(aValue);
  }
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::BreitWigner::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::BreitWigner::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpHBOOK::ExpHBOOK(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpHBOOK::ExpHBOOK(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,double aA
,double aB
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aA,aB)
//////////////////////////////////////////////////////////////////////////////
// WARNING : 
//   Lib, HBOOK, ROOT : exp(aA + aB * x)
//   AIDA : par[0] * exp(par[1]*x)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(2);
  fParamNames[0] = "A";
  fParamNames[1] = "B";
  fParams.resize(2);
  fParams[0] = aA;
  fParams[1] = aB;
  fVariableNames.push_back("x[0]");
  fCodelet = "class:ExpHBOOK";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpHBOOK::ExpHBOOK(
 const ExpHBOOK& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpHBOOK::~ExpHBOOK(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::ExpHBOOK::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::ExpHBOOK)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::ExpHBOOK)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::ExpHBOOK::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::ExpHBOOK(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::ExpHBOOK::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::ExpHBOOK";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::ExpHBOOK::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="A") return fParams[0];
  else if(aName=="B") return fParams[1];
  else {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpHBOOK::parameter :"
        << " unknown parameter " << aName << "."
        << Lib::endl;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpHBOOK::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=2) {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpHBOOK::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpHBOOK::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpHBOOK::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  if(index==0) {
    fFunction.setA(aValue);
  } else if(index==1) {
    fFunction.setB(aValue);
  }
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::ExpHBOOK::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpHBOOK::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpAIDA::ExpAIDA(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpAIDA::ExpAIDA(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,double aP0
,double aP1
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(::log(aP0),aP1)
//////////////////////////////////////////////////////////////////////////////
// WARNING : 
//   Lib, HBOOK, ROOT : exp(aA + aB * x)
//   AIDA : par[0] * exp(par[1]*x)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(2);
  fParamNames[0] = "amplitude";
  fParamNames[1] = "exponent";
  fParams.resize(2);
  fParams[0] = aP0;
  fParams[1] = aP1;
  fVariableNames.push_back("x[0]");
  fCodelet = "class:ExpAIDA";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpAIDA::ExpAIDA(
 const ExpAIDA& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ExpAIDA::~ExpAIDA(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::ExpAIDA::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::ExpAIDA)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
SLASH_STORE_SOURCE(BatchLab::Function::ExpAIDA)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpAIDA::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Function::ExpAIDA)
  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;
  if(!BaseFunction::visit(aVisitor)) return false;
  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpAIDA::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;
  int version;
  if(!aVisitor.visit(version)) return false;
  if(!BaseFunction::read(aVisitor)) return false;
  if(!aVisitor.end(*this)) return false;

  //fFunction.setParameters(fParams);
  fFunction.set(::log(fParams[0]),fParams[1]);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::ExpAIDA::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::ExpAIDA(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::ExpAIDA::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::ExpAIDA";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::ExpAIDA::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="amplitude") return fParams[0];
  else if(aName=="exponent") return fParams[1];
  else {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpAIDA::parameter :"
        << " unknown parameter " << aName << "."
        << Lib::endl;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpAIDA::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=2) {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpAIDA::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.set(::log(fParams[0]),fParams[1]);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpAIDA::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::ExpAIDA::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  if(index==0) {
    fFunction.setA(::log(aValue));
  } else if(index==1) {
    fFunction.setB(aValue);
  }
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::ExpAIDA::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ExpAIDA::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Polynomial::Polynomial(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
,fFunction(std::vector<double>())
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Polynomial::Polynomial(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aParams
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aParams)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(aParams.size()); 
  unsigned int number = aParams.size(); //It is the degree.
  for(unsigned int i=0;i<number;i++) {
    std::string s;
    Lib::smanip::printf(s,32,"p%d",i);
    fParamNames[i] = s;
  }
  fParams = aParams;
  fVariableNames.push_back("x[0]");
  Lib::smanip::printf(fCodelet,9+32,"class:Polynomial:%d",number);
  //fCodelet = "class:Polynomial";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Polynomial::Polynomial(
 const Polynomial& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Polynomial::~Polynomial(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Polynomial::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Polynomial)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Polynomial)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Polynomial::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Polynomial(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Polynomial::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Polynomial";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Polynomial::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName.size()>=1) {
    if(aName[0]=='p'){
      std::string s = aName.substr(1,aName.size()-1);
      unsigned int index;
      if(Lib::smanip::touint(s,index)) {
        if(index<fParams.size()) return fParams[index];
      }
    }
  }
  Lib::Out out(printer());
  out << "BatchLab::Function::Polynomial::parameter :"
      << " unknown parameter " << aName << "."
      << Lib::endl;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Polynomial::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Polynomial::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  fFunction.setParameter(index,aValue);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Polynomial::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=fParams.size()) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Polynomial::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.setParameters(aParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Polynomial::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Polynomial::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Cauchy::Cauchy(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Cauchy::Cauchy(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,double aHeight
,double aCenter
,double aWidth
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aHeight,aCenter,aWidth)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.resize(3);
  fParamNames[0] = "amplitude"; //AIDA convention.
  fParamNames[1] = "origin";
  fParamNames[2] = "width";
  fParams.resize(3);
  fParams[0] = aHeight;
  fParams[1] = aCenter;
  fParams[2] = aWidth;
  fVariableNames.push_back("x[0]");
  fCodelet = "class:Cauchy";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Cauchy::Cauchy(
 const Cauchy& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Cauchy::~Cauchy(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Cauchy::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Cauchy)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Cauchy)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Cauchy::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Cauchy(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Cauchy::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Cauchy";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Cauchy::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="amplitude") return fParams[0];
  else if(aName=="origin") return fParams[1];
  else if(aName=="width") return fParams[2];
  else {
    Lib::Out out(printer());
    out << "BatchLab::Function::Cauchy::parameter :"
        << " unknown parameter " << aName << "."
        << Lib::endl;
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Cauchy::setParameters(
 const std::vector<double>& aParams
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Cauchy::setParameter :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Cauchy::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Cauchy::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  if(index==0) {
    fFunction.setHeight(aValue);
  } else if(index==1) {
    fFunction.setCenter(aValue);
  } else if(index==2) {
    fFunction.setWidth(aValue);
  }
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Cauchy::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Cauchy::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(aPoint.size()==0) {
    aValue = 0;
    return false;
  }
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Scripted::Scripted(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
,fFunction("x*0.",aPrinter)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Scripted::Scripted(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::string& aScript
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aScript,aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames = fFunction.parameterNames();
  fParams = std::vector<double>(fParamNames.size());
  fVariableNames = fFunction.variableNames();
  fCodelet = "scripted:" + aScript;
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Scripted::Scripted(
 const Scripted& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Scripted::~Scripted(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Scripted::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fFunction.dimension();
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::string serror;
  if(!fFunction.value(aPoint,aValue,serror)) {

    if(serror!=Lib_valuemanip_error_div_zero) {

      Lib::Out out(printer());
      out << "BatchLab::Function::Scripted::value :"
          << " get value failed for codelet" 
          << " " << Lib::sout(fCodelet) 
          << " at point";   
     {unsigned int number = aPoint.size();
      for(unsigned int i=0;i<number;i++) {
        out << " " << aPoint[i];
      }}
      out << Lib::endl;
      out << serror << Lib::endl;

    }
    return false;
  } 
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Scripted::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Scripted)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
SLASH_STORE_SOURCE(BatchLab::Function::Scripted)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Function::Scripted)
  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;
  if(!BaseFunction::visit(aVisitor)) return false;
  if(!aVisitor.end(*this)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aVisitor.begin(*this)) return false;
  int version;
  if(!aVisitor.visit(version)) return false;
  if(!BaseFunction::read(aVisitor)) return false;
  if(!aVisitor.end(*this)) return false;

  //fCodelet = "scripted:" + aScript;
  std::string::size_type pos = fCodelet.find(':');
  if(pos==std::string::npos) return false;
  // ab:cdef
  // 0123456
  std::string script = fCodelet.substr(pos+1,fCodelet.length()-(pos+1));
  fFunction.set(script);
  fFunction.setParameters(fParams);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Scripted::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Scripted(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Scripted::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Scripted";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Scripted::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Scripted::parameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return 0;
  }
  return fParams[index];
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Scripted::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=fParams.size()) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Scripted::setParameters :"
        << " parameter number mismatch. "
        << (unsigned int)fParams.size() << " expected but " 
        << (unsigned int)aParams.size() << " received."
        << Lib::endl;
    return false;
  }
  fFunction.setParameters(aParams);
  fParams = aParams;
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Scripted::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFunction.isValid(); 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Composite::Composite(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
,fFunction("",aPrinter)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Composite::Composite(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::string& aScript
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aScript,aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames = fFunction.parameterNames();
  fParams = std::vector<double>(fParamNames.size());
  fVariableNames = fFunction.variableNames();
  fCodelet = "composite:" + aScript;
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Composite::Composite(
 const Composite& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Composite::~Composite(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Composite::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fFunction.dimension();
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Composite::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  std::string serror;
  if(!fFunction.value(aPoint,aValue,serror)) {
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::Function::Composite::value :"
          << " get value failed for codelet" 
          << " " << Lib::sout(fCodelet) 
          << " at point";
     {unsigned int number = aPoint.size();
      for(unsigned int i=0;i<number;i++) {
        out << " " << aPoint[i];
      }}
      out << Lib::endl;
      out << serror << Lib::endl;
    }
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Composite::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Composite)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Composite)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Composite::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Composite(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Composite::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Composite";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Composite::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Composite::parameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return 0;
  }
  return fParams[index];
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Composite::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Composite::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  fFunction.setParameters(fParams);
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Composite::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=fParams.size()) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Composite::setParameters :"
        << " parameter number mismatch. "
        << (unsigned int)fParams.size() << " expected but " 
        << (unsigned int)aParams.size() << " received."
        << Lib::endl;
    return false;
  }
  fFunction.setParameters(aParams);
  fParams = aParams;
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Composite::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFunction.isValid(); 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Compiled::Compiled(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
,fFunction(0)
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Compiled::Compiled(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,Slash::Data::IFunction* aFunction
,const std::vector<Slash::Data::IArray*> aArrays //For OpenPAW/pafunc.
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(0)
//////////////////////////////////////////////////////////////////////////////
// WARNING : aFunction probably already managed by the Lib::FunctionManager.
// Used by BatchLab::BaseFunction* BatchLab::FFactory::create.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFunction->numberOfParameters()>aFunction->dimension()) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Compiled::Compiled :"
        << " for model " << Lib::sout(aFunction->model())
        << " in file " << Lib::sout(aFunction->file())
//        << " with signature " << Lib::sout(aFunction->signature())
        << ", dimension mismatch"
        << " parameters = " << aFunction->numberOfParameters() 
        << " > dimension = " << aFunction->dimension()
        << "."
        << Lib::endl;
    return;
  }

  // Have a local copy of aArrays.
 {fArrays.clear();
  std::vector<Slash::Data::IArray*>::const_iterator it;
  for(it=aArrays.begin();it!=aArrays.end();++it){
    Slash::Data::IArray* from = *it;    
    BatchLab::Array* to = new BatchLab::Array(printer(),"","",from->orders());
    to->copy(*from);
    fArrays.push_back(to);
  }}

  unsigned int parn = aFunction->numberOfParameters();
  unsigned int varn = aFunction->dimension()-parn;
  unsigned int an = fArrays.size();    
  if(an) varn -= an;

 {fVariableNames.resize(varn);
  for(unsigned int i=0;i<varn;i++) {
    std::string s;
    Lib::smanip::printf(s,32,"x[%d]",i);
    fVariableNames[i] = s;
  }}

 {fParamNames.resize(parn);
  fParams.resize(parn,0);
  for(unsigned int i=0;i<parn;i++) {
    std::string s;
    Lib::smanip::printf(s,32,"p%d",i);
    fParamNames[i] = s;
  }}

  // codelet syntax is : compiled:<model>;<file>
  // compiled:<file> alone is not sufficient since a 
  // source file may contain multiple functions !
  // For the second separator, we can't take ':', since
  // it is part of the file path syntax on Windows (drive separator).

  fFunction = aFunction;

  fModel = aFunction->model();
  fFile = aFunction->file();
  //fSignature = aFunction->signature();
  fCodelet = "compiled:" + fModel + ";" + fFile;

}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Compiled::Compiled(
 const Compiled& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
,fModel(aFrom.fModel)
,fFile(aFrom.fFile)
,fSignature(aFrom.fSignature)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {fArrays.clear();
  std::vector<Slash::Data::IArray*>::const_iterator it;
  for(it=aFrom.fArrays.begin();it!=aFrom.fArrays.end();++it){
    const Slash::Data::IArray* from = *it;    
    BatchLab::Array* to = new BatchLab::Array(printer(),"","",from->orders());
    to->copy(*from);
    fArrays.push_back(to);
  }}
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Compiled::~Compiled(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Data::IArray*>::iterator it;
  for(it=fArrays.begin();it!=fArrays.end();++it) delete *it;    
  fArrays.clear();
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::Function::Compiled::reset(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fParamNames.clear();
  fParams.clear();
  fVariableNames.clear();
  fCodelet = "";
  fModel = "";
  fFile = "";
  fSignature = "";
  fFunction = 0;
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Compiled::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fVariableNames.size();
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IFunction* BatchLab::Function::Compiled::slashFunction(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFunction;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Compiled::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(!fFunction) {
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::Function::Compiled::value :"
          << " not initailized."
          << Lib::endl;
    }
    aValue = 0;
    return false;
  }

  unsigned int xn = fVariableNames.size();
  unsigned int pn = fParams.size();
  if(aPoint.size()!=xn) {
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::Function::Compiled::value :"
          << " dimension mismatch."
          << " (" << (unsigned int)aPoint.size() << "," << xn << ")."
          << Lib::endl;
    }
    aValue = 0;
    return false;
  }

  unsigned int an = fArrays.size();    
  if(an) {
    // Handle OpenPAW/pafunc f77 function signature. (exa : urban/airclas.for).

    unsigned int argn = xn+pn+an;
    if(fFunction->dimension()!=argn) {
      Lib::Out out(printer());
      out << "BatchLab::Function::Compiled::value :"
          << " internal dimension mismatch."
          << " (" << fFunction->dimension() << "," << argn << ")."
          << Lib::endl;
      aValue = 0;
      return false;
    }

    Slash::Core::IValue* retsig;
    std::vector<Slash::Core::IValue*> funcSig = fFunction->signature(retsig);
    
    double double_dummy;
    float float_dummy;
    int int_dummy;

    std::vector<Slash::Core::IValue*> args;
    args.resize(argn,0);
    unsigned int argi = 0;
   {for(unsigned int i=0;i<xn;i++) {
      Slash::Core::IValue* argSig = funcSig[argi];
      if(argSig->type()==Slash::Core::IValue::INT_STAR) {
        int_dummy = (int)aPoint[i];
        args[argi] = new Lib::Value(&int_dummy);
      } else if(argSig->type()==Slash::Core::IValue::FLOAT_STAR) {
        float_dummy = (float)aPoint[i];
        args[argi] = new Lib::Value(&float_dummy);
      } else if(argSig->type()==Slash::Core::IValue::DOUBLE_STAR) {
        double_dummy = aPoint[i];
        args[argi] = new Lib::Value(&double_dummy);
      } else if(argSig->type()==Slash::Core::IValue::FLOAT) {
        args[argi] = new Lib::Value((float)aPoint[i]);
      } else if(argSig->type()==Slash::Core::IValue::DOUBLE) {
        args[argi] = new Lib::Value(aPoint[i]);
      } else {
        Lib::Out out(printer());
        out << "BatchLab::Function::Compiled::value :"
            << " function arg type " << Lib::sout(argSig->stype()) 
            << " not handled."
            << Lib::endl;
       {for(unsigned int i=0;i<args.size();i++) delete args[i];}
        aValue = 0;
        return false;
      }
      argi++;
    }}
   {for(unsigned int i=0;i<pn;i++) {
      Slash::Core::IValue* argSig = funcSig[argi];
      if(argSig->type()==Slash::Core::IValue::DOUBLE) {
        args[argi] = new Lib::Value(fParams[i]);
      } else {
        Lib::Out out(printer());
        out << "BatchLab::Function::Compiled::value :"
            << " function arg type " << Lib::sout(argSig->stype()) 
            << " not handled."
            << Lib::endl;
       {for(unsigned int i=0;i<args.size();i++) delete args[i];}
        aValue = 0;
        return false;
      }
      argi++;
    }}
   {for(unsigned int i=0;i<an;i++) {
      Slash::Core::IValue* argSig = funcSig[argi];
      if(argSig->type()==Slash::Core::IValue::DOUBLE_STAR) {

        std::vector<double>& va = fArrays[i]->vector();
        if(!va.size()) {
          Lib::Out out(printer());
          out << "BatchLab::Function::Compiled::value :"
              << " IArray as an empty vector<double>."
              << Lib::endl;
         {for(unsigned int i=0;i<args.size();i++) delete args[i];}
          aValue = 0;
          return false;
        }
        double* p = &(va[0]); //FIXME : safe ?
        args[argi] = new Lib::Value(p);

      } else {
        Lib::Out out(printer());
        out << "BatchLab::Function::Compiled::value :"
            << " function arg type " << Lib::sout(argSig->stype()) 
            << " not handled."
            << Lib::endl;
       {for(unsigned int i=0;i<args.size();i++) delete args[i];}
        aValue = 0;
        return false;
      }
      argi++;
    }}
    
    Lib::Value ret;
    if(!fFunction->value(args,ret)) {
      if(verboseLevel()) {
        Lib::Out out(printer());
        out << "BatchLab::Function::Compiled::value :"
            << " get value failed for codelet" 
            << " " << Lib::sout(fCodelet) 
            << " at point";
       {unsigned int number = aPoint.size();
        for(unsigned int i=0;i<number;i++) {
          out << " " << aPoint[i];
        }}
        out << Lib::endl;
      }
     {for(unsigned int i=0;i<args.size();i++) delete args[i];}
      return false;
    }
   {for(unsigned int i=0;i<args.size();i++) delete args[i];}

    if(ret.type()==Slash::Core::IValue::FLOAT) {
      aValue = ret.get_float();
    } else if(ret.type()==Slash::Core::IValue::DOUBLE) {
      aValue = ret.get_double();
    } else {
      Lib::Out out(printer());
      out << "BatchLab::Function::Compiled::value :"
          << " returned Lib::Value type " 
          << Lib::sout(ret.stype()) 
          << " not treated."
          << Lib::endl;
      return false;
    }

  } else {

    std::vector<double> args;
    args.resize(xn+pn,0);
    unsigned int argi = 0;
   {for(unsigned int i=0;i<xn;i++) {
      args[argi] = aPoint[i];
      argi++;
    }}
   {for(unsigned int i=0;i<pn;i++) {
      args[argi] = fParams[i];
      argi++;
    }}

    if(!fFunction->value(args,aValue)) {
      if(verboseLevel()) {
        Lib::Out out(printer());
        out << "BatchLab::Function::Compiled::value :"
            << " get value failed for codelet" 
            << " " << Lib::sout(fCodelet) 
            << " at point";
       {unsigned int number = aPoint.size();
        for(unsigned int i=0;i<number;i++) {
          out << " " << aPoint[i];
        }}
        out << Lib::endl;
      }
      return false;
    }
  }


//{Lib::Out out(printer());
// out << "BatchLab::Function::Compiled::value :"
//     << " at point";
// unsigned int number = aPoint.size();
// for(unsigned int i=0;i<number;i++) {
//   out << " " << aPoint[i];
// }
// out << " value : " << aValue;
// out << Lib::endl;}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Compiled::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Compiled)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
SLASH_STORE_SOURCE(BatchLab::Function::Compiled)
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Compiled::visit(
 Slash::Store::IConstVisitor& aVisitor
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SLASH_STORE_BEGIN(BatchLab::Function::Compiled)
  int version = 1;
  if(!aVisitor.visit("fVersion",version)) return false;
  if(!BaseFunction::visit(aVisitor)) return false;

  //printf("debug : BatchLab::Function::Compiled::visit : %ld\n",this);
  //printf("codelet : \"%s\"\n",fCodelet.c_str());
  //printf("model : \"%s\"\n",fModel.c_str());
  //printf("file : \"%s\"\n",fFile.c_str());
  //printf("signature : \"%s\"\n",fSignature.c_str());

  if(!aVisitor.visit("fModel",fModel)) return false;
  if(!aVisitor.visit("fFile",fFile)) return false;
  if(!aVisitor.visit("fSignature",fSignature)) return false;

  if(!aVisitor.end(*this)) return false;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Compiled::read(
 Slash::Store::IVisitor& aVisitor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  reset();

  if(!aVisitor.begin(*this)) return false;
  int version;
  if(!aVisitor.visit(version)) return false;
  if(!BaseFunction::read(aVisitor)) return false;

  if(!aVisitor.visit(fModel)) return false;
  if(!aVisitor.visit(fFile)) return false;
  if(!aVisitor.visit(fSignature)) return false;

  if(!aVisitor.end(*this)) return false;

  //printf("debug : BatchLab::Function::Compiled::read : %ld\n",this);
  //printf("codelet : \"%s\"\n",fCodelet.c_str());
  //printf("model : \"%s\"\n",fModel.c_str());
  //printf("file : \"%s\"\n",fFile.c_str());
  //printf("signature : \"%s\"\n",fSignature.c_str());

 //{unsigned int number = fParamNames.size();
 // printf("debug : compiled : %d\n",number);
 // for(unsigned int i=0;i<number;i++) {
 //   printf("debug : compiled : %d : %s %g\n",
 //     i,fParamNames[i].c_str(),fParams[i]);
 //}}

  Slash::Core::ISession& sess = aVisitor.session();

  BatchLab::FunctionCatalog catalog(sess);
  Slash::Data::IFunction* func = catalog.find(fFile,fModel);
  if(!func) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Compiled::read :"
        << " Problem creating " << Lib::sout(fModel)
        << Lib::endl;
    reset();
    return true; //Do not stop the IO.
  }

  //fFunction.setParameters(fParams);

  fFunction = func;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Compiled::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Compiled(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Compiled::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Compiled";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Compiled::parameter(
 const std::string& aName
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName.size()>=1) {
    if(aName[0]=='p'){
      std::string s = aName.substr(1,aName.size()-1);
      unsigned int index;
      if(Lib::smanip::touint(s,index)) {
        if(index<fParams.size()) return fParams[index];
      }
    }
  }
  Lib::Out out(printer());
  out << "BatchLab::Function::Compiled::parameter :"
      << " unknown parameter " << aName << "."
      << Lib::endl;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Compiled::setParameter(
 const std::string& aName
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  int index = indexOfParameter(aName);
  if(index==NotFound) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Compiled::setParameter :"
        << " parameter name " << Lib::sout(aName) << " not found."
        << Lib::endl;
    return false;
  }
  fParams[index] = aValue;
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Compiled::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=fParams.size()) {
    Lib::Out out(printer());
    out << "BatchLab::Function::Compiled::setParameters :"
        << " bad input vector size."
        << Lib::endl;
    return false;
  }
  fParams = aParams;
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array::Array(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array::Array(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aXs
,const std::vector<double>& aVs
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aXs,aVs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fCodelet = "class:Array";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array::Array(
 const Array& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array::Array(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fCodelet = "class:Array";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array::~Array(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Array::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Array::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Array)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Array)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Array::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Array(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Array::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Array";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Array::parameter(
 const std::string&
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Array::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFunction.setParameters(aParams)) return false;
  //FIXME : should set fParams and fParamNames
  fParams.clear();
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& BatchLab::Function::Array::parameters(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(BatchLab::Function::Array);
  const std::vector<double>& Xs = fFunction.xs();
  const std::vector<double>& Vs = fFunction.vs();
  unsigned int nx = Xs.size();
  self.fParams.resize(1+1+nx+nx);
  int pari = 0;
  self.fParams[pari] = 1;
  pari++;
  self.fParams[pari] = nx;
  pari++;
  unsigned int index;
  for(index=0;index<nx;index++) {
    self.fParams[pari] = Xs[index];
    pari++;
  }
  for(index=0;index<nx;index++) {
    self.fParams[pari] = Vs[index];
    pari++;
  }
  return fParams;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Array::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  aValue = fFunction.value(aPoint[0]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array2D::Array2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array2D::Array2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aXs
,const std::vector<double>& aYs
,const std::vector<double>& aVs
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aXs,aYs,aVs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fVariableNames.push_back("x[1]");
  fCodelet = "class:Array2D";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array2D::Array2D(
 const Array2D& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array2D::Array2D(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fVariableNames.push_back("x[1]");
  fCodelet = "class:Array2D";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::Array2D::~Array2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::Array2D::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 2;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::Array2D::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::Array2D)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::Array2D)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::Array2D::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::Array2D(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::Array2D::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::Array2D";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::Array2D::parameter(
 const std::string&
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Array2D::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFunction.setParameters(aParams)) return false;
  //FIXME : should set fParams and fParamNames
  fParams.clear();
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& BatchLab::Function::Array2D::parameters(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(BatchLab::Function::Array2D);
  const std::vector<double>& Xs = fFunction.xs();
  const std::vector<double>& Ys = fFunction.ys();
  const std::vector<double>& Vs = fFunction.vs();
  unsigned int nx = Xs.size();
  unsigned int ny = Ys.size();
  unsigned int nv = nx * ny;
  self.fParams.resize(1+1+1+nx+ny+nv);
  int pari = 0;
  self.fParams[pari] = 2;
  pari++;
  self.fParams[pari] = nx;
  pari++;
  self.fParams[pari] = ny;
  pari++;
  unsigned int index;
  for(index=0;index<nx;index++) {
    self.fParams[pari] = Xs[index];
    pari++;
  }
  for(index=0;index<ny;index++) {
    self.fParams[pari] = Ys[index];
    pari++;
  }
  for(index=0;index<nv;index++) {
    self.fParams[pari] = Vs[index];
    pari++;
  }
  return fParams;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::Array2D::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  aValue = fFunction.value(aPoint[0],aPoint[1]);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::ArrayComplex(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
)
:BatchLab::BaseFunction(aPrinter,aName,"")
//////////////////////////////////////////////////////////////////////////////
// Empty object used by IO readers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::ArrayComplex(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fCodelet = "class:ArrayComplex";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::ArrayComplex(
 const ArrayComplex& aFrom
)
:AIDA::IManagedObject(aFrom)
,Slash::Store::IObject(aFrom)
,AIDA::IFunction(aFrom)
,Slash::Store::IStorable(aFrom)
,Slash::Meta::IObject(aFrom)
,BaseFunction(aFrom)
,fFunction(aFrom.fFunction)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::ArrayComplex(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aXs
,const std::vector< std::complex<double> >& aVs
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aXs,aVs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fCodelet = "class:ArrayComplex";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::ArrayComplex(
 Slash::Core::IWriter& aPrinter
,const std::string& aName
,const std::string& aTitle
,const std::vector<double>& aXs
,const std::vector<double>& aRs
,const std::vector<double>& aIs
)
:BatchLab::BaseFunction(aPrinter,aName,aTitle)
,fFunction(aXs,aRs,aIs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVariableNames.push_back("x[0]");
  fCodelet = "class:ArrayComplex";
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Function::ArrayComplex::~ArrayComplex(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
int BatchLab::Function::ArrayComplex::dimension(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Function::ArrayComplex::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::Function::ArrayComplex)
  else if_Lib_SCast(Slash::Meta::IObject)
  else return BaseFunction::cast(aClass);
}
VISIT_SOURCE(BatchLab::Function::ArrayComplex)
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::Function::ArrayComplex::copy(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return new BatchLab::Function::ArrayComplex(*this);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::Function::ArrayComplex::storeClassName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::Function::ArrayComplex";
}
//////////////////////////////////////////////////////////////////////////////
double BatchLab::Function::ArrayComplex::parameter(
 const std::string&
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ArrayComplex::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFunction.setParameters(aParams)) return false;
  //FIXME : should set fParams and fParamNames
  fParams.clear();
  emitUpdate();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& BatchLab::Function::ArrayComplex::parameters(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(BatchLab::Function::ArrayComplex);
  const std::vector<double>& Xs = fFunction.xs();
  const std::vector< std::complex<double> >& Vs = fFunction.vs();
  unsigned int nx = Xs.size();
  self.fParams.resize(1+1+nx+nx+nx);
  int pari = 0;
  self.fParams[pari] = 1;
  pari++;
  self.fParams[pari] = nx;
  pari++;
  unsigned int index;
  for(index=0;index<nx;index++) {
    self.fParams[pari] = Xs[index];
    pari++;
  }
  for(index=0;index<nx;index++) {
    self.fParams[pari] = Vs[index].real();
    pari++;
    self.fParams[pari] = Vs[index].imag();
    pari++;
  }
  return fParams;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Function::ArrayComplex::value(
 const std::vector<double>& aPoint
,double& aValue
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  aValue = fFunction.value(aPoint[0]);
  return true;
}
