// this :
#include <BatchLab/Core/MidnightFitter.h>

// Slash :
#include <Slash/Core/IWriter.h>

// Lib :
#include <Lib/Cast.h>
#include <Lib/Out.h>
#include <Lib/Debug.h>

// AIDA :
#include <AIDA/IFunction.h>

// Midnight :
#include <Midnight/Fitter.h>

//#include <math.h>

namespace BatchLab {

class FCN {
public:
  virtual double operator()(const std::vector<double>& aParams) const {
    Lib_self(FCN);
    self.fChi2 = 0;
    unsigned int npt = fXs.size();
    fFunction.setParameters(aParams);
 //{printf("debug : param number : %d\n",aParams.size());
 // for(unsigned int i=0;i<aParams.size();i++) {
 // printf("debug :    %d %g\n",i,aParams[i]);
 //}}
    for(unsigned int i=0;i<npt;i++) {
      self.fX[0] = fXs[i];
      double value = fFunction.value(fX);
      double chinew = (fVs[i] - value)/fEs[i];
      self.fChi2 += (chinew * chinew);
    }
    return self.fChi2;
  }
  virtual double chi2() const { return fChi2;}
public:
  FCN(AIDA::IFunction& aFunction,const std::vector<double>& aXs,const std::vector<double>& aVs,const std::vector<double>& aEs)
  :fFunction(aFunction),fXs(aXs),fVs(aVs),fEs(aEs),fChi2(0){
    fX.resize(1,0);
  }
  virtual ~FCN() {}
private:
  AIDA::IFunction& fFunction;
  const std::vector<double>& fXs;
  const std::vector<double>& fVs;
  const std::vector<double>& fEs;
  std::vector<double> fX;
  double fChi2;
};
}

//////////////////////////////////////////////////////////////////////////////
BatchLab::MidnightFitter::MidnightFitter(
 Slash::Core::IWriter& aPrinter
,AIDA::IFunctionFactory& aFunctionFactory
)
:BaseFitter(aPrinter,aFunctionFactory,"midnight")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("BatchLab::MidnightFitter");
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::MidnightFitter::~MidnightFitter(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("BatchLab::MidnightFitter");
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::MidnightFitter::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::MidnightFitter)
  else return BaseFitter::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::MidnightFitter::baseFit(
 AIDA::IFunction& aFunction
,const std::vector<double>& aXs
,const std::vector<double>& aVs
,const std::vector<double>& aEs
,double& aChi2
,std::vector<double>& aErrs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aChi2 = 0;
  aErrs.clear();
  const std::vector<double>& fparams = aFunction.parameters();
  if(fparams.size()<=0) {
    Lib::Out out(fPrinter);
    out << "BatchLab::MidnightFitter::baseFit :"
        << " given function has no parameters." << Lib::endl;
    return false;
  }
  std::vector<std::string> names = aFunction.parameterNames();
  if(names.size()!=fparams.size()) {
    Lib::Out out(fPrinter);
    out << "BatchLab::MidnightFitter::baseFit :"
        << " bad given function parameter names size." << Lib::endl;
    return false;
  }

  FCN fcn(aFunction,aXs,aVs,aEs);

  Midnight::Fitter minuit(0);
  minuit.SetFCN(function,&fcn);
  minuit.SetPrinter(&fPrinter);

  int status;
  for(unsigned int index=0;index<names.size();index++) {
    double start = fparams[index];
    AIDA::IFitParameterSettings& ps = fitParameterSettings(names[index]);
    double step = ps.stepSize();
    //if(step==0) step = 0.01;
    minuit.mnparm(index,
		  names[index].c_str(),
		  start,
		  step,
		  0,0,status);
  }

  double args[10];
  args[0] = 0;
  minuit.mnexcm("SET PRI",args,1,status);
  minuit.mnexcm("MIGRAD",args,0,status);
  //minuit.mnexcm("MINOS",args,0,status);
  
  // Output.
  aChi2 = fcn.chi2();

 {aErrs.resize(names.size());
  for(unsigned int index=0;index<names.size();index++) {
    std::string s;
    double value,err;
    double bmin,bmax;
    minuit.mnpout(index,s,
		  value,err,
		  bmin,bmax,status);
    aErrs[index] = err;
  }}

  return true;  
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::MidnightFitter::function(
 int& aParamn
,double* //aGrad
,double& aValue
,double* aParams
,int //aStatus
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BatchLab::FCN* FCN = (BatchLab::FCN*)aTag;
  std::vector<double> params;
  params.resize(aParamn,0);
  for(int index=0;index<aParamn;index++) params[index] = aParams[index];
  aValue = (*FCN)(params);
}

