//this :
#include <BatchLab/Core/amanip.h>

#include <AIDA/IConstants.h>
#include <AIDA/ITuple.h>
#include <AIDA/IFilter.h>
#include <AIDA/IEvaluator.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IAxis.h>

#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/Cast.h>
#include <Lib/smanip.h>
#include <Lib/sjust.h>
#include <Lib/sout.h>

#include <BatchLab/Core/BaseTuple.h> //FIXME
#include <BatchLab/Core/Histogram.h> //FIXME

typedef unsigned char byte;
#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::is_column_type(
 const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string type = aString;
  Lib::smanip::tolowercase(type);
  if( (type=="float") ||
      (type=="double") ||
      (type=="char") ||
      (type=="short") ||
      (type=="int") ||
      (type=="long") ||
      (type=="boolean") ||
      (type=="string") ||
      (type=="ITuple") ) return true;
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::compatible_tuples(
 Slash::Core::IWriter& aPrinter
,const AIDA::ITuple& a1
,const AIDA::ITuple& a2
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(a2.columns()!=a1.columns()) {
    Lib::Out out(aPrinter);
    out << "BatchLab::amanip::comaptible_tuples :"
        << " booking mismatch :"
        << " columns " << a2.columns() 
        << " (" << a1.columns() << " expected)." 
        << Lib::endl;
    return false;
  } else {
    int coln = a1.columns();
    for(int coli=0;coli<coln;coli++) {
      if(a2.columnName(coli)!=a1.columnName(coli)) {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::comaptible_tuples :"
            << " booking mismatch :"
            << " column name " << a2.columnName(coli) 
            << " (" << a1.columnName(coli) << " expected)." 
            << Lib::endl;
        return false;
      } else if(a2.columnType(coli)!=a1.columnType(coli)) {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::comaptible_tuples :"
            << " booking mismatch :"
            << " column type " << a2.columnType(coli) 
            << " (" << a1.columnType(coli) << " expected)." 
            << Lib::endl;
        return false;
      }
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::copy_tuple(
 Slash::Core::IWriter& aPrinter
,AIDA::ITuple& aFrom
,AIDA::ITuple& aTo
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!compatible_tuples(aPrinter,aFrom,aTo)) return false;  
  int coln = aFrom.columns();
  if(!coln) return true;
  aFrom.start();
  while(aFrom.next()) {
    for(int index=0;index<coln;index++) {
      std::string type = aFrom.columnType(index);
      if(type=="float") {
        if(!aTo.fill(index,aFrom.getFloat(index))) return false;
      } else if(type=="double") {
        if(!aTo.fill(index,aFrom.getDouble(index))) return false;
      } else if(type=="char") {
        if(!aTo.fill(index,aFrom.getChar(index))) return false;
      } else if(type=="short") {
        if(!aTo.fill(index,aFrom.getShort(index))) return false;
      } else if(type=="int") {
        if(!aTo.fill(index,aFrom.getInt(index))) return false;
      } else if(type=="long") {
        if(!aTo.fill(index,aFrom.getLong(index))) return false;
      } else if(type=="boolean") {
        if(!aTo.fill(index,aFrom.getBoolean(index))) return false;
      } else if(type=="string") {
        if(!aTo.fill(index,aFrom.getString(index))) return false;
      } else if(type=="ITuple") {
        if(!aTo.fill(index,*aFrom.getObject(index))) return false;
      } else {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::copy_tuple :" 
            << " column type " << type << " not handled."
            << Lib::endl;
        return false;
      }
    }
    aTo.addRow();
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::copy_tuple_filtered(
 Slash::Core::IWriter& aPrinter
,AIDA::ITuple& aFrom
,AIDA::ITuple& aTo
,AIDA::IFilter& aFilter
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!compatible_tuples(aPrinter,aFrom,aTo)) return false;  
  if(!aFilter.initialize(aFrom)) {
    Lib::Out out(aPrinter);
    out << "BatchLab::amanip::copy_tuple_filtered :"
        << " Filter initialization failed for tuple " 
        << Lib::sout(aFrom.title())
        << Lib::endl;
    return false;
  }
  int coln = aFrom.columns();
  if(!coln) return true;
  aFrom.start();
  while(aFrom.next()) {
    if(!aFilter.accept()) continue;
    for(int index=0;index<coln;index++) {
      std::string type = aFrom.columnType(index);
      if(type=="float") {
        if(!aTo.fill(index,aFrom.getFloat(index))) return false;
      } else if(type=="double") {
        if(!aTo.fill(index,aFrom.getDouble(index))) return false;
      } else if(type=="char") {
        if(!aTo.fill(index,aFrom.getChar(index))) return false;
      } else if(type=="short") {
        if(!aTo.fill(index,aFrom.getShort(index))) return false;
      } else if(type=="int") {
        if(!aTo.fill(index,aFrom.getInt(index))) return false;
      } else if(type=="long") {
        if(!aTo.fill(index,aFrom.getLong(index))) return false;
      } else if(type=="boolean") {
        if(!aTo.fill(index,aFrom.getBoolean(index))) return false;
      } else if(type=="string") {
        if(!aTo.fill(index,aFrom.getString(index))) return false;
      } else if(type=="ITuple") {
        if(!aTo.fill(index,*aFrom.getObject(index))) return false;
      } else {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::copy_tuple_filtered :" 
            << " column type " << type << " not handled."
            << Lib::endl;
        return false;
      }
    }
    aTo.addRow();
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::tuple_getDouble(
 Slash::Core::IWriter& aPrinter
,const AIDA::ITuple& aTuple
,int aIndex
,double& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string type = aTuple.columnType(aIndex);
  if(type=="float") {
    aValue = aTuple.getFloat(aIndex);
  } else if(type=="double") {
    aValue = aTuple.getDouble(aIndex);
  } else if(type=="char") {
    aValue = aTuple.getChar(aIndex);
  } else if(type=="byte") {
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
    const BaseTuple* baseTuple = Lib_ConstCast(aTuple,BatchLab::BaseTuple);
    if(!baseTuple) {
      Lib::Out out(aPrinter);
      out << "BatchLab::amanip::tuple_getDouble : "
          << " tuple not a BatchLab::BaseTuple."
          << Lib::endl;
      aValue = 0;
      return false;
    }
    aValue = baseTuple->getByte(aIndex);
#else
    aValue = aTuple.getByte(aIndex);
#endif
  } else if(type=="short") {
    aValue = aTuple.getShort(aIndex);
  } else if(type=="int") {
    aValue = aTuple.getInt(aIndex);
  } else if(type=="long") {
    aValue = aTuple.getLong(aIndex);
  } else if(type=="boolean") {
    aValue = aTuple.getBoolean(aIndex);
  } else {
    aValue = 0;
    Lib::Out out(aPrinter);
    out << "BatchLab::amanip::tuple_getDouble : "
        << " make no sense on column type " << type << "."
        << Lib::endl;
    return false;
  } 
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::tuple_getString(
 Slash::Core::IWriter& aPrinter
,const AIDA::ITuple& aTuple
,int aIndex
,std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string type = aTuple.columnType(aIndex);
  if(type=="float") {
    aValue = d2s(aTuple.getFloat(aIndex));
  } else if(type=="double") {
    aValue = d2s(aTuple.getDouble(aIndex));
  } else if(type=="char") {
    aValue = Lib::smanip::tostring(aTuple.getChar(aIndex));
  } else if(type=="byte") {
    const BaseTuple* baseTuple = Lib_ConstCast(aTuple,BatchLab::BaseTuple);
    if(!baseTuple) {
      Lib::Out out(aPrinter);
      out << "BatchLab::amanip::tuple_getString : "
          << " tuple not a BatchLab::BaseTuple."
          << Lib::endl;
      aValue = "";
      return false;
    }
    aValue = Lib::smanip::tostring((char)baseTuple->getByte(aIndex));//FIXME
  } else if(type=="short") {
    aValue = Lib::smanip::tostring(aTuple.getShort(aIndex));
  } else if(type=="int") {
    aValue = Lib::smanip::tostring(aTuple.getInt(aIndex));
  } else if(type=="long") {
    aValue = Lib::smanip::tostring((Lib::int64)aTuple.getLong(aIndex));
  } else if(type=="boolean") {
    aValue = Lib::smanip::tostring(aTuple.getBoolean(aIndex));
  } else if(type=="string") {
    aValue = aTuple.getString(aIndex);
  } else {
    aValue = "";
    Lib::Out out(aPrinter);
    out << "BatchLab::amanip::tuple_getString : "
        << " make no sense on column type " << type << "."
        << Lib::endl;
    return false;
  } 
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::tuple_fill(
 Slash::Core::IWriter& aPrinter
,AIDA::ITuple& aTuple
,int aIndex
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string type = aTuple.columnType(aIndex);
  if(type=="float") {
    double val;
    if(!Lib::smanip::todouble(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a double."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,(float)val);
  } else if(type=="double") {
    double val;
    if(!Lib::smanip::todouble(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a double."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,val);
  } else if(type=="char") {
    if(!aValue.size()) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " empty string."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,(char)aValue[0]);
  } else if(type=="byte") {
    BaseTuple* baseTuple = Lib_Cast(aTuple,BatchLab::BaseTuple);
    if(!baseTuple) {
      Lib::Out out(aPrinter);
      out << "BatchLab::amanip::tuple_fill : "
          << " tuple not a BatchLab::BaseTuple."
          << Lib::endl;
      return false;
    }
    int val;
    if(!Lib::smanip::toint(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a int."
          << Lib::endl;
      return false;
    }
    return baseTuple->fill(aIndex,(byte)val); //FIXME
  } else if(type=="short") {
    int val;
    if(!Lib::smanip::toint(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a int."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,(short)val);
  } else if(type=="int") {
    int val;
    if(!Lib::smanip::toint(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a int."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,(int)val);
  } else if(type=="long") {
    Lib::int64 val;
    if(!Lib::smanip::toint64(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a int64."
          << Lib::endl;
      return false;
    }
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
    return aTuple.fill(aIndex,(long)val);
#else
    return aTuple.fill(aIndex,(AIDA::int64)val);
#endif
  } else if(type=="boolean") {
    bool val;
    if(!Lib::smanip::tobool(aValue,val)) {
      Lib::Out out(aPrinter);
       out << "BatchLab::amanip::tuple_fill : "
          << " can't convert " << Lib::sout(aValue) << " to a bool."
          << Lib::endl;
      return false;
    }
    return aTuple.fill(aIndex,val);
  } else if(type=="string") {
    return aTuple.fill(aIndex,aValue);
  } else {
    Lib::Out out(aPrinter);
    out << "BatchLab::amanip::tuple_fill : "
        << " make no sense on column type " << type << "."
        << Lib::endl;
    return false;
  } 
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::amanip::tuple_name(
 const AIDA::ITuple& aTuple
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const AIDA::IManagedObject* mobject = 
    Lib_ConstCast(aTuple,AIDA::IManagedObject);
  if(!mobject) return "";
  return mobject->name();
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::tuple_evaluateMinMax(
 Slash::Core::IWriter& aPrinter
,AIDA::ITuple& aTuple
,AIDA::IEvaluator& aEvaluator
,AIDA::IFilter& aFilter
,double& aMin
,double& aMax
,std::vector<double>* aVector
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = 0;
  aMax = 0;
  if(aVector) aVector->clear();

  if(aTuple.rows()<=0) {
    Lib::Out out(aPrinter);
    out << "tuple_evaluateMin :"
        << " Tuple " << Lib::sout(tuple_name(aTuple)) << " empty." 
        << Lib::endl;
    return false;
  }

  if(!aEvaluator.initialize(aTuple)) {
    Lib::Out out(aPrinter);
    out << "tuple_evaluateMin :"
        << " Evaluator initialization failed for tuple " 
        << Lib::sout(aTuple.title())
        << Lib::endl;
    return false;
  }

  if(!aFilter.initialize(aTuple)) {
    Lib::Out out(aPrinter);
    out << "tuple_evaluateMin :"
        << " Filter initialization failed for tuple " 
        << Lib::sout(aTuple.title())
        << Lib::endl;
    return false;
  }

  bool first = true;
  aTuple.start();
  while(aTuple.next()) {
    if(!aFilter.accept()) continue;
    double value = aEvaluator.evaluateDouble();
    if(first) {
      aMin = value;
      aMax = value;
      first = false;
    } else {
      aMin = MINIMUM(value,aMin);
      aMax = MAXIMUM(value,aMax);
    }
    if(aVector) aVector->push_back(value);
  }

/*
  if(first) {
    Lib::Out out(aPrinter);
    out << "tuple_evaluateMin :"
        << " nothing selected." 
        << Lib::endl;
    return false;
  }
*/

  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IHistogram1D& aH1D
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
// A la HPRINT.
//
// From cern/src/packlib/hbook/code/hprst.F :
// 3000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//    +'* R . M . S =',E11.4)
// 4000 FORMAT(' * ENTRIES =',I11,6X,'* ALL CHANNELS =',E11.4,6X,
//     +'* UNDERFLOW =',E11.4,6X,'* OVERFLOW =',E11.4,/,
//     +' * BIN WID =',E11.4,6X,'* MEAN VALUE   =',E11.4,6X,
//     +'* R . M . S =',E11.4,6X,'* NEQUIVAL =',E11.4)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aH1D.allEntries()) {

    out << "****** AIDA   NO ENTRIES FOR HISTOGRAM = " << aName
        << Lib::endl;

  } else {

    out << aH1D.dimension() << "D " << aH1D.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << Lib::sjust(aH1D.allEntries(),field)
        << " * ALL CHANNELS = " << Lib::sjust(aH1D.sumBinHeights(),field)
        << " * UNDERFLOW = " << Lib::sjust(aH1D.binHeight(AIDA::IAxis::UNDERFLOW_BIN),field)
        << " * OVERFLOW = " << Lib::sjust(aH1D.binHeight(AIDA::IAxis::OVERFLOW_BIN),field)
        << Lib::endl;
    out << " * BIN WID = " << Lib::sjust(aH1D.axis().binWidth(0),field)
        << " * MEAN VALUE   = " << Lib::sjust(aH1D.mean(),field)
        << " * R . M . S = " << Lib::sjust(aH1D.rms(),field)
        << Lib::endl;
    out << " * MIN = " << Lib::sjust(aH1D.minBinHeight(),field+4)
        << " * MAX = " << Lib::sjust(aH1D.maxBinHeight(),field)
        << Lib::endl;
      
  }
}

#include <AIDA/IManagedObject.h>

#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IHistogram3D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IProfile2D.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>
#include <AIDA/IFunction.h>
#include <AIDA/IDataPointSet.h>
#include <AIDA/ITuple.h>
#include <Slash/Data/IArray.h>
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::amanip::managed_object_type(
 const AIDA::IManagedObject& aObject
) 
//////////////////////////////////////////////////////////////////////////////
// FIXME : AIDA-3.3 : should use aObject.type()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(Lib_ConstCast(aObject,AIDA::IHistogram1D)) {
    return "AIDA::IHistogram1D";
  } else if(Lib_ConstCast(aObject,AIDA::IHistogram2D)) {
    return "AIDA::IHistogram2D";
  } else if(Lib_ConstCast(aObject,AIDA::IHistogram3D)) {
    return "AIDA::IHistogram3D";
  } else if(Lib_ConstCast(aObject,AIDA::IProfile1D)) {
    return "AIDA::IProfile1D";
  } else if(Lib_ConstCast(aObject,AIDA::IProfile2D)) {
    return "AIDA::IProfile2D";
  } else if(Lib_ConstCast(aObject,AIDA::ICloud1D)) {
    return "AIDA::ICloud1D";
  } else if(Lib_ConstCast(aObject,AIDA::ICloud2D)) {
    return "AIDA::ICloud2D";
  } else if(Lib_ConstCast(aObject,AIDA::ICloud3D)) {
    return "AIDA::ICloud3D";
  } else if(Lib_ConstCast(aObject,AIDA::IDataPointSet)) {
    return "AIDA::IDataPointSet";
  } else if(Lib_ConstCast(aObject,AIDA::IFunction)) {
    return "AIDA::IFunction";
  } else if(Lib_ConstCast(aObject,AIDA::ITuple)) {
    return "AIDA::ITuple";
  } else if(Lib_ConstCast(aObject,Slash::Data::IArray)) {
    return "Slash::Data::IArray";
  } else {
    return "unknown";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::amanip::managed_object_title(
 const AIDA::IManagedObject& aObject
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(const AIDA::IBaseHistogram* h = 
       Lib_ConstCast(aObject,AIDA::IBaseHistogram)) {
    return h->title();
  } else if(const AIDA::IDataPointSet* dps = 
              Lib_ConstCast(aObject,AIDA::IDataPointSet)) {
    return dps->title();
  } else if(const AIDA::IFunction* f = 
              Lib_ConstCast(aObject,AIDA::IFunction)) {
    return f->title();
  } else if(const AIDA::ITuple* tuple = Lib_ConstCast(aObject,AIDA::ITuple)) {
    return tuple->title();
  } else if(const Slash::Data::IArray* v = 
              Lib_ConstCast(aObject,Slash::Data::IArray)) {
    return v->title();
  } else {
    return "";
  }
}
//FIXME : we are lacking a ITuple::columnDefault.
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::tuple_getBooking(
 Slash::Core::IWriter& aPrinter
,const AIDA::ITuple& aTuple
,std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
// aString should be initialized before call.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aString += "{";
  int coln = aTuple.columns();
  for(int index=0;index<coln;index++) {
    if(index) aString += ",";
    std::string type = aTuple.columnType(index);
    aString += type + " ";
    std::string name = aTuple.columnName(index);
    aString += name + " = ";
    if(type=="ITuple") {
      const AIDA::ITuple* tupleTemplate = aTuple.getTuple(index);
      if(!tupleTemplate) {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::tuple_getBooking : "
            << " tuple column without a template tuple."
            << Lib::endl;
        aString = "";
        return false;
      }
      const BaseTuple* baseTuple = 
        Lib_ConstCast(*tupleTemplate,BatchLab::BaseTuple);
      if(!baseTuple) {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::tuple_getBooking : "
            << " tuple column not a BatchLab::BaseTuple."
            << Lib::endl;
        aString = "";
        return false;
      }
      if(!tuple_getBooking(aPrinter,*baseTuple,aString)) {
        aString = "";
        return false;
      }
    } else {
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
      const BaseTuple* baseTuple = Lib_ConstCast(aTuple,BatchLab::BaseTuple);
      if(!baseTuple) {
        Lib::Out out(aPrinter);
        out << "BatchLab::amanip::tuple_getBooking : "
            << " tuple not a BatchLab::BaseTuple."
            << Lib::endl;
        aString = "";
        return false;
      }
      std::string def = baseTuple->columnDefaultValue(index);
      aString += def;
#else
      std::string def = aTuple.columnDefaultValue(index);
      aString += def;
#endif
    }  
       
  }  
  aString += "}";
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::ITuple& aTuple
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  out << " ********************************************************" 
      << Lib::endl;

  out << " * NTUPLE NAME= " << aName
      << " ENTRIES= " << aTuple.rows()
      << " TITLE= " << aTuple.title()
      << Lib::endl;

  out << " ********************************************************" 
      << Lib::endl;
  out << " *  Var numb  *   Name    *    Lower     *    Upper     *" 
      << Lib::endl;
  out << " ********************************************************" 
      << Lib::endl;

    
  int coln = aTuple.columns();
  for(int coli=0;coli<coln;coli++) {
    out << " *" 
        << Lib::sjust(Lib::smanip::tostring(coli+1),12,Lib::smanip::middle) 
        << "* " 
        << Lib::sjust(aTuple.columnName(coli),10) 
        << "* " 
        << Lib::sjust(Lib::smanip::tostring(aTuple.columnMin(coli)),13) 
        << "* " 
        << Lib::sjust(Lib::smanip::tostring(aTuple.columnMax(coli)),13) 
        << "*" 
        << Lib::endl;
  }

  out << " ********************************************************" 
      << Lib::endl;

/*

 ********************************************************
 * NTUPLE ID=   10  ENTRIES=   3354   CERN Population 
 ********************************************************
 *  Var numb  *   Name    *    Lower     *    Upper     *
 ********************************************************
 *      1     * CATEGORY  * 0.102000E+03 * 0.567000E+03 *
 *      2     * DIVISION  * 0.100000E+01 * 0.130000E+02 *
 *      3     * FLAG      * 0.000000E+00 * 0.310000E+02 *
 *      4     * AGE       * 0.210000E+02 * 0.640000E+02 *
 *      5     * SERVICE   * 0.000000E+00 * 0.350000E+02 *
 *      6     * CHILDREN  * 0.000000E+00 * 0.600000E+01 *
 *      7     * GRADE     * 0.300000E+01 * 0.140000E+02 *
 *      8     * STEP      * 0.000000E+00 * 0.150000E+02 *
 *      9     * NATION    * 0.100000E+01 * 0.150000E+02 *
 *     10     * HRWEEK    * 0.200000E+01 * 0.440000E+02 *
 *     11     * COST      * 0.686000E+03 * 0.188530E+05 *
 ********************************************************

*/

}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IProfile1D& aProfile
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aProfile.allEntries()) {

    out << "****** AIDA   NO ENTRIES FOR PROFILE = " << aName
        << Lib::endl;

  } else {

    out << aProfile.dimension() << "D " << aProfile.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << Lib::sjust(aProfile.allEntries(),field)
        << " * ALL CHANNELS = " << Lib::sjust(aProfile.sumBinHeights(),field)
        << " * UNDERFLOW = " << Lib::sjust(aProfile.binHeight(AIDA::IAxis::UNDERFLOW_BIN),field)
        << " * OVERFLOW = " << Lib::sjust(aProfile.binHeight(AIDA::IAxis::OVERFLOW_BIN),field)
        << Lib::endl;
    out << " * BIN WID = " << Lib::sjust(aProfile.axis().binWidth(0),field)
        << " * MEAN VALUE   = " << Lib::sjust(aProfile.mean(),field)
        << " * R . M . S = " << Lib::sjust(aProfile.rms(),field)
        << Lib::endl;
    out << " * MIN = " << Lib::sjust(aProfile.minBinHeight(),field+4)
        << " * MAX = " << Lib::sjust(aProfile.maxBinHeight(),field)
        << Lib::endl;
      
  }

}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IHistogram2D& aHisto
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aHisto.allEntries()) {

    out << "****** AIDA   NO ENTRIES FOR HISTOGRAM = " << aName
        << Lib::endl;

  } else {

    out << aHisto.dimension() << "D " << aHisto.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << aHisto.allEntries() << Lib::endl;

    //   6 | 7 | 8
    //  -----------
    //   3 | 4 | 5
    //  -----------
    //   0 | 1 | 2

    double height_0 = aHisto.binHeight(AIDA::IAxis::UNDERFLOW_BIN,
                                   AIDA::IAxis::UNDERFLOW_BIN);
    double height_2 = aHisto.binHeight(AIDA::IAxis::OVERFLOW_BIN,
                                   AIDA::IAxis::UNDERFLOW_BIN);
    double height_6 = aHisto.binHeight(AIDA::IAxis::UNDERFLOW_BIN,
                                   AIDA::IAxis::OVERFLOW_BIN);
    double height_8 = aHisto.binHeight(AIDA::IAxis::OVERFLOW_BIN,
                                   AIDA::IAxis::OVERFLOW_BIN);
      
    int i,j;
      
    double height_1 = 0;
    double height_7 = 0;
    for(i=0;i<aHisto.xAxis().bins();i++){
      height_1 += aHisto.binHeight(i,AIDA::IAxis::UNDERFLOW_BIN);
      height_7 += aHisto.binHeight(i,AIDA::IAxis::OVERFLOW_BIN);
    }
      
    double height_3 = 0;
    double height_5 = 0;
    for(j=0;j<aHisto.yAxis().bins();j++){
      height_3 += aHisto.binHeight(AIDA::IAxis::UNDERFLOW_BIN,j);
      height_5 += aHisto.binHeight(AIDA::IAxis::OVERFLOW_BIN,j);
    }

    double height_4 = 0;
    for(i=0;i<aHisto.xAxis().bins();i++){
      for(j=0;j<aHisto.yAxis().bins();j++){
        height_4 += aHisto.binHeight(i,j);
      }
    }
      
    std::string bar(32,'-');
    out << Lib::sjust(height_6,10,Lib::smanip::middle)
        << "|" << Lib::sjust(height_7,10,Lib::smanip::middle) 
        << "|" << Lib::sjust(height_8,10,Lib::smanip::middle) 
        << Lib::endl;
    out << bar << Lib::endl;
    out << Lib::sjust(height_3,10,Lib::smanip::middle)
        << "|" << Lib::sjust(height_4,10,Lib::smanip::middle)
        << "|" << Lib::sjust(height_5,10,Lib::smanip::middle)
        << Lib::endl;
    out << bar << Lib::endl;
    out << Lib::sjust(height_0,10,Lib::smanip::middle)
        << "|" << Lib::sjust(height_1,10,Lib::smanip::middle)
        << "|" << Lib::sjust(height_2,10,Lib::smanip::middle)
        << Lib::endl;

    out << " * MIN = " << Lib::sjust(aHisto.minBinHeight(),field)
        << " * MAX = " << Lib::sjust(aHisto.maxBinHeight(),field)
        << Lib::endl;
  }

}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IFunction& aFunc
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  std::vector<std::string> names = aFunc.parameterNames();
  std::vector<double> vals = aFunc.parameters();

  out << aFunc.dimension() << "D " << aFunc.title() << Lib::endl;
  out << " * NAME = " << aName << Lib::endl;

  unsigned int number = names.size();
  for(unsigned int index=0;index<number;index++){
    out << " * PARAM   = " << Lib::sjust(names[index],field)
        << " * VALUE = " << Lib::sjust(vals[index],field)
        << Lib::endl;
  }
  out << " * CODELET = " << aFunc.codeletString() << Lib::endl;

}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::ICloud& aCloud
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aCloud.entries()) {

    out << "****** AIDA   NO ENTRIES FOR CLOUD = " << aName
        << Lib::endl;

  } else {

    out << aCloud.dimension() << "D " << aCloud.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << Lib::sjust(aCloud.entries(),field)
        << Lib::endl;
      
    out << " * SUM OF WEIGHTS = " << Lib::sjust(aCloud.sumOfWeights(),field)
        << Lib::endl;

    out << " * IS CONVERTED TO HISTOGRAM = " 
        << Lib::smanip::tostring(aCloud.isConverted())
        << Lib::endl;

    if(const AIDA::ICloud1D* cl1D = Lib_ConstCast(aCloud,AIDA::ICloud1D)) {

        out << " * MEAN VALUE = " << Lib::sjust(cl1D->mean(),field)
            << " * R . M . S = " << Lib::sjust(cl1D->rms(),field)
            << Lib::endl;

        out << " * MIN = " << Lib::sjust(cl1D->lowerEdge(),field+7)
            << " * MAX = " << Lib::sjust(cl1D->upperEdge(),field)
            << Lib::endl;

    } else if(const AIDA::ICloud2D* cl2D = 
                Lib_ConstCast(aCloud,AIDA::ICloud2D)) {

        out << " * X MEAN VALUE = " << Lib::sjust(cl2D->meanX(),field)
            << " * X R . M . S = " << Lib::sjust(cl2D->rmsX(),field)
            << Lib::endl;

        out << " * X MIN = " << Lib::sjust(cl2D->lowerEdgeX(),field+7)
            << " * X MAX = " << Lib::sjust(cl2D->upperEdgeX(),field)
            << Lib::endl;

        out << " * Y MEAN VALUE = " << Lib::sjust(cl2D->meanY(),field)
            << " * Y R . M . S = " << Lib::sjust(cl2D->rmsY(),field)
            << Lib::endl;

        out << " * Y MIN = " << Lib::sjust(cl2D->lowerEdgeY(),field+7)
            << " * Y MAX = " << Lib::sjust(cl2D->upperEdgeY(),field)
            << Lib::endl;

    } else if(const AIDA::ICloud3D* cl3D = 
                Lib_ConstCast(aCloud,AIDA::ICloud3D)) {

        out << " * X MEAN VALUE = " << Lib::sjust(cl3D->meanX(),field)
            << " * X R . M . S = " << Lib::sjust(cl3D->rmsX(),field)
            << Lib::endl;

        out << " * X MIN = " << Lib::sjust(cl3D->lowerEdgeX(),field+7)
            << " * X MAX = " << Lib::sjust(cl3D->upperEdgeX(),field)
            << Lib::endl;

        out << " * Y MEAN VALUE = " << Lib::sjust(cl3D->meanY(),field)
            << " * Y R . M . S = " << Lib::sjust(cl3D->rmsY(),field)
            << Lib::endl;

        out << " * Y MIN = " << Lib::sjust(cl3D->lowerEdgeY(),field+7)
            << " * Y MAX = " << Lib::sjust(cl3D->upperEdgeY(),field)
            << Lib::endl;

        out << " * Z MEAN VALUE = " << Lib::sjust(cl3D->meanZ(),field)
            << " * Z R . M . S = " << Lib::sjust(cl3D->rmsZ(),field)
            << Lib::endl;

        out << " * Z MIN = " << Lib::sjust(cl3D->lowerEdgeZ(),field+7)
            << " * Z MAX = " << Lib::sjust(cl3D->upperEdgeZ(),field)
            << Lib::endl;
    }
  }

}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IDataPointSet& aDPS
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  int field = 13; //HBOOK had 17.

  if(!aDPS.size()) {

    out << "****** AIDA   NO ENTRIES FOR DATA POINT SET = " << aName
        << Lib::endl;

  } else {

    out << aDPS.dimension() << "D " << aDPS.title() << Lib::endl;
    out << " * NAME = " << aName << Lib::endl;

    out << " * ENTRIES = " << Lib::sjust(aDPS.size(),field)
        << Lib::endl;
      
  }
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IBaseHistogram& aObject
,const std::string& aName
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  if(const AIDA::IHistogram1D* h1 = 
       Lib_ConstCast(aObject,AIDA::IHistogram1D)) {

    hprint(*h1,aName,aPrinter);

  } else if(const AIDA::IProfile1D* pf1 = 
              Lib_ConstCast(aObject,AIDA::IProfile1D)) {

    hprint(*pf1,aName,aPrinter);

  } else if(const AIDA::IHistogram2D* h2 = 
              Lib_ConstCast(aObject,AIDA::IHistogram2D)) {

    hprint(*h2,aName,aPrinter);

  } else if(const AIDA::ICloud* cl = Lib_ConstCast(aObject,AIDA::ICloud)) {

    hprint(*cl,aName,aPrinter);

  } else {

    out << "****** AIDA NO INFOS FOR OBJECT = " << aName
        << Lib::endl;

  }
}
//////////////////////////////////////////////////////////////////////////////
void BatchLab::amanip::hprint(
 const AIDA::IManagedObject& aObject
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  if(const AIDA::IHistogram1D* h1 = 
       Lib_ConstCast(aObject,AIDA::IHistogram1D)) {

    hprint(*h1,aObject.name(),aPrinter);

  } else if(const AIDA::IProfile1D* pf1 = 
              Lib_ConstCast(aObject,AIDA::IProfile1D)) {

    hprint(*pf1,aObject.name(),aPrinter);

  } else if(const AIDA::IHistogram2D* h2 = 
              Lib_ConstCast(aObject,AIDA::IHistogram2D)) {

    hprint(*h2,aObject.name(),aPrinter);

  } else if(const AIDA::IFunction* f = 
              Lib_ConstCast(aObject,AIDA::IFunction)) {

    hprint(*f,aObject.name(),aPrinter);

  } else if(const AIDA::ITuple* tuple = Lib_ConstCast(aObject,AIDA::ITuple)) {

    hprint(*tuple,aObject.name(),aPrinter);

  } else if(const AIDA::ICloud* cl = Lib_ConstCast(aObject,AIDA::ICloud)) {

    hprint(*cl,aObject.name(),aPrinter);

  } else if(const AIDA::IDataPointSet* dps = 
              Lib_ConstCast(aObject,AIDA::IDataPointSet)) {

    hprint(*dps,aObject.name(),aPrinter);

  } else {

    out << "****** AIDA NO INFOS FOR OBJECT = " << aObject.name()
        << Lib::endl;

  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::write(
 AIDA::ITuple& aObject
,Lib::Out& aOut
,bool aHeader
,bool aTabSep
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aHeader) {
    // Write title :
    aOut << aObject.title() << Lib::endl;
  }

  // Check column types :
  int coln = aObject.columns();
  if(coln<=0) return true;
  for(int index=0;index<coln;index++) {
    std::string type = aObject.columnType(index);
    if( (type=="ITuple") ||
        (type=="string") ||
        (type=="char") ||
        (type=="byte") ||
        (type=="boolean")
      ) {
      Lib::Out out(aPrinter);
      out << "BatchLab::amanip::writeTuple :" 
          << " column type " << Lib::sout(type) << " not supported."
          << Lib::endl;
      return false;
    }
  }
  
  if(aHeader) {
    // Write column names :
    for(int index=0;index<coln;index++) {
      if(index) aOut << "\t";
      aOut << aObject.columnName(index);
    }
    aOut << Lib::endl;
  }

  // rows :
  aObject.start();
  while(aObject.next()) {
    for(int index=0;index<coln;index++) {
      std::string value;
      if(!amanip::tuple_getString(aPrinter,aObject,index,value)) {
        return false;
      }
      if(index) {
        if(aTabSep) aOut << "\t";
        else aOut << " ";
      }
      aOut << value;
    }
    aOut << Lib::endl;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::write(
 const AIDA::IHistogram1D& aObject
,Lib::Out& aOut
,bool aHeader
,bool //aTabSep
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Histogram1D* object = 
    Lib_ConstCast(aObject,BatchLab::Histogram1D); //FIXME : use AIDA only.

  if(aHeader) {

  aOut << Lib::sout(aObject.title()) << Lib::endl;

  // <axis> :
 {const AIDA::IAxis& axis = aObject.axis();
  aOut << "x"
       << " " << axis.bins() 
       << " " << BatchLab::amanip::d2s(axis.lowerEdge()) 
       << " " << BatchLab::amanip::d2s(axis.upperEdge()); 
  if(axis.isFixedBinning()) {
  } else {
    int number = axis.bins()-1;
    for(int index=0;index<number;index++) {
      aOut << " " << BatchLab::amanip::d2s(axis.binUpperEdge(index));
    }
  }
  aOut << Lib::endl;}

  // <statistics> : 
  aOut << aObject.entries() << Lib::endl;
  aOut << "x"
       << " " << d2s(aObject.mean()) 
       << " " << d2s(aObject.rms()) 
       << Lib::endl;

  // bins :
#define OUT_BIN_1D(aIndex)\
    aOut << binToString(aIndex)\
         << " " << aObject.binEntries(aIndex)\
         << " " << d2s(aObject.binHeight(aIndex))\
         << " " << d2s(aObject.binError(aIndex))\
         << " " << d2s(aObject.binMean(aIndex));\
    if(object) {\
    aOut << " " << d2s(object->binStdDev(aIndex));\
    }\
    aOut << Lib::endl;

  OUT_BIN_1D(AIDA::IAxis::UNDERFLOW_BIN)
  OUT_BIN_1D(AIDA::IAxis::OVERFLOW_BIN)

  } //aHeader

 {int xbins = aObject.axis().bins();
  for(int index=0;index<xbins;index++) {
    OUT_BIN_1D(index)
  }}

#undef OUT_BIN_1D

  return true;  
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::amanip::write(
 const AIDA::IHistogram2D& aObject
,Lib::Out& aOut
,bool aHeader
,bool //aTabSep
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const BatchLab::Histogram2D* object = 
    Lib_ConstCast(aObject,BatchLab::Histogram2D); //FIXME : use AIDA only.

  if(aHeader) {

  aOut << Lib::sout(aObject.title()) << Lib::endl;

  // <axis> :
 {const AIDA::IAxis& axis = aObject.xAxis();
  aOut << "x"
       << " " << axis.bins() 
       << " " << BatchLab::amanip::d2s(axis.lowerEdge()) 
       << " " << BatchLab::amanip::d2s(axis.upperEdge()); 
  if(axis.isFixedBinning()) {
  } else {
    int number = axis.bins()-1;
    for(int index=0;index<number;index++) {
      aOut << " " << BatchLab::amanip::d2s(axis.binUpperEdge(index));
    }
  }
  aOut << Lib::endl;}

 {const AIDA::IAxis& axis = aObject.yAxis();
  aOut << "y"
       << " " << axis.bins() 
       << " " << BatchLab::amanip::d2s(axis.lowerEdge()) 
       << " " << BatchLab::amanip::d2s(axis.upperEdge()); 
  if(axis.isFixedBinning()) {
  } else {
    int number = axis.bins()-1;
    for(int index=0;index<number;index++) {
      aOut << " " << BatchLab::amanip::d2s(axis.binUpperEdge(index));
    }
  }
  aOut << Lib::endl;}

  // <statistics> : 
  aOut << aObject.entries() << Lib::endl;
  aOut << "x"
       << " " << d2s(aObject.meanX()) 
       << " " << d2s(aObject.rmsX()) 
       << Lib::endl;
  aOut << "y"
       << " " << d2s(aObject.meanY()) 
       << " " << d2s(aObject.rmsY()) 
       << Lib::endl;

  // bins :
#define OUT_BIN_2D(aIndexX,aIndexY)\
      aOut << binToString(aIndexX)\
           << " " << binToString(aIndexY)\
           << " " << aObject.binEntries(aIndexX,aIndexY)\
           << " " << d2s(aObject.binHeight(aIndexX,aIndexY))\
           << " " << d2s(aObject.binError(aIndexX,aIndexY))\
           << " " << d2s(aObject.binMeanX(aIndexX,aIndexY))\
           << " " << d2s(aObject.binMeanY(aIndexX,aIndexY));\
      if(object) {\
      aOut << " " << d2s(object->binStdDevX(aIndexX,aIndexY));\
      aOut << " " << d2s(object->binStdDevY(aIndexX,aIndexY));\
      }\
      aOut << Lib::endl;

  OUT_BIN_2D(AIDA::IAxis::UNDERFLOW_BIN,AIDA::IAxis::UNDERFLOW_BIN)
  OUT_BIN_2D(AIDA::IAxis::UNDERFLOW_BIN,AIDA::IAxis::OVERFLOW_BIN)
  OUT_BIN_2D(AIDA::IAxis::OVERFLOW_BIN,AIDA::IAxis::UNDERFLOW_BIN)
  OUT_BIN_2D(AIDA::IAxis::OVERFLOW_BIN,AIDA::IAxis::OVERFLOW_BIN)

  } //aHeader

 {int xbins = aObject.xAxis().bins();
  int ybins = aObject.yAxis().bins();
  int indexX,indexY;
  for(indexX=0;indexX<xbins;indexX++) {
    for(indexY=0;indexY<ybins;indexY++) {
      OUT_BIN_2D(indexX,indexY)
    }
  }}

#undef OUT_BIN_2D

  return true;  
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::amanip::binToString(
 int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aIndex==AIDA::IAxis::UNDERFLOW_BIN) return "UNDERFLOW";
  else if(aIndex==AIDA::IAxis::OVERFLOW_BIN) return "OVERFLOW";
  else {
    std::string s;
    Lib::smanip::printf(s,64,"%d",aIndex);
    return s;
  }
}
#include <stdio.h>
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::amanip::d2s(
 double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  char s[64];
  ::sprintf(s,"%.25g",aValue);
  return std::string(s);
}
