// this :
#include <BatchLab/FS/Tree.h>

#include <Slash/Store/IFolderVisitor.h>
#include <Slash/Store/IObject.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>
#include <Lib/ItemML.h> //FIXME : for Lib::ItemMLFactory
#include <Lib/GZipWriter.h>
#include <Lib/FileWriter.h>

#include <AIDA/IManagedObject.h>
#include <AIDA/IConstants.h>
#include <AIDA/ITuple.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>

#include <BatchLab/XML/AIDA_ML.h>
#include <BatchLab/Version.h>

#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#include <BatchLab/Core/amanip.h>
#else
#endif

namespace BatchLab {

/*
class AIDA_FS : public Lib::BaseFS {
public:
  AIDA_FS(BaseTree& aTree)
  :fTree(aTree)
  ,fVerbose(fTree.verboseLevel())
  //,fVerbose(1)
  {}
public:
  virtual bool visitEntry(const std::string& aEntry,
                          unsigned int aSize,
                          const char* aBuffer) {
    if(fVerbose) {
      Lib::Out out(fTree.printer());
      out << "AIDA_FS::visitEntry :" 
          << " entry " << Lib::sout(aEntry)
          << " (size " << aSize << ")..."
          << Lib::endl;
    }

    Lib::ItemMLFactory factory;
    AIDA_ML aidaML(factory,fTree);
    if(!aidaML.loadBuffer(aSize,aBuffer,fTree.printer()) || aidaML.errors()) {
      Lib::Out out(fTree.printer());
      out << "AIDA_FS::visitEntry :" 
          << " problem loading entry " << Lib::sout(aEntry)
          << ". Continue anyway."
          << Lib::endl;
      return true;
    }
    if(fVerbose) {
      Lib::Out out(fTree.printer());
      out << "AIDA_FS::visitEntry :" 
          << " load of " << Lib::sout(aEntry) << " done."
          << Lib::endl;
    }
    return true;
  }
private:
  BaseTree& fTree;
  int fVerbose;
};
*/

}
//////////////////////////////////////////////////////////////////////////////
BatchLab::FS::Tree::Tree(
 Slash::Core::ISession& aSession
,const std::string& aStoreName
,bool aReadOnly
,bool aCreateNew
,const std::string& aOptions
,FS::Manager& aManager
,int aVerboseLevel
)
:MemoryTree(aSession,aStoreName) //FIXME : have a name # storeName
,fManager(aManager)
,fStoreName(aStoreName)
,fNameCounter(0)
,fReadOnly(aReadOnly)
,fCreateNew(aCreateNew)
,fCompress(false)
,fIsValid(true)
,fStreaming(XML)
,fHeader(true)
,fTabSep(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setVerboseLevel(aVerboseLevel);

  std::string value;
  if(Lib::smanip::option(aOptions,"compress",value)) {
    bool bval;
    if(Lib::smanip::tobool(value,bval)) {
      fCompress = bval;
    }
  }

  if(Lib::smanip::option(aOptions,"streaming",value)) {
    Lib::smanip::tolowercase(value);
    if(value=="ascii") fStreaming = ASCII;
  }

  if(Lib::smanip::option(aOptions,"header",value)) {
    bool bval;
    if(Lib::smanip::tobool(value,bval)) {
      fHeader = bval;
    }
  }
  if(Lib::smanip::option(aOptions,"tab",value)) {
    bool bval;
    if(Lib::smanip::tobool(value,bval)) {
      fTabSep = bval;
    }
  }

  //printf("debug : FS::Tree :<-- \"%s\" \"%s\" %d -->\n",
  // aStoreName.c_str(),aOptions.c_str(),fCompress);

/*FIXME
  if(!Lib::BaseFS::available()) {
    Lib::Out out(printer());
    out << "BatchLab::FS::Tree::FS::Tree :"
        << " FS not available. " 
        << " check installation of Lib package "
        << " and Lib::BaseFS in particular. "
        << Lib::endl;
    fCompress = false;
  }
  
  if(fCreateNew) {
    // Handled in the commit.
  } else {
    // Read the existing file :
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::FS::Tree::FS::Tree :"
          << " open " << Lib::sout(aStoreName) 
          << " with compression=" 
          << std::string(fCompress?"yes":"no") << "..." << Lib::endl;
    }
    AIDA_FS aidaFS(*this);
    if(!aidaFS.loadFile(aStoreName,printer())) {
      Lib::Out out(printer());
      out << "BatchLab::FS::Tree::FS::Tree :"
          << " problem loading file " << Lib::sout(aStoreName)
          << Lib::endl;
      fIsValid = false;
      return;
    }
    if(verboseLevel()) {
      Lib::Out out(printer());
      out << "BatchLab::FS::Tree::FS::Tree :"
          << " load of " << Lib::sout(aStoreName) << " done."
          << Lib::endl;
    }
  }
*/
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::FS::Tree::~Tree(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::FS::Tree::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(BatchLab::FS::Tree)
  else return MemoryTree::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::FS::Tree::storeName(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStoreName;
}
//////////////////////////////////////////////////////////////////////////////
std::string BatchLab::FS::Tree::storeType(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "BatchLab::FS::Tree";
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fIsValid;
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::FS::Tree::eStreaming BatchLab::FS::Tree::streaming(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStreaming;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::header(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fHeader;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::tabSep(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTabSep;
}

//////////////////////////////////////////////////////////////////////////////
/// Data specific area ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

namespace BatchLab {
  class FS_StoreVisitor : public virtual Slash::Store::IFolderVisitor {
  public:
    FS_StoreVisitor(FS::Tree& aTree,
                     Lib::ItemMLFactory& aFactory,
                     //Lib::FS_Writer& aWriter,
                     Slash::Core::IWriter& aPrinter)
    :fTree(aTree)
    ,fAIDA_ML(aFactory,aTree)
    //,fWriter(aWriter)
    ,fPrinter(aPrinter)
    ,fVerboseLevel(fTree.verboseLevel())
    //,fVerboseLevel(1)
    {}
  public:
    virtual bool visit(const Slash::Store::IFolder& aFolder,
                       const Slash::Store::IObject& aObject){
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "BatchLab::FS_StoreVisitor::visit :" 
            << " path : " << Lib::sout(aFolder.path())
            << " name : " << Lib::sout(aObject.name())
            << Lib::endl;
      }

      std::string name = aObject.name();

      // Append the type/class of object :
      name += ".";
      if(const AIDA::IManagedObject* amo = 
          Lib_ConstCast(aObject,AIDA::IManagedObject)) {
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
        name += amanip::managed_object_type(*amo);
#else
        name += amo->type();
#endif
        Lib::smanip::replace(name,"AIDA::","");
      } else {
        name += aObject.storeClassName();
      }

      Slash::Core::IWriter* writer = 0;
      if(fTree.compress()) {
        writer = new Lib::GZipWriter(name,fPrinter);
        if(!writer->enabled()) {
          delete writer;
          return false;
        }
      } else {
        writer = new Lib::FileWriter(name,fPrinter);
        if(!writer->enabled()) {
          delete writer;
          return false;
        }
      }
      Lib::Out out(*writer);

      if(fTree.streaming()==FS::Tree::ASCII) {

        if(const AIDA::ITuple* object = Lib_ConstCast(aObject,AIDA::ITuple)) {

          if(!amanip::write(*(const_cast<AIDA::ITuple*>(object)),
                            out,
                            fTree.header(),fTree.tabSep(),
                            fPrinter)) {
            //continue anyway.
          }
 
          delete writer;

          return true;
        } else if(const AIDA::IHistogram1D* object = 
                    Lib_ConstCast(aObject,AIDA::IHistogram1D)) {

          if(!amanip::write(*object,out,
                            fTree.header(),fTree.tabSep())) {
            //continue anyway.
          }
 
          delete writer;

          return true;
        } else if(const AIDA::IHistogram2D* object = 
                    Lib_ConstCast(aObject,AIDA::IHistogram2D)) {

          if(!amanip::write(*object,out,
                            fTree.header(),fTree.tabSep())) {
            //continue anyway.
          }
 
          delete writer;

          return true;
        } else {

          Lib::Out out(fPrinter);
          out << "BatchLab::FS_StoreVisitor::visit :" 
              << " path : " << Lib::sout(aFolder.path())
              << " name : " << Lib::sout(aObject.name())
              << " : stream in XML"
              << " (Only tuples are supported in ascii streaming)."
              << Lib::endl;

        }

      } 
      //else { //XML

        // The name may contain env variables.

        // Header :
        out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << Lib::endl;
        out << "<!DOCTYPE aida SYSTEM \"http://aida.freehep.org/schemas/3.0/aida.dtd\">" << Lib::endl;
        out << "<aida version=" << Lib::sout(AIDA_VERSION) << ">" << Lib::endl;
        out << "  <implementation package=" << Lib::sout("BatchLab") 
            << " version=" << Lib::sout(BATCHLAB_VERSION) << "/>" << Lib::endl;

        if(!fAIDA_ML.write(*writer,aFolder.path(),aObject)) {
          //continue anyway.
        }

        // Footer :
        out << "</aida>" << Lib::endl;

      //}

      delete writer;

      return true;
    }
    virtual bool beginVisit(const Slash::Store::IFolder&,
                            const Slash::Store::IFolder& aFolder){ 
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "BatchLab::FS_StoreVisitor::beginVisit :" 
            << " path : " << Lib::sout(aFolder.path())
            << Lib::endl;
      }
      return Lib::dirmanip::mkcd(aFolder.name());
    }
    virtual bool endVisit(const Slash::Store::IFolder&,
                          const Slash::Store::IFolder& aFolder){ 
      if(fVerboseLevel) {
        Lib::Out out(fPrinter);
        out << "BatchLab::FS_StoreVisitor::endVisit :" 
            << " path : " << Lib::sout(aFolder.path())
            << Lib::endl;
      }
      return Lib::dirmanip::go("..");
    }
    virtual bool isRecursive() const { return true;}
  private:
    FS::Tree& fTree;  
    AIDA_ML fAIDA_ML;
    //Lib::FS_Writer& fWriter;
    Slash::Core::IWriter& fPrinter;
    int fVerboseLevel;
};
}

//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::commit(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fReadOnly) return false;
  std::string fileName = name(); //!!!
  if(fCreateNew) {
  } else {
    //Can't update for the moment.
    return false; //FIXME.
  }

  // The name may contain env variables.
  std::string fullName = Lib::fmanip::name(fileName);

  std::string pwd;
  if(!Lib::dirmanip::directory(pwd)) {
    Lib::Out out(printer());
    out << "BatchLab::FS::Tree::commit :" 
        << " can't get current directory."
        << Lib::endl;
    return false;
  }

 {std::string path,name,suffix;
  Lib::smanip::pathNameSuffix(fullName,path,name,suffix);
  if(path.size() && !Lib::dirmanip::go(path)) {
    Lib::Out out(printer());
    out << "BatchLab::FS::Tree::commit :" 
        << " can't go in " << Lib::sout(path)
        << Lib::endl;
    return false;
  }
  if(fCreateNew) {
    if(Lib::dirmanip::exists(name)) {
      if(!Lib::dirmanip::remove(name)) {
        Lib::Out out(printer());
        out << "BatchLab::FS::Tree::commit :" 
            << " can't remove " << Lib::sout(name)
            << Lib::endl;
        return false;
      }
    }
  }
  if(!Lib::dirmanip::mkcd(name)) {
    Lib::Out out(printer());
    out << "BatchLab::FS::Tree::commit :" 
        << " can't mkcd " << Lib::sout(name)
        << Lib::endl;
    return false;
  }}

  Lib::ItemMLFactory factory; //FIXME
  FS_StoreVisitor visitor(*this,factory,printer());
  rootFolder().visit(visitor);

  if(pwd.size() && !Lib::dirmanip::go(pwd)) {
    Lib::Out out(printer());
    out << "BatchLab::FS::Tree::commit :" 
        << " can't go in " << Lib::sout(pwd)
        << Lib::endl;
    return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::close(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::FS::Tree::compress(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCompress;
}

