// this :
#include <BatchLab/Rio/THistogram.h>

// Rio :
#include <Rio/Interfaces/IClass.h>
#include <Rio/Interfaces/IBuffer.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/sout.h>

// HCL :
#include <HCL/Histogram.h>

// BatchLab :
#include <BatchLab/Core/Histogram.h>
#include <BatchLab/Rio/Annotation.h>

static bool read_v0(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v1(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v2(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v3(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v4(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v5(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v6(::Rio::IBuffer&,HCL::HistogramData&);
static bool read_v7(::Rio::IBuffer&,HCL::HistogramData&);

static bool Axis_write(::Rio::IBuffer&,HCL::Axis&);
static bool Axis_read_v0_v3(::Rio::IBuffer&,HCL::Axis&);
static bool Axis_read_v4_v6(::Rio::IBuffer&,HCL::Axis&);
static bool Axis_read_v7(::Rio::IBuffer&,HCL::Axis&);

RIO_META_SOURCE(BatchLab::Rio::THistogram)
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::THistogram::THistogram(
 const ::Rio::IDictionary& aDictionary
,Slash::Core::IWriter& aWriter
)
:Named(aDictionary,"","")
,fClass(BatchLab::Rio::THistogram::is(aDictionary))
,fWriter(aWriter)
,fData(0)
//////////////////////////////////////////////////////////////////////////////
// To read data.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::Rio::THistogram::~THistogram(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
BatchLab::BaseHistogram* BatchLab::Rio::THistogram::data(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData;
}
//////////////////////////////////////////////////////////////////////////////
void* BatchLab::Rio::THistogram::cast(
 const ::Rio::IClass& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(&aClass==&fClass) 
    return Rio_SCast(BatchLab::Rio::THistogram);
  else 
    return Named::cast(aClass);
}
//////////////////////////////////////////////////////////////////////////////
bool BatchLab::Rio::THistogram::stream(
 ::Rio::IBuffer& aBuffer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aBuffer.isReading()) {

    fData = 0;

    short v;
    if(!aBuffer.readVersion(v)) return false;
    if(!Named::stream(aBuffer)) return false;
    HCL::HistogramData data;
    BatchLab::Annotation annotation;
    if(v==0) {
      if(!read_v0(aBuffer,data)) return false;
    } else if(v==1) {
      if(!read_v1(aBuffer,data)) return false;
    } else if(v==2) {
      if(!read_v2(aBuffer,data)) return false;
    } else if(v==3) {
      if(!read_v3(aBuffer,data)) return false;
    } else if(v==4) {
      if(!read_v4(aBuffer,data)) return false;
    } else if(v==5) {
      if(!read_v5(aBuffer,data)) return false;
    } else if(v==6) {
      if(!read_v6(aBuffer,data)) return false;
    } else if(v==7) {
      if(!read_v7(aBuffer,data)) return false;
    } else if(v==8) {
      if(!BatchLab::Rio::Annotation::read(aBuffer,annotation)) return false;
      if(!read_v7(aBuffer,data)) return false;
    } else {
      return false;
    }

    data.fDoubles.resize(data.fDimension,0);
    data.fInts.resize(data.fDimension,0);

    HCL::Histogram hcl(data);

    BatchLab::BaseHistogram* bobject = 0;
    int dim = data.fDimension;
    bool isProfile = data.fProfile;
    if(dim==1) {
      if(isProfile) {
        bobject = new BatchLab::Profile1D(fWriter,fName,hcl);
      } else {
        bobject = new BatchLab::Histogram1D(fWriter,fName,hcl);
      }
    } else if(dim==2) {
      if(isProfile) {
        bobject = new BatchLab::Profile2D(fWriter,fName,hcl);
      } else {
        bobject = new BatchLab::Histogram2D(fWriter,fName,hcl);
      }
    } else if(dim==3) {
      bobject = new BatchLab::Histogram3D(fWriter,fName,hcl);
    } else {
      Lib::Out out(fWriter);
      out << "BatchLab::Rio::Histogram::streamer : object " 
          << Lib::sout(fName)
          << " of dimension " << dim << " not treated." 
          << Lib::endl;      
      return false;
    }
    bobject->setAnnotation(annotation);

    fData = bobject;

  } else {
    if(!aBuffer.writeVersion(fClass.version())) return false;
    if(!Named::stream(aBuffer)) return false;

    BatchLab::Annotation& annotation = fData->getAnnotation();
    if(!BatchLab::Rio::Annotation::write(aBuffer,annotation)) return false;

    HCL::Histogram& hclHisto = (HCL::Histogram&)fData->bareHistogram();
    HCL::HistogramData& data = hclHisto.getData(); 

    if(!aBuffer.write(data.fTitle)) return false;
    if(!aBuffer.write(data.fDimension)) return false;
    if(!aBuffer.write(data.fBinNumber)) return false;
    if(!::Rio::writeArray<int>(aBuffer,data.fBinEntries)) return false;
    if(!::Rio::writeArray<double>(aBuffer,data.fBinSw)) return false;
    if(!::Rio::writeArray<double>(aBuffer,data.fBinSw2)) return false;
    if(!::Rio::writeArray2<double>(aBuffer,data.fBinSxw)) return false;
    if(!::Rio::writeArray2<double>(aBuffer,data.fBinSx2w)) return false;
    for(int iaxis=0;iaxis<data.fDimension;iaxis++) {
      if(!Axis_write(aBuffer,data.fAxes[iaxis])) return false;
    }
    if(!aBuffer.write(data.fMode)) return false;
    // Profile :
    if(!aBuffer.write(data.fProfile)) return false;
    if(data.fProfile) {
      if(!::Rio::writeArray<double>(aBuffer,data.fBinSvw)) return false;
      if(!::Rio::writeArray<double>(aBuffer,data.fBinSv2w)) return false;
      if(!aBuffer.write(data.fCutV)) return false;
      if(!aBuffer.write(data.fMinV)) return false;
      if(!aBuffer.write(data.fMaxV)) return false;
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Axis_write(
 ::Rio::IBuffer& aBuffer
,HCL::Axis& aAxis
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aBuffer.write(aAxis.fOffset)) return false;
  if(!aBuffer.write(aAxis.fNumberOfBins)) return false;
  if(!aBuffer.write(aAxis.fMinimumValue)) return false;
  if(!aBuffer.write(aAxis.fMaximumValue)) return false;
  if(!aBuffer.write(aAxis.fFixed)) return false;
  if(!aBuffer.write(aAxis.fBinWidth)) return false;
  if(!aBuffer.write((int)aAxis.fEdges.size())) return false;
  for(unsigned int count=0;count<aAxis.fEdges.size();count++) {
    if(!aBuffer.write(aAxis.fEdges[count])) return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int new_bin_number(const std::vector<HCL::Axis>& aAxes) {
  int number = 1;
  for(unsigned int iaxis=0;iaxis<aAxes.size();iaxis++) 
    number *= (aAxes[iaxis].bins()+2);
  return number;
}
template <class T> void add_outflow(
 const std::vector<HCL::Axis>& aAxes
,std::vector<T>& aVector) {
  int iaxis;
  // aAxes[].fOffset contains the offset without outflow.
  int dim = aAxes.size();
  // new size and offsets :
  std::vector<int> aoff(dim);
  int newn = 1;
  for(iaxis=0;iaxis<dim;iaxis++) newn *= (aAxes[iaxis].bins()+2);
  aoff[0] = 1;
  for(iaxis=1;iaxis<dim;iaxis++)
    aoff[iaxis] = aoff[iaxis-1] * (aAxes[iaxis-1].bins()+2);
  // copy :
  std::vector<T> tmp = aVector;
  int oldn = (int)tmp.size();
  aVector.resize(newn,0);
  // Copy :
  std::vector<int> is(dim);
  int offset;
  for(int index=0;index<oldn;index++) {
    // Get new offset of index :
    offset = index;
    for(iaxis=dim-1;iaxis>=0;iaxis--) { 
      is[iaxis] = offset/aAxes[iaxis].fOffset;
      offset -= is[iaxis] * aAxes[iaxis].fOffset;
    }
    // new offset :
    offset = 0;
    for(iaxis=0;iaxis<dim;iaxis++) offset += is[iaxis] * aoff[iaxis];
    aVector[offset] = tmp[index];
  }
}
void add_outflow(std::vector<HCL::Axis>& aAxes) {
  // Restore new offsets :
  aAxes[0].fOffset = 1;
  for(unsigned int iaxis=1;iaxis<aAxes.size();iaxis++)
    aAxes[iaxis].fOffset = aAxes[iaxis-1].fOffset * (aAxes[iaxis-1].bins()+2);
}
//////////////////////////////////////////////////////////////////////////////
bool read_v0(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  std::string sdummy;
  if(!aBuffer.read(sdummy)) return false;
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v0_v3(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  HCL::Axis axisOfValues;
  if(!Axis_read_v0_v3(aBuffer,axisOfValues)) return false;
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v0 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v1(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v0_v3(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  HCL::Axis axisOfValues;
  if(!Axis_read_v0_v3(aBuffer,axisOfValues)) return false;
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v1 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v2(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  std::string sdummy;
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(sdummy)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v0_v3(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  HCL::Axis axisOfValues;
  if(!Axis_read_v0_v3(aBuffer,axisOfValues)) return false;
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v2 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v3(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;

  int l;
  if(!aBuffer.read(l)) return false;
  char* str = new char[l+1];
  for (int i = 0; i < l; i++) {
    if(!aBuffer.read(str[i])) { delete [] str;return false;}
  }
  str[l] = '\0';
  aData.fTitle = str;
  delete [] str;

  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v0_v3(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  HCL::Axis axisOfValues;
  if(!Axis_read_v0_v3(aBuffer,axisOfValues)) return false;
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v3 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v4(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;

  int l;
  if(!aBuffer.read(l)) return false;
  char* str = new char[l+1];
  for (int i = 0; i < l; i++) {
    if(!aBuffer.read(str[i])) { delete [] str;return false;}
  }
  str[l] = '\0';
  aData.fTitle = str;
  delete [] str;

  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v4_v6(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v4 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v5(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v4_v6(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  if(!aBuffer.read(aData.fMode)) return false;

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow(aData.fAxes);

  // Not here in v5 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty);
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSvw.resize(aData.fBinNumber,0);
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v6(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!aBuffer.read(idummy)) return false; //fEntries
  if(!aBuffer.read(idummy)) return false; //fOutFlow
  if(!aBuffer.read(ddummy)) return false; //fSw
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v4_v6(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  if(!aBuffer.read(aData.fMode)) return false;

  // Profile :
  if(!aBuffer.read(aData.fProfile)) return false;
  if(aData.fProfile) {
    if(!::Rio::readArray<double>(aBuffer,aData.fBinSvw)) return false;
    if((int)aData.fBinSvw.size()!=aData.fBinNumber) return false;
    if(!aBuffer.read(aData.fCutV)) return false;
    if(!aBuffer.read(aData.fMinV)) return false;
    if(!aBuffer.read(aData.fMaxV)) return false;
  }

  // Add outflow :
  aData.fBinNumber = new_bin_number(aData.fAxes);
  add_outflow<int>(aData.fAxes,aData.fBinEntries);
  add_outflow<double>(aData.fAxes,aData.fBinSw);
  add_outflow<double>(aData.fAxes,aData.fBinSw2);
  add_outflow<double>(aData.fAxes,aData.fBinSvw);
  add_outflow(aData.fAxes);

  // Not here in v6 :
  std::vector<double> empty;
  empty.resize(aData.fDimension,0);
  aData.fBinSxw.resize(aData.fBinNumber,empty); //Forget to write in v6 !
  aData.fBinSx2w.resize(aData.fBinNumber,empty);
  if(aData.fProfile) {
    aData.fBinSv2w.resize(aData.fBinNumber,0);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool read_v7(
 ::Rio::IBuffer& aBuffer
,HCL::HistogramData& aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aBuffer.read(aData.fTitle)) return false;
  if(!aBuffer.read(aData.fDimension)) return false;
  if(!aBuffer.read(aData.fBinNumber)) return false;
  if(!::Rio::readArray<int>(aBuffer,aData.fBinEntries)) return false;
  if((int)aData.fBinEntries.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw)) return false;
  if((int)aData.fBinSw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray<double>(aBuffer,aData.fBinSw2)) return false;
  if((int)aData.fBinSw2.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray2<double>(aBuffer,aData.fBinSxw)) return false;
  if((int)aData.fBinSxw.size()!=aData.fBinNumber) return false;
  if(!::Rio::readArray2<double>(aBuffer,aData.fBinSx2w)) return false;
  if((int)aData.fBinSx2w.size()!=aData.fBinNumber) return false;
  if(aData.fDimension>0) {
    aData.fAxes.resize(aData.fDimension);
    for(int iaxis=0;iaxis<aData.fDimension;iaxis++) {
      if(!Axis_read_v7(aBuffer,aData.fAxes[iaxis])) return false;
    }
  }
  if(!aBuffer.read(aData.fMode)) return false;
  // Profile :
  if(!aBuffer.read(aData.fProfile)) return false;
  if(aData.fProfile) {
    if(!::Rio::readArray<double>(aBuffer,aData.fBinSvw)) return false;
    if((int)aData.fBinSvw.size()!=aData.fBinNumber) return false;
    if(!::Rio::readArray<double>(aBuffer,aData.fBinSv2w)) return false;
    if((int)aData.fBinSv2w.size()!=aData.fBinNumber) return false;
    if(!aBuffer.read(aData.fCutV)) return false;
    if(!aBuffer.read(aData.fMinV)) return false;
    if(!aBuffer.read(aData.fMaxV)) return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Axis_read_v0_v3(
 ::Rio::IBuffer& aBuffer
,HCL::Axis& aAxis
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  if(!aBuffer.read(aAxis.fMinimumValue)) return false;
  if(!aBuffer.read(aAxis.fMaximumValue)) return false;
  if(!aBuffer.read(aAxis.fOffset)) return false;
  if(!aBuffer.read(aAxis.fNumberOfBins)) return false;
  if(!aBuffer.read(idummy)) return false; //fOverFlow
  if(!aBuffer.read(idummy)) return false; //fUnderFlow
  if(!aBuffer.read(aAxis.fBinWidth)) return false;
  if(!aBuffer.read(ddummy)) return false; //fSxw
  if(!aBuffer.read(ddummy)) return false; //fSx2w
  aAxis.fFixed = true;
  aAxis.fEdges.clear();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Axis_read_v4_v6(
 ::Rio::IBuffer& aBuffer
,HCL::Axis& aAxis
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int idummy;
  double ddummy;
  if(!aBuffer.read(aAxis.fOffset)) return false;
  if(!aBuffer.read(idummy)) return false; //fOverFlow
  if(!aBuffer.read(idummy)) return false; //fUnderFlow
  if(!aBuffer.read(ddummy)) return false; //fSxw
  if(!aBuffer.read(ddummy)) return false; //fSx2w
  if(!aBuffer.read(aAxis.fNumberOfBins)) return false;
  if(!aBuffer.read(aAxis.fMinimumValue)) return false;
  if(!aBuffer.read(aAxis.fMaximumValue)) return false;
  if(!aBuffer.read(aAxis.fFixed)) return false;
  if(!aBuffer.read(aAxis.fBinWidth)) return false;
  int edgen;
  if(!aBuffer.read(edgen)) return false;
  for(int count=0;count<edgen;count++) {
    double value;
    if(!aBuffer.read(value)) return false;
    aAxis.fEdges.push_back(value);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Axis_read_v7(
 ::Rio::IBuffer& aBuffer
,HCL::Axis& aAxis
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aBuffer.read(aAxis.fOffset)) return false;
  if(!aBuffer.read(aAxis.fNumberOfBins)) return false;
  if(!aBuffer.read(aAxis.fMinimumValue)) return false;
  if(!aBuffer.read(aAxis.fMaximumValue)) return false;
  if(!aBuffer.read(aAxis.fFixed)) return false;
  if(!aBuffer.read(aAxis.fBinWidth)) return false;
  int edgen;
  if(!aBuffer.read(edgen)) return false;
  for(int count=0;count<edgen;count++) {
    double value;
    if(!aBuffer.read(value)) return false;
    aAxis.fEdges.push_back(value);
  }
  return true;
}
