// this :
#include <BatchLab/XML/DataReader.h>

#include <AIDA/IConstants.h>

#include <Slash/Core/ISession.h>
#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/ItemML.h>
#include <Lib/Element.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <BatchLab/Version.h>
#include <BatchLab/Core/amanip.h>

// HCL :
#include <HCL/Histogram.h>

// BatchLab :
#include <BatchLab/Core/Histogram.h>
#include <BatchLab/Core/MemoryTuple.h>
#include <BatchLab/Core/Cloud.h>
#include <BatchLab/Core/DataPointSet.h>
#include <BatchLab/Core/Function.h>
#include <BatchLab/Core/FFactory.h>
#include <BatchLab/XML/ReadContext.h>

static int sNameCounter = 0; //FIXME

static Slash::Store::IObject* read_Histogram(BatchLab::XML::ReadContext&,std::string&,
                                       int,bool);

static bool readHistogramAxis(Lib::ItemML&,
			      int,int&,int&,double&,double&,
                              std::vector<double>&,bool&,Slash::Core::IWriter&);
static bool readHistogramData(Lib::ItemML&,HCL::Histogram&,Slash::Core::IWriter&);
static bool readTupleColumns(Lib::ItemML&,bool&,std::string&,Slash::Core::IWriter&);
static bool readTupleRows(Lib::ItemML&,AIDA::ITuple&,bool&,Slash::Core::IWriter&);

static Slash::Store::IObject* read_Cloud(BatchLab::XML::ReadContext&,std::string&,int);
static bool readCloudData(Lib::ItemML&,BatchLab::Cloud1D&,int,Slash::Core::ISession&);
static bool readCloudData(Lib::ItemML&,BatchLab::Cloud2D&,int,Slash::Core::ISession&);
static bool readCloudData(Lib::ItemML&,BatchLab::Cloud3D&,int,Slash::Core::ISession&);

static bool readDataPointSetData(Lib::ItemML&,AIDA::IDataPointSet&,Slash::Core::IWriter&);

class FunctionBooking {
public:
  std::string fArgument;
  std::vector< std::pair<double,double> > fRanges;
  std::vector< std::pair<std::string,double> > fParameters;
};

static bool readFunctionBooking(Lib::ItemML&,FunctionBooking&,Slash::Core::IWriter&);

//////////////////////////////////////////////////////////////////////////////
static std::string buildName(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name;
  if(!aItemML.attributeValue("","name",name)) { 
    Lib::smanip::printf(name,256,"BatchLab_XML_Object_%d",sNameCounter);
    sNameCounter++;
    aItemML.setAttributeValue("","name",name);
  }
  return name;
}
//////////////////////////////////////////////////////////////////////////////
static bool axisIndex(
 int aDimension
,const std::string& aAxis
,int& aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aDimension==1) {
    if(aAxis=="x") {aIndex = 0;return true;}
  } else if(aDimension==2) {
    if(aAxis=="x") {aIndex = 0;return true;}
    else if(aAxis=="y") {aIndex = 1;return true;}
  } else if(aDimension==3) {
    if(aAxis=="x") {aIndex = 0;return true;}
    else if(aAxis=="y") {aIndex = 1;return true;}
    else if(aAxis=="z") {aIndex = 2;return true;}
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Histogram1D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Histogram(aContext,aPath,1,false);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Histogram2D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Histogram(aContext,aPath,2,false);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Histogram3D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Histogram(aContext,aPath,3,false);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Profile1D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Histogram(aContext,aPath,1,true);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Profile2D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Histogram(aContext,aPath,2,true);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* read_Histogram(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
,int aDimension
,bool aIsProfile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int aVerboseLevel = aContext.verboseLevel();
  Slash::Core::IWriter& printer = aContext.session().printer();
  Lib::ItemML& aItemML = aContext.itemML();                      

  aPath = "";

  std::string xmlName = buildName(aItemML);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Histogram :" 
        << " name " << Lib::sout(xmlName)
        << "..." << Lib::endl;
  }

  std::string spath;
  aItemML.attributeValue("","path",spath);

  std::string stitle;
  aItemML.attributeValue("","title",stitle);

  // Booking parameters :
  std::vector<int> bins(aDimension);
  std::vector<double> mns(aDimension);
  std::vector<double> mxs(aDimension);
  std::vector< std::vector<double> > edges(aDimension);

  // Jump in subitems to find axes items :
  int NotFound = -1;
  int found = 0;
  bool isVariableBins = false;
  {const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    int iaxis;
    int num;
    double mn,mx;
    std::vector<double> borders;
    bool variableBins;
    if(!readHistogramAxis(*(*it),aDimension,
                          iaxis,num,mn,mx,borders,variableBins,printer)) 
      return 0;
    if(iaxis!=NotFound) {
      if((iaxis<0)||(iaxis>=aDimension)) return 0;
      bins[iaxis] = num;
      mns[iaxis] = mn;
      mxs[iaxis] = mx;
      edges[iaxis] = borders;
      if(variableBins) isVariableBins = true;
      found++;
    }
  }}
  if(found!=aDimension) return 0;
  if(isVariableBins) {
    // Axes consistency :
    for(int iaxis=0;iaxis<aDimension;iaxis++) {
      if(edges[iaxis].size()<=2) return 0;
    }
  }
  
  // Create a native histogram :
  HCL::Histogram* hclHistogram = 0;
  if(isVariableBins)
    hclHistogram = new HCL::Histogram(aDimension,edges,aIsProfile);
  else 
    hclHistogram = new HCL::Histogram(aDimension,bins,mns,mxs,aIsProfile);
  
  HCL::HistogramData& dHistogram = hclHistogram->getData(); 
  if(dHistogram.fBinNumber<=0) return 0;

  dHistogram.fTitle = stitle;
  
  // Sub items :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!readHistogramData(*(*it),*hclHistogram,printer)) {
      delete hclHistogram;
      return 0;
    }
  }
  
  hclHistogram->updateFastGetters();

  Slash::Store::IObject* object = 0;
  if(aIsProfile) {
    // Create a BatchLab::Profile :
    if(aDimension==1) {
      object = new BatchLab::Profile1D(printer,xmlName,*hclHistogram);
    } else if(aDimension==2) {
      object = new BatchLab::Profile2D(printer,xmlName,*hclHistogram);
    }
  } else {
    // Create a BatchLab::Histogram :
    if(aDimension==1) {
      object = new BatchLab::Histogram1D(printer,xmlName,*hclHistogram);
    } else if(aDimension==2) {
      object = new BatchLab::Histogram2D(printer,xmlName,*hclHistogram);
    } else if(aDimension==3) {
      object = new BatchLab::Histogram3D(printer,xmlName,*hclHistogram);
    }
  }

  delete hclHistogram;
  
  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Histogram :" 
        << " name " << Lib::sout(xmlName)
        << " done." << Lib::endl;
  }

  aPath = spath;

  return object;
}
//////////////////////////////////////////////////////////////////////////////
bool readHistogramAxis(
 Lib::ItemML& aItemML                      
,int aDimension
,int& aAxis
,int& aNumberOfBins
,double& aMin
,double& aMax
,std::vector<double>& aEdges
,bool& aVariableBins
,Slash::Core::IWriter& //aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NotFound = -1;
  aAxis = NotFound;
  aNumberOfBins = 0;
  aMin = 0;
  aMax = 0;
  aEdges.clear();
  aVariableBins = false;

  const std::string& tagName = aItemML.tagName();

  std::string svalue;
  int ival;

  if(tagName=="axis") {
 
    if(!aItemML.attributeValue("","numberOfBins",svalue)) return false;
    if(!Lib::smanip::toint(svalue,ival)) return false;
    aNumberOfBins = ival;

    if(!aItemML.attributeValue("","min",svalue)) return false;
    if(!Lib::smanip::todouble(svalue,aMin)) return false;

    if(!aItemML.attributeValue("","max",svalue)) return false;
    if(!Lib::smanip::todouble(svalue,aMax)) return false;

    if(!aItemML.attributeValue("","direction",svalue)) return false;
    if(!axisIndex(aDimension,svalue,aAxis)) return false;

    aEdges.push_back(aMin);
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="binBorder") {
        if(!(*it)->attributeValue("value",svalue)) return false;
        double value;
        if(!Lib::smanip::todouble(svalue,value)) return false;
        aEdges.push_back(value);
        aVariableBins = true;
      }
    }
    aEdges.push_back(aMax);
    if(aVariableBins) { // Variable bins histo.
      if((int)aEdges.size()!=aNumberOfBins+1) return false;
    }

  } 

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool readHistogramData(
 Lib::ItemML& aItemML                      
,HCL::Histogram& aHistogram
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  HCL::HistogramData& dHistogram = aHistogram.getData(); 

  std::string svalue;
  int ival;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="statistics") {
 
    if(!aItemML.attributeValue("","entries",svalue)) return false;
    if(!Lib::smanip::toint(svalue,ival)) return false;
    // dHistogram.fEntries = ival;

    int found = 0;
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="statistic") {
        double mean,rms;
        if(!(*it)->attributeValue("mean",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,mean)) return false;
          
        if(!(*it)->attributeValue("rms",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,rms)) return false;
          
        if(!(*it)->attributeValue("direction",svalue)) return false;
        int iaxis;
        if(!axisIndex(dHistogram.fDimension,svalue,iaxis)) return false;

        //dHistogram.fAxes[iaxis].fSxw = mean; //Temporarily put mean on fSxw.
        //dHistogram.fAxes[iaxis].fSx2w = rms; //Temporarily put mean on fSx2w.

        found++;
      }
    }
    if(found!=dHistogram.fDimension) return false;

    return true;

  } else if(tagName=="axis") {
 
    return true;

  } 

  int dimension = 0;
  if(tagName=="data1d") {
    dimension = 1;
  } else if(tagName=="data2d") {
    dimension = 2;
  } else if(tagName=="data3d") {
    dimension = 3;
  }

  if(dimension) {

    if(dimension!=dHistogram.fDimension) return false;

    std::string sbin;
    Lib::smanip::printf(sbin,32,"bin%dd",dimension);

    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()==sbin) {

        std::vector<int> is(dimension);

        int entries = 0;
        double height = 0;
        double error = 0;
        double weightedMean = 0;
        double weightedMeanX = 0;
        double weightedMeanY = 0;
        double weightedMeanZ = 0;
        double weightedRms = 0;
        double weightedRmsX = 0;
        double weightedRmsY = 0;
        double weightedRmsZ = 0;
        double rms = 0;

#define HASNO(aWhat) \
         {Lib::Out out(aPrinter);\
          out << "BatchLab::XML::readHistogramData :" \
              << " a <bin1d> has no " << #aWhat \
              << " attribute." \
              << Lib::endl;\
          return false;}

        // Required :
        if(!(*it)->attributeValue("entries",svalue)) HASNO(entries)
        if(!Lib::smanip::toint(svalue,entries)) {
          Lib::Out out(aPrinter);
          out << "BatchLab::XML::readHistogramData :" 
              << " problem converting a entries attribute to an int." 
              << " Value was " << Lib::sout(svalue) << "."
              << Lib::endl;
          return false;
        }

#define ERRCV(aWhat) \
         {Lib::Out out(aPrinter);\
          out << "BatchLab::XML::readHistogramData :" \
              << " problem converting a " << #aWhat \
              << " attribute to a double." \
              << " Value was " << Lib::sout(svalue) << "."\
              << Lib::endl;\
          return false;}

        // Optional :
        if((*it)->attributeValue("height",svalue)) { //FIXME : optional ?
          if(!Lib::smanip::todouble(svalue,height)) ERRCV(height)
        }  
        
        if((*it)->attributeValue("error",svalue)) { //FIXME : optional ?
          if(!Lib::smanip::todouble(svalue,error)) ERRCV(error)
        }

        if((*it)->attributeValue("rms",svalue)) {
          if(!Lib::smanip::todouble(svalue,rms)) ERRCV(rms)
        }

        if(dimension==1) {
          if((*it)->attributeValue("weightedMean",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMean)) ERRCV(weightedMean)
          }
          if((*it)->attributeValue("weightedRms",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRms)) ERRCV(weightedRms)
          }
        } else if(dimension==2) {
          if((*it)->attributeValue("weightedMeanX",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMeanX)) ERRCV(weightedMeanX)
          }
          if((*it)->attributeValue("weightedMeanY",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMeanY)) ERRCV(weightedMeanY)
          }
          if((*it)->attributeValue("weightedRmsX",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRmsX)) ERRCV(weightedRmsX)
          }
          if((*it)->attributeValue("weightedRmsY",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRmsY)) ERRCV(weightedRmsY)
          }
        } else if(dimension==3) {
          if((*it)->attributeValue("weightedMeanX",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMeanX)) ERRCV(weightedMeanX)
          }
          if((*it)->attributeValue("weightedMeanY",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMeanY)) ERRCV(weightedMeanY)
          }
          if((*it)->attributeValue("weightedMeanZ",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedMeanZ)) ERRCV(weightedMeanZ)
          }
          if((*it)->attributeValue("weightedRmsX",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRmsX)) ERRCV(weightedRmsX)
          }
          if((*it)->attributeValue("weightedRmsY",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRmsY)) ERRCV(weightedRmsY)
          }
          if((*it)->attributeValue("weightedRmsZ",svalue)) {
            if(!Lib::smanip::todouble(svalue,weightedRmsZ)) ERRCV(weightedRmsZ)
          }
        }
          
        int index;
        for(index=0;index<dimension;index++) {
          std::string s = "binNum";
          if(dimension!=1) {
            if(index==0) s += "X";
            else if(index==1) s += "Y";
            else if(index==2) s += "Z";
          }
          if(!(*it)->attributeValue(s,svalue)) {
            Lib::Out out(aPrinter);
            out << "BatchLab::XML::readHistogramData :" 
                << " a <bin1d> has no " << s << Lib::endl;
            return false;
          }
          if(svalue=="UNDERFLOW") {
            is[index] = AIDA::IAxis::UNDERFLOW_BIN;
          } else if(svalue=="OVERFLOW") {
            is[index] = AIDA::IAxis::OVERFLOW_BIN;
          } else {
            if(!Lib::smanip::toint(svalue,ival)) {
              Lib::Out out(aPrinter);
              out << "BatchLab::XML::readHistogramData :" 
                  << " problem converting binNum to an int." 
                  << " Value was " << Lib::sout(svalue) << "."
                  << Lib::endl;
              return false;
            }
            int ibin = ival;
            if( (ibin<0) || (ibin>=dHistogram.fAxes[index].bins()) ) { 
              Lib::Out out(aPrinter);
              out << "BatchLab::XML::readHistogramData :" 
                  << " a binNum is out of range." 
                  << Lib::endl;
              return false;
            }
            is[index] = ibin;
          }
        }

        // If we are here, then we have a valid bin :
        int offset = aHistogram.getOffset(is);
        dHistogram.fBinEntries[offset] = entries;
        if(!aHistogram.isProfile()) {

          // From HCL::Histogram, we have :
          //  height = sw
          //  error = sqrt(sw)
          //  weightedMean = sxw/sw
          //  weightedRms = sqrt(fabs(sx2w/sw - (sxw/sw)**2))

          double sw = height;
          dHistogram.fBinSw[offset] = sw;
          dHistogram.fBinSw2[offset] = error * error;
          if(dimension==1) {
            dHistogram.fBinSxw[offset][0] = weightedMean * sw;
            dHistogram.fBinSx2w[offset][0] = 
              (weightedRms * weightedRms + weightedMean * weightedMean) * sw;
          } else if(dimension==2){
            // X
            dHistogram.fBinSxw[offset][0] = weightedMeanX * sw;
            dHistogram.fBinSx2w[offset][0] = 
              (weightedRmsX*weightedRmsX + weightedMeanX*weightedMeanX) * sw;
            // Y :
            dHistogram.fBinSxw[offset][1] = weightedMeanY * sw;
            dHistogram.fBinSx2w[offset][1] = 
              (weightedRmsY*weightedRmsY + weightedMeanY*weightedMeanY) * sw;
          } else if(dimension==3){
            // X
            dHistogram.fBinSxw[offset][0] = weightedMeanX * sw;
            dHistogram.fBinSx2w[offset][0] = 
              (weightedRmsX*weightedRmsX + weightedMeanX*weightedMeanX) * sw;
            // Y :
            dHistogram.fBinSxw[offset][1] = weightedMeanY * sw;
            dHistogram.fBinSx2w[offset][1] = 
              (weightedRmsY*weightedRmsY + weightedMeanY*weightedMeanY) * sw;
            // Z :
            dHistogram.fBinSxw[offset][2] = weightedMeanZ * sw;
            dHistogram.fBinSx2w[offset][2] = 
              (weightedRmsZ*weightedRmsZ + weightedMeanZ*weightedMeanZ) * sw;
          }

        } else { // Profile :

          // bin writing is :
          // " height=" << Lib::sout(aObject.binHeight(aIndex))
          // " error=" << Lib::sout(aObject.binError(aIndex))
          // " weightedMean=" << Lib::sout(aObject.binMean(aIndex))
          // " rms=" << Lib::sout(aObject.binRms(aIndex))
          // " weightedRms=" << Lib::sout(binStdDev(aIndex));

          // From HCL::Profile, we have :
          //  height = svw / sw
          //  error = sqrt(fabs(sv2w/sw - (svw/sw)**2))/sqrt(sw)
          //  rms = sqrt(fabs(sv2w/sw - (svw/sw)**2))
          //  weightedMean = sxw/sw
          //  weightedRms = sqrt(fabs(sx2w/sw - (sxw/sw)**2))

          // Then :
          //  sw = (rms/error)**2
          //  svw = sw * height
          //  sv2w = sw * (rms**2 + height**2)
          //  sxw = weightedMean * sw
          //  sx2w = (weightedRms*weightedRms+weightedMean*weightedMean) * sw;

          double sw = 0;
          if(error==0) {
            // sv2w/sw = (svw/sw)**2
            // h = svw/sw
            //FIXME : we lack an info to get sw.
            // We assume that at fill time weight==1 then :
            //   sw == n
            sw = (double)entries;
          } else {
            double r_e = rms/error;
            sw = r_e * r_e;
          }
          dHistogram.fBinSw[offset] = sw;
          dHistogram.fBinSw2[offset] = 0; //FIXME
          if(dimension==1) {
            dHistogram.fBinSxw[offset][0] = weightedMean * sw;
            dHistogram.fBinSx2w[offset][0] = 
              (weightedRms * weightedRms + weightedMean * weightedMean) * sw;
          } else if(dimension==2){
            dHistogram.fBinSxw[offset][0] = weightedMeanX * sw;
            dHistogram.fBinSxw[offset][1] = weightedMeanY * sw;
            dHistogram.fBinSx2w[offset][0] = 
              (weightedRmsX*weightedRmsX + weightedMeanX*weightedMeanX) * sw;
            dHistogram.fBinSx2w[offset][1] = 
              (weightedRmsY*weightedRmsY + weightedMeanY*weightedMeanY) * sw;
          }
          dHistogram.fBinSvw[offset] = sw * height;
          dHistogram.fBinSv2w[offset] = sw * (rms * rms + height * height);
  
        }
      }
    }
    return true;
  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Tuple(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int aVerboseLevel = aContext.verboseLevel();
  Slash::Core::IWriter& printer = aContext.session().printer();
  Lib::ItemML& aItemML = aContext.itemML();                      

  aPath = "";

  std::string xmlName = buildName(aItemML);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Tuple :" 
        << " name " << Lib::sout(xmlName)
        << "..." << Lib::endl;
  }

  std::string spath;
  aItemML.attributeValue("","path",spath);

  std::string stitle;
  aItemML.attributeValue("","title",stitle);

  //FIXME annotation

  // Booking parameters :
  std::string booking;

  // Jump in subitems to find columns items :
  bool found = false;
 {const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!readTupleColumns(*(*it),found,booking,printer)) return 0;
    if(found) break;
  }}
  if(!found) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Tuple :" 
        << " for tuple name " << Lib::sout(xmlName)
        << " unable to read columns..." << Lib::endl;
    return 0;
  }

  // Create a BatchLab::MemoryTuple :
  BatchLab::MemoryTuple* object =  
    new BatchLab::MemoryTuple(printer,xmlName,stitle,booking);
  if(object->columns()<=0) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Tuple :" 
        << " for tuple name " << Lib::sout(xmlName)
        << " unable to create a BatchLab::MemoryTuple." << Lib::endl;
    delete object;
    object = 0;
    return 0;
  }
  
  // Get rows in sub items :
  found = false;
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!readTupleRows(*(*it),*object,found,printer)) {
      Lib::Out out(printer);
      out << "BatchLab::XML::read_Tuple :" 
          << " for tuple name " << Lib::sout(xmlName)
          << " unable to read rows." << Lib::endl;
      delete object;
      object = 0;
      return 0;
    }
    if(found) break;
  }
  
  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Tuple :" 
        << " name " << Lib::sout(xmlName)
        << " done." << Lib::endl;
  }
  
  aPath = spath;

  return object;
}
//////////////////////////////////////////////////////////////////////////////
bool readTupleColumns(
 Lib::ItemML& aItemML                      
,bool& aFound
,std::string& aBooking
,Slash::Core::IWriter& //aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aFound = false;

  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  aBooking = "";
  if(tagName=="columns") {
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="column") {
        if(aBooking!="") aBooking += ",";
        if(!(*it)->attributeValue("type",svalue)) {
          aBooking = "";
          return false;
        }
        aBooking += svalue + " ";
        if(!(*it)->attributeValue("name",svalue)) {
          aBooking = "";
          return false;
        }
        aBooking += svalue;
        if((*it)->attributeValue("booking",svalue)) {
          aBooking += " = " + svalue;
        } else if((*it)->attributeValue("default",svalue)) {
          aBooking += " = " + svalue;
        }
      }
    }
    aFound = true;
  } 

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool readTupleRows(
 Lib::ItemML& aItemML                      
,AIDA::ITuple& aTuple
,bool& aFound
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aFound = false;

  const std::string& tagName = aItemML.tagName();

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="columns") {

    return true;

  } else if(tagName=="rows") {

    // Sub items :
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      if(!readTupleRows(*(*it),aTuple,aFound,aPrinter)) return false;
    }
    aFound = true;

    return true;

  } else if(tagName=="row") {
 
    std::vector<int> ituples;
    std::vector<int> inot_tuples;
    int coln = aTuple.columns();
    for(int index=0;index<coln;index++) {
      std::string type = aTuple.columnType(index);
      if(type=="ITuple") {
        ituples.push_back(index);
      } else {
        inot_tuples.push_back(index);
      }
    }

    std::string svalue;
    int icol = 0;
   {const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="entry") {
        if(!(*it)->attributeValue("value",svalue)) {
          Lib::Out out(aPrinter);
          out << "BatchLab::XML::readTupleRows :" 
              << " can't get \"value\" attribute." << Lib::endl;
          return false;
        }
        if(!BatchLab::amanip::tuple_fill(aPrinter,aTuple,inot_tuples[icol],svalue)) {
          Lib::Out out(aPrinter);
          out << "BatchLab::XML::readTupleRows :" 
              << " can't fill tuple column." << Lib::endl;
          return false;
        }
        icol++;
      }
    }}

    // Sub items (entryTuple) :
    icol = 0;
   {const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      const std::string& tagName = (*it)->tagName();
      if(tagName=="entryTuple") {
        AIDA::ITuple* tuple = aTuple.getTuple(ituples[icol]);
        if(!tuple) {
          Lib::Out out(aPrinter);
          out << "BatchLab::XML::readTupleRows :" 
              << " can't get template tuple for column " 
              << Lib::sout(ituples[icol]) << "."
              << Lib::endl;
          return false;
        }
       // Read rows of tuple :
       {const std::list<Lib::ItemML*>& list = (*it)->children();
        std::list<Lib::ItemML*>::const_iterator it;
        for(it=list.begin();it!=list.end();++it) {
          bool found;
          if(!readTupleRows(*(*it),*tuple,found,aPrinter)) return false;
        }}
        icol++;
      }
    }}

    if(!aTuple.addRow()) {
      Lib::Out out(aPrinter);
      out << "BatchLab::XML::readTupleRows :" 
          << " can't add row to tuple." << Lib::endl;
      return false;
    }

    return true;

  }

  Lib::Out out(aPrinter);
  out << "BatchLab::XML::readTupleRows :" 
      << " unknown item class " << Lib::sout(tagName) << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Cloud1D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Cloud(aContext,aPath,1);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Cloud2D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Cloud(aContext,aPath,2);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Cloud3D(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return read_Cloud(aContext,aPath,3);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* read_Cloud(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
,int aDimension
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int aVerboseLevel = aContext.verboseLevel();
  Slash::Core::IWriter& printer = aContext.session().printer();
  Lib::ItemML& aItemML = aContext.itemML();                      

  aPath = "";

  std::string xmlName = buildName(aItemML);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Cloud :" 
        << " name " << Lib::sout(xmlName)
        << "..." << Lib::endl;
  }

  std::string spath;
  aItemML.attributeValue("","path",spath);

  std::string stitle;
  aItemML.attributeValue("","title",stitle);

  std::string svalue;
  int ival;

  // Booking parameters :
  int maxEntries = -1; //UNLIMITED
  if(aItemML.attributeValue("","maxEntries",svalue)) {
    if(!Lib::smanip::toint(svalue,ival)) return 0;
    maxEntries = ival;
  }

  // Create a BatchLab::Cloud :
  Slash::Store::IObject* object = 0;
  if(aDimension==1) {

    BatchLab::Cloud1D* cloud = 
      new BatchLab::Cloud1D(printer,xmlName,stitle,maxEntries);
    object = cloud;

    // Data sub items :
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      if(!readCloudData(*(*it),*cloud,aVerboseLevel,aContext.session())) {
        delete cloud;
        return 0;
      }
    }
  
  } else if(aDimension==2) {

    BatchLab::Cloud2D* cloud = 
      new BatchLab::Cloud2D(printer,xmlName,stitle,maxEntries);
    object = cloud;

    // Data sub items :
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      if(!readCloudData(*(*it),*cloud,aVerboseLevel,aContext.session())) {
        delete cloud;
        return 0;
      }
    }
  
  } else if(aDimension==3) {

    BatchLab::Cloud3D* cloud = 
      new BatchLab::Cloud3D(printer,xmlName,stitle,maxEntries);
    object = cloud;

    // Data sub items :
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      if(!readCloudData(*(*it),*cloud,aVerboseLevel,aContext.session())) {
        delete cloud;
        return 0;
      }
    }
  
  }

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Cloud :" 
        << " name " << Lib::sout(xmlName)
        << " done." << Lib::endl;
  }

  aPath = spath;  

  return object;
}
//////////////////////////////////////////////////////////////////////////////
bool readCloudData(
 Lib::ItemML& aItemML                      
,BatchLab::Cloud1D& aCloud
,int aVerboseLevel
,Slash::Core::ISession& aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="entries1d") {
 
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="entry1d") {
        double x;
        if(!(*it)->attributeValue("valueX",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,x)) return false;
        double w = 1;
        if((*it)->attributeValue("weight",svalue)) {
          if(!Lib::smanip::todouble(svalue,w)) return false;
        }
        if(!aCloud.fill(x,w)) return false;
      }
    }
    return true;

  } else if(tagName=="histogram1d") {

    BatchLab::XML::ReadContext ctx(aVerboseLevel,aSession,aItemML);
    std::string spath;
    Slash::Store::IObject* object = BatchLab::XML::read_Histogram1D(ctx,spath);
    if(!object) return false;
    if(BatchLab::Histogram1D* h = Lib_Cast(*object,BatchLab::Histogram1D)) {
      aCloud.setHistogram(h);
    }
    return true;

  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool readCloudData(
 Lib::ItemML& aItemML                      
,BatchLab::Cloud2D& aCloud
,int aVerboseLevel
,Slash::Core::ISession& aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="entries2d") {
 
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="entry2d") {
        double x;
        if(!(*it)->attributeValue("valueX",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,x)) return false;
        double y;
        if(!(*it)->attributeValue("valueY",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,y)) return false;
        double w = 1;
        if((*it)->attributeValue("weight",svalue)) {
          if(!Lib::smanip::todouble(svalue,w)) return false;
        }
        if(!aCloud.fill(x,y,w)) return false;
      }
    }
    return true;

  } else if(tagName=="histogram2d") {

    BatchLab::XML::ReadContext ctx(aVerboseLevel,aSession,aItemML);
    std::string spath;
    Slash::Store::IObject* object = BatchLab::XML::read_Histogram2D(ctx,spath);
    if(!object) return false;
    if(BatchLab::Histogram2D* h = Lib_Cast(*object,BatchLab::Histogram2D)) {
      aCloud.setHistogram(h);
    }
    return true;

  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool readCloudData(
 Lib::ItemML& aItemML                      
,BatchLab::Cloud3D& aCloud
,int aVerboseLevel
,Slash::Core::ISession& aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="entries3d") {
 
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="entry3d") {
        double x;
        if(!(*it)->attributeValue("valueX",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,x)) return false;
        double y;
        if(!(*it)->attributeValue("valueY",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,y)) return false;
        double z;
        if(!(*it)->attributeValue("valueZ",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,z)) return false;
        double w = 1;
        if((*it)->attributeValue("weight",svalue)) {
          if(!Lib::smanip::todouble(svalue,w)) return false;
        }
        if(!aCloud.fill(x,y,z,w)) return false;
      }
    }
    return true;

  } else if(tagName=="histogram3d") {

    BatchLab::XML::ReadContext ctx(aVerboseLevel,aSession,aItemML);
    std::string spath;
    Slash::Store::IObject* object = BatchLab::XML::read_Histogram3D(ctx,spath);
    if(!object) return false;
    if(BatchLab::Histogram3D* h = Lib_Cast(*object,BatchLab::Histogram3D)) {
      aCloud.setHistogram(h);
    }
    return true;

  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_DataPointSet(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int aVerboseLevel = aContext.verboseLevel();
  Slash::Core::IWriter& printer = aContext.session().printer();
  Lib::ItemML& aItemML = aContext.itemML();                      

  aPath = "";

  std::string xmlName = buildName(aItemML);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_DataPointSet :" 
        << " name " << Lib::sout(xmlName)
        << "..." << Lib::endl;
  }

  std::string spath;
  aItemML.attributeValue("","path",spath);

  std::string stitle;
  aItemML.attributeValue("","title",stitle);

  std::string svalue;
  int ival;

  // Booking parameters :
  int dimension = 0;
  if(!aItemML.attributeValue("","dimension",svalue)) return 0;
  if(!Lib::smanip::toint(svalue,ival)) return 0;
  dimension = ival;

  // Create a BatchLab::DataPointSet :
  BatchLab::DataPointSet* object = 
    new BatchLab::DataPointSet(printer,xmlName,stitle,dimension);

  // Data sub items :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!readDataPointSetData(*(*it),*object,printer)) {
      delete object;
      return 0;
    }
  }
  
  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::dataPointSet :" 
        << " name " << Lib::sout(xmlName)
        << " done." << Lib::endl;
  }

  aPath = spath;  

  return object;
}
//////////////////////////////////////////////////////////////////////////////
bool readDataPointSetData(
 Lib::ItemML& aItemML                      
,AIDA::IDataPointSet& aObject
,Slash::Core::IWriter& //aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="dataPoint") {
 
    AIDA::IDataPoint* point = aObject.addPoint();
    if(!point) return false;

    int coordinate = 0;
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="measurement") {
        if(coordinate>=aObject.dimension()) return false;
        double value;
        if(!(*it)->attributeValue("value",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,value)) return false;
        double errorPlus = 0;
        if((*it)->attributeValue("errorPlus",svalue)) {
          if(!Lib::smanip::todouble(svalue,errorPlus)) return false;
        }
        double errorMinus = 0;
        if((*it)->attributeValue("errorMinus",svalue)) {
          if(!Lib::smanip::todouble(svalue,errorMinus)) return false;
        }

        AIDA::IMeasurement* measurement = point->coordinate(coordinate);
        if(!measurement) return false;
        measurement->setValue(value);
        measurement->setErrorPlus(errorPlus);
        measurement->setErrorMinus(errorMinus);

        coordinate++;
      }
    }
    return true;

  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Store::IObject* BatchLab::XML::read_Function(
 BatchLab::XML::ReadContext& aContext
,std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int aVerboseLevel = aContext.verboseLevel();
  Slash::Core::IWriter& printer = aContext.session().printer();
  Lib::ItemML& aItemML = aContext.itemML();                      

  aPath = "";

  std::string xmlName = buildName(aItemML);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Function :" 
        << " name " << Lib::sout(xmlName)
        << "..." << Lib::endl;
  }

  std::string spath;
  aItemML.attributeValue("","path",spath);

  std::string stitle;
  aItemML.attributeValue("","title",stitle);

  // Booking parameters :
  /*FIXME
  bool isNormalized = false;
  if(aItemML.attributeValue("","isNormalized",svalue)) {
    if(!Lib::smanip::tobool(svalue,isNormalized)) return 0;
    }*/

  std::string codelet;
  if(!aItemML.propertyValue("codelet",codelet)) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Function :" 
        << " for " << Lib::sout(xmlName)
        << ", no codelet found." << Lib::endl;
    return 0;
  }

  FunctionBooking booking;
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!readFunctionBooking(*(*it),booking,printer)) {
      Lib::Out out(printer);
      out << "BatchLab::XML::read_Function :" 
          << " for " << Lib::sout(xmlName)
          << ", cant' read booking." << Lib::endl;
      return 0;
    }
  }
  
  Lib::smanip::strip(codelet);
  BatchLab::BaseFunction* object = 
    BatchLab::FFactory::create(aContext.session(),xmlName,stitle,codelet);
  if(!object) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Function :" 
        << " can't create function with codelet " 
        << Lib::sout(codelet) << "."
        << Lib::endl;
    return 0;
  }

  std::vector<std::string> params = object->parameterNames();
  if(params.size()!=booking.fParameters.size()) {
    Lib::Out out(printer);
    out << "BatchLab::XML::read_Function :" 
        << " for function " << Lib::sout(xmlName)
        << " parameter number mismatch."
        << " " << (unsigned int)params.size() << " expected." 
        << " " << (unsigned int)booking.fParameters.size() << " given."
        << Lib::endl;
    delete object;
    return 0;
  }

  // Find list of parameters values :
  std::vector<double> values;
  for(unsigned int index=0;index<params.size();index++) {
    bool found = false;
    for(unsigned int i=0;i<booking.fParameters.size();i++) {
      if(booking.fParameters[i].first==params[index]) {
        values.push_back(booking.fParameters[i].second);
        found = true;
        break;
      }
    }
    if(!found) {
      Lib::Out out(printer);
      out << "BatchLab::XML::read_Function :" 
          << " for function " << Lib::sout(xmlName)
          << " parameter " << Lib::sout(params[index]) << " not found."
          << Lib::endl;
      delete object;
      return 0;
    }
  }

  object->setParameters(values);

  if(aVerboseLevel) {
    Lib::Out out(printer);
    out << "BatchLab::XML::readFunction :" 
        << " name " << Lib::sout(xmlName)
        << " done." << Lib::endl;
  }

  aPath = spath;  

  return object;
}
//////////////////////////////////////////////////////////////////////////////
bool readFunctionBooking(
 Lib::ItemML& aItemML                      
,FunctionBooking& aBooking
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();

  std::string svalue;

  if(tagName=="annotation") { //FIXME

    return true;
 
  } else if(tagName=="arguments") {

    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      if(!readFunctionBooking(*(*it),aBooking,aPrinter)) return false;
    }
    return true;
  
  } else if(tagName=="argument") {

    if(!aItemML.attributeValue("","name",aBooking.fArgument)) return false;

    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="range") {
        double mn = 0;        
        if((*it)->attributeValue("min",svalue)) {
          if(!Lib::smanip::todouble(svalue,mn)) return false;
        }
        double mx = 0;        
        if((*it)->attributeValue("max",svalue)) {
          if(!Lib::smanip::todouble(svalue,mx)) return false;
        }
        aBooking.fRanges.push_back(std::pair<double,double>(mn,mx));
      }
    }
    return true;

  } else if(tagName=="parameters") {
 
    const std::list<Lib::Element*> props = aItemML.properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if((*it)->name()=="parameter") {
        std::string name;
        if(!(*it)->attributeValue("name",name)) return false;
        double value;
        if(!(*it)->attributeValue("value",svalue)) return false;
        if(!Lib::smanip::todouble(svalue,value)) return false;
        aBooking.fParameters.push_back
          (std::pair<std::string,double>(name,value));
      }
    }
    return true;

  }

  return false;
}
