//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
// $Id: main_A01.cc,v 1.1 2007/03/07 15:04:12 barrand Exp $
// --------------------------------------------------------------
//
// --------------------------------------------------------------
//      GEANT 4 - A01app
// --------------------------------------------------------------
// Comments
//   Tutorial for Geant4 lectures
// --------------------------------------------------------------


#include "G4RunManager.hh"
#include "G4UImanager.hh"
//UIOnX #include "G4UIterminal.hh"
//UIOnX #include "G4UItcsh.hh"
// G4Lab :
#include <G4Lab/UIOnX.h> //UIOnX
#include <G4Lab/Helpers.h> //UIOnX
#include "Environment.h"
//#include <Lib/Debug.h>

#include "A01DetectorConstruction.hh"
#include "A01PhysicsList.hh"
#include "A01PrimaryGeneratorAction.hh"

//#include "A01TrackingAction.hh"

#include "A01EventAction.hh"

#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif

#ifdef G4ANALYSIS_USE
#include "A01AnalysisManager.hh"
#endif

int main(int argc,char** argv)
{
  if(argc<=0) return 1;
  if(!check_envs(argv[0])) return 1;

  // RunManager construction
  G4RunManager* runManager = new G4RunManager;

#ifdef G4VIS_USE
  // Visualization manager construction
  G4VisManager* visManager = new G4VisExecutive;
  visManager->Initialize();
#endif

#ifdef G4ANALYSIS_USE
  //AIDA::IAnalysisFactory* aida = AIDA_createAnalysisFactory();
  //A01AnalysisManager::setAIDA(aida);
#endif

  //Lib::Debug::checkByClass(true);
  G4Lab::UIOnX* session = 0;
  if(argc<=1)  {
    std::string gui = "$G4EXAMPLESROOT/scripts/OnX/G4Examples.onx"; //UIOnX
    session = new G4Lab::UIOnX(*runManager,gui,argc,argv); //UIOnX
    // Compell the GUI creation here.
    // Else, the A01EventAction, by creating a plotter, will compell
    // a GUI which is not the G4Examples.onx one.
    session->createUI();
#ifdef G4ANALYSIS_USE
    A01AnalysisManager::setAIDA(session->findAIDA());
#endif
  }

  // mandatory user initialization classes
  runManager->SetUserInitialization(new A01DetectorConstruction);
  runManager->SetUserInitialization(new A01PhysicsList);

  // initialize Geant4 kernel
  runManager->Initialize();

  // mandatory user action class
  runManager->SetUserAction(new A01PrimaryGeneratorAction);

  // optional user action classes
  runManager->SetUserAction(new A01EventAction);
  //runManager->SetUserAction(new A01TrackingAction);

  if(argc>1)
  // execute an argument macro file if exist
  {
    G4UImanager* UImanager = G4UImanager::GetUIpointer();
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    UImanager->ApplyCommand(command+fileName);
  }
  else
  // start interactive session
  {
    //UIOnX G4UIsession* session = new G4UIterminal();
    session->SessionStart();
    //UIOnX delete session;
  }

#ifdef G4VIS_USE
  delete visManager;
#endif

  delete runManager;

  delete session; //UIOnX

#ifdef G4ANALYSIS_USE
  //UIOnX delete aida;
#endif

  return 0;
}

