//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(IUI&);
//     void callback_with_arguments(IUI&,const std::vector<std::string>&);
//   }
//

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/ISoViewer.h>

// Lib :
#include <Lib/System.h>
#include <Lib/Manager.h>
#include <Lib/Printer.h>
#include <Lib/Out.h>

// G4Lab :
#include <G4Lab/Interfaces/IGeant4Manager.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// G4Examples/scripts/OnX/G4Examples.onx //////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4Examples_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) {
    soViewer->setDecoration(true);
    soViewer->setViewing(true);
  }

  if(!aUI.setCallback("OnX_File_exit","activate","DLD","G4Lab G4Lab_exit")) {
    aUI.echo("G4Examples_Initialize : OnX_File_exit set callback failed.");
  }

  Slash::Core::ISession& session = aUI.session();

  session.setParameter("modeling.highlightColor","white");
  session.setParameter("modeling.trajectories","immediate_all");
  session.setParameter("event.scan","yes");
    
  IGeant4Manager* g4Manager = 
    Lib_findManager(session,"Geant4Manager",IGeant4Manager);
  if(!g4Manager) {
    aUI.echo("G4Examples_Initialize : Geant4Manager not found.");
  } else {
    // Declare a callback executed at each end of event :
    g4Manager->setEventEndScript("DLD","G4Examples G4Examples_eventEnd");
    //Declare G4Lab types (PV, Trajectory, HitsCollections, etc...).
    g4Manager->initialize(); 
  }
}
//////////////////////////////////////////////////////////////////////////////
void G4Examples_Finalize(
 Slash::UI::IUI&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}

} // extern "C"
