//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(IUI&);
//     void callback_with_arguments(IUI&,const std::vector<std::string>&);
//   }
//


#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/ISoViewer.h>

// Lib :
#include <Lib/System.h>
#include <Lib/Manager.h>
#include <Lib/Printer.h>
#include <Lib/Out.h>

// G4Lab :
#include <G4Lab/Interfaces/IGeant4Manager.h>
extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// G4Lab/scripts/OnX/Default.onx ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4LabDefault_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) {
    soViewer->setDecoration(true);
    soViewer->setViewing(true);
  }

  if(!aUI.setCallback("OnX_File_exit","activate","DLD","G4Lab G4Lab_exit")) {
    aUI.echo("G4LabDefault_Initialize : OnX_File_exit set callback failed.");
  }

  Slash::Core::ISession& session = aUI.session();

  session.setParameter("modeling.highlightColor","white");
  session.setParameter("modeling.trajectories","immediate_all");
  session.setParameter("event.scan","yes");
    
  IGeant4Manager* g4Manager = 
    Lib_findManager(session,"Geant4Manager",IGeant4Manager);
  if(!g4Manager) {
    aUI.echo("G4LabDefault_Initialize : Geant4Manager not found.");
  } else {
    // Declare a callback executed at each end of event :
    g4Manager->setEventEndScript("DLD","G4LabDefault G4LabDefault_eventEnd");
    //Declare G4Lab types (PV, Trajectory, HitsCollections, etc...).
    g4Manager->initialize(); 
  }
}
//////////////////////////////////////////////////////////////////////////////
void G4LabDefault_Finalize(
 Slash::UI::IUI&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}

}

#include <Lib/mmanip.h>

#include <Inventor/nodes/SoOrthographicCamera.h>

// HEPVis :
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
// Inventor related callbacks ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4LabDefault_eventEnd(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  soRegion->clear("dynamicScene");

  data_collect(session,"Trajectory");
  data_visualize(session);

  std::string svalue;
  if(!session.parameterValue("modeling.trajectories",svalue)) return;

  if( (svalue=="immediate_all") || 
      (svalue=="immediate_time") ||
      (svalue=="pickable") ) {
    std::string event_scan;
    session.parameterValue("event.scan",event_scan);
    if(event_scan=="yes") {
      //Enter secondary interactive loop to
      // play with the event :
      aUI.echo("Play with the event...");
      aUI.steer();
    } else {
      aUI.synchronize();
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void G4LabDefault_Page_create(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  soPage->deleteRegions();
  soPage->titleVisible.setValue(FALSE);
  soPage->title.setValue("Default");
  soPage->createRegions("SoDisplayRegion",1,1,0);

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  soRegion->color.setValue(SbColor(0,0,0));
}


} // extern "C"
