/*
   Example of a program started "a la Geant4 examples way".
  OnX is attached through the G4Lab::UIOnX (which inherits G4UISession).
 */

// Geant4 :
#include <G4RunManager.hh>
#include <G4UserSteppingAction.hh>

// G4Lab :
#include <G4Lab/UIOnX.h>
#include <G4Lab/TrackingAction.h>

// G4Simple :
#include <G4Simple/Detector.h>
#include <G4Simple/Primary.h>
#include <G4Simple/PhysicsList.h>
#include <G4Simple/RunAction.h>
#include <G4Simple/EventAction.h>
#include <G4Simple/SteppingAction.h>

// AIDA :
#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/Manager.h>
#include <Lib/Printer.h>
#include <Lib/argZero.h>

#include <Lib/dirmanip.h>
#include <Lib/smanip.h>

//////////////////////////////////////////////////////////////////////////////
int main(
 int aArgc
,char** aArgv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgc<=0) return 1;

  // The below Lib::argZero is only for
  // applications coming with the OSC distrib.
  if(!Lib::argZero::checkPackEnv(aArgv[0],"G4Lab")) return 1;
  if(!Lib::argZero::checkPackEnv(aArgv[0],"G4Simple")) return 1;
  bool is_Mac_app;
  if(!Lib::argZero::startMacApp(aArgv[0],"G4Simple",is_Mac_app)) return 1;

  G4RunManager* runManager = new G4RunManager;

  runManager->SetUserInitialization(new G4Simple::Detector);
  runManager->SetUserInitialization(new G4Simple::PhysicsList); 
  runManager->SetUserAction(new G4Simple::PrimaryGeneratorAction);
  runManager->SetUserAction(new G4Simple::EventAction);
  runManager->SetUserAction(new G4Lab::TrackingAction);

  // Need to pass the G4RunManager at UIOnX creation
  // so that the G4Lab::Manager get it properly.

  std::string gui = "$G4SIMPLEROOT/scripts/OnX/G4Simple_session.onx";
  G4Lab::UIOnX* session = new G4Lab::UIOnX(*runManager,gui,aArgc,aArgv);

  runManager->SetUserAction(new G4Simple::RunAction(session->printer()));

  AIDA::IAnalysisFactory* aida = session->findAIDA();
  runManager->SetUserAction(new G4Simple::SteppingAction(aida));

  // Main UI loop :
  // GUI is constructed here.
  // Create callbacks are executed here.
  if(is_Mac_app) session->createUI("Qt");

  session->SessionStart();

  delete session;

  delete runManager;

  return 0;
}

