//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

//
// This file is used when running from the OnX_onx program.
//


#include <BatchLab/Helpers/AIDA.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>

// Lib :
#include <Lib/Manager.h>
#include <Lib/Out.h>

// Geant4 :
#include <G4RunManager.hh>

// G4Lab :
#include <G4Lab/Manager.h>
#include <G4Lab/TrackingAction.h>

// G4Simple :
#include <G4Simple/Detector.h>
#include <G4Simple/Primary.h>
#include <G4Simple/PhysicsList.h>
#include <G4Simple/RunAction.h>
#include <G4Simple/EventAction.h>
#include <G4Simple/SteppingAction.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// G4Simple/scripts/OnX/G4Simple.onx ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4SimpleOnX_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  if(Lib_findManager(session,"Geant4Manager",IGeant4Manager)) return;

  G4RunManager* runManager = new G4RunManager;

  // The last "true" argument is to ask to the G4Lab::~Manager to 
  // delete the passed G4RunManager.
  G4Lab::Manager* g4Manager = 
    new G4Lab::Manager(session,"Geant4Manager",runManager,true);
  session.addManager(g4Manager);

  runManager->SetUserInitialization(new G4Simple::Detector);
  runManager->SetUserInitialization(new G4Simple::PhysicsList); 
  runManager->SetUserAction(new G4Simple::PrimaryGeneratorAction);
  runManager->SetUserAction(new G4Simple::EventAction);
  runManager->SetUserAction(new G4Lab::TrackingAction);

  // Actions that need some UI or analysis facilites :
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) {
    Lib::Out out(session.printer());
    out << "G4Simple::Manager : AIDA not found." << Lib::endl;
  }

  runManager->SetUserAction(new G4Simple::SteppingAction(aida));    
  runManager->SetUserAction(new G4Simple::RunAction(&(session.printer())));

  runManager->Initialize();

  // If having HitsCollections, and in order to declare them to the 
  // type manager, the below must be done after having declared 
  // the Geant4 user classes to the G4RunManager
  // and having initialized the Geant4 run manager.
  g4Manager->initialize();

}
//////////////////////////////////////////////////////////////////////////////
void G4SimpleOnX_Finalize(
 Slash::Core::ISession&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // At this moment the Geant4Manager is already deleted.
}

} // extern "C"
