// This :
#include <G4Simple/CellHit.h>

// Lib :
#include <Lib/Debug.h>
 
// Geant4 :
#include <G4Allocator.hh>
#include <G4UnitsTable.hh>
#include <G4ios.hh>
#include <G4AttDefStore.hh>
#include <G4AttDef.hh>
#include <G4AttValue.hh>

static G4Allocator<G4Simple::CellHit> sHitAllocator; //Beurk.

void* G4Simple::CellHit::operator new(size_t) {
  return (void*)sHitAllocator.MallocSingle();
}

void G4Simple::CellHit::operator delete(void* aHit)
{
  sHitAllocator.FreeSingle((G4Simple::CellHit*)aHit);
}

G4Simple::CellHit::CellHit():fLV(0),fEnergy(0){
  Lib::Debug::increment("G4Simple::CellHit");
}

G4Simple::CellHit::CellHit(const G4Simple::CellHit& aFrom):G4VHit() {
  fEnergy = aFrom.fEnergy;
  fLV = aFrom.fLV;
  fTSF = aFrom.fTSF;
  Lib::Debug::increment("G4Simple::CellHit");
}

G4Simple::CellHit::~CellHit() {
  Lib::Debug::decrement("G4Simple::CellHit");
}

const G4Simple::CellHit& G4Simple::CellHit::operator=(
  const G4Simple::CellHit& aFrom) {
  fEnergy = aFrom.fEnergy;
  fLV = aFrom.fLV;
  fTSF = aFrom.fTSF;
  return *this;
}

G4int G4Simple::CellHit::operator==(const G4Simple::CellHit& aFrom) const {
  return (this==&aFrom) ? 1 : 0;
}

void G4Simple::CellHit::Draw() {}

void G4Simple::CellHit::Print() {
  G4cout << " Energy " << G4BestUnit(fEnergy,"Energy") << G4endl;
}

const std::map<G4String,G4AttDef>* G4Simple::CellHit::GetAttDefs() const {
  G4bool isNew;
  std::map<G4String,G4AttDef>* attDefs = 
    G4AttDefStore::GetInstance("CellHit",isNew);

  if (isNew) {    
    G4String Energy("Energy");
    (*attDefs)[Energy] = G4AttDef(Energy,"Energy","Dump","","G4double");    

    G4String LV("LV");
    (*attDefs)[LV] = 
      G4AttDef(LV,"Logical Volume","Display","","G4LogicalVolume");

    G4String TSF("TSF");
    (*attDefs)[TSF] = 
      G4AttDef(TSF,"3D Transformation","Display","","G4Transform3D");
 
  }

  return attDefs;
}

std::vector<G4AttValue>* G4Simple::CellHit::CreateAttValues() const {

  std::vector<G4AttValue>* values = new std::vector<G4AttValue>;

  std::ostringstream os;
  os.str("");
  os << fEnergy;
  values->push_back(G4AttValue("Energy",os.str(),""));
    
  os.str("");
  os << fLV;
  values->push_back(G4AttValue("LV",os.str(),""));

  os.str("");
  os << &fTSF;
  values->push_back(G4AttValue("TSF",os.str(),""));

  return values;
}

void G4Simple::CellHit::SetLV(G4LogicalVolume* aLV) { 
  fLV = aLV; 
}
void G4Simple::CellHit::SetTSF(const G4Transform3D& aTSF) { 
  fTSF = aTSF; 
}
void G4Simple::CellHit::SetEnergy(G4double aEnergy) {
  fEnergy = aEnergy;
}
G4double G4Simple::CellHit::GetEnergy() const {
  return fEnergy;
}
