// This :
#include <G4Simple/CellSD.h>

// Geant4 :
#include <G4HCofThisEvent.hh>
#include <G4Step.hh>
#include <G4ThreeVector.hh>
#include <G4SDManager.hh>

// G4Simple :
#include <G4Simple/CellHit.h>

G4Simple::CellSD::CellSD()
:G4VSensitiveDetector("CellSD")
,fCellHits(0)
,fCellHC_ID(-1)
{
  collectionName.insert("CellHC");
}

G4Simple::CellSD::~CellSD(){ 
}

void G4Simple::CellSD::Initialize(G4HCofThisEvent*)
{
  fCellHits = new CellHits(SensitiveDetectorName,collectionName[0]); 
}

G4bool G4Simple::CellSD::ProcessHits(G4Step* aStep,G4TouchableHistory*)
{
  G4Track* currentTk = aStep->GetTrack();
  if (!currentTk) return false;

  //Get PreStep point to test boundary
  G4StepPoint* point = aStep->GetPreStepPoint();
  if (!point) return false;

  // Get PhysicalVolume :  
  G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  G4TouchableHandle touchable = preStepPoint->GetTouchableHandle();
  G4VPhysicalVolume* pv = touchable->GetVolume();
  G4AffineTransform trans = touchable->GetHistory()->GetTopTransform();
  G4RotationMatrix rot = trans.NetRotation();
  trans.Invert();
  G4ThreeVector pos = trans.NetTranslation();
  G4Transform3D tsf(rot,pos);

  if (point->GetStepStatus() == fGeomBoundary) {
    //particle enter the volume => new hit
    CellHit* hit = new CellHit();
    //hit->SetTrackID    (currentTk->GetTrackID());
    //hit->SetTrackPdgID (currentTk->GetDefinition()->GetPDGEncoding());
    hit->SetEnergy(aStep->GetTotalEnergyDeposit());
    //hit->SetPos(aStep->GetPreStepPoint()->GetPosition());
    
    hit->SetLV(pv->GetLogicalVolume());
    hit->SetTSF(tsf);

    fCellHits->insert(hit);
  }
  
  return true;
}

void G4Simple::CellSD::EndOfEvent(G4HCofThisEvent* aHCE) {
  
  if(verboseLevel>0) PrintAll();

  if(fCellHC_ID<0) { 
    fCellHC_ID = 
      G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]); 
  }

  aHCE->AddHitsCollection( fCellHC_ID,fCellHits); 
  fCellHits = 0;
}

void G4Simple::CellSD::PrintAll() {
  
  G4int NbHits = fCellHits->entries();
  G4cout << "--------<"<< GetName() << "> Hits Collection contains " 
         << NbHits << " hits " << G4endl;
  for (G4int i=0;i<NbHits;i++) {
    CellHit* hit = (*fCellHits)[i];
    hit->Print();
  }
}

