//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
// HEPVis and Inventor related callbacks.
//

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include <Inventor/nodes/SoOrthographicCamera.h>

// HEPVis :
#include <HEPVis/SbMath.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void G4Simple_eventEnd(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  soRegion->clear("dynamicScene");

  data_collect(session,"CellHC","");
  data_visualize(session);

  // action could be customised through the InputNextEventAction dialog.
  std::string action;
  if(!session.parameterValue("g4Lab.nextEventAction",action)) return;

  if(action=="Trajectories_all_immediate") {
    session.setParameter("modeling.trajectories","immediate_all");

    data_collect(session,"Trajectory");
    data_visualize(session);

  } else if(action=="Trajectories_all_pickable") {
    session.setParameter("modeling.trajectories","pickable");

    data_collect(session,"Trajectory");
    data_visualize(session);

  } else if(action=="Trajectories_leptons") {
    session.setParameter("modeling.trajectories","pickable");

    data_collect(session,"Trajectory","particle==\"gamma\"");
    data_visualize(session);
    data_collect(session,"Trajectory","particle==\"e-\"");
    data_visualize(session);
    data_collect(session,"Trajectory","particle==\"e+\"");
    data_visualize(session);

    // Fill histos with collected things 
    // by using the Lab data_fill1D facility :
    data_collect(session,"Trajectory","particle==\"e-\"");
    //FIXME data_fill1D("h_e_E","energy");
    // Geant4 energy unit is MeV.
    data_collect(session,"Trajectory",
                 "((particle==\"e-\") && (energy>10)) && (energy<=20)");
    //FIXME data_fill1D("h_e_E_cut","energy");

  } else if(action=="Trajectories_baryons") {
    session.setParameter("modeling.trajectories","pickable");

    data_collect(session,"Trajectory","particle==\"pi+\"");
    data_visualize(session);

    data_collect(session,"Trajectory","particle==\"pi-\"");
    data_visualize(session);

  } else if(action=="none") {
  }

  std::string svalue;
  if(!session.parameterValue("modeling.trajectories",svalue)) return;

  if( (svalue=="immediate_all") || 
      (svalue=="immediate_time") ||
      (svalue=="pickable") ) {
    std::string event_scan;
    session.parameterValue("event.scan",event_scan);
    if(event_scan=="yes") {
      //Enter secondary interactive loop to
      // play with the event :
      aUI.echo("Play with the event...");
      aUI.steer();
    } else {
      aUI.synchronize();
    }
  }

}
//////////////////////////////////////////////////////////////////////////////
void G4Simple_Page_create(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  soPage->deleteRegions();
  soPage->titleVisible.setValue(FALSE);
  soPage->title.setValue("G4Simple");
  soPage->createRegions("SoDisplayRegion",1,1,0);

  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  soRegion->color.setValue(SbColor(0,0,0));
}

//////////////////////////////////////////////////////////////////////////////
static void InputLayout_side_front(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //////////////////////////////////////////////////////////////////////////
  // Prepare the page :
  //////////////////////////////////////////////////////////////////////////
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  soPage->deleteRegions();
  soPage->titleVisible.setValue(FALSE);
  soPage->createRegions("SoDisplayRegion",1,2,1);

  // First region ; side view :
  soPage->setCurrentRegion(0);
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  SoRegion* fromRegion = soRegion;
  soRegion->color.setValue(SbColor(0,0,0));

  SoCamera* soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(-5000,0,0);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),-1.57F));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  // Second region ; front view :
  soPage->setCurrentRegion(1);
  soRegion = soPage->currentRegion();
  if(!soRegion) return;
  soRegion->color.setValue(SbColor(0,0,0));
  soRegion->connectFrom(fromRegion);

  soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(0,0,5000);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  ////////////////////////////////////////////////////////////////////////////
  // Visualize :
  ////////////////////////////////////////////////////////////////////////////

  soPage->setCurrentRegion(0);

  Slash::Core::ISession& session = aUI.session();

  data_collect(session,"PV","name==\"BarrelPV\"");
  data_visualize(session);

  data_collect(session,"PV","name==\"LeftCapPV\"");
  data_visualize(session);
}
//////////////////////////////////////////////////////////////////////////////
static void InputLayout_CAD(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //////////////////////////////////////////////////////////////////////////
  // Prepare the page :
  //////////////////////////////////////////////////////////////////////////
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  soPage->deleteRegions();
  soPage->titleVisible.setValue(FALSE);
  soPage->createRegions("SoDisplayRegion",2,2,1);

  // Connect the second, third, fourth region to the first one.

  // First region ; side view, +z at right :
  soPage->setCurrentRegion(0);
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  SoRegion* fromRegion = soRegion;
  soRegion->color.setValue(SbColor(0,0,0));

  SoCamera* soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(-5000,0,0);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),-FM_PI_2));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  // Second region ; front view :
  soPage->setCurrentRegion(1);
  soRegion = soPage->currentRegion();
  if(!soRegion) return;
  soRegion->color.setValue(SbColor(0,0,0));
  soRegion->connectFrom(fromRegion);

  soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(0,0,5000);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  // Third region ; top view :
  soPage->setCurrentRegion(2);
  soRegion = soPage->currentRegion();
  if(!soRegion) return;
  soRegion->color.setValue(SbColor(0,0,0));
  soRegion->connectFrom(fromRegion);

  soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(0,5000,9);
  soCamera->orientation.setValue(SbRotation(SbVec3f(-1,-1,-1),2*FM_PI/3));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  // Fourth region ; 3D view :
  soPage->setCurrentRegion(3);
  soRegion = soPage->currentRegion();
  if(!soRegion) return;
  soRegion->color.setValue(SbColor(0,0,0));
  soRegion->connectFrom(fromRegion);

  soCamera = soRegion->getCamera();
  if(soCamera->isOfType(SoOrthographicCamera::getClassTypeId()))
    ((SoOrthographicCamera*)soCamera)->height.setValue(5000);
  soCamera->position.setValue(-296,225,-226);
  soCamera->orientation.setValue
    (SbRotation(SbVec3f(-0.147F,-0.92F,-0.361F),2.348F));
  soCamera->nearDistance.setValue(1);
  soCamera->farDistance.setValue(100000);
  soCamera->focalDistance.setValue(5000);

  ////////////////////////////////////////////////////////////////////////////
  // Visualize :
  ////////////////////////////////////////////////////////////////////////////

  soPage->setCurrentRegion(0);

  Slash::Core::ISession& session = aUI.session();

  data_collect(session,"PV","name==\"BarrelPV\"");
  data_visualize(session);

  data_collect(session,"PV","name==\"LeftCapPV\"");
  data_visualize(session);
}
//////////////////////////////////////////////////////////////////////////////
void G4Simple_InputLayout_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string layout;
  if(!aUI.parameterValue("G4Simple_InputLayout_input_layout.value",layout)) return;

  if(layout=="side_front") {
    InputLayout_side_front(aUI);
  } else if(layout=="CAD") {
    InputLayout_CAD(aUI);
  }
}

} // extern "C"
