#include <stdlib.h>
#include <math.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoElapsedTime.h>
#include <Inventor/engines/SoCalculator.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodes/SoArrow.h>

//
// Test of SoArrow class
// Joe Boudreau Sep 1997
//
// This demo draws a Arrow
//
 
int main(int, char ** argv) {
 // Put everything in a block for SunOS/CC memory decount :
 {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoArrow::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  // Set up the material for the object to be drawn

  SoMaterial    *RedPlastic = new SoMaterial;
  RedPlastic->diffuseColor.setValue(1, 0, 0);

  SoMaterial    *BluePlastic = new SoMaterial;
  BluePlastic->diffuseColor.setValue(0, 0, 1);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoElapsedTime *TheCounter = new SoElapsedTime;
  TheCounter->speed=3;

  SoCalculator *calc = new SoCalculator;
  calc->a.connectFrom(&TheCounter->timeOut);
  calc->expression.set1Value(0,"oA=vec3f(-.5+-.5*sin(a),0,0)");
  calc->expression.set1Value(1,"oB=vec3f(.5+.5*sin(a),0,0)");
  
  // Make the Arrow

  SoArrow *Arrow1  = new SoArrow();
  SoArrow *Arrow2  = new SoArrow();
  Arrow1->tail.setValue(-5, 0, 0);
  Arrow2->tail.setValue( 5, 0, 0);
  Arrow1->tip.setValue(0, 0, 0);
  Arrow2->tip.setValue(0, 0, 0);
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))  Arrow1->generateAlternateRep(0);
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))  Arrow2->generateAlternateRep(0);
  Arrow1->tip.connectFrom(&calc->oA);
  Arrow2->tip.connectFrom(&calc->oB);

  root->addChild(RedPlastic);
  root->addChild(Arrow1);
  root->addChild(BluePlastic);
  root->addChild(Arrow2);


  // illustrate how the size and shape of the "shaft" can be modified
  SoCalculator *calc2 = new SoCalculator;
  calc2->a.connectFrom(&TheCounter->timeOut);
  calc2->expression.set1Value(0,"oa=0.04+0.02*sin(0.2*a)");


  // make another two arrows
  SoArrow *Arrow3  = new SoArrow();
  Arrow3->tail.setValue(0.F, 6.F, 0.F);
  Arrow3->tip.setValue(0.F, 1.F, 0.F);
  Arrow3->conicalShaft.setValue(TRUE);
  Arrow3->size.connectFrom(&calc2->oa);
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))  Arrow3->generateAlternateRep(0);
  SoArrow *Arrow4  = new SoArrow();
  Arrow4->tail.setValue(0.F, -6.F, 0.F);
  Arrow4->tip.setValue(0.F, -1.F, 0.F);
  Arrow4->conicalShaft.setValue(TRUE);
  Arrow4->size.setValue(0.04F);
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))  Arrow4->generateAlternateRep(0);

  root->addChild(RedPlastic);
  root->addChild(Arrow3);
  root->addChild(BluePlastic);
  root->addChild(Arrow4);


  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  // View the new object in an Examiner Viewer

  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;}

#ifdef SoFreePackage //Cleanup
  SoArrow::clearClass();
  SoGui::clear();  
#endif

  return 0; 

}

