#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>

#include <HEPVis/SbArguments.h>
#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodekits/SoAxis.h>
#include <HEPVis/nodes/SoTextHershey.h>
#include <HEPVis/nodes/SoTextStyle.h>
#include <HEPVis/nodes/SoLineStyle.h>
#include <HEPVis/nodes/SoTextTTF.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

#include <stdlib.h> //exit
#include <iostream>

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbArguments args(aArgc,aArgv);

  if(args.isAnArgument("-h")) {
    std::cout << "HEPVis::Axis :" << std::endl;
    std::cout << "Usage :" << std::endl;
    std::cout << " OS> Axis<GUI> <options>" << std::endl;
    std::cout << "Options :" << std::endl;
    std::cout << " -h : to get help." << std::endl;
    std::cout << " -altrep : to produce an alternate representation." 
              << std::endl;
    std::cout << " -write : to write scene in a file (default is out.iv)." 
              << std::endl;
    std::cout << " -file : name of the file (with .iv or .wrl extension)." 
              << std::endl;
    std::cout << "For example :" << std::endl;
    std::cout << " OS> AxisXt -altrep -write" << std::endl;
    std::cout << " OS> ivview out.iv" << std::endl;
    return 0;
  }

  // Set up the main window
  SoGuiWidget topWidget=SoGui::init(aArgv[0]);
  if (!topWidget) exit(1);

  // initialize HEPVis used classes :
  SoTextHershey::initClass();
  SoTextTTF::initClass();
  SoTextStyle::initClass();
  SoLineStyle::initClass();
  SoAxis::initClass();
  SoAlternateRepAction::initClass();

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  SoAxis* axis = new SoAxis;
  //axis->minimumValue.setValue(0);
  //axis->maximumValue.setValue(1);
  //axis->verbose.setValue(TRUE);
  axis->title.setValue("An SoAxis");
  axis->getLineStyle()->color.setValue(SbColor(1,0,0));
  axis->getLabelsStyle()->color.setValue(SbColor(0,0,1));
  sceneGraph->addChild(axis);

  if (args.isAnArgument("-write")) {
    SbBool altRep = args.isAnArgument("-altrep")?TRUE:FALSE;
    SbString file;
    args.find("-file",file);
    if(file=="") file = "out.iv";
    SoTools::write(*sceneGraph,file,"",FALSE,altRep);
  }

  SoGuiExaminerViewer* examinerViewer = new SoGuiExaminerViewer(topWidget);
  examinerViewer->setBackgroundColor(SbColor(1,1,1));
  examinerViewer->setSceneGraph(sceneGraph);
  examinerViewer->show();

  SoGui::show(topWidget);
  SoGui::mainLoop();

  sceneGraph->unref();
  delete examinerViewer;

  return 0; 
}
