#include <stdlib.h>
#include <math.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoElapsedTime.h>
#include <Inventor/engines/SoCalculator.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodes/SoEllipticalPrism.h>

// Test of SoEllipticalPrism class
// Joe Boudreau Nov. 11, 1996
//
// This demo draws a EllipticalPrism
 
int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoEllipticalPrism::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  // Set up the material for the object to be drawn

  SoMaterial    *BluePlastic = new SoMaterial;

  BluePlastic->diffuseColor.setValue(0, 1, 1);
  root->addChild(BluePlastic);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoElapsedTime *TheCounter = new SoElapsedTime;

  SoCalculator *TheCalculator = new SoCalculator();
  TheCalculator->a.connectFrom(&TheCounter->timeOut);
  TheCalculator->expression.setValue("oa=4*sin(a)*sin(a)");

  // Make the EllipticalPrism

  SoEllipticalPrism *TheEllipticalPrism   = new SoEllipticalPrism();
  TheEllipticalPrism->eigenvalues.setValue(1,3);
  TheEllipticalPrism->extent.setValue(4);
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP")) TheEllipticalPrism->generateAlternateRep(0);
  root->addChild(TheEllipticalPrism);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  TheEllipticalPrism->extent.connectFrom(&TheCalculator->oa);  
  // View the new object in an Examiner Viewer

  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;

#ifdef SoFreePackage //Cleanup
  SoEllipticalPrism::clearClass();
  SoGui::clear();  
#endif

  return 0; 
}

