#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoDirectionalLight.h>

#include <HEPVis/nodes/SoViewportRegion.h>
#include <HEPVis/nodes/SoGrid.h>
#include <HEPVis/nodes/SoBackPrimitive.h>

#include <HEPVis/nodekits/SoShell.h>
#include <HEPVis/nodekits/SoFileSelectionBox.h>
#include <HEPVis/nodekits/SoArrowButton.h>
#include <HEPVis/nodekits/SoScrolledList.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

static void okCB(void*,SoNode*);
//////////////////////////////////////////////////////////////////////////////
int main(
 int
,char ** aArgv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoGuiWidget widget=SoGui::init(aArgv[0]);
  if(widget==NULL) exit(1);

  SoViewportRegion::initClass();
  SoGrid::initClass();
  SoBackPrimitive::initClass();

  SoPrimitive::initClass();
  SoShell::initClass();
  SoForm::initClass();
  SoArrowButton::initClass();
  SoList::initClass();
  SoScrolledList::initClass();
  SoFileSelectionBox::initClass();

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  SoViewportRegion* viewportRegion = new SoViewportRegion;
  sceneGraph->addChild(viewportRegion);

  SoLightModel* lightModel = new SoLightModel;
  lightModel->model.setValue(SoLightModel::PHONG);
  sceneGraph->addChild(lightModel);

  SoDirectionalLight* light = new SoDirectionalLight;
  light->color.setValue(1,1,1);
  light->direction.setValue(SbVec3f(1,-1,-1));
  sceneGraph->addChild(light);

  SoOrthographicCamera* camera = new SoOrthographicCamera;
  camera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA);
  camera->position.setValue(0,0,4);
  camera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
  camera->height.setValue(2);
  camera->nearDistance.setValue(1);
  camera->farDistance.setValue(100);
  camera->focalDistance.setValue(4);
  sceneGraph->addChild(camera);

  SoShell* shell = new SoShell;
  //shell->background.setValue(0.5F,0.5F,0.5F);
  sceneGraph->addChild(shell);

  SoFileSelectionBox* fileSelectionBox = new SoFileSelectionBox;
  fileSelectionBox->addOkCallback(okCB,NULL);
  shell->addChild(fileSelectionBox);

  SoGuiExaminerViewer *viewer=new SoGuiExaminerViewer(widget);
  viewer->setSize(SbVec2s(200,200));
  viewer->setSceneGraph(sceneGraph);
  viewer->setViewing(FALSE);
  viewer->setDecoration(FALSE);
  viewer->setHeadlight(FALSE);
  viewer->viewAll();
  viewer->show();

  SoGui::show(widget);
  SoGui::mainLoop();

  sceneGraph->unref();
  delete viewer;

  return 0; 
}
//////////////////////////////////////////////////////////////////////////////
static void okCB (
 void* /*aUserData*/
,SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoFileSelectionBox* fileSelectionBox = (SoFileSelectionBox*)aNode;
  if(fileSelectionBox==NULL) return;
  const SbString& file = fileSelectionBox->getSelectedFile();
  printf("Selected file : %s\n",file.getString());
}
