#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoMaterial.h>


#include <HEPVis/SbHatching.h>
#include <HEPVis/SbTessContour.h>
#include <HEPVis/ListContour.h>

#include <float.h>

#define LIST_SET(list,number,x,y,z) list[number].setValue((x),(y),(z))


#define DRAW_SURFACE_SB
//#define DRAW_CONTOUR_AFTER
#define DRAW_CONTOUR_HATCH

static float xmn = -10;
static float xmx = 10;
static float ymn = -10;
static float ymx = 10;
static float zmin = -10;

// Function to draw the contours :
// In case we are in a inside-border around limits given to CListContour
// we return a big lower value (something lower than function
// minimum within limits). We do so to help the Contour algorithm to 
// treat borders properly (mainly to always have closed contours).

static double func(double x,double y,void*) {
  double dx = (xmx-xmn)*0.01;
  double dy = (ymx-ymn)*0.01;
  if((x<(xmn+dx))|| 
     (x>(xmx-dx))||
     (y<(ymn+dy))||
     (y>(ymx-dy))
    ) return -FLT_MAX;
  // 0.7854 = pi/4
  return 10 * (cos(x+0.7854)+sin(y+0.7854));
}

int main(int, char ** argv) {
  
  // Set up the main window
  SoGuiWidget topWidget=SoGui::init(argv[0]);
  if (!topWidget) exit(1);
    
  SoSeparator* root = new SoSeparator;
  root->ref();
  
  SoLightModel* lightModel = new SoLightModel;
  lightModel->model.setValue(SoLightModel::BASE_COLOR);
  root->addChild(lightModel);
  
  unsigned int hatchMin= 0;
  unsigned int hatchMax= 20;
  unsigned int numberOfHatches = 200;
  float dz = 20;
  int zn = 20;

  double limits[4];
  limits[0] = xmn;
  limits[1] = xmx;    
  limits[2] = ymn;    
  limits[3] = ymx;    
  CListContour listContour;
  listContour.SetFieldFcn(func,(void*)0);

  listContour.SetLimits(limits);
  std::vector<double> zs(zn+1);
  float zmax = zmin + dz;
  double zd = (zmax-zmin)/zn;
  for(int zi=0;zi<=zn;zi++) zs[zi] = zmin + zi * zd;
  listContour.SetPlanes(zs);
  
  SbVec3f* listPoints = new SbVec3f[5000];
  SbVec3f* listClosePoints = new SbVec3f[500];
  
  listContour.Generate();
  
  
  if (!listContour.CompactStrips()) {
    printf("  compact strip failed\n");
    return false;
  }
  
  for (unsigned int hatchToDraw=hatchMin;hatchToDraw<=hatchMax;hatchToDraw++) {
    
    int noClosePoints =0;
    int noPoints =2; // because of hatch, we want to have the same
    // hatch vector for all planes, so we force the first 2 points to be //
    // in all levels

    SoSeparator* functionNode = new SoSeparator;
    root->addChild(functionNode);
        
    float planeHeight = (float)listContour.GetPlane(hatchToDraw);
    CLineStripList* pStripList = listContour.GetLines(hatchToDraw);
    if(!pStripList) break;

    unsigned int nbContours = 0;
    // check all lines and save already close ones
    std::vector<std::vector<SbVec3f> > contourVector;

    contourVector.resize(pStripList->size());
    
    float yContourOp = 
      float((limits[3]-limits[2])/(double)(listContour.GetRowSec()));
    float xContourOp = 
      float((limits[1]-limits[0])/(double)(listContour.GetColSec()));
    for (CLineStripList::iterator pos=pStripList->begin();
         pos != pStripList->end() ; pos++) {
      CLineStrip* pStrip = (*pos);
      if (pStrip->size() >2) {
        contourVector[nbContours].resize(0);
        contourVector[nbContours].clear();
        for (CLineStrip::iterator pos=pStrip->begin();pos != pStrip->end();pos++) {
          contourVector[nbContours].push_back(
            SbVec3f(
              float(limits[0]+(*pos)%(listContour.GetColSec()+1)*xContourOp),
              float(limits[2]+(*pos)/(listContour.GetColSec()+1)*yContourOp),
              planeHeight));
        }
        nbContours++;
      }
    }
#ifdef DRAW_SURFACE_SB
    //    printf("Draw surface with SbTessContour for %d/%d\n",hatchToDraw,hatchMax);
    SbTessContour* ct2 = new SbTessContour();
    std::vector<SbTessTriangle> tri = ct2->getFilledArea(contourVector);
    delete ct2;
    SoSeparator* filled = new SoSeparator; 
    functionNode->addChild(filled);
    
    SoMaterial* material4 =  new SoMaterial();
    material4->diffuseColor.setValue(0.25F+0.75F*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)), 0,0.5F+0.25F*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)));
    filled->addChild(material4);

    SoCoordinate3* coordinate3_surface = new SoCoordinate3;
    filled->addChild(coordinate3_surface);
    SoIndexedFaceSet* indexedFaceSet_surface = new SoIndexedFaceSet;
    filled->addChild(indexedFaceSet_surface);
    for (unsigned int a=0;a<tri.size();a++) {
      coordinate3_surface->point.set1Value(a*3,
                                   (float)tri[a].pointA[0]+(xmx-xmn)*1.5F,
                                   (float)tri[a].pointA[1],
                                   (float)tri[a].pointA[2]);
      coordinate3_surface->point.set1Value(a*3+1,
                                   (float)tri[a].pointB[0]+(xmx-xmn)*1.5F,
                                   (float)tri[a].pointB[1],
                                   (float)tri[a].pointB[2]);
      coordinate3_surface->point.set1Value(a*3+2,
                                   (float)tri[a].pointC[0]+(xmx-xmn)*1.5F,
                                   (float)tri[a].pointC[1],
                                   (float)tri[a].pointC[2]);
      indexedFaceSet_surface->coordIndex.set1Value(a*4,a*3);
      indexedFaceSet_surface->coordIndex.set1Value(a*4+1,a*3+1);
      indexedFaceSet_surface->coordIndex.set1Value(a*4+2,a*3+2);
      indexedFaceSet_surface->coordIndex.set1Value(a*4+3,SO_END_FACE_INDEX);
    }
#endif
    
    for(unsigned int a=0;a<contourVector.size();a++) {
      listClosePoints[noClosePoints] = contourVector[a][0].getValue();
      noClosePoints++;
      for(unsigned int b=0;b<contourVector[a].size();b++) {
        listPoints[noPoints] = contourVector[a][b].getValue();
        noPoints++;
      }
    }
    
    //close all lines
    for (int a=noClosePoints-2;a>=0;a--) {
      if (listPoints[noPoints-1] != listClosePoints[a].getValue()) {
        listPoints[noPoints] = listClosePoints[a].getValue();
        noPoints ++;
      }
    }
    
    
    if (noPoints >2) {
      // to have the same angle for all levels
      listPoints[0] = listPoints[2];
      listPoints[1] = listPoints[2]+SbVec3f(1.0,0.0,0.0); // still a bug here in hatch if we try
      // (x*x*sin(x)+y*sin(1.5*y)) in -10,10 x,y,z
      listPoints[noPoints] = listPoints[2]+SbVec3f(0.0,1.0,0.0);
      noPoints ++;

#ifdef DRAW_CONTOUR_HATCH
      //      printf("Draw hatch for %d/%d\n",hatchToDraw,hatchMax);

      SbHatching sbHatching;
      sbHatching.setSpacing((xmx-xmn)/numberOfHatches); 
      sbHatching.setOffset(0.0F); 
              
      int res = sbHatching.checkPolyline(listPoints,noPoints);
      if (res == TRUE) res = sbHatching.computePolyline(listPoints,noPoints);
      unsigned int numPoints = sbHatching.getNoPoints();
      unsigned int numVertices = sbHatching.getNoVertices();
      if((res==TRUE) && numPoints && numVertices) {
        
        SoSeparator* hatchSep = new SoSeparator(); 
        functionNode->addChild(hatchSep); 

        SoMaterial* material_Contour_Hatch =  new SoMaterial();
        material_Contour_Hatch->diffuseColor.setValue(0.5F+0.25F*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)),0.25F+0.75F*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)), 0);
        hatchSep->addChild(material_Contour_Hatch);
        
        // So that the below [Face,Line]Set find the right SoCoordinate3 :
        SoCoordinate3* coordinate3_hatch = new SoCoordinate3;
        hatchSep->addChild(coordinate3_hatch);
        
        int icoord = 0;
        SbVec3f* xyz = new SbVec3f[numPoints];
        const std::vector<SbVec3f>& hatchPointsList = sbHatching.getPoints();
        for (unsigned int a=0;a<numPoints;a++) xyz[a] = hatchPointsList[a];
        coordinate3_hatch->point.setValues(icoord,numPoints,xyz);
        delete [] xyz;
        
        int32_t* coord = new int32_t[numVertices*3];
        for (unsigned int a=0;a<numVertices;a++) {
          int ia = 3 * a;
          coord[ia] = icoord;
          icoord++;
          coord[ia+1] = icoord;
          icoord++;
          coord[ia+2] = SO_END_LINE_INDEX;
        }
        SoIndexedLineSet* indexedLineSet_hatch = new SoIndexedLineSet;
        hatchSep->addChild(indexedLineSet_hatch);
        indexedLineSet_hatch->coordIndex.setValues(0,numVertices*3,coord);
        delete [] coord;
      }
#endif
#ifdef DRAW_CONTOUR_AFTER

      //decal all in Z
      int* coord = new int[noPoints];

      for (int a=0;a<noPoints;a++) {
        coord[a] = a;
        //listPoints[a][0] -= (xmx-xmn)*1.5;
      }
      coord[noPoints] = SO_END_LINE_INDEX;
        
      SoSeparator* contourAfterSep = new SoSeparator; 
      functionNode ->addChild(contourAfterSep);

      SoMaterial* material3 =  new SoMaterial();
      material3->diffuseColor.setValue(0,  0.6+0.4*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)), 0.1+0.9*(((float)hatchToDraw-(float)hatchMin)/((float)hatchMax-(float)hatchMin)));
      contourAfterSep->addChild(material3);

      int32_t pointn = noPoints;
      SoCoordinate3* coordinate3_after = new SoCoordinate3;
      coordinate3_after->point.setValues(0,pointn,listPoints);
      contourAfterSep->addChild(coordinate3_after);

      SoIndexedLineSet* indexedLineSet_after = new SoIndexedLineSet;
      indexedLineSet_after->coordIndex.setValues(0,noPoints,coord);
      contourAfterSep->addChild(indexedLineSet_after);
      
      delete [] coord;
#endif
    }
  }
  
  printf("End of computing...\n");
#ifdef DRAW_SURFACE_SB
  printf("In Pink : Contour filled with SbTessContour.\n");
#endif
#ifdef DRAW_CONTOUR_AFTER
  printf("In Green : Contour send to SbHatching (stranges line closes contours).\n");
#endif
#ifdef DRAW_CONTOUR_HATCH
  printf("In Yellow : Hatch surface made by SbHatching.\n");
#endif

  SoGuiExaminerViewer* examinerViewer = new SoGuiExaminerViewer(topWidget);
  examinerViewer->setSceneGraph(root);
  examinerViewer->viewAll();
  examinerViewer->show();

  SoGui::show(topWidget);
  SoGui::mainLoop();

  root->unref();
  delete [] listPoints;
  delete [] listClosePoints;
  delete examinerViewer;
#ifdef SoFreePackage //Cleanup
  SoGui::clear();  
#endif
  
  return 0; 
}
