#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/SbArguments.h>
#include <HEPVis/nodes/SoGrid.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

#include <stdlib.h>
#include <iostream>

//
// Test/demo of SoGrid class
// Guy Barrand September, 3, 2005
//
 
//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbArguments args(aArgc,aArgv);

  if(args.isAnArgument("-h")) {
    std::cout << "HEPVis::Grid :" << std::endl;
    std::cout << "Usage :" << std::endl;
    std::cout << " OS> Grid<GUI> <options>" << std::endl;
    std::cout << "Options :" << std::endl;
    std::cout << " -h : to get help." << std::endl;
    std::cout << " -altrep : to produce an alternate representation." 
              << std::endl;
    std::cout << " -write : to apply SoWriteAction." << std::endl;
    std::cout << " -file : for output of SoWriteAction." << std::endl;
    std::cout << "For example :" << std::endl;
    std::cout << " OS> GridXt -altrep -write" << std::endl;
    std::cout << " UNIX> GridXt -altrep -write > out.iv" << std::endl;
    std::cout << " UNIX> ivview out.iv" << std::endl;
    return 0;
  }

  // Set up the main window
 
  SoGuiWidget widget = SoGui::init(aArgv[0]);
  if (!widget) exit(1);

  // You must initialize a new class

  SoGrid::initClass();
  SoAlternateRepAction::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  // Set up the material for the object to be drawn

  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(1, 0, 0);
  sceneGraph->addChild(material);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoTimeCounter* timeCounter = new SoTimeCounter;
  timeCounter->min =  0; 
  timeCounter->max = 10; 
  timeCounter->frequency= 0.1F;
  
  // Make the SoGrid

  SoGrid* soGrid = new SoGrid();
  soGrid->rows = 20;
  soGrid->columns = 10;
  if(args.isAnArgument("-altrep")) {
    soGrid->generateAlternateRep(0);
  }
  soGrid->columns.connectFrom(&timeCounter->output);
  sceneGraph->addChild(soGrid);

  if(args.isAnArgument("-write")) {
    SbString file;
    args.find("-file",file);
    SoWriteAction writeAction;
    writeAction.getOutput()->openFile(file.getString());
    writeAction.getOutput()->setBinary(FALSE);
    writeAction.apply(sceneGraph);
  }

  // View the new object in an Examiner Viewer

  SoGuiExaminerViewer* examinerVieweer=new SoGuiExaminerViewer(widget);
  examinerVieweer->setSceneGraph(sceneGraph);
  examinerVieweer->viewAll();
  examinerVieweer->show();

  SoGui::show(widget);
  SoGui::mainLoop();

  sceneGraph->unref();
  delete examinerVieweer;

  return 0; 
}

