////////////////////////////////////////////////////////////
//
// In order to find the stars.fit file, this program should 
// be run from the HEPVis/data directory.
//
////////////////////////////////////////////////////////////

#include <HEPVis/Gui/SoGui.h>

// Inventor :

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/errors/SoDebugError.h>

// HEPVis :
#include <HEPVis/nodes/SoImage.h>
#include <HEPVis/actions/SoGL2PSAction.h>

#include <math.h>
#include <stdlib.h>
#include <string.h>

void keystroke(void*,SoEventCallback*);

SoGuiExaminerViewer* examinerViewer = NULL;

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  HEPVis_SoImage::Format fmt = HEPVis_SoImage::GIF;
  if(aArgc==2) {
    // A file name is given as first argument.
    // Guess its type (GIF, JPEG, FITS).
    if(strstr(aArgs[1],".gif")!=NULL) 
      fmt = HEPVis_SoImage::GIF;
    else if(strstr(aArgs[1],".jpg")!=NULL) 
      fmt = HEPVis_SoImage::JPEG;
    else if(strstr(aArgs[1],".fits")!=NULL) 
      fmt = HEPVis_SoImage::FITS;
    else if(strstr(aArgs[1],".fit")!=NULL) 
      fmt = HEPVis_SoImage::FITS;
    else if(strstr(aArgs[1],".tex")!=NULL) 
      fmt = HEPVis_SoImage::TEX;
    else if(strstr(aArgs[1],".dvi")!=NULL) 
      fmt = HEPVis_SoImage::DVI;
    else {
      printf("File extension .gif,.jpg,.fits,.fit,.tex,.dvi expected for \"%s\".\n",aArgs[1]);
      return 0;
    }
  } else {
    printf("Give a .gif or a .fit file as first argument.\n");
    printf("The HEPVis/data directory contains some.\n");
    return 0;
  }
    

  SoGuiWidget myWindow = SoGui::init(aArgs[0]);
  if (myWindow==NULL) exit(1);

  HEPVis_SoImage::initClass();
  SoGL2PSAction::initClass();

  SoSeparator* root = new SoSeparator;
  root->ref();

  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(1.0, 1.0, 0.0);
  root->addChild(material);

  SoCube* cube = new SoCube;
  root->addChild(cube);

  SoTransform* transform = new SoTransform;
  transform->translation.setValue(1.1F,1.1F,0);
  root->addChild(transform);

  HEPVis_SoImage* pixmap = new HEPVis_SoImage;
  pixmap->fileName.setValue(aArgs[1]);
  pixmap->format.setValue(fmt);
  root->addChild(pixmap);

  SoEventCallback* eventCallback = new SoEventCallback;
  eventCallback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
                                  keystroke,0);
  root->addChild(eventCallback);                                  

  // View the new object in an Examiner Viewer

  examinerViewer = new SoGuiExaminerViewer(myWindow);
  const SbViewportRegion& vpRegion = examinerViewer->getViewportRegion();
  SoGL2PSAction* action = new SoGL2PSAction(vpRegion);
  action->setFileName("out.ps");
  examinerViewer->setGLRenderAction(action);
  examinerViewer->setSize(SbVec2s(400,400));
  examinerViewer->setSceneGraph(root);
  examinerViewer->viewAll();
  examinerViewer->show();
  examinerViewer->setBackgroundColor(SbColor(0.0,0.0,0.0));

  SoDebugError::postInfo("Image",
    "type P key in picking mode to produce a postscript file.");

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete examinerViewer;
  delete action;

#ifdef SoFreePackage
  HEPVis_SoImage::clearClass();
  SoGL2PSAction::clearClass();

  SoGui::clear();  
#endif

  return EXIT_SUCCESS;
}
void keystroke(void*,SoEventCallback* eventCB){
  if (eventCB->isHandled()) return;

  const SoKeyboardEvent* event = (SoKeyboardEvent*)eventCB->getEvent();

  if (SO_KEY_PRESS_EVENT(event,P)) {
    if(examinerViewer) {
      SoDebugError::postInfo("Image","produce out.ps...");

      SoGL2PSAction* action = 
        (SoGL2PSAction*)examinerViewer->getGLRenderAction();
      action->enableFileWriting();
      examinerViewer->render();
      action->disableFileWriting();

      eventCB->setHandled();
    }
  }
}

