
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <HEPVis/Gui/SoGui.h>
#include <HEPVis/Gui/SoGuiPainterViewer.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodekits/SoNodeKitListPart.h>
#include <Inventor/nodekits/SoShapeKit.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/errors/SoDebugError.h>

#include <HEPVis/SbMath.h>
#include <HEPVis/nodes/SoViewportRegion.h>
#include <HEPVis/nodes/SoMarkerSet.h>
#include <HEPVis/nodes/SoTubs.h>
#include <HEPVis/nodes/SoBox.h>
#include <HEPVis/nodes/SoTrd.h>
#include <HEPVis/nodekits/SoDetectorTreeKit.h>
#include <HEPVis/nodekits/SoDetectorExample.h>
#include <HEPVis/actions/SoPainterAction.h>
#include <HEPVis/actions/SoGL2PSAction.h>

void keystroke(void*,SoEventCallback*);

SoGuiExaminerViewer* examinerViewer = NULL;
SoGuiPainterViewer* painterViewer = NULL;

int main(int argc, char** argv) {

  // Set up the main window
  
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoTubs::initClass();
  SoBox::initClass();
  SoTrd::initClass();
  SoDetectorTreeKit::initClass();
  SoDetectorExample::initClass();
  SoViewportRegion::initClass();
  HEPVis_SoMarkerSet::initClass();
  SoPainterAction::initClass(); //After nodes.
  SoGL2PSAction::initClass(); //After nodes.

  // Set up the root of the scene database and
  // reference it

  SoSeparator* root = new SoSeparator;
  root->ref();
  // Add a camera shared by the two viewers :
  SoOrthographicCamera* camera = new SoOrthographicCamera;
  root->addChild(camera);

  // In case only painter viewer is used :
  SoTransform* transform = new SoTransform;
  root->addChild(transform);

  SoDetectorExample* soDetExa = new SoDetectorExample;
  root->addChild(soDetExa);

  SoEventCallback* eventCallback = new SoEventCallback;
  eventCallback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
                                  keystroke,0);
  root->addChild(eventCallback);                                  

  // View the new object in an Examiner Viewer

  if((argc==2)&&(strcmp(argv[1],"-painter")==0)) {
    painterViewer = new SoGuiPainterViewer(myWindow); 
    painterViewer->setSize(SbVec2s(400,400));
    camera->position.setValue(0,0,100);
    camera->focalDistance.setValue(100);
    camera->nearDistance.setValue(1);
    camera->farDistance.setValue(1000);
    camera->height.setValue(100);
    transform->rotation.setValue(SbRotation(SbVec3f(1,1,0),FM_PI/4));
    //transform->rotation.setValue(SbRotation(SbVec3f(0,1,0),FM_PI_2));
  } else if((argc==2)&&(strcmp(argv[1],"+painter")==0)) {
    examinerViewer = new SoGuiExaminerViewer(myWindow);
    painterViewer = new SoGuiPainterViewer(myWindow,"Painter",FALSE); 
    painterViewer->setSize(SbVec2s(300,300));
  } else {
    examinerViewer = new SoGuiExaminerViewer(myWindow);
  }

  if(examinerViewer!=NULL) {
    examinerViewer->setSize(SbVec2s(400,400));
    examinerViewer->setSceneGraph(root);
    examinerViewer->viewAll();
    examinerViewer->setViewing(FALSE);
    examinerViewer->show();
  }

  if(painterViewer!=NULL) {
    painterViewer->setSceneGraph(root);
    //painterViewer->viewAll();
    //painterViewer->enableSort(TRUE);
    painterViewer->show();
  }

  SoDebugError::postInfo("PainterViewer",
    "type P key in picking mode to produce a postscript file.");

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  if(painterViewer!=NULL) delete painterViewer;
  if(examinerViewer!=NULL) delete examinerViewer;

#ifdef SoFreePackage //Cleanup
  SoDetectorTreeKit::clearClass();
  SoTubs::clearClass();
  SoBox::clearClass();
  SoTrd::clearClass();
  SoViewportRegion::clearClass();
  HEPVis_SoMarkerSet::clearClass();
  SoPainterAction::clearClass(); //After nodes.
  SoGL2PSAction::clearClass(); //After nodes.

  SoGui::clear();  
#endif

  return 0; 
}
void keystroke(void*,SoEventCallback* eventCB){
  if (eventCB->isHandled()) return;

  const SoKeyboardEvent* event = (SoKeyboardEvent*)eventCB->getEvent();

  if (SO_KEY_PRESS_EVENT(event,P)) {
    if(examinerViewer) {
      SoDebugError::postInfo("PainterViewer","produce out.ps...");
      const SbViewportRegion& vpRegion = examinerViewer->getViewportRegion();
      SoGL2PSAction* action = new SoGL2PSAction(vpRegion);
      action->setFileName("out.ps");
      action->apply(examinerViewer->getSceneGraph());
      delete action;
    }
  }
}

