#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/SbMath.h>
#include <HEPVis/nodes/SoPolyVol.h>
#include <HEPVis/nodes/SoCoordinateAxis.h>

#include <stdlib.h>

//
// Test of SoPolyVol class, after JB
//
// This demo draws a PolyVol
//
 
int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoPolyVol::initClass();
  SoCoordinateAxis::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  // Set up the material for the object to be drawn

  SoMaterial    *RedPlastic = new SoMaterial;
  RedPlastic->diffuseColor.setValue(1, 0, 0);
  root->addChild(RedPlastic);

  // Make the PolyVol
  const int nsides=5;
  const float sideHalf[5]={1.0,1.0,1.0,1.0,1.0};
  const float seventyTwo=2*FM_PI/5;
  const float angles[5]={0.0,seventyTwo,2*seventyTwo,3*seventyTwo,4*seventyTwo};
  
  SoPolyVol *ThePolyVol   = new SoPolyVol();
  ThePolyVol->setPolyVolPts(nsides,1.5,&angles[0],&sideHalf[0]);

  SoCoordinateAxis *TheAxis   = new SoCoordinateAxis();
  TheAxis->fNDivision = 3;
  TheAxis->fDivisionLength = 0.2F;

  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))  ThePolyVol->generateAlternateRep(0);
  root->addChild(ThePolyVol);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  root->addChild(TheAxis);
  // View the new object in an Examiner Viewer
  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->viewAll();
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

#ifdef SoFreePackage //Cleanup
  root->unref();
  delete WriteAction;
  delete myViewer;
  SoPolyVol::clearClass();
  SoCoordinateAxis::clearClass();
  SoGui::clear();  
  printf("PolyVols : exiting.\n");
#endif

  return 0; 
}

