
#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLight.h>

// intersection :
#include <Inventor/nodes/SoShape.h>
#include <Inventor/collision/SoIntersectionDetectionAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/manips/SoTransformBoxManip.h>

#include <HEPVis/SbArguments.h>
#include <HEPVis/SbFile.h>
#include <HEPVis/SbStyle.h>
#include <HEPVis/SbString.h>
#include <HEPVis/misc/SoTools.h>

#include <iostream>

#ifdef HEPVIS_READ_HEPVIS
extern "C" {
  void HEPVisUtils_initClasses();
  void HEPVisGeometry_initClasses();
  void HEPVisDetector_initClasses();
  void HEPVisPlotter_initClasses();
  void HEPVisGUI_initClasses();
}
#endif

void highlight(SoNode* sceneGraphNode,const SbColor& color) {
  SoSearchAction searchAction;
  searchAction.setFind(SoSearchAction::TYPE);
  searchAction.setType(SoMaterial::getClassTypeId());
  searchAction.apply(sceneGraphNode);
  SoPath* path = searchAction.getPath();
  if(path) {
    SoNode* parent = path->getNodeFromTail(1);
    if(parent && parent->isOfType(SoGroup::getClassTypeId())) {
      // replace the SoMaterial that may come from a style cache.
      SoMaterial* old_mat = (SoMaterial*)path->getTail();
      SoMaterial* new_mat = new SoMaterial;
      new_mat->diffuseColor.setValue(color);
      ((SoGroup*)parent)->replaceChild(old_mat,new_mat);
    }
  }
}

void manip(SoNode* sceneGraphNode) {
  SoSearchAction searchAction;
  searchAction.setFind(SoSearchAction::TYPE);
  searchAction.setType(SoTransform::getClassTypeId());
  searchAction.apply(sceneGraphNode);
  SoPath* path = searchAction.getPath();
  if(path) {
    SoNode* node = path->getTail();
    if(node->isOfType(SoTransform::getClassTypeId())) { 
      SoTransformManip* manip = new SoTransformBoxManip;
      if(manip) {
        manip->ref();
        manip->replaceNode(path);
      }
    }
  }
}

SoIntersectionDetectionAction::Resp intersectionCB(
 void* //aTag
,const SoIntersectingPrimitive* a1
,const SoIntersectingPrimitive* a2
) {
  std::cout << "intersection :" << std::endl;
 {const SoIntersectingPrimitive* ip = a1;
  if(ip->type==SoIntersectingPrimitive::TRIANGLE) {
    std::cout << " p1 triangle :" << std::endl;
    std::cout << " "
              << " " << ip->vertex[0][0]
              << " " << ip->vertex[0][1]
              << " " << ip->vertex[0][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[1][0]
              << " " << ip->vertex[1][1]
              << " " << ip->vertex[1][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[2][0]
              << " " << ip->vertex[2][1]
              << " " << ip->vertex[2][2] << std::endl;
  } else if(ip->type==SoIntersectingPrimitive::SEGMENT) {
    std::cout << " p1 segment :" << std::endl;
    std::cout << " "
              << " " << ip->vertex[0][0]
              << " " << ip->vertex[0][1]
              << " " << ip->vertex[0][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[1][0]
              << " " << ip->vertex[1][1]
              << " " << ip->vertex[1][2] << std::endl;
  }}

 {const SoIntersectingPrimitive* ip = a1;
  if(ip->type==SoIntersectingPrimitive::TRIANGLE) {
    std::cout << " p2 triangle :" << std::endl;
    std::cout << " "
              << " " << ip->vertex[0][0]
              << " " << ip->vertex[0][1]
              << " " << ip->vertex[0][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[1][0]
              << " " << ip->vertex[1][1]
              << " " << ip->vertex[1][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[2][0]
              << " " << ip->vertex[2][1]
              << " " << ip->vertex[2][2] << std::endl;
  } else if(ip->type==SoIntersectingPrimitive::SEGMENT) {
    std::cout << " p2 segment :" << std::endl;
    std::cout << " "
              << " " << ip->vertex[0][0]
              << " " << ip->vertex[0][1]
              << " " << ip->vertex[0][2] << std::endl;
    std::cout << " "
              << " " << ip->vertex[1][0]
              << " " << ip->vertex[1][1]
              << " " << ip->vertex[1][2] << std::endl;
  }}

 {std::cout << " p1 path :" << std::endl;
  SoFullPath* path = (SoFullPath*)a1->path;
  if(path) {
    for(int i=0;i<path->getLength();i++) {
      SoNode* n = path->getNodeFromTail(i);
      if(n->isOfType(SoShape::getClassTypeId()))  {
        std::cout << "  type : " << n->getTypeId().getName().getString()
                   << std::endl;
        SbName name = n->getName();
        if(name.getLength()) {
          std::cout << "  name : \"" << name.getString() << "\"" << std::endl;
        }
        break;
      }
    }
  }}

 {std::cout << " p2 path :" << std::endl;
  SoFullPath* path = (SoFullPath*)a2->path;
  if(path) {
    for(int i=0;i<path->getLength();i++) {
      SoNode* n = path->getNodeFromTail(i);
      if(n->isOfType(SoShape::getClassTypeId()))  {
        std::cout << "  type : " << n->getTypeId().getName().getString() 
                   << std::endl;
        SbName name = n->getName();
        if(name.getLength()) {
          std::cout << "  name : \"" << name.getString() << "\"" << std::endl;
        }
        break;
      }
    }
  }}


  // look if shape is under an "OnX kind scene graph: :
 {SoFullPath* path = (SoFullPath*)a1->path;
  if(path) {
    for(int i=0;i<path->getLength();i++) {
      SoNode* n = path->getNodeFromTail(i);
      if(n->getName()=="sceneGraph") {
        std::cout << " p1 highlight." << std::endl;
        highlight(n,SbColor(0,0,1));
        manip(n);
        break;
      }
    }
  }}
 {SoFullPath* path = (SoFullPath*)a2->path;
  if(path) {
    for(int i=0;i<path->getLength();i++) {
      SoNode* n = path->getNodeFromTail(i);
      if(n->getName()=="sceneGraph") {
        std::cout << " p2 highlight." << std::endl;
        highlight(n,SbColor(0,1,0));
        manip(n);
        break;
      }
    }
  }}

  //return SoIntersectionDetectionAction::NEXT_PRIMITIVE;
  //return SoIntersectionDetectionAction::NEXT_SHAPE;
  return SoIntersectionDetectionAction::ABORT;
}

int main(int aArgc, char** aArgv) {
  SbArguments args(aArgc,aArgv);
  if(args.isAnArgument("-h")) {
    std::cout << "HEPVis::Read :" << std::endl
              << "Usage :" << std::endl
              << " OS> HEPVis_Read_<GUI> <options> <file>" << std::endl
              << " OS> view_hiv <options> <file>" << std::endl
              << " OS> view_iv <options> <file>" << std::endl
              << "Options :" << std::endl
              << " -h : to get help." << std::endl
              << " -file=<string> : file to view." << std::endl
              << " -viewAll : to set the camera to view all the scene." 
              << std::endl
              << " -background=<color> : to set the window background." 
              << " -check_intersection" 
              << std::endl;
    return 0;
  }


  SbString file;
  if(SbString_getenv("HEPVIS_FILE",file)==FALSE) {
    if(args.find("-file",file)==FALSE) {
      if(args.file(file)==FALSE) {
#ifdef HEPVIS_READ_HEPVIS
        file = "out.hiv";
#else
        file = "out.iv";
#endif
      }
    }
  }

  SbColor back(0.6F,0.6F,0.6F);
 {SbString s;
  if(args.find("-background",s)==TRUE) {
    SbStyle::stringToColor(s,back);
  }}

  SbBool viewAll = args.isAnArgument("-viewAll");

  // Set up the main window
 
  SoGuiWidget widget=SoGui::init(aArgv[0]);
  if(!widget) return 1;

#ifdef HEPVIS_READ_HEPVIS
  HEPVisUtils_initClasses();
  HEPVisGeometry_initClasses();
  HEPVisDetector_initClasses();
  HEPVisPlotter_initClasses();
  HEPVisGUI_initClasses();
#endif

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  // Solve environment variables :
  SbString name = SbFile::getFullName(file);

  SoInput soInput;
  if(!soInput.openFile(name.getString())) return 1; 

  SoSeparator* separator = SoDB::readAll(&soInput);
  if(!separator) return 1;

  sceneGraph->addChild(separator);

  SbBool hasLight = 
    SoTools::searchNode(separator,SoLight::getClassTypeId()) ? TRUE:FALSE;

  if(args.isAnArgument("-check_intersection")==TRUE) {
    SoIntersectionDetectionAction action;
    action.addIntersectionCallback(intersectionCB);
    action.apply(sceneGraph);
  }

  SoGuiExaminerViewer* viewer = new SoGuiExaminerViewer(widget);
  viewer->setBackgroundColor(back);
  viewer->setSceneGraph(sceneGraph);
  viewer->setHeadlight(hasLight==TRUE?FALSE:TRUE);
  // We do not viewAll by default since the file
  // may come with its own camera setting.
  // (If no camera given, then the viewer->setSceneGraph does a viewAll.
  if(viewAll==TRUE) viewer->viewAll();
  viewer->show();

  SoGui::show(widget);
  SoGui::mainLoop();

  sceneGraph->unref();
  delete viewer;

  return 0; 
}
