//  January/2008 : G.Barrand : creation.

//
// See doc/Game.txt for explanation
// of the controls and for a simple
// demo scenario.
//


#include <HEPVis/Gui/SoGui.h>

#include <Inventor/SoDB.h>
#include <Inventor/errors/SoDebugError.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodekits/SoNodeKit.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/engines/SoCalculator.h>
#include <Inventor/sensors/SoTimerSensor.h>
//#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/fields/SoSFUInt32.h>
#include <Inventor/manips/SoTransformManip.h>

#include <HEPVis/SbMath.h>
#include <HEPVis/SbColors.h>
//#include <HEPVis/SbTestPlotter.h>
#include <HEPVis/SbFile.h>
#include <HEPVis/SbString.h>
#include <HEPVis/SbArguments.h>
#include <HEPVis/SbSystem.h>
#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodes/SoAreaStyle.h>
#include <HEPVis/nodes/SoTextStyle.h>
#include <HEPVis/nodes/SoStyle.h>
#include <HEPVis/nodes/SoFile.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>
#include <HEPVis/nodekits/SoFormRegion.h>
#include <HEPVis/nodekits/SoTextRegion.h>
#include <HEPVis/nodekits/SoSimpleButton.h>
//#include <HEPVis/nodekits/SoPlotter.h>
#include <HEPVis/nodekits/SoDetectorTreeKit.h>
//#include <HEPVis/nodekits/SoText.h>

static SbBool curl_file(const SbString&,const SbString&);

// To customize the program from a .hiv
class SoMain : public SoNode {
  SO_NODE_HEADER(SoMain);
public:
  SoSFBool fullScreen;
  SoSFBool checkFloor;
  SoSFFloat floor;
  SoSFInt32 videoWidth;
  SoSFInt32 videoHeight;
  SoMFString files;
  SoMFString legends;
  SoSFFloat slideTimeInterval;
  SoSFBool slidesAtStartup;
  SoSFBool animateAtStartup;
  SoSFBool viewAllAtStartup;
  SoSFBool headLight;
  SoSFFloat deltaTranslation;
  SoSFFloat deltaRotation;
  SoSFUInt32 moveTimerInterval;
  SoSFUInt32 keyRepeatDelay;
  SoSFUInt32 keyRepeatInterval;
  SoSFBool lockScreen;
  SoSFString password;
  SoSFBool renderCaching;
  SoSFColor landColor;
  SoSFBool legendVisible;
public:
  SoMain() {
    SO_NODE_CONSTRUCTOR(SoMain);
    SO_NODE_ADD_FIELD(fullScreen,(TRUE));
    SO_NODE_ADD_FIELD(checkFloor,(TRUE));
    SO_NODE_ADD_FIELD(floor,(0.0));
    SO_NODE_ADD_FIELD(videoWidth,(640));
    SO_NODE_ADD_FIELD(videoHeight,(480));
    SO_NODE_ADD_FIELD(files,(""));
    SO_NODE_ADD_FIELD(legends,(""));
    SO_NODE_ADD_FIELD(slideTimeInterval,(10)); //sec
    SO_NODE_ADD_FIELD(slidesAtStartup,(FALSE));
    SO_NODE_ADD_FIELD(viewAllAtStartup,(FALSE));
    SO_NODE_ADD_FIELD(animateAtStartup,(FALSE));
    SO_NODE_ADD_FIELD(headLight,(FALSE));

    SO_NODE_ADD_FIELD(deltaTranslation,(1.0F));
    SO_NODE_ADD_FIELD(deltaRotation,(3.1416F/180.0F));
    SO_NODE_ADD_FIELD(moveTimerInterval,(30));
    SO_NODE_ADD_FIELD(keyRepeatDelay,(200));
    SO_NODE_ADD_FIELD(keyRepeatInterval,(10));
    SO_NODE_ADD_FIELD(lockScreen,(FALSE));
    SO_NODE_ADD_FIELD(password,(""));
    SO_NODE_ADD_FIELD(renderCaching,(TRUE));
    SO_NODE_ADD_FIELD(landColor,(0.5F,0.5F,0.5F));
    SO_NODE_ADD_FIELD(legendVisible,(TRUE));

    files.deleteValues(0);
    legends.deleteValues(0);
  }
  static void initClass() {
    SO_NODE_INIT_CLASS(SoMain,SoNode,"Node");
  }
protected:
  virtual ~SoMain(){}
};
SO_NODE_SOURCE(SoMain)

#include <iostream>
#include <vector>

extern "C" {
  void HEPVisUtils_initClasses();
  void HEPVisGeometry_initClasses();
  void HEPVisDetector_initClasses();
  //void HEPVisGUI_initClasses();
  void HEPVisPlotter_initClasses();
}

static void detPickCB(void*,void*);
static void activateCB(void*,SoNode*);
static void sensorCB(void*,SoSensor*);
static void unlockCB(void*);
//static void textCB(void*,SoNode*);
static SoGroup* createGroupFiles(const std::vector<SbString>&);

#define ID_Quit 0
#define ID_Slides 1
#define ID_Reset 2
#define ID_Animate 3
#define ID_Lock 4
#define ID_Sensitivity 5
#define ID_Manips 6
#define num_IDs 7

//Darwin : "main" becomes "SDL_main" through a cpp macro in SDL.h.

class add_pickCB : public virtual SbVisitor {
public:
  virtual ~add_pickCB(){}
public:
  virtual SbBool visit(SoFullPath& aPath) {
    SoNode* node = aPath.getTail();
    if(!node) return TRUE;
    SoDetectorTreeKit* kit = (SoDetectorTreeKit*)node;
    kit->addPickCallback(fCB,fTag);
    return TRUE;
  }
public:
  SoCallbackListCB* fCB;
  void* fTag;
};

#ifdef HEPVisSDL
class GameViewer : public HEPVis::SoSDL_Viewer {
public:
  GameViewer(SoSDL_Widget* aWidget)
  : HEPVis::SoSDL_Viewer(aWidget)
  {}
  virtual ~GameViewer(){}
public:
  virtual SbBool processEvent(SDL_Event& aEvent,SbBool& aRender){
    if(processEventLock(aEvent,aRender)==TRUE) return TRUE;
    return HEPVis::SoSDL_Viewer::processEvent(aEvent,aRender);
  }
};
#else
#define GameViewer SoGuiExaminerViewer
#endif

static SoSeparator* sSceneGraph = 0;
static SoCamera* sCamera = 0;
static SoRotationXYZ* sRotation = 0;
static SoDisplayRegion* sLand = 0;
static SoSeparator* sSlides = 0;

static SoFormRegion* sUI = 0;
static SoTextRegion* sLegend = 0;
static SoSimpleButton* s_button_sensitivity = 0;
static SoGuiExaminerViewer* sViewer = 0;

static SoTimerSensor* sTimerSensor = 0;
static SoTimeCounter* sTimeCounter = 0;
static unsigned int s_delta_step = 0;
static int sIndex = 0; //could be <0
static SoGroup* sGroupFiles = 0;

static std::vector<SbString>* arg_legends = 0;
static double arg_slideTimeInterval = 10;
static float arg_deltaTranslation = 1.0F;
static float arg_deltaRotation = 3.1416F/180.0F;
static SbBool arg_lockScreen = FALSE;
static SbString* arg_password = 0;
static SbColor* arg_landColor = 0;
static SbBool arg_legendVisible = TRUE;

//////////////////////////////////////////////////////////////////////////////
int main( 
 int aArgc
,char* aArgv[]
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbArguments args(aArgc,aArgv);
  if(args.isAnArgument("-h")) {
    std::cout << "HEPVis::Show :" << std::endl;
    std::cout << "Usage :" << std::endl;
    std::cout << " OS> Show_<GUI> <options>" << std::endl;
    std::cout << "Options :" << std::endl;
    std::cout << " -h : to get help." << std::endl;
    std::cout << " -lock : to lock screen." << std::endl;
    std::cout << " -password=<string> : to lock screen." << std::endl;
    std::cout << " -not_full_screen : to not start in full screen." 
              << std::endl;
    std::cout << " -videWidth=<int> : video width (default is 640)." 
              << std::endl;
    std::cout << " -videHeight=<int> : video height (default is 480)." 
              << std::endl;
    return 0;
  }

#ifdef __APPLE__
 {SbString arg0 = (aArgc>=1?aArgv[0]:"");
  SbString dir_root;
  if(SbSystemIsMacApp(arg0,dir_root)==TRUE) {    
    SbString TTFPATH = dir_root;
    TTFPATH += SbString("/Resources/HEPVis/fonts");
    SbSystemPutenv("TTFPATH",TTFPATH);

    SbString HEPVIS_DATA = dir_root;
    HEPVIS_DATA += SbString("/Resources/HEPVis/data");
    SbSystemPutenv("HEPVIS_DATA",HEPVIS_DATA);
  }}
#endif

  if(SbString_isenv("HEPVIS_DATA")==FALSE) {
    SbSystemPutenv("HEPVIS_DATA",".");
  }

  ///////////////////////////////////////////////
  // Args :
  ///////////////////////////////////////////////

  SbString arg_main_hiv;
  if(args.find("-main_hiv",arg_main_hiv)==FALSE) {
    if(args.file(arg_main_hiv)==FALSE) {
      arg_main_hiv = "$HEPVIS_DATA/HEPVis_Show.hiv";
    }
  }

  SbBool arg_renderCaching = TRUE;

  std::vector<SbString> arg_files;
  arg_legends = new std::vector<SbString>();

  int arg_videoWidth = 640;
  int arg_videoHeight = 480;

  SbBool arg_fullScreen = TRUE;
  SbBool arg_checkFloor = TRUE;
  float arg_floor = 0.0F;

  SbBool arg_slidesAtStartup = FALSE;
  SbBool arg_animateAtStartup = FALSE;
  SbBool arg_viewAllAtStartup = FALSE;
  SbBool arg_headLight = FALSE;

  unsigned int arg_moveTimerInterval = 30;
  unsigned int arg_keyRepeatDelay = 200;
  unsigned int arg_keyRepeatInterval = 10;

  arg_landColor = new SbColor(0.5F,0.5F,0.5F);

  ///////////////////////////////////////////////
  // SoDB init and get args from SoMain :
  ///////////////////////////////////////////////
  SoSeparator* soLayout = 0;

 {if(!SoDB::isInitialized()) SoDB::init();
  SoNodeKit::init();
  HEPVisUtils_initClasses();
  HEPVisGeometry_initClasses();
  HEPVisDetector_initClasses();
  HEPVisPlotter_initClasses();
  //HEPVisGUI_initClasses();

  HEPVis_SoFile::setCurlFunction(curl_file); //FIXME
  SoMain::initClass();

 {SoInput soInput;
  SbString file = SbFile::getFullName(arg_main_hiv);
  if(soInput.openFile(file.getString())==TRUE) {
    SoSeparator* soSep = SoDB::readAll(&soInput);
    soSep->ref();
    if(soSep) {
      SoMain* soMain = 
        (SoMain*)SoTools::searchNode(soSep,SoMain::getClassTypeId());
      if(soMain) {
        arg_checkFloor = soMain->checkFloor.getValue();
        arg_floor = soMain->floor.getValue();
        arg_videoWidth = soMain->videoWidth.getValue();
        arg_videoHeight = soMain->videoHeight.getValue();
        arg_fullScreen = soMain->fullScreen.getValue();
       {int n = soMain->files.getNum();
        arg_files.clear();
        for(int i=0;i<n;i++) {
          arg_files.push_back(soMain->files[i].getString());
        }}
       {int n = soMain->legends.getNum();
        arg_legends->clear();
        for(int i=0;i<n;i++) {
          arg_legends->push_back(soMain->legends[i].getString());
        }}
        arg_slideTimeInterval = soMain->slideTimeInterval.getValue();
        arg_slidesAtStartup = soMain->slidesAtStartup.getValue();
        arg_animateAtStartup = soMain->animateAtStartup.getValue();
        arg_viewAllAtStartup = soMain->viewAllAtStartup.getValue();
        arg_headLight = soMain->headLight.getValue();
        arg_deltaTranslation = soMain->deltaTranslation.getValue();
        arg_deltaRotation = soMain->deltaRotation.getValue();
        arg_moveTimerInterval = soMain->moveTimerInterval.getValue();
        arg_keyRepeatDelay = soMain->keyRepeatDelay.getValue();
        arg_keyRepeatInterval = soMain->keyRepeatInterval.getValue();
        arg_lockScreen = soMain->lockScreen.getValue();
        delete arg_password;
        arg_password = new SbString(soMain->password.getValue());
        arg_renderCaching = soMain->renderCaching.getValue();
        delete arg_landColor;
        arg_landColor = new SbColor(soMain->landColor.getValue());
        arg_legendVisible = soMain->legendVisible.getValue();
      }
      sCamera = 
        (SoCamera*)SoTools::searchNode(soSep,"slides_camera",
                                       SoCamera::getClassTypeId());
      if(!sCamera) {
        std::cout << "HEPVis::Show :" << std::endl;
        std::cout << " no slides_camera SoCamera found in file." 
                  << std::endl;
        return 1;
      } else {
        sCamera->ref();
      }

      soLayout = 
        (SoSeparator*)SoTools::searchNode(soSep,"layout_separator",
                                          SoSeparator::getClassTypeId());
      if(!soLayout) {
        std::cout << "HEPVis::Show :" << std::endl;
        std::cout << " no layout_separator SoSeparator found in file." 
                  << std::endl;
        return 1;
      } else {
        soLayout->ref();
        sSlides = 
          (SoSeparator*)SoTools::searchNode(soLayout,"slides_separator",
                                            SoSeparator::getClassTypeId());
        if(!sSlides) {
          std::cout << "HEPVis::Show :" << std::endl;
          std::cout << " no slides_separator SoSeparator found in file." 
                    << std::endl;
          return 1;
        } else {
          add_pickCB v;
          v.fCB = detPickCB;
          v.fTag = sSlides;
          SoTools::visit(*sSlides,SoDetectorTreeKit::getClassTypeId(),v);
        }
        
        sRotation = 
          (SoRotationXYZ*)SoTools::searchNode(soLayout,
                                              "animate_rotation",
                                              SoRotationXYZ::getClassTypeId());
        if(!sRotation) {
          std::cout << "HEPVis::Show :" << std::endl;
          std::cout << " WARNING :"
                    << " no animate_rotation SoRotationXYZ found in file." 
                    << std::endl;
          //return 1;
        }
      }
    }
    soSep->unref();
  }}
  } //end SoDB::init

  if(!soLayout) return 1;
  if(!sSlides) return 1;
  if(!sCamera) return 1;
  // At the point we should have a soLayout and a sSlides nodes.

  ///////////////////////////////////////////////
  // args from command line :
  ///////////////////////////////////////////////

  if(args.isAnArgument("-not_cache")) arg_renderCaching = FALSE;

  if(args.isAnArgument("-lock")) arg_lockScreen = TRUE;

 {SbString s;
  if(args.find("-password",s)==TRUE) {
    arg_password = new SbString(s);
  } else {
    if(SbString_getenv("HEPVIS_PASSWORD",s)==TRUE) {
      arg_password = new SbString(s);
    }
  }}

 {SbString s;  
  if(args.find("-videoWidth",s)==TRUE) {
    int v;
    if(SbStringConvertToInt(s,v)==TRUE) arg_videoWidth = v;
  }}
 {SbString s;  
  if(args.find("-videoHeight",s)==TRUE) {
    int v;
    if(SbStringConvertToInt(s,v)==TRUE) arg_videoHeight = v;
  }}

  if(args.isAnArgument("-not_full_screen")) arg_fullScreen = FALSE;

  ///////////////////////////////////////////////
  // Get files :
  ///////////////////////////////////////////////
  sGroupFiles = createGroupFiles(arg_files);

  ///////////////////////////////////////////////
  // Init :
  ///////////////////////////////////////////////

#ifdef HEPVisSDL
  Uint32 flags = SDL_FULLSCREEN;
  if(arg_fullScreen==FALSE) flags = 0;
  SoGuiWidget topWidget = SoGui::init(arg_videoWidth,arg_videoHeight,flags);
  if(!topWidget) return 1;
  SDL_EnableKeyRepeat(arg_keyRepeatDelay,arg_keyRepeatInterval);
#else
  SoGuiWidget topWidget = SoGui::init(aArgv[0]);
  if(!topWidget) return 1;
#endif

  sSceneGraph = new SoSeparator;
  sSceneGraph->ref();

  sLand = new SoDisplayRegion;
  sLand->color.setValue(*arg_landColor);
  sSceneGraph->addChild(sLand);

  ///////////////////////////////////////////////
  // Camera (us), taken from file :
  ///////////////////////////////////////////////
  sLand->setCamera((SoCamera*)sCamera->copy());

 {SoSeparator* scene = sLand->getStaticScene();
  if(arg_renderCaching==TRUE) {
    scene->renderCaching.setValue(SoSeparator::ON);
  }
  scene->addChild(soLayout); //will ref soLayout.
  soLayout->unref();}

  ///////////////////////////////////////////////
  // Legend :
  ///////////////////////////////////////////////
  sLegend = new SoTextRegion;
  //sLegend->color.setValue(SbColor_red);
  sLegend->origin.setValue(SbVec2f(0.5F,0));
  //ui->size.setValue(SbVec2f(0.4F,0.1F));
  sLegend->size.setValue(SbVec2f(0.5F,0.1F));
  sLegend->visible.setValue(arg_legendVisible);
  sLegend->horizontalJustification.setValue(SoTextRegion::CENTER);
  sLegend->text.set1Value(0,"Type Esc key...");
  sSceneGraph->addChild(sLegend);

  ///////////////////////////////////////////////
  // Shooting cross :
  ///////////////////////////////////////////////
  SoRegion* shootRegion = new SoRegion;
  float w = 0.008F;
  float h = 0.008F;
  shootRegion->origin.setValue(SbVec2f(0.5F*(1.0F-w),0.5F*(1.0F-h)));
  shootRegion->size.setValue(SbVec2f(w,h));
  shootRegion->visible.setValue(FALSE);
  sSceneGraph->addChild(shootRegion);

  ///////////////////////////////////////////////
  // GUI :
  ///////////////////////////////////////////////
  sUI = new SoFormRegion;
  sUI->origin.setValue(SbVec2f(0,0));
  //sUI->origin.setValue(SbVec2f(0,0.5F));
  //sUI->size.setValue(SbVec2f(0.4F,0.1F));
  float ui_w = num_IDs * 0.07F;
  sUI->size.setValue(SbVec2f(ui_w,0.05F));
  sUI->columns.setValue(num_IDs);
  sUI->rows.setValue(1);
  sUI->visible.setValue(FALSE);
  sSceneGraph->addChild(sUI);

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  //button->getLabelStyle()->color.setValue(SbColor(0.7F,0,7F,0.7F));
  button->getLabelStyle()->fontName.setValue("Text3/defaultFont");
   //button->shadowThickness.setValue(0.1F);
   //button->marginWidth.setValue(0.1F);
   //button->marginHeight.setValue(0.1F);
  button->label.setValue("Q");
  button->addCallback(activateCB,(void*)ID_Quit);
  sUI->shapes.set1Value(ID_Quit,button);}

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  button->getLabelStyle()->fontName.setValue("Text3/defaultFont");
  button->label.setValue("Slides");
  button->addCallback(activateCB,(void*)ID_Slides);
  sUI->shapes.set1Value(ID_Slides,button);}

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  button->getLabelStyle()->fontName.setValue("Text3/defaultFont");
  button->label.setValue("Reset");
  button->addCallback(activateCB,(void*)ID_Reset);
  sUI->shapes.set1Value(ID_Reset,button);}

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  button->getLabelStyle()->fontName.setValue("Text3/defaultFont");
  button->label.setValue("Animate");
  button->addCallback(activateCB,(void*)ID_Animate);
  sUI->shapes.set1Value(ID_Animate,button);}

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  button->getLabelStyle()->fontName.setValue("Text3/defaultFont");
  button->label.setValue("Lock");
  button->addCallback(activateCB,(void*)ID_Lock);
  sUI->shapes.set1Value(ID_Lock,button);}

 {s_button_sensitivity = new SoSimpleButton;
  s_button_sensitivity->getBackStyle()->color.setValue(SbColor_white);
  s_button_sensitivity->getLabelStyle()->fontName.setValue
    ("Text3/defaultFont");
  s_button_sensitivity->label.setValue("0");
  s_button_sensitivity->addCallback(activateCB,(void*)ID_Sensitivity);
  sUI->shapes.set1Value(ID_Sensitivity,s_button_sensitivity);}

 {SoSimpleButton* button = new SoSimpleButton;
  button->getBackStyle()->color.setValue(SbColor_white);
  button->getLabelStyle()->fontName.setValue
    ("Text3/defaultFont");
  button->label.setValue("Manips");
  button->addCallback(activateCB,(void*)ID_Manips);
  sUI->shapes.set1Value(ID_Manips,button);}

/*
 {SoFormRegion* sPWD = new SoFormRegion;
  sPWD->origin.setValue(SbVec2f(0.5F,0.5F));
  //sUI->origin.setValue(SbVec2f(0,0.5F));
  //sUI->size.setValue(SbVec2f(0.4F,0.1F));
  sPWD->size.setValue(SbVec2f(0.2F,0.05F));
  sPWD->columns.setValue(1);
  sPWD->rows.setValue(1);
  sPWD->visible.setValue(TRUE);
  sSceneGraph->addChild(sPWD);

  SoText* text = new SoText();
  //text->shadowThickness.setValue(0.1F);
  //text->marginWidth.setValue(0.1F);
  //text->marginHeight.setValue(0.1F);
  text->shadowThickness.setValue(0);
  text->marginWidth.setValue(0);
  text->marginHeight.setValue(0);
  text->editable.setValue(TRUE);
  text->value.set1Value(0,"xxx");
  text->addActivateCallback(textCB,NULL);
  sPWD->shapes.set1Value(0,text);}
*/

  // Viewer :
  sViewer = new GameViewer(topWidget);
  sViewer->setSceneGraph(sSceneGraph);
  sViewer->setCamera(sLand->getCamera());
  sViewer->setHeadlight(arg_headLight);
  if(arg_viewAllAtStartup==TRUE) sViewer->viewAll();
#ifdef HEPVisSDL
  sViewer->setShootRegion(shootRegion);
  sViewer->setOptionsRegion(sUI);
  // adapt delta movement to the scene :
  sViewer->setDefaultDeltaTranslation(arg_deltaTranslation);
  sViewer->setDefaultDeltaRotation(arg_deltaRotation);
  sViewer->checkFloor(arg_checkFloor);
  sViewer->setFloor(arg_floor); //should take into account the camera height.
  sViewer->setMoveTimerInterval(arg_moveTimerInterval);
#endif
  sViewer->show();

  SoGui::show(topWidget);

  if(arg_slidesAtStartup) activateCB((void*)ID_Slides,0);
  if(arg_animateAtStartup) activateCB((void*)ID_Animate,0);
  if(arg_lockScreen==TRUE) activateCB((void*)ID_Lock,0);

  SoGui::mainLoop();
    
  SoTools::removeManips(*sSceneGraph);

  sSceneGraph->unref();

  sGroupFiles->unref();

  delete sViewer;

  delete arg_legends;
  delete arg_password;
  delete arg_landColor;

#ifdef HEPVisSDL
  SoGui::clear();  
#endif
	
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
static void detPickCB(
 void* aTag
,void* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoDetectorTreeKit* soDet = (SoDetectorTreeKit*)((SoNode*)aNode);

/*
  //FIXME : highlight/unhighlight logic
  SoMaterial* material = 
    (SoMaterial*)sCurrentDet->getPart("appearance.material",TRUE);
  SbColor color = material->diffuseColor[0];
  if(color[0]==1) {
    material->diffuseColor.setValue(0,1,0);
  } else {
    material->diffuseColor.setValue(1,0,0);
  }
*/

  //std::cout << "HEPVis_Game::detPickCB :" << std::endl
  //          << soDet->getName().getString() << std::endl;

  SoNode* node = soDet->getPart("transform",TRUE);
  if(node) {
    SoNode* sceneGraph = (SoNode*)aTag;
    SoTools::setManip(sceneGraph,(SoTransform*)node);
#ifdef HEPVisSDL
    SDL_ShowCursor(1);
#endif
  }
}

//////////////////////////////////////////////////////////////////////////////
static void activateCB (
 void* aUserData
,SoNode*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned long what = (unsigned long)aUserData;
  if(what==ID_Quit) {
#ifdef HEPVisSDL
    SoSDL::toQuit();
#else
    ::exit(0);
#endif

  } else if(what==ID_Slides) {

    if(sGroupFiles->getNumChildren()>0) {
      if(sGroupFiles->getNumChildren()==1) { 
        //do not spawn the looping for one slide.
        sensorCB(0,0);
      } else {
        if(!sTimerSensor) {
          //std::cout << "debug : start timer sensor" << std::endl;
          sTimerSensor = new SoTimerSensor(sensorCB,NULL);
          sTimerSensor->setInterval(SbTime(arg_slideTimeInterval));
          sTimerSensor->schedule();
  
          sLegend->text.set1Value(0,"Start...");
  
          sensorCB(0,0); //enforce first scene now.
  
        } else {
          //std::cout << "debug : stop timer sensor" << std::endl;
          sTimerSensor->unschedule();
          delete sTimerSensor;
          sTimerSensor = 0;
  
          sLegend->text.set1Value(0,"");
        }
      }
    }

  } else if(what==ID_Reset) {

    sLand->setCamera((SoCamera*)sCamera->copy());
    sViewer->setCamera(sLand->getCamera());
    if(sRotation) sRotation->angle.setValue(0);

   {SoSeparator* scene = sLand->getStaticScene();
    SoTools::removeManips(*scene);
    if(SoTools::searchNode(scene,SoTransformManip::getClassTypeId())==NULL) {
#ifdef HEPVisSDL
      SDL_ShowCursor(0);
#endif
    }}

  } else if(what==ID_Animate) {

    if(!sRotation) {
      sLegend->text.set1Value(0,"No rotation node given");
    } else {

      if(!sTimeCounter) {

        sTimeCounter = new SoTimeCounter;
        //sTimeCounter->on.setValue(FALSE);
        sTimeCounter->min.setValue(0); 
        sTimeCounter->max.setValue(628); 
        sTimeCounter->frequency.setValue(0.1F);
 
        SoCalculator* calculator = new SoCalculator();
        calculator->a.connectFrom(&(sTimeCounter->output));
        calculator->expression.set1Value(0,"oa=a/100.");
        sRotation->angle.connectFrom(&(calculator->oa));
 
      } else {
        sRotation->angle.disconnect();
        sTimeCounter = 0;
      }
    }

  } else if(what==ID_Lock) {

    if(!arg_password || (arg_password->getLength()<=0) ) {
      sLegend->text.set1Value(0,"No password given");
    } else {
#ifdef HEPVisSDL
      sViewer->lock(*arg_password,unlockCB);
#endif
      sLegend->color.setValue(SbColor_pink);
      sUI->visible.setValue(FALSE);
    }

  } else if(what==ID_Sensitivity) {

#ifdef HEPVisSDL
    s_delta_step++;
    if(s_delta_step>10) s_delta_step = 0;

   {float fact = 1.0F/FPOW(10,s_delta_step);
    sViewer->setDefaultDeltaTranslation(fact * arg_deltaTranslation);}

   {float fact = 1.0F/FPOW(2,s_delta_step);
    sViewer->setDefaultDeltaRotation(fact * arg_deltaRotation);}

   {SbString s;
    SbStringPrintF(s,32,"%d",s_delta_step);
    s_button_sensitivity->label.setValue(s);}
#endif

  } else if(what==ID_Manips) {

    SoSeparator* scene = sLand->getStaticScene();
    if(SoTools::searchNode(scene,SoTransformManip::getClassTypeId())!=NULL) {
      SoTools::removeManips(*scene);
    } else {
      if(!sSlides || (sSlides->getNumChildren()<=0)) { //else, too much.
        SoTools::setManips(*scene);
      }
    }

#ifdef HEPVisSDL
    if(SoTools::searchNode(scene,SoTransformManip::getClassTypeId())!=NULL) {
      SDL_ShowCursor(1);
    } else {
      SDL_ShowCursor(0);
    }
#endif
  }

}

void sensorCB(void*,SoSensor*) {
  //std::cout << "sensorCB : begin." << std::endl;

  if(sGroupFiles->getNumChildren()<=0) return;
  if(!sSlides) return;

  if(sIndex>=sGroupFiles->getNumChildren()) {
    //std::cout << "sensorCB : clear." << std::endl;

    SoTools::removeManips(*sSlides);

    sSlides->removeAllChildren();

#ifdef HEPVisSDL
   {SoSeparator* scene = sLand->getStaticScene();
    if(SoTools::searchNode(scene,SoTransformManip::getClassTypeId())==NULL) {
      SDL_ShowCursor(0);
    }}
#endif

    sLegend->text.set1Value(0,"Start...");

    sIndex = 0;
    return;
  }

  if(sIndex<0) {
    sIndex += 1;
    return;
  }

  sSlides->addChild(sGroupFiles->getChild(sIndex));

  if(arg_legends && (sIndex<arg_legends->size())) {
    sLegend->text.set1Value(0,(*arg_legends)[sIndex]);
  } else {
    sLegend->text.set1Value(0,"");
  }

  sIndex++;
}
void unlockCB(void* aTag) {
  sLegend->color.setValue(SbColor_white);  
}

//////////////////////////////////////////////////////////////////////////////
static SoGroup* createGroupFiles(
  const std::vector<SbString>& aFiles
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoGroup* soGroup = new SoGroup;
  soGroup->ref();

  std::string::size_type number = aFiles.size();
  for(unsigned int index=0;index<number;index++) {
    const SbString& file = aFiles[index];

    // look if web file :
    SbString name;
    if(file.getSubString(0,4)=="http:") {
      if(curl_file(file,"tmp.hiv")==FALSE) {
        std::cout << "Can't get URL \"" << file.getString() << "\"" 
                  << std::endl;
        continue;
      } else {
        name = "tmp.hiv";
      }
    } else {
      name = SbFile::getFullName(file);
    }

    //std::cout << "load " << std::string(name.getString()) 
    //          << "..." << std::endl;

    SoInput soInput;
    if(soInput.openFile(name.getString())==FALSE) {
      std::cout << "Can't read file \"" << name.getString() << "\"" 
                << std::endl;
    } else {
      SoSeparator* separator = SoDB::readAll(&soInput);
      if(separator) {
        soGroup->addChild(separator);
 
        add_pickCB v;
        v.fCB = detPickCB;
        v.fTag = sSlides;
        SoTools::visit(*separator,SoDetectorTreeKit::getClassTypeId(),v);

      }
    }

    if(name=="tmp.hiv") SbFile::remove(name);


  }
  return soGroup;
}

/*
//////////////////////////////////////////////////////////////////////////////
static void textCB (
 void* //aUserData
,SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoText* text = (SoText*)aNode;
  if(text==NULL) return;
  const SbString& s = text->value[0];
  printf("%s\n",s.getString());
}

*/


#ifdef HEPVIS_HAS_CURL
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>
#include <string>

class File {
public:
  File(const SbString& a_name):m_name(a_name),m_FILE(0){}
public:
  SbString m_name;
  FILE* m_FILE;
};

static int my_fwrite(
 void* a_buffer
,size_t a_size
,size_t a_nmemb
,void* a_tag
)
{
  File* out = (File*)a_tag;

  if(out && !out->m_FILE) {
    /* open file for writing */
    out->m_FILE = ::fopen(out->m_name.getString(),"wb");
    if(!out->m_FILE)
      return -1; /* failure, can't open file to write */
  }
  return ::fwrite(a_buffer,a_size,a_nmemb,out->m_FILE);
}


SbBool curl_file(const SbString& a_url,const SbString& a_out_file) {
  File ftpfile(a_out_file);

  curl_global_init(CURL_GLOBAL_DEFAULT);

  SbBool status = FALSE;

  CURL* curl = curl_easy_init();
  if(curl) {
    curl_easy_setopt(curl,CURLOPT_URL,a_url.getString());
    curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION,my_fwrite);
    curl_easy_setopt(curl,CURLOPT_WRITEDATA,&ftpfile);

    //curl_easy_setopt(curl,CURLOPT_VERBOSE,1);

    CURLcode res = curl_easy_perform(curl);

    curl_easy_cleanup(curl);

    if(ftpfile.m_FILE) ::fclose(ftpfile.m_FILE);

    status = (res!=CURLE_OK?FALSE:TRUE);
  }

  curl_global_cleanup();

  if(status==FALSE) SbFile::remove(a_out_file);

  return status;
}

#else
SbBool curl_file(const SbString&,const SbString&) { return FALSE;}
#endif
