#include <math.h>
#include <stdlib.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodes/SoSiStrips.h>
#include <HEPVis/nodekits/SoSiStripDetKit.h>

// Test of SoSiStripDet class
// Joe Boudreau Nov. 11, 1996

// This demo draws a SiStripDet

short Strip[8]  = {100, 101, 102, 103, 130, 131, 132, 133};
short ADC[8]    = {50,   80, 110,  70,  20,  34,  18,   5};
 
int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoSiStrips::initClass();
  SoSiStripDetKit::initClass();
  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  SoSiStripDetKit* TheSiStripDet = new SoSiStripDetKit();
  SoMaterial* RedPlastic = 
    (SoMaterial *) TheSiStripDet->getPart("waferAppearance.material",TRUE);
  RedPlastic->diffuseColor.setValue(1, 0, 0);
  SoMaterial* GreenPlastic = 
    (SoMaterial *) TheSiStripDet->getPart("stripsAppearance.material",TRUE);
  GreenPlastic->diffuseColor.setValue(0,1,0);  

  SoSiStrips* TheSiStrips = 
    (SoSiStrips*)TheSiStripDet->getPart("strips",TRUE);
  TheSiStrips->strip.setValues(0,8,Strip);
  TheSiStrips->adc.setValues(0,8,ADC);
  root->addChild(TheSiStripDet);

  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP")) TheSiStripDet->generateAlternateRep(0);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  // View the new object in an Examiner Viewer

  TheSiStripDet->setAlignable(TRUE);
  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;

#ifdef SoFreePackage //Cleanup
  SoSiStripDetKit::clearClass();
  SoSiStrips::clearClass();
  SoGui::clear();  
#endif

  return 0; 

}

