#include <math.h>
#include <stdlib.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/engines/SoCalculator.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodes/SoTrap.h>
#include <HEPVis/nodes/SoCoordinateAxis.h>

//
// Test of SoTrap class
// Joe Boudreau Nov. 11, 1996
//
// This demo draws a G4Trap
//
 
int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoTrap::initClass();
  SoCoordinateAxis::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  SoCoordinateAxis *TheAxis   = new SoCoordinateAxis();
  TheAxis->fNDivision = 5;
  TheAxis->fDivisionLength = 50;
  root->addChild(TheAxis);

  // Set up the material for the object to be drawn

  SoMaterial    *RedPlastic = new SoMaterial;
  RedPlastic->diffuseColor.setValue(1, 0, 0);
  root->addChild(RedPlastic);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoTimeCounter *TheCounter = new SoTimeCounter;
  TheCounter->max  = 360; 
  TheCounter->step = 4;
  TheCounter->frequency=0.2F;
  
  SoCalculator *TheCalculator = new SoCalculator();
  TheCalculator->a.connectFrom(&TheCounter->output);
  TheCalculator->expression.setValue("oa=a*M_PI/180.0");

  // Make the G4Trap

  SoTrap *TheG4Trap    = new SoTrap();
  TheG4Trap->pDz        = 200;
  TheG4Trap->pTheta     = 1;
  TheG4Trap->pPhi       = 0;
  TheG4Trap->pDy1       = 150;
  TheG4Trap->pDx1       = 125;
  TheG4Trap->pDx2       = 175;
  TheG4Trap->pDy2       = 165;
  TheG4Trap->pDx3       = 138;
  TheG4Trap->pDx4       = 193;
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP")) TheG4Trap->generateAlternateRep(0);
  TheG4Trap->pPhi.connectFrom(&TheCalculator->oa);
  root->addChild(TheG4Trap);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  // View the new object in an Examiner Viewer


  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->viewAll();
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;

#ifdef SoFreePackage //Cleanup
  SoTrap::clearClass();
  SoCoordinateAxis::clearClass();
  SoGui::clear();  
#endif

  return 0; 

}

