#include <math.h>
#include <stdlib.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/nodes/SoTrd.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

//
// Test of SoTrd class
// Joe Boudreau Nov. 11, 1996
//
// This demo draws a G4Trd
//
 
int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoTrd::initClass();
  SoAlternateRepAction::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  // Set up the material for the object to be drawn

  SoMaterial    *RedPlastic = new SoMaterial;

  RedPlastic->diffuseColor.setValue(1, 0, 0);
  root->addChild(RedPlastic);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoTimeCounter *TheCounter = new SoTimeCounter;
  TheCounter->min      =  50; 
  TheCounter->max      = 450; 
  TheCounter->frequency= 0.1F;
  
  // Make the G4Trd

  SoTrd *TheG4Trd    = new SoTrd();
  TheG4Trd->fDx1       = 450;
  TheG4Trd->fDy1       = 450;
  TheG4Trd->fDz        = 100;
  TheG4Trd->fDx2       =  50;
  TheG4Trd->fDy2       =  50;
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))TheG4Trd->generateAlternateRep(0);
  TheG4Trd->fDx1.connectFrom(&TheCounter->output);
  TheG4Trd->fDy1.connectFrom(&TheCounter->output);
  root->addChild(TheG4Trd);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  // View the new object in an Examiner Viewer

  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->viewAll();
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;

#ifdef HEPVisSDL
  SoGui::clear();  
#endif

  return 0; 

}

