#include <math.h>
#include <stdlib.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/engines/SoTimeCounter.h>
#include <Inventor/engines/SoCalculator.h>
#include <Inventor/actions/SoWriteAction.h>

#include <HEPVis/SbMath.h>
#include <HEPVis/nodes/SoTubs.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

//
// Test of SoTubs class
// Joe Boudreau Nov. 11, 1996
//
// This demo draws a G4Tubs
//


int main(int, char ** argv) {

  // Set up the main window
 
  SoGuiWidget myWindow=SoGui::init(argv[0]);
  if (!myWindow) exit(1);

  // You must initialize a new class

  SoTubs::initClass();
  SoAlternateRepAction::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  // Set up the material for the object to be drawn

  SoMaterial    *RedPlastic = new SoMaterial;

  RedPlastic->diffuseColor.setValue(1, 0, 0);
  root->addChild(RedPlastic);

  // Users will expect a new node to change appearance when a field
  // is changed.  Here, we connect one of the fields to an engine
  // which should animate the object if this feature is properly
  // implemented.

  SoTimeCounter *TheCounter = new SoTimeCounter;
  TheCounter->max  = 360; 
  TheCounter->step = 4;
  TheCounter->frequency = 0.2F;
  
  SoCalculator *TheCalculator = new SoCalculator();
  TheCalculator->a.connectFrom(&TheCounter->output);
  TheCalculator->expression.setValue("oa=a*M_PI/180.0");
  
  // Make the G4Tubs

  SoTubs *TheG4Tubs   = new SoTubs();
  TheG4Tubs->pRMin = 0.5F;
  TheG4Tubs->pRMax = 1;
  TheG4Tubs->pDz   = 1;
  TheG4Tubs->pDPhi = 1.5F*FM_PI;
  if (getenv("HEPVIS_ALTERNATEREP")||getenv("ALTERNATEREP"))TheG4Tubs->generateAlternateRep(0);
  TheG4Tubs->pDPhi.connectFrom(&TheCalculator->oa);
  root->addChild(TheG4Tubs);

  SoWriteAction *WriteAction= new SoWriteAction();
  WriteAction->getOutput()->setBinary(FALSE);
  WriteAction->apply(root);

  // View the new object in an Examiner Viewer

  SoGuiExaminerViewer *myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->viewAll();
  myViewer->show();

  SoGui::show(myWindow);
  SoGui::mainLoop();

  root->unref();
  delete WriteAction;
  delete myViewer;

#ifdef SoFreePackage //Cleanup
  SoTubs::clearClass();
  SoGui::clear();  
#endif

  return 0; 

}

