#ifndef WIN32

#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoResetTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>

#include <HEPVis/Xt/viewers/SoXtHepEventViewer.h>

#define NOBJECTS 4
SoNode *object[NOBJECTS]     = {NULL,NULL,NULL,NULL};
SoTranslation *xf[8]={NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
SoResetTransform *rs = NULL; 
SoSelection *theObject       = NULL;
SoSeparator *permanent=NULL;
SoSeparator *volat=NULL;
SoXtHepEventViewer *myViewer = NULL;
int EventNumber=0;


// The NextEvent callback--attached to the viewer
int NextEvent(void *) {
  theObject->deselectAll();
  myViewer->setRunNumber(0);
  myViewer->setEventNumber(EventNumber);  
  volat->removeAllChildren();

  volat->addChild(xf[0]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[1]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[2]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[3]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[4]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[5]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[6]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  volat->addChild(xf[7]);
  volat->addChild(object[(EventNumber)%NOBJECTS]);
  volat->addChild(rs);

  EventNumber++;
  return 0;
}

// The rewind callback--attached to the viewer
int Rewind(void *) {
  EventNumber=0;
  NextEvent(NULL);
  return 0;
}

// The SelectEvent callback--attached to the viewer
int SelectEvent(int run, int event, void *) {
  if (run!=0) return 1;
  EventNumber=event;
  NextEvent(NULL);
  return 0;
}

// The FileOpen callback--attached to the viewer.
int OpenFile(char *filename, void *) {
  myViewer->clearText();
  myViewer->addText("Located file ");
  myViewer->addText(filename);
  return 0;
}

// The select object callback--attacked to the selection object
void SelectObject(void *, SoPath *selectionPath) {
  myViewer->clearText();
  myViewer->addText("Selected object was ");
  myViewer->addText((char *) selectionPath->getTail()->getTypeId().getName().getString());
  myViewer->iconize(selectionPath,selectionPath->getTail()->getTypeId().getName().getString());
}

// The deselect object callback--attacked to the selection object
void DeselectObject(void *, SoPath *) {
  myViewer->clearText();
}

int main(int, char **argv)
{
  // Initialize Inventor and Xt, which must be done
  // before any Inventor calls are made.
  Widget myWindow = SoXt::init(argv[0]);
  
  theObject = new SoSelection;
  object[0] = new SoCone;
  object[1] = new SoCylinder;
  object[2] = new SoCube;
  object[3] = new SoSphere;


  xf[0] = new SoTranslation();
  xf[0]->translation.setValue( 5,  5,  5);
  xf[0]->ref();
  xf[1] = new SoTranslation();
  xf[1]->translation.setValue( 5,  5, -5);
  xf[1]->ref();
  xf[2] = new SoTranslation();
  xf[2]->translation.setValue( 5, -5,  5);
  xf[2]->ref();
  xf[3] = new SoTranslation();
  xf[3]->translation.setValue( 5, -5, -5);
  xf[3]->ref();
  xf[4] = new SoTranslation();
  xf[4]->translation.setValue(-5,  5,  5);
  xf[4]->ref();
  xf[5] = new SoTranslation();
  xf[5]->translation.setValue(-5,  5, -5);
  xf[5]->ref();
  xf[6] = new SoTranslation();
  xf[6]->translation.setValue(-5, -5,  5);
  xf[6]->ref();
  xf[7] = new SoTranslation();
  xf[7]->translation.setValue(-5, -5, -5);
  xf[7]->ref();

  rs    = new SoResetTransform(); rs->ref();
  // create the viewer in the toplevel window
  // and set some scene to display
  myViewer = new SoXtHepEventViewer(myWindow);
  
  // make a scene
  SoSeparator *root = new SoSeparator();
  root->ref();


  // make some red material
  SoMaterial *redPlastic = new SoMaterial();
  redPlastic->diffuseColor.setValue(1,0,0);
  redPlastic->emissiveColor.setValue(0,0,1);
  root->addChild(redPlastic);

  // make a place to hold the objects
  root->addChild(theObject);
  myViewer->setSceneGraph(root);

  // reference the objects
  object[0]->ref();
  object[1]->ref();
  object[2]->ref();
  object[3]->ref();

  // set up the viewer's callbacks:
  myViewer->setRewindCallback(Rewind);
  myViewer->setNextEventCallback(NextEvent);
  myViewer->setEventSelectCallback(SelectEvent);
  myViewer->setFileOpenCallback(OpenFile);
  theObject->addSelectionCallback(SelectObject);
  theObject->addDeselectionCallback(DeselectObject);

  permanent = new SoSeparator;
  volat  = new SoSeparator;
  volat->renderCaching=SoSeparator::OFF;
  theObject->addChild(permanent);
  theObject->addChild(volat);

  // make some red material
  SoMaterial *bluePlastic = new SoMaterial();
  bluePlastic->diffuseColor.setValue(0,0,1);
  permanent->addChild(bluePlastic);
  permanent->addChild(new SoSphere);

  // set the highlight style:
  for (int i=0;i<4;i++) {
    myViewer->getViewer(i)->setGLRenderAction(new SoLineHighlightRenderAction);
    myViewer->getViewer(i)->redrawOnSelectionChange(theObject);
  }
  myViewer->getViewer(SoXtHepEventViewer::EXAMINER);

  // manage and map window on screen
  myViewer->show();
  SoXt::show(myWindow); 
  
  // main loop
  SoXt::mainLoop();

  return 0;
}
#else
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#define main HEPVis_main
int main(int, char ** argv) {
 return 0;
}
#endif
