#include <math.h>
#include <stdlib.h>

#include <HEPVis/Gui/SoGui.h>

#include <Inventor/SoPickedPoint.h>
#include <Inventor/SbLinear.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodekits/SoNodeKitListPart.h>
#include <Inventor/nodekits/SoShapeKit.h>
#include <Inventor/nodes/SoSphere.h>

#ifndef WIN32
//
#define HEP_VIEWER
#endif

#ifdef HEP_VIEWER
#include <HEPVis/Xt/viewers/SoXtHepViewer.h>
#endif

#include <HEPVis/SbMath.h>
#include <HEPVis/nodes/SoTubs.h>
#include <HEPVis/nodes/SoBox.h>
#include <HEPVis/nodes/SoTrd.h>
#include <HEPVis/nodes/SoPolyhedron.h>
#include <HEPVis/nodekits/SoDetectorTreeKit.h>
#include <HEPVis/nodekits/SoDetectorExample.h>
#include <HEPVis/nodekits/SoMeterStickKit.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

#ifdef HEP_VIEWER
SoXtHepViewer *myViewer = NULL;
#else
SoGuiExaminerViewer *myViewer = NULL;
#endif

//
// Quit Callback
//
int 
quitCB(void* ) {
  printf("Exiting.\n");
  exit (0);
  return 1;
}

int main(int, char **argv)
{
  // Initialize Inventor and Xt, which must be done
  // before any Inventor calls are made.

  SoGuiWidget myWindow = SoGui::init(argv[0]);
  if (!myWindow) exit(1);
  
  // You must initialize a new class
  SoSFPolyhedron::initClass();
  SoMeterStickKit::initClass();
  SoBox::initClass();
  SoTrd::initClass();
  SoTubs::initClass();
  SoPolyhedron::initClass();
  SoDetectorTreeKit::initClass();
  SoDetectorExample::initClass();
  SoAlternateRepAction::initClass();

  // Set up the root of the scene database and
  // reference it

  SoSeparator *root=new SoSeparator;
  root->ref();

  SoMeterStickKit *ruler = new SoMeterStickKit;
  ruler->type.setValue(SoMeterStickKit::RULER);
  ruler->rotationSnap.setValue(TRUE);
  root->addChild(ruler);

  SoDetectorExample* SVX = new SoDetectorExample();
  root->addChild(SVX);

  // View the new object in an Examiner Viewer
#ifdef HEP_VIEWER
  myViewer = new SoXtHepViewer(myWindow);
  myViewer->hideTextArea();
  myViewer->resizeMain(400, 400);
  myViewer->setQuitCallback(quitCB,myViewer);
  myViewer->setSceneGraph(root);
  myViewer->getCurrentViewer()->viewAll();
  myViewer->getCurrentViewer()->saveHomePosition();
#else
  myViewer=new SoGuiExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->viewAll();
#endif
  myViewer->show();
  SoGui::show(myWindow); 
  ruler->touch();
  // main loop
  SoGui::mainLoop();

  delete myViewer;
  root->unref();

  return 0;
}


