
#include <Inventor/SoDB.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLight.h>
#include <Inventor/nodekits/SoNodeKit.h>

#include <HEPVis/SbArguments.h>
#include <HEPVis/SbFile.h>
#include <HEPVis/SbStyle.h>
#include <HEPVis/SbString.h>
#include <HEPVis/misc/SoTools.h>

#include <iostream>

extern "C" {
  void HEPVisUtils_initClasses();
  void HEPVisGeometry_initClasses();
  void HEPVisDetector_initClasses();
  void HEPVisPlotter_initClasses();
  void HEPVisGUI_initClasses();
}

int main(int aArgc,char** aArgv) {
  ///////////////////////////////////////////////
  // Args :
  ///////////////////////////////////////////////

  SbArguments args(aArgc,aArgv);
  if(args.isAnArgument("-h")) {
    std::cout << "HEPVis_convert :" << std::endl
              << "Usage :" << std::endl
              << " OS> HEPVis_convert <arguments>" 
              << std::endl
              << "Arguments :" << std::endl
              << " -h : to get help." << std::endl
//FIXME       << " -compress : produce compressed ascii file." << std::endl
              << " -in=<string> : input file name." 
              << " -out=<string> : output file name." 
              << " -format=<string> : output format. For example vrml2." 
              << std::endl
              << " -binary : produce binary file." << std::endl
              << std::endl;
    return 0;
  }

  SbString in_file;
  if(args.find("-in",in_file)==FALSE) {
    std::cout << "-in argument lacking." << std::endl;
    return 1;
  }
  SbString out_file;
  if(args.find("-out",out_file)==FALSE) {
    std::cout << "-out argument lacking." << std::endl;
    return 1;
  }
  SbString format;
  if(args.find("-format",format)==FALSE) {
    std::cout << "-format argument lacking." << std::endl;
    return 1;
  }

  if( (format!="wrl") && (format!="vrml2") &&
      (format!="iv")  && (format!="inventor") &&
      (format!="pov") ){
    std::cout << "unknown format." << std::endl;
    return 1;
  } 

  SbBool binary = args.isAnArgument("-binary");

  ///////////////////////////////////////////////
  // convert :
  ///////////////////////////////////////////////
  
  if(!SoDB::isInitialized()) SoDB::init();
  SoNodeKit::init();

  HEPVisUtils_initClasses();
  HEPVisGeometry_initClasses();
  HEPVisDetector_initClasses();
  HEPVisPlotter_initClasses();
  HEPVisGUI_initClasses();

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  // Solve environment variables :
  SbString name = SbFile::getFullName(in_file);

  SoInput soInput;
  if(!soInput.openFile(name.getString())) return 1; 

  SoSeparator* separator = SoDB::readAll(&soInput);
  if(!separator) return 1;

  sceneGraph->addChild(separator);

  SoTools::write(*sceneGraph,out_file,format,binary,FALSE);

  sceneGraph->unref();

  return 0; 
}
