#include <math.h>
#include <HEPVis/SbBasic.h>

#ifdef HEPVisXt
#include <HEPVis/Xt/PainterWidget.h>
#include <HEPVis/Xt/SbPainterX11.h>
#endif
#ifdef HEPVisWin
#include <windowsx.h> //For GetWindowInstance.
#include <HEPVis/Win/SbPainterWin.h>
#endif
#ifdef HEPVisQt
#include <HEPVis/Qt/SbPainterQt.h>
#endif
#ifdef HEPVisGtk
#include <HEPVis/Gtk/SbPainterGtk.h>
#endif

#include <Inventor/sensors/SoNodeSensor.h>
#include <Inventor/actions/SoHandleEventAction.h>
#include <Inventor/events/SoMouseButtonEvent.h>

#include <HEPVis/actions/SoPainterAction.h>
#include <HEPVis/SbPainterPS.h>

#include <HEPVis/Gui/SoGuiPainterViewer.h>

#ifdef HEPVisWin
#ifndef Widget
#define Widget HWND
#endif
static const char className[] = "SoWinPainterViewer";
#endif

static SbViewportRegion badRegion;
//////////////////////////////////////////////////////////////////////////////
SoGuiPainterViewer::SoGuiPainterViewer(
 SoGuiWidget aParent
,const char* aName
,SbBool aBuildInsideParent
,SbBool aBuildNow
)
:SoGuiComponent(aParent,aName,aBuildInsideParent)
,widget(NULL)
,renderAction(NULL)
,scene(NULL)
,sceneSensor(NULL)
,painter(NULL)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HEPVisXt
  painter = new SbPainterX11;
#endif
#ifdef HEPVisWin
  painter = new SbPainterWin;
#endif
#ifdef HEPVisQt
  painter = new SbPainterQt;
#endif
#ifdef HEPVisGtk
  painter = new SbPainterGtk;
#endif
  sceneSensor = new SoNodeSensor(
		     (SoSensorCB*)SoGuiPainterViewer::sceneSensorCallback,
		     (void*)this);
  sceneSensor->setPriority(10000);
  renderAction = new SoPainterAction(SbViewportRegion(),painter);
  handleEventAction = new SoHandleEventAction(SbViewportRegion());
  if(aBuildNow==TRUE) {
    setBaseWidget(buildWidget(getParentWidget()));
  }
  bkgColor.setValue(0.,0.,0.);
}
//////////////////////////////////////////////////////////////////////////////
SoGuiPainterViewer::~SoGuiPainterViewer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setSceneGraph(NULL);
  //if(getWidget()!=NULL) XtDestroyWidget(getWidget());
  delete sceneSensor;
  delete renderAction;
  delete handleEventAction;
  delete painter;
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::setSceneGraph (
 SoNode* aScene
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(scene!=NULL) {
    if(sceneSensor!=NULL) sceneSensor->detach();
    scene->unref();
  }
  scene = aScene;
  if(scene!=NULL) {
    scene->ref();
    if(sceneSensor!=NULL) sceneSensor->attach(scene);
  }
}
//////////////////////////////////////////////////////////////////////////////
SoNode* SoGuiPainterViewer::getSceneGraph (
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return scene;
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::setViewportRegion (
 const SbViewportRegion& aNewRegion
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(renderAction==NULL) return;
  renderAction->setViewportRegion(aNewRegion); 
}
//////////////////////////////////////////////////////////////////////////////
const SbViewportRegion& SoGuiPainterViewer::getViewportRegion (
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(renderAction==NULL) return badRegion;
  return renderAction->getViewportRegion(); 
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::setBackgroundColor (
 const SbColor& aC
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  bkgColor = aC;
}
//////////////////////////////////////////////////////////////////////////////
const SbColor& SoGuiPainterViewer::getBackgroundColor (
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return bkgColor;
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::producePostScript(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPainterPS* painterPS = new SbPainterPS;
  SoPainterAction* action = new SoPainterAction(getViewportRegion(),painterPS);
  action->setBackgroundColor(getBackgroundColor());
  action->apply(scene);
  delete action;
  delete painterPS;
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::enableSort(
 SbBool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(renderAction==NULL) return;
  renderAction->enableSort(aValue); 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SoGuiWidget SoGuiPainterViewer::buildWidget(
 SoGuiWidget aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParent==NULL) return NULL;
#ifdef HEPVisXt
  widget = XtCreateManagedWidget("HEPVisPainter",painterWidgetClass,
				 aParent,NULL,0);
  if(widget==NULL) {
    printf("SoGuiPainterViewer::buildWidget : unable to create a widget.\n");
    return NULL;
  }
  XtAddCallback(widget,XtNexposeCallback,
		(XtCallbackProc)SoGuiPainterViewer::exposeCB,(XtPointer)this);
  XtAddCallback(widget,XtNresizeCallback,
		(XtCallbackProc)SoGuiPainterViewer::resizeCB,(XtPointer)this);
  EventMask mask = 
    ButtonPressMask | 
    ButtonReleaseMask | 
    Button1MotionMask |
    Button2MotionMask;
  XtAddEventHandler(widget,mask,False,
		    (XtEventHandler)SoGuiPainterViewer::windowEventCB,
		    (XtPointer)this);
#endif
#ifdef HEPVisWin
  static SbBool done = FALSE;
  if(done==FALSE) {
    WNDCLASS wc;
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC)windowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = ::GetModuleHandle(NULL);
    wc.hIcon = ::LoadIcon(NULL,IDI_APPLICATION);
    wc.hCursor = ::LoadCursor(NULL,IDC_CROSS);
    wc.hbrBackground = GetStockBrush(BLACK_BRUSH);
    wc.lpszMenuName = className;
    wc.lpszClassName = className;
    ::RegisterClass(&wc);
    done = TRUE;
  }  
  RECT rect;
  ::GetWindowRect((HWND)aParent,&rect);
  int width = rect.right-rect.left;
  int height = rect.bottom-rect.top;
  widget = ::CreateWindow(className,"HEPVisPainter", 
			  WS_CHILD | WS_VISIBLE,
			  0,0,
			  width,height,
			  (HWND)aParent,NULL,::GetModuleHandle(NULL),NULL);
  ::SetWindowLong((HWND)widget,GWL_USERDATA,LONG(this));
  //printf("debug : CreateWin : %lu %d %d\n",this,width,height);
  sizeChanged(SbVec2s(width,height));
#endif
  return widget;
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::sceneSensorCallback (
 SoGuiPainterViewer* This
,SoSensor*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(This==NULL) return;
#ifdef HEPVisXt
  //printf("debug : sensor\n");
  This->redraw();
#endif
#ifdef HEPVisWin
  HWND window = (HWND)This->widget;
  if(window==NULL) return;
  ::InvalidateRect(window,NULL,FALSE);
#endif
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::redraw(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  if(renderAction==NULL) return;
  if(scene==NULL) return;
  if(painter==NULL) return;
  if(widget==NULL) return;
  if(sceneSensor==NULL) return;
  if(isVisible()==FALSE) return;
  //printf("debug : redraw\n");
  renderAction->setBackgroundColor(bkgColor);
  sceneSensor->detach();
  renderAction->apply(scene);
  sceneSensor->attach(scene);
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::sizeChanged(
 const SbVec2s& aSize
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(renderAction!=NULL) {
    SbViewportRegion vpr = renderAction->getViewportRegion();
    vpr.setWindowSize(aSize); 
    renderAction->setViewportRegion(vpr);
  }
  if(handleEventAction!=NULL) {
    SbViewportRegion vpr = handleEventAction->getViewportRegion();
    vpr.setWindowSize(aSize); 
    handleEventAction->setViewportRegion(vpr);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::processEvent(
 SoGuiEvent* aEvent
)
//////////////////////////////////////////////////////////////////////////////
// Propagate some event to scene graph :
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HEPVisXt
  if( (aEvent->type==ButtonPress) || (aEvent->type==ButtonRelease) ) {
    
    // Build SoEvent.
    SoMouseButtonEvent event;
    XButtonEvent* bevent = (XButtonEvent*)(void*)aEvent;
    
    switch(bevent->button){
    case Button1: event.setButton(SoMouseButtonEvent::BUTTON1); break;
    case Button2: event.setButton(SoMouseButtonEvent::BUTTON2); break;
    case Button3: event.setButton(SoMouseButtonEvent::BUTTON3); break;
    default:      event.setButton(SoMouseButtonEvent::ANY);     break;
    }
    if(aEvent->type==ButtonPress)   
      event.setState(SoButtonEvent::DOWN);
    else if(aEvent->type==ButtonRelease) 
      event.setState(SoButtonEvent::UP);
    else
      event.setState(SoButtonEvent::UNKNOWN);
      
    unsigned int state = bevent->state;
    if((state & ShiftMask)   == ShiftMask)    event.setShiftDown (TRUE);
    if((state & ControlMask) == ControlMask ) event.setCtrlDown  (TRUE);
    if((state & LockMask)    == LockMask )    event.setAltDown   (TRUE);  
    
    // Position is relative to lower left corner of window :
    const SbVec2s& win = getViewportRegion().getWindowSize();
    int height = win[1];
    event.setPosition(SbVec2s(bevent->x,height - bevent->y));
    
    treatMouseButton(&event);
  }
#endif	
#ifdef HEPVisWin
  switch (aEvent->message) { 
    case WM_LBUTTONDOWN:
    case WM_MBUTTONDOWN:
    case WM_RBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_MBUTTONUP:
    case WM_RBUTTONUP:
      {SoMouseButtonEvent event;      
      switch(aEvent->message){
      case WM_LBUTTONDOWN:
      case WM_LBUTTONUP:
	event.setButton(SoMouseButtonEvent::BUTTON1); 
	break;
      case WM_MBUTTONDOWN:
      case WM_MBUTTONUP:
	event.setButton(SoMouseButtonEvent::BUTTON2); 
	break;
      case WM_RBUTTONDOWN:
      case WM_RBUTTONUP:
	event.setButton(SoMouseButtonEvent::BUTTON3); 
	break;
      default:      
	event.setButton(SoMouseButtonEvent::ANY);     
	break;
      }
      
      switch(aEvent->message){
      case WM_LBUTTONDOWN:
      case WM_RBUTTONDOWN:
      case WM_MBUTTONDOWN:
	event.setState(SoButtonEvent::DOWN);
	break;
      case WM_LBUTTONUP:
      case WM_MBUTTONUP:
      case WM_RBUTTONUP:
	event.setState(SoButtonEvent::UP);
	break;
      default:      
	event.setState(SoButtonEvent::UNKNOWN);
	break;
      }    
      
      unsigned int state = (unsigned int)aEvent->wParam;
      if((state & MK_SHIFT)   == MK_SHIFT)    event.setShiftDown (TRUE);
      if((state & MK_CONTROL) == MK_CONTROL)  event.setCtrlDown  (TRUE);
      
      // Position is relative to lower left corner of window :
      const SbVec2s& win = getViewportRegion().getWindowSize();
      int height = win[1];
      event.setPosition (SbVec2s(LOWORD(aEvent->lParam),
				 height - HIWORD(aEvent->lParam)));
      
      treatMouseButton(&event);
      
      }break;
  }
#endif
#ifdef HEPVisQt
  aEvent = 0;
#endif
#ifdef HEPVisGtk
  aEvent = 0;
#endif
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::treatMouseButton(
 SoEvent* aEvent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aEvent==NULL) return;
  if(handleEventAction==NULL) return;
  if(scene==NULL) return;
  handleEventAction->setEvent(aEvent);
  handleEventAction->apply(scene);
}
#ifdef HEPVisXt
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::exposeCB(
 Widget
,SoGuiPainterViewer* This
,XtPointer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(This==NULL) return;
  if(This->painter==NULL) return;
  if(This->widget==NULL) return;
  //printf("debug : expose\n");
  This->painter->setWindow(XtDisplay(This->widget),XtWindow(This->widget));
  This->redraw();
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::resizeCB(
 Widget aWidget
,SoGuiPainterViewer* This
,XtPointer
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(This==NULL) return;
  if(This->painter==NULL) return;
  if(This->widget==NULL) return;
  //printf("debug : resize\n");
  This->painter->setWindow(XtDisplay(This->widget),XtWindow(This->widget));
  SbVec2s size = SoXt::getWidgetSize(aWidget);
  This->sizeChanged(size);
}
//////////////////////////////////////////////////////////////////////////////
void SoGuiPainterViewer::windowEventCB (
 Widget
,SoGuiPainterViewer* This
,XAnyEvent* aEvent
,Boolean* /*aDispatch*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(This==NULL) return;
  This->processEvent(aEvent);
}
#endif
#ifdef HEPVisWin
//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK SoGuiPainterViewer::windowProc ( 
 HWND aWindow
,UINT aMessage
,WPARAM aWParam
,LPARAM aLParam
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  switch (aMessage) { 
  case WM_PAINT:{
    //printf("debug : WM_PAINT\n");
    PAINTSTRUCT ps;
    HDC hDC = ::BeginPaint(aWindow,&ps);
    SoGuiPainterViewer* This = 
       (SoGuiPainterViewer*)::GetWindowLong(aWindow,GWL_USERDATA);
    if( (This!=NULL) && (This->painter!=NULL) ) {
      This->painter->setWindow(aWindow);
      This->painter->setHDC(hDC);
      This->redraw();
      This->painter->setHDC(NULL);
      This->painter->setWindow(NULL);
    }
    EndPaint(aWindow,&ps);
  }return 0;
  case WM_SIZE:{
    SoGuiPainterViewer* This = 
      (SoGuiPainterViewer*)::GetWindowLong(aWindow,GWL_USERDATA);
    if(This!=NULL) {
      int width = LOWORD(aLParam);
      int height = HIWORD(aLParam);
      //printf("debug : WM_SIZE %d %d \n",width,height);
      This->sizeChanged(SbVec2s(width,height));
    }
  }return 0;
  default:
   {MSG msg;
    msg.hwnd = aWindow;
    msg.message = aMessage;
    msg.wParam = aWParam;
    msg.lParam = aLParam;
    //msg.time
    //msg.pt
    SoGuiPainterViewer* This = 
      (SoGuiPainterViewer*)::GetWindowLong(aWindow,GWL_USERDATA);
    if(This!=NULL) {
      This->processEvent(&msg);
    }
   }
  return ::DefWindowProc(aWindow,aMessage,aWParam,aLParam);
  }
}

#endif
