// this :
#include <HEPVis/SDL/viewers/SoSDL_Viewer.h>

#include <HEPVis/SbColors.h>
#include <HEPVis/nodes/SoAreaStyle.h>
#include <HEPVis/nodekits/SoFormRegion.h>
#include <HEPVis/nodekits/SoSimpleButton.h>

HEPVis::SoSDL_Viewer::SoSDL_Viewer(SoSDL_Widget* aWidget)
: ::SoSDL_Viewer(aWidget)
,fShootRegion(0)
,fUI(0)
,fUI_current(0)
,fJoy_state(FALSE)
{}

HEPVis::SoSDL_Viewer::~SoSDL_Viewer(){}

void HEPVis::SoSDL_Viewer::setShootRegion(SoRegion* aRegion) {
  fShootRegion = aRegion;
}
void HEPVis::SoSDL_Viewer::setOptionsRegion(SoFormRegion* aRegion) {
  fUI = aRegion;  
}
SoRegion* HEPVis::SoSDL_Viewer::optionRegion() const {
  return fUI;  
}
SoRegion* HEPVis::SoSDL_Viewer::shootRegion() const {
  return fShootRegion;
}

SbBool HEPVis::SoSDL_Viewer::processEvent(SDL_Event& aEvent,SbBool& aRender){
  if(aEvent.type==SDL_JOYBUTTONDOWN) {
    if((aEvent.jbutton.button==4)|| //shoot buttons
       (aEvent.jbutton.button==5)) {
      return shoot(TRUE,aRender);
    } else if( (aEvent.jbutton.button==8) ||
               (aEvent.jbutton.button==9) ){ //ESC, options button
      return showUI(aRender);
    } else if( (aEvent.jbutton.button==6) ||
               (aEvent.jbutton.button==7) ){ //show, hide shoot region.
      return showShoot(aRender);
    }
  } else if(aEvent.type==SDL_JOYBUTTONUP) {
    if((aEvent.jbutton.button==4)|| //shoot buttons
       (aEvent.jbutton.button==5)) {
      return shoot(FALSE,aRender);
    }
  } else if(aEvent.type==SDL_JOYAXISMOTION) {
    if(fUI && (fUI->visible.getValue()==TRUE) ) { //move in menu.
      if(aEvent.jaxis.axis==0) {
        aRender = FALSE;
        if((aEvent.jaxis.value>-8000)&&(aEvent.jaxis.value<8000)) {
          fJoy_state = FALSE;
        } else {
          if(fJoy_state==TRUE) {
            fJoy_state = FALSE;
          } else {
            fJoy_state = TRUE;
            if(aEvent.jaxis.value>0) { //RIGHT
              nextUI();
              aRender = TRUE;
            } else { //LEFT
              prevUI();
              aRender = TRUE;
            }
          }
        }            
        return TRUE;
      } else if(aEvent.jaxis.axis==1) {
        if((aEvent.jaxis.value>-8000)&&(aEvent.jaxis.value<8000)) {
        } else {
          if(aEvent.jaxis.value>0) { //DOWN
            //aRender = TRUE;
            //return TRUE; 
          } else { //UP
            //aRender = TRUE;
            //return TRUE; 
          }
        }            
        return TRUE; 
      }
    }
  } else if(aEvent.type==SDL_KEYUP) {
    if(aEvent.key.keysym.sym==SDLK_RETURN) {
      return shoot(FALSE,aRender);
    }
  } else if(aEvent.type==SDL_KEYDOWN) {
    if(aEvent.key.keysym.sym==SDLK_ESCAPE) { //map the options
      return showUI(aRender);
    }
    if(aEvent.key.keysym.sym==SDLK_RETURN) {
      return shoot(TRUE,aRender);
    }
    if(aEvent.key.keysym.sym==SDLK_r) {
      return showShoot(aRender);
    }
    if(fUI && (fUI->visible.getValue()==TRUE) ) { //move in menu.
      if(aEvent.key.keysym.sym==SDLK_RIGHT) {
        nextUI();
        aRender = TRUE;
        return TRUE;
      }
      if(aEvent.key.keysym.sym==SDLK_LEFT) {
        prevUI();
        aRender = TRUE;
        return TRUE;
      }
      if(aEvent.key.keysym.sym==SDLK_UP) {
        return TRUE;
      }
      if(aEvent.key.keysym.sym==SDLK_DOWN) {
        return TRUE;
      }
    } 
  }

  return ::SoSDL_Viewer::processEvent(aEvent,aRender);
}

SbBool HEPVis::SoSDL_Viewer::showUI(SbBool& aRender) {
  aRender = FALSE;
  if(!fUI) return FALSE;
  if(fUI->visible.getValue()==FALSE) {
    fUI->visible.setValue(TRUE);  
    int shapen = fUI->shapes.getNum();
    if( (shapen>0) && (fUI_current>=0) && (fUI_current<shapen) ) {
      SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
      button->getBackStyle()->color.setValue(SbColor_red);
    }
    aRender = TRUE;
    return TRUE;
  } else {
    fUI->visible.setValue(FALSE);  
    aRender = TRUE;
    return TRUE;
  }
}  
void HEPVis::SoSDL_Viewer::nextUI() {
 {SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
  button->getBackStyle()->color.setValue(SbColor_white);}
  fUI_current++;
  if(fUI_current>=fUI->shapes.getNum()) fUI_current = 0;
  SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
  button->getBackStyle()->color.setValue(SbColor_red);
}
void HEPVis::SoSDL_Viewer::prevUI() {
 {SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
  button->getBackStyle()->color.setValue(SbColor_white);}
  fUI_current--;
  if(fUI_current<0) fUI_current = fUI->shapes.getNum()-1;
  SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
  button->getBackStyle()->color.setValue(SbColor_red);
}
SbBool HEPVis::SoSDL_Viewer::activateUI(SbBool aDown) {
  if(!fUI) return FALSE;
  if(fUI->visible.getValue()==FALSE) return FALSE;
  if(fUI_current==-1) return FALSE;
  if(aDown==TRUE) return TRUE; //invokeCallbacks button on up only.
  SoSimpleButton* button = (SoSimpleButton*)fUI->shapes[fUI_current];
  button->invokeCallbacks();
  return TRUE;
}
SbBool HEPVis::SoSDL_Viewer::showShoot(SbBool& aRender) {
  aRender = FALSE;
  if(!fShootRegion) return FALSE;
  fShootRegion->visible.setValue
    (fShootRegion->visible.getValue()==FALSE?TRUE:FALSE);
  aRender = TRUE;
  return TRUE;
}
SbBool HEPVis::SoSDL_Viewer::shoot(SbBool aDown,SbBool& aRender) {
  if(activateUI(aDown)==TRUE) {
    aRender = TRUE;
    return TRUE;
  } else {
    if(fShootRegion) {
      if(fShootRegion->visible.getValue()==TRUE) {
        if(aDown==TRUE) {
          return emulateLeftButtonDown(aRender);
        } else {
          return emulateLeftButtonUp(aRender);
        }
      } else { 
        return FALSE;
      }
    } else {
      return FALSE;
    }
  }
}
