// this :
#include <HEPVis/SbRotation.h>

#include <math.h>

#include <Inventor/SbVec3d.h>

HEPVis::SbRotation::SbRotation()
//NOTE : the default HEPVis::SbRotation() corresponds to a unit matrix.
//WARNING : Inventor/SbRotation()::quad is not initialized !
//          So that the HEPVis::SbRotation() does not the same than
//          the Inventor::SbRotation().
:quat(0,0,0,1) 
{
}

HEPVis::SbRotation::SbRotation(const SbVec3d& axis,double radians) {
  //FIXME : if(axis.length()==0) //throw
  quat[3] = ::cos(radians/2);
  double sineval = ::sin(radians/2);
  SbVec3d a = axis;
  a.normalize();
  quat[0] = a[0] * sineval;
  quat[1] = a[1] * sineval;
  quat[2] = a[2] * sineval;
}

void HEPVis::SbRotation::multVec(const SbVec3d& src,SbVec3d& dst) const {
  //SbMatrix mat;
  //mat.setRotate(*this);

  double x = quat[0];
  double y = quat[1];
  double z = quat[2];
  double w = quat[3];

  double matrix[4][4];
  matrix[0][0] = w*w + x*x - y*y - z*z;
  matrix[0][1] = 2*x*y + 2*w*z;
  matrix[0][2] = 2*x*z - 2*w*y;
  matrix[0][3] = 0;

  matrix[1][0] = 2*x*y-2*w*z;
  matrix[1][1] = w*w - x*x + y*y - z*z;
  matrix[1][2] = 2*y*z + 2*w*x;
  matrix[1][3] = 0;

  matrix[2][0] = 2*x*z + 2*w*y;
  matrix[2][1] = 2*y*z - 2*w*x;
  matrix[2][2] = w*w - x*x - y*y + z*z;
  matrix[2][3] = 0;

  matrix[3][0] = 0;
  matrix[3][1] = 0;
  matrix[3][2] = 0;
  matrix[3][3] = w*w + x*x + y*y + z*z;

  //mat.multVecMatrix(src, dst);
  //if(SbMatrixP::isIdentity(this->matrix)) { dst = src; return; }

  double* t0 = matrix[0];
  double* t1 = matrix[1];
  double* t2 = matrix[2];
  double* t3 = matrix[3];

  SbVec3d s = src;

  double W = s[0]*t0[3] + s[1]*t1[3] + s[2]*t2[3] + t3[3];

  dst[0] = (s[0]*t0[0] + s[1]*t1[0] + s[2]*t2[0] + t3[0])/W;
  dst[1] = (s[0]*t0[1] + s[1]*t1[1] + s[2]*t2[1] + t3[1])/W;
  dst[2] = (s[0]*t0[2] + s[1]*t1[2] + s[2]*t2[2] + t3[2])/W;
}

