// this :
#include <HEPVis/actions/SoGL2PSAction.h>

// Inventor :
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/elements/SoViewportRegionElement.h>
#include <Inventor/elements/SoCacheElement.h>
#include <Inventor/errors/SoDebugError.h>

#include <HEPVis/SbStyles.h>
#include <HEPVis/SbString.h>

#include "HEPVis_gl2ps.h"

#include <stdio.h>

SO_ACTION_SOURCE(SoGL2PSAction)
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::initClass(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SO_ACTION_INIT_CLASS(SoGL2PSAction,SoGLRenderAction);

  SO_ACTION_ADD_METHOD(SoSeparator,separatorAction);
  SO_ACTION_ADD_METHOD(SoDrawStyle,drawStyleAction);
}
//////////////////////////////////////////////////////////////////////////////
SoGL2PSAction::SoGL2PSAction(
 const SbViewportRegion& aViewPortRegion
)
:SoGLRenderAction(aViewPortRegion)
,fFileFormat(EPS)
,fFileName("out.ps")
,fFile(0)
,fPageOptions(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SO_ACTION_CONSTRUCTOR(SoGL2PSAction);

  fPageOptions = GL2PS_OCCLUSION_CULL | 
                 GL2PS_BEST_ROOT | 
                 GL2PS_SILENT | 
                 GL2PS_DRAW_BACKGROUND;
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::setFileFormat(
 Format aFileFormat
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFileFormat = aFileFormat;
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoGL2PSAction::setPageOptions(
 const SbString& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPList words = SbStringGetWords(aOptions,";");
  int wordn = words.getLength();
  for(int wordi=0;wordi<wordn;wordi++) {
    SbString& word = *((SbString*)words[wordi]);
    if(!word.getLength()) continue;
    if(word=="DEFAULTS") {
      fPageOptions = GL2PS_OCCLUSION_CULL | 
                     GL2PS_BEST_ROOT | 
                     GL2PS_SILENT | 
                     GL2PS_DRAW_BACKGROUND;
    } else if(word=="NONE") {
      fPageOptions = 0;
    } else if(word=="DRAW_BACKGROUND") {
      fPageOptions |= GL2PS_DRAW_BACKGROUND;
    } else if(word=="SIMPLE_LINE_OFFSET") {
      fPageOptions |= GL2PS_SIMPLE_LINE_OFFSET;
    } else if(word=="SILENT") {
      fPageOptions |= GL2PS_SILENT;
    } else if(word=="BEST_ROOT") {
      fPageOptions |= GL2PS_BEST_ROOT;
    } else if(word=="OCCLUSION_CULL") {
      fPageOptions |= GL2PS_OCCLUSION_CULL;
    } else if(word=="NO_TEXT") {
      fPageOptions |= GL2PS_NO_TEXT;
    } else if(word=="LANDSCAPE") {
      fPageOptions |= GL2PS_LANDSCAPE;
    } else if(word=="NO_PS3_SHADING") {
      fPageOptions |= GL2PS_NO_PS3_SHADING;
    } else if(word=="NO_PIXMAP") {
      fPageOptions |= GL2PS_NO_PIXMAP;
    } else if(word=="USE_CURRENT_VIEWPORT") {
      fPageOptions |= GL2PS_USE_CURRENT_VIEWPORT;
    } else if(word=="COMPRESS") {
      fPageOptions |= GL2PS_COMPRESS;
    } else if(word=="NO_BLENDING") {
      fPageOptions |= GL2PS_NO_BLENDING;
    } else if(word=="TIGHT_BOUNDING_BOX") {
      fPageOptions |= GL2PS_TIGHT_BOUNDING_BOX;
    } else {
      SoDebugError::postInfo("SoGL2PSAction::setPageOptions",
        "bad option \"%s\".\n",word.getString());
    }
  } 
  SbStringDelete(words);
  return TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::setFileName(
 const char* aFileName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFileName = aFileName;
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::enableFileWriting(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fFileFormat==GL2PS_PDF) {
    fFile = ::fopen(fFileName.getString(),"wb");
  } else {
    fFile = ::fopen(fFileName.getString(),"w");
  }
  if(!fFile) {
    SoDebugError::post("SoGL2PSAction::enableFileWriting",
                       "Cannot open file %s",fFileName.getString());
    return;
  }
#ifdef __COIN__
#else //SGI
  const SbViewportRegion& vpr = getViewportRegion();
  SoViewportRegionElement::set(getState(),vpr);
  gl2psBegin();
#endif
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::disableFileWriting(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef __COIN__
#else //SGI
  gl2psEndViewport();
  gl2psEndPage();        
#endif
  ::fclose(fFile);
  fFile = 0;
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoGL2PSAction::fileWritingEnabled(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fFile?TRUE:FALSE);
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoGL2PSAction::addBitmap(
 int aWidth
,int aHeight
,float aXorig
,float aYorig
,float aXmove
,float aYmove
)
/////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFile) return FALSE;
  GLboolean valid;
  glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID,&valid);
  if(valid==GL_FALSE) return FALSE;
  float pos[4];
  glGetFloatv(GL_CURRENT_RASTER_POSITION,pos);
  int xoff = -(int)(aXmove + aXorig);
  int yoff = -(int)(aYmove + aYorig);
  int x = (int)(pos[0] + xoff);
  int y = (int)(pos[1] + yoff);
  // Should clip against viewport area :
  GLint vp[4];
  glGetIntegerv(GL_VIEWPORT,vp);
  GLsizei w = aWidth;
  GLsizei h = aHeight;
  if(x+w>(vp[0]+vp[2])) w = vp[0]+vp[2]-x;
  if(y+h>(vp[1]+vp[3])) h = vp[1]+vp[3]-y;
  int s = 3 * w * h;
  if(s<=0) return FALSE;
  float* image = new float[s];
  glReadPixels(x,y,w,h,GL_RGB,GL_FLOAT,image);
  GLint status = gl2psDrawPixels(w,h,xoff,yoff,GL_RGB,GL_FLOAT,image);
  delete [] image;
  return (status!=GL2PS_SUCCESS ? FALSE : TRUE);
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::beginViewport(
 SbBool aDrawBack
)
/////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFile) return;
  GLint vp[4];
  glGetIntegerv(GL_VIEWPORT,vp);
  if(aDrawBack) {
    gl2psBeginViewport(vp);
  } else {
    GLint opts = 0;
    gl2psGetOptions(&opts);
    gl2psSetOptions(0);
    gl2psBeginViewport(vp);
    gl2psSetOptions(opts);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::endViewport(
)
/////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFile) return;
  gl2psEndViewport();
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::beginTraversal(
 SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fFile) {
#ifdef __COIN__
    const SbViewportRegion& vpr = getViewportRegion();
    SoViewportRegionElement::set(getState(),vpr);
    gl2psBegin();
    traverse(aNode);
    gl2psEndViewport();
    gl2psEndPage();        
#else //SGI
    SoGLRenderAction::beginTraversal(aNode);
#endif
  } else {
    SoGLRenderAction::beginTraversal(aNode);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::gl2psBegin(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFile) return;

  int sort = GL2PS_BSP_SORT;
  //int sort = GL2PS_SIMPLE_SORT;
    
  const SbViewportRegion& vpr = getViewportRegion();
  SoViewportRegionElement::set(getState(),vpr);
 
  const SbVec2s& win = vpr.getWindowSize();
  GLint vp[4];
  vp[0] = 0;
  vp[1] = 0;
  vp[2] = win[0];
  vp[3] = win[1];

  int bufsize = 0;
  gl2psBeginPage("title","HEPVis::SoGL2PSAction", 
                 vp,
                 fFileFormat, 
                 sort, 
                 fPageOptions, 
                 GL_RGBA,0, NULL,0,0,0,
                 bufsize, 
                 fFile,fFileName.getString());    

  gl2psBeginViewport(vp);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::separatorAction(
 SoAction* aThis
,SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoGL2PSAction* This = (SoGL2PSAction*)aThis;
  if(This->fFile) {
    //SoSeparator may use render caching.
    aNode->doAction(aThis);
  } else {
#ifdef __COIN__
    SoNode::GLRenderS(aThis,aNode);
#else //SGI
    GLRenderS(aThis,aNode);
#endif
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::drawStyleAction(
 SoAction* aThis
,SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef __COIN__
  SoNode::GLRenderS(aThis,aNode);
#else //SGI
  GLRenderS(aThis,aNode);
#endif
  SoGL2PSAction* This = (SoGL2PSAction*)aThis;
  if(This->fFile) {
    SoDrawStyle* soDrawStyle = (SoDrawStyle*)aNode;
    if(soDrawStyle->style.getValue()==SoDrawStyle::LINES) { 
      float w = soDrawStyle->lineWidth.getValue();
      gl2psLineWidth(w);    
      unsigned short pattern = soDrawStyle->linePattern.getValue();
      if(pattern==SbLinePattern_solid) 
        gl2psDisable(GL2PS_LINE_STIPPLE);
      else 
        gl2psEnable(GL2PS_LINE_STIPPLE);
    }
  }
}
#ifdef __COIN__
#else //SGI
//////////////////////////////////////////////////////////////////////////////
void SoGL2PSAction::GLRenderS(
 SoAction* aThis
,SoNode* aNode
)
//////////////////////////////////////////////////////////////////////////////
// From SoNode::GLRenderS (private method in SGI).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //Coin :
  // if ((action->getCurPathCode() != SoAction::OFF_PATH) ||
  //    node->affectsState()) {
  //     <the below>
  // }

  SoGL2PSAction* This = (SoGL2PSAction*)aThis;
  if(!This->abortNow()) {
    aNode->GLRender(This);
  } else {
    SoCacheElement::invalidate(aThis->getState());
  }
}
#endif
