// this :
#include <HEPVis/misc/SoStyleCache.h>

#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoResetTransform.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoNormal.h>

#include <HEPVis/nodes/SoHighlightMaterial.h>

//////////////////////////////////////////////////////////////////////////////
SoStyleCache::SoStyleCache(
) 
:fMaterials(0)
,fHighlightMaterials(0)
,fLineStyles(0)
,fFilled(0)
,fPoints(0)
,fPhong(0)
,fBaseColor(0)
,fPerFace(0)
,fPerFaceIndexed(0)
,fOverall(0)
,fNormalZ(0)
,fResetTransform(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
SoStyleCache::~SoStyleCache(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void SoStyleCache::clearCache(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fMaterials->removeAllChildren();
  fHighlightMaterials->removeAllChildren();
  fLineStyles->removeAllChildren();

  removeAllChildren();

  fMaterials = 0;
  fHighlightMaterials = 0;
  fLineStyles = 0;
  fFilled = 0;
  fPoints = 0;
  fPhong = 0;
  fBaseColor = 0;
  fPerFace = 0;
  fPerFaceIndexed = 0;
  fOverall = 0;
  fNormalZ = 0;
  fResetTransform = 0;
}
//////////////////////////////////////////////////////////////////////////////
SoMaterial* SoStyleCache::getMaterial(
 const SbColor& aRGB
,float aTransparency
,SbBool aSetEmissiveColor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fMaterials) {
    fMaterials = new SoGroup;
    addChild(fMaterials);
  }
  int number = fMaterials->getNumChildren();
  for(int index=0;index<number;index++) { 
    SoMaterial* material = (SoMaterial*)fMaterials->getChild(index);
    if( (material->diffuseColor[0]==aRGB) &&
        (material->transparency[0]==aTransparency) &&
        ( ((aSetEmissiveColor==TRUE) && (material->emissiveColor[0]==aRGB)) ||
          ((aSetEmissiveColor==FALSE) && 
             (material->emissiveColor[0]==SbColor(0.0F,0.0F,0.0F))) )
      ) {
      return material;
    }
  }
  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(aRGB);
  if(aSetEmissiveColor==TRUE) material->emissiveColor.setValue(aRGB);
  material->transparency.setValue(aTransparency);
  fMaterials->addChild(material);
  return material;
}
//////////////////////////////////////////////////////////////////////////////
SoMaterial* SoStyleCache::getMaterial(
 float aRed
,float aGreen
,float aBlue
,float aTransparency
,SbBool aSetEmissiveColor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return getMaterial
           (SbColor(aRed,aGreen,aBlue),aTransparency,aSetEmissiveColor);
}
//////////////////////////////////////////////////////////////////////////////
SoHighlightMaterial* SoStyleCache::getHighlightMaterial(
 const SbColor& aRGB
,const SbColor& aHighlightRGB
,float aTransparency
,SbBool aSetEmissiveColor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fHighlightMaterials) {
    fHighlightMaterials = new SoGroup;
    addChild(fHighlightMaterials);
  }
  int number = fHighlightMaterials->getNumChildren();
  for(int index=0;index<number;index++) { 
    SoHighlightMaterial* material = 
      (SoHighlightMaterial*)fHighlightMaterials->getChild(index);
    if( (material->diffuseColor[0]==aRGB) &&
        (material->highlightColor[0]==aHighlightRGB) &&
        (material->transparency[0]==aTransparency) &&
        ( ((aSetEmissiveColor==TRUE) && (material->emissiveColor[0]==aRGB)) ||
          ((aSetEmissiveColor==FALSE) && 
             (material->emissiveColor[0]==SbColor(0.0F,0.0F,0.0F))) )
      ) {
      return material;
    }
  }
  SoHighlightMaterial* material = new SoHighlightMaterial;
  material->diffuseColor.setValue(aRGB);
  if(aSetEmissiveColor==TRUE) material->emissiveColor.setValue(aRGB);
  material->highlightColor.setValue(aHighlightRGB);
  material->transparency.setValue(aTransparency);
  fHighlightMaterials->addChild(material);
  return material;
}
//////////////////////////////////////////////////////////////////////////////
SoHighlightMaterial* SoStyleCache::getHighlightMaterial(
 float aRed
,float aGreen
,float aBlue
,float aHRed
,float aHGreen
,float aHBlue
,float aTransparency
,SbBool aSetEmissiveColor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return getHighlightMaterial(SbColor(aRed,aGreen,aBlue),
                              SbColor(aHRed,aHGreen,aHBlue),
                              aTransparency,aSetEmissiveColor);
}
//////////////////////////////////////////////////////////////////////////////
SoDrawStyle* SoStyleCache::getLineStyle(
 SbLinePattern aPattern
,float aWidth
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fLineStyles) {
    fLineStyles = new SoGroup;
    addChild(fLineStyles);
  }
  int number = fLineStyles->getNumChildren();
  for(int index=0;index<number;index++) { 
    SoDrawStyle* drawStyle = (SoDrawStyle*)fLineStyles->getChild(index);
    if( (drawStyle->style.getValue()==SoDrawStyle::LINES) &&
        (drawStyle->lineWidth.getValue()==aWidth) &&
        (drawStyle->linePattern.getValue()==aPattern) ) {
      return drawStyle;
    }
  }
  SoDrawStyle* drawStyle = new SoDrawStyle;
  drawStyle->style.setValue(SoDrawStyle::LINES);
  drawStyle->lineWidth.setValue(aWidth);
  drawStyle->linePattern.setValue(aPattern);
  fLineStyles->addChild(drawStyle);
  return drawStyle;
}
//////////////////////////////////////////////////////////////////////////////
SoDrawStyle* SoStyleCache::getFilled(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fFilled) {
    fFilled = new SoDrawStyle;
    fFilled->style.setValue(SoDrawStyle::FILLED);
    addChild(fFilled);
  } 
  return fFilled;
}
//////////////////////////////////////////////////////////////////////////////
SoDrawStyle* SoStyleCache::getPoints(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fPoints) {
    fPoints = new SoDrawStyle;
    fPoints->style.setValue(SoDrawStyle::POINTS);
    addChild(fPoints);
  } 
  return fPoints;
}
//////////////////////////////////////////////////////////////////////////////
SoLightModel* SoStyleCache::getLightModelPhong(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fPhong) {
    fPhong = new SoLightModel;
    fPhong->model.setValue(SoLightModel::PHONG);
    addChild(fPhong);
  } 
  return fPhong;
}
//////////////////////////////////////////////////////////////////////////////
SoLightModel* SoStyleCache::getLightModelBaseColor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fBaseColor) {
    fBaseColor = new SoLightModel;
    fBaseColor->model.setValue(SoLightModel::BASE_COLOR);
    addChild(fBaseColor);
  } 
  return fBaseColor;
}
//////////////////////////////////////////////////////////////////////////////
SoResetTransform* SoStyleCache::getResetTransform(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fResetTransform) {
    fResetTransform = new SoResetTransform;
    addChild(fResetTransform);
  } 
  return fResetTransform;
}
//////////////////////////////////////////////////////////////////////////////
SoNormalBinding* SoStyleCache::getNormalBindingOverall(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fOverall) {
    fOverall = new SoNormalBinding;
    fOverall->value = SoNormalBinding::OVERALL;
    addChild(fOverall);
  } 
  return fOverall;
}
//////////////////////////////////////////////////////////////////////////////
SoNormalBinding* SoStyleCache::getNormalBindingPerFace(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fPerFace) {
    fPerFace = new SoNormalBinding;
    fPerFace->value = SoNormalBinding::PER_FACE;
    addChild(fPerFace);
  } 
  return fPerFace;
}
//////////////////////////////////////////////////////////////////////////////
SoNormalBinding* SoStyleCache::getNormalBindingPerFaceIndexed(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fPerFaceIndexed) {
    fPerFaceIndexed = new SoNormalBinding;
    fPerFaceIndexed->value = SoNormalBinding::PER_FACE_INDEXED;
    addChild(fPerFaceIndexed);
  } 
  return fPerFaceIndexed;
}
//////////////////////////////////////////////////////////////////////////////
SoNormal* SoStyleCache::getNormalZ(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNormalZ) {
    fNormalZ = new SoNormal;
    SbVec3f nz(0,0,1);
    fNormalZ->vector.setValues(0,1,&nz);
    addChild(fNormalZ);
  } 
  return fNormalZ;
}
