/*-----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Node:             SoXtGui                                            */
/* Description:      HEP GUI Class                                           */
/*                  (draws heavily on Jeff Kallenbach's SoXtHepViewer class) */
/*                                                                           */
/* Author:           Lucas Taylor  July 3 1998                               */
/*                                                                           */
/*---------------------------------------------------------------------------*/

// this :
#include "HEPVis/Xt/SoXtGui.h"

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtPrintDialog.h>
#include <Inventor/Xt/SoXtMaterialEditor.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/Xt/viewers/SoXtFlyViewer.h>
#include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
#include <Inventor/Xt/viewers/SoXtWalkViewer.h>

#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEnvironment.h>
#include <Inventor/nodes/SoLabel.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoVertexProperty.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoAnnotation.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoColorIndex.h>
#include <Inventor/nodes/SoTranslation.h>

#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/actions/SoBoxHighlightRenderAction.h>
#include <Inventor/actions/SoCallbackAction.h>

#include <Inventor/sensors/SoTimerSensor.h>

#include <Inventor/engines/SoCompose.h>
#include <Inventor/engines/SoCalculator.h>

#include "HEPVis/Xt/SoXtColorEditor.h"

#include <X11/StringDefs.h>
#include <Xm/MainW.h>
#include <Xm/PanedW.h>
#include <X11/Shell.h>
//#include <X11/Xirisw/GlxMDraw.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/ArrowB.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>

#include <math.h>
//#include <ctype.h>
#include <unistd.h> // used for execlp and fork (for help callback)

#include <iostream>
using std::cout;
using std::endl;

String  
SoXtGui::_defaultSoXtGuiResources[] = {
        NULL
};
SbString* SoXtGui::helpURL = NULL;

static bool HEPVis_IsDigit(char aChar ){
  switch(aChar) {
  case '0':return true;
  case '1':return true;
  case '2':return true;
  case '3':return true;
  case '4':return true;
  case '5':return true;
  case '6':return true;
  case '7':return true;
  case '8':return true;
  case '9':return true;
  }
  return false;
}

////////////////////////////////////////////////////////////////////////////////
//
// Return pointer to current viewer
//
////////////////////////////////////////////////////////////////////////////////
SoXtFullViewer *
SoXtGui::getCurrentViewer() const {
  return _theFullViewer;
}

////////////////////////////////////////////////////////////////////////////////
//
// Return pointer to material editor
//
////////////////////////////////////////////////////////////////////////////////
SoXtMaterialEditor *
SoXtGui::getMaterialEditor() const {
  return _materialEditor;
}



////////////////////////////////////////////////////////////////////////////////
//
// Return pointer to particular viewer
//
////////////////////////////////////////////////////////////////////////////////
SoXtFullViewer *
SoXtGui::getViewer(int i) const {
  if (i==EXAMINER) return _theExaminerViewer;
  if (i==PLANE)    return _thePlaneViewer;
  if (i==WALK)     return _theWalkViewer;
  if (i==FLY)      return _theFlyViewer;
  return NULL; 
}

////////////////////////////////////////////////////////////////////////////////
//
// Return the menu bar widget (to allow user to add new items)
//
////////////////////////////////////////////////////////////////////////////////
Widget 
SoXtGui::getMenuBar() const {
  return _menuBar;
}

////////////////////////////////////////////////////////////////////////////////
//
// Return the file menu widget (to allow user to add new items)
//
////////////////////////////////////////////////////////////////////////////////
Widget 
SoXtGui::getFileMenu() const {
  return _fileMenu;
}

////////////////////////////////////////////////////////////////////////////////
//
// Return the view menu widget (to allow user to add new items)
//
////////////////////////////////////////////////////////////////////////////////
Widget 
SoXtGui::getViewMenu() const {
  return _viewMenu;
}

////////////////////////////////////////////////////////////////////////////////
//
// Return the help menu widget (to allow user to add new items)
//
////////////////////////////////////////////////////////////////////////////////
Widget 
SoXtGui::getHelpMenu() const {
  return _helpMenu;
}




////////////////////////////////////////////////////////////////////////////////
//
// Define URL for WWW help
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::setHelpURL(SbString TextItem) {
    
  cout << endl << "SoXtGui: help URL set to: " << TextItem.getString() << endl;
  
  *helpURL = TextItem.getString();
  
}

////////////////////////////////////////////////////////////////////////////////
//
// Callback routine for "Help" --> "WWW browser"
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::helpButtonCB(Widget, XtPointer , XtPointer) 
 {

  pid_t child;
  char *browser;
  const char *theHelpURL = helpURL->getString();

  browser = getenv("WWW_BROWSER");
  if (browser == NULL) {
     browser = "netscape";
     }
 
  if ((child=fork() )==0) {
     execlp(browser,browser,theHelpURL,NULL);     
     }

 }



////////////////////////////////////////////////////////////////////////////////
//
// Callback routine for "Clear text window"  
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::clearTextCB(Widget, XtPointer ClientData, XmPushButtonCallbackStruct *){
 
  SoXtGui *This = (SoXtGui *) ClientData;
  This->clearText();
  
}



////////////////////////////////////////////////////////////////////////////////
//
// Construct and display error message box
//  
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::setErrorMessage(char *text) {
  int ac;
  Arg args[20];
  ac = 0;
  XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("HEPVis warning:"));  ac++;
  XtSetArg(args[ac], XmNmessageString, XmStringCreateSimple(text));             ac++;
  Widget messageBox = XmCreateWarningDialog(_MainWindow, "HEPVis warning", args, ac);
  XtUnmanageChild(XmMessageBoxGetChild(messageBox,XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(messageBox,XmDIALOG_CANCEL_BUTTON));
  XtManageChild(messageBox);
}

////////////////////////////////////////////////////////////////////////////////
//
// Access to the text window
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::addText(char *TextItem) {
  if  (XtIsManaged(_theTextArea))
    XmTextReplace(_theTextArea,XmTextGetLastPosition(_theTextArea),XmTextGetLastPosition(_theTextArea),TextItem);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::clearText(){
  if  (XtIsManaged(_theTextArea))
    XmTextReplace(_theTextArea,0,XmTextGetLastPosition(_theTextArea),"");
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::showTextArea(){
  XtManageChild(_theTextArea);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::hideTextArea(){
  XtUnmanageChild(_theTextArea);
}

////////////////////////////////////////////////////////////////////////////////
//
// Resize window
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::resizeMain(short height, short width){

  Widget topShell = SoXt::getShellWidget(_MainWindow);
  if (!topShell) {
    setErrorMessage("No Shell in hierarchy - can't resize");
    return;
  }

  if ((height < 1)||(width < 1)) {
    setErrorMessage ("Illegal window dimensions - using defaults.\n");
    height = 812; width = 580;
  }
  SbVec2s dimen = SbVec2s (height,width);  
  SoXt::setWidgetSize(topShell,dimen);
}


////////////////////////////////////////////////////////////////////////////////
//
// Trigger new event
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::invokeTrig(Widget, XtPointer ClientData, 
XmPushButtonCallbackStruct *){
  int status=0;
  SoXtGui *This = (SoXtGui *) ClientData;
  if (This->_trigCallback) status = (*(This->_trigCallback)) (This->_trigCallData);
  if (status) This->setErrorMessage("No event trigger callback defined.");
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::setTrigCallback(TrigCallbackProc procedure, void *call_data) {
  _trigCallback = procedure;
  _trigCallData=call_data;
}







////////////////////////////////////////////////////////////////////////////////
//
// Printing
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::printCallback(Widget /*w*/, XtPointer clientData, 
XmPushButtonCallbackStruct */*callbackStruct*/) {
  SoXtGui *This = (SoXtGui *) clientData;
  print(This);
}

////////////////////////////////////////////////////////////////////////////////
//
// Close
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::close(Widget, XtPointer ClientData, XmPushButtonCallbackStruct 
*){
  SoXtGui *This = (SoXtGui *) ClientData;
  delete This;
}

////////////////////////////////////////////////////////////////////////////////
//
// Quit
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::invokeQuit(Widget, XtPointer ClientData, 
XmPushButtonCallbackStruct *){
  int status=0;
  SoXtGui *This = (SoXtGui *) ClientData;
  if (This->_quitCallback) status = (*(This->_quitCallback)) (This->_quitCallData);
  if (status) This->setErrorMessage("No quit.");
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::setQuitCallback(QuitCallbackProc procedure, void *call_data) {
  _quitCallback = procedure;
  _quitCallData=call_data;
}

////////////////////////////////////////////////////////////////////////////////
//
// Check text input for digits
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::verifyNumeric
  (Widget, XtPointer, XmTextVerifyCallbackStruct *call) {
  for (int i=0; i < call->text->length; i++) {	
    char c = call->text->ptr[i];
    if (!HEPVis_IsDigit(c)) {
      call->doit = False;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
//
// Anti-aliasing callback
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleAACB
  (Widget, XtPointer clientData, XmToggleButtonCallbackStruct *callData) {

  SoXtGui *This = (SoXtGui *) clientData;
  This->_theExaminerViewer->setAntialiasing(callData->set,1);
  This->_theFlyViewer->setAntialiasing(callData->set,1);
  This->_theWalkViewer->setAntialiasing(callData->set,1);
  This->_thePlaneViewer->setAntialiasing(callData->set,1);
}

////////////////////////////////////////////////////////////////////////////////
//
// Show text window callback
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleShowTextWindow
  (Widget, XtPointer clientData, XmToggleButtonCallbackStruct *callData) {

  SoXtGui *This = (SoXtGui *) clientData;

  if (callData->set == TRUE) {
    This->showTextArea();
  }
  else {
    This->hideTextArea();
  }
}


////////////////////////////////////////////////////////////////////////////////
//
// Perf Meter callback
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::togglePMCB
  (Widget , XtPointer clientData,XmToggleButtonCallbackStruct *callData) {

  SoXtGui *This = (SoXtGui *) clientData;
  if (callData->set == TRUE) {
    This->Metered = 1;
    instrumentViewer (This);
    instrumentScene (This);
  }
  else {
    This->Metered = 0;
    unInstrumentViewer (This);
    delete This->perfSensor;
  }
}

////////////////////////////////////////////////////////////////////////////////
//
// Remove performance meter from scene
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::unInstrumentViewer(SoXtGui *This)
{
  This->getCurrentViewer()->setOverlaySceneGraph((SoSeparator *) NULL);
}
////////////////////////////////////////////////////////////////////////////////
//
// Label (Axes) callback
//
////////////////////////////////////////////////////////////////////////////////
void SoXtGui::toggleAxesCB
  (Widget, XtPointer clientData, XmToggleButtonCallbackStruct *callData) {

  SoXtGui *This = (SoXtGui *) clientData;
  if (callData->set ==TRUE) {
    addLabels(This); }
  else {
    unInstrumentViewer(This); }
}
////////////////////////////////////////////////////////////////////////////////
//
// Add a set of co-ordinate axes in the overlay
//
////////////////////////////////////////////////////////////////////////////////
void SoXtGui::addLabels(SoXtGui *This)
{

//// Logo

  int color_index = 1;
  SbColor overlayColor(1, 0, 0);
  SoLightModel *lightModel = new SoLightModel;
  lightModel->model.setValue(SoLightModel::BASE_COLOR);

  SoColorIndex *colorIndex = new SoColorIndex;
  colorIndex->index = color_index;

  SoShapeHints *shapeHints = new SoShapeHints;
  shapeHints->faceType.setValue(SoShapeHints::UNKNOWN_FACE_TYPE);
  shapeHints->vertexOrdering.setValue(SoShapeHints::COUNTERCLOCKWISE);
//  SoCoordinate3 *coords = new SoCoordinate3();


  SoDrawStyle *drawStyle = new SoDrawStyle;
  drawStyle->style = SoDrawStyle::FILLED;

  SoSeparator *logoSep = new SoSeparator;
  SoTransform *logoTransform = new SoTransform;
  logoTransform->scaleFactor.setValue(0.4F,0.4F,0.4F);
  logoTransform->translation.setValue(0.52F,0.6F,0.0F);
  logoSep->addChild(lightModel);
  logoSep->addChild(colorIndex);
  logoSep->addChild(logoTransform);
  logoSep->addChild(drawStyle);
  logoSep->addChild(shapeHints);
//  logoSep->addChild(coords);

//  No Logo in generic viewer	

//////// Axes

//make a set of axes
  SoSeparator *axisSep = new SoSeparator;

//the lines for the axes
  SoLineSet *theAxes = new SoLineSet;
  int32_t numVertices[3] = {2,2,2}; // three line segments with 2 vertices each
  const SbVec3f xyz[6] = {
    SbVec3f(0,0,0),SbVec3f(1,0,0),SbVec3f(0,0,0),
    SbVec3f(0,1,0),SbVec3f(0,0,0),SbVec3f(0,0,1)
  };
  SoVertexProperty *vertProperty = new SoVertexProperty;
  vertProperty->vertex.setValues(0,6,xyz);
  theAxes->vertexProperty.setValue(vertProperty);
  theAxes->numVertices.setValues(0,3,numVertices);
  axisSep->addChild(theAxes);

//Labels for the axes
  SoFont *axisFont = new SoFont;
  axisFont->name.setValue("Times-Roman");
  axisFont->size.setValue(24.0);
  axisSep->addChild(axisFont);

  SoText2 *xLabel = new SoText2;
  SoText2 *yLabel = new SoText2;
  SoText2 *zLabel = new SoText2;
  SoTranslation *xTranslation = new SoTranslation;
  SoTranslation *yTranslation = new SoTranslation;
  SoTranslation *zTranslation = new SoTranslation;
  SoSeparator *xSep = new SoSeparator;
  SoSeparator *ySep = new SoSeparator;
  SoSeparator *zSep = new SoSeparator;

  xLabel->string = "x";
  yLabel->string = "y";
  zLabel->string = "z";
  xTranslation->translation.setValue(1.1F,0,0);
  yTranslation->translation.setValue(0,1.1F,0);
  zTranslation->translation.setValue(0,0,1.1F);


  axisSep->addChild(xSep);
  xSep->addChild(xTranslation);
  xSep->addChild(xLabel);

  axisSep->addChild(ySep);
  ySep->addChild(yTranslation);
  ySep->addChild(yLabel);

  axisSep->addChild(zSep);
  zSep->addChild(zTranslation);
  zSep->addChild(zLabel);

  SoDrawStyle *lineDrawStyle = new SoDrawStyle;
  lineDrawStyle->style = SoDrawStyle::LINES;
  SoTransform *testTransform = new SoTransform;
  testTransform->scaleFactor.setValue(0.2F,0.2F,0.2F);
  testTransform->translation.setValue(-0.7F,-0.6F,0.0F);


////////////////////////////////
  for (int iViewer=0; iViewer < 4; iViewer++) {
    SoSeparator *topSep = new SoSeparator;

    SoXtFullViewer *theViewer = This->getViewer(iViewer);

    // Must get camera for each viewer; duplicate inverse rotations
    SoSeparator *testSep = new SoSeparator;
    SoCamera *theCamera = theViewer->getCamera();
    SoDecomposeRotation * viewCameraRot = new SoDecomposeRotation;
    viewCameraRot->rotation.connectFrom(&theCamera->orientation);
    SoCalculator *rotCalculator = new SoCalculator;
    rotCalculator->expression.set1Value(0,"oa = -a");
    rotCalculator->a.connectFrom(&viewCameraRot->angle);
    
    SoComposeRotation *realCameraRot = new SoComposeRotation;
    realCameraRot->angle.connectFrom(&rotCalculator->oa);
    realCameraRot->axis.connectFrom(&viewCameraRot->axis);
    
    SoRotation *cameraRotation = new SoRotation;
    cameraRotation->rotation.connectFrom(&realCameraRot->rotation);
    testSep->addChild(lineDrawStyle);
    testSep->addChild(lightModel);
    testSep->addChild(colorIndex);
    testSep->addChild(testTransform);
    testSep->addChild(cameraRotation);
    //  testSep->addChild(box);
    testSep->addChild(axisSep);
    
    topSep->addChild(testSep);
    topSep->addChild(logoSep);
    theViewer->setOverlayColorMap(color_index, 1, &overlayColor);
    theViewer->setOverlaySceneGraph(topSep);
  }
}
////////////////////////////////////////////////////////////////////////////////
//
// Add performance meter to scene in overlay
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::instrumentViewer(SoXtGui *This)
{

  //
  // Create the overlay scene    
  //
  SoSeparator *perfMeter = new SoSeparator;

  //
  // Add a callback node which will set the viewport 
  //
  SoCallback *overlayCB = new SoCallback;
  overlayCB->setCallback(overlayViewportCB);
  perfMeter->addChild(overlayCB);

  //
  // Add an orthographic camera
  //
  perfMeter->addChild(new SoOrthographicCamera());

  //
  // Add the text string to display the tris per second
  //
  char str[150];
  SoFont *perfMeterFont = new SoFont;
  perfMeterFont->size = 18;
  perfMeterFont->name = "Helvetica";
  perfMeter->addChild(perfMeterFont);
  This->perfMeterText = new SoText2;
  This->perfMeterText->justification = SoText2::LEFT;
  sprintf (str, "%d tris/frame",This->numTris);
  This->perfMeterText->string.setValue(str);
  perfMeter->addChild(This->perfMeterText);

  SbColor col(1, 1, 1);
  This->getCurrentViewer()->setOverlayColorMap(1, 1, &col);
  This->getCurrentViewer()->setOverlaySceneGraph(perfMeter);

  //
  // Update the performance meter every two seconds
  //
  This->perfSensor = 
	      new SoTimerSensor(perfSensorCB, This);
  This->perfSensor->setInterval(3.0);
  This->perfSensor->schedule();

  //
  // Reset the meter to only show the number of tris/frame -
  // otherwise the numbers might look misleading
  //
  This->getCurrentViewer()->addStartCallback(viewStartCB, This);
}

void
SoXtGui::overlayViewportCB(void *, SoAction *action)
{
    if (action->isOfType(SoGLRenderAction::getClassTypeId())) 
        glViewport(0, 0, 40, 40);
}

////////////////////////////////////////////////////////////////////////////////
//
// A sensor callback in the overlay scene updates the
// performance meter while the object is animating.
// 
////////////////////////////////////////////////////////////////////////////////
void
SoXtGui::perfSensorCB(void *data, SoSensor *)
{
  SoXtGui *This= (SoXtGui *) data;
  SoText2 *perfText = This->perfMeterText;
  char str[100];

  if (((SoXtExaminerViewer *) This->getCurrentViewer())->isAnimating()) {
    if (This->wasAnimating) {

      SbTime timeDiff = SbTime::getTimeOfDay() - This->startTime;
      double frames = 
	  This->numFramesRendered * 1000.0 / timeDiff.getMsecValue();
      frames = floor(frames * 100.0 + 0.5) / 100.0;

      // 
      // Update the perf meter display if the numbers have changed
      // 
      if (frames != This->lastFrames) {
          sprintf (str, 
		  "%d tris/frame,   %.01f frames/sec,   %.01f tris/sec", 
		  This->numTris, frames, frames * This->numTris);
    	  perfText->string.setValue(str);
	  This->lastFrames = frames;
      }

      // 
      // Don't reset the timer here.  By using the total
      // time from when the object started animating
      // we will get more accurate numbers - especially 
      // for slow objects.
      // 
    }
    else {
      // 
      // Reset the timer when we detect the object has
      // started animating.
      // 
      This->numFramesRendered = 0;
      This->startTime = SbTime::getTimeOfDay();
      This->wasAnimating = TRUE;
    }
  }
  else
    This->wasAnimating = FALSE;
}

////////////////////////////////////////////////////////////////////////////////
//
// Reset the meter to only show the tris/sec number.
// Otherwise the numbers might look misleading
//
////////////////////////////////////////////////////////////////////////////////
void
SoXtGui::viewStartCB(void *data, SoXtViewer *)
{
    SoXtGui *This = (SoXtGui *) data;
    if (This->Metered == 0) return;

    char str[150];
    SoText2 *perfText = This->perfMeterText;

    sprintf (str, "%d tris/frame", This->numTris);
    perfText->string.setValue(str);
}
	
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::countUpdatesCB(void * data, SoAction *action) {
  if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
    SoXtGui *This = (SoXtGui *) data;
    This->numFramesRendered++;
  }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void
SoXtGui::countTriangleCB(void *userData, SoCallbackAction *,
     		const SoPrimitiveVertex *, 
		const SoPrimitiveVertex *,
                const SoPrimitiveVertex *)
{
    int32_t *curCount = (int32_t *) userData;
    (*curCount)++;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
int
SoXtGui::countTriangles(SoNode *root)
{
    SoCallbackAction ca;
    int32_t numTris = 0;

    root->ref();

    ca.addTriangleCallback(SoShape::getClassTypeId(), countTriangleCB,
                           (void *) &numTris);
    ca.apply(root);

    root->unrefNoDelete();

    return (numTris);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::instrumentScene(SoXtGui *This) {
  
  
  This->ancestor->renderCaching = SoSeparator::OFF;
  This->ancestor->boundingBoxCaching = SoSeparator::OFF;
  This->ancestor->pickCulling = SoSeparator::OFF;
  
  SoCallback *countUpdates = new SoCallback;
  countUpdates->setCallback(SoXtGui::countUpdatesCB,This);
  
  This->ancestor->insertChild(countUpdates,0);

  This->numTris = countTriangles(This->ancestor);
}



////////////////////////////////////////////////////////////////////////////////
//
// Default Quit Callback
//
////////////////////////////////////////////////////////////////////////////////
int SoXtGui::quitCallback(void *) {
  exit (0);
  return 0;
}

////////////////////////////////////////////////////////////////////////////////
//
// Spawn new SoXtGui - create shell, create new viewer, set
// scene graph, show viewer
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::spawnView
  (Widget , XtPointer clientData, XmPushButtonCallbackStruct *) {

  SoXtGui *This = (SoXtGui *) clientData;

  Widget TheNewWindow = 
    XtAppCreateShell ("Inventor","Inventor",
                      topLevelShellWidgetClass,
                      XtDisplay (This->_MainWindow),
                      NULL,0);


  if (!TheNewWindow) {
    This->setErrorMessage("Cannot create Shell for new view.\n");
    return;
  }
  SoXtGui *TheNewViewer = new SoXtGui(TheNewWindow);
  
  //
  // Text Area in same state
  //
  if (!(XtIsManaged(This->_theTextArea))) TheNewViewer->hideTextArea();

  //
  // Copy Scene Graph
  //  
  TheNewViewer->setSceneGraph(This->getSceneGraph());

  //
  // Window same size
  //
  SbVec2s dimen = SoXt::getWidgetSize(This->_MainWindow);
  short h, w;
  dimen.getValue(h, w);
  TheNewViewer->resizeMain(h,w);

  //
  // Background Color
  //
  SbColor col = This->getCurrentViewer()->getBackgroundColor();
  TheNewViewer->_theExaminerViewer->setBackgroundColor(col);
  TheNewViewer->_thePlaneViewer->setBackgroundColor(col);
  TheNewViewer->_theWalkViewer->setBackgroundColor(col);
  TheNewViewer->_theFlyViewer->setBackgroundColor(col);

  //
  // Inherit Transparency and set correct button
  //
  XmToggleButtonSetState(TheNewViewer->_screenDoorButton,XmToggleButtonGetState(This->_screenDoorButton),0);
  XmToggleButtonSetState(TheNewViewer->_blendedButton,XmToggleButtonGetState(This->_blendedButton),0);
  XmToggleButtonSetState(TheNewViewer->_sortedButton,XmToggleButtonGetState(This->_sortedButton),0);
  XmToggleButtonSetState(TheNewViewer->_sortedBlendedButton,XmToggleButtonGetState(This->_sortedBlendedButton),0);

  int i;
  SoGLRenderAction::TransparencyType tt = (This->getCurrentViewer())->getTransparencyType();
  for (i=0;i<4;i++) {
    TheNewViewer->getViewer(i)->setTransparencyType(tt);
  }

  //
  // Inherit Highlight Style and set correct button
  //
  XmToggleButtonSetState(TheNewViewer->_boxHighlightButton,0,0);
  XmToggleButtonSetState(TheNewViewer->_linesHighlightButton,0,0);
  XmToggleButtonSetState(TheNewViewer->_noHighlightButton,0,0);
  for (i=0;i<4;i++) {
    
    if (XmToggleButtonGetState(This->_boxHighlightButton)){
      XmToggleButtonSetState(TheNewViewer->_boxHighlightButton,1,0);
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }  
    if (XmToggleButtonGetState(This->_linesHighlightButton)){
      XmToggleButtonSetState(TheNewViewer->_linesHighlightButton,1,0);
      SoLineHighlightRenderAction *rA = new SoLineHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }  
    if (XmToggleButtonGetState(This->_noHighlightButton)) {
      XmToggleButtonSetState(TheNewViewer->_noHighlightButton,1,0);
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(FALSE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }
  }
 
  //
  // Display new view
  //
  TheNewViewer->show();
  SoXt::show(TheNewWindow); 
  TheNewViewer->_theFullViewer->viewAll(); 
}

////////////////////////////////////////////////////////////////////////////////
//
// Change viewing style
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleViewCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {

  //
  // Turn this widget on, turn all the others off!
  //
  SoXtGui *This = (SoXtGui *) clientData;
  XmToggleButtonSetState(This->_examinerButton,0,0);
  XmToggleButtonSetState(This->_planeButton,0,0);
  XmToggleButtonSetState(This->_walkButton,0,0);
  XmToggleButtonSetState(This->_flyButton,0,0);
  XmToggleButtonSetState(w,1,0);

  //
  // Save the properties of the old viewer
  //
  SoCamera *theCamera = This->_theFullViewer->getCamera();
  SoXtViewer::DrawStyle theInterDrawStyle = This->_theFullViewer->getDrawStyle(SoXtViewer::INTERACTIVE);
  SoXtViewer::DrawStyle theStillDrawStyle = This->_theFullViewer->getDrawStyle(SoXtViewer::STILL);
  This->_theFullViewer->hide();

  if (w==This->_examinerButton) This->_theFullViewer=This->_theExaminerViewer;
  if (w==This->_planeButton)    This->_theFullViewer=This->_thePlaneViewer;
  if (w==This->_walkButton)     This->_theFullViewer=This->_theWalkViewer;
  if (w==This->_flyButton)      This->_theFullViewer=This->_theFlyViewer;

  if (This->_theFullViewer == This->_theExaminerViewer)
    XtSetSensitive(This->_perfMeterButton,TRUE);
  else
    XtSetSensitive(This->_perfMeterButton,FALSE);

  if (w==This->_planeButton)    {
    This->_thePlaneViewer->getCamera()->orientation.setValue(SbVec3f(0,0,1),0);
    This->_thePlaneViewer->viewAll();
  }
  This->_theFullViewer->setCamera(theCamera);
  This->_theFullViewer->setDrawStyle(SoXtViewer::INTERACTIVE,theInterDrawStyle);
  This->_theFullViewer->setDrawStyle(SoXtViewer::STILL,theStillDrawStyle);
  This->_theFullViewer->show(); 
}

////////////////////////////////////////////////////////////////////////////////
//
// Change Transparency
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleTransCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {
  SoXtGui *This = (SoXtGui *) clientData;
  XmToggleButtonSetState(This->_screenDoorButton,0,0);
  XmToggleButtonSetState(This->_blendedButton,0,0);
  XmToggleButtonSetState(This->_sortedButton,0,0);
  XmToggleButtonSetState(This->_sortedBlendedButton,0,0);
  XmToggleButtonSetState(w,1,0);

  for (int i=0;i<4;i++) {
    if (w==This->_screenDoorButton)    This->getViewer(i)->setTransparencyType(SoGLRenderAction::SCREEN_DOOR);
    if (w==This->_blendedButton)       This->getViewer(i)->setTransparencyType(SoGLRenderAction::BLEND);
    if (w==This->_sortedButton)        This->getViewer(i)->setTransparencyType(SoGLRenderAction::DELAYED_BLEND);
    if (w==This->_sortedBlendedButton) This->getViewer(i)->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
  }
 
}

////////////////////////////////////////////////////////////////////////////////
//
// Change Highlighting
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleHLCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {

  SoXtGui *This = (SoXtGui *) clientData;

  //
  // Store Transparency type for reset
  //  
  SoGLRenderAction::TransparencyType tt = (This->getCurrentViewer())->getTransparencyType();

  XmToggleButtonSetState(This->_boxHighlightButton,0,0);
  XmToggleButtonSetState(This->_linesHighlightButton,0,0);
  XmToggleButtonSetState(This->_noHighlightButton,0,0);
  XmToggleButtonSetState(w,1,0);

  for (int i=0;i<4;i++) {
    
    if (w==This->_boxHighlightButton){
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      SbColor myHighlightColor(1.0, 1.0, 1.0);      // set the highlight color to white
      rA->setColor(myHighlightColor);
      (This->getViewer(i))->setGLRenderAction(rA);
    }  
    if (w==This->_linesHighlightButton){
      SoLineHighlightRenderAction *rA = new SoLineHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      SbColor myHighlightColor(1.0, 1.0, 1.0);      // set the highlight color to white
      rA->setColor(myHighlightColor);
      This->getViewer(i)->setGLRenderAction(rA);
    }  
    if (w==This->_noHighlightButton) {
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(FALSE);
      rA->setTransparencyType(tt);
      This->getViewer(i)->setGLRenderAction(rA);
    }
  }
 
}

////////////////////////////////////////////////////////////////////////////////
//
// Spawn background color editor
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::editBackgroundColor(Widget, XtPointer clientData, 
XmPushButtonCallbackStruct *) {
    SoXtGui *This = (SoXtGui *) clientData;
    if (This->_backgroundColorEditor == NULL) {
	This->_backgroundColorEditor = new HEPVis_SoXtColorEditor;
	This->_backgroundColorEditor->setColor(SbColor(0, 0, 0));
        This->_backgroundColorEditor->setTitle( "Background Color" );
        This->_backgroundColorEditor->addColorChangedCallback
          (SoXtGui::backgroundColorCB,This);
    }

    This->_backgroundColorEditor->show();
}

////////////////////////////////////////////////////////////////////////////////
//
// Spawn material editor
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::editMaterial(Widget, XtPointer clientData, 
XmPushButtonCallbackStruct *) {
    SoXtGui *This = (SoXtGui *) clientData;
    if (This->_materialEditor == NULL) {
	This->_materialEditor = new SoXtMaterialEditor;
        This->_materialEditor->setTitle( "Material Editor" );

	//        This->_materialEditor->addColorChangedCallback
	//          (SoXtGui::backgroundColorCB,This);

    }

    This->_materialEditor->show();
}



////////////////////////////////////////////////////////////////////////////////
//
// Called after bg color editor used, to change bg color in all views
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::backgroundColorCB(void *clientData, const SbColor *color) {
    SoXtGui *This = (SoXtGui *) clientData;
    This->_theExaminerViewer->setBackgroundColor(*color);
    This->_theFlyViewer->setBackgroundColor(*color);
    This->_theWalkViewer->setBackgroundColor(*color);
    This->_thePlaneViewer->setBackgroundColor(*color);
}

////////////////////////////////////////////////////////////////////////////////
//
// Control method for printing
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::print(SoXtGui *This)
{

  //
  // Create print dialog if neccessary
  //
  if (This->printDialog == NULL) {
    This->printDialog = new SoXtPrintDialog;
    This->printDialog->setTitle("HEPVis print dialog");
    This->printDialog->setBeforePrintCallback(SoXtGui::beforePrint, (void *) This);
    This->printDialog->setAfterPrintCallback (SoXtGui::afterPrint,  (void *) This);
  }
  
  //
  // Send the render area size and scene graph to the print dialog
  //
  Widget widget = This->_theFullViewer->getRenderAreaWidget();
  if (widget != NULL) {
    Arg args[2];
    int n = 0;
    SbVec2s sz;
    XtSetArg(args[n], XtNwidth,  &sz[0]); n++;
    XtSetArg(args[n], XtNheight, &sz[1]); n++;
    XtGetValues(widget, args, n);
    This->printDialog->setPrintSize(sz);
  }
  This->printDialog->show();
}

////////////////////////////////////////////////////////////////////////////////
//
// Callback routine inputs a filename, writes scenegraph to iv file (ascii)
//
////////////////////////////////////////////////////////////////////////////////
void
SoXtGui::saveIV (Widget, void * clientData, XmFileSelectionBoxCallbackStruct *data) {

  SoXtGui *This = (SoXtGui *) clientData;

  char *filename;
  XmStringGetLtoR(data->value,
		  (XmStringCharSet) XmSTRING_DEFAULT_CHARSET, &filename);

  SoWriteAction myAction;
  myAction.getOutput()->openFile(filename);
  myAction.getOutput()->setBinary(FALSE);

  myAction.apply(This->sceneGraph);
  myAction.getOutput()->closeFile();
  XtFree(filename);
}

////////////////////////////////////////////////////////////////////////////////
//
// Control routine for "Save File"
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::saveIVCallback(Widget /*w*/, XtPointer clientData, 
XmPushButtonCallbackStruct */*callbackStruct*/) {
  SoXtGui *This = (SoXtGui *) clientData;
  Widget fileDialog = NULL;
  
  if (fileDialog == NULL) {
    Arg args[5];
    int n = 0;
    
    //
    // Unmanage when ok/cancel are pressed
    //
    
    XtSetArg(args[n], XmNautoUnmanage, TRUE); n++;
    fileDialog = XmCreateFileSelectionDialog(
					     This->_MainWindow, "SaveIV Dialog", args, n);
    
    XtAddCallback(fileDialog, XmNokCallback,
		  (XtCallbackProc) SoXtGui::saveIV, This);
  }
  
  //
  // Manage the dialog
  //
  
  XtManageChild(fileDialog);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::beforePrint(void *tt, SoXtPrintDialog *) {
    SoXtGui *This = (SoXtGui *) tt;
    This->printDialog->setSceneGraph(This->ancestor);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::afterPrint(void *, SoXtPrintDialog *) {
}

////////////////////////////////////////////////////////////////////////////////
//
// Constructor
//
////////////////////////////////////////////////////////////////////////////////
SoXtGui::SoXtGui( Widget parent, const char *name,  
  SbBool buildInsideParent):SoXtComponent(parent,name,buildInsideParent) {

  //
  // Viewer:
  //
  setTitle(name);
  printDialog=NULL;
  create(parent);
  setQuitCallback(SoXtGui::quitCallback,this);
  
  helpURL=new SbString("http://fnpspa.fnal.gov/graphics/HEPVis/www");
  cout << "SoXtGui: default help URL set to: " << helpURL->getString()  << endl;

  //
  // Scene:
  //
  ancestor = new SoSeparator();
  ancestor->ref();

  lightsCameraEnvironment   = new SoGroup;
  environment     	    = new SoEnvironment;
  lightGroup 	    	    = new SoGroup;
  envLabel	  	    = new SoLabel;

  envLabel->label.setValue("HEPVis Environment");
  lightsCameraEnvironment->addChild(envLabel);
  lightsCameraEnvironment->addChild(environment);
  lightsCameraEnvironment->addChild(lightGroup);
  ancestor->addChild(lightsCameraEnvironment);

  //
  // Lights;
  //
  _topFlood = new SoDirectionalLight();
  _topFlood->direction.setValue(0,-1,0);
  lightGroup->addChild(_topFlood);

  _bottomFlood = new SoDirectionalLight();
  _bottomFlood->direction.setValue(0,1,0);
  lightGroup->addChild(_bottomFlood);

  _leftFlood = new SoDirectionalLight();
  _leftFlood->direction.setValue(1,0,0);
  lightGroup->addChild(_leftFlood);

  _rightFlood = new SoDirectionalLight();
  _rightFlood->direction.setValue(-1,0,0);
  lightGroup->addChild(_rightFlood);

  _topFlood->on=False;
  _bottomFlood->on=False;
  _leftFlood->on=False;
  _rightFlood->on=False;

  perspectiveCamera = new SoPerspectiveCamera();
  perspectiveCamera->ref();

  lightsCameraEnvironment->addChild(perspectiveCamera);

  sceneGraph = new SoSeparator();
  ancestor->addChild(sceneGraph);

  _theExaminerViewer->setSceneGraph(ancestor);
  _theWalkViewer->setSceneGraph(ancestor);
  _theFlyViewer->setSceneGraph(ancestor);
  _thePlaneViewer->setSceneGraph(ancestor);

  _theExaminerViewer->setCamera(perspectiveCamera);
  _theWalkViewer->setCamera(perspectiveCamera);
  _theFlyViewer->setCamera(perspectiveCamera);
  _thePlaneViewer->setCamera(perspectiveCamera);

  _theFullViewer->viewAll();
  _theFullViewer->saveHomePosition();
  _theFullViewer->show();

  _backgroundColorEditor=NULL;
  _materialEditor=NULL;

  Metered = 0;
  numTris = 0;
  numFramesRendered = 0;
  wasAnimating = FALSE;
  startTime=0.0; 
  lastFrames = 0;
  perfSensor=NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Sets scene graph
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::setSceneGraph(SoNode *node) {
  ancestor->removeChild(sceneGraph);
  ancestor->addChild(node);
  sceneGraph=node;
  
}

////////////////////////////////////////////////////////////////////////////////
//
// Returns scene graph
//
////////////////////////////////////////////////////////////////////////////////
SoNode *
SoXtGui::getSceneGraph() const {
  return sceneGraph;
}

////////////////////////////////////////////////////////////////////////////////
//
// destructor
//
////////////////////////////////////////////////////////////////////////////////
SoXtGui::~SoXtGui() 
{
  delete printDialog;
  delete _backgroundColorEditor;
  delete _materialEditor;
  delete _theExaminerViewer;
  delete _thePlaneViewer;
  delete _theWalkViewer;
  delete _theFlyViewer;
  ancestor->removeAllChildren();
  ancestor->unref();
  XtDestroyWidget(getParentWidget());
  delete perfSensor;
  delete helpURL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Called to apply light request
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::toggleLights
  (Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *callData){

  SoXtGui *This = (SoXtGui *) clientData;
  if (w == This->_headlightButton) {
    for (int i=0;i<4;i++) This->getViewer(i)->setHeadlight(callData->set);
  }
  else if (w == This->_leftFloodButton) {
    This->_leftFlood->on=callData->set;
  }
  else if (w == This->_rightFloodButton) {
    This->_rightFlood->on=callData->set;
  }
  else if (w == This->_topFloodButton) {
    This->_topFlood->on=callData->set;
  }
  else if (w == This->_bottomFloodButton) {
    This->_bottomFlood->on=callData->set;
  }
}





////////////////////////////////////////////////////////////////////////////////
//
// Create viewer - set up widgets, etc.
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::create(Widget parent) 
{
    Arg      args[256];
    Cardinal ac=0;
    
    XtInitializeWidgetClass(xmMainWindowWidgetClass);
    XtInitializeWidgetClass(xmPanedWindowWidgetClass);
    XtInitializeWidgetClass(xmTextWidgetClass);
    XtInitializeWidgetClass(xmFormWidgetClass);
    XtInitializeWidgetClass(xmLabelWidgetClass);
    XtInitializeWidgetClass(xmTextFieldWidgetClass);
    XtInitializeWidgetClass(xmArrowButtonWidgetClass);
    XtInitializeWidgetClass(xmCascadeButtonWidgetClass);
    XtInitializeWidgetClass(xmRowColumnWidgetClass);
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    XtInitializeWidgetClass(xmToggleButtonWidgetClass);
    
    //
    // Create an unmanaged widget as the top of the hierarchy.
    //
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 69); ac++;
    XtSetArg(args[ac], XmNy, 454); ac++;
    XtSetArg(args[ac], XmNwidth, 812); ac++;
    XtSetArg(args[ac], XmNheight, 580); ac++;
    char *_name = (char *) getTitle();
    _MainWindow = XmCreateMainWindow(parent,_name,args, ac);
    setBaseWidget(_MainWindow);
    
    //
    // Create a paned window (panes are stacked vertically) 
    //
    
    ac = 0;
    XtSetArg(args[ac], XmNwidth, 812); ac++;
    XtSetArg(args[ac], XmNheight, 560); ac++;
    _panedWindow = XmCreatePanedWindow(_MainWindow,"panedWindow",args, ac);
    XtManageChild(_panedWindow);
    
    //
    // Create viewers to go in the top pane of the window
    //
    
    _theExaminerViewer = new SoXtExaminerViewer(_panedWindow);
    _thePlaneViewer    = new SoXtPlaneViewer(_panedWindow);
    _theWalkViewer     = new SoXtWalkViewer(_panedWindow);
    _theFlyViewer      = new SoXtFlyViewer(_panedWindow);
    _theFullViewer     = _theExaminerViewer; //by default;



  
    //
    // Create text area to go in the next pane of the window
    //
    
    ac = 0;
    XtSetArg(args[ac], XmNwidth,                           800); ac++;
    XtSetArg(args[ac], XmNheight,                           40); ac++;
    XtSetArg(args[ac], XmNeditable,                      FALSE); ac++;
    XtSetArg(args[ac], XmNscrollBarDisplayPolicy,  XmAUTOMATIC); ac++;  // ScrolledWindow handles scrolling (not application)
    XtSetArg(args[ac], XmNscrollingPolicy,         XmSTATIC);    ac++;  // always show vertical scroll bar
    _theTextArea = XmCreateScrolledText(_panedWindow, "theTextArea", args,ac);
    XtManageChild(_theTextArea);

//     //
//     // Create form to go in the next pane of the window 
//     // (not filled with anything by default, hence not managed by default)
//     //
//     
//     ac = 0;
//     XtSetArg(args[ac], XmNresizePolicy, XmRESIZE_ANY); ac++;
//     XtSetArg(args[ac], XmNwidth, 806); ac++;
//     XtSetArg(args[ac], XmNheight,  5); ac++;
//     _filterForm = XmCreateForm(_panedWindow, "filterForm", args, ac);
//     //  XtManageChild(_filterForm);
//     
//     ac = 0;
//     XtSetArg(args[ac], XmNpaneMinimum,  1); ac++;
//     XtSetArg(args[ac], XmNpaneMaximum, 85); ac++;
//     XtSetValues(_filterForm, args, ac);
//     
// 
    
    //======================================================================
    //
    // Create menu bar
    // ===============
    // 
        
    ac = 0;
    XtSetArg(args[ac], XmNwidth,                       812); ac++;
    XtSetArg(args[ac], XmNheight,                       35); ac++;
    _menuBar = XmCreateMenuBar(_MainWindow,"menuBar",args,ac);
    XtManageChild(_menuBar);
    
    
    //----------------------------------------------------------------------
    //
    // Create the "File" menu
    // ----------------------
    // 
    
    ac = 0;
    XtSetArg(args[ac],XmNlabelString,XmStringCreateSimple("File")); ac++;
    XtSetArg(args[ac], XmNx, 5); ac++;
    XtSetArg(args[ac], XmNy, 5); ac++;
    XtSetArg(args[ac], XmNwidth, 45); ac++;
    XtSetArg(args[ac], XmNheight, 25); ac++;
    _fileMenuButton = XmCreateCascadeButton(_menuBar,"fileMenuButton",args,ac);
    
    XtManageChild(_fileMenuButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx,                             0); ac++;
    XtSetArg(args[ac], XmNy,                             0); ac++;
    XtSetArg(args[ac], XmNwidth,                        54); ac++;
    XtSetArg(args[ac], XmNheight,                      112); ac++;
    XtSetArg(args[ac], XmNtearOffModel, XmTEAR_OFF_ENABLED); ac++;
    _fileMenu = XmCreatePulldownMenu(XtParent(_fileMenuButton),"fileMenu",args,ac);
    
    
    //
    // Create item:  "New HepViewer"
    // -----------------------------
    //
    // Should parameters such as visibility, etc. should be the same
    // or independent for the different viewers .. how to print several views into single picture etc. 
    //
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("New HepViewer"));  ac++;
    _newExaminerButton = XmCreatePushButton(_fileMenu,"newExaminerButton",args,ac);
    XtAddCallback(_newExaminerButton,XmNactivateCallback,(XtCallbackProc) SoXtGui::spawnView, this);
    XtManageChild(_newExaminerButton);

    
       
    //
    // Create item:  "Open"
    // --------------------
    //   
    // This is not managed! Not by this class!  At least not yet!
    // The unmanaged button is placed here so that it isn't at the bottom
    // of the menu; the subclasses can manage it and bind it as they please.
    //   
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Open"));  ac++;
    _openButton = XmCreatePushButton(_fileMenu,"openButton", args, ac);

    
    //
    // Create item:  "New event"
    // ------------------------
    //
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("New event"));  ac++;
    _trigButton = XmCreatePushButton(_fileMenu,"trigButton",args, ac);
    XtAddCallback(_trigButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::invokeTrig, (XtPointer) this);
    XtManageChild(_trigButton);
    


    //
    // Create item:  "Save IV"
    // -----------------------
    //
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Save IV"));  ac++;
    _saveButton = XmCreatePushButton(_fileMenu,"saveButton",args,ac);
    XtAddCallback(_saveButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::saveIVCallback, (XtPointer) this);
    XtManageChild(_saveButton);
    
    
    
    //
    // Create item:  "Print"
    // ---------------------
    //
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Print"));  ac++;
    _printButton = XmCreatePushButton(_fileMenu,"printButton",args, ac);
    XtAddCallback(_printButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::printCallback, (XtPointer) this);
    XtManageChild(_printButton);
    
       
    //
    // Create item:  "Close"
    // ---------------------
    //
    // I took this out until we decide what to do with multiple viewers/viewports.
    // (we don't want to close the ONLY viewer we have!!!)
    //
    // ac = 0;
    // XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Close"));  ac++;
    // _closeButton = XmCreatePushButton(_fileMenu,"closeButton",args, ac);
    // XtAddCallback(_closeButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::close, (XtPointer) this);
    // XtManageChild(_closeButton);
    // 
  
      
    //
    // Create item:  "Quit"
    // --------------------
    //
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Quit"));  ac++;
    _quitButton = XmCreatePushButton(_fileMenu,"quitButton",args, ac);
    XtAddCallback(_quitButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::invokeQuit, (XtPointer) this);
    XtManageChild(_quitButton);
    
    
    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _fileMenu); ac++;
    XtSetValues(_fileMenuButton, args, ac);



       
    //----------------------------------------------------------------------
    //
    // Create the "Viewer" menu
    // ------------------------
    //
    // 
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,  XmStringCreateSimple("Viewer"));  ac++;
    XtSetArg(args[ac], XmNx,                                        50);  ac++;
    XtSetArg(args[ac], XmNy,                                         5);  ac++;
    XtSetArg(args[ac], XmNwidth,                                    54);  ac++;
    XtSetArg(args[ac], XmNheight,                                   25);  ac++;       
       
    _viewMenuButton = XmCreateCascadeButton(_menuBar, "viewMenuButton",args,ac);
    XtManageChild(_viewMenuButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 119); ac++;
    XtSetArg(args[ac], XmNy, 484); ac++;
    XtSetArg(args[ac], XmNwidth, 222); ac++;
    XtSetArg(args[ac], XmNheight, 156); ac++;
    XtSetArg(args[ac], XmNtearOffModel, XmTEAR_OFF_ENABLED); ac++;
    _viewMenu = XmCreatePulldownMenu(XtParent(_viewMenuButton),"viewMenu",args,ac);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _viewMenu); ac++;
    XtSetValues(_viewMenuButton, args, ac);
    
    

    //
    // Create item:  "Viewer style"
    // ----------------------------
    //    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Viewer style"));  ac++;
    _viewStyle = XmCreateCascadeButton(_viewMenu,"viewStyle",args,ac);
    XtManageChild(_viewStyle);
    
    ac = 0;
    XtSetArg(args[ac], XmNx,      335); ac++;
    XtSetArg(args[ac], XmNy,      593); ac++;
    XtSetArg(args[ac], XmNwidth,  102); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _pulldownMenu = XmCreatePulldownMenu(XtParent(_viewStyle),"pulldownMenu",args,ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Examiner"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _examinerButton = XmCreateToggleButton(_pulldownMenu, "examinerButton",args, ac);
    XtAddCallback(_examinerButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleViewCB, this);
    XtManageChild(_examinerButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Plane"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _planeButton = XmCreateToggleButton(_pulldownMenu,"planeButton",args, ac);
    XtAddCallback(_planeButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleViewCB, this);
    XtManageChild(_planeButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Walk"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _walkButton = XmCreateToggleButton(_pulldownMenu,"walkButton",args, ac);
    XtAddCallback(_walkButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleViewCB, this);
    XtManageChild(_walkButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Fly"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _flyButton = XmCreateToggleButton(_pulldownMenu,"flyButton",args,ac);
    XtAddCallback(_flyButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleViewCB, this);
    XtManageChild(_flyButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _pulldownMenu); ac++;
    XtSetValues(_viewStyle, args, ac);

    //
    // Create item:  "Viewer size"
    // ---------------------------
    //                


    

    //
    // Create item:  "Lights"
    // ----------------------
    //           
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Lights"));  ac++;       
    _lightsButton = XmCreateCascadeButton(_viewMenu,"lightsButton",args,ac);
    XtManageChild(_lightsButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 491); ac++;
    XtSetArg(args[ac], XmNwidth, 176); ac++;
    XtSetArg(args[ac], XmNheight, 129); ac++;
    _lightsMenu = XmCreatePulldownMenu(XtParent(_lightsButton),"lightsMenu",args,ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Headlight"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;         
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _headlightButton = XmCreateToggleButton(_lightsMenu,"headlightButton",args,ac);
    XtManageChild(_headlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Left floodlight"));  ac++;  
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _leftFloodButton = XmCreateToggleButton(_lightsMenu,"leftFloodButton",args,ac);
    XtManageChild(_leftFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Right floodlight"));  ac++;       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _rightFloodButton = XmCreateToggleButton(_lightsMenu,"rightFloodButton",args,ac);
    XtManageChild(_rightFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Top floodlight"));  ac++;       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _topFloodButton = XmCreateToggleButton(_lightsMenu,"topFloodButton",args,ac);
    XtManageChild(_topFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Bottom floodlight"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _bottomFloodButton = XmCreateToggleButton(_lightsMenu,"bottomFloodButton",args,ac);
    XtManageChild(_bottomFloodButton);
    
    XtAddCallback(_headlightButton,  XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleLights,this);
    XtAddCallback(_topFloodButton,   XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleLights,this);
    XtAddCallback(_bottomFloodButton,XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleLights,this);
    XtAddCallback(_leftFloodButton,  XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleLights,this);
    XtAddCallback(_rightFloodButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleLights,this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _lightsMenu); ac++;
    XtSetValues(_lightsButton, args, ac);
    
    
    //
    // Create item:  "Transparency style"
    // ----------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Transparency style"));  ac++;
    _transparencyButton = XmCreateCascadeButton(_viewMenu,"transparencyButton",args,ac);
    XtManageChild(_transparencyButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 516); ac++;
    XtSetArg(args[ac], XmNwidth, 144); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _transparencyMenu = XmCreatePulldownMenu(XtParent(_transparencyButton),"transparencyMenu",args,ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Screen Door"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _screenDoorButton = XmCreateToggleButton(_transparencyMenu,"screenDoorButton",args,ac);
    XtManageChild(_screenDoorButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Blended"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _blendedButton = XmCreateToggleButton(_transparencyMenu,"blendedButton",args,ac);
    XtManageChild(_blendedButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Sorted"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _sortedButton = XmCreateToggleButton(_transparencyMenu,"sortedButton",args,ac);
    XtManageChild(_sortedButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Sorted Blended"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _sortedBlendedButton = XmCreateToggleButton(_transparencyMenu,"sortedBlendedButton",args,ac);
    XtManageChild(_sortedBlendedButton);
    
    XtAddCallback(_screenDoorButton,    XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleTransCB, this);
    XtAddCallback(_blendedButton,       XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleTransCB, this);
    XtAddCallback(_sortedButton,        XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleTransCB, this);
    XtAddCallback(_sortedBlendedButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleTransCB, this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _transparencyMenu); ac++;
    XtSetValues(_transparencyButton, args, ac);
 
    //
    // Create item:  "Highlight style"
    // -------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Highlight style"));  ac++;
    _highlightButton = XmCreateCascadeButton(_viewMenu,"highlightButton",args,ac);
    XtManageChild(_highlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 516); ac++;
    XtSetArg(args[ac], XmNwidth, 144); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _highlightMenu = XmCreatePulldownMenu(XtParent(_highlightButton),"highlightMenu",args,ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Box"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _boxHighlightButton = XmCreateToggleButton(_highlightMenu,"boxHighlightButton",args,ac);
    XtManageChild(_boxHighlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Lines"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _linesHighlightButton = XmCreateToggleButton(_highlightMenu,"linesHighlightButton",args,ac);
    XtManageChild(_linesHighlightButton);
        
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("None"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _noHighlightButton = XmCreateToggleButton(_highlightMenu,"noHighlightButton",args,ac);
    XtManageChild(_noHighlightButton);
    
    XtAddCallback(_boxHighlightButton,   XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleHLCB, this);
    XtAddCallback(_linesHighlightButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleHLCB, this);
    XtAddCallback(_noHighlightButton,    XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleHLCB, this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _highlightMenu); ac++;
    XtSetValues(_highlightButton, args, ac);

    //
    // Create item:  "Background colour"
    // --------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Background colour"));  ac++;
    _editBackgroundButton = XmCreatePushButton(_viewMenu,"editBackgroundButton",args,ac);
    XtAddCallback(_editBackgroundButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::editBackgroundColor,this);
    XtManageChild(_editBackgroundButton);
    
    //
    // Create item:  "Material Editor"
    // --------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Material Editor"));  ac++;
    _editMaterialButton = XmCreatePushButton(_viewMenu,"editMaterialButton",args,ac);
    XtAddCallback(_editMaterialButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::editMaterial,this);
    XtManageChild(_editMaterialButton);

    //
    // Create item:  "Antialiasing"
    // ----------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Antialiasing"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _antialiasingButton = XmCreateToggleButton(_viewMenu,"antialiasingButton",args, ac);
    XtAddCallback(_antialiasingButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleAACB, this);
    XtManageChild(_antialiasingButton);

    //
    // Create item:  "Performance monitor"
    // ----------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Performance monitor")); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _perfMeterButton = XmCreateToggleButton (_viewMenu,"perfMeterButton",args, ac);
    XtAddCallback(_perfMeterButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::togglePMCB, this);
    XtManageChild(_perfMeterButton);

    //
    // Create item:  "Labels"
    // ----------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Axes")); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _axesButton = XmCreateToggleButton (_viewMenu,"axesButton",args, ac);
    XtAddCallback(_axesButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleAxesCB, this);
    XtManageChild(_axesButton);

//      
//    Don't add this button (although it does work) 
//    It would be more useful if the viewer expanded/contracted to 
//    allow for space (not) used by the text widget but for now it doesn't
//      
//      
//     //
//     // Create item:  "Show text window"
//     // -------------------------------
//     //                
//     ac = 0;
//     XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Show text window"));  ac++;
//     XtSetArg(args[ac], XmNvisibleWhenOff,                                  True);  ac++;
//     _showTextWindowButton = XmCreateToggleButton(_viewMenu,"showTextWindowButton",args, ac);
//     XtAddCallback(_showTextWindowButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtGui::toggleShowTextWindow, this);
//     XtManageChild(_showTextWindowButton);
//     XmToggleButtonSetState(_showTextWindowButton,TRUE,TRUE);      // set the toggle button to be on

    //
    // Create item:  "Clear text window"
    // ----------------------------------
    //                
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Clear text window"));  ac++;
    _clearTextButton = XmCreatePushButton(_viewMenu,"clearTextButton",args, ac);
    XtAddCallback(_clearTextButton,XmNactivateCallback, (XtCallbackProc) SoXtGui::clearTextCB, (XtPointer) this);
    XtManageChild(_clearTextButton);

    //-------------------------------------------------------------------------
    //
    // Add the "Help" menu
    // -------------------
 
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,  XmStringCreateSimple("Help")); ac++;
    XtSetArg(args[ac], XmNx,					  50); ac++;
    XtSetArg(args[ac], XmNy,					   5); ac++;
    XtSetArg(args[ac], XmNwidth,				  54); ac++;
    XtSetArg(args[ac], XmNheight,				  25); ac++;
    _helpMenuButton = XmCreateCascadeButton(_menuBar, "helpMenuButton",args,ac);
    XtManageChild(_helpMenuButton);
 
    ac = 0;
    XtSetArg(args[ac], XmNx,					   0); ac++;
    XtSetArg(args[ac], XmNy,					   0); ac++;
    XtSetArg(args[ac], XmNwidth,				  54); ac++;
    XtSetArg(args[ac], XmNheight,				 112); ac++;
    XtSetArg(args[ac], XmNtearOffModel, 	  XmTEAR_OFF_ENABLED); ac++;
    _helpMenu = XmCreatePulldownMenu(XtParent(_helpMenuButton),"helpMenu",args,ac);
 
    //
    // Tell the menu bar that this is the help widget (then it's put on the far right)
    //
    XtVaSetValues (_menuBar, XmNmenuHelpWidget, _helpMenuButton, NULL);
 
    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId,			   _helpMenu); ac++;
    XtSetValues(_helpMenuButton, args, ac);
 
    //
    // Add items to the "Help" menu
    // ----------------------------
 
    ac = 0;
    XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("WWW browser"));  ac++;
    _helpPopupButton = XmCreatePushButton(_helpMenu,"helpPopupButton",args,ac);
    XtAddCallback(_helpPopupButton,XmNactivateCallback, (XtCallbackProc)  helpButtonCB, (XtPointer) NULL);
    XtManageChild(_helpPopupButton);


}
////////////////////////////////////////////////////////////////////////////////
//
// Build widget for controlling Twigs
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::createTwigWidget (Twig *theTopTwig)
{
  
  Arg      args[256];
  Cardinal ac=0;
  char     title[20];
  Twig   *theChildTwig;   
      

  cout << endl << endl << "----------------------------------------" << endl; 
    
  if (_menuBar==NULL) {
    cout << "SoXtGui::createTwigWidget>  no menu bar widget" << endl;
    return;
    } 
   
  cout << endl
       << "SoXtGui::createTwigWidget> Build widget for Twig =  " 
       << (theTopTwig->getName()).getString() 
       << "    which has " 
       << theTopTwig->numberOfChildren() 
       << " children..." 
       << endl << endl; 
    
  theTopTwig->setParentWidget(_MainWindow);  
    
  //-------------------------------------------------------------------------
  //
  // Add the "Twig" menu
  // -------------------

//  strcpy(title,"Show_");
  strcpy(title,"");
  const char *chname = (theTopTwig->getName()).getString();
  strcat(title,chname);
 
  cout << "SoXtGui::createTwigWidget> adding Widget:   " << title << endl;
  
  ac = 0;
  XtSetArg(args[ac], XmNlabelString,  XmStringCreateSimple(title)); ac++;
  XtSetArg(args[ac], XmNx,                                      50); ac++;
  XtSetArg(args[ac], XmNy,                                       5); ac++;
  XtSetArg(args[ac], XmNwidth,                                  54); ac++;
  XtSetArg(args[ac], XmNheight,                                 25); ac++;       
  _twigMenuButton = XmCreateCascadeButton(_menuBar, "twigMenuButton",args,ac);
  XtManageChild(_twigMenuButton);
  
  ac = 0;
  XtSetArg(args[ac], XmNx,                                       0); ac++;
  XtSetArg(args[ac], XmNy,                                       0); ac++;
  XtSetArg(args[ac], XmNwidth,                                  54); ac++;
  XtSetArg(args[ac], XmNheight,                                112); ac++;
  XtSetArg(args[ac], XmNtearOffModel,           XmTEAR_OFF_ENABLED); ac++;
  _twigMenu = XmCreatePulldownMenu(XtParent(_twigMenuButton),"twigMenu",args,ac);
  
  ac = 0;
  XtSetArg(args[ac], XmNsubMenuId,                       _twigMenu); ac++;
  XtSetValues(_twigMenuButton, args, ac);
    
  //-------------------------------------------------------------------------
  //
  // Add a button on the menu for each sub-Twig 
  // ------------------------------------------
    
  if (theTopTwig->numberOfChildren() <= 0) {
    cout << "SoXtGui::createTwigWidget> no children for this twig => cannot build sub-menu items" << endl;
    }
  else {
      
     for (int iChild = 0; iChild < theTopTwig->numberOfChildren(); iChild++) {
     
       theChildTwig = theTopTwig->getChildByIndex(iChild);
       
       if (theChildTwig == NULL) {
          cout << "SoXtGui::createTwigWidget> serious error !!! Bad Twig structure !!!" << endl;
          exit(1);
       }
 
       theChildTwig->setParentWidget(_MainWindow);  
    

//     strcpy(title,"Show ");
       strcpy(title,"");
       const char *chname = (theChildTwig->getName()).getString();
       strcat(title,chname);
 
       cout << "SoXtGui::createTwigWidget> adding Widget:   " << title << endl;
  
       ac = 0;
       XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple(title));  ac++;
       Widget _tempWidget = XmCreatePushButton(_twigMenu,"dummy",args,ac);
       XtAddCallback(_tempWidget,XmNactivateCallback, (XtCallbackProc) SoXtGui::createTwigDialog, (XtPointer) theChildTwig);
       XtManageChild(_tempWidget);
 
     }
  }
  
  //-------------------------------------------------------------------------
  //
  // Add a button to print out the Twig tree  
  // ---------------------------------------
    
  ac = 0;
  XtSetArg(args[ac], XmNlabelString,XmStringCreateSimple("Print Tree"));  ac++;
  Widget _printTreeWidget = XmCreatePushButton(_twigMenu,"dummy",args,ac);
  XtAddCallback(_printTreeWidget,XmNactivateCallback, (XtCallbackProc) SoXtGui::printTreeCB, (XtPointer) theTopTwig);
  XtManageChild(_printTreeWidget);

  cout << endl  
       << endl << "SoXtGui::createTwigWidget>   Ended"                 
       << endl << "----------------------------------------"
       << endl ; 
  
}


////////////////////////////////////////////////////////////////////////////////
//
// Callback routine for "print IV Tree"  
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::printTreeCB(Widget, XtPointer ClientData, XmPushButtonCallbackStruct *){
 
  Twig   *theTwig = (Twig *) ClientData;
  theTwig->printTree();  
}

////////////////////////////////////////////////////////////////////////////////
//
// Callback routine for "print IV Tree"  
//
////////////////////////////////////////////////////////////////////////////////
void 
SoXtGui::createTwigDialog(Widget, XtPointer ClientData, XmPushButtonCallbackStruct *){
 
  Twig   *theTwig = (Twig *) ClientData;
  theTwig->createShowDialog();  
}
#ifdef __KCC
//////////////////////
//
//  X stream routines
//
//////////////////////
InfoStreamBuf::int_type InfoStreamBuf::overflow(int_type int_char) {
  std::char_traits<char_type> isTraits;
  const char_type aChar = isTraits.to_char_type(int_char);
  if (int_char != traits_type::eof() && aChar != '\n') {
    my_buffer.append(1,aChar); }
  else {
    if (int_char != traits_type::eof() ) my_buffer.append(1,aChar);
    char *local_string = new char[my_buffer.size()+1];
    my_buffer.copy(local_string,my_buffer.size());
    local_string[my_buffer.size()] = '\0';
    theGui->addText(local_string);
    delete [] local_string;
    my_buffer = ""; //reset the buffer
  }
  return int_char;
}
#endif //__KCC
