// this :
#include <HEPVis/Xt/viewers/SoXtHepEventViewer.h>

#include <math.h>
#include <ctype.h>

#include <X11/StringDefs.h>

#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/ArrowB.h>
#include <Xm/PushB.h>
#include <Xm/FileSB.h>
#include <Xm/List.h>
#include <Xm/Form.h>

#include <Inventor/errors/SoDebugError.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoBlinker.h>
#include <Inventor/SoPath.h>
#include <algorithm>
#include <iostream>


void SoXtHepEventViewer::iconize(SoPath *ppath, const char *name) {

  SoFullPath *path = (SoFullPath *) ppath; 

  SoNode *parent = path->getNodeFromTail(1);
  int     index  = path->getIndexFromTail(0);
  SoNode *node   = path->getTail();

  if (parent->isOfType(SoGroup::getClassTypeId()) ){
    
    SoGroup *group = (SoGroup *) parent;
    
    
    Icon icon;
    icon.node   = node;
    icon.parent = group;
    icon.index  = index;

    std::list<Icon>::iterator iC;
    iC = std::find(_iconList.begin(),_iconList.end(),icon);
    if (iC==_iconList.end()) {
      SoSwitch *sw = new SoSwitch();
      sw->ref();
      sw->addChild(path->getTail());
      group->replaceChild(index,sw);
      icon.sw = sw;
      iC=_iconList.insert(_iconList.end(),icon);
    }
    
    _activeIcons.push_back(iC);
    XmListAddItem(_theIconBox, XmStringCreateSimple((char *) name), 0);

  }
  
}


String  SoXtHepEventViewer::_defaultSoXtHepEventViewerResources[] = {
        NULL
};

void SoXtHepEventViewer::setEventSelectCallback(EventSelectCallbackProc proceedure, void *EventSelectCallData){
  int ac;
  Arg args[20];
  if (proceedure) {
    ac = 0;
    XtSetArg(args[ac], XmNeditable, True); ac++;
    XtSetValues(_runNumberField, args, ac);
    XtSetValues(_eventNumberField, args, ac);
  }
  else {
    ac = 0;
    XtSetArg(args[ac], XmNeditable, False); ac++;
    XtSetValues(_runNumberField, args, ac);
    XtSetValues(_eventNumberField, args, ac);
  }
  _eventSelectCallback=proceedure;
  _eventSelectCallData=EventSelectCallData;
}

void SoXtHepEventViewer::setNextEventCallback  (NextEventCallbackProc   proceedure, void *NextEventCallData){
  if (proceedure) {
    XtManageChild(_nextEventButton);
  }
  else {
    XtUnmanageChild(_nextEventButton);
  }
  _nextEventCallback=proceedure;
  _nextEventCallData=NextEventCallData;
}

void SoXtHepEventViewer::setRewindCallback     (RewindCallbackProc      proceedure, void *RewindCallData){
  if (proceedure) {
    XtManageChild(_rewindButton);
  }
  else {
    XtUnmanageChild(_rewindButton);
  }
  _rewindCallback=proceedure;
  _rewindCallData=RewindCallData;
}

void SoXtHepEventViewer::setFileOpenCallback   (FileOpenCallbackProc    proceedure, void *FileOpenCallData){
  if (proceedure) {
    XtManageChild(_openButton);
  }
  else {
    XtUnmanageChild(_openButton);
  }
  _fileOpenCallback=proceedure;
  _fileOpenCallData=FileOpenCallData;
}

void SoXtHepEventViewer::setRunNumber(int number) {
  char digitString[256];
  sprintf(digitString,"%d",number);
  XmTextFieldSetString(_runNumberField,digitString);
}

void SoXtHepEventViewer::setEventNumber(int number) {
  char digitString[256];
  sprintf(digitString,"%d",number);
  XmTextFieldSetString(_eventNumberField,digitString);
}


void SoXtHepEventViewer::openCallback(Widget, XtPointer clientData, XmPushButtonCallbackStruct *){
  Widget fileDialog = NULL;
  Arg args[5];
  int n = 0;

  SoXtHepEventViewer *This = (SoXtHepEventViewer *) clientData;

  XtSetArg(args[n], XmNautoUnmanage, TRUE); n++;
  fileDialog = XmCreateFileSelectionDialog(
					   This->_MainWindow, "File Dialog", args, n);

  XtAddCallback(fileDialog, XmNokCallback,
  		(XtCallbackProc)SoXtHepEventViewer::invokeFileOpen, This);
  
  XtManageChild(fileDialog);
}

//
// Callback routine that gets called form the open file dialog.
void SoXtHepEventViewer::invokeFileOpen (Widget, void * clientData, XmFileSelectionBoxCallbackStruct *data)
{
  int status = 0;
  SoXtHepEventViewer *This = (SoXtHepEventViewer *) clientData;
  char *filename;
  XmStringGetLtoR(data->value,
		  (XmStringCharSet) XmSTRING_DEFAULT_CHARSET, &filename);

  if (This->_fileOpenCallback) status = (*(This->_fileOpenCallback)) (filename,This->_fileOpenCallData);
  char statusString[256];
  sprintf(statusString,"Cannot open input file %s", filename);
  if (status) This->setErrorMessage(statusString);
  XtFree(filename);
  This->clear();
}

void SoXtHepEventViewer::invokeSelectEvent (Widget, XtPointer ClientData, XmAnyCallbackStruct *){
  int status=0;
  SoXtHepEventViewer *This = (SoXtHepEventViewer *) ClientData;
  char *runString   = XmTextFieldGetString(This->_runNumberField);
  char *eventString = XmTextFieldGetString(This->_eventNumberField);
  int theRun=0,theEvent=0;
  if (!runString || !sscanf(runString,"%d",&theRun)) {
    SoDebugError::post("SoXtHepEventViewer::invokeSelectEvent"," cannot parse run number");
    return;
  }

  if (!eventString || !sscanf(eventString,"%d",&theEvent)) {
    SoDebugError::post("SoXtHepEventViewer::invokeSelectEvent"," cannot parse run number");
    return;
  }
  if (This->_eventSelectCallback) status = (*(This->_eventSelectCallback)) (theRun, theEvent, This->_eventSelectCallData);
  char statusString[256];
  sprintf(statusString,"Cannot locate run=%d, event=%d.", theRun, theEvent);
  if (status) This->setErrorMessage(statusString);
  XtFree(runString);
  XtFree(eventString);
  This->clear();
}


void SoXtHepEventViewer::invokeNextEvent(Widget, XtPointer ClientData, XmPushButtonCallbackStruct *){
  int status=0;
  SoXtHepEventViewer *This = (SoXtHepEventViewer *) ClientData;
  if (This->_nextEventCallback) status = (*(This->_nextEventCallback)) (This->_nextEventCallData);
  if (status) This->setErrorMessage("Cannot fetch next event.");
  This->clear();
}

void SoXtHepEventViewer::deiconize(Widget, XtPointer ClientData, XmListCallbackStruct *cb){
  SoXtHepEventViewer *This = (SoXtHepEventViewer *) ClientData;

  for (int N=cb->selected_item_count-1; N>=0; N--) {
    int selectedItem = cb->selected_item_positions[N]-1;
    std::vector<std::list<Icon>::iterator>::iterator cI=This->_activeIcons.begin()+selectedItem;
    (*(*cI)).sw->whichChild=SO_SWITCH_ALL;
    This->_activeIcons.erase(cI);
  }
  XmListDeletePositions(This->_theIconBox,cb->selected_item_positions, cb->selected_item_count);
}

void SoXtHepEventViewer::invokeRewind(Widget, XtPointer ClientData, XmPushButtonCallbackStruct *){
  int status=0;
  SoXtHepEventViewer *This = (SoXtHepEventViewer *) ClientData;
  if (This->_rewindCallback) status = (*(This->_rewindCallback)) (This->_rewindCallData);
  if (status) This->setErrorMessage("Cannot locate beginning of event stream.");
  This->clear();
}

SoXtHepEventViewer::SoXtHepEventViewer( Widget parent, const char *aName,  SbBool buildInsideParent):SoXtHepViewer(parent,aName,buildInsideParent) {

  // Viewer:
  setTitle(aName);
  create(parent);

}



SoXtHepEventViewer::~SoXtHepEventViewer() 
{
}

void SoXtHepEventViewer::create(Widget) 
{
    Arg      args[256];
    Cardinal ac=0;
    
    //
    // Create an unmanaged widget as the top of the hierarchy.
    //
    
    ac = 0;
    XtSetArg(args[ac], XmNresizePolicy, XmRESIZE_ANY); ac++;
    XtSetArg(args[ac], XmNwidth, 806); ac++;
    XtSetArg(args[ac], XmNheight, 85); ac++;
    _filterForm = XmCreateForm(_panedWindow, "filterForm", args, ac);
    XtManageChild(_filterForm);


    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Event"));  ac++;
    XtSetArg(args[ac], XmNx, 370); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 65); ac++;
    XtSetArg(args[ac], XmNheight, 33); ac++;
    _eventLabel = XmCreateLabel(_filterForm,"eventLabel",args, ac);
    XtManageChild(_eventLabel);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Run")); ac++;
    XtSetArg(args[ac], XmNx, 200); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 72); ac++;
    XtSetArg(args[ac], XmNheight, 33); ac++;
    _runLabel = XmCreateLabel(_filterForm,"runLabel",args, ac);
    XtManageChild(_runLabel);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 450); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 100); ac++;
    XtSetArg(args[ac], XmNheight, 40); ac++;
    XtSetArg(args[ac], XmNeditable, False); ac++;
    _eventNumberField = XmCreateTextField(_filterForm,"eventNumberField",args, ac);
    XtAddCallback(_eventNumberField,XmNmodifyVerifyCallback, (XtCallbackProc) SoXtHepViewer::verifyNumeric, (XtPointer) this);
    XtAddCallback(_eventNumberField,XmNactivateCallback, (XtCallbackProc) SoXtHepEventViewer::invokeSelectEvent, (XtPointer) this);
    XtManageChild(_eventNumberField);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 300); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 100); ac++;
    XtSetArg(args[ac], XmNheight, 40); ac++;
    XtSetArg(args[ac], XmNeditable, False); ac++;
    _runNumberField = XmCreateTextField(_filterForm,"runNumberField",args, ac);
    XtAddCallback(_runNumberField,XmNmodifyVerifyCallback, (XtCallbackProc) SoXtHepViewer::verifyNumeric, (XtPointer) this);
    XtAddCallback(_runNumberField,XmNactivateCallback, (XtCallbackProc) SoXtHepEventViewer::invokeSelectEvent, (XtPointer) this);
    XtManageChild(_runNumberField);
    
    ac = 0;
    XtSetArg(args[ac], XmNarrowDirection, XmARROW_RIGHT); ac++;
    XtSetArg(args[ac], XmNx, 110); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 74); ac++;
    XtSetArg(args[ac], XmNheight, 40); ac++;
    _nextEventButton = XmCreateArrowButton(_filterForm,"nextEventButton",args, ac);
    XtAddCallback(_nextEventButton,XmNactivateCallback, (XtCallbackProc) SoXtHepEventViewer::invokeNextEvent, (XtPointer) this);

    ac = 0;
    XtSetArg(args[ac], XmNarrowDirection, XmARROW_LEFT); ac++;
    XtSetArg(args[ac], XmNx, 20); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth, 74); ac++;
    XtSetArg(args[ac], XmNheight, 40); ac++;
    _rewindButton = XmCreateArrowButton(_filterForm,"rewindButton",args, ac);
    XtAddCallback(_rewindButton,XmNactivateCallback, (XtCallbackProc) SoXtHepEventViewer::invokeRewind, (XtPointer) this);

    ac = 0;
    XtSetArg(args[ac], XmNx, 580); ac++;
    XtSetArg(args[ac], XmNy, 20); ac++;
    XtSetArg(args[ac], XmNwidth,  160); ac++;
    XtSetArg(args[ac], XmNheight, 160); ac++;
    XtSetArg(args[ac], XmNselectionPolicy, XmEXTENDED_SELECT); ac++;
    XtSetArg(args[ac], XmNtopAttachment,   XmATTACH_FORM);     ac++;
    XtSetArg(args[ac], XmNbottomAttachment,   XmATTACH_FORM);     ac++;
    XtSetArg(args[ac], XmNscrollBarDisplayPolicy, XmSTATIC);      ac++;
    _theIconBox = XmCreateScrolledList(_filterForm,"iconBox",args, ac);
    XtAddCallback(_theIconBox, XmNdefaultActionCallback,  (XtCallbackProc) SoXtHepEventViewer::deiconize, (XtPointer) this);
    XtManageChild(_theIconBox);
    


    // This was created in the parent class especially for this class.
    // Not just make it active!
    XtAddCallback(_openButton,XmNactivateCallback, (XtCallbackProc) SoXtHepEventViewer::openCallback, (XtPointer) this);

    ac=0;
    XtSetArg(args[ac], XmNpaneMinimum,  1); ac++;
    XtSetArg(args[ac], XmNpaneMaximum, 400); ac++;
    XtSetValues(_filterForm, args, ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    XtSetArg(args[ac], XmNtopOffset, 20); ac++;
    XtSetArg(args[ac], XmNleftOffset, 185); ac++;
    XtSetValues(_runLabel, args, ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    XtSetArg(args[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
    XtSetArg(args[ac], XmNtopOffset, 20); ac++;
    XtSetArg(args[ac], XmNleftOffset, 265); ac++;
    XtSetArg(args[ac], XmNrightOffset, 0); ac++;
    XtSetValues(_runNumberField, args, ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    XtSetArg(args[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
    XtSetArg(args[ac], XmNtopOffset, 20); ac++;
    XtSetArg(args[ac], XmNleftOffset, 110); ac++;
    XtSetValues(_nextEventButton, args, ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
    XtSetArg(args[ac], XmNrightAttachment, XmATTACH_NONE); ac++;
    XtSetArg(args[ac], XmNtopOffset, 20); ac++;
    XtSetArg(args[ac], XmNleftOffset, 20); ac++;
    XtSetValues(_rewindButton, args, ac);

    
}

void SoXtHepEventViewer::clear() {

  // Go through the list of visible items.  Weed out everybody
  // who is used by no one else but us..

  std::vector<std::list<Icon>::iterator>::iterator cI = _activeIcons.begin();
  while (cI!=_activeIcons.end()) {
    SoSwitch *theSwitch = (*(*cI)).sw;
    if (theSwitch->getRefCount()==1) {
      theSwitch->unref();
      XmListDeletePos(_theIconBox,cI-_activeIcons.begin()+1);
      _iconList.erase(*cI);
      cI=_activeIcons.erase(cI);
    }
    else {
      cI++;
    }
  }


  // Then go through the rest of the list..
  std::list<Icon>::iterator cL = _iconList.begin();
  while (cL!=_iconList.end()) {
    SoSwitch *theSwitch = (*cL).sw;
    if (theSwitch->getRefCount()==1) {
      theSwitch->unref();
      cL=_iconList.erase(cL);
    }
    else {
      cL++;
    }
  }
  


  
}
