/*-----------------------------HEPVis----------------------------------------*/
/*                                                                           */
/* Node:             SoXtHepViewer                                           */
/* Description:      Viewer Class                                            */
/* Author:           Jeff Kallenbach Apr 16 1997                             */
/*                                                                           */
/*---------------------------------------------------------------------------*/

// this :
#include <HEPVis/Xt/viewers/SoXtHepViewer.h>

#include <math.h>
//#include <iostream.h>
//#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/MainW.h>
#include <Xm/PanedW.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/TextF.h>
#include <Xm/ArrowB.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#if (SO_VERSION>=2) && (SO_VERSION_REVISION>=5)
#include <Inventor/manips/SoClipPlaneManip.h>
#endif

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/SoXtPrintDialog.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/Xt/viewers/SoXtFlyViewer.h>
#include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
#include <Inventor/Xt/viewers/SoXtWalkViewer.h>

#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/actions/SoLineHighlightRenderAction.h>
#include <Inventor/actions/SoBoxHighlightRenderAction.h>
#include <Inventor/nodes/SoLabel.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoCallback.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/engines/SoCompose.h>
#include <Inventor/engines/SoTransformVec3f.h>

#include <HEPVis/Xt/SoXtColorEditor.h>
#include <Inventor/actions/SoSearchAction.h>


static bool HEPVis_IsDigit(char aChar ){
  switch(aChar) {
  case '0':return true;
  case '1':return true;
  case '2':return true;
  case '3':return true;
  case '4':return true;
  case '5':return true;
  case '6':return true;
  case '7':return true;
  case '8':return true;
  case '9':return true;
  }
  return false;
}

String  
SoXtHepViewer::_defaultSoXtHepViewerResources[] = {
        NULL
};

int SoXtHepViewer::_numberOfInstances = 0;
//
// Return pointer to current viewer
//
SoXtFullViewer *
SoXtHepViewer::getCurrentViewer() const {
  return _theFullViewer;
}

//
// Return pointer to particular viewer
//
SoXtFullViewer *
SoXtHepViewer::getViewer(int i) const {
  if (i==EXAMINER) return _theExaminerViewer;
  if (i==PLANE)    return _thePlaneViewer;
  if (i==WALK)     return _theWalkViewer;
  if (i==FLY)      return _theFlyViewer;
  return NULL; 
}

//
// Construct menu bar
//
Widget 
SoXtHepViewer::getMenuBar() const {
  return _menuBar;
}

//
// Construct and display error message box
//  
void 
SoXtHepViewer::setErrorMessage(char *text) {
  int ac;
  Arg args[20];
  ac = 0;
  XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("HEPVis warning:")); 
 ac++;
  XtSetArg(args[ac], XmNmessageString, XmStringCreateSimple(text));  ac++;
  Widget messageBox = XmCreateWarningDialog(_MainWindow, "HEPVis warning", 
args, ac);
  XtUnmanageChild(XmMessageBoxGetChild(messageBox,XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(messageBox,XmDIALOG_CANCEL_BUTTON));
  XtManageChild(messageBox);
}

//
// Access to the text window
//
void 
SoXtHepViewer::addText(char *TextItem) {
  if  (XtIsManaged(_theTextArea))
    XmTextReplace(_theTextArea,XmTextGetLastPosition(_theTextArea),XmTextGetLastPosition(_theTextArea),TextItem);
}

void 
SoXtHepViewer::clearText(){
  if  (XtIsManaged(_theTextArea))
    XmTextReplace(_theTextArea,0,XmTextGetLastPosition(_theTextArea),"");
}

void 
SoXtHepViewer::showTextArea(){
  XtManageChild(_theTextArea);
}

void 
SoXtHepViewer::hideTextArea(){
  XtUnmanageChild(_theTextArea);
}

//
// Resize window
//
void 
SoXtHepViewer::resizeMain(short height, short width){

  Widget topShell = SoXt::getShellWidget(_MainWindow);
  if (!topShell) {
    setErrorMessage("No Shell in hierarchy - can't resize");
    return;
  }

  if ((height < 1)||(width < 1)) {
    setErrorMessage ("Illegal window dimensions - using defaults.\n");
    height = 812; width = 580;
  }

  Arg         args[1];
  char        s[32];
  sprintf     (s,"%dx%d",width,height);
  XtSetArg    (args[0],XtNgeometry,XtNewString(s));
  XtSetValues (topShell,args,1);

}

//
// Printing
//
void 
SoXtHepViewer::printCallback(Widget /*w*/, XtPointer clientData, 
XmPushButtonCallbackStruct */*callbackStruct*/) {
  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  print(This);
}

//
// Close
//
void 
SoXtHepViewer::close(Widget, XtPointer ClientData, XmPushButtonCallbackStruct*){
  SoXtHepViewer *This = (SoXtHepViewer *) ClientData;
  if (_numberOfInstances==1) {
    int status = (*(This->_quitCallback)) (This->_quitCallData);
    if (status) This->setErrorMessage("No Quit");
    return;
  }
  This->hide();
  delete This;
}

//
// Quit
//
void 
SoXtHepViewer::invokeQuit(Widget, XtPointer ClientData, 
XmPushButtonCallbackStruct *){
  int status=0;
  SoXtHepViewer *This = (SoXtHepViewer *) ClientData;
  if (This->_quitCallback) status = (*(This->_quitCallback)) (This->_quitCallData);
  if (status) This->setErrorMessage("No quit.");
}

void 
SoXtHepViewer::setQuitCallback(QuitCallbackProc procedure, void *call_data) {
  _quitCallback = procedure;
  _quitCallData=call_data;
}

//
// Check text input for digits
//
void 
SoXtHepViewer::verifyNumeric
  (Widget, XtPointer, XmTextVerifyCallbackStruct *call) {
  for (int i=0; i < call->text->length; i++) {	
    char c = call->text->ptr[i];
    if (!HEPVis_IsDigit(c)) {
      call->doit = False;
    }
  }
}

//
// Dragger callback
//
void 
SoXtHepViewer::toggleDraggerCB
  (Widget, XtPointer clientData, XmToggleButtonCallbackStruct *callData) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;

  SoGetBoundingBoxAction bboxAction(This->getCurrentViewer()->getViewportRegion());
  bboxAction.apply(This->getSceneGraph());

#if (SO_VERSION>=2) && (SO_VERSION_REVISION>=5)
  This->_theDragger->setValue
    (bboxAction.getBoundingBox(), SbVec3f(1.0, 0.0, 0.0), 0.2);
  This->_draggerSwitch->whichChild = callData->set ? SO_SWITCH_ALL: SO_SWITCH_NONE;
#else
  callData = 0;
#endif

}

//
// Anti-aliasing callback
//
void 
SoXtHepViewer::toggleAACB
  (Widget, XtPointer clientData, XmToggleButtonCallbackStruct *callData) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  This->_theExaminerViewer->setAntialiasing(callData->set,1);
  This->_theFlyViewer->setAntialiasing(callData->set,1);
  This->_theWalkViewer->setAntialiasing(callData->set,1);
  This->_thePlaneViewer->setAntialiasing(callData->set,1);
}

//
// Perf Meter callback
//
void 
SoXtHepViewer::togglePMCB
  (Widget , XtPointer clientData,XmToggleButtonCallbackStruct *callData) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  if (callData->set == TRUE) {
    This->Metered = 1;
    instrumentViewer (This);
    instrumentScene (This);
  }
  else {
    This->Metered = 0;
    unInstrumentViewer (This);
    delete This->perfSensor;
  }
}

//
// Remove performance meter from scene
//
void 
SoXtHepViewer::unInstrumentViewer(SoXtHepViewer *This)
{
  std::cout << "Instrumenting view" << std::endl;

  This->getCurrentViewer()->setOverlaySceneGraph((SoSeparator *) NULL);
}

//
// Add performance meter to scene in overlay
//
void 
SoXtHepViewer::instrumentViewer(SoXtHepViewer *This)
{
  //
  // Create the overlay scene    
  //
  SoSeparator *perfMeter = new SoSeparator;
  //
  // Add a callback node which will set the viewport 
  //
  SoCallback *overlayCB = new SoCallback;
  overlayCB->setCallback(overlayViewportCB);
  perfMeter->addChild(overlayCB);

  //
  // Add an orthographic camera
  //
  perfMeter->addChild(new SoOrthographicCamera());

  //
  // Add the text string to display the tris per second
  //
  char str[150];
  SoFont *perfMeterFont = new SoFont;
  perfMeterFont->size = 18;
  perfMeterFont->name = "Helvetica";
  perfMeter->addChild(perfMeterFont);
  This->perfMeterText = new SoText2;
  This->perfMeterText->justification = SoText2::LEFT;
  sprintf (str, "%d tris/frame",This->numTris);
  This->perfMeterText->string.setValue(str);
  perfMeter->addChild(This->perfMeterText);

  SbColor col(1, 1, 1);
  This->getCurrentViewer()->setOverlayColorMap(1, 1, &col);
  This->getCurrentViewer()->setOverlaySceneGraph(perfMeter);

  //
  // Update the performance meter every two seconds
  //
  This->perfSensor = 
	      new SoTimerSensor(perfSensorCB, This);
  This->perfSensor->setInterval(3.0);
  This->perfSensor->schedule();

  //
  // Reset the meter to only show the number of tris/frame -
  // otherwise the numbers might look misleading
  //
  This->getCurrentViewer()->addStartCallback(viewStartCB, This);
}

void
SoXtHepViewer::overlayViewportCB(void *, SoAction *action)
{
    if (action->isOfType(SoGLRenderAction::getClassTypeId())) 
        glViewport(0, 0, 40, 40);
}

//
// A sensor callback in the overlay scene updates the
// performance meter while the object is animating.
// 
void
SoXtHepViewer::perfSensorCB(void *data, SoSensor *)
{
  SoXtHepViewer *This= (SoXtHepViewer *) data;
  SoText2 *perfText = This->perfMeterText;
  char str[100];

  if (((SoXtExaminerViewer *) This->getCurrentViewer())->isAnimating()) {
    if (This->wasAnimating) {

      SbTime timeDiff = SbTime::getTimeOfDay() - This->startTime;
      double frames = 
	  This->numFramesRendered * 1000.0 / timeDiff.getMsecValue();
      frames = floor(frames * 100.0 + 0.5) / 100.0;

      // 
      // Update the perf meter display if the numbers have changed
      // 
      if (frames != This->lastFrames) {
          sprintf (str, 
		  "%d tris/frame,   %.01f frames/sec,   %.01f tris/sec", 
		  This->numTris, frames, frames * This->numTris);
    	  perfText->string.setValue(str);
	  This->lastFrames = frames;
	  This->addText(str);
	  This->addText("\n");
      }

      // 
      // Don't reset the timer here.  By using the total
      // time from when the object started animating
      // we will get more accurate numbers - especially 
      // for slow objects.
      // 
    }
    else {
      // 
      // Reset the timer when we detect the object has
      // started animating.
      // 
      This->numFramesRendered = 0;
      This->startTime = SbTime::getTimeOfDay();
      This->wasAnimating = TRUE;
    }
  }
  else
    This->wasAnimating = FALSE;
}

//
// Reset the meter to only show the tris/sec number.
// Otherwise the numbers might look misleading
//
void
SoXtHepViewer::viewStartCB(void *data, SoXtViewer *)
{
    SoXtHepViewer *This = (SoXtHepViewer *) data;
    if (This->Metered == 0) return;

    char str[150];
    SoText2 *perfText = This->perfMeterText;

    sprintf (str, "%d tris/frame", This->numTris);
    perfText->string.setValue(str);
}
	
void 
SoXtHepViewer::countUpdatesCB(void * data, SoAction *action) {
  if (action->isOfType(SoGLRenderAction::getClassTypeId())) {
    SoXtHepViewer *This = (SoXtHepViewer *) data;
    This->numFramesRendered++;
  }
}

void
SoXtHepViewer::countTriangleCB(void *userData, SoCallbackAction *,
     		const SoPrimitiveVertex *, 
		const SoPrimitiveVertex *,
                const SoPrimitiveVertex *)
{
#ifdef INVENTOR2_0
  long    *curCount = (long    *) userData;
#else
  int32_t *curCount = (int32_t *) userData;
#endif
  (*curCount)++;
}

int
SoXtHepViewer::countTriangles(SoNode *root)
{
    SoCallbackAction ca;
#ifdef INVENTOR2_0
    long    numTris = 0;
#else
    int32_t numTris = 0;
#endif

    root->ref();

    ca.addTriangleCallback(SoShape::getClassTypeId(), countTriangleCB,
                           (void *) &numTris);
    ca.apply(root);

    root->unrefNoDelete();

    return (numTris);
}

void 
SoXtHepViewer::instrumentScene(SoXtHepViewer *This) {
  
  std::cout << "Instrumenting scene" << std::endl;
  This->ancestor->renderCaching = SoSeparator::OFF;
  This->ancestor->boundingBoxCaching = SoSeparator::OFF;
  This->ancestor->pickCulling = SoSeparator::OFF;
  
  SoCallback *countUpdates = new SoCallback;
  countUpdates->setCallback(SoXtHepViewer::countUpdatesCB,This);
  
  This->ancestor->insertChild(countUpdates,0);

  This->numTris = countTriangles(This->ancestor);
  // cout << "triangles : " << This->numTris <<endl;
}

//
// Default Quit Callback
//
int SoXtHepViewer::quitCallback(void *) {
  exit (0);
  return 0;
}

//
// Spawn new SoXtHepViewer - create shell, create new viewer, set
// scene graph, show viewer
//
void 
SoXtHepViewer::spawnView
  (Widget , XtPointer clientData, XmPushButtonCallbackStruct *) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;

  Widget TheNewWindow = 
    XtAppCreateShell ("Inventor","Inventor",
                      topLevelShellWidgetClass,
                      XtDisplay (This->_MainWindow),
                      NULL,0);


  if (!TheNewWindow) {
    This->setErrorMessage("Cannot create Shell for new view.\n");
    return;
  }
  SoXtHepViewer *TheNewViewer = new SoXtHepViewer(TheNewWindow);
  
  //
  // Text Area, Icon box  in same state
  //
  if (!(XtIsManaged(This->_theTextArea))) TheNewViewer->hideTextArea();

  //
  // Copy Scene Graph
  //  
  TheNewViewer->setSceneGraph(This->getSceneGraph());

  //
  // Window same size
  //
  SbVec2s dimen = SoXt::getWidgetSize(This->_MainWindow);
  short h, w;
  dimen.getValue(h, w);
  TheNewViewer->resizeMain(h,w);

  //
  // Background Color
  //
  SbColor col = This->getCurrentViewer()->getBackgroundColor();
  TheNewViewer->_theExaminerViewer->setBackgroundColor(col);
  TheNewViewer->_thePlaneViewer->setBackgroundColor(col);
  TheNewViewer->_theWalkViewer->setBackgroundColor(col);
  TheNewViewer->_theFlyViewer->setBackgroundColor(col);

  //
  // Inherit Transparency and set correct button
  //
  XmToggleButtonSetState(TheNewViewer->_screenDoorButton,XmToggleButtonGetState(This->_screenDoorButton),0);
  XmToggleButtonSetState(TheNewViewer->_blendedButton,XmToggleButtonGetState(This->_blendedButton),0);
  XmToggleButtonSetState(TheNewViewer->_sortedButton,XmToggleButtonGetState(This->_sortedButton),0);
  XmToggleButtonSetState(TheNewViewer->_sortedBlendedButton,XmToggleButtonGetState(This->_sortedBlendedButton),0);

  int i;
  SoGLRenderAction::TransparencyType tt = (This->getCurrentViewer())->getTransparencyType();
  for (i=0;i<4;i++) {
    TheNewViewer->getViewer(i)->setTransparencyType(tt);
  }

  //
  // Inherit Highlight Style and set correct button
  //
  XmToggleButtonSetState(TheNewViewer->_boxHighlightButton,0,0);
  XmToggleButtonSetState(TheNewViewer->_linesHighlightButton,0,0);
  XmToggleButtonSetState(TheNewViewer->_noHighlightButton,0,0);
  for (i=0;i<4;i++) {
    
    if (XmToggleButtonGetState(This->_boxHighlightButton)){
      XmToggleButtonSetState(TheNewViewer->_boxHighlightButton,1,0);
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }  
    if (XmToggleButtonGetState(This->_linesHighlightButton)){
      XmToggleButtonSetState(TheNewViewer->_linesHighlightButton,1,0);
      SoLineHighlightRenderAction *rA = new SoLineHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }  
    if (XmToggleButtonGetState(This->_noHighlightButton)) {
      XmToggleButtonSetState(TheNewViewer->_noHighlightButton,1,0);
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(FALSE);
      rA->setTransparencyType(tt);
      TheNewViewer->getViewer(i)->setGLRenderAction(rA);
    }
  }
 
  //
  // Display new view
  //
  TheNewViewer->show();
  SoXt::show(TheNewWindow); 
  TheNewViewer->_theFullViewer->viewAll(); 

  TheNewViewer->setQuitCallback(This->_quitCallback,TheNewViewer);
}

//
// Change viewing style
//
void 
SoXtHepViewer::toggleViewCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {
  //
  // Turn this widget on, turn all the others off!
  //
  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  XmToggleButtonSetState(This->_examinerButton,0,0);
  XmToggleButtonSetState(This->_planeButton,0,0);
  XmToggleButtonSetState(This->_walkButton,0,0);
  XmToggleButtonSetState(This->_flyButton,0,0);
  XmToggleButtonSetState(w,1,0);

  //
  // Save the properties of the old viewer
  //
  SoCamera *theCamera = This->_theFullViewer->getCamera();
  SoXtViewer::DrawStyle theInterDrawStyle  = This->_theFullViewer->getDrawStyle(SoXtViewer::INTERACTIVE);
  SoXtViewer::DrawStyle theStillDrawStyle = This->_theFullViewer->getDrawStyle(SoXtViewer::STILL);

  This->_theFullViewer->hide();
  if (w==This->_examinerButton) This->_theFullViewer=This->_theExaminerViewer;
  if (w==This->_planeButton)    This->_theFullViewer=This->_thePlaneViewer;
  if (w==This->_walkButton)     This->_theFullViewer=This->_theWalkViewer;
  if (w==This->_flyButton)      This->_theFullViewer=This->_theFlyViewer;

  if (This->_theFullViewer == This->_theExaminerViewer)
    XtSetSensitive(This->_perfMeterButton,TRUE);
  else
    XtSetSensitive(This->_perfMeterButton,FALSE);

  if (w==This->_planeButton)    {
    //    This->_thePlaneViewer->getCamera()->orientation.setValue(SbVec3f(0,0,1),0);
    //This->_thePlaneViewer->viewAll();
  }


  SoSearchAction action;
  action.setType(SoSeparator::getClassTypeId(),TRUE);
  action.setInterest(SoSearchAction::ALL);
  action.apply(This->ancestor);
  
  SoPathList &result=action.getPaths();
  int L=result.getLength();
  for (int l=0;l<L;l++) {
    SoSeparator *sep = (SoSeparator * )result[l]->getTail();
    if (sep->getNumChildren()>0) sep->getChild(0)->touch();
  }
 


  This->_theFullViewer->setDrawStyle(SoXtViewer::INTERACTIVE,theInterDrawStyle);
  This->_theFullViewer->setDrawStyle(SoXtViewer::STILL,theStillDrawStyle);
  This->_theFullViewer->getCamera()->orientation.setValue(theCamera->orientation.getValue());
  This->_theFullViewer->show(); 
 
   
  
}

//
// Change Transparency
//
void 
SoXtHepViewer::toggleTransCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {
  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  XmToggleButtonSetState(This->_screenDoorButton,0,0);
  XmToggleButtonSetState(This->_blendedButton,0,0);
  XmToggleButtonSetState(This->_sortedButton,0,0);
  XmToggleButtonSetState(This->_sortedBlendedButton,0,0);
  XmToggleButtonSetState(w,1,0);

  for (int i=0;i<4;i++) {
    if (w==This->_screenDoorButton)         This->getViewer(i)->setTransparencyType(SoGLRenderAction::SCREEN_DOOR);
    if (w==This->_blendedButton)            This->getViewer(i)->setTransparencyType(SoGLRenderAction::BLEND);
    if (w==This->_sortedButton)             This->getViewer(i)->setTransparencyType(SoGLRenderAction::DELAYED_BLEND);
    if (w==This->_sortedBlendedButton)      This->getViewer(i)->setTransparencyType(SoGLRenderAction::SORTED_OBJECT_BLEND);
  }
 
}

//
// Change Highlighting
//
void 
SoXtHepViewer::toggleHLCB(Widget w, XtPointer clientData, 
XmToggleButtonCallbackStruct *) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;

  //
  // Store Transparency type for reset
  //  
  SoGLRenderAction::TransparencyType tt = (This->getCurrentViewer())->getTransparencyType();

  XmToggleButtonSetState(This->_boxHighlightButton,0,0);
  XmToggleButtonSetState(This->_linesHighlightButton,0,0);
  XmToggleButtonSetState(This->_noHighlightButton,0,0);
  XmToggleButtonSetState(w,1,0);

  for (int i=0;i<4;i++) {
    
    if (w==This->_boxHighlightButton){
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      (This->getViewer(i))->setGLRenderAction(rA);
    }  
    if (w==This->_linesHighlightButton){
      SoLineHighlightRenderAction *rA = new SoLineHighlightRenderAction();
      rA->setVisible(TRUE);
      rA->setTransparencyType(tt);
      This->getViewer(i)->setGLRenderAction(rA);
    }  
    if (w==This->_noHighlightButton) {
      SoBoxHighlightRenderAction *rA = new SoBoxHighlightRenderAction();
      rA->setVisible(FALSE);
      rA->setTransparencyType(tt);
      This->getViewer(i)->setGLRenderAction(rA);
    }
  }
 
}

//
// Spawn background color editor
//
void 
SoXtHepViewer::editBackgroundColor(Widget, XtPointer clientData, 
XmPushButtonCallbackStruct *) {
    SoXtHepViewer *This = (SoXtHepViewer *) clientData;
    if (This->_backgroundColorEditor == NULL) {
      This->_backgroundColorEditor = 
	new HEPVis_SoXtColorEditor(This->_MainWindow,"Background",FALSE);
      This->_backgroundColorEditor->setColor(SbColor(0, 0, 0));
      This->_backgroundColorEditor->setTitle( "Background Color" );
      This->_backgroundColorEditor->addColorChangedCallback
	(SoXtHepViewer::backgroundColorCB,This);
    }

    This->_backgroundColorEditor->show();
}

//
// Called after bg color editor used, to change bg color in all views
//
void 
SoXtHepViewer::backgroundColorCB(void *clientData, const SbColor *color) {
    SoXtHepViewer *This = (SoXtHepViewer *) clientData;
    This->_theExaminerViewer->setBackgroundColor(*color);
    This->_theFlyViewer->setBackgroundColor(*color);
    This->_theWalkViewer->setBackgroundColor(*color);
    This->_thePlaneViewer->setBackgroundColor(*color);
}

//
// Control method for printing
//
void 
SoXtHepViewer::print(SoXtHepViewer *This)
{

  //
  // Create print dialog if neccessary
  //
  if (This->printDialog == NULL) {
    This->printDialog = new SoXtPrintDialog;
    This->printDialog->setTitle("HEPVis print dialog");
    This->printDialog->setBeforePrintCallback(SoXtHepViewer::beforePrint, 
(void *) This);
    This->printDialog->setAfterPrintCallback(SoXtHepViewer::afterPrint, (void 
*) This);
  }
  
  //
  // Send the render area size and scene graph to the print dialog
  //
  Widget widget = This->_theFullViewer->getRenderAreaWidget();
  if (widget != NULL) {
    Arg args[2];
    int n = 0;
    SbVec2s sz;
    XtSetArg(args[n], XtNwidth, &sz[0]); n++;
    XtSetArg(args[n], XtNheight, &sz[1]); n++;
    XtGetValues(widget, args, n);
    This->printDialog->setPrintSize(sz);
  }
  This->printDialog->show();
}

//
// Callback routine inputs a filename, writes scenegraph to iv file (ascii)
//
void
SoXtHepViewer::saveIV (Widget, void * clientData, XmFileSelectionBoxCallbackStruct *data) {

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;

  char *filename;
  XmStringGetLtoR(data->value,
		  (XmStringCharSet) XmSTRING_DEFAULT_CHARSET, &filename);

  SoWriteAction myAction;
  myAction.getOutput()->openFile(filename);
  myAction.getOutput()->setBinary(FALSE);

  myAction.apply(This->sceneGraph);
  myAction.getOutput()->closeFile();
  XtFree(filename);
}

//
// Control routine for "Save File"
//
void 
SoXtHepViewer::saveIVCallback(Widget /*w*/, XtPointer clientData, 
XmPushButtonCallbackStruct */*callbackStruct*/) {
  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  Widget fileDialog = NULL;
  
  if (fileDialog == NULL) {
    Arg args[5];
    int n = 0;
    
    //
    // Unmanage when ok/cancel are pressed
    //
    
    XtSetArg(args[n], XmNautoUnmanage, TRUE); n++;
    fileDialog = XmCreateFileSelectionDialog(
					     This->_MainWindow, "SaveIV Dialog", args, n);
    
    XtAddCallback(fileDialog, XmNokCallback,
		  (XtCallbackProc) SoXtHepViewer::saveIV, This);
  }
  
  //
  // Manage the dialog
  //
  
  XtManageChild(fileDialog);
}

void 
SoXtHepViewer::beforePrint(void *tt, SoXtPrintDialog *) {
    SoXtHepViewer *This = (SoXtHepViewer *) tt;
    This->printDialog->setSceneGraph(This->ancestor);
}

void 
SoXtHepViewer::afterPrint(void *, SoXtPrintDialog *) {
}

//
// Constructor
//
SoXtHepViewer::SoXtHepViewer( Widget parent, const char* aName,  
	      SbBool buildInsideParent):SoXtComponent(parent,aName,buildInsideParent) {
  _numberOfInstances++;
  //
  // Viewer:
  //
  setTitle(aName);
  printDialog=NULL;
  create(getParentWidget());
  setQuitCallback(SoXtHepViewer::quitCallback,this);

  //
  // Scene:
  //
  ancestor = new SoSeparator();
  ancestor->ref();

  lightsCamera              = new SoGroup;
  lightGroup 	    	    = new SoGroup;

  ancestor->addChild(lightsCamera);

  //
  // Lights;
  //
  _topFlood = new SoDirectionalLight();
  lightGroup->addChild(_topFlood);

  _bottomFlood = new SoDirectionalLight();
  lightGroup->addChild(_bottomFlood);

  _leftFlood = new SoDirectionalLight();
  lightGroup->addChild(_leftFlood);

  _rightFlood = new SoDirectionalLight();
  lightGroup->addChild(_rightFlood);

  _topFlood->on=False;
  _bottomFlood->on=False;
  _leftFlood->on=False;
  _rightFlood->on=False;

  lightsCamera->addChild(lightGroup);

  sceneGraph = new SoSeparator();
  ancestor->addChild(sceneGraph);

  _theExaminerViewer->setSceneGraph(ancestor);
  _theWalkViewer->setSceneGraph(ancestor);
  _theFlyViewer->setSceneGraph(ancestor);
  _thePlaneViewer->setSceneGraph(ancestor);


  //COMPOSE A MATRIX, CONNECT IT TO THE PERSPECTIVE CAMERA
  SoComposeMatrix *myMatrix=new SoComposeMatrix;
  myMatrix->rotation.connectFrom(&_theExaminerViewer->getCamera()->orientation);

  //TAKE THE OUTPUT OF THE MATRIX COMPOSITION, USE IT TO
  //COMPUTE A DIRECTION FOR THE LEFT LIGHT:
  SoTransformVec3f *leftTransform=new SoTransformVec3f;
  leftTransform->vector.setValue(1/sqrt(2.0),0,-1/sqrt(2.0));
  leftTransform->matrix.connectFrom(&myMatrix->matrix);

  SoTransformVec3f *rightTransform=new SoTransformVec3f;
  rightTransform->vector.setValue(-1/sqrt(2.0),0,-1/sqrt(2.0));
  rightTransform->matrix.connectFrom(&myMatrix->matrix);

  SoTransformVec3f *topTransform=new SoTransformVec3f;
  topTransform->vector.setValue(0,-1/sqrt(2.0),-1/sqrt(2.0));
  topTransform->matrix.connectFrom(&myMatrix->matrix);

  SoTransformVec3f *bottomTransform=new SoTransformVec3f;
  bottomTransform->vector.setValue(0,1/sqrt(2.0),-1/sqrt(2.0));
  bottomTransform->matrix.connectFrom(&myMatrix->matrix);


  
  // CONNECT TH CAMERA DIRECTION TO THE NEW MATRIX
  _leftFlood->direction.connectFrom(&leftTransform->direction);
  _rightFlood->direction.connectFrom(&rightTransform->direction);
  _topFlood->direction.connectFrom(&topTransform->direction);
  _bottomFlood->direction.connectFrom(&bottomTransform->direction);


  _theFullViewer->viewAll();
  _theFullViewer->saveHomePosition();
  _theFullViewer->show();

  _backgroundColorEditor=NULL;


  //
  // Add the dragger node
  //
#if (SO_VERSION>=2) && (SO_VERSION_REVISION>=5)
  _draggerSwitch = new SoSwitch();
  ancestor->addChild(_draggerSwitch);
  _theDragger = new SoClipPlaneManip;
  _draggerSwitch->addChild(_theDragger);
#endif

  Metered = 0;
  numTris = 0;
  numFramesRendered = 0;
  wasAnimating = FALSE;
  startTime=0.0; 
  lastFrames = 0;
  perfSensor=NULL;
}

//
// Sets scene graph
//
void 
SoXtHepViewer::setSceneGraph(SoSeparator *node) {
  ancestor->removeChild(sceneGraph);
  ancestor->addChild(node);
  sceneGraph=node;
}

//
// Returns scene graph
//
SoSeparator *
SoXtHepViewer::getSceneGraph() const {
  return sceneGraph;
}

//
// destructor
//
SoXtHepViewer::~SoXtHepViewer() 
{

  delete printDialog;
  delete _backgroundColorEditor;
  ancestor->removeAllChildren();
  ancestor->unref();
  delete _theExaminerViewer;
  delete _thePlaneViewer;
  delete _theWalkViewer;
  delete _theFlyViewer;
  delete perfSensor;
  unregisterWidget (_MainWindow);
  _numberOfInstances--;
}

//
// Called to apply light request
//
void 
SoXtHepViewer::toggleLights
  (Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *callData){

  SoXtHepViewer *This = (SoXtHepViewer *) clientData;
  if (w == This->_headlightButton) {
    for (int i=0;i<4;i++) This->getViewer(i)->setHeadlight(callData->set);
  }
  else if (w == This->_leftFloodButton) {
    This->_leftFlood->on=callData->set;
  }
  else if (w == This->_rightFloodButton) {
    This->_rightFlood->on=callData->set;
  }
  else if (w == This->_topFloodButton) {
    This->_topFlood->on=callData->set;
  }
  else if (w == This->_bottomFloodButton) {
    This->_bottomFlood->on=callData->set;
  }
}

//
// Create viewer - set up widgets, etc.
//
void 
SoXtHepViewer::create(Widget parent) 
{
    Arg      args[256];
    Cardinal ac=0;
    
    XtInitializeWidgetClass(xmMainWindowWidgetClass);
    XtInitializeWidgetClass(xmPanedWindowWidgetClass);
    XtInitializeWidgetClass(xmTextWidgetClass);
    XtInitializeWidgetClass(xmFormWidgetClass);
    XtInitializeWidgetClass(xmLabelWidgetClass);
    XtInitializeWidgetClass(xmTextFieldWidgetClass);
    XtInitializeWidgetClass(xmArrowButtonWidgetClass);
    XtInitializeWidgetClass(xmCascadeButtonWidgetClass);
    XtInitializeWidgetClass(xmRowColumnWidgetClass);
    XtInitializeWidgetClass(xmPushButtonWidgetClass);
    XtInitializeWidgetClass(xmToggleButtonWidgetClass);
    
    //
    // Create an unmanaged widget as the top of the hierarchy.
    //
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 69); ac++;
    XtSetArg(args[ac], XmNy, 454); ac++;
    XtSetArg(args[ac], XmNwidth, 812); ac++;
    XtSetArg(args[ac], XmNheight, 580); ac++;
    char *_name = (char *) getTitle();
    _MainWindow = XmCreateMainWindow(parent,_name,args, ac);
    XtManageChild(_MainWindow);
    registerWidget (_MainWindow);
    setBaseWidget  (_MainWindow);
    
    ac = 0;
    XtSetArg(args[ac], XmNwidth, 812); ac++;
    XtSetArg(args[ac], XmNheight, 560); ac++;
    _panedWindow = XmCreatePanedWindow(_MainWindow,"panedWindow",args, ac);
    XtManageChild(_panedWindow);

    _theExaminerViewer = new SoXtExaminerViewer(_panedWindow,NULL,TRUE, SoXtFullViewer::BUILD_ALL, SoXtViewer::BROWSER);
    _thePlaneViewer    = new SoXtPlaneViewer(_panedWindow, NULL, TRUE, SoXtFullViewer::BUILD_ALL, SoXtViewer::BROWSER);
    _theWalkViewer     = new SoXtWalkViewer(_panedWindow,NULL,TRUE,SoXtFullViewer::BUILD_ALL, SoXtViewer::BROWSER);
    _theFlyViewer      = new SoXtFlyViewer(_panedWindow,NULL, TRUE,SoXtFullViewer::BUILD_ALL, SoXtViewer::BROWSER);
    _theFullViewer     = _theExaminerViewer; //by default;
  
    ac = 0;
    XtSetArg(args[ac], XmNeditMode, XmMULTI_LINE_EDIT);ac++;
    XtSetArg(args[ac], XmNscrollingPolicy, XmAUTOMATIC); ac++;
    _theTextArea = XmCreateScrolledText(_panedWindow, "theTextWindow", args,ac);
    XtManageChild(_theTextArea);


    ac = 0;
    //    XtSetArg(args[ac], XmNwidth, 812); ac++;
    XtSetArg(args[ac], XmNheight, 35); ac++;
    XtSetArg(args[ac], XmNresizeHeight, False); ac++;
    _menuBar = XmCreateMenuBar(_MainWindow,
        "menuBar",
        args, 
        ac);
    XtManageChild(_menuBar);
    
    ac = 0;
    XtSetArg(args[ac],XmNlabelString,XmStringCreateSimple("File")); ac++;
    XtSetArg(args[ac], XmNx, 5); ac++;
    XtSetArg(args[ac], XmNy, 5); ac++;
    XtSetArg(args[ac], XmNwidth, 45); ac++;
    XtSetArg(args[ac], XmNheight, 25); ac++;
    _fileMenuButton = XmCreateCascadeButton(_menuBar,
        "fileMenuButton",
        args, 
        ac);
    XtManageChild(_fileMenuButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 0); ac++;
    XtSetArg(args[ac], XmNy, 0); ac++;
    XtSetArg(args[ac], XmNwidth, 54); ac++;
    XtSetArg(args[ac], XmNheight, 112); ac++;
    _fileMenu = XmCreatePulldownMenu(XtParent(_fileMenuButton),
        "fileMenu",
        args, 
        ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("New HepViewer"));  ac++;
    _newExaminerButton = XmCreatePushButton(_fileMenu,
        "newExaminerButton",args,ac);
    XtAddCallback(_newExaminerButton,XmNactivateCallback,(XtCallbackProc) SoXtHepViewer::spawnView, this);
    XtManageChild(_newExaminerButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Open"));  ac++;
    _openButton = XmCreatePushButton(_fileMenu,"openButton", args, ac);
    // This is not managed! Not by this class!  At least not yet!
    // The unmanaged button is placed here so that it isn't at the bottom
    // of the menu; the subclasses can manage it and bind it as they please.

    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Save"));  ac++;
    _saveButton = XmCreatePushButton(_fileMenu,"saveButton",args,ac);
    XtAddCallback(_saveButton,XmNactivateCallback, (XtCallbackProc) 
SoXtHepViewer::saveIVCallback, (XtPointer) this);
    XtManageChild(_saveButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Print"));  ac++;
    _printButton = XmCreatePushButton(_fileMenu,"printButton",args, ac);
    XtAddCallback(_printButton,XmNactivateCallback, (XtCallbackProc) 
SoXtHepViewer::printCallback, (XtPointer) this);
    XtManageChild(_printButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Close"));  ac++;
    _closeButton = XmCreatePushButton(_fileMenu,"closeButton",args, ac);
    XtAddCallback(_closeButton,XmNactivateCallback, (XtCallbackProc) 
SoXtHepViewer::close, (XtPointer) this);
    XtManageChild(_closeButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Quit"));  ac++;
    _quitButton = XmCreatePushButton(_fileMenu,"quitButton",args, ac);
    XtAddCallback(_quitButton,XmNactivateCallback, (XtCallbackProc) 
SoXtHepViewer::invokeQuit, (XtPointer) this);
    XtManageChild(_quitButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _fileMenu); ac++;
    XtSetValues(_fileMenuButton, args, ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("View"));  ac++;
    XtSetArg(args[ac], XmNx, 50); ac++;
    XtSetArg(args[ac], XmNy, 5); ac++;
    XtSetArg(args[ac], XmNwidth, 54); ac++;
    XtSetArg(args[ac], XmNheight, 25); ac++;       
       
    _viewMenuButton = XmCreateCascadeButton(_menuBar,
        "viewMenuButton",
        args, 
        ac);
    XtManageChild(_viewMenuButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 119); ac++;
    XtSetArg(args[ac], XmNy, 484); ac++;
    XtSetArg(args[ac], XmNwidth, 222); ac++;
    XtSetArg(args[ac], XmNheight, 156); ac++;
    _viewMenu = XmCreatePulldownMenu(XtParent(_viewMenuButton),
        "viewMenu",
        args, 
        ac);
    
 //
 // Lights
 //
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Lights"));  ac++;
       
    _lightsButton = XmCreateCascadeButton(_viewMenu,
        "lightsButton",
        args, 
        ac);
    XtManageChild(_lightsButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 491); ac++;
    XtSetArg(args[ac], XmNwidth, 176); ac++;
    XtSetArg(args[ac], XmNheight, 129); ac++;
    _lightsMenu = XmCreatePulldownMenu(XtParent(_lightsButton),
        "lightsMenu",
        args, 
        ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Head Light"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;         
       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _headlightButton = XmCreateToggleButton(_lightsMenu,
        "headlightButton",
        args, 
        ac);
    XtManageChild(_headlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Left Flood Light"));  ac++;
       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _leftFloodButton = XmCreateToggleButton(_lightsMenu,
        "leftFloodButton",
        args, 
        ac);
    XtManageChild(_leftFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Right Flood Light"));  ac++;
       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _rightFloodButton = XmCreateToggleButton(_lightsMenu,
        "rightFloodButton",
        args, 
        ac);
    XtManageChild(_rightFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Top Flood Light"));  ac++;
       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _topFloodButton = XmCreateToggleButton(_lightsMenu,
        "topFloodButton",
        args, 
        ac);
    XtManageChild(_topFloodButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Bottom Flood Light"));  ac++;
       
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _bottomFloodButton = XmCreateToggleButton(_lightsMenu,
        "bottomFloodButton",
        args, 
        ac);
    XtManageChild(_bottomFloodButton);
    
    XtAddCallback(_headlightButton,XmNvalueChangedCallback,(XtCallbackProc) 
SoXtHepViewer::toggleLights,this);
    XtAddCallback(_topFloodButton,XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleLights,this);
    XtAddCallback(_bottomFloodButton,XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleLights,this);
    XtAddCallback(_leftFloodButton,XmNvalueChangedCallback,(XtCallbackProc) 
SoXtHepViewer::toggleLights,this);
    XtAddCallback(_rightFloodButton,XmNvalueChangedCallback,(XtCallbackProc) 
SoXtHepViewer::toggleLights,this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _lightsMenu); ac++;
    XtSetValues(_lightsButton, args, ac);
    
 //
 // Transparency style
 //
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Transparency"));  ac++;
    _transparencyButton = XmCreateCascadeButton(_viewMenu,
        "transparencyButton",
        args, 
        ac);
    XtManageChild(_transparencyButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 516); ac++;
    XtSetArg(args[ac], XmNwidth, 144); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _transparencyMenu = XmCreatePulldownMenu(XtParent(_transparencyButton),
        "transparencyMenu",
        args, 
        ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Screen Door"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _screenDoorButton = XmCreateToggleButton(_transparencyMenu,
        "screenDoorButton",
        args, 
        ac);
    XtManageChild(_screenDoorButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Blended"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _blendedButton = XmCreateToggleButton(_transparencyMenu,
        "blendedButton",
        args, 
        ac);
    XtManageChild(_blendedButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Sorted"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _sortedButton = XmCreateToggleButton(_transparencyMenu,
        "sortedButton",
        args, 
        ac);
    XtManageChild(_sortedButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Sorted Blended"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _sortedBlendedButton = XmCreateToggleButton(_transparencyMenu,
        "sortedBlendedButton",
        args, 
        ac);
    XtManageChild(_sortedBlendedButton);
    
    XtAddCallback(_screenDoorButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleTransCB, this);
    XtAddCallback(_blendedButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleTransCB, this);
    XtAddCallback(_sortedButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleTransCB, this);
    XtAddCallback(_sortedBlendedButton, XmNvalueChangedCallback, 
(XtCallbackProc) SoXtHepViewer::toggleTransCB, this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _transparencyMenu); ac++;
    XtSetValues(_transparencyButton, args, ac);
 
 //
 // Highlight style
 //
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Highlight Style"));  ac++;
    _highlightButton = XmCreateCascadeButton(_viewMenu,
        "highlightButton",
        args, 
        ac);
    XtManageChild(_highlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 516); ac++;
    XtSetArg(args[ac], XmNwidth, 144); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _highlightMenu = XmCreatePulldownMenu(XtParent(_highlightButton),
        "highlightMenu",
        args, 
        ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Box"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _boxHighlightButton = XmCreateToggleButton(_highlightMenu,
        "boxHighlightButton",
        args, 
        ac);
    XtManageChild(_boxHighlightButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Lines"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _linesHighlightButton = XmCreateToggleButton(_highlightMenu,
        "linesHighlightButton",
        args, 
        ac);
    XtManageChild(_linesHighlightButton);
        
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("None"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _noHighlightButton = XmCreateToggleButton(_highlightMenu,
        "noHighlightButton",
        args, 
        ac);
    XtManageChild(_noHighlightButton);
    
    XtAddCallback(_boxHighlightButton, XmNvalueChangedCallback, 
(XtCallbackProc) SoXtHepViewer::toggleHLCB, this);
    XtAddCallback(_linesHighlightButton, XmNvalueChangedCallback, 
(XtCallbackProc) SoXtHepViewer::toggleHLCB, this);
    XtAddCallback(_noHighlightButton, XmNvalueChangedCallback, 
(XtCallbackProc) SoXtHepViewer::toggleHLCB, this);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _highlightMenu); ac++;
    XtSetValues(_highlightButton, args, ac);

    //
    // Background Color
    // 
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Edit Background Color"));  ac++;
    _editBackgroundButton = XmCreatePushButton(_viewMenu,
        "editBackgroundButton",
        args, 
        ac);
    XtAddCallback(_editBackgroundButton,XmNactivateCallback, (XtCallbackProc) 
SoXtHepViewer::editBackgroundColor,this);
    XtManageChild(_editBackgroundButton);
    
    //
    // Change Style of Main Viewer
    // 
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("View Style"));  ac++;
    _viewStyle = XmCreateCascadeButton(_viewMenu,
        "viewStyle",
        args, 
        ac);
    XtManageChild(_viewStyle);
    
    ac = 0;
    XtSetArg(args[ac], XmNx, 335); ac++;
    XtSetArg(args[ac], XmNy, 593); ac++;
    XtSetArg(args[ac], XmNwidth, 102); ac++;
    XtSetArg(args[ac], XmNheight, 104); ac++;
    _pulldownMenu = XmCreatePulldownMenu(XtParent(_viewStyle),
        "pulldownMenu",
        args, 
        ac);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Examiner"));  ac++;
    XtSetArg(args[ac], XmNset, True); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _examinerButton = XmCreateToggleButton(_pulldownMenu, "examinerButton", 
args, ac);
    XtAddCallback(_examinerButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleViewCB, this);
    XtManageChild(_examinerButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Plane"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _planeButton = XmCreateToggleButton(_pulldownMenu,"planeButton",args, ac);
    XtAddCallback(_planeButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleViewCB, this);
    XtManageChild(_planeButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, 
        XmStringCreateSimple("Walk"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _walkButton = XmCreateToggleButton(_pulldownMenu,"walkButton",args, ac);
    XtAddCallback(_walkButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleViewCB, this);
    XtManageChild(_walkButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Fly"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _flyButton = XmCreateToggleButton(_pulldownMenu,"flyButton",args,ac);
    XtAddCallback(_flyButton, XmNvalueChangedCallback, (XtCallbackProc) 
SoXtHepViewer::toggleViewCB, this);
    XtManageChild(_flyButton);
    
    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _pulldownMenu); ac++;
    XtSetValues(_viewStyle, args, ac);

    ac = 0;
    XtSetArg(args[ac], XmNsubMenuId, _viewMenu); ac++;
    XtSetValues(_viewMenuButton, args, ac);
    
#if (SO_VERSION>=2) && (SO_VERSION_REVISION>=5)
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Clip Plane Dragger"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _draggerButton = XmCreateToggleButton(_viewMenu,"draggerButton",args, ac);
    XtAddCallback(_draggerButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtHepViewer::toggleDraggerCB, this);
    XtManageChild(_draggerButton);
#endif
    ac = 0;
    XtSetArg(args[ac], XmNlabelString, XmStringCreateSimple("Antialiasing"));  ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _antialiasingButton = XmCreateToggleButton(_viewMenu,"antialiasingButton",args, ac);
    XtAddCallback(_antialiasingButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtHepViewer::toggleAACB, this);
    XtManageChild(_antialiasingButton);

    ac = 0;
    XtSetArg
      (args[ac], XmNlabelString, XmStringCreateSimple("Perf Meter")); ac++;
    XtSetArg(args[ac], XmNvisibleWhenOff, True); ac++;
    _perfMeterButton = XmCreateToggleButton
      (_viewMenu,"perfMeterButton",args, ac);
    XtAddCallback(_perfMeterButton, XmNvalueChangedCallback, (XtCallbackProc) SoXtHepViewer::togglePMCB, this);
    XtManageChild(_perfMeterButton);

}

