
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/dirmanip.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>

#include <stdlib.h>

class Finder : public virtual Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& /*aDirectory*/,bool& aProcess) {
    aProcess = true;
    return true;
  }
  virtual bool file(const std::string& aFile) {
    Lib::Out out(fPrinter);
    if(fPrint) out << aFile  << Lib::endl;
    if(fSearch!="") {
      std::vector<std::string> text;
      if(!Lib::fmanip::read(aFile,text)) {
        Lib::Out out(fPrinter);
        out << "Can't read \"" << aFile << "\"" << Lib::endl;
        return false;
      }
      for(unsigned int index=0;index<text.size();index++) {
        std::string s = text[index];
        if(s.find(fSearch)!=std::string::npos) {
          out << "Found at line " << (index+1) << " in \"" << aFile << "\" :" << Lib::endl;
          out << s << Lib::endl;
        }
      }
    }
    return true;
  }
public:
  Finder(Slash::Core::IWriter& aPrinter,const std::string& aSearch,bool aPrint)
    :fPrinter(aPrinter),fSearch(aSearch),fPrint(aPrint){}
  virtual ~Finder() {}
private:
  Slash::Core::IWriter& fPrinter;  
  std::string fSearch;
  bool fPrint;
};

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::Arguments args(aArgc,aArgv);

  std::string cmd = 
    "  OS> Lib_find -path=<starting path> -file=<file pattern> -search=<string to search in files> -print";

  std::string path;
  if(!args.find("-path",path)) path = ".";

  std::string filter;
  if(!args.find("-file",filter)) filter = "*";

  bool print = args.isAnArgument("-print");

  std::string what;
  if(!args.find("-search",what) && !print) {
    out << cmd<< Lib::endl;
    return EXIT_FAILURE;
  }

  Lib::Directory tree;
  tree.setPath(path);
  if(!Lib::dirmanip::content(tree,filter)) {
    out << "Can't get files of \"" << path << "\"" << Lib::endl;
    return EXIT_FAILURE;
  }

  Finder finder(printer,what,print);
  tree.visit(finder);

  return EXIT_SUCCESS;
}

// embed code :
#include <Printer.cxx>
#include <Out.cxx>
#include <Arguments.cxx>
#include <smanip.cxx>
#include <sout.cxx>
#include <cmanip.cxx>
#include <fmanip.cxx>
#include <dirmanip.cxx>
#include <System.cxx>
