%{

// this :
#include <Lib/FortranFormat.h>

#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/FortranFormatLexer.h>
#include <Lib/Debug.h>
#include <Lib/smanip.h>
#include <Lib/Cast.h>

static Lib::FortranFormat* sParser = 0;

extern int yylex();
int yyparse();
static void yyerror(char*); 

//#define LIB_DEBUG

%}

%union {
  unsigned long fInteger;
  Lib::DataDescriptor* fDesc;
  unsigned int fType;
}

%token <fInteger> NUMBER_INTEGER NUMBER_EXPONENT

%token T_X T_I T_F T_E T_G T_A T_L T_SLASH

%type <fDesc> data_desc_tree
%type <fDesc> data_desc_list
%type <fDesc> data_desc_leaf
%type <fDesc> data_desc_one
%type <fType> data_type

%left '-' '+'
%left '/' '*'
%nonassoc UMINUS

%%

result 
: error {
  // To reset lex and yacc in case of error.
} | data_desc_tree {
  sParser->fTree.fChildren.push_back($1);
  $1->fParent = &(sParser->fTree);
  sParser->fTmpDescs.pop_back();
};

data_desc_tree 
: '(' data_desc_list ')' {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_tree : (data_desc_list)" 
        << Lib::endl;
  }
  $2->fMultiplicity = 1;
  $$ = $2;
} | NUMBER_INTEGER '(' data_desc_list ')' {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_tree : integer(data_desc_list)" 
        << Lib::endl;
  }
  $3->fMultiplicity = $1;
  $$ = $3;
};

data_desc_list 
: data_desc_leaf {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_list : data_desc_leaf" 
        << Lib::endl;
  }
  Lib::DataDescriptor* list = new Lib::DataDescriptor();
  list->fChildren.push_back($1);
  $1->fParent = list;
  sParser->fTmpDescs.pop_back();
  sParser->fTmpDescs.push_back(list);//To cleanup in case of error.
  $$ = list;
} | data_desc_tree {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_list : data_desc_tree" 
        << Lib::endl;
  }
  Lib::DataDescriptor* list = new Lib::DataDescriptor();
  list->fChildren.push_back($1);
  $1->fParent = list;
  sParser->fTmpDescs.pop_back();
  sParser->fTmpDescs.push_back(list);//To cleanup in case of error.
  $$ = list;
} | data_desc_list ',' data_desc_list {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_list : data_desc_tree,data_desc_tree" 
        << Lib::endl;
  }
  Lib::DataDescriptor* list = new Lib::DataDescriptor();
  list->fChildren.push_back($1);
  $1->fParent = list;
  list->fChildren.push_back($3);
  $3->fParent = list;
  sParser->fTmpDescs.pop_back();
  sParser->fTmpDescs.pop_back();
  sParser->fTmpDescs.push_back(list);//To cleanup in case of error.
  $$ = list;
};

data_desc_leaf 
: data_desc_one {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_leaf : data_desc_one" 
        << Lib::endl;
  }
  $1->fMultiplicity = 1;
  $$ = $1;
} | NUMBER_INTEGER data_desc_one {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_leaf : integer data_desc_one" 
        << Lib::endl;
  }
  $2->fMultiplicity = $1;
  $$ = $2;
};

data_desc_one
: data_type {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_one : data_type" 
        << Lib::endl;
  }
  Lib::DataDescriptor* leaf = new Lib::DataDescriptor();
  sParser->fTmpDescs.push_back(leaf);//To cleanup in case of error.
  leaf->fType = $1;
  $$ = leaf;
} | data_type NUMBER_INTEGER {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_one : data_type integer" 
        << Lib::endl;
  }
  Lib::DataDescriptor* leaf = new Lib::DataDescriptor();
  sParser->fTmpDescs.push_back(leaf);//To cleanup in case of error.
  leaf->fType = $1;
  leaf->fWidth = $2;
  $$ = leaf;
} | data_type NUMBER_INTEGER '.' NUMBER_INTEGER {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_one : data_type integer.integer" 
        << Lib::endl;
  }
  Lib::DataDescriptor* leaf = new Lib::DataDescriptor();
  sParser->fTmpDescs.push_back(leaf);//To cleanup in case of error.
  leaf->fType = $1;
  leaf->fWidth = $2;
  leaf->fPrecision = $4;
  $$ = leaf;
} | data_type NUMBER_INTEGER '.' NUMBER_INTEGER NUMBER_EXPONENT {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    out << "Lib::FortranFormat::parse :"
        << " data_desc_one : data_type integer.integerEinteger" 
        << Lib::endl;
  }
  Lib::DataDescriptor* leaf = new Lib::DataDescriptor();
  sParser->fTmpDescs.push_back(leaf);//To cleanup in case of error.
  leaf->fType = $1;
  leaf->fWidth = $2;
  leaf->fPrecision = $4;
  leaf->fExponent = $5;
  $$ = leaf;
};

data_type  
: T_X { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      Lib::Out out(sParser->printer());
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_X" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_X;
} | T_I { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      Lib::Out out(sParser->printer());
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_I" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_I;
} | T_F { 
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_F" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_F;
} | T_E {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_E" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_E;
} | T_G {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_G" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_G;
} | T_A {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_A" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_A;
} | T_L {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_L" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_L;
} | T_SLASH {
  if(sParser->fVerboseLevel) {
    Lib::Out out(sParser->printer());
    if(sParser->fVerboseLevel) {
      out << "Lib::FortranFormat::parse :"
          << " data_type : T_SLASH" 
          << Lib::endl;
    }
  }
  $$ = DataDescriptor_SLASH;
}
;

%%

//////////////////////////////////////////////////////////////////////////////
Lib::FortranFormat::FortranFormat(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
) 
:fVerboseLevel(aVerboseLevel)
,fString(aString)
,fPrinter(aPrinter)
,fLexer(aPrinter,aVerboseLevel)
,fSuccess(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranFormat::~FortranFormat(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranFormat::FortranFormat(
 const FortranFormat& aFrom
) 
:fVerboseLevel(aFrom.fVerboseLevel)
,fString(aFrom.fString)
,fPrinter(aFrom.fPrinter)
,fLexer(fPrinter,fVerboseLevel)
,fSuccess(false)
//////////////////////////////////////////////////////////////////////////////
// private.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::FortranFormat& Lib::FortranFormat::operator=(
 const FortranFormat&
) 
//////////////////////////////////////////////////////////////////////////////
// private.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::FortranFormat::setScript(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fString = aString;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::FortranFormat::script(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fString;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::FortranFormat::parse(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  if(fString=="") return false;
  fLexer.set(fString);
  sParser = this;
  fSuccess = true;
  yyparse();
  sParser = 0;
  if(!fSuccess) clear();
  return fSuccess;
}
//////////////////////////////////////////////////////////////////////////////
const Lib::DataDescriptor& Lib::FortranFormat::result(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTree;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::FortranFormat::clear(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTree.clear();

  // In case of error :
  for(unsigned int index=0;index<fTmpDescs.size();index++) 
    delete fTmpDescs[index];
  fTmpDescs.clear();
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::FortranFormat::isSuccess(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSuccess;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::FortranFormat::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
const Lib::FortranFormatLexer& Lib::FortranFormat::lexer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fLexer;
}
//////////////////////////////////////////////////////////////////////////////
static void yyerror(
 char* aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(sParser) {
    Lib::Out out(sParser->printer());
    out << "FortranFormat : " << aString << Lib::endl;
    sParser->fSuccess = false;
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor::DataDescriptor(
)
:fParent(0)
,fIndex(0)
,fMultiplicity(1)
,fType(0) //Unknown by default
,fWidth(0)
,fPrecision(0)
,fExponent(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::DataDescriptor");
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor::~DataDescriptor(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
  Lib::Debug::decrement("Lib::DataDescriptor");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::DataDescriptor::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<fChildren.size();index++) 
    delete fChildren[index];
  fChildren.clear();
}
//////////////////////////////////////////////////////////////////////////////
void Lib::DataDescriptor::reset(
 bool aDoThis
)
//////////////////////////////////////////////////////////////////////////////
// Descent the tree to reset all the fIndex to 0.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aDoThis) fIndex = 0;
  for(unsigned int index=0;index<fChildren.size();index++)
    fChildren[index]->reset();
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptorIterator* Lib::DataDescriptor::createIterator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(DataDescriptor);
  return new Lib::DataDescriptorIterator(self);
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor* Lib::DataDescriptor::firstLeaf(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<fChildren.size();index++) {
    Lib::DataDescriptor* leaf = fChildren[index]->firstLeaf();
    if(leaf) return leaf;
  }
  if(!fChildren.size()) return 0;
  return fChildren[0];
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor* Lib::DataDescriptor::nextChild(
 Lib::DataDescriptor* aChild
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fChildren.size(); 
  if(!aChild && number) return fChildren[0];
  for(unsigned int index=0;index<number;index++) {
    if(aChild==fChildren[index]) {
      if((index+1)<number) {
        return fChildren[index+1];
      } else { 
        return 0;
      }
    }
  }
  return 0; //Should never pass here.
}
//////////////////////////////////////////////////////////////////////////////
void Lib::DataDescriptor::dump(
 Slash::Core::IWriter& aPrinter
,int aIndent
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);
  std::string sindent(aIndent,' ');

  if(!fChildren.size()) { // Leaf :
    out << sindent;
    out << fMultiplicity;
    if(fType==DataDescriptor_X) out << "X";
    else if(fType==DataDescriptor_I) out << "I";
    else if(fType==DataDescriptor_F) out << "F";
    else if(fType==DataDescriptor_E) out << "E";
    else if(fType==DataDescriptor_G) out << "G";
    else if(fType==DataDescriptor_A) out << "A";
    else if(fType==DataDescriptor_SLASH) out << "/";
    else out << "?";
    out << fWidth;
    out << ".";
    out << fPrecision;
    out << "E";
    out << fExponent;
    out << Lib::endl;    
  } else {

    out << sindent;
    out << fMultiplicity;
    out << Lib::endl;    

    unsigned int index;
    for(index=0;index<fChildren.size();index++) 
      fChildren[index]->dump(aPrinter,aIndent+2);
  }

}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::DataDescriptor::cformat(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(DataDescriptor);

       if(fType==DataDescriptor_X) {
    return "";
  } else if(fType==DataDescriptor_I) {
    std::string cformat;
    smanip::printf(cformat,64,"%%%d.%dd",fWidth,fPrecision);
    return cformat;
  } else if(fType==DataDescriptor_F) {
    // # : to force having a decimal point, even if precision is 0.
    std::string cformat;
    smanip::printf(cformat,64,"%%#%d.%df",fWidth,fPrecision);
    return cformat;
  } else if(fType==DataDescriptor_E) {
    std::string cformat;
    smanip::printf(cformat,64,"%%%d.%dE",fWidth,fPrecision);
    return cformat;
  } else if(fType==DataDescriptor_G) {
    std::string cformat;
    smanip::printf(cformat,64,"%%%d.%dG",fWidth,fPrecision);
    return cformat;
  } else if(fType==DataDescriptor_A) {
    return "%s";
  } else if(fType==DataDescriptor_SLASH) {
    return "";
  } else {
    return "";  
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptorIterator::DataDescriptorIterator(
 Lib::DataDescriptor& aTree
)
:fTree(aTree)
,fDesc(0)
,fFinished(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::DataDescriptorIterator");
  fTree.reset();
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptorIterator::~DataDescriptorIterator(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("Lib::DataDescriptorIterator");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::DataDescriptorIterator::reset(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTree.reset();
  fDesc = 0;
  fFinished = false;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::DataDescriptorIterator::finished(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFinished;
}
//////////////////////////////////////////////////////////////////////////////
Lib::DataDescriptor* Lib::DataDescriptorIterator::next(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fFinished) return 0;
  if(!fDesc) { //Begin of iteration.
    fDesc = &fTree;
    fDesc->fIndex = 0;
  } else { // Repeat again ?
    if((fDesc->fIndex+1)<(int)fDesc->fMultiplicity) {
      fDesc->fIndex++;
      fDesc->reset(false);
      return fDesc; // The same.
    }
  }
  // New desc :
  while(true) {
    Lib::DataDescriptor* leaf = fDesc->firstLeaf();
    if(leaf) {
      fDesc = leaf;
      fDesc->fIndex = 0;
   /*{Lib::DataDescriptor* parent = fDesc->fParent;
      while(parent) {
        parent->fIndex = 0;
        parent = parent->fParent;          
      }}*/
      return leaf;
    }
    // Back to parent and after current fDesc :
    Lib::DataDescriptor* parent = fDesc->fParent;
    while(1) {
      if(!parent) { //Tree finished.      
        fDesc = 0;
        fFinished = true;
        return 0;
      }
      fDesc = parent->nextChild(fDesc);
      if(fDesc) break;
      // parent children exhausted.
      // Repeat again ?
      if((parent->fIndex+1)<(int)parent->fMultiplicity) {
        parent->fIndex++;
        parent->reset(false);
        fDesc = parent;
        break;
      } else {
        fDesc = parent;
        parent = parent->fParent;          
      }
    }
  }
}
