// this :
#include <Lib/Functions.h>


// Slash :
#include <Slash/Core/IValue.h>

// Lib :
#include <Lib/SpaceAlgebra.h>
#include <Lib/Processor.h>
#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/nmanip.h>
#include <Lib/mmanip.h>
#include <Lib/Debug.h>
#include <Lib/Value.h>
#include <Lib/Cast.h>

// Else :
#include <math.h>

#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
Lib::Function::Gaussian::Gaussian(
 double aScale
,double aMean
,double aSigma
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aScale,aMean,aSigma);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Gaussian::Gaussian(
 const Gaussian& aFrom
)
:fScale(aFrom.fScale)
,fMean(aFrom.fMean)
,fSigma(aFrom.fSigma)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Gaussian& Lib::Function::Gaussian::operator=(
 const Gaussian& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScale = aFrom.fScale;
  fMean = aFrom.fMean;
  fSigma = aFrom.fSigma;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Gaussian::set(
 double aScale
,double aMean
,double aSigma
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScale = aScale;
  fMean = aMean;
  fSigma = aSigma;
  if(fSigma<=0) fSigma = 1;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Gaussian::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) return false;
  fScale = aParams[0];
  fMean = aParams[1];
  fSigma = aParams[2];
  if(fSigma<=0) fSigma = 1;
  return true;
}
/////////////////////////////////////////////////////////////////////////
double Lib::Function::Gaussian::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value = (aX - fMean)/fSigma;
  return fScale * ::exp(-0.5 * value * value);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Gaussian::setScale(
 double aScale
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScale = aScale;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Gaussian::setMean(
 double aMean
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fMean = aMean;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Gaussian::setSigma(
 double aSigma
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSigma = aSigma;
  if(fSigma<=0) fSigma = 1;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::BreitWigner::BreitWigner(
 double aHeight
,double aCenter
,double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aHeight,aCenter,aWidth);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::BreitWigner::BreitWigner(
 const BreitWigner& aFrom
)
:fHeight(aFrom.fHeight)
,fCenter(aFrom.fCenter)
,fWidth(aFrom.fWidth)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::BreitWigner& Lib::Function::BreitWigner::operator=(
 const BreitWigner& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aFrom.fHeight;
  fCenter = aFrom.fCenter;
  fWidth = aFrom.fWidth;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::BreitWigner::set(
 double aHeight
,double aCenter
,double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aHeight;
  fCenter = aCenter;
  fWidth = aWidth;
  if(fWidth<=0) fWidth = 1;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::BreitWigner::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) return false;
  fHeight = aParams[0];
  fCenter = aParams[1];
  fWidth = aParams[2];
  if(fWidth<=0) fWidth = 1;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::BreitWigner::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value = 2.*(aX - fCenter)/fWidth;
  return fHeight/(1. + value * value);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::BreitWigner::setHeight(
 double aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aHeight;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::BreitWigner::setCenter(
 double aCenter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCenter = aCenter;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::BreitWigner::setWidth(
 double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fWidth = aWidth;
  if(fWidth<=0) fWidth = 1;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Exponential::Exponential(
 double aA
,double aB
)
//////////////////////////////////////////////////////////////////////////////
// exp(aA + aB * aX)  //Same as in PAW fitting definition.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aA,aB);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Exponential::Exponential(
 const Exponential& aFrom
)
:fA(aFrom.fA)
,fB(aFrom.fB)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Exponential& Lib::Function::Exponential::operator=(
 const Exponential& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fA = aFrom.fA;
  fB = aFrom.fB;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Exponential::set(
 double aA
,double aB
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fA = aA;
  fB = aB;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Exponential::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=2) return false;
  fA = aParams[0];
  fB = aParams[1];
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Exponential::setA(
 double aA
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fA = aA;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Exponential::setB(
 double aB
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fB = aB;
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Exponential::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return ::exp(fA + fB * aX);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Polynomial::Polynomial(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
// P[0] + P[1] x + P[2] x**2 + ...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  setParameters(aParams);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Polynomial::Polynomial(
 const Polynomial& aFrom
)
:fPs(aFrom.fPs)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Polynomial& Lib::Function::Polynomial::operator=(
 const Polynomial& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPs = aFrom.fPs;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Polynomial::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPs = aParams;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Polynomial::setParameter(
 unsigned int aIndex
,double aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aIndex>=fPs.size()) return;
  fPs[aIndex] = aValue;
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Polynomial::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value = 0;
  double xxx = 1;
  unsigned int pn = fPs.size();
  for(unsigned int i=0;i<pn;i++) {
    value += fPs[i] * xxx;
    xxx = xxx * aX;
  }
  return value;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Cauchy::Cauchy(
 double aHeight
,double aCenter
,double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aHeight,aCenter,aWidth);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Cauchy::Cauchy(
 const Cauchy& aFrom
)
:fHeight(aFrom.fHeight)
,fCenter(aFrom.fCenter)
,fWidth(aFrom.fWidth)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Cauchy& Lib::Function::Cauchy::operator=(
 const Cauchy& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aFrom.fHeight;
  fCenter = aFrom.fCenter;
  fWidth = aFrom.fWidth;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Cauchy::set(
 double aHeight
,double aCenter
,double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aHeight;
  fCenter = aCenter;
  fWidth = aWidth;
  if(fWidth<=0) fWidth = 1;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Cauchy::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=3) return false;
  fHeight = aParams[0];
  fCenter = aParams[1];
  fWidth = aParams[2];
  if(fWidth<=0) fWidth = 1;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Cauchy::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
// From Wikipedia.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double b = fWidth/2.;
  double value = (aX - fCenter)/b;
  return fHeight/((value * value + 1)*b*Lib::mmanip::pi());
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Cauchy::setHeight(
 double aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fHeight = aHeight;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Cauchy::setCenter(
 double aCenter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCenter = aCenter;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Cauchy::setWidth(
 double aWidth
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fWidth = aWidth;
  if(fWidth<=0) fWidth = 1;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array::Array(
 const std::vector<double>& aXs
,const std::vector<double>& aVs
)
:fIndex(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aXs,aVs);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array::Array(
 const Array& aFrom
)
:fXs(aFrom.fXs)
,fVs(aFrom.fVs)
,fIndex(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array& Lib::Function::Array::operator=(
 const Array& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fXs = aFrom.fXs;
  fVs = aFrom.fVs;
  fIndex = 0;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array::Array(
)
:fIndex(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::Array::xs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::Array::vs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVs;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Array::set(
 const std::vector<double>& aXs
,const std::vector<double>& aVs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset :
  fXs.clear();
  fVs.clear();
  fIndex = 0;
  if(aXs.size()!=aVs.size()) return false;
  fXs = aXs;
  fVs = aVs;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Array::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = aParams.size();
  if(number<1) return false;
  int parami = 0;
  int dim = (int)aParams[parami];parami++;
  if(dim!=1) return false;
  if(number<1+dim) return false;
  std::vector<int> ns(dim);
  int index;
  int xn = 0;
  int vn = 1;
  for(index=0;index<dim;index++) {
    ns[index] = (int)aParams[parami];
    parami++;
    xn = xn + ns[index];
    vn = vn * ns[index];
  }
  if(1+dim+xn+vn!=number) return false; // Bad data number.
  std::vector< std::vector<double> > xs(dim);
  for(index=0;index<dim;index++) {
    int nx = ns[index];
    xs[index].resize(nx);
    for(int j=0;j<nx;j++) {
      xs[index][j] = aParams[parami];
      parami++;
    }
  }
  std::vector<double> vs(vn);
  for(index=0;index<vn;index++) {
    vs[index] = aParams[parami];
    parami++;
  }
  return set(xs[0],vs);
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Array::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
// In general, for 1D function, value() is called over successive, closed 
// value of the abscisse. We try to optimize the search of abscisse 
// by handling 'fIndex'.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = fXs.size();
  if(number<=0) return 0;
  int end = number-1;
  if(aX<fXs[0])   return 0; // Not in range.
  if(fXs[end]<aX) return 0; // Not in range.

  Lib_self(Array);

  // In range :
  if( (fIndex<0) || (end<=fIndex)) self.fIndex = 0;
  for(int count=fIndex;count<end;count++){
    if( (fXs[count]<=aX) && (aX<=fXs[count+1]) ) {
      self.fIndex = count;
      return interpolate(count,aX);
    }
  }
  //  Not found, could be in under part of fIndex ; 
  // reset index and search again :
  self.fIndex = 0;

  return value(aX);
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Array::interpolate(
 int aIndex
,double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double a = (fVs[aIndex+1] - fVs[aIndex])/(fXs[aIndex+1] - fXs[aIndex]);
  double b = fVs[aIndex] - a * fXs[aIndex];
  return a * aX + b; 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array2D::Array2D(
 const std::vector<double>& aXs
,const std::vector<double>& aYs
,const std::vector<double>& aVs
)
:fI(0)
,fJ(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs, aYs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aXs,aYs,aVs);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array2D::Array2D(
 const Array2D& aFrom
)
:fXs(aFrom.fXs)
,fYs(aFrom.fYs)
,fVs(aFrom.fVs)
,fI(0)
,fJ(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array2D& Lib::Function::Array2D::operator=(
 const Array2D& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fXs = aFrom.fXs;
  fYs = aFrom.fYs;
  fVs = aFrom.fVs;
  fI = 0;
  fJ = 0;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Array2D::Array2D(
)
:fI(0)
,fJ(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs, aYs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::Array2D::xs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::Array2D::ys(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fYs;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::Array2D::vs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVs;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Array2D::set(
 const std::vector<double>& aXs
,const std::vector<double>& aYs
,const std::vector<double>& aVs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset :
  fXs.clear();
  fYs.clear();
  fVs.clear();
  fI = 0;
  fJ = 0;
  if( (aXs.size()*aYs.size()) != aVs.size()) return false;
  fXs = aXs;
  fYs = aYs;
  fVs = aVs;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Array2D::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = aParams.size();
  if(number<1) return false;
  int parami = 0;
  int dim = (int)aParams[parami];parami++;
  if(dim!=2) return false;
  if(number<1+dim) return false;
  std::vector<int> ns(dim);
  int index;
  int xn = 0;
  int vn = 1;
  for(index=0;index<dim;index++) {
    ns[index] = (int)aParams[parami];
    parami++;
    xn = xn + ns[index];
    vn = vn * ns[index];
  }
  if(1+dim+xn+vn!=number) return false; // Bad data number.
  std::vector< std::vector<double> > xs(dim);
  for(index=0;index<dim;index++) {
    int nx = ns[index];
    xs[index].resize(nx);
    for(int j=0;j<nx;j++) {
      xs[index][j] = aParams[parami];
      parami++;
    }
  }
  std::vector<double> vs(vn);
  for(index=0;index<vn;index++) {
    vs[index] = aParams[parami];
    parami++;
  }
  return set(xs[0],xs[1],vs);
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Array2D::value(
 double aX
,double aY
) const
//////////////////////////////////////////////////////////////////////////////
// Oftenly, value is done by looping on successive rows.
// Optimize the search by handling fI, fJ indices.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int xn = fXs.size();
  int yn = fYs.size();
  if( (xn<=0) || (yn<=0) ) return 0;

  int xend = xn-1;
  int yend = yn-1;

  // In range ?
  if(aX<fXs[0]) return 0;
  if(fXs[xend]<aX) return 0;
  if(aY<fYs[0]) return 0;
  if(fYs[yend]<aY) return 0;

  Lib_self(Array2D);

  // In range :
  if( (fI<0) || (xend<=fI)) self.fI = 0;
  if( (fJ<0) || (yend<=fJ)) self.fJ = 0;

  for(int cy=fJ;cy<yend;cy++){
    if( (fYs[cy]<=aY) && (aY<=fYs[cy+1]) ) {

      for(int cx=fI;cx<xend;cx++){
        if( (fXs[cx]<=aX) && (aX<=fXs[cx+1]) ) {
          self.fI = cx;
          self.fJ = cy;
          return interpolate(cx,cy,aX,aY);
        }
      }

    }
    //  Not found in upper part of current row ;
    // start at beginning of next row.
    self.fI = 0;
  }

  //  Not found, could be in under part ; 
  // reset indices and search again :
  self.fI = 0;
  self.fJ = 0;

  return value(aX,aY);
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::Array2D::interpolate(
 int aI
,int aJ
,double aX
,double aY
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  A3d P0(fXs[aI]  ,fYs[aJ]  ,0);
  A3d P1(fXs[aI+1],fYs[aJ]  ,0);
  A3d P2(fXs[aI]  ,fYs[aJ+1],0);
  A3d P (aX,aY,0);
  A3d V10 = P1;V10.subtract(P0);
  A3d V21 = P2;V21.subtract(P1);
  A3d V02 = P0;V02.subtract(P2);
  A3d VP0 = P; VP0.subtract(P0);
  A3d VP1 = P; VP1.subtract(P1);
  A3d VP2 = P; VP2.subtract(P2);
  if(   
     (V10.cross(VP0).value2()>=0)  &&
     (V21.cross(VP1).value2()>=0)  &&
     (V02.cross(VP2).value2()>=0) ) { 
    // Inside lower triangle :
    A3d Q0(fXs[aI]  ,fYs[aJ]  ,fVs[offset(aI,aJ)]);
    A3d Q1(fXs[aI+1],fYs[aJ]  ,fVs[offset(aI+1,aJ)]);
    A3d Q2(fXs[aI]  ,fYs[aJ+1],fVs[offset(aI,aJ+1)]);
    SpacePlane p(Q0,Q1,Q2);
    A3d i;
    if(p.intersect(SpaceLine(P,A3d(aX,aY,1)),i)) {
      return i.value2();
    }
  } else {
    // Inside upper triangle :
    A3d Q0(fXs[aI+1],fYs[aJ]  ,fVs[offset(aI+1,aJ)]);
    A3d Q1(fXs[aI+1],fYs[aJ+1],fVs[offset(aI+1,aJ+1)]);
    A3d Q2(fXs[aI]  ,fYs[aJ+1],fVs[offset(aI,aJ+1)]);
    SpacePlane p(Q0,Q1,Q2);
    A3d i;
    if(p.intersect(SpaceLine(P,A3d(aX,aY,1)),i)) {
      return i.value2();
    }
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
int Lib::Function::Array2D::offset(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (aI + aJ * fXs.size());
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::ArrayComplex::ArrayComplex(
)
:fIndex(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::ArrayComplex::ArrayComplex(
 const ArrayComplex& aFrom
)
:fXs(aFrom.fXs)
,fVs(aFrom.fVs)
,fIndex(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::ArrayComplex& Lib::Function::ArrayComplex::operator=(
 const ArrayComplex& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fXs = aFrom.fXs;
  fVs = aFrom.fVs;
  fIndex = 0;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::ArrayComplex::ArrayComplex(
 const std::vector<double>& aXs
,const std::vector< std::complex<double> >& aVs
)
:fIndex(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aXs,aVs);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::ArrayComplex::ArrayComplex(
 const std::vector<double>& aXs
,const std::vector<double>& aRs
,const std::vector<double>& aIs
)
:fIndex(0)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aXs,aRs,aIs);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::ArrayComplex::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = aParams.size();
  if(number<1) return false;
  int parami = 0;
  int dim = (int)aParams[parami];parami++;
  if(dim!=1) return false;
  if(number<1+dim) return false;
  std::vector<int> ns(dim);
  int index;
  int xn = 0;
  int vn = 1;
  for(index=0;index<dim;index++) {
    ns[index] = (int)aParams[parami];
    parami++;
    xn = xn + ns[index];
    vn = vn * ns[index];
  }
  if(1+dim+xn+2*vn!=number) return false; // Bad data number.
  std::vector< std::vector<double> > xs(dim);
  for(index=0;index<dim;index++) {
    int nx = ns[index];
    xs[index].resize(nx);
    for(int j=0;j<nx;j++) {
      xs[index][j] = aParams[parami];
      parami++;
    }
  }
  std::vector<double> vRs(vn);
  std::vector<double> vIs(vn);
  for(index=0;index<vn;index++) {
    vRs[index] = aParams[parami];
    parami++;
    vIs[index] = aParams[parami];
    parami++;
  }
  return set(xs[0],vRs,vIs);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::ArrayComplex::set(
 const std::vector<double>& aXs
,const std::vector< std::complex<double> >& aVs
)
//////////////////////////////////////////////////////////////////////////////
// Assume that aXs are given in increasing order.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset :
  fXs.clear();
  fVs.clear();
  fIndex = 0;
  if(aXs.size()!=aVs.size()) return false;
  fXs = aXs;
  fVs = aVs;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::ArrayComplex::set(
 const std::vector<double>& aXs
,const std::vector<double>& aRs
,const std::vector<double>& aIs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset :
  fXs.clear();
  fVs.clear();
  fIndex = 0;
  if(aXs.size()!=aRs.size()) return false;
  if(aXs.size()!=aIs.size()) return false;
  fXs = aXs;
  unsigned int number = aXs.size();
  fVs.resize(number);
  for(unsigned int count=0;count<number;count++){
    fVs[count] = std::complex<double>(aRs[count],aIs[count]);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<double>& Lib::Function::ArrayComplex::xs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fXs;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector< std::complex<double> >& Lib::Function::ArrayComplex::vs(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVs;
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::ArrayComplex::value(
 double aX
) const
//////////////////////////////////////////////////////////////////////////////
// In general, for 1D function, value() is called over successive, closed 
// value of the abscisse. We try to optimize the search of abscisse 
// by handling 'fIndex'.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number = fXs.size();
  if(number<=0) return 0;
  int end = number-1;
  if(aX<fXs[0])   return ::sqrt(std::norm(fVs[0]));   // Not in range.
  if(fXs[end]<aX) return ::sqrt(std::norm(fVs[end])); // Not in range.

  Lib_self(ArrayComplex);

  // In range :
  if( (fIndex<0) || (end<=fIndex)) self.fIndex = 0;
  for(int count=fIndex;count<end;count++){
    if( (fXs[count]<=aX) && (aX<=fXs[count+1]) ) {
      self.fIndex = count;
      return interpolate(count,aX);
    }
  }
  //  Not found, could be in under part of fIndex ; 
  // reset index and search again :
  self.fIndex = 0;

  return value(aX);
}
//////////////////////////////////////////////////////////////////////////////
double Lib::Function::ArrayComplex::interpolate(
 int aIndex
,double aX
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double a = (::sqrt(std::norm(fVs[aIndex+1])) - 
              ::sqrt(std::norm(fVs[aIndex])))/(fXs[aIndex+1] - fXs[aIndex]);
  double b = ::sqrt(std::norm(fVs[aIndex])) - a * fXs[aIndex];
  return a * aX + b; 
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Scripted::Scripted(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
)
:fScript(aString)
,fIsValid(false)
,fDimension(0)
,fPrinter(aPrinter)
,fProcessor("",aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aString);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Scripted::Scripted(
 const Scripted& aFrom
)
:fScript(aFrom.fScript)
,fIsValid(false)
,fDimension(aFrom.fDimension)
,fParameterNames(aFrom.fParameterNames)
,fParameterValues(aFrom.fParameterValues)
,fVariableNames(aFrom.fVariableNames)
,fPrinter(aFrom.fPrinter)
,fProcessor("",aFrom.fPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aFrom.fScript);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Scripted::~Scripted(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Scripted& Lib::Function::Scripted::operator=(
 const Scripted& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScript = aFrom.fScript;
  fDimension = aFrom.fDimension;
  fParameterNames = aFrom.fParameterNames;
  fParameterValues = aFrom.fParameterValues;
  fVariableNames = aFrom.fVariableNames;

  //fPrinter = aFrom.fPrinter;
  fIsValid = false;

  set(aFrom.fScript);
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Scripted::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fIsValid; 
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::Function::Scripted::dimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDimension; 
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Function::Scripted::script(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScript;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Function::Scripted::parameterNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParameterNames;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double> Lib::Function::Scripted::parameterValues(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParameterValues;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Function::Scripted::variableNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVariableNames;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Scripted::set(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  reset();
  //Check for AIDA/SLAC variable syntax convention x[<int>] :
  std::string ss = aString;
  std::string ns;
  bool slac_syntax = false;
  while(true) {
    std::string::size_type lb = ss.find("x[");
    if(lb==std::string::npos){
      ns += ss;
      break;
    }
    std::string::size_type rb = ss.find(']',lb+2);
    if(rb==std::string::npos) {
      Lib::Out out(fPrinter);
      out << "Lib::Function::Scripted::set :"
          << " bad [] balance in " << Lib::sout(aString)
          << Lib::endl;
      reset();
      return false;
    }
    //     abx[012]ef
    //     0 2    7 9   size=10
    std::string snum = ss.substr(lb+2,rb-lb-2);
    int coord;
    if(!Lib::smanip::toint(snum,coord)) {
      Lib::Out out(fPrinter);
      out << "Lib::Function::Scripted::set :"
          << " integer expected within [] in " << Lib::sout(aString)
          << ". Found " << Lib::sout(snum) << "."
          << Lib::endl;
      reset();
      return false;
    }
    slac_syntax = true;
    ns += ss.substr(0,lb)+ "x"+ snum;
    ss = ss.substr(rb+1,ss.size()-rb);
  }
  //printf("debug : ns \"%s\"\n",ns.c_str());

  // Find variables in the script :
  fProcessor.setScript(ns);
  std::vector<std::string> vars;
  if(!fProcessor.findVariables(vars)) {
    Lib::Out out(fPrinter);
    out << "Lib::Function::Scripted::set :"
        << " Lib::Processor can't find variables in " << Lib::sout(aString)
        << Lib::endl;
    reset();
    return false;
  }

  //vars had been passed to Lib::smanip::unique.

  //  In vars, some are expected to be of the form x<int>.
  // The x<int> are the variables of the scripted function. 
  // The rest are the parameters of the scripted function. 
  // The variables are used to find the dimension of the function.
  //  If x0 only is found, then it is a one dim function.
  // If x0,x1 only are found, then it is a two dim function, etc...

  std::vector<int> coords;
  bool found_x = false;
  bool found_y = false;
  bool found_z = false;
 {for(unsigned int index=0;index<vars.size();index++) {
    int coord;
    if(vars[index]=="x") {
      found_x = true;
    } else if(vars[index]=="y") {
      found_y = true;
    } else if(vars[index]=="z") {
      found_z = true;
    } else if(isVariable(vars[index],coord)) {
      coords.push_back(coord);
    } else {
      fParameterNames.push_back(vars[index]);
    }
  }}


  vars.clear(); //Reorder with params first and variables last.
 {for(unsigned int index=0;index<fParameterNames.size();index++) {
    vars.push_back(fParameterNames[index]);
  }}

  // Sort coords :
  if(coords.size()) {
    if(found_x||found_y||found_z) {      
      Lib::Out out(fPrinter);
      out << "Lib::Function::Scripted::set :"
          << " inconsistent variable specification in " << Lib::sout(aString)
          << Lib::endl;
      reset();
      return false;
    }    
    Lib::nmanip::sort(coords);
    for(unsigned int index=0;index<coords.size();index++) {
      std::string s;
      Lib::smanip::printf(s,32,"x%d",coords[index]);
      vars.push_back(s);
      if(slac_syntax) {
        Lib::smanip::printf(s,32,"x[%d]",coords[index]);
        fVariableNames.push_back(s);
      } else {
        fVariableNames.push_back(s);
      }
    }
  } else {
    if(found_x) {
      vars.push_back("x");
      fVariableNames.push_back("x");
    }
    if(found_y) {
      vars.push_back("y");
      fVariableNames.push_back("y");
    }
    if(found_z) {
      vars.push_back("z");
      fVariableNames.push_back("z");
    }
  }

  if(!fProcessor.compile(vars)) {
    Lib::Out out(fPrinter);
    out << "Lib::Function::Scripted::set :"
        << " Lib::Processor can't compile " << Lib::sout(aString)
        << Lib::endl;
    reset();
    return false;
  }

  fParameterValues.resize(fParameterNames.size(),0);

  fInput.resize
    (fParameterNames.size()+fVariableNames.size(),Lib::Value((double)0));

  fScript = aString;
  fDimension = fVariableNames.size();
  fIsValid = true;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Scripted::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aParams.size()!=fParameterValues.size()) return false;
  fParameterValues = aParams;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Scripted::value(
 const std::vector<double>& aXs
,double& aValue
,std::string& aError
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(Scripted);

  if(!isValid()) {
    aValue = 0;
    aError = "Lib::Function::Scripted::value : is not valid.";
    return false;
  }
  if(aXs.size()!=fDimension) {
    aValue = 0;
    aError = "Lib::Function::Scripted::value : xs.size()/dimension mismatch.";
    return false;
  }

  unsigned int ii = 0;
 {unsigned int number = fParameterValues.size(); 
  for(unsigned int index=0;index<number;index++) {
    self.fInput[ii].set(fParameterValues[index]);
    ii++;
  }}
 {unsigned int number = aXs.size(); 
  for(unsigned int index=0;index<number;index++) {
    self.fInput[ii].set(aXs[index]);
    ii++;
  }}

  Lib::Value var;
  if(!self.fProcessor.execute(fInput,var,aError)) {
    aValue = 0;
    return false;
  }

  if(var.type()!=Slash::Core::IValue::DOUBLE) {
    aError = std::string("Lib::Function::Scripted::value :")
           + " result is not a double Variable.";
    aValue = 0;
    return false;
  }

  //aValue = var.toDouble(); //FIXME
  aValue = var.get_double();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Scripted::isVariable(
 const std::string& aVar
,int& aIndex
)
//////////////////////////////////////////////////////////////////////////////
// If aVar is not of the form x<int> return false.
// If yes, return true with aIndex=<int>.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aVar.size()<=1) {
    aIndex = 0;
    return false;
  }
  if(aVar[0]!='x') {
    aIndex = 0;
    return false;
  }
  if(!Lib::smanip::toint(aVar.substr(1,aVar.size()-1),aIndex)) {
    aIndex = 0;
    return false;;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Function::Scripted::reset(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset :
  fIsValid = false;
  fScript = "";
  fDimension = 0;
  fParameterNames.clear();
  fParameterValues.clear();
  fVariableNames.clear();
  fProcessor.clear();
  fInput.clear();
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Composite::Composite(
 const std::string& aString
,Slash::Core::IWriter& aPrinter
)
:fScript(aString)
,fScripted("0",aPrinter)
,fPrinter(aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aString);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Composite::Composite(
 const Composite& aFrom
)
:fScript(aFrom.fScript)
,fScripted(aFrom.fScripted)
,fPrinter(aFrom.fPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  set(aFrom.fScript);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Composite::~Composite(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Composite& Lib::Function::Composite::operator=(
 const Composite& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScript = aFrom.fScript;
  fScripted = aFrom.fScripted;
  //fPrinter = aFrom.fPrinter;
  set(aFrom.fScript);
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Composite::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.isValid(); 
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::Function::Composite::dimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.dimension(); 
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Function::Composite::script(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScript;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Function::Composite::parameterNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.parameterNames();
}
//////////////////////////////////////////////////////////////////////////////
std::vector<double> Lib::Function::Composite::parameterValues(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.parameterValues();
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Function::Composite::variableNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.variableNames();
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Composite::setParameters(
 const std::vector<double>& aParams
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.setParameters(aParams);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Composite::value(
 const std::vector<double>& aXs
,double& aValue
,std::string& aError
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScripted.value(aXs,aValue,aError);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Composite::set(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// exa : G+E
// FIXME : have more operations.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  std::string script; 
  std::vector<std::string> words;
  Lib::smanip::words(aString,"+",words);
  unsigned int wordn = words.size();
  for(unsigned int index=0;index<wordn;index++) {
    if(index) script += "+";
    std::string& word = words[index];
    Lib::smanip::strip(word);
    Lib::smanip::touppercase(word);
    std::string p; //variable prefix.
#define pc p.c_str()
    if(word=="E") {
      if(wordn>=2) Lib::smanip::printf(p,32,"E_%d_",index);
      Lib::smanip::printf(s,132,"%samplitude*exp(x*%sexponent)",pc,pc);
    } else if(word=="EHBOOK") {
      if(wordn>=2) Lib::smanip::printf(p,32,"EHBOOK_%d_",index);
      Lib::smanip::printf(s,132,"expo(x,%sA,%sB)",pc,pc);
    } else if(word=="G") {
      if(wordn>=2) Lib::smanip::printf(p,32,"G_%d_",index);
      Lib::smanip::printf
        (s,132,"gauss(x,%samplitude,%smean,%ssigma)",pc,pc,pc);
    } else if(word=="BW") {
      if(wordn>=2) Lib::smanip::printf(p,32,"BW_%d_",index);
      Lib::smanip::printf
        (s,132,"breit(x,%samplitude,%sorigin,%swidth)",pc,pc,pc);
    } else if(word=="P1") {
      if(wordn>=2) Lib::smanip::printf(p,32,"P1_%d_",index);
      Lib::smanip::printf(s,132,"pol1(x,%sp0,%sp1)",pc,pc);
    } else if(word=="P2") {
      if(wordn>=2) Lib::smanip::printf(p,32,"P2_%d_",index);
      Lib::smanip::printf(s,132,"pol2(x,%sp0,%sp1,%sp2)",pc,pc,pc);
    } else if(word=="P3") {
      if(wordn>=2) Lib::smanip::printf(p,32,"P3_%d_",index);
      Lib::smanip::printf
        (s,132,"pol3(x,%sp0,%sp1,%sp2,%sp3)",pc,pc,pc,pc);
    } else if(word=="P4") {
      if(wordn>=2) Lib::smanip::printf(p,32,"P4_%d_",index);
      Lib::smanip::printf
        (s,132,"pol4(x,%sp0,%sp1,%sp2,%sp3,%sp4)",pc,pc,pc,pc,pc);
#undef pc
    } else {
      Lib::Out out(fPrinter);
      out << "Lib::Function::Composite::set :" 
          << " in " << Lib::sout(aString)
          << ", unknown function " << Lib::sout(word)
          << Lib::endl;
      fScript = "";
      return false;
    }
    script += s;
  }

  //printf("debug : composite : \"%s\"\n",script.c_str());

  if(!fScripted.set(script)) {
    fScript = "";
    return false;
  }
  fScript = aString;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Compiled::Compiled(
 Slash::Core::IWriter& aPrinter
,const std::string& aFile
,const std::string& aModel
,void* aAddress
,const std::vector<Lib::Value>& aArgs
,unsigned int aParn
)
:fPrinter(aPrinter)
,fFile(aFile)
,fModel(aModel)
,fAddress(aAddress)
,fParn(aParn)
,fReturn(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("Lib::Function::Compiled");

  unsigned int argn = aArgs.size();
  for(unsigned int argi=0;argi<argn;argi++) {
    const Lib::Value& v = aArgs[argi];
    if( (v.type()==Slash::Core::IValue::DOUBLE_STAR) ||
        (v.type()==Slash::Core::IValue::INT_STAR)    ||
        (v.type()==Slash::Core::IValue::FLOAT_STAR)  ||
        (v.type()==Slash::Core::IValue::DOUBLE)      ||
        (v.type()==Slash::Core::IValue::INT)         ||
        (v.type()==Slash::Core::IValue::FLOAT)       ){
      //ok
    } else {
      //unknown type :
      Lib::Out out(fPrinter);
      out << "Lib::Function::Compiled::Compiled :"
          << " " << Lib::sout(v.stype()) << " not handled."
          << Lib::endl;
      fAddress = 0;
      fDims.clear();
      return;
    }
    fDims.push_back(v.dimension());
    //printf("debug :  arg \"%s\" %d\n",NAME.c_str(),dim);
  }

  // to optimize :
 {fValues.resize(argn,0);
  for(unsigned int argi=0;argi<argn;argi++) {
    const Lib::Value& v = aArgs[argi];
    if(v.type()==Slash::Core::IValue::DOUBLE) {
      fValues[argi] = new Lib::Value(double(0));
    } else if(v.type()==Slash::Core::IValue::FLOAT) {
      fValues[argi] = new Lib::Value(float(0));
    } else if(v.type()==Slash::Core::IValue::INT) {
      fValues[argi] = new Lib::Value(int(0));
    } else if(v.type()==Slash::Core::IValue::DOUBLE_STAR) {
      unsigned int dim = fDims[argi];
      double* array = dim ? new double[dim] : 0;
      fValues[argi] = new Lib::Value(array);
    } else if(v.type()==Slash::Core::IValue::INT_STAR) {
      unsigned int dim = fDims[argi];
      int* array = dim ? new int[dim] : 0;
      fValues[argi] = new Lib::Value(array);
    } else if(v.type()==Slash::Core::IValue::FLOAT_STAR) {
      unsigned int dim = fDims[argi];
      float* array = dim? new float[dim] : 0;
      fValues[argi] = new Lib::Value(array);
    }
  }}
  fReturn = new Lib::Value;

}
//////////////////////////////////////////////////////////////////////////////
Lib::Function::Compiled::~Compiled(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fReturn;
 {unsigned int argn = fValues.size();
  for(unsigned int argi=0;argi<argn;argi++) {
    Slash::Core::IValue* ivalue = fValues[argi];
    if(ivalue->type()==Slash::Core::IValue::FLOAT_STAR) {
      if(ivalue->get_float_star()) delete [] ivalue->get_float_star();
    } else if(ivalue->type()==Slash::Core::IValue::DOUBLE_STAR) {
      if(ivalue->get_double_star()) delete [] ivalue->get_double_star();
    } else if(ivalue->type()==Slash::Core::IValue::INT_STAR) {
      if(ivalue->get_int_star()) delete [] ivalue->get_int_star();
    }
    delete ivalue;
  }}
  Lib::Debug::decrement("Lib::Function::Compiled");
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Function::Compiled::file(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFile;
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Function::Compiled::model(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fModel;
}
//////////////////////////////////////////////////////////////////////////////
void* Lib::Function::Compiled::address(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fAddress;
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::Function::Compiled::dimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int dim = 0;
  unsigned int dimn = fDims.size();
  for(unsigned int i=0;i<dimn;i++) dim += (fDims[i]?fDims[i]:1);
  return dim;
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::Function::Compiled::numberOfParameters(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParn;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Compiled::value(
 const std::vector<double>& aArgs
,double& aValue
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAddress) {
    aValue = 0;
    return false;
  }

  if(aArgs.size()!=dimension()) {
    aValue = 0;
    return false;
  }

  bool status = true;

  unsigned int argn = fValues.size();
  unsigned int aArgn = aArgs.size();

 {unsigned int argc = 0;
  for(unsigned int argi=0;argi<argn;argi++) {
    Slash::Core::IValue* ivalue = fValues[argi];

    if(ivalue->type()==Slash::Core::IValue::DOUBLE) {
      if((argc+1)>aArgn) {
        status = false;
        break;
      } else {
        ivalue->set(aArgs[argc]);
        argc++;
      }
    } else if(ivalue->type()==Slash::Core::IValue::FLOAT) {
      if((argc+1)>aArgn) {
        status = false;
        break;
      } else {
        ivalue->set((float)aArgs[argc]);
        argc++;
      }
    } else if(ivalue->type()==Slash::Core::IValue::INT) {
      if((argc+1)>aArgn) {
        status = false;
        break;
      } else {
        ivalue->set((int)aArgs[argc]);
        argc++;
      }
    } else if(ivalue->type()==Slash::Core::IValue::DOUBLE_STAR) {
      unsigned int dim = fDims[argi];
      if((argc+dim)>aArgn) {
        status = false;
        break;
      } else {
        double* array = ivalue->get_double_star();
        for(unsigned int i=0;i<dim;i++) {
          array[i] = aArgs[argc];
          argc++;
        }
      }

    } else if(ivalue->type()==Slash::Core::IValue::INT_STAR) {
      unsigned int dim = fDims[argi];
      if((argc+dim)>aArgn) {
        status = false;
        break;
      } else {
        int* array = ivalue->get_int_star();
        for(unsigned int i=0;i<dim;i++) {
          array[i] = (int)aArgs[argc];
          argc++;
        }
      }

    } else if(ivalue->type()==Slash::Core::IValue::FLOAT_STAR) {
      unsigned int dim = fDims[argi];
      if((argc+dim)>aArgn) {
        status = false;
        break;
      } else {
        float* array = ivalue->get_float_star();
        for(unsigned int i=0;i<dim;i++) {
          array[i] = (float)aArgs[argc];
          argc++;
        }
      }

    } else {
      status = false;
      break;
    }
  }
  if(argc!=dimension()) {
    status = false;
  }}
  //if(!status) {
    //printf("debug : Lib::Function::Compiled::value : arg setup failed.\n");
  //}

  if(status) {
    typedef bool (*Function)(const std::vector<Slash::Core::IValue*>&,
                             Slash::Core::IValue&);
    status = ((Function)fAddress)(fValues,*fReturn);
  }
 
  if(status) {
    status = true;
    if(fReturn->type()==Slash::Core::IValue::FLOAT) {
      aValue = fReturn->get_float();
    } else if(fReturn->type()==Slash::Core::IValue::DOUBLE) {
      aValue = fReturn->get_double();
    } else {
      status = false;    
    }
  }

  return status;  
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Compiled::value(
 const std::vector<Slash::Core::IValue*>& aArgs
,Slash::Core::IValue& aValue
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAddress) {
    aValue.setNone();
    return false;
  }

  typedef bool (*Function)(const std::vector<Slash::Core::IValue*>&,
                           Slash::Core::IValue&);

  return ((Function)fAddress)(aArgs,aValue);
}
//////////////////////////////////////////////////////////////////////////////
std::vector<Slash::Core::IValue*> Lib::Function::Compiled::signature(
 Slash::Core::IValue*& aReturn
) const
//////////////////////////////////////////////////////////////////////////////
// args + return value.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aReturn = fReturn;
  return fValues;
}
/*
//////////////////////////////////////////////////////////////////////////////
bool Lib::Function::Compiled::build_signature(
 const std::vector<Lib::Value>& aArgs
,std::string& aString
) const
//////////////////////////////////////////////////////////////////////////////
// signature is a list of "," separated of :
//    <string> [INTEGER,FLOAT,DOUBLE,..] <int>
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sargs;
  unsigned int argn = aArgs.size();
  for(unsigned int argi=0;argi<argn;argi++) {
    const Lib::Value& v = aArgs[argi];
    if(!v.label().size()) {
      Lib::Out out(fPrinter);
      out << "Lib::Function::Compiled::build_signature :" 
          << " empty variable name."
          << Lib::endl;
      aString = "";
      return false;
    }
    if(argi) sargs += ",";
    sargs += v.label();
    sargs += " ";
    sargs += v.stype();
    sargs += " ";
    sargs += smanip::tostring((int)aArgs[argi].dimension());
  }
  aString = sargs;
  return true;
}
*/
