// this :
#include <Lib/InternetSocket.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Debug.h>
#include <Lib/System.h>
#include <Lib/smanip.h>

#ifdef WIN32
#include <io.h>
#include <winsock.h>
#define CLIENT_ADDR_LENGTH int
#else
#include<sys/socket.h>
#include<netinet/in.h>
#include<netdb.h>
#include <errno.h>
#include <unistd.h>  //close,sleep
#ifdef __alpha
#define CLIENT_ADDR_LENGTH int
#else
#define CLIENT_ADDR_LENGTH socklen_t
#endif
#endif

#include <string.h> //strerror

// #include <sys/systeminfo.h>
// #define  GET_HOSTNAME( hostname, length )  sysinfo( SI_HOSTNAME, hostname, length ) 
#define  GET_HOSTNAME( hostname, length )  gethostname( hostname, length ) 


const int  MAX_BINDING_TRIAL = 10;
const int  BINDING_TRIAL_INTERVAL = 5;
const int  MAX_CONNECT_TRIAL = 10;

//////////////////////////////////////////////////////////////////////////////
Lib::InternetSocket::InternetSocket(
 Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
:BaseSocket(aPrinter,aVerboseLevel)
,fEnv("LIB_HOST")
,fPort(50701)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Debug::increment("Lib::InternetSocket");
#ifdef WIN32
  WSADATA WSAData;
  int initwinsock = 0;
  if (initwinsock = ::WSAStartup(MAKEWORD(2, 0), &WSAData)) {
    Out out(fPrinter);
    out << "Lib::InternetSocket::InternetSocket :"
        << " starting sockets failed." 
        << Lib::endl;
  }
#endif

 {std::string s;
  if(Lib::System::getenv("LIB_PORT",s)) {
    int port;
    if(Lib::smanip::toint(s,port)) {
      fPort = port;
      if(fVerboseLevel) {
        Out out(fPrinter);
        out << "Lib::InternetSocket::InternetSocket :"
            << " port taken from LIB_PORT " << Lib::sout(s) << "."
            << Lib::endl;
      }
    }
  }}

}
//////////////////////////////////////////////////////////////////////////////
Lib::InternetSocket::~InternetSocket(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  ::WSACleanup();
#endif
  Debug::decrement("Lib::InternetSocket");
}
//////////////////////////////////////////////////////////////////////////////
void Lib::InternetSocket::setPort( 
 int aPort 
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fPort = aPort ; 
}
//////////////////////////////////////////////////////////////////////////////
int Lib::InternetSocket::port(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fPort; 
}
//////////////////////////////////////////////////////////////////////////////
void Lib::InternetSocket::setEnv( 
  const std::string& aEnv
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fEnv = aEnv; 
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::InternetSocket::accept(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSocket!=(-1)) return true; //Done.

  Out out(fPrinter);

  int fd_socket_waiting;

  int num_binding_trial = 0;
  while(1) {
    // controlling retry
    num_binding_trial++ ;
    if( ( num_binding_trial % BINDING_TRIAL_INTERVAL ) == 0 ) { 
      out << "Lib::InternetSocket::accept :"
          << " trying to bind soket. Please wait." 
          << Lib::endl;
    }
    if( num_binding_trial > MAX_BINDING_TRIAL ) { 
      out << "Lib::InternetSocket::accept :"
          << " cannot use port " << fPort << "." 
          << Lib::endl;
      return false;
    }

    // make socket
    fd_socket_waiting = ::socket(AF_INET,SOCK_STREAM,0);
    if( fd_socket_waiting < 0 ) { 
      out << "Lib::InternetSocket::accept :"
          << " socket : " << std::string(strerror(errno))
          << Lib::endl;
      return false;
    }

    // Force reuse of the server socket (i.e. do not wait for the time
    // out to pass).
    int val = 1;
    if(::setsockopt(fd_socket_waiting,
                    SOL_SOCKET,SO_REUSEADDR,(char*)&val,sizeof(val)) == -1) {
      out << "Lib::InternetSocket::accept :"
          << " setsockopt : " << std::string(strerror(errno))
          << Lib::endl;
      return false;
    }

    // get server name
    char server_hostname[32];
    if( GET_HOSTNAME( server_hostname, sizeof( server_hostname ) ) < 0 ) { 
      out << "Lib::InternetSocket::accept :"
          << " gethostname : " << std::string(strerror(errno))
          << Lib::endl;
      return false;
    }
    if(fVerboseLevel) {
      out << "Lib::InternetSocket::accept :"
          << " hostname : " << Lib::sout(server_hostname) << "."
          << Lib::endl;
    }

    // get IP address of server from its name
    hostent* server_host_p;
    if( !(server_host_p = ::gethostbyname( server_hostname ) ) ) { 
      out << "Lib::InternetSocket::accept :"
          << " gethostbyname : " << std::string(strerror(errno))
          << Lib::endl;
      return false;
    }

    // binding
    sockaddr_in server_address;
    ::memset((char*)&server_address,'\0',sizeof(server_address)) ; 

    // clear server_address
    server_address.sin_family = AF_INET ;
    server_address.sin_port = htons(fPort);
    ::memcpy((char*)(&server_address.sin_addr), 
             (char*)(server_host_p->h_addr), 
             server_host_p->h_length);

    if(::bind(fd_socket_waiting,
              (struct sockaddr*)(&server_address),sizeof(server_address))<0 ) {
      if(fVerboseLevel) {
        out << "Lib::InternetSocket::accept :"
            << " bind : " << std::string(strerror(errno))
            << " : retry..." 
            << Lib::endl;
      }
      ::close(fd_socket_waiting);

    } else {
      if(fVerboseLevel) {
        out << "Lib::InternetSocket::accept :"
            << " socket is bound on port " << fPort << "." 
            << Lib::endl;
      }
      break ;
    }
  }


  // listen (wait connection)
  if(::listen(fd_socket_waiting,1) == -1 ) { 
    out << "Lib::InternetSocket::accept :"
            << " listen : " << std::string(strerror(errno))
            << Lib::endl;
    return false;
  }

  // accept connection
  sockaddr_in client_address;
  CLIENT_ADDR_LENGTH client_addr_length = sizeof(client_address);
  fSocket = ::accept(fd_socket_waiting, 
                     (struct sockaddr*)(&client_address), 
                     &client_addr_length ) ;
  ::close(fd_socket_waiting);
  if(fSocket<0) { 
    out << "Lib::InternetSocket::accept :"
            << " accept : " << std::string(strerror(errno))
            << Lib::endl;
    fSocket = -1;
    return false;
  }

  if(fVerboseLevel) {
    out << "Lib::InternetSocket::accept :"
        << " accept ok."
       << Lib::endl;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::InternetSocket::connect(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSocket!=(-1)) return true; //Done.

  Out out(fPrinter);

  std::string server_hostname;
  if(!Lib::System::getenv(fEnv,server_hostname)) {
    out << "Lib::InternetSocket::connect :"
        << " environment variable " << fEnv << " not defined." 
        << Lib::endl;
    return false;
  }

  // make socket :
  fSocket = ::socket(AF_INET, SOCK_STREAM,0);
  if( fSocket < 0 ) { 
    out << "Lib::InternetSocket::accept :"
            << " socket : " << std::string(strerror(errno))
            << Lib::endl;
    fSocket = -1;
    return false;
  }

  // get IP address of server from its name :
  hostent* server_host_p = ::gethostbyname(server_hostname.c_str()) ;
  if(!server_host_p) {
    out << "Lib::InternetSocket::accept :"
            << " gethostbyname : " << std::string(strerror(errno))
            << Lib::endl;
    ::close(fSocket);
    fSocket = -1;
    return false;
  }

  // connection and binding 
  sockaddr_in server_address;
  ::memset((char*)&server_address,'\0',sizeof(server_address)); 
  // clear server_address
  server_address.sin_family = AF_INET ;
  server_address.sin_port = htons(fPort);
  ::memcpy((char*)(&server_address.sin_addr), 
           (char*)(server_host_p->h_addr), 
           server_host_p->h_length);

  int connection_status = -1;
  int num_trial = 0;
  while( (connection_status<0) && (num_trial<=MAX_CONNECT_TRIAL) ) {
    num_trial++ ; 
    connection_status = ::connect 
      (fSocket,(struct sockaddr*)(&server_address),sizeof(server_address));
    if( connection_status  <0 ) {
      if(fVerboseLevel) {
        out << "Lib::InternetSocket::connect :"
            << " connect : " << std::string(strerror(errno))
            << " : retry..."
            << Lib::endl;
      }
    } else {
      if(fVerboseLevel) {
        out << "Lib::InternetSocket::connect :"
            << " connect to " << server_hostname << " ok."
            << Lib::endl;
      }
      return true;
    }
#ifdef WIN32
    ::Sleep(1000);                
#else
    ::sleep(1);                
#endif
  }

  out << "Lib::InternetSocket::connect :"
      << " can't connect."
      << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::InternetSocket::disconnect(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  closeSocket();
}
