//this :
#include <Lib/Messages.h>

#include <Slash/Core/IAction.h>
#include <Slash/Core/IRelation.h>
#include <Lib/Cast.h>
#include <Lib/Debug.h>

Lib::BaseMessage::BaseMessage(){ 
  Lib::Debug::increment("Lib::BaseMessage"); 
  //printf("debug : create %ld \n",this);
}
Lib::BaseMessage::~BaseMessage(){ 
  //printf("debug : delete %ld \n",this);
  Lib::Debug::decrement("Lib::BaseMessage"); 
}

Lib::ExitMessage::ExitMessage(int aCode):BaseMessage(),fCode(aCode){
  Lib::Debug::increment("Lib::ExitMessage"); 
}
Lib::ExitMessage::~ExitMessage(){
  Lib::Debug::decrement("Lib::ExitMessage"); 
}
void* Lib::ExitMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::ExitMessage)
  else return 0;
}
std::string Lib::ExitMessage::message() const { 
  return std::string("Lib::ExitMessage.");
}
int Lib::ExitMessage::code() const { 
  return fCode;
}

Lib::DeleteMessage::DeleteMessage(const std::string& aWhat)
:BaseMessage(),fWhat(aWhat){
  Lib::Debug::increment("Lib::DeleteMessage"); 
}
Lib::DeleteMessage::~DeleteMessage(){
  Lib::Debug::decrement("Lib::DeleteMessage"); 
}
void* Lib::DeleteMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::DeleteMessage)
  else return 0;
}
std::string Lib::DeleteMessage::message() const { 
  return std::string("Lib::DeleteMessage : \"") + fWhat + std::string("\".");
}
const std::string& Lib::DeleteMessage::what() const { return fWhat;}


Lib::UpdateMessage::UpdateMessage(const std::string& aWhat,Slash::Core::IRelation* aRelation)
  :BaseMessage(),fWhat(aWhat),fRelation(aRelation){
  Lib::Debug::increment("Lib::UpdateMessage"); 
}
Lib::UpdateMessage::~UpdateMessage(){
  Lib::Debug::decrement("Lib::UpdateMessage"); 
}
void* Lib::UpdateMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::UpdateMessage)
  else return 0;
}
std::string Lib::UpdateMessage::message() const { 
  return std::string("Lib::UpdateMessage : \"") + fWhat + std::string("\".");
}
const std::string& Lib::UpdateMessage::what() const { return fWhat;}
Slash::Core::IRelation* Lib::UpdateMessage::relation() const { return fRelation;}

Lib::CreateMessage::CreateMessage(const std::string& aWhat)
:BaseMessage(),fWhat(aWhat){
  Lib::Debug::increment("Lib::CreateMessage"); 
}
Lib::CreateMessage::~CreateMessage(){
  Lib::Debug::decrement("Lib::CreateMessage"); 
}
void* Lib::CreateMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::CreateMessage)
  else return 0;
}
std::string Lib::CreateMessage::message() const { 
  return std::string("Lib::CreateMessage : \"") + fWhat + std::string("\".");
}
const std::string& Lib::CreateMessage::what() const { return fWhat;}


Lib::ExecuteMessage::ExecuteMessage(Slash::Core::IAction& aAction):BaseMessage(),fAction(aAction){
  Lib::Debug::increment("Lib::ExecuteMessage"); 
}
Lib::ExecuteMessage::~ExecuteMessage(){
  Lib::Debug::decrement("Lib::ExecuteMessage"); 
}
Slash::Core::IAction& Lib::ExecuteMessage::action() const { return fAction;}
void* Lib::ExecuteMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::ExecuteMessage)
  else return 0;
}
std::string Lib::ExecuteMessage::message() const { 
  return std::string("Lib::ExecuteMessage : \"") 
    + fAction.name() + std::string("\".");
}

Lib::RelationMessage::RelationMessage(Slash::Core::IRelation& aRelation)
:BaseMessage()
,fRelation(aRelation){
  Lib::Debug::increment("Lib::RelationMessage"); 
}
Lib::RelationMessage::~RelationMessage(){
  Lib::Debug::decrement("Lib::RelationMessage"); 
}
Slash::Core::IRelation& Lib::RelationMessage::relation() const { return fRelation;}
void* Lib::RelationMessage::cast(const std::string& aClass) const {
  if_Lib_SCast(Lib::RelationMessage)
  else return 0;
}
std::string Lib::RelationMessage::message() const { 
  return "";
}

