%{

// this :
#include <Lib/ProcessorLexer.h>

// Slash :
#include <Slash/Core/IWriter.h>

// Lib :
namespace Lib {class Expression;}
#include <Lib/ProcessorTokens.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/mmanip.h>

#include <string.h>

//#define DEBUG

#ifndef __APPLE__
extern "C" {extern int isatty (int ) throw();}
#endif

#define _UNISTD_H_
#define YY_NEVER_INTERACTIVE 1
#undef YY_INPUT
#define YY_INPUT(aBuffer,aResult,aMaxSize) (aResult=Lib::ProcessorLexer::input(aBuffer,aMaxSize)) 
#define NotFound (-1)

static Lib::ProcessorLexer* sLexer = 0;

%}

/* signus handled in by the parser :
number [-+]?([0-9]+|(([0-9]+\.[0-9]*)|([0-9]*\.[0-9]+))([eE][-+]?[0-9]+)?)
integer [0-9]+
*/
integer ([0-9]+)([eE][+]?[0-9]+)?
hexaint 0x[0-9a-fA-F]+
number (([0-9]+\.[0-9]*)|([0-9]*\.[0-9]+))([eE][-+]?[0-9]+)?
name   [a-zA-Z_][a-zA-Z0-9_]*
string (\"[^\"\n]*\")|('[^'\n]*')

%%


[\t ]+ /* ignore whitespace */;

{hexaint} { 
#ifdef DEBUG
  sLexer->out().println("debug : Lexer.l : integer :%s|",yytext);
#endif
  unsigned int i;
  if(sscanf(yytext,"0x%x",&i)!=1) {
    Lib::Out out(sLexer->printer());
    out << "Lib::ProcessorLexer :"
        << " problem converting " << Lib::sout(yytext) 
        << " to unsigned int."
        << Lib::endl;
    Processor_lval.fInteger = 0;
  } else {
    Processor_lval.fInteger = i;
  }
  return NUMBER_INTEGER;
}

{integer} { 
  char* s;
#ifdef DEBUG
  sLexer->out().println("debug : Lexer.l : integer :%s|",yytext);
#endif
  unsigned int i,j;
  if( (sscanf(yytext,"%ue%u",&i,&j)==2)  ||
      (sscanf(yytext,"%ue+%u",&i,&j)==2) ||
      (sscanf(yytext,"%uE%u",&i,&j)==2)  ||
      (sscanf(yytext,"%uE+%u",&i,&j)==2) ){
    Processor_lval.fInteger = i * Lib::mmanip::pow(10,j);
  } else if(sscanf(yytext,"%u",&i)==1) {
    Processor_lval.fInteger = i;
  } else {
    Lib::Out out(sLexer->printer());
    out << "Lib::ProcessorLexer :"
        << " problem converting " << Lib::sout(yytext) 
        << " to long."
        << Lib::endl;
    Processor_lval.fInteger = 0;
  }
  return NUMBER_INTEGER;
}

{number} {
  char* s;
#ifdef DEBUG
  sLexer->out().println("debug : Lexer.l : number :%s|",yytext);
#endif
  Processor_lval.fNumber = strtod(yytext,&s);
  if((s==yytext)||(s!=(yytext+strlen(yytext)))) {
    Lib::Out out(sLexer->printer());
    out << "Lib::ProcessorLexer :"
        << " problem converting " << Lib::sout(yytext) 
        << " to double."
        << Lib::endl;
    Processor_lval.fNumber = 0.;
  }
  return NUMBER_REAL;
}

cos   return T_COS;
sin   return T_SIN;
tan   return T_TAN;
acos  return T_ACOS;
asin  return T_ASIN;
atan  return T_ATAN;
sqrt  return T_SQRT;
pow   return T_POW;
log   return T_LOG;
log10 return T_LOG10;
exp   return T_EXP;
fabs  return T_FABS;
abs  return T_FABS;
cosh  return T_COSH;
sinh  return T_SINH;
tanh  return T_TANH;

gauss return T_GAUSS;
breit return T_BW;
expo  return T_EXPO;
pol1  return T_POL1;
pol2  return T_POL2;
pol3  return T_POL3;
pol4  return T_POL4;
cauchy  return T_CAUCHY;

M_PI return T_PI;

true return T_TRUE;
TRUE return T_TRUE;
True return T_TRUE;
false return T_FALSE;
FALSE return T_FALSE;
False return T_FALSE;

int return T_TOINT;
mod return T_MOD;
array return T_ARRAY;
PI return T_PI;
pi return T_PI;
or return T_OR;
OR return T_OR;
and return T_AND;
AND return T_AND;
nco return T_NCO;
vsum return T_VSUM;
rndm return T_RNDM;

{name} {
#ifdef DEBUG
  sLexer->out().println("debug : Lexer.l : name :%s|",yytext);
#endif
  sLexer->variables().push_back(yytext);
  Processor_lval.fIndex = sLexer->variables().size()-1;
  return T_NAME;
}

{string} {
#ifdef DEBUG
  sLexer->out().println("debug : Lexer.l : string :%s|",yytext);
#endif
  // Remove leading and trailing double quote :
  // "abcd"
  // 012345
  std::string s = yytext;
  sLexer->variables().push_back(s.substr(1,s.length()-2)); 
  int index = sLexer->variables().size()-1;
  Processor_lval.fIndex = index;
  return STRG;
}

. return yytext[0];

%%
//////////////////////////////////////////////////////////////////////////////
Lib::ProcessorLexer::ProcessorLexer(
 Slash::Core::IWriter& aPrinter
)
:fCurrent(0)
,fEnd(0)
,fPrinter(aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::ProcessorLexer& Lib::ProcessorLexer::operator=(
 const ProcessorLexer& /*aProcessorLexer*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Reset other things :
  fString = "";
  fCurrent = 0;
  fEnd = 0;
  fVariables.clear();
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::ProcessorLexer::set(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fString = aString;
  sLexer = 0;
  fCurrent = (char*)fString.c_str();
  fEnd = fCurrent + ::strlen(fCurrent);
  sLexer = this;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::ProcessorLexer::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string>& Lib::ProcessorLexer::variables(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVariables;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::ProcessorLexer::variable(
 unsigned int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVariables[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
int Lib::ProcessorLexer::input(
 char* aBuffer
,int aMaxSize
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!sLexer) return 0;
  if(aMaxSize<=0) return 0;
  int rest = sLexer->fEnd - sLexer->fCurrent;
  int n = aMaxSize>rest ? rest : aMaxSize;
  if(n>0) {
    memcpy(aBuffer,sLexer->fCurrent,n);
    sLexer->fCurrent += n;
  }
  return n;
}
//////////////////////////////////////////////////////////////////////////////
int yywrap(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 1;
}

