// this :
#include <Lib/Session.h>

#include <Slash/Core/IManager.h>
#include <Lib/Out.h>
#include <Lib/sout.h>

//////////////////////////////////////////////////////////////////////////////
Lib::Session::Session(
 Slash::Core::IWriter& aPrinter
)
:fPrinter(aPrinter)
,fVerboseLevel(0)
,fName("Lib_Session")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Session::~Session(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::iterator it;
  while(!fManagers.empty()) {
    it = fManagers.end();
    it--;
    if(fVerboseLevel) {
      Lib::Out out(fPrinter);
      out << "Lib::Session::~Session :"
          << " delete manager " << Lib::sout((*it)->name()) 
          << "." << Lib::endl;
    }
    Slash::Core::IManager* manager = *it;
    fManagers.erase(it); //WARNING : must be done before the below.
    delete manager; //WARNING : some ~manager may rm themself from the list.
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::Session::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int Lib::Session::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::addManager(
 Slash::Core::IManager* aManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fManagers.push_back(aManager);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::removeManager(
 Slash::Core::IManager* aManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::iterator it;
  for(it=fManagers.begin();it!=fManagers.end();) {
    if((*it)==aManager) {
      it = fManagers.erase(it);
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IManager* Lib::Session::findManager(
 const std::string& aString
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::const_iterator it;
  for(it=fManagers.begin();it!=fManagers.end();++it) {
    if(aString==(*it)->name()) return *it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::destroyManager(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::iterator it;
  for(it=fManagers.begin();it!=fManagers.end();) {
    Slash::Core::IManager* manager = *it;
    if(manager->name()==aName) {
      it = fManagers.erase(it);
      delete manager;
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
unsigned int Lib::Session::numberOfManagers(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fManagers.size();
}
//////////////////////////////////////////////////////////////////////////////
const Slash::Core::IManager* Lib::Session::manager(
 unsigned int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fManagers[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Session::managerNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> names;
  std::vector<Slash::Core::IManager*>::const_iterator it;
  for(it=fManagers.begin();it!=fManagers.end();++it) {
    names.push_back((*it)->name());
  }
  return names;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& Lib::Session::printer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Session::load(
 const std::string&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::redirectOutput(
 bool 
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::flush(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Session::setParameter(
 const std::string&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Session::parameterValue(
 const std::string&
,std::string&)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> Lib::Session::availableParameters(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return std::vector<std::string>();
}
//////////////////////////////////////////////////////////////////////////////
std::vector< std::pair<std::string,std::string> > Lib::Session::arguments(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fArguments;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Session::setArguments(
 const std::vector< std::pair<std::string,std::string> >& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fArguments = aArgs;
}
