// From HEPVis/SbFile.

// this :
#include <Lib/dirmanip.h>

#include <Lib/smanip.h>

#ifdef WIN32
#include <windows.h>
#include <direct.h>
#else //UNIX
#include <dirent.h>
#if defined(_POSIX_SOURCE)
#define REAL_DIR_ENTRY(dp) 1
#else
#define REAL_DIR_ENTRY(dp) (dp->d_ino != 0)
#endif
#include <unistd.h>
#endif

#include <sys/stat.h>

//////////////////////////////////////////////////////////////////////////////
Lib::Directory::Directory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Directory::~Directory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Lib::Directory::Directory(
 const Directory& aDirectory
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPath = aDirectory.fPath;
  fFiles = aDirectory.fFiles;
  fDirs = aDirectory.fDirs;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::Directory::visit(
 Slash::Core::IDirectoryVisitor& aVisitor
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool process;
  if(!aVisitor.directory(fPath,process)) return false;
  if(!process) return true;
  unsigned int index;
  unsigned int filen = fFiles.size();
  for(index=0;index<filen;index++) {
    bool is;
    if(!dirmanip::isDirectory(fFiles[index],is)) return false;
    if(!is) {
      if(!aVisitor.file(fFiles[index])) return false;
    }
  }
  unsigned int dirn = fDirs.size();
  for(index=0;index<dirn;index++) {
    if(!fDirs[index].visit(aVisitor)) return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Directory::addDirectory(
 Directory& aDirectory
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fDirs.push_back(aDirectory);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Directory::addFile(
 const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFiles.push_back(aFile);
}
//////////////////////////////////////////////////////////////////////////////
void Lib::Directory::setPath(
 const std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPath = aPath;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<Lib::Directory>& Lib::Directory::directories(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDirs;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& Lib::Directory::path(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPath;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::create(
 const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  return (::mkdir(aPath.c_str())==0 ? true : false); 
#else
  return (::mkdir(aPath.c_str(), 0755)==0 ? true : false); 
#endif
}
#include <stdio.h> // For ::remove, ::fopen.
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::remove(
 const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  struct stat finfo;
  if(::stat(aPath.c_str(),&finfo) < 0) return false;
#ifdef WIN32
  if(!(finfo.st_mode & S_IFDIR)) return false;
#else
  if(!S_ISDIR(finfo.st_mode)) return false;
#endif
  // Empty the directory :
  std::vector<std::string> files;
  entries(aPath,files);
  unsigned int filen = files.size();
  for(unsigned int count=0;count<filen;count++) {
    const std::string& entry = files[count];     

    //FIXME : optimize : do not pass by entries().    
   {std::string path,name,suffix;
    Lib::smanip::pathNameSuffix(entry,path,name,suffix);
    if((name==".")||(name=="..")) continue;}

    bool isDir;
    if(!isDirectory(entry,isDir)) return false;
    if(isDir) {
      if(!dirmanip::remove(entry)) {
        //printf("debug : rm dir %s failed.\n",entry.c_str());
        return false;
      }
    } else {
      if(::remove(entry.c_str())!=0) {
        //printf("debug : rm file %s failed.\n",entry.c_str());
        return false;
      }
    }
  }
  // Remove the directory :
  return (::rmdir(aPath.c_str())==0?true:false);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::removeInDirectory(
 const std::string& aPath
,const std::string& aPattern
)
//////////////////////////////////////////////////////////////////////////////
// Remove file matching aPattern in the aPath directory.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  struct stat finfo;
  if(::stat(aPath.c_str(),&finfo) < 0) return false;
#ifdef WIN32
  if(!(finfo.st_mode & S_IFDIR)) return false;
  std::string sep = "\\";
#else
  if(!S_ISDIR(finfo.st_mode)) return false;
  std::string sep = "/";
#endif
  std::vector<std::string> files;
  entries(aPath,files,false);
  bool status = true;
  unsigned int filen = files.size();
  for(unsigned int count=0;count<filen;count++) {
    if(smanip::match(files[count],aPattern)) {
      std::string file = aPath+sep+files[count];
      if(::remove(file.c_str())!=0) status = false;
    }
  }
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::entries(
 const std::string& aPath
,std::vector<std::string>& aList
,bool aFullPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aList.clear();
  struct stat finfo;
  if (::stat(aPath.c_str(),&finfo) < 0)  return false;
#ifdef WIN32
  if (!(finfo.st_mode & S_IFDIR)) return false;
  std::string entry = aPath;
  if (!(entry[entry.size()] == '/' || entry[entry.size()] == '\\' ))
    entry += "\\";
  entry += "*";
  WIN32_FIND_DATA findFileData;
  HANDLE dir = ::FindFirstFile(entry.c_str(),&findFileData);
  if(dir == INVALID_HANDLE_VALUE) return false;
  // Get file names :
  for (;;) {
    if(!::FindNextFile(dir,&findFileData)) break;
    std::string name = (const char*)findFileData.cFileName;
    // Be sure we can work on the file :
    std::string fname = aPath+"\\"+name;
    if (::stat(fname.c_str(),&finfo) < 0)  continue;
    if(aFullPath)
      aList.push_back(fname);
    else
      aList.push_back(name);
  }
  ::FindClose(dir);
#else
  if (!S_ISDIR(finfo.st_mode)) return false;
  DIR* dir = ::opendir(aPath.c_str());
  if(!dir) return false;
  // Get file names :
  for (;;) {
    struct dirent* dp = ::readdir(dir);
    //struct direct* dp;
    if (dp==NULL) break;
    if (REAL_DIR_ENTRY(dp)) {
      std::string name = dp->d_name;
      // Be sure we can work on the file :
      std::string fname = aPath+"/"+name;
      if (::stat(fname.c_str(),&finfo) < 0)  continue;
      if(aFullPath)
        aList.push_back(fname);
      else
        aList.push_back(name);
    }
  }
  ::closedir(dir);
#endif
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::isEmpty(
 const std::string& aPath
,bool& aIsEmpty
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aIsEmpty = true;
  struct stat finfo;
  if (::stat(aPath.c_str(),&finfo) < 0)  return false;
#ifdef WIN32
  if (!(finfo.st_mode & S_IFDIR)) return false;
  std::string entry = aPath;
  if (!(entry[entry.size()] == '/' || entry[entry.size()] == '\\' ))
    entry += "\\";
  entry += "*";
  WIN32_FIND_DATA findFileData;
  HANDLE dir = ::FindFirstFile(entry.c_str(),&findFileData);
  if(dir == INVALID_HANDLE_VALUE) return false;
  for (;;) {
    if(!::FindNextFile(dir,&findFileData)) break;
    std::string name = (const char*)findFileData.cFileName;
    if(name==".") continue;
    if(name=="..") continue;
    aIsEmpty = false;
    ::FindClose(dir);
    return true;  
  }
  ::FindClose(dir);
#else
  if (!S_ISDIR(finfo.st_mode)) return false;
  DIR* dir = ::opendir(aPath.c_str());
  if(!dir) return false;
  for (;;) {
    struct dirent* dp = ::readdir(dir);
    //struct direct* dp;
    if (dp==NULL) break;
    if (REAL_DIR_ENTRY(dp)) {
      std::string name = dp->d_name;
      if(name==".") continue;
      if(name=="..") continue;
      aIsEmpty = false;
      ::closedir(dir);
      return true;  
    }
  }
  ::closedir(dir);
#endif
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::directory(
 std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
// Return current directory.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
#define LIB_MAXPATHLEN 1024
  char cwd[LIB_MAXPATHLEN];
#ifdef WIN32
  //  driveletter = 0 means return the working directory for the default drive.
  if(::_getdcwd(0,cwd,LIB_MAXPATHLEN)==NULL) return false;
#else
  if(::getcwd(cwd,LIB_MAXPATHLEN)==NULL) return false;
#endif
  aValue = cwd;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::go(
 const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(::chdir(aPath.c_str())!=0) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::exists(
 const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  struct stat finfo;
  if (::stat(aPath.c_str(),&finfo) < 0) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::isDirectory(
 const std::string& aPath
,bool& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = false;
  struct stat finfo;
  if (::stat(aPath.c_str(),&finfo) < 0) return false;
  aValue = ( ((finfo.st_mode & S_IFMT) == S_IFDIR) ? true : false);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::isDot(
 const std::string& aPath
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  char sep = '\\';
#else
  char sep = '/';
#endif
  unsigned int l = aPath.size();
  if((l==1) && (aPath[0]=='.') ) return true;
  if((l==2) && (aPath[0]=='.') && (aPath[l]=='.') ) return true;
  if((l>=2) && (aPath[l-1]=='.') && (aPath[l-2]==sep) ) return true;
  if((l>=3) && (aPath[l-1]=='.') && (aPath[l-2]=='.') && (aPath[l-3]==sep) )
    return true;
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::content(
 Directory& aTree
,const std::string& aPattern
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> files;
  if(!entries(aTree.path(),files)) return false;
  //printf("debug : Lib::dirmanip::content : \"%s\" %d files.\n",
  //       aTree.path().c_str(),(int)files.size());
  unsigned int filen = files.size();
  for(unsigned int index=0;index<filen;index++) {
    const std::string& name = files[index];
    bool is;
    if(!isDirectory(name,is)) {
      //printf("debug : Lib::dirmanip::content : problem with \"%s\".\n",
      //       name.c_str());
      return false;
    }
    if(is) {
      if(!isDot(name)) {
        Directory dir;
        aTree.addDirectory(dir);
        Directory& tree = aTree.directories()[aTree.directories().size()-1];
        tree.setPath(name);
        if(!content(tree,aPattern)) return false;
      }
    } else {
      if( (aPattern=="") || (aPattern=="*") ) {
        aTree.addFile(name);
      } else {
        bool match = smanip::match(name,aPattern);
        //printf("debug : Lib::dirmanip::content : \"%s\" match \"%s\" ? %d\n",
        //       name.c_str(),aPattern.c_str(),match);
        if(match) aTree.addFile(name);
      }
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::isAnEntry(
 const std::string& aPath
,const std::string& aName
,bool& aFound
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aFound = false;
  struct stat finfo;
  if (::stat(aPath.c_str(),&finfo) < 0)  return false;
#ifdef WIN32
  if (!(finfo.st_mode & S_IFDIR)) return false;
  std::string entry = aPath;
  if (!(entry[entry.size()] == '/' || entry[entry.size()] == '\\' ))
    entry += "\\";
  entry += "*";
  WIN32_FIND_DATA findFileData;
  HANDLE dir = ::FindFirstFile(entry.c_str(),&findFileData);
  if(dir == INVALID_HANDLE_VALUE) return false;
  // Get file names :
  for (;;) {
    if(!::FindNextFile(dir,&findFileData)) break;
    std::string name = (const char*)findFileData.cFileName;
    if(name==aName) {
      aFound = true;
      ::FindClose(dir);
      return true;
    }
  }
  ::FindClose(dir);
#else
  if (!S_ISDIR(finfo.st_mode)) return false;
  DIR* dir = ::opendir(aPath.c_str());
  if(!dir) return false;
  // Get file names :
  for (;;) {
    struct dirent* dp = ::readdir(dir);
    //struct direct* dp;
    if (dp==NULL) break;
    if (REAL_DIR_ENTRY(dp)) {
      std::string name = dp->d_name;
      if(name==aName) {
        aFound = true;
        ::closedir(dir);
        return true;
      }
    }
  }
  ::closedir(dir);
#endif
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::isAnEntry(
 const std::string& aPath
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool found;
  isAnEntry(aPath,aName,found);
  return found;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::mkcd(
 const std::string& aDirectory
)
//////////////////////////////////////////////////////////////////////////////
// aDirectory should be a single directory name, and not a file system path.
// Then it must not contain : ., .., /, \ etc...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool isDir;
  if(!Lib::dirmanip::isDirectory(aDirectory,isDir)) {
    if(!Lib::dirmanip::create(aDirectory)) return false;
  } else {
    if(!isDir) return false;
  }
  return Lib::dirmanip::go(aDirectory);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::mkcd(
 const std::vector<std::string>& aPath
)
//////////////////////////////////////////////////////////////////////////////
// aPath[index] should be a single directory name, and not a file system path.
// Then they must not contain : ., .., /, \ etc...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPath.size()) if(!goRoot()) return false;
  unsigned int pathn = aPath.size();
  for(unsigned int index=0;index<pathn;index++) {
    if(!mkcd(aPath[index])) return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static std::string Lib_System_getenv(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv(aString.c_str());
  return std::string(env?env:"");
}
//////////////////////////////////////////////////////////////////////////////
std::string Lib::dirmanip::home(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  std::string homedrive = Lib_System_getenv("HOMEDRIVE");
  std::string drive = (homedrive!="" ? homedrive : std::string("C:"));
  std::string homepath = Lib_System_getenv("HOMEPATH");
  if(homepath=="") return drive+"\\";
  else return drive+homepath;
#else
  return Lib_System_getenv("HOME");
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::goHome(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = home();
  if(s=="") return false;
  return go(s);
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::goRoot(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef WIN32
  std::string homedrive = Lib_System_getenv("HOMEDRIVE");
  std::string drive = (homedrive!="" ? homedrive : std::string("C:"));
  return go(drive+"\\");
#else
  return go("/");
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::find(
 const std::vector<std::string>& aPaths
,const std::string& aPackage
,const std::string& aVersion
,std::string& aPath
,std::string& aError
) 
//////////////////////////////////////////////////////////////////////////////
// aVersion could be empty and can contain wildcards.
// The list of paths is searched first to last.
// return true and aPath="" mean not found.
// return false : something went wrong in directory manipulation.
//                aError contains then an error message.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPackage=="") {
    aError = "Lib::dirmanip::find : empty package name.";
    return false;
  }

  std::string::size_type star = aVersion.find('*');
  bool vers_wildcard = (star==std::string::npos?false:true);

#ifdef WIN32
  std::string sep = "\\";
#else
  std::string sep = "/";
#endif

  unsigned int pathn = aPaths.size();
  //fprintf(stderr,"obuild_find : number of path %d\n",pathn);
  for(unsigned int pathi=0;pathi<pathn;pathi++) {
    std::string dir = aPaths[pathi];
    //fprintf(stderr,"Lib::dirmanip::find : look directory \"%s\"\n",
    //  dir.c_str());
    bool found;
    if(!Lib::dirmanip::isAnEntry(dir,aPackage,found)) {
      aPath = "";
      Lib::smanip::printf(aError,128+dir.size(),
        "Lib::dirmanip::find : can't open directory \"%s\"\n",dir.c_str());
      return false;
    }
    if(found) {
      if(aVersion=="") {
        dir += sep+aPackage;
        aPath = dir;
        aError = "";
        return true;
      }
      // Look version :
      dir += sep+aPackage;
      if(vers_wildcard) {
        std::vector<std::string> files;
        if(Lib::dirmanip::entries(dir,files,false)) {
          unsigned int number = files.size();
          for(unsigned int index=0;index<number;index++) {
            const std::string& file = files[index];
            //std::string p,n,s;
            //Lib::smanip::pathNameSuffix(file,p,n,s);
            if(Lib::smanip::match(file,aVersion)) {
              dir += sep+file;
              aPath = dir;
              aError = "";
              return true;
            }
          } 
        } 
      } else {
        bool found;
        if(!Lib::dirmanip::isAnEntry(dir,aVersion,found)) {
          aPath = "";
          Lib::smanip::printf(aError,128+dir.size(),
            "Lib::dirmanip::find : can't open directory \"%s\"\n",dir.c_str());
          return false;
        }
        dir += sep+aVersion;
        aPath = dir;
        aError = "";
        return true;
      }
    }
  }

  aPath = ""; // return true and aPath="" mean not found.
  aError = "";
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::removeIfEmpty(
 const std::string& aDirectory
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!exists(aDirectory)) return true;
  bool empty;
  if(!isEmpty(aDirectory,empty)) return false;
  if(!empty) return true;
  if(!remove(aDirectory)) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static bool Lib_System_getenv(
 const std::string& aString
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const char* env = ::getenv(aString.c_str());
  if(env) {
    aValue = std::string(env?env:"");
    return true;
  } else {
    aValue = "";
    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
static bool Lib_fmanip_exists(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  FILE* file = ::fopen(aString.c_str(),"rb");
  if(!file) return false;
  ::fclose(file);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool Lib::dirmanip::binaryPath(
 const std::string& aArg0
,std::string& aPath
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArg0=="") {
    aPath = "";
    return false;
  }
#ifdef WIN32
  std::string sep = "\\";
  std::string psep = ";";
#else
  std::string sep = "/";
  std::string psep = ":";
#endif
  std::string path,name,suffix;
  Lib::smanip::pathNameSuffix(aArg0,path,name,suffix);
  if(path=="") {
    std::string PATH;
    if(Lib_System_getenv("PATH",PATH)) {
      std::string program = aArg0;
#if defined(WIN32) || (defined(__CYGWIN__) && defined(__GNUC__))
      if(program.find(".exe")==std::string::npos) program += ".exe";
#endif
      std::vector<std::string> paths = Lib::smanip::words(PATH,psep);
      unsigned int pathn = paths.size();
      for(unsigned int pathi=0;pathi<pathn;pathi++) {
        std::string dir = paths[pathi];
        if(Lib_fmanip_exists(dir+sep+program)) {
          path = dir;
          break;
        }
      }
    }
  }
  if(path=="") {
    aPath = "";
    return false;
  }
  if(!Lib::smanip::isAbsolutePath(path)) {
    std::string pwd;
    if(!Lib::dirmanip::directory(pwd)) {
      aPath = "";
      return false;
    }
    path = pwd+sep+path;
  }
  //printf("debug : path : %s\n",path.c_str());
  aPath = path;
  return true;
}
