#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/mmanip.h>
#include <Lib/Array.h>

//#include <Lib/mxmanip.h>

#include <math.h>

//////////////////////////////////////////////////////////////////////////////
/// Real matrices ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

typedef Lib::ADD<double> RMatrix;

#define Matrix_print Lib::print

// OSF1 has problem with ::fabs passed in argument of isLogConvergent.
double my_fabs(double a) { return ::fabs(a);}

//////////////////////////////////////////////////////////////////////////////
int main(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  // Check :
 {RMatrix A(2);
  //double a = Lib::mmanip::half_pi();
  //double a = Lib::mmanip::pi();
  //double b = 0;
  double b = 0.234;
  double a = 0.1234;
  A.setValue(0,0,b);
  A.setValue(0,1,a);
  A.setValue(1,0,-a);
  A.setValue(1,1,b);
  Matrix_print(A,printer,"A");

  RMatrix exp_A = Lib::exp(A);
  Matrix_print(exp_A,printer,"exp_A");

  RMatrix log_exp_A = Lib::log(exp_A);
  Matrix_print(log_exp_A,printer,"log_exp_A");}

  RMatrix A(3);
 {double nx = 1;
  double ny = 0;
  double nz = 0;
  double theta = Lib::mmanip::half_pi()/4;
  //   0  nz -ny
  // -nz   0  nx
  //  ny -nx   0
  A.setValue(0,1, nz);
  A.setValue(0,2,-ny);
  A.setValue(1,0,-nz);
  A.setValue(1,2, nx);
  A.setValue(2,0, ny);
  A.setValue(2,1,-nx);
  A.multiply(theta);
  // Should be :
  // 0  0     0 
  // 0  0     0.392
  // 0 -0.392 0
  Matrix_print(A,printer,"A");}

  RMatrix B(3);
 {double nx = 0;
  double ny = 1;
  double nz = 0;
  double theta = Lib::mmanip::half_pi()/4;
  B.setValue(0,1, nz);
  B.setValue(0,2,-ny);
  B.setValue(1,0,-nz);
  B.setValue(1,2, nx);
  B.setValue(2,0, ny);
  B.setValue(2,1,-nx);
  B.multiply(theta);
  Matrix_print(B,printer,"B");}

  // Should be :
  // 1  0     0 
  // 0  0.923 0.382
  // 0 -0.382 0.923
  RMatrix exp_A = Lib::exp(A);
  Matrix_print(exp_A,printer,"exp_A");
  // Should be :
  //  0.923 0 -0.382
  //  0     1  0 
  //  0.382 0  0.923
  RMatrix exp_B = Lib::exp(B);
  Matrix_print(exp_B,printer,"exp_B");

  RMatrix exp_A_exp_B = exp_A * exp_B;
  Matrix_print(exp_A_exp_B,printer,"exp_A_exp_B");

  if(!Lib::isLogConvergent<double>(exp_A,my_fabs)) {
    out << "Log of exp_A not convergent." << Lib::endl;
  }
  if(!Lib::isLogConvergent<double>(exp_B,my_fabs)) {
    out << "Log of exp_B not convergent." << Lib::endl;
  }
  if(!Lib::isLogConvergent<double>(exp_A_exp_B,my_fabs)) {
    out << "Log of exp_A_exp_B not convergent." << Lib::endl;
  }

  RMatrix l = Lib::log(exp_A_exp_B);
  Matrix_print(l,printer,"log_exp_A_exp_B");
  double vx = l.value(1,2);
  double vy = l.value(2,0);
  double vz = l.value(0,1);
  double angle = ::sqrt(vx*vx+vy*vy+vz*vz);

  out << " nx = " << vx/angle << Lib::endl;
  out << " ny = " << vy/angle << Lib::endl;
  out << " nz = " << vz/angle << Lib::endl;
  out << " angle = " << angle << " rad" << Lib::endl;
  out << " angle = " << 180*angle/Lib::mmanip::pi() << " deg" << Lib::endl;

  // The Baker Hausdorf formula should be able
  // to get l by using directly A and B and the
  // commutator of A and B...

  return 0;
}

