
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/smanip.h>

#include <Lib/Array.h>

//typedef std::complex<double> z;

class Symbol {
public:
 Symbol():fValue("none"){}
 Symbol(const std::string& aValue):fValue(aValue){}
 Symbol(int aValue){
   Lib::smanip::printf(fValue,32,"%d",aValue);
 }
 virtual ~Symbol(){}
 Symbol(const Symbol& aFrom):fValue(aFrom.fValue) {}
public:
 Symbol& operator=(const Symbol& aFrom) {   
   fValue = aFrom.fValue;
   return *this;
 }
 Symbol operator*(const Symbol& aFrom) {   
   if(fValue=="1") 
     return Symbol(aFrom.fValue);
   else if(aFrom.fValue=="1") 
     return Symbol(fValue);
   else if(fValue=="0") 
     return Symbol("0");
   else if(aFrom.fValue=="0") 
     return Symbol("0");
   else
     return Symbol("("+fValue + ")*(" + aFrom.fValue+")");
 }
 Symbol operator+(const Symbol& aFrom) {   
   if(fValue=="0") 
     return Symbol(aFrom.fValue);
   else if(aFrom.fValue=="0") 
     return Symbol(fValue);
   else
     return Symbol(fValue + "+" + aFrom.fValue);
 }
 Symbol operator-(const Symbol& aFrom) {   
   if(fValue=="0") 
     return Symbol("-" + aFrom.fValue);
   else if(aFrom.fValue=="0") 
     return Symbol(fValue);
   else
     return Symbol(fValue + "-" + aFrom.fValue);
 }
 Symbol operator+=(const Symbol& aFrom) {   
   Symbol s = operator+(aFrom);
   *this = s;
   return s;
 }
 Symbol operator-=(const Symbol& aFrom) {   
   Symbol s = operator-(aFrom);
   *this = s;
   return s;
 }
 std::string value() const { return fValue;}
private:
 std::string fValue;
};

//////////////////////////////////////////////////////////////////////////////
bool det_id_44(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::ADD<Symbol> mtx(4);
  mtx.setIdentity();    

  Symbol s = mtx.determinant();
  out << "Lib_symbolic :" 
      << s.value()
      << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool det_A_33(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::ADD<Symbol> mtx(3);
  //mtx.setIdentity();    
  // 0 t0 -t1  t3
  // 0  0  t2 -t4
  // 0  0   0  t5
  // 0  0   0   0

  mtx.setValue(0,1,Symbol("t0"));
  mtx.setValue(0,2,Symbol("m_t1"));
  mtx.setValue(0,3,Symbol("t3"));
  mtx.setValue(1,2,Symbol("t2"));
  mtx.setValue(1,3,Symbol("m_t4"));
  mtx.setValue(2,3,Symbol("t5"));

  mtx.setValue(1,0,Symbol("m_t0"));
  mtx.setValue(2,0,Symbol("t1"));
  mtx.setValue(3,0,Symbol("m_t3"));
  mtx.setValue(2,1,Symbol("m_t2"));
  mtx.setValue(3,1,Symbol("t4"));
  mtx.setValue(3,2,Symbol("m_t5"));

  Symbol s = mtx.determinant();
  out << "Lib_symbolic :" 
      << s.value()
      << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool det_A_44(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);

  Lib::ADD<Symbol> mtx(4);
  //mtx.setIdentity();    
  // 0 t0 -t1  t3 -t6
  // 0  0  t2 -t4  t7
  // 0  0   0  t5 -t8
  // 0  0   0   0  t9
  // 0  0   0   0   0

  mtx.setValue(0,1,Symbol("t0"));
  mtx.setValue(0,2,Symbol("m_t1"));
  mtx.setValue(0,3,Symbol("t3"));
  mtx.setValue(0,4,Symbol("m_t6"));
  mtx.setValue(1,2,Symbol("t2"));
  mtx.setValue(1,3,Symbol("m_t4"));
  mtx.setValue(1,4,Symbol("t7"));
  mtx.setValue(2,3,Symbol("t5"));
  mtx.setValue(2,4,Symbol("m_t8"));
  mtx.setValue(3,4,Symbol("t9"));

  mtx.setValue(1,0,Symbol("m_t0"));
  mtx.setValue(2,0,Symbol("t1"));
  mtx.setValue(3,0,Symbol("m_t3"));
  mtx.setValue(4,0,Symbol("t6"));
  mtx.setValue(2,1,Symbol("m_t2"));
  mtx.setValue(3,1,Symbol("t4"));
  mtx.setValue(4,1,Symbol("m_t7"));
  mtx.setValue(3,2,Symbol("m_t5"));
  mtx.setValue(4,2,Symbol("t8"));
  mtx.setValue(4,3,Symbol("m_t9"));

  Symbol s = mtx.determinant();
  out << "Lib_symbolic :" 
      << s.value()
      << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
int main(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);
  out << "Lib_symbolic :" 
      << " 2." 
      << Lib::endl;

  det_id_44();
  det_A_33();
  det_A_44();

  return 0;
}
