
#include <Lib/Arguments.h>
#include <Lib/System.h>
#include <Lib/argZero.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Printer.h>
#include <Lib/fmanip.h>
#include <Lib/Debug.h>

#include <Lib/InternetSocket.h>
#include <Slash/Core/IFileGetter.h>

#include <OnX/Core/Main.h>

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgc<=0) return Lib::System::exitFailure();

 {bool is_Mac_app = false;
  if(!Lib::argZero::startMacApp(aArgv[0],"OnX",is_Mac_app)) return 1;}

  Lib::Printer printer;

 {Lib::Arguments args(aArgc,aArgv);

  int verboseLevel = 0;
 {std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose=="") verboseLevel = 1;
  }}

  Lib::InternetSocket* server = 
    new Lib::InternetSocket(printer,verboseLevel);
  //server->setEnv("ONXDISPLAY");
  if(!server->accept()) {
    delete server;
    return Lib::System::exitFailure();
  }

  // wait a "file protocol" :
  std::string s;
  if(!server->waitString(s)) {
    delete server;
    return Lib::System::exitFailure();
  }
  std::string ONX_PROTOCOL_FILE = "OnX::file::";
  if(s.substr(0,ONX_PROTOCOL_FILE.size())!=ONX_PROTOCOL_FILE) {
    Lib::Out out(printer);
    out << "OnX_server :"
        << " error in protocol " << Lib::sout(ONX_PROTOCOL_FILE) << "."
        << " Received " << Lib::sout(s) << "."
        << Lib::endl;
    delete server;
    return Lib::System::exitFailure();
  }
  std::string filename = 
    s.substr(ONX_PROTOCOL_FILE.size(),s.size()-ONX_PROTOCOL_FILE.size());
  if(verboseLevel) {
    Lib::Out out(printer);
    out << "OnX_server :"
        << " file name " << Lib::sout(filename) << "."
        << Lib::endl;
  }
  // wait content of file :
  std::vector<std::string> text;
  if(!server->waitText(text)) {
    delete server;
    return Lib::System::exitFailure();
  }

  Lib::fmanip::write(filename,text);
  args.add("-onx_file",filename);

  if(!server->sendString("hello from server")) {
    delete server;
    return Lib::System::exitFailure();
  }

  OnX::Main onxMain(true,true,args.tovector());
  if(!onxMain.isValid()) {
    delete server;
    return Lib::System::exitFailure();
  }
  onxMain.steer();

  delete server;}

  Lib::Debug::balance(printer);

  // Some libraries (for exa Coin, gtk) may have declared 
  // C functions with ::atexit. To ease debugging we call
  // explicitly ::exit here.
  Lib::System::exit();

  return Lib::System::exitSuccess();
}
