#
# Visualize a detector by using its representation
# done with the HEPVis SoDetectorExample nodekit.
#
# This script could be executed on an OnX Python callback ;
# for example on a MenuItem with :
#    <activate exec="Python">import OnX;OnX.x('OnX_exa_detector')</activate>
# or from the OnX UI Python prompt with :
#   Python> import OnX_exa_detector
# or from the python shell with :
#       OS> <setup OnX>
#       OS> <start Python>
#       >>> import OnX_exa_detector
#

import OnX

import CoinPython as Inventor

import HEPVis

onx_main = None
if OnX.session() == None:  # Start from scratch (for exa from the Python shell)
  import OnX
  onx_main = OnX.run(0,0) # Create the ONXFILE UI but do not ui.steer.
  ui = OnX.session().ui()
  ui.currentPage().currentRegion().clear()  # To clear the OnX banner.
 
ui = OnX.session().ui()
soPage = ui.currentPage()
soPage.title.setValue("Python / Polyhedron")

soRegion = soPage.currentRegion()
soRegion.color.setValue(Inventor.SbColor(0.8,0.8,0.8))
soSep = soRegion.cast_SoDisplayRegion().getStaticScene()

soScale = Inventor.SoScale()
soScale.scaleFactor.setValue(Inventor.SbVec3f(0.1,0.1,0.1))
soSep.addChild(soScale)

# SoPolyhedron : a trd :
soSep_trd = Inventor.SoSeparator()
soSep.addChild(soSep_trd)

soMaterial = Inventor.SoMaterial()
soMaterial.diffuseColor.setValue(1,0,0)
soSep_trd.addChild(soMaterial)

soLigthModel = Inventor.SoLightModel()
soLigthModel.model.setValue(Inventor.SoLightModel.BASE_COLOR)
soSep_trd.addChild(soLigthModel)

trd2 = HEPVis.SbPolyhedronTrd2(10,0,10,0,10)
so_trd2 = HEPVis.SoPolyhedron(trd2)
so_trd2.solid.setValue(Inventor.FALSE)
so_trd2.reducedWireFrame.setValue(Inventor.TRUE)
soSep_trd.addChild(so_trd2)

# SoPolyhedron : boolean operation :
soSep_bool_op = Inventor.SoSeparator()
soSep.addChild(soSep_bool_op)

soLigthModel = Inventor.SoLightModel()
soLigthModel.model.setValue(Inventor.SoLightModel.PHONG)
soSep_trd.addChild(soLigthModel)

box_1 = HEPVis.SbPolyhedronBox(20,10,10)
bool_op = HEPVis.SbPolyhedron(box_1);
box_2 = HEPVis.SbPolyhedronBox(30,5,5)
box_2.Transform(HEPVis.SbRotation(Inventor.SbVec3d(0,1,0),1.57),Inventor.SbVec3d(0,0,0))
bool_op = bool_op.subtract(box_2);
bool_op.Transform(HEPVis.SbRotation(),Inventor.SbVec3d(40,0,0))
so_bool_op = HEPVis.SoPolyhedron(bool_op)
#so_bool_op.solid.setValue(Inventor.FALSE)
#so_bool_op.reducedWireFrame.setValue(Inventor.TRUE)
so_bool_op.solid.setValue(Inventor.TRUE)
soSep_bool_op.addChild(so_bool_op)

del soScale
del soSep
del soRegion
del soPage

if onx_main != None:
  ui.show()
  ui.steer()
  del onx_main
  
