#
# Manipulate data (here some "OnX_Shape") by using its
# data accessor "type".
# All the code is in the OnX callbacks :
#   OnX/source/Callback/ShapeExa.cxx file.
# The "user data class", OnX::Shape, is manipulated
# by the below "data_xxx" system through its
# OnX::ShapeType accessor class.
#
# This script could be executed on an OnX Python callback ;
# for example on a MenuItem with :
#    <activate exec="Python">import OnX;OnX.x('OnX_exa_shapes')</activate>
# or from the OnX UI Python prompt with :
#   Python> import OnX_exa_shapes
# or from the python shell with :
#       OS> <setup OnX>
#       OS> <start Python>
#       >>> import OnX_exa_shapes
#

import OnX

import CoinPython as Inventor

import HEPVis

onx_main = None
if OnX.session() == None:  # Start from scratch (for exa from the Python shell)
  import OnX
  onx_main = OnX.run(0,0) # Create the ONXFILE UI but do not ui.steer.
  ui = OnX.session().ui()
  ui.currentPage().currentRegion().clear()  # To clear the OnX banner.
 
session = OnX.session()

ui = session.ui()
da = session.accessorManager() # data accessor.

ui.executeScript("DLD","OnX OnX_shape_initialize")

soPage = ui.currentPage()
soPage.title.setValue("OnX/Shapes")
soRegion = soPage.currentRegion()
soRegion.color.setValue(Inventor.SbColor(0.8,0.8,0.8))
del soRegion
del soPage

session.setParameter('modeling.modeling','solid')

session.setParameter('modeling.transparency','0.4')
session.setColor('red')
da.collect('OnX_Shape','type=="Cube"')
da.visualize()

session.setParameter('modeling.transparency','0')
session.setColor('yellow')
da.collect('OnX_Shape','type=="Hole"')
da.visualize()

session.setColor('blue')
session.setParameter('modeling.modeling','texts')
#da.collect('OnX_Shape','type=="Sphere" && size==1')
da.collect('OnX_Shape','type=="Sphere"')
da.visualize()

widget = ui.currentWidget()
if widget != None:
  soViewer = widget.cast_ISoViewer()
  if soViewer != None:
    soViewer.setFeedbackVisibility(1)
  
if onx_main != None:
  ui.show()
  ui.steer()
  del onx_main
