#ifndef OnX_Environment_h
#define OnX_Environment_h

#include <Slash/Core/IWriter.h>

#include <Lib/System.h>
#include <Lib/dirmanip.h>
#include <Lib/argZero.h>
#include <Lib/Out.h>

//////////////////////////////////////////////////////////////////////////////
inline bool check_ONXROOT(
 const std::string& aArg0
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
,bool aWarn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string fs = Lib::System::fileSeparator();
  std::string to = "Resources"+fs+"OnX";

  std::string dir_root;
  bool is_Mac_app;
  bool status = 
    Lib::argZero::checkEnv(aArg0,"ONXROOT",to,aPrinter,aWarn,
                        dir_root,is_Mac_app);
  if(status) {
    if(dir_root=="") {
      // Assume that ONXROOT is defined through the dev setup.
      return true;
    } else {
      // Activated through an app in a OSC binary kit.
      // Have to define ONXROOT.
    }
  } else {
    // May be activated from a user app that uses a OSC binary kit.
    // To detect this situation, we look for the AIDA_HOME_DIR 
    // that should be defined by the [aida-]setup of these kits.
    std::string AIDA_HOME_DIR;
    if(!Lib::System::getenv("AIDA_HOME_DIR",AIDA_HOME_DIR)) {
      Lib::Out out(aPrinter);
      out << "check_ONXROOT :"
          << " cant guess where OpenScientist had been installed."
          << Lib::endl;
      return false;
    } else {
      dir_root = AIDA_HOME_DIR;
    }
  }

  if(!Lib::System::putenv
        ("ONXROOT",dir_root+fs+"Resources"+fs+"OnX")) return false;

  if(!Lib::System::envPathAppend
    ("TTFPATH",dir_root+fs+"Resources"+fs+"HEPVis"+fs+"fonts")) return false;
  
#ifdef WIN32
 {std::string Resources_Python = dir_root+fs+"Resources"+fs+"Python";
  if(Lib::dirmanip::exists(Resources_Python)) {
    // Python is in the distribution :
    if(!Lib::System::putenv("ONX_HAS_PYTHON","yes")) return false;
    //Override.
    if(!Lib::System::putenv("PYTHONHOME",Resources_Python)) return false;
    if(!Lib::System::envPathAppend("PYTHONPATH",Resources_Python+fs+"DLLs"))
      return false;
  }}
#else
#ifdef __APPLE__
  Lib::System::putenv("ONX_HAS_PYTHON","yes");
#else  // Linux
 {std::string Resources_Python = dir_root+fs+"Resources"+fs+"Python";
  if(Lib::dirmanip::exists(Resources_Python)) {
    // Python is in the distribution :
    if(!Lib::System::putenv("ONX_HAS_PYTHON","yes")) return false;
    //Override.
    if(!Lib::System::putenv("PYTHONHOME",Resources_Python)) return false;
    //Lib::System::envPathAppend("PYTHONPATH",Resources_Python+fs+"DLLs");
  }}
#endif
#endif

  if(!Lib::System::envPathAppend("PYTHONPATH",
    dir_root+fs+"Resources"+fs+"OnX"+fs+"scripts"+fs+"Python")) return false;
  if(!Lib::System::envPathAppend("PYTHONPATH",
    dir_root+fs+"Resources"+fs+"CoinPython"+fs+"scripts")) return false;
  if(!Lib::System::envPathAppend("PYTHONPATH",
    dir_root+fs+"Resources"+fs+"HEPVis"+fs+"scripts"+fs+"Python")) 
      return false;
  if(!Lib::System::envPathAppend("PYTHONPATH",dir_root+fs+"bin")) return false;

#ifdef __APPLE__
  if(is_Mac_app) {
    if(!Lib::System::envPathAppend("DYLD_LIBRARY_PATH",dir_root+fs+"bin"))
      return false;
    if(!Lib::System::isenv("DISPLAY")) {
      if(!Lib::System::putenv("DISPLAY",":0.0")) return false;
    }
  }
#endif

  return true;
}

#endif
