// this :
#include "Accessors.h"

// Inventor :
#include <Inventor/nodes/SoShape.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodekits/SoBaseKit.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/manips/SoTransformManip.h>

// HEPVis :
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodes/SoHighlightMaterial.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/Data/IProcessor.h>
#include <Slash/Data/IAccessor.h>
#include <Slash/Data/IIterator.h>
#include <Slash/UI/IUI.h>

// Lib :
#include <Lib/Property.h>
#include <Lib/Value.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Manager.h>
#include <Lib/Debug.h>

// OnX :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/UI.h>

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::AccessorAccessor::AccessorAccessor(
 Slash::Core::ISession& aSession
)
:BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("Lib_Accessor")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("name",Lib::Property::STRING);
  //addProperty("address",Lib::Property::POINTER);
}
//////////////////////////////////////////////////////////////////////////////
OnX::AccessorAccessor::~AccessorAccessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::AccessorAccessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnX {
class AccessorAccessorIterator : public virtual Slash::Data::IIterator {
public: //Slash::Data::IIterator
  virtual Slash::Data::IAccessor::Data object() {
    if(fIterator==fList.end()) return 0;
    return *fIterator;
  }
  virtual void next() { ++fIterator;}
  virtual void* tag() { return 0;}
public:
  AccessorAccessorIterator(const std::vector<Slash::Data::IAccessor*>& aList)
  :fList(aList) {
    fIterator = fList.begin();
    Lib::Debug::increment("OnX::AccessorAccessorIterator");
  }
  virtual ~AccessorAccessorIterator() {
    Lib::Debug::decrement("OnX::AccessorAccessorIterator");
  }
private:
  const std::vector<Slash::Data::IAccessor*>& fList;
  std::vector<Slash::Data::IAccessor*>::const_iterator fIterator;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnX::AccessorAccessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(fSession,"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return 0;
  return new AccessorAccessorIterator(accessorManager->accessors());
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnX::AccessorAccessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IAccessor* obj = (Slash::Data::IAccessor*)aData;
  if(aName=="address") {
    return new Lib::Value((void*)obj);
  } else if(aName=="name") {
    return new Lib::Value(obj->name());
  } else {
    return new Lib::Value();
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::SceneGraphAccessor::SceneGraphAccessor(
 Slash::Core::ISession& aSession
)
:BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("SceneGraph")
,fSceneGraphs(0)
,fNumber(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("name",Lib::Property::STRING);
  addProperty("highlight",Lib::Property::BOOLEAN);
  addProperty("address",Lib::Property::POINTER);
}
//////////////////////////////////////////////////////////////////////////////
OnX::SceneGraphAccessor::~SceneGraphAccessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::SceneGraphAccessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnX {
class SceneGraphAccessorIterator : public virtual Slash::Data::IIterator {
public: //Slash::Data::IIterator
  virtual Slash::Data::IAccessor::Data object() {
    if(fIndex>=fNumber) return 0;
    return &(fList[fIndex]);
  }
  virtual void next() {fIndex++;}
  virtual void* tag() { return 0;}
public:
  SceneGraphAccessorIterator(OnX::SceneGraph* aList,unsigned int aNumber)
    :fIndex(0),fList(aList),fNumber(aNumber) {
    Lib::Debug::increment("OnX::SceneGraphAccessorIterator");
  }
  virtual ~SceneGraphAccessorIterator() {
    Lib::Debug::decrement("OnX::SceneGraphAccessorIterator");
  }
private:
  unsigned int fIndex;
  OnX::SceneGraph* fList;
  unsigned int fNumber;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnX::SceneGraphAccessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  getSceneGraphs();
  return new SceneGraphAccessorIterator(fSceneGraphs,fNumber);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnX::SceneGraphAccessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SceneGraph* obj = (SceneGraph*)aData;
  if(aName=="address") {
    return new Lib::Value((void*)obj);
  } else if(aName=="name") {
    SoSearchAction searchAction;
    searchAction.setFind(SoSearchAction::TYPE);
    searchAction.setType(SoShape::getClassTypeId());
    searchAction.apply(obj->fNode);
    SoPath* path = searchAction.getPath();
    std::string spath = path ? path->getTail()->getName().getString():"nil";
    //printf("debug : SceneGraphAccessor::getProp: %ld name:%s|\n",
    //   obj->fNode,spath.c_str());
    return new Lib::Value(spath);
  } else if(aName=="highlight") {
    SoSearchAction searchAction;
    searchAction.setFind(SoSearchAction::TYPE);
    searchAction.setType(SoHighlightMaterial::getClassTypeId());
    searchAction.apply(obj->fNode);
    SoPath* path = searchAction.getPath();
    bool value = false;
    if(path) {
      SoHighlightMaterial* material = (SoHighlightMaterial*)path->getTail();
      if(material->getSavedMaterial()) {
        value = true;
      }
    }
    return new Lib::Value(value);
  } else {
    return new Lib::Value();
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::SceneGraphAccessor::destroy(
 Slash::Data::IAccessor::Data aData
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SceneGraph* obj = (SceneGraph*)aData;
  //printf("debug : destroy %lx\n",aData);
  if(!obj->fParent) return;
  if(obj->fParent->isOfType(SoGroup::getClassTypeId())==FALSE) return;
  ((SoGroup*)obj->fParent)->removeChild(obj->fNode);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::SceneGraphAccessor::set(
 Slash::Data::IAccessor::Data aData
,const std::string& aWhat
,const std::string& aValue
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : set %lx %s %s\n",aData,aWhat.c_str(),aValue.c_str());

  SceneGraph* obj = (SceneGraph*)aData;
  if(!obj->fNode) return;

  // Look first for a name :
  SoSearchAction searchAction;
  searchAction.setFind(SoSearchAction::NAME);
  searchAction.setName(aWhat.c_str());
  searchAction.setInterest(SoSearchAction::ALL);
  searchAction.apply(obj->fNode);
  SoPathList pathList = searchAction.getPaths();
  int number = pathList.getLength();
  if(!number) {
    // Not found ; look for a type :
    SoType type = SoType::fromName(aWhat.c_str());
    if(type==SoType::badType()) {
      Lib::Out out(fSession.printer());
      out << "Unknown type " << aWhat << Lib::endl;
      return;
    }
    SoSearchAction searchAction;
    searchAction.setFind(SoSearchAction::TYPE);
    searchAction.setType(type);
    searchAction.setInterest(SoSearchAction::ALL);
    searchAction.apply(obj->fNode);
    pathList = searchAction.getPaths();
    number = pathList.getLength();
  }

  std::vector<std::string> words;
  Lib::smanip::words(aValue," ",words);

  if( (words.size()==2) && (words[0]=="manip") && (words[1]=="off") ) { 
    for(int index=0;index<number;index++) {
      SoFullPath* path = (SoFullPath*)pathList[index];
      SoNode* node = path->getTail();
      if(!node) continue;
      if(node->isOfType(SoTransformManip::getClassTypeId())) { 
        ((SoTransformManip*)node)->replaceManip(path,NULL);
        ((SoTransformManip*)node)->unref();
      }
    }
  } else if( (words.size()==2) && (words[0]=="manip") ) { 
    SoType type = SoType::fromName(words[1].c_str());
    if(type==SoType::badType()) {
      Lib::Out out(fSession.printer());
      out << "Unknown type \"" << words[1] << "\"" << Lib::endl;
      return;
    }
    if(!type.isDerivedFrom(SoTransformManip::getClassTypeId())) {
      Lib::Out out(fSession.printer());
      out << "Accessor \"" << words[1] 
          << "\" not a SoTransformManip." << Lib::endl;
      return;
    }
    for(int index=0;index<number;index++) {
      SoFullPath* path = (SoFullPath*)pathList[index];
      SoNode* node = path->getTail();
      if(!node) continue;
      if(node->isOfType(SoTransform::getClassTypeId())) { 
        SoTransformManip* manip = (SoTransformManip*)type.createInstance();
        if(manip) {
          manip->ref();
          manip->replaceNode(path);
        }
      }
    }
  } else if( (words.size()==2) && (words[0]=="solid") ) {

    bool value = false;
    if(!Lib::smanip::tobool(words[1],value)) { 
      Lib::Out out(fSession.printer());
      out << "\"" << words[1] << "\" not a boolean." << Lib::endl;
    }
    
    for(int index=0;index<number;index++) {
      SoFullPath* path = (SoFullPath*)pathList[index];
      SoNode* node = path->getTail();
      if(node) node->set(aValue.c_str());
    }

    // Have to toggle the light model :
   {SoSearchAction searchAction;
    searchAction.setFind(SoSearchAction::TYPE);
    searchAction.setType(SoLightModel::getClassTypeId());
    searchAction.setInterest(SoSearchAction::ALL);
    searchAction.apply(obj->fNode);
    pathList = searchAction.getPaths();
    number = pathList.getLength();
    if(number) {
      SoFullPath* path = (SoFullPath*)pathList[0];
      SoNode* node = path->getTail();
      if(node) {
        if(node->getRefCount()==1) {
          ((SoLightModel*)node)->model.setValue(value ?
                                                SoLightModel::PHONG:
                                                SoLightModel::BASE_COLOR);
        } else {
          //FIXME
          // Probably a node from a style cache. 
          // We can't edit it. Then we replace it with a new node.
        }
      }
    }}

    // Have to set the draw style :
   {SoSearchAction searchAction;
    searchAction.setFind(SoSearchAction::TYPE);
    searchAction.setType(SoDrawStyle::getClassTypeId());
    searchAction.setInterest(SoSearchAction::ALL);
    searchAction.apply(obj->fNode);
    pathList = searchAction.getPaths();
    number = pathList.getLength();
    if(number) {
      SoFullPath* path = (SoFullPath*)pathList[0];
      SoNode* node = path->getTail();
      if(node) ((SoDrawStyle*)node)->style.setValue(value ?
                                                    SoDrawStyle::FILLED:
                                                    SoDrawStyle::LINES);
    }}

  } else {
    for(int index=0;index<number;index++) {
      SoFullPath* path = (SoFullPath*)pathList[index];
      SoNode* node = path->getTail();
      if(node) node->set(aValue.c_str());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::SceneGraphAccessor::getSceneGraphs(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clear();

  Slash::UI::IUI* ui = Lib_findUI(fSession);
  if(!ui) return;
  SoPage* soPage = ui_SoPage(*ui);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  SoNode* node = soRegion->findNode("part","topSeparator");
  if(!node) {
    Lib::Out out(fSession.printer());
    out << "OnX::SceneGraphAccessor::getSceneGraphs :" 
        << " part not found." << Lib::endl;
    return;
  }
 
  // Need to search in nodekits (for exa SoDetectorTreeKit) :
  SbBool oldsearch = SoBaseKit::isSearchingChildren();
  SoBaseKit::setSearchingChildren(TRUE);
  SoSearchAction searchAction;
  searchAction.setFind(SoSearchAction::NAME);
  searchAction.setName("sceneGraph");
  //searchAction.setSearchingAll(TRUE); //to search all under an SoSwitch.
  searchAction.setInterest(SoSearchAction::ALL);
  searchAction.apply(node);
  SoBaseKit::setSearchingChildren(oldsearch);

  SoPathList& pathList = searchAction.getPaths();
  unsigned int number = pathList.getLength();
  if(!number) return;

  fSceneGraphs = new SceneGraph[number];
  if(!fSceneGraphs) return;

  for(unsigned int index=0;index<number;index++) {
    SoFullPath* path = (SoFullPath*)pathList[index];
    SceneGraph* sg = &(fSceneGraphs[index]);
    sg->fNode = path->getTail();
    int l = path->getLength();
    sg->fParent = l>=2 ? path->getNodeFromTail(1) : 0;
  }

  fNumber = number;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::SceneGraphAccessor::clear(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete [] fSceneGraphs;
  fSceneGraphs = 0;
  fNumber = 0;
}
