//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
//  This file does not need HEPVis and Inventor to compile.
//

// Helpers :
#include <OnX/Helpers/OnX.h>

//Slash:
#include <Slash/Core/IWriter.h>
#include <Slash/UI/ICallbackData.h>
#include <Slash/UI/ICyclic.h>
#include <Slash/UI/IStyleManager.h>

// Lib : 
#include <Lib/System.h>
#include <Lib/Printer.h>
#include <Lib/smanip.h>
#include <Lib/dirmanip.h>
#include <Lib/fmanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/sjust.h>
#include <Lib/Cloner.h>
#include <Lib/Arguments.h>

// OnX :
#include <OnX/Version.h>
#include <OnX/Core/Messages.h>

#include "Accessors.h"

#include <stdlib.h> //exit

#define MAXIMUM(a,b) ((a)>(b)?a:b)

//atexit
extern "C" {
  //static void OnX_atexit();
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  //if(session.verboseLevel()) ::atexit(OnX_atexit);

  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
  if(accessorManager) {
    if(!accessorManager->findAccessor("Lib_Accessor"))
      accessorManager->addAccessor(new OnX::AccessorAccessor(session));
    if(!accessorManager->findAccessor("SceneGraph"))
      accessorManager->addAccessor(new OnX::SceneGraphAccessor(session));
  }


  //  The defaults in OnX::Session constructor are such that
  // session.printer is enabled (on the UI) and
  // std[out,err],std::[cout,cerr] are not redirected 
  // to session.printer (then they still write on terminal).
  //  Then :
  // - session.output=UI redirect all on UI.
  // - session.output=terminal redirect all on the terminal.
  // - session.output=defout redirect only session.printer to UI
  //   which is then the default of OnX::Session constructor.  

 {std::string svalue;
  if(session.parameterValue("session.output",svalue)) {
    if( (svalue=="UI") && aUI.isComponent("consoleText")) {
      session.redirectOutput(true);
    } else if(svalue=="terminal") {
      session.printer().disable();
    } else if(svalue=="defout") {
      //session.printer().disable();
    }
  }}

}
//////////////////////////////////////////////////////////////////////////////
/// sys //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void sys_copy(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  sys_copy(aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void sys_remove(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=1) return;
  sys_remove(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void sys_rm(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=1) return;
  sys_rm(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void sys_pwd(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aUI.session().printer());
  std::string s;
  if(!Lib::dirmanip::directory(s)) {
    out << "sys_pwd :" 
        << "Can't get current directory."
        << Lib::endl;
    return;
  }
  out << s << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void sys_env(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=1) return;
  Lib::Out out(aUI.session().printer());
  std::string s;
  if(!Lib::System::getenv(aArgs[0],s)) {
    out << "sys_env :" 
        << "Environment variable " << aArgs[0] << " not defined."
        << Lib::endl;
    return;
  }
  out << s << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
/// session //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void session_exit(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // The below exit may call some atexit function (for exa with gtk)
  // that may do some print. We have to switch off OnX output.
  aUI.session().redirectOutput(false);

  int code = 0;
  if(aArgs.size()==1) {
    if(!Lib::smanip::toint(aArgs[0],code)) code = 0;
  }
  ::exit(code);
}
//////////////////////////////////////////////////////////////////////////////
void session_setParameter(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] what
// args[1,n-1] : partial value
//  value is built with : args[1] + " " + args[2] + ..... + args[n-1]
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  if(aArgs.size()==1) {
    aUI.session().setParameter(aArgs[0],"");
  } else {
    std::string value = ui_cbk(aUI,aArgs[1]);
    for(unsigned int index=2;index<aArgs.size();index++) 
      value += " " + ui_cbk(aUI,aArgs[index]);
    aUI.session().setParameter(aArgs[0],value);
  }
}
//////////////////////////////////////////////////////////////////////////////
void session_redirectOutput(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  bool value;
  if(!Lib::smanip::tobool(aArgs[0],value)) return;
  aUI.session().redirectOutput(value);
}
//////////////////////////////////////////////////////////////////////////////
void session_toggle_verbose(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  session.setVerboseLevel(session.verboseLevel()==0?1:0);
}
//////////////////////////////////////////////////////////////////////////////
void session_toggle_helpOutput(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  std::string svalue;
  if(!session.parameterValue("session.helpOnDialog",svalue)) return;
  session.setParameter("session.helpOnDialog",svalue=="yes"?"no":"yes");
}
//////////////////////////////////////////////////////////////////////////////
void session_load(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] library
// args[1] package
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  aUI.session().load(aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void session_load_styles(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] style file
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=1) return;
  Slash::UI::IStyleManager* styleManager = 
    Lib_findManager(aUI.session(),"StyleManager",Slash::UI::IStyleManager);
  if(!styleManager) return;
  styleManager->loadFile(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void session_modeling_none(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  session_modeling_none(aUI.session());
}
//////////////////////////////////////////////////////////////////////////////
void session_dump_parameters(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aUI.session().printer());
  std::vector<std::string> keys = aUI.session().availableParameters();
  unsigned int l = 0;
  for(unsigned int index=0;index<keys.size();index++) {
    const std::string& key = keys[index];
    l = MAXIMUM(l,key.size());
  }
  for(unsigned int index=0;index<keys.size();index++) {
    const std::string& key = keys[index];
    std::string value;
    aUI.session().parameterValue(key,value);
    out << Lib::sjust(key,l) << " = " << value << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void session_flush(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.session().flush();
}
//////////////////////////////////////////////////////////////////////////////
/// ui ///////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void ui_exit(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int exitCode = 0;
  if(aArgs.size()==1) {
    int l;
    if(Lib::smanip::toint(aArgs[0],l)) {
      exitCode = l;
    } else {
      aUI.echo("ui_exit : "+aArgs[0]+" not a number.");
    }
  }
  aUI.exit(exitCode);
}
//////////////////////////////////////////////////////////////////////////////
void ui_break_modal(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.exit(ONX_BREAK_MODAL);
}
//////////////////////////////////////////////////////////////////////////////
void ui_echo(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) {
    aUI.echo("");
  } else {
    std::string line = ui_cbk(aUI,aArgs[0]);
    for(unsigned int index=1;index<aArgs.size();index++) {
      line += " " + ui_cbk(aUI,aArgs[index]);
    }
    aUI.echo(line);
  }
}
//////////////////////////////////////////////////////////////////////////////
void ui_echo_file(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] : file name.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  ui_echo_file(aUI,aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_warn(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string line;
  for(unsigned int index=0;index<aArgs.size();index++) {
    if(index) line += " ";
    line += aArgs[index];
  }
  aUI.warn(line);
}
//////////////////////////////////////////////////////////////////////////////
void ui_ask(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string line;
  for(unsigned int index=0;index<aArgs.size();index++) {
    if(index) line += " ";
    line += aArgs[index];
  }
  aUI.echo(aUI.ask(line));
}
//////////////////////////////////////////////////////////////////////////////
void ui_setParameter(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] what
// args[1,n-1] : partial value
//  value is built with : args[1] + " " + args[2] + ..... + args[n-1]
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  std::string value;
  if(aArgs.size()!=1) {
    value = ui_cbk(aUI,aArgs[1]);
    for(unsigned int index=2;index<aArgs.size();index++) 
      value += " " + ui_cbk(aUI,aArgs[index]);
  }
  aUI.setParameter(aArgs[0],value);
}
//////////////////////////////////////////////////////////////////////////////
void ui_parameterValue(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] widget.parameter
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  std::string value;
  if(!aUI.parameterValue(aArgs[0],value)) return;
  aUI.echo(value);
}
//////////////////////////////////////////////////////////////////////////////
void ui_setParameterWith(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  std::string value;
  if(!aUI.parameterValue(aArgs[1],value)) return;
  aUI.setParameter(aArgs[0],value);
}
//////////////////////////////////////////////////////////////////////////////
void ui_executeFile(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] interpreter
// aArgs[1] file
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  ui_execute_file(aUI,aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_show_dialog(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  ui_show_dialog(aUI,aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_hide(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  aUI.hide(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_show(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  aUI.show(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_help_text(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs : text to print in the help dialog (or terminal).
//         It is a list of strings separated by \n.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  std::string line = ui_cbk(aUI,aArgs[0]);
  for(unsigned int index=1;index<aArgs.size();index++) {
    line += " " + ui_cbk(aUI,aArgs[index]);
  }
  Lib::smanip::replace(line,"\\n","\n");
  ui_help_text(aUI,line);
}
//////////////////////////////////////////////////////////////////////////////
void ui_help_file(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] : file name.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  ui_help_file(aUI,aArgs[0]);
}
/////////////////////////////////////////////////////////////////////////////
void ui_showFileChooser(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] lib
// args[1] callback
// args[2] dirmask
// args[3] default directory
// args[...] passed to the ok callback.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<4) return;

  std::string ok_args;
  for(unsigned int index=4;index<aArgs.size();index++) {
    if(index!=4) ok_args += " ";
    ok_args += ui_cbk(aUI,aArgs[index]);
  }


  aUI.setCallback("fileChooser","ok","DLD",aArgs[0]+" "+aArgs[1]+" "+ok_args);
  aUI.setCallback("fileChooser","cancel",
                  "DLD","OnX ui_hide fileChooser");
  aUI.setParameter("fileChooser.dirMask",aArgs[2]);
  aUI.setParameter("fileChooser.directory",aArgs[3]);
  aUI.setParameter("fileChooser.mode","open");
  aUI.show("fileChooser");
}
//////////////////////////////////////////////////////////////////////////////
void ui_showFileSaver(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] lib
// args[1] callback
// args[2] dirmask
// args[3] default directory
// args[...] passed to the ok callback.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<4) return;

  std::string ok_args;
  for(unsigned int index=4;index<aArgs.size();index++) {
    if(index!=4) ok_args += " ";
    ok_args += ui_cbk(aUI,aArgs[index]);
  }


  aUI.setCallback("fileChooser","ok","DLD",aArgs[0]+" "+aArgs[1]+" "+ok_args);
  aUI.setCallback("fileChooser","cancel",
                  "DLD","OnX ui_hide fileChooser");
  aUI.setParameter("fileChooser.dirMask",aArgs[2]);
  aUI.setParameter("fileChooser.directory",aArgs[3]);
  aUI.setParameter("fileChooser.mode","save");
  aUI.show("fileChooser");
}
//////////////////////////////////////////////////////////////////////////////
void ui_showColorChooser(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
// args[0] lib
// args[1] callback
// args[..] passed to the ok callback.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<2) return;

  std::string ok_args;
  for(unsigned int index=2;index<aArgs.size();index++) {
    if(index!=2) ok_args += " ";
    ok_args += ui_cbk(aUI,aArgs[index]);
  }

  aUI.setCallback("colorChooser","ok","DLD",aArgs[0]+" "+aArgs[1]+" "+ok_args);
  aUI.show("colorChooser");
}
//////////////////////////////////////////////////////////////////////////////
void ui_setCallback(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] widget
// args[1] callback
// args[2] interpreter
// args[3,n-1] : script
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<2) return;
  std::string script;
  for(unsigned int index=3;index<aArgs.size();index++) {
    if(index!=3) script += " ";
    script += ui_cbk(aUI,aArgs[index]);
  }
  aUI.setCallback(ui_cbk(aUI,aArgs[0]),
                  ui_cbk(aUI,aArgs[1]),
                  ui_cbk(aUI,aArgs[2]),
                  script);
}
//////////////////////////////////////////////////////////////////////////////
void ui_executeCallbacks(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  aUI.executeCallbacks(aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_executeScript(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()<=1) return;
  std::string value = aArgs[1];
  for(unsigned int index=2;index<aArgs.size();index++) 
    value += " " + aArgs[index];
  aUI.executeScript(aArgs[0],value);
}
//////////////////////////////////////////////////////////////////////////////
void ui_destroyComponent(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  aUI.destroyComponent(aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_save(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.save();
}
//////////////////////////////////////////////////////////////////////////////
void ui_add_prompt(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
// Add a prompt in the consolePrompt list.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  ui_add_prompt(aUI,aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void ui_setCurrentWidget(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  std::string name = aArgs[0];
  Slash::UI::IWidget* widget = aUI.findWidget(name);
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "ui_setCurrentWidget :" 
        << "Can't find widget " << Lib::sout(name)
        << Lib::endl;
    return;
  }
  aUI.setCurrentWidget(widget);
}
//////////////////////////////////////////////////////////////////////////////
/// data /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void data_collect(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()==1) {
    data_collect(aUI.session(),aArgs[0]);
  } else if(aArgs.size()==2) {
    data_collect(aUI.session(),aArgs[0],aArgs[1]);
  }
}
//////////////////////////////////////////////////////////////////////////////
void data_visualize(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()==1) {
    data_visualize(aUI.session(),aArgs[0]);
  } else {
    data_visualize(aUI.session());
  }
}
//////////////////////////////////////////////////////////////////////////////
void data_number(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  data_number(aUI.session());
}
//////////////////////////////////////////////////////////////////////////////
void data_dump(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  std::string svalue;
  session.parameterValue("AccessorManager.dumpMode",svalue);
  data_dump(session,svalue);
}
//////////////////////////////////////////////////////////////////////////////
/// Widgets //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void console_execute(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sexec;
  if(!aUI.parameterValue("consolePrompt.value",sexec)) return;
  aUI.executeScript(sexec,aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void colorChooser_ok(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  session_setColor(aUI.session(),aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void colorChooser_show(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.setCallback("colorChooser","ok","DLD","OnX colorChooser_ok");
  aUI.show("colorChooser");
}
//////////////////////////////////////////////////////////////////////////////
void tree_set_with_file(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  tree_set_with_file(aUI,aArgs[0]+".items",aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void tree_set_managers(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  tree_set_with_strings(aUI,"mainTree.items",aUI.session().managerNames());
}
//////////////////////////////////////////////////////////////////////////////
void tree_set_styles(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IStyleManager* styleManager = 
    Lib_findManager(aUI.session(),"StyleManager",Slash::UI::IStyleManager);
  if(!styleManager) return;
  tree_set_with_strings(aUI,"mainTree.items",styleManager->styles());
}
//////////////////////////////////////////////////////////////////////////////
/// OnX/scripts/OnX/OnX.onx //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_mainTree_select(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path = aUI.callbackValue();
  std::vector<std::string> lines;
  Lib::smanip::lines(path,lines);
  std::string item = lines.size() ? lines[lines.size()-1] : "";
  aUI.echo("item :"+item+"\npath :\n"+path);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_OptionMenu_change(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string item = aUI.callbackValue();
  if(item=="Load a tree from file") {
    tree_set_with_file(aUI,"mainTree.items","$ONXROOT/scripts/OnX/Tree.xml");
  } else if(item=="Managers") {
    tree_set_with_strings(aUI,"mainTree.items",aUI.session().managerNames());
  } else if(item=="Styles") {
    Slash::UI::IStyleManager* styleManager = 
      Lib_findManager(aUI.session(),"StyleManager",Slash::UI::IStyleManager);
    if(!styleManager) return;
    tree_set_with_strings(aUI,"mainTree.items",styleManager->styles());
  } else if(item=="Echo selection") {
    std::string selection;
    if(!aUI.parameterValue("mainTree.selection",selection)) return;
    aUI.echo(selection);
  } else if(item=="Test open") {
    aUI.executeScript("DLD","OnX OnX_TestTree_initialize");
  }
}
//////////////////////////////////////////////////////////////////////////////
/// OnX/scripts/OnX/File.onx /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_File_import(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  aUI.hide("fileChooser");
  ui_execute_file(aUI,"Python",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_tcl(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  ui_execute_file(aUI,"Tcl",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_CINT(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  ui_execute_file(aUI,"CINT",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_KUIP(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  ui_execute_file(aUI,"KUIP",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_ps(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
// To visualize PS files.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string cmd;
  if(Lib::System::getenv("ONXPSVIEW",cmd)) {
    Lib::System::execute(cmd+" "+aUI.callbackValue());
  } else {
    if(Lib::System::isTypeOf("darwin")) {
      Lib::System::execute("open "+aUI.callbackValue());
    } else if(Lib::System::isKindOf("Windows")) {
      Lib::System::execute("gsview32.exe "+aUI.callbackValue());
    } else {
      Lib::System::execute("ghostview "+aUI.callbackValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_gif(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
// To visualize GIF files.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string cmd;
  if(Lib::System::getenv("ONXGIFVIEW",cmd)) {
    Lib::System::execute(cmd+" "+aUI.callbackValue());
  } else {
    if(Lib::System::isTypeOf("darwin")) {
      Lib::System::execute("open "+aUI.callbackValue());
    } else if(Lib::System::isKindOf("Windows")) {
      Lib::System::execute("mspaint "+aUI.callbackValue());
    } else {
      Lib::System::execute("xv "+aUI.callbackValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_jpeg(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
// To visualize JPEG files (.jpg).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string cmd;
  if(Lib::System::getenv("ONXJPEGVIEW",cmd)) {
    Lib::System::execute(cmd+" "+aUI.callbackValue());
  } else {
    if(Lib::System::isTypeOf("darwin")) {
      Lib::System::execute("open "+aUI.callbackValue());
    } else if(Lib::System::isKindOf("Windows")) {
      Lib::System::execute("mspaint "+aUI.callbackValue());
    } else {
      Lib::System::execute("xv "+aUI.callbackValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_pdf(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
// To visualize PDF files (.pdf(.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string cmd;
  if(Lib::System::getenv("ONXPDFVIEW",cmd)) {
    Lib::System::execute(cmd+" "+aUI.callbackValue());
  } else {
    if(Lib::System::isTypeOf("darwin")) {
      Lib::System::execute("open "+aUI.callbackValue());
    } else if(Lib::System::isKindOf("Windows")) {
      Lib::System::execute("AcroRd32.exe "+aUI.callbackValue());
    } else {
      Lib::System::execute("xpdf "+aUI.callbackValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_iv(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
// To visualize Inventor files (.iv).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string cmd;
  if(Lib::System::getenv("ONXIVVIEW",cmd)) {
    Lib::System::execute(cmd+" "+aUI.callbackValue());
  } else {
    if(Lib::System::isKindOf("Windows")) {
      Lib::System::execute("ivview.exe "+aUI.callbackValue());
    } else {
      Lib::System::execute("ivview "+aUI.callbackValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_File_style(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Load a style file.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  Slash::UI::IStyleManager* styleManager = 
    Lib_findManager(aUI.session(),"StyleManager",Slash::UI::IStyleManager);
  if(!styleManager) return;
  styleManager->loadFile(aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
/// OnX/scripts/OnX/Pages.onx ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Pages is a TabStack that contains "viewers".
// A "viewer" is not necessary an Inventro one (PageViewer).
// It could be an OpenGL one )OpenGLArea).
//////////////////////////////////////////////////////////////////////////////
void OnX_Tab_select(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.setCurrentWidget(aUI.findWidget(aUI.callbackValue()));
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Pages_HelpText(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Display the Inventor viewer (PageViewer) help.

  std::vector<std::string> text;
  
  if(!Lib::fmanip::read("$ONXROOT/scripts/Help/Viewer.help",text)) return;

  aUI.setParameter("HelpText.value",Lib::smanip::tostring(text,"\n"));
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_NextDialog_Cancel(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.session().setParameter("ui.exitSecondaryLoop","yes");
  aUI.exit();
  aUI.hide("OnX_NextDialog_dialog");
}

//void OnX_widget_gl2ps(Slash::UI::IUI&,const std::vector<std::string>&);

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPrint_write(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  if(!aArgs.size()) return;
  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_InputPrint_write : "
        << " no current widget."
        << Lib::endl;
    return;
  }
  if(aArgs[0]=="OpenGLArea_PS") {
    aUI.executeScript
      ("DLD","OnX OnX_widget_gl2ps "+widget->name()+" "+aUI.callbackValue());
  } else if(aArgs[0]=="JPEG") {
    aUI.load("main",
      Lib::fmanip::name("$ONXROOT/scripts/OnX/JPEG_Dialog.onx"));
    aUI.setParameter("OnX_JPEG_Dialog_file.label",
      aUI.callbackValue());
    aUI.show("OnX_JPEG_Dialog");
  } else if(aArgs[0]=="GL2PS") {
    aUI.load("main",
      Lib::fmanip::name("$ONXROOT/scripts/OnX/Input_gl2ps.onx"));
    aUI.setParameter("OnX_Input_gl2ps_input_GL2PS_FILE.value",
      aUI.callbackValue());
    aUI.show("OnX_Input_gl2ps_dialog");
  } else if( (aArgs[0]=="HIV") || (aArgs[0]=="IV") ||
             (aArgs[0]=="WRL") || (aArgs[0]=="POV") ){
    aUI.load("main",
      Lib::fmanip::name("$ONXROOT/scripts/OnX/InputWrite.onx"));
    aUI.setParameter("OnX_InputWrite_input_file.value",
      aUI.callbackValue());
    aUI.show("OnX_InputWrite_dialog");
  } else {
    widget->write(aUI.callbackValue(),aArgs[0]);
  }
}

//////////////////////////////////////////////////////////////////////////////
void OnX_JPEG_Dialog_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string file;
  if(!aUI.parameterValue("OnX_JPEG_Dialog_file.label",file)) return;
  std::string quality;
  if(!aUI.parameterValue("OnX_JPEG_Dialog_quality.value",quality)) return;
  aUI.hide("OnX_JPEG_Dialog");

  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_JPEG_Dialog_ok : "
        << " no current widget."
        << Lib::endl;
    return;
  }

  widget->write(file,"JPEG",quality);
}
/////////////////////////////////////////////////////////////////////////////
void OnX_Input_gl2ps_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string file;
  if(!aUI.parameterValue("OnX_Input_gl2ps_input_GL2PS_FILE.value",file)) return;

  std::string options = "NONE";

  bool GL2PS_DRAW_BACKGROUND;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_DRAW_BACKGROUND.value",GL2PS_DRAW_BACKGROUND)) return;
  if(GL2PS_DRAW_BACKGROUND) options += ";DRAW_BACKGROUND";

  bool GL2PS_SIMPLE_LINE_OFFSET;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_SIMPLE_LINE_OFFSET.value",GL2PS_SIMPLE_LINE_OFFSET)) return;
  if(GL2PS_SIMPLE_LINE_OFFSET) options += ";SIMPLE_LINE_OFFSET";

  bool GL2PS_SILENT;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_SILENT.value",GL2PS_SILENT)) return;
  if(GL2PS_SILENT) options += ";SILENT";

  bool GL2PS_BEST_ROOT;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_BEST_ROOT.value",GL2PS_BEST_ROOT)) return;
  if(GL2PS_BEST_ROOT) options += ";BEST_ROOT";

  bool GL2PS_OCCLUSION_CULL;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_OCCLUSION_CULL.value",GL2PS_OCCLUSION_CULL)) return;
  if(GL2PS_OCCLUSION_CULL) options += ";OCCLUSION_CULL";

  bool GL2PS_NO_TEXT;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_NO_TEXT.value",GL2PS_NO_TEXT)) return;
  if(GL2PS_NO_TEXT) options += ";NO_TEXT";

  bool GL2PS_LANDSCAPE;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_LANDSCAPE.value",GL2PS_LANDSCAPE)) return;
  if(GL2PS_LANDSCAPE) options += ";LANDSCAPE";

  bool GL2PS_NO_PS3_SHADING;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_NO_PS3_SHADING.value",GL2PS_NO_PS3_SHADING)) return;
  if(GL2PS_NO_PS3_SHADING) options += ";NO_PS3_SHADING";

  bool GL2PS_NO_PIXMAP;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_NO_PIXMAP.value",GL2PS_NO_PIXMAP)) return;
  if(GL2PS_NO_PIXMAP) options += ";NO_PIXMAP";

  bool GL2PS_USE_CURRENT_VIEWPORT;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_USE_CURRENT_VIEWPORT.value",GL2PS_USE_CURRENT_VIEWPORT)) return;
  if(GL2PS_USE_CURRENT_VIEWPORT) options += ";USE_CURRENT_VIEWPORT";

  bool GL2PS_COMPRESS;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_COMPRESS.value",GL2PS_COMPRESS)) return;
  if(GL2PS_COMPRESS) options += ";COMPRESS";

  bool GL2PS_NO_BLENDING;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_NO_BLENDING.value",GL2PS_NO_BLENDING)) return;
  if(GL2PS_NO_BLENDING) options += ";NO_BLENDING";

  bool GL2PS_TIGHT_BOUNDING_BOX;
  if(!ui_parameterValue(aUI,"OnX_Input_gl2ps_input_GL2PS_TIGHT_BOUNDING_BOX.value",GL2PS_TIGHT_BOUNDING_BOX)) return;
  if(GL2PS_TIGHT_BOUNDING_BOX) options += ";TIGHT_BOUNDING_BOX";

  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_Input_gl2ps_ok : "
        << " no current widget."
        << Lib::endl;
    return;
  }

  //printf("debug : file : \"%s\"\n",file.c_str());
  //printf("debug : options : \"%s\"\n",options.c_str());

  widget->write(file,"GL2PS",options);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPrint_system(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] command
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  if(!aArgs.size()) return;
  Lib::System::execute(aArgs[0]+" "+aUI.callbackValue());
}
/////////////////////////////////////////////////////////////////////////////
void OnX_InputPrint_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string action;
  if(!aUI.parameterValue("OnX_InputPrint_input_action.value",action)) return;

  aUI.setCallback("fileChooser","ok",
                  "DLD","OnX OnX_InputPrint_write "+action);
  aUI.setCallback("fileChooser","cancel",
                  "DLD","OnX ui_hide fileChooser");
  aUI.setParameter("fileChooser.mode","save");

  if(action=="PS") {
    aUI.setParameter("fileChooser.dirMask","*.ps");
  } else if(action=="PSVec") {
    aUI.setParameter("fileChooser.dirMask","*.ps");
  } else if(action=="GL2PS") {
    aUI.setParameter("fileChooser.dirMask","*.ps");
  } else if(action=="GL2PDF") {
    aUI.setParameter("fileChooser.dirMask","*.pdf");
  } else if(action=="GL2SVG") {
    aUI.setParameter("fileChooser.dirMask","*.svg");
  } else if(action=="GL2TEX") {
    aUI.setParameter("fileChooser.dirMask","*.tex");
  } else if(action=="GL2PGF") {
    aUI.setParameter("fileChooser.dirMask","*.pgf");
  } else if(action=="GIF") {
    aUI.setParameter("fileChooser.dirMask","*.gif");
  } else if(action=="JPEG") {
    aUI.setParameter("fileChooser.dirMask","*.jpg");
  } else if(action=="OpenGLArea_PS") {
    aUI.setParameter("fileChooser.dirMask","*.ps");

  } else if(action=="HIV") {
    aUI.setParameter("fileChooser.dirMask","*.hiv");
  } else if(action=="IV") {
    aUI.setParameter("fileChooser.dirMask","*.iv");
  } else if(action=="WRL") {
    aUI.setParameter("fileChooser.dirMask","*.wrl");
  } else if(action=="POV") {
    aUI.setParameter("fileChooser.dirMask","*.pov");
  }

  aUI.show("fileChooser");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void OnX_InputAccessorManager_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(aUI.session(),"AccessorManager",Slash::Data::IProcessor);
  if(!accessorManager) return;
  aUI.setParameter("OnX_InputAccessorManager_input_dataType.items",
    Lib::smanip::tostring(accessorManager->accessorNames(),"\n"));
  aUI.setParameter("OnX_InputAccessorManager_input_action.items",
    Lib::smanip::tostring(accessorManager->actionNames(),"\n"));
}
/////////////////////////////////////////////////////////////////////////////
void OnX_InputAccessorManager_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// input fields :
//  string dataType
//  string dataFilter
//  string action
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string type;
  if(!aUI.parameterValue("OnX_InputAccessorManager_input_dataType.value",type)) return;
  std::string filter;
  if(!aUI.parameterValue("OnX_InputAccessorManager_input_dataFilter.value",filter)) return;
  std::string action;
  if(!aUI.parameterValue("OnX_InputAccessorManager_input_action.value",action)) return;
  std::string args;
  if(!aUI.parameterValue("OnX_InputAccessorManager_input_args.value",args)) return;

  Slash::Core::ISession& session = aUI.session();

  if(action=="dump") {
    data_collect(session,type,filter);
    std::string svalue;
    session.parameterValue("AccessorManager.dumpMode",svalue);
    data_dump(session,svalue);
  } else if(action=="erase") {
    data_collect(session,"SceneGraph","name==\""+type+"/*\"");
    data_destroy(session);
  } else { //generic
    data_collect(session,type,filter);
    Slash::Data::IProcessor* accessorManager = 
      Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
    if(!accessorManager) return;
    std::vector<std::string> words;
    Lib::smanip::words(args," ",words);
    accessorManager->execute(action,words);
  }
}
/////////////////////////////////////////////////////////////////////////////
void OnX_InputParameter_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// input fields :
//  string color
//  string modeling
//  string transparency
//  string action
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string action;
  if(!aUI.parameterValue("OnX_InputParameter_input_action.value",action)) 
    return;

  std::string v;
  if(action == "modeling.color") {
    if(!aUI.parameterValue("OnX_InputParameter_input_color.value",v)) return;
  } else if(action == "modeling.modeling") {
    if(!aUI.parameterValue("OnX_InputParameter_input_modeling.value",v)) 
      return;
  } else if(action == "modeling.transparency") {
    if(!aUI.parameterValue("OnX_InputParameter_input_transparency.value",v)) 
      return;
  } else if(action == "modeling.linePattern") {
    if(!aUI.parameterValue("OnX_InputParameter_input_linePattern.value",v)) 
      return;
  } else if(action == "modeling.lineWidth") {
    if(!aUI.parameterValue("OnX_InputParameter_input_lineWidth.value",v)) 
      return;
  } else {
    aUI.echo("Unknown action.");
    return;
  }

  std::vector<std::string> args;
  args.push_back(action);
  args.push_back(v);
  session_setParameter(aUI,args);

}
/////////////////////////////////////////////////////////////////////////////
void OnX_InputSceneGraph_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// input fields :
//  string what
//  string color
//  string modeling
//  string transparency
//  string manip
//  string action
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  std::string what;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_what.value",what)) return;

  std::string color;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_color.value",color)) return;
  std::string modeling;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_modeling.value",modeling)) return;

  std::string transparency;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_transparency.value",transparency)) return;
  double d;
  if(!Lib::smanip::todouble(transparency,d)) d = 0;

  std::string manip;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_manip.value",manip)) return;

  std::string action;
  if(!aUI.parameterValue("OnX_InputSceneGraph_input_action.value",action)) return;

  if(what=="Highlighted") {
    data_collect(session,"SceneGraph","highlight==true");
  } else if(what=="Not highlighted") {
    data_collect(session,"SceneGraph","highlight==false");
  }

  if(action == "Set color") {
    std::string value;
    Lib::smanip::torgbs(color,value);
    //FIXME : does not work if working on an highlight object !
    data_set(session,"SoMaterial","diffuseColor " + value);
  } else if(action == "Set transparency") {
    //FIXME : does not work if working on an highlight object !
    data_set(session,"SoMaterial","transparency " + transparency);
  } else if(action == "Set manip") {
    data_set(session,"SoTransform","manip " + manip);
  } else if(action == "Set modeling") {
    if(modeling=="Solid") {
      data_set(session,"SoPolyhedron","solid TRUE");
    } else if(modeling=="Wire frame") {
      data_set(session,"SoPolyhedron","solid FALSE");
      data_set(session,"SoPolyhedron","reducedWireFrame FALSE");
    } else if(modeling=="Reduced wire frame") {
      data_set(session,"SoPolyhedron","solid FALSE");
      data_set(session,"SoPolyhedron","reducedWireFrame TRUE");
    }
  } else if(action == "Erase") {
    data_destroy(session);
  }
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void OnX_InputShape_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// input fields :
//  string shape
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  std::string shape;
  if(!aUI.parameterValue("OnX_InputShape_input_shape.value",shape)) return;

  data_collect(session,"OnX_Shape","type==\""+shape+"\"");
  data_visualize(session);

}
/////////////////////////////////////////////////////////////////////////////
void OnX_MenubarEditor_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.setParameter("OnX_MenubarEditor_option_menubar.items",
     Lib::smanip::tostring(aUI.findChildren("menuBar",
                                      "labelFromClass",
                                      "CascadeButton"),"\n"));
}
/////////////////////////////////////////////////////////////////////////////
void OnX_MenubarEditor_show_menu(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string menuLabel;
  if(!aUI.parameterValue("OnX_MenubarEditor_option_menubar.value",menuLabel)) return;

  std::vector<std::string> nameFromLabel = 
    aUI.findChildren("menuBar","nameFromLabel",menuLabel);

  if(nameFromLabel.size() == 1) {
    std::string name = nameFromLabel[0];
    std::vector<std::string> items = 
      aUI.findChildren(name,"labelFromClass","MenuItem");
    aUI.setParameter("OnX_MenubarEditor_option_menu.items",
                     Lib::smanip::tostring(items,"\n"));
  }
}
/////////////////////////////////////////////////////////////////////////////
void OnX_MenubarEditor_show_script(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string itemLabel;
  if(!aUI.parameterValue("OnX_MenubarEditor_option_menu.value",itemLabel)) return;

  std::vector<std::string> nameFromLabel = 
    aUI.findChildren("menuBar","nameFromLabel",itemLabel);

  if(nameFromLabel.size() == 1) {
    std::string name = nameFromLabel[0];
    std::string value;
    if(!aUI.parameterValue(name+".activate",value)) return;
    aUI.echo(value);
  }
}
//////////////////////////////////////////////////////////////////////////////
/// OnX/scripts/OnX/NewPackage.onx ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_NewPackage_create(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name;
  if(!aUI.parameterValue("entry.value",name)) return;

  std::string which;
  if(!aUI.parameterValue("which.value",which)) return;

  if(which=="Template One") {
    std::string dname = 
      Lib::fmanip::name("$ONXROOT/templates/OnXTemplateOne");
    bool status =  
      Lib::Cloner::clone(dname,"OnXTemplateOne",name,aUI.session().printer());
    if(status) 
      aUI.echo("Cloning success.");
    else 
      aUI.echo("Cloning failure.");
  } else {
    std::string dname = 
      Lib::fmanip::name("$ONXROOT/templates/OnXTemplateTwo");
    bool status = 
      Lib::Cloner::clone(dname,"OnXTemplateTwo",name,aUI.session().printer());
    if(status) 
      aUI.echo("Cloning success.");
    else 
      aUI.echo("Cloning failure.");
  }
}
//////////////////////////////////////////////////////////////////////////////
/// Input*.onx ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_Input_file(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
// Used by the Input palette with a fileSaver/Chooser input.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  if(!aArgs.size()) return;
  std::string widget = aArgs[0];
  aUI.setParameter(widget+".value",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Input_colorChooser(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// Used by the Input palette with a colorChooser input.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("colorChooser");
  if(!aArgs.size()) return;
  std::string widget = aArgs[0];
  aUI.setParameter(widget+".default",aUI.callbackValue());
}
//////////////////////////////////////////////////////////////////////////////
void OnX_debug(
 Slash::UI::IUI&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);
  out << "OnX_debug" << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_About(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = std::string("OnX version ")+ONX_VERSION;
  s += "\n\n";
  s += "Author : Guy Barrand";
  s += "\n\n";
  s += "URL : http://OpenScientist.lal.in2p3.fr";
  ui_help_text(aUI,s);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/* No more used.
<OnX version="v15r0">
  <leaveSteering exec="DLD">OnX OnX_LeaveSteering</leaveSteering>
  ...
</OnX>
//////////////////////////////////////////////////////////////////////////////
void OnX_LeaveSteering(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Do nothing if started from Python.
  Lib::Arguments args(aUI.session().arguments());
  std::string s;
  if(!args.find("-from",s)) return;
  Lib::smanip::tolowercase(s);
  if(s!="python") return;

  aUI.echo("To give control to the GUI again :");
  aUI.echo("  >>> onx.steer()");
  aUI.echo("");
  aUI.echo("To delete the OnX object (and then the GUI) :");
  aUI.echo("  >>> del onx");
}
*/
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void OnX_Input_exa_py_create(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path = Lib::fmanip::name("$ONXROOT/examples/Python");
  std::vector<std::string> files;
  if(!Lib::dirmanip::entries(path,files,false)) return;
  files = Lib::smanip::match(files,"*.py");
  std::string s = Lib::smanip::tostring(files,"\n");
  aUI.setParameter("OnX_Input_exa_py_input_files.items",s);
}
/////////////////////////////////////////////////////////////////////////////
void OnX_Input_exa_py_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string file;
  if(!aUI.parameterValue("OnX_Input_exa_py_input_files.value",file)) return;
  // Send to the EditExec palette :
  file = "$ONXROOT/examples/Python/"+file;
  editExec_setFile(aUI,file);
}

} // extern "C"

#include <stdio.h>
#include <iostream>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_printf(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// To test the stdout redirection.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()) {
    std::string line = ui_cbk(aUI,aArgs[0]);
    for(unsigned int index=1;index<aArgs.size();index++) {
      line += " " + ui_cbk(aUI,aArgs[index]);
    }
    ::printf(line.c_str());
  }
  ::printf("\n");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_cout(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// To test the std::cout redirection.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()) {
    std::string line = ui_cbk(aUI,aArgs[0]);
    for(unsigned int index=1;index<aArgs.size();index++) {
      line += " " + ui_cbk(aUI,aArgs[index]);
    }
    std::cout << line;
  }
  std::cout << std::endl;
}

}

// Example of working on picked objects.
#include <Slash/Data/IHandlersIterator.h>
#include <Slash/Data/IHandler.h>
#include <Slash/Data/IAccessor.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_exa_picked_objects(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// DLD version.
// See OnX/examples/Python/Onx_exa_picked_objects for a Python version.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  Slash::Data::IProcessor* da = 
    Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
  if(!da) return;

 {std::vector<std::string> args;
  args.push_back("SceneGraph");
  args.push_back("highlight==true");
  da->execute("collect",args);}

 {std::vector<std::string> args;
  args.push_back("name");
  args.push_back("");
  da->execute("filter",args);}

  Lib::Out out(session.printer());
  out << "Picked objects :" << Lib::endl;

  Slash::Data::IHandlersIterator* hit = da->handlersIterator();
  if(hit) {
    for(;const Slash::Data::IHandler* h = hit->handler();hit->next()) {
      Slash::Data::IAccessor* accessor = h->type();
      void* obj = h->object();
      //void* tag = h->tag();
      out << "Accessor " << Lib::sout(accessor->name())
          << " object " << obj
          << Lib::endl;
    }  
    delete hit;
  }
}

}
