//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

//
//  This file does not need HEPVis and Inventor to compile.
//

// Helpers :
#include <OnX/Helpers/OnX.h>

// Lib : 
#include <Lib/Out.h>
#include <Lib/sout.h>

extern "C" {

/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_create(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  Slash::UI::IScriptManager* sm = Lib_findManager(session,"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return;
  aUI.setParameter("OnX_EditExec_interpreter.items",
                   Lib::smanip::tostring(sm->names(),"\n"));
}
/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string file = aUI.callbackValue();
  editExec_setFile(aUI,file,false);
  // Save the name of the file on some session parameter :
  aUI.session().setParameter("ui.editExec.file",file);
}
/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_save(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Get the name of the file :
  std::string file;
  aUI.session().parameterValue("ui.editExec.file",file);

  std::string svalue;
  if(!aUI.parameterValue("OnX_EditExec_text.value",svalue)) return;

  if(!Lib::fmanip::write(file,svalue)) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_EditExec_save :"
        << " can't write in " << Lib::sout(file)
        << Lib::endl;
  }
}
/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_save_as(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sinterp;
  if(!aUI.parameterValue("OnX_EditExec_interpreter.value",sinterp)) return;

  std::string ssuf = ui_interpreter_suffix(aUI,sinterp);

  aUI.setCallback("fileChooser","ok",
                  "DLD","OnX OnX_EditExec_write");
  aUI.setCallback("fileChooser","cancel",
                  "DLD","OnX ui_hide fileChooser");
  aUI.setParameter("fileChooser.dirMask","*."+ssuf);
  aUI.setParameter("fileChooser.mode","save");
  aUI.show("fileChooser");
}
/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_write(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string file = aUI.callbackValue();

  std::string svalue;
  if(!aUI.parameterValue("OnX_EditExec_text.value",svalue)) return;

  if(!Lib::fmanip::write(file,svalue)) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_EditExec_write :"
        << " can't write in " << Lib::sout(file)
        << Lib::endl;
  }
}
/////////////////////////////////////////////////////////////////////////////
void OnX_EditExec_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string svalue;
  if(!aUI.parameterValue("OnX_EditExec_text.value",svalue)) return;
  std::string sinterp;
  if(!aUI.parameterValue("OnX_EditExec_interpreter.value",sinterp)) return;

  Slash::UI::IScriptManager* sm = 
    Lib_findManager(aUI.session(),"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return;
  Slash::UI::IInterpreter* interp = sm->findInterpreter(sinterp);
  if(!interp) return;
  std::string ssuf = interp->suffix();
  std::string scmd = interp->execFileCommand();

  // Save text in a temporary file :
  std::string name;
  if(Lib::fmanip::tmpname(name,".","OnX_","."+ssuf)) {
    if(Lib::fmanip::write(name,svalue)) {

      std::string file = Lib::fmanip::name(name);
      Lib::smanip::replace(scmd,"onx_file",file);

      sm->record(sinterp,scmd); //history
      interp->executeScript(scmd);

      Lib::fmanip::remove(file);
    }
  } else {
    Lib::Out out(aUI.session().printer());
    out << "OnX_EditExec_ok : can't open a temporary file." << Lib::endl;
  }
}

} // extern "C"
