//
//  All functions here should be InputPlotter callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
// HEPVis and Inventor related callbacks.
//

// Helpers :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

// Slash :
#include <Slash/Core/IWriter.h>

// Lib : 
#include <Lib/Printer.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>

#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoPlotter.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_InputAxis_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//
// validate inputAxis values
//
// Input fields : 
//   bool auto
//   bool log
//   real amin
//   real amax
//   int steps
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  /////////////////////////////
  // Get input fields values :
  /////////////////////////////

  std::string value;
  if(!aUI.parameterValue("OnX_InputAxis_x_automated.set",value)) return;
  bool xautomate;
  if(!Lib::smanip::tobool(value,xautomate)) return;

  if(!aUI.parameterValue("OnX_InputAxis_x_amin.value",value)) return;
  double xminimum;
  if(!Lib::smanip::todouble(value,xminimum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_x_amax.value",value)) return;
  double xmaximum;
  if(!Lib::smanip::todouble(value,xmaximum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_x_AxisLogScale.set",value)) return;
  bool xlog;
  if(!Lib::smanip::tobool(value,xlog)) return;

  if(!aUI.parameterValue("OnX_InputAxis_y_automated.set",value)) return;
  bool yautomate;
  if(!Lib::smanip::tobool(value,yautomate)) return;

  if(!aUI.parameterValue("OnX_InputAxis_y_amin.value",value)) return;
  double yminimum;
  if(!Lib::smanip::todouble(value,yminimum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_y_amax.value",value)) return;
  double ymaximum;
  if(!Lib::smanip::todouble(value,ymaximum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_y_AxisLogScale.set",value)) return;
  bool ylog;
  if(!Lib::smanip::tobool(value,ylog)) return;

  if(!aUI.parameterValue("OnX_InputAxis_z_automated.set",value)) return;
  bool zautomate;
  if(!Lib::smanip::tobool(value,zautomate)) return;

  if(!aUI.parameterValue("OnX_InputAxis_z_amin.value",value)) return;
  double zminimum;
  if(!Lib::smanip::todouble(value,zminimum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_z_amax.value",value)) return;
  double zmaximum;
  if(!Lib::smanip::todouble(value,zmaximum)) return;

  if(!aUI.parameterValue("OnX_InputAxis_z_AxisLogScale.set",value)) return;
  bool zlog;
  if(!Lib::smanip::tobool(value,zlog)) return;

  /////////////////////////////
  /////////////////////////////
  /////////////////////////////

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotter* soPlotter = ((SoPlotterRegion*)soRegion)->getPlotter();

  soPlotter->xAxisAutomated.setValue(xautomate?TRUE:FALSE);
  soPlotter->xAxisMinimum.setValue((float)xminimum);
  soPlotter->xAxisMaximum.setValue((float)xmaximum);
  soPlotter->xAxisLogScale.setValue(xlog?TRUE:FALSE);
  soPlotter->yAxisAutomated.setValue(yautomate?TRUE:FALSE);
  soPlotter->yAxisMinimum.setValue((float)yminimum);
  soPlotter->yAxisMaximum.setValue((float)ymaximum);
  soPlotter->yAxisLogScale.setValue(ylog?TRUE:FALSE);
  soPlotter->zAxisAutomated.setValue(zautomate?TRUE:FALSE);
  soPlotter->zAxisMinimum.setValue((float)zminimum);
  soPlotter->zAxisMaximum.setValue((float)zmaximum);
  soPlotter->zAxisLogScale.setValue(zlog?TRUE:FALSE);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputAxis_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Get all parameters from plotter and update the display with thoses value
// Update all the InputAxis 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotter* soPlotter = ((SoPlotterRegion*)soRegion)->getPlotter();

  SbString xautomate;
  soPlotter->xAxisAutomated.get(xautomate);
  aUI.setParameter("OnX_InputAxis_x_automated.set",xautomate.getString());

  SbString xminimum;
  soPlotter->xAxisMinimum.get(xminimum);
  aUI.setParameter("OnX_InputAxis_x_amin.value",xminimum.getString());

  SbString xmaximum;
  soPlotter->xAxisMaximum.get(xmaximum);
  aUI.setParameter("OnX_InputAxis_x_amax.value",xmaximum.getString());

  SbString xlog;
  soPlotter->xAxisLogScale.get(xlog);
  aUI.setParameter("OnX_InputAxis_x_AxisLogScale.set",xlog.getString());

  SbString yautomate;
  soPlotter->yAxisAutomated.get(yautomate);
  aUI.setParameter("OnX_InputAxis_y_automated.set",yautomate.getString());

  SbString yminimum;
  soPlotter->yAxisMinimum.get(yminimum);
  aUI.setParameter("OnX_InputAxis_y_amin.value",yminimum.getString());

  SbString ymaximum;
  soPlotter->yAxisMaximum.get(ymaximum);
  aUI.setParameter("OnX_InputAxis_y_amax.value",ymaximum.getString());

  SbString ylog;
  soPlotter->yAxisLogScale.get(ylog);
  aUI.setParameter("OnX_InputAxis_y_AxisLogScale.set",ylog.getString());

  SbString zautomate;
  soPlotter->zAxisAutomated.get(zautomate);
  aUI.setParameter("OnX_InputAxis_z_automated.set",zautomate.getString());

  SbString zminimum;
  soPlotter->zAxisMinimum.get(zminimum);
  aUI.setParameter("OnX_InputAxis_z_amin.value",zminimum.getString());

  SbString zmaximum;
  soPlotter->zAxisMaximum.get(zmaximum);
  aUI.setParameter("OnX_InputAxis_z_amax.value",zmaximum.getString());

  SbString zlog;
  soPlotter->zAxisLogScale.get(zlog);
  aUI.setParameter("OnX_InputAxis_z_AxisLogScale.set",zlog.getString());
}

} // extern "C"
