//
//  All functions here should be InputPlotter callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

// Helpers :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

// Slash :
#include <Slash/Core/IWriter.h>

// Lib : 
#include <Lib/Printer.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>

// HEPVis :
#include <HEPVis/SbString.h>
#include <HEPVis/SbMath.h>
#include <HEPVis/SbStyle.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoAxis.h>
#include <HEPVis/nodekits/SoPlotter.h>
#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodes/SoLineStyle.h>
#include <HEPVis/nodes/SoTextStyle.h>
#include <HEPVis/nodes/SoStyle.h>

//////////////////////////////////////////////////////////////////////////////
/// So to UI /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void So_UI_LineStyle(
 Slash::UI::IUI& aUI
,SoLineStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.pattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.width.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}
}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_TextStyle(
 Slash::UI::IUI& aUI
,SoTextStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
/*
  SbColor color = aStyle.color.getValue();
  SbString sbs;
  SbStyle::colorToString(color,sbs);
  aUI.setParameter(aField+"color.default",sbs.getString());

  float transparency = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",
    Lib::smanip::tostring(transparency));
*/
 {SbString v = aStyle.fontName.getValue();
  aUI.setParameter(aField+"fontName.default",v.getString());}

 {int v = aStyle.fontSize.getValue();
  aUI.setParameter(aField+"fontSize.default",Lib::smanip::tostring(v));}
}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_Style(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.linePattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.lineWidth.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}

 {SbString s = aStyle.fontName.getValue();
  aUI.setParameter(aField+"fontName.default",s.getString());}

 {int v = aStyle.fontSize.getValue();
  aUI.setParameter(aField+"fontSize.default",Lib::smanip::tostring(v));}
}
//FIXME : rm the below : do it with So_UI_LineStyle :
//////////////////////////////////////////////////////////////////////////////
static void So_UI_StyleLine(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.linePattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.lineWidth.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}
}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_StyleArea(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbString v;
  aStyle.areaStyle.get(v); //enum
  aUI.setParameter(aField+"areaStyle.default",v.getString());}

}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_StyleHatch(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {float v = aStyle.spacing.getValue();
  aUI.setParameter(aField+"spacing.default",Lib::smanip::tostring(v));}

 {float v = aStyle.stripWidth.getValue();
  aUI.setParameter(aField+"stripWidth.default",Lib::smanip::tostring(v));}

 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.linePattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.lineWidth.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}

 {float v = aStyle.angle.getValue();
  v = v * 180 / FM_PI;
  aUI.setParameter(aField+"angle.default",Lib::smanip::tostring(v));}
}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_StyleData(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
,bool aPainting
,bool aHatching
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.linePattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.lineWidth.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}

 {SbString v = aStyle.modeling.getValue();
  aUI.setParameter(aField+"modeling.default",v.getString());}

 {int v = aStyle.multiNodeLimit.getValue();
  aUI.setParameter(aField+"multiNodeLimit.default",Lib::smanip::tostring(v));}

 {SbString v;
  aStyle.areaStyle.get(v); //enum
  aUI.setParameter(aField+"areaStyle.default",v.getString());}

 {int v = aStyle.markerSize.getValue();
  aUI.setParameter(aField+"markerSize.default",Lib::smanip::tostring(v));}

 {SbString v;
  aStyle.markerStyle.get(v); //enum
  aUI.setParameter(aField+"markerStyle.default",v.getString());}

  if(aPainting) {
   {SbString v;
    aStyle.painting.get(v); //enum
    aUI.setParameter(aField+"painting.default",v.getString());}
  }

  if(aHatching) { 
   {SbString v;
    aStyle.hatching.get(v); //enum
    aUI.setParameter(aField+"hatching.default",v.getString());}
  }
 
}
//////////////////////////////////////////////////////////////////////////////
static void So_UI_StyleErrors(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SbColor v = aStyle.color.getValue();
  SbString s;
  SbStyle::colorToString(v,s);
  aUI.setParameter(aField+"color.default",s.getString());}

 {float v = aStyle.transparency.getValue();
  aUI.setParameter(aField+"transparency.value",Lib::smanip::tostring(v));}

 {SbLinePattern v = aStyle.linePattern.getValue();
  std::string s;
  Lib::smanip::printf(s,32,"0x%x",v);
  aUI.setParameter(aField+"linePattern.default",s);}

 {int v = aStyle.lineWidth.getValue();
  aUI.setParameter(aField+"lineWidth.default",Lib::smanip::tostring(v));}

 {int v = aStyle.markerSize.getValue();
  aUI.setParameter(aField+"markerSize.default",Lib::smanip::tostring(v));}

 {SbString v;
  aStyle.markerStyle.get(v); //enum
  aUI.setParameter(aField+"markerStyle.default",v.getString());}
}
//////////////////////////////////////////////////////////////////////////////
/// UI to So /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static void UI_So_LineStyle(
 Slash::UI::IUI& aUI
,SoLineStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) aStyle.color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        aStyle.transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.width.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.pattern.setValue((SbLinePattern)v);
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_TextStyle(
 Slash::UI::IUI& aUI
,SoTextStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  //FIXME color.
  //FIXME transparency.

  if(aUI.parameterValue(aField + "fontName.value",s)) {
    if(s!="unchanged") {
      aStyle.fontName.setValue(s.c_str());
    } 
  }

  if(aUI.parameterValue(aField + "fontSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.fontSize.setValue(v);
      }
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_Style(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) aStyle.color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        aStyle.transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "fontName.value",s)) {
    if(s!="unchanged") {
      aStyle.fontName.setValue(s.c_str());
    } 
  }

  if(aUI.parameterValue(aField + "fontSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.fontSize.setValue(v);
      }
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleLine(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) aStyle.color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        aStyle.transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        aStyle.linePattern.setValue((SbLinePattern)v);
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleArea(
 Slash::UI::IUI& aUI
,SoStyle& aStyle
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) aStyle.color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        aStyle.transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "areaStyle.value",s)) {
    if(s!="unchanged") {
      aStyle.areaStyle.set(s.c_str()); //enum
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleBins(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getBinsStyle(index);

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "modeling.value",s)) {
    if(s!="unchanged") {
      style->modeling.setValue(s.c_str());
    } 
  }

  if(aUI.parameterValue(aField + "multiNodeLimit.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->multiNodeLimit.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "areaStyle.value",s)) {
    if(s!="unchanged") {
      style->areaStyle.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "markerSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->markerSize.setValue(v);
      }
    }
  }


  if(aUI.parameterValue(aField + "markerStyle.value",s)) {
    if(s!="unchanged") {
      style->markerStyle.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "painting.value",s)) {
    if(s!="unchanged") {
      style->painting.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "hatching.value",s)) {
    if(s!="unchanged") {
      style->hatching.set(s.c_str()); //enum
    } 
  } 
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StylePoints(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getPointsStyle(index);

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "modeling.value",s)) {
    if(s!="unchanged") {
      style->modeling.setValue(s.c_str());
    } 
  }

  if(aUI.parameterValue(aField + "multiNodeLimit.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->multiNodeLimit.setValue(v);
      }
    }
  }


  if(aUI.parameterValue(aField + "areaStyle.value",s)) {
    if(s!="unchanged") {
      style->areaStyle.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "markerSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->markerSize.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "markerStyle.value",s)) {
    if(s!="unchanged") {
      style->markerStyle.set(s.c_str()); //enum
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleFunction(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getFunctionStyle(index);

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "modeling.value",s)) {
    if(s!="unchanged") {
      style->modeling.setValue(s.c_str());
    } 
  }

  if(aUI.parameterValue(aField + "multiNodeLimit.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->multiNodeLimit.setValue(v);
      }
    }
  }


  if(aUI.parameterValue(aField + "areaStyle.value",s)) {
    if(s!="unchanged") {
      style->areaStyle.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "markerSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->markerSize.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "markerStyle.value",s)) {
    if(s!="unchanged") {
      style->markerStyle.set(s.c_str()); //enum
    } 
  }

  if(aUI.parameterValue(aField + "painting.value",s)) {
    if(s!="unchanged") {
      style->painting.set(s.c_str()); //enum
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleRightHatch(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getRightHatchStyle(index);

  if(aUI.parameterValue(aField + "spacing.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->spacing.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "stripWidth.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->stripWidth.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "angle.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        v = v * FM_PI / 180;
        style->angle.setValue(v);
      }
    } 
  }
}
//FIXME : use the upper.
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleLeftHatch(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getLeftHatchStyle(index);

  if(aUI.parameterValue(aField + "spacing.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->spacing.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "stripWidth.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->stripWidth.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "angle.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        v = v * FM_PI / 180;
        style->angle.setValue(v);
      }
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
static void UI_So_StyleErrors(
 Slash::UI::IUI& aUI
,SoPlotter& aPlotter
,const std::string& aField
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;

  int index = 0;
  if(aUI.parameterValue(aField + "index.value",s)) {
    int v;
    if(Lib::smanip::toint(s,v)) index = v;
  }

  SoStyle* style = aPlotter.getErrorsStyle(index);

  if(aUI.parameterValue(aField + "color.value",s)) {
    if(s!="unchanged") {
      SbColor v;
      if(SbStyle::stringToColor(s.c_str(),v)) style->color.setValue(v);
    }
  }

  if(aUI.parameterValue(aField + "transparency.value",s)) {
    if(s!="unchanged") {
      float v;
      if(Lib::smanip::tofloat(s,v)) {
        style->transparency.setValue(v);
      }
    } 
  }

  if(aUI.parameterValue(aField + "lineWidth.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->lineWidth.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "linePattern.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->linePattern.setValue((SbLinePattern)v);
      }
    }
  }

  if(aUI.parameterValue(aField + "markerSize.value",s)) {
    if(s!="unchanged") {
      int v;
      if(Lib::smanip::toint(s,v)) {
        style->markerSize.setValue(v);
      }
    }
  }

  if(aUI.parameterValue(aField + "markerStyle.value",s)) {
    if(s!="unchanged") {
      style->markerStyle.set(s.c_str()); //enum
    } 
  }
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_updateComponentPanel(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Get all parameters from plotter and update the display with thoses value
// Only update the component tabstack 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

 {SbBool v = soPlotter->getGridStyle()->visible.getValue();
  aUI.setParameter("OnX_InputPlotter_gridStyle.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotter->getWallStyle()->visible.getValue();
  aUI.setParameter("OnX_InputPlotter_wallStyle.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotter->getInnerFrameStyle()->visible.getValue();
  aUI.setParameter("OnX_InputPlotter_innerFrameStyle.set",v?"TRUE":"FALSE");}
 
 {SbBool v = soPlotterRegion->infosRegionVisible.getValue();
  aUI.setParameter("OnX_InputPlotter_infosVisible.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotterRegion->legendRegionVisible.getValue();
  aUI.setParameter("OnX_InputPlotter_legendVisible.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotter->getTitleStyle()->visible.getValue();
  aUI.setParameter("OnX_InputPlotter_titleStyle.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotter->titleUp.getValue();
  aUI.setParameter("OnX_InputPlotter_titleUp.set",v?"TRUE":"FALSE");}

 {SbBool v = soPlotter->superposeBins.getValue();
  aUI.setParameter("OnX_InputPlotter_superposeBins.set",v?"TRUE":"FALSE");}
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_updateHatchEditor(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Get all parameters from plotter and update the display with thoses value
// Only update the hatch editor 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

  So_UI_StyleHatch(aUI,*soPlotter->getRightHatchStyle(0),
                   "OnX_InputPlotter_rightHatch_");

  So_UI_StyleHatch(aUI,*soPlotter->getLeftHatchStyle(0),
                   "OnX_InputPlotter_leftHatch_");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_updateAllTabStack(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Get all parameters from plotter and update the display with thoses value
// Only update the style and plotter tabstack from InputPlotter 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

  // Axes :
  //FIXME : TicksStyle, TitleStyle, MagStyle.
  So_UI_LineStyle(aUI,*soPlotter->getXAxis()->getLineStyle(),
                  "OnX_InputPlotter_xAxis_");
  So_UI_LineStyle(aUI,*soPlotter->getYAxis()->getLineStyle(),
                  "OnX_InputPlotter_yAxis_");
  So_UI_LineStyle(aUI,*soPlotter->getZAxis()->getLineStyle(),
                 "OnX_InputPlotter_zAxis_");

  So_UI_TextStyle(aUI,*soPlotter->getXAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_xAxis_");
  So_UI_TextStyle(aUI,*soPlotter->getYAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_yAxis_");
  So_UI_TextStyle(aUI,*soPlotter->getZAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_zAxis_");

  // Infos :
  aUI.setParameter("OnX_InputPlotter_infos_content.default",
                   soPlotter->getInfosStyle()->options.getValue().getString());
  So_UI_Style(aUI,*soPlotter->getInfosStyle(),
              "OnX_InputPlotter_infos_");

  // Title :
  So_UI_TextStyle(aUI,*soPlotter->getTitleStyle(),
              "OnX_InputPlotter_title_");

  // Grid :
  //FIXME : So_UI_LineStyle
  So_UI_StyleLine(aUI,*soPlotter->getGridStyle(),
                  "OnX_InputPlotter_grid_");

  // InnerFrame :
  //FIXME : So_UI_LineStyle
  So_UI_StyleLine(aUI,*soPlotter->getInnerFrameStyle(),
                  "OnX_InputPlotter_innerFrame_");

  // Wall :
  //FIXME : updateAreaStyle
  So_UI_StyleArea(aUI,*soPlotter->getWallStyle(),
                  "OnX_InputPlotter_wall_");

  // Bins :
  So_UI_StyleData(aUI,*soPlotter->getBinsStyle(0),
                  "OnX_InputPlotter_bins_",true,true);

  // Points :
  So_UI_StyleData(aUI,*soPlotter->getPointsStyle(0),
                  "OnX_InputPlotter_points_",false,false);

  // Function :
  So_UI_StyleData(aUI,*soPlotter->getFunctionStyle(0),
                  "OnX_InputPlotter_function_",true,false);

  // Errors :
  So_UI_StyleErrors(aUI,*soPlotter->getErrorsStyle(0),
                    "OnX_InputPlotter_errors_");
    
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Get all parameters from plotter and update the display with thoses value
// Update all the InputPlotter (no update for InputPlotterHatch) 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;

  OnX_InputPlotter_updateComponentPanel(aUI);
  OnX_InputPlotter_updateAllTabStack(aUI);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_setAllTabStack(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//
// Get all parameters from display and update the plotter fields with thoses
// value
// Get all parameters from the InputPlotter (style and plotter tabStack)
// (no parameters from InputPlotterHatch) 
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

  // Axes :
  UI_So_LineStyle(aUI,*soPlotter->getXAxis()->getLineStyle(),
                  "OnX_InputPlotter_xAxis_");
  UI_So_LineStyle(aUI,*soPlotter->getYAxis()->getLineStyle(),
                  "OnX_InputPlotter_yAxis_");
  UI_So_LineStyle(aUI,*soPlotter->getZAxis()->getLineStyle(),
                 "OnX_InputPlotter_zAxis_");

  UI_So_TextStyle(aUI,*soPlotter->getXAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_xAxis_");
  UI_So_TextStyle(aUI,*soPlotter->getYAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_yAxis_");
  UI_So_TextStyle(aUI,*soPlotter->getZAxis()->getLabelsStyle(),
                  "OnX_InputPlotter_zAxis_");

  // Infos :
 {std::string s;
  if(aUI.parameterValue("OnX_InputPlotter_infos_content.value",s)) {
    if(s!="unchanged") {
      soPlotter->getInfosStyle()->options.setValue(s.c_str());
    }
  }}
  UI_So_Style(aUI,*soPlotter->getInfosStyle(),
              "OnX_InputPlotter_infos_");

  // Title :
  UI_So_TextStyle(aUI,*soPlotter->getTitleStyle(),
              "OnX_InputPlotter_title_");

  // Grid :
  //FIXME : So_UI_LineStyle
  UI_So_StyleLine(aUI,*soPlotter->getGridStyle(),
                  "OnX_InputPlotter_grid_");

  // InnerFrame :
  //FIXME : So_UI_LineStyle
  UI_So_StyleLine(aUI,*soPlotter->getInnerFrameStyle(),
                  "OnX_InputPlotter_innerFrame_");

  // Wall :
  //FIXME : updateAreaStyle
  UI_So_StyleArea(aUI,*soPlotter->getWallStyle(),
                  "OnX_InputPlotter_wall_");

  // Bins :
  UI_So_StyleBins(aUI,*soPlotter,"OnX_InputPlotter_bins_");

  // Points :
  UI_So_StylePoints(aUI,*soPlotter,"OnX_InputPlotter_points_");

  // Function :
  UI_So_StyleFunction(aUI,*soPlotter,"OnX_InputPlotter_function_");

  // Errors :
  UI_So_StyleErrors(aUI,*soPlotter,"OnX_InputPlotter_errors_");

}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_setHatchTabStack(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//
// Get all parameters from display and update the plotter fields with thoses
// value
// Get all parameters from the InputPlotterHatch 
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

  UI_So_StyleRightHatch(aUI,*soPlotter,"OnX_InputPlotter_rightHatch_");

  UI_So_StyleLeftHatch(aUI,*soPlotter,"OnX_InputPlotter_leftHatch_");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_setComponentPanel(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//
// Get all parameters from display and update the plotter fields with thoses
// value
// Get all parameters from the InputPlotter component tabStack 
//
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
  SoPlotter* soPlotter = soPlotterRegion->getPlotter();

 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_wallStyle.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->getWallStyle()->visible.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_gridStyle.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->getGridStyle()->visible.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_innerFrameStyle.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->getInnerFrameStyle()->visible.setValue(v?TRUE:FALSE);}

 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_infosVisible.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotterRegion->infosRegionVisible.setValue(v?TRUE:FALSE);}

 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_legendVisible.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotterRegion->legendRegionVisible.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_titleStyle.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->getTitleStyle()->visible.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_titleUp.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->titleUp.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_superposeBins.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->superposeBins.setValue(v?TRUE:FALSE);}
  
 {std::string s;
  if(!aUI.parameterValue("OnX_InputPlotter_errors.set",s)) return;
  Lib::smanip::touppercase(s);
  bool v;
  if(!Lib::smanip::tobool(s,v)) return;
  soPlotter->getErrorsStyle(0)->visible.setValue(v?TRUE:FALSE);}
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// validate plotter values
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnX_InputPlotter_setComponentPanel(aUI);
  OnX_InputPlotter_setAllTabStack(aUI);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_InputPlotter_HatchEditor_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// validate hatchEditor values
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnX_InputPlotter_setHatchTabStack(aUI);
}

} // extern "C"
