//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

//
// gl2ps and OpenGL related callbacks.
//

// Helpers :
#include <OnX/Helpers/OnX.h>

// Slash :
#include <Slash/UI/IWidgetClass.h>
#include <Slash/UI/ICallbackData.h>

// Lib : 
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/ListContour.h>
#include <Lib/Functions.h>
#include <Lib/Cast.h>

// OnX :
#include <OnX/OpenGL/OpenGL.h>

#include "OnX_gl2ps.h"

static double contourFunction(double aX,double aY,void* aData) {
  Lib::Function::Scripted* func = (Lib::Function::Scripted*)aData;
  std::vector<double> xs(2);
  xs[0] = aX;
  xs[1] = aY;
  double value;
  std::string serror;
  func->value(xs,value,serror);
  return value;
}

//////////////////////////////////////////////////////////////////////////////
inline bool ui_gl2ps(
 Slash::UI::IWidget& aWidget
,const std::string& aFileName
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel = 0
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(aPrinter);

  unsigned int width,height;
  if(!aWidget.size(width,height)) {
    out << "ui_gl2ps :"
        << " Cannot get size of " << Lib::sout(aWidget.name())
        << Lib::endl;
    return false;
  }
 
  FILE* file = ::fopen(aFileName.c_str(),"w");
  if(!file) {
    out << "ui_gl2ps :"
        << " Cannot open file " << Lib::sout(aFileName)
        << Lib::endl;
    return false;
  }

  if(aVerboseLevel) {
    out << "ui_gl2ps :"
        << " write file " << Lib::sout(aFileName) << "..."
        << Lib::endl;
  }

  int options = GL2PS_OCCLUSION_CULL 
    | GL2PS_BEST_ROOT 
    | GL2PS_SILENT
    | GL2PS_DRAW_BACKGROUND;
  int sort = GL2PS_BSP_SORT;
  //int sort = GL2PS_SIMPLE_SORT;
    
  GLint vp[4];
  vp[0] = 0;
  vp[1] = 0;
  vp[2] = width;
  vp[3] = height;

  int bufsize = 0;

// Glint state = GL2PS_OVERFLOW;
//  while( state == GL2PS_OVERFLOW ){ 
//    buffsize += 1024*1024;
//    ...
//    state = gl2psEndPage();
//  }

  gl2psBeginPage("title","OnX_gl2ps", 
                 vp,GL2PS_EPS,sort,options, 
                 GL_RGBA,0, NULL,0,0,0,bufsize, 
                 file,aFileName.c_str());
    
  aWidget.executeCallbacks("paint");
    
  //int state =
  gl2psEndPage();
  ::fclose(file);

  return true;
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// OnX/scripts/OnX/InputPrint.onx ///////////////////////////////////////////
/// gl2ps ////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_widget_gl2ps(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;

  Slash::UI::IWidget* widget = aUI.findWidget(aArgs[0]);
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_widget_gl2ps : "
        << " widget " << Lib::sout(widget->name()) << " not found."
        << Lib::endl;
    return;
  }
  if(widget->widgetClass().name()!="OpenGLArea") {
    Lib::Out out(aUI.session().printer());
    out << "OnX_widget_gl2ps : "
        << " widget " << Lib::sout(widget->name()) 
        << " not an OpenGLArea. ("
        << Lib::sout(widget->widgetClass().name()) << ")."
        << Lib::endl;
    return;
  }

  ui_gl2ps(*widget,aArgs[1],
           aUI.session().printer(),aUI.session().verboseLevel());

}
//////////////////////////////////////////////////////////////////////////////
/// OpenGL ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_create_OpenGLArea(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Generate a name :
  int n;
  if(!ui_parameterValue(aUI,"ViewerTabStack.itemCount",n)) return;
  std::string name;
  if(!Lib::smanip::printf(name,64,"OpenGLArea_%d",n+1)) return;

  aUI.setParameter("Viewer_1.tabLabel",name);
  aUI.createComponent(name,"OpenGLArea","ViewerTabStack");
  aUI.setParameter("Viewer_1.tabLabel","Viewer_1");

  if(aArgs.size()) {
    if(aArgs[0]=="contour") {
      aUI.setCallback(name,"paint","DLD","OnX OnX_OpenGL_contour");
    } else if(aArgs[0]=="cube") {
      aUI.setCallback(name,"paint","DLD","OnX OnX_OpenGL_cube");
    }
  }
}

static void OpenGL_Contour(CListContour&);

namespace OnX {
class ContourManager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::ContourManager)
    else if_Lib_SCast(Slash::Core::IManager)
    else return 0;
  }
public:
  ContourManager(Slash::Core::ISession& aSession) 
  :fSession(aSession),fName("OnX_ContourManager"),fListContour(0),fFunction(0){
    fListContour = new CListContour;
    //fListContour->Generate();

    // WARNING : function not defined at x = 0 !!!
    fFunction = new Lib::Function::Scripted
        ("abs(sin(x)/x)*(cos(y)*y)",fSession.printer());
    //  ("sqrt(x*x+y*y)",fSession.printer());

    double limits[4];
    double epsilon = 0.01; //To avoid the singularity.
    limits[0] = -6+epsilon;
    limits[1] =  6+epsilon;    
    limits[2] = -6+epsilon;    
    limits[3] =  6+epsilon;    
    fListContour->SetLimits(limits);
    int zn = 100;
    std::vector<double> zs(zn+1);
    double zmn = -6;
    double zmx = 6;
    double zd = (zmx-zmn)/zn;
    for(int zi=0;zi<=zn;zi++) zs[zi] = zmn + zi * zd;
    fListContour->SetPlanes(zs);

    //fListContour->SetFirstGrid(32,32);
    //fListContour->SetSecondaryGrid(256,256);

    fListContour->SetFieldFcn(contourFunction,fFunction);
    fListContour->Generate();
  }
  virtual ~ContourManager(){
    delete fListContour;
    delete fFunction;
  }
  CListContour* listContour() const { return fListContour;}
private:
  Slash::Core::ISession& fSession;
  std::string fName;
  CListContour* fListContour;
  Lib::Function::Scripted* fFunction;
};
}
//////////////////////////////////////////////////////////////////////////////
void OnX_OpenGL_contour(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  OnX::ContourManager* contourManager = 
    Lib_findManager(session,"OnX_ContourManager",OnX::ContourManager);
  if(!contourManager) {
    contourManager = new OnX::ContourManager(session);
    session.addManager(contourManager);
  }

  CListContour* listContour = contourManager->listContour();

  std::string component = aUI.callbackData().component();
  unsigned int width,height;
  aUI.size(component,width,height);
  //printf("debug : %s : %d %d\n",component.c_str(),width,height);

  glEnable(GL_DEPTH_TEST);
  //glEnable(GL_SCISSOR_TEST);

  glEnable(GL_LINE_STIPPLE);
  glPolygonMode(GL_FRONT,GL_FILL);
  //  So that lightning is correctly computed 
  // when some scaling is applied.
  glEnable(GL_NORMALIZE);

  glShadeModel(GL_FLAT);
  //glEnable(GL_LIGHTING);
  //glDisable(GL_LIGHT0);

  glViewport(0,0,width,height);
  //glScissor(0,0,width,height);

  // In OpenGL, by default,
  // camera is placed at origin, looking at -z, up along y.
  GLint viewport[4];
  glGetIntegerv(GL_VIEWPORT,viewport);
  double aspect = ((double)viewport[2])/((double)viewport[3]);
  glMatrixMode(GL_PROJECTION); 
  glLoadIdentity();

  double limits[4];
  listContour->GetLimits(limits);
  double xb = limits[0];
  double xe = limits[1];
  double yb = limits[2];
  double ye = limits[3];

  double half_height = (ye-yb)/2;
  half_height *= 1.2;

  glOrtho(-aspect * half_height,aspect*half_height,
          -half_height,half_height,-100,100);

  // Better to clear after glViewport and glScissor (else problems with Mesa).
  glClearColor(1,1,1,0);
  glClear(GL_COLOR_BUFFER_BIT);
  glClear(GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glTranslated(-(xb+xe)/2,-(yb+ye)/2,0);

  OpenGL_Contour(*listContour);

  glFinish();
}
static void OpenGL_DrawCube();
//////////////////////////////////////////////////////////////////////////////
void OnX_OpenGL_cube(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string component = aUI.callbackData().component();
  unsigned int width,height;
  aUI.size(component,width,height);
  //printf("debug : %s : %d %d\n",component.c_str(),width,height);

  glEnable(GL_DEPTH_TEST);
  //glEnable(GL_SCISSOR_TEST);

  glEnable(GL_LINE_STIPPLE);
  glPolygonMode(GL_FRONT,GL_FILL);
  //  So that lightning is correctly computed 
  // when some scaling is applied.
  glEnable(GL_NORMALIZE);

  glShadeModel(GL_FLAT);
  //glEnable(GL_LIGHTING);
  //glDisable(GL_LIGHT0);

  glViewport(0,0,width,height);
  //glScissor(0,0,width,height);

  // In OpenGL, by default,
  // camera is placed at origin, looking at -z, up along y.
  GLint viewport[4];
  glGetIntegerv(GL_VIEWPORT,viewport);
  double aspect = ((double)viewport[2])/((double)viewport[3]);
  glMatrixMode(GL_PROJECTION); 
  glLoadIdentity();

  double half_height = 1;
  glOrtho(-aspect * half_height,aspect*half_height,
          -half_height,half_height,-100,100);

  // Better to clear after glViewport and glScissor (else problems with Mesa).
  glClearColor(0.8F,0.8F,0.8F,0);
  glClear(GL_COLOR_BUFFER_BIT);
  glClear(GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  //glScaled(1,1,1);
  // Rotate then translate :
  glTranslated(0,0,-10);
  glRotated(30,0,1,0);
  glRotated(30,1,0,0);

  OpenGL_DrawCube();

  glFinish();
}
//////////////////////////////////////////////////////////////////////////////
void OnX_OpenGL_event(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.echo("OnX_OpenGL_event");
  //printf("debug : OnX_OpenGL_event : \"%s\" %d %d\n",
    //aUI.callbackData().event().c_str(),
    //aUI.callbackData().x(),aUI.callbackData().y());
}
/////////////////////////////////////////////////////////////////////////////
void OnX_print_OpenGLArea(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Used in POVAMA.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.setCallback("fileChooser","ok",
                  "DLD","OnX OnX_write_OpenGLArea");
  aUI.setCallback("fileChooser","cancel",
                  "DLD","OnX ui_hide fileChooser");
  aUI.setParameter("fileChooser.dirMask","*.ps");
  aUI.setParameter("fileChooser.mode","save");
  aUI.show("fileChooser");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_write_OpenGLArea(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");

  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "OnX_write_OpenGLArea : "
        << " No current widget."
        << Lib::endl;
    return;
  }
  if(widget->widgetClass().name()!="OpenGLArea") {
    Lib::Out out(aUI.session().printer());
    out << "OnX_write_OpenGLArea : "
        << " widget " << Lib::sout(widget->name()) 
        << " not an OpenGLArea. ("
        << Lib::sout(widget->widgetClass().name()) << ")."
        << Lib::endl;
    return;
  }

  ui_gl2ps(*widget,aUI.callbackValue(),
           aUI.session().printer(),aUI.session().verboseLevel());
}

} // extern "C"

//////////////////////////////////////////////////////////////////////////////
static void OpenGL_DrawCube(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  glLineWidth(4);

  // Front :
  glColor3d(1,0,0);    //red
  glBegin(GL_POLYGON);
  glNormal3d(0,0,1);
  glVertex3d( 0.5,-0.5,0.5);
  glVertex3d( 0.5, 0.5,0.5);
  glVertex3d(-0.5, 0.5,0.5);
  glVertex3d(-0.5,-0.5,0.5);
  glEnd();

  // Back :
  glColor3d(0,1,0);   //green
  glBegin(GL_POLYGON);
  glNormal3d(0,0,-1);
  glVertex3d( 0.5,-0.5,-0.5);
  glVertex3d(-0.5,-0.5,-0.5);
  glVertex3d(-0.5, 0.5,-0.5);
  glVertex3d( 0.5, 0.5,-0.5);
  glEnd();

  glColor3d(1,1,0);   //yellow
  glBegin(GL_LINES);
  glVertex3d( 0.5,-0.5, 0.5);
  glVertex3d( 0.5,-0.5,-0.5);
  glVertex3d( 0.5, 0.5, 0.5);
  glVertex3d( 0.5, 0.5,-0.5);
  glVertex3d(-0.5, 0.5, 0.5);
  glVertex3d(-0.5, 0.5,-0.5);
  glVertex3d(-0.5,-0.5, 0.5);
  glVertex3d(-0.5,-0.5,-0.5);
  glEnd();
}
//////////////////////////////////////////////////////////////////////////////
static void OpenGL_Contour(
 CListContour& aListContour
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  glLineWidth(1);

  double limits[4];
  limits[0] = 0;
  limits[1] = 0;
  limits[2] = 0;
  limits[3] = 0;
  //aListContour.GetLimits(limits);
  for (unsigned int i=0;i<aListContour.GetNPlanes();i++)  {
    CLineStripList* pStripList = aListContour.GetLines(i);
    if(!pStripList) {
      //FIXME : should produce a warning.
      return;
    }
    for (CLineStripList::iterator pos=pStripList->begin(); 
         pos != pStripList->end() ; pos++) {
      CLineStrip* pStrip = (*pos);
      if(!pStrip) {
        //FIXME : should produce a warning.
        return;
      }
      if (pStrip->empty()) continue;

      // putting point at start and end of strip
      glBegin(GL_POINTS);
      glColor4f(1,0,0,.8f);
      // retreiving index
      unsigned int index=pStrip->front();
      // drawing
      double xb=aListContour.GetXi(index);
      double yb=aListContour.GetYi(index);
      glVertex2f((GLfloat)(limits[0]+xb),(GLfloat)(limits[2]+yb));
      // retreiving index
      //glColor4f(0,0,1,.8f);
      index=pStrip->back();
      // drawing
      double xe=aListContour.GetXi(index);
      double ye=aListContour.GetYi(index);
      //printf("debug : %g %g : %g %g\n",xb,yb,xe,ye);
      glVertex2f((GLfloat)(limits[0]+xe),(GLfloat)(limits[2]+ye));
      glEnd();

      glColor3f(i/(float)aListContour.GetNPlanes(),
                i/(float)aListContour.GetNPlanes(),
                1.f-i/(float)aListContour.GetNPlanes());

      // filling strip if closed...
      glBegin(GL_LINE_STRIP);
      for (CLineStrip::iterator pos2=pStrip->begin(); 
           pos2 != pStrip->end() ; pos2++) {
        // retreiving index
        index=(*pos2);
        // drawing
        double x=aListContour.GetXi(index);
        double y=aListContour.GetYi(index);
        glVertex2f((GLfloat)(limits[0]+x),(GLfloat)(limits[2]+y));
      }
      glEnd();
    }
  }
}
