//
//  All functions here should be InputPlotter callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

// Helpers :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

// Lib :
#include <Lib/Manager.h>

#include <HEPVis/SbTestPlotter.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodekits/SoAxis.h>
#include <HEPVis/nodekits/SoPlotter.h>

// Have a manager to delete histograms when quiting.
#include <Slash/Core/IManager.h>

namespace OnX {
class TestHistogramManager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::TestHistogramManager)
    else if_Lib_SCast(Slash::Core::IManager)
    else return 0;
  }
public:
  TestHistogramManager(Slash::Core::ISession& aSession) 
  :fSession(aSession),fName("OnX_TestHistogramManager"),fHistogram(0){}
  virtual ~TestHistogramManager(){ delete fHistogram;}
private:
  Slash::Core::ISession& fSession;
  std::string fName;
public:
  SbTestHistogram1D* fHistogram;
};
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_Plotter_test(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;

  Slash::Core::ISession& session = aUI.session();
  OnX::TestHistogramManager* histogramManager = 
    Lib_findManager(session,"OnX_TestHistogramManager",
                    OnX::TestHistogramManager);
  if(!histogramManager) {
    histogramManager = new OnX::TestHistogramManager(session);
    session.addManager(histogramManager);
  }

  SbTestHistogram1D* histo1D = new SbTestHistogram1D;
  histogramManager->fHistogram = histo1D;

  SoPlotter* plotter = soPlotterRegion->getPlotter();
  plotter->setName("plotter_1");
  plotter->title.setValue("Title");
  plotter->setPlottableObject(new SbTestPlottableBins1D(*histo1D));
  plotter->set("binsStyle[0]","modeling points color 0 0 1 markerStyle PLUS markerSize 9");
  plotter->getXAxis()->title.setValue("x axis title");
  plotter->getYAxis()->title.setValue("y axis title");
  plotter->getZAxis()->title.setValue("z axis title");

}

} // extern "C"
