//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
// SDL examples related callbacks.
//

// Helpers :
#include <OnX/Helpers/Inventor.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/UI/ICyclic.h>

// Lib : 
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/Arguments.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/System.h>

// Inventor :
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/actions/SoSearchAction.h>

// HEPVis :
#include <HEPVis/SbMath.h>
#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodes/SoGrid.h>
#include <HEPVis/nodes/SoViewportRegion.h>
#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>

#include <HEPVis/nodekits/SoDetectorTreeKit.h>
#include <HEPVis/nodekits/SoDetectorExample.h>

static void detPickCB(void*,void*);

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_banner(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoDisplayRegion::getClassTypeId())) return;
  SoDisplayRegion* displayRegion = (SoDisplayRegion*)soRegion;
  SoSeparator* scene = (SoSeparator*)displayRegion->getStaticScene();
  if(!scene) return;
  scene->removeAllChildren();

  soRegion->color.setValue(0.8F,0.8F,0.8F);

  // Light at ceil :
  SoPointLight* light = new SoPointLight;
  light->location.setValue(0,100,0); //y = 100.
  scene->addChild(light);

  std::string file = Lib::fmanip::name("$ONXROOT/scripts/Inventor/OnX.iv");
  std::string format = Lib::smanip::suffix(file);

  if(soRegion->readScene(file.c_str(),format.c_str())==FALSE) {
    aUI.echo("Can't read file"+file+".");
  }
}

namespace OnX {
class add_pickCB : public virtual SbVisitor {
public:
  virtual ~add_pickCB(){}
public:
  virtual SbBool visit(SoFullPath& aPath) {
    SoNode* node = aPath.getTail();
    if(!node) return TRUE;
    SoDetectorTreeKit* kit = (SoDetectorTreeKit*)node;
    kit->addPickCallback(fCB,fTag);
    return TRUE;
  }
public:
  SoCallbackListCB* fCB;
  void* fTag;
};
}

//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_default_scene(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;

  SoViewportRegion* vpr = soRegion->getViewportRegion();
  if(!vpr) return;
  vpr->horizontalBorder.setValue(0);
  vpr->verticalBorder.setValue(0);
  vpr->horizontalHighlightBorder.setValue(0);
  vpr->verticalHighlightBorder.setValue(0);

  if(!soRegion->isOfType(SoDisplayRegion::getClassTypeId())) return;
  SoDisplayRegion* displayRegion = (SoDisplayRegion*)soRegion;

  displayRegion->setCameraType(SoPerspectiveCamera::getClassTypeId());
  SoPerspectiveCamera* soCamera = 
    (SoPerspectiveCamera*)displayRegion->getCamera();
  soCamera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA);
  float d = 100;
  soCamera->position.setValue(0,2,d); //y = 2 (meter)
  soCamera->focalDistance.setValue(d);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
  soCamera->nearDistance.setValue(d*0.01F);
  soCamera->farDistance.setValue(100*d);
  soCamera->heightAngle.setValue(FM_PI/4.0F);

 {Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) return;
  float dr = 2.0F * 3.1416F/180.0F;
  widget->setParameter("deltaRotation",Lib::smanip::tostring(dr));
  widget->setParameter("deltaTranslation","3");} //default is 1.

  SoSeparator* scene = (SoSeparator*)displayRegion->getStaticScene();
  if(!scene) return;
  scene->removeAllChildren();

  soRegion->color.setValue(0.8F,0.8F,0.8F);

  // Light at ceil :
  SoPointLight* light = new SoPointLight;
  light->location.setValue(0,100,0); //y = 100.
  scene->addChild(light);

  ///////////////////////////////////////////////
  // Floor :
  ///////////////////////////////////////////////
  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(0.8F,0.8F,0.8F);
  scene->addChild(material);

  SoGrid* floor = new SoGrid;
  float halfSize = 1000;
  floor->position.setValue(SbVec3f(-halfSize,0,halfSize)); //y = 0
  floor->X.setValue(SbVec3f(1,0,0));
  floor->Y.setValue(SbVec3f(0,0,-1));
  floor->width.setValue(2.0F*halfSize);
  floor->height.setValue(2.0F*halfSize);
  scene->addChild(floor);

  ///////////////////////////////////////////////
  // Detector :
  ///////////////////////////////////////////////
 {SoSeparator* soSepDet = new SoSeparator;
  scene->addChild(soSepDet);

  SoTranslation* trans = new SoTranslation;
  trans->translation.setValue(SbVec3f(0,20,0)); //y = 20
  soSepDet->addChild(trans);

  SoRotationXYZ* soRotation = new SoRotationXYZ; //For animation.
  soRotation->axis.setValue(SoRotationXYZ::Y);
  //soRotation->angle.setValue(FM_PI_2);
  soSepDet->addChild(soRotation);

  SoDetectorExample* soDetExa = new SoDetectorExample;
  soSepDet->addChild(soDetExa);

  OnX::add_pickCB v;
  v.fCB = detPickCB;
  v.fTag = &aUI;
  SoTools::visit(*soDetExa,SoDetectorTreeKit::getClassTypeId(),v);
  }

}

#define CYCLIC "OnX_SDL_rotate_scene"

//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_rotate_scene(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic(CYCLIC);
  if(!cyclic) {
    cyclic = aUI.createCyclic(CYCLIC,250,"DLD","OnX OnX_SDL_rotate_update");
    if(!cyclic) return;
    cyclic->start();
  } else {
    if(!cyclic->started()) {
      cyclic->start();
    } else {
      cyclic->stop();
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_rotate_update(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoDisplayRegion::getClassTypeId())) return;
  SoDisplayRegion* displayRegion = (SoDisplayRegion*)soRegion;
  SoSeparator* scene = (SoSeparator*)displayRegion->getStaticScene();
  if(!scene) return;

  SoSearchAction searchAction;
  searchAction.setFind(SoSearchAction::TYPE);
  searchAction.setType(SoRotationXYZ::getClassTypeId());
  searchAction.apply(scene);

  SoPath* path = searchAction.getPath();
  if(!path) return;

  SoRotationXYZ* soRotation = (SoRotationXYZ*)path->getTail();
  float angle = soRotation->angle.getValue();
  float one_degree = 3.14/360.0;
  soRotation->angle.setValue(angle+one_degree);

}
//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_lock_screen(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string passwd;
  if(aArgs.size()==1) {
    passwd = aArgs[0];
  } else {
    Lib::Arguments args(aUI.session().arguments());
    if(!args.find("-password",passwd)) {
      if(!Lib::System::getenv("ONX_PASSWORD",passwd)) {
        Lib::Out out(aUI.session().printer());
        out << "No password given"
            << Lib::endl;
        return;
      }
    }
  }

  aUI.lockScreen(passwd,"DLD","OnX OnX_SDL_unlock_screen");

/*
#define LOCK_MENUITEM "OnX_SDL_lock"

  Slash::UI::IWidget* widget = aUI.findWidget(LOCK_MENUITEM);
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "Can't find widget " << Lib::sout(LOCK_MENUITEM)
        << Lib::endl;
    return;
  }

  widget->setParameter("background","green");
*/

 {Slash::UI::IWidget* widget = aUI.currentWidget();
  if(widget) widget->setParameter("showOptionsRegion","false");}
  
}
//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_unlock_screen(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
/*
  Slash::UI::IWidget* widget = aUI.findWidget(LOCK_MENUITEM);
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "Can't find widget " << Lib::sout(LOCK_MENUITEM)
        << Lib::endl;
    return;
  }
  widget->setParameter("background","white");
*/
}
//////////////////////////////////////////////////////////////////////////////
void OnX_SDL_remove_manips(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoDisplayRegion::getClassTypeId())) return;
  SoDisplayRegion* displayRegion = (SoDisplayRegion*)soRegion;
  SoSeparator* scene = (SoSeparator*)displayRegion->getStaticScene();
  if(!scene) return;

  SoTools::removeManips(*scene);

 {Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) return;
  widget->setParameter("showCursor","false");}
}

} // extern "C"

//////////////////////////////////////////////////////////////////////////////
static void detPickCB(
 void* aTag
,void* aNode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoDetectorTreeKit* soDet = (SoDetectorTreeKit*)((SoNode*)aNode);
  printf("OnX_SDL_default_scene::detPickCB : %s\n",
    soDet->getName().getString());

  SoNode* node = soDet->getPart("transform",TRUE);
  if(node) {
    Slash::UI::IUI* ui = (Slash::UI::IUI*)aTag;

   {Slash::UI::IWidget* widget = ui->currentWidget();
    if(widget) widget->setParameter("showCursor","true");

    Slash::UI::ISoViewer* soViewer = ui_SoViewer(*widget);
    if(soViewer) {
      SoNode* sceneGraph = soViewer->sceneGraph();
      SoTools::setManip(sceneGraph,(SoTransform*)node);
    }}
  }

}
