//
// Example of declaration of some "user" data (here class Shape) 
// to the AccessorManager.
//


#include "Shape.h" // "user" class to declare to the AccessorManager.

#include <Lib/Cast.h>

//////////////////////////////////////////////////////////////////////////////
// Code to declare the user class to OnX.  ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// The OnX_ShapeManager manages data (here instances of class OnX::Shape)
// or permits to access the entity that manages the data.
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Inheritance :
#include <Slash/Core/IManager.h>

#include <vector>
#include <Slash/Core/ISession.h>
#include <Lib/System.h>
#include <Lib/Randoms.h>
#include <Lib/Manager.h>
#include <Slash/UI/IStyleManager.h>

namespace OnX {
class ShapeManager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const;
  virtual void* cast(const std::string&) const;
public:
  ShapeManager(Slash::Core::ISession&);
  virtual ~ShapeManager();
  std::vector<Shape*>& shapes();
private:
  std::string fName;
  std::vector<Shape*> fShapes;
};
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeManager::ShapeManager(
 Slash::Core::ISession& aSession
)
:fName("OnX_ShapeManager")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create some data.
  fShapes.push_back(new Shape("Cube",1, 10, 10,0));
  fShapes.push_back(new Shape("Cube",2,-10, 10,0));
  fShapes.push_back(new Shape("Cube",3,-10,-10,0));
  fShapes.push_back(new Shape("Cube",4, 10,-10,0));
  //Have an intersection with the previous one :
  fShapes.push_back(new Shape("Cube",4, 13,-10,0));

  fShapes.push_back(new Shape("Sphere",1,0, 10, 10));
  fShapes.push_back(new Shape("Sphere",2,0,-10, 10));
  fShapes.push_back(new Shape("Sphere",3,0,-10,-10));
  fShapes.push_back(new Shape("Sphere",4,0, 10,-10));
  //Have an intersection with the previous one :
  fShapes.push_back(new Shape("Sphere",4,0, 17.9,-10));
  fShapes.push_back(new Shape("Cylinder",1, 10,0, 10));
  fShapes.push_back(new Shape("Cylinder",2,-10,0, 10));
  fShapes.push_back(new Shape("Cylinder",3,-10,0,-10));
  fShapes.push_back(new Shape("Cylinder",5, 10,0,-10));
  fShapes.push_back(new Shape("Cone",1,0,0,0));
  fShapes.push_back(new Shape("Hole",1,10,10,10));
  fShapes.push_back(new Shape("Hole",1,-10,-10,-10));
  // Have one centered at origin to test shape cuts :
  fShapes.push_back(new Shape("Hole",2,0,0,0));

  // To test line drawing :
  Lib::RandomFlat r;
  double s = 100; //scale.
  fShapes.push_back(new Shape("Track1",0,s*r.shoot(),s*r.shoot(),s*r.shoot()));
  fShapes.push_back(new Shape("Track2",0,s*r.shoot(),s*r.shoot(),s*r.shoot()));
  fShapes.push_back(new Shape("Track3",0,s*r.shoot(),s*r.shoot(),s*r.shoot()));
  fShapes.push_back(new Shape("Track4",0,s*r.shoot(),s*r.shoot(),s*r.shoot()));
  fShapes.push_back(new Shape("Track5",0,s*r.shoot(),s*r.shoot(),s*r.shoot()));

  Slash::UI::IStyleManager* styleManager = 
    Lib_findManager(aSession,"StyleManager",Slash::UI::IStyleManager);
  if(styleManager) {
    std::string file = Lib::System::getenv("ONXSHAPESTYLES");
    if(file=="") file = "$ONXROOT/scripts/Styles/Shape.styles";
    styleManager->loadFile(file);
  }
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeManager::~ShapeManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<fShapes.size();index++) delete fShapes[index];
  fShapes.clear();
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::ShapeManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::ShapeManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(OnX::ShapeManager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<OnX::Shape*>& OnX::ShapeManager::shapes(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fShapes;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// The Shape Accessor permits to manipulate the data 
// (with data_collect) and permits to order visualization (with data_visualize)
// of them. See OnX/source/Callbacks/Inventor.cxx/OnX_Examples_Shapes callback.
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Inheritance :
#include <Slash/Data/IIterator.h>
#include <OnX/Inventor/InventorAccessor.h>

#include <Lib/Property.h>
#include <Lib/Value.h>
#include <Lib/smanip.h>
namespace OnX {
class ShapeAccessor : public OnX::InventorAccessor {
public:
  ShapeAccessor(Slash::Core::ISession&,std::vector<Shape*>&);
  virtual ~ShapeAccessor();
public: //Slash::Data::IAccessor
  virtual std::string name() const;
  virtual Slash::Data::IIterator* iterator();
  virtual Slash::Core::IValue* findValue(Slash::Data::IAccessor::Data,const std::string&,void*);
public: //OnX::IAccessor
  virtual void visualize(Slash::Data::IAccessor::Data,void*);
private:
  std::string fName;
  std::vector<Shape*>& fShapes;
};
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeAccessor::ShapeAccessor(
 Slash::Core::ISession& aSession
,std::vector<Shape*>& aShapes
)
:InventorAccessor(aSession)
,fName("OnX_Shape")
,fShapes(aShapes)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("type",Lib::Property::STRING);
  addProperty("size",Lib::Property::DOUBLE);
  addProperty("x",Lib::Property::DOUBLE);
  addProperty("y",Lib::Property::DOUBLE);
  addProperty("z",Lib::Property::DOUBLE);
  addProperty("hits",Lib::Property::VECTOR_DOUBLE);
  addProperty("address",Lib::Property::POINTER);
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeAccessor::~ShapeAccessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::ShapeAccessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnX {
class ShapeAccessorIterator : public virtual Slash::Data::IIterator {
public: //Slash::Data::IIterator
  virtual Slash::Data::IAccessor::Data object() {
    if(fIndex>=fNumber) return 0;
    return fList[fIndex];
  }
  virtual void next() { fIndex++;}
  virtual void* tag() { return 0;}
public:
  ShapeAccessorIterator(std::vector<OnX::Shape*>& aList)
  :fIndex(0),fList(aList),fNumber(aList.size()) {
    Lib::Debug::increment("OnX::ShapeAccessorIterator");
  }
  virtual ~ShapeAccessorIterator() {
    Lib::Debug::decrement("OnX::ShapeAccessorIterator");
  }
private:
  unsigned int fIndex;
  std::vector<OnX::Shape*>& fList;
  unsigned int fNumber;
};
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnX::ShapeAccessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Emulated a HEP "Event" with tracks :
  Lib::RandomFlat r;
  double s = 100; //scale.
  for(unsigned int index=0;index<fShapes.size();index++) {
    if(Lib::smanip::match(fShapes[index]->type(),"Track*")) {
      fShapes[index]->setSize(s*r.shoot(),s*r.shoot(),s*r.shoot());
    }
  }
  return new ShapeAccessorIterator(fShapes);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnX::ShapeAccessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Shape* obj = (Shape*)aData;
  if(aName=="address") {
    return new Lib::Value((void*)obj);
  } else if(aName=="type") {
    return new Lib::Value(obj->type());
  } else if(aName=="size") {
    return new Lib::Value(obj->size());
  } else if(aName=="x") {
    return new Lib::Value(obj->x());
  } else if(aName=="y") {
    return new Lib::Value(obj->y());
  } else if(aName=="z") {
    return new Lib::Value(obj->z());
  } else if(aName=="hits") {
    unsigned int order = 10;
    std::vector<double> vec(order);
    for(unsigned int index=0;index<order;index++) vec[index] = double(index);
    return new Lib::Value(vec);
  } else {
    return new Lib::Value();
  }
}

// Inventor :
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoLineSet.h>

// HEPVis :
#include <HEPVis/SbPolyhedron.h>
#include <HEPVis/nodes/SoPolyhedron.h>
#include <HEPVis/nodes/SoTextTTF.h>
#include <HEPVis/nodes/SoHighlightMaterial.h>

//////////////////////////////////////////////////////////////////////////////
void OnX::ShapeAccessor::visualize(
 Slash::Data::IAccessor::Data aData
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion) return;

  Shape* obj = (Shape*)aData;

  std::string style = "OnX_Shape_"+obj->type();
  fillSoGC(style); //Get a style corresponding to shape type.

  // Build scene graph :
  SoSeparator* separator = new SoSeparator;
  separator->setName("sceneGraph");

  separator->addChild(fSoGC.getHighlightMaterial());
  separator->addChild(fSoGC.getDrawStyle());
  separator->addChild(fSoGC.getLightModel());

  // Build name (for picking) :
  std::string s;
  Lib::smanip::printf(s,128,"OnX_Shape/0x%lx",(unsigned long)obj);
  SbName name(s.c_str());

  SoTransform* transform = new SoTransform;
  transform->translation.setValue(SbVec3f((float)obj->x(),
                                          (float)obj->y(),
                                          (float)obj->z()));
  const SbString& modeling = fSoGC.getModeling();
  if(obj->type()=="Cube") {
    separator->addChild(transform);
    if(modeling==SbModeling_texts) {
      SoTextTTF* text = new SoTextTTF;
      //text->rescale.setValue(TRUE);
      text->size.setValue(5);
      text->string.set1Value(0,"Cube");
      text->setName(name);
      separator->addChild(text);
    } else {
/*
      SoCube* cube = new SoCube;
      cube->setName(name);
      cube->width = (float)obj->size();
      cube->height = (float)obj->size();
      separator->addChild(cube);
      cube->depth = (float)obj->size();
*/
      //To have shape cut.
      float sz2 = (float)obj->size()/2.F;
      SbPolyhedronBox box(sz2,sz2,sz2);
      SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(box);
      soPolyhedron->setName(name);
      separator->addChild(soPolyhedron);
    }
  } else if(obj->type()=="Sphere") {
    separator->addChild(transform);
    if(modeling==SbModeling_texts) {
      SoTextTTF* text = new SoTextTTF;
      text->setName(name);
      text->string.set1Value(0,"Sphere");
      text->size.setValue(20);
      // Ask to the node to rescale the text according the viewport size.
      // 200 is the viewport size for the wanted upper size.
      text->rescale.setValue(200);
      separator->addChild(text);
    } else {
/*
      SoSphere* sphere = new SoSphere;
      sphere->setName(name);
      sphere->radius = (float)obj->size();
      separator->addChild(sphere);
*/
      //To have shape cut.
      float radius = (float)obj->size();
      SbPolyhedronSphere sphere(0,radius,0,2*M_PI,0,M_PI);
      SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(sphere);
      soPolyhedron->setName(name);
      separator->addChild(soPolyhedron);
    }
  } else if(obj->type()=="Cone") {
    separator->addChild(transform);
    //WARNING : shape cut does not apply on SoCone.
    SoCone* cone = new SoCone;
    cone->setName(name);
    cone->bottomRadius = (float)obj->size();
    cone->height = (float)obj->size();
    separator->addChild(cone);
  } else if(obj->type()=="Cylinder") {
    separator->addChild(transform);
    //WARNING : shape cut does not apply on SoCone.
    SoCylinder* cylinder = new SoCylinder;
    cylinder->setName(name);
    cylinder->radius = (float)obj->size();
    cylinder->height = (float)obj->size();
    separator->addChild(cylinder);
  } else if(obj->type()=="Hole") {
    separator->addChild(transform);

    /* A flat box with a hole :
    SbPolyhedronBox box(1,1,0.1);
    SbPolyhedronTubs tubs(0,1,1,0,2*M_PI);
    SbPolyhedron op = box.subtract(tubs);
    */
    float scale = (float)obj->size();

    // A Tube with a transverse hole :
    SbPolyhedronTubs tubs_1(scale*0.7,scale*1.5,scale*2,0,2*M_PI);
    SbPolyhedronTubs tubs_2(scale*0,scale*0.5,scale*4,0,2*M_PI);
    tubs_2.Transform
      (HEPVis::SbRotation(SbVec3d(0,1,0),M_PI/2.),SbVec3d(0,0,0));
    SbPolyhedron op = tubs_1.subtract(tubs_2);

    SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(op);
    soPolyhedron->setName(name);
    separator->addChild(soPolyhedron);

  } else if(obj->type().substr(0,5)=="Track") {
    transform->unref();

    int32_t pointn = 2;
    SbVec3f points[2];
    points[0].setValue(0,0,0);
    points[1].setValue((float)obj->x(),(float)obj->y(),(float)obj->z());
 
    SoCoordinate3* coordinate3 = new SoCoordinate3;
    coordinate3->point.setValues(0,pointn,points);
    separator->addChild(coordinate3);

    SoLineSet* lineSet = new SoLineSet;
    lineSet->numVertices.setValues(0,1,&pointn);
    lineSet->setName(name);
    separator->addChild(lineSet);
  }
  
  fSoRegion->doIt(SbAddNode(separator,"dynamicScene"));
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// OnX callback to declare the ShapeManager to the Slash::Core::ISession 
// and declare ShapeAccessor to the accessor manager (access through 
// the Slash::Core::ISession).
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <Slash/Data/IProcessor.h>

//extern "C" {
//  void OnX_shape_type_initialize(Slash::UI::IUI&); // To check old "Types".
//}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_shape_initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed before any use of data_collect("OnX_Shape").
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  //if(session.verboseLevel()) ::atexit(OnX_atexit);

  OnX::ShapeManager* shapeManager = 
    Lib_findManager(session,"OnX_ShapeManager",OnX::ShapeManager);
  if(!shapeManager) {
    shapeManager = new OnX::ShapeManager(session);
    session.addManager(shapeManager);
  }

  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
  if(accessorManager && !accessorManager->findAccessor("OnX_Shape")) {
    accessorManager->addAccessor
      (new OnX::ShapeAccessor(session,shapeManager->shapes()));
  }

  //To test OSC_15.
  //OnX_shape_type_initialize(aUI);

}

}
