// To check that code using old "Type" classes still work.

#include "Shape.h" // "user" class to declare to the AccessorManager.

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// The Shape Type permits to manipulate the data 
// (with data_collect) and permits to order visualization (with data_visualize)
// of them. See OnX/source/Callbacks/Inventor.cxx/OnX_Examples_Shapes callback.
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Inheritance :
#include <Lib/Interfaces/IIterator.h>
#include <OnX/Inventor/InventorType.h>

#include <Lib/Property.h>
#include <Lib/Variable.h>
#include <Lib/smanip.h>
#include <Lib/Randoms.h>

namespace OnX {
class ShapeType : public OnX::InventorType {
public:
  ShapeType(ISession&,const std::vector<Shape*>&);
  virtual ~ShapeType();
public: //Lib::IType
  //virtual const std::string& name() const;
  virtual std::string name() const;
  virtual Lib::IIterator* iterator();
  virtual Lib::Variable value(Lib::Identifier,const std::string&,void*);
public: //OnX::IType
  virtual void visualize(Lib::Identifier,void*);
private:
  std::string fName;
  std::vector<Shape*> fShapes;
};
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeType::ShapeType(
 ISession& aSession
,const std::vector<Shape*>& aShapes
)
:InventorType(aSession)
,fName("OnX_Shape_old")
,fShapes(aShapes)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("type",Lib::Property::STRING);
  addProperty("size",Lib::Property::DOUBLE);
  addProperty("x",Lib::Property::DOUBLE);
  addProperty("y",Lib::Property::DOUBLE);
  addProperty("z",Lib::Property::DOUBLE);
  addProperty("address",Lib::Property::POINTER);
}
//////////////////////////////////////////////////////////////////////////////
OnX::ShapeType::~ShapeType(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fShapes.size();
  for(unsigned int index=0;index<number;index++) delete fShapes[index];
}
//////////////////////////////////////////////////////////////////////////////
//const std::string& OnX::ShapeType::name(
std::string OnX::ShapeType::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnX {
class ShapeTypeIterator : public virtual Lib::IIterator {
public: //Lib::IIterator
  virtual Lib::Identifier object() {
    if(fIndex>=fNumber) return 0;
    return fList[fIndex];
  }
  virtual void next() { fIndex++;}
  virtual void* tag() { return 0;}
public:
  ShapeTypeIterator(std::vector<OnX::Shape*>& aList)
  :fIndex(0),fList(aList),fNumber(aList.size()) {}
private:
  unsigned int fIndex;
  std::vector<OnX::Shape*>& fList;
  unsigned int fNumber;
};
}
//////////////////////////////////////////////////////////////////////////////
Lib::IIterator* OnX::ShapeType::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Emulated a HEP "Event" with tracks :
  Lib::RandomFlat r;
  double s = 100; //scale.
  for(unsigned int index=0;index<fShapes.size();index++)
    fShapes[index]->setSize(s*r.shoot(),s*r.shoot(),s*r.shoot());
  return new ShapeTypeIterator(fShapes);
}
//////////////////////////////////////////////////////////////////////////////
Lib::Variable OnX::ShapeType::value(
 Lib::Identifier aIdentifier
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Shape* obj = (Shape*)aIdentifier;
  if(aName=="address") {
    return Lib::Variable(printer(),(void*)obj);
  } else if(aName=="type") {
    return Lib::Variable(printer(),obj->type());
  } else if(aName=="size") {
    return Lib::Variable(printer(),obj->size());
  } else if(aName=="x") {
    return Lib::Variable(printer(),obj->x());
  } else if(aName=="y") {
    return Lib::Variable(printer(),obj->y());
  } else if(aName=="z") {
    return Lib::Variable(printer(),obj->z());
  } else {
    return Lib::Variable(printer());
  }
}

// Inventor :
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoLineSet.h>

// HEPVis :
#include <HEPVis/SbPolyhedron.h>
#include <HEPVis/nodes/SoPolyhedron.h>
#include <HEPVis/nodes/SoTextTTF.h>
#include <HEPVis/nodes/SoHighlightMaterial.h>

//////////////////////////////////////////////////////////////////////////////
void OnX::ShapeType::visualize(
 Lib::Identifier aIdentifier
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion) return;

  Shape* obj = (Shape*)aIdentifier;

  std::string style = "OnX_Shape_"+obj->type();
  fillSoGC(style); //Get a style corresponding to shape type.

  // Build scene graph :
  SoSeparator* separator = new SoSeparator;
  separator->setName("sceneGraph");

  separator->addChild(fSoGC.getHighlightMaterial());
  separator->addChild(fSoGC.getDrawStyle());
  separator->addChild(fSoGC.getLightModel());

  // Build name (for picking) :
  std::string s;
  Lib::smanip::printf(s,128,"OnX_Shape/0x%lx",(unsigned long)obj);
  SbName name(s.c_str());

  SoTransform* transform = new SoTransform;
  transform->translation.setValue(SbVec3f((float)obj->x(),
                                          (float)obj->y(),
                                          (float)obj->z()));
  const SbString& modeling = fSoGC.getModeling();
  if(obj->type()=="Cube") {
    separator->addChild(transform);
    if(modeling==SbModeling_texts) {
      SoTextTTF* text = new SoTextTTF;
      //text->rescale.setValue(TRUE);
      text->size.setValue(5);
      text->string.set1Value(0,"Cube");
      text->setName(name);
      separator->addChild(text);
    } else {
/*
      SoCube* cube = new SoCube;
      cube->setName(name);
      cube->width = (float)obj->size();
      cube->height = (float)obj->size();
      separator->addChild(cube);
      cube->depth = (float)obj->size();
*/
      //To have shape cut.
      float sz2 = (float)obj->size()/2.F;
      SbPolyhedronBox box(sz2,sz2,sz2);
      SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(box);
      soPolyhedron->setName(name);
      separator->addChild(soPolyhedron);
    }
  } else if(obj->type()=="Sphere") {
    separator->addChild(transform);
    if(modeling==SbModeling_texts) {
      SoTextTTF* text = new SoTextTTF;
      text->setName(name);
      text->string.set1Value(0,"Sphere");
      text->size.setValue(20);
      // Ask to the node to rescale the text according the viewport size.
      // 200 is the viewport size for the wanted upper size.
      text->rescale.setValue(200);
      separator->addChild(text);
    } else {
/*
      SoSphere* sphere = new SoSphere;
      sphere->setName(name);
      sphere->radius = (float)obj->size();
      separator->addChild(sphere);
*/
      //To have shape cut.
      float radius = (float)obj->size();
      SbPolyhedronSphere sphere(0,radius,0,2*M_PI,0,M_PI);
      SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(sphere);
      soPolyhedron->setName(name);
      separator->addChild(soPolyhedron);
    }
  } else if(obj->type()=="Cone") {
    separator->addChild(transform);
    //WARNING : shape cut does not apply on SoCone.
    SoCone* cone = new SoCone;
    cone->setName(name);
    cone->bottomRadius = (float)obj->size();
    cone->height = (float)obj->size();
    separator->addChild(cone);
  } else if(obj->type()=="Cylinder") {
    separator->addChild(transform);
    //WARNING : shape cut does not apply on SoCone.
    SoCylinder* cylinder = new SoCylinder;
    cylinder->setName(name);
    cylinder->radius = (float)obj->size();
    cylinder->height = (float)obj->size();
    separator->addChild(cylinder);
  } else if(obj->type()=="Hole") {
    separator->addChild(transform);

    /* A flat box with a hole :
    SbPolyhedronBox box(1,1,0.1);
    SbPolyhedronTubs tubs(0,1,1,0,2*M_PI);
    SbPolyhedron op = box.subtract(tubs);
    */
    float scale = (float)obj->size();

    // A Tube with a transverse hole :
    SbPolyhedronTubs tubs_1(scale*0.7,scale*1.5,scale*2,0,2*M_PI);
    SbPolyhedronTubs tubs_2(scale*0,scale*0.5,scale*4,0,2*M_PI);
    tubs_2.Transform
      (HEPVis::SbRotation(SbVec3d(0,1,0),M_PI/2.),SbVec3d(0,0,0));
    SbPolyhedron op = tubs_1.subtract(tubs_2);

    SoPolyhedron* soPolyhedron = fSoGC.getPolyhedron(op);
    soPolyhedron->setName(name);
    separator->addChild(soPolyhedron);

  } else if(obj->type().substr(0,5)=="Track") {
    transform->unref();

    int32_t pointn = 2;
    SbVec3f points[2];
    points[0].setValue(0,0,0);
    points[1].setValue((float)obj->x(),(float)obj->y(),(float)obj->z());
 
    SoCoordinate3* coordinate3 = new SoCoordinate3;
    coordinate3->point.setValues(0,pointn,points);
    separator->addChild(coordinate3);

    SoLineSet* lineSet = new SoLineSet;
    lineSet->numVertices.setValues(0,1,&pointn);
    lineSet->setName(name);
    separator->addChild(lineSet);
  }
  
  fSoRegion->doIt(SbAddNode(separator,"dynamicScene"));
}

#include <Slash/Data/IProcessor.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_shape_type_initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed before any use of data_collect("OnX_Shape").
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
  if(accessorManager && !accessorManager->findAccessor("OnX_Shape_old")) {

    std::vector<OnX::Shape*> shapes;
    // Create some data (deleted by OnX::ShapeType.
    shapes.push_back(new OnX::Shape("Cube",1, 10, 10,0));
    shapes.push_back(new OnX::Shape("Cube",2,-10, 10,0));
    shapes.push_back(new OnX::Shape("Cube",3,-10,-10,0));
    shapes.push_back(new OnX::Shape("Cube",4, 10,-10,0));
    shapes.push_back(new OnX::Shape("Sphere",1,0, 10, 10));
    shapes.push_back(new OnX::Shape("Sphere",2,0,-10, 10));
    shapes.push_back(new OnX::Shape("Sphere",3,0,-10,-10));
    shapes.push_back(new OnX::Shape("Sphere",4,0, 10,-10));
    shapes.push_back(new OnX::Shape("Cylinder",1, 10,0, 10));
    shapes.push_back(new OnX::Shape("Cylinder",2,-10,0, 10));
    shapes.push_back(new OnX::Shape("Cylinder",3,-10,0,-10));
    shapes.push_back(new OnX::Shape("Cylinder",5, 10,0,-10));
    shapes.push_back(new OnX::Shape("Cone",1,0,0,0));
    shapes.push_back(new OnX::Shape("Hole",1,10,10,10));
    shapes.push_back(new OnX::Shape("Hole",1,-10,-10,-10));
    // Have one centered at origin to test shape cuts :
    shapes.push_back(new OnX::Shape("Hole",2,0,0,0));

    accessorManager->addAccessor(new OnX::ShapeType(session,shapes));
  }
}

}
