//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
//  This file does not need HEPVis and Inventor to compile.
//

// Helpers :
#include <OnX/Helpers/OnX.h>

// OnX :
#include <Slash/UI/ICyclic.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// OnX/examples/OnX/Test.onx ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_toggle(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.echo("callbackValue() : "+aUI.callbackValue());
  std::string svalue;
  aUI.parameterValue("toggle_1.set",svalue);
  aUI.echo("parameterValue(toggle_1.set) : "+svalue);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_set_toggle(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool value;
  if(!ui_parameterValue(aUI,"toggle_1.set",value)) return;
  if(value) aUI.setParameter("toggle_1.set","false");
  else aUI.setParameter("toggle_1.set","true");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_tree(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  tree_set_with_file(aUI,"mainTree.items","$ONXROOT/scripts/OnX/Tree.xml");
  //tree_set_with_strings(aUI,"mainTree.items",aUI.session().managerNames(""));
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_add_menu(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create a new menu in the default menu bar :
  aUI.createComponent("onx_exa_casc","CascadeButton","menuBar");
  aUI.setParameter("onx_exa_casc.label","MyMenu");
  aUI.createComponent("onx_exa_menu","Menu","onx_exa_casc");

  // Create a first item in the menu :
  aUI.createComponent("onx_exa_item_1","MenuItem","onx_exa_menu");
  aUI.setParameter("onx_exa_item_1.label","Item 1");
  aUI.setCallback("onx_exa_item_1","activate","DLD","OnX ui_echo Item 1");

  // Create a second item in the menu :
  aUI.createComponent("onx_exa_item_2","MenuItem","onx_exa_menu");
  aUI.setParameter("onx_exa_item_2.label","Item 2");
  aUI.setCallback("onx_exa_item_2","activate","DLD","OnX ui_echo Item 2");
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_progressBar_start(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnX_Test_progressBar_cyclic");
  if(!cyclic) {
    cyclic = aUI.createCyclic("OnX_Test_progressBar_cyclic",
             250,"DLD","OnX OnX_Test_progressBar_update");
  }
  if(!cyclic) return;
  cyclic->start();
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_progressBar_update(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnX_Test_progressBar_cyclic");
  if(!cyclic) return;  
  std::string svalue;
  int number = cyclic->iterations();
  // Be in [0,100] :
  number = number - 100 * (int)(number/100);
  Lib::smanip::printf(svalue,32,"%d",number);
  aUI.setParameter("OnX_TestProgress_progressBar.value",svalue);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_progressBar_stop(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnX_Test_progressBar_cyclic");
  if(!cyclic) return;  
  cyclic->stop();
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_input_ok(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string svalue;
  aUI.parameterValue("OnX_TestInput_input_xxx_1.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_2.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_3.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_4.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_5.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_6.value",svalue);
  aUI.echo(svalue);
  aUI.parameterValue("OnX_TestInput_input_xxx_7.value",svalue);
  aUI.echo(svalue);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_tab_tree(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path = aUI.callbackValue();
  std::vector<std::string> items;
  Lib::smanip::lines(path,items);
  aUI.setParameter("OnX_TestTabStack_text.value","item :"+items[items.size()-1]+"\npath :\n"+path);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_Test_tab_selection(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path;
  if(!aUI.parameterValue("OnX_TestTabStack_tree.selection",path)) return;
  std::vector<std::string> items;
  Lib::smanip::lines(path,items);
  aUI.setParameter("OnX_TestTabStack_text.value","item :"+items[items.size()-1]+"\npath :\n"+path);
}

}

#include <stdio.h>
#include <iostream>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_Test_output(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  
  aUI.echo("UI.echo 1");
  ::printf("printf 1\n");
  std::cout << "cout 1" << std::endl;
  session.flush();

  aUI.echo("UI.echo 2");
  ::printf("printf 2\n");
  std::cout << "cout 2" << std::endl;
  session.flush();
}

}
