//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(IUI&);
//     void callback_with_arguments(IUI&,const std::vector<std::string>&);
//   }
//


// Lib :
#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/ItemML.h>
#include <Lib/BaseML.h>
#include <Lib/Interfaces/ISession.h>

// OnX :
#include <OnX/Interfaces/IUI.h>

#define getPropertyValue propertyValue

// Emulate Gaudi LHCb begin of transient store :
inline std::vector<std::string> uiSvc_dataChildren(
 ISession& aSession
,const std::string& aPath
) {
  std::vector<std::string> v;

  // /dd :
  if(aPath=="/dd") {
    v.push_back(aPath+"/"+"Structure");
    v.push_back(aPath+"/"+"Geometry");
  } else if(aPath=="/dd/Structure") {
    v.push_back(aPath+"/"+"LHCb");
  } else if(aPath=="/dd/Structure/LHCb") {
    v.push_back(aPath+"/"+"BeforeMagnet");
    v.push_back(aPath+"/"+"AfterMagnet");
  } else if(aPath=="/dd/Structure/LHCb/BeforeMagnet") { //Leaf
  } else if(aPath=="/dd/Structure/LHCb/AfterMagnet") { //Leaf

  } else if(aPath=="/dd/Geometry") {
    v.push_back(aPath+"/"+"LHCb");
  } else if(aPath=="/dd/Geometry/LHCb") { //Leaf

  } else if(aPath=="/Event") {
    v.push_back(aPath+"/"+"MC");
    v.push_back(aPath+"/"+"Rec");
  } else if(aPath=="/Event/MC") {
    v.push_back(aPath+"/"+"Particles");
    v.push_back(aPath+"/"+"Hits");
  } else if(aPath=="/Event/MC/Particles") { //Leaf
  } else if(aPath=="/Event/MC/Hits") { //Leaf
  } else if(aPath=="/Event/Rec") {
    v.push_back(aPath+"/"+"Particles");
    v.push_back(aPath+"/"+"Hits");
  } else if(aPath=="/Event/Rec/Particles") { //Leaf
  } else if(aPath=="/Event/Rec/Hits") { //Leaf

  } else { 
    Lib::Out out(aSession.printer());
    out << "uiSvc_dataChildren :" 
        << " unknown path " << Lib::sout(aPath)
        << Lib::endl;
  }

  return v;
}

//////////////////////////////////////////////////////////////////////////////
inline Lib::ItemML* Lib_ItemML_findItemFromPath(
 Lib::ItemML& aItemML
,const std::vector<std::string>& aPath
,unsigned int& aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aIndex>=aPath.size()) return 0;
  std::string s;
  if(aItemML.getPropertyValue("label",s)) { //if ! it is a <tree>, continue.
    if(aPath[aIndex]!=s) return 0; //Bad path, do not jump in children.
    // Matching item.
    if(aIndex==(aPath.size()-1)) return &(aItemML); //End of aPath.
    aIndex++;
  }
  // Look children :
  const std::list<Lib::ItemML*>& children = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=children.begin();it!=children.end();++it) {
    Lib::ItemML* item = Lib_ItemML_findItemFromPath(*(*it),aPath,aIndex);
    if(item) return item;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
inline bool Lib_ItemML_visitToXML(
 Lib::ItemML& aItemML
,std::string& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  bool hasLabel = aItemML.getPropertyValue("label",s);
  // !hasLabel if head <tree>.
  if(hasLabel) {
    aOut +=  "<treeItem>";
    aOut +=  "<label>" + s + "</label>";
  }
  const std::list<Lib::ItemML*>& children = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=children.begin();it!=children.end();++it) {
    if(!Lib_ItemML_visitToXML(*(*it),aOut)) return false;
  }
  if(hasLabel) aOut += "</treeItem>";
  return true;
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnX_TestTree_initialize(
 IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.setCallback("mainTree","open","DLD",
		  "OnX OnX_TestTree_tree_open_branch");
  aUI.setCallback("mainTree","close","DLD",
		  "OnX OnX_TestTree_tree_close_branch");

  std::string out;
  out += "<tree>";
  out += "  <treeItem>";
  out += "    <label>dd</label>";
  out += "    <treeItem><label></label></treeItem>";
  out += "  </treeItem>";
  out += "  <treeItem>";
  out += "    <label>Event</label>";
  out += "    <treeItem><label></label></treeItem>";
  out += "  </treeItem>";
  out += "</tree>";
  aUI.setParameter("mainTree.items",out);

}
//////////////////////////////////////////////////////////////////////////////
void OnX_TestTree_tree_open_branch(
 IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  ISession& session = aUI.session();

  std::string branch = aUI.callbackValue();
  //printf("debug : OnX_TestTree_tree_open_branch : branch \"%s\"\n",
  //branch.c_str());

  std::vector<std::string> items;
  Lib::smanip::words(branch,"\n",items,true);
  if(!items.size()) return;

  Lib::smanip::replace(branch,"\n","/");
  std::string s = "/" + branch;
  //aUI.echo("OnX_TestTree_tree_open_branch : " + s);
  std::vector<std::string> text = uiSvc_dataChildren(session,s);
  if(!text.size()) return; //no children.

  std::string svalue;
  aUI.parameterValue("mainTree.items",svalue);
  //printf("debug : items : \"%s\"\n",svalue.c_str());

  Lib::ItemMLFactory factory;
  Lib::BaseML treeML(factory);
  std::vector<std::string> tags;
  tags.push_back("tree");
  tags.push_back("treeItem");
  treeML.setTags(tags);
  if(!treeML.loadString(svalue,session.printer())) return;
  Lib::ItemML* top = treeML.topItem();
  if(!top) return;
  unsigned int pathi = 0;
  Lib::ItemML* branchML = Lib_ItemML_findItemFromPath(*top,items,pathi);
  if(!branchML) return;
  branchML->deleteChildren();
  for(unsigned int index=0;index<text.size();index++) {
    std::vector<Lib::ItemML::Attribute> atbs;
    Lib::ItemML* childML = factory.create("",atbs,branchML);
    if(!childML) return;
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(text[index],p,n,s);
    childML->addProperty("label",atbs,n);
    branchML->addChild(childML);

    if(uiSvc_dataChildren(session,text[index]).size()) { // Is it a leaf ?
      // Add an empty item for the moment :
      std::vector<Lib::ItemML::Attribute> atbs;
      Lib::ItemML* subChildML = factory.create("",atbs,childML);
      if(!subChildML) return;
      subChildML->addProperty("label",atbs,"");
      childML->addChild(subChildML);
    }
  }

  std::string out = "<tree>";
  if(!Lib_ItemML_visitToXML(*top,out)) return;
  out += "</tree>";

  aUI.setParameter("mainTree.items",out);
}
//////////////////////////////////////////////////////////////////////////////
void OnX_TestTree_tree_close_branch(
 IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  ISession& session = aUI.session();

  std::string branch = aUI.callbackValue();
  //printf("debug : OnX_TestTree_tree_close_branch : branch\"%s\"\n",
  //branch.c_str());

  std::vector<std::string> items;
  Lib::smanip::words(branch,"\n",items,true);
  if(!items.size()) return;

  std::string svalue;
  aUI.parameterValue("mainTree.items",svalue);
  //printf("debug : items : \"%s\"\n",svalue.c_str());

  Lib::ItemMLFactory factory;
  Lib::BaseML treeML(factory);
  std::vector<std::string> tags;
  tags.push_back("tree");
  tags.push_back("treeItem");
  treeML.setTags(tags);
  if(!treeML.loadString(svalue,session.printer())) return;
  Lib::ItemML* top = treeML.topItem();
  if(!top) return;
  unsigned int pathi = 0;
  Lib::ItemML* branchML = Lib_ItemML_findItemFromPath(*top,items,pathi);
  if(!branchML) return;
  branchML->deleteChildren();

 {// Add an empty item for the moment :
  //Lib::ItemML* subChildML = factory.create("",childML);
  std::vector<Lib::ItemML::Attribute> atbs;
  Lib::ItemML* subChildML = factory.create("",atbs,branchML);
  if(!subChildML) return;
  //subChildML->addProperty("label",std::vector<std::string>(),"");
  subChildML->addProperty("label",atbs,"");
  branchML->addChild(subChildML);}

  std::string out = "<tree>";
  if(!Lib_ItemML_visitToXML(*top,out)) return;
  out += "</tree>";

  aUI.setParameter("mainTree.items",out);
}

} // extern "C"

