//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

// Helpers :
#include <OnX/Helpers/OnX.h>

#include <Slash/UI/ICallbackData.h>
#include <Slash/UI/ISoViewer.h>

// Lib :
#include <Lib/System.h>

//////////////////////////////////////////////////////////////////////////////
inline void ui_setCurrentViewer(
 Slash::UI::IUI& aUI
,const std::string& aViewer
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Check that the widget is a viewer :
  Slash::UI::IWidget* widget = aUI.findWidget(aViewer);
  if(!widget) {
    Lib::Out out(aUI.session().printer());
    out << "ui_setCurrentViewer :" 
        << "Can't find widget " << Lib::sout(aViewer)
        << Lib::endl;
    return;
  }
  Slash::UI::ISoViewer* soViewer = Lib_Cast(*widget,Slash::UI::ISoViewer);
  if(!soViewer) {
    //Lib::Out out(aUI.session().printer());
    //out << "ui_setCurrentViewer :" 
    //    << " widget " << Lib::sout(aViewer)
    //    << " not a Slash::UI::ISoViewer."
    //    << Lib::endl;
    return;
  }
  aUI.setCurrentWidget(widget);
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// viewer ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void viewer_collect(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  data_collect(session,"SceneGraph","highlight==true");
  data_filter(session,"name");
  std::string svalue;
  session.parameterValue("AccessorManager.dumpMode",svalue);
  data_dump(session,svalue);
}
//////////////////////////////////////////////////////////////////////////////
void viewer_set_current(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  ui_setCurrentViewer(aUI,aArgs[0]);
}
//////////////////////////////////////////////////////////////////////////////
void viewer_popup(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string item = aUI.callbackValue();
  if(item == "No highlighted") {
    aUI.executeScript("DLD","OnX region_reset_highlight");
  } else if(item == "Dump highlighted") {
    viewer_collect(aUI);
  } else if(item == "Region to page") {
    aUI.executeScript("DLD","OnX page_currentRegionToPage");
  } else if(item == "Region on top") {
    aUI.executeScript("DLD","OnX page_currentRegionOnTop");
  } else if(item == "Decorations on") {
    aUI.executeScript("DLD","OnX ui_setParameter viewer.decoration TRUE");
  } else if(item == "Decorations off") {
    aUI.executeScript("DLD","OnX ui_setParameter viewer.decoration FALSE");
  }
}
//////////////////////////////////////////////////////////////////////////////
void viewer_setParameter(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// args[0] what
// args[1,n-1] : partial value
//  value is built with : args[1] + " " + args[2] + ..... + args[n-1]
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;
  std::string value;
  if(aArgs.size()!=1) {
    value = ui_cbk(aUI,aArgs[1]);
    for(unsigned int index=2;index<aArgs.size();index++) 
      value += " " + ui_cbk(aUI,aArgs[index]);
  }
  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) return;
  if(!Lib_Cast(*widget,Slash::UI::ISoViewer)) return;
  widget->setParameter(aArgs[0],value);
}
//////////////////////////////////////////////////////////////////////////////
void page_create(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name = aUI.callbackData().component();
  //aUI.echo("page_create for : "+name);

  // Add in the Viewers pdm :
  if(aUI.isComponent("viewers")) {
    std::string s = "viewers_item_" + name;
    aUI.createComponent(s,"MenuItem","viewers");
    aUI.setParameter(s+".label",name);
    aUI.setCallback(s,"activate","DLD","OnX viewer_set_current "+name);
  }

  // Set currentViewer :
  ui_setCurrentViewer(aUI,name);
}
//////////////////////////////////////////////////////////////////////////////
void page_destroy(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //if(aUI.beingDestroyed()) return; //No more needed since IUI::finalize().

  std::string name = aUI.callbackData().component();
  //aUI.echo("page_destroy for : "+name);

  // Remove from Viewers pdm :
 {std::string s = "viewers_item_" + name;
  if(aUI.isComponent(s)) aUI.destroyComponent(s);}

  // Set currentViewer to another PageViewer.
 {std::vector<std::string> viewers = 
    aUI.findChildren("main","nameFromClass","PageViewer");
  bool found = false;
  if(viewers.size()) {  
    for(unsigned int index=0;index<viewers.size();index++) {
      if(viewers[index]!=name) {
        ui_setCurrentViewer(aUI,viewers[index]);
        found = true;
      }
    }
  } 
  if(!found) {
    ui_setCurrentViewer(aUI,"");
  }}

}

}
