// this :
#include <OnX/Core/Action.h>

#include <Lib/Debug.h>
#include <Lib/Cast.h>
#include <Lib/smanip.h>

#include <Slash/UI/IUI.h>

//////////////////////////////////////////////////////////////////////////////
OnX::Action::Action(
 Slash::UI::IUI& aUI
,Slash::UI::IWidget* aWidget
,NativeWidget aComponent
,const std::string& aName
,int aID
,const std::string& aInterpreter
,const std::string& aScript
)
:fUI(aUI)
,fWidget(aWidget)
,fNativeWidget(aComponent)
,fName(aName)
,fID(aID)
,fInterpreter(aInterpreter)
,fScript(aScript)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnX::Action");
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action::Action(
 Slash::UI::IUI& aUI
//,NativeWidget aComponent
,const std::string& aName
,int aID
,const std::string& aInterpreter
,const std::string& aScript
,const CallbackData& aCallbackData
)
:fUI(aUI)
,fWidget(0)
,fNativeWidget(0)
,fName(aName)
,fID(aID)
,fInterpreter(aInterpreter)
,fScript(aScript)
,fCallbackData(aCallbackData)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnX::Action");
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action::Action(
 const Action& aAction
)
:IAction(aAction)
,fUI(aAction.fUI)
,fWidget(aAction.fWidget)
,fNativeWidget(aAction.fNativeWidget)
,fName(aAction.fName)
,fID(aAction.fID)
,fInterpreter(aAction.fInterpreter)
,fScript(aAction.fScript)
,fCallbackData(aAction.fCallbackData)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnX::Action");
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action::~Action(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("OnX::Action");
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::Action::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(OnX::Action)
  else if_Lib_SCast(Slash::Core::IAction)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action& OnX::Action::operator=(
 const Action& aAction
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //IRef : fUI = aAction.fUI;
  fWidget = aAction.fWidget;
  fNativeWidget = aAction.fNativeWidget;
  fName = aAction.fName;
  fID = aAction.fID;
  fInterpreter = aAction.fInterpreter;
  fScript = aAction.fScript;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Action::equal(
 const Action& aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (fName != aAction.fName) return false;
  if (fWidget != aAction.fWidget) return false;
  if (fNativeWidget != aAction.fNativeWidget) return false;
  if (fID != aAction.fID) return false;
  if (fInterpreter != aAction.fInterpreter) return false;
  if (fScript != aAction.fScript) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Action::execute(
 const std::vector<std::string>&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fUI.executeScript(fInterpreter,fScript);
  return "";
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Action::isBackground(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fInterpreter.find(" ")!=std::string::npos ? true : false);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Action::setBackground(
 bool aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aValue) { 
    if(isBackground()) return; //Done.
    fInterpreter = fInterpreter + " bg";
  } else {
    if(!isBackground()) return; //Done.
    std::vector<std::string> words;
    Lib::smanip::words(fInterpreter," ",words);
    fInterpreter = words[0];
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidget* OnX::Action::widget(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fWidget;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action::NativeWidget OnX::Action::nativeWidget(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fNativeWidget;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Action::setID(
 int aID
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fID = aID;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Action::name(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
const OnX::CallbackData& OnX::Action::callbackData(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCallbackData;
}
//////////////////////////////////////////////////////////////////////////////
int OnX::Action::getID(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fID;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI& OnX::Action::ui(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUI;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& OnX::Action::script(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScript;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Action::setScript(
 const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScript = aScript;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Action::setInterpreter(
 const std::string& aInterpreter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInterpreter = aInterpreter;
}
