// this :
#include <OnX/Core/BaseUI.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Slash/Core/IRelationManager.h>
#include <Slash/Core/IRelation.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IScriptManager.h>
#include <Slash/UI/ICyclic.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/IWidgetFactory.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/Messages.h>
#include <Lib/Manager.h>
#include <Lib/System.h>
#include <Lib/FileWriter.h>
#include <Lib/Debug.h>
#include <Lib/Element.h>
#include <Lib/ItemML.h>
#include <Lib/BaseML.h>
#include <Lib/Cast.h>

// OnX :
#include <OnX/Version.h>
#include <OnX/Core/CallbackData.h>
#include <OnX/Core/Messages.h>
#include <OnX/Core/WidgetClass.h>
#include <OnX/Core/Widget.h>
#include <OnX/Core/WidgetFactory.h>

#include <algorithm> //std::find

namespace OnX {

class UIML : public Lib::BaseML {
public:
  UIML(WidgetFactory& aFactory,int aVerbose):Lib::BaseML(aFactory,aVerbose){
    std::vector<std::string> tags(4);
    tags[0] = "OnX";
    tags[1] = "widget";
    tags[2] = "reference";
    tags[3] = "comment";
    setTags(tags);
  }
};

}

static bool saveItemML(Lib::ItemML&,const std::string&,int,Slash::Core::IWriter&);
static bool saveItemML(Lib::ItemML&,Lib::Out&,int,int,Slash::Core::IWriter&);
static bool isGeneratedName(const std::string&);


OnX::Execute::Execute(BaseUI& aUI,const std::string& aWhat)
:fUI(aUI),fWhat(aWhat){}
const std::string& OnX::Execute::what() const {return fWhat;}
OnX::BaseUI& OnX::Execute::baseUI(){return fUI;}

#ifdef WIN32
// disable the warning about the usage of "this" in the constructor.
#pragma warning(disable:4355)
#endif

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::BaseUI::BaseUI (
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fTopItemML(0)
,fScriptManager(0)
,fWidgetNameCounter(0)
,fBeingDestroyed(false)
,fVerboseLevel(0)
,fSteerLevel(0)
,fRelationManager(0)
,fCurrentWidget(0)
,fAskToTerminal(false)
//IB
,fWidgetFactory(*this,fSession.printer(),fWidgetClasses)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fScriptManager = Lib_findManager(fSession,"ScriptManager",Slash::UI::IScriptManager);
  fRelationManager = 
    Lib_findManager(fSession,"Lib::RelationManager",Slash::Core::IRelationManager);

  std::string svalue;
  if(fSession.parameterValue("UI_Manager.verbose",svalue)) {
    int value;
    if(Lib::smanip::toint(svalue,value)) fVerboseLevel = value;
  } else {
    fVerboseLevel = fSession.verboseLevel();
  }

  // IB :
  createWidgetTable();
  createLanguageList();
  createPropertyList();
}
//////////////////////////////////////////////////////////////////////////////
OnX::BaseUI::~BaseUI(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  //FIXME : Python : on a Python callback, someone
  //                 may do a sys.exit(0) that will
  //                 delete the ui object whilst in ui.steer() !
  //if(fSteerLevel) {
  //}

  fCurrentWidget = 0;

  // Remove relations concerning this UI :
  if(fRelationManager) fRelationManager->removeWith(this);

  std::list<Slash::UI::ICyclic*>::iterator it;
  for(it=fCyclics.begin();it!=fCyclics.end();it = fCyclics.erase(it)) 
    delete (*it);
 
 {for(unsigned int index=0;index<fFactories.size();index++) {
    delete fFactories[index];
  }
  fFactories.clear();}

  // IB :
 {for(unsigned int index=0;index<fWidgetClasses.size();index++) {
    delete fWidgetClasses[index];
  }
  fWidgetClasses.clear();}

/* Displaced in finalize().
  if(fTopItemML) {

    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "OnX::BaseUI::~BaseUI :"
          << " execute destroyCallbacks..." << Lib::endl;
    }

    Execute tag(*this,"destroy");

    //FIXME : the below heavily uses IUI, then with a concreete
    //        object under destruction...
    fTopItemML->postExecuteBackward(executeItemMLCallbacks,&tag);

    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "OnX::BaseUI::~BaseUI :"
          << " execute destroyCallbacks : end" << Lib::endl;
    }
  }
*/

  delete fTopItemML;
}
//////////////////////////////////////////////////////////////////////////////
int OnX::BaseUI::steerLevel( 
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSteerLevel;
}
//////////////////////////////////////////////////////////////////////////////
int OnX::BaseUI::steer( 
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return 0;
  executeCallbacks(*fTopItemML,"enterSteering");
  int value = nativeSteer();
  executeCallbacks(*fTopItemML,"leaveSteering");
  return value;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::startup(
 const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTopItemML = 0;
  UIML uiml(fWidgetFactory,fVerboseLevel);
  if(!uiml.loadFile(aFile,false,fSession.printer())) {
    return false;
  }
  Lib::ItemML* top = uiml.topItem();
  uiml.empty();
  if(!top) return false;
  if(!solveInputDialog(*top)) {
    delete top;
    return false;
  }
  if(!createNativeWidget(*top)) {
    delete top;
    return false;
  }
  fTopItemML = top;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::finalize(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return true;

  // Disbale terminal here (and not in <Xxx>UI)
  // and then before the destroy callbacks in order 
  // to avoid : DLD> OnX_Finalize.
  disableTerminal(); 

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::BaseUI::finalize :"
        << " execute destroyCallbacks..." << Lib::endl;
  }

  Execute tag(*this,"destroy");

  fTopItemML->postExecuteBackward(executeItemMLCallbacks,&tag);

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::BaseUI::finalize :"
        << " execute destroyCallbacks : end" << Lib::endl;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::beingDestroyed(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fBeingDestroyed;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ISession& OnX::BaseUI::session(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSession;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& OnX::BaseUI::printer(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSession.printer();
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::BaseUI::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::UI::IUI)
  else if_Lib_SCast(IIB)
  else if_Lib_SCast(OnX::BaseUI)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::executeScript(
 const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fScriptManager) return;
  std::vector<std::string> words;
  Lib::smanip::words(aInterp," ",words);
  if(words.size()==1) {
    fScriptManager->executeScript(aInterp,aScript);
  } else if(words.size()==2) {
    if(((words[1]=="bg")||(words[1]=="background")) ){
      if(fVerboseLevel) {
        Lib::Out out(fSession.printer());
        out << "OnX::BaseUI::executeScript :"
            << " post execution of "
            << " " << Lib::sout(words[0])
            << " " << Lib::sout(aScript)
            << Lib::endl;
      }
      // Process in background :
      // Have a copy of fCallbackData in Action.
      Action* action = 
        new Action(*this,"post",0,words[0],aScript,fCallbackData);
      notify(new Lib::ExecuteMessage(*action));
    } else {
      Lib::Out out(fSession.printer());
      out << "OnX::BaseUI::executeScript :"
          << " bad keyword "
          << " " << Lib::sout(words[1])
          << " for script " << Lib::sout(aScript)
          << Lib::endl;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::echo(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Out out(fSession.printer());
  out << aString << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::addDispatcher(
 Slash::UI::IUI::Dispatcher aDispatcher
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(std::find(fDispatchers.begin(),
               fDispatchers.end(),
               aDispatcher)!=fDispatchers.end()) return;
  fDispatchers.push_back(aDispatcher);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::removeDispatcher(
 Slash::UI::IUI::Dispatcher aDispatcher
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fDispatchers.remove(aDispatcher);
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::callbackValue(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCallbackData.value();
}
//////////////////////////////////////////////////////////////////////////////
const Slash::UI::ICallbackData& OnX::BaseUI::callbackData(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCallbackData;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::setCallbackData(
 const Slash::UI::ICallbackData& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCallbackData.copy(aFrom);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::setCallbackData(
 const CallbackData& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCallbackData = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::deleteChildren(
 const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::Action& OnX::BaseUI::addAction(
 Slash::UI::IUI& aUI
,Slash::UI::IWidget* aWidget
,OnX::Action::NativeWidget aComponent
,const std::string& aInterp
,const std::string& aScript
,const std::string& aName
,int aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fActions.push_back(Action(aUI,aWidget,aComponent,aName,aID,aInterp,aScript));
  return fActions.back();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::removeActions(
 OnX::Action::NativeWidget aComponent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();) {
    if((*it).nativeWidget()==aComponent) {
      it = fActions.erase(it);
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::removeAction(
 OnX::Action::NativeWidget aComponent
,const std::string& aName
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int index = 0;
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if( ((*it).nativeWidget()==aComponent) &&
        ((*it).name()==aName) ) {
      if(index==aIndex) {
        fActions.erase(it);
        return;
      }
      index++;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::BaseUI::getAction(
 OnX::Action::NativeWidget aComponent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if((*it).nativeWidget()==aComponent) return &(*it);
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::BaseUI::getAction(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if((*it).name()==aName) return &(*it);
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::BaseUI::getAction(
 int aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if((*it).getID()==aID) return &(*it);
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::BaseUI::getAction(
 OnX::Action::NativeWidget aComponent
,const std::string& aName
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Action>::iterator it;
  int index = 0;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if( ((*it).nativeWidget()==aComponent) &&
        ((*it).name()==aName) ) {
      if(index==aIndex)
        return &(*it);
      else 
        index++;
    }
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::BaseUI::disableCallbackScript(
 OnX::Action::NativeWidget aComponent
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
// Disable the scripts of the aName callback.
// It can be used to avoid recursive loop in case
// the same callback is indirectly triggered in the scripts.
// Usage :
//   std::vector<std::string> tmp = disableCallbackScript(...)
//   // call something which does a callback
//   enableCallbackScript(...,tmp)
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> scripts;
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if( ((*it).nativeWidget()==aComponent) &&
        ((*it).name()==aName) ) {
      scripts.push_back((*it).script());
      (*it).setScript("");
    }
  }
  return scripts;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::enableCallbackScript(
 OnX::Action::NativeWidget aComponent
,const std::string& aName
,const std::vector<std::string>& aScripts
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int index = 0;
  std::list<Action>::iterator it;
  for(it=fActions.begin();it!=fActions.end();++it) {
    if( ((*it).nativeWidget()==aComponent) &&
        ((*it).name()==aName) ) {
      if(index>=aScripts.size()) return false;
      (*it).setScript(aScripts[index]);
      index++;
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::load(
 const std::string& aParent
,const std::string& aToLoad
,bool aIsFile
)
//////////////////////////////////////////////////////////////////////////////
// Load a file or a string and add its items in the fTopItemML tree.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aParent);
  if(!element) return false;
  if(!element->getData1()) return false;
  UIML uiml(fWidgetFactory,fVerboseLevel);
  if(aIsFile) {
    if(!uiml.loadFile(aToLoad,false,fSession.printer())) return false;
  } else {
    if(!uiml.loadString(aToLoad,fSession.printer())) return false;
  }
  Lib::ItemML* top = uiml.topItem();
  if(top) {
    if(!solveInputDialog(*top)) return false;
    // Look if already loaded !
    Lib::ItemML* e = findFirstWidget(*top);
    if(e) {
      std::string name;
      e->attributeValue("","name",name);
      if(name=="") {
        Lib::Out out(fSession.printer());
        out << "OnX::BaseUI::load :"
            << " WARNING :" << Lib::endl;
        if(aIsFile)
          out << " in file " << Lib::sout(aToLoad) << "," << Lib::endl;
        out << " top widget without a name." << Lib::endl;
        out << " Specify a name in order to avoid twice widget creation." 
            << Lib::endl;
        return true;
      } else if(element->findItem(name)) {
        return true; // Already loaded.
      }
    }
    // Create widgets, set hierarchy.
    top->setParent(element);
    createNativeWidget(*top);
    uiml.empty();
    element->addChild(top);
    show(aParent);
    Execute tag(*this,"create");
    top->postExecute(executeItemMLCallbacks,&tag);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::save(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  return saveItemML(*fTopItemML,fTopItemML->file(),
                    fVerboseLevel,fSession.printer());
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::toWidgetAndResource(
 const std::string& aString
,std::string& aWidget
,std::string& aResource
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aWidget = "";
  aResource = "";
  std::vector<std::string> words;
  Lib::smanip::words(aString,".",words);
  if(words.size()!=2) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::toWidgetAndResource :"
        << " Bad syntax in " << Lib::sout(aString)
        << ". <widget>.<parameter> expected." << Lib::endl;
    return false;
  }
  if( (words[0]=="currentViewer") || (words[0]=="viewer") ) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::toWidgetAndResource :"
        << " words currentViewer and viewer are deprecated."
        << " Use ui_SoViewer helper." << Lib::endl;
    //FIXME : deprecate that. No reason that BaseUI contains
    //        any viewer logic.
    //aWidget = currentViewer();  
    //if(aWidget=="") return false; //No viewer around.
    return false;
  } else if(words[0]=="this") {
    aWidget = fCallbackData.component();
  } else {
    aWidget = words[0];
  }
  aResource = words[1];
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::propertyValue(
 const std::string& aWidget
,const std::string& aResource
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) return false;
  element->propertyValue(aResource,aValue);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidget* OnX::BaseUI::findWidget(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return 0;
  Lib::ItemML* itemML = fTopItemML->findItem(aName);
  if(!itemML) return 0;
  return Lib_Cast(*itemML,Slash::UI::IWidget);
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::BaseUI::findNativeWidget(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
// Should be templated.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return 0;
  Lib::ItemML* element = fTopItemML->findItem(aName);
  if(!element) return 0;
  return element->getData1();
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::isComponent(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (findNativeWidget(aName) ? true : false);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::createNativeWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!createIf(aItemML)) return true;

  const std::string& tagName = aItemML.tagName();

  if(tagName=="comment") {

    return true;

  } else if( (tagName=="OnX") ||
             (tagName=="reference") ) {

    return createChildren(aItemML);

  } else if(tagName=="widget") {

    std::string sclass;
    if(!aItemML.attributeValue("","class",sclass)) {
      Lib::Out out(fSession.printer());
      out << "BaseUI::createNativeWidget :" 
          << " widget without a class." << Lib::endl;
      aItemML.printXML(fSession.printer(),"  ");
      return false;
    }

    if(sclass=="InputDialog") {
      return createChildren(aItemML);
    }
    
    std::string name = buildName(aItemML);
    Lib::ItemML* parent = parentItemML(aItemML); 

    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "BaseUI::createNativeWidget :" 
          << " class " << Lib::sout(sclass)
          << " name " << Lib::sout(name)
          << "..." << Lib::endl;
    }

    return createNativeWidget(aItemML,sclass,name,parent);

  } else {

    Lib::Out out(fSession.printer());
    out << "BaseUI::createNativeWidget :"
        << " unknown XML tag " << Lib::sout(tagName) 
        << Lib::endl;
    return false;

  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidget* OnX::BaseUI::createWidget(
 const Slash::UI::IWidgetClass& aClass
,const std::string& aName
,Slash::UI::IWidget& aParent
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Look for a plugin factory :
  for(unsigned int index=0;index<fFactories.size();index++) {
    Slash::UI::IWidgetFactory* fac = fFactories[index];
    Slash::UI::IWidget* w = fac->createWidget(aClass,aName,aParent,aArgs);
    if(w) return w;
  }
  // Look to the predefined widgets :
  if(!fTopItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::createWidget :"
        << " topItemML not created." << Lib::endl;
    return 0;
  }
  Lib::ItemML* parentItemML = Lib_Cast(aParent,Lib::ItemML);
  if(!parentItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::createWidget :"
        << " IWidget parent is not a Lib::ItemML."
        << Lib::endl;
    return 0;
  }
  Lib::ItemML* element = 
    createComponent(aName,aClass.name(),*parentItemML);
  if(!element) return 0;
  if(!createNativeWidget(*element)) return 0;
  Slash::UI::IWidget* w = Lib_Cast(*element,Slash::UI::IWidget);
  if(!w) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::createWidget :"
        << " Lib::ItemML not a IWidget."
        << Lib::endl;
    return 0;
  }
  Execute tag(*this,"create");
  element->postExecute(executeItemMLCallbacks,&tag);
  return w;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::size(
 const Slash::UI::IWidget& aWidget
,unsigned int& aW
,unsigned int& aH
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  void* nwidget = aWidget.nativeWidget();
  if(!nwidget) return false;
  return nativeSize(nwidget,aW,aH);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::size(
 const std::string& aName
,unsigned int& aWidth
,unsigned int& aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  void* nwidget = findNativeWidget(aName);
  if(!nwidget) return false;
  return nativeSize(nwidget,aWidth,aHeight);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::createComponent(
 const std::string& aName
,const std::string& aClass
,const std::string& aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Look for a plugin factory :
  for(unsigned int index=0;index<fFactories.size();index++) {
    Slash::UI::IWidgetFactory* fac = fFactories[index];
    if(fac->createComponent(aName,aClass,aParent)) return true;
  }
  // Look to the predefined widgets :
  if(!fTopItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::createComponent :"
        << " topItemML not created." << Lib::endl;
    return false;
  }
  Lib::ItemML* parentItemML = fTopItemML->findItem(aParent);
  if(!parentItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::createComponent :"
        << " component " << Lib::sout(aParent) << " not found." << Lib::endl;
    return false;
  }
  Lib::ItemML* element = createComponent(aName,aClass,*parentItemML);
  if(!element) return false;
  if(!createNativeWidget(*element)) return false;
  Execute tag(*this,"create");
  element->postExecute(executeItemMLCallbacks,&tag);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::destroyComponent(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aName);
  if(!element) return false;

  Execute tag(*this,"destroy");
  element->postExecuteBackward(executeItemMLCallbacks,&tag);

  /*FIXME
  std::string sclass;
  element->attributeValue("","class",sclass);
  if(sclass=="OnX") {
  } else if(sclass=="comment") {
  } else if(sclass=="Reference") {    
    // Do it with a better implementation of OnX::Widget.
  } else if(sclass=="InputDialog") {
  }*/
  //Destroy native widgets.
  bool status = destroyWidget(*element);
  // Remove in parent :
  Lib::ItemML* parentItemML = element->parent();
  if(parentItemML) {
    parentItemML->removeChild(element);
  } else { 
    Lib::Out out(printer());
    out << "OnX::BaseUI::destroyComponent :"
        << " WARNING : element " << Lib::sout(aName) << " with no parent !" 
        << Lib::endl;
    delete element;
  }
  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::copyComponent(
 const std::string& aName
,const std::string& aComponent
,const std::string& aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::copyComponent :"
        << " topItemML not created." << Lib::endl;
    return false;
  }
  Lib::ItemML* fromItemML = fTopItemML->findItem(aComponent);
  if(!fromItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::copyComponent :"
        << " component " << Lib::sout(aComponent) << " not found." << Lib::endl;
    return false;
  }
  Lib::ItemML* parentItemML = fTopItemML->findItem(aParent);
  if(!parentItemML) {
    Lib::Out out(printer());
    out << "OnX::BaseUI::copyComponent :"
        << " component " << Lib::sout(aParent) << " not found." << Lib::endl;
    return false;
  }
  Lib::ItemML* element = fromItemML->createCopy(parentItemML);
  if(!element) return false;
  parentItemML->addChild(element);
  element->setAttributeValue("","name",aName);
  if(!createNativeWidget(*element)) return false;
  Execute tag(*this,"create");
  element->postExecute(executeItemMLCallbacks,&tag);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* OnX::BaseUI::parentItemML(
 Lib::ItemML& aItemML      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ItemML* parent = aItemML.parent(); 
  while(parent) {
    void* tkParent = parent->getData1();
    if(tkParent) return parent;
    parent = parent->parent();
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::BaseUI::findChildren(
 const std::string& aComponent
,const std::string& aWhat
,const std::string& aParameter
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> found;
  if(!fTopItemML) return found;
  Lib::ItemML* element = fTopItemML->findItem(aComponent);
  if(!element) return found;

  const std::list<Lib::ItemML*>& list = element->children();

  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(searchChild(*(*it),aWhat,aParameter,found)) break;
  }

  return found;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::searchChild(
 Lib::ItemML& aItemML
,const std::string& aWhat
,const std::string& aParameter
,std::vector<std::string>& aFound
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aWhat=="class") {
    // aParameter not used.
    std::string sclass;
    aItemML.attributeValue("","class",sclass);
    aFound.push_back(sclass);
  } else if(aWhat=="name") {
    // aParameter not used.
    std::string value;
    aItemML.attributeValue("","name",value);
    aFound.push_back(value);
  } else if(aWhat=="nameFromClass") {
    std::string sclass;
    aItemML.attributeValue("","class",sclass);
    if(sclass==aParameter) {
      std::string value;
      aItemML.attributeValue("","name",value);
      if(value!="") {
        aFound.push_back(value);
      } else {
        Lib::Out out(printer());
        out << "OnX::BaseUI::searchChild :" 
            << " found a component with matching class " << Lib::sout(sclass)
            << " but without name." << Lib::endl;
      }
    }
  } else if(aWhat=="labelFromClass") {
    std::string sclass;
    aItemML.attributeValue("","class",sclass);
    if(sclass==aParameter) {
      std::string value;
      aItemML.propertyValue("label",value);
      if(value!="") {
        aFound.push_back(value);
      } else {
        Lib::Out out(printer());
        out << "OnX::BaseUI::searchChild :" 
            << " found a component with matching class " << Lib::sout(sclass)
            << " but without label." << Lib::endl;
      }
    }
  } else if(aWhat=="nameFromLabel") {
    std::string label;
    aItemML.propertyValue("label",label);
    if(label==aParameter) {
      std::string value;
      aItemML.attributeValue("","name",value);
      if(value!="") {
        aFound.push_back(value);
        return true; //stop.
      } else {
        Lib::Out out(printer());
        out << "OnX::BaseUI::searchChild :" 
            << " found a component with matching label " << Lib::sout(label)
            << " but without name." << Lib::endl;
      }
    }
  }

  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(searchChild(*(*it),aWhat,aParameter,aFound)) return true;
  }

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::executeCallbacks(
 const std::string& aComponent
,const std::string& aCallback
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  if(aComponent=="*") {
    Execute tag(*this,aCallback);
    fTopItemML->postExecute(executeItemMLCallbacks,&tag);
  } else {
    Lib::ItemML* element = fTopItemML->findItem(aComponent);
    if(!element) return false;
    executeCallbacks(*element,aCallback);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::executeCallbacks(
 Lib::ItemML& aItemML
,const std::string& aCallback
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  if(widget) {
    return widget->executeCallbacks(aCallback);
  } else {
    // Callback of an ItemML which is not a IWidget. //FIXME.
    //FIXME : the below duplicate OnX::Widget::executeCallbacks code.
    std::string name;
    aItemML.attributeValue("","name",name);
    const std::list<Lib::Element*> props = aItemML.properties();
    bool found = false;
    std::list<Lib::Element*>::const_iterator it;
    for(it=props.begin();it!=props.end();++it) {
      if(aCallback==(*it)->name()) {
  
        std::string sexec;
        (*it)->attributeValue("exec",sexec);
  
        fCallbackData = CallbackData(0,name,"","",0,0);
        executeScript(sexec,(*it)->value());
        fCallbackData = CallbackData();
  
        found = true;
      }
    }
    return found;
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* OnX::BaseUI::createComponent(
 const std::string& aName
,const std::string& aClass
,Lib::ItemML& aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Lib::ItemML::Attribute> atts(2);
  atts[0] = Lib::ItemML::Attribute("class",aClass);
  atts[1] = Lib::ItemML::Attribute("name",aName);
  Lib::ItemML* element = fWidgetFactory.create("widget",atts,&aParent);
  if(!element) return 0;
  aParent.addChild(element);
  return element;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::executeItemMLCallbacks(
 Lib::ItemML& aItemML
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aItemML.tagName()=="comment") return false;
  OnX::Execute* tag = (OnX::Execute*)aTag;
  tag->baseUI().executeCallbacks(aItemML,tag->what());
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::buildName(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
// "OnX_Component_" is filtered in Lib::ItemML::save.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name;
  if(!aItemML.attributeValue("","name",name)) { 
    name = createWidgetName();
    //FIXME : do it here ?
    aItemML.setAttributeValue("","name",name);
  }
  return name;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::createChildren(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Create children :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!createNativeWidget(*(*it))) return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::solveInputDialog(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aItemML.tagName()=="widget") {
    std::string sclass;
    if(!aItemML.attributeValue("","class",sclass)){
      Lib::Out out(printer());
      out << "BaseUI::solveInputDialog :" 
          << " ItemML without a class." << Lib::endl;
      aItemML.printXML(printer(),"  ");
      return false;
    }
    if(sclass=="InputDialog") {
      if(!createInputDialog(aItemML)) return false;
    }
  }

  // Solve children :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!solveInputDialog(*(*it))) return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::createInputDialog(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
// Assume that class of the component element is "Form".
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string inputName = buildName(aItemML);

  //Lib::Out out(printer());
  //out << "InputDialog \"" << inputName << "\"." << Lib::endl;
  
  Lib::ItemML* dialogItemML = 
    createComponent(inputName+"_dialog","Dialog",aItemML);
  std::string value;
  aItemML.propertyValue("label",value);
  std::vector<Lib::ItemML::Attribute> atts;
  dialogItemML->addProperty("title",atts,value);

  std::string vbox = inputName + "_vbox";
  Lib::ItemML* vBoxItemML = createComponent(vbox,"VBox",*dialogItemML);
  vBoxItemML->addProperty("expand",atts,"first");
  vBoxItemML->addProperty("sizeOfFixed",atts,"30");

  std::string vcontainer = inputName + "_vcontainer";
  Lib::ItemML* vContainerItemML = 
    createComponent(vcontainer,"VContainer",*vBoxItemML);

  /*
  Lib::ItemML* labelItemML = 
    createComponent(inputName + "_label","Label",*vContainerItemML);
  labelItemML->addProperty("label",atts,value);
  */


  // Loop on input properties :
  std::vector<std::string> inputNames;
  const std::list<Lib::Element*>& properties = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;
  for(it=properties.begin();it!=properties.end();++it) {
    if((*it)->name()=="input") {
      std::string name;
      if(!(*it)->attributeValue("name",name)) {
        Lib::Out out(printer());
        out << "OnX::BaseUI::createInputDialog :" 
            << " An input property must have a name." << Lib::endl;
        return false;
      }
      std::string type;
      (*it)->attributeValue("type",type);
      if((type=="") || (type=="text") ) {
        Lib::ItemML* hBoxItemML = 
          createComponent(inputName + "_hbox_" + name,"HBox",
            *vContainerItemML);
        Lib::ItemML* labelItemML = 
          createComponent(inputName + "_label_" + name,"Label",*hBoxItemML);
        std::vector<Lib::ItemML::Attribute> atts;
        int length = 0;
        if((*it)->attributeValue("label",value)) {
          labelItemML->addProperty("label",atts,value);
          length = value.size();
        } else {
          labelItemML->addProperty("label",atts,name);
          length = name.size();
        }

        Lib::smanip::printf(value,64,"%d",length * 10);
        hBoxItemML->addProperty("expand",atts,"second");
        hBoxItemML->addProperty("sizeOfFixed",atts,value);

        Lib::ItemML* entryItemML = 
          createComponent(inputName + "_input_" + name,"Entry",*hBoxItemML);
        entryItemML->addProperty("background",atts,"white");
        (*it)->attributeValue("default",value);
        entryItemML->addProperty("value",atts,value);

      } else if( (type=="option")|| (type=="combo") ) {

        Lib::ItemML* hBoxItemML = 
          createComponent(inputName+"_hbox_"+name,"HBox",*vContainerItemML);

        Lib::ItemML* labelItemML = 
          createComponent(inputName + "_label_" + name,"Label",*hBoxItemML);
        std::vector<Lib::ItemML::Attribute> atts;
        int length = 0;
        if((*it)->attributeValue("label",value)) {
          labelItemML->addProperty("label",atts,value);
          length = value.size();
        } else {
          labelItemML->addProperty("label",atts,name);
          length = name.size();
        }

        Lib::smanip::printf(value,64,"%d",length * 10);
        hBoxItemML->addProperty("expand",atts,"second");
        hBoxItemML->addProperty("sizeOfFixed",atts,value);

        Lib::ItemML* optionItemML = createComponent
          (inputName + "_input_" + name,
           (type=="combo"?"ComboBox":"OptionMenu"),
           *hBoxItemML);
        (*it)->attributeValue("items",value);
        optionItemML->addProperty("items",atts,value);
        (*it)->attributeValue("default",value);
        optionItemML->addProperty("default",atts,value);

      } else if( (type=="fileChooser")||(type=="fileSaver")) {
        // Variant of type "text" with the label being
        // now a button mapping a file chooser.

        Lib::ItemML* hBoxItemML = 
          createComponent(inputName + "_hbox_" + name,
            "HBox",*vContainerItemML);
        Lib::ItemML* buttonItemML = 
          createComponent(inputName + "_pushbutton_" + name,
            "PushButton",*hBoxItemML);

        std::vector<Lib::ItemML::Attribute> atts;
        int length = 0;
        if((*it)->attributeValue("label",value)) {
          buttonItemML->addProperty("label",atts,value);
          length = value.size();
        } else {
          buttonItemML->addProperty("label",atts,name);
          length = name.size();
        }

        Lib::smanip::printf(value,64,"%d",length * 10);
        hBoxItemML->addProperty("expand",atts,"second");
        hBoxItemML->addProperty("sizeOfFixed",atts,value);

        std::string entryName = inputName + "_input_" + name;
        Lib::ItemML* entryItemML = 
          createComponent(entryName,"Entry",*hBoxItemML);
        entryItemML->addProperty("background",atts,"white");
        (*it)->attributeValue("default",value);
        entryItemML->addProperty("value",atts,value);

        // Set the push button script. 
        // It maps a file chooser by passing the name of the entry.
        //  The "ok" of the chooser calls 
        // the OnX_Input_fileChooser DLD callback 
        // that deposit the choosed file on the entry widget.
        atts.clear();
        atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
        std::string s = 
          (type=="fileChooser"?"ui_showFileChooser":"ui_showFileSaver");
        buttonItemML->addProperty
          ("activate",atts,"OnX "+s+" OnX OnX_Input_file *.* . "+entryName);
        atts.clear();

      } else if(type=="colorChooser") {
        // Variant of type "combo" with the label being
        // now a button mapping a color chooser.

        Lib::ItemML* hBoxItemML = 
          createComponent(inputName+"_hbox_"+name,"HBox",*vContainerItemML);

        Lib::ItemML* buttonItemML = 
          createComponent(inputName + "_pushbutton_" + name,"PushButton",
            *hBoxItemML);

        std::vector<Lib::ItemML::Attribute> atts;
        int length = 0;
        if((*it)->attributeValue("label",value)) {
          buttonItemML->addProperty("label",atts,value);
          length = value.size();
        } else {
          buttonItemML->addProperty("label",atts,name);
          length = name.size();
        }

        Lib::smanip::printf(value,64,"%d",length * 10);
        hBoxItemML->addProperty("expand",atts,"second");
        hBoxItemML->addProperty("sizeOfFixed",atts,value);

        std::string comboName = inputName + "_input_" + name;
        Lib::ItemML* optionItemML = createComponent
          (comboName,"ComboBox",*hBoxItemML);
        (*it)->attributeValue("items",value);
        optionItemML->addProperty("items",atts,value);
        (*it)->attributeValue("default",value);
        optionItemML->addProperty("default",atts,value);

        // Set the push button script. 
        // It maps a color chooser by passing the name of the combo. 
        // The "ok" of the chooser calls 
        // the OnX_Input_colorChooser DLD callback 
        // that deposit the choosed color on the 
        // combo.
        atts.clear();
        atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
        buttonItemML->addProperty("activate",atts,
          "OnX ui_showColorChooser OnX OnX_Input_colorChooser "+comboName);
        atts.clear();

      } else if( (type=="scale")||(type=="scrollBar")) {

        // Box(Box2(Label,Entry),Scale)
        Lib::ItemML* hBoxItemML = 
          createComponent(inputName + "_hbox_" + name,"HBox",
            *vContainerItemML);
        hBoxItemML->addProperty("expand",atts,"second");
        hBoxItemML->addProperty("sizeOfFixed",atts,"150");

        // Box2(Label,Entry)
        Lib::ItemML* hBox2ItemML = 
          createComponent(inputName + "_hbox2_" + name,"HBox",
            *hBoxItemML);

        Lib::ItemML* labelItemML = 
          createComponent(inputName + "_label_" + name,"Label",*hBox2ItemML);
        std::vector<Lib::ItemML::Attribute> atts;
        int length = 0;
        if((*it)->attributeValue("label",value)) {
          labelItemML->addProperty("label",atts,value);
          length = value.size();
        } else {
          labelItemML->addProperty("label",atts,name);
          length = name.size();
        }

        Lib::smanip::printf(value,64,"%d",length * 10);
        hBox2ItemML->addProperty("expand",atts,"second");
        hBox2ItemML->addProperty("sizeOfFixed",atts,value);

        std::string entryName = inputName + "_input_" + name;
        Lib::ItemML* entryItemML = 
          createComponent(entryName,"Entry",*hBox2ItemML);
        entryItemML->addProperty("background",atts,"white");
        (*it)->attributeValue("default",value);
        entryItemML->addProperty("value",atts,value);

        std::string sliderName = inputName + "_slider_" + name;

        atts.clear();
        atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
        entryItemML->addProperty("activate",atts,
          "OnX ui_setParameter "+sliderName+".value ui_callbackValue");
        atts.clear();

        Lib::ItemML* sliderItemML = 
          createComponent(sliderName,
            (type=="scale"?"HScale":"HScrollBar"),
            *hBoxItemML);
        (*it)->attributeValue("default",value);
        sliderItemML->addProperty("value",atts,value);
        (*it)->attributeValue("minimum",value);
        sliderItemML->addProperty("minimum",atts,value);
        (*it)->attributeValue("maximum",value);
        sliderItemML->addProperty("maximum",atts,value);

        atts.clear();
        atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
        sliderItemML->addProperty("valueChanged",atts,
          "OnX ui_setParameter "+entryName+".value ui_callbackValue");
        sliderItemML->addProperty("drag",atts,
          "OnX ui_setParameter "+entryName+".value ui_callbackValue");
        sliderItemML->addProperty("increment",atts,
          "OnX ui_setParameter "+entryName+".value ui_callbackValue");
        sliderItemML->addProperty("decrement",atts,
          "OnX ui_setParameter "+entryName+".value ui_callbackValue");
        atts.clear();

      } else {
        Lib::Out out(printer());
        out << "OnX::BaseUI::createInputDialog :" 
            << " unknown type " << Lib::sout(type)
            << Lib::endl;
        return false;
      }

      inputNames.push_back(name);
    }
  }

  int margin = 5;
  int hchild = 35;
  if(cast("OnX::NextStepUI")) { //FIXME. And remove the Slash::UI::IUI::cast ?
    margin = 0;
    hchild = 30;
  }
  
  // WIN32 : set geometry (mainly height) of the dialog :
  std::string sgeom;
  // WinTk::[V,H}Container has a margin of 5 :
  int childn = inputNames.size()+1;
  Lib::smanip::printf(sgeom,256,"%dx%d",300,margin + childn * (hchild + margin));
  dialogItemML->addProperty("geometry",atts,sgeom);

  Lib::ItemML* hContainerItemML = 
    createComponent(inputName + "_hcontainer","HContainer",*vBoxItemML);

  // Ok button :
  Lib::ItemML* okItemML = 
    createComponent(inputName + "_ok","PushButton",*hContainerItemML);

  // Set the script :

  // Get the "ok" script :
  std::string sexec;
  if(aItemML.propertyValue("ok",value))
    aItemML.attributeValue("ok","exec",sexec);

  std::string script;
  if(sexec=="Python") {
    for(unsigned int index=0;index<inputNames.size();index++) {
      script += inputNames[index];
      script += " = OnX.ui_parameterValue(\"";
      script += inputName;
      script += "_input_";
      script += inputNames[index];
      script += ".value\")\n";
    }
    script += value;
  }

  //out << "InputDialog script " << Lib::sout(script) << "." << Lib::endl;

  atts.clear();
  atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
  okItemML->addProperty("activate",atts,"OnX ui_hide "+inputName+"_dialog");
  atts.clear();

  if(sexec=="Python") {
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","Python"));
    okItemML->addProperty("activate",atts,"import OnX;"+script);
    atts.clear();
  } else if( (sexec=="DLD") || (sexec=="C++") ) {
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
    okItemML->addProperty("activate",atts,value);
    atts.clear();
  }
  okItemML->addProperty("label",atts,"Ok");

  // Apply button :
  Lib::ItemML* applyItemML = 
    createComponent(inputName + "_apply","PushButton",*hContainerItemML);
  if(sexec=="Python") {
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","Python"));
    applyItemML->addProperty("activate",atts,"import OnX;"+script);
    atts.clear();
  } else if( (sexec=="DLD") || (sexec=="C++") ) {
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
    applyItemML->addProperty("activate",atts,value);
    atts.clear();
  }
  applyItemML->addProperty("label",atts,"Apply");

  // Cancel button :
  Lib::ItemML* cancelItemML = 
    createComponent(inputName + "_cancel","PushButton",*hContainerItemML);
  atts.clear();
  atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
  cancelItemML->addProperty
    ("activate",atts,"OnX ui_hide " + inputName + "_dialog");
  atts.clear();
  cancelItemML->addProperty("label",atts,"Cancel");

  // Update button :
  // Get the "update" script :
  if(aItemML.propertyValue("update",value))
    aItemML.attributeValue("update","exec",sexec);
  if(value!="") {
    Lib::ItemML* updateItemML = 
      createComponent(inputName + "_update","PushButton",*hContainerItemML);
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
    updateItemML->addProperty("activate",atts,value);
    atts.clear();
    updateItemML->addProperty("label",atts,"Update");
  }

  // Help button :
  // Get the "help" script :
  //aItemML.propertyValue("help",value);
  std::string sfile;
  aItemML.attributeValue("help","file",sfile);
  if(sfile!="") {
    Lib::ItemML* helpItemML = 
      createComponent(inputName + "_help","PushButton",*hContainerItemML);
    atts.clear();
    atts.push_back(Lib::ItemML::Attribute("exec","DLD"));
    helpItemML->addProperty("activate",atts,"OnX ui_help_file "+sfile);
    atts.clear();
    helpItemML->addProperty("label",atts,"Help");
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::notification(
 Slash::Core::IMessage* aMessage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::BaseUI::notification :"
        << " receive message " << aMessage->message() << Lib::endl;
  }
  if(Lib::ExecuteMessage* mess = 
              Lib_Cast(*aMessage,Lib::ExecuteMessage)) {
    Slash::Core::IAction& action = mess->action();
    //FIXME : is action still valid ?
    Action* oaction = Lib_Cast(action,OnX::Action);
    if(oaction) {
      fCallbackData = oaction->callbackData();
      std::vector<std::string> args;
      oaction->execute(args);
      fCallbackData = CallbackData();
    }
    delete &action;
  } else if(Lib::UpdateMessage* mess = 
              Lib_Cast(*aMessage,Lib::UpdateMessage)) {
    std::string s = mess->what();
    if(s=="*") {
      executeCallbacks("*","update");
    } else if(fTopItemML) {
      Lib::ItemML* element = fTopItemML->findItem(s);
      if(element) executeCallbacks(*element,"update");
    }
    Slash::Core::IRelation* relation = mess->relation();
    if(relation && fRelationManager && fRelationManager->isValid(*relation)) {
      relation->unSchedule();
    }
  } else if(Lib::RelationMessage* mess = 
              Lib_Cast(*aMessage,Lib::RelationMessage)) {
    Slash::Core::IRelation& relation = mess->relation();
    if(fRelationManager && fRelationManager->isValid(relation)) {
      relation.unSchedule();
    }
  }
  delete aMessage;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::removeCallback(
 const std::string& aName
,const std::string& aWhich
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aName);
  if(!element) return false;
  OnX::Action::NativeWidget widget = element->getData1();
  if(!widget) return false;
  Action* action = getAction(widget,aWhich,aIndex);
  if(!action) return false;
  deleteCallback(*element,aWhich,*action);
  removeAction(widget,aWhich,aIndex);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::setCallback(
 const std::string& aName
,const std::string& aWhich
,const std::string& aInterp
,const std::string& aScript
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
// If aIndex found override script.
// If not found add a new callback.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aName);
  if(!element) return false;
  OnX::Action::NativeWidget widget = element->getData1();
  if(!widget) return false;
  bool newCallback = false;
  Action* action = getAction(widget,aWhich,aIndex);
  if(!action) {
    newCallback = true;
    action = addCallback(*element,aWhich,aInterp,aScript);
  }
  if(!action) return false;
  action->setInterpreter(aInterp);
  action->setScript(aScript);
  // Update element :
  if (newCallback) {
    std::vector<Lib::ItemML::Attribute> att;
    att.push_back(Lib::ItemML::Attribute("exec",aInterp));
    element->addProperty(aWhich,att,aScript);
  } else {
    element->setAttributeValue(aWhich,"exec",aInterp,aIndex);
    element->setPropertyValue(aWhich,aScript,aIndex);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::createIf(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string svalue;
  if(!aItemML.attributeValue("","if",svalue)) return true;
  std::vector<std::string> words = Lib::smanip::words(svalue,".");
  if((words.size()==2) && (words[0]=="env")) {
    // if="env.<env>"           return true if <env> exists.
    // if="env.<env>=<value>"   return true if <env>==<value>
    std::vector<std::string> words2 = Lib::smanip::words(words[1],"=");
    if(words2.size()==1) { //if <env> exist.
      std::string value;
      return Lib::System::getenv(words2[0],value);
    } else if(words2.size()==2) { //if <env> == <value>
      std::string value;
      if(!Lib::System::getenv(words2[0],value)) return false;
      return (value==words2[1]);
    }
  } else if((words.size()==2) && (words[0]=="exec")) {
    // if="exec.<interp>"  return true if <interp> is operational.
    std::string sexec = words[1];
    if(!fScriptManager) return false;
    Slash::UI::IInterpreter* interpreter = 
      fScriptManager->findInterpreter(sexec);
    if(!interpreter) return false;
    return interpreter->isOperational();
  } else {
    // if="<param>"           return true if <param> is a session parameter.
    // if="<param>=<value>"   return true if <param>==<value>
    std::vector<std::string> words = Lib::smanip::words(svalue,"=");
    if(words.size()==1) {
      std::string value;
      return fSession.parameterValue(words[0],value);
    } else if(words.size()==2) {
      std::string value;
      if(!fSession.parameterValue(words[0],value)) return false;
      return (words[1]==value);
    }
  }
  return false;
}    
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* OnX::BaseUI::findFirstWidget(
 Lib::ItemML& aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : stop descending under comment ?
  if(aItemML.tagName()=="widget") return &aItemML;

  // Look children :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML* element = findFirstWidget(*(*it));
    if(element) return element;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
static bool saveItemML(
 Lib::ItemML& aItemML
,const std::string& aFile
,int aVerboseLevel
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFile=="") return false;
  if(!aItemML.saveFlag()) return true;
  std::string sfile = Lib::fmanip::name(aFile);
  std::string name =  sfile + "_new";
  Lib::FileWriter fp(name,aPrinter);
  if(!fp.enabled()) {
    Lib::Out out(aPrinter);
    out << "OnX::BaseUI::save :" 
        << " can't open file " << name << Lib::endl;
    return false;
  }
  Lib::Out out(fp);
  if(aVerboseLevel) {
    Lib::Out out(aPrinter);
    out << "OnX::BaseUI::save :" 
        << " save file " << Lib::sout(sfile) << "..." << Lib::endl;
  }
  bool status = saveItemML(aItemML,out,0,aVerboseLevel,aPrinter);
  fp.disable();
  if(!status) {
    Lib::fmanip::remove(name);
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnX::BaseUI::save :" 
          << " unable to save file " << Lib::sout(sfile) << "..." << Lib::endl;
    }
  } else {
    // Replace file :
    Lib::fmanip::move(name,sfile);
  }
  return status;
}
//////////////////////////////////////////////////////////////////////////////
static bool saveItemML(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,int aShift
,int aVerboseLevel
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aItemML.saveFlag()) return true;
  
  std::string spaces;
  for(int i=0;i<aShift;i++) spaces += " ";

  const std::string& tagName = aItemML.tagName();

  std::string sname;
  aItemML.attributeValue("","name",sname);

  const std::list<Lib::ItemML*>& childs = aItemML.children();

  if(tagName=="reference") {
    std::string sfile;
    aItemML.attributeValue("","file",sfile);
    aOut << spaces 
         << "<reference"
         << " file="  << Lib::sout(sfile) << "/>" 
         << Lib::endl;
    
    // Save GUI associated to the reference :
    // Element should have one child :
    if(childs.size()!=1) return false;
    return saveItemML(*(childs.front()),sfile,aVerboseLevel,aPrinter);
  }

  if (aItemML.file() != "") {
    // If file field is present, we have to take care that file
    // is not already open by a parent.
    // (case of reference file (and others ?))
    Lib::ItemML* parentItemML = aItemML.parent();
    if (parentItemML) {
      std::string pfile;
      parentItemML->attributeValue("","file",pfile);
      if( (parentItemML->tagName() == "reference") && 
          (pfile == aItemML.file()) ) {
      } else if (aItemML.file() == parentItemML->file()) {
      } else { 
        std::string file = aItemML.file();
        aItemML.setFile(""); //to avoid recursivity.
        bool status = saveItemML(aItemML,file,aVerboseLevel,aPrinter);
        aItemML.setFile(file);
        return status;
      }
    }
  }
  
  if(tagName=="OnX") { 
    // Is in principle at shift=0. 
    aOut << spaces << "<OnX version=" << Lib::sout(ONX_VERSION) ;
  } else if(tagName=="comment") {
    aOut << spaces << "<comment";

    const std::vector<Lib::ItemML::Attribute>& atbs = aItemML.attributes();
    unsigned int atbn = atbs.size();
    for(unsigned int index=0;index<atbn;index++) {
      const std::string& atb = atbs[index].first;
      const std::string& atbv = atbs[index].second;
      if( (atb=="name") && isGeneratedName(atbv)) {
        // Do not save generated names.
      } else if(atb!="class") {
        aOut << " " << atb << "=" << Lib::sout(atbv);
      }
    }

  } else { //FIXME : check that tagName is "widget".
    aOut << spaces << "<widget";

    const std::vector<Lib::ItemML::Attribute>& atbs = aItemML.attributes();
    unsigned int atbn = atbs.size();
    for(unsigned int index=0;index<atbn;index++) {
      const std::string& atb = atbs[index].first;
      const std::string& atbv = atbs[index].second;
      if( (atb=="name") && (atbv.substr(0,14)=="OnX_Component_")) {
        // Do not save generated names.
      } else {
        aOut << " " << atb << "=" << Lib::sout(atbv);
      }
    }

  }

  const std::list<Lib::Element*>& props = aItemML.properties();

  if(!props.size() && !childs.size() ) {

    aOut << "/>" << Lib::endl;

  } else {

    aOut << ">" << Lib::endl;

    std::list<Lib::Element*>::const_iterator itp;
    for(itp=props.begin();itp!=props.end();++itp) {
      const std::vector<Lib::ItemML::Attribute>& atbs = (*itp)->attributes();

      bool isCallback = false;
      aOut << spaces << "  <" << (*itp)->name();
      unsigned int atbn = atbs.size();
      for(unsigned int index=0;index<atbn;index++) {
        const std::string& atb = atbs[index].first;
        const std::string& atbv = atbs[index].second;
        aOut << " " << atb << "=" << Lib::sout(atbv);
        if(atb=="exec") isCallback = true;
      }
      if((*itp)->value()=="") {
        aOut << "/>" << Lib::endl;
      } else {
        aOut << ">";
        std::string value = Lib::smanip::toxml((*itp)->value());
        if(isCallback) {
          if(value.find("\\n\\")==std::string::npos) {
            aOut << value;
          } else {
            aOut << Lib::endl;
            Lib::smanip::replace(value,"\\n\\","@OnX@");
            Lib::smanip::replace(value,"@OnX@","\\n\\\n");
            Lib::smanip::strip(value,Lib::smanip::trailing,' ');
            aOut << value;
            unsigned int l = value.size();
            if(l && value[l-1]!='\n') aOut << Lib::endl;
            aOut << spaces << "  ";
          }
        } else {
          aOut << value;
        }
        aOut << "</" << (*itp)->name() << ">" << Lib::endl;
      }
    }

    if(tagName!="InputDialog") {
      //Children :
      std::list<Lib::ItemML*>::const_iterator it;
      for(it=childs.begin();it!=childs.end();++it) {
        if(!saveItemML(*(*it),aOut,aShift+2,aVerboseLevel,aPrinter)) 
          return false;
      }
    }  
  
    aOut << spaces << "</" << tagName << ">" << Lib::endl;

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::ICyclic* OnX::BaseUI::findCyclic(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::UI::ICyclic*>::const_iterator it;
  for(it=fCyclics.begin();it!=fCyclics.end();++it) {
    if((*it)->name()==aName) return *it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::steerModal(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  while(steer()!=ONX_BREAK_MODAL) {}
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::ask(
 const std::string& aPrompt
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  load("main",Lib::fmanip::name("$ONXROOT/scripts/OnX/AskDialog.onx"));
  setParameter("OnX_AskDialog_prompt.label",aPrompt);
  setParameter("OnX_AskDialog_entry.value","");
  show("OnX_AskDialog_dialog");
  steerModal();
  //echo("debug : exit steer loop.");
  std::string value;
  parameterValue("OnX_AskDialog_entry.value",value);
  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::BaseUI::askString :"
        << value
        << Lib::endl;
  }
  return value;
}
//////////////////////////////////////////////////////////////////////////////
// Laurent ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
std::vector< std::pair<bool,std::string> > OnX::BaseUI::parseDecorations(
 const std::string& aDecoration
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector< std::pair<bool,std::string> > decos;
  std::vector<std::string> items;
  Lib::smanip::lines(aDecoration,items);
  unsigned int itemn = items.size();

  //look the first, if it is not all or none, we go threw the next
  bool close_button = true;
  bool iconify_button = true;
  bool min_max_button = true;
  bool resize_handle = true;
  bool border = true;
  bool caption = true;
  bool system_menu = true;
  if (itemn >0) {
    if (items[0]=="all"){ // do nothing 
      close_button = true;
    } else if (items[0]=="none"){ // remove all
      close_button = false;
      iconify_button = false;
      min_max_button = false;
      resize_handle = false;
      border = false;
      caption = false;           
      system_menu = false;
    }

    // To have a icon, close, or minimize-maximize button, you should
    // have a caption
    // If you have a system menu, you will also have a close button

    Lib::Out out(printer());
    for(unsigned int index=0;index<itemn;index++){
      if (items[index]=="no_close_button")        close_button = false;
      else if (items[index]=="close_button")      close_button = true;
      else if (items[index]=="no_iconify_button") iconify_button = false;
      else if (items[index]=="iconify_button")    iconify_button = true;
      else if (items[index]=="no_system_menu")    system_menu = false;
      else if (items[index]=="system_menu")       system_menu = true;
      else if (items[index]=="no_min_max_button") min_max_button = false;
      else if (items[index]=="min_max_button")    min_max_button = true;
      else if (items[index]=="no_resize_handle")  resize_handle = false;
      else if (items[index]=="resize_handle")     resize_handle = true;
      else if (items[index]=="no_border")         border = false;
      else if (items[index]=="border")            border = true;
      else if (items[index]=="no_caption")        caption = false;
      else if (items[index]=="caption")           caption = true;
      else if (items[index]=="all") {}
      else if (items[index]=="none") {}
      else if (items[index]!="") 
        out << "OnX::BaseUI::parseDecorations :"
            << " WARNING : The decoration " << Lib::sout(items[index]) 
            << " is not recognised." 
            << " Legal options are "
            << " none"
            << " [no_]close_button"
            << " [no_]iconify_button" 
            << " [no_]system_menu" 
            << " [no_]min_max_button [no_]resize_handle [no_]border"
            << " [no_]caption all none." 
            << Lib::endl; 
    }
    if (!caption && 
        ((close_button )|| (iconify_button) || 
         (min_max_button) || (system_menu)))
      out << "OnX::BaseUI::parseDecorations :" 
          << " WARNING : You want to create a widget whithout caption, " 
          << "but you should have a caption to have a"
          << " close,iconify,system menu " 
          << "or min max button" 
          << Lib::endl;
    decos.push_back(std::pair<bool,std::string>(false,"no_decorations"));
  } else {
    decos.push_back(std::pair<bool,std::string>(true,"no_decorations"));
  }
  decos.push_back(std::pair<bool,std::string>(close_button,"close_button"));
  decos.push_back(std::pair<bool,std::string>(iconify_button,"iconify_button"));
  decos.push_back(std::pair<bool,std::string>(min_max_button,"min_max_button"));
  decos.push_back(std::pair<bool,std::string>(resize_handle,"resize_handle"));
  decos.push_back(std::pair<bool,std::string>(border,"border"));
  decos.push_back(std::pair<bool,std::string>(caption,"caption"));
  decos.push_back(std::pair<bool,std::string>(system_menu,"system_menu"));
  return decos;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::addFactory(
 Slash::UI::IWidgetFactory* aFactory
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFactories.push_back(aFactory);
}
//////////////////////////////////////////////////////////////////////////////
// IB ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::addWidgetClass(
 Slash::UI::IWidgetClass* aClass
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fWidgetClasses.push_back(aClass);
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::createWidgetName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name;
  Lib::smanip::printf(name,256,"OnX_Component_%d",fWidgetNameCounter);
  fWidgetNameCounter++;
  return name;
}
//////////////////////////////////////////////////////////////////////////////
static bool isGeneratedName(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //OnX_Component_
  //012345678901234
  if(aString.size()<=14) return false;
  int i;
  return Lib::smanip::toint(aString.substr(14,aString.size()-14),i);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::createWidgetTable(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> sChilds;
  std::vector<std::string> sAttrs;
  std::vector<std::string> sProps;
  std::vector<std::string> sCbs;

  //tooltip, pixmap : only at the creation
  std::string containers = "\
VContainer\n\
HContainer\n\
TabStack\n\
VBox\n\
HBox\n\
VPaned\n\
HPaned\n\
Reference";
  std::string leaves = "\
ScrolledText\n\
ScrolledList\n\
ScrolledTree\n\
VScrollBar\n\
HScrollBar\n\
VScale\n\
HScale\n\
ComboBox\n\
Label\n\
PushButton\n\
CommandLine\n\
OptionMenu\n\
ProgressBar\n\
OpenGLArea\n\
PageViewer\n\
PlanePageViewer\n\
Entry\n\
Password\n\
ToggleButton";

  // CallbackList "valueChanged activate create select complete collect drag increment decrement popup update" paint event"


  // Containers

  Lib::smanip::lines("Window\nDialog\nReference",sChilds);
  Lib::smanip::lines("name\nclass\nversion",sAttrs);
  Lib::smanip::lines("",sProps);
  Lib::smanip::lines("create",sCbs);
  //FIXME fWidgetClasses.push_back(new WidgetClass("OnX",sChilds,sAttrs,sProps,sCbs));

  // no more attributes except "name" and "class"
  Lib::smanip::lines("name\nclass",sAttrs);

  Lib::smanip::lines(std::string("Window\nMenuBar\nDialog\nToolBar\nWarningDialog")+"\n"+containers +"\n"+leaves,sChilds);
  Lib::smanip::lines("geometry\ndecoration\ntitle\nshow\nbackground\nforeground",sProps);
  Lib::smanip::lines("create",sCbs);
  fWidgetClasses.push_back(new WidgetClass("Window",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("CascadeButton\nReference",sChilds);
  Lib::smanip::lines("background\nforeground",sProps);
  Lib::smanip::lines("",sCbs);
  fWidgetClasses.push_back(new WidgetClass("MenuBar",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("MenuItem\nMenuSeparator\nToggleMenuItem\nCascadeButton\nReference",sChilds);
  Lib::smanip::lines("background\nforeground",sProps);
  Lib::smanip::lines("",sCbs);
  fWidgetClasses.push_back(new WidgetClass("Menu",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("Menu\nReference",sChilds);
  Lib::smanip::lines("label\npixmap\ntooltip\nbackground\nforeground",sProps);
  Lib::smanip::lines("activate",sCbs);
  fWidgetClasses.push_back(new WidgetClass("CascadeButton",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("ToolBarItem\nComboBox\nLabel\nPushButton\nCommandLine\nOptionMenu\nProgressBar\nPassword\nEntry",sChilds);
  Lib::smanip::lines("background\nforeground",sProps);
  Lib::smanip::lines("",sCbs);
  fWidgetClasses.push_back(new WidgetClass("ToolBar",sChilds,sAttrs,sProps,sCbs));


  // sChild = container & leaves
  Lib::smanip::lines(containers+"\n"+leaves,sChilds);
  //  Lib::smanip::lines("name",sAttrs);

  Lib::smanip::lines("expand\nbackground",sProps);
  Lib::smanip::lines("",sCbs);
  fWidgetClasses.push_back(new WidgetClass("VBox",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("HBox",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background",sProps);
  fWidgetClasses.push_back(new WidgetClass("VContainer",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("HContainer",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("splitPosition\nbackground",sProps);
  fWidgetClasses.push_back(new WidgetClass("VPaned",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("HPaned",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("tabLabel\nbackground\nforeground",sProps);
  Lib::smanip::lines("select",sCbs);

  fWidgetClasses.push_back(new WidgetClass("TabStack",sChilds,sAttrs,sProps,sCbs));
  Lib::smanip::lines("geometry\ndecoration\ntitle\nshow\nbackground\nforeground",sProps);
  Lib::smanip::lines("create",sCbs);
  fWidgetClasses.push_back(new WidgetClass("Dialog",sChilds,sAttrs,sProps,sCbs));


  // leaves : no more sChilds
  sChilds.clear();
  sChilds.resize(0);
  //  Lib::smanip::lines("name",sAttrs);

  Lib::smanip::lines("selection\nitems\nbackground\nforeground",sProps);
  Lib::smanip::lines("select",sCbs);
  fWidgetClasses.push_back(new WidgetClass("ScrolledList",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("title\ndirMask\ndirectory\nbackground\nforeground",sProps);
  Lib::smanip::lines("ok\ncancel",sCbs);
  fWidgetClasses.push_back(new WidgetClass("FileSelection",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background\nforeground",sProps);
  Lib::smanip::lines("ok",sCbs);
  fWidgetClasses.push_back(new WidgetClass("ColorSelection",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background\nforeground",sProps);
  Lib::smanip::lines("select\nselectBranch\nopen\nclose",sCbs);
  fWidgetClasses.push_back(new WidgetClass("ScrolledTree",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\nbackground\nforeground",sProps);
  Lib::smanip::lines("drag\nvalueChanged\nincrement\ndecrement",sCbs);
  fWidgetClasses.push_back(new WidgetClass("VScrollBar",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("HScrollBar",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\nbackground\nforeground\nminimum\nmaximum",sProps);
  Lib::smanip::lines("drag",sCbs);
  fWidgetClasses.push_back(new WidgetClass("VScale",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("HScale",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\ndefault\nitems\nbackground\nforeground",sProps);
  Lib::smanip::lines("valueChanged",sCbs);
  fWidgetClasses.push_back(new WidgetClass("ComboBox",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\nbackground\nforeground",sProps);
  Lib::smanip::lines("activate\ncomplete",sCbs);
  fWidgetClasses.push_back(new WidgetClass("CommandLine",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\nlabel\nitems\ndefault\nbackground\nforeground",sProps);
  Lib::smanip::lines("valueChanged",sCbs);
  fWidgetClasses.push_back(new WidgetClass("OptionMenu",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("popupItems\nbackground",sProps);
  Lib::smanip::lines("collect\npopup\nupdate",sCbs);
  fWidgetClasses.push_back(new WidgetClass("PageViewer",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("PlanePageViewer",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("",sProps);
  Lib::smanip::lines("",sCbs);
  fWidgetClasses.push_back(new WidgetClass("SoExaminerViewer",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("SoPlaneViewer",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("SoWalkViewer",sChilds,sAttrs,sProps,sCbs));
  fWidgetClasses.push_back(new WidgetClass("SoFlyViewer",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background",sProps);
  Lib::smanip::lines("paint\nevent",sCbs);
  fWidgetClasses.push_back(new WidgetClass("OpenGLArea",sChilds,sAttrs,sProps,sCbs));


  // activate callbacks
  Lib::smanip::lines("activate",sCbs);
  //  Lib::smanip::lines("name",sAttrs);
  //  Lib::smanip::lines("",sChilds);

  Lib::smanip::lines("background\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("Password",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("value\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("Entry",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("label\npixmap\ntooltip\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("ToolBarItem",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("label\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("PushButton",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("set\nlabel\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("ToggleButton",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background\nforeground\nlabel",sProps);
  fWidgetClasses.push_back(new WidgetClass("MenuItem",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("label\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("ToggleMenuItem",sChilds,sAttrs,sProps,sCbs));

  // no more callbacks
  Lib::smanip::lines("",sCbs);
  //  Lib::smanip::lines("name",sAttrs);
  //  Lib::smanip::lines("",sChilds);

  Lib::smanip::lines("value\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("ScrolledText",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background\nforeground\ntitle\ngeometry\ndecoration",sProps);
  fWidgetClasses.push_back(new WidgetClass("WarningDialog",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("label\nlabelString\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("Label",sChilds,sAttrs,sProps,sCbs));

  Lib::smanip::lines("background\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("MenuSeparator",sChilds,sAttrs,sProps,sCbs));
 
  Lib::smanip::lines("value\nbackground\nforeground",sProps);
  fWidgetClasses.push_back(new WidgetClass("ProgressBar",sChilds,sAttrs,sProps,sCbs));

  //no name
  //  Lib::smanip::lines("",sChilds);
  Lib::smanip::lines("class\nfile",sAttrs);
  Lib::smanip::lines("",sProps);
  //FIXME fWidgetClasses.push_back(new WidgetClass("Reference",sChilds,sAttrs,sProps,sCbs));

  return;
}

//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::createLanguageList(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // FIXME : get this list from the System manager
 
  fCallbackLanguageList.push_back("DLD");
  fCallbackLanguageList.push_back("tcl");
  fCallbackLanguageList.push_back("cint");
  fCallbackLanguageList.push_back("kuip");
  fCallbackLanguageList.push_back("system");
  fCallbackLanguageList.push_back("sh");
  fCallbackLanguageList.push_back("DOS");
  fCallbackLanguageList.push_back("java");
}
//////////////////////////////////////////////////////////////////////////////
void OnX::BaseUI::createPropertyList(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPropertyList.push_back("foreground background");
  fPropertyList.push_back("color");

  fPropertyList.push_back("title label labelString tooltip dirMask tabLabel");
  fPropertyList.push_back("string");

  fPropertyList.push_back("directory pixmap file");
  fPropertyList.push_back("file");

  fPropertyList.push_back("value");
  fPropertyList.push_back("value");

  fPropertyList.push_back("set show opened");
  fPropertyList.push_back("bool");

  fPropertyList.push_back("selection default");
  fPropertyList.push_back("selection");

  fPropertyList.push_back("geometry");
  fPropertyList.push_back("geometry");

  fPropertyList.push_back("decoration");
  fPropertyList.push_back("decoration");

  fPropertyList.push_back("expand");
  fPropertyList.push_back("expand");

  fPropertyList.push_back("splitPosition sizeOfFixed value minimum maximum");
  fPropertyList.push_back("int");

  fPropertyList.push_back("items popupItems");
  fPropertyList.push_back("items");
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::propertyType(
 const std::string& aName
,std::string& aValue
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for (unsigned int index=0;index<fPropertyList.size();index += 2) {
    if((" "+fPropertyList[index]+" ").find(" "+aName+" ") != std::string::npos) {
      aValue = fPropertyList[index+1];
      return true;
    }
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
const std::vector<std::string>& OnX::BaseUI::callbackLanguageList(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCallbackLanguageList;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidgetClass* OnX::BaseUI::findWidgetClass(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fWidgetClasses.size();
  for (unsigned int index=0;index<number;index++) {
    if (fWidgetClasses[index]->name() == aName) {
      return fWidgetClasses[index];
    } 
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* OnX::BaseUI::topItemML(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTopItemML;
}
//////////////////////////////////////////////////////////////////////////////
Lib::ItemMLFactory& OnX::BaseUI::itemMLFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fWidgetFactory;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::loadReferenceFile(
 Lib::ItemML& aItemML
,const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
// Load a file and add its items in the aItemML tree
// insert a <reference> tag before the insertion of item
// this is useful for inserting file from other files.onx 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  
  // build a new component
  // build a label/title propertie if necessairy
  std::vector<Lib::ItemML::Attribute> atts;
  Lib::ItemML* element = fWidgetFactory.create("reference",atts,&aItemML);
  if(!element) return false;
  
  std::string name = buildName(*element);
  if (!element->setAttributeValue("","file",aFile))  return false;
  
  UIML uiml(fWidgetFactory,fVerboseLevel);
  if(!uiml.loadFile(aFile,false,fSession.printer()))    return false;
  Lib::ItemML* top = uiml.topItem();
  if(!top) return false; //FIXME : return true ? file could be empty.
  if(!solveInputDialog(*top)) return false;
  // Create widgets, set hierarchy.
  aItemML.addChild(element);
  top->setParent(element);
  createNativeWidget(*top);
  uiml.empty();
  element->addChild(top);
    
  std::string parentName;
  aItemML.attributeValue("","name",parentName);
  show(parentName);
  Execute tag(*this,"create");
  top->postExecute(executeItemMLCallbacks,&tag);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::tree(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return "";
  return "<tree><treeItem>"+subTree(*fTopItemML,2)+"</treeItem></tree>";
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::BaseUI::subTree(
 Lib::ItemML& aItemML
,int depth
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string line;
  std::string value;
  std::string spaceItem = "";
  std::string spaceRoot = "";
  //spaceItem.insert(0,depth*2,' ');
  //spaceRoot.insert(0,(depth-1)*2,' ');

  aItemML.attributeValue("","class",value);
  line += spaceItem + "<label>"+value;
  aItemML.attributeValue("","name",value);
  value = Lib::smanip::toxml(value);

  line += " name=\""+value+"\"";
  
  const std::vector<Lib::ItemML::Attribute>& atbs = aItemML.attributes();
  unsigned int atbn = atbs.size();
  for(unsigned int index=0;index<atbn;index++) {
    const std::string& atb = atbs[index].first;
    if ((atb != "name") && (atb !="class")){
      line += " "+atb+"=\""+Lib::smanip::toxml(atbs[index].second)+"\"";
    }
  }
  line += "</label>\n";

  std::list<Lib::Element*>::const_iterator it;
  const std::list<Lib::Element*>& props = aItemML.properties();
  for(it=props.begin();it!=props.end();++it) {
    line += spaceRoot + "<treeItem>";
    value = Lib::smanip::toxml((*it)->value());
    
    const std::vector<Lib::ItemML::Attribute>& atbs = (*it)->attributes();
    if (atbs.size() == 0){
      line += spaceItem+"<label>"+(*it)->name()+"=\""+value+"\"</label>";      

    } else {
      line += spaceItem+"<label>"+(*it)->name()+" ";      
      for (unsigned int b=0;b<atbs.size();b++) {
        line += atbs[b].first+"=\""+atbs[b].second+"\" ";
      }
      line += "value=\""+value+"\"</label>";
    }
    line += spaceRoot + "</treeItem>";
  }

  //line += spaceItem + "<opened>false</opened>";
 {const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    line += spaceRoot + "<treeItem>";
    line += subTree(*(*it),depth+1);
    line += spaceRoot + "</treeItem>";
  }}
  return line;
}
//////////////////////////////////////////////////////////////////////////////
// current widget ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidget* OnX::BaseUI::currentWidget(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCurrentWidget;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::setCurrentWidget(
 Slash::UI::IWidget* aWidget
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCurrentWidget = aWidget;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::enableTerminal(
 const std::string& //aPrompt
,const std::string& //aInterp
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::disableTerminal(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::redirectAskToTerminal(
 bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fAskToTerminal = aValue;
  return true;
} 
//////////////////////////////////////////////////////////////////////////////
bool OnX::BaseUI::lockScreen(
 const std::string&
,const std::string&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
} 
