// this :
#include <OnX/Core/Interpreter.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Slash/Core/ILibraryManager.h>

#include <Lib/Manager.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/System.h>

//////////////////////////////////////////////////////////////////////////////
OnX::Interpreter::Interpreter(
 Slash::Core::ISession& aSession
,const std::string& aName
,const std::string& aLibrary
,const std::string& aPackage
,const std::string& aSuffix
,const std::string& aExecFileCmd
,const std::string& aMendatoryEnvironmentVariable
,Slash::UI::IInterpreter::Function aFunction
,void* aTag
)
:fName(aName)
,fLibrary(aLibrary)
,fPackage(aPackage)
,fSuffix(aSuffix)
,fExecFileCmd(aExecFileCmd)
,fMendatoryEnvironmentVariable(aMendatoryEnvironmentVariable)
,fLoaded(false)
,fFunction(aFunction)
,fTag(aTag)
,fSession(aSession)
,fOperationalChecked(false)
,fOperationalStatus(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnX::Interpreter::~Interpreter(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Interpreter::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Interpreter::package(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPackage;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Interpreter::suffix(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSuffix;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Interpreter::execFileCommand(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fExecFileCmd;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Interpreter::setFunction(
 Slash::UI::IInterpreter::Function aFunction
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aFunction) return;
  fFunction = aFunction;
  fTag = aTag;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IInterpreter::Function OnX::Interpreter::function(
 void*& aTag
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aTag = fTag;
  return fFunction;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Interpreter::load(
 bool aQuiet
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fLoaded) return true;
  if(fLibrary!="") {
    Slash::Core::ILibraryManager* libraryManager = 
      Lib_findManager(fSession,"LibraryManager",Slash::Core::ILibraryManager);
    if(!libraryManager) return false;
    if(!libraryManager->addLibrary(fName,fLibrary,fPackage,aQuiet)) 
      return false;
  }
  fLoaded = true;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Interpreter::executeScript(
 const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aScript=="") return true;
  if(!load()) {
    Lib::Out out(fSession.printer());
    out << "OnX::Interpreter::executeScript :"
        << " interpreter " << Lib::sout(fName) 
        << " loading failed." << Lib::endl;
    out << aScript << Lib::endl;
    return false;
  } else {
    if(fFunction) {
      if(fSession.verboseLevel()) {
        Lib::Out out(fSession.printer());
        out << "OnX::Interpreter::executeScript :"
            << " with interpreter " << Lib::sout(fName) 
            << " : " << Lib::endl;
        out << aScript << Lib::endl;
      }
      return fFunction(aScript,fTag);
    } else {
      Lib::Out out(fSession.printer());
      out << "OnX::Interpreter::executeScript :"
          << " Interpreter (" << fName << ") " 
          << "native function not found for script :" << Lib::endl;
      out << aScript << Lib::endl;
      return false;
    }
  }

  //FIXME fSession.flush(); do it here ?

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool  OnX::Interpreter::isOperational(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fOperationalChecked) return fOperationalStatus;

  fOperationalChecked = true;

  fOperationalStatus = false;
  if(fMendatoryEnvironmentVariable!="") {
    bool status = Lib::System::isenv(fMendatoryEnvironmentVariable);
    if(fSession.verboseLevel()) {
      Lib::Out out(fSession.printer());
      out << "OnX::Interpreter::isOperational :"
          << " mendatoryEnvironmentVariable " 
          << Lib::sout(fMendatoryEnvironmentVariable) 
          << " defined : " << status
          << Lib::endl;
    }
    if(!status) return false;
  }
  if(!load(true)) return false;
  if(!fFunction) return false;
  fOperationalStatus = true;
  return true;
}
