// this :
#include <OnX/Core/Main.h>

#include <Slash/Core/IManager.h>
#include <Slash/Core/ILibraryManager.h>
#include <Slash/Core/ILibrary.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IScriptManager.h>
#include <Slash/UI/IUI_Manager.h>

#include <Lib/Printer.h>
#include <Lib/Debug.h>
#include <Lib/smanip.h>
#include <Lib/dirmanip.h>
#include <Lib/fmanip.h>
#include <Lib/System.h>
#include <Lib/Arguments.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Manager.h>
#include <Lib/Cast.h>
#include <Lib/Upgrader.h>
#include <Lib/Cloner.h>
#include <Lib/DLL.h>

#include <OnX/Version.h>
#include <OnX/Core/Session.h>
#include <OnX/Core/Environment.h>

// Embedded code :
#include <OnX/Core/OnX_onx_help.h>

//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::Main::defaultKnownOptions(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> options;
  options.push_back("-h");
  options.push_back("-version");
  options.push_back("-verbose");
  options.push_back("-stdout");
  options.push_back("-defout");
  options.push_back("-uiout");
  options.push_back("-ui");
  options.push_back("-toolkit");
  options.push_back("-Xt");
  options.push_back("-xt");
  options.push_back("-Xm");
  options.push_back("-xm");
  options.push_back("-Motif");
  options.push_back("-motif");
  options.push_back("-Win");
  options.push_back("-win");
  options.push_back("-Win32");
  options.push_back("-win32");
  options.push_back("-Gtk");
  options.push_back("-gtk");
  options.push_back("-Qt");
  options.push_back("-qt");
  options.push_back("-NextStep");
  options.push_back("-nextstep");
  options.push_back("-Batch");
  options.push_back("-batch");
  options.push_back("-Python");
  options.push_back("-python");
  options.push_back("-Tcl");
  options.push_back("-tcl");
  options.push_back("-KUIP");
  options.push_back("-kuip");
  options.push_back("-DLD");
  options.push_back("-dld");
  options.push_back("-session");
  options.push_back("-upgrade");
  options.push_back("-orders");
  options.push_back("-bg");
  options.push_back("-debug_check_class");
  options.push_back("-style");
  options.push_back("-thread");
  options.push_back("-newapp");
  options.push_back("-newpack");
  options.push_back("-template");
  options.push_back("-banner");
  options.push_back("-from");
  options.push_back("-debug");
  options.push_back("-onx_file");
  options.push_back("-ib");
  options.push_back("-enable_terminal");
  options.push_back("-SDL");
  options.push_back("-sdl");
  // SDL_UI :
  options.push_back("-password");
  options.push_back("-video_width");
  options.push_back("-video_height");
  options.push_back("-not_full_screen");
  return options;  
}
//////////////////////////////////////////////////////////////////////////////
OnX::Main::Main(
 bool aCreateUI
,bool aShowUI
,const std::vector<std::string>& aArgs
,const std::vector<std::string>& aOpts
)
:fDummy(false)
,fStatus(false)
,fVerboseLevel(0)
,fSession(0)
,fUI(0)
,fIB(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> options = 
    aOpts.size()?aOpts:defaultKnownOptions();

  Lib::Arguments args(aArgs);
#ifdef __APPLE__
  // Darwin : If started with "open", an encoded pid is passed in argument.
 {std::string key;
  std::string val;
  if(args.match("-psn_*",key,val)) args.remove(key);}
#endif

 {std::string senv;
  if(Lib::System::getenv("ONXARGS",senv)) {
    args.add(Lib::smanip::words(senv," "));
  }}

  if(args.isAnArgument("-debug_check_class")) {
    Lib::Debug::checkByClass(true);
  }

  Lib::Printer printer;

  if(!args.knownOptions(options)) {
    Lib::Out out(printer);
    out << "OnX::Main::Main :"
        << " Some options are unknown." << Lib::endl;
    out << "Passed options was :" << Lib::endl;
    out << args.tostring() << Lib::endl;
    out << "To have help, type :" << Lib::endl;
    out << " OS> onx -h" << Lib::endl;
    //out << "Continue anyway..." << Lib::endl;
    fStatus = false;
    return;
  }

  if(args.isAnArgument("-version")) {
    Lib::Out out(printer);
    out << "OnX::Main::Main :"
        << " version " << ONX_VERSION 
        << Lib::endl;
    fStatus = true;
    return;
  }

  std::string sverbose;
  if(args.find("-verbose",sverbose)) {
    if(sverbose=="") fVerboseLevel = 1;
  }

  std::string sstyle;
  args.find("-style",sstyle);

  std::string arg0(aArgs.size()?aArgs[0]:"");

  if(args.isAnArgument("-h")) {
    std::vector<std::string> text;
    if(!Lib::System::isenv("ONXROOT")) {
      text.resize(OnX_onx_help_linen);
      for(unsigned int index=0;index<OnX_onx_help_linen;index++) {
        std::string s = OnX_onx_help_lines[index];
        Lib::smanip::replace(s,"@@double_quote@@","\"");
        Lib::smanip::replace(s,"@@back_slash@@","\\");
        text[index] = s;
      }
    } else {
      std::string helpFile = "$ONXROOT/scripts/Help/OnX_onx.help";
      if(!Lib::fmanip::read(helpFile,text)) {
        Lib::Out out(printer);
        out << "OnX::Main::Main :"
            << " help file " << Lib::sout(helpFile) << " not found."
            << Lib::endl;
        fStatus = false;
        return;
      }
    }
    Lib::Out out(printer);
    for(unsigned int index=0;index<text.size();index++) {
      out << text[index] << Lib::endl;
    }
    fStatus = true;
    return;
  }

  if(args.isAnArgument("-upgrade")) {
    if(!Lib::System::isenv("ONXROOT")) {
      if(!check_ONXROOT(arg0,printer,fVerboseLevel,true)) {
        fStatus = false;
        return;
      }
    } 
    std::string file;
    if(!args.file(file)) file = ".";
    std::string orders;
    if(!args.find("-orders",orders)) {
      orders = "$ONXROOT/scripts/upgrade/";
      orders += ONX_VERSION_VRP;
    }
    Lib::Upgrader upgrader(printer,fVerboseLevel,orders);
    upgrader.upgrade(file);
    fStatus = true;
    return;
  }


 {std::string name;
  if(args.find("-newapp",name)||args.find("-newpack",name)) {
    if(!Lib::System::isenv("ONXROOT")) {
      if(!check_ONXROOT(arg0,printer,fVerboseLevel,true)) {
        fStatus = false;
        return;
      }
    } 
    if(name=="") name="MyOnXApp";
    std::string path;
    std::string from;
    args.find("-template",from);
    if(from=="") {
      from="OnXTemplateOne";
      path="$ONXROOT/templates/" + from;   
      //path="$OSC_HOME_DIR/Resources/OnX/templates/" + from;   
    } else if(from=="gl") {
      from="OnXTemplateThree";
      path="$ONXROOT/templates/" + from;   
    } else if(from=="iv") {
      from="OnXTemplateFour";
      path="$ONXROOT/templates/" + from;   
    } else if(from=="g4") {
      from="G4LabTemplateOne";
      path="$OSC_HOME_DIR/Resources/G4Lab/templates/" + from;   
    }

    path = Lib::fmanip::name(path);
    bool debug = args.isAnArgument("-debug");
    bool verbose = args.isAnArgument("-verbose");
    fStatus = Lib::Cloner::clone(path,from,name,printer,verbose,debug);
    return;
  }}

 {std::string sessionFile;
  if(!args.find("-session",sessionFile)) {
    if(Lib::System::isenv("ONXSESSION")) {
      sessionFile = Lib::System::getenv("ONXSESSION");
      if(!Lib::fmanip::exists(sessionFile)) {
        Lib::Out out(printer);
        out << "OnXLab::Main::Main :"
            << " Environment variable ONXSESSION does not refer"
            << " to an existing file."
            << " Load the embedded Session.onx." 
            << Lib::endl;
        sessionFile = "";
      }
    } else {
      if(!check_ONXROOT(arg0,printer,fVerboseLevel,false)) {
        sessionFile = "";
      } else {
        sessionFile = "$ONXROOT/scripts/OnX/Session.onx";
      }
    }
  }
  fSession = new OnX::Session(sessionFile,fVerboseLevel);}

  Lib::Out out(fSession->printer());
  fSession->setParameter("session.binary",arg0);
  fSession->setArguments(args.args());
  if(sstyle!="") fSession->setParameter("UI_Manager.style",sstyle);

  if(sverbose!="") {
    // Assume that the given value is the name of a manager.
    // For exa :
    //   OS> onx -verbose=UI_Manager
    fSession->setParameter(sverbose+".verbose","1");
  }

  if(args.isAnArgument("-stdout")) {
    fSession->setParameter("session.output","terminal");
  }
  if(args.isAnArgument("-defout")) {
    fSession->setParameter("session.output","defout");
  }
  if(args.isAnArgument("-uiout")) {
    fSession->setParameter("session.output","UI");
  }

  std::string interp;
  std::string cmd;
  if(args.find("-Python",cmd)||args.find("-python",cmd)) {
    interp = "Python";
  } else if(args.find("-Tcl",cmd)||args.find("-tcl",cmd)) {
    interp = "Tcl";
  } else if(args.find("-KUIP",cmd)||args.find("-kuip",cmd)) {
    interp = "KUIP";
  } else if(args.find("-DLD",cmd)||args.find("-dld",cmd)) {
    interp = "DLD";
  }
  if(args.isAnArgument("-bg")) interp += " bg";

  if(args.isAnArgument("-thread")) {
    fSession->setParameter("UI_Manager.thread","yes");
  }

  fArgs = args.tovector();

  fIB = args.isAnArgument("-ib");

  if(args.find("-enable_terminal",fTerm)) {
    if(fTerm=="") fTerm = "DLD";
    fSession->setParameter("session.output","terminal");
  }

  if(aCreateUI) {
    Slash::UI::IUI* ui = createUI("","",aShowUI);
    if(!ui) {
      delete fSession;
      fSession = 0;
      fStatus = false;
      return;
    } 
  }

  if(interp!="") {
    if(fVerboseLevel) {
      Lib::Out out(fSession->printer());
      out << "OnX::Main::Main :"
          << " execute " << Lib::sout(interp)
          << " : " << Lib::sout(cmd) << "." << Lib::endl;
    }
    // Do not use anymore a UI to execute a script
    // so that someone can execute a script without creating a UI.
    //ui->executeScript(interp,cmd);
    Slash::UI::IScriptManager* sm = 
      Lib_findManager(*fSession,"ScriptManager",Slash::UI::IScriptManager);
    if(!sm) {
      Lib::Out out(printer);
      out << "OnX::Main::Main :"
          << " ScriptManager not found." 
          << " can't execute " << Lib::sout(interp)
          << " : " << Lib::sout(cmd) << "."
          << Lib::endl;
    } else {
      sm->executeScript(interp,cmd);
    }
  }

  fStatus = true;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Main::~Main(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fUI) fUI->hide();

  delete fSession; //Will delete all managers (then the UI).

  Lib::Printer printer;
  if(!fDummy) {
    Lib::Debug::balance(printer);
  }

  if(fVerboseLevel) {
    Lib::Out out(printer);
    out << "OnX::Main::~Main :"
        << " Exiting..." << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI* OnX::Main::createUI(
 const std::string& aDriver
,const std::string& aFile
,bool aShow
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // For the moment only one Slash::UI::IUI handled.
  if(fUI) return fUI; //Done.  //FIXME : ?

  if(!fSession) return 0;

  Slash::UI::IUI_Manager* uiManager = 
    Lib_findManager(*fSession,"UI_Manager",Slash::UI::IUI_Manager);
  if(!uiManager) {
    Lib::Out out(fSession->printer());
    out << "OnX::Main::createUI :"
        << " UI_Manager not found." << Lib::endl;
    return 0;
  }

  // GUI driver :
  std::string driver;
  if(aDriver=="") {
    driver = findDriver(fSession->printer());
  } else {
    driver = aDriver;
  }

  std::string file;
  if(aFile=="") {
    if(!findFile(fSession->printer(),file)) return 0;
  } else {
    file = aFile;
  }

  if(driver=="") {
    Lib::Out out(fSession->printer());
    out << "OnX::Main::createUI :"
        << " no driver specify." << Lib::endl;
    return 0;
  }

  if(file=="") {
    Lib::Out out(fSession->printer());
    out << "OnX::Main::createUI :"
        << " no OnX GUI XML file given." << Lib::endl;
    return 0;
  }

  Slash::UI::IUI* ui = uiManager->create(driver,file,fArgs,aShow);
  if(!ui) {
    Lib::Out out(fSession->printer());
    out << "OnX::Main::createUI :"
        << " can't create UI for driver " << Lib::sout(driver) 
        << " and file " << Lib::sout(file) 
        << Lib::endl;
    return 0;
  }

  fUI = ui;

  if(fIB) {
    ui->executeScript("DLD",
      "OnX ui_show_dialog $ONXROOT/scripts/OnX/IB/IB.onx OnX_IB_dialog");
  }

  if(fTerm!="") ui->enableTerminal(fTerm+"> ",fTerm);

  return ui;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Main::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStatus;
}
//////////////////////////////////////////////////////////////////////////////
int OnX::Main::steer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fUI) return Lib::System::exitSuccess();
  return fUI->steer();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Main::show(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fUI) return;
  fUI->show();
  fUI->steer();
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI* OnX::Main::ui(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUI;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::ISession* OnX::Main::session(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSession;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Main::loadInterpreter(
 const std::string& aInterp
,bool aQuiet
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSession) return false;
  Slash::UI::IScriptManager* sm = 
    Lib_findManager(*fSession,"ScriptManager",Slash::UI::IScriptManager);
  if(!sm) return false;
  Slash::UI::IInterpreter* interp = sm->findInterpreter(aInterp);
  if(!interp) return false;
  return interp->load(aQuiet);
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Main::findDriver(
 Slash::Core::IWriter& aPrinter
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Arguments args(fArgs);

  // GUI driver :
  std::string driver;
 {std::string value;
  if(fSession && 
     fSession->parameterValue("UI_Manager.toolkit",value) &&
     (value!="") ) {
    driver = value;
  }}
  if(driver=="") args.find("-toolkit",driver);
  if(driver=="") {
    if(args.isAnArgument("-Xt")||
       args.isAnArgument("-xt")||
       args.isAnArgument("-Xm")||
       args.isAnArgument("-xm")||
       args.isAnArgument("-Motif")||
       args.isAnArgument("-motif")) {
      driver = "Xt";
    } else if(args.isAnArgument("-Win")||
              args.isAnArgument("-win")||
              args.isAnArgument("-Win32")||
              args.isAnArgument("-win32")) {
      driver = "Win";
    } else if(args.isAnArgument("-NextStep")||
              args.isAnArgument("-nextstep")) {
      driver = "NextStep";
    } else if(args.isAnArgument("-Gtk")||
              args.isAnArgument("-gtk")) {
      driver = "Gtk";
    } else if(args.isAnArgument("-Qt")||
              args.isAnArgument("-qt")) {
      driver = "Qt";
    } else if(args.isAnArgument("-SDL")||
              args.isAnArgument("-sdl")) {
      driver = "SDL";
    } else if(args.isAnArgument("-Batch")||
              args.isAnArgument("-batch")) {
      driver = "";
    }
  }

  if(driver=="") { // Guess something :
#if defined(WIN32)
    driver = "Win";
#else
#ifdef __APPLE__
    if(Lib::DLL::isValid("OnXNextStep",aPrinter)) {
      driver = "NextStep";
    } else if(Lib::DLL::isValid("OnXXt",aPrinter)) {
      driver = "Xt";
    } else if(Lib::DLL::isValid("OnXQt",aPrinter)) {
      driver = "Qt";
    } else {
      driver = "Xt";
    }
#else
    driver = "Xt";
#endif
#endif
  }

  return driver;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Main::findFile(
 Slash::Core::IWriter& aPrinter
,std::string& aFile
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Arguments args(fArgs);

  std::string file;
  if(args.find("-onx_file",file)) {
  } else if(args.file(file)) {
  } else {
    if(Lib::System::isenv("ONXFILE")) {
      file = Lib::System::getenv("ONXFILE");
      if(!Lib::fmanip::exists(file)) {
        Lib::Out out(fSession->printer());
        out << "OnXLab::Main::findFile :"
            << " Environment variable ONXFILE does not refer"
            << " to an existing file."
            << Lib::endl;
        aFile = "";
        return false;
      }
    } else {
      std::string arg0(fArgs.size()?fArgs[0]:"");
      if(!check_ONXROOT(arg0,fSession->printer(),fVerboseLevel,true)) {
        aFile = "";
        return false;
      } else {
        file = "$ONXROOT/scripts/OnX/OnX.onx";
      }
    }
  }

  aFile = file;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::Main::args(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fArgs;
}

#ifdef ONX_SERVER

class FileGetter : public virtual Slash::Core::IFileGetter {
public: //Slash::Core::IFileGetter:
  virtual bool fetch(const std::string& aFileName,std::string& aNewName) {
    if(!fServer.isConnected()) return false;
    // Ask to the client the file :
    std::string ONX_PROT_FILE = "OnX::file::";
    if(!fServer.sendString(ONX_PROT_FILE+aFileName)) {
      fServer.disconnect();
      return false;
    }
    std::vector<string> text;
    if(!fServer.waitText(text)) {
      fServer.disconnect();
      return false;
    }
    std::string path,name,suffix;
    Lib::smanip::pathNameSuffix(aFileName,path,name,suffix);
    aNewName = name;
    Lib::fmanip::write(aNewName,text);
    return true;  
  }  
public:
  FileGetter(Lib::InternetSocket& aServer)
  :fServer(aServer) {
  }
private:
  Lib::InternetSocket& fServer;
};

#endif
