// this :
#include <OnX/Core/ScriptManager.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/ILibraryManager.h>
#include <Lib/Manager.h>
#include <Lib/Out.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/DLL.h>
#include <Lib/Cast.h>

#include <OnX/Core/Interpreter.h>

//////////////////////////////////////////////////////////////////////////////
OnX::ScriptManager::ScriptManager(
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fName("ScriptManager")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnX::ScriptManager::~ScriptManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::UI::IInterpreter*>::iterator it;
  for(it=fList.begin();it!=fList.end();it = fList.erase(it)) delete (*it);
  closeLog();
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::ScriptManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::ScriptManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::UI::IScriptManager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IInterpreter* OnX::ScriptManager::addInterpreter(
 const std::string& aName
,const std::string& aLibrary
,const std::string& aPackage
,const std::string& aSuffix
,const std::string& aFileCmd
,const std::string& aMendatoryEnvironmentVariable
,Slash::UI::IInterpreter::Function aFunction
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="") return 0;
  Slash::UI::IInterpreter* interp = 
    new Interpreter(fSession,aName,
                    aLibrary,aPackage,aSuffix,aFileCmd,
                    aMendatoryEnvironmentVariable,
                    aFunction,aTag);
  fList.push_back(interp);
  return interp;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IInterpreter* OnX::ScriptManager::findInterpreter(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::UI::IInterpreter*>::const_iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    if((*it)->name()==aName) return (*it);
  }
  // Exact matching failed ; tolerate case insensitive :
  std::string sa = aName;
  Lib::smanip::tolowercase(sa);
  for(it=fList.begin();it!=fList.end();++it) {
    std::string si = (*it)->name();
    Lib::smanip::tolowercase(si);
    if(si==sa) return (*it);
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IInterpreter* OnX::ScriptManager::findInterpreterFromSuffix(
 const std::string& aSuffix
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list<Slash::UI::IInterpreter*>::const_iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    if((*it)->suffix()==aSuffix) return (*it);
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::ScriptManager::names(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> vs;
  std::list<Slash::UI::IInterpreter*>::const_iterator it;
  for(it=fList.begin();it!=fList.end();++it) {
    vs.push_back((*it)->name());
  }
  return vs;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::ScriptManager::recordExecutedScripts(
 bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aValue) {
    if(fLogFile!="") return; //done
    // Remove existing log files :
    Lib::fmanip::remove("OnX.log");
    Lib::dirmanip::removeInDirectory(".","OnX_*.log");
    // Get a name for the log file :
    if(Lib::fmanip::exists("OnX.log_tmp")) { // Another OnX process is at work.
      Lib::fmanip::tmpname(fLogFile,".","OnX_",".log_tmp");
    } else {
      fLogFile = "OnX.log_tmp";
    }
  } else {
    closeLog();
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::ScriptManager::executeScript(
 const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aScript=="") return true;

  record(aInterp,aScript);

  Slash::UI::IInterpreter* interpreter = findInterpreter(aInterp);
  if(!interpreter) {
    Lib::Out out(fSession.printer());
    out << "OnX::ScriptManager::executeScript :"
        << " Unknown interpreter : " << aInterp << Lib::endl;
    return false;
  } else {
    bool status = interpreter->executeScript(aScript);
    fSession.flush(); //FIXME : do it here ? Needed only in interactive mode...
    return status;
  }
}
#include <stdio.h>
//////////////////////////////////////////////////////////////////////////////
bool OnX::ScriptManager::record( 
 const std::string& aInterp
,const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fLogFile=="") return true;
  FILE* file = ::fopen(fLogFile.c_str(),"ab");
  if(!file) return false;
  ::fprintf(file,"%s>%s\n",aInterp.c_str(),aString.c_str());
  ::fclose (file);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::ScriptManager::closeLog(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fLogFile=="") return;
  // Rename OnX_<number>.log_tmp in OnX_<number>.log :
  Lib::fmanip::rename(fLogFile,fLogFile.substr(0,fLogFile.size() - 4));
  fLogFile = "";
}
