// this :
#include <OnX/Core/Session.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IManager.h>
#include <Slash/Core/IWriter.h>
#include <Slash/Core/ILibraryManager.h>
#include <Slash/Core/ILibrary.h>

#include <Slash/UI/IUI.h>

#include <Lib/Manager.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/DLL.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/StreamBuffer.h>
#include <Lib/LibraryManager.h>
#include <Lib/RelationManager.h>
#include <Lib/AccessorManager.h>
#include <Lib/System.h>
#include <Lib/ItemML.h>
#include <Lib/Element.h>
#include <Lib/BaseML.h>

#include <OnX/Core/ScriptManager.h>
#include <OnX/Core/StyleManager.h>
#include <OnX/Core/UI_Manager.h>
#include <OnX/Helpers/UI.h>

// Embedded code :
#include <OnX/Core/Session_onx.h>

namespace OnX {
class Printer : public Slash::Core::IWriter {
public: //Slash::Core::IWriter
  virtual bool write(const std::string&);
  virtual void disable();
  virtual void enable();
  virtual bool enabled() const;
  virtual bool flush();
public:
  Printer(Slash::Core::ISession&); 
  virtual ~Printer();
private:
  Slash::Core::ISession& fSession;
  bool fEnabled;
  int fLevel;
  void* fFILE;
};
}

// Should be after any inclusion of string :
#ifdef HasAnsiStream
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

// To redirect stdout :
#include <stdio.h>
#ifdef WIN32
#include <io.h>
#else
#include <unistd.h>
#endif

//#define ONX_TEST_ATEXIT

#ifdef ONX_TEST_ATEXIT
#include <stdlib.h> //atexit
#include <stdio.h>
static void session_atexit() {
  ::printf("session_atexit.\n");
}
#endif


//////////////////////////////////////////////////////////////////////////////
OnX::StreamDestination::StreamDestination (
 Slash::Core::IWriter& aPrinter
) 
:fPrinter(aPrinter)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnX::StreamDestination::~StreamDestination (
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//

{
}
//////////////////////////////////////////////////////////////////////////////
int OnX::StreamDestination::receive(
 const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPrinter.write(aString);
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static bool system_Function(const std::string&,void*);
static bool sh_Function(const std::string&,void*);
static bool DOS_Function(const std::string&,void*);
static bool session_Function(const std::string&,void*);
static bool dld_Function(const std::string&,void*);
static bool cpp_Function(const std::string&,void*);
//////////////////////////////////////////////////////////////////////////////
OnX::Session::Session(
 const std::string& aFile
,int aVerboseLevel
)
:fPrinter(0)
,fOutOld(0)
,fOutNew(0)
,fErrOld(0)
,fErrNew(0)
,fOutFile(0)
,fErrFile(0)
,fParameters("","")
,fVerboseLevel(aVerboseLevel)
,fName("OnX_Session")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef ONX_TEST_ATEXIT
  ::printf("OnX::Session::Session : set atexit(session_atexit).\n");
  ::atexit(session_atexit);
#endif

  fPrinter = new OnX::Printer(*this);

  setParameter("session.output","UI");
  setParameter("AccessorManager.dumpMode","table");
  setParameter("session.helpOnDialog","yes");
  // Used in the OnX/NextDialog dialog :
  setParameter("ui.exitSecondaryLoop","no");

  // Session XML file :
  Lib::ItemML* top = 0;
  Lib::ItemMLFactory factory;
  Lib::BaseML sessionML(factory);
  std::vector<std::string> tags;
  tags.push_back("session");
  tags.push_back("manager");
  tags.push_back("interpreter");
  sessionML.setTags(tags);
  if(aFile!="") {
    sessionML.loadFile(aFile,false,*fPrinter);
    top = sessionML.topItem();
  } else {
    std::string s = Lib::smanip::tostring(Session_onx_linen,
                                          Session_onx_lines);
    Lib::smanip::replace(s,"@@double_quote@@","\"");
    Lib::smanip::replace(s,"@@back_slash@@","\\");
    sessionML.loadString(s,*fPrinter);
    top = sessionML.topItem();
  }
  std::string svalue;
  int ivalue;
  if(top) {

   {const std::list<Lib::Element*>& properties = top->properties();
    std::list<Lib::Element*>::const_iterator it;
    for(it=properties.begin();it!=properties.end();++it) {
      std::string name = (*it)->name();
      std::string value = (*it)->value();
      //printf("debug : \"%s\" \"%s\"\n",name.c_str(),value.c_str());
      if(name=="verbose") {
        if(Lib::smanip::toint(value,ivalue)) fVerboseLevel = ivalue;
      } else if(name=="output") {
        setParameter("session.output",value);
      } else {
        setParameter(name,value);
      }
    }}

    // Set some manager attribute (verbose) in the session parameters :
    // <manager>.<attribute>:<value>
    const std::list<Lib::ItemML*> managers = top->children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=managers.begin();it!=managers.end();++it) {
      std::string sname;
      if((*it)->attributeValue("","name",sname)) {
        std::string param = sname + ".verbose";
        if((*it)->propertyValue("verbose",svalue) && 
           Lib::smanip::toint(svalue,ivalue) ) {
          setParameter(param,Lib::smanip::tostring(ivalue));
        } else {
          setParameter(param,Lib::smanip::tostring(fVerboseLevel));
        }
      }
    }

  }

  // Should be the first.
  if(!findManager("LibraryManager")) {
    Lib::LibraryManager* libraryManager = new Lib::LibraryManager(*this);
    addManager(libraryManager);
  }

  if(!findManager("AccessorManager")) {
    Lib::AccessorManager* accessorManager = new Lib::AccessorManager(*fPrinter);
    addManager(accessorManager);

    Lib::ItemML* item = 0;
    if(top && (item = top->findItem("AccessorManager")) ) {
      if(item->propertyValue("verbose",svalue)) {
        if(Lib::smanip::toint(svalue,ivalue))
          accessorManager->setVerboseLevel(ivalue);
      }
    } else {
      accessorManager->setVerboseLevel(fVerboseLevel);
    }

    accessorManager->addAction(new Lib::CollectAction(*accessorManager));
    accessorManager->addAction(new Lib::FilterAction(*accessorManager));
    accessorManager->addAction(new Lib::DumpAction(*accessorManager));
    accessorManager->addAction(new Lib::NumberAction(*accessorManager));
    accessorManager->addAction(new Lib::GetValuesAction(*accessorManager));
    accessorManager->addAction(new Lib::DestroyAction(*accessorManager));
    accessorManager->addAction(new Lib::SetAction(*accessorManager));
    accessorManager->addAction(new Lib::CalculateAction(*accessorManager));
    accessorManager->addAction(new Lib::VisualizeAction(*accessorManager));
  }

  if(!findManager("RelationManager")) {
    addManager(new Lib::RelationManager(*this));
  }

  if(!findManager("ScriptManager")) {
    OnX::ScriptManager* scriptManager = new OnX::ScriptManager(*this);
    addManager(scriptManager);

    // Default interpreters :
    scriptManager->addInterpreter("system","","","","onx_file","",
                                  system_Function,0);
    scriptManager->addInterpreter("sh","","","sh","onx_file","",
                                  sh_Function,0);
    scriptManager->addInterpreter("DOS","","","bat","call onx_file","",
                                  DOS_Function,0);

    scriptManager->addInterpreter("Session","","","","","",
                                  session_Function,(Slash::Core::ISession*)this);

    scriptManager->addInterpreter("DLD","","","","","",
                                  dld_Function,(Slash::Core::ISession*)this);
    // deprecated
    scriptManager->addInterpreter("C++","","","","","",
                                  cpp_Function,(Slash::Core::ISession*)this);

    Lib::ItemML* item = 0;
    if(top && (item = top->findItem("ScriptManager")) ) {
      if(item->propertyValue("recordExecutedScripts",svalue)) {
        bool bvalue;
        if(Lib::smanip::tobool(svalue,bvalue))
          scriptManager->recordExecutedScripts(bvalue);
      }
      const std::list<Lib::ItemML*> interpreters = item->children();
      std::list<Lib::ItemML*>::const_iterator it;
      for(it=interpreters.begin();it!=interpreters.end();++it) {
        std::string sname;
        if((*it)->attributeValue("","name",sname)) {
          std::string slibrary;
          if((*it)->propertyValue("library",slibrary)) {
            std::string spackage;
            if((*it)->propertyValue("package",spackage)) {
              std::string ssuffix;
              if((*it)->propertyValue("suffix",ssuffix)) {
                std::string scmd;
                if((*it)->propertyValue("execFileCommand",scmd)) {
                  std::string senv;
                  (*it)->propertyValue("mendatoryEnvironmentVariable",senv);

                  if(fVerboseLevel) {
                    Lib::Out out(*fPrinter);
                    out << "OnX::Session::Session :"
                        << " from file " << Lib::sout(aFile)
                        << " addInterpreter :" 
                        << " name " << Lib::sout(sname)
                        << " library " << Lib::sout(slibrary)
                        << " package " << Lib::sout(spackage)
                        << " suffix " << Lib::sout(ssuffix)
                        << " execFileCommand " << Lib::sout(scmd)
                        << " mendatoryEnvironmentVariable " << Lib::sout(senv)
                        << Lib::endl;
                  }
                  scriptManager->addInterpreter(sname,slibrary,spackage,
                                                ssuffix,scmd,senv,0,0);
                }
              }
            }
          }
        }
      }
    }
  }

  if(!findManager("StyleManager")) {
    addManager(new OnX::StyleManager(*this));
  }

  
  if(!findManager("UI_Manager")) {
    OnX::UI_Manager* uiManager = new OnX::UI_Manager(*this);
    addManager(uiManager);
    //FIXME : put the below params on the UI_Manager.
    Lib::ItemML* item = 0;
    if(top && (item = top->findItem("UI_Manager")) ) {
      if(item->propertyValue("toolkit",svalue) && (svalue!="")) {
        setParameter("UI_Manager.toolkit",svalue);
      }
      if(item->propertyValue("threaded",svalue) && (svalue!="")) {
        setParameter("UI_Manager.threaded",svalue);
      }
      if(item->propertyValue("style",svalue) && (svalue!="")) {
        setParameter("UI_Manager.style",svalue);
      }
    }
  }

  // Test dynamic loading :
  Lib::DLL* testDLL = new Lib::DLL("OnXTestDLD",*fPrinter,fVerboseLevel);
  if(testDLL) {
    typedef void(*FT)(int);
    FT OnX_Test_Init = (FT)testDLL->findProcedure("OnX_Test_Init");
    if(OnX_Test_Init) OnX_Test_Init(fVerboseLevel);
    delete testDLL;
  }
}
//////////////////////////////////////////////////////////////////////////////
OnX::Session::~Session(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  stdoutToDefault();

  fPrinter->disable(); // Can't delete fPrinter here.

  // LibManager must be deleted last !

  while(!fManagers.empty()) {
    std::vector<Slash::Core::IManager*>::iterator it = fManagers.end();
    it--;
    if(fVerboseLevel) {
      Lib::Out out(*fPrinter);
      out << "OnX::Session::~Session :"
          << " delete manager " << Lib::sout((*it)->name()) 
          << "." << Lib::endl;
    }
    Slash::Core::IManager* manager = *it;
    fManagers.erase(it); //WARNING : must be done before the below.
    delete manager; //WARNING : some ~manager may rm themself from the list.
  }

  coutToDefault();

  delete fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Session::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::setVerboseLevel(
 int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fVerboseLevel = aVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
int OnX::Session::verboseLevel(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVerboseLevel;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IWriter& OnX::Session::printer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return *fPrinter;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Session::load(
 const std::string& aFile
,const std::string& aPackage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ILibraryManager* libraryManager = 
    Lib_findManager(*this,"LibraryManager",Slash::Core::ILibraryManager);
  if(!libraryManager) return false;
  return libraryManager->addLibrary(aPackage,aFile,aPackage) ? true : false;
}
#include <stdio.h>
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::redirectOutput(
 bool aYesNo
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aYesNo) {
    coutToOnX();    // cout to fPrinter.
    stdoutToOnX();  // stdout to fPrinter.
    fPrinter->enable(); // Slash::Core::IWriter to Slash::UI::IUI.
  } else {
    stdoutToDefault(); // stdout to terminal.
    coutToDefault();   // cout to terminal.
    fPrinter->disable();   // Slash::Core::IWriter to stdout.
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::coutToOnX(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
//#define NO_COUT
#if !defined(NO_COUT) && !defined(__hpux)
  if(!fOutNew) {
    fOutNew = new Lib::StreamBuffer(new OnX::StreamDestination(*fPrinter));
#ifdef HasAnsiStream
    fOutOld = cout.rdbuf(fOutNew);
#else
    cout = (streambuf*)fOutNew;
#endif
  }
  if(!fErrNew) {
    fErrNew = new Lib::StreamBuffer(new OnX::StreamDestination(*fPrinter));
#ifdef HasAnsiStream
    fErrOld = cerr.rdbuf(fErrNew);
#else
    cerr = (streambuf*)fErrNew;
#endif
  }
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::coutToDefault(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //  On HP the cout = oldStreamBuffer induces 
  // a crash at return of the main() of oxm program.
#if !defined(NO_COUT) && !defined(__hpux)
  if(fOutNew) {
#ifdef HasAnsiStream
    cout.rdbuf(fOutOld);
#else
    cout = fOutOld;
#endif
    delete fOutNew;
    fOutNew = 0;
  }
  if(fErrNew) {
#ifdef HasAnsiStream
    cerr.rdbuf(fErrOld);
#else
    cout = fErrOld;
#endif
    delete fErrNew;
    fErrNew = 0;
  }
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::stdoutToOnX(
)
//////////////////////////////////////////////////////////////////////////////
// See CINT/disp.c, pause.c code.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Avoid to redirect stdout, stderr.
  // On Darwin and Windows we loose the Python output !
  //return;

  //if ((::dup2(::fileno(stdout), ::fileno(stderr))) >= 0) {
  //} 
  /*FIXME :
    Linux : if starting with the mouse, we loose output 
     (redirected to /dev/null ???)
  */

  if(!fOutFile) {
    if(Lib::fmanip::tmpname(fOutName,".","OnX_",".out")) {
#ifdef WIN32
      fOutDefault = "CONOUT$";
#else
      char* stdoutName = ttyname(STDOUT_FILENO);
      // Could be NULL if stdout had been redirected on the command line.
      fOutDefault = stdoutName ? std::string(stdoutName) : std::string("");
#endif
      if(fOutDefault!="") {
        if (::freopen(fOutName.c_str(), "w", stdout)) {
          fOutFile = ::fopen(fOutName.c_str(), "r");
        } else {
          Lib::fmanip::remove(fOutName);
        }
      }
    }
  }
  if(!fErrFile) {
    if(Lib::fmanip::tmpname(fErrName,".","OnX_",".err")) {
#ifdef WIN32
      fErrDefault = "CONOUT$";
#else
      char* stderrName = ttyname(STDERR_FILENO);
      // Could be NULL if stderr had been redirected on the command line.
      fErrDefault = stderrName ? std::string(stderrName) : std::string("");
#endif
      if(fErrDefault!="") {
        if (::freopen(fErrName.c_str(), "w", stderr)) {
          fErrFile = ::fopen(fErrName.c_str(), "r");
        } else {
          Lib::fmanip::remove(fErrName);
        }
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::stdoutToDefault(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fOutFile) {
    if (::freopen(fOutDefault.c_str(), "w", stdout)) {}
    ::fclose((FILE*)fOutFile);
    fOutFile = 0;
    Lib::fmanip::remove(fOutName);
  }
  if(fErrFile) {
    if (::freopen(fErrDefault.c_str(), "w", stderr)) {}
    ::fclose((FILE*)fErrFile);
    fErrFile = 0;
    Lib::fmanip::remove(fErrName);
  }
}
#define READBUFSIZE 32768  //16384  //65536;
//#define READBUFSIZE 16
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::flush(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fPrinter) return;
  char buf[READBUFSIZE];
  int len;
  if(fOutFile) {
    ::fflush(stdout);
    do {
      len = ::fread(buf,1,READBUFSIZE-1,(FILE*)fOutFile);
      if(len<=0) break;
      buf[len] = '\0';
      fPrinter->write(buf);
    } while (len > 0);
  }
  if(fErrFile) {
    ::fflush(stderr);
    do {
      len = ::fread(buf,1,READBUFSIZE-1,(FILE*)fErrFile);
      if(len<=0) break;
      buf[len] = '\0';
      fPrinter->write(buf);
    } while (len > 0);
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::addManager(
 Slash::Core::IManager* aManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(findManager(aManager->name())) {
    Lib::Out out(*fPrinter);
    out << "OnX::Session::addManager :"
        << " WARNING : a manager with name "
        << Lib::sout(aManager->name()) << " already exists." << Lib::endl;
  }
  if(fVerboseLevel) {
    Lib::Out out(*fPrinter);
    out << "OnX::Session::addManager :"
        << " " << Lib::sout(aManager->name()) << "." << Lib::endl;
  }
  fManagers.push_back(aManager);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::removeManager(
 Slash::Core::IManager* aManager
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fVerboseLevel) {
    Lib::Out out(*fPrinter);
    out << "OnX::Session::removeManager :"
        << " " << Lib::sout(aManager->name()) << "." << Lib::endl;
  }
  std::vector<Slash::Core::IManager*>::iterator it;
  for(it=fManagers.begin();it!=fManagers.end();) {
    if((*it)==aManager) {
      it = fManagers.erase(it);
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IManager* OnX::Session::findManager(
 const std::string& aString
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::const_iterator it;
  for(it=fManagers.begin();it!=fManagers.end();++it) {
    if(aString==(*it)->name()) return *it;
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
unsigned int OnX::Session::numberOfManagers(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fManagers.size();
}
//////////////////////////////////////////////////////////////////////////////
const Slash::Core::IManager* OnX::Session::manager(
 unsigned int aIndex
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fManagers[aIndex];
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::Session::managerNames(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> names;
  std::vector<Slash::Core::IManager*>::const_iterator it;
  for(it=fManagers.begin();it!=fManagers.end();++it) {
    names.push_back((*it)->name());
  }
  return names;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::destroyManager(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<Slash::Core::IManager*>::iterator it;
  for(it=fManagers.begin();it!=fManagers.end();) {
    Slash::Core::IManager* manager = *it;
    if(manager->name()==aName) {
      it = fManagers.erase(it);
      if(fVerboseLevel) {
        Lib::Out out(*fPrinter);
        out << "OnX::Session::destroyManager :"
            << " " << Lib::sout(aName) << "." 
            << Lib::endl;
      }
      delete manager;
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Session::setParameter(
 const std::string& aKey
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aKey=="") return false;
  if(fParameters.isKey(aKey)) { 
    fParameters.setValue(aKey,aValue);
  } else {
    fParameters.add(aKey,"",aValue,"","");
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Session::parameterValue(
 const std::string& aKey
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fParameters.isKey(aKey)) {
    aValue = fParameters.value(aKey);
    return true;
  } else {
    aValue = "";
    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> OnX::Session::availableParameters(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fParameters.keys();
}
//////////////////////////////////////////////////////////////////////////////
std::vector< std::pair<std::string,std::string> > OnX::Session::arguments(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fArguments;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Session::setArguments(
 const std::vector< std::pair<std::string,std::string> >& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fArguments = aArgs;
}

#include <stdio.h>
#include <stdarg.h>

OnX::Printer::Printer(Slash::Core::ISession& aSession)
:fSession(aSession)
,fEnabled(true)
,fFILE(0)
{
  //fFILE = ::fopen("OnX_debug","wb");
}
OnX::Printer::~Printer() {
  if(fFILE) ::fclose((FILE*)fFILE);
}
bool OnX::Printer::write(const std::string& aString) {
  if(fFILE) {
    ::fprintf((FILE*)fFILE,"%s",aString.c_str());
  } else {
    Slash::UI::IUI* ui = 0;
    if(fEnabled && (ui = Lib_findUI(fSession)) ) {
      ui->put(aString);
    } else {
      ::printf("%s",aString.c_str());
    }
  }
  return true;
}
void OnX::Printer::disable() {
  fEnabled = false;
}
void OnX::Printer::enable() {
  fEnabled = true;
}
bool OnX::Printer::enabled() const {
  return fEnabled;
}
bool OnX::Printer::flush() {
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool system_Function(
 const std::string& aString
,void*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::System::execute(aString);
  return true;  
}
//////////////////////////////////////////////////////////////////////////////
bool sh_Function(
 const std::string& aString
,void*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(Lib::System::isKindOf("UNIX")) Lib::System::execute(aString);
  return true;  
}
//////////////////////////////////////////////////////////////////////////////
bool DOS_Function(
 const std::string& aString
,void*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(Lib::System::isKindOf("Windows")) Lib::System::execute(aString);
  return true;  
}
//////////////////////////////////////////////////////////////////////////////
bool dld_Function(
 const std::string& aString
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aTag;
  Lib::Out out(session->printer());

  if(session->verboseLevel()) {
    out << "OnX::Session::dld_Function :"
        << " execute " << Lib::sout(aString) << "..."
        << Lib::endl;
  }

  Slash::UI::IUI* ui = Lib_findUI(*session);
  if(!ui) {
    if(session->verboseLevel()) {
      out << "OnX::Session::dld_Function :"
          << " Slash::UI::IUI not found" 
          << Lib::endl;
    }
    return false;
  }

  std::string sdll;
  std::string scbk;

  std::vector<std::string> words;
  Lib::smanip::words(aString," ",words);
  if(words.size()<2) {
    out << "OnX::Session::dld_Function :"
        << " bad syntax in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }
  sdll = words[0];
  scbk = words[1];
  Lib::smanip::strip(sdll);
  Lib::smanip::strip(scbk);

  std::vector<std::string> args;
  for(unsigned int index=2;index<words.size();index++) 
    args.push_back(words[index]);

  if(sdll=="") {
    out << "OnX::Session::dld_Function :"
        << " dll not specified in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }
  if(scbk=="") {
    out << "OnX::Session::dld_Function :"
        << " cbk not specified in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }

  Slash::Core::ILibraryManager* libraryManager = 
    Lib_findManager(*session,"LibraryManager",Slash::Core::ILibraryManager);
  if(libraryManager) {
    Slash::Core::ILibrary* library = libraryManager->addLibrary(sdll,sdll,"");
    if(library) {
      typedef void(*Callback)(Slash::UI::IUI&,const std::vector<std::string>&);
      Callback cbk = (Callback)library->findProcedure(scbk);
      if(cbk) cbk(*ui,args);
    }
  }

  return true;  
}
//////////////////////////////////////////////////////////////////////////////
bool cpp_Function(
 const std::string& aString
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aTag;

  bool warn = true;
  std::string svalue;
  if(session->parameterValue("session.dld_cpp_warning",svalue)) {
    if(!Lib::smanip::tobool(svalue,warn)) warn = true;
  }
  if(warn) {
    Lib::Out out(session->printer());
    out << "OnX::ScriptManager::executeScript :"
        << " keyword " << Lib::sout("C++") 
        << " deprecated, use " << Lib::sout("DLD") << ". Script is :" 
        << Lib::endl;    
    out << aString << Lib::endl;    
  }

  return dld_Function(aString,aTag);  
}
//////////////////////////////////////////////////////////////////////////////
bool session_Function(
 const std::string& aString
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aTag;
  Lib::Out out(session->printer());

  if(session->verboseLevel()) {
    out << "OnX::Session::session_Function :"
        << " execute " << Lib::sout(aString) << "..."
        << Lib::endl;
  }

  std::string sdll;
  std::string scbk;

  std::vector<std::string> words;
  Lib::smanip::words(aString," ",words);
  if(words.size()<2) {
    out << "OnX::Session::session_Function :"
        << " bad syntax in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }
  sdll = words[0];
  scbk = words[1];
  Lib::smanip::strip(sdll);
  Lib::smanip::strip(scbk);

  std::vector<std::string> args;
  for(unsigned int index=2;index<words.size();index++) 
    args.push_back(words[index]);

  if(sdll=="") {
    out << "OnX::Session::session_Function :"
        << " dll not specified in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }
  if(scbk=="") {
    out << "OnX::Session::session_Function :"
        << " cbk not specified in " << Lib::sout(aString) << "." 
        << Lib::endl;
    return false;
  }

  Slash::Core::ILibraryManager* libraryManager = 
    Lib_findManager(*session,"LibraryManager",Slash::Core::ILibraryManager);
  if(libraryManager) {
    Slash::Core::ILibrary* library = libraryManager->addLibrary(sdll,sdll,"");
    if(library) {
      typedef void(*Callback)(Slash::Core::ISession&,const std::vector<std::string>&);
      Callback cbk = (Callback)library->findProcedure(scbk);
      if(cbk) cbk(*session,args);
    }
  }

  return true;
}
