#include <OnX/Core/Term.h>

#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>

#define HAS_TERM

#ifdef HAS_TERM
#include <OnX/Core/Getline.h>
#endif


//////////////////////////////////////////////////////////////////////////////
OnX::Term::Term(
 Slash::UI::IUI& aUI
,const std::string& aPrompt
,const std::string& aInterp
)
:fUI(aUI)
,fPrompt(aPrompt)
,fInterp(aInterp)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  OnX_Term_Gl_histinit("OnX_term.hist");
  //WARNING : the prompt address is put in the static gl_prmpt !
  OnX_Term_Getlinem(-1,fPrompt.c_str());
#endif
}
//////////////////////////////////////////////////////////////////////////////
OnX::Term::~Term(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  OnX_Term_Gl_erase_line();
  //FIXME : have to reset the cursor to the begin of line and 
  //        erase the prompt. This would avoid to have in 
  //        OpenPAWFinalize an extra 
  //          out << Lib::endl;
  OnX_Term_Getlinem(2,0);
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Term::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  return true;
#else 
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Term::cyclicExecute(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  //printf("debug : term cyclic\n");
  int is_stdin;
  if(!OnX_Gl_is_there_stdin_input(&is_stdin)) {
    Lib::Out out(fUI.session().printer());
    out << "OnX::Term::cyclicExecute :"
        << " problem with select."
        << Lib::endl;
  } else {
    if(is_stdin) {
      char* p = OnX_Term_Getlinem(1,0);
      if(p) {
        if(*p) {
          std::string cmd(p);
          Lib::smanip::strip(cmd,Lib::smanip::trailing,'\n');

          OnX_Term_Gl_histadd((char*)cmd.c_str());
          //::printf("|%s|\n",cmd.c_str());

          fUI.executeScript(fInterp,cmd);

          OnX_Term_Getlinem(-1,fPrompt.c_str());

        } else { // ^D
          fUI.executeScript("DLD","OnX ui_exit");
          OnX_Term_Getlinem(-1,fPrompt.c_str());
        }
      }
    }
  } 
#endif
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::Term::ask(
 const std::string& aPrompt
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  char* p = OnX_Term_Getline(aPrompt.c_str());
  if(!(*p)) {
    //^D
    return "";
  } else {
    std::string cmd(p);
    Lib::smanip::strip(cmd,Lib::smanip::trailing,'\n');
    return cmd;
  }
#else
  return "";
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::Term::initPrompt(
 const std::string& aPrompt
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  OnX_Term_Getlinem(-1,aPrompt.c_str());
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Term::getInput(
 eKind& aWhat
,std::string& aInput
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  char* p = OnX_Term_Getlinem(1,0);
  if(!p) {
    aInput = "";
    aWhat = no_input;
  } else {
    if(*p) {
      aInput = std::string(p);
      Lib::smanip::strip(aInput,Lib::smanip::trailing,'\n');
      aWhat = input;
    } else { //^D
      aInput = "";
      aWhat = escape;
    }
  }
  return true;
#else
  aInput = "";
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::Term::is_there_stdin_input(
 bool& aIs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_TERM
  int is_stdin;
  if(!OnX_Gl_is_there_stdin_input(&is_stdin)) {
    aIs = false;
    return false;
  }
  aIs = is_stdin?true:false; 
  return true;
#else
  return false;
#endif
}
