// this :
#include <OnX/Core/UI_Manager.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/ILibrary.h>
#include <Slash/Core/ILibraryManager.h>
#include <Lib/Manager.h>
#include <Lib/Cast.h>
#include <Lib/Out.h>
#include <Lib/sout.h>

#include <Slash/UI/IUI.h>

//////////////////////////////////////////////////////////////////////////////
OnX::UI_Manager::UI_Manager(
 Slash::Core::ISession& aSession
)
:fSession(aSession)
,fName("UI_Manager")
,fUI(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnX::UI_Manager::~UI_Manager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fUI) {
    fUI->finalize();
    delete fUI;
  }
  fUI = 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::UI_Manager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::UI_Manager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::UI::IUI_Manager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::UI_Manager::add(
 Slash::UI::IUI* aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fUI) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::add :"
        << " a UI already exist." 
        << Lib::endl;
    return false;
  }
  fUI = aUI; //take ownership.
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI* OnX::UI_Manager::find(
  const std::string& //aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUI;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI* OnX::UI_Manager::create(
 const std::string& aDriver
,const std::string& aFile
,const std::vector<std::string>& aArgs
,bool aShow
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // For the moment only one Slash::UI::IUI handled.
  if(fUI) return fUI; //Done.

  std::string library = "OnX"+aDriver;

  Slash::Core::ILibraryManager* libraryManager = 
    Lib::findManager<Slash::Core::ILibraryManager>(fSession,
                                      "LibraryManager",
                                      "Slash::Core::ILibraryManager");
  if(!libraryManager) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " LibraryManager not found." << Lib::endl;
    return 0;
  }
  Slash::Core::ILibrary* dll = 
    libraryManager->addLibrary("OnX_driver",library,"");
  if(!dll) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Unable to add DLL " << Lib::sout(library) 
        << Lib::endl;
    return 0;
  }

  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Load " << Lib::sout(aDriver) << " driver..." << Lib::endl;
  }

  std::string function = "OnX_create"+aDriver+"UI";
  Factory uiFactory = (Factory)dll->findProcedure(function);
  if(!uiFactory) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Function " << Lib::sout(function) << " not found." << Lib::endl;
    return 0;
  }

  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Load file " << Lib::sout(aFile) << "." << Lib::endl;
  }

  fUI = uiFactory(fSession,aFile,aArgs);
  if(!fUI) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Creation of UI for " << Lib::sout(aFile) << " failed." 
        << Lib::endl;
    return 0;
  }

  if(!fUI->isValid()) {
    delete fUI;
    fUI = 0; // To redirect output to stdout before the below print.
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " Problem when creating the UI for file " 
        << Lib::sout(aFile) << "." 
        << Lib::endl;
    return 0;
  }

  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "OnX::UI_Manager::create :"
        << " loading " << Lib::sout(aFile) << " ok." << Lib::endl;
  }

  if(aShow) fUI->show();
  fUI->executeCallbacks("*","create");

  return fUI;
}
