// this :
#include <OnX/Core/WidgetFactory.h>

// Slash :
#include <Slash/UI/IWidgetClass.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/ISoViewer.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/Cast.h>


#include <OnX/Core/Widget.h>

namespace OnX {

class SoViewer : public OnX::Widget, public virtual Slash::UI::ISoViewer {
public: //Slash::UI::ISoViewer
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::SoViewer)
    else if_Lib_SCast(Slash::UI::ISoViewer)
    else return OnX::Widget::cast(aClass);
  }
  virtual SoNode* sceneGraph() const {return fSoNode;}
  virtual void setSceneGraph(SoNode* aNode) { fSoNode = aNode;}
  virtual void setBackgroundColor(double aRed,double aGreen,double aBlue) {
    //FIXME : avoid passing by IUI.
    std::string s;
    Lib::smanip::printf(s,96,"%g %g %g",aRed,aGreen,aBlue);
    fUI.setParameter(name()+".backgroundColor",s);
  }
  virtual void setFeedbackVisibility(bool aValue) {
    //FIXME : avoid passing by IUI.
    fUI.setParameter
      (name()+".feedbackVisibility",Lib::smanip::tostring(aValue));
  }
  virtual void setAutoClipping(bool aValue) {
    //FIXME : avoid passing by IUI.
    fUI.setParameter
      (name()+".autoClipping",Lib::smanip::tostring(aValue));
  }
  virtual void setDecoration(bool aValue) {
    //FIXME : avoid passing by IUI.
    fUI.setParameter
      (name()+".decoration",Lib::smanip::tostring(aValue));
  }
  virtual void setViewing(bool aValue) {
    //FIXME : avoid passing by IUI.
    fUI.setParameter
      (name()+".viewing",Lib::smanip::tostring(aValue));
  }
  virtual void setHeadlight(bool aValue) {
    fUI.setParameter(name()+".headLight",Lib::smanip::tostring(aValue));
  }
  virtual void render() {
    //FIXME : avoid passing by IUI.
    fUI.setParameter(name()+".render","");
  }
  virtual void viewAll() {
    //FIXME : avoid passing by IUI.
    fUI.setParameter(name()+".view","all");
  }
public:
  SoViewer(Slash::UI::IUI& aUI,
           Lib::ItemMLFactory& aFactory,
           Lib::ItemML* aParent,
           Slash::UI::IWidgetClass& aWidgetClass)
  :OnX::Widget(aUI,aFactory,aParent,aWidgetClass)
  ,fSoNode(0)
  {}
  virtual ~SoViewer(){}
private:
  SoNode* fSoNode;
};

  /*
class ScrolledTree : public Lib::ItemML, public virtual IScrolledTree {
public: //Slash::UI::IWidget
  virtual Slash::UI::IWidgetClass& widgetClass() const { return fWidgetClass;}
  virtual std::string name() const { 
    std::string value;
    attributeValue("","name",value);
    return value;
  }
  virtual void* nativeWidget() const {
    return getData1();
  }
public: //Slash::UI::IScrolledTree
  virtual std::vector<std::string> children(const std::string&) {
  }
public: //IScrolledTree
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::ScrolledTree)
    else if_Lib_SCast(IScrolledTree)
    else if_Lib_SCast(Slash::UI::IWidget)
    else return Lib::ItemML::cast(aClass);
  }
public:
  ScrolledTree(Lib::ItemMLFactory& aFactory,
               Lib::ItemML* aParent,
               Slash::UI::IWidgetClass& aWidgetClass)
  :Lib::ItemML("widget",aFactory,aParent)
  ,fWidgetClass(aWidgetClass) {
  virtual ~ScrolledTree(){
  }
private:
  Slash::UI::IWidgetClass& fWidgetClass;
};

class OpenGLArea : public Lib::ItemML, public virtual IOpenGLArea {
public: //Slash::UI::IWidget
  virtual Slash::UI::IWidgetClass& widgetClass() const { return fWidgetClass;}
  virtual std::string name() const { 
    std::string value;
    attributeValue("","name",value);
    return value;
  }
  virtual void* nativeWidget() const {
    return getData1();
  }
  virtual void refresh() {
  }
public: //IOpenGLArea
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::OpenGLArea)
    else if_Lib_SCast(IOpenGLArea)
    else if_Lib_SCast(Slash::UI::IWidget)
    else return Lib::ItemML::cast(aClass);
  }
public:
  OpenGLArea(Lib::ItemMLFactory& aFactory,
             Lib::ItemML* aParent,
             Slash::UI::IWidgetClass& aWidgetClass)
  :Lib::ItemML("widget",aFactory,aParent)
  ,fWidgetClass(aWidgetClass) {
  virtual ~OpenGLArea(){
  }
private:
  Slash::UI::IWidgetClass& fWidgetClass;
};
  */

}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::WidgetFactory::WidgetFactory(
 Slash::UI::IUI& aUI
,Slash::Core::IWriter& aPrinter
,const std::vector<Slash::UI::IWidgetClass*>& aWidgetClasses
)
:Lib::ItemMLFactory()
,fUI(aUI)
,fPrinter(aPrinter)
,fWidgetClasses(aWidgetClasses)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnX::WidgetFactory::~WidgetFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IWidgetClass* OnX::WidgetFactory::findWidgetClass(
 const std::string& aName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = fWidgetClasses.size();
  for (unsigned int index=0;index<number;index++) {
    if (fWidgetClasses[index]->name()==aName) {
      return fWidgetClasses[index];
    } 
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* OnX::WidgetFactory::create(
 const std::string& aTagName
,const std::vector<Lib::ItemML::Attribute>& aAtts
,Lib::ItemML* aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool save = true;
 {unsigned int linen = aAtts.size();
  for(unsigned int index=0;index<linen;index++) {
    if(aAtts[index].first=="save") {
      if(!Lib::smanip::tobool(aAtts[index].second,save)) save = true;
      break;
    }
  }}

  if(aTagName=="widget") {

    std::string sclass;

   {unsigned int linen = aAtts.size();
    for(unsigned int index=0;index<linen;index++) {
      if(aAtts[index].first=="class") {
        sclass = aAtts[index].second;
        break;
      }
    }}

    if(sclass=="InputDialog") {
      Lib::ItemML* itemML = new Lib::ItemML(aTagName,*this,aParent);
      itemML->setAttributes(aAtts);
      itemML->setSaveFlag(save);
      return itemML;
    }

    Slash::UI::IWidgetClass* widgetClass = findWidgetClass(sclass);  
    if(widgetClass) {
      if( (widgetClass->name()=="PageViewer") ||
          (widgetClass->name()=="PlanePageViewer") ||
          (widgetClass->name()=="SoExaminerViewer") ||
          (widgetClass->name()=="SoPlaneViewer") ||
          (widgetClass->name()=="SoWalkViewer") ||
          (widgetClass->name()=="SoFlyViewer") ) {
        Lib::ItemML* itemML = new SoViewer(fUI,*this,aParent,*widgetClass);
        itemML->setAttributes(aAtts);
        itemML->setSaveFlag(save);
        return itemML;
	/*
      } else if(widgetClass->name()=="ScrolledTree") {
        Lib::ItemML* itemML = new OpenGLArea(*this,aParent,*widgetClass);
        itemML->setAttributes(aAtts);
        itemML->setSaveFlag(save);
        return itemML;
      } else if(widgetClass->name()=="OpenGLArea") {
        Lib::ItemML* itemML = new OpenGLArea(*this,aParent,*widgetClass);
        itemML->setAttributes(aAtts);
        itemML->setSaveFlag(save);
        return itemML;
        */
      } else {
        Lib::ItemML* itemML = new Widget(fUI,*this,aParent,*widgetClass);
        itemML->setAttributes(aAtts);
        itemML->setSaveFlag(save);
        return itemML;
      }
    } else {
      Lib::Out out(fPrinter);
      out << "OnX::WidgetFactory::create :"
          << " Slash::UI::IWidgetClass not found for " << Lib::sout(sclass) << "."
          << Lib::endl;
      return 0;
    }
  } else { //OnX, reference, comment are not widgets.
    //Lib::Out out(fPrinter);
    //out << "OnX::WidgetFactory::create :"
    //  << " WARNING : Slash::UI::IWidgetClass not found for " << Lib::sout(aClass) << "."
    //    << Lib::endl;
    Lib::ItemML* itemML = new Lib::ItemML(aTagName,*this,aParent);
    itemML->setAttributes(aAtts);
    itemML->setSaveFlag(save);
    return itemML;
  }
}



