// this :
#include <OnX/Gtk/GtkUI.h>

#include <Slash/Core/ISession.h>
#include <Slash/UI/IWidgetClass.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/sout.h>
#include <Lib/fmanip.h>
#include <Lib/System.h>
#include <Lib/Messages.h>
#include <Lib/Debug.h>
#include <Lib/BaseML.h>
#include <Lib/Element.h>
#include <Lib/ItemML.h>
#include <Lib/Cast.h>

#include <OnX/Gtk/gtkcommandline.h>
#include <OnX/Gtk/gtkwarningdialog.h>
#include <OnX/Gtk/GtkCyclic.h>
#include <OnX/Gtk/GtkTools.h>

//#ifndef __APPLE__
#define ONX_HAS_INVENTOR
#define ONX_HAS_GTK_GL_AREA
//#endif

#ifdef ONX_HAS_INVENTOR
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/Gtk/SoGtk.h>
#include <Inventor/Gtk/viewers/SoGtkExaminerViewer.h>
#include <Inventor/Gtk/viewers/SoGtkPlaneViewer.h>
#include <Inventor/Gtk/viewers/SoGtkFlyViewer.h>
#include <Inventor/Gtk/viewers/SoGtkWalkViewer.h>
#include <OnX/Gtk/gtkpageviewer.h>
#include <OnX/Inventor/Inventor.h>
#include <OnX/Inventor/HEPVis.h>
#endif

#ifdef ONX_HAS_GTK_GL_AREA
#include <gtkgl/gtkglarea.h>
#endif

#ifdef __sun
#include <time.h> // For SunOS-CC-5.1.
#endif

#include <gtk/gtk.h>

#define gtk_WINDOW           1
#define gtk_SCROLLED_TEXT    2
#define gtk_SCROLLED_LIST    3
#define gtk_SCROLLED_TREE    4
#define gtk_DIALOG           5
#define gtk_TOOLBAR_ITEM     6
#define gtk_MENU_SEPARATOR   7
#define gtk_PASSWORD         8
#define gtk_MENUBAR          9
#define gtk_TOOLBAR         10
#define gtk_VCONTAINER      11
#define gtk_HCONTAINER      12

#define gtk_SOEXAMINERVIEWER 13
#define gtk_SOPLANEVIEWER    14
#define gtk_SOFLYVIEWER      15
#define gtk_SOWALKVIEWER     16

#define gtk_PAGEVIEWER       17
#define gtk_PLANEPAGEVIEWER  18

static GtkWidget* findInHierarchy(GtkWidget*,const std::string&);

#ifdef HAS_GTK1
#else

struct _GtkNotebookPage  // copy from gtknotebook.c by l.garnier
{
  GtkWidget* child;
  GtkWidget* tab_label;
  GtkWidget* menu_label;
  GtkWidget* last_focus_child;  // Last descendant of the page that had focus
    
  guint default_menu : 1;   // If true, we create the menu label ourself
  guint default_tab  : 1;   // If true, we create the tab label ourself
  guint expand       : 1;
  guint fill         : 1;
  guint pack         : 1;
    
  GtkRequisition requisition;
  GtkAllocation allocation;
    
  guint mnemonic_activate_signal;
};
#endif

static int sMessageID = 0;

#define NotFound (-1)

//////////////////////////////////////////////////////////////////////////////
OnX::GtkUI::GtkUI(
 Slash::Core::ISession& aSession
,const std::string& aFile
,const std::vector<std::string>& aArgs
)
:BaseUI(aSession)
,fTopWindow(0)
,fConsoleWindow(0)
,fConsoleText(0)
#ifdef HAS_GTK1
,fWarningDialog(0)
#endif
,fFixedFont(0)
,fExitCode(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int argc;
  char** argv;
  Lib::smanip::newArgcv(aArgs,argc,argv);
  gtk_init(&argc, &argv);
  Lib::smanip::delArgcv(argc,argv); //Free now ?

  std::string sgtkrc;
  if(!Lib::System::getenv("ONXGTKRC",sgtkrc)) 
    sgtkrc = "$ONXROOT/scripts/gtk/OnX.gtkrc";
  std::string name = Lib::fmanip::name(sgtkrc);
  if(name!="") {
    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "OnX::GtkUI::GtkUI :"
          << " load resource file \"" << name << "\"." 
          << Lib::endl;
    }
    gtk_rc_parse(name.c_str());
  }

  //  It is the UI that takes care of the visualisation viewers.
  // It needs some widget to be initialised.
  GtkWidget* topWidget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
#ifdef ONX_HAS_INVENTOR
  SoGtk::init(topWidget);
  HEPVis::initClasses();
  OnX::Inventor::initialize(aSession);
#endif

  // Realize top in order to have a window to create pixmaps.
  gtk_widget_realize(topWidget);  
  fTopWindow = topWidget->window;

  fWidgets.push_back(topWidget);

  // Returned value could be NULL.
  fFixedFont = gdk_font_load("*courier*-r-*--14-*");

  if(!startup(aFile)) return;

  fConsoleWindow = (GtkWidget*)findNativeWidget("consoleWindow");
  fConsoleText = (GtkWidget*)findNativeWidget("consoleText");
#ifdef HAS_GTK1
  fWarningDialog = (GtkWidget*)findNativeWidget("warningDialog");
#endif
}
//////////////////////////////////////////////////////////////////////////////
OnX::GtkUI::~GtkUI(
) 
//////////////////////////////////////////////////////////////////////////////
// Should do a gtk_widget_destroy(top widgets) ; not sure
// that embedded viewers are ready for that !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fBeingDestroyed = true;

  // Treat messages emitted between the quiting of the steering
  // and here. They may come from the destruction of various
  // managers in the ~Session.
  synchronize();

  // FIXME : Darwin : malloc error at exit in Gtk2 in the below
  //  gtk_widget_destroy. It comes probably from a bad memory management 
  //  in the SoGtkExaminerViewer.

  std::list<GtkWidget*>::reverse_iterator it;
  for(it=fWidgets.rbegin();it!=fWidgets.rend();++it) {
    if(fVerboseLevel) {
      std::string name = gtk_widget_get_name(*it);
      Lib::Out out(fSession.printer());
      out << "OnX::GtkUI::~GtkUI :"
          << " gtk_widget_destroy \"" << name << "\"..."<< Lib::endl;
    }
    gtk_widget_destroy(*it);
  }
  fWidgets.clear();
  fConsoleWindow = 0;
  fConsoleText = 0;

  if(sMessageID) gtk_timeout_remove(sMessageID);

  // Should destroy the main viewer.
#ifdef ONX_HAS_INVENTOR
  OnX::Inventor::finalize(fSession);
#endif
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::GtkUI::cast( 
 const std::string& aClassName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aClassName=="OnX::GtkUI") {
    return Lib_SCast(GtkUI);
  } else {
    return BaseUI::cast(aClassName);
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::put(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Append line at end of report region.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fConsoleText) return;
#ifdef HAS_GTK1
  gtk_text_freeze (GTK_TEXT (fConsoleText));
  gtk_text_insert (GTK_TEXT (fConsoleText),
                   fFixedFont,0,0, 
                   aString.c_str(), -1);
  // Crashes on Windows :
  gtk_text_set_point(GTK_TEXT(fConsoleText),
                     gtk_text_get_length(GTK_TEXT(fConsoleText)));
  gtk_adjustment_set_value(GTK_TEXT(fConsoleText)->vadj,
                           GTK_TEXT(fConsoleText)->vadj->upper);
  gtk_text_thaw(GTK_TEXT(fConsoleText));
#else
  GtkTextBuffer* buffer = 
    gtk_text_view_get_buffer(GTK_TEXT_VIEW(fConsoleText));
  if(!buffer) return;

  GtkTextIter iter;  
  gtk_text_buffer_get_end_iter(buffer,&iter);
  gtk_text_buffer_insert(buffer,&iter,aString.c_str(),aString.length());

  GtkTextIter iter_start;
  GtkTextIter iter_end;
  gtk_text_buffer_get_bounds(buffer, &iter_start, &iter_end);
  GtkTextMark* mark = 
    gtk_text_buffer_create_mark(buffer, NULL, &iter_end, FALSE);
  if(!mark) return;
  gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW (fConsoleText), mark);
  gtk_text_buffer_delete_mark(buffer, mark);
#endif
  // If not done, map the console window :
  if(fConsoleWindow) gtk_widget_show_all(fConsoleWindow);
}
#ifdef HAS_GTK1
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::warningCallback(
 GtkWidget*
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnX::GtkUI* This = (OnX::GtkUI*)aTag;
  GtkWarningDialog* warningDialog = GTK_WARNING_DIALOG(This->fWarningDialog);
  gtk_widget_hide(GTK_WIDGET(warningDialog));
  //gtk_window_set_modal(GTK_WINDOW(warningDialog),FALSE);
  gtk_main_quit();
}
#endif
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::warn(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Modal method.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_GTK1
  if(!fWarningDialog) {
    echo("WARNING : "+aString);
    return;
  } 
  gtk_warning_dialog_set_message(GTK_WARNING_DIALOG(fWarningDialog),
                                 (char*)aString.c_str());
  gtk_signal_connect(GTK_OBJECT(GTK_WARNING_DIALOG(fWarningDialog)->ok), 
                     "clicked",
                     GTK_SIGNAL_FUNC(warningCallback), 
                     (gpointer)this);
  //FIXME gtk_window_set_modal(GTK_WINDOW(fWarningDialog),TRUE);
  gtk_widget_show_all(fWarningDialog);  
  steer(); //FIXME : use set_modal.
#else
  GtkWidget* warningDialog = 
    gtk_message_dialog_new (GTK_WINDOW(GTK_WINDOW_TOPLEVEL),
      GTK_DIALOG_MODAL,
      GTK_MESSAGE_WARNING,
      GTK_BUTTONS_OK,
      aString.c_str());
  gtk_dialog_run(GTK_DIALOG(warningDialog));
  gtk_widget_destroy(warningDialog);
#endif
}
//////////////////////////////////////////////////////////////////////////////
int OnX::GtkUI::nativeSteer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return 0;

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::GtkUI::steer :"
        << " jumping in gtk_main..."
        << Lib::endl;
  }

  fExitCode = 0;

  fSteerLevel++;
  gtk_main ();
  fSteerLevel--;

  return fExitCode;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::synchronize(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  gdk_flush();
  while(g_main_iteration(FALSE)) {}

  //FIXNE : insted have the below ? 
  //  while(gtk_events_pending()) gtk_main_iteration();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::exit(
 int aExitCode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fExitCode = aExitCode;
  gtk_main_quit();
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::lock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::unlock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}


namespace OnX {
class GtkMessage {
public:
  GtkMessage(GtkUI& aUI,Slash::Core::IMessage* aMess):fUI(aUI),fMessage(aMess){
    Lib::Debug::increment("OnX::GtkMessage");
  }
  ~GtkMessage() {
    Lib::Debug::decrement("OnX::GtkMessage");
  }
  GtkUI& fUI;
  Slash::Core::IMessage* fMessage;
};
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::notify(
 Slash::Core::IMessage* aMessage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //idmess = gtk_timeout_add(33,(GtkFunction)timer_message,0);
  sMessageID = 
    gtk_idle_add((GtkFunction)timer_message,new GtkMessage(*this,aMessage));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
gint OnX::GtkUI::timer_message(
 gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  sMessageID = 0;
  OnX::GtkMessage* guiMessage = (OnX::GtkMessage*)aTag;
  OnX::GtkUI& This = guiMessage->fUI;
  Slash::Core::IMessage* mess = (Slash::Core::IMessage*)guiMessage->fMessage;
  delete guiMessage;
  This.notification(mess);
  return FALSE;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::show(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
// Show the shell widget.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="" && fTopItemML) {
    showAll(*fTopItemML);
    return;
  }
  GtkWidget* widget = (GtkWidget*)findNativeWidget(aName);
  if(!widget) return;
  GtkWidget* shell = gtk_widget_get_toplevel(widget);
  if(!shell) return;
  gtk_widget_show_all(shell);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::hide(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
// Hide the shell widget.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* widget = (GtkWidget*)findNativeWidget(aName);
  if(!widget) return;
  GtkWidget* shell = gtk_widget_get_toplevel(widget);
  if(!shell) return;
  gtk_widget_hide(shell);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::refresh(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* widget = (GtkWidget*)findNativeWidget(aName);
  if(!widget) return false;
#ifdef ONX_HAS_GTK_GL_AREA
  if(GTK_IS_GL_AREA(widget)) {
    // Send an expose event :
    GdkEventExpose event;
    event.type = GDK_EXPOSE;
    event.window = widget->window;
    event.count = 0;
    event.area.x = 0;
    event.area.y = 0;
    event.area.width = widget->allocation.width;
    event.area.height = widget->allocation.height;
    gtk_widget_event(widget,(GdkEvent*)&event);
  }
#endif
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::position(
 const std::string& aName
,int& aX
,int& aY
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aX = 0; 
  aY = 0;
  GtkWidget* widget = (GtkWidget*)findNativeWidget(aName);
  if(!widget) return false;
  aX = widget->allocation.x;
  aY = widget->allocation.y;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::deleteChildren(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* gtkWidget = (GtkWidget*)findNativeWidget(aName);
  if(!gtkWidget) return;
  if(!GTK_IS_CONTAINER(gtkWidget)) return;
  gtk_container_foreach(GTK_CONTAINER(gtkWidget),
                        (GtkCallback)gtk_widget_destroy,0);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::parameterValue(
 const std::string& aWhat
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;


  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::GtkUI::parameterValue :"
        << " widget \"" << aWidget << "\""
        << " not found." << Lib::endl;
    return false;
  }
  GtkWidget* gtkWidget = (GtkWidget*)element->getData1();
  if(gtkWidget) {
    if(aResource=="selection") {
      if(GTK_IS_NOTEBOOK(gtkWidget)) {
#ifdef HAS_GTK1
        GtkNotebookPage* page = GTK_NOTEBOOK(gtkWidget)->cur_page;
#else
        GtkNotebook* notebook = GTK_NOTEBOOK(gtkWidget);
        GList* children        = notebook->children;
        GtkNotebookPage* page = (GtkNotebookPage*)children->data;
#endif
        if(page) {
          gchar* s = GTK_LABEL(page->tab_label)->label;
          aValue = s?s:"";
        }
        return true;
      } else if(GTK_IS_LIST(gtkWidget)) {
        GList* dlist = GTK_LIST(gtkWidget)->selection;
        while(dlist) {
          GtkWidget* item = GTK_WIDGET(dlist->data);
          GtkLabel* label = GTK_LABEL(GTK_BIN(item)->child);
          gchar* s = 0;
          gtk_label_get(label,&s);
          aValue = s?s:"";
          dlist=dlist->next;
          if(dlist) aValue = "\n";
        }
        return true;
#ifdef HAS_GTK1
      } else if(GTK_IS_TREE(gtkWidget)) {
        aValue = getTreeSelection(gtkWidget);
        return true;
#else
      } else if(GTK_IS_TREE_VIEW(gtkWidget)) {
        aValue = getTreeSelection(gtkWidget);
        return true;
#endif
      }
    } else if(aResource=="label") {
      if(GTK_IS_LABEL(gtkWidget)) {
        gchar* s = 0;
        gtk_label_get(GTK_LABEL(gtkWidget),&s);
        aValue = s?s:"";
        return true;
      } else if(GTK_IS_MENU_ITEM(gtkWidget)) {
        GtkWidget* child = GTK_BIN(gtkWidget)->child;
        if(child) {
          gchar* s = 0;
          gtk_label_get(GTK_LABEL(child),&s);
          aValue = s?s:"";
          return true;
        }
      } else if (GTK_IS_OPTION_MENU(gtkWidget)) {
        GList* hBoxList = 
          gtk_container_get_children
            (GTK_CONTAINER(gtk_widget_get_parent(gtkWidget))); //REVISIT
        int number = g_list_length(hBoxList);
        for(int index=0;index<number;index++) {
          if(GTK_IS_LABEL(g_list_nth_data(hBoxList,index))) {
            gchar* s = 0;
            gtk_label_get(GTK_LABEL(g_list_nth_data(hBoxList,index)),&s);
            aValue = s?s:"";
            return true;
          }
        }
        g_list_free(hBoxList);
      } else if(GTK_IS_BUTTON(gtkWidget)) {
        GtkWidget *label = GTK_BIN(GTK_BUTTON(gtkWidget))->child;
        if(GTK_IS_LABEL(label)) {
          gchar* s = 0;
          gtk_label_get(GTK_LABEL(label),&s);
          aValue = s?s:"";
          return true;
        } else if(GTK_IS_TOOLBAR(gtkWidget->parent)){
          //this is probably a ToolBarItem
          GtkToolbar *toolbar = GTK_TOOLBAR(gtkWidget->parent);
          for (GList* children = toolbar->children; 
            children; children = children->next) {
                 GtkToolbarChild *child = (GtkToolbarChild*)(children->data);
            if (child->widget == gtkWidget)
              if(GTK_IS_LABEL(child->label)) {
                gchar* s = 0;
                gtk_label_get(GTK_LABEL(child->label),&s);
                aValue = s?s:"";
                return true;
              }
          }
        }
      }
    } else if(aResource=="tooltip") {
      if(GTK_IS_TOOLBAR(gtkWidget->parent)){
        GtkToolbar *toolbar = GTK_TOOLBAR(gtkWidget->parent);
        if (GTK_IS_TOOLTIPS(toolbar->tooltips)){
          // find the widget child
          bool found = false;
          for (GList* children = toolbar->children; 
               children; children = children->next) {
            GtkToolbarChild *child = (GtkToolbarChild*)(children->data);
            if (child->widget == gtkWidget) {
              GtkTooltipsData* d = gtk_tooltips_data_get(child->widget);
              if (d) {
                aValue = d->tip_text?d->tip_text:"";
                  found = true;
                break;
              }
            }
          }
          if (found) return true;
        }
      }
    } else if(aResource=="itemCount") {
      if(GTK_IS_NOTEBOOK(gtkWidget)) {
        int value  = g_list_length(GTK_NOTEBOOK(gtkWidget)->children);
        Lib::smanip::printf(aValue,64,"%d",value);
        return true;
      }
    } else if(aResource=="title") {
      if(GTK_IS_WINDOW(gtkWidget)) {
        const gchar * s = gtk_window_get_title(GTK_WINDOW(gtkWidget));
        aValue = s?s:"";
        return true;
      } else if(GTK_IS_WINDOW(gtk_widget_get_toplevel(gtkWidget))) {
        const gchar * s = 
          gtk_window_get_title(GTK_WINDOW(gtk_widget_get_toplevel(gtkWidget)));
        aValue = s?s:"";
        return true;
      }
    } else if(aResource=="value") {
      if(GTK_IS_ENTRY(gtkWidget)) {
        const gchar* s = gtk_entry_get_text(GTK_ENTRY(gtkWidget));
        aValue = s?s:"";
        return true;
      } else if(GTK_IS_COMBO(gtkWidget)) {
        const gchar* s = 
          gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(gtkWidget)->entry));
        aValue = s?s:"";
        return true;
      } else if(GTK_IS_OPTION_MENU(gtkWidget)) {
        if(gtk_option_menu_get_value(GTK_OPTION_MENU(gtkWidget),aValue))
          return true;
      } else if(GTK_IS_RANGE(gtkWidget)) {
        float value = gtk_range_get_adjustment(GTK_RANGE(gtkWidget))->value;
        Lib::smanip::printf(aValue,64,"%d",(int)value);
        return true;
#ifdef HAS_GTK1
      } else if (GTK_IS_TEXT(gtkWidget)) {
        gchar* s = gtk_editable_get_chars(&(GTK_TEXT(gtkWidget)->editable),0,-1);
        aValue = s?s:"";
        g_free(s);
        return true;
      }
#else
      } else if (GTK_IS_TEXT_VIEW(gtkWidget)) {
        GtkTextIter start;
        GtkTextIter end;
        GtkTextBuffer* text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gtkWidget));
        gtk_text_buffer_get_start_iter(text_buffer,&start);
        gtk_text_buffer_get_end_iter(text_buffer,&end);
        gchar* s = gtk_text_buffer_get_text(text_buffer,&start, &end,TRUE);

        aValue = s?s:"";
        g_free(s);
        return true;
      }
#endif
    } else if(aResource=="set") {
      if(GTK_IS_CHECK_MENU_ITEM(gtkWidget)) {
        aValue = GTK_CHECK_MENU_ITEM(gtkWidget)->active ? "true" : "false";
        return true;
      } else if(GTK_IS_TOGGLE_BUTTON(gtkWidget)) {
        aValue = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtkWidget)) ?
          "true" : "false";
        return true;
      }
    }
  }
  // Not found, then look in XML elements :
  if(aResource=="class") {
    return element->attributeValue("",aResource,aValue);
  }
  return element->propertyValue(aResource,aValue);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::setParameter(
 const std::string& aWhat
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(!fTopItemML) return false;

  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::GtkUI::setParameter :"
        << " widget \"" << aWidget << "\""
        << " resource \"" << aResource << "\""
        << " value \"" << aValue << "\"."
        << Lib::endl;
  }

  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::GtkUI::setParameter :"
        << " widget \"" << aWidget << "\""
        << " not found." << Lib::endl;
    return false;
  }

  GtkWidget* gtkWidget = (GtkWidget*)element->getData1();
  if(!gtkWidget) return false;

  std::string svalue;

  if(aResource=="background") {
    if(GTK_IS_MENU_ITEM(gtkWidget)) {
      if(gtk_menu_item_set_background(GTK_MENU_ITEM(gtkWidget),aValue))
        return true;
    } else if (GTK_IS_LABEL(gtkWidget)) {
      if(gtk_widget_set_background(gtk_widget_get_parent(gtkWidget),aValue))
        return true;
    } else if (GTK_IS_BUTTON(gtkWidget)) {
      if(gtk_button_set_background(GTK_BUTTON(gtkWidget),aValue))
        return true;
#ifdef HAS_GTK1
    } else if (GTK_IS_TREE(gtkWidget)) {
#else
    } else if (GTK_IS_TREE_VIEW(gtkWidget)) {
#endif
      if(gtk_widget_set_background2(gtkWidget,aValue))
        return true;
    } else {
      if(gtk_widget_set_background(gtkWidget,aValue))
        return true;
    }
  } else if(aResource=="foreground") {
    if(GTK_IS_LABEL(gtkWidget)) {
      if(gtk_widget_set_foreground(gtk_widget_get_parent(gtkWidget),aValue))
        return true;
    } else if(GTK_IS_MENU_ITEM(gtkWidget)) {
      if(gtk_menu_item_set_foreground(GTK_MENU_ITEM(gtkWidget),aValue))
        return true;
    } else if(GTK_IS_BUTTON(gtkWidget)) {
      if(gtk_button_set_foreground(GTK_BUTTON(gtkWidget),aValue))
        return true;
    } else {
      if(gtk_widget_set_foreground(gtkWidget,aValue))
        return true;
    }
  } else if(aResource=="label") {
    if(GTK_IS_LABEL(gtkWidget)) {
      gtk_label_set_text(GTK_LABEL(gtkWidget),aValue.c_str());
      return true;
    } else if(GTK_IS_MENU_ITEM(gtkWidget)) {
      GtkWidget* child = GTK_BIN(gtkWidget)->child;
      if(child) {
        gtk_label_set_text(GTK_LABEL(child),aValue.c_str());
        return true;
      }
    } else if(GTK_IS_BUTTON(gtkWidget)) {
      if (GTK_IS_OPTION_MENU(gtkWidget)) {
        GList* hBoxList = 
          gtk_container_get_children
            (GTK_CONTAINER(gtk_widget_get_parent(gtkWidget)));
        bool found = false;
        int number = g_list_length(hBoxList);
        for(int index=0;index<number;index++) {
          if(GTK_IS_LABEL(g_list_nth_data(hBoxList,index))) {
            gtk_label_set_text
              (GTK_LABEL(g_list_nth_data(hBoxList,index)),aValue.c_str());
            found = true;
          }
        }
        g_list_free(hBoxList);
        if (found) return true;
      }
      else {
        GtkWidget *label = GTK_BIN (GTK_BUTTON(gtkWidget))->child;
        if(GTK_IS_LABEL(label)) {
          gtk_label_set_text(GTK_LABEL(label),aValue.c_str());
          return true;
        } else if(GTK_IS_TOOLBAR(gtkWidget->parent)){
          //this is probably a ToolBarItem
          // FIXME : we can't set the label or icon after the 
          // creation of a new toolbaritem (in UI builder for ex)
          // if no one has been specified at widget creation.

          GtkToolbar *toolbar = GTK_TOOLBAR(gtkWidget->parent);
          bool found = false;
          for (GList* children = toolbar->children; 
               children; children = children->next) {
            GtkToolbarChild *child = (GtkToolbarChild*)(children->data);
            if (child->widget == gtkWidget)
              if(GTK_IS_LABEL(child->label)) {
                 gtk_label_set_text(GTK_LABEL(child->label),aValue.c_str());
                 found = true;
                 break;
              }
          }
          if (found) return true;
        }
      }
    }
  } else if(aResource=="tooltip") {
    if(GTK_IS_TOOLBAR(gtkWidget->parent)){
      GtkToolbar *toolbar = GTK_TOOLBAR(gtkWidget->parent);
      if (GTK_IS_TOOLTIPS(toolbar->tooltips)){
        // find the widget child
        bool found = false;
        for (GList* children = toolbar->children; 
             children; children = children->next) {
          GtkToolbarChild *child = (GtkToolbarChild*)(children->data);
          if (child->widget == gtkWidget) {
            gtk_tooltips_set_tip (toolbar->tooltips,child->widget,aValue.c_str(), NULL);
            found = true;
            break;
          }
        }
        if (found) return true;
      }
    }
  } else if(aResource=="pixmap") {
    if(GTK_IS_TOOLBAR(gtkWidget->parent)){
      // FIXME : we can't set the pixmap after the 
      // creation of a new toolbaritem (in UI builder for ex)
      // if no one has been specified at widget creation.

      GtkToolbar* toolbar = GTK_TOOLBAR(gtkWidget->parent);
      // find the widget child
      bool found = false;
      for (GList * children = toolbar->children; 
           children; children = children->next) {
        GtkToolbarChild* child = (GtkToolbarChild*)(children->data);
        if (child->widget == gtkWidget) {
          if(child->icon && GTK_IS_PIXMAP(child->icon)) {
            gtk_pixmap_set_from_file
              (GTK_PIXMAP(child->icon),fTopWindow,aValue);
            found = true;
            break;
          }
        }
      }
      if (found) return true;
    }
  } else if(aResource=="dirMask") {
    if(GTK_IS_FILE_SELECTION(gtkWidget)) {
      svalue = aValue;
      Lib::smanip::getenv(svalue);
      //FIXME : how to do a selection ?
      return true;
    }
  } else if(aResource=="directory") {
    if(GTK_IS_FILE_SELECTION(gtkWidget)) {
      std::string directory = aValue;
      if(directory!="") {
        directory += Lib::System::fileSeparator();
      }
      Lib::smanip::getenv(directory);
      gtk_file_selection_set_filename(GTK_FILE_SELECTION(gtkWidget),
                                      directory.c_str());
      return true;
    }
  } else if(aResource=="selection") {
    if(GTK_IS_LIST(gtkWidget)) {
      gtk_list_unselect_all(GTK_LIST(gtkWidget));
      GList* dlist = GTK_LIST(gtkWidget)->children;
      while(dlist) {
        GtkWidget* item = GTK_WIDGET(dlist->data);
        GtkLabel* label = GTK_LABEL(GTK_BIN(item)->child);
        gchar* s = 0;
        gtk_label_get(label,&s);
        if(s && (aValue==std::string(s))) {
          gtk_list_select_child(GTK_LIST(gtkWidget),item);
          return true;
        }
        dlist=dlist->next;
      }
#ifdef HAS_GTK1
    } else if(GTK_IS_TREE(gtkWidget)) {
      setTreeSelection(gtkWidget,aValue);
      return true;
#else
    } else if(GTK_IS_TREE_VIEW(gtkWidget)) {
      setTreeSelection(gtkWidget,aValue);
      return true;
#endif
    }
  } else if(aResource=="set") {
    bool value;
    if(Lib::smanip::tobool(aValue,value)) {
      if(GTK_IS_CHECK_MENU_ITEM(gtkWidget)) {
        gtk_check_menu_item_set_active(
          GTK_CHECK_MENU_ITEM(gtkWidget),value ? TRUE : FALSE);
        return true;
      } else if(GTK_IS_TOGGLE_BUTTON(gtkWidget)) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkWidget),
          value ? TRUE : FALSE);
        return true;
      }
    }
  } else if(aResource=="splitPosition") {
    if(GTK_IS_PANED(gtkWidget)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        gtk_paned_set_position(GTK_PANED(gtkWidget),value);
        return true;
      }     
    } 
  } else if(aResource=="expand") {
    if ((GTK_IS_VBOX(gtkWidget)) || 
        (GTK_IS_HBOX(gtkWidget))) {
      return gtk_box_set_expand(GTK_BOX(gtkWidget),aValue);
    }
  } else if(aResource=="value") {
    if(GTK_IS_ENTRY(gtkWidget)) {
      gtk_entry_set_text(GTK_ENTRY(gtkWidget),aValue.c_str());
      return true;
#ifdef HAS_GTK1
    } else if(GTK_IS_TEXT(gtkWidget)) {
      gtk_text_freeze(GTK_TEXT(gtkWidget));
      gtk_text_set_point(GTK_TEXT(gtkWidget),0);
      gtk_text_forward_delete(GTK_TEXT(gtkWidget),
                              gtk_text_get_length(GTK_TEXT(gtkWidget)));
      gtk_text_insert(GTK_TEXT(gtkWidget),
                      fFixedFont,0,0, 
                      aValue.c_str(),-1);
      gtk_text_thaw(GTK_TEXT(gtkWidget));
      return true;
#else
    } else if(GTK_IS_TEXT_VIEW(gtkWidget)) {
       gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(gtkWidget)),aValue.c_str(),-1);
       return true;
#endif
    } else if(GTK_IS_PROGRESS_BAR(gtkWidget)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        gfloat percent = GTK_PROGRESS_BAR(gtkWidget)->blocks;
        percent = ((gfloat)value)/percent;
        if(percent<0) percent = 0;
        else if(percent>1) percent = 1;
        gtk_progress_bar_update(GTK_PROGRESS_BAR(gtkWidget),percent);
        return true;
      }
    } else if(GTK_IS_RANGE(gtkWidget)) {
      double value;
      if(Lib::smanip::todouble(aValue,value)) {
        gtk_adjustment_set_value
          (gtk_range_get_adjustment(GTK_RANGE(gtkWidget)),(float)value);
        return true;
      }
    }
  } else if(aResource=="maximum") {
    if(GTK_IS_SCALE(gtkWidget)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        GtkAdjustment* adjustment = 
          gtk_range_get_adjustment(GTK_RANGE(gtkWidget));
        float xvalue = adjustment->value;
        float slider = adjustment->page_size;
        adjustment->upper = value+slider;
        gtk_adjustment_set_value(adjustment,(int)xvalue);
        return true;
      }
    }
  } else if(aResource=="minimum") {
    if(GTK_IS_SCALE(gtkWidget)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        GtkAdjustment* adjustment = 
          gtk_range_get_adjustment(GTK_RANGE(gtkWidget));
        float xvalue = adjustment->value;
        adjustment->lower = value;
        gtk_adjustment_set_value(adjustment,(int)xvalue);
        return true;
      }
    }
  } else if(aResource=="default") {
    if(GTK_IS_OPTION_MENU(gtkWidget)) {
      gtk_option_menu_set_default(GTK_OPTION_MENU(gtkWidget),aValue); 
      return true;
    } else if(GTK_IS_COMBO(gtkWidget)) {
      GtkEntry* entry = GTK_ENTRY(GTK_COMBO(gtkWidget)->entry);
      gtk_entry_set_text(entry,(char*)(aValue.c_str()));
      return true;
    }
  } else if(aResource=="items") {
    if(GTK_IS_LIST(gtkWidget)) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      gtk_list_clear_items(GTK_LIST(gtkWidget),0,-1);
      unsigned int itemn = items.size();
      for(unsigned int index=0;index<itemn;index++){
        GtkWidget *item = 
          gtk_list_item_new_with_label((char*)items[index].c_str());
          gtk_widget_show(item);
        gtk_container_add(GTK_CONTAINER(gtkWidget),item);
      }
      return true;
    } else if(GTK_IS_COMBO(gtkWidget)) {
      GtkEntry* entry = GTK_ENTRY(GTK_COMBO(gtkWidget)->entry);
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      std::string sdef;
      element->propertyValue("default",sdef);
      GList* list = 0;
      bool found = false;
      for(unsigned int index=0;index<items.size();index++) {
        list = g_list_append(list,(char*)items[index].c_str());
        if(items[index]==sdef) found = true;
      }
      gtk_combo_set_popdown_strings(GTK_COMBO(gtkWidget),list);
      g_list_free(list);
      if(found) {
        gtk_entry_set_text(entry,(char*)(sdef.c_str()));
      }
      return true;
    } else if(GTK_IS_OPTION_MENU(gtkWidget)) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      std::string def;
      element->propertyValue("default",def);
#ifdef HAS_GTK1
      GList* children = 
        gtk_option_menu_get_menu_items(GTK_OPTION_MENU(gtkWidget));
      while (children) {
        removeActions(gtkWidget);
        children = children->next;
      }
#endif
      gtk_option_menu_initialize(GTK_OPTION_MENU(gtkWidget),items,def);
#ifdef HAS_GTK1
      // Have to set callbacks on each menu items !
      children = gtk_option_menu_get_menu_items(GTK_OPTION_MENU(gtkWidget));
      while(children) {
        GtkWidget* child = (GtkWidget*)children->data;
        if (GTK_IS_MENU_ITEM(child)) {
          addCallbacks(*element,
                       gtkWidget,"valueChanged",
                       GTK_OBJECT(child),"activate");
        }
        children = children->next;
      }
#endif
      return true;
#ifdef HAS_GTK1
    } else if(GTK_IS_TREE(gtkWidget)) {

      // retrieve the old tree
      std::string oldXMLTree;
      oldXMLTree = gtk_tree_get_XML(GTK_TREE(gtkWidget),1);

      // retrieve the old selection
      std::string selection = getTreeSelection(gtkWidget);

      // clear the old tree
      GList *list = GTK_TREE(gtkWidget)->children;
      while (list) {
        gtk_tree_remove_item(GTK_TREE(gtkWidget),GTK_WIDGET(list->data));
        list = GTK_TREE(gtkWidget)->children;
        if (!list) break;
        list = list->next;
      }

      // reset all tree root to good values
      // if we don't do this, we get a ***GTK CRITICAL*** when
      //  we do setParameter(selection)
      GTK_TREE(gtkWidget)->children = NULL;
      GTK_TREE(gtkWidget)->root_tree = GTK_TREE(gtkWidget);
      GTK_TREE(gtkWidget)->selection = NULL;
      GTK_TREE(gtkWidget)->tree_owner = NULL;
      GTK_TREE(gtkWidget)->selection_mode = GTK_SELECTION_SINGLE;
      GTK_TREE(gtkWidget)->indent_value = 9;
      GTK_TREE(gtkWidget)->current_indent = 0;
      GTK_TREE(gtkWidget)->level = 0;
      GTK_TREE(gtkWidget)->view_mode = GTK_TREE_VIEW_LINE;
      GTK_TREE(gtkWidget)->view_line = 1;

      // load the new tree
      Lib::ItemMLFactory factory;
      Lib::BaseML treeML(factory,fSession.verboseLevel());
      std::vector<std::string> tags;
      tags.push_back("tree");
      tags.push_back("treeItem");
      treeML.setTags(tags);
      if(treeML.loadString(aValue,fSession.printer())) {
        Lib::ItemML* top = treeML.topItem();
        if(top) {
          if (oldXMLTree !="") {
            oldXMLTree = "<tree>"+oldXMLTree+"</tree>";
            Lib::BaseML oldTreeML(factory,fSession.verboseLevel());
            std::vector<std::string> tags2;
            tags2.push_back("tree");
            tags2.push_back("treeItem");
            oldTreeML.setTags(tags2);
            if(oldTreeML.loadString(oldXMLTree,fSession.printer())) {
              Lib::ItemML* old = oldTreeML.topItem();
              if (old) {
                top->updateTree(*old);
              }
            }
          }
          createTree(gtkWidget,top);
        }
      }
      // disable the scripts of the callback to avoid recursive loop 
      // if someone has define the same callback in the script :
      std::vector<std::string> scripts = 
        disableCallbackScript(gtkWidget,"select");
      // restore selection
      setTreeSelection(gtkWidget,selection);
      // enable callback scripts :
      enableCallbackScript(gtkWidget,"select",scripts);

      return true;
#else
    } else if(GTK_IS_TREE_VIEW(gtkWidget)) {
      // retrieve the old selection
      std::string selection = getTreeSelection(gtkWidget);

      gtk_container_foreach(GTK_CONTAINER(gtkWidget),
                            (GtkCallback)gtk_widget_destroy,0);
      Lib::ItemMLFactory factory;
      Lib::BaseML treeML(factory,fSession.verboseLevel());
      std::vector<std::string> tags;
      tags.push_back("tree");
      tags.push_back("treeItem");
      treeML.setTags(tags);
      if(treeML.loadString(aValue,fSession.printer())) {
        Lib::ItemML* top = treeML.topItem();
        //top->dump();
      if(top) createTree(gtkWidget,top);
      }

      // disable the scripts of the callback to avoid recursive loop 
      // if someone has define the same callback in the script :
      std::vector<std::string> scripts = 
        disableCallbackScript(gtkWidget,"select");
      // restore selection
      setTreeSelection(gtkWidget,selection);
      // enable callback scripts :
      enableCallbackScript(gtkWidget,"select",scripts);

      return true;
#endif
    }
#ifdef ONX_HAS_INVENTOR
  } else if(aResource=="popupItems") {
    if(GTK_IS_PAGE_VIEWER(gtkWidget)) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      for(unsigned int index=0;index<items.size();index++)
        gtk_page_viewer_add_popup_entry(GTK_PAGE_VIEWER(gtkWidget),
                                        items[index]);
      return true;
    }
  } else if(aResource=="title") {
    if(GTK_IS_WINDOW(gtkWidget)) {
      gtk_window_set_title(GTK_WINDOW(gtkWidget),aValue.c_str());
      return true;
    } else if(GTK_IS_WINDOW(gtk_widget_get_toplevel(gtkWidget))) {
        gtk_window_set_title(GTK_WINDOW(gtk_widget_get_toplevel(gtkWidget)),aValue.c_str());
      return true;
    }

#endif
  } else if(aResource=="mode") {
    return true;
  } else if(aResource=="tabLabel") { //Constraint resource
    element->setPropertyValue("tabLabel",aValue);
    return true;
#ifdef ONX_HAS_INVENTOR
  } else { // Viewer parameter ?
    if(GTK_IS_PAGE_VIEWER(gtkWidget)) {
      if(gtk_page_viewer_set_parameter(GTK_PAGE_VIEWER(gtkWidget),
                                       aResource,aValue)) return true;
    }
#endif
  }

  Lib::Out out(fSession.printer());
  out << "OnX::GtkUI::setParameter :"
      << " unable to set parameter for : widget \"" << aWidget << "\""
      << " widget \"" << aWidget << "\""
      << " resource \"" << aResource << "\""
      << " value \"" << aValue << "\"."
      << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::write(
 const std::string& aWidget
,const std::string& aWhat
,const std::string& aFile
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef ONX_HAS_INVENTOR
  GtkWidget* widget = (GtkWidget*)findNativeWidget(aWidget);
  if(!widget) return false;
  if(!GTK_IS_PAGE_VIEWER(widget)) return false;
  return gtk_page_viewer_write(GTK_PAGE_VIEWER(widget),aWhat,aFile,aOptions);
#else
  std::string nowarnings = aWidget + aWhat + aFile + aOptions;
  return false;
#endif
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#ifdef HAS_GTK1
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::createTree(
 GtkWidget* aTree
,Lib::ItemML* aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::list<Lib::ItemML*>& list = aItemML->children();
  if(!list.size()) return true;

  if(!aTree) {
    void* data;
    void* tree_item;
    int dataInt;
    aItemML->getData(data,tree_item,dataInt);
    GtkWidget* gtkWidget = gtk_tree_new();
    gtk_tree_set_selection_mode(GTK_TREE(gtkWidget),GTK_SELECTION_SINGLE);
    gtk_tree_set_view_mode(GTK_TREE(gtkWidget),GTK_TREE_VIEW_ITEM);
    gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item),gtkWidget);
    aItemML->setData(gtkWidget,0,0);
  } else {
    aItemML->setData(aTree,0,0);
  }

  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string slabel;
    std::string sopened;
    (*it)->propertyValue("label",slabel);
    void* data;
    void* ptree;
    int dataInt;
    (*it)->parent()->getData(ptree,data,dataInt);
    void* tree;
    (*it)->getData(tree,data,dataInt);
    GtkWidget* gtkWidget = gtk_tree_item_new_with_label((char*)slabel.c_str());
    gtk_widget_show(gtkWidget);
    gtk_tree_append(GTK_TREE(ptree),gtkWidget);
    (*it)->setData(tree,gtkWidget,0);
    bool opened = false;
    if((*it)->propertyValue("opened",sopened)) {
      Lib::smanip::tobool(sopened,opened);
    }
    if(!createTree(0,*it)) return false;

    if (opened) {
       gtk_tree_item_expand((GtkTreeItem*)gtkWidget);
    }
  }
  return true;
}
#else
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::createTree(
 GtkWidget* aView
,Lib::ItemML* aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // clear all the old tree if it is present
  if(! GTK_IS_TREE_VIEW(aView)) return false;
  GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(aView)); 
  if (model) {

    // view old model content
    Lib::ItemML* oldTreeItemML = 0;
    GtkTreeIter top;
    gtk_tree_model_get_iter_first(model,&top);
    std::string line;
    line = "<tree>"+
      gtk_tree_model_get_XML(model,GTK_TREE_VIEW(aView),&top)+"</tree>";

    Lib::ItemMLFactory factory;
    Lib::BaseML treeML(factory,fSession.verboseLevel());
    std::vector<std::string> tags;
    tags.push_back("tree");
    tags.push_back("treeItem");
    treeML.setTags(tags);
    if(treeML.loadString(line,fSession.printer())) {
      oldTreeItemML = treeML.topItem();
    }
    if (!oldTreeItemML) return false;
    aItemML->updateTree(*oldTreeItemML);
    //FIXME test for GUI Tree
    /*
      printf("GUITree \n%s",getGUITree().c_str());
      if(treeML.loadString(getGUITree(),fSession.printer())) {
      aItemML = treeML.topItem();
      if (!aItemML) return false;
      }
    */
    // end FIXME
    GtkTreeStore* treestore = gtk_tree_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(GTK_TREE_VIEW(aView), GTK_TREE_MODEL(treestore));
    fillModel(aView,treestore,NULL,aItemML);
    
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(aView)), GTK_SELECTION_SINGLE); 
  } else {
    GtkTreeViewColumn* col = gtk_tree_view_column_new();  
    
    gtk_tree_view_append_column(GTK_TREE_VIEW(aView), col);
    GtkCellRenderer* renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_add_attribute(col,renderer,"text",GTK_TREE_COL_TEXT);
    
    //model
    GtkTreeStore* treestore = gtk_tree_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(GTK_TREE_VIEW(aView), GTK_TREE_MODEL(treestore));
    fillModel(aView,treestore,NULL,aItemML);
    //end model
    
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(aView)), GTK_SELECTION_SINGLE); 
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::fillModel(
 GtkWidget* aWidget
,GtkTreeStore* aTreestore
,GtkTreeIter* aParent
,Lib::ItemML* aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkTreeIter child;
  const std::list<Lib::ItemML*>& list = aItemML->children();
  if(!list.size()) return true;
  
  aItemML->setData(aWidget,0,0);

  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string slabel;
    (*it)->propertyValue("label",slabel);
    void* data;
    void* ptree;
    int dataInt;
    (*it)->parent()->getData(ptree,data,dataInt);
    void* tree;
    (*it)->getData(tree,data,dataInt);
    gtk_tree_store_append(aTreestore, &child, aParent);
    gtk_tree_store_set(aTreestore, &child,
                       GTK_TREE_COL_TEXT,slabel.c_str(),-1);
    (*it)->setData(tree,aWidget,0);
    std::string sopened;

    bool opened = false;
    if((*it)->propertyValue("opened",sopened)) {
      Lib::smanip::tobool(sopened,opened);
    }

    bool fill = fillModel(aWidget,aTreestore,&child,*it);

    if (opened) {
      GtkTreePath* path = gtk_tree_model_get_path(GTK_TREE_MODEL(aTreestore),&child);
      gtk_tree_view_expand_to_path(GTK_TREE_VIEW(aWidget),path); 
      gtk_tree_path_free(path);
    }
    if (!fill)  return false;
  }

  return true;
}
#endif
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::createNativeWidget(
 Lib::ItemML& aItemML                      
,const std::string& aClass
,const std::string& aName
,Lib::ItemML* aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
    int compoundId = getCompound(aClass);
    GtkType gtkType = getType(aClass);
    if( (compoundId==0) && (gtkType==GTK_TYPE_INVALID) ) {
      Lib::Out out(fSession.printer());
      out << "GtkUI::createNativeWidget :"
          << " unknown widget type : " << aClass << Lib::endl;
      return false;
    }
    
    // Get parent :
    GtkWidget* gtkParent = 0;
    GtkWidget* gtkParentContainer = 0;
    int parentCompoundId = 0;
    GtkType gtkParentType = 0;
    if(aParent) {
      gtkParent = (GtkWidget*)aParent->getData1();
      gtkParentContainer = (GtkWidget*)aParent->getData2();
      std::string sparentclass; 
      aParent->attributeValue("","class",sparentclass);
      parentCompoundId = getCompound(sparentclass);
      if(gtkParent) gtkParentType = GTK_WIDGET_TYPE(gtkParent);

      // Get the parent number of child
      if (( (gtkParentType==gtk_vpaned_get_type()) ||
            (gtkParentType==gtk_hpaned_get_type()) ||
            (gtkParentType==gtk_vbox_get_type()) ||
            (gtkParentType==gtk_hbox_get_type()) ) &&
          (g_list_length(gtk_container_get_children(GTK_CONTAINER(gtkParent))) >1 ) &&
          (parentCompoundId !=gtk_VCONTAINER) &&
          (parentCompoundId !=gtk_HCONTAINER)) {
        Lib::Out out(fSession.printer());
        out << "GtkUI::createNativeWidget :"
            << " could not create a 3rd widget in  : " 
            << sparentclass <<Lib::endl;
        return false;
      }
    }   
    
    // List of gtk1.2 widgets without window (gadgets) :
    // GtkAlignment, GtkArrow, GtkBin, GtkBox, GtkImage, GtkItem
    // GtkLabel, GtkPixmap, GtkScrolledWindow, GtkSeparator, GtkTable
    // GtkAspectFrame, GtkFrame, GtkVBox, GtkHBox, GtkVSeparator, GtkHSeparator


    // List of gtk2 widgets without window (gadgets) :
    // GtkAlignment, GtkArrow, GtkBin, GtkBox, GtkImage
    // GtkLabel,            GtkScrolledWindow, GtkSeparator, GtkTable
    // GtkAspectFrame, GtkFrame, GtkVBox, GtkHBox, GtkVSeparator, GtkHSeparator
    // GtkButton, GtkCheckButton, GtkFixed, GtkMenuItem, GtkNotebook,
    // GtkPaned, GtkRadioButton, GtkRange, GtkToolbar,

    //  These are put in an GtkEventBox in order to get a window
    // to set background color.

    // Create the gtk widget :
    std::string svalue;
    std::string sdef;
    GtkWidget* gtkWidget = 0;
    GtkWidget* gtkContainer = 0;
    if(gtkType==gtk_check_menu_item_get_type()) {
      aItemML.propertyValue("label",svalue);
      gtkWidget = gtk_check_menu_item_new_with_label(svalue.c_str());
    } else if(gtkType==gtk_vbox_get_type()) {
      gtkWidget = gtk_vbox_new (FALSE, 0);
    } else if(gtkType==gtk_label_get_type()) {
      // GtkLabel has no window, we parent it with a windowed container.
      gtkContainer = gtk_event_box_new();
      if(!aItemML.propertyValue("label",svalue))
        aItemML.propertyValue("labelString",svalue);
      gtkWidget = gtk_label_new(svalue.c_str());
      gtk_container_add(GTK_CONTAINER(gtkContainer),gtkWidget);
    } else if(gtkType==gtk_button_get_type()) {
      aItemML.propertyValue("label",svalue);
      gtkWidget = gtk_button_new_with_label(svalue.c_str());
    } else if(gtkType==gtk_toggle_button_get_type()) {
      aItemML.propertyValue("label",svalue);
      gtkWidget = gtk_toggle_button_new_with_label(svalue.c_str());
    } else if(gtkType==gtk_menu_item_get_type()) {
      aItemML.propertyValue("label",svalue);
      gtkWidget = gtk_menu_item_new_with_label(svalue.c_str());
      gtk_widget_show(gtkWidget); //For Viewers pdm.
  /*} else if(gtkType==gtk_tree_item_get_type()) {
      aItemML.propertyValue("label",svalue);
      gtkWidget = gtk_tree_item_new_with_label((char*)svalue.c_str());
      gtk_widget_show(gtkWidget); // Seems needed. */

    } else if(gtkType==gtk_option_menu_get_type()) {
      gtkContainer = gtk_hbox_new(FALSE,10);
      aItemML.propertyValue("label",svalue);
      GtkWidget* label = gtk_label_new((char*)svalue.c_str());
      gtk_container_add(GTK_CONTAINER(gtkContainer),label);
      gtkWidget = gtk_option_menu_new();
      gtk_container_add(GTK_CONTAINER(gtkContainer),gtkWidget);
      GtkWidget* menu = gtk_menu_new();
      gtk_option_menu_set_menu(GTK_OPTION_MENU(gtkWidget),menu);
    } else if(gtkType==gtk_hscrollbar_get_type()) {
      int slider = 10;
      GtkObject* adjustment = gtk_adjustment_new(0,0,1000+slider,1,1,slider);
      gtkWidget = gtk_hscrollbar_new(GTK_ADJUSTMENT(adjustment));
    } else if(gtkType==gtk_vscrollbar_get_type()) {
      int slider = 10;
      GtkObject* adjustment = gtk_adjustment_new(0,0,1000+slider,1,1,slider);
      gtkWidget = gtk_vscrollbar_new(GTK_ADJUSTMENT(adjustment));

    } else if((gtkType==gtk_hscale_get_type()) ||
              (gtkType==gtk_vscale_get_type()) ) {
      int slider = 10;
      int current = 0;
      int mn = 0;
      if(aItemML.propertyValue("minimum",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          mn = value;
        }
      }
      int mx = 1000+slider;
      if(aItemML.propertyValue("maximum",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
#ifdef HAS_GTK1
          mx = value+slider;
#else
          mx = value;
#endif
        }
      }
      if(aItemML.propertyValue("value",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          current = value;
        }
      }
      GtkObject* adjustment = gtk_adjustment_new(current,mn,mx,1,1,slider);
      if(gtkType==gtk_hscale_get_type()) {
        gtkWidget = gtk_hscale_new(GTK_ADJUSTMENT(adjustment));
      } else {
        gtkWidget = gtk_vscale_new(GTK_ADJUSTMENT(adjustment));
      }

    } else if(gtkType==gtk_progress_bar_get_type()) {
      gtkWidget = gtk_progress_bar_new();
      gtk_progress_bar_set_discrete_blocks(GTK_PROGRESS_BAR(gtkWidget),100);
      gtk_progress_bar_update(GTK_PROGRESS_BAR(gtkWidget),0);
#ifdef ONX_HAS_GTK_GL_AREA
    } else if(gtkType==gtk_gl_area_get_type()) {

      int glAttributes[16];
      int i = 0;
      glAttributes[i] = GDK_GL_RGBA; i++;
      glAttributes[i] = GDK_GL_DOUBLEBUFFER; i++;
      glAttributes[i] = GDK_GL_DEPTH_SIZE; i++;
      glAttributes[i] = 1; i++;
      glAttributes[i] = GDK_GL_STENCIL_SIZE; i++;
      glAttributes[i] = 1; i++;
      glAttributes[i] = GDK_GL_NONE; i++;
      gtkWidget = gtk_gl_area_new(glAttributes);    
#endif
    } else if(gtkType==gtk_warning_dialog_get_type()) {
      gtkWidget = gtk_warning_dialog_new();
      gtk_window_set_position(GTK_WINDOW(gtkWidget),GTK_WIN_POS_CENTER);
      gtk_window_set_default_size(GTK_WINDOW(gtkWidget),300,100);

    } else if(compoundId==gtk_WINDOW) {
      // Emulate a "main window" with internal menubar and toolbar.
      gtkContainer = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      fWidgets.push_back(gtkContainer);
      GtkWidget* box = gtk_vbox_new(FALSE, 0);
      gtk_container_add(GTK_CONTAINER(gtkContainer),box);
      GtkWidget* menuBar = gtk_menu_bar_new();
      gtk_widget_set_name(menuBar,(char*)"OnX_gtk_menuBar");
#ifdef HAS_GTK1
      GtkWidget* toolBar = 
        gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_ICONS);
#else
      GtkWidget* toolBar = gtk_toolbar_new();
      gtk_toolbar_set_orientation
        (GTK_TOOLBAR(toolBar),GTK_ORIENTATION_HORIZONTAL);
      gtk_toolbar_set_style(GTK_TOOLBAR(toolBar),GTK_TOOLBAR_ICONS);
#endif
      gtk_widget_set_name(toolBar,(char*)"OnX_gtk_toolBar");
      gtk_box_pack_start(GTK_BOX(box),menuBar,FALSE,TRUE,0);
      gtk_box_pack_start(GTK_BOX(box),toolBar,FALSE,TRUE,0);
      gtkWidget = gtk_vbox_new(FALSE, 0);
      gtk_box_pack_start(GTK_BOX(box),gtkWidget,TRUE,TRUE,0);
    } else if(compoundId==gtk_MENUBAR) {
      gtkWidget = findInHierarchy(gtkParentContainer,"OnX_gtk_menuBar");
      if(!gtkWidget) {
        Lib::Out out(fSession.printer());
        out << "GtkUI::createNativeWidget :"
            << " MenuBar not found in parent."
            << Lib::endl;
        return false;
      }
    } else if(compoundId==gtk_TOOLBAR) {
      gtkWidget = findInHierarchy(gtkParentContainer,"OnX_gtk_toolBar");
      if(!gtkWidget) {
        Lib::Out out(fSession.printer());
        out << "GtkUI::createNativeWidget :"
            << " ToolBar not found in parent."
            << Lib::endl;
        return false;
      }
    } else if(compoundId==gtk_MENU_SEPARATOR) {
      gtkContainer = gtk_menu_item_new();
      gtkWidget = gtk_hseparator_new();
      gtk_container_add(GTK_CONTAINER(gtkContainer),gtkWidget);
      gtk_widget_show(gtkWidget);
    } else if(compoundId==gtk_DIALOG) {
#ifdef HAS_GTK1
      gtkWidget = gtk_window_new(GTK_WINDOW_DIALOG);
#else
      gtkWidget = gtk_dialog_new();
      gtkContainer = (GTK_DIALOG(gtkWidget))->vbox;
      //      gtk_window_set_modal (GTK_WINDOW(gtkWidget), FALSE);
#endif
      // args are allow_shrink,allow_grow,auto_shrink
      //gtk_window_set_policy(GTK_WINDOW(gtkWidget),FALSE,FALSE,FALSE);
      gtk_window_set_policy(GTK_WINDOW(gtkWidget),FALSE,TRUE,FALSE);
      //gtk_widget_set_usize(gtkWidget,100,100);
      gtk_window_set_position(GTK_WINDOW(gtkWidget),GTK_WIN_POS_CENTER);
      fWidgets.push_back(gtkWidget);
    } else if(compoundId==gtk_PASSWORD) {
      gtkWidget = gtk_entry_new();
      gtk_entry_set_visibility(GTK_ENTRY(gtkWidget),FALSE);
    } else if(compoundId==gtk_SCROLLED_TEXT) {
      
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER (gtkContainer), 10);
      
      GtkWidget* table = gtk_table_new (2, 2, FALSE);
      gtk_table_set_row_spacing (GTK_TABLE (table), 0, 2);
      gtk_table_set_col_spacing (GTK_TABLE (table), 0, 2);
      gtk_box_pack_start (GTK_BOX (gtkContainer), table, TRUE, TRUE, 0);
      
      // Create the GtkText widget :
#ifdef HAS_GTK1
      gtkWidget = gtk_text_new (0,0);
      gtk_text_set_editable (GTK_TEXT(gtkWidget), TRUE);
#else
      gtkWidget = gtk_text_view_new();
      gtk_widget_set_scroll_adjustments(gtkWidget,0,0);
      gtk_text_view_set_editable (GTK_TEXT_VIEW(gtkWidget), TRUE);
#endif
      gtk_table_attach (GTK_TABLE (table),gtkWidget, 
                        0, 1, 0, 1,
                        (GtkAttachOptions)
                        (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
                        (GtkAttachOptions)
                        (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 
                        0, 0);
      // Add a vertical scrollbar to the GtkText widget :
#ifdef HAS_GTK1
      GtkWidget* vscrollbar = gtk_vscrollbar_new (GTK_TEXT (gtkWidget)->vadj);
#else
      GtkWidget* vscrollbar = gtk_vscrollbar_new (GTK_TEXT_VIEW (gtkWidget)->vadjustment);
#endif
      gtk_table_attach (GTK_TABLE (table), vscrollbar, 
                        1, 2, 0, 1,
                         (GtkAttachOptions)GTK_FILL,
                        (GtkAttachOptions)
                        (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 
                        0, 0);
      //gtk_text_set_editable(GTK_TEXT(fConsoleText),FALSE);
    } else if(compoundId==gtk_SCROLLED_LIST) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      GtkWidget* scrolled_window = gtk_scrolled_window_new(0,0);
      gtk_container_add(GTK_CONTAINER(gtkContainer),scrolled_window);
      gtkWidget = gtk_list_new();
      gtk_scrolled_window_add_with_viewport(
                                       GTK_SCROLLED_WINDOW(scrolled_window),
                                       gtkWidget);
    } else if(compoundId==gtk_SCROLLED_TREE) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      GtkWidget* scrolled_window = gtk_scrolled_window_new(0,0);
#ifdef HAS_GTK1
      gtk_container_add(GTK_CONTAINER(gtkContainer),scrolled_window);
      gtkWidget = gtk_tree_new();
      gtk_tree_set_selection_mode(GTK_TREE(gtkWidget),
                                  GTK_SELECTION_SINGLE);
      gtk_scrolled_window_add_with_viewport(
                                       GTK_SCROLLED_WINDOW(scrolled_window),
                                       gtkWidget);
#else
      gtkWidget = gtk_tree_view_new();
      gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(gtkWidget)),GTK_SELECTION_SINGLE);
      gtk_container_add(GTK_CONTAINER(scrolled_window), gtkWidget);

      gtk_container_add(GTK_CONTAINER(gtkContainer),scrolled_window);

      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
                                     GTK_POLICY_AUTOMATIC,
                                     GTK_POLICY_AUTOMATIC);
#endif
    } else if(compoundId==gtk_VCONTAINER) {
      gtkWidget = gtk_vbox_new(FALSE,0);
    } else if(compoundId==gtk_HCONTAINER) {
      gtkWidget = gtk_hbox_new(FALSE,0);
    } else if(compoundId==gtk_TOOLBAR_ITEM) {

      // Assume that parent is a gtk_toolbar...

      aItemML.propertyValue("label",svalue);
      std::string shelp;
      aItemML.propertyValue("tooltip",shelp);
      std::string sfile;
      aItemML.propertyValue("pixmap",sfile);
      std::string fileName = Lib::fmanip::name(sfile);

      GtkWidget* icon = 0;
      if((fileName!="") && fTopWindow) {
        GtkPixmap* pixmap = (GtkPixmap*)gtk_type_new(gtk_pixmap_get_type());
        gtk_pixmap_set_build_insensitive(pixmap,TRUE);
        if(!gtk_pixmap_set_from_file(pixmap,fTopWindow,sfile)) {
          gtk_widget_destroy(GTK_WIDGET(pixmap));
        } else {
          icon = GTK_WIDGET(pixmap);
        }        
      }

      gtkWidget = gtk_toolbar_append_item(GTK_TOOLBAR(gtkParent),
                                          svalue.c_str(),
                                          shelp.c_str(),
                                          0,icon,0,0);
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"clicked");
#ifdef ONX_HAS_INVENTOR
    } else if(compoundId==gtk_SOEXAMINERVIEWER) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      std::string name = aName + "_viewer";
      SoGtkExaminerViewer* viewer = 
        new SoGtkExaminerViewer(gtkContainer,name.c_str(),TRUE);
      gtkWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
    } else if(compoundId==gtk_SOPLANEVIEWER) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      std::string name = aName + "_viewer";
      SoGtkPlaneViewer* viewer = 
        new SoGtkPlaneViewer(gtkContainer,name.c_str(),TRUE);
      gtkWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
    } else if(compoundId==gtk_SOFLYVIEWER) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      std::string name = aName + "_viewer";
      SoGtkFlyViewer* viewer = 
        new SoGtkFlyViewer(gtkContainer,name.c_str(),TRUE);
      gtkWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
    } else if(compoundId==gtk_SOWALKVIEWER) {
      gtkContainer = gtk_vbox_new (FALSE, 10);
      gtk_container_set_border_width (GTK_CONTAINER(gtkContainer), 10);
      std::string name = aName + "_viewer";
      SoGtkWalkViewer* viewer = 
        new SoGtkWalkViewer(gtkContainer,name.c_str(),TRUE);
      gtkWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);

    } else if( (compoundId==gtk_PAGEVIEWER) ||
               (compoundId==gtk_PLANEPAGEVIEWER) ){
      std::string vclass = "Examiner";
      if(compoundId==gtk_PLANEPAGEVIEWER) vclass = "Plane";
      gtkWidget = gtk_page_viewer_new(vclass.c_str());
     {Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      setCurrentWidget(widget);}
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) {
        Lib::Out out(fSession.printer());
        out << "GtkUI::createNativeWidget : ItemML not a Slash::UI::ISoViewer."
            << Lib::endl;
        return false;
      }
      SoNode* soNode = 
        gtk_page_viewer_get_scene_graph(GTK_PAGE_VIEWER(gtkWidget));
      soViewer->setSceneGraph(soNode);
#endif
    } else {
      // Generic case :
      gtkWidget = (GtkWidget*)gtk_type_new(gtkType);
      if(GTK_IS_WINDOW(gtkWidget)) fWidgets.push_back(gtkWidget);
    }

    if(!gtkWidget) return false;
    if(!gtkContainer) gtkContainer = gtkWidget;
    aItemML.setData(gtkWidget,gtkContainer,0);

    // Set the widget name and customize according 
    // to resources in the rc files. A specific widget
    // can be customized in a rc file by some :
    //    widget "*<widdget name>" style "<specific style>".
    // See scripts/gtk/OnX.gtkrc for examples.
    gtk_widget_set_name(gtkWidget,aName.c_str());
    
    // Put in parent :
#ifdef HAS_GTK1
    if( (compoundId==gtk_DIALOG)
        || (gtkType==gtk_file_selection_get_type())
        || (gtkType==gtk_color_selection_dialog_get_type())
        || (gtkType==gtk_warning_dialog_get_type())
#else
    if((gtkType==gtk_file_selection_get_type())
       || (gtkType==gtk_color_selection_dialog_get_type()) 
       || (gtkType==gtk_warning_dialog_get_type())
#endif
        ) {
      if(gtkParent) {
        GtkWidget* shell = gtk_widget_get_toplevel(gtkParent);
        if(shell) gtk_window_set_transient_for(GTK_WINDOW(gtkContainer),
                                               GTK_WINDOW(shell));
      }
    } else if(compoundId==gtk_WINDOW) {
    } else if(compoundId==gtk_DIALOG) {
    } else if(compoundId==gtk_MENUBAR) {
    } else if(compoundId==gtk_TOOLBAR) {
    } else if(parentCompoundId==gtk_VCONTAINER) {
      gtk_container_add(GTK_CONTAINER(gtkParent),gtkContainer);
    } else if(parentCompoundId==gtk_HCONTAINER) {
      gtk_container_add(GTK_CONTAINER(gtkParent),gtkContainer);
    } else if(parentCompoundId==gtk_WINDOW) {
      gtk_box_pack_start (GTK_BOX (gtkParent), gtkContainer, TRUE, TRUE, 0);
#ifdef HAS_GTK1
#else
    // if the gtkDIALOG is the parent, we have to pack in
    } else if(parentCompoundId==gtk_DIALOG) {
      gtk_box_pack_start (GTK_BOX (GTK_DIALOG(gtkParent)->vbox), gtkContainer, TRUE, TRUE, 0);
#endif
    } else if(gtkParent) {
      GtkType gtkParentType = GTK_WIDGET_TYPE(gtkParent);
      if( (gtkParentType==gtk_vbox_get_type()) ||
          (gtkParentType==gtk_hbox_get_type()) ) {
        std::string sexpand;
        if(!aParent->propertyValue("expand",sexpand)) sexpand = "second";
        gtk_box_set_expand(GTK_BOX(gtkParent),sexpand,gtkContainer);
      } else if(gtkParentType==gtk_menu_bar_get_type()) {
        // Cascade button in a menu bar :
        gtk_menu_bar_append (GTK_MENU_BAR(gtkParent),gtkContainer);
      } else if(gtkParentType==gtk_menu_item_get_type()) {
        // Pulldown menu attached to a cascade button :
        gtk_menu_item_set_submenu (GTK_MENU_ITEM(gtkParent),gtkContainer);
      } else if(gtkParentType==gtk_menu_get_type()) {
        // Button in a menu or MenuSeparator :
        gtk_menu_append (GTK_MENU(gtkParent),gtkContainer);
      } else if(gtkParentType==gtk_notebook_get_type()) {
        if(!aItemML.propertyValue("tabLabel",svalue)) svalue = aName;
        gtk_notebook_append_page(GTK_NOTEBOOK(gtkParent),
                                 gtkContainer,
                                 gtk_label_new(svalue.c_str()));
        gtk_widget_show_all(gtkContainer); //FIXME : for PageViewers.
      } else if(compoundId==gtk_TOOLBAR_ITEM) {
        // Done at creation.
      } else {
        // Generic case :
        gtk_container_add(GTK_CONTAINER(gtkParent),gtkContainer);
      }
    }
    
    // Set resources :
    if(compoundId==gtk_WINDOW) {
      if(aItemML.propertyValue("title",svalue)) {
        gtk_window_set_title(GTK_WINDOW(gtkContainer),svalue.c_str());
      }
      if(aItemML.propertyValue("geometry",svalue)) {
        int x,y,w,h;
        if(!Lib::smanip::togeometry(svalue,x,y,w,h)) {
          Lib::Out out(fSession.printer());
          out << "GtkUI::createNativeWidget :"
              << " " << svalue << " : bad value for geometry." << Lib::endl;
        } else {
          gtk_window_set_default_size(GTK_WINDOW(gtkContainer),w,h);
#ifdef HAS_GTK1
          // Is there a simplest way to position a window ?
          // We can't call gdk_move_window here since the 
          // widget is not yet realized.
          if(x || y) {          
            gtk_signal_connect(GTK_OBJECT(gtkContainer), 
                               "realize",
                               GTK_SIGNAL_FUNC(realizeCallback),
                               &aItemML);
          }
#else
          gtk_window_move(GTK_WINDOW(gtkContainer),x,y);
#endif
        }
      }
#ifdef HAS_GTK1
      gtk_window_set_policy(GTK_WINDOW(gtkContainer),TRUE,TRUE,FALSE);
#else
      gtk_window_set_resizable(GTK_WINDOW(gtkContainer),TRUE);
#endif
      
    } else if(compoundId==gtk_DIALOG) {
      if(aItemML.propertyValue("title",svalue)) {
        gtk_window_set_title(GTK_WINDOW(gtkWidget),svalue.c_str());
      }
      if(aItemML.propertyValue("geometry",svalue)) {
        int w,h;
        if(sscanf(svalue.c_str(),"%dx%d",&w,&h)!=2) {
          Lib::Out out(fSession.printer());
          out << "GtkUI::createNativeWidget :"
              << " " << svalue << " : bad value for geometry." << Lib::endl;
        } else {
          gtk_window_set_default_size(GTK_WINDOW(gtkWidget),w,h);
        }
      }

    } else if(compoundId==gtk_SCROLLED_LIST) {
      aItemML.propertyValue("selection",sdef);
      if(aItemML.propertyValue("items",svalue)) {
        gtk_list_clear_items(GTK_LIST(gtkWidget),0,-1);
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        unsigned int itemn = items.size();
        int found = NotFound;
        for(unsigned int index=0;index<itemn;index++){
          GtkWidget *item = 
            gtk_list_item_new_with_label((char*)items[index].c_str());
          gtk_container_add(GTK_CONTAINER(gtkWidget),item);
          if(items[index]==sdef) found = index;
        }
        if(found!=NotFound) gtk_list_select_item(GTK_LIST(gtkWidget),found);
      }
      addCallbacks(aItemML,
                   gtkWidget,"select",
                   GTK_OBJECT(gtkWidget),"selection_changed");
    } else if(compoundId==gtk_PASSWORD) {
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"activate");
    } else if(compoundId==gtk_SCROLLED_TREE) {
#ifdef HAS_GTK1
      addCallbacks(aItemML,
                   gtkWidget,"select",
                   GTK_OBJECT(gtkWidget),"selection_changed");
#else
      addCallbacks(aItemML,
                   gtkWidget,
                   "select",
                   GTK_OBJECT(gtkWidget),
                   "row-activated",
                   (GtkSignalFunc)treeViewActivatedCallback);
      addCallbacks(aItemML,
                   gtkWidget,
                   "selectBranch",
                   GTK_OBJECT(gtkWidget),
                   "row-activated",
                   (GtkSignalFunc)treeViewActivatedCallback);
      addCallbacks(aItemML,
                   gtkWidget,
                   "close",
                   GTK_OBJECT(gtkWidget),
                   "row_collapsed",
                   (GtkSignalFunc)treeViewOpenCloseCallback);
      addCallbacks(aItemML,
                   gtkWidget,
                   "open",
                   GTK_OBJECT(gtkWidget),
                   "row-expanded",
                   (GtkSignalFunc)treeViewOpenCloseCallback);
#endif
    } else if( (gtkType==gtk_hpaned_get_type()) ||
               (gtkType==gtk_vpaned_get_type()) ) {
      if(aItemML.propertyValue("splitPosition",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          gtk_paned_set_position(GTK_PANED(gtkWidget),value);
        }      
      }
    } else if( (gtkType==gtk_check_menu_item_get_type()) ||
               (gtkType==gtk_menu_item_get_type()) ){
      //FIXME : pass here too for cascade button...      
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"activate");
    } else if(gtkType==gtk_button_get_type()) {
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"clicked");
    } else if(gtkType==gtk_notebook_get_type()) {
      addCallbacks(aItemML,
                   gtkWidget,"select",
                   GTK_OBJECT(gtkWidget),"switch_page",
                   (GtkSignalFunc)notebookCallback);
    } else if(gtkType==gtk_file_selection_get_type()) {

      gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(gtkWidget));

      addCallbacks(aItemML,gtkWidget,"ok",
        GTK_OBJECT(GTK_FILE_SELECTION(gtkWidget)->ok_button),"clicked");
      addCallbacks(aItemML,gtkWidget,"cancel",
        GTK_OBJECT(GTK_FILE_SELECTION(gtkWidget)->cancel_button),"clicked");

      std::string directory;
      if(!aItemML.propertyValue("directory",directory)) directory = "";
      if(directory!="") {
        directory += Lib::System::fileSeparator();
      }

      Lib::smanip::getenv(directory);
      gtk_file_selection_set_filename(GTK_FILE_SELECTION(gtkWidget),
                                      directory.c_str());

      //FIXME : how to do a selection ?
      if(aItemML.propertyValue("dirMask",svalue)) {
        Lib::smanip::getenv(svalue);
      }

    } else if(gtkType==gtk_entry_get_type()) {
      std::string svalue;
      if(aItemML.propertyValue("value",svalue)) {
        gtk_entry_set_text(GTK_ENTRY(gtkWidget),svalue.c_str());
      }
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"activate");
    } else if(gtkType==gtk_command_line_get_type()) {
      addCallbacks(aItemML,
                   gtkWidget,"activate",
                   GTK_OBJECT(gtkWidget),"activate");
      addCallbacks(aItemML,
                   gtkWidget,"complete",
                   GTK_OBJECT(gtkWidget),"complete");
    } else if( (gtkType==gtk_hscrollbar_get_type()) ||
               (gtkType==gtk_vscrollbar_get_type()) ||
               (gtkType==gtk_hscale_get_type()) ||
               (gtkType==gtk_vscale_get_type())  )  {
      GtkAdjustment* adjustment = 
        gtk_range_get_adjustment(GTK_RANGE(gtkWidget));
      addCallbacks(aItemML,
                   gtkWidget,"drag",
                   GTK_OBJECT(adjustment),"value_changed");
#ifdef HAS_GTK1
    } else if(gtkType==gtk_tree_get_type()) {
      gtk_tree_set_selection_mode (GTK_TREE(gtkWidget),
                                   GTK_SELECTION_SINGLE);
#else
    } else if(gtkType==gtk_tree_view_get_type()) {
      gtk_tree_selection_set_mode (gtk_tree_view_get_selection(GTK_TREE_VIEW(gtkWidget)),
                                   GTK_SELECTION_SINGLE);
#endif
    } else if(gtkType==gtk_option_menu_get_type()) {
      aItemML.propertyValue("default",sdef);
      if(aItemML.propertyValue("items",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        gtk_option_menu_initialize(GTK_OPTION_MENU(gtkWidget),items,sdef);
      }
#ifdef HAS_GTK1
      // Have to set callbacks on each menu items !
      GList* children = 
        gtk_option_menu_get_menu_items(GTK_OPTION_MENU(gtkWidget));
      while(children) {
        GtkWidget* child = (GtkWidget*)children->data;
        if (GTK_IS_MENU_ITEM(child)) {
          addCallbacks(aItemML,
                       gtkWidget,"valueChanged",
                       GTK_OBJECT(child),"activate");
        }
        children = children->next;
      }
#else
      addCallbacks(aItemML,
                   gtkWidget,"valueChanged",
                   GTK_OBJECT(gtkWidget),"changed");
#endif
    } else if(gtkType==gtk_combo_get_type()) {
      //REVIST : put all the below in GtkTools ?

      aItemML.propertyValue("default",sdef);
      GtkEntry* entry = GTK_ENTRY(GTK_COMBO(gtkWidget)->entry);
      if(aItemML.propertyValue("items",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        GList* list = 0;
        bool found = false;
        for(unsigned int index=0;index<items.size();index++) {
          list = g_list_append(list,(char*)items[index].c_str());
          if(items[index]==sdef) found = true;
        }
        gtk_combo_set_popdown_strings(GTK_COMBO(gtkWidget),list);
        g_list_free(list);
        if(found) {
          gtk_entry_set_text(entry,(char*)sdef.c_str());
        }
      }

      // Disable activate, else a return in the entry will popup the list !
      gtk_combo_disable_activate(GTK_COMBO(gtkWidget));
      GtkList* list = GTK_LIST(GTK_COMBO(gtkWidget)->list);

      const std::list<Lib::Element*> props = aItemML.properties();
      std::list<Lib::Element*>::const_iterator it;
      
      for(it=props.begin();it!=props.end();++it) {
        if("valueChanged"==(*it)->name()) {
          std::string sexec;
          (*it)->attributeValue("exec",sexec);
          Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
          Action& action = addAction(*this,widget,gtkWidget,sexec,
                                     (*it)->value(),
                                     "valueChanged");
          gtk_signal_connect(GTK_OBJECT(entry),
                             "activate",
                             (GtkSignalFunc)OnX::GtkUI::callback,
                             (gpointer)&action);
          gtk_signal_connect(GTK_OBJECT(list),
                             "button_release_event",
                             (GtkSignalFunc)listCallback,
                             (gpointer)&action); 
        }
      }
    } else if(gtkType==gtk_color_selection_dialog_get_type()) {
      addCallbacks(aItemML,gtkWidget,"ok",
       GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(gtkWidget)->ok_button),"clicked");
    } else if(gtkType==gtk_toolbar_get_type()) {
      gtk_toolbar_set_orientation(GTK_TOOLBAR(gtkWidget),
                                  GTK_ORIENTATION_HORIZONTAL);
      //gtk_toolbar_set_style(GTK_TOOLBAR(gtkWidget),GTK_TOOLBAR_BOTH);
      gtk_toolbar_set_style(GTK_TOOLBAR(gtkWidget),GTK_TOOLBAR_ICONS);
#ifdef ONX_HAS_GTK_GL_AREA
    } else if(gtkType==gtk_gl_area_get_type()) {
      //REVISIT      
      const std::list<Lib::Element*> props = aItemML.properties();
      std::list<Lib::Element*>::const_iterator it;
      for(it=props.begin();it!=props.end();++it) {
        if("event"==(*it)->name()) {
          std::string sexec;
          (*it)->attributeValue("exec",sexec);
          Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
          Action& action = addAction(*this,widget,gtkWidget,sexec,
                                     (*it)->value(),
                                     "event");
          gtk_signal_connect(GTK_OBJECT(gtkWidget),
                             "button_press_event",
                             (GtkSignalFunc)eventCallback,
                             (gpointer)&action); 
          gtk_signal_connect(GTK_OBJECT(gtkWidget),
                             "button_release_event",
                             (GtkSignalFunc)eventCallback,
                             (gpointer)&action); 
          gtk_signal_connect(GTK_OBJECT(gtkWidget),
                             "motion_notify_event",
                             (GtkSignalFunc)eventCallback,
                             (gpointer)&action); 
        }
      }
      addCallbacks(aItemML,
                   gtkWidget,"paint",
                   GTK_OBJECT(gtkWidget),"expose_event",
                   (GtkSignalFunc)exposeCallback);
      gtk_widget_add_events(gtkWidget,
        GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|GDK_BUTTON_MOTION_MASK);
#endif
#ifdef ONX_HAS_INVENTOR
    } else if( (compoundId==gtk_PAGEVIEWER) ||
               (compoundId==gtk_PLANEPAGEVIEWER) ){

      if(aItemML.propertyValue("popupItems",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++)
          gtk_page_viewer_add_popup_entry(GTK_PAGE_VIEWER(gtkWidget),
                                          items[index]);
      }

      addCallbacks(aItemML,
                   gtkWidget,"collect",
                   GTK_OBJECT(gtkWidget),"collect",
                   (GtkSignalFunc)pageCallback);
      addCallbacks(aItemML,
                   gtkWidget,"popup",
                   GTK_OBJECT(gtkWidget),"popup",
                   (GtkSignalFunc)pageCallback);
      addCallbacks(aItemML,gtkWidget,"update",0,"",0);

      gtk_signal_connect(GTK_OBJECT(gtkWidget), 
                         "destroy",
                         GTK_SIGNAL_FUNC(destroyCallback),
                         this);
      
#endif
    } 
        
    if((compoundId==gtk_WINDOW)|| (compoundId==gtk_DIALOG)) {
      
      // default values for decorations are in BaseUI.cxx
      bool close_button = true;
      bool iconify_button = true;
      bool min_max_button = true;
      bool resize_handle = true;
      bool border = true;
      bool caption = true;
      bool no_decorations = false;
      bool system_menu = true;

      aItemML.propertyValue("decoration",svalue);
      std::vector< std::pair<bool,std::string> > decoItems = 
        parseDecorations(svalue);
      std::vector< std::pair<bool,std::string> >::iterator it;
      for(it=decoItems.begin();it!=decoItems.end();++it) {
        if ((*it).second == "close_button") close_button = (*it).first; 
        if ((*it).second == "iconify_button") iconify_button = (*it).first; 
        if ((*it).second == "min_max_button") min_max_button = (*it).first; 
        if ((*it).second == "resize_handle") resize_handle = (*it).first; 
        if ((*it).second == "border") border = (*it).first;
        if ((*it).second == "caption") caption = (*it).first; 
        if ((*it).second == "system_menu") system_menu = (*it).first;
        if ((*it).second == "no_decorations") no_decorations = (*it).first;
      }
      Lib::Out out(fSession.printer());
      std::string msg = "GtkUI::createNativeWidget : WARNING : ";
      
      if (system_menu)
        if (!no_decorations) 
          out << msg <<"This toolkit haven't got any system menu" << Lib::endl;
        // to avoid error on exit from dialogs, if no close button, 
        // we block any delete events
        if(compoundId==gtk_DIALOG) {
          gtk_signal_connect(GTK_OBJECT(gtkWidget), 
                             "delete_event",
                             GTK_SIGNAL_FUNC(avoidDeleteCallback),NULL);
        } else {
          gtk_signal_connect(GTK_OBJECT(gtkContainer), 
                             "delete_event",
                             GTK_SIGNAL_FUNC(avoidDeleteCallback),NULL);
        }
      
#ifdef HAS_GTK1
      if (!no_decorations) {
        if (!close_button)
          out << msg <<"This toolkit can't hide the close button" << Lib::endl;
        if (!iconify_button)
          out << msg  <<"This Toolkit can't hide the iconify button " << Lib::endl;
        if (!min_max_button)
          out << msg  <<"This toolkit can't hide the minimize-maximize button" << Lib::endl;
        if (!resize_handle)
          out << msg  <<"This toolkit can't hide the resize handle" << Lib::endl;
        if (!border)
          out << msg  <<"This toolkit can't hide the window border" << Lib::endl;
        if (!caption)
          out << msg  <<"This toolkit can't hide the window border" << Lib::endl;
      }
#else
      if (!no_decorations) {
        if (!iconify_button) {
          if(compoundId==gtk_DIALOG)
            out << msg <<"DialogWidget: You cannot hide a iconify button" << Lib::endl;
          else
            out << msg  <<"To hide iconify button, you should try no_caption option" << Lib::endl;
        }
        if (!min_max_button) {
          if(compoundId!=gtk_DIALOG)
            out << msg  <<"To hide minimize-maximize button, you should try no_caption option" << Lib::endl;
        }
      }
      if(compoundId==gtk_DIALOG)
        gtk_window_set_resizable (GTK_WINDOW(gtkWidget),resize_handle);
      else 
        gtk_window_set_resizable (GTK_WINDOW(gtkContainer),resize_handle);
      if (!caption) {
        if(compoundId==gtk_DIALOG) {
          if (!no_decorations)
            out <<msg <<"DialogWidget: You cannot hide caption on a dialog widget" << Lib::endl;
        } else gtk_window_set_decorated(GTK_WINDOW(gtkContainer),FALSE);
      } else {
        if (compoundId==gtk_DIALOG) {
          if (!no_decorations) 
            out <<msg <<"DialogWidget: Caption is always visible on a dialog widget" << Lib::endl;
        } else gtk_window_set_decorated(GTK_WINDOW(gtkContainer),TRUE);
      }
      if (!border) {
        if(compoundId==gtk_DIALOG) {
          if (!no_decorations)
            out <<msg <<"DialogWidget: You cannot hide border on a dialog widget" << Lib::endl;
        } else gtk_window_set_decorated(GTK_WINDOW(gtkContainer),FALSE);
      } else {
        if (compoundId==gtk_DIALOG) {
          if (!no_decorations)
            out <<msg <<"DialogWidget: Border is always visible on a dialog widget" << Lib::endl;
        } else gtk_window_set_decorated(GTK_WINDOW(gtkContainer),TRUE);
      }
#endif
    }

    if(aItemML.propertyValue("background",svalue)) {
      if(GTK_IS_MENU_ITEM(gtkWidget)) {
        gtk_menu_item_set_background(GTK_MENU_ITEM(gtkWidget),svalue);
      } else if (GTK_IS_BUTTON(gtkWidget)) {
        gtk_button_set_background(GTK_BUTTON(gtkWidget),svalue);
#ifdef HAS_GTK1
      } else if (GTK_IS_TREE(gtkWidget)) {
#else
      } else if (GTK_IS_TREE_VIEW(gtkWidget)) {
#endif
        gtk_widget_set_background2(gtkWidget,svalue);
      } else {
        gtk_widget_set_background(gtkWidget,svalue);
      }
    }

    if(aItemML.propertyValue("foreground",svalue)) {
      if(GTK_IS_LABEL(gtkWidget)) {
        gtk_widget_set_foreground(gtk_widget_get_parent(gtkWidget),svalue);
      } else if(GTK_IS_MENU_ITEM(gtkWidget)) {
        gtk_menu_item_set_foreground(GTK_MENU_ITEM(gtkWidget),svalue);
      } else if(GTK_IS_BUTTON(gtkWidget)) {
        gtk_button_set_foreground(GTK_BUTTON(gtkWidget),svalue);
      } else {
        gtk_widget_set_foreground(gtkWidget,svalue);
      }
    }

    bool value;
    if(aItemML.propertyValueBoolean("grabFocus",value)) {
      gtk_widget_grab_focus(gtkWidget);
    }  

    return createChildren(aItemML);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::destroyWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* widget = (GtkWidget*)aItemML.getData2();
  if(!widget) return false;
  if(fVerboseLevel) {
    std::string name = gtk_widget_get_name(widget);
    Lib::Out out(fSession.printer());
    out << "GtkUI::destroyWidget :"
        << " destroy Gtk widget \""
        << name << "\"." << Lib::endl;
  }
  removeActions(widget);
  if(GTK_IS_WINDOW(widget)) fWidgets.remove(widget);
  gtk_widget_destroy(widget);
  aItemML.setData(0,0,0);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::GtkUI::getCallbackName(
 GtkType aType
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aName == "valueChanged") {
    if (aType == gtk_option_menu_get_type()){
#ifdef HAS_GTK1
      return "activate";
#else
      return "changed";
#endif
    }
  } else if (aName == "activate") {
    if (aType == gtk_button_get_type()){
      return "clicked";
    } else if ((aType == gtk_entry_get_type()) ||
               (aType == gtk_menu_item_get_type()) ||
               (aType == gtk_command_line_get_type())) {
      return "activate";
    }
  } else if (aName == "select") {
    if ((aType == gtk_list_get_type()) ||
        (aType == gtk_hscrollbar_get_type()) ||
        (aType == gtk_vscrollbar_get_type()) ||
        (aType == gtk_hscale_get_type()) ||
        (aType == gtk_vscale_get_type())){
      return "selection_changed";
    }
#ifdef HAS_GTK1
    else if (aType == gtk_tree_get_type()){
      return "selection_changed";
#else
    else if (aType == gtk_tree_view_get_type()){
      return "row-activated";
#endif
    } else if (aType == gtk_notebook_get_type()) {
      return "switch_page";
    } 
  } 
  else if (aName == "drag")     return "value_changed";
  else if (aName == "complete") return "complete";
  else if (aName == "paint")    return "paint";
  else if (aName == "collect")  return "collect";
  else if (aName == "popup")    return "popup";
  else if (aName == "ok")       return "clicked";
  else if (aName == "cancel")   return "clicked";
  else if (aName == "open")     return "row-expanded";
  else if (aName == "close")    return "row-collapsed";
  else if (aName == "selectBranch") return "row-activated";

  return "";
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::deleteCallback(
 Lib::ItemML& aItemML
,const std::string& aNameXML
,const OnX::Action& aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkObject* widget = (GtkObject*)aItemML.getData1();
  if(!widget) return;
  GtkType type = GTK_OBJECT_TYPE(widget);

  // function callback
  GtkSignalFunc callbackFunction = (GtkSignalFunc)callback;
  if (type == gtk_notebook_get_type()) {
    callbackFunction = (GtkSignalFunc)notebookCallback;
  }
#ifdef HAS_GTK1
#else
  else if (type == gtk_tree_view_get_type()) {
    if ((aNameXML == "select") ||
        (aNameXML == "selectBranch")) {
      callbackFunction = (GtkSignalFunc)treeViewActivatedCallback;
    } else if ((aNameXML == "open") ||
               (aNameXML == "close")) {
      callbackFunction = (GtkSignalFunc)treeViewOpenCloseCallback;
    }
  }
#endif
#ifdef ONX_HAS_GTK_GL_AREA
  else if(type == gtk_gl_area_get_type()) {
    if (aNameXML == "paint") {
      callbackFunction = (GtkSignalFunc)exposeCallback;
    } else {
      callbackFunction = (GtkSignalFunc)eventCallback;
    }
  }
#else
  else if(type == gtk_drawing_area_get_type()) {
    callbackFunction = (GtkSignalFunc)pageCallback;
  }
#endif

  // specials widget cases
  if (type == gtk_file_selection_get_type()) {
    if (aNameXML == "ok") {
      widget = GTK_OBJECT(GTK_FILE_SELECTION((GtkWidget*)widget)->ok_button);
      gtk_signal_disconnect_by_data(widget,(gpointer)&aAction);
    } else if (aNameXML == "cancel") {
      widget = 
        GTK_OBJECT(GTK_FILE_SELECTION((GtkWidget*)widget)->cancel_button);
      gtk_signal_disconnect_by_data(widget,(gpointer)&aAction);
    }
  } else if ((type == gtk_hscrollbar_get_type()) ||
             (type == gtk_vscrollbar_get_type()) ||
             (type == gtk_hscale_get_type()) ||
             (type == gtk_vscale_get_type())) {
    GtkAdjustment* adjustment = 
      gtk_range_get_adjustment(GTK_RANGE((GtkWidget*)widget));
    widget = GTK_OBJECT(adjustment);
    gtk_signal_disconnect_by_data(widget,(gpointer)&aAction);
  }
  else if (type == gtk_color_selection_dialog_get_type()) {
    widget =  GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG((GtkWidget*)widget)->ok_button);
    gtk_signal_disconnect_by_data(widget,(gpointer)&aAction);
  }
#ifdef HAS_GTK1
  else if ((aNameXML == "valueChanged") && 
           (type == gtk_option_menu_get_type())) {
    GList* item = //REVISIT
      gtk_container_get_children
        (GTK_CONTAINER(gtk_widget_get_parent((GtkWidget*)widget)));
    while (item) {
      //FIXME : to be tested !!
      gtk_signal_disconnect_by_data(GTK_OBJECT(item->data),(gpointer)&aAction);
      item = item->next;
    }
  }
#endif
  else if ((aNameXML == "valueChanged") && (type == gtk_combo_get_type())) {

    GtkEntry* entry = GTK_ENTRY(GTK_COMBO((GtkWidget*)widget)->entry);
    gtk_signal_disconnect_by_data(GTK_OBJECT(entry),(gpointer)&aAction);

    GtkList* list = GTK_LIST(GTK_COMBO((GtkWidget*)widget)->list);
    gtk_signal_disconnect_by_data(GTK_OBJECT(list),(gpointer)&aAction);    

  } else { //Generic case :
    gtk_signal_disconnect_by_data(widget,(gpointer)&aAction);
  }

}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::GtkUI::addCallback(
 Lib::ItemML& aItemML
,const std::string& aNameXML
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  if(!widget) return 0;
  GtkWidget* gtkWidget = (GtkWidget*)aItemML.getData1();
  if(!gtkWidget) return 0;
  GtkType type = GTK_OBJECT_TYPE(gtkWidget);
  GtkObject* objectGUI = GTK_OBJECT(gtkWidget);

  // function callback
  GtkSignalFunc callbackFunction = (GtkSignalFunc)callback;
  if (type == gtk_notebook_get_type()) {
    callbackFunction = (GtkSignalFunc)notebookCallback;
  }
#ifdef HAS_GTK1
#else
  else if (type == gtk_tree_view_get_type()) {
    if ((aNameXML == "select") ||
        (aNameXML == "selectBranch")) {
      callbackFunction = (GtkSignalFunc)treeViewActivatedCallback;
    } else if ((aNameXML == "open") ||
               (aNameXML == "close")) {
      callbackFunction = (GtkSignalFunc)treeViewOpenCloseCallback;
    }
  }
#endif
#ifdef ONX_HAS_GTK_GL_AREA
  else if(type == gtk_gl_area_get_type()) {
    if (aNameXML == "paint") {
      callbackFunction = (GtkSignalFunc)exposeCallback;
    } else {
      callbackFunction = (GtkSignalFunc)eventCallback;
    }
  }
#else
  else if(type == gtk_drawing_area_get_type()) {
    callbackFunction = (GtkSignalFunc)pageCallback;
  }
#endif
  
  // specials widget cases
  if (type == gtk_file_selection_get_type()) {
    if (aNameXML == "ok") {
      objectGUI = 
        GTK_OBJECT(GTK_FILE_SELECTION((GtkWidget*)widget)->ok_button);
    } else if (aNameXML == "cancel") {
      objectGUI = 
        GTK_OBJECT(GTK_FILE_SELECTION((GtkWidget*)widget)->cancel_button);
    } else {
      return 0;
    }
  } else if ((type == gtk_hscrollbar_get_type()) ||
             (type == gtk_vscrollbar_get_type()) ||
             (type == gtk_hscale_get_type()) ||
             (type == gtk_vscale_get_type())) {
    GtkAdjustment* adjustment = 
      gtk_range_get_adjustment(GTK_RANGE((GtkWidget*)widget));
    objectGUI = GTK_OBJECT(adjustment);
  }
  else if (type == gtk_color_selection_dialog_get_type()) {
    objectGUI = 
      GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG((GtkWidget*)widget)->ok_button);
  }
#ifdef HAS_GTK1
  else if ((aNameXML == "valueChanged") && 
           (type == gtk_option_menu_get_type())) {
    GList* item = //REVISIT
      gtk_container_get_children
        (GTK_CONTAINER(gtk_widget_get_parent((GtkWidget*)widget)));
    Action& action = 
      addAction(*this,widget,gtkWidget,aInterp,aScript,"valueChanged");
    while (item) {
      gtk_signal_connect
       (GTK_OBJECT(item->data),"activate",callbackFunction,(gpointer)&action);
      item = item->next;
    }
    return &action;
  }
#endif
  else if ((aNameXML == "valueChanged") && (type == gtk_combo_get_type())) {
    Action& action = 
      addAction(*this,widget,gtkWidget,aInterp,aScript,"valueChanged");

    objectGUI = GTK_OBJECT(GTK_ENTRY(GTK_COMBO((GtkWidget*)widget)->entry));
    gtk_signal_connect
      (objectGUI,"activate",callbackFunction,(gpointer)&action);
    
    objectGUI = GTK_OBJECT(GTK_LIST(GTK_COMBO((GtkWidget*)widget)->list));
    gtk_signal_connect
      (objectGUI,
       "button_release_event",(GtkSignalFunc)listCallback,(gpointer)&action);
    return &action;
  }
  else if ((aNameXML == "event") && (type == gtk_gl_area_get_type())) {
    Action& action = 
      addAction(*this,widget,gtkWidget,aInterp,aScript,"event");
    gtk_signal_connect(objectGUI,
                       "button_press_event",
                       (GtkSignalFunc)eventCallback,
                       (gpointer)&action);
    gtk_signal_connect(objectGUI,
                       "button_release_event",
                       (GtkSignalFunc)eventCallback,
                       (gpointer)&action);
    gtk_signal_connect(objectGUI,
                       "motion_notify_event",
                       (GtkSignalFunc)eventCallback,
                       (gpointer)&action);
    return &action;
  }

  // Generic case :
  std::string gtkCbkName = getCallbackName(type,aNameXML);
  if(gtkCbkName=="") return 0;

  Action& action = 
    addAction(*this,widget,gtkWidget,aInterp,aScript,aNameXML);
  gtk_signal_connect
    (objectGUI,gtkCbkName.c_str(),callbackFunction,(gpointer)&action);

  return &action;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::addCallbacks(
 Lib::ItemML& aItemML
,GtkWidget* aWidget
,const std::string& aNameXML
,GtkObject* aObjectGUI
,const std::string& aNameGUI
,GtkSignalFunc aCallback
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  if(!widget) return;

  const std::list<Lib::Element*> props = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;

  for(it=props.begin();it!=props.end();++it) {
    if(aNameXML==(*it)->name()) {
      std::string sexec;
      (*it)->attributeValue("exec",sexec);
      Action& action = addAction(*this,widget,aWidget,sexec,
                                 (*it)->value(),
                                 aNameXML);
      if(aNameGUI!="") {
        gtk_signal_connect(aObjectGUI, 
                           aNameGUI.c_str(),
                           aCallback, 
                           (gpointer)&action);
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::destroyCallback(
 GtkObject* //aObject
,gpointer //aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //GtkUI* This = (GtkUI*)aTag; 
}
//////////////////////////////////////////////////////////////////////////////
gboolean OnX::GtkUI::avoidDeleteCallback(
 GtkObject* /*aObject*/
,gpointer /*aTag*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return TRUE;
}
#ifdef HAS_GTK1
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::realizeCallback(
 GtkObject* aObject
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : inefficient to get the x,y again through the element...
  Lib::ItemML* element = (Lib::ItemML*)aTag;
  std::string svalue;
  if(element->propertyValue("geometry",svalue)) {
   int x,y,w,h;
   if(Lib::smanip::togeometry(svalue,x,y,w,h)) {
     gdk_window_set_hints(GTK_WIDGET(aObject)->window,
                          x,y,0,0,0,0,GDK_HINT_POS);
     gdk_window_move (GTK_WIDGET(aObject)->window, x, y);
   }
  }
}
#endif
//////////////////////////////////////////////////////////////////////////////
int OnX::GtkUI::getCompound(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if(aName=="Window")        return gtk_WINDOW;
  else if(aName=="MenuBar")       return gtk_MENUBAR;
  else if(aName=="ToolBar")       return gtk_TOOLBAR;
  else if(aName=="ScrolledText")  return gtk_SCROLLED_TEXT;
  else if(aName=="ScrolledList")  return gtk_SCROLLED_LIST;
  else if(aName=="ScrolledTree")  return gtk_SCROLLED_TREE;
  else if(aName=="Dialog")        return gtk_DIALOG;
  else if(aName=="ToolBarItem")   return gtk_TOOLBAR_ITEM;
  else if(aName=="MenuSeparator") return gtk_MENU_SEPARATOR;
  else if(aName=="Password")      return gtk_PASSWORD;
  else if(aName=="VContainer")    return gtk_VCONTAINER;
  else if(aName=="HContainer")    return gtk_HCONTAINER;

#ifdef ONX_HAS_INVENTOR
  else if(aName=="SoExaminerViewer") return gtk_SOEXAMINERVIEWER;
  else if(aName=="SoPlaneViewer")    return gtk_SOPLANEVIEWER;
  else if(aName=="SoFlyViewer")      return gtk_SOFLYVIEWER;
  else if(aName=="SoWalkViewer")     return gtk_SOWALKVIEWER;
  else if(aName=="PageViewer")       return gtk_PAGEVIEWER;
  else if(aName=="PlanePageViewer")  return gtk_PLANEPAGEVIEWER;
#endif
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
GtkType OnX::GtkUI::getType(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if(aName=="VBox")           return gtk_vbox_get_type();
  else if(aName=="HBox")           return gtk_hbox_get_type();
  else if(aName=="VPaned")         return gtk_vpaned_get_type();
  else if(aName=="HPaned")         return gtk_hpaned_get_type();
  else if(aName=="MenuItem")       return gtk_menu_item_get_type();
  else if(aName=="Menu")           return gtk_menu_get_type();
  else if(aName=="CommandLine")    return gtk_command_line_get_type();
  else if(aName=="Label")          return gtk_label_get_type();
  else if(aName=="PushButton")     return gtk_button_get_type();
  else if(aName=="ToggleButton")   return gtk_toggle_button_get_type();
  else if(aName=="FileSelection")  return gtk_file_selection_get_type();
  else if(aName=="Entry")          return gtk_entry_get_type();
  else if(aName=="OptionMenu")     return gtk_option_menu_get_type();
  else if(aName=="ComboBox")       return gtk_combo_get_type();
  else if(aName=="HScrollBar")     return gtk_hscrollbar_get_type();
  else if(aName=="VScrollBar")     return gtk_vscrollbar_get_type();
  else if(aName=="HScale")         return gtk_hscale_get_type();
  else if(aName=="VScale")         return gtk_vscale_get_type();
  else if(aName=="ProgressBar")    return gtk_progress_bar_get_type();

  else if(aName=="TabStack")       return gtk_notebook_get_type();
  else if(aName=="ToggleMenuItem") return gtk_check_menu_item_get_type();
  else if(aName=="CascadeButton")  return gtk_menu_item_get_type();
  else if(aName=="ColorSelection") return gtk_color_selection_dialog_get_type();
  else if(aName=="WarningDialog")  return gtk_warning_dialog_get_type();
#ifdef ONX_HAS_GTK_GL_AREA
  else if(aName=="OpenGLArea")     return gtk_gl_area_get_type();
#else
  else if(aName=="OpenGLArea")     return gtk_drawing_area_get_type();
#endif

  else return GTK_TYPE_INVALID;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::callback(
 GtkObject* aObject
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;

  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  std::string svalue;
  

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());
  //WARNING : take care of inheritance !

  if(GTK_IS_COMMAND_LINE(widget)) {
    gchar* cmd = gtk_command_line_get_value(GTK_COMMAND_LINE(aObject));
    svalue = cmd ? cmd : "";
  } else if(GTK_IS_ENTRY(widget)) {
    std::string cmd = gtk_entry_get_text(GTK_ENTRY(aObject));
    gtk_entry_set_text (GTK_ENTRY(aObject),"");
    svalue  = cmd;
  } else if(GTK_IS_FILE_SELECTION(widget)) {
    svalue = 
      gtk_file_selection_get_filename(GTK_FILE_SELECTION(widget));
  } else if(GTK_IS_COLOR_SELECTION_DIALOG(widget)) {
    gdouble rgba[4];
    gtk_color_selection_get_color(
      GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(widget)->colorsel),rgba);
    std::string s;
    Lib::smanip::printf(s,3 * 16,"%g %g %g",rgba[0],rgba[1],rgba[2]);
    svalue = s;
  } else if(GTK_IS_LIST(widget)) {
    GList* dlist = GTK_LIST(aObject)->selection;
    if(dlist) {
      svalue  = "";
      while(dlist) {
        GtkWidget* item = GTK_WIDGET(dlist->data);
        GtkLabel* label = GTK_LABEL(GTK_BIN(item)->child);
        gchar* s = 0;
        gtk_label_get(label,&s);
        svalue = s?s:"";
        dlist=dlist->next;
        if(dlist) svalue = "\n";
      }
    }
#ifdef HAS_GTK1
  } else if(GTK_IS_TREE(widget)) {
    GList* dlist = GTK_TREE_SELECTION(aObject);
    if(dlist) {
      GtkWidget* item = GTK_WIDGET(dlist->data);
      if (!GTK_TREE_ITEM_SUBTREE(item)) {
        svalue = gtk_tree_item_get_path(GTK_TREE_ITEM(item));
        if (svalue.size() == 0) return;
      } else if (GTK_TREE_ITEM_SUBTREE(item)) {
        svalue = gtk_tree_item_get_path(GTK_TREE_ITEM(item));
        if (svalue.size() == 0) return;
      }
    }
#else 
  // for GTK 2.2, we have a treeViewActivatedCallback
#endif
  } else if(GTK_IS_RANGE(widget)) {
    float value = gtk_range_get_adjustment(GTK_RANGE(widget))->value;
    Lib::smanip::printf(svalue,64,"%d",(int)value);
  } else if(GTK_IS_CHECK_MENU_ITEM(widget)) {
    svalue = GTK_CHECK_MENU_ITEM(aObject)->active ? "true" : "false";
  } else if(GTK_IS_OPTION_MENU(widget)) {
#ifdef HAS_GTK1
    if(GTK_BIN(widget)->child) {
      char* str = 0;
      gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child),&str);
      if(str) svalue = str;
    }
#else
    if (gtk_option_menu_get_menu(GTK_OPTION_MENU(widget)))  {
      if (gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(widget)))))  {
        GtkLabel* label = GTK_LABEL(GTK_BIN(gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(widget)))))->child);
        if(label) {
          char* str = 0;
          gtk_label_get(label,&str);
          if(str) svalue = str;
        }
      }
    }
#endif
  } else if(GTK_IS_COMBO(widget)) {
    GtkEntry* entry = GTK_ENTRY(GTK_COMBO(widget)->entry);
    std::string cmd = gtk_entry_get_text(entry);
    svalue = cmd;
  }

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::callback :"
        << " value \"" << svalue << "\"."        << Lib::endl;
  }

  This->fCallbackData = 
    CallbackData(action->widget(),gtk_widget_get_name(widget),svalue,"",0,0);
  
  // Execute script :
  action->execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();
}
#ifdef HAS_GTK1
#else
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::treeViewActivatedCallback(
 GtkTreeView* aView
,GtkTreePath*
,GtkTreeViewColumn*
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
// select and selectBranch callbacks
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  if(GTK_IS_TREE_VIEW(widget)) {
    GtkTreeSelection* dselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(aView));
    if (dselection) {
      GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(aView)); 
      if(!model) return;
      GtkTreeIter child;
      if (gtk_tree_selection_get_selected(dselection, &model, &child)) {
        if (gtk_tree_model_iter_has_child(model,&child)) {
          if (action->name() == "select") return;
        } else {
          if (action->name() != "select") return;
        }
        std::string svalue = gtk_tree_selection_get_path(dselection);
        This->fCallbackData = 
          CallbackData(action->widget(),
                       gtk_widget_get_name(widget),svalue,"",0,0);
        // Execute script :
        action->execute(std::vector<std::string>());
        This->fCallbackData = CallbackData();
      }
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::treeViewOpenCloseCallback(
 GtkTreeView* aView
,GtkTreeIter* aIter
,GtkTreePath*
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
// open and close callbacks
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  if(GTK_IS_TREE_VIEW(widget)) {
    GtkTreeSelection* dselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(aView));
    if (dselection) {
      GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(aView)); 
      if(!model) return;
      GtkTreeIter child;
      if (!gtk_tree_model_iter_has_child(model,aIter)) {
        return;
      }
      if (gtk_tree_selection_get_selected(dselection, &model, &child)) {
          std::string svalue = gtk_tree_selection_get_path(dselection);
          This->fCallbackData = 
            CallbackData(action->widget(),
                         gtk_widget_get_name(widget),svalue,"",0,0);
          // Execute script :
          action->execute(std::vector<std::string>());
          This->fCallbackData = CallbackData();
      }
    }
  }
}

#endif
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::pageCallback(
 GtkObject*
,GtkCallbackStruct* aData
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  std::string svalue = aData->value;
  int cbkx = aData->x;
  int cbky = aData->y;

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::pageCallback :"
        << " value \"" << svalue << "\"."        << Lib::endl;
  }

  This->fCallbackData = 
    CallbackData(action->widget(),
                 gtk_widget_get_name(GTK_WIDGET(action->nativeWidget())),
                 svalue,"",cbkx,cbky);
  
  // Execute script :
  action->execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::notebookCallback(
 GtkObject*
,GtkNotebookPage* aPage
,guint
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  // The notebookCallback is called when children are destroyed.
  // If quiting and action is background, we do not send the 
  // execution message (else we have a bad memory balance).
  if(This->beingDestroyed() && action->isBackground()) return;

  std::string svalue = std::string(GTK_LABEL(aPage->tab_label)->label);

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::notebookCallback :"
        << " value \"" << svalue << "\"."        << Lib::endl;
  }

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  This->fCallbackData = 
    CallbackData(action->widget(),gtk_widget_get_name(widget),svalue,"",0,0);
  
  // Execute script :
  action->execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::listCallback(
 GtkObject*
,GdkEvent*
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  std::string svalue;
  
  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  if(GTK_IS_COMBO(widget)) {
    GtkEntry* entry = GTK_ENTRY(GTK_COMBO(widget)->entry);
    std::string cmd = gtk_entry_get_text(entry);
    svalue = cmd;
  }

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::listCallback :"
        << " value \"" << svalue << "\"."        << Lib::endl;
  }


  This->fCallbackData = 
    CallbackData(action->widget(),gtk_widget_get_name(widget),svalue,"",0,0);
  
  // Execute script :
  action->execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::exposeCallback(
 GtkObject*
,GdkEventExpose*
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  std::string svalue;

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::exposeCallback :"
        << " value \"" << svalue << "\"."        << Lib::endl;
  }

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  This->fCallbackData = 
    CallbackData(action->widget(),gtk_widget_get_name(widget),svalue,"",0,0);
  
#ifdef ONX_HAS_GTK_GL_AREA
  if(gtk_gl_area_make_current(GTK_GL_AREA(widget))) {
    // Execute script :
    action->execute(std::vector<std::string>());
    gtk_gl_area_swap_buffers(GTK_GL_AREA(widget));
  }
#endif

  This->fCallbackData = CallbackData();

}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::eventCallback(
 GtkObject*
,GdkEvent* aEvent
,gpointer aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Action* action =(Action*)aTag;
  if(!action) return;
  GtkUI* This = Lib_Cast(action->ui(),OnX::GtkUI);
  if(!This) return;

  std::string svalue;
  std::string sevent;
  int cbkx = 0;
  int cbky = 0;

  if(aEvent->type==GDK_BUTTON_PRESS) {
    sevent = "ButtonPress";
    cbkx = (int)((GdkEventButton*)aEvent)->x;
    cbky = (int)((GdkEventButton*)aEvent)->y;
  } else if(aEvent->type==GDK_BUTTON_RELEASE) {
    sevent = "ButtonRelease";
    cbkx = (int)((GdkEventButton*)aEvent)->x;
    cbky = (int)((GdkEventButton*)aEvent)->y;
  } else if(aEvent->type==GDK_MOTION_NOTIFY) {
    sevent = "MotionNotify";
    cbkx = (int)((GdkEventMotion*)aEvent)->x;
    cbky = (int)((GdkEventMotion*)aEvent)->y;
  }

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::GtkUI::eventCallback :"
        << " event " << sevent 
        << " x " << cbkx
        << " y " << cbky
        << "."        << Lib::endl;
  }

  GtkWidget* widget = GTK_WIDGET(action->nativeWidget());

  This->fCallbackData =  
    CallbackData(action->widget(),
                 gtk_widget_get_name(widget),svalue,sevent,cbkx,cbky);
  
  // Execute script :
  action->execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();

}
//////////////////////////////////////////////////////////////////////////////
GtkWidget* findInHierarchy(
 GtkWidget* aWidget
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aWidget) return 0;
  const gchar* s = gtk_widget_get_name(aWidget);
  if(s && (aName==s)) return aWidget;
  // Look children :
  if(!GTK_IS_CONTAINER(aWidget)) return 0;
  GList* dlist = gtk_container_children(GTK_CONTAINER(aWidget));
  if(!dlist) return 0;
  GList* item = dlist;
  while(item) {
    GtkWidget* w = findInHierarchy((GtkWidget*)item->data,aName);
    if(w) return w;
    item = item->next;
  }
  g_list_free(dlist);
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::showAll(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* widget = (GtkWidget*)aItemML.getData1();
  if(widget) {
    std::string sshow;
    // If show property not here, we show the panel.
    // If show is here and is not "no", we show the panel.
    if( !aItemML.propertyValue("show",sshow) || (sshow!="no") ) {
      GtkWidget* shell = gtk_widget_get_toplevel(widget);
      if(shell) gtk_widget_show_all(shell);
    }
  } else { // Look children :
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) showAll(*(*it));
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::UI::ICyclic* OnX::GtkUI::createCyclic(
 const std::string& aName
,int aDelay
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = new GtkCyclic(fSession,aName,aDelay,aInterp,aScript);
  fCyclics.push_back(cyclic);
  return cyclic;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::GtkUI::widgetName(
 Lib::ItemML& aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* widget = (GtkWidget*)aItemML.getData1();
  if(!widget) return "";
  const gchar* cname = gtk_widget_get_name(GTK_WIDGET(widget));
  return std::string(cname?cname:"");
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::GtkUI::getTreeSelection(
 GtkWidget* aWidget
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_GTK1
  GList* dlist = GTK_TREE_SELECTION(aWidget);
  if(dlist) {
    GtkWidget* item = GTK_WIDGET(dlist->data);
    if(GTK_IS_TREE_ITEM(item)) {
      return gtk_tree_item_get_path(GTK_TREE_ITEM(item));
    }
  }
  return "";
#else
  GtkTreeSelection* dselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(aWidget));
  return gtk_tree_selection_get_path(dselection);
#endif
}

//////////////////////////////////////////////////////////////////////////////
void OnX::GtkUI::setTreeSelection(
 GtkWidget* aWidget
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef HAS_GTK1
  // Clear existing selection :
  GList* dlist = GTK_TREE_SELECTION(aWidget);
  while(dlist) {
    if (GTK_IS_TREE_ITEM(dlist->data)) {
      GtkTreeItem* item = GTK_TREE_ITEM(dlist->data);
      if(GTK_IS_TREE(GTK_WIDGET(item)->parent))
        gtk_tree_unselect_child(GTK_TREE(GTK_WIDGET(item)->parent),
                                GTK_WIDGET(item));
    }
    dlist = dlist->next;
  }
  
  // Set new selection :
  std::vector<std::string> items;
  Lib::smanip::lines(aValue,items);
  unsigned int itemn = items.size();
  bool found = false;
  GtkTree* tree = GTK_TREE(aWidget);
  
  for(unsigned int index=0;index<itemn;index++){
    if(!tree) {found = false;break;}
    GList* dlist = tree->children;
    while(dlist) {
      GtkTreeItem* item = GTK_TREE_ITEM(dlist->data);
      GtkLabel* label = GTK_LABEL(GTK_BIN(item)->child);
      gchar* s = 0;
      gtk_label_get(label,&s);
      if(s && (items[index]==std::string(s))) {
        if((index == itemn-1) && (GTK_IS_WIDGET(item)))
          gtk_tree_select_child(tree,GTK_WIDGET(item));
        found = true;
        tree = item->subtree ? GTK_TREE(item->subtree) : 0;
        break;
      }
      dlist = dlist->next;
    }
  }
  if(found) return;
  dlist = GTK_TREE_SELECTION(aWidget);
  while(dlist) {
    GtkTreeItem* item = GTK_TREE_ITEM(dlist->data);
    if(GTK_IS_TREE(GTK_WIDGET(item)->parent))
      gtk_tree_unselect_child(GTK_TREE(GTK_WIDGET(item)->parent),
                              GTK_WIDGET(item));
    dlist = dlist->next;
  }
#else
  std::vector<std::string> items;
  Lib::smanip::lines(aValue,items);
  // Try to open all the items selected 
  // if SELECTION_TYPE is single, we expand all 
  // items and select the last.
  for(unsigned int index=0;index<items.size();index++) {
    gtk_tree_view_expand_item(GTK_TREE_VIEW(aWidget),items[index]);
  }
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::GtkUI::nativeSize(
 void* aNative
,unsigned int& aWidth
,unsigned int& aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  GtkWidget* gtkWidget = (GtkWidget*)aNative;
  aWidth = gtkWidget->allocation.width;
  aHeight = gtkWidget->allocation.height;
  return true;
}
