
#include <Slash/Core/ISession.h>
#include <Slash/Core/IManager.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IScriptManager.h>
#include <Slash/UI/IInterpreter.h>

#include <Lib/Out.h>
#include <Lib/OpaqueManager.h>
#include <Lib/Manager.h>
#include <Lib/smanip.h>

#include <OnX/Helpers/UI.h>

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/kout.h> // Special to KUIP LAL.

extern "C" {
  void OnXKuipInitialize(Slash::Core::ISession&);
  void OnXKuipFinalize(Slash::Core::ISession&);
}

extern "C" {
  void kuinit_(int);
  char* disp_prompt_C(char*,void*);
  int disp_exit_C(void*);
}

extern "C" {
  void onx_kuip_def_();
}

static bool KUIP_Function(const std::string&,void*);
static void putV(void*,const char*,va_list);

//////////////////////////////////////////////////////////////////////////////
void OnXKuipInitialize(
 Slash::Core::ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
// Executed when dynamic load is done.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NWORDS=15000;
  Kuinit(NWORDS);
  ku_exec("/KUIP/SET_SHOW/RECALL_STYLE 'NONE'"); //ifdef WIN32
  ku_exec("/KUIP/SET_SHOW/BREAK OFF");
  
  if(!aSession.findManager("KuipManager")) {
    aSession.addManager(new Lib::OpaqueManager("KuipManager"));
  }

  Slash::UI::IScriptManager* scriptManager = 
    Lib_findManager(aSession,"ScriptManager",Slash::UI::IScriptManager);
  if(scriptManager) {
    Slash::UI::IInterpreter* interp = scriptManager->findInterpreter("KUIP");
    if(interp) {
      interp->setFunction(KUIP_Function,0);
    }
  }

  ku_action_data(&aSession); 
  ku_prompt(disp_prompt_C); 
  ku_exit(disp_exit_C); 
  ku_quit(disp_exit_C); 
  // Special to KUIP LAL :
  k_set_out_func((koutFunc)putV,&(aSession.printer())); 

  //  Should create here a KUIP/LOAD command for dynamic loading
  // KUIP modules (libs containing KUIP commands).
  //
  //  Below should be dynamicly loaded by the upper
  // non-existing KUIP/LOAD command.
  onx_kuip_def_(); 
}
//////////////////////////////////////////////////////////////////////////////
void OnXKuipFinalize(
 Slash::Core::ISession& aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  k_set_out_func(0,0);
  aSession.destroyManager("KuipManager");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int disp_exit_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;
  Slash::UI::IUI* ui = Lib_findUI(*session);
  if(!ui) return -1;
  ui->exit();
  return 0;
}
#include <stdlib.h>
#include <string.h>
//////////////////////////////////////////////////////////////////////////////
char* disp_prompt_C(
 char* aPrompt
,void* aSession
)
//////////////////////////////////////////////////////////////////////////////
// We should never pass here !
// KUIP free the return string ; need to allocate one.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#define STRDUP(str) strcpy((char*)malloc((unsigned)strlen(str)+1),str)

  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;
  Slash::UI::IUI* ui = Lib_findUI(*session);
  if(!ui) return STRDUP("Q");

  std::string value = ui->ask(aPrompt);

  return STRDUP(value.c_str());
}
//////////////////////////////////////////////////////////////////////////////
bool KUIP_Function(
 const std::string& aString
,void*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> text;
  Lib::smanip::words(aString,"\\n\\",text);
  // Do not use ku_exec to avoid to be prompted 
  // in case of syntax error.
  //ku_exec((char*)aString);
  for(unsigned int index=0;index<text.size();index++) {
    ku_exec_no_help((char*)text[index].c_str());
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void putV(
 void* aPrinter
,const char* aFormat
,va_list aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  char s[1024];
  vsprintf(s,aFormat,aArgs);
  Lib::Out out(*((Slash::Core::IWriter*)aPrinter));
  out << s;
}
