#include <Slash/Core/ISession.h>
#include <Slash/Core/ILibraryManager.h>
#include <Slash/Core/IOpaqueManager.h>
#include <Slash/UI/IUI.h>

#include <Lib/Out.h>
#include <Lib/Manager.h>
#include <Lib/Cast.h>
#include <Lib/smanip.h>

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/UI.h>
#include <OnX/Helpers/Inventor.h>

#include <Inventor/nodes/SoSeparator.h>

#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>
#include <HEPVis/nodekits/SoDetectorExample.h>

extern "C" {
  void onx_kuip(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>
#include <kuip/kout.h> // Special to KUIP LAL.

//////////////////////////////////////////////////////////////////////////////
void onx_kuip(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IOpaqueManager* kuipManager = 
    Lib_findManager(*session,"KuipManager",Slash::Core::IOpaqueManager);
  if(!kuipManager) return;

  Slash::UI::IUI* ui = Lib_findUI(*session);
  if(!ui) return;

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();

  ///////////////////////////////////////////////////////
  /// LIB ///////////////////////////////////////////////
  ///////////////////////////////////////////////////////
  if(cmd_path=="/LIB/LOAD") {
    std::string LIB = ku_gets();
    std::string PACK = ku_gets();

    Slash::Core::ILibraryManager* libraryManager = 
      Lib_findManager(*session,"LibraryManager",Slash::Core::ILibraryManager);
    if(libraryManager) {
      libraryManager->addLibrary(PACK,LIB,PACK);
    }

    return;
  }

  ///////////////////////////////////////////////////////
  /// UI ////////////////////////////////////////////////
  ///////////////////////////////////////////////////////
  if(cmd_path=="/ONX/UI/SET/PARAMETER") {
    std::string CPARAM = ku_gets();
    std::string CVALUE = ku_gets();

    std::vector<std::string> words = 
      Lib::smanip::words(CPARAM,".");    
    if((words.size()==2) && (words[0]=="viewer")) {
      Slash::UI::IWidget* widget = ui->currentWidget();
      if(!widget) return;
      if(!Lib_Cast(*widget,Slash::UI::ISoViewer)) return;
      widget->setParameter(words[1],CVALUE);
    } else {
      ui->setParameter(CPARAM,CVALUE);
    }
    
    return;
  }

  if(cmd_path=="/ONX/UI/LOAD") {

    std::string FILE = ku_gets();

    ui->load("main",Lib::fmanip::name(FILE));

    return;
  }

  if(cmd_path=="/ONX/UI/SHOW") {

    std::string WIDGET = ku_gets();

    ui->show(WIDGET);

    return;
  }

  if(cmd_path=="/ONX/UI/DIALOG") {

    std::string NAME = ku_gets();

    Lib::smanip::touppercase(NAME);

    if(NAME=="PRINT") {
      ui->load
        ("main",Lib::fmanip::name("$ONXROOT/scripts/OnX/InputPrint.onx"));
      ui->show("OnX_InputPrint_dialog");
    } else {
      Lib::Out out(session->printer());
      out << "Unknown dialog box " << Lib::sout(NAME)
          << "."
          << Lib::endl;
    }
 
    return;
  }

  ///////////////////////////////////////////////////////
  /// PAGE //////////////////////////////////////////////
  ///////////////////////////////////////////////////////
  if(cmd_path=="/ONX/PAGE/SET/CURRENTREGION") {
    int N = ku_geti();
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    soPage->setCurrentRegion(N);
    return;
  }

  if(cmd_path=="/ONX/PAGE/SET/PARAMETER") {
    std::string CPARAM = ku_gets();
    std::string CVALUE = ku_gets();
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoTools::setKitPartValue(soPage,CPARAM.c_str(),CVALUE.c_str());
    return;
  }

  if(cmd_path=="/ONX/PAGE/CLEAR") {
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    soPage->clearRegions();
    return;
  }

  if(cmd_path=="/ONX/PAGE/REGIONS/HIGHLIGHTABLE") {
    std::string CVALUE = ku_gets();
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    bool b;
    if(!Lib::smanip::tobool(CVALUE,b)) return;
    soPage->setRootRegionsHighlightable(b?TRUE:FALSE);
    return;
  }

  ///////////////////////////////////////////////////////
  /// REGION ////////////////////////////////////////////
  ///////////////////////////////////////////////////////
  if(cmd_path=="/ONX/REGION/SET/PARAMETER") {
    std::string CPARAM = ku_gets();
    std::string CVALUE = ku_gets();
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    SoTools::setKitPartValue(soRegion,CPARAM.c_str(),CVALUE.c_str());
    return;
  }

  if(cmd_path=="/ONX/REGION/CLEAR") {
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    soRegion->clear();
    return;
  }

  ///////////////////////////////////////////////////////
  /// DETECTOR //////////////////////////////////////////
  ///////////////////////////////////////////////////////
  if(cmd_path=="/ONX/DETECTOR/NEW") {
    return;
  }

  if(cmd_path=="/ONX/DETECTOR/SHOW") {
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    if(!soRegion->isOfType(SoDisplayRegion::getClassTypeId())) return;
    SoDisplayRegion* displayRegion = (SoDisplayRegion*)soRegion;
    SoSeparator* sep = (SoSeparator*)displayRegion->getStaticScene();
    if(!sep) return;
    sep->addChild(new SoDetectorExample);    
    return;
  }

  if(cmd_path=="/ONX/DETECTOR/DELETE") {
    return;
  }

}
