// this :
#import <OnX/NextStep/NSPageViewer.h>

#import <AppKit/NSApplication.h>
#import <AppKit/NSWindow.h>
#import <SCExaminerController.h>
#import <SCView.h>

#import <Inventor/nodes/SoLightModel.h>
#import <Inventor/nodes/SoCamera.h>
#import <Inventor/nodes/SoOrthographicCamera.h>
#import <Inventor/nodes/SoLightModel.h>

#import <HEPVis/misc/SoTools.h>
#import <HEPVis/nodekits/SoPage.h>
#import <HEPVis/nodekits/SoRegion.h>

#import <Lib/Debug.h>

#import <Onx/NextStep/NSAction.h>

class PageSetRegionCallback : public virtual SbSetRegionCallback {
public:
  PageSetRegionCallback(SCExaminerController* aExaminerController)
   :fExaminerController(aExaminerController){}
  virtual void setCamera(SoCamera* aCamera) {
    [fExaminerController setCamera:aCamera];
  }
private:
  SCExaminerController* fExaminerController;
};

@implementation NSPageViewer
- (id)initWithFrame:(NSRect)aRect {
  fExaminerController = 0;
  fPageView = 0;
  fPage = 0;
  fDestroyActions = 0;
  if(self = [super init]) {
    Lib::Debug::increment("OnX::NSPageViewer");

    fDestroyActions = [[NSMutableArray alloc] init];

    fPageView = [[SCView alloc] initWithFrame:aRect];
    //NOTE : inheriting SCExaminerController crashes at exit.
    fExaminerController = [[SCExaminerController alloc] init];
    // Create the view<->controller connection
    [fExaminerController setView:fPageView];
    [fPageView setController:fExaminerController];
    // The magic activate
    [fExaminerController activate];
    [fPageView reshape];

    //SoSceneManager* sceneManager = [examinerController sceneManager];
    //const SbViewportRegion& vpRegion = sceneManager->getViewportRegion();
    //SoGL2PSAction* action = new SoGL2PSAction(vpRegion);
    //this->setGLRenderAction(action);

    fPage = new SoPage;
    fPage->ref();
    // SetRegionCallback object deleted by the SoPage.
    fPage->setSetRegionCallback
      (new PageSetRegionCallback(fExaminerController));
    
    [fExaminerController setSceneGraph:(SoGroup*)fPage];

    //FIXME [fExaminerController setHeadlight:FALSE];

    fPage->highlightRegion(fPage->getRootRegion(0));
  }
  return self;
}
- (void) dealloc {
  int number = [fDestroyActions count];
  int index;
  for(index=0;index<number;index++) {
    [(OnX_NSAction*)[fDestroyActions objectAtIndex:index] execute];
  }
  [fDestroyActions release];
  Lib::Debug::decrement("OnX::NSPageViewer");
  [fExaminerController setSceneGraph:nil];
  //FIXME : pb in SC12 : fPage refcount is not 1 !
  while(fPage->getRefCount()) fPage->unref();
  [fPageView release];
  [fExaminerController release];
  [super dealloc];  
}
- (void)addDestroyAction:(OnX_NSAction*)aAction {
  [fDestroyActions addObject:aAction];
}
- (SoPage*)page {
  return fPage;
}
- (SoNode*)sceneGraph {
  return fPage;
}
- (NSView*)pageView {
  return fPageView;
}
- (BOOL)setParameter:(NSString*)aWhat value:(NSString*)aValue {
    if( ([aWhat isEqualToString:@"view"]==YES) && 
        ([aValue isEqualToString:@"all"]==YES) ) {
      [fExaminerController viewAll];
      return YES;
    } else if([aWhat isEqualToString:@"autoClipping"]==YES) {
    /*FIXME
      bool value = true;
      if(!Lib::smanip::tobool(aValue,value)) value = true;
      if(value) {
        if(this->isAutoClipping()==FALSE) 
          this->setAutoClipping(TRUE);
      } else {
        if(this->isAutoClipping()==TRUE) 
          this->setAutoClipping(FALSE);
      }*/
      return true;
    } else if([aWhat isEqualToString:@"decoration"]==YES) {
    /*FIXME 
      bool value = true;
      if(!Lib::smanip::tobool(aValue,value)) value = true;
      if(value) {
        if(this->isDecoration()==FALSE) 
          this->setDecoration(TRUE);
      } else {
        if(this->isDecoration()==TRUE) 
          this->setDecoration(FALSE);
      }*/
      return true;
    } else if([aWhat isEqualToString:@"feedbackVisibility"]==YES) {
    /*FIXME
      bool value = true;
      if(!Lib::smanip::tobool(aValue,value)) value = true;
      if(value) {
        if(this->isFeedbackVisible()==FALSE) 
          this->setFeedbackVisibility(TRUE);
      } else {
        if(this->isFeedbackVisible()==TRUE) 
          this->setFeedbackVisibility(FALSE);
      }*/
      return true;
    } else if([aWhat isEqualToString:@"headLight"]==YES) {
    /*FIXME
      bool value = true;
      if(!Lib::smanip::tobool(aValue,value)) value = true;
      if(value) {
        if(this->isHeadlight()==FALSE) 
          this->setHeadlight(TRUE);
      } else {
        if(this->isHeadlight()==TRUE) 
          this->setHeadlight(FALSE);
      }*/
      return true;
    } else if([aWhat isEqualToString:@"viewing"]==YES) {
    /*FIXME 
      bool value = true;
      if(Lib::smanip::tobool(aValue,value)) {
        if(value) {
          if(this->isViewing()==FALSE) this->setViewing(TRUE);
        } else {
          if(this->isViewing()==TRUE) this->setViewing(FALSE);
        }
        return true;
      }*/
      return true;
    } else if([aWhat isEqualToString:@"backgroundColor"]==YES) {
      //FIXME 
      return true;
    }
    return NO;
}
@end
