// this :
#import <OnX/NextStep/NextStepUI.h>

// Slash :
#import <Slash/Core/ISession.h>
#import <Slash/UI/IScriptManager.h>
#import <Slash/UI/IWidgetClass.h>
#import <Slash/UI/IWidget.h>
#import <Slash/UI/ISoViewer.h>

#import <Lib/Out.h>
#import <Lib/sout.h>
#import <Lib/smanip.h>
#import <Lib/fmanip.h>
#import <Lib/Messages.h>
#import <Lib/BaseML.h>
#import <Lib/Element.h>
#import <Lib/ItemML.h>
#import <Lib/Manager.h>
#import <Lib/Cast.h>

#import <OnX/NextStep/NextStepCyclic.h>
#import <OnX/NextStep/NextStepTk.h>
#import <OnX/NextStep/NSAction.h>
#import <OnX/Core/Messages.h>

#import <AppKit/NSAlert.h>

// Inventor :
#import <SCController.h>
#import <OnX/Inventor/HEPVis.h>

#import <OnX/NextStep/NSPageViewer.h>

// Handle some classID to avoid too much string comparison in createWidget.
#define nextstep_Window                1
#define nextstep_CommandLine           2
#define nextstep_PushButton            3
#define nextstep_VBox                  4
#define nextstep_HBox                  5
#define nextstep_MenuBar               6
#define nextstep_CascadeButton         7
#define nextstep_Menu                  8
#define nextstep_MenuItem              9
#define nextstep_ScrolledTree         10
#define nextstep_PageViewer           11
#define nextstep_FileSelectionDialog  12
#define nextstep_ScrolledList         13
#define nextstep_ScrolledText         14
#define nextstep_VPaned               15
#define nextstep_HPaned               16
#define nextstep_ToggleMenuItem       17
#define nextstep_ColorSelectionDialog 18
#define nextstep_Label                19
#define nextstep_Dialog               20
#define nextstep_VContainer           21
#define nextstep_HContainer           22
#define nextstep_Entry                23
#define nextstep_ComboBox             24
#define nextstep_VScrollBar           25
#define nextstep_HScrollBar           26
#define nextstep_ToolBar              27
#define nextstep_ToolBarItem          28
#define nextstep_MenuSeparator        29
#define nextstep_ProgressBar          30
#define nextstep_TabStack             31
#define nextstep_OptionMenu           32
#define nextstep_Password             33
#define nextstep_VScale               34
#define nextstep_HScale               35
#define nextstep_OpenGLArea           36
#define nextstep_ToggleButton         37
#define nextstep_WarningDialog        38

@interface OnX_NSCallback : OnX_NSAction {
  OnX::Action* fAction;
}
- (id)initWithAction:(OnX::Action*)aAction;
- (void)dealloc;
- (void)execute;
@end
@implementation OnX_NSCallback
- (id)initWithAction:(OnX::Action*)aAction {
  fAction = 0;
  if(self = [super init]) {
    fAction = aAction;
  }
  return self;
}
- (void)dealloc {
  //NSLog(@"debug : dealloc cbk");
  [super dealloc];
}
- (void)execute {
  OnX::NextStepUI::callback(*fAction);
}
@end

@interface OnX_NSMessageWindow : NSWindow {
  OnX::NextStepUI* fUI;
}
- (id)initWithUI:(OnX::NextStepUI*)aUI;
- (void)dealloc;
@end
@implementation OnX_NSMessageWindow
- (id)initWithUI:(OnX::NextStepUI*)aUI {
  fUI = 0;
  NSRect rect;
  if(self = [super initWithContentRect:rect styleMask:0 backing:NSBackingStoreBuffered defer:NO]) {
    fUI = aUI;
  }
  return self;
}
- (void)dealloc {  
  [super dealloc];
}
- (void)sendEvent:(NSEvent*)aEvent {
  if([aEvent type]==NSApplicationDefined) {
    Slash::Core::IMessage* mess = (Slash::Core::IMessage*)[aEvent data1];
    fUI->notification(mess);
  } else {
    [super sendEvent:aEvent];
  }
}
@end

@interface OnX_NSDestroyAction : OnX_NSAction {
  OnX::NextStepUI* fUI;
  NSString* fViewer;
}
- (id)initWithUI:(OnX::NextStepUI*)aUI andViewer:(NSString*)aViewer;
- (void)dealloc;
- (void)execute;
@end
@implementation OnX_NSDestroyAction
- (id)initWithUI:(OnX::NextStepUI*)aUI andViewer:(NSString*)aViewer {
  fUI = 0;
  fViewer = 0;
  if(self = [super init]) {
    fUI = aUI;
    fViewer = [[NSString alloc] initWithString:aViewer];
  }
  return self;
}
- (void)dealloc {
  //NSLog(@"debug : dealloc cbk");
  [fViewer release];
  [super dealloc];
}
- (void)execute {
}
@end

@interface OnX_NSApplicationDelegate : NSObject {
  Slash::Core::ISession* fSession;
}
- (id)initWithSession:(Slash::Core::ISession*)aSession;
- (void)dealloc;
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)aSender;
@end
@implementation OnX_NSApplicationDelegate
- (id)initWithSession:(Slash::Core::ISession*)aSession {
  fSession = 0;
  if(self = [super init]) {
    fSession = aSession;
  }
  return self;
}
- (void)dealloc {
  [super dealloc];
}
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication*)aSender {
  //NSLog(@"debug : delegate terminate\n");
  // Remove .log_tmp, .err, .out files :
  Slash::UI::IScriptManager* scriptManager = 
    Lib_findManager(*fSession,"ScriptManager",Slash::UI::IScriptManager);
  if(scriptManager) {
    scriptManager->recordExecutedScripts(false);
  }
  fSession->redirectOutput(false);
  return NSTerminateNow;
}
@end

//////////////////////////////////////////////////////////////////////////////
OnX::NextStepUI::NextStepUI(
 Slash::Core::ISession& aSession
,const std::string& aFile
,const std::vector<std::string>& aArgs
)
:BaseUI(aSession)
,fAutoreleasePool(0)
,fApplication(0)
,fAppDelegate(0)
,fConsole(0)
,fMessageWindow(0)
,fAlert(0)
,fExitCode(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : ================================== : 100\n");
  // Initialize NS (NextStep) :
  fAutoreleasePool = [[NSAutoreleasePool alloc] init];
  fApplication = [NSApplication sharedApplication];  
  // fApplication is created with a class factory.
  // According NextStep conventions, it is then autoreleased, 
  // we do not have to release it at the end ; this will be done   
  // by the pool.

  fAppDelegate = [[OnX_NSApplicationDelegate alloc] initWithSession:&aSession];
  [fApplication setDelegate:fAppDelegate];

  // Get application name from aArgv[0] or from 
  // the name of the main .onx file :
  std::string app = "OnX";
  std::string path,name,suffix;
  std::string ss = (aArgs.size()>=1 ? aArgs[0] : aFile);
  if(Lib::smanip::pathNameSuffix(ss,path,name,suffix)) {
    app = suffix!="" ? name.substr(0,name.size()-(suffix.size()+1)) : name;
  }

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::NextStepUI :"
        << " application name is " << Lib::sout(app) << "."
        << Lib::endl;
  }

  NSString* nsapp = [[NSString alloc] initWithCString:app.c_str()];
  NSImage* image = [NSImage imageNamed:nsapp];
  [fApplication setApplicationIconImage:image];

  // Could have the menubar done in a .nib with InterfaceBuilder.
  // If so get it with [fApplication mainMenu]
  // It seems that we can't pass an absolute path name out of the .app.
  BOOL status = [NSBundle loadNibNamed:nsapp owner:fApplication];
  if(status==NO) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::NextStepUI :"
        << " " << app << ".nib not found."
        << Lib::endl;

    // Menubar :
    NSMenu* menu = [[NSMenu allocWithZone:[NSMenu menuZone]] init];
    [fApplication setMainMenu:menu];
    [menu release];

    //FIXME : is the nib file had not been found, the Apple application 
    // pdm is empty ! How to add a "Quit" item in the Apple appliction pdm ?
    // We have tried to use [fApplication setAppleMenu:our_pdm] 
    // without success.
  }
  [nsapp release];

  // Message window :
  fMessageWindow = [[OnX_NSMessageWindow alloc] initWithUI:this];

  // Initialize Inventor :
  [SCController initCoin];
  HEPVis::initClasses();

  if(!startup(aFile)) return;

  fConsole = (NSText*)findNativeWidget("consoleText");
  fAlert = (NSAlert*)findNativeWidget("warningDialog");
}
//////////////////////////////////////////////////////////////////////////////
OnX::NextStepUI::~NextStepUI(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fBeingDestroyed = true;

  // Treat messages emitted between the quiting of the steering
  // and here. They may come from the destruction of various
  // managers in the ~Session.
  synchronize();

  std::list< std::pair<NSObject*,std::string> >::iterator it;
  for(it=fPool.begin();it!=fPool.end();it=fPool.erase(it)) {
    [(*it).first release];
  }

  fConsole = 0;
  [fMessageWindow release];

  [fApplication setDelegate:nil];
  [fAppDelegate release];

  [fAutoreleasePool release];
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::NextStepUI::cast( 
 const std::string& aClassName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aClassName=="OnX::NextStepUI") {
    return Lib_SCast(NextStepUI);
  } else {
    return BaseUI::cast(aClassName);
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::put(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Append line at end of report region.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fConsole) return;  
  NSString* as = [[NSString alloc] initWithCString:aString.c_str()];
  // insertText positions the view at end of the text.
  [fConsole insertText:as];
  [as release];
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::warn(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Modal method.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAlert) {
    echo("WARNING : "+aString);
    return;
  }
  NSString* nss = [[NSString alloc] initWithCString:aString.c_str()];
  [fAlert setMessageText:nss];
  [nss release];
  [fAlert runModal];
}
//////////////////////////////////////////////////////////////////////////////
int OnX::NextStepUI::nativeSteer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fExitCode = 0;
  if(!fTopItemML) return 0;
  fSteerLevel++;
  [fApplication run];
  fSteerLevel--;
  return fExitCode;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::synchronize(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::exit(
 int aExitCode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fApplication) return;
  fExitCode = aExitCode;
  [fApplication stop:nil];
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::lock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::unlock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::notify(
 Slash::Core::IMessage* aMessage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fMessageWindow) return false;

  int data1 = 0;
  int data2 = 0;
  if(sizeof(void*)==4) {
    data1 = (int)aMessage;
  //} else if(sizeof(void*)==8) {
  //data1 = aMessage & 0xFFFF;
  //data2 = (aMessage >> 32) & 0xFFFF;
  } else {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::notify :"
        << " can't send message on a " << int(8 * sizeof(void*))
        << " bits machine."
        << Lib::endl;
    return false;
  }

  NSPoint loc;
  int num = [fMessageWindow windowNumber];
  NSEvent* event = 
    [NSEvent otherEventWithType:NSApplicationDefined 
                       location:loc  
                  modifierFlags:0 
                      timestamp:0 
                   windowNumber:num 
                        context:nil 
                        subtype:0 
                          data1:data1
                          data2:data2];
  [fApplication postEvent:event atStart:NO];

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::show(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="" && fTopItemML) {
    showAll(*fTopItemML);
    return;
  }
  NSObject* widget = (NSObject*)findNativeWidget(aName);
  if(!widget) return;
  if([widget isKindOfClass:[OnX_NSDialog class]]==YES) {
    NSRect rect = [(NSWindow*)widget frame];
    NSRect srect = [[NSScreen mainScreen] visibleFrame];
    if(rect.size.width>srect.size.width) 
      rect.size.width = srect.size.width;
    if(rect.size.height>srect.size.height) 
      rect.size.height = srect.size.height;
    NSPoint origin;
    origin.x = (srect.size.width - rect.size.width)/2;
    if(origin.x<0) origin.x = 0;
    origin.y = (srect.size.height - rect.size.height)/2;
    if(origin.y<0) origin.y = 0;
    [(NSWindow*)widget setFrameOrigin:origin];
    [(NSWindow*)widget makeKeyAndOrderFront:fApplication];
  } else if([widget isKindOfClass:[NSWindow class]]==YES) {
    [(NSWindow*)widget makeKeyAndOrderFront:fApplication];
  } else if([widget isKindOfClass:[OnX_NSFileChooser class]]==YES) {
    [(OnX_NSFileChooser*)widget show];
  } else if([widget isKindOfClass:[OnX_NSColorChooser class]]==YES) {
    [(OnX_NSColorChooser*)widget show:fApplication];
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::hide(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)findNativeWidget(aName);
  if(!widget) return;
  if([widget isKindOfClass:[NSWindow class]]==YES) {
    [(NSWindow*)widget close];
  } else if([widget isKindOfClass:[OnX_NSColorChooser class]]==YES) {
    [(OnX_NSColorChooser*)widget hide];
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::refresh(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)findNativeWidget(aName);
  if(!widget) return false;
  if([widget isKindOfClass:[OnX_NSOpenGLArea class]]==YES) {
    [(OnX_NSOpenGLArea*)widget refresh];
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::nativeSize(
 void* aNative
,unsigned int& aWidth
,unsigned int& aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)aNative;
  NSRect rect = getFrame(widget);
  aWidth = (unsigned int)rect.size.width;
  aHeight = (unsigned int)rect.size.height;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::position(
 const std::string& aName
,int& aX
,int& aY
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aX = 0; 
  aY = 0;
  NSObject* widget = (NSObject*)findNativeWidget(aName);
  if(!widget) return false;
  NSRect rect = getFrame(widget);
  aX = (int)rect.origin.x;
  aY = (int)rect.origin.y;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::parameterValue(
 const std::string& aWhat
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::parameterValue :"
        << " widget " << Lib::sout(aWidget)
        << " resource " << Lib::sout(aResource) << "..."
        << Lib::endl;
  }


  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::parameterValue :"
        << " widget " << Lib::sout(aWidget)
        << " not found." << Lib::endl;
    return false;
  }

  NSObject* widget = (NSObject*)element->getData1();
  if(widget) {
    if(aResource=="set") {
      if([widget isKindOfClass:[OnX_NSToggleMenuItem class]]==YES) {
        int state = [(NSMenuItem*)widget state];
        aValue = state==NSOnState ? "true" : "false";
        return true;
      } else if([widget isKindOfClass:[OnX_NSToggleButton class]]==YES) {
        int state = [(NSButton*)widget state];
        aValue = state==NSOnState ? "true" : "false";
        return true;
      }
    } else if(aResource=="label") {
      if([widget isKindOfClass:[OnX_NSMenuItem class]]==YES) {
        NSString* title = [(OnX_NSMenuItem*)widget title];
        aValue = title ? std::string([title cString]) : "";
        return true;
      }
    } else if(aResource=="selection") {
      if([widget isKindOfClass:[OnX_NSTree class]]==YES) {
        NSString* selection = [(OnX_NSTree*)widget selectedPath];
        aValue = selection ? std::string([selection cString]) : "";
        return true;
      } else if([widget isKindOfClass:[OnX_NSList class]]==YES) {
        NSString* selection = [(OnX_NSList*)widget selection];
        aValue = selection ? std::string([selection cString]) : "";
        return true;
      } else if([widget isKindOfClass:[NSTabView class]]==YES) {
        NSTabViewItem* item = [(NSTabView*)widget selectedTabViewItem];
        if(item) {
          NSString* label = [item label];
          aValue = label ? std::string([label cString]) : "";
        }
        return true;
      }
    } else if(aResource=="itemCount") {
      if([widget isKindOfClass:[NSTabView class]]==YES) {
        int value  = [[(NSTabView*)widget tabViewItems] count];
        Lib::smanip::printf(aValue,64,"%d",value);
        return true;
      }
    } else if(aResource=="value") {
      if([widget isKindOfClass:[OnX_NSPopUpButton class]]==YES) {
        NSMenuItem* menuItem = 
         (NSMenuItem*)[(OnX_NSPopUpButton*)widget selectedItem];
        NSString* title = [menuItem title];
        aValue = std::string([title cString]);
        return true;
      } else if([widget isKindOfClass:[NSText class]]==YES) {
        // ScrolledText
        NSString* value = [(NSText*)widget string];
        aValue = std::string([value cString]);
        return true;
      } else if([widget isKindOfClass:[NSTextField class]]==YES) {
        // Entry and CommandLine
        NSString* value = [(NSTextField*)widget stringValue];
        aValue = std::string([value cString]);
        return true;
      } else if([widget isKindOfClass:[NSProgressIndicator class]]==YES) {
        double value = [(NSProgressIndicator*)widget doubleValue];
        Lib::smanip::printf(aValue,64,"%d",(int)value);
        return true;
      } else if([widget isKindOfClass:[NSSlider class]]==YES) {
        double value = [(NSSlider*)widget doubleValue];
        Lib::smanip::printf(aValue,64,"%d",(int)value);
        return true;
      } else if([widget isKindOfClass:[NSScroller class]]==YES) {
        double value = [(NSScroller*)widget doubleValue];
        Lib::smanip::printf(aValue,64,"%d",(int)value);
        return true;
      }
    }
  }
  // Not found, then look in XML elements :
  if(aResource=="class") {
    return element->attributeValue("",aResource,aValue);
  }
  return element->propertyValue(aResource,aValue);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::setParameter(
 const std::string& aWhat
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;

  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::setParameter :"
        << " widget " << Lib::sout(aWidget)
        << " resource " << Lib::sout(aResource)
        << " value " << Lib::sout(aValue) << "."
        << Lib::endl;
  }

  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::NextStepUI::setParameter :"
        << " widget " << Lib::sout(aWidget)
        << " not found." << Lib::endl;
    return false;
  }

  NSObject* widget = (NSObject*)element->getData1();

  std::string svalue;

  if(aResource=="label") {
    if([widget isKindOfClass:[NSButton class]]==YES) {
      NSString* nss = [[NSString alloc] initWithCString:aValue.c_str()];
      [(NSButton*)widget setTitle:nss];
      [nss release];
      element->setPropertyValue("label",aValue);
      return true;
    } else if([widget isKindOfClass:[NSMenuItem class]]==YES) {
      NSString* nss = [[NSString alloc] initWithCString:aValue.c_str()];
      [(NSMenuItem*)widget setTitle:nss];
      [nss release];
      element->setPropertyValue("label",aValue);
      return true;
    }
/* FIXME
  } else if(aResource=="dirMask") {
    if(widget->type()=="FileSelectionDialog") {
      svalue = aValue;
      Lib::smanip::getenv(svalue);
      ((NextStepTk::FileSelectionDialog*)widget)->setDirMask(svalue);
      return true;
    }
  } else if(aResource=="directory") {
    if(widget->type()=="FileSelectionDialog") {
      svalue = aValue;
      Lib::smanip::getenv(svalue);
      ((NextStepTk::FileSelectionDialog*)widget)->setDirectory(svalue);
      return true;
    }
*/
  } else if(aResource=="mode") {
    if([widget isKindOfClass:[OnX_NSFileChooser class]]==YES) {
      if(aValue=="save") {
        [(OnX_NSFileChooser*)widget setModeOpen:false];
      } else {
        [(OnX_NSFileChooser*)widget setModeOpen:true];
      }
      return true;
    }
  } else if(aResource=="set") {
    bool value;
    if(Lib::smanip::tobool(aValue,value)) {
      if([widget isKindOfClass:[OnX_NSToggleMenuItem class]]==YES) {
        [(NSMenuItem*)widget setState:value?NSOnState:NSOffState];
        return true;
      } else if([widget isKindOfClass:[OnX_NSToggleButton class]]==YES) {
        [(NSButton*)widget setState:value?NSOnState:NSOffState];
        return true;
      }
    }
  } else if(aResource=="value") {
    if([widget isKindOfClass:[NSText class]]==YES) {
      // ScrolledText
      NSString* nss = [[NSString alloc] initWithCString:aValue.c_str()];
      [(NSText*)widget setString:nss];
      [nss release];
      return true;
    } else if([widget isKindOfClass:[NSTextField class]]==YES) {
      // Entry, CommandLine and Password
      NSString* nss = [[NSString alloc] initWithCString:aValue.c_str()];
      [(NSTextField*)widget setStringValue:nss];
      [nss release];
      return true;
    } else if([widget isKindOfClass:[OnX_NSPopUpButton class]]==YES) {
      NSString* nss = [[NSString alloc] initWithCString:aValue.c_str()];
      [(OnX_NSPopUpButton*)widget selectItemWithTitle:nss];
      [nss release];
      return true;
    } else if([widget isKindOfClass:[NSProgressIndicator class]]==YES) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        // [0,100]
        [(NSProgressIndicator*)widget setDoubleValue:(double)value];
        return true;
      }
      return true;
    } else if([widget isKindOfClass:[NSSlider class]]==YES) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        // [0,100]
        [(NSSlider*)widget setDoubleValue:(double)value];
        return true;
      }
      return true;
    } else if([widget isKindOfClass:[NSScroller class]]==YES) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        // [0,100]
        [(NSScroller*)widget setDoubleValue:(double)value];
        return true;
      }
      return true;
    }
  } else if(aResource=="selection") {
    if([widget isKindOfClass:[OnX_NSList class]]==YES) {
      NSString* nss = 
        [[NSString alloc] initWithCString:aValue.c_str()];
      //FIXME [(OnX_NSList*)widget setSelection:nss];
      [nss release];
      return true;
/* FIXME
    if(widget->type()=="ScrolledList") {
      if(((NextStepTk::ScrolledList*)widget)->setSelection(aValue)) return true;
    } else if(widget->type()=="Tree") {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      if(((NextStepTk::Tree*)widget)->setSelection(items)) return true;
*/
    }
  } else if(aResource=="items") {
    if([widget isKindOfClass:[OnX_NSPopUpButton class]]==YES) {
      OnX_NSPopUpButton* optionMenu = (OnX_NSPopUpButton*)widget;
      [optionMenu removeAllItems];
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      for(unsigned int index=0;index<items.size();index++) {
        NSString* nsitem = 
          [[NSString alloc] initWithCString:items[index].c_str()];
        [optionMenu addItemWithTitle:nsitem];
        [nsitem release];
      }
      return true;
    } else if([widget isKindOfClass:[OnX_NSComboBox class]]==YES) {
      OnX_NSComboBox* comboBox = (OnX_NSComboBox*)widget;
      [comboBox removeAllItems];
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      for(unsigned int index=0;index<items.size();index++) {
        NSString* nsitem = 
          [[NSString alloc] initWithCString:items[index].c_str()];
        [comboBox addItemWithObjectValue:nsitem];
        [nsitem release];
      }
    } else if([widget isKindOfClass:[OnX_NSList class]]==YES) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      OnX_NSTreeItem* rootItem = [[OnX_NSTreeItem alloc] initWithParent:0];
      for(unsigned int index=0;index<items.size();index++) {
        OnX_NSTreeItem* item = 
          [[OnX_NSTreeItem alloc] initWithParent:rootItem];
        NSString* nsLabel = 
          [[NSString alloc] initWithCString:items[index].c_str()];
        [item setLabel:nsLabel];
        [rootItem addObject:item];
        [item release];
        [nsLabel release];
      }
      [(OnX_NSList*)widget setRootItem:rootItem];
      return true;
    } else if([widget isKindOfClass:[OnX_NSTree class]]==YES) {
      OnX_NSTree* tree = (OnX_NSTree*)widget;
      Lib::ItemMLFactory factory;
      Lib::BaseML treeML(factory,fSession.verboseLevel());
      std::vector<std::string> tags;
      tags.push_back("tree");
      tags.push_back("treeItem");
      treeML.setTags(tags);
      if(treeML.loadString(aValue,fSession.printer())) {
        Lib::ItemML* top = treeML.topItem();
        //top->dump();
        if(top) {
          OnX_NSTreeItem* item = [[OnX_NSTreeItem alloc] initWithParent:0];
          createTree(item,top);
          [tree setRootItem:item];
        }
        return true;
      }
    }
/*FIXME
  } else if(aResource=="popupItems") {
    if(widget->type()=="PageViewer") {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      for(unsigned int index=0;index<items.size();index++)
        ((NextStepTk::PageViewer*)widget)->addPopupEntry(items[index]);
      return true;
    }
*/
  } else if(aResource=="tabLabel") { //Constraint resource.
    element->setPropertyValue("tabLabel",aValue);
    return true;
  } else { // Viewer parameter ?
    if([widget isKindOfClass:[NSPageViewer class]]==YES) {
      NSString* what = [[NSString alloc] initWithCString:aResource.c_str()];
      NSString* value = [[NSString alloc] initWithCString:aValue.c_str()];
      BOOL status = [(NSPageViewer*)widget setParameter:what value:value];
      [what release];
      [value release];
      if(status==YES) return true;
    }
  }

  Lib::Out out(fSession.printer());
  out << "OnX::NextStepUI::setParameter :"
      << " unable to set parameter for : widget " << Lib::sout(aWidget)
      << " widget " << Lib::sout(aWidget)
      << " resource " << Lib::sout(aResource)
      << " value " << Lib::sout(aValue) << "."
      << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::write(
 const std::string& aWidget
,const std::string& //aWhat
,const std::string& //aFile
,const std::string& //aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)findNativeWidget(aWidget);
  if(!widget) return false;
  if([widget isKindOfClass:[NSPageViewer class]]==YES) {
    //FIXME return ((NextStepTk::PageViewer*)widget)->write(aWhat,aFile);
    //return true;
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::createTree(
 OnX_NSTreeItem* aTreeItem
,Lib::ItemML* aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::list<Lib::ItemML*>& list = aItemML->children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string slabel;
    (*it)->propertyValue("label",slabel);
    OnX_NSTreeItem* item = [[OnX_NSTreeItem alloc] initWithParent:aTreeItem];
    NSString* nsLabel = [[NSString alloc] initWithCString:slabel.c_str()];
    [item setLabel:nsLabel];
    [aTreeItem addObject:item];
    [item release];
    [nsLabel release];
    if(!createTree(item,*it)) return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::createNativeWidget(
 Lib::ItemML& aItemML                      
,const std::string& aClass
,const std::string& aName
,Lib::ItemML* aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
    int nextstepClass = getType(aClass);
    if(nextstepClass==0) {
      Lib::Out out(fSession.printer());
      out << "NextStepUI::createNativeWidget : unknown widget type : " 
          << aClass << Lib::endl;
      return false;
    }

    // Get parent :
    NSObject* nextStepParent = 0;
    NSObject* nextStepParentContainer = 0;
    int nextStepParentClass = 0;
    if(aParent) {
      //printf("debug : parent :%s|\n",parent->getValue("class"));
      nextStepParent = (NSObject*)aParent->getData1();
      nextStepParentContainer = (NSObject*)aParent->getData2();
      //FIXME nextStepParentClass = getType(nextStepParent->type());
    }

    std::string svalue;

    // What is changing :
    unsigned int autoResizeMask = NSViewWidthSizable | NSViewHeightSizable;

#define CHECK_PARENT \
      if(!nextStepParent) {\
        Lib::Out out(fSession.printer());\
        out << "NextStepUI::createNativeWidget : " << aClass \
            << " need a parent." << Lib::endl;\
        return false;\
      }  

    // Create the NextStepTk widget :
    NSObject* nextStepWidget = 0;
    NSObject* nextStepContainer = 0;
    if(nextstepClass==nextstep_Window) {
      NSRect srect = [[NSScreen mainScreen] visibleFrame];
      NSRect rect;
      float woffset = 100;
      float hoffset = 100;
      rect.origin.x = woffset;
      rect.origin.y = hoffset;
      rect.size.width = srect.size.width - 2 * woffset;
      rect.size.height = srect.size.height - hoffset;;
      if(aItemML.propertyValue("geometry",svalue)) {
        int x,y,w,h;
        if(!Lib::smanip::togeometry(svalue,x,y,w,h)) {
          Lib::Out out(fSession.printer());
          out << svalue << " : bad value for geometry." << Lib::endl;
        } else {
          rect.origin.x = x;
          // NS origin is lower-left corner, x toright, y toup.
          rect.origin.y = srect.size.height - h - y;
          rect.size.width = w;
          rect.size.height = h;
        }
      }
      unsigned int mask = 
        NSResizableWindowMask | 
        NSMiniaturizableWindowMask;
      NSWindow* window = [[NSWindow alloc] initWithContentRect:rect styleMask:mask backing:NSBackingStoreBuffered defer:NO]; //We are owner. 
      [window setShowsResizeIndicator:YES];
      [window setReleasedWhenClosed:NO];
      [window makeKeyAndOrderFront:fApplication];
      nextStepWidget = window;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_Dialog) {
      NSRect srect = [[NSScreen mainScreen] visibleFrame];
      NSRect rect;
      float woffset = 100;
      float hoffset = 100;
      rect.origin.x = woffset;
      rect.origin.y = hoffset;
      rect.size.width = srect.size.width - 2 * woffset;
      rect.size.height = srect.size.height - hoffset;;
      OnX_NSDialog* window = [[OnX_NSDialog alloc] initWithContentRect:rect];
      NSString* title = [[NSString alloc] initWithCString:"Dialog"];
      [window setTitle:title];
      [title release];
      nextStepWidget = window;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_WarningDialog) {
      NSAlert* alert = [[NSAlert alloc] init];
      nextStepWidget = alert;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_Label) {
      CHECK_PARENT
      bool bitmap = aItemML.propertyValue("pixmap",svalue);
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSButton* button = [[NSButton alloc] initWithFrame:rect];
      [button setBordered:NO];
      [button setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,button);
      nextStepWidget = button;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_PushButton) {
      CHECK_PARENT
      bool bitmap = aItemML.propertyValue("pixmap",svalue);
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSButton* button = [[OnX_NSButton alloc] initWithFrame:rect];
      [button setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,button);
      nextStepWidget = button;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_ToggleButton) {
      CHECK_PARENT
      bool bitmap = aItemML.propertyValue("pixmap",svalue);
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSToggleButton* button = [[OnX_NSToggleButton alloc] initWithFrame:rect];
      [button setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,button);
      nextStepWidget = button;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_VScrollBar) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      //OnX_NSScroller* scroller = [[OnX_NSScroller alloc] initWithFrame:rect];
      OnX_NSSlider* scroller = [[OnX_NSSlider alloc] initWithFrame:rect];
      [scroller setMinValue:0];
      [scroller setMaxValue:1000];
      [scroller setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,scroller);
      nextStepWidget = scroller;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_HScrollBar) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      //OnX_NSScroller* scroller = [[OnX_NSScroller alloc] initWithFrame:rect];
      OnX_NSSlider* scroller = [[OnX_NSSlider alloc] initWithFrame:rect];
      [scroller setMinValue:0];
      [scroller setMaxValue:1000];
      [scroller setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,scroller);
      nextStepWidget = scroller;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_VScale) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSSlider* slider = [[OnX_NSSlider alloc] initWithFrame:rect];
      [slider setMinValue:0];
      [slider setMaxValue:1000];
      [slider setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,slider);
      nextStepWidget = slider;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_HScale) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSSlider* slider = [[OnX_NSSlider alloc] initWithFrame:rect];
      //[slider rotateByAngle:-90];
      [slider setMinValue:0];
      [slider setMaxValue:1000];
      [slider setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,slider);
      nextStepWidget = slider;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_CommandLine) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSTextField* command = [[OnX_NSTextField alloc] initWithFrame:rect];
      [command setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,command);
      nextStepWidget = command;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_Password) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSSecureTextField* command = 
        [[OnX_NSSecureTextField alloc] initWithFrame:rect];
      [command setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,command);
      nextStepWidget = command;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_Entry) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSTextField* command = [[NSTextField alloc] initWithFrame:rect];
      [command setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,command);
      nextStepWidget = command;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_VBox) {
      CHECK_PARENT

      std::string sexpand;
      if(!aItemML.propertyValue("expand",sexpand)) sexpand = "second";
      OnX_Expand expand = OnX_SECOND;
      if(sexpand=="first") expand = OnX_FIRST;
      else if(sexpand=="both") expand = OnX_BOTH;

      int sizeOfFixed = -1;
      if(aItemML.propertyValue("sizeOfFixed",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          sizeOfFixed = value;
        }      
      } else if(aItemML.propertyValue("splitPosition",svalue)) { 
        //deprecated.
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget : "
            << " splitPosition resource deprecated for VBox." 
            << " Use sizeOfFixed." 
            << Lib::endl;
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          sizeOfFixed = value;
        }      
      }

      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSVBox* vbox = [[OnX_NSVBox alloc] initWithFrame:rect];
      [vbox setExpand:expand];
      [vbox setSizeOfFixed:sizeOfFixed];
      [vbox setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,vbox);
      nextStepWidget = vbox;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_HBox) {
      CHECK_PARENT

      std::string sexpand;
      if(!aItemML.propertyValue("expand",sexpand)) sexpand = "second";
      OnX_Expand expand = OnX_SECOND;
      if(sexpand=="first") expand = OnX_FIRST;
      else if(sexpand=="both") expand = OnX_BOTH;

      int sizeOfFixed = -1;
      if(aItemML.propertyValue("sizeOfFixed",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          sizeOfFixed = value;
        }      
      } else if(aItemML.propertyValue("splitPosition",svalue)) { 
        //deprecated.
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget : "
            << " splitPosition resource deprecated for HBox." 
            << " Use sizeOfFixed." 
            << Lib::endl;
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          sizeOfFixed = value;
        }      
      }

      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSHBox* hbox = [[OnX_NSHBox alloc] initWithFrame:rect];
      [hbox setExpand:expand];
      [hbox setSizeOfFixed:sizeOfFixed];
      [hbox setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,hbox);
      nextStepWidget = hbox;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_VPaned) {
      CHECK_PARENT
      int splitPosition = -1;
      if(aItemML.propertyValue("splitPosition",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          splitPosition = value;
        }      
      }
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSVPaned* vpaned = [[OnX_NSVPaned alloc] initWithFrame:rect];
      [vpaned setSplitPosition:splitPosition];
      [vpaned setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,vpaned);
      nextStepWidget = vpaned;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_HPaned) {
      CHECK_PARENT
      int splitPosition = -1;
      if(aItemML.propertyValue("splitPosition",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          splitPosition = value;
        }      
      }
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSHPaned* hpaned = [[OnX_NSHPaned alloc] initWithFrame:rect];
      [hpaned setSplitPosition:splitPosition];
      [hpaned setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,hpaned);
      nextStepWidget = hpaned;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_VContainer) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSVContainer* vcontainer = 
        [[OnX_NSVContainer alloc] initWithFrame:rect];
      [vcontainer setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,vcontainer);
      nextStepWidget = vcontainer;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_HContainer) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSHContainer* hcontainer = 
        [[OnX_NSHContainer alloc] initWithFrame:rect];
      [hcontainer setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,hcontainer);
      nextStepWidget = hcontainer;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_MenuBar) {
      //nextStepParent should be a NSWindow :
      if([nextStepParent isKindOfClass:[NSWindow class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSWindow." << Lib::endl;
        return false;
      }
      NSMenu* menuBar = [fApplication mainMenu];
      nextStepWidget = menuBar;
    } else if(nextstepClass==nextstep_CascadeButton) {
      CHECK_PARENT
      //nextStepParent should be a NSMenu (menubar) :
      if([nextStepParent isKindOfClass:[NSMenu class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSMenu." << Lib::endl;
        return false;
      }
      NSMenu* menuBar = (NSMenu*)nextStepParent;
      std::string slabel;
      aItemML.propertyValue("label",slabel);
      NSString* title = [[NSString alloc] initWithCString:slabel.c_str()];
      NSMenuItem* casc = 
        [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:title 
                            action:NULL  
                            keyEquivalent:@""]; 
      [title release];
      [menuBar addItem:casc];
      [casc release];
      nextStepWidget = casc;
    } else if(nextstepClass==nextstep_Menu) {
      CHECK_PARENT
      //nextStepParent should be a NSMenuItem (cascade).
      if([nextStepParent isKindOfClass:[NSMenuItem class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSMenuItem." << Lib::endl;
        return false;
      }
      NSMenuItem* casc = (NSMenuItem*)nextStepParent;
      NSMenu* pdm = [[NSMenu allocWithZone:[NSMenu menuZone]] init];
      [pdm setTitle:[casc title]];
      [casc setSubmenu:pdm];
      [pdm release];
      nextStepWidget = pdm;
    } else if(nextstepClass==nextstep_MenuItem) {
      CHECK_PARENT
      //nextStepParent should be a NSMenu (pdm) :
      if([nextStepParent isKindOfClass:[NSMenu class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSMenu." << Lib::endl;
        return false;
      }
      NSMenu* pdm = (NSMenu*)nextStepParent;
      std::string slabel;
      aItemML.propertyValue("label",slabel);
      NSString* title = [[NSString alloc] initWithCString:slabel.c_str()];
      OnX_NSMenuItem* item = 
        [[OnX_NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:title 
                            action:NULL  
                            keyEquivalent:@""]; 
      [title release];
      [pdm addItem:item];
      [item release];
      nextStepWidget = item;
    } else if(nextstepClass==nextstep_ToggleMenuItem) {
      CHECK_PARENT
      //nextStepParent should be a NSMenu (pdm) :
      if([nextStepParent isKindOfClass:[NSMenu class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSMenu." << Lib::endl;
        return false;
      }
      NSMenu* pdm = (NSMenu*)nextStepParent;
      std::string slabel;
      aItemML.propertyValue("label",slabel);
      NSString* title = [[NSString alloc] initWithCString:slabel.c_str()];
      OnX_NSToggleMenuItem* item = 
        [[OnX_NSToggleMenuItem allocWithZone:[NSMenu menuZone]] 
                               initWithTitle:title 
                                      action:NULL  
                               keyEquivalent:@""]; 
      [title release];
      [pdm addItem:item];
      [item release];
      nextStepWidget = item;
    } else if(nextstepClass==nextstep_MenuSeparator) {
      CHECK_PARENT
      NSMenu* pdm = (NSMenu*)nextStepParent;
      NSMenuItem* item = [NSMenuItem separatorItem];
      [pdm addItem:item];
      //class created [item release];
      nextStepWidget = item;
    } else if(nextstepClass==nextstep_OpenGLArea) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSOpenGLArea* openGL = [[OnX_NSOpenGLArea alloc] initWithFrame:rect];
      [openGL setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,openGL);
      nextStepWidget = openGL;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_PageViewer) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSPageViewer* nsPageViewer = [[NSPageViewer alloc] initWithFrame:rect];
      [[nsPageViewer pageView] setAutoresizingMask:autoResizeMask];
      addInParent(aItemML,nextStepParent,[nsPageViewer pageView]);
      nextStepWidget = nsPageViewer;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );

     {Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      setCurrentWidget(widget);}

      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :" 
            << " ItemML not a Slash::UI::ISoPageViewer." 
            << Lib::endl;
        return false;
      }
      SoNode* soNode = [(NSPageViewer*)nsPageViewer sceneGraph];
      soViewer->setSceneGraph(soNode);
    } else if(nextstepClass==nextstep_FileSelectionDialog) {
      CHECK_PARENT
      OnX_NSFileChooser* panel = [[OnX_NSFileChooser alloc] init];
      nextStepWidget = panel;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_ColorSelectionDialog) {
      CHECK_PARENT
      OnX_NSColorChooser* panel = [[OnX_NSColorChooser alloc] init];
      nextStepWidget = panel;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_ScrolledList) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSScrollView* scrollView = [[NSScrollView alloc] initWithFrame:rect];
      [scrollView setHasVerticalScroller:YES];
      [scrollView setAutoresizingMask:autoResizeMask]; //Before addInParent.

      OnX_NSList* list = [[OnX_NSList alloc] initWithFrame:rect];
      [list setAutoresizingMask:autoResizeMask]; //Before addInParent.
      [scrollView setDocumentView:list];
      [list release];
      addInParent(aItemML,nextStepParent,scrollView);
      nextStepWidget = list;
      nextStepContainer = scrollView;
      fPool.push_back( std::pair<NSObject*,std::string>(scrollView,"") );
    } else if(nextstepClass==nextstep_ScrolledText) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSScrollView* scrollView = [[NSScrollView alloc] initWithFrame:rect];
      [scrollView setHasVerticalScroller:YES];
      [scrollView setAutoresizingMask:autoResizeMask]; //Before addInParent.
      NSText* text = [[NSText alloc] initWithFrame:rect];
      [scrollView setDocumentView:text];
      [text release];
      addInParent(aItemML,nextStepParent,scrollView);
      nextStepWidget = text;
      nextStepContainer = scrollView;
      fPool.push_back( std::pair<NSObject*,std::string>(scrollView,"") );
    } else if(nextstepClass==nextstep_ScrolledTree) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSScrollView* scrollView = [[NSScrollView alloc] initWithFrame:rect];
      [scrollView setHasVerticalScroller:YES];
      [scrollView setAutoresizingMask:autoResizeMask]; //Before addInParent.

      OnX_NSTree* tree = [[OnX_NSTree alloc] initWithFrame:rect];
      [tree setAutoresizingMask:autoResizeMask]; //Before addInParent.
      [scrollView setDocumentView:tree];
      [tree release];
      addInParent(aItemML,nextStepParent,scrollView);
      nextStepWidget = tree;
      nextStepContainer = scrollView;
      fPool.push_back( std::pair<NSObject*,std::string>(scrollView,"") );
    } else if(nextstepClass==nextstep_ToolBar) {
      //nextStepParent should be a NSWindow :
      if([nextStepParent isKindOfClass:[NSWindow class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSWindow." << Lib::endl;
        return false;
      }
      NSWindow* window = (NSWindow*)nextStepParent;        
      if(!window) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :" 
            << " " << aClass << " : NSWindow not found." << Lib::endl;
        return false;
      }  
      NSString* identifier = 
        [[NSString alloc] initWithCString:aName.c_str()];
      OnX_NSToolbar* toolbar = 
        [[OnX_NSToolbar alloc] initWithIdentifier:identifier];
      [identifier release];
      [toolbar setDisplayMode:NSToolbarDisplayModeIconAndLabel];

      [window setToolbar:toolbar];
      nextStepWidget = toolbar;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_ToolBarItem) {
      CHECK_PARENT
      //nextStepParent should be a NSToolbar :
      if([nextStepParent isKindOfClass:[OnX_NSToolbar class]]==NO) {
        Lib::Out out(fSession.printer());
        out << "NextStepUI::createNativeWidget :"
            << " " << aClass << " : parent not a NSToolbar." << Lib::endl;
        return false;
      }
      OnX_NSToolbar* toolbar = (OnX_NSToolbar*)nextStepParent;

      std::string spixmap;
      aItemML.propertyValue("pixmap",spixmap);
      std::string shelp;
      aItemML.propertyValue("tooltip",shelp);
      std::string slabel;
      aItemML.propertyValue("label",slabel);

      std::string sspixmap = Lib::fmanip::name(spixmap);
      //FIXME : NSImage does not understand .xpm files.
      Lib::smanip::replace(sspixmap,".xpm",".gif");

      NSString* identifier = 
        [[NSString alloc] initWithCString:aName.c_str()];
      NSString* label = [[NSString alloc] initWithCString:slabel.c_str()];
      NSString* pixmap = [[NSString alloc] initWithCString:sspixmap.c_str()];
      NSString* toolTip = [[NSString alloc] initWithCString:shelp.c_str()];

      OnX_NSToolbarItem* toolbarItem = 
        [[OnX_NSToolbarItem alloc] initWithItemIdentifier:identifier];
      [toolbarItem autorelease];

      [toolbarItem setLabel:label];

      [toolbarItem setToolTip:toolTip];

      //[toolbarItem setPaletteLabel:label];
      //FIXME : handle .xpm (.png, .jpg, .gif ok).

      NSImage* image = [[NSImage alloc] initWithContentsOfFile:pixmap];
      [toolbarItem setImage:image];
      [image release]; 

      // Put in parent :
      [toolbar addToolbarItem:toolbarItem];

      unsigned int number = [[toolbar items] count];
      [toolbar insertItemWithItemIdentifier:identifier atIndex:number];

      [identifier release];
      [label release];
      [pixmap release];
      [toolTip release];

      nextStepWidget = toolbarItem;
    } else if(nextstepClass==nextstep_ComboBox) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSComboBox* comboBox = [[OnX_NSComboBox alloc] initWithFrame:rect];
      [comboBox setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,comboBox);
      nextStepWidget = comboBox;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_OptionMenu) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSPopUpButton* optionMenu = 
        [[OnX_NSPopUpButton alloc] initWithFrame:rect];
      [optionMenu setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,optionMenu);
      nextStepWidget = optionMenu;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_ProgressBar) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      NSProgressIndicator* progress = 
        [[NSProgressIndicator alloc] initWithFrame:rect];
      [progress setAutoresizingMask:autoResizeMask]; //Before addInParent.
      [progress setIndeterminate:NO];
      [progress setStyle:NSProgressIndicatorBarStyle];
      addInParent(aItemML,nextStepParent,progress);
      nextStepWidget = progress;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else if(nextstepClass==nextstep_TabStack) {
      CHECK_PARENT
      NSRect rect = getFrame(nextStepParent);
      rect.origin.x = 0;
      rect.origin.y = 0;
      OnX_NSTabView* tabView = [[OnX_NSTabView alloc] initWithFrame:rect];
      [tabView setAutoresizingMask:autoResizeMask]; //Before addInParent.
      addInParent(aItemML,nextStepParent,tabView);
      nextStepWidget = tabView;
      fPool.push_back( std::pair<NSObject*,std::string>(nextStepWidget,aName) );
    } else {
      Lib::Out out(fSession.printer());
      out << "NextStepUI::createNativeWidget :" << aClass 
          << " unknown case." << Lib::endl;
      return false;
    }
    
    //NOTE : NS widgets has no name. It is handled in fPool.
    aItemML.setData(nextStepWidget,nextStepContainer?nextStepContainer:nextStepWidget,0);

    // Set resources :
    if(nextstepClass==nextstep_Window) {
      if(aItemML.propertyValue("title",svalue)) {
        NSString* title = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSWindow*)nextStepWidget setTitle:title];
        [title release];
      }
    } else if(nextstepClass==nextstep_Dialog)  {
      if(aItemML.propertyValue("geometry",svalue)) {
        int w,h;
        if(sscanf(svalue.c_str(),"%dx%d",&w,&h)!=2) {
          Lib::Out out(fSession.printer());
          out << svalue << " : bad value for geometry." << Lib::endl;
        } else {
          NSWindow* window = (NSWindow*)nextStepWidget;
          NSSize size;
          size.width = w;
          size.height = h;
          [window setContentSize:size];
        }
      }
      if(aItemML.propertyValue("title",svalue)) {
        NSString* title = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSWindow*)nextStepWidget setTitle:title];
        [title release];
      }
    } else if(nextstepClass==nextstep_Label) {
      if(aItemML.propertyValue("label",svalue) ||
         aItemML.propertyValue("labelString",svalue) ) {
        NSString* title = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSButton*)nextStepWidget setTitle:title];
        [title release];
      }
    } else if(nextstepClass==nextstep_PushButton) {
      if(aItemML.propertyValue("label",svalue) ||
         aItemML.propertyValue("labelString",svalue) ) {
        NSString* title = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSButton*)nextStepWidget setTitle:title];
        [title release];
      }
      if(aItemML.propertyValue("pixmap",svalue)) {
        //FIXME ((NextStepTk::Button*)nextStepWidget)->setPixmap(svalue);
      }
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_ToggleButton) {
      if(aItemML.propertyValue("label",svalue)) {
        NSString* title = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSButton*)nextStepWidget setTitle:title];
        [title release];
      }
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_HScrollBar) {
      addCallbacks(aItemML,nextStepWidget,"drag");
    } else if(nextstepClass==nextstep_VScrollBar) {
      addCallbacks(aItemML,nextStepWidget,"drag");
    } else if(nextstepClass==nextstep_HScale) {
      addCallbacks(aItemML,nextStepWidget,"drag");
    } else if(nextstepClass==nextstep_VScale) {
      addCallbacks(aItemML,nextStepWidget,"drag");
    } else if(nextstepClass==nextstep_ComboBox) {
      std::string def;
      aItemML.propertyValue("default",def);
      OnX_NSComboBox* comboBox = (OnX_NSComboBox*)nextStepWidget;
      NSString* nss = [[NSString alloc] initWithCString:def.c_str()];
      [comboBox setStringValue:nss];
      [nss release];
      if(aItemML.propertyValue("items",svalue)) {
        [comboBox removeAllItems];
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++) {
          NSString* nsitem = 
            [[NSString alloc] initWithCString:items[index].c_str()];
          [comboBox addItemWithObjectValue:nsitem];
          [nsitem release];
        }
      }
      addCallbacks(aItemML,nextStepWidget,"valueChanged");
    } else if(nextstepClass==nextstep_OptionMenu) {
      if(aItemML.propertyValue("label",svalue)) {
      }
      std::string def;
      aItemML.propertyValue("default",def); //FIXME : handle def
      if(aItemML.propertyValue("items",svalue)) {
        OnX_NSPopUpButton* optionMenu = (OnX_NSPopUpButton*)nextStepWidget;
        [optionMenu removeAllItems];
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++) {
          NSString* nsitem = 
            [[NSString alloc] initWithCString:items[index].c_str()];
          [optionMenu addItemWithTitle:nsitem];
          [nsitem release];
        }
      }
      addCallbacks(aItemML,nextStepWidget,"valueChanged");
    } else if(nextstepClass==nextstep_ScrolledList) {
      std::string def;
      aItemML.propertyValue("selection",def); //FIXME : handle
      if(aItemML.propertyValue("items",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        OnX_NSTreeItem* rootItem = [[OnX_NSTreeItem alloc] initWithParent:0];
        for(unsigned int index=0;index<items.size();index++) {
          OnX_NSTreeItem* item = 
            [[OnX_NSTreeItem alloc] initWithParent:rootItem];
          NSString* nsLabel = 
            [[NSString alloc] initWithCString:items[index].c_str()];
          [item setLabel:nsLabel];
          [rootItem addObject:item];
          [item release];
          [nsLabel release];
        }
        [(OnX_NSList*)nextStepWidget setRootItem:rootItem];
      }
      addCallbacks(aItemML,nextStepWidget,"select");
    } else if(nextstepClass==nextstep_ScrolledText) {
      if(aItemML.propertyValue("value",svalue)) {
        NSString* nss = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSText*)nextStepWidget setString:nss];
        [nss release];
      }
    } else if(nextstepClass==nextstep_Entry) {
      if(aItemML.propertyValue("value",svalue)) {
        NSString* nss = [[NSString alloc] initWithCString:svalue.c_str()];
        [(NSTextField*)nextStepWidget setStringValue:nss];
        [nss release];
      }
    } else if(nextstepClass==nextstep_ScrolledTree) {
      addCallbacks(aItemML,nextStepWidget,"select");
    } else if(nextstepClass==nextstep_ToolBarItem) {
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_CommandLine) {
      addCallbacks(aItemML,nextStepWidget,"activate");
      addCallbacks(aItemML,nextStepWidget,"complete");
    } else if(nextstepClass==nextstep_Password) {
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_TabStack) {
      addCallbacks(aItemML,nextStepWidget,"select");
    } else if(nextstepClass==nextstep_MenuItem) {
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_ToggleMenuItem) {
      addCallbacks(aItemML,nextStepWidget,"activate");
    } else if(nextstepClass==nextstep_OpenGLArea) {
      addCallbacks(aItemML,nextStepWidget,"paint");
      addCallbacks(aItemML,nextStepWidget,"event");
    } else if(nextstepClass==nextstep_PageViewer) {

/*FIXME
      if(aItemML.propertyValue("popupItems",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++)
          ((NextStepTk::PageViewer*)nextStepWidget)->addPopupEntry(items[index]);
      }

      ((NextStepTk::PageViewer*)nextStepWidget)->
        setVerboseLevel(fVerboseLevel);

      addCallbacks(aItemML,nextStepWidget,"collect");
      addCallbacks(aItemML,nextStepWidget,"popup");
      addCallbacks(aItemML,nextStepWidget,"update");

*/
      NSString* nss = [[NSString alloc] initWithCString:aName.c_str()];
      OnX_NSDestroyAction* act = 
        [[OnX_NSDestroyAction alloc] initWithUI:this andViewer:nss];
      fPool.push_back( std::pair<NSObject*,std::string>(act,"destroyAction") );
      [nss release];
      [(NSPageViewer*)nextStepWidget addDestroyAction:act];

    } else if(nextstepClass==nextstep_FileSelectionDialog) {
      addCallbacks(aItemML,nextStepWidget,"ok");
      addCallbacks(aItemML,nextStepWidget,"cancel");

/*FIXME
      if(aItemML.propertyValue("dirMask",svalue)) {
        Lib::smanip::getenv(svalue);
        ((NextStepTk::FileSelectionDialog*)nextStepWidget)->setDirMask(svalue);
      }

      if(aItemML.propertyValue("directory",svalue)) {
        Lib::smanip::getenv(svalue);
        ((NextStepTk::FileSelectionDialog*)nextStepWidget)->setDirectory(svalue);
      }

*/
    } else if(nextstepClass==nextstep_ColorSelectionDialog) {
      addCallbacks(aItemML,nextStepWidget,"ok");
      addCallbacks(aItemML,nextStepWidget,"cancel");
    }

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "NextStepUI::createNativeWidget :" 
        << " class " << Lib::sout(aClass)
        << " name " << Lib::sout(aName)
        << " done." << Lib::endl;
  }

  return createChildren(aItemML);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::destroyWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Destroy sub NextStepTk::Component :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!destroyWidget(*(*it))) return false;
  }
  // Destroy this component :
  return destroyOneWidget(aItemML);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::destroyOneWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)aItemML.getData1();
  if(widget) {
    if(fVerboseLevel) {
      std::string name;
      findName(widget,name);
      Lib::Out out(fSession.printer());
      out << "NextStepUI::destroyWidget :"
          << " destroy widget with name " << Lib::sout(name) << "..." 
          << Lib::endl;
    }
    removeActions(widget);
    if([widget isKindOfClass:[NSMenuItem class]]==YES) {
      [[(NSMenuItem*)widget menu] removeItem:widget];
    }
    removeInPool(widget);
  }
  NSObject* widgetContainer = (NSObject*)aItemML.getData2();
  if(widgetContainer && (widgetContainer!=widget) ) {
    if(fVerboseLevel) {
      std::string name;
      findName(widgetContainer,name);
      Lib::Out out(fSession.printer());
      out << "NextStepUI::destroyWidget :"
          << " destroy widget with name " << Lib::sout(name) << "..." 
          << Lib::endl;
    }
    removeActions(widgetContainer);
    removeInPool(widgetContainer);
  }
  aItemML.setData(0,0,0);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::deleteCallback(
 Lib::ItemML& /*aItemML */
,const std::string& /*aNameXML*/
,const Action& /*aAction*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : to implement.
  Lib::Out out(fSession.printer());
  out << "NextStepUI::deleteCallback :"
      << " dummy."
      << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::NextStepUI::addCallback(
 Lib::ItemML& aItemML
,const std::string& aName
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : not yet tested.

  NSObject* aWidget = (NSObject*)aItemML.getData1();
  if(!aWidget) return 0;

  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);

  Action& action = addAction(*this,widget,aWidget,aInterp,aScript,aName);

  OnX_NSCallback* cbk = [[OnX_NSCallback alloc] initWithAction:&action];

  if([aWidget isKindOfClass:[NSMenuItem class]]==YES) {
    OnX_NSMenuItem* widget = (OnX_NSMenuItem*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[NSButton class]]==YES) {
    OnX_NSButton* widget = (OnX_NSButton*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[NSToolbarItem class]]==YES) {
    OnX_NSToolbarItem* widget = (OnX_NSToolbarItem*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSOpenGLArea class]]==YES) {
    OnX_NSOpenGLArea* widget = (OnX_NSOpenGLArea*)aWidget;
    if(aName=="paint") {
      [widget addPaintAction:cbk];
    } else if(aName=="event") {
      [widget addEventAction:cbk];
    }
  } else if([aWidget isKindOfClass:[OnX_NSFileChooser class]]==YES) {
    OnX_NSFileChooser* widget = (OnX_NSFileChooser*)aWidget;
    if(aName=="ok") {
      [widget addOkAction:cbk];
    } else if(aName=="cancel") {
      [widget addCancelAction:cbk];
    }
  } else if([aWidget isKindOfClass:[OnX_NSColorChooser class]]==YES) {
    OnX_NSColorChooser* widget = (OnX_NSColorChooser*)aWidget;
    if(aName=="ok") {
      [widget addOkAction:cbk];
    } else if(aName=="cancel") {
    }
  } else if([aWidget isKindOfClass:[OnX_NSTextField class]]==YES) {
    OnX_NSTextField* widget = (OnX_NSTextField*)aWidget;
    if(aName=="activate") {
      [widget addActivateAction:cbk];
    } else if(aName=="complete") {
      [widget addCompleteAction:cbk];
    }
  } else if([aWidget isKindOfClass:[OnX_NSTree class]]==YES) {
    OnX_NSTree* widget = (OnX_NSTree*)aWidget;
    [widget addSelectAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSComboBox class]]==YES) {
    OnX_NSComboBox* widget = (OnX_NSComboBox*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSSecureTextField class]]==YES) {
    OnX_NSSecureTextField* widget = (OnX_NSSecureTextField*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSSlider class]]==YES) {
    OnX_NSSlider* widget = (OnX_NSSlider*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSScroller class]]==YES) {
    OnX_NSScroller* widget = (OnX_NSScroller*)aWidget;
    [widget addActivateAction:cbk];
  } else if([aWidget isKindOfClass:[OnX_NSTabView class]]==YES) {
    OnX_NSTabView* widget = (OnX_NSTabView*)aWidget;
    [widget addSelectAction:cbk];
  }

  [cbk release];
  return &action;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::addCallbacks(
 Lib::ItemML& aItemML
,NSObject* aWidget
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  const std::list<Lib::Element*> props = aItemML.properties();
  //bool found = false;
  std::list<Lib::Element*>::const_iterator it;
  for(it=props.begin();it!=props.end();++it) {
    if(aName==(*it)->name()) {
      std::string sexec;
      (*it)->attributeValue("exec",sexec);
      Action& action = 
        addAction(*this,widget,aWidget,sexec,(*it)->value(),aName);

      OnX_NSCallback* cbk = [[OnX_NSCallback alloc] initWithAction:&action];

      if([aWidget isKindOfClass:[OnX_NSMenuItem class]]==YES) {
        OnX_NSMenuItem* widget = (OnX_NSMenuItem*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSButton class]]==YES) {
        OnX_NSButton* widget = (OnX_NSButton*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSPopUpButton class]]==YES) {
        OnX_NSPopUpButton* widget = (OnX_NSPopUpButton*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSToolbarItem class]]==YES) {
        OnX_NSToolbarItem* widget = (OnX_NSToolbarItem*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSOpenGLArea class]]==YES) {
        OnX_NSOpenGLArea* widget = (OnX_NSOpenGLArea*)aWidget;
        if(aName=="paint") {
          [widget addPaintAction:cbk];
        } else if(aName=="event") {
          [widget addEventAction:cbk];
        }
      } else if([aWidget isKindOfClass:[OnX_NSFileChooser class]]==YES) {
        OnX_NSFileChooser* widget = (OnX_NSFileChooser*)aWidget;
        if(aName=="ok") {
          [widget addOkAction:cbk];
        } else if(aName=="cancel") {
          [widget addCancelAction:cbk];
        }
      } else if([aWidget isKindOfClass:[OnX_NSColorChooser class]]==YES) {
        OnX_NSColorChooser* widget = (OnX_NSColorChooser*)aWidget;
        if(aName=="ok") {
          [widget addOkAction:cbk];
        } else if(aName=="cancel") {
        }
      } else if([aWidget isKindOfClass:[OnX_NSTextField class]]==YES) {
        OnX_NSTextField* widget = (OnX_NSTextField*)aWidget;
        if(aName=="activate") {
          [widget addActivateAction:cbk];
        } else if(aName=="complete") {
          [widget addCompleteAction:cbk];
        }
      } else if([aWidget isKindOfClass:[OnX_NSTree class]]==YES) {
        OnX_NSTree* widget = (OnX_NSTree*)aWidget;
        [widget addSelectAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSList class]]==YES) {
        OnX_NSList* widget = (OnX_NSList*)aWidget;
        [widget addSelectAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSComboBox class]]==YES) {
        OnX_NSComboBox* widget = (OnX_NSComboBox*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSSecureTextField class]]==YES) {
        OnX_NSSecureTextField* widget = (OnX_NSSecureTextField*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSSlider class]]==YES) {
        OnX_NSSlider* widget = (OnX_NSSlider*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSScroller class]]==YES) {
        OnX_NSScroller* widget = (OnX_NSScroller*)aWidget;
        [widget addActivateAction:cbk];
      } else if([aWidget isKindOfClass:[OnX_NSTabView class]]==YES) {
        OnX_NSTabView* widget = (OnX_NSTabView*)aWidget;
        [widget addSelectAction:cbk];
      }

      [cbk release];

      //found = true;
    }
  }
/*
  if(!found) {
    //  Create a callback even if no activate property given
    // (someone may want to add some action by program later).
    Action& action = addAction(*this,aWidget,"","",aName);

    OnX_NSCallback* cbk = [[OnX_NSCallback alloc] initWithAction:&action];

    if([aWidget isKindOfClass:[NSMenuItem class]]==YES) {
      OnX_NSMenuItem* widget = (OnX_NSMenuItem*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[NSButton class]]==YES) {
      OnX_NSButton* widget = (OnX_NSButton*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[NSToolbarItem class]]==YES) {
      OnX_NSToolbarItem* widget = (OnX_NSToolbarItem*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSOpenGLArea class]]==YES) {
      OnX_NSOpenGLArea* widget = (OnX_NSOpenGLArea*)aWidget;
      if(aName=="paint") {
        [widget addPaintAction:cbk];
      } else if(aName=="event") {
        [widget addEventAction:cbk];
      }
    } else if([aWidget isKindOfClass:[OnX_NSFileChooser class]]==YES) {
      OnX_NSFileChooser* widget = (OnX_NSFileChooser*)aWidget;
      if(aName=="ok") {
        [widget addOkAction:cbk];
      } else if(aName=="cancel") {
        [widget addCancelAction:cbk];
      }
    } else if([aWidget isKindOfClass:[OnX_NSColorChooser class]]==YES) {
      OnX_NSColorChooser* widget = (OnX_NSColorChooser*)aWidget;
      if(aName=="ok") {
        [widget addOkAction:cbk];
      } else if(aName=="cancel") {
      }
    } else if([aWidget isKindOfClass:[OnX_NSTextField class]]==YES) {
      OnX_NSTextField* widget = (OnX_NSTextField*)aWidget;
      if(aName=="activate") {
        [widget addActivateAction:cbk];
      } else if(aName=="complete") {
        [widget addCompleteAction:cbk];
      }
    } else if([aWidget isKindOfClass:[OnX_NSTree class]]==YES) {
      OnX_NSTree* widget = (OnX_NSTree*)aWidget;
      [widget addSelectAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSComboBox class]]==YES) {
      OnX_NSComboBox* widget = (OnX_NSComboBox*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSSecureTextField class]]==YES) {
      OnX_NSSecureTextField* widget = (OnX_NSSecureTextField*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSSlider class]]==YES) {
      OnX_NSSlider* widget = (OnX_NSSlider*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSScroller class]]==YES) {
      OnX_NSScroller* widget = (OnX_NSScroller*)aWidget;
      [widget addActivateAction:cbk];
    } else if([aWidget isKindOfClass:[OnX_NSTabView class]]==YES) {
      OnX_NSTabView* widget = (OnX_NSTabView*)aWidget;
      [widget addSelectAction:cbk];
    }

    [cbk release];
  }
*/
}
//////////////////////////////////////////////////////////////////////////////
int OnX::NextStepUI::getType(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if(aName=="Window")         return nextstep_Window;
  else if(aName=="VBox")           return nextstep_VBox;
  else if(aName=="HBox")           return nextstep_HBox;
  else if(aName=="VPaned")         return nextstep_VPaned;
  else if(aName=="HPaned")         return nextstep_HPaned;
  else if(aName=="MenuBar")        return nextstep_MenuBar;
  else if(aName=="CascadeButton")  return nextstep_CascadeButton;
  else if(aName=="Menu")           return nextstep_Menu;
  else if(aName=="MenuItem")       return nextstep_MenuItem;
  else if(aName=="ToggleMenuItem") return nextstep_ToggleMenuItem;
  else if(aName=="Label")          return nextstep_Label;
  else if(aName=="PushButton")     return nextstep_PushButton;
  else if(aName=="ToggleButton")   return nextstep_ToggleButton;
  else if(aName=="CommandLine")    return nextstep_CommandLine;
  else if(aName=="PageViewer")     return nextstep_PageViewer;
  else if(aName=="ScrolledList")   return nextstep_ScrolledList;
  else if(aName=="ScrolledText")   return nextstep_ScrolledText;
  else if(aName=="Dialog")         return nextstep_Dialog;
  else if(aName=="VContainer")     return nextstep_VContainer;
  else if(aName=="HContainer")     return nextstep_HContainer;
  else if(aName=="Entry")          return nextstep_Entry;
  else if(aName=="ComboBox")       return nextstep_ComboBox;
  else if(aName=="VScrollBar")     return nextstep_VScrollBar;
  else if(aName=="HScrollBar")     return nextstep_HScrollBar;
  else if(aName=="VScale")         return nextstep_VScale;
  else if(aName=="HScale")         return nextstep_HScale;
  else if(aName=="ToolBar")        return nextstep_ToolBar;
  else if(aName=="ToolBarItem")    return nextstep_ToolBarItem;
  else if(aName=="MenuSeparator")  return nextstep_MenuSeparator;
  else if(aName=="ProgressBar")    return nextstep_ProgressBar;
  else if(aName=="TabStack")       return nextstep_TabStack;
  else if(aName=="OptionMenu")     return nextstep_OptionMenu;
  else if(aName=="OpenGLArea")     return nextstep_OpenGLArea;

  else if(aName=="Password")       return nextstep_Password;
  else if(aName=="ScrolledTree")   return nextstep_ScrolledTree;
  else if(aName=="FileSelection")  return nextstep_FileSelectionDialog;
  else if(aName=="ColorSelection") return nextstep_ColorSelectionDialog;
  else if(aName=="WarningDialog")  return nextstep_WarningDialog;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::callback(
 Action& aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NextStepUI* This = Lib_Cast(aAction.ui(),OnX::NextStepUI);
  if(!This) return;

  std::string svalue;
  std::string sevent;
  int cbkx = 0;
  int cbky = 0;


  NSObject* component = (NSObject*)aAction.nativeWidget();
  if([component isKindOfClass:[OnX_NSFileChooser class]]==YES) {
    NSString* fileName = [(OnX_NSFileChooser*)component fileName];
    svalue = fileName ? std::string([fileName cString]) : "";
  } else if([component isKindOfClass:[OnX_NSTree class]]==YES) {
    NSString* selection = [(OnX_NSTree*)component selectedPath];
    if(!selection) return; //expansion ; not a leaf.
    svalue = std::string([selection cString]);
  } else if([component isKindOfClass:[OnX_NSList class]]==YES) {
    NSString* selection = [(OnX_NSList*)component selection];
    if(!selection) return; //expansion ; not a leaf.
    svalue = std::string([selection cString]);
  } else if([component isKindOfClass:[OnX_NSColorChooser class]]==YES) {
    NSString* selection = [(OnX_NSColorChooser*)component choosenColor];
    if(!selection) return;
    svalue = std::string([selection cString]);
  } else if([component isKindOfClass:[OnX_NSOpenGLArea class]]==YES) {
    NSEventType eventType = [(OnX_NSOpenGLArea*)component eventType];
    NSPoint location = [(OnX_NSOpenGLArea*)component eventLocation];
    if(eventType==NSLeftMouseDown) {
      sevent = "ButtonPress";
      cbkx = (int)location.x;
      cbky = (int)location.y;
    } else if(eventType==NSLeftMouseUp) {
      sevent = "ButtonRelease";
      cbkx = (int)location.x;
      cbky = (int)location.y;
    } else if(eventType==NSMouseMoved) {
      sevent = "MotionNotify";
      cbkx = (int)location.x;
      cbky = (int)location.y;
    }
  } else if([component isKindOfClass:[OnX_NSToggleMenuItem class]]==YES) {
    int state = [(NSMenuItem*)component state];
    svalue = state==NSOnState ? "true" : "false";
  } else if([component isKindOfClass:[OnX_NSToggleButton class]]==YES) {
    int state = [(NSButton*)component state];
    svalue = state==NSOnState ? "true" : "false";
  } else if([component isKindOfClass:[NSPopUpButton class]]==YES) {
    NSMenuItem* menuItem = 
      (NSMenuItem*)[(NSPopUpButton*)component selectedItem];
    NSString* title = [menuItem title];
    svalue = std::string([title cString]);
  } else if([component isKindOfClass:[NSTextField class]]==YES) {
    // Entry, CommandLine and Password
    NSString* value = [(NSTextField*)component stringValue];
    svalue = std::string([value cString]);
  } else if([component isKindOfClass:[NSSecureTextField class]]==YES) {
    // Entry, CommandLine and Password
    NSString* value = [(NSTextField*)component stringValue];
    svalue = std::string([value cString]);
  } else if([component isKindOfClass:[NSComboBox class]]==YES) {
    NSString* value = [(NSTextField*)component stringValue];
    svalue = std::string([value cString]);
  } else if([component isKindOfClass:[NSSlider class]]==YES) {
    double value  = [(NSSlider*)component doubleValue];
    Lib::smanip::printf(svalue,64,"%d",(int)value);
  } else if([component isKindOfClass:[NSScroller class]]==YES) {
    double value  = [(NSScroller*)component doubleValue];
    Lib::smanip::printf(svalue,64,"%d",(int)value);
  } else if([component isKindOfClass:[NSTabView class]]==YES) {
    NSTabViewItem* item = [(NSTabView*)component selectedTabViewItem];
    if(item) {
      NSString* label = [item label];
      svalue = label ? std::string([label cString]) : "";
    }
  } else if([component isKindOfClass:[NSPageViewer class]]==YES) {
/*FIXME
    svalue = aData.value;
    cbkx = aData.x;
    cbky = aData.y;
*/
  }

  if(This->session().verboseLevel()) {
    Lib::Out out(This->session().printer());
    out << "OnX::NextStepUI::callback :"
        << " value " << Lib::sout(svalue) << "."        << Lib::endl;
  }

  std::string name;
  This->findName(component,name);
  This->fCallbackData = 
    CallbackData(aAction.widget(),name,svalue,sevent,cbkx,cbky);

  // Execute script :
  aAction.execute(std::vector<std::string>());

  This->fCallbackData = CallbackData();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::showAll(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)aItemML.getData1();
  if(widget) {
    // If show property not here, we show the panel.
    // If show is here and is not "no", we show the panel.
    std::string sshow;
    if( !aItemML.propertyValue("show",sshow) || (sshow!="no") ) {
      if(fVerboseLevel) {
        Lib::Out out(fSession.printer());
        out << "OnX::NextStepUI::showAll :"
            << " show " << Lib::sout(widgetName(aItemML)) << "..."
            << Lib::endl;
      }
      //FIXME widget->show();
      if(fVerboseLevel) {
        Lib::Out out(fSession.printer());
        out << "OnX::NextStepUI::showAll :"
            << " show " << Lib::sout(widgetName(aItemML)) << " done."
            << Lib::endl;
      }
    }
  } else {
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) showAll(*(*it));
  }
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::ICyclic* OnX::NextStepUI::createCyclic(
 const std::string& aName
,int aDelay
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = new NextStepCyclic(fSession,aName,aDelay,aInterp,aScript);
  fCyclics.push_back(cyclic);
  return cyclic;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::NextStepUI::widgetName(
 Lib::ItemML& aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  NSObject* widget = (NSObject*)aItemML.getData1();
  if(!widget) return "";
  std::string name;
  if(!findName(widget,name)) return "";
  return name;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OnX::NextStepUI::findName(
 NSObject* aObject
,std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list< std::pair<NSObject*,std::string> >::iterator it;
  for(it=fPool.begin();it!=fPool.end();++it) {
    if((*it).first==aObject) {
      aName = (*it).second;
      return true;
    }
  }
  aName = "";
  return false;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::addInParent(
 Lib::ItemML& aItemML
,NSObject* aParent
,NSObject* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( ([aParent isKindOfClass:[NSWindow class]]==YES) &&
      ([aObject isKindOfClass:[NSView class]]==YES) ) {
    NSView* parentView = [(NSWindow*)aParent contentView];
    [parentView addSubview:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSVBox class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSVBox*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSHBox class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSHBox*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSVContainer class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSVContainer*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSHContainer class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSHContainer*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSVPaned class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSVPaned*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[OnX_NSHPaned class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(OnX_NSHPaned*)aParent addView:(NSView*)aObject];
  } else if( ([aParent isKindOfClass:[NSTabView class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    std::string svalue;
    aItemML.propertyValue("tabLabel",svalue);
    NSString* nss = [[NSString alloc] initWithCString:svalue.c_str()];
    NSTabViewItem* tabViewItem = [[NSTabViewItem alloc] init];    
    [tabViewItem setView:aObject];
    [tabViewItem setLabel:nss];
    [(NSTabView*)aParent addTabViewItem:tabViewItem];
    [nss release];
  } else if( ([aParent isKindOfClass:[NSView class]]==YES) &&
             ([aObject isKindOfClass:[NSView class]]==YES) ) {
    [(NSView*)aParent addSubview:(NSView*)aObject];
  }
}
//////////////////////////////////////////////////////////////////////////////
NSRect OnX::NextStepUI::getFrame(
 NSObject* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if([aObject isKindOfClass:[NSWindow class]]==YES) {
    NSRect rect = [[(NSWindow*)aObject contentView] frame];
    //printf("debug : getFrame Window rec %g %g %g %g\n",
      //rect.origin.x,rect.origin.y,rect.size.width,rect.size.height);
    return rect;
  } else if([aObject isKindOfClass:[NSPageViewer class]]==YES) {
    NSRect rect = [[(NSPageViewer*)aObject pageView] frame];
    return rect;
  } else if([aObject isKindOfClass:[NSView class]]==YES) {
    NSRect rect = [(NSView*)aObject frame];
    return rect;
  } else {
    NSRect rect = {0,0,0,0};
    return rect;
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::removeInPool(
 NSObject* aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::list< std::pair<NSObject*,std::string> >::iterator it;
  for(it=fPool.begin();it!=fPool.end();) {
    if((*it).first==aObject) {
      it = fPool.erase(it);
      [aObject release];
    } else {
      ++it;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::NextStepUI::steerModal(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  while(steer()!=ONX_BREAK_MODAL) {}
  // Have to run again here :
  [fApplication run];
}
