// this :
#include <OnX/Qt/QPageViewer.h>

#include <qevent.h>
#include <qpainter.h>

#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/Qt/viewers/SoQtPlaneViewer.h>
#include <OnX/Inventor/GuiViewer.h>

#include <OnX/Qt/QtTools.h>

#if QT_VERSION < 0x040000
#include <qpopupmenu.h>
#define Qt_ButtonState ButtonState
#define Qt_gray gray
#else
#include <q3popupmenu.h> //Assume Qt4 and -DQT3_SUPPORT
#define QPopupMenu Q3PopupMenu
#define Qt_ButtonState Qt::ButtonState
#define Qt_gray Qt::gray
#endif

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

namespace OnX {
class QtPageViewerCollectCallback : public SbCollectCallback {
public: //SbCollectCallback
  virtual void execute(const SoCanvas&,const SoNodeList& aValue) {
    std::string value;
    int number = aValue.getLength();
    for(int index=0;index<number;index++) {
      if(index) value += "\n";
      value += std::string(aValue[index]->getName().getString());      
    }
    fPageViewer.emitCollect(value);
  }
public:
  QtPageViewerCollectCallback(QPageViewer& aPageViewer)
    :fPageViewer(aPageViewer){}
private:
  QPageViewer& fPageViewer;
};
}

//////////////////////////////////////////////////////////////////////////////
OnX::QPageViewer::QPageViewer(
 QWidget* aParent
,const char* aName
,const char* aViewerClass
)
:QWidget(aParent,aName)
,fViewer(0)
,fRubberMove(false)
,fBeginMove(0,0)
,fEndMove(0,0)
,fPopX(0),fPopY(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aViewerClass && !strcmp(aViewerClass,"Plane")) {
    fViewer = new SoQtPlaneViewer(aParent,"PAGEVIEWER_VIEWER",TRUE);
  } else {
    fViewer = new SoQtExaminerViewer(aParent,"PAGEVIEWER_VIEWER",TRUE);
  }
  GuiViewer_initialize<SoQtViewer>(*fViewer);

  SoPage* soPage = GuiViewer_SoPage<SoQtViewer>(*fViewer);
  if(soPage) {
    soPage->setCollectCallback(new QtPageViewerCollectCallback(*this));
  }

  // Create a "pick mode" popup menu :
  //QWidget parent = getNormalWidget();
  fPopupMenu = new QPopupMenu(aParent);
  fViewer->setEventCallback(eventCB,this);
}
//////////////////////////////////////////////////////////////////////////////
OnX::QPageViewer::~QPageViewer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fPopupMenu;
  if(fViewer) {
    GuiViewer_finalize<SoQtViewer>(*fViewer);
    if(fViewer->isOfType(SoQtExaminerViewer::getClassTypeId())) {
      delete (SoQtExaminerViewer*)fViewer;
    } else if(fViewer->isOfType(SoQtPlaneViewer::getClassTypeId())) {
      delete (SoQtPlaneViewer*)fViewer;
    } else {
      SoDebugError::postInfo
        ("OnX::QPageViewer::~QPageViewer","unknown viewer class");
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QPageViewer::setParameter(
 const std::string& aWhat
,const std::string& aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(!fViewer) return false;
  if(fViewer->isOfType(SoQtExaminerViewer::getClassTypeId())) {    
    if(GuiExaminerViewer_setParameter<SoQtExaminerViewer>
         (*((SoQtExaminerViewer*)fViewer),aWhat,aValue)) return true;
    SoDebugError::postInfo("OnX::QPageViewer::setParameter",
                           "\"%s\" not a parameter.",aWhat.c_str());
  } else if(fViewer->isOfType(SoQtPlaneViewer::getClassTypeId())) {
    if(GuiViewer_setParameter<SoQtFullViewer>
             (*fViewer,aWhat,aValue)) return true;
    SoDebugError::postInfo("OnX::QPageViewer::setParameter",
                           "\"%s\" not a parameter.",aWhat.c_str());
  } else {
    SoDebugError::postInfo
      ("OnX::QPageViewer::setParameter","unknown viewer class");
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QPageViewer::write(
 const std::string& aWhat
,const std::string& aFile
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(!fViewer) return false;
  return GuiViewer_write<SoQtViewer>(*fViewer,aWhat,aFile,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
SoNode* OnX::QPageViewer::sceneGraph(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fViewer) return 0;
  return fViewer->getSceneGraph();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewer::emitCollect(
  const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QCallbackData data;
  data.value = aValue;
  emit collect(data);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewer::emitPopup(
  const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QCallbackData data;
  data.value = aValue;
  data.x = fPopX;
  data.y = fPopY;
  emit popup(data);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnX::QPageViewerPopupCallback::QPageViewerPopupCallback(
 QPageViewer& aPageViewer
,const std::string& aLabel
)
:fPageViewer(aPageViewer),fLabel(aLabel)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewerPopupCallback::execute(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPageViewer.emitPopup(fLabel);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewer::addPopupEntry(
 const std::string& aLabel
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : who delete the QPageViewerPopupCallback ?
  if(!fPopupMenu) return;
  fPopupMenu->insertItem(aLabel.c_str(),
                         new QPageViewerPopupCallback(*this,aLabel),
                         SLOT(execute()));
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewer::drawRect(
 QPainter& aPainter
,const QPoint& aBegin
,const QPoint& aEnd
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if QT_VERSION < 0x040000
  aPainter.moveTo(aBegin);
  aPainter.lineTo(aBegin.x(),aEnd.y());
  aPainter.lineTo(aEnd.x(),aEnd.y());
  aPainter.lineTo(aEnd.x(),aBegin.y());
  aPainter.lineTo(aBegin);
#else
  aPainter.drawRect(aBegin.x(),aBegin.y(),aEnd.x(),aEnd.y());
#endif
}
//////////////////////////////////////////////////////////////////////////////
SbBool OnX::QPageViewer::eventCB(
 void* aTag
,QEvent* aEvent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QPageViewer* This = (QPageViewer*)aTag;

  Qt_ButtonState Qt_CtrlButton = Qt::ControlButton;

  //if(aEvent->type()==QEvent::ContextMenu)

  if(aEvent->type()==QEvent::MouseButtonPress) {
    QMouseEvent* event = (QMouseEvent*)aEvent;
    if(event->button()==Qt::RightButton) {
      if(This->fViewer->isViewing()==TRUE) return FALSE;

      This->fPopupMenu->popup
        (This->mapToGlobal(QPoint(event->x(),event->y())));

      This->fPopX = event->x();
      int hwin = This->fViewer->getNormalWidget()->height();
      This->fPopY = hwin - event->y(); //GL coordinates.

      return FALSE; //So that SoPage::mouseEventCB is treated.
    } else if(event->button()==Qt::LeftButton) {
      if( ((event->state() & Qt::ShiftButton)==Qt::ShiftButton) &&
          ((event->state() & Qt_CtrlButton)==Qt_CtrlButton) ) {

        This->fRubberMove = true;
        This->fBeginMove.setX(event->x());
        This->fBeginMove.setY(event->y());
        This->fEndMove = This->fBeginMove;

        QPainter painter(This->fViewer->getNormalWidget());
        painter.setPen(Qt_gray);
        painter.setBrush(Qt_gray);
#if QT_VERSION < 0x040000
        painter.setRasterOp(XorROP);
#else
        painter.setCompositionMode(QPainter::CompositionMode_Xor);
#endif
        This->drawRect(painter,This->fBeginMove,This->fEndMove);

        return TRUE;
      }
    }
  } else if(aEvent->type()==QEvent::MouseButtonRelease) {
    QMouseEvent* event = (QMouseEvent*)aEvent;
    if(event->button()==Qt::LeftButton) {
      if(This->fRubberMove) {
        This->fRubberMove = false;

        QPainter painter(This->fViewer->getNormalWidget());
        painter.setPen(Qt_gray);
        painter.setBrush(Qt_gray);
#if QT_VERSION < 0x040000
        painter.setRasterOp(XorROP);
#else
        painter.setCompositionMode(QPainter::CompositionMode_Xor);
#endif
      
        // Erase previous rectangle :
        This->drawRect(painter,This->fBeginMove,This->fEndMove);

        int x,y,w,h;
        This->getRectangle(This->fBeginMove.x(),This->fBeginMove.y(),
                           This->fEndMove.x(),This->fEndMove.y(),
                           x,y,w,h);
        
        int wwin = This->fViewer->getNormalWidget()->width();
        int hwin = This->fViewer->getNormalWidget()->height();

        // We pass in GL coordinate (origin is bottom-left, y toup) :
        int xgl = x;
        int ygl = hwin - (y+h);

        if(This->fViewer->isViewing()==TRUE) {
          GuiViewer_map<SoQtViewer>(*(This->fViewer),
                            SbVec2s((short)xgl,(short)ygl),
                            SbVec2s((short)w,(short)h),
                            SbVec2s((short)wwin,(short)hwin));
        } else {
          GuiViewer_collect<SoQtViewer>(*(This->fViewer),xgl,ygl,w,h);
        }

        return TRUE;
      }
    }
  } else if(aEvent->type()==QEvent::MouseMove) {
    QMouseEvent* event = (QMouseEvent*)aEvent;
    if((event->state() & Qt::LeftButton)!=Qt::LeftButton) return FALSE;

    if( ((event->state() & Qt::ShiftButton)==Qt::ShiftButton) &&
        ((event->state() & Qt_CtrlButton)==Qt_CtrlButton) ) {
      if(This->fRubberMove) {

        QWidget* w = This->fViewer->getNormalWidget();

        if( (event->x()<0) || (event->y()<0) 
            || (event->x()>=w->width()) || (event->y()>=w->height()) ){
          // Out of the window :
          return TRUE;
        }

        QPainter painter(w);
        painter.setPen(Qt_gray);
        painter.setBrush(Qt_gray);
#if QT_VERSION < 0x040000
        painter.setRasterOp(XorROP);
#else
        painter.setCompositionMode(QPainter::CompositionMode_Xor);
#endif

        // Erase previous rectangle :
        This->drawRect(painter,This->fBeginMove,This->fEndMove);

        // New one :
        This->fEndMove.setX(event->x());
        This->fEndMove.setY(event->y());
        This->drawRect(painter,This->fBeginMove,This->fEndMove);

        return TRUE;
      }
    } else {
      //Picking mode : not discarded. We can be in resize/move region.
      if(This->fViewer->isViewing()==FALSE) return FALSE; //Picking mode.
      //GL coordinates.
      int hwin = This->fViewer->getNormalWidget()->height();
      int px = event->x();
      int py = hwin - event->y();
      // Discard mouse move event not in current region in viewing mode.
      if(!GuiViewer_isInCurrentRegion<SoQtViewer>
            (*(This->fViewer),SbVec2s(px,py))) return TRUE;
    }
  } else if(aEvent->type()==QEvent::KeyPress) {
    if(This->fViewer->isViewing()!=TRUE) return FALSE;
    QKeyEvent* event = (QKeyEvent*)aEvent;
    if(event->key()==Qt::Key_BackSpace) {
      GuiViewer_popCamera<SoQtViewer>(*(This->fViewer));
      return TRUE;
    }
  }

  return FALSE;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QPageViewer::getRectangle(
 int aBeginX
,int aBeginY
,int aEndX
,int aEndY
,int& aX
,int& aY
,int& aWidth
,int& aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aX = MINIMUM(aBeginX,aEndX);
  aY = MINIMUM(aBeginY,aEndY);
  aWidth = MAXIMUM(aBeginX,aEndX) - aX;
  aHeight = MAXIMUM(aBeginY,aEndY) - aY;
}
