// this :
#include <OnX/Qt/QtTools.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/smanip.h>

#include <qstring.h>
#include <qwidget.h>
#include <qcombobox.h>
#include <qdialog.h>
#include <qpushbutton.h>

#if QT_VERSION < 0x040000
#include <qobjectlist.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qbutton.h>
#include <qlistview.h>
#else
#include <qobject.h>
#include <q3vbox.h>
#include <q3hbox.h>
#include <q3button.h>
#include <q3listview.h>
#define QHBox Q3HBox
#define QVBox Q3VBox
#define QButton Q3Button
#define QListViewItem Q3ListViewItem
#endif

//////////////////////////////////////////////////////////////////////////////
std::string OnX::q2std(
 const QString& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aString.isEmpty()) return "";
  const char* s = aString.latin1();
  return (s?s:"");
}
//////////////////////////////////////////////////////////////////////////////
Qt::WFlags OnX::decorationFlags(
 const std::vector< std::pair<bool,std::string> >& aItems
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Qt::WFlags Qt_decor = Qt::WStyle_Customize;

  // default values for decorations are in BaseUI.cxx
  bool close_button = true;
  bool iconify_button = true;
  bool min_max_button = true;
  bool resize_handle = true;
  bool border = true;
  bool caption = true;
  bool no_decorations = false;
  bool system_menu = true;
    
  std::vector< std::pair<bool,std::string> >::const_iterator it;
  for(it=aItems.begin();it!=aItems.end();++it) {
    if ((*it).second == "close_button") close_button = (*it).first; 
    if ((*it).second == "iconify_button") iconify_button = (*it).first; 
    if ((*it).second == "min_max_button") min_max_button = (*it).first; 
    if ((*it).second == "resize_handle") resize_handle = (*it).first; 
    if ((*it).second == "border") border = (*it).first;
    if ((*it).second == "caption") caption = (*it).first; 
    if ((*it).second == "system_menu") system_menu = (*it).first;
    if ((*it).second == "no_decorations") no_decorations = (*it).first;
  }

  Lib::Out out(aPrinter);
  std::string msg = "OnX::QtTools::decorationFlags : WARNING :";
    
  if (!close_button) {
    if (!no_decorations)
      out << msg 
          << "You should set no_caption if you want"  
          << " to hide the close button, " 
          << "but you will also hide all buttons." 
          << Lib::endl;
  }
  if (iconify_button) Qt_decor |= Qt::WStyle_Minimize;
  if (min_max_button) Qt_decor |= Qt::WStyle_Maximize;
  if (!resize_handle)
    if (!no_decorations)
      out << msg
          << "You should set no_border if you want" 
          << " to hide the resize handle, " 
          << "but you will also hide all border and buttons."
          << Lib::endl;
  if (border) Qt_decor |= Qt::WStyle_NormalBorder;
  else        Qt_decor |= Qt::WStyle_NoBorder;
  if (caption) Qt_decor |= Qt::WStyle_Title;
  if (system_menu) Qt_decor |=  Qt::WStyle_SysMenu;

  Qt_decor |= Qt::WType_TopLevel;

  //if(aMeta.inherits("QDialog")) Qt_decor |= Qt::WType_Dialog;

  return Qt_decor;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
int OnX::QWidgetGetNumberOfChildren(
 QWidget& aWidget
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if QT_VERSION < 0x040000
  const QObjectList* list = aWidget.children();
  if(!list) return 0;
  return list->count();
#else
  const QObjectList& list = aWidget.children();
  return list.count();
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QWidgetSetBackground(
 QWidget& aWidget
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QPalette p = aWidget.palette();
  p.setColor(QColorGroup::Background, QColor(aValue.c_str()));
  p.setColor(QColorGroup::Base, QColor(aValue.c_str()));
  aWidget.setPalette(p);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QWidgetSetForeground(
 QWidget& aWidget
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aWidget.setPaletteForegroundColor(QColor(aValue.c_str()));
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX::QButtonSetBackground(
 QButton& aWidget
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QPalette p = aWidget.palette();
  p.setColor(QColorGroup::Background, QColor(aValue.c_str()));
  p.setColor(QColorGroup::Button, QColor(aValue.c_str()));
  p.setColor(QColorGroup::Base, QColor(aValue.c_str()));
  aWidget.setPalette(p);
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QComboBoxSetBackground(
 QComboBox& aWidget
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QPalette p = aWidget.palette();
  p.setColor(QColorGroup::Background, QColor(aValue.c_str()));
  p.setColor(QColorGroup::Button, QColor(aValue.c_str()));
  p.setColor(QColorGroup::Base, QColor(aValue.c_str()));
  aWidget.setPalette(p);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QHBoxSetExpand(
 QHBox& aWidget
,const std::string& aExpand
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if QT_VERSION < 0x040000
  QObjectList* children = (QObjectList*)aWidget.children();
  if(!children) return false;
#else
  const QObjectList& list = aWidget.children();
  QObjectList* children = (QObjectList*)&list;
#endif
  QWidget* first = 0;
  QWidget* second = 0;
  // WARNING : QHBox have one extra QHBoxLayout child.
  if (children->count() == 2) {
    if ((((QObject*)(children->at(0)))->inherits("QBoxLayout")) &&
        (((QObject*)(children->at(1)))->inherits("QWidget"))) {
      first = ((QWidget*)(children->at(1)));
      // no use of second in this case
    } else {
      return false;
    }
  } else if (children->count() == 3) {
    if ((((QObject*)(children->at(0)))->inherits("QBoxLayout")) &&
        (((QObject*)(children->at(1)))->inherits("QWidget")) &&
        (((QObject*)(children->at(2)))->inherits("QWidget"))) { 
      first = ((QWidget*)(children->at(1)));
      second = ((QWidget*)(children->at(2)));
    } else {
      return false;
    }
  } else {
    return false;
  }

  if(aExpand=="second") {
    if(children->count()==2) {
      aWidget.setStretchFactor(first,0);
    } else if(children->count()==3) {
      QSizePolicy        sp = second->sizePolicy();
      sp.setHorData(QSizePolicy::Expanding);
      second->setSizePolicy(sp);
      aWidget.setStretchFactor(second,10);
      
      sp = first->sizePolicy();
      sp.setHorData(QSizePolicy::Preferred);
      first->setSizePolicy(sp);
      aWidget.setStretchFactor(first,0);
    }
  } else if(aExpand=="first") {
    if(children->count()==2) {
      QSizePolicy        sp = first->sizePolicy();
      sp.setHorData(QSizePolicy::Expanding);
      first->setSizePolicy(sp);
      aWidget.setStretchFactor(first,10);
    } else if(children->count()==3) {
      QSizePolicy        sp = first->sizePolicy();
      sp.setHorData(QSizePolicy::Expanding);
      first->setSizePolicy(sp);
      aWidget.setStretchFactor(first,10);
        
      sp = second->sizePolicy();
      sp.setHorData(QSizePolicy::Preferred);
      second->setSizePolicy(sp);
      aWidget.setStretchFactor(second,0);
    }
  } else { // expand both :
    if(children->count()>=2) {
      QSizePolicy sp = first->sizePolicy();
      sp.setHorData(QSizePolicy::Preferred);
      first->setSizePolicy(sp);
      aWidget.setStretchFactor(first,0);

      if(children->count()>=3) {
        sp = second->sizePolicy();
        sp.setHorData(QSizePolicy::Preferred);
        second->setSizePolicy(sp);
        aWidget.setStretchFactor(second,0);
      }
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QVBoxSetExpand(
 QVBox& aWidget
,const std::string& aExpand
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if QT_VERSION < 0x040000
  QObjectList* children = (QObjectList*)aWidget.children();
  if(!children) return false;
#else
  const QObjectList& list = aWidget.children();
  QObjectList* children = (QObjectList*)&list;
#endif
  QWidget* first = 0;
  QWidget* second = 0;
  // WARNING : QVBox has one extra QVBoxLayout child.
   if (children->count() == 2) {
    if ((((QObject*)(children->at(0)))->inherits("QBoxLayout")) &&
        (((QObject*)(children->at(1)))->inherits("QWidget"))) {
      first = ((QWidget*)(children->at(1)));

      QSizePolicy sp = first->sizePolicy();
        
      sp.setVerData(QSizePolicy::Expanding);
      first->setSizePolicy(sp);
      aWidget.setStretchFactor(first,10);
      return true;
    } else {
      return false;
    }
  } else if (children->count() == 3) {
    if ((((QObject*)(children->at(0)))->inherits("QBoxLayout")) &&
        (((QObject*)(children->at(1)))->inherits("QWidget")) &&
        (((QObject*)(children->at(2)))->inherits("QWidget"))) { 
      first = ((QWidget*)(children->at(1)));
      second = ((QWidget*)(children->at(2)));
    } else {
      return false;
    }
  } else {
    return false;
  }

  if(children->count()!=3) return true;

  if(aExpand=="second") { 
    QSizePolicy sp = second->sizePolicy();

    sp.setVerData(QSizePolicy::Expanding);
    second->setSizePolicy(sp);
    aWidget.setStretchFactor(second,10);
    
    sp = first->sizePolicy();
    sp.setVerData(QSizePolicy::Preferred);
    first->setSizePolicy(sp);
    aWidget.setStretchFactor(first,0);
  }
  else if(aExpand=="first") {
    QSizePolicy sp = first->sizePolicy();

    sp.setVerData(QSizePolicy::Expanding);
    first->setSizePolicy(sp);
    aWidget.setStretchFactor(first,10);
    
    sp = second->sizePolicy();
    sp.setVerData(QSizePolicy::Preferred);
    second->setSizePolicy(sp);
    aWidget.setStretchFactor(second,0);
  }
  else { // expand both :    
    QSizePolicy sp = second->sizePolicy();
    sp = second->sizePolicy();
    sp.setVerData(QSizePolicy::Preferred);
    second->setSizePolicy(sp);
    aWidget.setStretchFactor(second,0);

    sp = first->sizePolicy();
    sp.setVerData(QSizePolicy::Preferred);
    first->setSizePolicy(sp);
    aWidget.setStretchFactor(first,0);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QListViewItemGetPath(
 QListViewItem* aItem
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string path;
  QListViewItem* item = aItem;
  while(item) {
    std::string opath = path;
    path = "\n";
    path += OnX::q2std(item->text(0));
    path += opath;
    item = item->parent();
  }
  // Remove the leading \n
  if(path.size()) path = path.substr(1,path.size()-1);
  return path;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QListViewItemGetXML(
 QListViewItem* aItem
)
//////////////////////////////////////////////////////////////////////////////
//return a XML string representing this tree
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string spaceItem;
  std::string spaceRoot;
  std::string line;
  do  {
    line += spaceRoot + "<treeItem>";
    line += spaceItem + "<label>";
    std::string s = Lib::smanip::toxml(q2std(aItem->text(0)));
    line.append(s);
    line += "</label>";
    line += spaceItem + "<opened>";
    if (aItem->isOpen()) line.append("true");
    else line.append("false");
    line += "</opened>";
    if (aItem->firstChild()) {
      line += QListViewItemGetXML(aItem->firstChild());
    }    
    line += spaceRoot + "</treeItem>";
    aItem = aItem->nextSibling();
  }
  while (aItem);
  return line;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QDialogDisableEnter(
 QDialog& aDialog
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#if QT_VERSION < 0x040000
  QObjectList* list = aDialog.queryList("QPushButton");
  QObjectListIt it(*list);
  QPushButton* pb;
  while ( (pb = (QPushButton*)it.current()) ) {
    pb->setAutoDefault(false);
    ++it;
  }
  delete list;
#else
  QList<QPushButton*> list = aDialog.findChildren<QPushButton*>();
  int number = list.size();
  for (int index=0;index<number;index++) {
    list.at(index)->setAutoDefault(false);
  }
#endif
}
