// this :
#include <OnX/Qt/QtUI.h>

#include <Slash/Core/ISession.h>
#include <Slash/UI/IWidgetClass.h>
#include <Slash/UI/ISoViewer.h>
#include <Slash/UI/IWidget.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/Debug.h>
#include <Lib/Messages.h>
#include <Lib/BaseML.h>
#include <Lib/Element.h>
#include <Lib/ItemML.h>
#include <Lib/Cast.h>

#include <OnX/Qt/QtCyclic.h>
#include <OnX/Qt/QCommandLine.h>
#include <OnX/Qt/QPageViewer.h>
#include <OnX/Qt/QtTools.h>
#include <OnX/Qt/QTk.h>

#define ONX_HAS_INVENTOR
#ifdef ONX_HAS_INVENTOR
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/Qt/viewers/SoQtPlaneViewer.h>
//#include <Inventor/Qt/viewers/SoQtWalkViewer.h>
#include <Inventor/Qt/viewers/SoQtFlyViewer.h>
#include <OnX/Inventor/Inventor.h>
#include <OnX/Inventor/HEPVis.h>
#include <HEPVis/nodekits/SoPage.h>
#endif

#include <qapplication.h>
#include <qmenubar.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qpainter.h>
#include <qlineedit.h>
#include <qsplitter.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qslider.h>
#include <qmessagebox.h>
#include <qgl.h>
#include <qmetaobject.h>
#include <qeventloop.h>

#include <qmainwindow.h>
#define S_QMainWindow "QMainWindow"

#if QT_VERSION < 0x040000
#include <qvbox.h>
#include <qhbox.h>
#include <qpopupmenu.h>
#include <qtextview.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qhbuttongroup.h>
#include <qgrid.h>
#include <qheader.h>
#include <qobjectlist.h>
#include <qprogressbar.h>
#include <qfiledialog.h>
#define S_QPopupMenu "QPopupMenu"
#define S_QTextView "QTextView"
#define S_QListBox "QListBox"
#define S_QListView "QListView"
#define S_QProgressBar "QProgressBar"
#define S_QButton "QButton"
#define S_QVBox "QVBox"
#define S_QHBox "QHBox"
#define S_QFileDialog  "QFileDialog"
#else
#include <q3button.h>
#include <q3vbox.h>
#include <q3hbox.h>
#include <q3popupmenu.h>
#include <q3textview.h>
#include <q3listbox.h>
#include <q3listview.h>
#include <q3buttongroup.h>
#include <q3grid.h>
#include <q3header.h>
#include <qobject.h>
#include <q3progressbar.h>
#include <q3valuelist.h>
//#include <q3filedialog.h>
#include <qfiledialog.h>
#include <qtoolbar.h>
//#include <q3mainwindow.h>
#include <qmenudata.h> // for QMenuItem definition
#define QButton Q3Button
#define QHBox Q3HBox
#define QVBox Q3VBox
#define QPopupMenu Q3PopupMenu
#define QTextView Q3TextView
#define QListBox Q3ListBox
#define QListView Q3ListView
#define QListViewItem Q3ListViewItem
#define QButtonGroup Q3ButtonGroup
#define QGrid Q3Grid
#define QProgressBar Q3ProgressBar
#define QValueList Q3ValueList
#define QTextEdit Q3TextEdit
//#define QMainWindow Q3MainWindow
#define S_QPopupMenu "Q3PopupMenu"
#define S_QTextView "Q3TextView"
#define S_QListBox "Q3ListBox"
#define S_QListView "Q3ListView"
#define S_QListViewItem "Q3ListViewItem"
#define S_QProgressBar "Q3ProgressBar"
#define S_QButton "Q3Button"
#define S_QVBox "Q3VBox"
#define S_QHBox "Q3HBox"
//#define S_QMainWindow "Q3MainWindow"
#define S_QFileDialog  "OnX_QFileDialog"
#endif

#define S_QDialog      "QDialog"
#define S_QGLWidget    "QGLWidget"
#define S_QTabWidget   "QTabWidget"
#define S_QSlider      "QSlider"
#define S_QLineEdit    "QLineEdit"
#define S_QComboBox    "QComboBox"
#define S_QScrollBar   "QScrollBar"
#define S_QCheckBox    "QCheckBox"
#define S_QLabel       "QLabel"
#define S_QComboBox    "QComboBox"
#define S_QCheckBox    "QCheckBox"
#define S_QSplitter    "QSplitter"
#define S_QMenuBar     "QMenuBar"
#define S_QToolBar     "QToolBar"
#define S_QPushButton  "QPushButton"
#define S_QLineEdit    "QLineEdit"

#define S_OnX_QToggleMenuItem "OnX_QToggleMenuItem"
#define S_OnX_QMenuItem       "OnX_QMenuItem"
#define S_OnX_QMenu           "OnX_QMenu"
#define S_OnX_QColorDialog    "OnX_QColorDialog"
#define S_OnX_QOpenGLArea     "OnX_QOpenGLArea"

static QObject* findParent(QObject*,const std::string&);

//////////////////////////////////////////////////////////////////////////////
OnX::QtUI::QtUI(
 Slash::Core::ISession& aSession
,const std::string& aFile
,const std::vector<std::string>& aArgs
)
:BaseUI(aSession)
,fQApplication(0)
,fQApplication_owner(false)
,fFirstWindow(true)
,fConsoleWindow(0)
,fConsoleText(0)
,fWarningDialog(0)
,fCounter(0)
,fMenuItemCounter(1)
,fMessageWidget(0)
,fTermCyclic(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (!qApp) {    
    //QApplication::setColorSpec( QApplication::ManyColor );
    int argc;
    char** argv;
    Lib::smanip::newArgcv(aArgs,argc,argv);
#if QT_VERSION < 0x040000
    qApp = new QApplication (argc, argv);
    fQApplication = qApp;
    fQApplication_owner = true;
#else
    fQApplication = new QApplication (argc, argv);
    fQApplication_owner = true;
#endif
    Lib::smanip::delArgcv(argc,argv);
  } else {
    fQApplication = qApp;
    fQApplication_owner = false;
  }  

  if(!fQApplication) {
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::QtUI :"
        << " null QApplication."
        << Lib::endl;
    return;
  }

  std::string value;
  if(aSession.parameterValue("UI_Manager.style",value) &&(value!="")) {
    // The string must be one of the QStyleFactory::keys(),
    // typically one of :
    //  "windows", "motif", "cde", "motifplus", "platinum", "sgi"
    //  "compact", "windowsxp", "aqua", "macintosh"
    QApplication::setStyle(value.c_str());
  }

  fMessageWidget = new OnX_QMessageWidget();

  QWidget* qWidget = new QVBox();
#ifdef ONX_HAS_INVENTOR
  SoQt::init(qWidget);
  HEPVis::initClasses();
  OnX::Inventor::initialize(aSession);
#endif

  if(!startup(aFile)) {
    delete fMessageWidget;
    if(fQApplication_owner) delete fQApplication;
    fQApplication = 0;
    fMessageWidget = 0;
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::QtUI :"
        << " onx file startup failure."
        << Lib::endl;
    return;
  }

  fConsoleWindow = (QWidget*)findNativeWidget("consoleWindow");
  fConsoleText = (QWidget*)findNativeWidget("consoleText");
  if(fConsoleText) {
    fConsoleText->setFont(QFont("courier"));
    //((QTextView*)fConsoleText)->setTextFormat(Qt::PlainText);
  }
  fWarningDialog = (QMessageBox*)findNativeWidget("warningDialog");
}
//////////////////////////////////////////////////////////////////////////////
OnX::QtUI::~QtUI(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fBeingDestroyed = true;

  // Treat messages emitted between the quiting of the steering
  // and here. They may come from the destruction of various
  // managers in the ~Session.
  synchronize();

  std::list<QWidget*>::reverse_iterator it;
  for(it=fWidgets.rbegin();it!=fWidgets.rend();++it) {
    if(fVerboseLevel) {
      std::string name = (*it)->name();
      std::string className = (*it)->className();
      Lib::Out out(fSession.printer());
      out << "OnX::QtUI::~QtUI :"
          << " delete "
          << "\"" << name << "\" of class " 
          << "\"" << className << "\"..." 
          << Lib::endl;
    }
    delete (*it);
  }
  fWidgets.clear();

  fConsoleWindow = 0;
  fConsoleText = 0;

  delete fMessageWidget;
  if(fQApplication_owner) delete fQApplication;
  
 {std::vector<QtCallback*>::iterator it;
  for(it=fQtCallbacks.begin();
      it!=fQtCallbacks.end();
      it=fQtCallbacks.erase(it)) 
    delete (*it);}
  
#ifdef ONX_HAS_INVENTOR
  OnX::Inventor::finalize(fSession); 
#endif
}
//////////////////////////////////////////////////////////////////////////////
void* OnX::QtUI::cast( 
 const std::string& aClassName
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aClassName=="OnX::QtUI") {
    return Lib_SCast(QtUI);
  } else {
    return BaseUI::cast(aClassName);
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fQApplication) return false;
  if(!fTopItemML) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
QApplication* OnX::QtUI::getQApplication(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fQApplication;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::put(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Append line at end of report region.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fConsoleText) return;
  ((QTextView*)fConsoleText)->insert(aString.c_str());
  if(fConsoleWindow) fConsoleWindow->show();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::warn(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
// Modal method.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fWarningDialog) {
    echo("WARNING : "+aString);
    return;
  }
  fWarningDialog->setText(aString.c_str());
  fWarningDialog->exec();
}
//////////////////////////////////////////////////////////////////////////////
int OnX::QtUI::nativeSteer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fQApplication) return 0;
  if(!fTopItemML) return 0;
  fSteerLevel++;

  QEventLoop* eventLoop = new QEventLoop();
  fEventLoopStack.push_front(eventLoop);
  int code = eventLoop->exec();

 {QEventLoop* eloop = fEventLoopStack.front();
  if(eloop!=eventLoop) {
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::nativeSteer :"
        << " problem with QEventLoops. fSteerLevel " << fSteerLevel << "."
        << Lib::endl;
    return 0;
  }}

  fEventLoopStack.pop_front();
  delete eventLoop;

  //int code = fQApplication->exec();
  fSteerLevel--;
  return code;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::synchronize(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //if(!fQApplication) return;
  //fQApplication->processEvents(); 

  if(fEventLoopStack.empty()) return;
  QEventLoop* eventLoop = fEventLoopStack.front();
  eventLoop->processEvents();
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::exit(
 int aExitCode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //if(!fQApplication) return;
  //fQApplication->exit(aExitCode);

  if(fEventLoopStack.empty()) return;
  QEventLoop* eventLoop = fEventLoopStack.front();
  eventLoop->exit(aExitCode);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::lock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fQApplication) return false;
  fQApplication->lock();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::unlock(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fQApplication) return false;
  fQApplication->unlock();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::notify(
 Slash::Core::IMessage* aMessage
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fMessageWidget) return false;

  //FIXME : or done with fEventLoopStack ?
  if(!fQApplication) return false;
#if QT_VERSION < 0x040000
  QApplication::postEvent(fMessageWidget,
    new QCustomEvent(QEvent::User,new QtMessage(*this,aMessage)));
#else
  fQApplication->postEvent(fMessageWidget,
    new OnX::QCustomEvent(*this,aMessage));
#endif
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::setCallback(
 const std::string& aName
,const std::string& aWhich
,const std::string& aInterp
,const std::string& aScript
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  Lib::ItemML* element = fTopItemML->findItem(aName);
  if(!element) return false;
  QWidget* widget = (QWidget*)element->getData1();
  if(!widget) return false;
  bool newCallback = false;
  Action* action = 0;
  // MenuItem : 
  if(widget->inherits(S_OnX_QMenuItem)) {
    int qtID = element->getDataInt();
    //if(!qtID) return false; // CascadeButton (QPopupMenu).
    action = getAction(qtID);
  } else {
    action = getAction(widget,aWhich,aIndex);
  }
  if(!action) {
    newCallback = true;
    action = addCallback(*element,aWhich,aInterp,aScript);
  }
  if(!action) return false;
  action->setInterpreter(aInterp);
  action->setScript(aScript);
  // Update element :
  if (newCallback) {
    std::vector<Lib::ItemML::Attribute> att;
    att.push_back(Lib::ItemML::Attribute("exec",aInterp));
    element->addProperty(aWhich,att,aScript);
  } else {
    element->setAttributeValue(aWhich,"exec",aInterp,aIndex);
    element->setPropertyValue(aWhich,aScript,aIndex);
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::show(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aName=="" && fTopItemML) {
    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "OnX::QtUI::show..."
          << Lib::endl;
    }
    showAll(*fTopItemML);
    return;
  }
  QWidget* widget = (QWidget*)findNativeWidget(aName);
  if(!widget) return;
#if QT_VERSION < 0x040000
  widget->show();

  QWidget* shell = widget->topLevelWidget();
  if(!shell) return;
  if(shell->inherits(S_QDialog)) {
    if(shell->isShown()) return; //To avoid a recenter if already shown.
    int sw = shell->width();
    int sh = shell->height();
    QWidget* parent = shell->parentWidget();
    if(parent->inherits(S_QMainWindow)) {
      int w = parent->width();
      int h = parent->height();
      int x = parent->x();
      int y = parent->y();
      shell->move(x+(w-sw)/2,y+(h-sh)/2);
    }
  }
  shell->show(); 
#else
  // In Qt4 QWidget::show(),hide() are no more virtual !
  if(widget->inherits(S_QFileDialog)) {
    ((OnX_QFileDialog*)widget)->map();
  } else if(widget->inherits(S_OnX_QColorDialog)) {
    ((OnX_QColorDialog*)widget)->map();
  } else {
    QWidget* shell = widget->topLevelWidget();
    if(!shell) return;
    if(shell->inherits(S_QDialog)) {
      if(shell->isShown()) return; //To avoid a recenter if already shown.
      int sw = shell->width();
      int sh = shell->height();
      QWidget* parent = shell->parentWidget();
      if(parent->inherits(S_QMainWindow)) {
        int w = parent->width();
        int h = parent->height();
        int x = parent->x();
        int y = parent->y();
        shell->move(x+(w-sw)/2,y+(h-sh)/2);
      }
    }
    shell->show(); 
  }
#endif
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::hide(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)findNativeWidget(aName);
  if(!widget) return;
#if QT_VERSION < 0x040000
  QWidget* shell = widget->topLevelWidget();
  if(!shell) return;
  shell->hide();
#else
  if(widget->inherits(S_QFileDialog)) {
  } else if(widget->inherits(S_OnX_QColorDialog)) {
  } else {
    QWidget* shell = widget->topLevelWidget();
    if(!shell) return;
    shell->hide(); 
  }
#endif
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::refresh(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)findNativeWidget(aName);
  if(!widget) return false;
  if(widget->inherits(S_QGLWidget)) {
    ((QGLWidget*)widget)->updateGL();
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::nativeSize(
 void* aNative
,unsigned int& aWidth
,unsigned int& aHeight
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)aNative;
  aWidth = widget->width();
  aHeight = widget->height();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::position(
 const std::string& aName
,int& aX
,int& aY
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aX = 0; 
  aY = 0;
  QWidget* widget = (QWidget*)findNativeWidget(aName);
  if(!widget) return false;
  aX = widget->x();
  aY = widget->y();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::parameterValue(
 const std::string& aWhat
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;

  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::parameterValue :"
        << " widget \"" << aWidget << "\""
        << " not found." << Lib::endl;
    return false;
  }

  QWidget* widget = (QWidget*)element->getData1();
  if(widget) {
    if(aResource=="background") {
      QColor color = widget->paletteBackgroundColor();
      aValue = Lib::smanip::tostring(((double)color.red())/255.)+" " +
               Lib::smanip::tostring(((double)color.green())/255.)+" "+
               Lib::smanip::tostring(((double)color.blue())/255.);
      return true;
    } else if(aResource=="foreground") {
      QColor color = widget->paletteForegroundColor();
      aValue = Lib::smanip::tostring(((double)color.red())/255.)+" " +
               Lib::smanip::tostring(((double)color.green())/255.)+" "+
               Lib::smanip::tostring(((double)color.blue())/255.);
      return true;

    } else if(aResource=="items") {
      if(widget->inherits(S_QListView)) {
        if(((QListView*)widget)->firstChild()) {
          aValue = "<tree>"+QListViewItemGetXML(((QListView*)widget)->firstChild())+"</tree>";
        } else {
          aValue = "";
        }
        return true;
      }
    } else if(aResource=="selection") {
      if(widget->inherits(S_QTabWidget)) {
        QTabWidget* w = (QTabWidget*)widget;
        if(w->currentPage()) {
          aValue = q2std(w->tabLabel(w->currentPage()));
        }
        return true;
      } else if(widget->inherits(S_QListBox)) {
        aValue = q2std(((QListBox*)widget)->currentText());
        return true;
      } else if(widget->inherits(S_QListView)) {
        QListViewItem* item = ((QListView*)widget)->selectedItem();
        aValue = item?QListViewItemGetPath(item):"";
        return true;
      }
    } else if(aResource=="itemCount") {
      if(widget->inherits(S_QTabWidget)) {
        QTabWidget* w = (QTabWidget*)widget;
        Lib::smanip::printf(aValue,64,"%d",w->count());
        return true;
      }
    } else if(aResource=="value") {
      if(widget->inherits(S_QLineEdit)) {
        aValue = q2std(((QLineEdit*)widget)->text());
        return true;
      } else if(widget->inherits(S_QComboBox)) {
        aValue = q2std(((QComboBox*)widget)->currentText());
        return true;
      } else if(widget->inherits(S_QScrollBar)) {
        Lib::smanip::printf(aValue,64,"%d",((QScrollBar*)widget)->value());
        return true;
      } else if(widget->inherits(S_QSlider)) {
        Lib::smanip::printf(aValue,64,"%d",((QSlider*)widget)->value());
        return true;
      } else if(widget->inherits(S_QTextView)) {
        aValue = q2std(((QTextEdit*)widget)->text());
        return true;
      } else if(widget->inherits(S_QProgressBar)) {
        aValue = Lib::smanip::tostring(((QProgressBar*)widget)->progress());
        return true;
      }

    } else if(aResource=="minimum") {
      if(widget->inherits(S_QScrollBar)) {
        Lib::smanip::printf(aValue,64,"%d",((QScrollBar*)widget)->minValue());
        return true;
      } else if(widget->inherits(S_QSlider)) {
        Lib::smanip::printf(aValue,64,"%d",((QSlider*)widget)->minValue());
        return true;
      }

    } else if(aResource=="maximum") {
      if(widget->inherits(S_QScrollBar)) {
        Lib::smanip::printf(aValue,64,"%d",((QScrollBar*)widget)->maxValue());
        return true;
      } else if(widget->inherits(S_QSlider)) {
        Lib::smanip::printf(aValue,64,"%d",((QSlider*)widget)->maxValue());
        return true;
      }

    } else if(aResource=="set") {
      if(widget->inherits(S_QCheckBox)) {
        aValue = ((QCheckBox*)widget)->isChecked()?"true":"false";
        return true;
      } else if(widget->inherits(S_OnX_QToggleMenuItem)) {
        int qtID = element->getDataInt();
        QPopupMenu& popup = ((OnX_QMenu*)widget)->popupMenu();
        QMenuItem* menuItem = popup.findItem(qtID);
        if(menuItem) {
          aValue = popup.isItemChecked(qtID)?"true":"false";
          return true;
        }
      }
    } else if(aResource=="label") {
      if(widget->inherits(S_QLabel)) {
        aValue = q2std(((QLabel*)widget)->text());
        return true;
      } else if(widget->inherits(S_QButton)) {
        aValue = q2std(((QButton*)widget)->text());
        return true;
      } else if(widget->inherits(S_QPopupMenu)) {
#if QT_VERSION < 0x040000
        QWidget* containerWidget = (QWidget*)element->getData2();
        QMenuData* menuData = 0;
        if(containerWidget->inherits(S_QMenuBar)) {
          menuData = (QMenuData*)(QMenuBar*)containerWidget;
        } else if(containerWidget->inherits(S_QPopupMenu)) {
          menuData = (QMenuData*)(QPopupMenu*)containerWidget;
        }
        if(menuData) {
          int index;
          //FIXME : be careful with findPopup method, it's an internal one
          QMenuItem* menuItem = 
            menuData->findPopup((QPopupMenu*)widget,&index);
          if(menuItem) {
            aValue = q2std(menuItem->text());
            return true;
          }
        }
#else 
        // FIXME should be verifed !
        aValue = q2std(((QMenu*)(QPopupMenu*)widget)->title());
        return true;
#endif
      } else if(widget->inherits(S_OnX_QMenuItem)) {
        int qtID = element->getDataInt();
        QPopupMenu& popup = ((OnX_QMenu*)widget)->popupMenu();
        QMenuItem* menuItem = popup.findItem(qtID);
        if(menuItem) {
          aValue = q2std(menuItem->text());
          return true;
        }
      }
    }
  }
  // Not found, then look in XML elements :
  if(aResource=="class") {
    return element->attributeValue("",aResource,aValue);
  }
  return element->propertyValue(aResource,aValue);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::setParameter(
 const std::string& aWhat
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopItemML) return false;
  std::string aWidget;
  std::string aResource;
  if(!toWidgetAndResource(aWhat,aWidget,aResource)) return false;

  if(fVerboseLevel) {
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::setParameter :"
        << " widget \"" << aWidget << "\""
        << " resource \"" << aResource << "\""
        << " value \"" << aValue << "\"."
        << Lib::endl;
  }

  Lib::ItemML* element = fTopItemML->findItem(aWidget);
  if(!element) {
    Lib::Out out(fSession.printer());
    out << "OnX::QtUI::setParameter :"
        << " widget \"" << aWidget << "\""
        << " not found." << Lib::endl;
    return false;
  }

  QWidget* widget = (QWidget*)element->getData1();
  if(!widget) return false;

  std::string svalue;

  if(aResource=="background") {
    if(widget->inherits(S_QButton)) {
      QButtonSetBackground(*((QButton*)widget),aValue);
    } else if(widget->inherits(S_QComboBox)) {
      QComboBoxSetBackground(*((QComboBox*)widget),aValue);
    } else {
      QWidgetSetBackground(*widget,aValue);
    }
    return true;
  } else if(aResource=="foreground") {
    QWidgetSetForeground(*widget,aValue);
    return true;
  } else if(aResource=="decoration") {
    if(widget->inherits("OnX::QPageViewer")) {
      if(((QPageViewer*)widget)->setParameter(aResource,aValue)) return true;
    } else {
      QWidget* top = widget->topLevelWidget();
      int sx = top->x();
      int sy = top->y();
#ifdef __APPLE__
      if(sy<=23) sy = 23;  //To be under Apple menubar.
#endif
      Qt::WFlags flags = 
        decorationFlags(parseDecorations(aValue),fSession.printer());
      if(top->inherits(S_QDialog)) flags |= Qt::WType_Dialog;
      top->reparent(top->parentWidget(),flags,top->pos(),top->isVisible());
      top->move(sx,sy);
      top->resize(top->size());
      return true;
    }
  } else if(aResource=="label") {
    if(widget->inherits(S_QLabel)) {
      ((QLabel*)widget)->setText(aValue.c_str());
      return true;
    } else if(widget->inherits(S_QPopupMenu)) {
#if QT_VERSION < 0x040000
      // Hell, what a mess to set a label !
      // FIXME : the item in the menu bar has bad size in case 
      //         the menu bar is mapped when changing the label.
      QWidget* containerWidget = (QWidget*)element->getData2();
      QMenuData* menuData = 0;
      if(containerWidget->inherits(S_QMenuBar)) {
        menuData = (QMenuData*)(QMenuBar*)containerWidget;
      } else if(containerWidget->inherits(S_QPopupMenu)) {
        menuData = (QMenuData*)(QPopupMenu*)containerWidget;
      }        
      if(menuData) {
        int index;
        QMenuItem* menuItem = menuData->findPopup((QPopupMenu*)widget,&index);
        if(menuItem) {
          menuItem->setText(aValue.c_str());
          return true;
        }
      }
#else
      ((QMenu*)widget)->setTitle(aValue.c_str());
      return true;
#endif
    } else if(widget->inherits(S_OnX_QMenuItem)) {
      int qtID = element->getDataInt();
      QPopupMenu& popup = ((OnX_QMenu*)widget)->popupMenu();
      QMenuItem* menuItem = popup.findItem(qtID);
      if(menuItem) {
        menuItem->setText(aValue.c_str());
        return true;
      }
    } else if(widget->inherits(S_QButton)) {
      ((QButton*)widget)->setText(aValue.c_str());
      return true;
    }
  } else if(aResource=="expand") {
    // Should be in that order because QVBos herited QHBox
    if(widget->inherits(S_QVBox)) {
      return QVBoxSetExpand(*((QVBox*)widget),aValue);
    } else if(widget->inherits(S_QHBox)) {
      return QHBoxSetExpand(*((QHBox*)widget),aValue);
    }
  } else if(aResource=="dirMask") {
    if(widget->inherits(S_QFileDialog)) {
      svalue = aValue;
      Lib::smanip::getenv(svalue);
#if QT_VERSION < 0x040000
      ((QFileDialog*)widget)->setFilter(svalue.c_str());
#else
      ((OnX_QFileDialog*)widget)->setFilter(svalue);
#endif
      return true;
    }
  } else if(aResource=="directory") {
    if(widget->inherits(S_QFileDialog)) {
      svalue = aValue;
      Lib::smanip::getenv(svalue);
#if QT_VERSION < 0x040000
      ((QFileDialog*)widget)->setDir(svalue.c_str());
#else
      ((OnX_QFileDialog*)widget)->setDirectory(svalue);
#endif
      return true;
    }
  } else if(aResource=="mode") {
    if(widget->inherits(S_QFileDialog)) {
#if QT_VERSION < 0x040000
      if(aValue=="save") {
        ((QFileDialog*)widget)->setMode(QFileDialog::AnyFile);
      } else {
        ((QFileDialog*)widget)->setMode(QFileDialog::ExistingFile);
      }
#else
      if(aValue=="save") {
        ((OnX_QFileDialog*)widget)->setAcceptMode(OnX_QFileDialog::AcceptSave);
      } else {
        ((OnX_QFileDialog*)widget)->setAcceptMode(OnX_QFileDialog::AcceptOpen); 
      }
#endif
      return true;
    }
  } else if(aResource=="selection") {
    if(widget->inherits(S_QListView)) {
      ((QListView*)widget)->clearSelection();
      // For ScrolledList and ScrolledTree :
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      unsigned int itemn = items.size();
      bool found = true;
      for(unsigned int index=0;index<itemn;index++){
        QListViewItem* item = 
          ((QListView*)widget)->findItem(items[index].c_str(),0);
        if(!item) {
          found = false;
          break;
        } else {
          if(index!=itemn-1) {
            item->setOpen(true);
          }
          ((QListView*)widget)->setSelected(item,true);
        }
      }
      if(found) return true;
      else ((QListView*)widget)->clearSelection();
    } 
  } else if(aResource=="set") {
    bool value;
    if(Lib::smanip::tobool(aValue,value)) {
      if(widget->inherits(S_QCheckBox)) {
        ((QCheckBox*)widget)->setChecked(value);
        return true;
      } else if(widget->inherits(S_OnX_QToggleMenuItem)) {
        int qtID = element->getDataInt();
        QPopupMenu& popup = ((OnX_QMenu*)widget)->popupMenu();
        QMenuItem* menuItem = popup.findItem(qtID);
        if(menuItem) {
          popup.setItemChecked(qtID,value);
          return true;
        }
      }
    }
  } else if(aResource=="value") {
    if(widget->inherits(S_QScrollBar)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QScrollBar*)widget)->setValue(value);
        return true;
      }
    } else if(widget->inherits(S_QSlider)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QSlider*)widget)->setValue(value);
        return true;
      }
    } else if(widget->inherits(S_QLineEdit)) {
      ((QLineEdit*)widget)->setText(aValue.c_str());
      return true;
    } else if(widget->inherits(S_QTextView)) {
      ((QTextView*)widget)->setText(aValue.c_str());
      return true;
    } else if(widget->inherits(S_QProgressBar)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QProgressBar*)widget)->setProgress(value);
        return true;
      }
    }
  } else if(aResource=="minimum") {
    if(widget->inherits(S_QScrollBar)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QScrollBar*)widget)->setMinValue(value);
        return true;
      }
    } else if(widget->inherits(S_QSlider)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QSlider*)widget)->setMinValue(value);
        return true;
      }
    }
  } else if(aResource=="maximum") {
    if(widget->inherits(S_QScrollBar)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QScrollBar*)widget)->setMaxValue(value);
        return true;
      }
    } else if(widget->inherits(S_QSlider)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        ((QSlider*)widget)->setMaxValue(value);
        return true;
      }
    }
  } else if(aResource=="default") {
    if(widget->inherits(S_QComboBox)) {
      int number = ((QComboBox*)widget)->count();
      for(int index=0;index<number;index++) {
        if (q2std(((QComboBox*)widget)->text(index))==aValue) {
          ((QComboBox*)widget)->setCurrentItem(index);
          return true;
        }
      }
      if(((QComboBox*)widget)->editable()) {
        ((QComboBox*)widget)->setCurrentText(aValue.c_str());
        return true;
      }
    }
  } else if(aResource=="items") {
    if(widget->inherits(S_QListBox)) {
      ((QListBox*)widget)->clear();
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      unsigned int itemn = items.size();
      for(unsigned int index=0;index<itemn;index++){
        ((QListBox*)widget)->insertItem((char*)items[index].c_str());
      }
      return true;
    } else if(widget->inherits(S_QListView)) {
      //WARNING : could be executed from a execute3 callback.
      //          The the below clear() induces a crash.

      // disable the scripts of the callback to avoid recursive loop 
      // if someone has define the same callback in the script.
      std::vector<std::string> scripts = 
        disableCallbackScript(widget,"select");
      // retrieve the old tree in a string
      std::string oldXMLTree;
      if(((QListView*)widget)->firstChild())
        oldXMLTree = QListViewItemGetXML(((QListView*)widget)->firstChild());
      //retrieve the ol selection
      QListViewItem* item = ((QListView*)widget)->selectedItem();
      std::string selection = item?QListViewItemGetPath(item):"";
      // clear the old tree in memory
      ((QListView*)widget)->clear();
      if(!((QListView*)widget)->columns()) {
        ((QListView*)widget)->addColumn("");
        ((QListView*)widget)->setRootIsDecorated ( true );
      }
      if(aValue.find("<tree>")!=std::string::npos) {
        //if(aValue==oldXMLTree) return true;
        Lib::ItemMLFactory factory;
        Lib::BaseML treeML(factory,fSession.verboseLevel());
        std::vector<std::string> tags;
        tags.push_back("tree");
        tags.push_back("treeItem");
        treeML.setTags(tags);
        if(treeML.loadString(aValue,fSession.printer())) {
          Lib::ItemML* top = treeML.topItem();
          //top->dump();
          if(top) {
            if (oldXMLTree !="") {
              oldXMLTree = "<tree>"+oldXMLTree+"</tree>";
              Lib::BaseML oldTreeML(factory,fSession.verboseLevel());
              std::vector<std::string> tags2;
              tags2.push_back("tree");
              tags2.push_back("treeItem");
              oldTreeML.setTags(tags2);
              if(oldTreeML.loadString(oldXMLTree,fSession.printer())) {
                Lib::ItemML* old = oldTreeML.topItem();
                if (old) {
                  top->updateTree(*old);
                }
              }
            }
            createTree(widget,top);
          }
        }
      } else {
        std::vector<std::string> items;
        Lib::smanip::lines(aValue,items);
        unsigned int itemn = items.size();
        for(unsigned int index=0;index<itemn;index++){
          new QListViewItem((QListView*)widget,(char*)items[index].c_str());
        }
      }
      //restore old selection
      ((QListView*)widget)->clearSelection();
      std::vector<std::string> items;
      Lib::smanip::lines(selection,items);
      unsigned int itemn = items.size();
      bool found = true;
      for(unsigned int index=0;index<itemn;index++){
        QListViewItem* item = 
          ((QListView*)widget)->findItem(items[index].c_str(),0);
        if(!item) {
          found = false;
          break;
        } else {
          if(index!=itemn-1) {
            ((QListView*)widget)->blockSignals(true);
            item->setOpen(true);
            ((QListView*)widget)->blockSignals(false);
          }
          ((QListView*)widget)->setSelected(item,true);
        }
      }
      // enable callback script
      enableCallbackScript(widget,"select",scripts);
      if(found) return true;
      else ((QListView*)widget)->clearSelection();
      return true;
    } else if(widget->inherits(S_QComboBox)) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      unsigned int itemn = items.size();
      std::string def;
      element->propertyValue("default",def);
      ((QComboBox*)widget)->clear();
      ((QComboBox*)widget)->setCurrentItem(0);
      for(unsigned int index=0;index<itemn;index++) {
        ((QComboBox*)widget)->insertItem(items[index].c_str());
        if(items[index]==def) {
          ((QComboBox*)widget)->setCurrentItem(index);
        }
      }
      return true;
    }
  } else if(aResource=="splitPosition") {
    if(widget->inherits(S_QSplitter)) {
      int value;
      if(Lib::smanip::toint(aValue,value)) {
        QSplitter* w = (QSplitter*)widget;
        QValueList<int> sizes = w->sizes();
        if(sizes.size()==2) {
          sizes[0] = value;
          sizes[1] = (w->orientation()==Qt::Vertical ? 
                      w->height() : w->width());
          sizes[1] -= value;
          w->setSizes(sizes);
        }
        return true;
      }
    }
  } else if(aResource=="popupItems") {
    if(widget->inherits("OnX::QPageViewer")) {
      std::vector<std::string> items;
      Lib::smanip::lines(aValue,items);
      for(unsigned int index=0;index<items.size();index++)
        ((QPageViewer*)widget)->addPopupEntry(items[index]);
      return true;
    }
  } else if(aResource=="tabLabel") { //Constraint resource
    element->setPropertyValue("tabLabel",aValue);
    return true;
  } else if(aResource=="title") {
    widget->topLevelWidget()->setCaption(aValue.c_str());
    return true;
  } else if(aResource=="geometry") {
    int x,y,w,h;
    if(!Lib::smanip::togeometry(aValue.c_str(),x,y,w,h)) {
      Lib::Out out(fSession.printer());
      out << "QtUI::setParameter :"
          << " \"" << aValue << "\" : bad value for geometry." 
          << Lib::endl;
    } else {
#ifdef __APPLE__
      if(y<=23) y = 23;  //To be under Apple menubar.
#endif
      //WARNING : setGeometry positions client area.
      // for mainWindow it's a qtWidget and for QtDialog it'a a qtContainer
      // but qtWidget = qtContainer ...so...
      widget->topLevelWidget()->move(QPoint(x,y));
      widget->topLevelWidget()->resize(w,h);
      return true;
    }
  } else { // Viewer parameter ?
    if(widget->inherits("OnX::QPageViewer")) {
      if(((QPageViewer*)widget)->setParameter(aResource,aValue)) return true;
    }
  }

  Lib::Out out(fSession.printer());
  out << "OnX::QtUI::setParameter :"
      << " unable to set parameter for :"
      << " widget \"" << aWidget << "\""
      << " resource \"" << aResource << "\""
      << " value \"" << aValue << "\"."
      << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::write(
 const std::string& aWidget
,const std::string& aWhat
,const std::string& aFile
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)findNativeWidget(aWidget);
  if(!widget) return false;
  if(!widget->inherits("OnX::QPageViewer")) return false;
  return ((QPageViewer*)widget)->write(aWhat,aFile,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::createTree(
 QWidget* aTree
,Lib::ItemML* aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::list<Lib::ItemML*>& list = aItemML->children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string sopened;
    bool opened = false;

    std::string slabel;
    (*it)->propertyValue("label",slabel);
    if((*it)->propertyValue("opened",sopened)) {
      Lib::smanip::tobool(sopened,opened);
    }

    void* data1;
    void* data2;
    int dataInt;
    (*it)->parent()->getData(data1,data2,dataInt);
    QListViewItem* item = 0;
    if(!data1) {
      item = new QListViewItem((QListView*)aTree,slabel.c_str());
    } else {
      item = new QListViewItem((QListViewItem*)data1,slabel.c_str());
    }

    aTree->blockSignals(true);
    item->setOpen(opened);
    aTree->blockSignals(false);

    (*it)->setData(item,0,0);
    if(!createTree(aTree,*it)) return false;
  }
  return true;
}

//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::createNativeWidget(
 Lib::ItemML& aItemML                      
,const std::string& aClass
,const std::string& aName
,Lib::ItemML* aParent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
    std::string qtClass = getType(aClass);
    if(qtClass=="") {
      Lib::Out out(fSession.printer());
      out << "QtUI::createNativeWidget :"
          << " unknown widget type : " 
          << aClass << Lib::endl;
      return false;
    }
    
    // Get parent :
    QWidget* qtContainer = 0;
    QWidget* qtParent = 0;
    std::string qtParentClass;
    std::string sparentclass;
    if(aParent) {
      qtParent = (QWidget*)aParent->getData1();
      aParent->attributeValue("","class",sparentclass);
      qtParentClass = getType(sparentclass);
    } 

#if QT_VERSION < 0x040000
#define QT_PARENT (qtParent)
#else
#define QT_PARENT (qtParent->inherits(S_QTabWidget)?0:qtParent)
#endif

    // Check parent number of children :
    if (((qtParentClass=="HBox") ||
         (qtParentClass=="VBox") ||
         (qtParentClass=="HPaned") ||
         (qtParentClass=="VPaned")) && 
         (QWidgetGetNumberOfChildren(*qtParent)>2) ) {
      Lib::Out out(fSession.printer());
      out << "QtUI::createNativeWidget :"
          << " could not create a third widget in  : " 
          << sparentclass
          << Lib::endl;
      return false;
    }
    
    // Create the Qt widget :
    std::string svalue;
    std::string sdef;
    QWidget* qtWidget = 0;
    int qtID = 0;

    //FIXME: Use QMetaObject to create widget in a generic way ?

    if(qtClass=="Window") {
      qtParent = 0;
      aItemML.propertyValue("decoration",svalue);
      Qt::WFlags flags = 
        decorationFlags(parseDecorations(svalue),fSession.printer());
      qtWidget = new QMainWindow(qtParent,(char*)aName.c_str(),flags);
      if(fFirstWindow) {
        fQApplication->setMainWidget(qtWidget);
        fFirstWindow = false;
        fWidgets.push_back(qtWidget);
      }
    } else if(qtClass=="CascadeButton") {
      QWidget* parent = 0;
#if QT_VERSION < 0x040000
      QMenuData* menuData = 0;
      if(qtParent->inherits(S_QMenuBar)) {
        parent = ((QMenuBar*)qtParent)->parentWidget();
        menuData = ((QMenuBar*)qtParent);
        qtContainer = qtParent; //Used by setParameter.
      } else if(qtParent->inherits(S_OnX_QMenu)) {
        parent = &((OnX_QMenu*)qtParent)->popupMenu();
        menuData = &((OnX_QMenu*)qtParent)->popupMenu();
        qtContainer = parent; //Used by setParameter.
      } else {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " CascadeButton needs a MenuBar as parent " 
            << "(" << std::string(qtParent->className()) << ") found."
            << Lib::endl;
        return false;
      }
      qtWidget = new QPopupMenu(parent,aName.c_str());
      ((QPopupMenu*)qtWidget)->setCheckable(TRUE);
      aItemML.propertyValue("label",svalue);
      menuData->insertItem(svalue.c_str(),(QPopupMenu*)qtWidget);
#else
      if(qtParent->inherits(S_QMenuBar)) {
        parent = ((QMenuBar*)qtParent)->parentWidget();
        qtContainer = qtParent; //Used by setParameter.
      } else if(qtParent->inherits(S_OnX_QMenu)) {
        parent = &((OnX_QMenu*)qtParent)->popupMenu();
        qtContainer = parent; //Used by setParameter.
      } else {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " CascadeButton needs a MenuBar as parent " 
            << "(" << std::string(qtParent->className()) << ") found."
            << Lib::endl;
        return false;
      }
      qtWidget = new QPopupMenu(parent,aName.c_str());
      ((QPopupMenu*)qtWidget)->setCheckable(TRUE);
      aItemML.propertyValue("label",svalue);
      if(qtParent->inherits(S_QMenuBar)) {
	((QMenuBar*)qtParent)->insertItem
            (svalue.c_str(),(QPopupMenu*)qtWidget);
      } else if(qtParent->inherits(S_OnX_QMenu)) {
	((OnX_QMenu*)qtParent)->popupMenu().insertItem
            (svalue.c_str(),(QPopupMenu*)qtWidget);
      } else {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " problem when creating CascadeButton in " 
            << "(" << std::string(qtParent->className()) << ")."
            << " " << std::string(parent->className())
            << Lib::endl;
        return false;
      }
#endif
    } else if(qtClass=="Menu") {
      if(!qtParent->inherits(S_QPopupMenu)) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " Menu needs a CascadeButton as parent." << Lib::endl;
        return false;
      }
      qtWidget = new OnX_QMenu(aName.c_str(),*((QPopupMenu*)qtParent));
      fWidgets.push_back(qtWidget); //FIXME : should have a parent !
    } else if(qtClass=="MenuItem") {
      if(!qtParent->inherits(S_OnX_QMenu)) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " MenuItem needs a Menu as parent." << Lib::endl;
        return false;
      }
      QPopupMenu& popup = ((OnX_QMenu*)qtParent)->popupMenu();
      qtID = 10000+fMenuItemCounter;
      fMenuItemCounter++;
      qtWidget = new OnX_QMenuItem(aName.c_str(),popup,qtID);
      fWidgets.push_back(qtWidget); //FIXME : should have a parent !
    } else if(qtClass=="ToggleMenuItem") {
      if(!qtParent->inherits(S_OnX_QMenu)) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " ToggleMenuItem needs a Menu as parent." << Lib::endl;
        return false;
      }
      QPopupMenu& popup = ((OnX_QMenu*)qtParent)->popupMenu();
      qtID = fMenuItemCounter;
      fMenuItemCounter++;
      popup.setCheckable(TRUE);
      qtWidget = new OnX_QToggleMenuItem(aName.c_str(),popup,qtID);
      fWidgets.push_back(qtWidget); //FIXME : should have a parent !
    } else if(qtClass=="MenuSeparator") {
      if(!qtParent->inherits(S_OnX_QMenu)) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " MenuSeparator needs a Menu as parent." << Lib::endl;
        return false;
      }
      QPopupMenu& popup = ((OnX_QMenu*)qtParent)->popupMenu();
      popup.insertSeparator();
      qtWidget = new OnX_QMenuItem(aName.c_str(),popup);
      fWidgets.push_back(qtWidget); //FIXME : should have a parent !
    } else if(qtClass=="Dialog") {
      aItemML.propertyValue("decoration",svalue);
      Qt::WFlags flags = 
        decorationFlags(parseDecorations(svalue),fSession.printer());
      flags |= Qt::WType_Dialog;
      qtContainer = new QDialog(qtParent,aName.c_str(),false,flags);
      QBoxLayout* layout = 
        new QBoxLayout(qtContainer,QBoxLayout::TopToBottom);
      std::string name = aName + "_vbox";
      qtWidget = new QVBox(qtContainer,name.c_str());
      layout->addWidget(qtWidget);
    } else if(qtClass=="FileSelection") {
      std::string dirMask;
      if(!aItemML.propertyValue("dirMask",dirMask)) dirMask = "*";
      Lib::smanip::getenv(dirMask);
      std::string directory;
      if(!aItemML.propertyValue("directory",directory)) directory = "";
      Lib::smanip::getenv(directory);
#if QT_VERSION < 0x040000
      qtWidget = new QFileDialog(directory.c_str(),
                                 dirMask.c_str(),
                                 qtParent,aName.c_str(),FALSE);
#else
      qtWidget = new OnX_QFileDialog(aName.c_str());      
      ((OnX_QFileDialog*)qtWidget)->setAcceptMode(OnX_QFileDialog::AcceptOpen);
      ((OnX_QFileDialog*)qtWidget)->setFilter(dirMask);
      ((OnX_QFileDialog*)qtWidget)->setDirectory(directory);
      fWidgets.push_back(qtWidget);
#endif
    } else if(qtClass=="ColorSelection") {
      qtWidget = new OnX_QColorDialog(aName.c_str());      
      fWidgets.push_back(qtWidget);
    } else if(qtClass=="WarningDialog") {
      /*FIXME
      aItemML.propertyValue("decoration",svalue);
      Qt::WFlags flags = 
        decorationFlags(parseDecorations(svalue),fSession.printer());
      flags |= Qt::WType_Dialog;
      */
      qtWidget = new QMessageBox(qtParent,aName.c_str());      
      fWidgets.push_back(qtWidget);
    } else if(qtClass=="MenuBar") {
      QMainWindow* mainWindow = 
        (QMainWindow*)findParent(qtParent,S_QMainWindow);
      if(!mainWindow) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " MenuBar needs a Window as parent " 
            << "(" << std::string(qtParent->className()) << ") found."
            << Lib::endl;
        return false;
      }
      qtWidget = mainWindow->menuBar();
      //qtWidget->setName(aName.c_str());
    } else if(qtClass=="ToolBar") {
      QMainWindow* mainWindow = 
        (QMainWindow*)findParent(qtParent,S_QMainWindow);
      if(!mainWindow) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " ToolBar needs a Window as parent " 
            << "(" << std::string(qtParent->className()) << ") found."
            << Lib::endl;
        return false;
      }
#if QT_VERSION < 0x040000
      qtWidget = new QToolBar(mainWindow,aName.c_str());
      mainWindow->addDockWindow((QDockWindow*)qtWidget);
#else
      qtWidget = new QToolBar(aName.c_str(),(QWidget*)mainWindow);
      mainWindow->addToolBar((QToolBar*)qtWidget);
#endif
    } else if(qtClass=="ToolBarItem") {
      if(!qtParent->inherits(S_QToolBar)) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " ToolBarItem needs a QToolBar as parent." << Lib::endl;
        return false;
      }
#if QT_VERSION < 0x040000
      qtWidget = new QToolButton(qtParent,aName.c_str());
#else
      qtWidget = new QToolButton(0,aName.c_str());
      ((QToolBar*)qtParent)->addWidget(qtWidget);
#endif
    ////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////
    } else if(qtClass=="ScrolledTree") {
      qtWidget = new QListView(QT_PARENT,aName.c_str());
      ((QListView*)qtWidget)->header()->hide();
    } else if(qtClass=="ScrolledList") {
      qtWidget = new QListView(QT_PARENT,aName.c_str());
      ((QListView*)qtWidget)->header()->hide();
    } else if(qtClass=="VBox") {
      qtWidget = new QVBox(QT_PARENT,aName.c_str());
    } else if(qtClass=="HBox") {
      qtWidget = new QHBox(QT_PARENT,aName.c_str());
    } else if(qtClass=="VContainer") {
      qtWidget = new QVBox(QT_PARENT,aName.c_str());
    } else if(qtClass=="HContainer") {
      qtWidget = new QHBox(QT_PARENT,aName.c_str());
    } else if(qtClass=="VPaned") {
      qtWidget = new OnX_QSplitter(Qt::Vertical,QT_PARENT,aName.c_str());
      if(aItemML.propertyValue("splitPosition",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          ((OnX_QSplitter*)qtWidget)->setSplitPosition(value);
        }
      }
    } else if(qtClass=="HPaned") {
      qtWidget = new OnX_QSplitter(Qt::Horizontal,QT_PARENT,aName.c_str());
      if(aItemML.propertyValue("splitPosition",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          ((OnX_QSplitter*)qtWidget)->setSplitPosition(value);
        }
      }      
    } else if(qtClass=="Label") {
      qtWidget = new QLabel(QT_PARENT,aName.c_str());
    } else if(qtClass=="PushButton") {
      qtWidget = new QPushButton(QT_PARENT,aName.c_str());
    } else if(qtClass=="ToggleButton") {
      qtWidget = new QCheckBox(QT_PARENT,aName.c_str());
    } else if(qtClass=="ScrolledText") {
      qtWidget = new QTextView(QT_PARENT,aName.c_str());
      ((QTextView*)qtWidget)->setTextFormat(Qt::PlainText);
      ((QTextView*)qtWidget)->setReadOnly(false);
    } else if(qtClass=="Entry") {
      qtWidget = new QLineEdit(QT_PARENT,aName.c_str());
    } else if(qtClass=="Password") {
      qtWidget = new QLineEdit(QT_PARENT,aName.c_str());
      ((QLineEdit*)qtWidget)->setEchoMode(QLineEdit::Password);
    } else if(qtClass=="CommandLine") {
      qtWidget = new OnX::QCommandLine(QT_PARENT,aName.c_str());
    } else if(qtClass=="Tree") {
      qtWidget = new QListView(QT_PARENT,aName.c_str());
    } else if(qtClass=="OptionMenu") {
      qtWidget = new QComboBox(QT_PARENT,aName.c_str());
      ((QComboBox*)qtWidget)->setEditable(false);
    } else if(qtClass=="ComboBox") {
      qtWidget = new QComboBox(QT_PARENT,aName.c_str());
      ((QComboBox*)qtWidget)->setEditable(true);
    } else if(qtClass=="HScrollBar") {
      qtWidget = new QScrollBar(Qt::Horizontal,QT_PARENT,aName.c_str());
    } else if(qtClass=="VScrollBar") {
      qtWidget = new QScrollBar(Qt::Vertical,QT_PARENT,aName.c_str());
    } else if(qtClass=="HScale") {
      qtWidget = new QSlider(Qt::Horizontal,QT_PARENT,aName.c_str());
    } else if(qtClass=="VScale") {
      qtWidget = new QSlider(Qt::Vertical,QT_PARENT,aName.c_str());
    } else if(qtClass=="ProgressBar") {
      qtWidget = new QProgressBar(QT_PARENT,aName.c_str());
    } else if(qtClass=="TabStack") {
      qtWidget = new QTabWidget(QT_PARENT,aName.c_str());
    } else if(qtClass=="OpenGLArea") {
      qtWidget = new OnX_QOpenGLArea(QT_PARENT,aName.c_str());

#ifdef ONX_HAS_INVENTOR
    } else if(qtClass=="SoExaminerViewer") {
      std::string name = aName + "_frame";
      qtContainer =  new QFrame(QT_PARENT,name.c_str());
      SoQtExaminerViewer* viewer = 
        new SoQtExaminerViewer(qtContainer,aName.c_str(),TRUE);
      qtWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
    } else if(qtClass=="SoPlaneViewer") {
      std::string name = aName + "_frame";
      qtContainer =  new QFrame(QT_PARENT,name.c_str());
      SoQtPlaneViewer* viewer = 
        new SoQtPlaneViewer(qtContainer,aName.c_str(),TRUE);
      qtWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
/*FIXME
    } else if(qtClass=="SoWalkViewer") {
      std::string name = aName + "_frame";
      qtContainer =  new QFrame(QT_PARENT,name.c_str());
      SoQtWalkViewer* viewer = 
        new SoQtWalkViewer(qtContainer,aName.c_str(),TRUE);
      qtWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
*/
    } else if(qtClass=="SoFlyViewer") {
      std::string name = aName + "_frame";
      qtContainer =  new QFrame(QT_PARENT,name.c_str());
      SoQtFlyViewer* viewer = 
        new SoQtFlyViewer(qtContainer,aName.c_str(),TRUE);
      qtWidget = viewer->getWidget();
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) return false;
      SoSeparator* sg = new SoSeparator;
      viewer->setSceneGraph(sg);
      soViewer->setSceneGraph(sg);
    } else if( (qtClass=="PageViewer") ||
               (qtClass=="PlanePageViewer") ){
      std::string vclass = "Examiner";
      if(qtClass=="PlanePageViewer") vclass = "Plane";
      std::string name = aName + "_frame";
      qtContainer =  new QFrame(QT_PARENT,name.c_str());
      qtWidget = new QPageViewer(qtContainer,aName.c_str(),vclass.c_str());
     {Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      setCurrentWidget(widget);}
      Slash::UI::ISoViewer* soViewer = Lib_Cast(aItemML,Slash::UI::ISoViewer);
      if(!soViewer) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget : ItemML not a Slash::UI::ISoViewer."
            << Lib::endl;
        return false;
      }
      SoNode* soNode = ((QPageViewer*)qtWidget)->sceneGraph();
      soViewer->setSceneGraph(soNode);
#endif
    }
    
    if(!qtWidget) return false;
    if(!qtContainer) qtContainer = qtWidget;
    aItemML.setData(qtWidget,qtContainer?qtContainer:qtWidget,qtID);
    
    // Manage :
    //qtWidget->show();

    // Put in parent :
    if(qtParent) {

      if(qtParent->inherits(S_QTabWidget)) {
        if(!aItemML.propertyValue("tabLabel",svalue)) svalue = aName;
        ((QTabWidget*)qtParent)->addTab(qtContainer,svalue.c_str());
      } else if(qtParentClass=="Window") {
        if(qtClass=="ToolBar") {
        } else if(qtClass=="MenuBar") {
        } else if(qtContainer->inherits(S_OnX_QColorDialog)) {
        } else if(qtContainer->inherits(S_QDialog)) {
        } else if(((QMainWindow*)qtParent)->centralWidget()) {
          Lib::Out out(fSession.printer());
          out << "QtUI::createNativeWidget :"
              << " A Window can have only one non-dialog child." 
              << Lib::endl;
        } else {
          ((QMainWindow*)qtParent)->setCentralWidget(qtContainer);
        }
      } else if(qtParentClass=="HBox") {
        std::string sexpand;
        if(!aParent->propertyValue("expand",sexpand)) sexpand = "second";
        QHBoxSetExpand(*((QHBox*)qtParent),sexpand);
      } else if(qtParentClass=="VBox") {
        std::string sexpand;
        if(!aParent->propertyValue("expand",sexpand)) sexpand = "second";
        QVBoxSetExpand(*((QVBox*)qtParent),sexpand);
      }
    }

    // Set resources :
    if(aItemML.propertyValue("background",svalue)) {
      if(qtWidget->inherits(S_QButton)) {
        QButtonSetBackground(*((QButton*)qtWidget),svalue);
      } else if(qtWidget->inherits(S_QComboBox)) {
        QComboBoxSetBackground(*((QComboBox*)qtWidget),svalue);
      } else {
        QWidgetSetBackground(*qtWidget,svalue);
      }
    }
    if(aItemML.propertyValue("foreground",svalue)) {
      QWidgetSetForeground(*qtWidget,svalue);
    }
    if( (qtClass=="Window") || 
        (qtClass=="Dialog") ||
        (qtClass=="WarningDialog") ) {
      if(aItemML.propertyValue("title",svalue)) {
        qtWidget->topLevelWidget()->setCaption(svalue.c_str());
      }
      if(aItemML.propertyValue("geometry",svalue)) {
        int x,y,w,h;
        if(!Lib::smanip::togeometry(svalue,x,y,w,h)) {
          Lib::Out out(fSession.printer());
          out << "QtUI::createNativeWidget :"
              << " \"" << svalue << "\" : bad value for geometry." 
              << Lib::endl;
        } else {
#ifdef __APPLE__
	  if(y<=0) y += 23;  //To be under Apple menubar.
#endif
 	  qtWidget->topLevelWidget()->move(QPoint(x,y));
	  qtWidget->topLevelWidget()->resize(w,h);
	}
      }

    } else if( (qtClass=="MenuItem") || 
               (qtClass=="ToggleMenuItem") ){
      //std::string schecked;
      //aItemML.propertyValue("checked",schecked);

      aItemML.propertyValue("label",svalue);

      QPopupMenu& popup = ((OnX_QMenuItem*)qtWidget)->popupMenu();
      
      // Need a callback to create the item.
      Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      Action& action = addAction(*this,widget,qtWidget,"","","");
      QtCallback* qtCbk = new QtCallback(action);
      fQtCallbacks.push_back(qtCbk);

      int actualID = popup.insertItem(svalue.c_str(),
                                      qtCbk,SLOT(execute()),
                                      0,qtID);
      if(actualID!=qtID) {
        Lib::Out out(fSession.printer());
        out << "QtUI::createNativeWidget :"
            << " create MenuItem : actualID!=qtID"
            << " (" << actualID << "," << qtID << ")."
            << Lib::endl;
        return false;
      }

      addCallbacks(aItemML,&popup,qtID,"activate");
    } else if(qtClass=="Label") {
      if(aItemML.propertyValue("label",svalue) || 
         aItemML.propertyValue("labelString",svalue) ) {
        ((QLabel*)qtWidget)->setText(svalue.c_str());
      }
    } else if(qtClass=="PushButton") {
      if(aItemML.propertyValue("label",svalue)) {
        ((QPushButton*)qtWidget)->setText(svalue.c_str());
      }
      // Callbacks :
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(clicked()));
    } else if(qtClass=="TabStack") {
      addCallbacks(aItemML,qtWidget,
                   "select",SIGNAL(currentChanged(QWidget*)));
    } else if(qtClass=="ToolBarItem") {
      if(aItemML.propertyValue("label",svalue)) {
        ((QToolButton*)qtWidget)->setText(svalue.c_str());
      }
      if(aItemML.propertyValue("tooltip",svalue)) {
        QToolTip::add(qtWidget,svalue.c_str());
      }
      if(aItemML.propertyValue("pixmap",svalue)) {
        std::string fileName = Lib::fmanip::name(svalue);
        QPixmap pixmap;
        pixmap.load(fileName.c_str());
        ((QToolButton*)qtWidget)->setPixmap(pixmap);
      }
      // Callbacks :
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(clicked()));
    } else if(qtClass=="ToggleButton") {
      if(aItemML.propertyValue("label",svalue)) {
        ((QCheckBox*)qtWidget)->setText(svalue.c_str());
      }
      // Callbacks :
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(toggled(bool)));
    } else if(qtClass=="FileSelection") {
#if QT_VERSION < 0x040000
      addCallbacks(aItemML,qtWidget,
                   "ok",SIGNAL(fileSelected(const QString&)));
#else
      addCallbacks(aItemML,qtWidget,"ok",SIGNAL(ok()));
#endif
    } else if(qtClass=="ColorSelection") {
      /*
      std::string sexec;
      std::string script;
      if(aItemML.propertyValue("ok",script))
        aItemML.attributeValue("ok","exec",sexec);
      Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      Action& action = addAction(*this,widget,qtWidget,sexec,script,"ok");
      ((OnX_QColorDialog*)qtWidget)->setAction(&action);
      */
      addCallbacks(aItemML,qtWidget,"ok",SIGNAL(ok()));
    } else if(qtClass=="CommandLine") {
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(returnPressed()));
      addCallbacks(aItemML,qtWidget,"complete",SIGNAL(complete()));
    } else if(qtClass=="ProgressBar") {
      ((QProgressBar*)qtWidget)->setTotalSteps(100);
      ((QProgressBar*)qtWidget)->setProgress(0);
    } else if(qtClass=="ScrolledList") {
      aItemML.propertyValue("selection",sdef);
      if(aItemML.propertyValue("items",svalue)) {
        ((QListView*)qtWidget)->clear();
        if(!((QListView*)qtWidget)->columns()) {
          ((QListView*)qtWidget)->addColumn("");
          //((QListView*)qtWidget)->setRootIsDecorated(true);
        }
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        unsigned int itemn = items.size();
        for(unsigned int index=0;index<itemn;index++){
          QListViewItem* item = new QListViewItem((QListView*)qtWidget,
                                                  (char*)items[index].c_str());
          if(items[index]==sdef) 
            ((QListView*)qtWidget)->setSelected(item,true);
        }
      }
      addCallbacks(aItemML,qtWidget,"select",SIGNAL(selectionChanged()));
    } else if(qtClass=="ScrolledTree") {
      addCallbacks(aItemML,qtWidget,"select",SIGNAL(selectionChanged()));
      addCallbacks(aItemML,qtWidget,"selectBranch",SIGNAL(selectionChanged()));
#if QT_VERSION < 0x040000
      addCallbacks(aItemML,qtWidget,
        "open",SIGNAL(expanded(QListViewItem*)),
               SLOT(execute3(QListViewItem*)));
      addCallbacks(aItemML,qtWidget,
        "close",SIGNAL(collapsed(QListViewItem*)),
               SLOT(execute3(QListViewItem*)));
#else
      addCallbacks(aItemML,qtWidget,
        "open",SIGNAL(expanded(Q3ListViewItem*)),
               SLOT(execute3(Q3ListViewItem*)));
      addCallbacks(aItemML,qtWidget,
        "close",SIGNAL(collapsed(Q3ListViewItem*)),
                SLOT(execute3(Q3ListViewItem*)));
#endif
    } else if(qtClass=="Password") {
      std::string svalue;
      if(aItemML.propertyValue("value",svalue)) {
        ((QLineEdit*)qtWidget)->setText(svalue.c_str());
      }
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(returnPressed()));
    } else if(qtClass=="Entry") {
      std::string svalue;
      if(aItemML.propertyValue("value",svalue)) {
        ((QLineEdit*)qtWidget)->setText(svalue.c_str());
      }
      addCallbacks(aItemML,qtWidget,"activate",SIGNAL(returnPressed()));
    } else if( (qtClass=="HScale") || (qtClass=="VScale") ||
               (qtClass=="HScrollBar") || (qtClass=="VScrollBar") ) {
      int mn = 0;
      if(aItemML.propertyValue("minimum",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          mn = value;
        }
      }
      int mx = 1000;
      if(aItemML.propertyValue("maximum",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          mx = value;
        }
      }
      int val = 0;
      if(aItemML.propertyValue("value",svalue)) {
        int value;
        if(Lib::smanip::toint(svalue,value)) {
          val = value;
        }
      }
      if( (qtClass=="HScale") || (qtClass=="VScale") ) {
        ((QSlider*)qtWidget)->setMinValue(mn);
        ((QSlider*)qtWidget)->setMaxValue(mx);
        ((QSlider*)qtWidget)->setValue(val);
      } else {
        ((QScrollBar*)qtWidget)->setMinValue(mn);
        ((QScrollBar*)qtWidget)->setMaxValue(mx);
        ((QScrollBar*)qtWidget)->setValue(val);
      }
      // Callbacks :
      addCallbacks(aItemML,qtWidget,"drag",SIGNAL(sliderMoved(int)));
    } else if( (qtClass=="ComboBox") || (qtClass=="OptionMenu")) {
      if(aItemML.propertyValue("label",svalue)) {
        //((QComboBox*)qtWidget)->
      }
      aItemML.propertyValue("default",sdef);
      if(aItemML.propertyValue("items",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++) {
          ((QComboBox*)qtWidget)->insertItem(items[index].c_str());
          if(items[index]==sdef) {
            ((QComboBox*)qtWidget)->setCurrentItem(index);
          }
        }
      }
      addCallbacks(aItemML,qtWidget,"valueChanged",SIGNAL(activated(int)));
    } else if(qtClass=="OpenGLArea") {
      addCallbacks(aItemML,qtWidget,"paint",
                   SIGNAL(paint(QCallbackData&)),
                   SLOT(execute2(QCallbackData&)));
      addCallbacks(aItemML,qtWidget,"event",
                   SIGNAL(eventHandler(QCallbackData&)),
                   SLOT(execute2(QCallbackData&)));
    } else if( (qtClass=="PageViewer") ||
               (qtClass=="PlanePageViewer") ){
      if(aItemML.propertyValue("popupItems",svalue)) {
        std::vector<std::string> items;
        Lib::smanip::lines(svalue,items);
        for(unsigned int index=0;index<items.size();index++)
          ((QPageViewer*)qtWidget)->addPopupEntry(items[index]);
      }

      addCallbacks(aItemML,qtWidget,"collect",
                   SIGNAL(collect(QCallbackData&)),
                   SLOT(execute2(QCallbackData&)));
      addCallbacks(aItemML,qtWidget,"popup",
                   SIGNAL(popup(QCallbackData&)),
                   SLOT(execute2(QCallbackData&)));
      addCallbacks(aItemML,qtWidget,"update","","");

      Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
      Action& action2 = addAction(*this,widget,qtWidget,"","","destroy");
      QtCallback* qtCbk2 = new QtCallback(action2);
      fQtCallbacks.push_back(qtCbk2);
      QObject::connect(qtWidget,SIGNAL(destroyed()),
                       qtCbk2,SLOT(destroy()));

    }      

    // Constraint resources

    bool status = createChildren(aItemML);

    // Post children create resources :
    if(qtClass=="Dialog") {
      QDialogDisableEnter(*((QDialog*)qtWidget));
    }

    return status;

}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::destroyWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Destroy sub widgets :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!destroyWidget(*(*it))) return false;
  }
  // Destroy this widget :
  return destroyOneWidget(aItemML);
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::destroyOneWidget(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)aItemML.getData1();
  if(widget) {
    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "QtUI::destroyWidget :"
          << " destroy Qt widget \"" 
          << std::string(widget->name()) << "\"." << Lib::endl;
    }
    fWidgets.remove(widget);
    delete widget;
  }
  QWidget* widgetContainer = (QWidget*)aItemML.getData2();
  if(widgetContainer && (widgetContainer!=widget) ) {
    if(fVerboseLevel) {
      Lib::Out out(fSession.printer());
      out << "QtUI::destroyWidget :"
          << " destroy Qt widget \"" 
          << std::string(widgetContainer->name()) << "\"." << Lib::endl;
    }
    fWidgets.remove(widgetContainer);
    delete widgetContainer;
  }
  aItemML.setData(0,0,0);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QtUI::getCallbackName(
 const QObject& aMeta
,const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if (aName == "activate") {
    if(aMeta.inherits(S_QPushButton)) {
      return SIGNAL(clicked());
    } else if(aMeta.inherits(S_QPopupMenu)) {
      return SIGNAL(clicked());
    } else if(aMeta.inherits(S_QCheckBox)) {
      return SIGNAL(toggled(bool));
    } else if(aMeta.inherits(S_QLineEdit)) {
      return SIGNAL(returnPressed());
    } else if(aMeta.inherits("OnX::QCommandLine")) {
      return SIGNAL(returnPressed());
    }
  } else if (aName == "valueChanged") {
    if(aMeta.inherits(S_QScrollBar)) {
      return SIGNAL(activated(int));
    } else if(aMeta.inherits(S_QComboBox)) {
      return SIGNAL(activated(int));
    }
  } else if (aName == "select") {
    if(aMeta.inherits(S_QTabWidget)) {
      return SIGNAL(currentChanged(QWidget*));
    } else if(aMeta.inherits(S_QListView)) {
      return SIGNAL(selectionChanged());
    }
  } else if (aName == "ok") {
    if(aMeta.inherits(S_QFileDialog)) {
#if QT_VERSION < 0x040000
      return SIGNAL(fileSelected(const QString&));
#else
      return SIGNAL(ok());
#endif
    } else if(aMeta.inherits(S_OnX_QColorDialog)) {
      return SIGNAL(ok());
    }
  }
  // unique callback name
  else if (aName =="complete") return SIGNAL(complete());
  else if (aName =="collect" ) return SIGNAL(collect(QCallbackData&));
  else if (aName =="popup")    return SIGNAL(popup(QCallbackData&));
  else if (aName =="paint")    return SIGNAL(paint(QCallbackData&));
  else if (aName =="event")    return SIGNAL(eventHandler(QCallbackData&));
  else if (aName =="drag")     return SIGNAL(sliderMoved(int));
#if QT_VERSION < 0x040000
  else if (aName =="open")     return SIGNAL(expanded(QListViewItem*));
  else if (aName =="close")    return SIGNAL(collapsed(QListViewItem*));
  else if (aName =="selectBranch") 
    return SIGNAL(selectionChanged(QListViewItem*));
#else
  else if (aName =="open")     return SIGNAL(expanded(Q3ListViewItem*));
  else if (aName =="close")    return SIGNAL(collapsed(Q3ListViewItem*));
  else if (aName =="selectBranch") 
    return SIGNAL(selectionChanged(Q3ListViewItem*));
#endif

  // FIXME
  //  else if (aName =="cancel") return ;
  //  else if (aName =="increment") return ;
  //  else if (aName =="decrement") return ;
  
  return "";
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::deleteCallback(
 Lib::ItemML& aItemML
,const std::string& aNameXML
,const OnX::Action& aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)aItemML.getData1();
  if(!widget) return;

  std::string qtCbkName = getCallbackName(*widget,aNameXML);
  if(qtCbkName=="") return;

  std::string slot;
  if(widget->inherits("OnX::QPageViewer")) {
    slot = SLOT(execute2(QCallbackData&));    
  } else if(widget->inherits(S_OnX_QOpenGLArea)) {
    slot = SLOT(execute2(QCallbackData&));    
  } else {
    slot = SLOT(execute());
  }

  std::vector<QtCallback*>::iterator it;
  for (it=fQtCallbacks.begin();it!=fQtCallbacks.end();++it) {
    OnX::Action& action = (*it)->action();
    if (action.equal(aAction)){
      QObject::disconnect(widget,qtCbkName.c_str(),(*it),slot.c_str());
      delete (*it);
      fQtCallbacks.erase(it);
      return;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
OnX::Action* OnX::QtUI::addCallback(
 Lib::ItemML& aItemML
,const std::string& aNameXML
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  if(!widget) return 0;

  QWidget* qtWidget = (QWidget*)aItemML.getData1();
  if(!qtWidget) return 0;

  std::string qtCbkName = getCallbackName(*qtWidget,aNameXML);
  if(qtCbkName=="") return 0;

  if(qtWidget->inherits(S_OnX_QMenuItem)) { // MenuItem : 

    //FIXME : not tested yet.

    int qtID = aItemML.getDataInt();

    QPopupMenu& popup = ((OnX_QMenu*)qtWidget)->popupMenu();

    Action& action = addAction(*this,
                               widget,
                               &popup,
                               aInterp,
                               aScript,
                               aNameXML,
                               qtID);
    QtCallback* qtCbk = new QtCallback(action);
    fQtCallbacks.push_back(qtCbk);
    popup.connectItem(qtID,qtCbk,SLOT(execute()));
    
    return &action;
  } else {

    Action& action = addAction(*this,widget,qtWidget,aInterp,aScript,aNameXML);
    QtCallback* qtCbk = new QtCallback(action);
    fQtCallbacks.push_back(qtCbk);

    std::string slot;
    if(qtWidget->inherits("OnX::QPageViewer")) {
      slot = SLOT(execute2(QCallbackData&));    
    } else if(qtWidget->inherits(S_OnX_QOpenGLArea)) {
      slot = SLOT(execute2(QCallbackData&));    
    } else {
      slot = SLOT(execute());
    }

    QObject::connect(qtWidget,qtCbkName.c_str(),qtCbk,slot.c_str());

    return &action;
  }
  
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::addCallbacks(
 Lib::ItemML& aItemML
,QWidget* aWidget
,const std::string& aNameXML
,const std::string& aNameGUI
,const std::string& aSlot
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  std::string slot = (aSlot==""?std::string(SLOT(execute())):aSlot);
  const std::list<Lib::Element*> props = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;
  for(it=props.begin();it!=props.end();++it) {
    if(aNameXML==(*it)->name()) {
      std::string sexec;
      (*it)->attributeValue("exec",sexec);
      Action& action = addAction(*this,
                                 widget,
                                 aWidget,
                                 sexec,
                                 (*it)->value(),
                                 aNameXML);
      QtCallback* qtCbk = new QtCallback(action);
      fQtCallbacks.push_back(qtCbk);
      if(aNameGUI!="")
        QObject::connect(aWidget,aNameGUI.c_str(),qtCbk,slot.c_str());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::addCallbacks(
 Lib::ItemML& aItemML
,QPopupMenu* aPopup
,int a_qtID
,const std::string& aNameXML
)
//////////////////////////////////////////////////////////////////////////////
// For MenuItems
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = Lib_Cast(aItemML,Slash::UI::IWidget);
  const std::list<Lib::Element*> props = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;
  for(it=props.begin();it!=props.end();++it) {
    if(aNameXML==(*it)->name()) {
      std::string sexec;
      (*it)->attributeValue("exec",sexec);
      Action& action = addAction(*this,
                                 widget,
                                 aPopup,
                                 sexec,
                                 (*it)->value(),
                                 aNameXML,
                                 a_qtID);
      QtCallback* qtCbk = new QtCallback(action);
      fQtCallbacks.push_back(qtCbk);
      aPopup->connectItem(a_qtID,qtCbk,SLOT(execute()));
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
int OnX::QtUI::getCompound(
 const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QtUI::getType(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if(aName=="Window")         return aName;
  else if(aName=="VBox")           return aName;
  else if(aName=="HBox")           return aName;
  else if(aName=="VPaned")         return aName;
  else if(aName=="HPaned")         return aName;
  else if(aName=="MenuBar")        return aName;
  else if(aName=="Label")          return aName;
  else if(aName=="PushButton")     return aName;
  else if(aName=="ToggleButton")   return aName;
  else if(aName=="FileSelection")  return aName;
  else if(aName=="ColorSelection") return aName;
  else if(aName=="ScrolledList")   return aName;
  else if(aName=="ScrolledText")   return aName;
  else if(aName=="ScrolledTree")   return aName;
  else if(aName=="Entry")          return aName;
  else if(aName=="Password")       return aName;
  else if(aName=="CommandLine")    return aName;
  else if(aName=="Dialog")         return aName;
  else if(aName=="VContainer")     return aName;
  else if(aName=="HContainer")     return aName;
  else if(aName=="ComboBox")       return aName;
  else if(aName=="HScrollBar")     return aName;
  else if(aName=="VScrollBar")     return aName;
  else if(aName=="HScale")         return aName;
  else if(aName=="VScale")         return aName;
  else if(aName=="PageViewer")     return aName;
  else if(aName=="PlanePageViewer")return aName;
  else if(aName=="ToolBar")        return aName;
  else if(aName=="ToolBarItem")    return aName;
  else if(aName=="ProgressBar")    return aName;
  else if(aName=="TabStack")       return aName;

  else if(aName=="CascadeButton")  return aName;
  else if(aName=="Menu")           return aName;
  else if(aName=="MenuItem")       return aName;
  else if(aName=="ToggleMenuItem") return aName;
  else if(aName=="MenuSeparator")  return aName;
  else if(aName=="OptionMenu")     return aName;
  else if(aName=="OpenGLArea")     return aName;
  else if(aName=="WarningDialog")  return aName;

  // Inventor viewers :
  else if(aName=="SoExaminerViewer") return aName;
  else if(aName=="SoPlaneViewer")    return aName;
  //else if(aName=="SoFlyViewer")      return aName;
  //else if(aName=="SoWalkViewer")   return aName;

  return "";
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnX::QtUI::showAll(
 Lib::ItemML& aItemML                      
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)aItemML.getData2();
  if(widget) {
    // If show property not here, we show the panel.
    // If show is here and is not "no", we show the panel.
    std::string sshow;
    if( !aItemML.propertyValue("show",sshow) || (sshow!="no") ) {
      QWidget* shell = widget->topLevelWidget();
      if(shell) shell->show();
    }
  } else {
    const std::list<Lib::ItemML*>& list = aItemML.children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) showAll(*(*it));
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
Slash::UI::ICyclic* OnX::QtUI::createCyclic(
 const std::string& aName
,int aDelay
,const std::string& aInterp
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = 
    new QtCyclic(fSession,aName,aDelay,aInterp,aScript);
  fCyclics.push_back(cyclic);
  return cyclic;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QtUI::widgetName(
 Lib::ItemML& aItemML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QWidget* widget = (QWidget*)aItemML.getData1();
  if(!widget) return "";
  const char* cname = widget->name();
  return std::string(cname?cname:"");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
QObject* findParent(
 QObject* aObject
,const std::string& aClass
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  QObject* object = aObject;
  while(1) {
    if(!object) return 0;
    if(object->inherits(aClass.c_str())) return object;
    object = object->parent();
  }
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include <OnX/Core/Term.h>

namespace OnX {
class TermCyclic 
:public QtCyclic
,public Term {
public: //Slash::UI::ICyclic
  virtual void execute() { cyclicExecute(); }
public:
  TermCyclic(Slash::UI::IUI& aUI,
             const std::string& aPrompt,
             const std::string& aInterp)
  // Passing 0 millisec will process when idling.
  :QtCyclic(aUI.session(),"onx_terminal",10,"","") //10 is millisec.
  ,Term(aUI,aPrompt,aInterp)
  {}
  virtual ~TermCyclic() {}
public:
  virtual std::string ask(const std::string& aPrompt) {
    //return Term::ask(aPrompt);
    initPrompt(aPrompt);
    while(true) {    
/*
      bool is_Qt;
      if(!is_there_Qt_input(fUI.appContext(),is_X11)) { //FIXME
        Lib::Out out(fXtUI.session().printer());
        out << "OnX::TermCyclic::ask :"
            << " problem with X11 select."
            << Lib::endl;
        return "";
      }
      if(is_Qt) {
        fUI.synchronize();
      } else {
*/
        fUI.synchronize();

        bool is_stdin;
        if(!is_there_stdin_input(is_stdin)) {
          Lib::Out out(fUI.session().printer());
          out << "OnX::TermCyclic::ask :"
              << " problem with stdin select."
              << Lib::endl;
          return "";
        }
        if(is_stdin) {
          eKind kind;
          std::string cmd;
          if(!getInput(kind,cmd)) {
            Lib::Out out(fUI.session().printer());
            out << "OnX::TermCyclic::ask :"
                << " problem to get input."
                << Lib::endl;
            return "";
          }
          if(kind==input) {
            //initPrompt(fPrompt);
            return cmd;
          } else if(kind==escape) {
            //initPrompt(fPrompt);
            //return cmd;
          } //no_input, continue.
        }
      } 
//    }
  }
};
}

//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::enableTerminal(
 const std::string& aPrompt
,const std::string& aInterp
)
//////////////////////////////////////////////////////////////////////////////
//WARNING : it assumes that stdout is directed to the terminal !
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fTermCyclic) return true;
  fTermCyclic = new TermCyclic(*this,aPrompt,aInterp);
  if(!fTermCyclic->isValid()) {
    delete fTermCyclic;
    fTermCyclic = 0;
    return false;
  }
  fTermCyclic->start();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnX::QtUI::disableTerminal(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTermCyclic) return true;
  delete fTermCyclic;
  fTermCyclic = 0;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string OnX::QtUI::ask(
 const std::string& aPrompt
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fAskToTerminal && fTermCyclic) {
    return fTermCyclic->ask(aPrompt);
  } else {
    return BaseUI::ask(aPrompt);
  }
}
/*
bool is_there_Qt_input(bool& aIs) {
  if(fEventLoopStack.empty()) return;
  QEventLoop* eventLoop = fEventLoopStack.front();
  eventLoop->processEvents();
  return true;
}
*/
