#ifndef OnX_SDL_Viewer_h
#define OnX_SDL_Viewer_h

// inheritance :
#include <Slash/UI/IWidgetClass.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>

#include <Lib/smanip.h>
#include <Lib/Cast.h>

#include <SDL_mouse.h>

#include <HEPVis/SDL/viewers/SoSDL_Viewer.h>
#include <HEPVis/nodekits/SoRegion.h>

namespace OnX {

class SDL_ViewerClass : public virtual Slash::UI::IWidgetClass {
public: //Slash::UI::IWidgetClass
  virtual std::string name() const {
    return "OnX::SDL_ViewerClass";
  }
  virtual std::vector<std::string> attributes() const {
    return std::vector<std::string>();
  }
  virtual std::vector<std::string> children() const {
    return std::vector<std::string>();
  }
  virtual std::vector<std::string> properties() const {
    return std::vector<std::string>();
  }
  virtual std::vector<std::string> callbacks() const {
    return std::vector<std::string>();
  }
  virtual void setAttributes (const std::vector<std::string>&) {
  }
public:
  SDL_ViewerClass(){}
  virtual ~SDL_ViewerClass(){}
};

class SDL_Viewer 
:public virtual Slash::UI::IWidget
,public virtual Slash::UI::ISoViewer {
public: //Slash::UI::IWidget
  virtual const Slash::UI::IWidgetClass& widgetClass() const {
    return fWidgetClass;
  }
  virtual std::string name() const {
    return "";
  }
  virtual void* nativeWidget() const {
    return 0;
  }
  virtual bool executeCallbacks(const std::string&) {
    return false;
  }
  virtual bool size(unsigned int&,unsigned int&) {
    return false;
  }
  virtual bool write(const std::string& file,
                     const std::string& format,
                     const std::string& options = "") {
    return false;
  }
  virtual bool setParameter(const std::string& aWhat,
                            const std::string& aValue) {
    if(aWhat=="deltaRotation") {
      float value;
      if(!Lib::smanip::tofloat(aValue,value)) return false;
      fSoSDLViewer.setDefaultDeltaRotation(value);
    } else if(aWhat=="deltaTranslation") {
      float value;
      if(!Lib::smanip::tofloat(aValue,value)) return false;
      fSoSDLViewer.setDefaultDeltaTranslation(value);

    } else if(aWhat=="showCursor") {
      bool value;
      if(!Lib::smanip::tobool(aValue,value)) return false;
      SDL_ShowCursor(value?1:00);

    } else if(aWhat=="showOptionsRegion") {
      bool value;
      if(!Lib::smanip::tobool(aValue,value)) return false;
      SoRegion* soRegion = fSoSDLViewer.optionRegion();
      if(soRegion) soRegion->visible.setValue(value?TRUE:FALSE);
    }
    return false;
  }
  virtual bool parameterValue(const std::string&,std::string&) {
    return false;
  }
public: //Slash::UI::ISoViewer
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnX::SDL_Viewer)
    else if_Lib_SCast(Slash::UI::ISoViewer)
    else if_Lib_SCast(Slash::UI::IWidget)
    else return 0;
  }
  virtual SoNode* sceneGraph() const {
    return fSoNode;
  }
  virtual void setSceneGraph(SoNode* aNode) { 
    fSoNode = aNode;
  }
  virtual void setBackgroundColor(double aRed,double aGreen,double aBlue) {
  }
  virtual void setFeedbackVisibility(bool aValue) {
  }
  virtual void setAutoClipping(bool aValue) {
  }
  virtual void setDecoration(bool aValue) {
  }
  virtual void setViewing(bool aValue) {
  }
  virtual void setHeadlight(bool aValue) {
  }
  virtual void render() {
    fSoSDLViewer.render();
  }
  virtual void viewAll() {
    fSoSDLViewer.viewAll();
  }
public:
  SDL_Viewer( ::HEPVis::SoSDL_Viewer& aViewer)
  :fSoSDLViewer(aViewer)
  ,fSoNode(0){}
  virtual ~SDL_Viewer(){}
private:
  SDL_ViewerClass fWidgetClass;
  ::HEPVis::SoSDL_Viewer& fSoSDLViewer;
  SoNode* fSoNode;
};

}

#endif
