//////////////////////////////////////////////////////////////////////////////
/// Pre SWIG /////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
/// Code to wrap /////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

%include <OnX/Core/Main.h>

//////////////////////////////////////////////////////////////
/// Post SWIG ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

%inline {

// Tcl :
// In OnXTcl/Tcl.cxx we define the OnX_sessionPointer
// Tcl command that returns on a string the 
// session address. With the below, we can
// reconvert it to a SWIG wrapped pointer in the
// OnX/scripts/Tcl/init.tcl script.
Slash::Core::ISession* OnX_cast_ISession(const std::string& aString) {
  void* p;
  // Read with %lx and, if failure, with %lu.
  if(!Lib::smanip::topointer(aString,p)) return 0;
  return (Slash::Core::ISession*)p;
}

// To test dynammic loading :
void OnX_SWIG_Hello() {printf("OnX SWIG Hello\n");}

// To test std::vector<std::string> to native types :
std::vector<std::string> OnX_test_std_vector_std_string(const std::vector<std::string>& aVector) {
  return aVector;
}

// To test std::vector<double> to native types :
std::vector<double> OnX_test_std_vector_double(const std::vector<double>& aVector) {
  return aVector;
}

}

#ifdef SWIGPYTHON

%inline {

void set_PyOS_InputHook(Slash::UI::IUI& aUI) {
  if(PyOS_InputHook==NULL) {
    //see OnX_SWIG_Python.cxx.
    save_PyOS_InputHook = NULL;
    PyOS_InputHook = OnX_PyOS_InputHook;
    sUI = &aUI;
  } else if(PyOS_InputHook==OnX_PyOS_InputHook) {
    sUI = &aUI;
  } else {
    aUI.session().printer().write
       ("OnX::set_PyOS_InputHook : PyOS_InputHook already used. Take control anyway.\n");
    save_PyOS_InputHook = PyOS_InputHook;
    PyOS_InputHook = OnX_PyOS_InputHook;
    sUI = &aUI;
  }
}
void reset_PyOS_InputHook() {
  PyOS_InputHook = save_PyOS_InputHook;
  sUI = 0;
}

}


%pythoncode {
#print 'debug : OnX_SWIG_OnX.i'

def sys_import(aModule):
  import sys
  if sys.modules.has_key(aModule):
    reload(sys.modules[aModule])
  else:
    exec('import %s' % aModule)

# WARNING : take care of having defined no "x" variable before the below ; else
#           a OnX.x('<module>') will not call the below !
def x(aModule): # very short cut.
  sys_import(aModule)

def xn(aModule,aCount):
  import OnX
  for i in range(0,aCount):
    OnX.x(aModule)

def import_py(a_dir,a_name):
  # Example : OnX.import_file('$ONXROOT/examples/Python','OnX_exa_detector')
  if fmanip.exists(a_name+'.pyc') == 0 : 
    # Have a local copy of the .py file. 
    # Else the .pyc may be attempted to be created on a 
    # non-writable area (for example the one of the release area).
    if fmanip.exists(a_name+'.py') == 0 : 
      if fmanip.exists(a_dir+'/'+a_name+'.py') == 0 : 
        print 'File '+a_dir+'/'+a_name+'.py does not exist.'
        return
      else :
        import compiler
        if fmanip.copy(fmanip.name(a_dir+'/'+a_name+'.py'),'.') == 0 :
          print 'Copy of file '+a_dir+'/'+a_name+'.py failed.'
          return
        compiler.compileFile(a_name+'.py')
        fmanip.remove(a_name+'.py')
    else :
      import compiler
      compiler.compileFile(a_name+'.py')
  sys.path.append('') #THE trick to load from current directory.
  sys_import(a_name)
  del sys.path[-1]

def x_exa(a_name):
  import_py('$ONXROOT/examples/Python',a_name)

def dumpMain():
  import sys
  print dir(sys.modules['__main__'])

def exists(name):
  try:
    eval(name)
    return 1
  except NameError:
    return 0

def sys2onx():
  # Use this function to redirect properly the output to the "UI console".  
  # On some platform (Darwin,Windows), if the OnX::Session redirect 
  # std[out,err] to session.printer (by using the OnX::Session::stdoutToOnX()
  # method), the Python output does not follow. Doing a :
  #   import OnX;OnX.sys2onx() 
  # on a callback, redirect the Python output properly.
  import sys
  sys.stdout = session().printer() 
  sys.stderr = session().printer()

def save2sys():
  import sys
  sys.stdout = onx_stdout_save
  sys.stderr = onx_stderr_save

# Only actions done in this module.
import sys
onx_stderr_save = sys.stderr
onx_stdout_save = sys.stdout

def session():
  import sys
  try:
    import OnX_PythonManager # could fail if the OnX Python driver not loaded.
    # The below could fail even if the OnX_PythonManager module exists.
    # This happens when OnX is deleted. This induces the
    # destruction of the OnX::PythonManager
    # that will remove the "sessionPointer" function from
    # the OnX_PythonManager module dictionary.
    string_session = OnX_PythonManager.sessionPointer()
    #print string_session
    return OnX_cast_ISession(string_session)
  except: # case of OnX having not yet loaded the Python interpreter.
    return None
    
def main(aSteer = 1,aThreaded = 0):
  import sys
  args = sys.argv
  #args.append('-stdout')
  #args.append('-from=Python') #Used in OnX_LeaveSteering.
  #args.append('-verbose')
  if aThreaded == 1: args.append('-thread')

  create_UI = 0   # Do not create UI now ; setup Python scripting before.
  show_UI = 0
  onx_main = Main(create_UI,show_UI,args)

  #FIXME : restore Python from GUI in case of thread.
  #FIXME : we should %extend Main to compell Python loading.
  #FIXME : or use '-from=Python' to load OnX-Python in Main() ?
  if aThreaded == 0:
    onx_main.loadInterpreter('Python');

  # Create ONXFILE UI :
  ui = onx_main.createUI('','',aSteer) # if not steering, do not show GUI.

  if ui == None :
    print 'Cannot create GUI.'
    del onx_main
    return None

  if aSteer == 1:
    if aThreaded == 1:
      # never exit the below loop.
      import time
      while 1:
        ui.synchronize()
        time.sleep( 0.01 )
    else:
      ui.steer()

  return onx_main

def run(aSteer = 1,aThreaded = 0):
  if aThreaded == 1:
    import thread
    thread.start_new_thread(main,(aSteer,aThreaded,)) # non blocking.
    return None
  else:
    return main(aSteer,0) # blocking

def bg(aUI = None):
  ui = None
  if aUI == None :
    if session() == None :
      print 'session() is None. Pass a IUI in argument.'
      return
    else:
      ui = session().ui()
  else:
    ui = aUI      
  def timer_sync(a_UI):
    # never exit the below loop.
    import time
    while 1:
      a_UI.synchronize()
      time.sleep( 0.01 )
  import thread
  thread.start_new_thread(timer_sync,(ui,)) # non blocking.

def ui_parameterValue(aWhat):
  # Used by palettes that uses Python on their ok callback.
  # See : OnX::BaseUI::createInputDialog(
  return session().ui().parameterValue(aWhat)

}
#endif
