
// this :
#include "OptionMenuP.h"

// Private :
#include <X11/StringDefs.h>

#include <Xm/RowColumn.h>
#include <Xm/PushB.h>

#include <Lib/smanip.h>
#include <OnX/Xt/XtTools.h>

static void InitializeClass(void);
static void InitializeWidget(Widget, Widget,ArgList,Cardinal*);
static void ValueChangedCallback(Widget,XtPointer,XtPointer);

static XtResource resources [] ={
 {(char*)XmNvalueChangedCallback,(char*)XtCCallback,(char*)XtRCallback,
  sizeof(XtCallbackList),
  XtOffset(OptionMenuWidget,optionMenu.valueChangedCallback),
  XtRImmediate,(XtPointer)NULL},
};

#define athis ((OptionMenuWidget)This)->optionMenu

OptionMenuClassRec optionMenuClassRec = {
/* Core Class Part */
{
    (WidgetClass)&xmFormClassRec, /* pointer to superclass ClassRec   */
    (char*)"OptionMenu",             /* widget resource class name       */
    sizeof(OptionMenuRec),             /* size in bytes of widget record   */
    InitializeClass,            /* class_initialize                 */
    NULL,                         /* dynamic initialization           */
    FALSE,                       /* has class been initialized?      */
    InitializeWidget,           /* initialize                       */
    NULL,                        /* notify that initialize called    */
    XtInheritRealize,           /* XCreateWindow for widget         */
    NULL,                             /* widget semantics name to proc map*/
    0,                                 /* number of entries in actions     */
    resources,                  /* resources for subclass fields    */
    XtNumber(resources),        /* number of entries in resources   */
    NULLQUARK,                        /* resource class quarkified        */
    TRUE,                        /* compress MotionNotify for widget */
    TRUE,                        /* compress Expose events for widget*/
    TRUE,                        /* compress enter and leave events  */
    TRUE,                        /* select for VisibilityNotify      */
    NULL,                        /* free data for subclass pointers  */
    XtInheritResize,            /* geom manager changed widget size */
    XtInheritExpose,            /* expose                           */
    NULL,                        /* set subclass resource values     */
    NULL,                        /* notify that SetValues called     */
    XtInheritSetValuesAlmost,   /* SetValues got "Almost" geo reply */
    NULL,                        /* notify that get_values called    */
    XtInheritAcceptFocus,       /* assign input focus to widget     */
    XtVersion,                        /* version of intrinsics used       */
    NULL,                        /* list of callback offsets         */
    XtInheritTranslations,      /* translations                     */        
    XtInheritQueryGeometry,     /* return preferred geometry        */
    XtInheritDisplayAccelerator,/* display your accelerator         */
    NULL                         /* pointer to extension record      */
},
/* Composite Class Part */
{
    XtInheritGeometryManager,   /* geometry manager for children   */
    XtInheritChangeManaged,     /* change managed state of child   */
    XtInheritInsertChild,       /* physically add child to parent  */
    XtInheritDeleteChild,       /* physically remove child         */
    NULL                         /* pointer to extension record     */
},
/* constraint_class fields */
{
    NULL,                         /* constraint resource     */
    0,                            /* number of constraints   */
    sizeof(XmFormConstraintRec),  /* constraint size         */
    NULL,                         /* initialization          */
    NULL,                         /* destroy proc            */
    NULL,                         /* set_values proc         */
    NULL,                         /* extension               */
},
/* manager_class fields   */
{                        
    XtInheritTranslations,                /* translations           */
    NULL,                                 /* syn_resources          */
    0,                                    /* num_syn_resources      */
    NULL,                                 /* syn_cont_resources     */
    0,                                    /* num_syn_cont_resources */
    XmInheritParentProcess,               /* parent_process         */
    NULL,                                 /* extension              */
},
/* bulletin_board_class fields */
{                        
    FALSE,                                /* always_install_accelerators */
    NULL,                                 /* geo_matrix_create  */
    XmInheritFocusMovedProc,              /* focus_moved_proc   */
    NULL,                                 /* extension          */
},

/* form_class fields  */
{   
    NULL,                     /* extension          */
},


/* OptionMenu */
{
    NULL
}
   
};

WidgetClass optionMenuWidgetClass = (WidgetClass) &optionMenuClassRec;

//////////////////////////////////////////////////////////////////////////////
static void InitializeClass(
 void
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
static void InitializeWidget(
 Widget
,Widget This
,ArgList
,Cardinal*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  This->core.width = 0;
  This->core.height = 0;

  athis.pdm = XmCreatePulldownMenu(This,
                                   (char*)"OPTIONMENU_PULLDOWNMENU",0,0);
  Arg args[5];
  XtSetArg(args[0],XmNsubMenuId,athis.pdm);
  XtSetArg(args[1],XmNtopAttachment,XmATTACH_FORM);
  XtSetArg(args[2],XmNbottomAttachment,XmATTACH_FORM);
  XtSetArg(args[3],XmNleftAttachment,XmATTACH_FORM);
  XtSetArg(args[4],XmNrightAttachment,XmATTACH_FORM);
  athis.rowColumn = XmCreateOptionMenu(This,
                                       (char*)"OPTIONMENU_ROWCOLUMN",args,5);
  XtManageChild(athis.rowColumn);

}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OptionMenuInitialize(
 Widget This
,const std::vector<std::string>& aItems
,const std::string& aDefault 
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Arg args[1];
  // Set history to null :
  XtSetArg(args[0],XmNmenuHistory,0);
  XtSetValues(athis.rowColumn,args,1);
  // kill existing pull children :
  std::vector<Widget> items;
  XWidgetGetChildren(athis.pdm,items);
  for(unsigned int index=0;index<items.size();index++) {
    XtDestroyWidget(items[index]);
  }
  // If no option given, create a "blank" push button :
  if(aItems.size()==0) { 
    XmString cps = XmStringLtoRCreate((char*)"",
                                      XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0],XmNlabelString,cps);
    std::string name(XtName(This));
    name += "_1";
    Widget button = 
      XtCreateManagedWidget(name.c_str(),
                            xmPushButtonWidgetClass,athis.pdm,args,1);
    XmStringFree(cps);
    XtSetArg(args[0],XmNmenuHistory,button);
    XtSetValues(athis.rowColumn,args,1);
    XtAddCallback(button,XmNactivateCallback,
                  ValueChangedCallback,(XtPointer)This);
    return;
  }
  // Creaet new pull children :
  Widget whistory  = NULL;
  for(unsigned int count=0;count<aItems.size();count++) { 
    XmString cps = XmStringLtoRCreate((char*)aItems[count].c_str(),
                                      XmSTRING_DEFAULT_CHARSET);
    XtSetArg(args[0],XmNlabelString,cps);
    std::string name;
    Lib::smanip::printf(name,::strlen(XtName(This))+32,"%s_%d",
                  XtName(This),count+1);
    Widget button = 
      XtCreateManagedWidget(name.c_str(),
                            xmPushButtonWidgetClass,athis.pdm,args,1);
    if(whistory==NULL) whistory = button;  // history = first option.
    XmStringFree(cps);
    if( (aDefault!="") && (aDefault==aItems[count])) whistory = button;
    XtAddCallback(button,XmNactivateCallback,
                  ValueChangedCallback,(XtPointer)This);
  }
  if(whistory==NULL) return;
  // Set history :
  XtSetArg(args[0],XmNmenuHistory,whistory);
  XtSetValues(athis.rowColumn,args,1);
}
//////////////////////////////////////////////////////////////////////////////
void OptionMenuSetDefault(
 Widget This
,const std::string& aDefault 
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Widget whistory  = NULL;
  std::vector<Widget> items;
  XWidgetGetChildren(athis.pdm,items);
  for(unsigned int index=0;index<items.size();index++) {
    std::string label = XmGetString(items[index],XmNlabelString);
    if( (aDefault!="") && (aDefault==label)) whistory = items[index];
  }
  if(whistory==NULL) return;
  // Set history :
  Arg args[1];
  XtSetArg(args[0],XmNmenuHistory,whistory);
  XtSetValues(athis.rowColumn,args,1);
}
//////////////////////////////////////////////////////////////////////////////
bool OptionMenuGetOption(
 Widget This
,std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aValue = "";
  Arg args[1];
  Widget whistory;
  XtSetArg(args[0],XmNmenuHistory,&whistory);
  XtGetValues(athis.rowColumn,args,1);
  if(!whistory) return false;
  XmString cps = NULL;
  XtSetArg(args[0],XmNlabelString,&cps);
  XtGetValues(whistory,args,1);
  XmConvertString(cps,aValue);
  XmStringFree(cps);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Widget OptionMenuGetLabel(
 Widget This
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return athis.rowColumn;
}
//////////////////////////////////////////////////////////////////////////////
void ValueChangedCallback(
 Widget
,XtPointer aTag
,XtPointer aData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Widget This = (Widget)aTag;

  OptionMenuCallbackStruct value;

  value.reason = XmCR_VALUE_CHANGED;
  value.event = ((XmAnyCallbackStruct*)aData)->event;

  OptionMenuGetOption(This,value.value);

  XtCallCallbacks(This,XmNvalueChangedCallback,(XtPointer)&value);
}
