/*
 * Example of an OnX client program.
 */

#include <Lib/Printer.h>
#include <Lib/System.h>
#include <Lib/fmanip.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Debug.h>
#include <Lib/Arguments.h>

#include <Lib/InternetSocket.h>

//////////////////////////////////////////////////////////////////////////////
int main (
 int aArgc
,char** aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;

 {Lib::Arguments args(aArgc,aArgv);

  int verbose = 0;
  if(args.isAnArgument("-verbose")) verbose = 1;

  Lib::Out out(printer);

  Lib::InternetSocket client(printer,verbose);
  client.setEnv("ONXDISPLAY");

  if(!client.connect()) {
    out << "OnX_client : "
        << " can't connect to server."
        << Lib::endl;
    return Lib::System::exitFailure();
  }

  // send a "file protocol" :
  if(!client.sendString("OnX::file::Xxx.onx")) {
    out << "OnX_client : "
        << " can't send string to server."
        << Lib::endl;
    return Lib::System::exitFailure();
  }

  // send content of file :
  std::vector<std::string> text;
  Lib::fmanip::read("$ONXROOT/scripts/OnX/OnX.onx",text);
  if(!client.sendText(text)) {
    out << "OnX_client : "
        << " can't send text to server."
        << Lib::endl;
    return Lib::System::exitFailure();
  }

  // Treat events coming from the OnX server :
  while(1) {
    std::string s;
    printf("debug : go to waitString...\n");
    if(!client.waitString(s)) {
      return Lib::System::exitFailure();
    }
    printf("debug : from server : %s\n",s.c_str());

    std::string ONX_PROT_FILE = "OnX::file::";
    if(s.substr(0,ONX_PROT_FILE.size())==ONX_PROT_FILE) {
      std::string filename = 
        s.substr(ONX_PROT_FILE.size(),s.size()-ONX_PROT_FILE.size());
      std::vector<std::string> text;
      if(!Lib::fmanip::read(filename,text)) {
        out << "OnX_client : "
            << " can't find file " << Lib::sout(filename) 
            << Lib::endl;
        return Lib::System::exitFailure();
      }
      if(!client.sendText(text)) {
        out << "OnX_client : "
            << " can't send file " << Lib::sout(filename) << " to server."
            << Lib::endl;
        return Lib::System::exitFailure();
      }
    } else {
       out << "OnX_client :"
           << " receive event : " << Lib::sout(s) << " from server."
           << Lib::endl;
    }
  }

  }
  Lib::Debug::balance(printer);

  return Lib::System::exitSuccess();
}
