#!/bin/sh -f

#
#  To produce Darwin .app.
#
# Usage :
#     csh> source ../cmt/setup.csh
#    (csh> . ../cmt/setup.sh)
#    UNIX> ./mak_OnX_ons_app
#
# To analyse dependancies :
#    Darwin> find ../Darwin/OnX_ons.app/Contents/${dir_lib} -name '*' -exec ottol -L {} \;

if test -z $ONXROOT ;
then
  echo "OnX setup not executed."
  exit
fi

packroot=${ONXROOT}
app=OnX_ons

# Get binary :
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents
/bin/rm -R -f MacOS
mkdir MacOS
cd MacOS
cp ${packroot}/${PLATFORMBIN}/${app}.exe ${app}

# Get libraries :
dir_lib=Frameworks
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents
/bin/rm -R -f ${dir_lib}
mkdir ${dir_lib}
cd ${dir_lib}
# copy dylibs :
cp /sw/lib/libfreetype.6.dylib .
cp /sw/lib/libexpat.0.dylib .
cp ${GL2PSROOT}/${PLATFORMBIN}/libosc_gl2ps.dylib .
cp ${JPEGROOT}/${PLATFORMBIN}/libosc_jpeg.dylib .
cp ${DVI2BITMAPROOT}/${PLATFORMBIN}/libosc_dvi2bitmap.dylib .
cp ${COINGLROOT}/${PLATFORMBIN}/libCoin.dylib .
cp ${COINNEXTSTEPROOT}/${PLATFORMBIN}/libCoinNextStep.dylib .
cp ${HEPVISROOT}/${PLATFORMBIN}/libHEPVisGUI.dylib .
cp ${HEPVISROOT}/${PLATFORMBIN}/libHEPVisPlotter.dylib .
cp ${HEPVISROOT}/${PLATFORMBIN}/libHEPVisDetector.dylib .
cp ${HEPVISROOT}/${PLATFORMBIN}/libHEPVisGeometry.dylib .
cp ${HEPVISROOT}/${PLATFORMBIN}/libHEPVisUtils.dylib .
cp ${LIBROOT}/${PLATFORMBIN}/libLibXML.dylib .
cp ${LIBROOT}/${PLATFORMBIN}/libLibZip.dylib .
cp ${LIBROOT}/${PLATFORMBIN}/libLibUtils.dylib .
cp ${KUIPROOT}/${PLATFORMBIN}/libKUIP.dylib .
cp ${ONXROOT}/${PLATFORMBIN}/libOnXNextStep.dylib .
cp ${ONXROOT}/${PLATFORMBIN}/libOnXInventor.dylib .
cp ${ONXROOT}/${PLATFORMBIN}/libOnXCore.dylib .
# copy bundles :
cp ${ONXROOT}/${PLATFORMBIN}/OnXDLL.bundle .
cp ${ONXROOT}/${PLATFORMBIN}/OnXNextStep.bundle .
cp ${ONXROOT}/${PLATFORMBIN}/OnX.bundle .
cp ${ONXROOT}/${PLATFORMBIN}/OnXPython.bundle .
cp ${ONXROOT}/${PLATFORMBIN}/OnXPythonSWIG.bundle .

swlibs='libfreetype.6.dylib libexpat.0.dylib'

dylibs='libosc_gl2ps.dylib libosc_dvi2bitmap.dylib libosc_jpeg.dylib libLibUtils.dylib libLibXML.dylib libLibZip.dylib libCoin.dylib libCoinNextStep.dylib libHEPVisUtils.dylib libHEPVisDetector.dylib libHEPVisGeometry.dylib libHEPVisPlotter.dylib libHEPVisGUI.dylib libKUIP.dylib libOnXCore.dylib libOnXInventor.dylib libOnXNextStep.dylib'

# Instrument the libraries :
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents/${dir_lib}
for swlibi in $swlibs
do 
  install_name_tool -id @executable_path/../${dir_lib}/${swlibi} ${swlibi}
  for swlibj in $swlibs
  do 
    install_name_tool -change /sw/lib/${swlibj} @executable_path/../${dir_lib}/${swlibj} ${swlibi}
  done
done
for dylibi in $dylibs
do 
  install_name_tool -id @executable_path/../${dir_lib}/${dylibi} ${dylibi}
  for swlibj in $swlibs
  do 
    install_name_tool -change /sw/lib/${swlibj} @executable_path/../${dir_lib}/${swlibj} ${dylibi}
  done
  for dylibj in $dylibs
  do 
    install_name_tool -change ${dylibj} @executable_path/../${dir_lib}/${dylibj} ${dylibi}
  done
done

bundles='OnXDLL.bundle OnXNextStep.bundle OnX.bundle OnXPython.bundle OnXPythonSWIG.bundle'

# Instrument the bundles :
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents/${dir_lib}
for bundlei in $bundles
do 
  install_name_tool -id @executable_path/../${dir_lib}/${bundlei} ${bundlei}
  for swlibj in $swlibs
  do 
    install_name_tool -change /sw/lib/${swlibj} @executable_path/../${dir_lib}/${swlibj} ${bundlei}
  done
  for dylibj in $dylibs
  do 
    install_name_tool -change ${dylibj} @executable_path/../${dir_lib}/${dylibj} ${bundlei}
  done
done

# Instrument the binary :
#set -x
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents/MacOS
binary=${app}
for swlibi in $swlibs
do 
  install_name_tool -change /sw/lib/${swlibi} @executable_path/../${dir_lib}/${swlibi} ${binary}
done
for dylibi in $dylibs
do 
  install_name_tool -change ${dylibi} @executable_path/../${dir_lib}/${dylibi} ${binary}
done

# Copy scripts :
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents/Resources
/bin/rm -R -f OnX
mkdir OnX
cd OnX
cp -R $ONXROOT/scripts .
cp -R $ONXROOT/examples .

# Copy fonts :
cd ${packroot}/${PLATFORMBIN}/${app}.app/Contents/Resources/OnX
/bin/rm -R -f fonts
mkdir fonts
cd fonts
find $TTFPATH -name '*.ttf' -exec cp {} . \;

