// 
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//
// This callback file contains examples over AIDA only.
//


#include <BatchLab/Helpers/AIDA.h>

#include <AIDA/AIDA.h>

#include <Slash/UI/IUI.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/System.h>
#include <Lib/Randoms.h>
#include <Lib/Arguments.h>
#include <Lib/fmanip.h>

#include <math.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLab_Banner(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Arguments args(aUI.session().arguments());
  bool b;
  if(args.find("-banner",b)&&(!b)) return; //For exa from opy.

  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) {
    Lib::Out out(session.printer());
    out << "OnXLab_Banner :" 
        << " aida not found." 
        << Lib::endl;
    return;
  } 

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) {
    Lib::Out out(session.printer());
    out << "OnXLab_Banner :" 
        << " memoryTree not found." 
        << Lib::endl;
    return;
  }

  if(memoryTree->find("h")) memoryTree->rm("h");

  //FIXME : rm fit function : memoryTree->rm("h");

  AIDA::IHistogram1D* h = 
    create_histogram1D(*aida,*memoryTree,
      "h","The famous normal distribution.",100,-5,5);
  if(!h) {
    Lib::Out out(session.printer());
    out << "OnXLab_Banner :" 
        << " can't create histogram." 
        << Lib::endl;
    return;
  }

  Lib::RandomGauss rg = Lib::RandomGauss(0,1);
  for(int index=0;index<10000;index++) h->fill(rg.shoot(),1);

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(!plotter) {
    Lib::Out out(session.printer());
    out << "OnXLab_Banner :" 
        << " can't create plotter." 
        << Lib::endl;
    return;
  }
  plotter->createRegions(1,1,0);
  plotter->setTitle("OnXLab");

  AIDA::IPlotterRegion& region = plotter->currentRegion();

  region.style().xAxisStyle().setLabel("Random gauss");
  region.style().yAxisStyle().setLabel("Entries");

  region.plot(*h);

  AIDA::IFitter* fitter = create_fitter(*aida);
  if(fitter) {
    AIDA::IFitResult* fitResult = fitter->fit(*h,"G");
    if(fitResult) {
      AIDA::IFunction& function = fitResult->fittedFunction();
      region.plot(function);
      delete fitResult;
    }
    delete fitter;
  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
/// OnXLab/scripts/OnX/Examples.onx //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_0(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Rio file example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Cleanup memoryTree :
  if(!memoryTree->rmdir("/")) return;

  // If already loaded close (not delete) the file :
  std::string fileName = 
    Lib::fmanip::name("$ONXLABROOT/examples/data/analysis.root");

  session.destroyManager(fileName);
  
  AIDA::ITree* tree = create_tree(*aida,fileName,"ROOT",true,false);
  if(!tree) return;

  tree->ls();

  // Create some factories to work with the tree :
  AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
  if(!tupleFactory) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) {
    delete tupleFactory;
    return;
  }

  plotter->setTitle("Rio example");
  //plotter->titleStyle().textStyle().setColor("blue");
  //plotter->titleStyle().textStyle().setFont("couri");

  plotter->createRegions(2,2,0);
    
  ///////////////////////////////////////////////////////////////////////////
  //First region :
  // A plot with two histograms.  
  ///////////////////////////////////////////////////////////////////////////
  {
    if(!tree->cd("histograms")) {
      Lib::Out out(session.printer());
      out << "OnXLab_Examples_0 :" 
          << " can't cd to \"histograms\"." 
          << Lib::endl;
    } else {

      tree->ls();

      AIDA::IPlotterRegion& region = plotter->currentRegion();
    
      AIDA::IHistogram1D* BW = find_histogram1D(*tree,"BW");
      AIDA::IHistogram1D* gauss = find_histogram1D(*tree,"gauss");
    
      // Plot :
      region.style().titleStyle().textStyle().setColor("red");
      region.style().titleStyle().textStyle().setFont("couri");

      region.style().yAxisStyle().tickLabelStyle().setColor("red");
      region.style().yAxisStyle().tickLabelStyle().setFont("couri");

      region.style().yAxisStyle().labelStyle().setColor("red");
      region.style().yAxisStyle().labelStyle().setFont("couri");

      //FIXME : Have to do the below with AIDA styles :
      region.setParameter("plotter.binsStyle[0].color","0 1 0");
      region.setParameter("plotter.binsStyle[0].modeling","boxes");
      region.setParameter("plotter.binsStyle[1].color","1 0 0");
      region.setParameter("plotter.binsStyle[1].modeling","wire_boxes");
  
      if( BW && gauss) {
        region.plot(*gauss); //Behind (fat). Green. (main)
        region.plot(*BW);    //In front (slim). Red.
      }

    }
  }

  ///////////////////////////////////////////////////////////////////////////
  // Second region :
  //  A scatter plot.
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();
    AIDA::IHistogram2D* gauss_BW = find_histogram2D(*tree,"gauss_BW");
    region.setParameter("infosRegionVisible","TRUE");
    region.setParameter("plotter.colorMapping","black 1 cyan 2 green 4 orange");
    if(gauss_BW) {
      region.plot(*gauss_BW);
    } else {
      Lib::Out out(session.printer());
      out << "OnXLab_Examples_0 :" 
          << " gauss_BW not found." << Lib::endl;
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  // Third region :
  // Get a tuple, create an histo from one of its column, plot it. 
  /////////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    if(!tree->cd("..")) {
      Lib::Out out(session.printer());
      out << "OnXLab_Examples_0 :" 
          << " can't cd to \"..\"." << Lib::endl;
    } else if(!tree->cd("tuples")) {
      Lib::Out out(session.printer());
      out << "OnXLab_Examples_0 :" 
          << " can't cd to \"tuples\"." << Lib::endl;
    } else {

      tree->ls();
    
      AIDA::ITuple* tuple1 = tupleFactory->create("tuple1","","");
      AIDA::IHistogram1D* t1_v2 = 
        create_histogram1D(*aida,*memoryTree,"t1_v2","tuple1/v2",10,0,10);
      AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("v2");
      AIDA::IFilter* filter = tupleFactory->createFilter("");
      tuple1->project(*t1_v2,*evaluator,*filter);
      delete evaluator;
      delete filter;

      region.plot(*t1_v2);
    }
  }

  ////////////////////////////////////////////////////////////////////////////
  // Fourth region :
  // Get a tuple, create an histo from one of its column, plot it. 
  ////////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    AIDA::ITuple* tuple2 = tupleFactory->create("tuple2","","");
    //tuple2->prnt();

    AIDA::IHistogram1D* t2_gauss = 
      create_histogram1D
      (*aida,*memoryTree,"t2_gauss","tuple2/rand_gauss",100,-5,5);
    AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("rand_gauss");
    std::string cuts("(fabs(rand_gauss)<=2.)&&(step>=1.)");
    AIDA::IFilter* filter = tupleFactory->createFilter(cuts);
    tuple2->project(*t2_gauss,*evaluator,*filter);
    delete evaluator;
    delete filter;

    region.plot(*t2_gauss);

    //////////////////////////////////////////////////////////////////////////
    // Fitting :
    //////////////////////////////////////////////////////////////////////////
    AIDA::IFitter* fitter = create_fitter(*aida);
    if(fitter) {
      AIDA::IFitResult* fitResult = fitter->fit(*t2_gauss,"G");
      if(fitResult) {
        AIDA::IFunction& function = fitResult->fittedFunction();
        region.plot(function);
        delete fitResult;
      }
      delete fitter;
    }
  }

  delete plotter;

  delete tupleFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_1(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Fitting example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get facilities :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IFitter* fitter = create_fitter(*aida);
  if(!fitter) return;

  plotter->setTitle("Fitting examples");
  plotter->createRegions(2,3,0);
    
  /////////// Gauss ///////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();

    Lib::RandomGauss r(0,1);
    AIDA::IHistogram1D* hgauss = 
      create_histogram1D(*aida,*memoryTree,"hgauss","Rand gauss",100,-5,5);
    if(!hgauss) return;

    for(int index=0;index<10000;index++)  hgauss->fill(r.shoot(),1);

    region.setTitle("Gaussian fit.");
    region.setParameter("plotter.titleAutomated","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");
    region.plot(*hgauss);

    AIDA::IFitResult* fitResult = fitter->fit(*hgauss,"G");
    if(fitResult) {
      AIDA::IFunction& fitFunction = fitResult->fittedFunction(); 
      region.plot(fitFunction);
      delete fitResult;
    }
  }

  /////////// Breit Wigner ///////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();
    AIDA::IHistogram1D* hbw = 
      create_histogram1D(*aida,*memoryTree,"hbw","Rand BW",100,-5,5);
    if(!hbw) return;

    Lib::RandomBreitWigner rbw(0,1);

    for(int index=0;index<10000;index++)  hbw->fill(rbw.shoot(),1);

    region.setTitle("Breit-Wigner fit.");
    region.setParameter("plotter.titleAutomated","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");
    region.plot(*hbw);

    AIDA::IFitResult* fitResult = fitter->fit(*hbw,"BW");
    if(fitResult) {
      AIDA::IFunction& fitFunction = fitResult->fittedFunction();
      region.plot(fitFunction);
      delete fitResult;
    }
  }

  /////////// Exponential ///////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    AIDA::IHistogram1D* hexp = 
      create_histogram1D(*aida,*memoryTree,"hexp","Exp",100,0,10);
    if(!hexp) return;

/*
    AIDA::IFunction* func = 
      create_function(*aida,*memoryTree,"exponential","E");
    if(!func) return;

    std::vector<double> params;
    params.push_back(0.4);
    params.push_back(1);
    func->setParameters(params);

    for(int index=0;index<100;index++)  {
      double x = index * 0.1 - 5 + 0.05;
      std::vector<double> args;
      args.push_back(x);
      hexp->fill(x,func->value(args));
    }
*/

    Lib::RandomExponential rexp(0.5); //0.5*exp(-x*0.5) //rate=0.5

    for(int index=0;index<10000;index++)  hexp->fill(rexp.shoot(),1);

    region.setTitle("Exponential fit.");
    region.setParameter("plotter.titleAutomated","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");
    region.plot(*hexp);

    AIDA::IFitResult* fitResult = fitter->fit(*hexp,"E");
    if(fitResult) {
      AIDA::IFunction& fitFunction = fitResult->fittedFunction();
      region.plot(fitFunction);
      delete fitResult;
    }
  }

  /////////// Polynomial ///////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    AIDA::IFunction* func = 
      create_function(*aida,*memoryTree,"p2","Polynomial 2","P2");
    if(!func) return;

    std::vector<double> params;
    params.push_back(1);
    params.push_back(2);
    params.push_back(3);
    func->setParameters(params);

    AIDA::IHistogram1D* hpol = 
      create_histogram1D(*aida,*memoryTree,"hp2","P2",100,-5,5);
    if(!hpol) return;

    for(int index=0;index<100;index++)  {
      double x = index * 0.1 - 5 + 0.05;
      std::vector<double> args;
      args.push_back(x);
      hpol->fill(x,func->value(args));
    }

    region.setTitle("Polynomial fit.");
    region.setParameter("plotter.titleAutomated","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");
    region.plot(*hpol);

    AIDA::IFitResult* fitResult = fitter->fit(*hpol,"P2");
    if(fitResult) {
      AIDA::IFunction& fitFunction = fitResult->fittedFunction();
      region.plot(fitFunction);
      delete fitResult;
    }
  }

  /////////// Gauss + pol2 /////////////////////////////////
  {
    //  Trickier fit. An expression is given using the usual 
    // mathematical functions (cos, exp, ...) and using 
    // the functions :
    //   gauss(x,H,M,W)    :  H * exp(-0.5 * ((x - M)/W)**2 );
    //   breit(x,H,M,W)    :  H/(1. + (2 * (x - M)/W)**2 );
    //   expo(x,A,B)       : exp(A + B x)
    //   pol1(x,A,B)       : A + B x
    //   pol2(x,A,B,C)     : A + B x + C x**2 
    //   pol3(x,A,B,C,D)   : A + B x + C x**2 + D x**3
    //   pol4(x,A,B,C,D,E) : A + B x + C x**2 + D x**3 + E x**4 
    //  These functions have parameters that are going to be 
    // used as fitting parameters.

    AIDA::IHistogram1D* hgauss_pol2 = 
      create_histogram1D(*aida,*memoryTree,"hgauss_pol2","Rand gauss",100,-5,5);
    if(!hgauss_pol2) return;

    Lib::RandomGauss r(0,1);
    for(int index=0;index<10000;index++)  {
      hgauss_pol2->fill(r.shoot(),1);
    }

    AIDA::IPlotterRegion& region = plotter->next();
    region.setTitle("Scripted fit.");
    region.setParameter("plotter.titleAutomated","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");
    region.plot(*hgauss_pol2);

    AIDA::IFunction* func = 
      create_scripted_function(*aida,*memoryTree,"scripted",1,"gauss(x,H,M,W)+pol2(x,A,B,C)");
    if(!func) {
      aUI.echo("Can't create scripted function.");
    } else {
      // Starting values :
      std::vector<double> params;
      params.push_back(400); // H
      params.push_back(0);   // M
      params.push_back(4);   // W
      params.push_back(1);   // A
      params.push_back(0);   // B
      params.push_back(-1);  // C
      func->setParameters(params);

      AIDA::IFitResult* fitResult = fitter->fit(*hgauss_pol2,*func);
      if(fitResult) {
        AIDA::IFunction& fitFunction = fitResult->fittedFunction();
        region.plot(fitFunction);
        delete fitResult;
      }
    }

  }

  delete fitter;
  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_2(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Histogram operations example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Factories :
  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  // Get facilities :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->createRegions(2,2,0);
  plotter->setTitle("Histo arithmetic");

  AIDA::IHistogram1D* BW = 
    histogramFactory->createHistogram1D("BW","Rand BW",100,-4,4);

  Lib::RandomBreitWigner r(0,1);
  for(int index = 0;index<10000;index++) BW->fill(r.shoot(),1);

  AIDA::IHistogram1D* h_add = histogramFactory->add("h_add",*BW,*BW);
  h_add->setTitle("BW+BW");
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.setParameter("color","0.8 0.8 0.8");
    region.setParameter("viewportRegion.borderColor","0 1 0");
    region.setParameter("viewportRegion.horizontalBorder","2");
    region.setParameter("viewportRegion.verticalBorder","2");
    region.plot(*h_add);
  }

  AIDA::IHistogram1D* h_div = histogramFactory->divide("h_div",*BW,*BW);
  h_div->setTitle("BW/BW");
  {
    AIDA::IPlotterRegion& region = plotter->next();
    region.plot(*h_div);
  }

  AIDA::IHistogram1D* h_mul = histogramFactory->multiply("h_mul",*BW,*BW);
  h_mul->setTitle("BW*BW");
  {
    AIDA::IPlotterRegion& region = plotter->next();
    region.plot(*h_mul);
  }

  {
    AIDA::IPlotterRegion& region = plotter->next();

    region.setParameter("plotter.superposeBins","TRUE");
    region.setParameter("plotter.binsStyle[0].color","1 0 0");
    region.setParameter("plotter.binsStyle[0].modeling","boxes");
    region.setParameter("plotter.binsStyle[1].color","0 0 1");
    region.setParameter("plotter.binsStyle[1].modeling","boxes");

    // Order is not important in "superpose" mode.
    region.plot(*BW);    //Red.
    region.plot(*h_add); //On top. Blue.
  }

  delete plotter;

  delete histogramFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_3(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// ROOT TH* reading example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->createRegions(1,2,0);
  plotter->setTitle("ROOT TH*");
    
  // Cleanup memoryTree :
  memoryTree->rmdir("/");

  // If already loaded close (not delete) the file :
  std::string fileName = 
    Lib::fmanip::name("$ONXLABROOT/examples/data/RootW.root");

  session.destroyManager(fileName);
  
  AIDA::ITree* rootTree = create_tree(*aida,fileName,"ROOT",true,false);
  if(!rootTree) return;

  rootTree->ls();

  // Get a TH1F :
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    AIDA::IHistogram1D* totalHistogram = 
      find_histogram1D(*rootTree,"totalHistogram");

    if(!totalHistogram)
      aUI.echo("OnXLab_Examples_3 : totalHistogram not found or is not a TH1F.");
    else
      region.plot(*totalHistogram);
  }

  // Get a TH2F :
  {
    AIDA::IPlotterRegion& region = plotter->next();
    
    AIDA::IHistogram2D* hpxpy = find_histogram2D(*rootTree,"hpxpy");

    if(!hpxpy)
      aUI.echo("OnXLab_Examples_3 : hpxpy not found or is not a TH2F.");
    else
      region.plot(*hpxpy);

  }

  delete plotter;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_4(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Function 2D example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IFunctionFactory* functionFactory = 
    aida->createFunctionFactory(*memoryTree); 
  if(!functionFactory) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Function 2D");
  plotter->createRegions(1,1,0);
    
  std::vector<double> params;
  // Grid of dimension 2 :
  params.push_back(2);
  int NX = 50;
  params.push_back(NX);
  int NY = 50;
  params.push_back(NY);
  int index;
  for(index=0;index<NX;index++) {
    double x = -10 + 0.399 * index;
    params.push_back(x);
  }
  for(index=0;index<NY;index++) {
    double y = -10 + 0.399 * index;
    params.push_back(y);
  }

  for(int J=0;J<NY;J++) {
    double y = -10 + 0.399 * J;
    for(int I=0;I<NX;I++) {
      double x = -10 + 0.399 * I;
      params.push_back((::sin(x)/x) * (::sin(y)/y));
    }
  }

  // Then give 1 + 1 + 1 + NX + NY + NX * NY values for parameters :

  AIDA::IFunction* function = 
    functionFactory->createFunctionByName("function2D","Grid2D");
  function->setTitle("sin(x)/x) * (sin(y)/y)");
  function->setParameters(params);

  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.style().xAxisStyle().setLabel("x1");
    region.style().yAxisStyle().setLabel("y1");
    region.style().zAxisStyle().setLabel("func");

    region.setParameter("plotter.xAxisAutomated","FALSE");
    region.setParameter("plotter.xAxisMinimum","-10");
    region.setParameter("plotter.xAxisMaximum","10");
    region.setParameter("plotter.yAxisAutomated","FALSE");
    region.setParameter("plotter.yAxisMinimum","-10");
    region.setParameter("plotter.yAxisMaximum","10");

    region.setParameter("plotter.wallStyle.visible","FALSE");
    region.setParameter("plotter.functionStyle[0].color","0 1 0");
    region.setParameter("plotter.functionStyle[0].painting","BY_VALUE");
    region.setParameter("plotter.functionStyle[0].multiNodeLimit","1000000");
    region.setParameter("plotter.colorMapping","black -0.2 cyan 0. green 0.1 orange 0.4 red");

    region.plot(*function);
  }

  delete plotter;

  delete functionFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_5(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Complex numbers and function example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IFunctionFactory* functionFactory = 
    aida->createFunctionFactory(*memoryTree); 
  if(!functionFactory) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Complex");
  plotter->createRegions(1,2,0);
    
  std::vector<double> params;
  // Grid of dimension 1 :
  params.push_back(1);
  int NX = 100;
  params.push_back(NX);
  int index;
  for(index=0;index<NX;index++) {
    double delta = 20./NX;
    double x = -10. + index * delta;
    params.push_back(x);
  }
  // Real, imaginary part of values :
  for(index=0;index<NX;index++) {
    double delta = 20./NX;
    double x = -10. + index * delta;
    double norme = ::sqrt(::exp(-(x*x)/2));
    params.push_back(norme*::cos(x));
    params.push_back(norme*::sin(x));
  }

  // Then give 1 + 1 + NX + NX + NX values for parameters.
  
  AIDA::IFunction* function = 
    functionFactory->createFunctionByName("complexFunction","ComplexGrid1D");
  function->setTitle("sqrt(exp(-(x*x)/2))*(cos(x)+i*sin(x))");
  function->setParameters(params);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {

    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.setParameter("plotter.xAxisAutomated","FALSE");
    region.setParameter("plotter.xAxisMinimum","-10");
    region.setParameter("plotter.xAxisMaximum","10");

    region.plot(*function);
  }

  ///////////////////////////////////////////////////////////////////////////
  // Second region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    region.setParameter("plotter.xAxisAutomated","FALSE");
    region.setParameter("plotter.xAxisMinimum","-10");
    region.setParameter("plotter.xAxisMaximum","10");
    region.setParameter("legendRegionVisible","TRUE");

    if(is_ComplexArrayFunction(*function)) {

      AIDA::IFunction* fr = convert_complexToRealFunction
        (*functionFactory,"real","Real part",*function,"real");
      if(fr) {
        fr->annotation().addItem("Legend","real part (red)");
        region.setParameter("plotter.functionStyle[0].color","1 0 0");
        region.plot(*fr);
      }
  
      AIDA::IFunction* fi = convert_complexToRealFunction
        (*functionFactory,"imag","Imag part",*function,"imag");
      if(fi) {
        fi->annotation().addItem("Legend","imag part (blue)");
        region.setParameter("plotter.functionStyle[1].color","0 0 1");
        region.plot(*fi);
      }
  
      AIDA::IFunction* fn = convert_complexToRealFunction
        (*functionFactory,"norm","Norme",*function,"norm");
      if(fn) {
        fn->annotation().addItem("Legend","norm (green)");
        region.setParameter("plotter.functionStyle[2].color","0 1 0");
        region.plot(*fn);
      }
  
      AIDA::IFunction* fa = convert_complexToRealFunction
        (*functionFactory,"abs","abs",*function,"abs");
      if(fa) {
        fa->annotation().addItem("Legend","abs (cyan)");
        region.setParameter("plotter.functionStyle[3].color","0 1 1");
        region.plot(*fa);
      }
    }
  }

  delete plotter;

  delete functionFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_6(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Simple ASCII tuple file reading example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Ascii tuple example");
  plotter->createRegions(1,1,0);
    
  // If already loaded close (not delete) the file :
  std::string fileName = 
    Lib::fmanip::name("$ONXLABROOT/examples/data/data.ascii");

  session.destroyManager(fileName);
  
  AIDA::ITree* tree = create_tree(*aida,fileName,"ASCII",true,false);
  if(!tree) return;

  // Create some factories to work with the tree :
  AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
  if(!tupleFactory) return;

  /////////////////////////////////////////////////////////////////////////////
  // Get a tuple, create an histo from one of its column, plot it. 
  /////////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    // Column names are v0 v1 ...
    AIDA::ITuple* tuple = tupleFactory->create("tuple","tuple","");

    AIDA::IHistogram1D* h = 
      create_histogram1D(*aida,*memoryTree,"h","tuple/v1",10,1,11);

    AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("v1");
    AIDA::IFilter* filter = tupleFactory->createFilter("");
    tuple->project(*h,*evaluator,*filter);
    delete evaluator;
    delete filter;

    region.plot(*h);
  }

  delete plotter;

  delete tupleFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_7(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// AIDA XML example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Cleanup memoryTree :
  memoryTree->rmdir("/");

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("AIDA XML example");
  plotter->createRegions(1,1,0);
    
  // If already loaded close (not delete) the file :
  std::string fileName = 
    Lib::fmanip::name("$ONXLABROOT/examples/data/exampleHistogram.aida");

  session.destroyManager(fileName);
  
  AIDA::ITree* tree = create_tree(*aida,fileName,"XML",true,false);
  if(!tree) return;

  tree->ls();

  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    AIDA::IHistogram1D* histo = find_histogram1D(*tree,"Aida 1D Histogram");
    
    if(histo) {
      region.plot(*histo);
    }
  }

  delete plotter;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_8(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Profile example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  // Cleanup memoryTree :
  memoryTree->rmdir("/");

  AIDA::IProfile1D* histo = 
    histogramFactory->createProfile1D("prof_1","Profile",100,-5,5);
  delete histogramFactory;

  if(!histo) return;

  Lib::RandomFlat f;
  Lib::RandomGauss r(0,1);
  for(int index=0;index<100000;index++) {
    histo->fill(r.shoot(),f.shoot()-0.5,1);
    //histo->fill(r.shoot(),r.shoot(),1);
  }
  
  Lib::Out out(session.printer());
  out << "OnXLab_Examples_8 :"
      << " Profile " << Lib::sout(histo->title()) 
      << " mean " << histo->mean()
      << " rms " << histo->rms()
      << Lib::endl;


  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Profile example");
  plotter->createRegions(1,1,0);
    
  AIDA::IPlotterRegion& region = plotter->currentRegion();
    
  region.plot(*histo);

  delete plotter;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_9(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Store/retrieve and histogram.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Get some factories :

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Store/retrieve an histogram");
  plotter->createRegions(1,1,0);

  std::string fileName = "OnXLab_example_9.root";    
  if(session.findManager(fileName)) session.destroyManager(fileName);

  {
    aUI.echo("Create a tree associated to a "+fileName+" file");

    AIDA::ITree* tree = create_tree(*aida,fileName,"ROOT",false,true);
    if(tree) {

      tree->mkdir("histograms");
      tree->cd("histograms");

      // Create an histogram in the ROOT tree :
      AIDA::IHistogram1D* histogram = 
        create_histogram1D(*aida,*tree,"h","Rand gauss",100,-5,5);
      if(histogram) {
        Lib::RandomGauss r(0,1);
        for(int index=0;index<10000;index++) histogram->fill(r.shoot(),1);
      }

      tree->commit();
      tree->ls();

      // Destroy the tree :
      session.destroyManager(fileName);
    }
  }

  
  //////////////////////////////////////////////////////
  // Reopen the file in read mode, get the histo and plot :
  //////////////////////////////////////////////////////
  {
    aUI.echo("Reopen the "+fileName+" file...");

    AIDA::ITree* tree = create_tree(*aida,fileName,"ROOT",true,false);
    if(tree) {

      tree->cd("histograms");

      AIDA::IHistogram1D* histogram = find_histogram1D(*tree,"h");
      if(histogram) {
        // Plot :
        AIDA::IPlotterRegion& region = plotter->currentRegion();
        region.plot(*histogram);
      }
    }

  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_10(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Cloud3D from a Function2D
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_Examples_4(aUI);

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IFunction* function = find_function(*memoryTree,"function2D");
  if(!function) {
    aUI.echo("OnXLab_Examples_10 : function2D not found.");
    return;
  }

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  if(!is_Array2DFunction(*function)) {
    aUI.echo("OnXLab_Examples_10 : function not a Array2DFunction.");
    return;
  }

  AIDA::ICloud* cloud = convert_functionToCloud(*histogramFactory,
                          "cloud3D","Cloud from function",*function);
  delete histogramFactory;
  if(!cloud) {
    aUI.echo("OnXLab_Examples_10 : can't convert function to cloud.");
    return;
  }
  
  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->createRegions(1,2,0);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.setParameter("plotter.xAxisAutomated","FALSE");
    region.setParameter("plotter.xAxisMinimum","-10");
    region.setParameter("plotter.xAxisMaximum","10");
    region.setParameter("plotter.yAxisAutomated","FALSE");
    region.setParameter("plotter.yAxisMinimum","-10");
    region.setParameter("plotter.yAxisMaximum","10");

    region.setParameter("plotter.wallStyle.visible","FALSE");
    region.setParameter("plotter.functionStyle[0].color","0 1 0");
    region.setParameter("plotter.functionStyle[0].painting","BY_VALUE");
    region.setParameter("plotter.functionStyle[0].multiNodeLimit","1000000");
    region.setParameter("plotter.colorMapping","black -0.2 cyan 0. green 0.1 orange 0.4 red");

    region.plot(*function);

  }
  ///////////////////////////////////////////////////////////////////////////
  //Second region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();
    
    region.setParameter("plotter.wallStyle.visible","FALSE");
    region.plot(*cloud);

  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_11(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Cloud3D slice XY
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  AIDA::ICloud3D* cloud = histogramFactory->createCloud3D("cloud3D","Cloud3D");
  if(!cloud) return;
  
  // Fill the cloud :
  Lib::RandomGauss r(0,1);
  for(int index=0;index<10000;index++)  {
    cloud->fill(r.shoot(),r.shoot(),r.shoot(),1);
  }

  // Convert to an histogram :
  cloud->convert(50,-10,10,50,-10,10,50,-10,10);
  const AIDA::IHistogram3D& histogram = cloud->histogram();
  // Get a 2D slice :
  AIDA::IHistogram2D* sliceXY = 
    histogramFactory->projectionXY("slice_XY",histogram);

  delete histogramFactory;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Cloud 3D");

  plotter->createRegions(1,1,0);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.plot(*sliceXY);

  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_12(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Cloud2D
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  AIDA::ICloud2D* cloud = histogramFactory->createCloud2D("cloud2D","Cloud2D");
  delete histogramFactory;
  if(!cloud) return;

  // Fill the cloud :
  Lib::RandomGauss r(0,1);
  for(int index=0;index<10000;index++)  {
    cloud->fill(r.shoot(),r.shoot(),1);
  }

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Cloud 2D");

  plotter->createRegions(1,1,0);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.plot(*cloud);

  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_13(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Histogram2D, lego plot
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IHistogram2D* histogram = 
    create_histogram2D(*aida,*memoryTree,"Histogram2D","Histogram2D",
     100,-5,5,100,-5,5);
  if(!histogram) return;
  
  // Fill :
  Lib::RandomGauss r(0,1);
  for(int index=0;index<10000;index++)  {
    histogram->fill(r.shoot(),r.shoot(),1);
  }

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Histogram 2D");

  plotter->createRegions(1,1,0);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.setParameter("plotter.colorMapping",
                        "black 0 cyan 5 green 10 orange 15 red");

    region.setParameter("plotter.wallStyle.visible","FALSE");
    region.setParameter("plotter.binsStyle[0].modeling","boxes");
    region.setParameter("plotter.binsStyle[0].painting","BY_VALUE");
    region.setParameter("plotter.binsStyle[0].multiNodeLimit","1000000000");

    region.style().xAxisStyle().setLabel("Random gauss");
    region.style().yAxisStyle().setLabel("Random gauss");
    region.style().zAxisStyle().setLabel("Entries");
    region.plot(*histogram);

    //Lego layout :
    region.setParameter("plotter.shapeAutomated","FALSE");
    region.setParameter("plotter.shape","XYZ");
  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_14(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Test mag, axes title.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IHistogram1D* histogram = 
    create_histogram1D(*aida,*memoryTree,"h_mag","Rand Gauss",100,0,100000);
  if(!histogram) return;
  
  // Fill :
  Lib::RandomGauss r(0,1);
  for(int index=0;index<100000;index++)  {
    histogram->fill(r.shoot(),1);
  }

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("OnXLab");

  plotter->createRegions(1,1,0);

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    region.style().xAxisStyle().setLabel("Random gauss");
    region.style().yAxisStyle().setLabel("Entries");

    region.plot(*histogram);
  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_15(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Hippodraw tuple file reading example.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("hippodraw tuple example");
  plotter->createRegions(1,1,0);
    
  // If already loaded close (not delete) the file :
  std::string fileName = 
    Lib::fmanip::name("$ONXLABROOT/examples/data/aptuple.tnt.gz");

  session.destroyManager(fileName);
  
  AIDA::ITree* tree = create_tree(*aida,fileName,"hippodraw",true,false);
  if(!tree) return;

  // Create some factories to work with the tree :
  AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
  if(!tupleFactory) return;

  /////////////////////////////////////////////////////////////////////////////
  // Get a tuple, create an histo from one of its column, plot it. 
  /////////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    // Tuple title and column names are read in the file.
    AIDA::ITuple* tuple = tupleFactory->create("aptuple","","");

    AIDA::IHistogram1D* h = 
      create_histogram1D(*aida,*memoryTree,"h_age","Age",50,20,70);

    AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("Age");
    AIDA::IFilter* filter = tupleFactory->createFilter("");
    tuple->project(*h,*evaluator,*filter);
    delete evaluator;
    delete filter;

    region.plot(*h);
  }

  delete plotter;

  delete tupleFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Monitoring_Panel_1(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Used by examples/Onx/Monitoring.onx.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IFunctionFactory* functionFactory = 
    aida->createFunctionFactory(*memoryTree); 
  if(!functionFactory) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Panel 1");
  plotter->createRegions(1,2,0);
    
  Lib::RandomFlat flat;

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {

    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    memoryTree->rm("param_1_1");

    AIDA::IFunction* function = 
      functionFactory->createFunctionByName("param_1_1","G");
    std::vector<double> params;
    params.push_back(400);
    params.push_back(0);
    params.push_back(flat.shoot()*10);
    function->setParameters(params);

    region.plot(*function);
  }

  ///////////////////////////////////////////////////////////////////////////
  // Second region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    memoryTree->rm("param_1_2");

    AIDA::IFunction* function = 
      functionFactory->createFunctionByName("param_1_2","G");
    std::vector<double> params;
    params.push_back(200);
    params.push_back(0);
    params.push_back(1);
    function->setParameters(params);

    region.plot(*function);

  }

  delete plotter;

  delete functionFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Monitoring_Panel_2(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Used by examples/Onx/Monitoring.onx.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IFunctionFactory* functionFactory = 
    aida->createFunctionFactory(*memoryTree); 
  if(!functionFactory) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->setTitle("Panel 2");
  plotter->createRegions(1,3,0);
    
  Lib::RandomFlat flat;

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  ///////////////////////////////////////////////////////////////////////////
  {

    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    memoryTree->rm("param_2_1");

    AIDA::IFunction* function = 
      functionFactory->createFunctionByName("param_2_1","G");
    std::vector<double> params;
    params.push_back(400);
    params.push_back(0);
    params.push_back(1);
    function->setParameters(params);

    region.plot(*function);
  }

  ///////////////////////////////////////////////////////////////////////////
  // Second region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    memoryTree->rm("param_2_2");

    AIDA::IFunction* function = 
      functionFactory->createFunctionByName("param_2_2","G");
    std::vector<double> params;
    params.push_back(200);
    params.push_back(0);
    params.push_back(1);
    function->setParameters(params);

    region.plot(*function);
  }

  ///////////////////////////////////////////////////////////////////////////
  // Third region :
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    memoryTree->rm("param_2_3");

    AIDA::IFunction* function = 
      functionFactory->createFunctionByName("param_2_3","G");
    std::vector<double> params;
    params.push_back(200);
    params.push_back(0);
    params.push_back(flat.shoot()*10);
    function->setParameters(params);

    region.plot(*function);

   {memoryTree->rm("param_2_4");

    AIDA::IFunction* function24 = 
      functionFactory->createFunctionByName("param_2_4","G");
    std::vector<double> params;
    params.push_back(200);
    params.push_back(0);
    params.push_back(flat.shoot()*10);
    function24->setParameters(params);

    region.plot(*function24);}

  }

  delete plotter;

  delete functionFactory;

}
//////////////////////////////////////////////////////////////////////////////
static double Mandel(double aX,double aY)
//////////////////////////////////////////////////////////////////////////////
// From pawex10.kumac mandel.f.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int NMAX = 30;
  double X = aX;
  double Y = aY;
  double XX = 0;
  double YY = 0;
  int N;
  for(N=1;N<=NMAX;N++) {
    double TT = XX*XX-YY*YY+X;
    YY = 2*XX*YY+Y;
    XX = TT;
    if(4<(XX*XX+YY*YY)) break;
  }
  return ((double)N)/((double)NMAX);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_mandel(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Factories :
  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;

  // Get facilities :
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;

  plotter->createRegions(1,1,0);
  plotter->setTitle("Mandel");

  AIDA::IHistogram2D* mandel = 
    histogramFactory->createHistogram2D("mandel","Mandel",
      100,-2.4,0.8,100,-1.2,1.2);


  for(int i=0;i<100;i++) {
    double x = mandel->xAxis().binLowerEdge(i)+mandel->xAxis().binWidth(i)/2;
    for(int j=0;j<100;j++) {
      double y = mandel->yAxis().binLowerEdge(j)+mandel->yAxis().binWidth(j)/2;
      mandel->fill(x,y,Mandel(x,y));
    }
  }

  {
    AIDA::IPlotterRegion& region = plotter->next();

    region.setParameter("plotter.binsStyle[0].modeling","wire_boxes");

    region.plot(*mandel);
  }

  delete plotter;

  delete histogramFactory;

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_animated_fit(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(memoryTree->find("h")) memoryTree->rm("h");
  if(memoryTree->find("fitHistogram_h")) memoryTree->rm("fitHistogram_h");

  AIDA::IHistogram1D* h = 
    create_histogram1D(*aida,*memoryTree,
      "h","The famous normal distribution.",100,-5,5);
  if(!h) return;

  Lib::RandomGauss rg = Lib::RandomGauss(0,1);
  for(int index=0;index<10000;index++) h->fill(rg.shoot(),1);

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(!plotter) return;
  plotter->createRegions(1,1,0);
  plotter->setTitle("OnXLab");

  AIDA::IPlotterRegion& region = plotter->currentRegion();

  region.style().xAxisStyle().setLabel("Random gauss");
  region.style().yAxisStyle().setLabel("Entries");

  region.setParameter("plotter.binsStyle[0].color","0 0 0");
  region.setParameter("plotter.binsStyle[0].modeling","top_lines");

  region.plot(*h);

  AIDA::IFitter* fitter = create_fitter(*aida);
  if(fitter) {
    AIDA::IFunctionFactory* functionFactory = 
      aida->createFunctionFactory(*memoryTree); 
    if(functionFactory) {
      AIDA::IFunction* function = 
        functionFactory->createFunctionByName("fitHistogram_h","G");
      if(function) {
        std::vector<double> params(3);
        params[0] = 100;  //amplitude
        params[1] = 0;    //mean
        params[2] = 3;    //sigma
        function->setParameters(params);
        //Ask to plot before doing the fit in order to see the fit evolution.
        region.plot(*function);
        AIDA::IFitResult* fitResult = fitter->fit(*h,*function);
        delete fitResult;
      }
      delete functionFactory;
    }
    delete fitter;
  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_animated_fit_2(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(memoryTree->find("h")) memoryTree->rm("h");
  if(memoryTree->find("fitHistogram_h")) memoryTree->rm("fitHistogram_h");

  //memoryTree->ls();

  AIDA::IHistogram1D* h = 
    create_histogram1D(*aida,*memoryTree,"h","G+BW distribution.",100,-5,5);
  if(!h) return;

 {Lib::RandomGauss rg = Lib::RandomGauss(0,1);
  for(int index=0;index<10000;index++) h->fill(rg.shoot(),1);}

 {Lib::RandomBreitWigner rbw = Lib::RandomBreitWigner(3,0.5);
  for(int index=0;index<10000;index++) h->fill(rbw.shoot(),1);}

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(!plotter) return;
  plotter->createRegions(1,1,0);
  plotter->setTitle("OnXLab");

  AIDA::IPlotterRegion& region = plotter->currentRegion();

  region.style().xAxisStyle().setLabel("Two random gauss");
  region.style().yAxisStyle().setLabel("Entries");

  region.setParameter("plotter.binsStyle[0].color","0 0 0");
  region.setParameter("plotter.binsStyle[0].modeling","top_lines");

  region.plot(*h);

  //std::string engine = "minuitc++";
  std::string engine = "midnight";
  AIDA::IFitter* fitter = create_fitter(*aida,engine);
  if(fitter) {
    AIDA::IFunctionFactory* functionFactory = 
      aida->createFunctionFactory(*memoryTree); 
    if(functionFactory) {
      AIDA::IFunction* function = 
        functionFactory->createFunctionFromScript
          ("fitHistogram_h",1,"gauss(x,H,M,W)+breit(x,A,B,C)","","","");
      if(function) {
        // Starting values :
        std::vector<double> params;
        params.push_back(300);  // H  should be around 400
        params.push_back(0);    // M  0
        params.push_back(4);    // W  1
        params.push_back(500);  // A  should be around 1200
        params.push_back(2.5);  // B  3
        params.push_back(0.2);  // C  0.5
        function->setParameters(params);
        //Ask to plot before doing the fit in order to see the fit evolution.
        region.plot(*function); 
        AIDA::IFitResult* fitResult = fitter->fit(*h,*function);
        delete fitResult;
      }
      delete functionFactory;
    }
    delete fitter;
  }

  delete plotter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_data_point_set(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Factories :
  AIDA::IDataPointSetFactory* dataPointSetFactory = 
    aida->createDataPointSetFactory(*memoryTree);
  if(!dataPointSetFactory) return;

  AIDA::IDataPointSet* dps1D = 
    dataPointSetFactory->create("dps1D","one dimensional IDataPointSet",1);
  if(dps1D) {
    // Fill the one dimensional IDataPointSet
    double yVals1D[] = { 0.32, 0.45, 0.36, 0.29, 0.34 };
    double yErrP1D[] = { 0.06, 0.07, 0.03, 0.07, 0.04 };

    int number = 5;
    for ( int i = 0; i<number; i++ ) {
      dps1D->addPoint();
      dps1D->point(i)->coordinate(0)->setValue( yVals1D[i] );
      dps1D->point(i)->coordinate(0)->setErrorPlus( yErrP1D[i] );
    }
  }

  // Create a two dimensional IDataPointSet.
  AIDA::IDataPointSet* dps2D = 
    dataPointSetFactory->create("dps2D","two dimensional IDataPointSet",2);
  if(dps2D) {
    // Fill the two dimensional IDataPointSet
    double yVals2D[] = { 0.12, 0.22, 0.35, 0.42, 0.54 , 0.61 };
    double yErrP2D[] = { 0.01, 0.02, 0.03, 0.03, 0.04 , 0.04 };
    double yErrM2D[] = { 0.02, 0.02, 0.02, 0.04, 0.06 , 0.05 };
    double xVals2D[] = { 1.5, 2.6, 3.4, 4.6, 5.5 , 6.4 };
    double xErrP2D[] = { 0.5, 0.5, 0.4, 0.4, 0.5 , 0.5 };

    int number = 6;
    for ( int i = 0; i<number; i++ ) {
      dps2D->addPoint();
      dps2D->point(i)->coordinate(0)->setValue( xVals2D[i] );
      dps2D->point(i)->coordinate(0)->setErrorPlus( xErrP2D[i] );
      dps2D->point(i)->coordinate(1)->setValue( yVals2D[i] );
      dps2D->point(i)->coordinate(1)->setErrorPlus( yErrP2D[i] );
      dps2D->point(i)->coordinate(1)->setErrorMinus( yErrM2D[i] );
    }
  }
      
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(plotter) {
    plotter->createRegions(2,1,0);
    if(dps1D) plotter->region(0)->plot(*dps1D);
    if(dps2D) plotter->region(1)->plot(*dps2D);
    delete plotter;
  }

  delete dataPointSetFactory;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Examples_functions(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) {
    Lib::Out out(session.printer());
    out << "OnXLab_Examples_functions :" 
        << " aida not found." 
        << Lib::endl;
    return;
  } 

  AIDA::ITree* mtree = find_memoryTree(*aida);
  if(!mtree) {
    Lib::Out out(session.printer());
    out << "OnXLab_Examples_functions :" 
        << " memoryTree not found." 
        << Lib::endl;
    return;
  }

  mtree->mkdir("functions");
  mtree->cd("functions");

  std::string codelet = 
    "Compiled:rmason_func;$ONXLABROOT/examples/cpp/OnXLab_exa_func_rmason.cpp";
  AIDA::IFunction* function = 
    create_function(*aida,*mtree,"rmason",codelet,codelet);

  mtree->cd("..");

  if(!function) {
    Lib::Out out(session.printer());
    out << "OnXLab_Examples_functions :" 
        << " can't get function " << Lib::sout(codelet) << "."
        << Lib::endl;
    return;
  }

  // Starting values :
  std::vector<double> params;
  params.push_back(1);       // gauss_H
  params.push_back(16000);   // gauss_M
  params.push_back(1000);    // gauss_W
  params.push_back(1);       // cauchy_H
  params.push_back(20000);   // cauchy_C
  params.push_back(2*0.2);   // cauchy_W
  params.push_back(0);       // pol_0
  params.push_back(0.00001); // pol_1
  function->setParameters(params);

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(!plotter) {
    Lib::Out out(session.printer());
    out << "OnXLab_Examples_functions :" 
        << " can't create plotter." 
        << Lib::endl;
    return;
  }
  plotter->createRegions(1,1,0);
  //plotter->setTitle("OnXLab");

  AIDA::IPlotterRegion& region = plotter->currentRegion();

  //region.style().xAxisStyle().setLabel("x");
  //region.style().yAxisStyle().setLabel("f(x)");

  region.setParameter("plotter.xAxisAutomated","FALSE");
  region.setParameter("plotter.xAxisMinimum","0");
  region.setParameter("plotter.xAxisMaximum","30000");

  function->annotation().addItem("PlottingNumberOfStepsX","100000");

  region.setParameter("plotter.yAxisAutomated","FALSE");
  region.setParameter("plotter.yAxisMinimum","0");
  region.setParameter("plotter.yAxisMaximum","3");

  region.plot(*function);

  delete plotter;
}


} // extern "C"
