// this :
#include "Accessors.h"

// Lib :
#include <Slash/Core/ISession.h>
#include <Slash/Data/IProcessor.h>
#include <Slash/Data/IIterator.h>
#include <Lib/Property.h>
#include <Lib/Value.h>
#include <Lib/Manager.h>
#include <Lib/smanip.h>
#include <Lib/Out.h>
#include <Lib/Cast.h>

// Lab :
#include <BatchLab/Helpers/AIDA.h>

// HEPVis :
#include <HEPVis/SbPlottableThings.h> //Interfaces

//////////////////////////////////////////////////////////////////////////////
OnXLab::Bin1D_Accessor::Bin1D_Accessor(
 Slash::Core::ISession& aSession
)
:Lib::BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("Bin1D")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("index",Lib::Property::INTEGER);
  addProperty("height",Lib::Property::DOUBLE);
  addProperty("error",Lib::Property::DOUBLE);
  addProperty("entries",Lib::Property::INTEGER);
  addProperty("lowerEdge",Lib::Property::DOUBLE);
  addProperty("upperEdge",Lib::Property::DOUBLE);
  addProperty("name",Lib::Property::STRING);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::Bin1D_Accessor::~Bin1D_Accessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::Bin1D_Accessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnXLab::Bin1D_Accessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnXLab::Bin1D_Accessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void* aTag
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPlottableBins1D* object = (SbPlottableBins1D*)aData;
  if(!object->isValid()) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::Bin1D_Accessor::value :"
        << " data object associated to plotted representation " 
        << "is no more valid." << Lib::endl;
    return new Lib::Value(); 
  }
  // aTag is a char* (see SoPlotter).
  std::string infos((char*)aTag);
  std::vector<int> vals;
  if(!Lib::smanip::toints(infos,vals)) return new Lib::Value();
  if(vals.size()!=1) return new Lib::Value();
  int i = vals[0];
  if(aName=="index") {
    return new Lib::Value(i);
  } else if(aName=="height") {
    return new Lib::Value((double)object->getBinSumOfWeights(i));
  } else if(aName=="error") {
    return new Lib::Value((double)object->getBinBar(i));
  } else if(aName=="entries") {
    return new Lib::Value(object->getBinNumberOfEntries(i));
  } else if(aName=="lowerEdge") {
    return new Lib::Value((double)object->getBinLowerEdge(i));
  } else if(aName=="upperEdge") {
    return new Lib::Value((double)object->getBinUpperEdge(i));
  } else if(aName=="name") {
    return new Lib::Value(std::string(object->getName()));
  } 
  return new Lib::Value();
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::Bin2D_Accessor::Bin2D_Accessor(
 Slash::Core::ISession& aSession
)
:Lib::BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("Bin2D")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("indexX",Lib::Property::INTEGER);
  addProperty("indexY",Lib::Property::INTEGER);
  addProperty("height",Lib::Property::DOUBLE);
  addProperty("error",Lib::Property::DOUBLE);
  addProperty("entries",Lib::Property::INTEGER);
  addProperty("lowerEdgeX",Lib::Property::DOUBLE);
  addProperty("upperEdgeX",Lib::Property::DOUBLE);
  addProperty("lowerEdgeY",Lib::Property::DOUBLE);
  addProperty("upperEdgeY",Lib::Property::DOUBLE);
  addProperty("name",Lib::Property::STRING);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::Bin2D_Accessor::~Bin2D_Accessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::Bin2D_Accessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnXLab::Bin2D_Accessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnXLab::Bin2D_Accessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void* aTag
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPlottableBins2D* object = (SbPlottableBins2D*)aData;
  if(!object->isValid()) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::Bin2D_Accessor::value :"
        << " data object associated to plotted representation " 
        << "is no more valid." << Lib::endl;
    return new Lib::Value(); 
  }
  // aTag is a char* (see SoPlotter).
  std::string infos((char*)aTag);
  std::vector<int> vals;
  if(!Lib::smanip::toints(infos,vals)) return new Lib::Value();
  if(vals.size()!=2) return new Lib::Value();
  int i = vals[0];
  int j = vals[1];
  if(aName=="indexX") {
    return new Lib::Value(i);
  } else if(aName=="indexY") {
    return new Lib::Value(j);
  } else if(aName=="height") {
    return new Lib::Value((double)object->getBinSumOfWeights(i,j));
  } else if(aName=="error") {
    return new Lib::Value((double)object->getBinBar(i,j));
  } else if(aName=="entries") {
    return new Lib::Value(object->getBinNumberOfEntries(i,j));
  } else if(aName=="lowerEdgeX") {
    return new Lib::Value((double)object->getBinLowerEdgeX(i));
  } else if(aName=="upperEdgeX") {
    return new Lib::Value((double)object->getBinUpperEdgeX(i));
  } else if(aName=="lowerEdgeY") {
    return new Lib::Value((double)object->getBinLowerEdgeY(j));
  } else if(aName=="upperEdgeY") {
    return new Lib::Value((double)object->getBinUpperEdgeY(j));
  } else if(aName=="name") {
    return new Lib::Value(std::string(object->getName()));
  } 
  return new Lib::Value();
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// AIDA :
#include <AIDA/ITree.h>
#include <AIDA/IManagedObject.h>

// Lab :
#include <BatchLab/Core/amanip.h>
#include <BatchLab/Core/MemoryTree.h>

//////////////////////////////////////////////////////////////////////////////
OnXLab::ManagedObjectAccessor::ManagedObjectAccessor(
 Slash::Core::ISession& aSession
)
:Lib::BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("ManagedObject")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("name",Lib::Property::STRING);
  addProperty("type",Lib::Property::STRING,20);
  addProperty("address",Lib::Property::POINTER);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::ManagedObjectAccessor::~ManagedObjectAccessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::ManagedObjectAccessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnXLab {
  class ManagedObjectIterator : public Slash::Data::IIterator {
  public: //Slash::Data::IIterator
    virtual Slash::Data::IAccessor::Data object() {
      if(fIterator==fList.end()) return 0;
      return *fIterator;
    }
    virtual void next() { ++fIterator;}
    virtual void* tag() { return 0;}
  public:
    ManagedObjectIterator(const std::vector<AIDA::IManagedObject*>& aList)
      :fList(aList){
      fIterator = fList.begin();
    }
  private:
    std::vector<AIDA::IManagedObject*> fList; // Local copy.
    std::vector<AIDA::IManagedObject*>::iterator fIterator;
  };
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnXLab::ManagedObjectAccessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BatchLab::MemoryTree* memoryTree = 
    Lib_findManager(fSession,"MemoryTree",BatchLab::MemoryTree);
  if(!memoryTree) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::ManagedObjectAccessor::iterator :"
        << " MemoryTree not found." << Lib::endl;
    return 0;
  }
  std::vector<AIDA::IManagedObject*> vec = memoryTree->allObjects();
  return new ManagedObjectIterator(vec);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnXLab::ManagedObjectAccessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IManagedObject* object = (AIDA::IManagedObject*)aData;
  if(aName=="name") {
    return new Lib::Value(object->name());
  } else if(aName=="type") {
    std::string s = BatchLab::amanip::managed_object_type(*object); //FIXME
    return new Lib::Value(s);
  } else if(aName=="address") {
    return new Lib::Value((void*)object);
  } else {
    return new Lib::Value();
  }
}
void OnXLab::ManagedObjectAccessor::beginVisualize() {}
void OnXLab::ManagedObjectAccessor::endVisualize() {}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::ManagedObjectAccessor::visualize(
 Slash::Data::IAccessor::Data aData
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IAnalysisFactory* aida = find_aida(fSession);
  if(!aida) return;
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IPlotterRegion& region = plotter->currentRegion();
  AIDA::IManagedObject* object = (AIDA::IManagedObject*)aData;
  if(AIDA::IBaseHistogram* h = Lib_Cast(*object,AIDA::IBaseHistogram)) {
    region.plot(*h);
  } else if(AIDA::IFunction* f = Lib_Cast(*object,AIDA::IFunction)) {
    region.plot(*f);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include <AIDA/IFunction.h>
//////////////////////////////////////////////////////////////////////////////
OnXLab::FunctionAccessor::FunctionAccessor(
 Slash::Core::ISession& aSession
)
:Lib::BaseAccessor(aSession.printer())
,fSession(aSession)
,fName("Function")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  addProperty("name",Lib::Property::STRING);
  addProperty("title",Lib::Property::STRING);
  addProperty("dimension",Lib::Property::INTEGER);
  addProperty("#params",Lib::Property::INTEGER);
  addProperty("codelet",Lib::Property::STRING);
  addProperty("address",Lib::Property::POINTER);
  addProperty("params",Lib::Property::STRING);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::FunctionAccessor::~FunctionAccessor(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::FunctionAccessor::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
namespace OnXLab {
  class FunctionIterator : public Slash::Data::IIterator {
  public: //Slash::Data::IIterator
    virtual Slash::Data::IAccessor::Data object() {
      if(fIterator==fList.end()) return 0;
      return *fIterator;
    }
    virtual void next() { ++fIterator;}
    virtual void* tag() { return 0;}
  public:
    FunctionIterator(const std::vector<AIDA::IFunction*>& aList)
      :fList(aList){
      fIterator = fList.begin();
    }
  private:
    std::vector<AIDA::IFunction*> fList; // Local copy.
    std::vector<AIDA::IFunction*>::iterator fIterator;
  };
}
//////////////////////////////////////////////////////////////////////////////
Slash::Data::IIterator* OnXLab::FunctionAccessor::iterator(
) 
//////////////////////////////////////////////////////////////////////////////
// Returned object should be deleted by the receiver.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  BatchLab::MemoryTree* memoryTree = 
    Lib_findManager(fSession,"MemoryTree",BatchLab::MemoryTree);
  if(!memoryTree) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::FunctionAccessor::iterator :"
        << " MemoryTree not found." << Lib::endl;
    return 0;
  }
  std::vector<AIDA::IManagedObject*> vec = memoryTree->allObjects();
  std::vector<AIDA::IFunction*> funcs;
  for(unsigned int index=0;index<vec.size();index++) {
    AIDA::IFunction* obj = Lib_Cast(*(vec[index]),AIDA::IFunction);
    if(obj) funcs.push_back(obj);
  }
  return new FunctionIterator(funcs);
}
//////////////////////////////////////////////////////////////////////////////
Slash::Core::IValue* OnXLab::FunctionAccessor::findValue(
 Slash::Data::IAccessor::Data aData
,const std::string& aName
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IFunction* object = (AIDA::IFunction*)aData;
  if(aName=="name") {
    AIDA::IManagedObject* mobject = Lib_Cast(*object,AIDA::IManagedObject);
    return new Lib::Value(mobject?mobject->name():"none");
  } else if(aName=="title") {
    return new Lib::Value(object->title());
  } else if(aName=="dimension") {
    return new Lib::Value(object->dimension());
  } else if(aName=="#params") {
    return new Lib::Value(object->numberOfParameters());
  } else if(aName=="codelet") {
    return new Lib::Value(object->codeletString());
  } else if(aName=="address") {
    return new Lib::Value((void*)object);
  } else if(aName=="params") {
    std::vector<std::string> names = object->parameterNames();
    std::vector<double> vals = object->parameters();
    std::string value;
    unsigned int number = names.size();
    for(unsigned int index=0;index<number;index++){
      std::string s;
      Lib::smanip::printf(s,32,"%g",vals[index]);
      value += names[index] + " = " + s + ";";
    }
    return new Lib::Value(value);
  } else {
    return new Lib::Value();
  }
}
void OnXLab::FunctionAccessor::endVisualize() {}
void OnXLab::FunctionAccessor::beginVisualize() {}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::FunctionAccessor::visualize(
 Slash::Data::IAccessor::Data aData
,void*
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IAnalysisFactory* aida = find_aida(fSession);
  if(!aida) return;
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IPlotterRegion& region = plotter->currentRegion();
  AIDA::IFunction* object = (AIDA::IFunction*)aData;
  region.plot(*object);
}
