//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI& ui);
//     void callback_with_arguments(Slash::UI::IUI& ui,
//                                  const std::vector<std::string>& args);
//   }
//

#include "Helpers.h"

#include <Slash/Data/IProcessor.h>
#include <Slash/UI/ICyclic.h>

#include <Lib/Manager.h>
#include <Lib/Randoms.h>
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/System.h>
#include <Lib/Cast.h>
#include <Lib/Arguments.h>

#include <BatchLab/Core/amanip.h>
#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/BaseTree.h> //FIXME

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include <OnXLab/Core/AnalysisFactory.h>
#include <OnXLab/Helpers/OnXLab.h>

#include "HistogramAction.h"
#include "TupleAction.h"
#include "Accessors.h"

extern "C" {
  static void OnXLab_open(Slash::UI::IUI&,const std::string&); 
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// OnXLab/scripts/OnX/OnXLab.onx ////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) {
    soViewer->setDecoration(false);
    soViewer->setViewing(false);
  }

  Slash::Core::ISession& session = aUI.session();

  if(!session.findManager("AnalysisFactory")) {
    session.addManager(new OnXLab::AnalysisFactory(session));
  }

  Slash::Data::IProcessor* accessorManager = 
    Lib_findManager(session,"AccessorManager",Slash::Data::IProcessor);
  if(accessorManager) {
    if(!accessorManager->findAccessor("ManagedObject")) {
      accessorManager->addAction(new OnXLab::HistogramAction(*accessorManager,session));
      accessorManager->addAction(new OnXLab::TupleAction(*accessorManager,session));
      accessorManager->addAccessor(new OnXLab::ManagedObjectAccessor(session));
      accessorManager->addAccessor(new OnXLab::Bin1D_Accessor(session));
      accessorManager->addAccessor(new OnXLab::Bin2D_Accessor(session));
      accessorManager->addAccessor(new OnXLab::FunctionAccessor(session));
    }
  }

  session.setParameter("mainTree.append","no");

  // Look if some data file had been passed in argument :
  //   OS> <application> -data_file=<file>
  // Example :
  //   OS> onxlab -data_file=my.data -file_type=ascii
 {Slash::Core::ISession& session = aUI.session();
  Lib::Arguments args(session.arguments());
  std::string type;
  args.find("-file_type",type);
  std::string file;
  if(args.find("-data_file",file)) {
    if(file!="") {
      AIDA::IAnalysisFactory* aida = find_aida(session);
      if(aida) {
        if(type=="") {
          std::string path,name,suffix;
          Lib::smanip::pathNameSuffix(file,path,name,suffix);
          Lib::smanip::tolowercase(suffix);
          type = suffix;
        } else {  
          Lib::smanip::tolowercase(type);
        }
        AIDA::ITree* tree = create_tree(*aida,file,type,true,false);
        if(tree) {
          if((type=="ascii")||(type=="tnt")) {
            // Read the file so that the tree widget shows the tuple :
            create_tuple(*aida,*tree,"tuple","","");
          }
        }
      }
    }
  }}

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_mainTree_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  std::vector<std::string> vs;
 {unsigned int number = session.numberOfManagers();
  for(unsigned int index=0;index<number;index++) {
    const Slash::Core::IManager* mgr = session.manager(index);
    if(dynamic_cast<const AIDA::ITree*>(mgr)) {
      vs.push_back(mgr->name());
    }
  }}

  // Build the XML script to be put on the tree widget :

  std::string sout = "<tree>";

  for(unsigned int index=0;index<vs.size();index++) {
    std::string treeName = vs[index];
    AIDA::ITree* tree = Lib_findManager(session,treeName,AIDA::ITree);
    if(tree) {
      //FIXME : avoid the casting to the OnXLab::Tree. We should
      //        be able to retreive the infos through AIDA.
      
      BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
      if(labTree) {
        sout += "<treeItem><label>"+Lib::smanip::toxml(treeName)+"</label>";

        std::string ss = labTree->hierarchy();
        Lib::smanip::replace(ss,"<tree>","");
        Lib::smanip::replace(ss,"</tree>","");
        sout += ss;
        sout += "</treeItem>";
      }
    }
  }
  sout += "</tree>";
  aUI.setParameter("mainTree.items",sout);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_mainTree_selection(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IManager* manager;
  std::string path,basename;
  AIDA::IManagedObject* object = mainTree_selection(aUI,manager,path,basename);
  if(!object) return;

  Slash::Core::ISession& session = aUI.session();

  bool append;
  session_parameterValue(session,"mainTree.append",append);

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IPlotterRegion& region = plotter->currentRegion();

  AIDA::ICloud* cl = Lib_Cast(*object,AIDA::ICloud);
  if(cl) {
    // Show the cloud explorer palette :
    aUI.executeScript("DLD","OnX ui_show_dialog $ONXLABROOT/scripts/OnX/CloudExplorer.onx OnXLab_CloudExplorer_dialog");

    aUI.executeScript("DLD","OnXLab OnXLab_CloudExplorer_update");

    region.clear();
    delete plotter;

    BatchLab::amanip::hprint(*object,session.printer());
    return;
  } 

  AIDA::IBaseHistogram* h = Lib_Cast(*object,AIDA::IBaseHistogram);
  if(h) {
    if(!append) region.clear();
    region.plot(*h);
    BatchLab::amanip::hprint(*object,session.printer());
    delete plotter;
    return;
  } 

  AIDA::IFunction* f = Lib_Cast(*object,AIDA::IFunction);
  if(f) {
    if(!append) region.clear();
    region.plot(*f);
    BatchLab::amanip::hprint(*object,session.printer());
    delete plotter;
    return;
  } 

  AIDA::IDataPointSet* dps = Lib_Cast(*object,AIDA::IDataPointSet);
  if(dps) {
    if(!append) region.clear();
    region.plot(*dps);
    BatchLab::amanip::hprint(*object,session.printer());
    delete plotter;
    return;
  }

  AIDA::ITuple* tuple = Lib_Cast(*object,AIDA::ITuple);
  if(tuple) {
    // Show the tuple explorer palette :
    aUI.executeScript("DLD","OnX ui_show_dialog $ONXLABROOT/scripts/OnX/TupleExplorer.onx OnXLab_TupleExplorer_dialog");

    aUI.executeScript("DLD","OnXLab OnXLab_TupleExplorer_update");

    region.clear();
    delete plotter;

    Lib::Out out(aUI.session().printer());
    out << "/////////////////" << Lib::endl;
    out << "name    : " << object->name() << Lib::endl;
    out << "title   : " << tuple->title() << Lib::endl;
    out << "columns : " << tuple->columns() << Lib::endl;
    out << "rows    : " << tuple->rows() << Lib::endl;
    return;
  }

  delete plotter;

  Lib::Out out(session.printer());
  out << "OnXLab_mainTree_selection :"
      << " don't know how to represent " << Lib::sout(basename)
      << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_tree_option(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  std::string item = aUI.callbackValue();

  if(item=="AIDA trees") {
    OnXLab_mainTree_update(aUI);

  } else if(item=="Clear selected tree") {
    std::string selection;
    if(!aUI.parameterValue("mainTree.selection",selection)) return;

    std::vector<std::string> items;
    Lib::smanip::lines(selection,items);
    if(!items.size()) return;

    AIDA::ITree* tree = Lib_findManager(session,items[0],AIDA::ITree);
    if(!tree) return;

    tree->rmdir("/");

  } else if(item=="Delete selected tree") {
    std::string selection;
    if(!aUI.parameterValue("mainTree.selection",selection)) return;

    std::vector<std::string> items;
    Lib::smanip::lines(selection,items);
    if(!items.size()) return;

    if(items[0]=="MemoryTree") {
      Lib::Out out(session.printer());
      out << "OnXLab_mainTree_selection :"
          << " can't delete the MemoryTree."
          << Lib::endl;
    } else {
      session.destroyManager(items[0]);
    }

  } else if(item=="Delete selected object") {

    Slash::Core::IManager* manager;
    std::string path,basename;
    AIDA::IManagedObject* object = 
      mainTree_selection(aUI,manager,path,basename);
    if(!object) return;
    delete object;

  } else if(item=="Managers") {
    tree_set_with_strings(aUI,"mainTree.items",session.managerNames());

  } else if(item=="Append on/off") {
    std::string svalue;
    if(!session.parameterValue("mainTree.append",svalue)) return;
    session.setParameter("mainTree.append",svalue=="yes"?"no":"yes");
  }
}
//////////////////////////////////////////////////////////////////////////////
/// OnXLab/scripts/OnX/File.onx //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_ROOT_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"ROOT");
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_AIDA_XML_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"XML");
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_AIDA_zip_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"zip");
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_AIDA_HDF5_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"HDF5");
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_ASCII_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"ASCII");
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_File_ZEBRA_open(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_open(aUI,"ZEBRA");
}
//////////////////////////////////////////////////////////////////////////////
/// OnXLab/scripts/OnX/InputHistogram.onx ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_InputHistogram_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Input fields : 
//   string sname
//   string stitle
//   int xnumber
//   real xmin 
//   real xmax
//   bool fill
//   int entries
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  /////////////////////////////
  // Get input fields values :
  /////////////////////////////
  std::string svalue;

  std::string name;
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_name.value",name)) return;

  std::string title;
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_title.value",title)) return;
  
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_xnumber.value",svalue)) return;
  int xnumber;
  if(!Lib::smanip::toint(svalue,xnumber)) return;

  double xmin;
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_xmin.value",svalue)) return;
  if(!Lib::smanip::todouble(svalue,xmin)) return;

  double xmax;
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_xmax.value",svalue)) return;
  if(!Lib::smanip::todouble(svalue,xmax)) return;

  bool fill;
  if(!aUI.parameterValue("OnXLab_InputHistogram_input_fill.value",svalue)) return;
  if(!Lib::smanip::tobool(svalue,fill)) return;

  if(!aUI.parameterValue("OnXLab_InputHistogram_input_entries.value",svalue)) return;
  int entries;
  if(!Lib::smanip::toint(svalue,entries)) return;

  /////////////////////////////
  /////////////////////////////
  /////////////////////////////
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IManagedObject* object = memoryTree->find(name);
  if(object) {
    aUI.echo("Histogram "+name+" already exists.");
  } else {
    if(xnumber<=0) {
      aUI.echo("Bad number of bins.");
    } else if(xmax<=xmin) {
      aUI.echo("Max <= min !");
    } else {
      
      AIDA::IHistogram1D* histogram = 
        create_histogram1D(*aida,*memoryTree,name,title,xnumber,xmin,xmax);
      if(!histogram) return;

      if(fill) {
        // Fill with random :;
        Lib::RandomGauss rg(xmin,xmax);
        for(int index=0;index<entries;index++)
          histogram->fill(rg.shoot(),1);
      }
      // Plot it :
      AIDA::IPlotter* plotter = create_plotter(*aida);
      if(plotter) {
        AIDA::IPlotterRegion& region = plotter->currentRegion();
        region.clear();
        region.plot(*histogram);
        delete plotter;
      }

      OnXLab_mainTree_update(aUI);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
/// OnXLab/scripts/OnX/InputFunction.onx /////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_InputFunction_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Input fields : 
//   string name
//   string title
//   string kind
//   string class_script
//   string scripted_script
//   string compiled_script
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  /////////////////////////////
  // Get input fields values :
  /////////////////////////////
  std::string svalue;

  std::string name;
  if(!aUI.parameterValue("OnXLab_InputFunction_input_name.value",name)) return;

  std::string title;
  if(!aUI.parameterValue("OnXLab_InputFunction_input_title.value",title)) return;
  
  std::string kind;
  if(!aUI.parameterValue("OnXLab_InputFunction_input_kind.value",kind)) return;

  std::string script;
  if(!aUI.parameterValue
    ("OnXLab_InputFunction_input_"+kind+"_script.value",script)) return;

  /////////////////////////////
  /////////////////////////////
  /////////////////////////////
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IManagedObject* object = memoryTree->find(name);
  if(object) {
    aUI.echo("Function "+name+" already exists.");
  } else {
    std::string path = name;    
    std::string codelet = kind+":"+script;
   
    AIDA::IFunction* function = 
      create_function(*aida,*memoryTree,path,title,codelet);
    if(!function) return;

    // Plot it :
    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      AIDA::IPlotterRegion& region = plotter->currentRegion();
      region.clear();
      region.plot(*function);
      delete plotter;
    }

    OnXLab_mainTree_update(aUI);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_LegendEditor_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IManager* manager;
  std::string path,basename;
  AIDA::IManagedObject* object = mainTree_selection(aUI,manager,path,basename);
  if(!manager) return;
  AIDA::ITree* tree = Lib_Cast(*manager,AIDA::ITree);
  if(!tree) return;
  if(object) {
    aUI.setParameter("OnXLab_LegendEditor_tree.value",manager->name());
    aUI.setParameter("OnXLab_LegendEditor_object.value",path);
  } else {
    aUI.setParameter("OnXLab_LegendEditor_tree.value","");
    aUI.setParameter("OnXLab_LegendEditor_object.value","");
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_LegendEditor_entry(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string legend = aUI.callbackValue();

  std::string stree;
  std::string path;
  aUI.parameterValue("OnXLab_LegendEditor_tree.value",stree);
  aUI.parameterValue("OnXLab_LegendEditor_object.value",path);

  Slash::Core::IManager* manager = aUI.session().findManager(stree);
  if(!manager) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_LegendEditor_entry :"
        << " manager " << Lib::sout(stree) << " not found."
        << Lib::endl;
    return;
  }
  AIDA::ITree* tree = Lib_Cast(*manager,AIDA::ITree);
  if(!tree) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_LegendEditor_entry :"
        << " manager " << Lib::sout(stree) << " is not an AIDA::ITree."
        << Lib::endl;
    return;
  }
  AIDA::IManagedObject* object = tree->find(path);
  if(!object) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_LegendEditor_entry :"
        << " object " << Lib::sout(path) << " not found in AIDA::ITree "
        << Lib::sout(stree)
        << Lib::endl;
    return;
  }

  if(AIDA::IBaseHistogram* bh = Lib_Cast(*object,AIDA::IBaseHistogram)) {
    bh->annotation().addItem("Legend",legend);
  } else if(AIDA::IDataPointSet* dps = Lib_Cast(*object,AIDA::IDataPointSet)) {
    dps->annotation().addItem("Legend",legend);
  } else if(AIDA::IFunction* f = Lib_Cast(*object,AIDA::IFunction)) {
    f->annotation().addItem("Legend",legend);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_plotter_initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(!plotter) return;

  plotter->createRegions(1,1,0);
  plotter->setTitle("");

  delete plotter;

  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) {
    soViewer->setDecoration(false);
    soViewer->setViewing(false);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Monitoring_Cyclic_Panel_1(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnXLab_Cyclic");
  if(!cyclic) cyclic = aUI.createCyclic("OnXLab_Cyclic",1,"","");
  if(!cyclic) return;
  cyclic->stop();
  cyclic->setDelay(1000);
  cyclic->setScript("DLD","OnXLab OnXLab_Monitoring_Panel_1");
  cyclic->start();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Monitoring_Cyclic_Panel_2(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnXLab_Cyclic");
  if(!cyclic) cyclic = aUI.createCyclic("OnXLab_Cyclic",1,"","");
  if(!cyclic) return;
  cyclic->stop();
  cyclic->setDelay(1000);
  cyclic->setScript("DLD","OnXLab OnXLab_Monitoring_Panel_2");
  cyclic->start();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Monitoring_Cyclic_stop(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ICyclic* cyclic = aUI.findCyclic("OnXLab_Cyclic");
  if(!cyclic) return;
  cyclic->stop();
}
//////////////////////////////////////////////////////////////////////////////
/// data /////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void data_fill1D(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  data_fill1D(aUI.session(),aArgs[0],aArgs[1]);
}
//////////////////////////////////////////////////////////////////////////////
void data_fill2D(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=3) return;
  data_fill2D(aUI.session(),aArgs[0],aArgs[1],aArgs[3]);
}
//////////////////////////////////////////////////////////////////////////////
void data_tuple(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  data_tuple(aUI.session(),aArgs[0],aArgs[1]);
}

// For backward compatibility.
void Lab_Initialize(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_Initialize : deprecated. Use OnXLab_Initialize.");
  OnXLab_Initialize(aUI);
}
void Lab_mainTree_update(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_mainTree_update depreacted. Use OnXLab_mainTree_update.");
  OnXLab_mainTree_update(aUI);
}
void Lab_mainTree_selection(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_mainTree_selection depreacted. Use OnXLab_mainTree_selection.");
  OnXLab_mainTree_selection(aUI);
}
void Lab_File_ROOT_open(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_File_ROOT_open depreacted. Use OnXLab_File_ROOT_open.");
  OnXLab_File_ROOT_open(aUI);
}
void Lab_File_AIDA_XML_open(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_File_AIDA_XML_open depreacted. Use OnXLab_File_AIDA_XML_open.");
  OnXLab_File_AIDA_XML_open(aUI);
}
void Lab_File_ASCII_open(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_File_ASCII_open depreacted. Use OnXLab_File_ASCII_open.");
  OnXLab_File_ASCII_open(aUI);
}
void Lab_File_ZEBRA_open(Slash::UI::IUI& aUI) {
  aUI.echo("Lab_File_ZEBRA_open depreacted. Use OnXLab_File_ZEBRA_open.");
  OnXLab_File_ZEBRA_open(aUI);
}

} // extern "C"

//////////////////////////////////////////////////////////////////////////////
static void OnXLab_open(
 Slash::UI::IUI& aUI
,const std::string& aType
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aUI.hide("fileChooser");
  std::string file = aUI.callbackValue();
  Slash::Core::ISession& session = aUI.session();
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;
  AIDA::ITree* tree = create_tree(*aida,file,aType,true,false);
  if(!tree) return;
  if((aType=="ASCII")||(aType=="TNT")) {
    // Read the file so that the tree widget shows the tuple :
    create_tuple(*aida,*tree,"tuple","","");
  }
}
