//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

#include <Lib/Manager.h>
#include <Lib/Cast.h>

#include <OnX/Helpers/OnX.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/amanipT.h>

#include "Helpers.h"

#define NotFound (-1)

namespace OnXLab {

class CloudExplorerManager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnXLab::CloudExplorerManager)
    else if_Lib_SCast(Slash::Core::IManager)
    else return 0;
  }
public:
  CloudExplorerManager(Slash::Core::ISession&)
  :fName("OnXLab_CloudExplorerManager"),fCloud(0),fRep(0){}
  virtual ~CloudExplorerManager(){
    delete fRep;
  }
  void setCloud(AIDA::ICloud* aCloud) { 
    fCloud = aCloud;
    delete fRep;
    fRep = 0;
  }
  AIDA::ICloud* cloud() const { return fCloud;}
  void setRep(AIDA::IBaseHistogram* aRep) {
    delete fRep;
    fRep = aRep;
  }
private:
  std::string fName;
  AIDA::ICloud* fCloud;
  AIDA::IBaseHistogram* fRep;
};
}

//////////////////////////////////////////////////////////////////////////////
static OnXLab::CloudExplorerManager* find_explorer(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return Lib_findManager
    (aUI.session(),"OnXLab_CloudExplorerManager",OnXLab::CloudExplorerManager);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ICloud* getCloud(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab::CloudExplorerManager* explorer = find_explorer(aUI);
  if(!explorer) return 0;
  return explorer->cloud();
}

#define CLOUD_HISTO(aCloud,aLower,aUpper,aValue) \
   {double xmin = 0;\
    double xmax = 0;\
    if(aXmax<=aXmin) { /* Guess the histogram booking parameters. */\
      xmin = aCloud->aLower();\
      xmax = aCloud->aUpper();\
      if(xmax==xmin) {\
        xmax = xmin + 1;\
      } else { /* Tilt xmax in order that max value is in the histo. */\
        xmax = xmax + (xmax-xmin) * 0.01;\
      }\
    } else {\
      xmin = aXmin;\
      xmax = aXmax;\
    }\
    /* WARNING : will be managed by the CloudExplorer. */\
    AIDA::IHistogram1D* histogram = \
      histogramFactory->createHistogram1D\
        (path_not_managed()+"/"+name,name,xnumber,xmin,xmax);\
    if(!histogram) {\
      delete histogramFactory;\
      return 0;\
    }\
    int n = aCloud->entries();\
    for(int index = 0;index<n;index++) {\
      histogram->fill(aCloud->aValue(index),aCloud->weight(index));\
    }\
    baseHistogram = histogram;}

//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* represent_cloud(
 Slash::Core::ISession& aSession
,AIDA::ICloud& aCloud
,int aCoord
,int aXnumber
,double aXmin
,double aXmax
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IWriter& printer = aSession.printer();
  Lib::Out out(printer);

  if(aCloud.dimension()<=0) {
    out << "represent_cloud :"
        << " cloud has fancy a dimension." << Lib::endl;
    return 0;
  }

  if( (aCoord<0) || (aCoord>=aCloud.dimension()) ) {
    out << "represent_cloud :"
        << " coordinate out of range ; given " << aCoord
        << " range [0," << aCloud.dimension() << "]." 
        << Lib::endl;
    return 0;
  }

  AIDA::IAnalysisFactory* aida = 
      Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
  if(!aida) return 0;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return 0;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return 0;

  AIDA::IBaseHistogram* baseHistogram = 0;

  std::string name = BatchLab::aida_object_name<AIDA::ICloud>(aCloud);

  int xnumber = aXnumber<=0 ? 100 : aXnumber;

  if(AIDA::ICloud1D* cl1D = Lib_Cast(aCloud,AIDA::ICloud1D)) {
   
    CLOUD_HISTO(cl1D,lowerEdge,upperEdge,value)

  } else if(AIDA::ICloud2D* cl2D = Lib_Cast(aCloud,AIDA::ICloud2D)) {
   
    if(aCoord==0) {
      CLOUD_HISTO(cl2D,lowerEdgeX,upperEdgeX,valueX)
    } else if(aCoord==1) {
      CLOUD_HISTO(cl2D,lowerEdgeY,upperEdgeY,valueY)
    }

  } else if(AIDA::ICloud3D* cl3D = Lib_Cast(aCloud,AIDA::ICloud3D)) {
   
    if(aCoord==0) {
      CLOUD_HISTO(cl3D,lowerEdgeX,upperEdgeX,valueX)
    } else if(aCoord==1) {
      CLOUD_HISTO(cl3D,lowerEdgeY,upperEdgeY,valueY)
    } else if(aCoord==2) {
      CLOUD_HISTO(cl3D,lowerEdgeZ,upperEdgeZ,valueZ)
    }

  }

  delete histogramFactory;

  return baseHistogram;
}
//////////////////////////////////////////////////////////////////////////////
static void cloud_histo_represent(
 Slash::UI::IUI& aUI
,AIDA::ICloud& aCloud
,int aCoord
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IPlotterRegion& region = plotter->currentRegion();

  region.clear();

  if(aCloud.isConverted()) {

    if(AIDA::ICloud1D* cl1D = Lib_Cast(aCloud,AIDA::ICloud1D)) {
      region.plot(cl1D->histogram());
    } else if(AIDA::ICloud2D* cl2D = Lib_Cast(aCloud,AIDA::ICloud2D)) {
      region.plot(cl2D->histogram());
    } else if(AIDA::ICloud3D* cl3D = Lib_Cast(aCloud,AIDA::ICloud3D)) {
      region.plot(cl3D->histogram());
    }

  } else {

    OnXLab::CloudExplorerManager* explorer = find_explorer(aUI);
    if(explorer) {
      int nbin;
      if(ui_parameterValue(aUI,"OnXLab_CloudExplorer_xnumber.value",nbin)) {
      double mn;
      if(ui_parameterValue(aUI,"OnXLab_CloudExplorer_xmin.value",mn)) {
      double mx;
      if(ui_parameterValue(aUI,"OnXLab_CloudExplorer_xmax.value",mx)) {
      AIDA::IBaseHistogram* rep = 
        represent_cloud(aUI.session(),aCloud,aCoord,nbin,mn,mx);
      if(rep) {
        explorer->setRep(rep);
        region.plot(*rep);
      }}}}
    }
  }

  delete plotter;
}

extern "C" {

void OnXLab_CloudExplorer_list_represent(Slash::UI::IUI&);

//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab::CloudExplorerManager* explorer = find_explorer(aUI);
  if(!explorer) {
    explorer = new OnXLab::CloudExplorerManager(aUI.session());
    aUI.session().addManager(explorer);
  }

  AIDA::ICloud* cloud = 0;
  Slash::Core::IManager* manager = 0;
  std::string path,basename;
 {AIDA::IManagedObject* object = mainTree_selection(aUI,manager,path,basename);
  if(manager) {
    AIDA::ITree* tree = Lib_Cast(*manager,AIDA::ITree);
    if(tree) {
      if(object) {
        cloud = Lib_Cast(*object,AIDA::ICloud);
        if(!cloud) {
          //aUI.echo(basename + " is not a cloud.");
        }
      } /*FIXME else {
        // May be a cloud in a Rio tree ?
        AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
        if(aida) {
          // Get the cloud from the rioTree :
          cloud = create_cloud(*aida,*tree,basename,"","");
          if(!cloud) {
            aUI.echo(basename + " not found.");
          }
        } 
      } */
    }
  }}

  explorer->setCloud(cloud);
  if(cloud) {
    int n = cloud->dimension();
    std::string s;
    for(int index = 0;index<n;index++) {
      s += Lib::smanip::int2coord(index) + "\n";
    }
    aUI.setParameter("OnXLab_CloudExplorer_list.items",s);
    aUI.setParameter("OnXLab_CloudExplorer_tree.value",manager->name());
    aUI.setParameter("OnXLab_CloudExplorer_cloud.value",path);
  } else {
    aUI.setParameter("OnXLab_CloudExplorer_list.items","");
    aUI.setParameter("OnXLab_CloudExplorer_tree.value","");
    aUI.setParameter("OnXLab_CloudExplorer_cloud.value","");
  }
  aUI.setParameter("OnXLab_CloudExplorer_xnumber.value","0");
  aUI.setParameter("OnXLab_CloudExplorer_xmin.value","0");
  aUI.setParameter("OnXLab_CloudExplorer_xmax.value","0");
  aUI.setParameter("OnXLab_CloudExplorer_xmin_scale.value","0");
  aUI.setParameter("OnXLab_CloudExplorer_xmax_scale.value","1000");

  if(cloud) {
    Lib::Out out(aUI.session().printer());
    out << "/////////////////" << Lib::endl;
    std::string name = BatchLab::aida_object_name<AIDA::ICloud>(*cloud);
    out << "name        : " << name << Lib::endl;
    out << "title       : " << cloud->title() << Lib::endl;
    out << "dimension   : " << cloud->dimension() << Lib::endl;
    out << "entries     : " << cloud->entries() << Lib::endl;
    out << "isConverted : " << cloud->isConverted() << Lib::endl;
  }

  OnXLab_CloudExplorer_list_represent(aUI);

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_infos(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string scoord;
  if(!aUI.parameterValue("OnXLab_CloudExplorer_list.value",scoord)) return;

  // Get the ICloud :
  AIDA::ICloud* cloud = getCloud(aUI);
  if(!cloud) return;

  Lib::Out out(aUI.session().printer());
  out << "/////////////////" << Lib::endl;
  out << "entries : " << cloud->entries() << Lib::endl;

  if(AIDA::ICloud1D* cl1D = Lib_Cast(*cloud,AIDA::ICloud1D)) {

    out << "min     : " << cl1D->lowerEdge() << Lib::endl;
    out << "max     : " << cl1D->upperEdge() << Lib::endl;
    out << "mean    : " << cl1D->mean() << Lib::endl;
    out << "rms     : " << cl1D->rms() << Lib::endl;

  } else if(AIDA::ICloud2D* cl2D = Lib_Cast(*cloud,AIDA::ICloud2D)) {

    out << "x min   : " << cl2D->lowerEdgeX() << Lib::endl;
    out << "x max   : " << cl2D->upperEdgeX() << Lib::endl;
    out << "x mean  : " << cl2D->meanX() << Lib::endl;
    out << "x rms   : " << cl2D->rmsX() << Lib::endl;

    out << "y min   : " << cl2D->lowerEdgeY() << Lib::endl;
    out << "y max   : " << cl2D->upperEdgeY() << Lib::endl;
    out << "y mean  : " << cl2D->meanY() << Lib::endl;
    out << "y rms   : " << cl2D->rmsY() << Lib::endl;

  } else if(AIDA::ICloud3D* cl3D = Lib_Cast(*cloud,AIDA::ICloud3D)) {

    out << "x min   : " << cl3D->lowerEdgeX() << Lib::endl;
    out << "x max   : " << cl3D->upperEdgeX() << Lib::endl;
    out << "x mean  : " << cl3D->meanX() << Lib::endl;
    out << "x rms   : " << cl3D->rmsX() << Lib::endl;

    out << "y min   : " << cl3D->lowerEdgeY() << Lib::endl;
    out << "y max   : " << cl3D->upperEdgeY() << Lib::endl;
    out << "y mean  : " << cl3D->meanY() << Lib::endl;
    out << "y rms   : " << cl3D->rmsY() << Lib::endl;

    out << "z min   : " << cl3D->lowerEdgeZ() << Lib::endl;
    out << "z max   : " << cl3D->upperEdgeZ() << Lib::endl;
    out << "z mean  : " << cl3D->meanZ() << Lib::endl;
    out << "z rms   : " << cl3D->rmsZ() << Lib::endl;
  }
  out << "isConverted : " << cloud->isConverted() << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_list_represent(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Get the ICloud :
  AIDA::ICloud* cloud = getCloud(aUI);
  if(!cloud) return;

  std::string rep;
  if(!aUI.parameterValue("OnXLab_CloudExplorer_rep.value",rep)) return;

  if(rep=="points") {

    aUI.setParameter("OnXLab_CloudExplorer_xnumber.value","0");
    aUI.setParameter("OnXLab_CloudExplorer_xmin.value","0");
    aUI.setParameter("OnXLab_CloudExplorer_xmax.value","0");
    aUI.setParameter("OnXLab_CloudExplorer_xmin_scale.value","0");
    aUI.setParameter("OnXLab_CloudExplorer_xmax_scale.value","1000");

    // Find AIDA :
    AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
    if(!aida) return;
    AIDA::IPlotter* plotter = create_plotter(*aida);        
    if(!plotter) return;
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.clear();
    if(!cloud->isConverted()) region.plot(*cloud);
    delete plotter;

  } else if(rep=="histo") {    

    int coord = 0; 
    if(cloud->isConverted()) {

      aUI.setParameter("OnXLab_CloudExplorer_xnumber.value","0");
      aUI.setParameter("OnXLab_CloudExplorer_xmin.value","0");
      aUI.setParameter("OnXLab_CloudExplorer_xmax.value","0");
      aUI.setParameter("OnXLab_CloudExplorer_xmin_scale.value","0");
      aUI.setParameter("OnXLab_CloudExplorer_xmax_scale.value","1000");

    } else {
      std::string s;
      if(!aUI.parameterValue("OnXLab_CloudExplorer_list.value",s)) return;
      if(!Lib::smanip::coord2int(s,coord)) return;

      double xmin = 0;
      double xmax = 0;

      if(AIDA::ICloud1D* cl1D = Lib_Cast(*cloud,AIDA::ICloud1D)) {
        xmin = cl1D->lowerEdge();
        xmax = cl1D->upperEdge();
      } else if(AIDA::ICloud2D* cl2D = Lib_Cast(*cloud,AIDA::ICloud2D)) {
        if(coord==0) {
          xmin = cl2D->lowerEdgeX();
          xmax = cl2D->upperEdgeX();
        } else if(coord==1) {
          xmin = cl2D->lowerEdgeY();
          xmax = cl2D->upperEdgeY();
        }
      } else if(AIDA::ICloud3D* cl3D = Lib_Cast(*cloud,AIDA::ICloud3D)) {
        if(coord==0) {
          xmin = cl3D->lowerEdgeX();
          xmax = cl3D->upperEdgeX();
        } else if(coord==1) {
          xmin = cl3D->lowerEdgeY();
          xmax = cl3D->upperEdgeY();
        } else if(coord==2) {
          xmin = cl3D->lowerEdgeZ();
          xmax = cl3D->upperEdgeZ();
        }
      }

      ui_setParameter(aUI,"OnXLab_CloudExplorer_xmin.value",xmin);
      ui_setParameter(aUI,"OnXLab_CloudExplorer_xmax.value",xmax);

      // Save for the scale drag callback :
      Slash::Core::ISession& session = aUI.session();
      session_setParameter(session,"OnXLab_CloudExplorer_xmin",xmin);
      session_setParameter(session,"OnXLab_CloudExplorer_xmax",xmax);

      aUI.setParameter("OnXLab_CloudExplorer_xmin_scale.value","0");
      aUI.setParameter("OnXLab_CloudExplorer_xmax_scale.value","1000");

    }

    cloud_histo_represent(aUI,*cloud,coord);
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_represent(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string rep;
  if(!aUI.parameterValue("OnXLab_CloudExplorer_rep.value",rep)) return;
  if(rep!="histo") return;
  AIDA::ICloud* cloud = getCloud(aUI);
  if(!cloud) return;
  std::string s;
  if(!aUI.parameterValue("OnXLab_CloudExplorer_list.value",s)) return;
  int i; 
  if(!Lib::smanip::coord2int(s,i)) return;
  cloud_histo_represent(aUI,*cloud,i);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_xmin_scale(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value = ui_cbk_double(aUI); //In [0,1000]

  Slash::Core::ISession& session = aUI.session();
  double xmin;
  if(!session_parameterValue(session,"OnXLab_CloudExplorer_xmin",xmin)) return;
  double xmax;
  if(!session_parameterValue(session,"OnXLab_CloudExplorer_xmax",xmax)) return;
  if(xmax<=xmin) return;

  double dx = xmax-xmin;
  dx *= value/1000.;
  double xmn = xmin + dx;
  ui_setParameter(aUI,"OnXLab_CloudExplorer_xmin.value",xmn);

  OnXLab_CloudExplorer_represent(aUI);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_CloudExplorer_xmax_scale(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double value = ui_cbk_double(aUI); //In [0,1000]

  Slash::Core::ISession& session = aUI.session();
  double xmin;
  if(!session_parameterValue(session,"OnXLab_CloudExplorer_xmin",xmin)) return;
  double xmax;
  if(!session_parameterValue(session,"OnXLab_CloudExplorer_xmax",xmax)) return;
  if(xmax<=xmin) return;

  double dx = xmax-xmin;
  dx *= value/1000.;
  double xmx = xmin + dx;
  ui_setParameter(aUI,"OnXLab_CloudExplorer_xmax.value",xmx);

  OnXLab_CloudExplorer_represent(aUI);
}

} // extern "C"
