//FIXME : pass through AIDA only !

// Lib :
#include <Lib/Processor.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/amanipT.h>

// AIDA :
#include <AIDA/IProfile1D.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/IFitResult.h>
#include <AIDA/IFitParameterSettings.h>

// Inventor :
#include <Inventor/errors/SoDebugError.h>

// HEPVis :
#include <HEPVis/SbPlottableThings.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodekits/SoPlotter.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLab_Fitter_fit(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find the histogram to plot :
  AIDA::IBaseHistogram* baseHisto = 0;
  //FIXME : query the plotter ?
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  SoRegion* soRegion = soPage->currentRegion();
  if(!soRegion) return;
  if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotter* soPlotter = ((SoPlotterRegion*)soRegion)->getPlotter();
  const SbPList& list = soPlotter->getPlottableObjects();
  int objn = list.getLength();
  std::string sclass;
  std::string sname;
  for(int index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)list[index];
    if(!object) continue; 
    if(!object->isValid()) {
      SoDebugError::postInfo
      ("SoPlotter::updateSceneGraph","object \"%s\" not valid.",
       object->getName());
      continue; 
    }
    if(Lib_Cast(*object,SbPlottableBins1D)) {
      const char* s = object->nativeObjectClass();
      sclass = s?s:"";
      if(sclass=="AIDA::IHistogram1D") {
        AIDA::IHistogram1D* aobject = 
          ((AIDA::IHistogram1D*)object->nativeObject());
        baseHisto = aobject;
        sname = BatchLab::aida_object_name<AIDA::IHistogram1D>(*aobject);
        break;
      } else if(sclass=="AIDA::IProfile1D") {
        AIDA::IProfile1D* aobject =  
          ((AIDA::IProfile1D*)object->nativeObject());
        baseHisto = aobject;
        sname = BatchLab::aida_object_name<AIDA::IProfile1D>(*aobject);
        break;
      } else if(sclass=="AIDA::ICloud1D") {
        AIDA::ICloud1D* aobject = ((AIDA::ICloud1D*)object->nativeObject());
        baseHisto = aobject;
        sname = BatchLab::aida_object_name<AIDA::ICloud1D>(*aobject);
        break;
      }
    } else if(Lib_Cast(*object,SbPlottablePoints2D)) {
      const char* s = object->nativeObjectClass();
      sclass = s?s:"";
      if(sclass=="AIDA::ICloud2D") {
        AIDA::ICloud2D* aobject = ((AIDA::ICloud2D*)object->nativeObject());
        baseHisto = aobject;
        sname = BatchLab::aida_object_name<AIDA::ICloud2D>(*aobject);
        break;
      }
    }
  }
  if(!baseHisto) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_fit : "
        << " found nothing to fit in the plotter." 
        << Lib::endl;
    return;
  } else {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_fit : "
        << " fit " << Lib::sout(sname) 
        << " of class " << Lib::sout(sclass) << "..." 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  std::string func_name;
  if(!aUI.parameterValue("OnXLab_Fitter_name.value",func_name)) return;
  std::string engine;
  if(!aUI.parameterValue("OnXLab_Fitter_engine.value",engine)) return;

  std::string path = func_name;

  AIDA::IManagedObject* object = memoryTree->find(path);
  if(!object) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_fit : "
        << " object " << Lib::sout(func_name) << " not found."
        << Lib::endl;
    return;
  }
  AIDA::IFunction* func = Lib_Cast(*object,AIDA::IFunction);
  if(!func) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_fit : "
        << " object " << Lib::sout(func_name) << " not an AIDA::IFunction."
        << Lib::endl;
    return;
  }

  std::vector<std::string> params = func->parameterNames();

  AIDA::IFitter* fitter = create_fitter(*aida,engine);
  if(!fitter) return;

  fitter->resetParameterSettings();

  std::vector<double> starts;
  std::vector<double> steps;
  for(unsigned int index=0;index<params.size();index++) {
    std::string sindex = Lib::smanip::tostring((int)index);
    double start;
    if(!ui_parameterValue
       (aUI,"OnXLab_Fitter_param_"+sindex+"_start.value",start)) {
      delete fitter;
      return;
    }
    starts.push_back(start);
    double step;
    if(!ui_parameterValue
       (aUI,"OnXLab_Fitter_param_"+sindex+"_step.value",step)) {
      delete fitter;
      return;
    }
    steps.push_back(step);
    //printf("debug : index %d start %g step %g\n",index,start,step);

    AIDA::IFitParameterSettings& ps = 
      fitter->fitParameterSettings(params[index]);
    ps.setStepSize(step);
  }

  func->setParameters(starts);

  AIDA::IFitResult* fitResult = fitter->fit(*baseHisto,*func);

  if(!fitResult) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_fit : "
        << " fit failed."
        << Lib::endl;
    delete fitter;
    return;
  }

  AIDA::IFunction& function = fitResult->fittedFunction();
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(plotter) {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.plot(function);
    delete plotter;
  }

  delete fitResult;

  delete fitter;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_Fitter_create_function(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Create function and set the param area in the palette.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  std::string func_name;
  if(!aUI.parameterValue("OnXLab_Fitter_name.value",func_name)) return;
  std::string func_title;
  if(!aUI.parameterValue("OnXLab_Fitter_title.value",func_title)) return;
  std::string codelet;
  if(!aUI.parameterValue("OnXLab_Fitter_codelet.value",codelet)) return;
  Lib::smanip::strip(codelet);

  std::string path = func_name;

 {AIDA::IManagedObject* object = memoryTree->find(path);
  if(object) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_create_function :"
        << " Function " << Lib::sout(func_name) << " already exists."
        << Lib::endl;
    return;
  }}

  AIDA::IFunction* func = 
    create_function(*aida,*memoryTree,path,func_title,codelet);
  if(!func) {
    Lib::Out out(aUI.session().printer());
    out << "OnXLab_Fitter_create_function :"
        << " Can't create function " << Lib::sout(func_name) << "."
        << Lib::endl;
    return;
  }

  //aUI.destroyComponent("OnXLab_Fitter_params_container");

  std::vector<std::string> params = func->parameterNames();

  std::string sxml;
  sxml += "<widget class=\"VContainer\" name=\"OnXLab_Fitter_params_container\">";

  sxml += "  <widget class=\"HBox\">";
  sxml += "    <expand>second</expand>";
  sxml += "    <sizeOfFixed>30</sizeOfFixed>";
  sxml += "    <widget class=\"Label\">";
  sxml += "      <label>Param</label>";
  sxml += "    </widget>";
  sxml += "    <widget class=\"HBox\">";
  sxml += "      <expand>both</expand>";
  sxml += "      <widget class=\"Label\">";
  sxml += "        <label>Start</label>";
  sxml += "      </widget>";
  sxml += "      <widget class=\"Label\">";
  sxml += "        <label>Step</label>";
  sxml += "      </widget>";
  sxml += "    </widget>";
  sxml += "  </widget>";

  for(unsigned int index=0;index<params.size();index++) {
    std::string sindex = Lib::smanip::tostring((int)index);

  sxml += "  <widget class=\"HBox\">";
  sxml += "    <expand>second</expand>";
  sxml += "    <sizeOfFixed>30</sizeOfFixed>";
  sxml += "    <widget class=\"Label\">";
  sxml += "      <label>"+params[index]+"</label>";
  sxml += "    </widget>";
  sxml += "    <widget class=\"HBox\">";
  sxml += "      <expand>both</expand>";
  sxml += "      <widget class=\"Entry\" name=\"OnXLab_Fitter_param_"+sindex+"_start\">";
  sxml += "        <value>0</value>";
  sxml += "      </widget>";
  sxml += "      <widget class=\"Entry\" name=\"OnXLab_Fitter_param_"+sindex+"_step\">";
  sxml += "        <value>0.1</value>";
  sxml += "      </widget>";
  sxml += "    </widget>";
  sxml += "  </widget>";
  }

  sxml += "</widget>";
  aUI.load("OnXLab_Fitter_params",sxml,false);
  aUI.show("OnXLab_Fitter_params_container");

}

} // extern "C"
