//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

//
// HEPVis and Inventor related callbacks.
//

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include <BatchLab/Helpers/AIDA.h>

#include <Lib/Randoms.h>
#include <Lib/System.h>
#include <Lib/fmanip.h>

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotation.h>

// HEPVis :
#include <HEPVis/nodes/SoImage.h>
#include <HEPVis/nodes/SoTextStyle.h>
#include <HEPVis/nodes/SoViewportRegion.h>
#include <HEPVis/nodekits/SoPage.h>
//#include <HEPVis/nodekits/SoPlotter.h>
#include <HEPVis/nodekits/SoDisplayRegion.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodekits/SoDetectorExample.h>
#include <HEPVis/nodekits/SoImageRegion.h>
#include <HEPVis/nodekits/SoTextRegion.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLab_ExaOnXLab_0(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;
  soPage->deleteRegions();

  SbBool titleVisible = soPage->titleVisible.getValue();
  float titleHeight = soPage->titleHeight.getValue();

  // Detector :
  {    
    float height = titleVisible?1-titleHeight:1;

    SoDisplayRegion* soRegion = 
      (SoDisplayRegion*)soPage->createRegion("SoDisplayRegion",
                                          0,0,1,height);  
    if(!soRegion) return;
    soPage->setCurrentRegion(soRegion);

    soRegion->color.setValue(SbColor(0.8F,0.8F,0.8F));
    
    SoSeparator* sep = soRegion->getStaticScene();

    SoScale* soScale = new SoScale;
    soScale->scaleFactor.setValue(0.1F,0.1F,0.1F);
    sep->addChild(soScale);

    sep->addChild(new SoDetectorExample());
  }    
   
  // Histo viewport :
  {
    // Find AIDA :
    AIDA::IAnalysisFactory* aida = find_aida(session);
    if(!aida) return;
    
    // Find memory tree :
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(!memoryTree) return;

    AIDA::IHistogram1D* h = 
      create_histogram1D(*aida,*memoryTree,"gauss","Gauss",100,-5,5);

    Lib::RandomGauss r(0,1);
    for(int index=0;index<10000;index++) h->fill(r.shoot(),1);

    AIDA::IPlotter* plotter = create_plotter(*aida);        
    if(!plotter) return;
    AIDA::IPlotterRegion* region = plotter->createRegion(0.01,0.01,0.3,0.3);
    if(!region) return;
    region->plot(*h);
    delete plotter;
  }

  soPage->titleVisible.setValue(FALSE);
  soPage->setCurrentRegion(0);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_ExaOnXLab_1(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  soPage->deleteRegions();
  soPage->titleVisible.setValue(FALSE);
  soPage->createRegions("SoPlotterRegion",1,1,0);

  /////////////////////////////////////////////////////////////////////////////
  // Histo plotting part :
  /////////////////////////////////////////////////////////////////////////////
  { 
    // Find AIDA :
    AIDA::IAnalysisFactory* aida = find_aida(session);
    if(!aida) return;
    
    // Find memory tree :
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(!memoryTree) return;

    AIDA::IPlotter* plotter = create_plotter(*aida);        
    if(!plotter) return;
    plotter->createRegions(1,1,0);
    plotter->setTitle("Histo and detector");
    
    AIDA::IHistogram1D* h = 
      create_histogram1D(*aida,*memoryTree,"h","Rand gauss",100,-5,5);
    
    Lib::RandomGauss r(0,1);
    for(int index=0;index<10000;index++) h->fill(r.shoot(),1);
    
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.setParameter("plotter.gridStyle.visible","FALSE");
    region.setParameter("plotter.wallStyle.visible","FALSE");
    //FIXME region.style().wallStyle().visible(false);
    //FIXME region.style().gridStyle().visible(false);

    region.plot(*h);
    
    delete plotter;
  } 
 
  {
    SoPage* soPage = ui_SoPage(aUI);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
    SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;

    ///////////////////////////////////////////////////////////////////////////
    // Add a detector in the plotter scene.
    ///////////////////////////////////////////////////////////////////////////
    //WARNING : lights accumulate. Then do not put the below
    // scene graph under plotter.etcSeparator which is under
    // the light of the SoPlotterRegion. We put it under
    // the etc separator of the plotter region.
    SoSeparator* etc = soPlotterRegion->getEtcSeparator();
    //SoSeparator* etc = soPlotterRegion->getPlotter()->getEtcSeparator();
    if(!etc) return;

    SoDirectionalLight* soLight = new SoDirectionalLight;
    soLight->color.setValue(1,1,1);
    soLight->direction.setValue(SbVec3f(1,-1,-10));
    etc->addChild(soLight);

    SoLightModel* soLightModel = new SoLightModel;
    soLightModel->model.setValue(SoLightModel::PHONG);
    etc->addChild(soLightModel);

    SoOrthographicCamera* soCamera = new SoOrthographicCamera;
    soCamera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA);
    soCamera->position.setValue(0,0,4);
    soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
    soCamera->height.setValue(2);
    soCamera->nearDistance.setValue(1);
    soCamera->farDistance.setValue(100);
    soCamera->focalDistance.setValue(4);
    etc->addChild(soCamera);

    SoTranslation* soTranslation = new SoTranslation;
    soTranslation->translation.setValue(0.8F,0.7F,0);
    etc->addChild(soTranslation);

    SoScale* soScale = new SoScale;
    soScale->scaleFactor.setValue(0.01F,0.01F,0.01F);
    etc->addChild(soScale);

    SoRotation* soRotation = new SoRotation;
    soRotation->rotation.setValue(SbRotation(SbVec3f(1,1,1),0.8F));
    etc->addChild(soRotation);

    etc->addChild(new SoDetectorExample());
  }
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_ExaOnXLab_2(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab_ExaOnXLab_1(aUI);

  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return;

  // A region with the Dirac equation done with a .gif produced 
  // from a .tex file :
  {
    SoImageRegion* soRegion = 
      (SoImageRegion*)soPage->createRegion("SoImageRegion",
                                           0.01F,0.85F,0.25F,0.25F);
    if(!soRegion) return;
    soPage->setCurrentRegion(soRegion);
    HEPVis_SoImage* soImage = soRegion->getImage();

    if(!Lib::System::isenv("ONXLABROOT")) {
      aUI.echo(" Environment variable ONXLABROOT not defined.");
    } else {
      std::string file = 
        Lib::fmanip::name("$ONXLABROOT/examples/data/Dirac.gif");

      soImage->fileName.setValue(file.c_str());

      aUI.echo(" The Dirac equation is an insertion");
      aUI.echo("of a GIF file done with the HEPVis/SoImage node.");
      aUI.echo("The GIF file had been produced from a latex file");
      aUI.echo("(examples/data/Dirac.tex) and with the dvi2bitmap program.");
    }
  }

  // A region with a Feynman diagram done with a .gif produced 
  // from a .tex file :
  {
    SoImageRegion* soRegion = 
      (SoImageRegion*)soPage->createRegion("SoImageRegion",
                                           0.5F,0.03F,0.25F,0.25F);
    if(!soRegion) return;
    soPage->setCurrentRegion(soRegion);
    HEPVis_SoImage* soImage = soRegion->getImage();

    if(!Lib::System::isenv("ONXLABROOT")) {
      aUI.echo(" Environment variable ONXLABROOT not defined.");
    } else {
      std::string file = 
        Lib::fmanip::name("$ONXLABROOT/examples/data/fmf_w_e_nu.gif");

      soImage->fileName.setValue(file.c_str());

      aUI.echo(" The Feynman equation is an insertion");
      aUI.echo("of a GIF file done with the HEPVis/SoImage node.");
      aUI.echo("The GIF file had been produced from a latex file");
      aUI.echo("(examples/data/fmf_w_e_nu.tex) by using the feynMF package");
      aUI.echo("and with the dvi2bitmap program.");
    }
  }

  // A region with a formula :
  {
    SoTextRegion* soRegion = 
      (SoTextRegion*)soPage->createRegion("SoTextRegion",
                                          0.4F,0.6F,0.5F,0.2F);
    if(!soRegion) return;
    soPage->setCurrentRegion(soRegion);

    soRegion->color.setValue(SbColor(0.8F,0.8F,0.8F));
    // PAW encoding :
    soRegion->text.set1Value(0,"W^+! -> e^+! [n]?e!");
    soRegion->text.set1Value(1,"W^-! -> [m]^-! |[n]?[m]!");

    SoTextStyle* soStyle = soRegion->getStyle(0);
    if(soStyle) {
      soStyle->encoding.setValue("PAW");
      soStyle->color.setValue(0,0,1);
      soStyle->fontName.setValue("times");
    }

    soStyle = soRegion->getStyle(1);
    if(soStyle) {
      soStyle->encoding.setValue("PAW");
      soStyle->color.setValue(1,0,0);
      soStyle->fontName.setValue("arialbd");
    }
  }

  // A region with LaJoconde :
  {
    SoImageRegion* soRegion = 
      (SoImageRegion*)soPage->createRegion("SoImageRegion",
                                           0.01F,0.03F,0.25F,0.25F);
    if(!soRegion) return;
    soPage->setCurrentRegion(soRegion);
    HEPVis_SoImage* soImage = soRegion->getImage();

    if(!Lib::System::isenv("ONXLABROOT")) {
      aUI.echo(" Environment variable ONXLABROOT not defined.");
    } else {
      std::string file = 
        Lib::fmanip::name("$ONXLABROOT/examples/data/LaJoconde.gif");

      soImage->fileName.setValue(file.c_str());

      soRegion->getViewportRegion()->horizontalShadow.setValue(10);
      soRegion->getViewportRegion()->verticalShadow.setValue(-10);

      soRegion->horizontalMargin.setValue(20);
      soRegion->verticalMargin.setValue(20);
    }
  }

  soPage->setCurrentRegion(0);

}

} // extern "C"
