// 
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//
// This callback file contains examples over AIDA only.
//

// Examples not stricly AIDA.


#include <OnXLab/Helpers/OnXLab.h>

#include <BatchLab/Helpers/AIDA.h>
#include <OnX/Helpers/OnX.h>

#include <Lib/Out.h>
#include <Lib/sout.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLab_collect_in_histo(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  std::string hname = "h_OnX_Shape_hits";
  if(memoryTree->find(hname)) memoryTree->rm(hname);

  AIDA::IHistogram1D* h = 
    create_histogram1D(*aida,*memoryTree,hname,"OnX Shape hits.",10,0,10);
  if(!h) return;

  aUI.executeScript("DLD","OnX OnX_shape_initialize");
  data_collect(session,"OnX_Shape","type==\"Cube\"");
  data_number(session);

  // Fill histogram with the data_collect :
  data_fill1D(session,hname,"hits");

  // Get current plotter :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(plotter) {
    plotter->createRegions(1,1,0);
    plotter->setTitle("Histo & data_collect");
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.plot(*h);
    delete plotter;
  }
}

//////////////////////////////////////////////////////////////////////////////
void OnXLab_collect_in_tuple(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  std::string tuple_name = "tuple_OnX_Shape";
  if(memoryTree->find(tuple_name)) memoryTree->rm(tuple_name);

  aUI.executeScript("DLD","OnX OnX_shape_initialize");
  data_collect(session,"OnX_Shape","");
  data_number(session);

  data_tuple(session,tuple_name,"OnX_Shape");

  Lib::Out out(aUI.session().printer());
  out << "Tuple " << Lib::sout(tuple_name)
      << " created in MemoryTree." 
      << Lib::endl;
}

} // extern "C"


