//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

#include <Lib/Manager.h>
#include <Lib/Time.h>

#include <OnX/Helpers/OnX.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Helpers/Filters.h>
#include <BatchLab/Core/amanip.h>
#include <BatchLab/Core/T_amanip.h>

#include "Helpers.h"

#define NotFound (-1)
#define NoLimit (-1)

namespace OnXLab {

class TupleExplorerManager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnXLab::TupleExplorerManager)
    else if_Lib_SCast(Slash::Core::IManager)
    else return 0;
  }
public:
  TupleExplorerManager(Slash::Core::ISession&)
  :fName("OnXLab_TupleExplorerManager"),fTuple(0),fRep(0){}
  virtual ~TupleExplorerManager(){
    delete fRep;
  }
  void setTuple(AIDA::ITuple* aTuple) { 
    fTuple = aTuple;
    delete fRep;
    fRep = 0;
  }
  AIDA::ITuple* tuple() const { return fTuple;}
  void setRep(AIDA::IBaseHistogram* aRep) {
    delete fRep;
    fRep = aRep;
  }
private:
  std::string fName;
  AIDA::ITuple* fTuple;
  AIDA::IBaseHistogram* fRep;
};
}

//////////////////////////////////////////////////////////////////////////////
static OnXLab::TupleExplorerManager* find_explorer(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return Lib_findManager
    (aUI.session(),"OnXLab_TupleExplorerManager",OnXLab::TupleExplorerManager);
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::ITuple* getTuple(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab::TupleExplorerManager* explorer = find_explorer(aUI);
  if(!explorer) return 0;
  return explorer->tuple();
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* represent_tuple(
 Slash::Core::ISession& aSession
,AIDA::ITuple& aTuple
,int aColumn
,int aXnumber
,double aXmin
,double aXmax
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IWriter& printer = aSession.printer();
  Lib::Out out(printer);

  if(aTuple.columns()<=0) {
    out << "represent_tuple :"
        << " tuple has no columns." << Lib::endl;
    return 0;
  }

  if( (aColumn<0) || (aColumn>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumn 
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  AIDA::IAnalysisFactory* aida = 
      Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
  if(!aida) return 0;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return 0;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return 0;

  std::string columnName = aTuple.columnName(aColumn);
  std::string name = BatchLab::amanip::tuple_name(aTuple) + "_" + columnName;

  double xmin = 0;
  double xmax = 0;
  if(aXmax<=aXmin) {
    // Guess the histogram booking parameters :
    Lib::Time begin = Lib::Time::now();
    //FIXME : optimize the below.
    xmin = aTuple.columnMin(aColumn);
    xmax = aTuple.columnMax(aColumn);
    if(aSession.verboseLevel()) {
      Lib::Time end = Lib::Time::elapsed(begin);
      out << "OnXLab::represent_tuple :"
          << " get min, max column elapsed time : " 
          << end.seconds() << " seconds."
          << Lib::endl;
    }
    if(xmax==xmin) {
      xmax = xmin + 1;
    } else {
      // Tilt xmax in order that max value is in the histo :
      xmax = xmax + (xmax-xmin) * 0.01;
    }
  } else {
    xmin = aXmin;
    xmax = aXmax;
  }

  int xnumber = aXnumber<=0 ? 100 : aXnumber;

  AIDA::IHistogram1D* histogram = 
    histogramFactory->createHistogram1D
      (path_not_managed()+"/"+name,name,xnumber,xmin,xmax);
  if(!histogram) {
    delete histogramFactory;
    return 0;
  }

  Lib::Time begin = Lib::Time::now();

  BatchLab::AcceptAll filter;
  BatchLab::T_amanip::project<AIDA::IHistogram1D>
       (printer,aTuple,*histogram,aColumn,filter);
  
  if(aSession.verboseLevel()) {
    Lib::Time end = Lib::Time::elapsed(begin);
    out << "OnXLab::represent_tuple :"
        << " fill histogram elapsed time : " 
        << end.seconds() << " seconds."
        << Lib::endl;
  }
  
  delete histogramFactory;
  
  return histogram;
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* represent_tuple(
 Slash::Core::ISession& aSession
,AIDA::ITuple& aTuple
,int aColumnX
,int aXnumber
,double aXmin
,double aXmax
,int aColumnY
,int aYnumber
,double aYmin
,double aYmax
,const std::string& aRep
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IWriter& printer = aSession.printer();
  Lib::Out out(printer);

  if(aTuple.columns()<=0) {
    out << "represent_tuple :"
        << " tuple has no columns." << Lib::endl;
    return 0;
  }

  if( (aColumnX<0) || (aColumnX>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumnX 
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  if( (aColumnY<0) || (aColumnY>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumnY
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  AIDA::IAnalysisFactory* aida = 
      Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
  if(!aida) return 0;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return 0;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return 0;

  std::string columnNameX = aTuple.columnName(aColumnX);
  std::string columnNameY = aTuple.columnName(aColumnY);
  std::string name = BatchLab::amanip::tuple_name(aTuple) 
    + "_" + columnNameX+ "_" + columnNameY;

  int xnumber = aXnumber<=0 ? 100 : aXnumber;
  double xmin = 0;
  double xmax = 0;
  if(aXmax<=aXmin) {
    xmin = aTuple.columnMin(aColumnX);
    xmax = aTuple.columnMax(aColumnX);
    if(xmax==xmin) {
      xmax = xmin + 1;
    } else {
      xmax = xmax + (xmax-xmin) * 0.01;
    }
  } else {
    xmin = aXmin;
    xmax = aXmax;
  }

  int ynumber = aYnumber<=0 ? 100 : aYnumber;
  double ymin = 0;
  double ymax = 0;
  if(aYmax<=aYmin) {
    ymin = aTuple.columnMin(aColumnY);
    ymax = aTuple.columnMax(aColumnY);
    if(ymax==ymin) {
      ymax = ymin + 1;
    } else {
      ymax = ymax + (ymax-ymin) * 0.01;
    }
  } else {
    ymin = aYmin;
    ymax = aYmax;
  }

  if(aRep=="Histogram2D") {
    AIDA::IHistogram2D* histogram = 
      histogramFactory->createHistogram2D(path_not_managed()+"/"+name,
                                          name,
                                          xnumber,xmin,xmax,
                                          ynumber,ymin,ymax);
    if(histogram) {
      BatchLab::AcceptAll filter;
      BatchLab::T_amanip::project<AIDA::IHistogram2D>
        (printer,aTuple,*histogram,aColumnX,aColumnY,filter);
    }

    delete histogramFactory;

    return histogram;

  } else if(aRep=="Profile1D") {
    AIDA::IProfile1D* profile = 
      histogramFactory->createProfile1D(path_not_managed()+"/"+name,
                                        name,
                                        xnumber,xmin,xmax,
                                        ymin,ymax);
    if(profile) {
      BatchLab::AcceptAll filter;
      BatchLab::T_amanip::project<AIDA::IProfile1D>
        (printer,aTuple,*profile,aColumnX,aColumnY,filter);
    }

    delete histogramFactory;

    return profile;

  } else if(aRep=="Cloud2D") {
    BatchLab::Filter2D filter(printer,aTuple,
                              aColumnX,xmin,xmax,
                              aColumnY,ymin,ymax);
    AIDA::ICloud2D* cloud = 
      histogramFactory->createCloud2D
        (path_not_managed()+"/"+name,name,NoLimit);
    if(cloud) {
      BatchLab::T_amanip::project<AIDA::ICloud2D>
        (printer,aTuple,*cloud,aColumnX,aColumnY,filter);
    }

    delete histogramFactory;

    return cloud;

  } else {
    out << "represent_tuple :"
        << " unknown representation " << aRep
        << Lib::endl;

    delete histogramFactory;

    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
static AIDA::IBaseHistogram* represent_tuple(
 Slash::Core::ISession& aSession
,AIDA::ITuple& aTuple
,int aColumnX
,int aXnumber
,double aXmin
,double aXmax
,int aColumnY
,int aYnumber
,double aYmin
,double aYmax
,int aColumnZ
,int aZnumber
,double aZmin
,double aZmax
,const std::string& aRep
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IWriter& printer = aSession.printer();
  Lib::Out out(printer);

  if(aTuple.columns()<=0) {
    out << "represent_tuple :"
        << " tuple has no columns." << Lib::endl;
    return 0;
  }

  if( (aColumnX<0) || (aColumnX>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumnX 
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  if( (aColumnY<0) || (aColumnY>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumnY
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  if( (aColumnZ<0) || (aColumnZ>=aTuple.columns()) ) {
    out << "represent_tuple :"
        << " column index out of range ; given " << aColumnZ
        << " range [0," << aTuple.columns() << "]." 
        << Lib::endl;
    return 0;
  }

  AIDA::IAnalysisFactory* aida = 
      Lib_findManager(aSession,"AnalysisFactory",AIDA::IAnalysisFactory);
  if(!aida) return 0;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return 0;

  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return 0;

  std::string columnNameX = aTuple.columnName(aColumnX);
  std::string columnNameY = aTuple.columnName(aColumnY);
  std::string columnNameZ = aTuple.columnName(aColumnZ);
  std::string name = BatchLab::amanip::tuple_name(aTuple) 
    + "_" + columnNameX + "_" + columnNameY+ "_" + columnNameZ;

  int xnumber = aXnumber<=0 ? 100 : aXnumber;
  double xmin = 0;
  double xmax = 0;
  if(aXmax<=aXmin) {
    xmin = aTuple.columnMin(aColumnX);
    xmax = aTuple.columnMax(aColumnX);
    if(xmax==xmin) {
      xmax = xmin + 1;
    } else {
      xmax = xmax + (xmax-xmin) * 0.01;
    }
  } else {
    xmin = aXmin;
    xmax = aXmax;
  }

  int ynumber = aYnumber<=0 ? 100 : aYnumber;
  double ymin = 0;
  double ymax = 0;
  if(aYmax<=aYmin) {
    ymin = aTuple.columnMin(aColumnY);
    ymax = aTuple.columnMax(aColumnY);
    if(ymax==ymin) {
      ymax = ymin + 1;
    } else {
      ymax = ymax + (ymax-ymin) * 0.01;
    }
  } else {
    ymin = aYmin;
    ymax = aYmax;
  }

  int znumber = aZnumber<=0 ? 100 : aZnumber;
  double zmin = 0;
  double zmax = 0;
  if(aZmax<=aZmin) {
    zmin = aTuple.columnMin(aColumnZ);
    zmax = aTuple.columnMax(aColumnZ);
    if(zmax==zmin) {
      zmax = zmin + 1;
    } else {
      zmax = zmax + (zmax-zmin) * 0.01;
    }
  } else {
    zmin = aZmin;
    zmax = aZmax;
  }

  if(aRep=="Histogram3D") {
    AIDA::IHistogram3D* histogram = 
      histogramFactory->createHistogram3D(path_not_managed()+"/"+name,
                                          name,
                                          xnumber,xmin,xmax,
                                          ynumber,ymin,ymax,
                                          znumber,zmin,zmax);
    if(histogram) {
      BatchLab::AcceptAll filter;
      BatchLab::T_amanip::project<AIDA::IHistogram3D>
        (printer,aTuple,*histogram,aColumnX,aColumnY,aColumnZ,filter);
    }

    delete histogramFactory;

    return histogram;

  } else if(aRep=="Profile2D") {
    AIDA::IProfile2D* profile = 
      histogramFactory->createProfile2D(path_not_managed()+"/"+name,
                                        name,
                                        xnumber,xmin,xmax,
                                        ynumber,ymin,ymax,
                                        zmin,zmax);
    if(profile) {
      BatchLab::AcceptAll filter;
      BatchLab::T_amanip::project<AIDA::IProfile2D>
        (printer,aTuple,*profile,aColumnX,aColumnY,aColumnZ,filter);
    }

    delete histogramFactory;

    return profile;

  } else if(aRep=="Cloud3D") {
    BatchLab::Filter3D filter(printer,aTuple,
                              aColumnX,xmin,xmax,
                              aColumnY,ymin,ymax,
                              aColumnZ,zmin,zmax);
    AIDA::ICloud3D* cloud = 
      histogramFactory->createCloud3D
        (path_not_managed()+"/"+name,name,NoLimit);
    if(cloud) {
      BatchLab::T_amanip::project<AIDA::ICloud3D>
        (printer,aTuple,*cloud,aColumnX,aColumnY,aColumnZ,filter);
    }

    delete histogramFactory;

    return cloud;

  } else {
    out << "represent_tuple :"
        << " unknown representation " << aRep
        << Lib::endl;

    delete histogramFactory;

    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
static void tuple_represent(
 Slash::UI::IUI& aUI
,AIDA::ITuple& aTuple
,int aColumnX
,int aColumnY
,int aColumnZ
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab::TupleExplorerManager* explorer = find_explorer(aUI);
  if(!explorer) return;

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;
  AIDA::IPlotter* plotter = create_plotter(*aida);        
  if(!plotter) return;
  AIDA::IPlotterRegion& region = plotter->currentRegion();

  region.clear();

  std::string srep;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_rep.value",srep)) return;
 {std::vector<std::string> words = Lib::smanip::words(srep," ");
  srep = words[0];}

  if(aColumnX==NotFound) {
  } else {

    int xnumber;
    if(!ui_parameterValue
      (aUI,"OnXLab_TupleExplorer_xnumber.value",xnumber)) return;
    double xmin;
    if(!ui_parameterValue(aUI,"OnXLab_TupleExplorer_xmin.value",xmin)) return;
    double xmax;
    if(!ui_parameterValue(aUI,"OnXLab_TupleExplorer_xmax.value",xmax)) return;

    if(aColumnY==NotFound) {

      AIDA::IBaseHistogram* rep = 
        represent_tuple(aUI.session(),aTuple,aColumnX,xnumber,xmin,xmax);
      if(rep) {
        explorer->setRep(rep);
        region.plot(*rep);
      }

    } else {

      int ynumber;
      if(!ui_parameterValue
        (aUI,"OnXLab_TupleExplorer_ynumber.value",ynumber)) return;
      double ymin;
      if(!ui_parameterValue
            (aUI,"OnXLab_TupleExplorer_ymin.value",ymin)) return;
      double ymax;
      if(!ui_parameterValue
            (aUI,"OnXLab_TupleExplorer_ymax.value",ymax)) return;

      if(aColumnZ==NotFound) {

        AIDA::IBaseHistogram* rep = 
          represent_tuple(aUI.session(),aTuple,
                          aColumnX,xnumber,xmin,xmax,
                          aColumnY,ynumber,ymin,ymax,srep);
        if(rep) {
          explorer->setRep(rep);
          region.plot(*rep);
        }

      } else {

        int znumber;
        if(!ui_parameterValue
          (aUI,"OnXLab_TupleExplorer_znumber.value",znumber)) return;
        double zmin;
        if(!ui_parameterValue
              (aUI,"OnXLab_TupleExplorer_zmin.value",zmin)) return;
        double zmax;
        if(!ui_parameterValue
              (aUI,"OnXLab_TupleExplorer_zmax.value",zmax)) return;

        AIDA::IBaseHistogram* rep = 
          represent_tuple(aUI.session(),aTuple,
                          aColumnX,xnumber,xmin,xmax,
                          aColumnY,ynumber,ymin,ymax,
                          aColumnZ,znumber,zmin,zmax,
                          srep);
        if(rep) {
          explorer->setRep(rep);
          region.plot(*rep);
        }

      }
    }

  }

  delete plotter;
}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLab_TupleExplorer_update(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  OnXLab::TupleExplorerManager* explorer = find_explorer(aUI);
  if(!explorer) {
    explorer = new OnXLab::TupleExplorerManager(aUI.session());
    aUI.session().addManager(explorer);
  }

  AIDA::ITuple* tuple = 0;
  Slash::Core::IManager* manager = 0;
  std::string path,basename;
 {AIDA::IManagedObject* object = mainTree_selection(aUI,manager,path,basename);
  if(manager) {
    AIDA::ITree* tree = Lib_Cast(*manager,AIDA::ITree);
    if(tree) {
      if(object) {
        tuple = Lib_Cast(*object,AIDA::ITuple);
        if(!tuple) {
          //aUI.echo(basename + " is not a tuple.");
        }
      } else {
        // May be a tuple in a Rio tree ?
        AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
        if(aida) {
          // Get the tuple from the rioTree :
          tuple = create_tuple(*aida,*tree,basename,"","");
          if(!tuple) {
            aUI.echo(basename + " not found.");
          }
        }
      }
    }
  }}

  explorer->setTuple(tuple);
  if(tuple) {
    int n = tuple->columns();
    std::string s = "none";
    for(int index = 0;index<n;index++) {
      s += "\n"+tuple->columnName(index);
    }
    aUI.setParameter("OnXLab_TupleExplorer_bind_x.items",s);
    aUI.setParameter("OnXLab_TupleExplorer_bind_y.items",s);
    aUI.setParameter("OnXLab_TupleExplorer_bind_z.items",s);
    aUI.setParameter("OnXLab_TupleExplorer_tree.value",manager->name());
    aUI.setParameter("OnXLab_TupleExplorer_tuple.value",path);
  } else {
    aUI.setParameter("OnXLab_TupleExplorer_bind_x.items","");
    aUI.setParameter("OnXLab_TupleExplorer_bind_y.items","");
    aUI.setParameter("OnXLab_TupleExplorer_bind_z.items","");
    aUI.setParameter("OnXLab_TupleExplorer_tree.value","");
    aUI.setParameter("OnXLab_TupleExplorer_tuple.value","");
  }
  aUI.setParameter("OnXLab_TupleExplorer_xnumber.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_xmin.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_xmax.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_xmin_scale.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_xmax_scale.value","1000");

  aUI.setParameter("OnXLab_TupleExplorer_ynumber.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_ymin.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_ymax.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_ymin_scale.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_ymax_scale.value","1000");

  aUI.setParameter("OnXLab_TupleExplorer_znumber.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_zmin.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_zmax.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_zmin_scale.value","0");
  aUI.setParameter("OnXLab_TupleExplorer_zmax_scale.value","1000");

  if(tuple) {
    Lib::Out out(aUI.session().printer());
    out << "/////////////////" << Lib::endl;
    out << "name    : " << BatchLab::amanip::tuple_name(*tuple) << Lib::endl;
    out << "title   : " << tuple->title() << Lib::endl;
    out << "columns : " << tuple->columns() << Lib::endl;
    out << "rows    : " << tuple->rows() << Lib::endl;
  }

}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_TupleExplorer_infos(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] x or y
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aArgs.size()) return;

  std::string scoord = aArgs[0];

  std::string column;
  if(!aUI.parameterValue
    ("OnXLab_TupleExplorer_bind_"+scoord+".value",column)) return;

  // Get the ITuple :
  AIDA::ITuple* tuple = getTuple(aUI);
  if(!tuple) return;
  int i = tuple->findColumn(column);
  if(i==NotFound) return;

  Lib::Out out(aUI.session().printer());
  out << "/////////////////" << Lib::endl;
  out << "name : " << tuple->columnName(i) << Lib::endl;
  out << "rows : " << tuple->rows() << Lib::endl;
  out << "type : " << tuple->columnType(i) << Lib::endl;
  out << "min  : " << tuple->columnMin(i) << Lib::endl;
  out << "max  : " << tuple->columnMax(i) << Lib::endl;
  out << "mean : " << tuple->columnMean(i) << Lib::endl;
  out << "rms  : " << tuple->columnRms(i) << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_TupleExplorer_bind(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Get the ITuple :
  AIDA::ITuple* tuple = getTuple(aUI);
  if(!tuple) return;

  int ix;
  std::string column;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_x.value",column)) return;
  if(column=="none") {
    aUI.setParameter("OnXLab_TupleExplorer_xnumber.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_xmin.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_xmax.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_xmin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_xmax_scale.value","1000");
    ix = NotFound;
  } else {
    ix = tuple->findColumn(column);
    if(ix==NotFound) return;
    double mn = tuple->columnMin(ix);
    double mx = tuple->columnMax(ix);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_xmin.value",mn);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_xmax.value",mx);
    // Save for the scale drag callback :
    Slash::Core::ISession& session = aUI.session();
    session_setParameter(session,"OnXLab_TupleExplorer_xmin",mn);
    session_setParameter(session,"OnXLab_TupleExplorer_xmax",mx);

    aUI.setParameter("OnXLab_TupleExplorer_xmin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_xmax_scale.value","1000");
  }

  int iy;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_y.value",column)) return;
  if(column=="none") {
    aUI.setParameter("OnXLab_TupleExplorer_ynumber.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_ymin.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_ymax.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_ymin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_ymax_scale.value","1000");
    iy = NotFound;
  } else {
    iy = tuple->findColumn(column);
    if(iy==NotFound) return;
    double mn = tuple->columnMin(iy);
    double mx = tuple->columnMax(iy);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_ymin.value",mn);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_ymax.value",mx);
    // Save for the scale drag callback :
    Slash::Core::ISession& session = aUI.session();
    session_setParameter(session,"OnXLab_TupleExplorer_ymin",mn);
    session_setParameter(session,"OnXLab_TupleExplorer_ymax",mx);

    aUI.setParameter("OnXLab_TupleExplorer_ymin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_ymax_scale.value","1000");
  }

  int iz;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_z.value",column)) return;
  if(column=="none") {
    aUI.setParameter("OnXLab_TupleExplorer_znumber.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_zmin.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_zmax.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_zmin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_zmax_scale.value","1000");
    iz = NotFound;
  } else {
    iz = tuple->findColumn(column);
    if(iz==NotFound) return;
    double mn = tuple->columnMin(iz);
    double mx = tuple->columnMax(iz);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_zmin.value",mn);
    ui_setParameter(aUI,"OnXLab_TupleExplorer_zmax.value",mx);
    // Save for the scale drag callback :
    Slash::Core::ISession& session = aUI.session();
    session_setParameter(session,"OnXLab_TupleExplorer_zmin",mn);
    session_setParameter(session,"OnXLab_TupleExplorer_zmax",mx);

    aUI.setParameter("OnXLab_TupleExplorer_zmin_scale.value","0");
    aUI.setParameter("OnXLab_TupleExplorer_zmax_scale.value","1000");
  }

  tuple_represent(aUI,*tuple,ix,iy,iz);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab_TupleExplorer_represent(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITuple* tuple = getTuple(aUI);
  if(!tuple) return;
  int ix;
  std::string column;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_x.value",column)) return;
  if(column=="none") {
    ix = NotFound;
  } else {
    ix = tuple->findColumn(column);
    if(ix==NotFound) return;
  }
  int iy;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_y.value",column)) return;
  if(column=="none") {
    iy = NotFound;
  } else {
    iy = tuple->findColumn(column);
    if(iy==NotFound) return;
  }
  int iz;
  if(!aUI.parameterValue("OnXLab_TupleExplorer_bind_z.value",column)) return;
  if(column=="none") {
    iz = NotFound;
  } else {
    iz = tuple->findColumn(column);
    if(iz==NotFound) return;
  }
  tuple_represent(aUI,*tuple,ix,iy,iz);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OnXLab_TupleExplorer_scale(
 Slash::UI::IUI& aUI
,const std::vector<std::string>& aArgs
) 
//////////////////////////////////////////////////////////////////////////////
// aArgs[0] x, y or z
// aArgs[1] min or max
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArgs.size()!=2) return;
  double value = ui_cbk_double(aUI); //In [0,1000]

  std::string scoord = aArgs[0];

  Slash::Core::ISession& session = aUI.session();
  double mn;
  if(!session_parameterValue
    (session,"OnXLab_TupleExplorer_"+scoord+"min",mn)) return;
  double mx;
  if(!session_parameterValue
    (session,"OnXLab_TupleExplorer_"+scoord+"max",mx)) return;
  if(mx<=mn) return;

  double dx = mx - mn;
  dx *= value/1000.;
  double v = mn + dx;
  ui_setParameter(aUI,"OnXLab_TupleExplorer_"+scoord+aArgs[1]+".value",v);

  OnXLab_TupleExplorer_represent(aUI);
}

} // extern "C"

