// this :
#include <OnXLab/Core/Main.h>

#include <Slash/Core/ISession.h>

#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Arguments.h>
#include <Lib/System.h>
#include <Lib/smanip.h>
#include <Lib/Cast.h>

#include <OnX/Core/Main.h>

#include <OnXLab/Core/AnalysisFactory.h>

//////////////////////////////////////////////////////////////////////////////
OnXLab::Main::Main(
 bool aCreateUI
,bool aShowUI
,const std::vector<std::string>& aArgs
)
:fOnX(0)
,fAnalysisFactory(0)
,fVerbose(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fOnX = new OnX::Main(aCreateUI,aShowUI,aArgs);

  if(!fOnX->isValid()) {
    delete fOnX;
    fOnX = 0;
    return;
  }
  Slash::Core::ISession* session = fOnX->session();
  if(!session) {
    delete fOnX;
    fOnX = 0;
    return;
  }

  fVerbose = session->verboseLevel()?true:false;

  Slash::Core::IManager* manager = session->findManager("AnalysisFactory");
  if(manager) {
    OnXLab::AnalysisFactory* factory = 
      Lib_Cast(*manager,OnXLab::AnalysisFactory);
    if(!factory) {
      Lib::Printer printer;  
      Lib::Out out(printer);
      out << "OnXLab::Main::Main :"
          << " WARNING : the session has already an AnalyisFactory" 
          << " but it is not an OnXLab::AnalyisFactory."
          << " Continue anyway." 
          << Lib::endl;
    }
    fAnalysisFactory = factory;
  } else {
    OnXLab::AnalysisFactory* factory = new OnXLab::AnalysisFactory(*session);
    session->addManager(factory);
    fAnalysisFactory = factory;
  }

  if(fVerbose) {    
    Lib::Printer printer;  
    Lib::Out out(printer);
    out << "OnXLab::Main::Main :"
        << " " << this << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::Main::~Main(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //The below will delete the ISession that will delete the AnalysisFactory.
  delete fOnX; 

  if(fVerbose) {
    Lib::Printer printer;  
    Lib::Out out(printer);
    out << "OnXLab::Main::~Main :"
        << " " << this << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Main::isValid(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return (fAnalysisFactory?true:false);
}
//////////////////////////////////////////////////////////////////////////////
OnX::Main* OnXLab::Main::onxMain(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fOnX;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
//////////////////////////////////////////////////////////////////////////////
AIDA::ITreeFactory* OnXLab::Main::createTreeFactory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTreeFactory();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IHistogramFactory* OnXLab::Main::createHistogramFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createHistogramFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITupleFactory* OnXLab::Main::createTupleFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTupleFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFunctionFactory* OnXLab::Main::createFunctionFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFunctionFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterFactory* OnXLab::Main::createPlotterFactory(
 int aArgc
,char** aArgv
,const std::string& aImp
,const std::string& aLib
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createPlotterFactory(aArgc,aArgv,aImp,aLib);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPointSetFactory* OnXLab::Main::createDataPointSetFactory(
 AIDA::ITree& aTree
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createDataPointSetFactory(aTree);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitFactory* OnXLab::Main::createFitFactory(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFitFactory();
}
#else
//////////////////////////////////////////////////////////////////////////////
AIDA::ITreeFactory* OnXLab::Main::createTreeFactory(
 const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTreeFactory(aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IHistogramFactory* OnXLab::Main::createHistogramFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createHistogramFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITupleFactory* OnXLab::Main::createTupleFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createTupleFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFunctionFactory* OnXLab::Main::createFunctionFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFunctionFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterFactory* OnXLab::Main::createPlotterFactory(
 int aArgc
,char** aArgv
,const std::string& aImp
,const std::string& aLib
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return 
    fAnalysisFactory->createPlotterFactory(aArgc,aArgv,aImp,aLib,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IDataPointSetFactory* OnXLab::Main::createDataPointSetFactory(
 AIDA::ITree& aTree
,const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createDataPointSetFactory(aTree,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IFitFactory* OnXLab::Main::createFitFactory(
 const std::string& aOptions
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createFitFactory(aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IGenericFactory* OnXLab::Main::createGenericFactory(
 const std::string& aFactoryType
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createGenericFactory(aFactoryType,aOptions);
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IGenericFactory* OnXLab::Main::createManagedObjectGenericFactory(
 const std::string& aFactoryType
,AIDA::ITree& aTree
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fAnalysisFactory) return 0;
  return fAnalysisFactory->createManagedObjectGenericFactory
           (aFactoryType,aTree,aOptions);
}
#endif
