// this :
#include <OnXLab/Core/TreeFactory.h>

#include <Slash/Core/IRelation.h>
#include <Slash/Core/IRelationManager.h>
#include <Lib/Debug.h>
#include <Lib/Cast.h>
#include <Lib/Manager.h>
#include <Lib/Messages.h>

#include <Slash/UI/IUI.h>
#include <OnX/Helpers/UI.h>

#include <BatchLab/Core/BaseTree.h> //FIXME

//////////////////////////////////////////////////////////////////////////////
OnXLab::TreeFactory::TreeFactory(
 Slash::Core::ISession& aSession
)
:BatchLab::TreeFactory(aSession)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnXLab::TreeFactory");
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::TreeFactory::~TreeFactory(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("OnXLab::TreeFactory");
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OnXLab::TreeFactory::create(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree =  BatchLab::TreeFactory::create();
  if(!tree) return 0;
  /*
  setUIRelation(*tree);

  //FIXME : we should be able to pass through AIDA :
  BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
  if(labTree) labTree->emitUpdate();
  */
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OnXLab::TreeFactory::createTree(
 const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree =  BatchLab::TreeFactory::create(aOptions);
  if(!tree) return 0;
  /*
  setUIRelation(*tree);

  //FIXME : we should be able to pass through AIDA :
  BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
  if(labTree) labTree->emitUpdate();
  */
  return tree;
}
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OnXLab::TreeFactory::createTree(
 const std::string& aStoreName
,const std::string& aStoreType
,int aMode
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree =  
    BatchLab::TreeFactory::createTree(aStoreName,aStoreType,aMode,aOptions);
  if(!tree) return 0;
  setUIRelation(*tree);

  //FIXME : we should be able to pass through AIDA :
  BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
  if(labTree) labTree->emitUpdate();

  return tree;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OnXLab::TreeFactory::createNamedTree(
 const std::string& aName
,const std::string& aStoreName
,const std::string& aStoreType
,int aMode
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree =  
    BatchLab::TreeFactory::createNamedTree
      (aName,aStoreName,aStoreType,aMode,aOptions);
  if(!tree) return 0;
  setUIRelation(*tree);

  //FIXME : we should be able to pass through AIDA :
  BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
  if(labTree) labTree->emitUpdate();

  return tree;
}
#endif
//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OnXLab::TreeFactory::create(
 const std::string& aStoreName
,const std::string& aStoreType
,bool aReadOnly
,bool aCreateNew
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree =  
    BatchLab::TreeFactory::create
      (aStoreName,aStoreType,aReadOnly,aCreateNew,aOptions);
  if(!tree) return 0;
  setUIRelation(*tree);

  //FIXME : we should be able to pass through AIDA :
  BatchLab::BaseTree* labTree = Lib_Cast(*tree,BatchLab::BaseTree);
  if(labTree) labTree->emitUpdate();

  return tree;
}

namespace OnXLab {
  class AIDA_ITree_IUI_Relation : public virtual Slash::Core::IRelation {
  public: // IRelation :
    virtual std::string name() const {return fName;}
    virtual void* cast(const std::string& aTo) const {
      if(aTo=="OnXLab::AIDA_ITree_IUI_Relation")  {
        return Lib_SCast(OnXLab::AIDA_ITree_IUI_Relation);
      } else if(aTo=="Slash::Core::IRelation") {
        return Lib_SCast(Slash::Core::IRelation);
      } else {
        return 0;
      }
    }
    virtual bool schedule() {
      if(fScheduled) return true;
      //printf("debug : OnXLab::TreeUI_Relation::notify\n");
      if(!fUI.notify(new Lib::UpdateMessage("mainTree",this))) return false;
      fScheduled = true;
      return true;
    }
    virtual void unSchedule() { 
      fScheduled = false;
    }
    virtual void* first() const { return &fTree;}
    virtual void* second() const { return &fUI;}
  public:
    AIDA_ITree_IUI_Relation(AIDA::ITree& aTree,Slash::UI::IUI& aUI)
    :fName("OnXLab::AIDA_ITree_IUI_Relation")
    ,fScheduled(false)
    ,fTree(aTree),fUI(aUI){
      Lib::Debug::increment("OnXLab::AIDA_ITree_IUI_Relation");
      //printf("debug : OnXLab::AIDA_ITree_IUI_Relation::AIDA_ITree_IUI_Relation : %ld\n",this);
    }
    virtual ~AIDA_ITree_IUI_Relation(){
      //printf("debug : OnXLab::AIDA_ITree_IUI_Relation::~AIDA_ITree_IUI_Relation : %ld\n",this);
      Lib::Debug::decrement("OnXLab::AIDA_ITree_IUI_Relation");
    }
  private:
    std::string fName;
    bool fScheduled;
    AIDA::ITree& fTree;
    Slash::UI::IUI& fUI;
  };
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::TreeFactory::setUIRelation(
 AIDA::ITree& aTree
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::IRelationManager* relationManager =
    Lib_findManager(fSession,"Lib::RelationManager",Slash::Core::IRelationManager);
  if(!relationManager) return;
  Slash::UI::IUI* ui = Lib_findUI(fSession);
  if(!ui) return;
  relationManager->add(new OnXLab::AIDA_ITree_IUI_Relation(aTree,*ui));
  //FIXME : aTree emitUpdate about its creation ?
}
