// this :
#include <OnXLab/Inventor/PlottableThings.h>

#include <HEPVis/SbPlottableThings.h>

#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/IProfile2D.h>
#include <AIDA/IAxis.h>
#include <AIDA/ITree.h>
#include <AIDA/ICloud1D.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/ICloud3D.h>
#include <AIDA/IFunction.h>
#include <AIDA/IDataPointSet.h>
#include <AIDA/IDataPoint.h>
#include <AIDA/IMeasurement.h>
#include <AIDA/IAnnotation.h>

#include <Lib/smanip.h>
#include <Lib/Cast.h>

#include <BatchLab/Core/Function.h>

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableHistogram1D::PlottableHistogram1D(
 const std::string& aName
,const AIDA::IHistogram1D* aHistogram
,INotifier* aNotifier
,bool aRetain
)
:fName(aName)
,fData(aHistogram)
,fDataClass("AIDA::IHistogram1D")
,fNotifier(aNotifier)
// Retain :
,fRetain(aRetain)
,f_dimension(0)
,f_minBinHeight(0)
,f_maxBinHeight(0)
,f_axis_bins(0)
,f_axis_lowerEdge(0)
,f_axis_upperEdge(0)
,f_allEntries(0)
,f_mean(0)
,f_rms(0)
,f_under(0)
,f_over(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  if(fRetain) { //PAW
    f_dimension = fData->dimension();
    f_minBinHeight = fData->minBinHeight();
    f_maxBinHeight = fData->maxBinHeight();
    //axis:
    f_axis_bins = fData->axis().bins();
    f_axis_lowerEdge = fData->axis().lowerEdge();
    f_axis_upperEdge = fData->axis().upperEdge();
    f_axis_binLowerEdge.resize(f_axis_bins);
    f_axis_binUpperEdge.resize(f_axis_bins);
   {for(int i=0;i<f_axis_bins;i++) {
      f_axis_binLowerEdge[i] = fData->axis().binLowerEdge(i);
      f_axis_binUpperEdge[i] = fData->axis().binUpperEdge(i);
    }}
    //bins:
    int n = f_axis_bins;
    f_binEntries.resize(n);
    f_binHeight.resize(n);
    f_binError.resize(n);
   {for(int i=0;i<n;i++) {
      f_binEntries[i] = fData->binEntries(i);
      f_binHeight[i] = fData->binHeight(i);
      f_binError[i] = fData->binError(i);
    }}
    //infos:
    f_allEntries = fData->allEntries();
    f_mean = fData->mean();
    f_rms = fData->rms();
    f_under = fData->binHeight(AIDA::IAxis::UNDERFLOW_BIN);
    f_over = fData->binHeight(AIDA::IAxis::OVERFLOW_BIN);
   {const AIDA::IAnnotation& annotation = fData->annotation();
    int number = annotation.size();
    for(int index=0;index<number;index++) {
      std::string key = annotation.key(index);
      std::string value = annotation.value(index);
      f_annotation.addItem(key,value);
    }}
    //forget data !
    fData = 0;
    delete fNotifier;
    fNotifier = 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableHistogram1D::~PlottableHistogram1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableHistogram1D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!strcmp(aClass,SbPlottableBins1D_s)) {
    return Lib_SCast(SbPlottableBins1D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableHistogram1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram1D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram1D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableHistogram1D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram1D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram1D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->dimension() : f_dimension;
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram1D::getInfos(
 const char* aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool show_name = false;
  bool show_entries = false;
  bool show_mean = false;
  bool show_rms = false;
  bool show_underflow = false;
  bool show_overflow = false;
  // Fit infos (if there are some) :
  bool show_fit_quality = false;
  bool show_fit_ndf = false;
  bool show_fit_errors = false;
  bool show_fit_parameters = false;
  
  std::vector<std::string> words;
  Lib::smanip::words(aOptions," ",words);
  for(unsigned int wordi=0;wordi<words.size();wordi++) {
    std::string word = words[wordi];
    Lib::smanip::tolowercase(word);
    if(word=="name") show_name = true; 
    if(word=="entries") show_entries = true; 
    if(word=="mean") show_mean = true; 
    if(word=="rms") show_rms = true; 
    if(word=="overflow") show_overflow = true; 
    if(word=="underflow") show_underflow = true; 
    if(word=="fit_quality") show_fit_quality = true; 
    if(word=="fit_ndf") show_fit_ndf = true; 
    if(word=="fit_errors") show_fit_errors = true; 
    if(word=="fit_parameters") show_fit_parameters = true; 
  }
  fInfos = "";
  if(show_name) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "Name\n"+fName;
  }
  if(show_entries) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "Entries\n"+Lib::smanip::tostring
      (fData ? fData->allEntries() : f_allEntries);
  }
  if(show_mean) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "Mean\n"+Lib::smanip::tostring
      (fData ? fData->mean() : f_mean);
  }
  if(show_rms) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "RMS\n"+Lib::smanip::tostring
      (fData ? fData->rms() : f_rms);
  }
  if(show_underflow) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "UDFLW\n"+Lib::smanip::tostring
      (fData ? fData->binHeight(AIDA::IAxis::UNDERFLOW_BIN) : f_under);
  }
  if(show_overflow) {
    if(fInfos!="") fInfos += "\n";
    fInfos += "OVFLW\n"+Lib::smanip::tostring
      (fData?fData->binHeight(AIDA::IAxis::OVERFLOW_BIN):f_over);
  }

  // See OnXLab/source/Core/BaseFitter.cxx/set_fit_infos for 
  // the annotation setup.
  if(show_fit_quality || show_fit_ndf ||
     show_fit_errors || show_fit_parameters) {
    const AIDA::IAnnotation* ANNOTATION  = &f_annotation;
    if(fData) ANNOTATION = &(fData->annotation());
    // ndf :
    int ndf = 0;
    std::string value = ANNOTATION->value("fit_ndf");
    Lib::smanip::toint(value,ndf);
    if(ndf) {
      if(show_fit_quality) {
        double quality = 0;
        value = ANNOTATION->value("fit_quality");
        Lib::smanip::todouble(value,quality);
        double chi2 = quality * ndf;
        //Need PAW encoding rendering for chi2 :
        if(show_fit_ndf) {
          if(fInfos!="") fInfos += "\n";
          fInfos += "[c]^2! / ndf\n"+ 
            Lib::smanip::tostring(chi2)+" / "+Lib::smanip::tostring(ndf);
        } else {
          if(fInfos!="") fInfos += "\n";
          fInfos += "[c]^2!\n"+Lib::smanip::tostring(chi2);
        }
      }
      if(show_fit_parameters) {
        int number = ANNOTATION->size();
        for(int index=0;index<number;index++) {
          std::string key = ANNOTATION->key(index);
          std::string value = ANNOTATION->value(index);
          if(key.substr(0,10)=="fit_param_") { 
            std::string name = key.substr(10,key.size()-10);
            std::vector<std::string> words;
            Lib::smanip::words(value," ",words);
            if(words.size()==2) { //value and error.
              double v;
              Lib::smanip::todouble(words[0],v);
              double e;
              Lib::smanip::todouble(words[1],e);
              if(show_fit_errors) {
                //Need PAW encoding rendering for +/- :
                if(fInfos!="") fInfos += "\n";
                fInfos += name+"\n"+
                  Lib::smanip::tostring(v)+" +&_ "+Lib::smanip::tostring(e);
              } else {
                if(fInfos!="") fInfos += "\n";
                fInfos += name+"\n"+Lib::smanip::tostring(v);
              }
            }
          }
        }
      }
    }
  }
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram1D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData ? fData->annotation().value("Legend"):
                    f_annotation.value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableHistogram1D::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    aMin = (float)fData->minBinHeight();
    aMax = (float)fData->maxBinHeight();
  } else {
    aMin = (float)f_minBinHeight;
    aMax = (float)f_maxBinHeight;
  }
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram1D::getAxisNumberOfBins(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->axis().bins() : f_axis_bins;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getAxisMinimum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->axis().lowerEdge() : (float)f_axis_lowerEdge;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getAxisMaximum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->axis().upperEdge() : (float)f_axis_upperEdge;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram1D::getBinNumberOfEntries(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->binEntries(aI) : f_binEntries[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getBinLowerEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->axis().binLowerEdge(aI) : 
                 (float)f_axis_binLowerEdge[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getBinUpperEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->axis().binUpperEdge(aI) : 
                 (float)f_axis_binUpperEdge[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getBinSumOfWeights(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->binHeight(aI) : (float)f_binHeight[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram1D::getBinBar(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->binError(aI) : (float)f_binError[aI];
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableHistogram2D::PlottableHistogram2D(
 const std::string& aName
,const AIDA::IHistogram2D* aHistogram
,INotifier* aNotifier
,bool aRetain
)
:fName(aName)
,fData(aHistogram)
,fDataClass("AIDA::IHistogram2D")
,fNotifier(aNotifier)
// Retain :
,fRetain(aRetain)
,f_dimension(0)
,f_minBinHeight(0)
,f_maxBinHeight(0)
,f_xAxis_bins(0)
,f_xAxis_lowerEdge(0)
,f_xAxis_upperEdge(0)
,f_yAxis_bins(0)
,f_yAxis_lowerEdge(0)
,f_yAxis_upperEdge(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  if(fRetain) { //PAW
    f_dimension = fData->dimension();
    f_minBinHeight = fData->minBinHeight();
    f_maxBinHeight = fData->maxBinHeight();
    //axes:
    f_xAxis_bins = fData->xAxis().bins();
    f_xAxis_lowerEdge = fData->xAxis().lowerEdge();
    f_xAxis_upperEdge = fData->xAxis().upperEdge();
    f_xAxis_binLowerEdge.resize(f_xAxis_bins);
    f_xAxis_binUpperEdge.resize(f_xAxis_bins);
   {for(int i=0;i<f_xAxis_bins;i++) {
      f_xAxis_binLowerEdge[i] = fData->xAxis().binLowerEdge(i);
      f_xAxis_binUpperEdge[i] = fData->xAxis().binUpperEdge(i);
    }}
    f_yAxis_bins = fData->yAxis().bins();
    f_yAxis_lowerEdge = fData->yAxis().lowerEdge();
    f_yAxis_upperEdge = fData->yAxis().upperEdge();
    f_yAxis_binLowerEdge.resize(f_yAxis_bins);
    f_yAxis_binUpperEdge.resize(f_yAxis_bins);
   {for(int j=0;j<f_yAxis_bins;j++) {
      f_yAxis_binLowerEdge[j] = fData->yAxis().binLowerEdge(j);
      f_yAxis_binUpperEdge[j] = fData->yAxis().binUpperEdge(j);
    }}
    //bins:
    int n = f_xAxis_bins * f_yAxis_bins;
    f_binEntries.resize(n);
    f_binHeight.resize(n);
    f_binError.resize(n);
   {for(int j=0;j<f_yAxis_bins;j++) {
      for(int i=0;i<f_xAxis_bins;i++) {
        int off = i + j * f_xAxis_bins;
        f_binEntries[off] = fData->binEntries(i,j);
        f_binHeight[off] = fData->binHeight(i,j);
        f_binError[off] = fData->binError(i,j);
      }
    }}
    fData = 0;
    delete fNotifier;
    fNotifier = 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableHistogram2D::~PlottableHistogram2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableHistogram2D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableBins2D_s)) {
    return Lib_SCast(SbPlottableBins2D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableHistogram2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram2D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram2D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableHistogram2D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram2D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram2D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->dimension() : f_dimension;
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram2D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    fInfos  = "Name\n"+fName;
    fInfos += "\n";
    fInfos += "Entries\n"+Lib::smanip::tostring(fData->allEntries());
    fInfos += "\n";
    fInfos += "Mean X\n"+Lib::smanip::tostring(fData->meanX());
    fInfos += "\n";
    fInfos += "Mean Y\n"+Lib::smanip::tostring(fData->meanY());
    fInfos += "\n";
    fInfos += "RMS X\n"+Lib::smanip::tostring(fData->rmsX());
    fInfos += "\n";
    fInfos += "RMS Y\n"+Lib::smanip::tostring(fData->rmsY());
    return fInfos.c_str();
  } else {
    fInfos = "";
    return fInfos.c_str();
  }
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableHistogram2D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    fLegend = fData->annotation().value("Legend");
  } else {
    fLegend = ""; //FIXME
  }
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableHistogram2D::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    aMin = (float)fData->minBinHeight();
    aMax = (float)fData->maxBinHeight();
  } else {
    aMin = (float)f_minBinHeight;
    aMax = (float)f_maxBinHeight;
  }
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram2D::getAxisNumberOfBinsX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->xAxis().bins() : f_xAxis_bins;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram2D::getAxisNumberOfBinsY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->yAxis().bins() : f_yAxis_bins;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getAxisMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->xAxis().lowerEdge() : 
                 (float)f_xAxis_lowerEdge;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getAxisMinimumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->yAxis().lowerEdge() :
                 (float)f_yAxis_lowerEdge;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getAxisMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->xAxis().upperEdge() :
                 (float)f_xAxis_upperEdge;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getAxisMaximumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->yAxis().upperEdge() : 
                 (float)f_yAxis_upperEdge;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableHistogram2D::getBinNumberOfEntries(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? fData->binEntries(aI,aJ) : 
                 f_binEntries[aI + aJ * f_xAxis_bins];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinLowerEdgeX(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->xAxis().binLowerEdge(aI) :
                 (float)f_xAxis_binLowerEdge[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinUpperEdgeX(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->xAxis().binUpperEdge(aI) :
                 (float)f_xAxis_binUpperEdge[aI];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinLowerEdgeY(
 int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->yAxis().binLowerEdge(aJ) :
                 (float)f_yAxis_binLowerEdge[aJ];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinUpperEdgeY(
 int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->yAxis().binUpperEdge(aJ) :
                 (float)f_yAxis_binUpperEdge[aJ];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinSumOfWeights(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->binHeight(aI,aJ) :
                 (float)f_binHeight[aI + aJ * f_xAxis_bins];
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableHistogram2D::getBinBar(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData ? (float)fData->binError(aI,aJ) :
                 (float)f_binError[aI + aJ * f_xAxis_bins];
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableProfile1D::PlottableProfile1D(
 const std::string& aName
,const AIDA::IProfile1D* aProfile
,INotifier* aNotifier
)
:fName(aName)
,fData(aProfile)
,fDataClass("AIDA::IProfile1D")
,fNotifier(aNotifier)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableProfile1D::~PlottableProfile1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableProfile1D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableBins1D_s)) {
    return Lib_SCast(SbPlottableBins1D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableProfile1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile1D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile1D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableProfile1D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile1D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile1D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile1D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  fInfos += "Entries\n"+Lib::smanip::tostring(fData->allEntries());
  fInfos += "\n";
  fInfos += "Mean\n"+Lib::smanip::tostring(fData->mean());
  fInfos += "\n";
  fInfos += "RMS\n"+Lib::smanip::tostring(fData->rms());
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile1D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile1D::getAxisNumberOfBins(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->axis().bins();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getAxisMinimum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->axis().lowerEdge();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getAxisMaximum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->axis().upperEdge();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableProfile1D::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = (float)fData->minBinHeight();
  aMax = (float)fData->maxBinHeight();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile1D::getBinNumberOfEntries(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->binEntries(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getBinLowerEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->axis().binLowerEdge(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getBinUpperEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->axis().binUpperEdge(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getBinSumOfWeights(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->binHeight(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile1D::getBinBar(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->binError(aI);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableProfile2D::PlottableProfile2D(
 const std::string& aName
,const AIDA::IProfile2D* aProfile
,INotifier* aNotifier
)
:fName(aName)
,fData(aProfile)
,fDataClass("AIDA::IProfile2D")
,fNotifier(aNotifier)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableProfile2D::~PlottableProfile2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableProfile2D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableBins2D_s)) {
    return Lib_SCast(SbPlottableBins2D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableProfile2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile2D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile2D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableProfile2D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile2D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile2D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile2D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  fInfos += "Entries\n"+Lib::smanip::tostring(fData->allEntries());
  fInfos += "\n";
  fInfos += "Mean X\n"+Lib::smanip::tostring(fData->meanX());
  fInfos += "\n";
  fInfos += "Mean Y\n"+Lib::smanip::tostring(fData->meanY());
  fInfos += "\n";
  fInfos += "RMS X\n"+Lib::smanip::tostring(fData->rmsX());
  fInfos += "\n";
  fInfos += "RMS Y\n"+Lib::smanip::tostring(fData->rmsY());
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableProfile2D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableProfile2D::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = (float)fData->minBinHeight();
  aMax = (float)fData->maxBinHeight();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile2D::getAxisNumberOfBinsX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->xAxis().bins();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile2D::getAxisNumberOfBinsY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->yAxis().bins();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getAxisMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->xAxis().lowerEdge();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getAxisMinimumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->yAxis().lowerEdge();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getAxisMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->xAxis().upperEdge();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getAxisMaximumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->yAxis().upperEdge();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableProfile2D::getBinNumberOfEntries(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->binEntries(aI,aJ);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinLowerEdgeX(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->xAxis().binLowerEdge(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinUpperEdgeX(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->xAxis().binUpperEdge(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinLowerEdgeY(
 int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->yAxis().binLowerEdge(aJ);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinUpperEdgeY(
 int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->yAxis().binUpperEdge(aJ);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinSumOfWeights(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->binHeight(aI,aJ);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableProfile2D::getBinBar(
 int aI
,int aJ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->binError(aI,aJ);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableFunction1D::PlottableFunction1D(
 const std::string& aName
,const AIDA::IFunction* aFunction
,INotifier* aNotifier
,bool //aRetain
)
:fName(aName)
,fData(aFunction)
,fDataClass("AIDA::IFunction")
,fNotifier(aNotifier)
,fBaseFunction(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  fBaseFunction = Lib_ConstCast(*aFunction,BatchLab::BaseFunction);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableFunction1D::~PlottableFunction1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableFunction1D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableFunction1D_s)) {
    return Lib_SCast(SbPlottableFunction1D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableFunction1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction1D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction1D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableFunction1D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction1D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableFunction1D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction1D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction1D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableFunction1D::value(
 float aX
,float& aValue
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<double> xs(1);
  xs[0] = (double)aX;
  if(fBaseFunction) {
    double value;
    if(!fBaseFunction->value(xs,value)) {
      aValue = 0;
      return false;
    }
    aValue = float(value);
    return true;
  } else {
    aValue = (float)fData->value(xs); //FIXME : throw in case of problem.
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableFunction1D::getNumberOfStepsX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingNumberOfStepsX");
  if(!s.size()) return 100;
  int v;
  if(!Lib::smanip::toint(s,v)) return 100;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction1D::getMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMinimumX");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction1D::getMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMaximumX");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableFunction2D::PlottableFunction2D(
 const std::string& aName
,const AIDA::IFunction* aFunction
,INotifier* aNotifier
,bool //aRetain
)
:fName(aName)
,fData(aFunction)
,fDataClass("AIDA::IFunction")
,fNotifier(aNotifier)
,fBaseFunction(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  //if(aRetain) {
  //  //FIXME : the aFunction may not be copiable.
  //  const Slash::Store::IObject* aobj = 
  //    Lib_ConstCast(*aFunction,Slash::Store::IObject);
  //  Slash::Store::IObject* obj = aobj->copy();
  //  fData = Lib_ConstCast(*obj,AIDA::IFunction);
  //}
  fBaseFunction = Lib_ConstCast(*aFunction,BatchLab::BaseFunction);
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableFunction2D::~PlottableFunction2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableFunction2D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableFunction2D_s)) {
    return Lib_SCast(SbPlottableFunction2D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableFunction2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction2D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction2D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableFunction2D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction2D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableFunction2D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction2D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableFunction2D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableFunction2D::value(
 float aX
,float aY
,float& aValue
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<double> xs(2);
  xs[0] = (double)aX;
  xs[1] = (double)aY;
  if(fBaseFunction) {
    double value;
    if(!fBaseFunction->value(xs,value)) {
      aValue = 0;
      return false;
    }
    aValue = float(value);
    return true;
  } else {
    aValue = (float)fData->value(xs);  //FIXME : throw in case of problem.
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableFunction2D::getNumberOfStepsX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingNumberOfStepsX");
  if(!s.size()) return 100;
  int v;
  if(!Lib::smanip::toint(s,v)) return 100;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction2D::getMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMinimumX");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction2D::getMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMaximumX");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableFunction2D::getNumberOfStepsY(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingNumberOfStepsY");
  if(!s.size()) return 100;
  int v;
  if(!Lib::smanip::toint(s,v)) return 100;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction2D::getMinimumY(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMinimumY");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableFunction2D::getMaximumY(
) const
//////////////////////////////////////////////////////////////////////////////
// Used by OpenPAW to fix the extend of the function really drawn.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = fData->annotation().value("PlottingMaximumY");
  if(!s.size()) return 0;
  float v;
  if(!Lib::smanip::tofloat(s,v)) return 0;
  return v;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//FIXME : two ways to display Cloud1D (x,w) or (count,x) (assuming w = 1)
// For the moment, we display (count,x). The xAxis is in [1,n+1]
// where n is the number of entries of the Cloud.
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud1D::PlottableCloud1D(
 const std::string& aName
,const AIDA::ICloud1D* aCloud
,INotifier* aNotifier
)
:fName(aName)
,fData(aCloud)
,fDataClass("AIDA::ICloud1D")
,fNotifier(aNotifier)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud1D::~PlottableCloud1D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableCloud1D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableBins1D_s)) {
    return Lib_SCast(SbPlottableBins1D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableCloud1D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud1D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud1D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableCloud1D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud1D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud1D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud1D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  fInfos += "Points\n"+Lib::smanip::tostring(fData->entries());
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud1D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableCloud1D::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = (float)fData->lowerEdge();
  aMax = (float)fData->upperEdge();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud1D::getAxisNumberOfBins(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->entries();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getAxisMinimum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)1;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getAxisMaximum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->entries()+1;
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud1D::getBinNumberOfEntries(
 int //aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getBinLowerEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(aI+1);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getBinUpperEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(aI+2);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getBinSumOfWeights(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->value(aI);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud1D::getBinBar(
 int //aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud2D::PlottableCloud2D(
 const std::string& aName
,const AIDA::ICloud2D* aCloud
,INotifier* aNotifier
,bool aRetain
)
:fName(aName)
,fData(aCloud)
,fDataClass("AIDA::ICloud2D")
,fNotifier(aNotifier)
// Retain :
,fRetain(aRetain)
,f_dimension(0)
,f_entries(0)
,f_lowerEdgeX(0)
,f_upperEdgeX(0)
,f_lowerEdgeY(0)
,f_upperEdgeY(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  if(fRetain) { //PAW
    f_dimension = fData->dimension();
    f_entries = fData->entries();
    f_lowerEdgeX = fData->lowerEdgeX();
    f_lowerEdgeY = fData->lowerEdgeY();
    f_upperEdgeX = fData->upperEdgeX();
    f_upperEdgeY = fData->upperEdgeY();
    f_valueX.resize(f_entries);
    f_valueY.resize(f_entries);
    for(int i=0;i<f_entries;i++) {
      f_valueX[i] = fData->valueX(i);
      f_valueY[i] = fData->valueY(i);
    }
   {const AIDA::IAnnotation& annotation = fData->annotation();
    int number = annotation.size();
    for(int index=0;index<number;index++) {
      std::string key = annotation.key(index);
      std::string value = annotation.value(index);
      f_annotation.addItem(key,value);
    }}
    //forget data !
    fData = 0;
    delete fNotifier;
    fNotifier = 0;
  }

}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud2D::~PlottableCloud2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableCloud2D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottablePoints2D_s)) {
    return Lib_SCast(SbPlottablePoints2D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableCloud2D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud2D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud2D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableCloud2D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud2D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud2D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fData?fData->dimension():f_dimension);
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud2D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  fInfos += "Points\n"+Lib::smanip::tostring(fData?fData->entries():f_entries);
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud2D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    fLegend = fData->annotation().value("Legend");
  } else { 
    fLegend = f_annotation.value("Legend");
  }
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud2D::getNumberOfPoints(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (fData?fData->entries():f_entries);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud2D::getAxisMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(fData?fData->lowerEdgeX():f_lowerEdgeX);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud2D::getAxisMinimumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(fData?fData->lowerEdgeY():f_lowerEdgeY);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud2D::getAxisMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(fData?fData->upperEdgeX():f_upperEdgeX);
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud2D::getAxisMaximumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)(fData?fData->upperEdgeY():f_upperEdgeY);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableCloud2D::getIthPoint(
 int aIndex
,float& aX
,float& aY
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    aX = (float)fData->valueX(aIndex);
    aY = (float)fData->valueY(aIndex);
  } else {
    aX = (float)f_valueX[aIndex];
    aY = (float)f_valueY[aIndex];
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud3D::PlottableCloud3D(
 const std::string& aName
,const AIDA::ICloud3D* aCloud
,INotifier* aNotifier
)
:fName(aName)
,fData(aCloud)
,fDataClass("AIDA::ICloud3D")
,fNotifier(aNotifier)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableCloud3D::~PlottableCloud3D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableCloud3D::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottablePoints3D_s)) {
    return Lib_SCast(SbPlottablePoints3D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableCloud3D::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud3D::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud3D::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableCloud3D::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud3D::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud3D::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->dimension();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud3D::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  fInfos += "Points\n"+Lib::smanip::tostring(fData->entries());
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableCloud3D::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fLegend = fData->annotation().value("Legend");
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableCloud3D::getNumberOfPoints(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fData->entries();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMinimumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->lowerEdgeX();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMinimumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->lowerEdgeY();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMinimumZ(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->lowerEdgeZ();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMaximumX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->upperEdgeX();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMaximumY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->upperEdgeY();
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableCloud3D::getAxisMaximumZ(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (float)fData->upperEdgeZ();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableCloud3D::getIthPoint(
 int aIndex
,float& aX
,float& aY
,float& aZ
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aX = (float)fData->valueX(aIndex);
  aY = (float)fData->valueY(aIndex);
  aZ = (float)fData->valueZ(aIndex);
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableDataPointSet::PlottableDataPointSet(
 const std::string& aName
,const AIDA::IDataPointSet* aDataPointSet
,INotifier* aNotifier
,bool aRetain
)
:fName(aName)
,fData(aDataPointSet)
,fDataClass("AIDA::IDataPointSet")
,fNotifier(aNotifier)
,fCoordinate(0) // Displayed coordinate.
//,fRep(0)
,fRep(1)
// Retain :
,fRetain(aRetain)
,f_dimension(0)
,f_size(0)
//////////////////////////////////////////////////////////////////////////////
// fRep = 0
//   Expect a DPS of any dimension>0.
//   X axis will be from [1,dps.size()+1] with dps.size() bins.
//   Y axis will display the fCoordinate measurement.
// fRep = 1
//   Expect a DPS of dim 2. 
//   Coordinate 0 will be displayed on plotter X axis.
//   Coordinate 1 will be displayed on plotter Y axis.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fTitle = fData->title();
  if(fData->dimension()!=2) fRep = 0;

  if(fRetain) { //PAW
    f_dimension = fData->dimension();
    f_size = fData->size();
    
    f_vs.resize(f_size);
    f_vms.resize(f_size);
    f_vps.resize(f_size);
    if(fRep==1) {
      f_xs.resize(f_size);
      f_xms.resize(f_size);
      f_xps.resize(f_size);
      for(int index=0;index<f_size;index++) {
        const AIDA::IDataPoint* point = fData->point(index);
        f_xs[index] = point->coordinate(0)->value();
        f_xms[index] = point->coordinate(0)->errorMinus();
        f_xps[index] = point->coordinate(0)->errorPlus();

        f_vs[index] = point->coordinate(1)->value();
        f_vms[index] = point->coordinate(1)->errorMinus();
        f_vps[index] = point->coordinate(1)->errorPlus();
      }
    } else {
      for(int index=0;index<f_size;index++) {
        const AIDA::IDataPoint* point = fData->point(index);
        f_vs[index] = point->coordinate(fCoordinate)->value();
        f_vms[index] = point->coordinate(fCoordinate)->errorMinus();
        f_vps[index] = point->coordinate(fCoordinate)->errorPlus();
      }
    }
    // Infos :
   {const AIDA::IAnnotation& annotation = fData->annotation();
    int number = annotation.size();
    for(int index=0;index<number;index++) {
      std::string key = annotation.key(index);
      std::string value = annotation.value(index);
      f_annotation.addItem(key,value);
    }}
    //forget data !
    fData = 0;
    delete fNotifier;
    fNotifier = 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlottableDataPointSet::~PlottableDataPointSet(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  delete fNotifier;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlottableDataPointSet::cast(
 const char* aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!::strcmp(aClass,SbPlottableObject_s)) {
    return Lib_SCast(SbPlottableObject);
  } else if(!::strcmp(aClass,SbPlottableBins1D_s)) {
    return Lib_SCast(SbPlottableBins1D);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlottableDataPointSet::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fNotifier) return true;
  return fNotifier->isValid();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableDataPointSet::getName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableDataPointSet::getLabel(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitle.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
void * OnXLab::PlottableDataPointSet::nativeObject(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (void*)fData; 
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableDataPointSet::nativeObjectClass(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDataClass.c_str(); 
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableDataPointSet::getDimension(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fData->dimension()<=0) return 0;
    if( (fRep==1) && (fData->dimension()!=2) ) return 0;
    return 1;
  } else {
    if(f_dimension<=0) return 0;
    if( (fRep==1) && (f_dimension!=2) ) return 0;
    return 1;
  }
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableDataPointSet::getInfos(
 const char* /*aOptions*/
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fInfos  = "Name\n"+fName;
  fInfos += "\n";
  if(fData) {
    fInfos += "Entries\n"+Lib::smanip::tostring(fData->size());
  } else {
    fInfos += "Entries\n"+Lib::smanip::tostring(f_size);
  }
  return fInfos.c_str();
}
//////////////////////////////////////////////////////////////////////////////
const char* OnXLab::PlottableDataPointSet::getLegend(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    fLegend = fData->annotation().value("Legend");
  } else {
    fLegend = f_annotation.value("Legend");
  }
  return fLegend.c_str();
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlottableDataPointSet::getBinsSumOfWeightsRange(
 float& aMin
,float& aMax
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = 0;
  aMax = 0;
  if(fData) {
    int number = fData->size();
    if(number<=0) return;
    int coord = (fRep==1?1:fCoordinate);
    float vmin = (float)fData->point(0)->coordinate(coord)->value();
    float vmax = (float)fData->point(0)->coordinate(coord)->value();
    for(int index=1;index<number;index++) {
      float value = (float)fData->point(index)->coordinate(coord)->value();
      vmin = MINIMUM(vmin,value);
      vmax = MAXIMUM(vmax,value);
    }
    aMin = vmin;
    aMax = vmax;
  } else {
    if(f_size<=0) return;
    float vmin = (float)f_vs[0];
    float vmax = vmin;
    for(int index=1;index<f_size;index++) {
      float value = (float)f_vs[index];
      vmin = MINIMUM(vmin,value);
      vmax = MAXIMUM(vmax,value);
    }
    aMin = vmin;
    aMax = vmax;
  }
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableDataPointSet::getAxisNumberOfBins(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    return fData->size();
  } else {
    return f_size;
  }
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getAxisMinimum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fRep==1) {
      const AIDA::IDataPoint* point = fData->point(0);
      return (float)(point->coordinate(0)->value() -
                     point->coordinate(0)->errorMinus());
    } else {
      return (float)1;
    }
  } else {
    if(fRep==1) {
      return float(f_xs[0]-f_xms[0]);
    } else {
      return (float)1;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getAxisMaximum(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fRep==1) {
      const AIDA::IDataPoint* point = fData->point(fData->size()-1);
      return (float)(point->coordinate(0)->value() +
                     point->coordinate(0)->errorPlus());
    } else {
      return (float)(fData->size()+1);
    }
  } else {
    if(fRep==1) {
      return float(f_xs[f_size-1]+f_xps[f_size-1]);
    } else {
      return (float)1;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::PlottableDataPointSet::getBinNumberOfEntries(
 int //aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return 1;
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getBinLowerEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(fData) {
    if(fRep==1) {
      const AIDA::IDataPoint* point = fData->point(aI);
      return (float)(point->coordinate(0)->value() -
                     point->coordinate(0)->errorMinus());
    } else {
      return (float)(aI+1);
    }
  } else {
    if(fRep==1) {
      return float(f_xs[aI]-f_xms[aI]);
    } else {
      return (float)(aI+1);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getBinUpperEdge(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fRep==1) {
      const AIDA::IDataPoint* point = fData->point(aI);
      return (float)(point->coordinate(0)->value() +
                     point->coordinate(0)->errorPlus());
    } else {
      return (float)(aI+2);
    }
  } else {
    if(fRep==1) {
      return float(f_xs[aI]+f_xps[aI]);
    } else {
      return (float)(aI+1);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getBinSumOfWeights(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fRep==1) {
      return (float)fData->point(aI)->coordinate(1)->value();
    } else {
      return (float)fData->point(aI)->coordinate(fCoordinate)->value();
    }
  } else {
    return (float)f_vs[aI];
  }
}
//////////////////////////////////////////////////////////////////////////////
float OnXLab::PlottableDataPointSet::getBinBar(
 int aI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fData) {
    if(fRep==1) {
      return float(fData->point(aI)->coordinate(1)->errorMinus()+
                   fData->point(aI)->coordinate(1)->errorPlus());
    } else {
      return 0;
    }
  } else {
    return (float)(f_vms[aI]+f_vps[aI]);
  }
}
