// this :
#include <OnXLab/Inventor/Plotter.h>

#include <AIDA/IHistogram.h>
#include <AIDA/IFunction.h>
#include <AIDA/ICloud.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IMessage.h>
#include <Slash/Core/IRelationManager.h>
#include <Slash/UI/IUI.h>

#include <Lib/Out.h>
#include <Lib/smanip.h>
#include <Lib/Debug.h>
#include <Lib/Manager.h>
#include <Lib/Cast.h>

#include <Inventor/errors/SoDebugError.h>

#include <HEPVis/SbPlottableThings.h>
#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodekits/SoPlotter.h>
#include <HEPVis/nodekits/SoRegion.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodekits/SoTextRegion.h>
#include <HEPVis/nodekits/SoPage.h>

#include <OnXLab/Inventor/Representer.h>

#define NotFound (-1)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterRegion::PlotterRegion(
 Slash::Core::IWriter& aPrinter
,Slash::Core::IRelationManager& aRelationManager
,int aVerboseLevel
,SoRegion* aSoRegion
,Slash::UI::IUI& aUI
)
:fPrinter(aPrinter)
,fRelationManager(aRelationManager)
,fVerboseLevel(aVerboseLevel)
,fSoRegion(aSoRegion)
,fUI(aUI)
,fSetParameter(aSoRegion)
,fPlotterStyle(fSetParameter,"plotter")
,fLayout(fSetParameter,"plotter")
,fRetain(false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnXLab::PlotterRegion");
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterRegion::PlotterRegion(
 const PlotterRegion& aRegion
)
:AIDA::IPlotterRegion()
,fPrinter(aRegion.fPrinter)
,fRelationManager(aRegion.fRelationManager)
,fVerboseLevel(aRegion.fVerboseLevel)
,fSoRegion(aRegion.fSoRegion)
,fUI(aRegion.fUI)
,fSetParameter(aRegion.fSoRegion)
,fPlotterStyle(fSetParameter,"plotter")
,fLayout(fSetParameter,"plotter")
,fRetain(aRegion.fRetain)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnXLab::PlotterRegion");
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterRegion::~PlotterRegion(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("OnXLab::PlotterRegion");
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterRegion& OnXLab::PlotterRegion::operator=(
 const PlotterRegion& aRegion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSoRegion = aRegion.fSoRegion;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
SoRegion* OnXLab::PlotterRegion::soRegion(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSoRegion;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IBaseHistogram& aObject
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IBaseHistogram& aObject
,const AIDA::IPlotterStyle&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IFunction& aObject
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,fUI,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IFunction& aObject
,const AIDA::IPlotterStyle&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,fUI,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IDataPointSet& aObject
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::plot(
 const AIDA::IDataPointSet& aObject
,const AIDA::IPlotterStyle&
,const std::string&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  SbPlottableObject* sbPlottable = 
    Representer::represent(aObject,fRelationManager,*soPlotter,fRetain,
                           fPrinter,fVerboseLevel);
  if(!sbPlottable) return false;
  fSoRegion->doIt(SbPlotterDoIt(sbPlottable));
  return true;
}
/*FIXME
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::plot(
 const AIDA::ICloud& aObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
*/
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::remove(
 const AIDA::IBaseHistogram&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::remove(
 const AIDA::IFunction&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::remove(
 const AIDA::IDataPointSet&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false; //FIXME
}
/*FIXME
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::remove(
 const AIDA::ICloud&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return false;
}
*/
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlotterRegion::clear(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSoRegion->clear("");
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::setParameter(
 const std::string& aParameter
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
// aString = field then do a set("<field> <value>")
// aString = part.field then do a set("<part>","<field> <value>")
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : we should have a AIDA::IPlotterRegion::refresh().
  //if(aParameter=="refresh") {
    //fSoRegion->refresh();
    //return true;
  //}
  if(aParameter=="retain") {
    Lib::smanip::tobool(aValue,fRetain);
    return true;
  }
  return (SoTools::setKitPartValue(fSoRegion,aParameter.c_str(),aValue.c_str())==TRUE?true:false);
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::PlotterRegion::parameterValue(
 const std::string& aParameter
) const 
//////////////////////////////////////////////////////////////////////////////
// aParameter = field then do a get("<field>")
// aParameter = part.field then do a getPart("<part>") and then
//  do a <field>.get("<value>")
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{  
  if(aParameter=="numberOfLegendRegions") { //OpenPAW/PAW_Manager::addLegend.
    if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return "";
    SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)fSoRegion;
    return Lib::smanip::tostring(soPlotterRegion->getNumberOfLegendRegions());
  }
  SbString value;
  SoTools::getKitPartValue(fSoRegion,aParameter.c_str(),value);
  return std::string(value.getString());
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterStyle & OnXLab::PlotterRegion::style(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPlotterStyle;
} 
// FIXME
bool OnXLab::PlotterRegion::setStyle(const AIDA::IPlotterStyle&) {return false;}
bool OnXLab::PlotterRegion::applyStyle(const AIDA::IPlotterStyle&) {return false;}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::PlotterRegion::setTitle(
 const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  soPlotter->titleAutomated.setValue(FALSE);
  soPlotter->title.setValue(aString.c_str());
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::setXLimits(
 double aMin
,double aMax
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  soPlotter->xAxisAutomated.setValue(FALSE);
  soPlotter->xAxisMinimum.setValue((float)aMin);
  soPlotter->xAxisMaximum.setValue((float)aMax);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::setYLimits(
 double aMin
,double aMax
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  soPlotter->yAxisAutomated.setValue(FALSE);
  soPlotter->yAxisMinimum.setValue((float)aMin);
  soPlotter->yAxisMaximum.setValue((float)aMax);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterRegion::setZLimits(
 double aMin
,double aMax
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fSoRegion->isOfType(SoPlotterRegion::getClassTypeId())) return false;
  SoPlotter* soPlotter = ((SoPlotterRegion*)fSoRegion)->getPlotter();
  soPlotter->zAxisAutomated.setValue(FALSE);
  soPlotter->zAxisMinimum.setValue((float)aMin);
  soPlotter->zAxisMaximum.setValue((float)aMax);
  return true;
}
AIDA::IPlotterLayout& OnXLab::PlotterRegion::layout() { return fLayout;}
bool OnXLab::PlotterRegion::setLayout(const AIDA::IPlotterLayout&) {return false;}
AIDA::IInfo& OnXLab::PlotterRegion::info() { return fInfo;}
std::vector<std::string> OnXLab::PlotterRegion::availableParameterOptions(
 const std::string&) const { return std::vector<std::string>();}
std::vector<std::string> OnXLab::PlotterRegion::availableParameters() const {
  return std::vector<std::string>();
}

#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
bool OnXLab::PlotterRegion::plot(
 const AIDA::IPlottable&
,const std::string&) {
  return false;
}
bool OnXLab::PlotterRegion::plot(
 const AIDA::IPlottable&
,const AIDA::IPlotterStyle&
,const std::string&) {
  return false;
}
bool OnXLab::PlotterRegion::remove(const AIDA::IPlottable&) { return false;}
bool OnXLab::PlotterRegion::refresh() { return false;}
std::string OnXLab::PlotterRegion::title() const { return "";}
double OnXLab::PlotterRegion::xLimitMin() const { return 0;}
double OnXLab::PlotterRegion::xLimitMax() const { return 0;}
double OnXLab::PlotterRegion::yLimitMin() const { return 0;}
double OnXLab::PlotterRegion::yLimitMax() const { return 0;}
double OnXLab::PlotterRegion::zLimitMin() const { return 0;}
double OnXLab::PlotterRegion::zLimitMax() const { return 0;}
void* OnXLab::PlotterRegion::cast(const std::string&) const { return 0;}
#endif
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OnXLab::Plotter::Plotter(
 Slash::Core::IWriter& aPrinter
,Slash::UI::IUI& aUI
,int aVerboseLevel
,SoPage& aSoPage
,const std::string& aName
)
:fPrinter(aPrinter)
,fUI(aUI)
,fVerboseLevel(aVerboseLevel)
,fSoPage(aSoPage)
,fName(aName)
,fSetParameter(&aSoPage)
,fTitleStyle(fSetParameter,"titleRegion.styles[0]")
,fRelationManager(0)
,fPlotterStyle(fSetParameter,"")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::increment("OnXLab::Plotter");
  fRelationManager = 
    Lib_findManager(aUI.session(),"Lib::RelationManager",Slash::Core::IRelationManager);
  if(!fRelationManager) {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::Plotter :"
        << " Lib::RelationManager not found.."
        << Lib::endl;
    return;
  }
  SoRegion* soRegion = currentSoRegion();
  if(!soRegion) {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::Plotter :"
        << " no SoRegion available."
        << Lib::endl;
    return;
  }
  plotterRegion(soRegion,fUI); // Have at least one entry in fRegions.
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::Plotter::~Plotter(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::decrement("OnXLab::Plotter");
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::isValid(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fSoPage.getNumberOfRootRegions()<=0) return false;
  SoRegion* soRegion = currentSoRegion();
  if(!soRegion) return false;  
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::Plotter::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if_Lib_SCast(Slash::UI::IPlotter)
  else if_Lib_SCast(AIDA::IPlotter)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
SoPage& OnXLab::Plotter::soPage(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fSoPage;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI& OnXLab::Plotter::ui(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUI;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterRegion* OnXLab::Plotter::plotterRegion(
 SoRegion* aSoRegion
,Slash::UI::IUI& aUI
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib_self(OnXLab::Plotter);
  std::list<PlotterRegion>::iterator it;
  for(it=self.fRegions.begin();it!=self.fRegions.end();++it) {
    if((*it).soRegion()==aSoRegion) return &(*it);
  }
  self.fRegions.push_back
    (PlotterRegion(fPrinter,*fRelationManager,fVerboseLevel,aSoRegion,aUI));
  return &(self.fRegions.back());
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::createRegions(
 int aCols
,int aRows
,int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aCols<0)&&(aRows<0)) { //FIXME : beurk. Should have an AIDA method.
    SoRegion* soRegion = 
      fSoPage.createGridRegion("SoPlotterRegion",-aCols,-aRows,aIndex);
    if(!soRegion) return false;
    if(fSoPage.getNumberOfRootRegions()==1) fSoPage.highlightRegion(soRegion);
    fSoPage.highlightRegion(soRegion);
  } else {
    fSoPage.createRegions("SoPlotterRegion",aCols,aRows,aIndex);
    fRegions.clear();
    SoRegion* soRegion = currentSoRegion();
    if(!soRegion) return false;
    plotterRegion(soRegion,fUI); // Have an entry in fRegions.
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterRegion* OnXLab::Plotter::createRegion(
 double aX
,double aY
,double aWidth
,double aHeight                           
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = fSoPage.createRegion
    ("SoPlotterRegion",(float)aX,(float)aY,(float)aWidth,(float)aHeight);
  if(!soRegion) return 0;
  if(fSoPage.getNumberOfRootRegions()==1) fSoPage.highlightRegion(soRegion);
  return plotterRegion(soRegion,fUI);
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::Plotter::clearRegions(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSoPage.clearRegions();
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::setParameter(
 const std::string& aParameter
,const std::string& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoTools::setKitPartValue(&fSoPage,aParameter.c_str(),aValue.c_str())==TRUE?true:false);
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::Plotter::parameterValue(
 const std::string& aParameter
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbString value;
  SoTools::getKitPartValue((SoBaseKit*)&fSoPage,aParameter.c_str(),value);
  return std::string(value.getString());
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterRegion& OnXLab::Plotter::next(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* highlighted = fSoPage.getHighlightedRegion();
  if(highlighted) {
    SoRegion* nextSoRegion = fSoPage.getNextRootRegion(highlighted);
    if(nextSoRegion) {
      fSoPage.highlightRegion(nextSoRegion);
    }
  }

  SoRegion* soRegion = currentSoRegion();
  if(!soRegion) {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::next :"
        << " no SoRegion available !"
        << Lib::endl;
  }
  return *plotterRegion(soRegion,fUI);
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::setCurrentRegionNumber(
 int aIndex
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = fSoPage.getRootRegion(aIndex);
  if(!soRegion) return false;
  fSoPage.highlightRegion(soRegion);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterRegion& OnXLab::Plotter::currentRegion(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = currentSoRegion();
  if(!soRegion) {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::currentRegion :"
        << " no region available !"
        << Lib::endl;
  }
  return *plotterRegion(soRegion,fUI);
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::Plotter::currentRegionNumber(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* highlighted = fSoPage.getHighlightedRegion();
  if(!highlighted) return NotFound;
  return fSoPage.getRootRegionIndex(highlighted);
}
//////////////////////////////////////////////////////////////////////////////
int OnXLab::Plotter::numberOfRegions(
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fSoPage.getNumberOfRootRegions();
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::writeToFile(
 const std::string& aFileName
,const std::string& aType
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fUI.synchronize();
  if( (aType=="PS") || (aType=="PostScript") ) {
    fUI.write(fName,"PS",aFileName);
  } else if(aType=="GL2PS") {
    fUI.write(fName,"GL2PS",aFileName);
  } else if(aType=="GL2PDF") {
    fUI.write(fName,"GL2PDF",aFileName);
  } else if(aType=="GIF") {
    fUI.write(fName,"GIF",aFileName);
  } else if(aType=="JPEG") {
    fUI.write(fName,"JPEG 50",aFileName);
  } else if(aType=="") {
    // Guess type from file suffix :
    std::string path,name,suffix;
    Lib::smanip::pathNameSuffix(aFileName,path,name,suffix);
    Lib::smanip::tolowercase(suffix);
    if(suffix=="ps") {
      fUI.write(fName,"PS",aFileName);
    } else if(suffix=="gif") {
      fUI.write(fName,"GIF",aFileName);
    } else {
      Lib::Out out(fPrinter);
      out << "OnXLab::Plotter::write : format " << suffix 
          << " not supported." << Lib::endl;
    }
  } else {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::write : format " << aType 
        << " not supported." << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::show(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fUI.show(fName);
  fUI.synchronize();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::hide(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fUI.hide(fName);
  fUI.synchronize();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::refresh(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSoPage.updateChildren(); //To update geom of regions.
  fSoPage.refreshRegions();
  fUI.synchronize();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::Plotter::interact(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fUI.steerLevel()) fUI.steer();
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::Plotter::setTitle(
 const std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Assume a title on one line.
  fSoPage.title.set1Value(0,aString.c_str());
  if(aString=="") {
    fSoPage.titleVisible.setValue(FALSE);
  } else {
    fSoPage.titleVisible.setValue(TRUE);
  }
  //  Force an update now so that further createRegion(x,y,w,h),
  // that uses NDC of the page, be correctly done.
  // Else AIDA example PlotExample.cpp will not be correct.
  fSoPage.updateChildren();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::ITitleStyle& OnXLab::Plotter::titleStyle(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fTitleStyle;
}
//////////////////////////////////////////////////////////////////////////////
void OnXLab::Plotter::destroyRegions(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSoPage.deleteRegions();
}
void OnXLab::Plotter::setTitleStyle(const AIDA::ITitleStyle&) {}
std::vector<std::string> OnXLab::Plotter::availableParameterOptions(const std::string&) const { return std::vector<std::string>();}
std::vector<std::string> OnXLab::Plotter::availableParameters() const {
  return std::vector<std::string>();
}
//////////////////////////////////////////////////////////////////////////////
AIDA::IPlotterRegion* OnXLab::Plotter::region(
 int aIndex
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* soRegion = fSoPage.getRootRegion(aIndex);
  if(!soRegion) {
    Lib::Out out(fPrinter);
    out << "OnXLab::Plotter::region :"
        << " SoRegion " << aIndex << " not found."
        << Lib::endl;
    return 0;
  }
  return plotterRegion(soRegion,fUI);
}
//////////////////////////////////////////////////////////////////////////////
SoRegion* OnXLab::Plotter::currentSoRegion(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoRegion* highlighted = fSoPage.getHighlightedRegion();
  if(!highlighted) return 0;
  if(fSoPage.isRootRegion(highlighted)==FALSE) return 0;
  return highlighted;
}
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
#else
std::string OnXLab::Plotter::title() const { return "";}
AIDA::IPlotterStyle& OnXLab::Plotter::style() {
  return fPlotterStyle;
}
void OnXLab::Plotter::setStyle(const AIDA::IPlotterStyle&) {}
#endif
