// this :
#include <OnXLab/Inventor/PlotterManager.h>

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Slash/UI/IUI.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>

// Lib:
#include <Lib/Manager.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/dirmanip.h>
#include <Lib/fmanip.h>
#include <Lib/Cast.h>
#include <Lib/System.h>

#include <OnX/Helpers/UI.h>

#include <OnXLab/Inventor/Plotter.h>

#include <HEPVis/nodekits/SoPage.h>

#include <OnX/Core/Environment.h>
#include <OnXLab/Core/Environment.h>

#include <OnXLab/Inventor/SimplePlotter_onx.h>

//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterManager::PlotterManager(
 Slash::Core::ISession& aSession
)
:fName("OnXLab_PlotterManager")
,fSession(aSession)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OnXLab::PlotterManager::~PlotterManager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
std::string OnXLab::PlotterManager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* OnXLab::PlotterManager::cast(
 const std::string& aClass
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
       if_Lib_SCast(OnXLab::PlotterManager)
  else if_Lib_SCast(Slash::UI::IPlotterManager)
  else if_Lib_SCast(Slash::Core::IManager)
  else return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool OnXLab::PlotterManager::setArguments(
 const std::vector<std::string>& aArgs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fArgs = aArgs;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IPlotter* OnXLab::PlotterManager::create(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IUI* ui = Lib_findUI(fSession);
  if(!ui) ui = loadUI();
  if(!ui) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::create :"
        << " no UI found." 
        << Lib::endl;
    return 0;
  }
  Slash::UI::IWidget* w = 0;

  std::string currentViewer;
 {Slash::UI::IWidget* cw = ui->currentWidget();
  if(cw && Lib_Cast(*cw,Slash::UI::ISoViewer)) currentViewer = cw->name();}

  std::string wname;
  if( (aName!="")&&(aName!=currentViewer) ) {
    wname = aName;
    w = ui->findWidget(wname);
    if(!w) {
      Lib::Out out(fSession.printer());
      out << "OnXLab::PlotterManager::create :"
        << " widget " << Lib::sout(wname) << " not found." 
        << " Try current viewer." 
        << Lib::endl;
    }
  }
  if(!w) {
    wname = currentViewer;
    w = ui->findWidget(wname);
    if(!w) {
      Lib::Out out(fSession.printer());
      out << "OnXLab::PlotterManager::create :"
        << " current viewer " << Lib::sout(wname) << " not found." 
        << Lib::endl;
    }
  }
  if(!w) return 0;

  Slash::UI::ISoViewer* soViewer = Lib_Cast(*w,Slash::UI::ISoViewer);
  if(!soViewer) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::create :"
        << " widget " << Lib::sout(wname) << " not an Slash::UI::ISoViewer." 
        << Lib::endl;
    return 0;
  }
  SoNode* soNode = soViewer->sceneGraph();
  if(!soNode) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::create :"
        << " Slash::UI::ISoViewer " << Lib::sout(wname) 
        << " with null SoNode." 
        << Lib::endl;
    return 0;
  }
  if(!soNode->isOfType(SoPage::getClassTypeId())) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::create :"
        << " SoNode not a SoPage."
        << Lib::endl;
    return 0;
  }
  SoPage* soPage = (SoPage*)soNode;    
  //FIXME : use Slash::UI::IWidget or 
  // Slash::UI::ISoViewer to set viewer background.
  ui->setParameter(wname+".backgroundColor","1 1 1");
  OnXLab::Plotter* plotter = new OnXLab::Plotter
    (fSession.printer(),*ui,fSession.verboseLevel(),*soPage,wname);
  if(plotter->isValid()) return plotter;
  Lib::Out out(fSession.printer());
  out << "OnXLab::PlotterManager::create :"
      << " OnXLab::Plotter not valid." 
      << Lib::endl;
  delete plotter;
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
Slash::UI::IUI* OnXLab::PlotterManager::loadUI(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IUI_Manager* uiManager = 
    Lib_findManager(fSession,"UI_Manager",Slash::UI::IUI_Manager);
  if(!uiManager) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::loadUI :"
        << " UI_Manager not found." << Lib::endl;
    return 0;
  }
  std::string driver; //FIXME : Get it from UI_Manager.
#if defined(WIN32)
  driver = "Win";
#else
  driver = "Xt";
#endif

  bool load_simple_plotter = false;

  std::string file;
  if(Lib::System::isenv("ONXLABPLOTTER")) {
    file = Lib::System::getenv("ONXLABPLOTTER");
    if(!Lib::fmanip::exists(file)) {
      if(fSession.verboseLevel()) {
        Lib::Out out(fSession.printer());
        out << "OnXLab::PlotterManager::loadUI :"
            << " Environment variable ONXLABPLOTTER does not refer"
            << " to an existing file."
            << " Load the embedded SimplePlotter.onx..." 
            << Lib::endl;
      }
      load_simple_plotter = true;
    }
  } else {
    std::string arg0 = (fArgs.size()?fArgs[0]:"");
    if(check_ONXROOT(arg0,fSession.printer(),fSession.verboseLevel(),false)) {
      if(check_ONXLABROOT(arg0,fSession.printer(),false)) {
        file = "$ONXLABROOT/scripts/OnX/Plotter.onx";
      } else {
        if(fSession.verboseLevel()) {
          Lib::Out out(fSession.printer());
          out << "OnXLab::PlotterManager::loadUI :"
              << " Load the embedded SimplePlotter.onx..." 
              << Lib::endl;
        }
        load_simple_plotter = true;
      }
    } else {
      if(fSession.verboseLevel()) {
        Lib::Out out(fSession.printer());
        out << "OnXLab::PlotterManager::loadUI :"
            << " Load the embedded SimplePlotter.onx..." 
            << Lib::endl;
      }
      load_simple_plotter = true;
    }
  }

  if(load_simple_plotter) {
    std::string s = Lib::smanip::tostring(SimplePlotter_onx_linen,
                                          SimplePlotter_onx_lines);
    Lib::smanip::replace(s,"@@double_quote@@","\"");
    Lib::smanip::replace(s,"@@back_slash@@","\\");
    if(Lib::fmanip::tmpname(file,".","SimplePlotter",".onx")) {
      if(!Lib::fmanip::write(file,s)) {
        Lib::Out out(fSession.printer());
        out << "OnXLab::PlotterManager::loadUI :"
            << " can't write in file " << Lib::sout(file)  
            << Lib::endl;
        Lib::fmanip::remove(file);
        return 0;
      }
    } else {
      Lib::Out out(fSession.printer());
      out << "OnXLab::PlotterManager::loadUI :"
          << " can't open a temporary file." 
          << Lib::endl;
      return 0;
    }
  }

  Slash::UI::IUI* ui = uiManager->create(driver,file,fArgs,false);

  if(load_simple_plotter) {
    if(!Lib::fmanip::remove(file)) {
      Lib::Out out(fSession.printer());
      out << "OnXLab::PlotterManager::loadUI :"
          << " can't remove file " << Lib::sout(file)  
          << Lib::endl;
    }
  }

  if(!ui) {
    Lib::Out out(fSession.printer());
    out << "OnXLab::PlotterManager::loadUI :"
        << " can't create UI for driver " << Lib::sout(driver) 
        << " and file " << Lib::sout(file) 
        << Lib::endl;
    return 0;
  }
  return ui;
}
