// this :
#include <OnXLab/Inventor/Representer.h>

// Slash :
#include <Slash/Core/IRelation.h>
#include <Slash/Core/IRelationManager.h>
#include <Slash/Core/IRelationVisitor.h>
#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>

// Lib :
#include <Lib/Out.h>
#include <Lib/Debug.h>
#include <Lib/Cast.h>
#include <Lib/sout.h>
#include <Lib/System.h>

// BatchLab :
#include <BatchLab/Core/Histogram.h>
#include <BatchLab/Core/Function.h>
#include <BatchLab/Core/Cloud.h>
#include <BatchLab/Core/DataPointSet.h>

// OnX :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include <OnXLab/Inventor/PlottableThings.h>

#include <HEPVis/nodekits/SoPlotter.h>

namespace OnXLab {

class BaseObject_SoPlotter_Relation : public virtual Slash::Core::IRelation {
public: // IRelation :
  virtual std::string name() const {return fName;}
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnXLab::BaseObject_SoPlotter_Relation)
    else if_Lib_SCast(Slash::Core::IRelation)
    else return 0;
  }
  virtual bool schedule() {
    fSoPlotter.raiseUpdateFlag(SoPlotter::SHAPE);
    fSoPlotter.touch();
    return true;
  }
  virtual void unSchedule() {}
  virtual void* first() const { return (void*)&fObject;}
  virtual void* second() const { return &fSoPlotter;}
public:
  BaseObject_SoPlotter_Relation
  (const BatchLab::BaseObject& aObject,SoPlotter& aSoPlotter)
  :fName("OnXLab::BaseObject_SoPlotter_Relation")
  ,fObject(aObject),fSoPlotter(aSoPlotter){
    Lib::Debug::increment("OnXLab::BaseObject_SoPlotter_Relation");
    fSoPlotter.ref();
    //printf("debug : OnXLab::Relation::Relation : %ld\n",this);
  }
  virtual ~BaseObject_SoPlotter_Relation(){
    //printf("debug : OnXLab::Relation::~Relation : %ld\n",this);
    fSoPlotter.unref();
    Lib::Debug::decrement("OnXLab::BaseObject_SoPlotter_Relation");
  }
private:
  std::string fName;
  const BatchLab::BaseObject& fObject;
protected:
  SoPlotter& fSoPlotter;
};

class Function_SoPlotter_Relation : public BaseObject_SoPlotter_Relation {
public: // IRelation :
  virtual void* cast(const std::string& aClass) const {
    if_Lib_SCast(OnXLab::Function_SoPlotter_Relation)
    else return Function_SoPlotter_Relation::cast(aClass);
  }
  virtual bool schedule() {
    fSoPlotter.raiseUpdateFlag(SoPlotter::FUNCTION);
    fSoPlotter.updateChildren();
    //fSoPlotter.touch();
    Slash::UI::ISoViewer* soViewer = ui_SoViewer(fUI);
    if(soViewer) soViewer->render();
    //fUI.synchronize();
    //printf("debug : sleep...\n");
    //Lib::System::sleep(10);
    return true;
  }
public:
  Function_SoPlotter_Relation
  (const BatchLab::BaseObject& aObject,
   SoPlotter& aSoPlotter,
   Slash::UI::IUI& aUI)
  :BaseObject_SoPlotter_Relation(aObject,aSoPlotter),fUI(aUI){}
  virtual ~Function_SoPlotter_Relation(){}
private:
  Slash::UI::IUI& fUI;  
};

class RelationVisitor1 : public virtual Slash::Core::IRelationVisitor {
public: //IRelationVisitor
  virtual bool visit(Slash::Core::IRelation& aRelation) {
    if(fFirst!=aRelation.first()) return true; //continue
    fFound = true;
    return false; //stop visit
  }
public:
  RelationVisitor1(const void* aFirst)
  :fFirst(aFirst),fFound(false){}
  virtual ~RelationVisitor1(){}
private:
  const void* fFirst;
public:
  bool fFound;
};

class RelationVisitor2 : public virtual Slash::Core::IRelationVisitor {
public: //IRelationVisitor
  virtual bool visit(Slash::Core::IRelation& aRelation) {
    if(fObject!=aRelation.first()) return false;
    if(fSoPlotter!=aRelation.second()) return false;
    return true; //remove
  }
public:
  RelationVisitor2(const BatchLab::BaseObject* aObject,SoPlotter* aSoPlotter)
  :fObject(aObject),fSoPlotter(aSoPlotter){}
  virtual ~RelationVisitor2(){}
private:
  const BatchLab::BaseObject* fObject;
  SoPlotter* fSoPlotter;
};

class Notifier : public virtual INotifier {
public: //INotifier
  virtual bool isValid() const {
    RelationVisitor1 visitor(&fObject);
    fRelationManager.visit(visitor);
    return visitor.fFound;
  }
  //virtual void destroy() const {}
public:
  Notifier(Slash::Core::IRelationManager& aRelationManager,
           const BatchLab::BaseObject& aObject,
           SoPlotter& aSoPlotter)
  :fRelationManager(aRelationManager),fObject(aObject),fSoPlotter(aSoPlotter){
    Lib::Debug::increment("OnXLab::Notifier");
  }
  virtual ~Notifier(){
    // A SbPlottableObject had been deleted.
    RelationVisitor2 visitor(&fObject,&fSoPlotter);
    fRelationManager.remove(visitor);
    Lib::Debug::decrement("OnXLab::Notifier");
  }
private:
  Slash::Core::IRelationManager& fRelationManager;
  const BatchLab::BaseObject& fObject;
  SoPlotter& fSoPlotter;
};

class DummyNotifier : public virtual INotifier {
public: //INotifier
  virtual bool isValid() const { return true;}
  virtual void destroy() const {}
public:
  DummyNotifier(){Lib::Debug::increment("OnXLab::DummyNotifier");}
  virtual ~DummyNotifier(){Lib::Debug::decrement("OnXLab::DummyNotifier");}
};

}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SbPlottableObject* OnXLab::Representer::represent(
 const AIDA::IBaseHistogram& aObject
,Slash::Core::IRelationManager& aRelationManager
,SoPlotter& aSoPlotter
,bool aRetain
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
// Returned value deleted by the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //////////////////
  // Histograms :
  //////////////////
  if(const BatchLab::Histogram1D* object = 
       Lib_ConstCast(aObject,BatchLab::Histogram1D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::Histogram1D*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableHistogram1D(object->name(),object,
                           new Notifier(aRelationManager,*object,aSoPlotter),
                           aRetain);
  } else if(const BatchLab::Histogram2D* object = 
              Lib_ConstCast(aObject,BatchLab::Histogram2D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::Histogram2D*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableHistogram2D(object->name(),object,
                           new Notifier(aRelationManager,*object,aSoPlotter),
                           aRetain);


  //////////////////
  // Profiles :
  //////////////////
  } else if(const BatchLab::Profile1D* object = 
              Lib_ConstCast(aObject,BatchLab::Profile1D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::Profile1D*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableProfile1D(object->name(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter));
  } else if(const BatchLab::Profile2D* object = 
              Lib_ConstCast(aObject,BatchLab::Profile2D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::Profile2D*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableProfile2D(object->name(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter));

  //////////////////
  // Clouds :
  //////////////////
  } else if(const BatchLab::Cloud1D* object = 
              Lib_ConstCast(aObject,BatchLab::Cloud1D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      const_cast<BatchLab::Cloud1D*>(object)->setRelationManager
        (&aRelationManager); //FIXME
      aRelationManager.add
        (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
      return new PlottableCloud1D(object->title(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter));
    }
  } else if(const BatchLab::Cloud2D* object = 
              Lib_ConstCast(aObject,BatchLab::Cloud2D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      const_cast<BatchLab::Cloud2D*>(object)->setRelationManager
        (&aRelationManager); //FIXME
      aRelationManager.add
        (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
      return new PlottableCloud2D(object->title(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter),
                          aRetain);
    }
  } else if(const BatchLab::Cloud3D* object = 
              Lib_ConstCast(aObject,BatchLab::Cloud3D)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      const_cast<BatchLab::Cloud3D*>(object)->setRelationManager
        (&aRelationManager); //FIXME
      aRelationManager.add
        (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
      return new PlottableCloud3D(object->title(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter));
    }

  //////////////////
  // DataPointSet :
  //////////////////
  } else if(const BatchLab::DataPointSet* object = 
              Lib_ConstCast(aObject,BatchLab::DataPointSet)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::DataPointSet*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableDataPointSet(object->title(),object,
                          new Notifier(aRelationManager,*object,aSoPlotter),
                          aRetain);
  } 

  /////////////////////////////////
  // Not a OnXLab object. Try anyway :
  /////////////////////////////////

  if(aVerboseLevel) {
    Lib::Out out(aPrinter);
    out << "OnXLab::Representer::represent :"
        << " " << Lib::sout(aObject.title()) << " not a OnXLab object." 
        << Lib::endl;
  }

  if(const AIDA::IHistogram1D* object = 
       Lib_ConstCast(aObject,AIDA::IHistogram1D)) {
    //FIXME : retain = true ? Suppress DummyNotifer ?
    // It will be more reliable.
    return new PlottableHistogram1D(object->title(),object,
                                  new DummyNotifier(),aRetain);
  } else if(const AIDA::IHistogram2D* object = 
              Lib_ConstCast(aObject,AIDA::IHistogram2D)){
    //FIXME : retain = true ? Suppress DummyNotifer ?
    // It will be more reliable.
    return new PlottableHistogram2D(object->title(),object,
                                    new DummyNotifier(),aRetain);

  } else if(const AIDA::ICloud1D* object = 
              Lib_ConstCast(aObject,AIDA::ICloud1D)) {
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      return new PlottableCloud1D(object->title(),object,
                                  new DummyNotifier());
    }
  } else if(const AIDA::ICloud2D* object = 
              Lib_ConstCast(aObject,AIDA::ICloud2D)) {
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      return new PlottableCloud2D(object->title(),object,
                                  new DummyNotifier(),aRetain);
    }
  } else if(const AIDA::ICloud3D* object = 
              Lib_ConstCast(aObject,AIDA::ICloud3D)) {
    if(object->isConverted()) {
      return represent(object->histogram(),aRelationManager,aSoPlotter,
                       aRetain,aPrinter,aVerboseLevel);
    } else {
      return new PlottableCloud3D(object->title(),object,
                                  new DummyNotifier());
    }
  } else {
    return 0;
  }

}
//////////////////////////////////////////////////////////////////////////////
SbPlottableObject* OnXLab::Representer::represent(
 const AIDA::IFunction& aObject
,Slash::Core::IRelationManager& aRelationManager
,SoPlotter& aSoPlotter
,bool aRetain
,Slash::UI::IUI& aUI
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
// Returned value deleted by the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(const BatchLab::BaseFunction* object = 
       Lib_ConstCast(aObject,BatchLab::BaseFunction)){
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " function." << Lib::endl;
    }
    const_cast<BatchLab::BaseFunction*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new Function_SoPlotter_Relation(*object,aSoPlotter,aUI));
    if(object->dimension()==1) {
      return new PlottableFunction1D(object->name(),&aObject,
                            new Notifier(aRelationManager,*object,aSoPlotter),
                            aRetain);
    } else if(object->dimension()==2) {
      return new PlottableFunction2D(object->name(),&aObject,
                            new Notifier(aRelationManager,*object,aSoPlotter),
                            aRetain);
    } else {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " function with dimension " << object->dimension()
          << " not handled."
          << Lib::endl;
      return 0;
    }
  } else {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " function (not a OnXLab one)." << Lib::endl;
    }
    if(aObject.dimension()==1) {
      return new PlottableFunction1D("Function",&aObject,  //FIXME
                                   new DummyNotifier(),aRetain);
    } else if(aObject.dimension()==2) {
      return new PlottableFunction2D("Function",&aObject,  //FIXME
                                   new DummyNotifier(),aRetain);
    } else {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " function with dimension " << aObject.dimension()
          << " not handled."
          << Lib::endl;
      return 0;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
SbPlottableObject* OnXLab::Representer::represent(
 const AIDA::IDataPointSet& aObject
,Slash::Core::IRelationManager& aRelationManager
,SoPlotter& aSoPlotter
,bool aRetain
,Slash::Core::IWriter& aPrinter
,int aVerboseLevel
)
//////////////////////////////////////////////////////////////////////////////
// Returned value deleted by the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(const BatchLab::DataPointSet* object = 
       Lib_ConstCast(aObject,BatchLab::DataPointSet)) {
    if(aVerboseLevel) {
      Lib::Out out(aPrinter);
      out << "OnXLab::Representer::represent :"
          << " " << Lib::sout(object->name()) << "." 
          << Lib::endl;
    }
    const_cast<BatchLab::DataPointSet*>(object)->setRelationManager
      (&aRelationManager); //FIXME
    aRelationManager.add
      (new BaseObject_SoPlotter_Relation(*object,aSoPlotter));
    return new PlottableDataPointSet(object->name(),&aObject,
                        new Notifier(aRelationManager,*object,aSoPlotter),
                        aRetain);
  } 

  // Not a OnXLab object. Try anyway.

  if(aVerboseLevel) {
    Lib::Out out(aPrinter);
    out << "OnXLab::Representer::represent :"
        << " " << Lib::sout(aObject.title()) << " not a OnXLab object."
        << Lib::endl;
  }

  if(const AIDA::IDataPointSet* object = 
       Lib_ConstCast(aObject,AIDA::IDataPointSet)) {
    return new PlottableDataPointSet(object->title(),object,
                                     new DummyNotifier(),aRetain);
  } else {
    return 0;
  }
}
