%module OnXLab

%include std_string.i
%include std_vector.i

%template() std::vector<std::string>;
%template() std::vector<double>;
%template() std::vector<int>;

#ifdef SWIGJAVA
%include java_typemaps.i
%apply ARG_STD_VECTOR_STD_STRING {std::vector<std::string>}
%apply ARG_CONST_STD_VECTOR_STD_STRING {const std::vector<std::string>&}
%apply RET_STD_VECTOR_STD_STRING {std::vector<std::string>}
%apply RET_CONST_STD_VECTOR_STD_STRING {const std::vector<std::string>&}

%apply ARG_STD_VECTOR_DOUBLE {std::vector<double>}
%apply ARG_CONST_STD_VECTOR_DOUBLE {const std::vector<double>&}
%apply RET_STD_VECTOR_DOUBLE {std::vector<double>}
%apply RET_CONST_STD_VECTOR_DOUBLE {const std::vector<double>&}

%apply ARG_STD_VECTOR_INT {std::vector<int>}
%apply ARG_CONST_STD_VECTOR_INT {const std::vector<int>&}
%apply RET_STD_VECTOR_INT {std::vector<int>}
%apply RET_CONST_STD_VECTOR_INT {const std::vector<int>&}
#endif

#ifdef SWIGJAVA

%typemap(javacode) AIDA::ITupleFactory %{
  //FIXME : have a typemap for Class.
  public ITuple create(String aPath,
                             String aTitle,
                             String[] aColumnNames,
                             Class[] aColumnTypes,
                             String aOptions) {
    String[] columnTypes = new String[aColumnTypes.length];
    for(int index=0;index<columnTypes.length;index++) {
             if(aColumnTypes[index]==Integer.TYPE) {
        columnTypes[index] = "int";
      } else if(aColumnTypes[index]==Float.TYPE) {
        columnTypes[index] = "float";
      } else if(aColumnTypes[index]==Double.TYPE) {
        columnTypes[index] = "double";
    //} else if(aColumnTypes[index]==String.TYPE) {
    //  columnTypes[index] = "string";
      } else {
        System.out.println
          ("OnXLab : %extend AIDA::ITupleFactory : type not treated.");
        return null;
      }
    }
    return create(aPath,aTitle,aColumnNames,columnTypes,aOptions);
  }
%}

/*
%typemap(javacode) AIDA::ITree %{
  public IBaseHistogram find(String aPath) {
    IManagedObject mobject = find(aPath);
    return mobject.cast_IBaseHistogram();
  }
%}
*/

#endif


%newobject OnXLab::Main::createTreeFactory;
%newobject OnXLab::Main::createHistogramFactory;
%newobject OnXLab::Main::createTupleFactory;
%newobject OnXLab::Main::createFunctionFactory;
%newobject OnXLab::Main::createPlotterFactory;
%newobject OnXLab::Main::createDataPointSetFactory;
%newobject OnXLab::Main::createFitFactory;

//////////////////////////////////////////////////////////////////////////////
/// Code to wrap /////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

%include AIDA.i

//namespace Slash {namespace UI {class IUI;}}

%include <OnXLab/Core/Main.h>

// Bridge OnX / OnXLab.
AIDA::IAnalysisFactory* find_aida(Slash::Core::ISession&); // From BatchLab/Helpers/AIDA.h

//////////////////////////////////////////////////////////////////////////////
/// Post SWIG ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

// BatchLab/Helpers/AIDA.h helpers are put as extensions.

/*
//FIXME : can we do the below without clash with OnX/Lib.i ?
namespace Lib {
class IProcessor {
public:
  virtual ~IProcessor();
public:
  virtual std::vector<std::string> typeNames() = 0;
};
}

//%extend Slash::Core::ISession {
//  AIDA::IAnalysisFactory* findAIDA() {
//    return find_aida(*self);
//  } 
//}

%extend Slash::Data::IProcessor {
  void fill1D(const std::string& aHistogram,
              const std::string& aExpressionX) { 
    data_fill1D(*self,aHistogram,aExpressionX);
  }
  void fill2D(const std::string& aHistogram,
              const std::string& aExpressionX,
              const std::string& aExpressionY) {
    data_fill2D(*self,aHistogram,aExpressionX,aExpressionY);
  }
  void tuple(const std::string& aPath,
             const std::string& aTitle) {
    data_tuple(*self,aPath,aTitle);
  }
}
*/

%extend AIDA::IFunction {
  AIDA::ICloud* convertToCloud(AIDA::IHistogramFactory& aFactory,
                        const std::string& aName,
                        const std::string& aTitle) {
    return convert_functionToCloud(aFactory,aName,aTitle,*self);
  }
  AIDA::IFunction* convertToRealFunction(AIDA::IFunctionFactory& aFactory,
                                  const std::string& aName,
                                  const std::string& aTitle,
                                  const std::string& aWhat) {
    return convert_complexToRealFunction(aFactory,aName,aTitle,*self,aWhat);
  }
  AIDA::IFunction* convertToComplexFunction(AIDA::IFunctionFactory& aFactory,
                                     const std::string& aName,
                                     const std::string& aTitle) {
    return convert_realToComplexFunction(aFactory,aName,aTitle,*self);
  }
}

%extend AIDA::ITuple {
  AIDA::ICloud* convertToCloud(AIDA::IHistogramFactory& aFactory,
                        const std::string& aName,
                        const std::string& aTitle,
                        AIDA::IEvaluator& aEvaluatorX,
                        AIDA::IEvaluator& aEvaluatorY,
                        AIDA::IFilter& aFilter) {
    return convert_tupleToCloud(aFactory,aName,aTitle,*self,
                                aEvaluatorX,aEvaluatorY,aFilter);
  }
  AIDA::IFunction* convertToFunction(AIDA::IFunctionFactory& aFactory,
                              const std::string& aName,
                              const std::string& aTitle,
                              const std::string& aX,
                              AIDA::IEvaluator& aEvaluator,
                              AIDA::IFilter& aFilter) {
    return convert_tupleToFunction(aFactory,aName,aTitle,*self,
                                   aX,aEvaluator,aFilter);
  }
}

// From Marcus Mendenhall :
%define AIDA_CAST(aClass)
  AIDA::aClass* cast_##aClass () {
    return Lib_Cast(*self,AIDA::aClass);
  }
%enddef

%extend AIDA::IManagedObject {
  AIDA_CAST(IBaseHistogram);
  AIDA_CAST(IHistogram1D);
  AIDA_CAST(IHistogram2D);
  AIDA_CAST(IHistogram3D);
  AIDA_CAST(IProfile1D);
  AIDA_CAST(IProfile2D);
  AIDA_CAST(ICloud1D);
  AIDA_CAST(ICloud2D);
  AIDA_CAST(ICloud3D);
  AIDA_CAST(IDataPointSet);
  AIDA_CAST(ITuple);
  AIDA_CAST(IFunction);
#ifdef SWIGPYTHON
  // From Marcus Mendenhall :
  %pythoncode {
    def smartCast(self):
      for convertor in (
        self.cast_IHistogram1D,
        self.cast_IHistogram2D,
        self.cast_IHistogram3D,
        self.cast_IProfile1D,
        self.cast_IProfile2D,
        self.cast_ICloud1D,
        self.cast_ICloud2D,
        self.cast_ICloud3D,
        self.cast_IDataPointSet,
        self.cast_ITuple,
        self.cast_IFunction):
        a = convertor()
        if a: 
          return a #a non-null value means it worked
  }
#endif
}

%extend AIDA::IBaseHistogram {
  AIDA_CAST(IHistogram1D);
  AIDA_CAST(IHistogram2D);
  AIDA_CAST(IHistogram3D);
  AIDA_CAST(IProfile1D);
  AIDA_CAST(IProfile2D);
  AIDA_CAST(ICloud1D);
  AIDA_CAST(ICloud2D);
  AIDA_CAST(ICloud3D);
}

%extend AIDA::ITupleEntry {
  //AIDA_CAST(ITuple); //no ITupleEntry::cast method in AIDA 3.2.1.
  AIDA::ITuple* cast_ITuple() {
    return dynamic_cast<AIDA::ITuple*>(self);
  }
}

// From BatchLab/Helpers/AIDA.h :
%extend AIDA::IAnalysisFactory {
  AIDA::ITree* find_memoryTree() {
    return find_memoryTree(*self);
  }
  AIDA::ITree* create_tree(const std::string& aStoreName,
                           const std::string& aStoreType,
                           bool aReadOnly,bool aCreateNew,
                           const std::string& aOptions = "") {
    return create_tree(*self,aStoreName,aStoreType,
	  	       aReadOnly,aCreateNew,aOptions);
  }
  AIDA::IHistogram1D* create_histogram1D(AIDA::ITree& aTree,
                                         const std::string& aName,
                                         const std::string& aTitle,
				 int aXnumber,double aXmin,double aXmax){
    return create_histogram1D(*self,aTree,aName,aTitle,
                              aXnumber,aXmin,aXmax);
  }
  AIDA::IHistogram1D* copy_histogram1D(AIDA::ITree& aTree,
                                       const std::string& aPath,
                                       AIDA::IHistogram1D& aHisto) {
    return copy_histogram1D(*self,aTree,aPath,aHisto);
  }
  AIDA::IHistogram2D* copy_histogram2D(AIDA::ITree& aTree,
                                       const std::string& aPath,
                                       AIDA::IHistogram2D& aHisto) {
    return copy_histogram2D(*self,aTree,aPath,aHisto);
  }
  AIDA::IBaseHistogram* copy_baseHistogram(AIDA::ITree& aTree,
                                       const std::string& aPath,
                                       AIDA::IBaseHistogram& aHisto) {
    return copy_baseHistogram(*self,aTree,aPath,aHisto);
  }
  AIDA::IHistogram2D* create_histogram2D(AIDA::ITree& aTree,
                                         const std::string& aName,
                                         const std::string& aTitle,
			 int aXnumber,double aXmin,double aXmax,
			 int aYnumber,double aYmin,double aYmax){
    return create_histogram2D(*self,aTree,aName,aTitle,
                              aXnumber,aXmin,aXmax,
                              aYnumber,aYmin,aYmax);
  }
  AIDA::IHistogram3D* create_histogram3D(AIDA::ITree& aTree,
                                         const std::string& aName,
                                         const std::string& aTitle,
			 int aXnumber,double aXmin,double aXmax,
			 int aYnumber,double aYmin,double aYmax,
			 int aZnumber,double aZmin,double aZmax){
    return create_histogram3D(*self,aTree,aName,aTitle,
                              aXnumber,aXmin,aXmax,
                              aYnumber,aYmin,aYmax,
                              aZnumber,aZmin,aZmax);
  }
  AIDA::ICloud2D* create_cloud2D(AIDA::ITree& aTree, 
                                 const std::string& aName,
                                 const std::string& aTitle,
                                 int aLimit = -1,
                                 const std::string& aOptions = "") {
    return create_cloud2D(*self,aTree,aName,aTitle,aLimit,aOptions);
  }

  AIDA::IDataPointSet* create_dataPointSet(AIDA::ITree& aTree,
                                           const std::string& aName,
                                           const std::string& aTitle,
                                           int aDimension) {
    return create_dataPointSet(*self,aTree,aName,aTitle,aDimension);
  }

  AIDA::IFitter* create_fitter(const std::string& aEngine = "") {
    return create_fitter(*self,aEngine);
  }

  AIDA::IFunction* create_function(AIDA::ITree& aTree,
                                   const std::string& aPath,
                                   const std::string& aTitle,
                                   const std::string& aModel){
    return create_function(*self,aTree,aPath,aTitle,aModel);
  }
  AIDA::IFunction* create_scripted_function(AIDA::ITree& aTree,
                                            const std::string& aName,
                                            int aDimension,
                                            const std::string& aScript) {
    return create_scripted_function(*self,aTree,aName,aDimension,aScript);
  }
  AIDA::IFunctionCatalog* find_function_catalog(AIDA::ITree& aTree) {
    return find_function_catalog(*self,aTree);
  }
  AIDA::ITuple* create_tuple(AIDA::ITree& aTree,
                             const std::string& aPath,
                             const std::string& aTitle,
                             const std::string& aColumnNames,
                             const std::string& aOptions = "") {
    return create_tuple(*self,aTree,aPath,aTitle,aColumnNames,aOptions);
  }
  AIDA::IFilter* create_filter(AIDA::ITree& aTree,
                               const std::string& aExpression) {
    return create_filter(*self,aTree,aExpression);
  }
  AIDA::IEvaluator* create_evaluator(AIDA::ITree& aTree,
                                     const std::string& aExpression) {
    return create_evaluator(*self,aTree,aExpression);
  }
  AIDA::IPlotter* create_plotter() {
    return create_plotter(*self);
  }
}


%extend AIDA::ITree {
  AIDA::IBaseHistogram* find_baseHistogram(const std::string& aName)  {
    return find_baseHistogram(*self,aName);
  }
  AIDA::IHistogram1D* find_histogram1D(const std::string& aName) {
    return find_histogram1D(*self,aName);
  }
  AIDA::IHistogram2D* find_histogram2D(const std::string& aName) {
    return find_histogram2D(*self,aName);
  }
  AIDA::IFunction* find_function(const std::string& aName) {
    return find_function(*self,aName);
  }
  AIDA::ITuple* find_tuple(const std::string& aName) {
    return find_tuple(*self,aName);
  }
#ifdef SWIGPYTHON
  // From Marcus Mendenhall :
  %pythoncode {
    def smartFind(self, *args, **kwargs):
      return self.find(*args, **kwargs).smartCast()
  }
#endif
}

/*
  std::string tuple_name(const AIDA::ITuple& aTuple) {
  }
  std::string histogram_name(const AIDA::IBaseHistogram& aHistogram) {
  }
*/

#ifdef SWIGPYTHON
%pythoncode {
#print 'debug : OnXLab_SWIG.i'

#
# Script execution context :
# ------------------------
#  An AIDA .py (for example myaida.py) is of the form :
#      import AIDA
#      aida = AIDA.createAnalysisFactory()
#      ...
#      del aida
# This kind of script can be executed in various contexts.
# For example it can be executed "from scratch"
# from the python shell with :
#    OS> <setup AIDA implementation>
#    OS> <start python shell>
#    >>> import myaida
# or it can be executed in a "Python callback" of
# an interactive program which had not been started
# from Python. For example, someone can execute
# the script from an OnX Python callback, with OnX
# having been started with :
#    OS> <setup AIDA implementation>
#    OS> onx
#    GUI> <click in a button executing myaida.py>
#  It is clear that the AIDA.createAnalysisFactory
# procedure must take into account all these
# possible contexts.
# 

def createAnalysisFactory():
  import OnX
  #FIXME : OnX could have been started, but the
  # OnX Python plugin may be not yet activated, so
  # the below OnX.session is still None...
  if OnX.session() == None:  # Start from scratch :
    #FIXME : case of OnX started without OnXPython loaded ?
    import OnXLab
    import sys
    args = sys.argv
    #args.append('-verbose')  
    # 0 = do not create UI now. 
    # The UI creation,if needed, is delegated to the plotter factory.
    # The plotter factory will use the OnXLab/scripts/OnX/Plotter.onx GUI.
    create_UI = 0
    show_UI = 0
    onxlab_main = OnXLab.Main(create_UI,show_UI,args) 
    #FIXME : we should %extend OnXLab::Main() to compell Python loading.
    #FIXME : or use '-load=Python' to load OnXPython in OnXLab::Main() ?
    onxlab_main.onxMain().loadInterpreter('Python');
    return onxlab_main
  else: # OnX started :
    if OnX.session().findManager('AnalysisFactory') != None:
      import OnXLab
      return OnXLab.find_aida(OnX.session())
    else:
      print 'Found OnX, but there is no AnalysisFactory manager !'
      return None
    

def x_exa(a_name):
  import OnX
  OnX.import_py('$ONXLABROOT/examples/Python',a_name)

}
#endif

#ifdef SWIGJAVA
%pragma(java) jniclasscode=%{
  static {
    try {
      System.loadLibrary("OnXLab_SWIG_Java");
    } catch (UnsatisfiedLinkError e) {
      System.err.println("Can't load OnXLab_SWIG_Java jni library.");
    }
  }
%}

%extend AIDA::IAnalysisFactory {
  // Reemulate SLAC java startup :
  static AIDA::IAnalysisFactory* create() {
    bool create_UI = false;
    bool show_UI = false;
    return new OnXLab::Main(create_UI,show_UI); 
  }
}

#endif
