// 
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//
// This callback file contains examples over AIDA only.
//


#include <BatchLab/Helpers/AIDA.h>

#include <AIDA/AIDA.h>

#include <Slash/UI/IUI.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLabTemplateOne_plot(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  // Get some factories :
  AIDA::IHistogramFactory* histogramFactory = 
    aida->createHistogramFactory(*memoryTree);
  if(!histogramFactory) return;
  AIDA::IFunctionFactory* functionFactory = 
    aida->createFunctionFactory(*memoryTree); 
  if(!functionFactory) return;
  AIDA::IPlotterFactory* plotterFactory = aida->createPlotterFactory();
  if(!plotterFactory) return;
  AIDA::ITreeFactory* treeFactory = aida->createTreeFactory();
  if(!treeFactory) return;

  // Cleanup memoryTree :
  if(!memoryTree->rmdir("/")) return;

  // Get a plotter in the GUI :
  AIDA::IPlotter* plotter = plotterFactory->create("");        
  if(!plotter) return;

  plotter->setParameter("page.title","OnXLabTemplateOne example");
  plotter->createRegions(2,2,0);
    
  // If already loaded close (not delete) the file :
  std::string fileName = "OnXLabTemplateOne.root";

  session.destroyManager(fileName);
  
  AIDA::ITree* tree = treeFactory->create(fileName,"ROOT",true,false);
  if(!tree) return;

  tree->ls();

  // Create some factories to work with the tree :
  AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
  if(!tupleFactory) return;

  ///////////////////////////////////////////////////////////////////////////
  //First region :
  // A XV plot with two histograms.  
  ///////////////////////////////////////////////////////////////////////////
  {
    if(!tree->cd("histograms")) {
      aUI.echo("OnXLabTemplateOne_plot : can't cd to \"histograms\".\n");
      return;
    }

    tree->ls();

    AIDA::IPlotterRegion& region = plotter->currentRegion();
    
    AIDA::IHistogram1D* BW = find_histogram1D(*tree,"BW");
    AIDA::IHistogram1D* gauss = find_histogram1D(*tree,"gauss");
    
    // Plot :
    region.setParameter("textStyle","color red fontName TTF/couri");
    region.setParameter("binsStyle","color red modeling boxes");
    region.setParameter("binsStyle.1","color green modeling boxes");
    
    if( BW && gauss) {
      region.plot(*BW);
      region.plot(*gauss);
    }
  }

  ///////////////////////////////////////////////////////////////////////////
  // Second region :
  // A XY plot.  
  ///////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();
    AIDA::IHistogram2D* gauss_BW = find_histogram2D(*tree,"gauss_BW");
    region.setParameter("infosVisible","FALSE");
    //plotter->setParameter("xyModeling","COLOR");
    region.setParameter("colorMapping","black 1 cyan 2 green 4 orange");
    if(gauss_BW) {
      region.plot(*gauss_BW);
    } else {
      aUI.echo("OnXLab_Examples_0 : gauss_BW not found.\n");
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  // Third region :
  // Get a tuple, create an histo from one of its column, plot it. 
  /////////////////////////////////////////////////////////////////////////////
  {
    AIDA::IPlotterRegion& region = plotter->next();

    if(!tree->cd("..")) {
      aUI.echo("OnXLab_Examples_0 : can't cd to \"..\".\n");
      return;
    }
    if(!tree->cd("tuples")) {
      aUI.echo("OnXLab_Examples_0 : can't cd to \"tuples\".\n");
      return;
    }

    tree->ls();
    
    AIDA::ITuple* tuple1 = tupleFactory->create("tuple1","","");
    AIDA::IHistogram1D* t1_v2 = 
      histogramFactory->createHistogram1D("t1_v2","tuple1/v2",10,0,10);
    region.setParameter("infosVisible","FALSE");
    AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("v2");
    AIDA::IFilter* filter = tupleFactory->createFilter("");
    tuple1->project(*t1_v2,*evaluator,*filter);
    delete evaluator;
    delete filter;

    region.plot(*t1_v2);
  }

  ////////////////////////////////////////////////////////////////////////////
  // Fourth region :
  // Get a tuple, create an histo from one of its column, plot it. 
  ////////////////////////////////////////////////////////////////////////////
  AIDA::IPlotterRegion& region = plotter->next();
  plotter->next();

  AIDA::ITuple* tuple2 = tupleFactory->create("tuple2","","");
  //tuple2->prnt();

  AIDA::IHistogram1D* t2_gauss = 
    histogramFactory->createHistogram1D
    ("t2_gauss","tuple2/rand_gauss",100,-5,5);
  AIDA::IEvaluator* evaluator = tupleFactory->createEvaluator("rand_gauss");
  std::string cuts("(fabs(rand_gauss)<=2.)&&(step>=1.)");
  AIDA::IFilter* filter = tupleFactory->createFilter(cuts);
  tuple2->project(*t2_gauss,*evaluator,*filter);
  delete evaluator;
  delete filter;

  region.plot(*t2_gauss);

  ////////////////////////////////////////////////////////////////////////////
  // Fitting :
  ////////////////////////////////////////////////////////////////////////////
  AIDA::IFitFactory* fitFactory = aida->createFitFactory(); 
  AIDA::IFitter* fitter = fitFactory->createFitter("","","");
  AIDA::IFitResult* fitResult = fitter->fit(*t2_gauss,"G");
  AIDA::IFunction& function = fitResult->fittedFunction();
  region.plot(function);
  delete fitter;
  delete fitFactory;
  delete fitResult;

  delete plotter;

  delete histogramFactory;
  delete tupleFactory;
  delete functionFactory;
  delete treeFactory;
  delete plotterFactory;

}

} // extern "C"
