//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

// Slash :
#include <Slash/UI/IUI.h>

// Lib : 
#include <Lib/Cast.h> //Helpers only, no link dependency :
#include <Lib/Out.h>
#include <Lib/Randoms.h>

// BatchLab :
#include <BatchLab/Helpers/AIDA.h> //Helpers.
#include <BatchLab/Core/Histogram.h>

namespace OnXLabTemplateThree {

class Histogram1D : public BatchLab::Histogram1D {
public: //AIDA/IBaseHistogram
  virtual void* cast(const std::string& aClass) const { //Safe cast.
    if_Lib_SCast(OnXLabTemplateThree::Histogram1D)
    else return BatchLab::Histogram1D::cast(aClass);
  }
public:
  Histogram1D(Slash::Core::IWriter& aPrinter,
              const std::string& aName,const std::string& aTitle,
              int aXnumber,double aXmin,double aXmax)
  :BatchLab::Histogram1D(aPrinter,aName,aTitle,aXnumber,aXmin,aXmax){
    Lib::Out out(printer());
    out << "OnXLabTemplateThree::Histogram1D::Histogram1D." 
        << Lib::endl;
  }
  Histogram1D(const std::string& aName,
              const Histogram1D& aFrom)
  :BatchLab::Histogram1D(aName,aFrom){}
  virtual ~Histogram1D(){}
  void extendOnXLabMethod() {
    Lib::Out out(printer());
    out << "OnXLabTemplateThree::Histogram1D::myMethod." 
        << Lib::endl;
  }
  virtual BaseObject* copy(const std::string& aName) const {
    return new Histogram1D(aName,*this);
  }
};

}


#include <BatchLab/Core/HistogramAllocator.h>

namespace OnXLabTemplateThree {

class HistogramAllocator : public BatchLab::HistogramAllocator {
public: 
  HistogramAllocator():fName("OnXLabTemplateThree::HistogramAllocator"){}
  virtual ~HistogramAllocator(){}
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const { //Safe cast.
    if_Lib_SCast(OnXLabTemplateThree::HistogramAllocator)
    else return BatchLab::HistogramAllocator::cast(aClass);
  }
public:
  // create methods having same signature than constructors :
  virtual BatchLab::Histogram1D* new_Histogram1D(
   Slash::Core::IWriter& aPrinter,const std::string& aName,const std::string& aTitle,
   int aXnumber,double aXmin,double aXmax) {
     return new OnXLabTemplateThree::Histogram1D
       (aPrinter,aName,aTitle,aXnumber,aXmin,aXmax);
  }
private:
  std::string fName;
};

}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXLabTemplateThree_initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//  Should be executed once and after the creation of the 
// OnXLab::AnalysisFactory that creates the OnXLab::HistogramAllocator. 
//  In OnXLabTemplateThree, this callback is executed on the create 
// callback of the :
//    scripts/OnX/OnXLabTemplateThree.onx 
// file loaded by onx.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Replace the OnXLab::HistogramAllocator by the OnXLabTemplateThree one :
  session.destroyManager("BatchLab::HistogramAllocator");
  session.addManager(new OnXLabTemplateThree::HistogramAllocator());

  std::vector<std::string> txt = session.managerNames("");
  Lib::Out out(session.printer());
  out << "Managers are :" << Lib::endl;
  for(unsigned int index=0;index<txt.size();index++) {
    out << txt[index] << Lib::endl;
  }
  
}
//////////////////////////////////////////////////////////////////////////////
void OnXLabTemplateThree_userMethod(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Executed from :
//   File/OnXLabTemplateThree::Histogram1D::extendOnXLabMethod
// menu item.
// Use my histogram managed now by BatchLab AIDA.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IAnalysisFactory* aida = find_aida(aUI.session());
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(memoryTree->find("OnXLabTemplateThree_h")) 
    memoryTree->rm("OnXLabTemplateThree_h");

  //  If OnXLabTemplateThree_initialize had been executed, the concrete 
  // BatchLab AIDA::IHistogramFactory will create a 
  // OnXLabTemplateThree::Histogram1D.

  AIDA::IHistogram1D* h = create_histogram1D(*aida,*memoryTree,
    "OnXLabTemplateThree_h","An OnXLabTemplateThree::Histogram1D.",100,-5,5);
  if(!h) return;

  Lib::RandomGauss rg = Lib::RandomGauss(0,1);
  for(int index=0;index<10000;index++) h->fill(rg.shoot(),1);

  OnXLabTemplateThree::Histogram1D* extendOnXLabHistogram = 
    Lib_Cast(*h,OnXLabTemplateThree::Histogram1D);  //Use the safe cast.
  if(!extendOnXLabHistogram) {
    Lib::Out out(aUI.session().printer());
    out << "AIDA::Histogram1D not a OnXLabTemplateThree::Histogram1D !" 
        << Lib::endl;
    return;
  }

  // Seems ok. Execute local method :
  extendOnXLabHistogram->extendOnXLabMethod();

  // Plot the histogram :
  AIDA::IPlotter* plotter = create_plotter(*aida);
  if(plotter) {
    AIDA::IPlotterRegion& region = plotter->currentRegion();
    region.plot(*h);
    delete plotter;
  }
}

} // extern "C"
