
/*
 * This example demonstrates histogram saving.
 */

#include <AIDA/AIDA.h>

#include "Random.h"

#include <stdlib.h>

int main(int argc,char* argv[]) {

  AIDA::IAnalysisFactory* af = AIDA_createAnalysisFactory();
  if(!af) return EXIT_FAILURE;

  AIDA::ITreeFactory* trf = af->createTreeFactory();
  if(!trf) return EXIT_FAILURE;
  AIDA::ITree* tree = trf->create("OnXLabTemplateTwo_out.aida","xml",false,true);
  delete trf;
  if(!tree) return EXIT_FAILURE;

  AIDA::IHistogramFactory* hf = af->createHistogramFactory(*tree);
  if(!hf) return EXIT_FAILURE;
  AIDA::IHistogram1D* h1d = hf->createHistogram1D("test 1d",50,-3,3);
  AIDA::IHistogram2D* h2d = hf->createHistogram2D("test 2d",50,-3,3,50,-3,3);
  delete hf;
  if(!h1d) return EXIT_FAILURE;
  if(!h2d) return EXIT_FAILURE;
      
  Random r;
  for (int i=0; i<10000; i++) {
    h1d->fill(r.nextGaussian());
    h2d->fill(r.nextGaussian(),r.nextGaussian());
  }
      
  tree->commit();

  delete af;

  return EXIT_SUCCESS;
}
