#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/Manager.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/sout.h>
#include <Lib/Cast.h>

#include <AIDA/IFitResult.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/TupleFactory.h> //FIXME : not AIDA.
#include <Slash/Store/ITree.h> //FIXME : not AIDA.

#include <OpenPAW/PAW_Manager.h>

//////////////////////////////////////////////////////////////////////////////
inline bool is_pawc_path(
 const std::string& aPATH
,bool& a_is
,std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aPATH.size()<2)||(aPATH.substr(0,2)!="//")) {
    a_is = false;
    aString = "";
    return true;
  }

  std::vector<std::string> words;
  Lib::smanip::words(aPATH,"/",words);

  if(!words.size()) {
    a_is = false;
    aString = "";
    return false;
  }

  std::string sunit = words[0];
  Lib::smanip::touppercase(sunit);
  if(sunit!="PAWC") {
    a_is = false;
    aString = "";
    return true;
  }

  aString = "";
  for(unsigned index = 1;index<words.size();index++){
    aString += "/";
    aString += words[index];
  }
  if(aString=="") aString = "/";

  a_is = true;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool is_unit_path(
 const std::string& aPATH
,bool& a_is
,std::string& aString
,int& aUnit
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aPATH.size()<2)||(aPATH.substr(0,2)!="//")) {
    a_is = false;
    aString = "";
    aUnit = 0;
    return true;
  }

  std::vector<std::string> words;
  Lib::smanip::words(aPATH,"/",words);

  if(!words.size()) {
    a_is = false;
    aString = "";
    aUnit = 0;
    return false;
  }

  std::string sunit = words[0];
  Lib::smanip::touppercase(sunit);

  if((sunit.size()<3) || (sunit.substr(0,3)!="LUN")) {
    a_is = false;
    aString = "";
    aUnit = 0;
    return true;
  }

  sunit = sunit.substr(3,sunit.size()-3);
  if(!Lib::smanip::toint(sunit,aUnit)) {
    a_is = false;
    aString = "";
    aUnit = 0;
    return false;
  }

  aString = "";
  for(unsigned index = 1;index<words.size();index++){
    aString += "/";
    aString += words[index];
  }
  if(aString=="") aString = "/";

  a_is = true;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline bool parse_path(
 Slash::Core::ISession& aSession
,const std::string& aPATH
,AIDA::ITree*& aTree
,std::string& aPath
,bool& aMemory
)
//////////////////////////////////////////////////////////////////////////////
// Input :
//   aPATH : can start with //PAWC, //LUN<unit>
// Output :
//   aTree : the AIDA::ITree.
//   aPath : path in the AIDA::ITree.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aTree = 0;
  aPath = "";
  aMemory = true;

  Slash::Core::IWriter& printer = aSession.printer();

  std::vector<std::string> words;
  Lib::smanip::words(aPATH,"/",words);

  OpenPAW::PAW_Manager* pawManager = find_PAW(aSession);
  if(!pawManager) return false;

  AIDA::IAnalysisFactory* aida = find_aida(aSession);
  if(!aida) return false;

  bool is_pawc;
  if(!is_pawc_path(aPATH,is_pawc,aPath)) {
    Lib::Out out(printer);
    out << "parse_path :" 
        << " syntax error in " << Lib::sout(aPATH) 
        << Lib::endl;
    return false;
  }
  if(is_pawc) {
    AIDA::ITree* tree = find_memoryTree(*aida);
    if(!tree) {
      Lib::Out out(printer);
      out << "parse_path :" 
          << " can't find memory tree." 
          << Lib::endl;
      return false;
    }

    aTree = tree;
    aMemory = true;
    return true;
  }

  bool is_unit;
  int unit;
  if(!is_unit_path(aPATH,is_unit,aPath,unit)) {
    Lib::Out out(printer);
    out << "parse_path :" 
        << " syntax error in " << Lib::sout(aPATH) 
        << Lib::endl;
    return false;
  }
  if(is_unit) {
    OpenPAW::Unit u = pawManager->units().value(unit);
    AIDA::ITree* tree = Lib_findManager(aSession,u.fString,AIDA::ITree);
    if(!tree) {
      Lib::Out out(printer);
      out << "parse_path :" 
          << " can't find tree " << Lib::sout(u.fString)
          << Lib::endl;
      return false;
    }

    aTree = tree;
    aMemory = false;
    return true;

  } 

  if(pawManager->currentTree()=="MemoryTree") {

    AIDA::ITree* tree = find_memoryTree(*aida);
    if(!tree) {
      Lib::Out out(printer);
      out << "parse_path :" 
          << " can't find memory tree." 
          << Lib::endl;
      return false;
    }

    aTree = tree;
    aPath = aPATH;
    aMemory = true;
    return true;

  }

  AIDA::ITree* tree = pawManager->findCurrentTree();
  if(!tree) {
    Lib::Out out(printer);
    out << "parse_path :" 
        << " can't find current tree " 
        << Lib::sout(pawManager->currentTree())
        << Lib::endl;
    return false;
  }

  aTree = tree;
  aPath = aPATH;
  aMemory = false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IBaseHistogram* baseHistogram_find(
 Slash::Core::ISession& aSession
,const std::string& aNAME
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree = 0;
  std::string PATH;
  bool memory;
  if(!parse_path(aSession,aNAME,tree,PATH,memory)) return 0;
  if(memory) {
    return find_baseHistogram(*tree,PATH);
  } else {
    AIDA::IAnalysisFactory* aida = find_aida(aSession);
    if(!aida) return 0;
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(!memoryTree) return 0;
    AIDA::IBaseHistogram* fromObject = find_baseHistogram(*tree,PATH);
    if(fromObject) {
      // If not from memory, PAW copy the object from source (file) 
      // to memory and return the in-memory one !
      if(find_baseHistogram(*memoryTree,PATH)) memoryTree->rm(PATH); //ok.
      AIDA::IAnalysisFactory* aida = find_aida(aSession);
      if(!aida) return 0;
      return copy_baseHistogram(*aida,*memoryTree,PATH,*fromObject);
    } else {
      return find_baseHistogram(*memoryTree,PATH);
    }
  } 
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram1D* histogram1D_find(
 Slash::Core::ISession& aSession
,const std::string& aNAME
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IBaseHistogram* base = baseHistogram_find(aSession,aNAME);
  if(!base) return 0;
  return Lib_Cast(*base,AIDA::IHistogram1D);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IHistogram2D* histogram2D_find(
 Slash::Core::ISession& aSession
,const std::string& aNAME
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IBaseHistogram* base = baseHistogram_find(aSession,aNAME);
  if(!base) return 0;
  return Lib_Cast(*base,AIDA::IHistogram2D);
}
//////////////////////////////////////////////////////////////////////////////
inline AIDA::ITuple* tuple_find(
 Slash::Core::ISession& aSession
,const std::string& aNAME
,AIDA::ITree*& aTree
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aTree = 0;
  AIDA::ITree* tree;
  std::string PATH;
  bool memory;
  if(!parse_path(aSession,aNAME,tree,PATH,memory)) return 0;
  if(memory) {
    aTree = tree;
    return find_tuple(*tree,PATH);
  } else {
    AIDA::IAnalysisFactory* aida = find_aida(aSession);
    if(!aida) return 0;
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(!memoryTree) return 0;
    //FIXME
    Slash::Store::ITree* baseTree = 
      Lib_Cast(*memoryTree,Slash::Store::ITree);
    if(!baseTree) return 0;
    AIDA::ITuple* fromObject = find_tuple(*tree,PATH);
    if(fromObject) {
      if(find_tuple(*memoryTree,PATH)) memoryTree->rm(PATH); //ok.
      // If not from memory, PAW copy the object from source (file) 
      // to memory and return the in-memory one !
      //FIXME : for tuple, we have it twice in memory !
      //FIXME : it is slow. Could we have some TupleProxy in memory.
      //return copy_tuple(*aida,*memoryTree,PATH,*fromObject);
      aTree = memoryTree;
      BatchLab::TupleFactory factory(aSession,*baseTree);
      return factory.createCopy(PATH,*fromObject);
    } else {
      aTree = memoryTree;
      return find_tuple(*memoryTree,PATH);
    }
  }
}

#include <Lib/FunctionManager.h>
#include <Slash/Data/IArray.h>
#include <BatchLab/Core/ArrayFactory.h> //FIXME : not AIDA.
//////////////////////////////////////////////////////////////////////////////
inline bool create_vectors(
 std::vector<Lib::FunctionManager::FunctionArgs> aArgs
,AIDA::ITree& aTree
,Slash::Core::IWriter& aPrinter
,std::vector< std::vector<Slash::Data::IArray*> >& a_vctss
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  a_vctss.clear();

  // Create vector if needed :
  Slash::Store::ITree* baseTree = Lib_Cast(aTree,Slash::Store::ITree);
  if(!baseTree) {
    Lib::Out out(aPrinter);
    out << "pafort :" 
        << " tree not a Slash::Store::ITree."
        << Lib::endl;
    a_vctss.clear();
    return false;
  }
  BatchLab::ArrayFactory factory(aPrinter,*baseTree); //FIXME

  // Handle VECTOR
  unsigned int funcn = aArgs.size();
  for(unsigned int funci=0;funci<funcn;funci++) {
    std::vector<Slash::Data::IArray*> vcts;
    unsigned int argn = aArgs[funci].size();
    for(unsigned int argi=0;argi<argn;argi++) {
      Lib::Value& v = aArgs[funci][argi];
      if(v.type()==Slash::Core::IValue::DOUBLE_STAR) {
        Slash::Data::IArray* vector = 0;
        if(v.dimension()==0) {
          // Expect that the vector already exists. Check that.
          AIDA::IManagedObject* mobj = aTree.find(v.label());
          if(!mobj) {
            Lib::Out out(aPrinter);
            out << "pafort :" 
                << " vector " << Lib::sout(v.label()) << " not found."
                << Lib::endl;
            a_vctss.clear();
            return false;
          }
          vector = Lib_SafeCast(*mobj,Slash::Data::IArray);
          if(!vector) {
            Lib::Out out(aPrinter);
            out << "pafort :" 
                << " object " << Lib::sout(v.label()) 
                << " not a Slash::Data::IArray.."
                << Lib::endl; 
            a_vctss.clear();
            return false;
          }

        } else { 

          AIDA::IManagedObject* mobj = aTree.find(v.label());
          if(mobj) {
            vector = Lib_SafeCast(*mobj,Slash::Data::IArray);
            if(!vector) {
              Lib::Out out(aPrinter);
              out << "pafort :" 
                  << " object " << Lib::sout(v.label()) 
                  << " not a Slash::Data::IArray.."
                  << Lib::endl; 
              a_vctss.clear();
              return false;
            }
            if(vector->size()!=v.dimension()) {
              Lib::Out out(aPrinter);
              out << "pafort :" 
                  << " dimension mismatch for vector " 
                  << Lib::sout(v.label()) 
                  << ". In code " << v.dimension()
                  << " whilst booked size is " << vector->size() 
                  << "."
                  << Lib::endl;
              a_vctss.clear();
              return false;
            }

          } else {
            // Create vector :
            std::vector<unsigned int> orders(1);
            orders[0] = v.dimension();
            vector = factory.create(v.label(),v.label(),orders);
            if(!vector) {
               Lib::Out out(aPrinter);
               out << "pafort :" 
                   << " can't create vector " 
                   << Lib::sout(v.label()) << "."
                   << Lib::endl;
               a_vctss.clear();
               return false;
            }
          }
        }
        vcts.push_back(vector);
      } else { //Not a vector
      }

    }
    a_vctss.push_back(vcts);

  }

  return true;
}
#include <BatchLab/Core/FunctionCatalog.h> //FIXME : not AIDA.
#include <BatchLab/Core/Function.h> //FIXME : not AIDA.
//////////////////////////////////////////////////////////////////////////////
inline AIDA::IFunction* create_function_vector(
 const std::string& aName
,const std::string& aPaths
,const std::string& aUFUNC
,Slash::Core::ISession& aSession
,AIDA::IAnalysisFactory& aAIDA
,AIDA::ITree& aTree
,bool aManaged
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aManaged) {
    if(aTree.find(aName)) aTree.rm(aName);
  }
  
  AIDA::IFunction* afunc = 0;

  std::string what;
  if(Lib::fmanip::is_f77(aUFUNC)) {
    // Extract name/parameters :
    std::string sfunc;
   {std::string name;
    std::vector<std::string> sargs;
    if(!Lib::smanip::nameArgs(aUFUNC,name,sargs)) {
      Lib::Out out(aSession.printer());
      out << "create_function_vector :" 
          << " Syntax error in " << Lib::sout(aUFUNC) 
          << Lib::endl;
      return 0;
    }
    sfunc = name;
    Lib::smanip::touppercase(sfunc);}

    //On the fly compilation. Search file...
    std::string UFUNC = Lib::fmanip::find(aPaths,",",aUFUNC);
    // Compiled:<model>;<file>
    //what = "Compiled:"+Lib::smanip::nosuffix(UFUNC)+";"+UFUNC;
    std::string model = Lib::smanip::nosuffix(UFUNC);
    std::string file = UFUNC;

    // Check for usage of VECTOR :
    std::vector<std::string> funcs;
    std::vector<Lib::FunctionManager::FunctionArgs> args;
    if(!Lib::FunctionManager::find_f77_functions
         (UFUNC,funcs,args,aSession.printer())) {
      Lib::Out out(aSession.printer());
      out << "create_function_vector :" 
          << " can't get arguments in " << Lib::sout(UFUNC) 
          << Lib::endl;
      return 0;
    }

    std::vector< std::vector<Slash::Data::IArray*> > vctss;
    if(!create_vectors(args,aTree,aSession.printer(),vctss)) {
      Lib::Out out(aSession.printer());
      out << "create_function_vector :" 
          << " problem creating vectors."
          << Lib::endl;
      return 0;
    }

    BatchLab::FunctionCatalog catalog(aSession);
    Slash::Data::IFunction* func = catalog.find(file,model);
    if(!func) {
      Lib::Out out(aSession.printer());
      out << "create_function_vector :" 
          << " function " << Lib::sout(model) << " not found"
          << " in file " << Lib::sout(file) << "."
          << Lib::endl;
      return 0;
    }
    //WARNING : func is managed by the Lib::FunctionManager.

    unsigned int funci = 0;
    bool found = false;
   {unsigned int funcn = funcs.size();
    sfunc = Lib::smanip::nosuffix(sfunc);
    for(funci=0;funci<funcn;funci++) {
      if(funcs[funci]==sfunc) { //Found function.
        found = true;
        break;
      }
    }}
    if(!found) {
      Lib::Out out(aSession.printer());
      out << "create_function_vector :" 
          << " can't find " << Lib::sout(sfunc) 
          << " in list of functions from " << Lib::sout(UFUNC)
          << Lib::endl;
      return 0;
    }

    // The below will do a local copy of the 
    //  given arrays (retain mode for the given vectors).
    BatchLab::Function::Compiled* bfunc = new BatchLab::Function::Compiled
      (aSession.printer(),aName,UFUNC,func,vctss[funci]);

    if(aManaged) {
      Slash::Store::ITree* tree = Lib_Cast(aTree,Slash::Store::ITree);
      if(!tree) {
        Lib::Out out(aSession.printer());
        out << "create_function_vector :" 
            << " tree is not a Slash::Store::ITree."
            << Lib::endl;
        delete bfunc;
        return 0;
      }
      if(!tree->manageObject(bfunc,"")) {
        Lib::Out out(aSession.printer());
        out << "create_function_vector :" 
            << " can't manage."
            << Lib::endl;
        delete bfunc;
        return 0;
      }
    }
    afunc = bfunc;

  } else if(Lib::fmanip::is_cpp(aUFUNC) ){
    //On the fly compilation. Search file...
    std::string UFUNC = Lib::fmanip::find(aPaths,",",aUFUNC);
    // Compiled:<model>;<file>
    what = "Compiled:"+Lib::smanip::nosuffix(UFUNC)+";"+UFUNC;
    std::string name;
    if(aManaged) name = aName;
    else         name = path_not_managed()+"/"+aName;
    afunc = create_function(aAIDA,aTree,name,UFUNC,what);
  } else {
    what = aUFUNC;
    Lib::smanip::tolowercase(what);
    std::string name;
    if(aManaged) name = aName;
    else         name = path_not_managed()+"/"+aName;
    afunc = create_function(aAIDA,aTree,name,aUFUNC,what);
  }

  return afunc;
}

