/*
 * There is no KUIP in this class.
 * It maintains the "PAW state".
 */


// this :
#include <OpenPAW/PAW_Manager.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/mmanip.h>
#include <Lib/Cast.h>

#include <stdlib.h> //strtol

#include <HEPVis/SbStyles.h> //for SbFont_ROOT_<code>

void att_opt_init(ATTs_type&,OPTs_type&);

#define MAXIMUM(a,b) ((a)>(b)?a:b)
#define NotFound (-1)

#define REGION_SET_MTYP \
  aRegion.setParameter(field+"markerStyle",get_MTYP((int)fATTs.value("MTYP")));

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OpenPAW::ColorEntry::ColorEntry(
)
:fRed(0),fGreen(0),fBlue(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::ColorEntry::ColorEntry(
 double aRed
,double aGreen
,double aBlue
)
:fRed(aRed),fGreen(aGreen),fBlue(aBlue)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::ColorEntry::ColorEntry(
 const ColorEntry& aFrom
)
:fRed(aFrom.fRed)
,fGreen(aFrom.fGreen)
,fBlue(aFrom.fBlue)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::ColorEntry& OpenPAW::ColorEntry::operator=(
 const ColorEntry& aFrom
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fRed = aFrom.fRed;fGreen = aFrom.fGreen;fBlue = aFrom.fBlue;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OpenPAW::Palette::Palette(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Default CERN/PAW palette elements (2,3,4,5,6,7)
  push_back(2); //red in default color table.
  push_back(3); //green in default color table.
  push_back(4); //blue in default color table.
  push_back(5); //yellow in default color table.
  push_back(6); //magenta in default color table.
  push_back(7); //cyan in default color table.
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::Palette::Palette(
 const std::vector<int>& aFrom
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  *((std::vector<int>*)this) = aFrom;
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::Palette::Palette(
 const Palette& aFrom
)
:std::vector<int>(aFrom)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::Palette& OpenPAW::Palette::operator=(
 const Palette& aFrom
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  *((std::vector<int>*)this) = aFrom;
  return *this;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
OpenPAW::PAW_Manager::PAW_Manager(
 Slash::Core::ISession& aSession
)
:fName("PAW_Manager")
,fSession(aSession)
,fUnits(NotFound,Unit())
,fVariables("",Lib::Value())
,fCuts(NotFound,"")
,fOPTs("","")
,fATTs("",0)
,fContourMode(true) //Contour or fill area.
,fCurrentTree("MemoryTree")
,fFunctionCounter(0)
,fNPX(100)
,fNPY(100)
,fNPZ(100)
,fZONE_K(NotFound)
,fDoZONE(true)
,fZONE(1,1,1,false)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  att_opt_init(fATTs,fOPTs);

  // Default CERN/PAW color table is :
  // 0 white, 1 black, 2 red, 3 green, 4 blue, 5 yellow, 6 magenta, 7 cyan.
  fColorTable.push_back(ColorEntry(1,1,1));
  fColorTable.push_back(ColorEntry(0,0,0));
  fColorTable.push_back(ColorEntry(1,0,0));
  fColorTable.push_back(ColorEntry(0,1,0));
  fColorTable.push_back(ColorEntry(0,0,1));
  fColorTable.push_back(ColorEntry(1,1,0));
  fColorTable.push_back(ColorEntry(1,0,1));
  fColorTable.push_back(ColorEntry(0,1,1));

  setPalette(0,Palette());  //index 0 is a dummy palette.
  setPalette(1,Palette());

  //CERN/PAW has PALNB in [1,50]. PALNB >= 51 are for OpenPAW internal usage.

  //HI/2D/CONTOUR CHOPT=0 (pawex14)
 {std::vector<int> pal;
  pal.push_back(1); //black
  pal.push_back(2); //red
  pal.push_back(3); //green
  pal.push_back(4); //blue
  setPalette(PALETTE_HISTOGRAM_2D_PLOT_CONTOUR_OPT_0,pal);}

 {std::vector<int> pal;
  pal.push_back(0);
  pal.push_back(5);
  pal.push_back(7);
  pal.push_back(3);
  pal.push_back(6);
  pal.push_back(2);
  pal.push_back(4);
  pal.push_back(1);
  setPalette(PALETTE_HISTOGRAM_PLOT_OPT_COL,pal);}
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::PAW_Manager::~PAW_Manager(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Close units :
  const Lib::Parameters<int,OpenPAW::Unit>::List& units = fUnits.list();
  Lib::Parameters<int,OpenPAW::Unit>::ConstIterator it;
  for(it=units.begin();it!=units.end();++it) {
    const OpenPAW::Unit& u = (*it).fValue;
    if(fSession.verboseLevel()) {
      Lib::Out out(fSession.printer());
      out << " OpenPAW::PAW_Manager::~PAW_Manager :" 
          << " close unit " << (*it).fKey 
          << " " << Lib::sout(u.fString) 
          << Lib::endl;
    }
    if(u.isFILE()) {
      ::fclose(u.fFILE);
    } else {
      fSession.destroyManager(u.fString);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string OpenPAW::PAW_Manager::name(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fName;
}
//////////////////////////////////////////////////////////////////////////////
void* OpenPAW::PAW_Manager::cast(
 const std::string& aTo
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aTo=="OpenPAW::PAW_Manager") {
    return Lib_SCast(OpenPAW::PAW_Manager);
  } else if(aTo=="Slash::Core::IManager") {
    return Lib_SCast(Slash::Core::IManager);
  } else {
    return 0;
  }
}
//////////////////////////////////////////////////////////////////////////////
Lib::Parameters<int,OpenPAW::Unit>& OpenPAW::PAW_Manager::units(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUnits;
}
//////////////////////////////////////////////////////////////////////////////
OpenPAW::Unit OpenPAW::PAW_Manager::findUnit(
 const std::string& aString
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const Lib::Parameters<int,OpenPAW::Unit>::List& lunits = fUnits.list();
  Lib::Parameters<int,OpenPAW::Unit>::ConstIterator it;
  for(it=lunits.begin();it!=lunits.end();++it) {
    const OpenPAW::Unit& u = (*it).fValue;
    if(u.fString==aString) return u;
  }
  return OpenPAW::Unit();
}
//////////////////////////////////////////////////////////////////////////////
Lib::Parameters<std::string,Lib::Value>& OpenPAW::PAW_Manager::variables(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fVariables;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Parameters<int,std::string>& OpenPAW::PAW_Manager::cuts(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCuts;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Parameters<std::string,std::string>& OpenPAW::PAW_Manager::opts(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fOPTs;
}
//////////////////////////////////////////////////////////////////////////////
Lib::Parameters<std::string,double>& OpenPAW::PAW_Manager::atts(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fATTs;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setUTIT(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fUTIT = aString;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::paletteFromHCOL(
) const
//////////////////////////////////////////////////////////////////////////////
// if HCL = <c>.<p>   <p> is for the palette with p in [1,50]
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  double HCOL = fATTs.value("HCOL");
  double rest = HCOL - (int)HCOL;
  if(rest==0) {
    if(fSession.verboseLevel()) {
      Lib::Out out(fSession.printer());
      out << "OpenPAW::PAW_Manager::paletteFromHCOL :" 
          << " HCOL= " << HCOL << " return 1"
          << Lib::endl;
    }
    return 1;
  }
  rest *= 10;
  int i1 = Lib::mmanip::round(rest);
  rest -= i1;
  rest *= 10;
  int i2 = Lib::mmanip::round(rest);
  if(i2==0) {
    if(fSession.verboseLevel()) {
      Lib::Out out(fSession.printer());
      out << "OpenPAW::PAW_Manager::paletteFromHCOL :" 
          << " HCOL= " << HCOL << " return " << i1
          << Lib::endl;
    }
    return i1;
  }
  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::paletteFromHCOL :" 
        << " HCOL= " << HCOL << " return " << i1*10+i2
        << Lib::endl;
  }
  return i1*10+i2;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setContourMode( 
 bool aMode
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fContourMode = aMode;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setCurrentTree(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCurrentTree = aString;
}
//////////////////////////////////////////////////////////////////////////////
const std::string& OpenPAW::PAW_Manager::currentTree(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCurrentTree;
}
//////////////////////////////////////////////////////////////////////////////
std::string OpenPAW::PAW_Manager::generateFunctionName(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name = "scripted_";
  name += Lib::smanip::tostring(fFunctionCounter);
  fFunctionCounter++;
  return name;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setFunctionPoints(
 int aNPX
,int aNPY
,int aNPZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fNPX = aNPX;
  fNPY = aNPY;
  fNPZ = aNPZ;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::NPX(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fNPX;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::NPY(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fNPY;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::NPZ(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fNPZ;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setZONE_K(
 int aZONE_K
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fZONE_K = aZONE_K;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::ZONE_K(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fZONE_K;
}

//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setDoZONE( 
 bool aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fDoZONE = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setZONE( 
 const ZONE& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fZONE = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setColor( 
 int aICOL
,double aRED
,double aGREEN
,double aBLUE
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aICOL<0) return;
  createColor(aICOL);
  fColorTable[aICOL].fRed = aRED;
  fColorTable[aICOL].fGreen = aGREEN;
  fColorTable[aICOL].fBlue = aBLUE;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::createColor( 
 int aICOL
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aICOL<0) return;
  if(aICOL<(int)fColorTable.size()) return;
  int n = (aICOL+1) - fColorTable.size();
  for(int index=0;index<n;index++) fColorTable.push_back(ColorEntry());
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setPalette( 
 int aPALNB
,const Palette& aPalette
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPALNB<0) return;
  if(aPALNB>=(int)fPalettes.size()) {
    int n = (aPALNB+1) - fPalettes.size();
    for(int index=0;index<n;index++) {
      fPalettes.push_back(Palette());
    }
  }
  fPalettes[aPALNB] = aPalette;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::showATTs(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::IWriter& printer = fSession.printer();
  Lib::Out out(printer);

  //print defaults and current values for all attributes.
  const Lib::Parameters<std::string,double>::List& attList = fATTs.list();
  Lib::Parameters<std::string,double>::List::const_iterator it;
  out << "+-------------+---------------+---------------+--------------------------------+" << Lib::endl;
  out << "|  Parameter  | Current value | Default value |          Explanation           |" << Lib::endl;
  out << "+-------------+---------------+---------------+--------------------------------+" << Lib::endl;
  for(it=attList.begin();it!=attList.end();++it) {
    std::string s = (*it).fKey; //Key
    Lib::smanip::justify(s,13,Lib::smanip::middle);
    out << "|" << s;
    Lib::smanip::printf(s,32,"%g",(*it).fValue); //value
    Lib::smanip::justify(s,15,Lib::smanip::middle);
    out << "|" << s;
    Lib::smanip::printf(s,32,"%g",(*it).fDefault); //default
    Lib::smanip::justify(s,15,Lib::smanip::middle);
    out << "|" << s;
    s = (*it).fLabel; //label
    Lib::smanip::justify(s,32,Lib::smanip::left);
    out << "|" << s << "|";
    out << Lib::endl;        
  }
  out << "+------------+---------------+---------------+--------------------------------+" << Lib::endl;

}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::showOPTs(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::IWriter& printer = fSession.printer();
  Lib::Out out(printer);

  //print defaults and current values for all attributes.
  const Lib::Parameters<std::string,std::string>::List& optList = fOPTs.list();
  Lib::Parameters<std::string,std::string>::List::const_iterator it;
  out << "+-------------+-------------+---------------+---------------------------------+" << Lib::endl;
  out << "|   Current   |   Default   |  Alternative  |           Explanation           |" << Lib::endl;
  out << "+-------------+-------------+---------------+---------------------------------+" << Lib::endl;
  for(it=optList.begin();it!=optList.end();++it) {
    std::string s = (*it).fValue;
    Lib::smanip::justify(s,13,Lib::smanip::middle);
    out << "|" << s;
    s = (*it).fDefault;
    Lib::smanip::justify(s,13,Lib::smanip::middle);
    out << "|" << s;
    s = (*it).fAlternative;
    Lib::smanip::justify(s,15,Lib::smanip::middle);
    out << "|" << s;
    s = (*it).fLabel;
    std::vector<std::string> words;
    Lib::smanip::words(s," ",words);
    unsigned int index;
    std::string s1;
    int l = 0;
    int istart = NotFound;
    for(index=0;index<words.size();index++) {
      if((l+words[index].size()+1)>31) {
        istart = index;
        break;
      }
      s1 += words[index];
      s1 += " ";
      l += words[index].size()+1;
    }
    std::string s2;
    if(istart!=NotFound) {
      for(index=(unsigned int)istart;index<words.size();index++) {
        s2 += words[index];
        s2 += " ";
      }
    }
    Lib::smanip::justify(s1,31,Lib::smanip::left);
    out << "| " << s1 << " |" << Lib::endl;        
    Lib::smanip::strip(s2);
    if(s2!="") {
      Lib::smanip::justify(s2,31,Lib::smanip::left);
      out << "|             |             |               | " 
          << s2 << " |" << Lib::endl;
    }

  }
  out << "+------------+---------------+---------------+--------------------------------+" << Lib::endl;

}
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::filterCuts(
 const std::string& aString
,std::string& aFilter
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = aString;

  // Treat cuts. replace $<number> with (<cut>)
  while(1) {
    std::string::size_type dollar = s.find('$');
    if(dollar==std::string::npos) break;

    const char* ss = s.c_str()+dollar+1;
    char* sss;
    long v = ::strtol(ss,&sss,10);
    if(ss==sss) { //Syntax error :
      aFilter = "";
      return false;
    }
    unsigned long l = sss - ss;     
    std::string scut = s.substr(dollar+1,l);
    //printf("debug : cut : \"%s\"\n",scut.c_str());      
    if(!fCuts.isKey((int)v)) {
      aFilter = "";
      return false;
    }
    std::string cut = fCuts.value((int)v);
    std::string sbef = s.substr(0,dollar);
    std::string saft = s.substr(dollar+1+l,s.size()-(dollar+1+l));
    //printf("debug : filter2C : bef \"%s\"\n",s.c_str());      
    s = sbef + "(" + cut + ")" + saft;
    //printf("debug : filter2C : aft \"%s\"\n",s.c_str());      
  }

  Lib::smanip::tolowercase(s);
  Lib::smanip::replace(s,"!=",".neq.");
  Lib::smanip::replace(s,"=",".eq.");

  Lib::smanip::replace(s,".neq.","!=");
  Lib::smanip::replace(s,".eq.","==");
  Lib::smanip::replace(s,".gt.",">");
  Lib::smanip::replace(s,".ge.",">=");
  Lib::smanip::replace(s,".lt.","<");
  Lib::smanip::replace(s,".le.","<=");
  Lib::smanip::replace(s,".and.","&&");
  Lib::smanip::replace(s,".or.","||");
  Lib::smanip::replace(s,".not.","!");

  aFilter = s;
  //printf("debug : filter2C : res \"%s\"\n",aFilter.c_str());      
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::parseTupleName(
 const std::string& aTNAME
,std::string& aNAME
,std::vector<std::string>& aVars
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aVars.clear();
  std::string::size_type ldot = aTNAME.find('.');
  if(ldot==std::string::npos){
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::parseTupleName :" 
        << " Bad syntax in " << Lib::sout(aTNAME) << "." << Lib::endl;
    return false;
  }
  aNAME = aTNAME.substr(0,ldot);
  std::string svars = aTNAME.substr(ldot+1,aTNAME.size()-(ldot+1));
  Lib::smanip::words(svars,"%",aVars);
  for(unsigned int index=0;index<aVars.size();index++) {
    std::string s;
    if(!filterCuts(aVars[index],s)) {
      Lib::Out out(fSession.printer());
      out << "OpenPAW::PAW_Manager::parseTupleName :" 
          << " problem treating " << Lib::sout(aVars[index]) 
          << Lib::endl;
      return false;
    }
    aVars[index] = s;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
int OpenPAW::PAW_Manager::getInfosOptions(
 std::string& aOptions
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int linen = 0;
  std::string opts;

  // STAT infos :
  std::string NSTA = fOPTs.value("NSTA");
  if(NSTA=="STA") {
    double STAT = fATTs.value("STAT"); 
    //p120 : set STAT [OURMEI]  //default 1111
    // O=1 overflow
    // U=1 underflow
    // R=1 RMS
    // M=1 mean
    // E=1 entries
    // I=1 ID
    int code = (int)STAT;
    int O = code/100000;
    int U = (code - O * 100000)/10000; 
    int R = (code - O * 100000 - U * 10000)/1000; 
    int M = (code - O * 100000 - U * 10000 - R * 1000)/100; 
    int E = (code - O * 100000 - U * 10000 - R * 1000 - M *100)/10; 
    int I = (code - O * 100000 - U * 10000 - R * 1000 - M *100 - E*10)/1; 
    //printf("debug : stat : %d %d %d %d %d %d\n",O,U,R,M,E,I);
    if(I) {opts += "name ";linen++;}
    if(E) {opts += "entries ";linen++;}
    if(M) {opts += "mean ";linen++;}
    if(R) {opts += "rms ";linen++;}
    if(U) {opts += "underflow ";linen++;}
    if(O) {opts += "overflow ";linen++;}
  }

  // FIT infos :
  std::string NFIT = fOPTs.value("NFIT");
  if(NFIT=="FIT") {
    double FIT = fATTs.value("FIT"); 
    // PAW set FIT [CEP] //default 101
    //    C = 1;  print Chisquare/Number of degress of freedom
    //    E = 1;  print errors (if e=1, v must be 1)
    //    P = 1;  print name/values of parameters
    //FIXME : from ROOT TPaveStats.
    //  The parameter mode can be = pcev  (default = 0111)
    //    v = 1;  print name/values of parameters
    //    e = 1;  print errors (if e=1, v must be 1)
    //    c = 1;  print Chisquare/Number of degrees of freedom
    //    p = 1;  print Probability
    //    When "v"=1 is specified, only the non-fixed parameters are shown.
    //    When "v"=2 all parameters are shown.
    //  Example: gStyle->SetOptFit(1011);
    //           print fit probability, parameter names/values and errors.
    int code = (int)FIT;
    int C = code/100;
    int E = (code - C * 100)/10; 
    int P = (code - C * 100 - E * 10)/1; 
    //printf("debug : fit : C %d E %d P %d\n",C,E,P);
    if(C) {
      opts += "fit_quality ";
      opts += "fit_ndf ";
      linen++;
    }
    if(E) {opts += "fit_errors ";linen++;}
    if(P) {opts += "fit_parameters ";linen++;}
  }

  aOptions = opts;

  return linen;
}
//////////////////////////////////////////////////////////////////////////////
// static ////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::parseName(
 const std::string& aString
,std::string& aNAME
,std::vector< std::pair<int,int> >& aDims
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : parse_name : \"%s\"\n",aString.c_str());
  aDims.clear();
  std::string::size_type lbrace = aString.find('(');
  if(lbrace!=std::string::npos){
    std::string::size_type rbrace = aString.find(')');
    if( (rbrace==std::string::npos) || (rbrace<lbrace) ) {
      Lib::Out out(aPrinter);
      out << "OpenPAW::PAW_Manager::parseName :" 
          << " Bad syntax in " << Lib::sout(aString) << "." << Lib::endl;
      return false;
    }
    aNAME = aString.substr(0,lbrace);
    std::string sdim = aString.substr(lbrace+1,rbrace-(lbrace+1));
    std::vector<std::string> words;
    Lib::smanip::words(sdim,",",words,true);
    if( (words.size()<=0) || (words.size()>=4)) {
      Lib::Out out(aPrinter);
      out << "OpenPAW::PAW_Manager::parseName :" 
          << " Dimension >=4 not supported." << Lib::endl;
      return false;
    }
    for(unsigned int index=0;index<words.size();index++) {
      int idim;
      if(words[index]=="") {
        aDims.push_back( std::pair<int,int>(NotFound,NotFound)); 
      } else if(!Lib::smanip::toint(words[index],idim)) {
        // Case of limits ?
        std::string::size_type colon = words[index].find(':');
        if(colon==std::string::npos){
          Lib::Out out(aPrinter);
          out << "OpenPAW::PAW_Manager::parseName :" 
              << " bad syntax in " << Lib::sout(words[index]) << "." 
              << Lib::endl;
          return false;
        } else {
          std::string left = words[index].substr(0,colon);
          std::string right = 
            words[index].substr(colon+1,words[index].size()-(colon+1));
          int mn;
          if(left=="") {
            mn = NotFound;
          } else {
            if(!Lib::smanip::toint(left,mn)) {
              Lib::Out out(aPrinter);
              out << "OpenPAW::PAW_Manager::parseName :" 
                  << " can't convert " << Lib::sout(left) << " to long." 
                  << Lib::endl;
              return false;
            }
            mn -= 1;
            if(mn<0) {
              Lib::Out out(aPrinter);
              out << "OpenPAW::PAW_Manager::parseName :" 
                  << " bad limits in " << Lib::sout(words[index]) << "." 
                  << Lib::endl;
              return false;
            }
          }
          int mx;
          if(right=="") {
            mx = NotFound;
          } else {
            if(!Lib::smanip::toint(right,mx)) {
              Lib::Out out(aPrinter);
              out << "OpenPAW::PAW_Manager::parseName :" 
                  << " can't convert " << Lib::sout(right) << " to long." 
                  << Lib::endl;
              return false;
            }
            mx -= 1;
            if(mx<0) {
              Lib::Out out(aPrinter);
              out << "OpenPAW::PAW_Manager::parseName :" 
                  << " bad limits in " << Lib::sout(words[index]) << "." 
                  << Lib::endl;
              return false;
            }
            if((mn!=NotFound)&&(mx<mn)) {
              Lib::Out out(aPrinter);
              out << "OpenPAW::PAW_Manager::parseName :" 
                  << " bad limits in " << Lib::sout(words[index]) << "." 
                  << Lib::endl;
              return false;
            }
          }
          //printf("debug : mn,mx1 : %d %d\n",mn,mx);
          aDims.push_back( std::pair<int,int>(mn,mx)); 
        }       
      } else {
        //printf("debug : mn,mx2 : %d %d\n",idim-1,idim-1);
        aDims.push_back( std::pair<int,int>(idim-1,idim-1)); 
      }
    }
  } else {
    aNAME = aString;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::isAnOption(
 const std::vector<std::string>& aOPTS
,const std::string& aEntered
,const std::string& aIsIn
) 
//////////////////////////////////////////////////////////////////////////////
//  For FUN2 ("CHOPT","BOXSSURF","S") yes
//  For FUN2 ("CHOPT","BOXSURF","S") no
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Check if aIsIn is in aEntered.
  // WARNING : an option may be a substring of another one
  // like "S" and "SURF" for FUN2 CHOPT.
  std::string s = aEntered;
  std::vector<std::string> sfound;
  unsigned int index;
  for(index=0;index<aOPTS.size();index++) {
    const std::string& OPT = aOPTS[index];
    if(OPT=="") {
      sfound.push_back("");
    } else if(s.size() && (s.find(OPT)!=std::string::npos) ) {
      sfound.push_back(OPT);
      Lib::smanip::replace(s,OPT,"");
    }
  }
  for(index=0;index<sfound.size();index++) {
    if(aIsIn==sfound[index]) return true;
  }
  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::hasUnkownOption(
 const std::vector<std::string>& aOPTS
,const std::string& aEntered
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // WARNING : an option may be a substring of another one
  // like "S" and "SURF" for FUN2 CHOPT.
  std::string s = aEntered;
  std::vector<std::string> sfound;
  unsigned int index;
  for(index=0;index<aOPTS.size();index++) {
    const std::string& OPT = aOPTS[index];
    if(OPT=="") {
      sfound.push_back("");
    } else if(s.size() && (s.find(OPT)!=std::string::npos) ) {
      sfound.push_back(OPT);
      Lib::smanip::replace(s,OPT,"");
    }
  }
  return (s.size()?true:false);
}
//////////////////////////////////////////////////////////////////////////////
/// Over AIDA ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include <AIDA/ITree.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterRegion.h>
#include <AIDA/IFunction.h>
#include <AIDA/ICloud2D.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IAnnotation.h>

#include <Lib/Manager.h>
#include <HEPVis/SbStyles.h> //#define only.
#include "RegionHelpers.h"

//////////////////////////////////////////////////////////////////////////////
AIDA::ITree* OpenPAW::PAW_Manager::findCurrentTree(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::ITree* tree = Lib_findManager(fSession,fCurrentTree,AIDA::ITree);
  if(!tree) {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::findCurrentTree :"
        << " tree " << Lib::sout(fCurrentTree) << " not found." 
        << Lib::endl;
  }
  return tree;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setPlotterZONE(
 AIDA::IPlotter& aPlotter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fDoZONE==false) return;

  aPlotter.setParameter("regionsResizePolicy","PAW_GRID");

  double XSIZ = fATTs.value("XSIZ"); 
  double XMGL = fATTs.value("XMGL");
  double XMGR = fATTs.value("XMGR");
  double XWIN = fATTs.value("XWIN");
  aPlotter.setParameter("pageWidth",Lib::smanip::tostring(XSIZ));
  aPlotter.setParameter("leftMargin",Lib::smanip::tostring(XMGL));
  aPlotter.setParameter("rightMargin",Lib::smanip::tostring(XMGR));
  aPlotter.setParameter("horizontalSpacing",Lib::smanip::tostring(XWIN));

  double YSIZ = fATTs.value("YSIZ"); 
  double YMGL = fATTs.value("YMGL");
  double YMGU = fATTs.value("YMGU");
  double YWIN = fATTs.value("YWIN");
  aPlotter.setParameter("pageHeight",Lib::smanip::tostring(YSIZ));
  aPlotter.setParameter("bottomMargin",Lib::smanip::tostring(YMGL));
  aPlotter.setParameter("topMargin",Lib::smanip::tostring(YMGU));
  aPlotter.setParameter("verticalSpacing",Lib::smanip::tostring(YWIN));

  //size of text in infos box.  
  // CSIZ is the height of hershey text in the infos box given in page unit.
  double CSIZ = fATTs.value("CSIZ");   //0.28 for a YSIZ of 20
  //double YSIZ = 20;//FIXME
  double cooking = 2.2;
  //double cooking = 1;
  std::string opts;
  int linen = getInfosOptions(opts);
  double h = linen * CSIZ/YSIZ * cooking;
  //In  infosRegionResizePolicy = ASPECT_RATIO width is automatic.
  //double w = 0; 
  //aPlotter.setParameter("infosRegionSize",
  //  Lib::smanip::tostring(w) + " " + Lib::smanip::tostring(h));

  if(fZONE.SUPER()) {

    if(!aPlotter.createRegions(-fZONE.NX(),
                               -fZONE.NY(),
                                fZONE.FIRST()-1)) {
      Lib::Out out(fSession.printer());
      out << "plotter_ZONE :" 
          << " can't create a grid region"
          << Lib::endl;
      return;
    }

    // Initialize the region with PAW defaults :
    AIDA::IPlotterRegion& region = aPlotter.currentRegion();
    regionInitialize(region);

  } else {

    if(!aPlotter.createRegions(fZONE.NX(),
                               fZONE.NY(),
                               fZONE.FIRST()-1)) {
      Lib::Out out(fSession.printer());
      out << "plotter_ZONE :" 
          << " can't create regions"
          << Lib::endl;
      return;
    }

    // Initialize region with PAW defaults :
    int number = aPlotter.numberOfRegions();
    for(int index=0;index<number;index++) {
      aPlotter.setCurrentRegionNumber(index);
      AIDA::IPlotterRegion& region = aPlotter.currentRegion();
      regionInitialize(region);
    }

    aPlotter.setCurrentRegionNumber(fZONE.FIRST()-1);
  }


  fZONE_K = NotFound;

  fDoZONE = false;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionInitialize(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aRegion.setParameter("viewportRegion.visible","TRUE");
  aRegion.setParameter("viewportRegion.sensitiveBorder","TRUE");
  aRegion.setParameter("viewportRegion.horizontalBorder","0");
  aRegion.setParameter("viewportRegion.verticalBorder","0");
  aRegion.setParameter("viewportRegion.horizontalShadow","0");
  aRegion.setParameter("viewportRegion.verticalShadow","0");
  //FIXME : the highlight should be available in GUI mode.
  aRegion.setParameter("viewportRegion.horizontalHighlightBorder","0");
  aRegion.setParameter("viewportRegion.verticalHighlightBorder","0");

  aRegion.setParameter("legendRegionVisible","FALSE");
  aRegion.setParameter("legendAutomated","FALSE");
  aRegion.setParameter("legendRegion.viewportRegion.horizontalShadow","0");
  aRegion.setParameter("legendRegion.viewportRegion.verticalShadow","0");
  aRegion.setParameter("legendRegion.viewportRegion.horizontalBorder","1");
  aRegion.setParameter("legendRegion.viewportRegion.verticalBorder","1");
  aRegion.setParameter("legendRegion.viewportRegion.horizontalOriginOffset","1");
  aRegion.setParameter("legendRegion.viewportRegion.verticalOriginOffset","1");

  //  HWID  //  1   // Histogram line WIDth
  double HWID = fATTs.value("HWID"); 
  HWID = MAXIMUM(int(HWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F

  //  FWID  //  1   // Function line WIDth 
  double FWID = fATTs.value("FWID"); 
  FWID = MAXIMUM(int(FWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F

  std::string s;
  for(int index=0;index<10;index++) {
    Lib::smanip::printf(s,32,"%d",index);

    // bins :
    aRegion.setParameter("plotter.binsStyle["+s+"].color","0 0 0");
    aRegion.setParameter("plotter.binsStyle["+s+"].linePattern",SOLID);
    aRegion.setParameter("plotter.binsStyle["+s+"].markerStyle","CROSS");
    aRegion.setParameter("plotter.binsStyle["+s+"].modeling",
      SbModeling_top_lines);
    aRegion.setParameter("plotter.binsStyle["+s+"].lineWidth",
      Lib::smanip::tostring(HWID));

    aRegion.setParameter("plotter.rightHatchStyle["+s+"].color","0 0 0");
    aRegion.setParameter("plotter.rightHatchStyle["+s+"].linePattern",SOLID);
    aRegion.setParameter("plotter.rightHatchStyle["+s+"].lineWidth","1");
    aRegion.setParameter("plotter.rightHatchStyle["+s+"].angle","0.7854");
    aRegion.setParameter("plotter.rightHatchStyle["+s+"].spacing","0.05");
    aRegion.setParameter("plotter.rightHatchStyle["+s+"].offset","0");

    aRegion.setParameter("plotter.leftHatchStyle["+s+"].color","0 0 0");
    aRegion.setParameter("plotter.leftHatchStyle["+s+"].linePattern",SOLID);
    aRegion.setParameter("plotter.leftHatchStyle["+s+"].lineWidth","1");
    aRegion.setParameter("plotter.leftHatchStyle["+s+"].angle","0.7854");
    aRegion.setParameter("plotter.leftHatchStyle["+s+"].spacing","0.05");
    aRegion.setParameter("plotter.leftHatchStyle["+s+"].offset","0");

    aRegion.setParameter("plotter.errorsStyle["+s+"].visible","FALSE");

    // points :
    aRegion.setParameter("plotter.pointsStyle["+s+"].color","0 0 0");
    aRegion.setParameter("plotter.pointsStyle["+s+"].linePattern",SOLID);
    aRegion.setParameter("plotter.pointsStyle["+s+"].markerStyle","CROSS");
    aRegion.setParameter("plotter.pointsStyle["+s+"].modeling",SbModeling_lines);

    // functions :
    aRegion.setParameter("plotter.functionStyle["+s+"].color","0 0 0");
    aRegion.setParameter("plotter.functionStyle["+s+"].linePattern",SOLID);
    aRegion.setParameter("plotter.functionStyle["+s+"].lineWidth",
      Lib::smanip::tostring(FWID));
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool OpenPAW::PAW_Manager::plotterNext(
 AIDA::IPlotter& aPlotter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  AIDA::IPlotterRegion& region = aPlotter.currentRegion();
  int number;
  if(!region_field_value
      (region,"numberOfPlottedObject",number,fSession.printer())) return false;
  if(number) {
    aPlotter.next();

    if(aPlotter.numberOfRegions()) {
      if(aPlotter.currentRegionNumber()==0) {
        aPlotter.clearRegions();
      } else {
        aPlotter.currentRegion().clear();
      }
    }

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSetLayout(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // To plot even if the object had been deleted.
  //FIXME : Histogram1D, 2D only. Have other kind of PAW data. (Functions ?)
  aRegion.setParameter("retain","true"); 

  aRegion.setParameter("infosRegionAttachedToDataFrame","TRUE");

  aRegion.setParameter("viewportRegion.background","TRUE");
  aRegion.setParameter("viewportRegion.backgroundColor","1 1 1");

  aRegion.setParameter("plotter.titleUp","FALSE");
  aRegion.setParameter("plotter.wallStyle.visible","FALSE");
  aRegion.setParameter("plotter.gridStyle.visible","FALSE");
  aRegion.setParameter("plotter.innerFrameStyle.visible","TRUE");
  aRegion.setParameter("plotter.rightAxisVisible","FALSE");
  aRegion.setParameter("plotter.topAxisVisible","FALSE");

  aRegion.setParameter("plotter.colorMapping","");

  std::string font = "Hershey";
  //std::string font = "defaultFont";
  aRegion.setParameter("infosRegion.textStyle.fontName",font);
  aRegion.setParameter("infosRegionResizePolicy","ASPECT_RATIO");
  aRegion.setParameter("infosRegion.viewportRegion.horizontalShadow","0");
  aRegion.setParameter("infosRegion.viewportRegion.verticalShadow","0");
  aRegion.setParameter("infosRegion.viewportRegion.horizontalBorder","1");
  aRegion.setParameter("infosRegion.viewportRegion.verticalBorder","1");
  aRegion.setParameter("infosRegion.viewportRegion.horizontalOriginOffset","1");
  aRegion.setParameter("infosRegion.viewportRegion.verticalOriginOffset","1");
  aRegion.setParameter("infosRegionStyle","PAW");
  aRegion.setParameter("infosRegion.viewportRegion.backgroundColor","1 1 1");
  aRegion.setParameter("infosRegion.inBetweenSpaces","10");

  aRegion.setParameter("plotter.xAxis.modeling","hplot");
  aRegion.setParameter("plotter.yAxis.modeling","hplot");
  aRegion.setParameter("plotter.zAxis.modeling","hplot");
  aRegion.setParameter("plotter.xAxis.titleStyle.encoding","PAW");
  aRegion.setParameter("plotter.yAxis.titleStyle.encoding","PAW");
  aRegion.setParameter("plotter.zAxis.titleStyle.encoding","PAW");

  aRegion.setParameter("plotter.xAxis.lineStyle.color","0 0 0");
  aRegion.setParameter("plotter.xAxis.ticksStyle.color","0 0 0");
  aRegion.setParameter("plotter.xAxis.labelsStyle.color","0 0 0");
  aRegion.setParameter("plotter.xAxis.titleStyle.color","0 0 0");
  aRegion.setParameter("plotter.xAxis.titleToAxisJustification","RIGHT");
  aRegion.setParameter("plotter.xAxis.magStyle.color","0 0 0");

  aRegion.setParameter("plotter.yAxis.lineStyle.color","0 0 0");
  aRegion.setParameter("plotter.yAxis.ticksStyle.color","0 0 0");
  aRegion.setParameter("plotter.yAxis.labelsStyle.color","0 0 0");
  aRegion.setParameter("plotter.yAxis.titleStyle.color","0 0 0");
  aRegion.setParameter("plotter.yAxis.titleToAxisJustification","RIGHT");
  aRegion.setParameter("plotter.yAxis.magStyle.color","0 0 0");

  aRegion.setParameter("plotter.zAxis.lineStyle.color","0 0 0");
  aRegion.setParameter("plotter.zAxis.ticksStyle.color","0 0 0");
  aRegion.setParameter("plotter.zAxis.labelsStyle.color","0 0 0");
  aRegion.setParameter("plotter.zAxis.titleStyle.color","0 0 0");
  aRegion.setParameter("plotter.zAxis.titleToAxisJustification","RIGHT");
  aRegion.setParameter("plotter.zAxis.magStyle.color","0 0 0");

/* 
  // If IPlotter (SoPage) regionsResizePolicy is PAW_GRID,
  // the IPlotterRegion (SoPlotterRegion) plotterViewportMapping 
  // along the SoPlotter :
  //  width, leftMargin, rightMargin
  //  height, bottomMargin, topMargin
  //  depth, downMargin, upMargin
  // are set automatically in SoPlotterRegion::placeInPage called 
  // by the SoPage::updateChildren. 
  // (And SoPlotterRegion.plotterViewportMapping is set to ADJUST_VIEWPORT.

  // Enforcing these parameters here without a aPlotter.refresh()
  // (to do a fSoPage.updateChildren) may lead to bad geometries,
  // especially is doing ZONEs.

  aRegion.setParameter("plotterViewportMapping","ADJUST_VIEWPORT");

  double XSIZ = 20;
  aRegion.setParameter("plotter.width",Lib::smanip::tostring(XSIZ));
  double XMGL = 2;
  double XMGR = 2;
  aRegion.setParameter("plotter.leftMargin",Lib::smanip::tostring(XMGL));
  aRegion.setParameter("plotter.rightMargin",Lib::smanip::tostring(XMGR));

  double YSIZ = 20;
  aRegion.setParameter("plotter.height",Lib::smanip::tostring(YSIZ));
  double YMGL = 2;
  double YMGU = 2;
  aRegion.setParameter("plotter.bottomMargin",Lib::smanip::tostring(YMGL));
  aRegion.setParameter("plotter.topMargin",Lib::smanip::tostring(YMGU));

  double ZSIZ = 20;  
  aRegion.setParameter("plotter.depth",Lib::smanip::tostring(ZSIZ));
  double ZMGD = 2;
  double ZMGU = 2;
  aRegion.setParameter("plotter.downMargin",Lib::smanip::tostring(ZMGD));
  aRegion.setParameter("plotter.upMargin",Lib::smanip::tostring(ZMGU));

*/

  double YHTI = fATTs.value("YHTI"); 
  double TSIZ = fATTs.value("TSIZ"); 

  aRegion.setParameter("titleRegion.styles[0].color","0 0 0");

  std::string HTIT = fOPTs.value("HTIT");
  if(HTIT=="HTIT") {
    aRegion.setParameter("plotter.titleStyle.visible","TRUE");
    aRegion.setParameter("plotter.titleAutomated","TRUE");
    aRegion.setParameter("plotter.titleUp","FALSE");
    aRegion.setParameter("plotter.titleHeight",Lib::smanip::tostring(TSIZ));
    aRegion.setParameter("plotter.titleToAxis",Lib::smanip::tostring(YHTI));
  } else if(HTIT=="UTIT") {
    if(fUTIT=="") {
      aRegion.setParameter("plotter.titleStyle.visible","FALSE");
    } else {
      std::string PTIT = fOPTs.value("PTIT"); //GB option.
      if(PTIT=="PTIT") {
        aRegion.setParameter("titleRegionVisible","FALSE");
        aRegion.setParameter("plotter.titleStyle.visible","TRUE");
        aRegion.setParameter("plotter.titleUp","FALSE");
        aRegion.setParameter("plotter.titleHeight",
                             Lib::smanip::tostring(TSIZ));
        aRegion.setParameter("plotter.titleToAxis",
                             Lib::smanip::tostring(YHTI));
      } else { //RTIT
        aRegion.setParameter("titleRegionVisible","TRUE");
        aRegion.setParameter("plotter.titleStyle.visible","FALSE");
      }
      aRegion.setParameter("plotter.titleAutomated","FALSE");
      aRegion.setParameter("plotter.title",fUTIT);
    }
  }

  //Axis title SIZe
  double ASIZ = fATTs.value("ASIZ");
  aRegion.setParameter
    ("plotter.xAxis.titleHeight",Lib::smanip::tostring(ASIZ));
  aRegion.setParameter
    ("plotter.yAxis.titleHeight",Lib::smanip::tostring(ASIZ));

  //y length of X axis ticks. Def 0.015 
  double XTIC = fATTs.value("YTIC"); 
  aRegion.setParameter
    ("plotter.xAxis.tickLength",Lib::smanip::tostring(XTIC));

  //x length of Y axis ticks. Def 0.015 
  double YTIC = fATTs.value("YTIC"); 
  aRegion.setParameter
    ("plotter.yAxis.tickLength",Lib::smanip::tostring(YTIC));

  //y distance of x tick label to data frame. Def 0.02 
  double YVAL = fATTs.value("YVAL"); 
  aRegion.setParameter
    ("plotter.xAxis.labelToAxis",Lib::smanip::tostring(YVAL));

  //x distance of y tick label to data frame. Def 0.02 
  double XVAL = fATTs.value("XVAL"); 
  aRegion.setParameter
    ("plotter.yAxis.labelToAxis",Lib::smanip::tostring(XVAL));

  //y distance of x title to data frame. Def 0.04  
  double YOPENPAW = fATTs.value("YOPENPAW"); 
  aRegion.setParameter
    ("plotter.xAxis.titleToAxis",Lib::smanip::tostring(YOPENPAW));

  //x distance of y title to data frame. Def 0.07  
  double XOPENPAW = fATTs.value("XOPENPAW"); 
  aRegion.setParameter
    ("plotter.yAxis.titleToAxis",Lib::smanip::tostring(XOPENPAW));

  // axis labels SIZe. Def : 0.28F.
  double VSIZ = fATTs.value("VSIZ"); 
  aRegion.setParameter
    ("plotter.xAxis.labelHeight",Lib::smanip::tostring(VSIZ));
  aRegion.setParameter
    ("plotter.yAxis.labelHeight",Lib::smanip::tostring(VSIZ));
  aRegion.setParameter
    ("plotter.zAxis.labelHeight",Lib::smanip::tostring(VSIZ));

  if(fOPTs.value("LINX")=="LOGX") {
    aRegion.setParameter("plotter.xAxisLogScale","TRUE");
  } else {
    aRegion.setParameter("plotter.xAxisLogScale","FALSE");
  }
  if(fOPTs.value("LINY")=="LOGY") {
    aRegion.setParameter("plotter.yAxisLogScale","TRUE");
  } else {
    aRegion.setParameter("plotter.yAxisLogScale","FALSE");
  }
  if(fOPTs.value("LINZ")=="LOGZ") {
    aRegion.setParameter("plotter.zAxisLogScale","TRUE");
  } else {
    aRegion.setParameter("plotter.zAxisLogScale","FALSE");
  }

 {int NDVX = (int)fATTs.value("NDVX"); 
  if(NDVX<0) NDVX *= -1;
  if(fZONE.NX()>=2) { 
    //seen with pawex22. Prim-div seems to be divided by the number of x zones.
    int N3 = NDVX/10000;    //See SbAxisHPLOT::paint
    int N2 = (NDVX - N3*10000)/100;    
    int N1 = NDVX - N2*100 - N3*10000;    
    NDVX = N1/fZONE.NX() + N2 * 100 + N3 * 10000;
  }
  aRegion.setParameter("plotter.xAxis.divisions",Lib::smanip::tostring(NDVX));}

 {int NDVY = int(fATTs.value("NDVY")); 
  if(NDVY<0) NDVY *= -1;
  aRegion.setParameter("plotter.yAxis.divisions",Lib::smanip::tostring(NDVY));}

  double NDVZ = fATTs.value("NDVZ"); 
  if(NDVZ<0) NDVZ *= -1;
  aRegion.setParameter
    ("plotter.zAxis.divisions",Lib::smanip::tostring(NDVZ));

  //  BWID  //  1   // Box line WIDth      
  double BWID = fATTs.value("BWID"); 
  BWID = MAXIMUM(int(BWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F
  aRegion.setParameter
    ("plotter.innerFrameStyle.lineWidth",Lib::smanip::tostring(BWID));
  aRegion.setParameter
    ("plotter.xAxis.lineStyle.width",Lib::smanip::tostring(BWID));
  aRegion.setParameter
    ("plotter.yAxis.lineStyle.width",Lib::smanip::tostring(BWID));

  //  PWID  //  1   // Picture line WIDth  

  //  XWID  //  1   // X ticks WIDth       
  double XWID = fATTs.value("XWID"); 
  XWID = MAXIMUM(int(XWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F
  aRegion.setParameter
    ("plotter.xAxis.ticksStyle.width",Lib::smanip::tostring(XWID));

  //  YWID  //  1   // Y ticks WIDth       
  double YWID = fATTs.value("YWID"); 
  YWID = MAXIMUM(int(YWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F
  aRegion.setParameter
    ("plotter.yAxis.ticksStyle.width",Lib::smanip::tostring(YWID));

  //  CWID  // 0.50 // Color map WIDth     

  std::string NGRI = fOPTs.value("NGRI");
  if(NGRI=="NGRI") {
    aRegion.setParameter("plotter.gridStyle.visible","FALSE");
  } else {
    aRegion.setParameter("plotter.gridStyle.visible","TRUE");
  }

  int GRID = (int)fATTs.value("GRID"); 
  // o00p
  // o = 1 = horizontal only.
  // o = 2 = vertical only. 
  // o = else = horizontal and vertical.
  // p = 2 = dashed.
  // p = 3 = dotted.
  // p = 4 = dash-dotted.
 {int pattern = GRID % 10;
  int orientation = GRID / 1000;
  std::string spattern = SbLinePattern_s_solid;
  if(pattern==2) spattern = SbLinePattern_s_dashed;
  else if(pattern==3) spattern = SbLinePattern_s_dotted;
  else if(pattern==4) spattern = SbLinePattern_s_dash_dotted;
  aRegion.setParameter("plotter.gridStyle.linePattern",spattern);
  //FIXME : have a field orientation on SoStyle.
  if(orientation==1)
    aRegion.setParameter("plotter.gridStyle.options","horizontal");
  else if(orientation==2)
    aRegion.setParameter("plotter.gridStyle.options","vertical");
  else
    aRegion.setParameter("plotter.gridStyle.options","");}

  std::string NSTA = fOPTs.value("NSTA");
  std::string NFIT = fOPTs.value("NFIT");
  if( (NSTA=="STA") || (NFIT=="FIT") )  {
    std::string opts;
    getInfosOptions(opts);
    aRegion.setParameter("infosRegionVisible","TRUE");
    aRegion.setParameter("plotter.infosStyle.options",opts);
  } else {
    aRegion.setParameter("infosRegionVisible","FALSE");
  }

  // Fonts :
  regionSet_VFON(aRegion);
  regionSet_TFON(aRegion);
  regionSet_LFON(aRegion);
  regionSet_GFON(aRegion);

}

//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::getFont(
 const std::string& aATT
,std::string& aFont
,double& aScale
) 
//////////////////////////////////////////////////////////////////////////////
// ATT value code :
//      2 : Hershey  
//    -fp : PAW code. f is the font and p the "precision". From page 129.
//   1000 + <ROOT_code> : ROOT code convention. From TAttText.cxx.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int X_FON = (int)fATTs.value(aATT);

  aScale = 0.016/0.28; // majic number scaling for TTF.

  // HIGZ :
  if(X_FON==2) {
    aFont = "Hershey";
    aScale = 1;
    return;

  // PAW code :
  } else if(X_FON==-10) {
    aFont = "TTF/timesi";
    return;
  } else if(X_FON==-20) {
    aFont = "TTF/timesbd";
    return;
  } else if(X_FON==-30) {
    aFont = "TTF/timesbi";
    return;

  } else if(X_FON==-40) {
    aFont = "TTF/Helvetica";
    return;
  } else if(X_FON==-50) {
  } else if(X_FON==-60) {
  } else if(X_FON==-70) {

  } else if(X_FON==-80) {
    aFont = "TTF/cour";
    return;
  } else if(X_FON==-90) {
    aFont = "TTF/couri";
    return;
  } else if(X_FON==-100) {
    aFont = "TTF/courbd";
    return;
  } else if(X_FON==-110) {
    aFont = "TTF/courbi";
    return;

  } else if(X_FON==-120) {
    aFont = "TTF/symbol";
    return;
  } else if(X_FON==-130) {
    aFont = "TTF/Times-Roman";
    return;
  } else if(X_FON==-140) {
    //ZapfDingbats

  } else if(X_FON==-150) {
    aFont = "TTF/timesi";
    return;
  } else if(X_FON==-160) {
    aFont = "TTF/timesbd";
    return;
  } else if(X_FON==-170) {
    aFont = "TTF/timesbi";
    return;

  } else if(X_FON==-180) {
    aFont = "TTF/Helvetica";
    return;
  } else if(X_FON==-190) {
  } else if(X_FON==-200) {
  } else if(X_FON==-210) {

  } else if(X_FON==-220) {
    aFont = "TTF/symbol";
    return;
  } else if(X_FON==-230) {
    aFont = "TTF/Times-Roman";
    return;
  } else if(X_FON==-240) {
    //ZapfDingbats

  // ROOT code :
  } else if(X_FON==132) {
    aFont = SbFont_ROOT_132;
    return;
  } else if(X_FON==12) {
    aFont = SbFont_ROOT_12;
    return;
  } else if(X_FON==22) {
    aFont = SbFont_ROOT_22;
    return;
  } else if(X_FON==32) {
    aFont = SbFont_ROOT_32;
    return;
  } else if(X_FON==42) {
    aFont = SbFont_ROOT_42;
    return;
  } else if(X_FON==52) {
    aFont = SbFont_ROOT_52;
    return;
  } else if(X_FON==62) {
    aFont = SbFont_ROOT_62;
    return;
  } else if(X_FON==72) {
    aFont = SbFont_ROOT_72;
    return;
  } else if(X_FON==82) {
    aFont = SbFont_ROOT_82;
    return;
  } else if(X_FON==92) {
    aFont = SbFont_ROOT_92;
    return;
  } else if(X_FON==102) {
    aFont = SbFont_ROOT_102;
    return;
  } else if(X_FON==112) {
    aFont = SbFont_ROOT_112;
    return;
  } else if(X_FON==122) {
    aFont = SbFont_ROOT_122;
    return;
  } else if(X_FON==142) {
    aFont = SbFont_ROOT_142;
    return;
  }

  Lib::Out out(fSession.printer());
  out << "OpenPAW::PAW_Manager::getFont :" 
      << " for ATT " << Lib::sout(aATT)
      << " unknown font code " << X_FON << "."
      << " Take Hershey (code 2)."
      << Lib::endl;

  aFont = "Hershey";
  aScale = 1;

}
//////////////////////////////////////////////////////////////////////////////
std::string OpenPAW::PAW_Manager::get_MTYP(
 int aMTYP
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Documented :
  if(aMTYP==31) 
    return "ASTERISK";
  else if(aMTYP==30)
    return "PENTA_STAR_LINE";
  else if(aMTYP==29) 
    return "PENTA_STAR_FILLED";
  else if(aMTYP==28) 
    return "SWISS_CROSS_LINE";
  else if(aMTYP==27) 
    return "DIAMOND_LINE";
  else if(aMTYP==26) 
    return "TRIANGLE_UP_LINE";
  else if(aMTYP==25) 
    return "SQUARE_LINE";
  else if(aMTYP==24) 
    return "CIRCLE_LINE";
  else if(aMTYP==23) 
    return "TRIANGLE_DOWN_FILLED";
  else if(aMTYP==22) 
    return "TRIANGLE_UP_FILLED";
  else if(aMTYP==21) 
    return "SQUARE_FILLED";
  else if(aMTYP==20) 
    return "CIRCLE_FILLED";

  // Not documented :
  else if(aMTYP==1) 
    return "DOT";
  else if(aMTYP==2)
    return "PLUS";
  else if(aMTYP==3)
    return "ASTERISK";
  else if(aMTYP==4)
    return "CIRCLE_LINE";
  else if(aMTYP==5)
    return "CROSS";
  else if(aMTYP==6)
    return "SWISS_CROSS_FILLED";
  else if(aMTYP==7)
    return "SQUARE_FILLED";
  else if(aMTYP==8)
    return "CIRCLE_FILLED";

  // OpenPAW :
  else if(aMTYP==123) 
    return "TRIANGLE_DOWN_LINE";

  else {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::get_MTYP :" 
        << " marker type " << aMTYP << " not yet supported." 
        << " We use dot for it for the moment." 
        << Lib::endl;
    return "DOT";
  }
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSet_VFON(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
// Axis labels font.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string font;
  double scale;
  getFont("VFON",font,scale);

  aRegion.setParameter("plotter.xAxis.labelsStyle.fontName",font);
  aRegion.setParameter("plotter.yAxis.labelsStyle.fontName",font);
  aRegion.setParameter("plotter.zAxis.labelsStyle.fontName",font);

  std::string smoothing = "TRUE";
  aRegion.setParameter("plotter.xAxis.labelsStyle.smoothing",smoothing);
  aRegion.setParameter("plotter.yAxis.labelsStyle.smoothing",smoothing);
  aRegion.setParameter("plotter.zAxis.labelsStyle.smoothing",smoothing);

 {std::string s = Lib::smanip::tostring(scale);
  aRegion.setParameter("plotter.xAxis.labelsStyle.scale",s);
  aRegion.setParameter("plotter.yAxis.labelsStyle.scale",s);
  aRegion.setParameter("plotter.zAxis.labelsStyle.scale",s);}
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSet_TFON(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
// Region title font.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string font;
  double scale;
  getFont("TFON",font,scale);

  aRegion.setParameter("plotter.titleStyle.fontName",font);

  std::string smoothing = "TRUE";
  aRegion.setParameter("plotter.titleStyle.smoothing",smoothing);

 {std::string s = Lib::smanip::tostring(scale);
  aRegion.setParameter("plotter.titleStyle.scale",s);}
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSet_LFON(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
// Axis title font.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string font;
  double scale;
  getFont("LFON",font,scale);

  aRegion.setParameter("plotter.xAxis.titleStyle.fontName",font);
  aRegion.setParameter("plotter.yAxis.titleStyle.fontName",font);
  aRegion.setParameter("plotter.zAxis.titleStyle.fontName",font);

  std::string smoothing = "TRUE";
  aRegion.setParameter("plotter.xAxis.titleStyle.smoothing",smoothing);
  aRegion.setParameter("plotter.yAxis.titleStyle.smoothing",smoothing);
  aRegion.setParameter("plotter.zAxis.titleStyle.smoothing",smoothing);

 {std::string s = Lib::smanip::tostring(scale);
  aRegion.setParameter("plotter.xAxis.titleStyle.scale",s);
  aRegion.setParameter("plotter.yAxis.titleStyle.scale",s);
  aRegion.setParameter("plotter.zAxis.titleStyle.scale",s);}
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSet_GFON(
 AIDA::IPlotterRegion& aRegion
) 
//////////////////////////////////////////////////////////////////////////////
// Region title font if using titleRegion.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string font;
  double scale;
  getFont("GFON",font,scale);

  aRegion.setParameter("titleRegion.styles[0].fontName",font);

  std::string smoothing = "TRUE";
  aRegion.setParameter("titleRegion.styles[0].smoothing",smoothing);

 {std::string s = Lib::smanip::tostring(scale);
  aRegion.setParameter("titleRegion.styles[0].scale",s);}
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSet_GFON(
 AIDA::IPlotter& aPlotter
) 
//////////////////////////////////////////////////////////////////////////////
// Page title font.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string font;
  double scale;
  getFont("GFON",font,scale);

  aPlotter.setParameter("titleRegion.styles[0].fontName",font);

  std::string smoothing = "TRUE";
  aPlotter.setParameter("titleRegion.styles[0].smoothing",smoothing);

 {std::string s = Lib::smanip::tostring(scale);
  aPlotter.setParameter("titleRegion.styles[0].scale",s);}
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::regionSetColormap(
 AIDA::IPlotterRegion& aRegion
,double aMin
,double aMax
,int aPALNB
) 
//////////////////////////////////////////////////////////////////////////////
// if palette contains : red, green, blue, yellow, magenta, cyan
// and aMin, aMax is 0 and 6 then we build :
//    red 0 red 1 green 2 blue 3 yellow 4 magenta 5 cyan 6 cyan
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aPALNB<0)||(aPALNB>=(int)fPalettes.size())) return;
  Palette& pal = fPalettes[aPALNB];
  if(!pal.size()) return;
 
  std::vector<std::string> scols;
 {for(unsigned int index=0;index<pal.size();index++) {
    unsigned char r = (unsigned char)(fColorTable[pal[index]].fRed * 255);
    unsigned char g = (unsigned char)(fColorTable[pal[index]].fGreen * 255);
    unsigned char b = (unsigned char)(fColorTable[pal[index]].fBlue * 255);
    std::string s;
    Lib::smanip::printf(s,64,"#%2x%2x%2x",r,g,b);
    Lib::smanip::replace(s," ","0");
    scols.push_back(s);
  }}

  std::vector< std::pair<double,double> > limits;
  unsigned int ncol = scols.size();
  double dw = (aMax - aMin)/ncol;
 {for(unsigned int index=0;index<ncol;index++) {
    double cmn = aMin + index * dw;
    double cmx = cmn + dw;
    limits.push_back(std::pair<double,double>(cmn,cmx));
  }}

  std::string cmap = scols[0] + " ";
 {for(unsigned int index=0;index<ncol;index++) {
    cmap += Lib::smanip::tostring(limits[index].first);
    cmap += " ";
    cmap += scols[index];
    cmap += " ";
  }}
  cmap += Lib::smanip::tostring(limits[ncol-1].second);
  cmap += " ";
  cmap += scols[ncol-1];
  cmap += " ";

  if(fSession.verboseLevel()) {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::regionSetColormap :" 
        << " min= " << aMin
        << " max= " << aMax
        << " PALNB= " << aPALNB
        << " cmap= " << Lib::sout(cmap)
        << Lib::endl;
  }  

  aRegion.setParameter("plotter.colorMapping",cmap);
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::plotFunction1D(
 AIDA::IPlotterRegion& aRegion
,AIDA::IFunction& aFunction
,double aXLOW
,double aXUP
,const std::string& aModel
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // For CERN/PAW, NPX is the number of points between
  // XLOW, XUP but by excluding these !

  if(aXUP<=aXLOW) {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::plotFunction1D :"
        << " XUP (" << aXUP << ") <= XLOW (" << aXLOW << ") !"
        << Lib::endl;
    return;
  }
  if(fNPX<=1) {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::plotFunction1D :"
        << " NPX <=1 !"
        << Lib::endl;
    return;
  }

  double df = (aXUP - aXLOW)/(fNPX-1+2);
  double xmn = aXLOW + df;
  double xmx = aXUP  - df;
  aFunction.annotation().addItem
    ("PlottingNumberOfStepsX",Lib::smanip::tostring(fNPX-1));
  aFunction.annotation().addItem
    ("PlottingMinimumX",Lib::smanip::tostring(xmn));
  aFunction.annotation().addItem
    ("PlottingMaximumX",Lib::smanip::tostring(xmx));

  aRegion.setParameter("plotter.shapeAutomated","FALSE");
  aRegion.setParameter("plotter.shape","XY");

  int number;
  region_field_value(aRegion,"numberOfPlottedFunction1D",number,
    fSession.printer());

  std::string field;
  Lib::smanip::printf(field,64,"plotter.functionStyle[%d].",number);

  if(aModel=="C") {

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    int PLCI = (int)fATTs.value("PLCI");
    region_set_COL(aRegion,field,PLCI);

    region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

    aRegion.plot(aFunction);

  } else if(aModel=="P") {

    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_points);

    // Parameters :
    int PMCI = (int)fATTs.value("PMCI");
    region_set_COL(aRegion,field,PMCI);

    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));
    REGION_SET_MTYP

    aRegion.plot(aFunction);

  } else if(aModel=="L") {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::plotFunction1D :"
        << " option L not yet supported."
        << Lib::endl;

  } else if(aModel=="*") {
    Lib::Out out(fSession.printer());
    out << "OpenPAW::PAW_Manager::plotFunction1D :"
        << " option * not yet supported."
        << Lib::endl;
  }

}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::plotFunction2D(
 AIDA::IPlotterRegion& aRegion
,AIDA::IFunction& aFunction
,double aXLOW
,double aXUP
,double aYLOW
,double aYUP
,double aZLOW
,double aZUP
,int aNLEVEL
,const std::string& aModel
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aRegion.setParameter("plotter.shapeAutomated","FALSE");
  aRegion.setParameter("plotter.shape","XY");

  int number;
  region_field_value(aRegion,"numberOfPlottedFunction2D",number,
    fSession.printer());
  std::string field;
  Lib::smanip::printf(field,64,"plotter.functionStyle[%d].",number);

  if(aModel=="CONT_COL") {

    aRegion.setParameter(field+"modeling",SbModeling_curve);

    int PALNB = paletteFromHCOL();
    if(fOPTs.value("COLL")=="COLL") {
      aRegion.setParameter(field+"painting","BY_LEVEL");
      regionSetColormap(aRegion,0,0,PALNB);
    } else {
      aRegion.setParameter(field+"painting","BY_VALUE");
      regionSetColormap(aRegion,aZLOW,aZUP,PALNB);
    }

  } else if(aModel=="CONT_COL_FILLED") {

    aRegion.setParameter(field+"modeling",SbModeling_filled_curve);

    int PALNB = paletteFromHCOL();
    if(fOPTs.value("COLL")=="COLL") {
      aRegion.setParameter(field+"painting","BY_LEVEL");
      regionSetColormap(aRegion,0,0,PALNB);
    } else {
      aRegion.setParameter(field+"painting","BY_VALUE");
      regionSetColormap(aRegion,aZLOW,aZUP,PALNB);
    }

  } else if(aModel=="CONT_DMOD") {

    aRegion.setParameter(field+"modeling",SbModeling_curve);

    aRegion.setParameter(field+"painting","UNIFORM");
    int HCOL = (int)fATTs.value("HCOL");
    int color = HCOL - ((int)(HCOL/1000)) * 1000;
    region_set_COL(aRegion,field,color);

  }
  region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

  aRegion.setParameter
    ("plotter.numberOfLevels",Lib::smanip::tostring(aNLEVEL));

  aFunction.annotation().addItem
    ("PlottingNumberOfStepsX",Lib::smanip::tostring(fNPX));
  aFunction.annotation().addItem
    ("PlottingMinimumX",Lib::smanip::tostring(aXLOW));
  aFunction.annotation().addItem
    ("PlottingMaximumX",Lib::smanip::tostring(aXUP));

  aFunction.annotation().addItem
    ("PlottingNumberOfStepsY",Lib::smanip::tostring(fNPY));
  aFunction.annotation().addItem
    ("PlottingMinimumY",Lib::smanip::tostring(aYLOW));
  aFunction.annotation().addItem
    ("PlottingMaximumY",Lib::smanip::tostring(aYUP));

  aRegion.plot(aFunction);

}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::plotCloud2D(
 AIDA::IPlotterRegion& aRegion
,AIDA::ICloud2D& aCloud
,const std::string& aModel
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number;
  region_field_value
    (aRegion,"numberOfPlottedPoints2D",number,fSession.printer());
  std::string field;
  Lib::smanip::printf(field,64,"plotter.pointsStyle[%d].",number);

  if(aModel=="L") {
    // Lines joining points.

    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_lines);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

  } else if(aModel=="P") {
    // (Poly)markers.

    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_points);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));
    REGION_SET_MTYP

  } else if(aModel=="C") {
    // Smooth curve joining points.

    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_curve);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

  }

  aRegion.plot(aCloud);
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::plotErrors2D(
 AIDA::IPlotterRegion& aRegion
,AIDA::IDataPointSet& aDPS
,int aMTYP
,double aMSCF
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int number;
  region_field_value
    (aRegion,"numberOfPlottedBins1D",number,fSession.printer());

 {std::string field;
  Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

  aRegion.setParameter(field+"visible","TRUE");
  aRegion.setParameter(field+"markerStyle",get_MTYP(aMTYP));
  region_set_MSCF(aRegion,field,aMSCF);
  aRegion.setParameter(field+"modeling",SbModeling_points);}

 {std::string field;
  Lib::smanip::printf(field,64,"plotter.errorsStyle[%d].",number);

  aRegion.setParameter(field+"visible","TRUE");
  aRegion.setParameter(field+"markerStyle","PLUS");

  //In CERN/PAW, HCOL change error rep color but not the one of the markers.
  aRegion.setParameter(field+"painting","UNIFORM");
  int HCOL = (int)fATTs.value("HCOL");
  int color = HCOL - ((int)(HCOL/1000)) * 1000;
  region_set_COL(aRegion,field,color);}

  aRegion.plot(aDPS);
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::plotHistogram1D(
 AIDA::IPlotterRegion& aRegion
,AIDA::IHistogram1D& aHistogram
,const std::string& aModel
,bool aLoopLineStyle
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : region_plot_histogram1D : \"%s\"\n",aModel.c_str());

  aRegion.setParameter("plotter.shapeAutomated","FALSE");
  aRegion.setParameter("plotter.shape","XY");

  int number;
  region_field_value
    (aRegion,"numberOfPlottedBins1D",number,fSession.printer());
  if(number==0) region_set_automated(aRegion,aHistogram);
  std::string field;
  Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

  if(aModel=="C") { // Smooth curve joining points.
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_curve);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

    aRegion.plot(aHistogram);

  } else if(aModel=="L") { // Line joining top middle of bins.
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_lines);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_DMOD(aRegion,field,(int)fATTs.value("DMOD")); 

    aRegion.plot(aHistogram);

  } else if(aModel=="E") { // Errors. 
    //FIXME : For a vector error = sqrt(value).
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_points);
    std::string s;
    Lib::smanip::printf(s,32,"%d",number);
    aRegion.setParameter("plotter.errorsStyle["+s+"].visible","TRUE");

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));
    REGION_SET_MTYP

    aRegion.plot(aHistogram);

  } else if(aModel=="P") { // Points at the top middle of bins.
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_points);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));
    REGION_SET_MTYP

    aRegion.plot(aHistogram);

  } else if(aModel=="*") { // '*' at the top middle of bins.
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_points);

    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    aRegion.setParameter(field+"markerSize","9");
    aRegion.setParameter(field+"markerStyle","ASTERISK");

    aRegion.plot(aHistogram);

  } else if(aModel=="B") { // Bounding line of bins reduced to half.
    // Modeling :
    aRegion.setParameter(field+"modeling",SbModeling_bar_chart);
    // Parameters :
    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");
    aRegion.setParameter(field+"linePattern",SOLID);
    aRegion.setParameter(field+"barOffset",
      Lib::smanip::tostring(fATTs.value("BARO")));
    aRegion.setParameter(field+"barWidth",
      Lib::smanip::tostring(fATTs.value("BARW")));

    setHTYP(aRegion,number,
      (int)fATTs.value("HTYP"),fATTs.value("BARO"),fATTs.value("BARW"));

    aRegion.plot(aHistogram);

  } else { // Top lines of bins. Default rep    
    // Modeling :
    if(fContourMode) { 
      aRegion.setParameter(field+"modeling",SbModeling_top_lines);
      std::string spattern = SOLID;
      //PAW loops on : SOLID, DOTTED, DASHED, DASH_DOTTED.
      if(aLoopLineStyle) {
        int lineStyle = number % 5;
             if(lineStyle==0) spattern = SOLID;
        else if(lineStyle==1) spattern = DASHED;
        else if(lineStyle==2) spattern = DOTTED;
        else if(lineStyle==3) spattern = DASH_DOTTED;
        else if(lineStyle==5) spattern = SOLID;
      } 
      aRegion.setParameter(field+"linePattern",spattern);
    } else {
      aRegion.setParameter(field+"modeling",SbModeling_boxes);
    }
    setHTYP(aRegion,number,
      (int)fATTs.value("HTYP"),fATTs.value("BARO"),fATTs.value("BARW"));

    aRegion.setParameter(field+"painting","UNIFORM");
    int HCOL = (int)fATTs.value("HCOL");
    int color = HCOL - ((int)(HCOL/1000)) * 1000;
    region_set_COL(aRegion,field,color);

    double HWID = fATTs.value("HWID"); 
    region_set_HWID(aRegion,field,HWID);

    aRegion.plot(aHistogram);
  }
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::addLegend(
 AIDA::IPlotterRegion& aRegion
,double aX
,double aY
,const std::string& aTEXT
,const std::string& //aModel
,int aIATT
,bool aSuperpose
) 
//////////////////////////////////////////////////////////////////////////////
// aX,aY are in axes coordinates.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string stext = aTEXT;
  Lib::smanip::replace(stext,"\"","\\\"");

  if(aSuperpose) { //Put all in one SoLegendRegion.

    aRegion.setParameter("legendRegionVisible","TRUE");
    aRegion.setParameter("legendAutomated","FALSE");
    aRegion.setParameter("legendRegionSize","0.5 0.1");
  
    aRegion.setParameter("legendRegionAttachedToInfosRegion","FALSE");
    aRegion.setParameter("legendRegionOriginUnit","AXIS");

   {std::string s;
    Lib::smanip::printf(s,64,"%g %g",aX,aY);
    aRegion.setParameter("legendRegionOrigin",s);}
  
    aRegion.setParameter("legendRegion.viewportRegion.visible","FALSE");

    std::string s = aRegion.parameterValue("legendRegion.text");
    Lib::smanip::strip(s,Lib::smanip::leading,'[');
    Lib::smanip::strip(s,Lib::smanip::trailing,']');
    Lib::smanip::strip(s);
    if(s=="\"\"") s = "";

    std::vector<std::string> words = Lib::smanip::words(s,",");
    unsigned int number = words.size();
    //printf("debug : number %d |%s|\n",number,aTEXT.c_str());
  
    std::string si = Lib::smanip::tostring(number);
    std::string field = "legendRegion.styles["+si+"]."; //SoStyle !
    aRegion.setParameter(field+"color","0 0 0");
    aRegion.setParameter(field+"encoding","PAW");
  
   {std::string font;
    double scale;
    getFont("CFON",font,scale);
    aRegion.setParameter(field+"fontName",font);
    std::string smoothing = "TRUE";
    aRegion.setParameter(field+"smoothing",smoothing);
    aRegion.setParameter(field+"scale",Lib::smanip::tostring(scale));}

    //if(aModel=="M") {
    // IATT is the marker type. 
    //}
    aRegion.setParameter(field+"markerStyle",get_MTYP(aIATT));
  
    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));
  
    if(number) s += ",";
    s += "\""+stext+"\"";
    aRegion.setParameter("legendRegion.text","["+s+"]");

  } else { // One SoLegendRegion per /GRAPHICS/HPLOT/KEY

    aRegion.setParameter("legendRegionVisible","FALSE");

    std::string sir = aRegion.parameterValue("numberOfLegendRegions");
    if(!sir.size()) return;
    std::string rfield = "legendRegions["+sir+"].";
  
    aRegion.setParameter(rfield+"visible","TRUE");
    aRegion.setParameter(rfield+"size","0.5 0.1");
  
    aRegion.setParameter(rfield+"verticalJustification","MIDDLE");
  
    aRegion.setParameter(rfield+"originUnit","AXIS");
  
   {std::string s;
    Lib::smanip::printf(s,64,"%g %g",aX,aY);
    aRegion.setParameter(rfield+"origin",s);}
  
    aRegion.setParameter(rfield+"viewportRegion.visible","FALSE");
  
    std::string s = aRegion.parameterValue(rfield+"text");
    Lib::smanip::strip(s,Lib::smanip::leading,'[');
    Lib::smanip::strip(s,Lib::smanip::trailing,']');
    Lib::smanip::strip(s);
    if(s=="\"\"") s = "";
    std::vector<std::string> words = Lib::smanip::words(s,",");
    unsigned int number = words.size();
    //printf("debug : number %d |%s|\n",number,aTEXT.c_str());

    std::string si = Lib::smanip::tostring(number);
    std::string field = rfield+"styles["+si+"]."; //SoStyle !
    aRegion.setParameter(field+"color","0 0 0");
    aRegion.setParameter(field+"encoding","PAW");
  
   {std::string font;
    double scale;
    getFont("CFON",font,scale);
    aRegion.setParameter(field+"fontName",font);
    std::string smoothing = "TRUE";
    aRegion.setParameter(field+"smoothing",smoothing);
    aRegion.setParameter(field+"scale",Lib::smanip::tostring(scale));}
  
    //if(aModel=="M") {
    // IATT is the marker type. 
    //}
    aRegion.setParameter(field+"markerStyle",get_MTYP(aIATT));
  
    region_set_MSCF(aRegion,field,fATTs.value("MSCF"));

    if(number) s += ",";
    s += "\""+stext+"\"";

    aRegion.setParameter(rfield+"text","["+s+"]");
  }
}
//////////////////////////////////////////////////////////////////////////////
void OpenPAW::PAW_Manager::setHTYP(
 AIDA::IPlotterRegion& aRegion
,int aIndex
,int aHTYP
,double aBARO
,double aBARW
) 
//////////////////////////////////////////////////////////////////////////////
// PAW encoding (manual p 122) :
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int code = aHTYP;
  if(code==0) code = 255; //No hatching.
  int i = code / 100;
  int j = (code - i * 100)/10;
  int k = code - i * 100 - j * 10;

  //printf("debug : %d : %d %d %d\n",code,i,j,k);

  if(i==0) return;

  // j-hatching on rightHatchStyle :
  // k-hatching on leftHatchStyle :

  std::string sindex;
  Lib::smanip::printf(sindex,32,"%d",aIndex);

  std::string field = "plotter.binsStyle["+sindex+"].";

  if((j==5)&&(k==5))
    aRegion.setParameter(field+"hatching","NONE");
  else if((j!=5)&&(k==5))
    aRegion.setParameter(field+"hatching","RIGHT");
  else if((j==5)&&(k!=5))
    aRegion.setParameter(field+"hatching","LEFT");
  else if((j!=5)&&(k!=5))
    aRegion.setParameter(field+"hatching","LEFT_AND_RIGHT");

  std::string ss;

  // In CERN/PAW, hatch spacing is fixed for the page (not for the region) !
  int NX = 0;
  int NY = 0;
  if(fZONE.SUPER()) {
    NX = -fZONE.NX();
    NY = -fZONE.NY();
  } else {
    NX = fZONE.NX();
    NY = fZONE.NY();
  }

  double spacing = NY * i * 0.004;

  if(j!=5) {
    field = "plotter.rightHatchStyle["+sindex+"].";
    aRegion.setParameter(field+"color","0 0 0");
    double angle = (j==4?45:j*10);
    angle = 180 - angle;
    angle *= Lib::mmanip::pi() / 180;
    Lib::smanip::printf(ss,32,"%g",spacing);
    aRegion.setParameter(field+"spacing",ss);
    Lib::smanip::printf(ss,32,"%g",angle);
    aRegion.setParameter(field+"angle",ss);
    aRegion.setParameter(field+"barOffset",Lib::smanip::tostring(aBARO));
    aRegion.setParameter(field+"barWidth",Lib::smanip::tostring(aBARW));
  }

  if(k!=5) {
    field = "plotter.leftHatchStyle["+sindex+"].";
    aRegion.setParameter(field+"color","0 0 0");
    double angle = (k==4?45:k*10);
    angle *= Lib::mmanip::pi() / 180;
    Lib::smanip::printf(ss,32,"%g",spacing);
    aRegion.setParameter(field+"spacing",ss);
    Lib::smanip::printf(ss,32,"%g",angle);
    aRegion.setParameter(field+"angle",ss);
    aRegion.setParameter(field+"barOffset",Lib::smanip::tostring(aBARO));
    aRegion.setParameter(field+"barWidth",Lib::smanip::tostring(aBARW));
  }

}
