#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/sout.h>

#include <AIDA/IFunctionCatalog.h>

//#include <BatchLab/Helpers/AIDA.h>
#include <OpenPAW/PAW_Manager.h>

#include <HEPVis/SbStyles.h> //#define only.

#include "RegionHelpers.h"

//////////////////////////////////////////////////////////////////////////////
inline void region_plot_histogram2D(
 Slash::Core::ISession& aSession
,AIDA::IPlotterRegion& aRegion
,AIDA::IHistogram2D& aHistogram
,const std::string& aModel
,Slash::Core::IWriter& aPrinter
,int aNLEVEL
,OpenPAW::PAW_Manager& aPAW_Manager
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  ATTs_type& atts = aPAW_Manager.atts();
  OPTs_type& opts = aPAW_Manager.opts();
  if( (aModel=="SURF") || (aModel=="SURF_COL") ) {
    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_curve);

    if(aModel=="SURF") {
      aRegion.setParameter(field+"painting","UNIFORM");
      aRegion.setParameter(field+"color","1 1 1");
    } else if(aModel=="SURF_COL") {
      int PALNB = aPAW_Manager.paletteFromHCOL();
      aRegion.setParameter(field+"painting","BY_VALUE");
      double mn = aHistogram.minBinHeight();
      double mx = aHistogram.maxBinHeight();
      aPAW_Manager.regionSetColormap(aRegion,mn,mx,PALNB);
    }

    aRegion.setParameter("plotter.wallStyle.visible","FALSE");
    aRegion.setParameter("plotter.gridStyle.visible","TRUE");

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XYZ");

    aRegion.plot(aHistogram);

  } else if(aModel=="LEGO") {
    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_boxes);

    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","1 1 1");

    aRegion.setParameter("plotter.wallStyle.visible","FALSE");
    aRegion.setParameter("plotter.gridStyle.visible","TRUE");

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XYZ");

    aRegion.plot(aHistogram);

  } else if(aModel=="TEXT") {
    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_texts);

    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XY");

    aRegion.plot(aHistogram);

  } else if(aModel=="BOX") {

    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_wire_boxes);

    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XY");

    aRegion.plot(aHistogram);

  } else if(aModel=="COL") {

    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_solid);

    //int PALNB = aPAW_Manager.paletteFromHCOL();
    int PALNB = PALETTE_HISTOGRAM_PLOT_OPT_COL;
    aRegion.setParameter(field+"painting","BY_VALUE");
    double mn = aHistogram.minBinHeight();
    double mx = aHistogram.maxBinHeight();
    aPAW_Manager.regionSetColormap(aRegion,mn,mx,PALNB);

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XY");

    aRegion.plot(aHistogram);

  } else if( (aModel=="CONT_COL") || 
             (aModel=="CONT_COL_FILLED") ||
             (aModel=="CONT_DMOD") ) {
    AIDA::IAnnotation& annotation = aHistogram.annotation();
    std::string codelet = annotation.value("Codelet");
    if(codelet=="") {
      Lib::Out out(aPrinter);
      out << "region_plot_histogram2D :" 
          << " histogram with title " << Lib::sout(aHistogram.title())
          << " does not have a function script."
          << " We use content of bins to draw contours."
          << Lib::endl;

      int number;
      region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
      if(number==0) region_set_automated(aRegion,aHistogram);
      std::string field;
      Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

      if(aModel=="CONT_COL_FILLED") {
        aRegion.setParameter(field+"modeling",SbModeling_filled_curve);
      } else {
        aRegion.setParameter(field+"modeling",SbModeling_curve);
      }  

      if( (aModel=="CONT_COL") || (aModel=="CONT_COL_FILLED") ){
        int PALNB = aPAW_Manager.paletteFromHCOL();
        if(opts.value("COLL")=="COLL") {
          aRegion.setParameter(field+"painting","BY_LEVEL");
          aPAW_Manager.regionSetColormap(aRegion,0,0,PALNB);
        } else {
          aRegion.setParameter(field+"painting","BY_VALUE");
          double mn = aHistogram.minBinHeight();
          double mx = aHistogram.maxBinHeight();
          aPAW_Manager.regionSetColormap(aRegion,mn,mx,PALNB);
        }
      } else if(aModel=="CONT_DMOD") {
        aRegion.setParameter(field+"painting","UNIFORM");
        int HCOL = (int)atts.value("HCOL");
        int color = HCOL - ((int)(HCOL/1000)) * 1000;
        region_set_COL(aRegion,field,color);
      }

      region_set_DMOD(aRegion,field,(int)atts.value("DMOD")); 
      aRegion.setParameter
        ("plotter.numberOfLevels",Lib::smanip::tostring(aNLEVEL));

      aRegion.setParameter("plotter.shapeAutomated","FALSE");
      aRegion.setParameter("plotter.shape","XY");

      aRegion.plot(aHistogram);

    } else { //codelet!=""

      AIDA::IAnalysisFactory* aida = find_aida(aSession);
      if(!aida) return;

      AIDA::ITree* memoryTree = find_memoryTree(*aida);
      if(!memoryTree) return;

      std::string funcName = aPAW_Manager.generateFunctionName();
      AIDA::IFunction* rep = 
        create_function(*aida,*memoryTree,funcName,codelet,codelet);
      if(!rep) {
        Lib::Out out(aPrinter);
        out << "region_plot_histogram2D :" 
            << " can't create function with codelet " << Lib::sout(codelet) 
            << Lib::endl;
        return;
      }
      
      double XLOW,XUP;
     {bool xauto;
      region_parameterValue(aRegion,"plotter.xAxisAutomated",xauto);
      if(xauto) {
        XLOW = aHistogram.xAxis().lowerEdge();
        XUP = aHistogram.xAxis().upperEdge();
      } else {
        region_parameterValue(aRegion,"plotter.xAxisMinimum",XLOW);
        region_parameterValue(aRegion,"plotter.xAxisMaximum",XUP);
      }}

      double YLOW,YUP;
     {bool yauto;
      region_parameterValue(aRegion,"plotter.yAxisAutomated",yauto);
      if(yauto) {
        YLOW = aHistogram.yAxis().lowerEdge();
        YUP = aHistogram.yAxis().upperEdge();
      } else {
        region_parameterValue(aRegion,"plotter.yAxisMinimum",YLOW);
        region_parameterValue(aRegion,"plotter.yAxisMaximum",YUP);
      }}
      

      double ZLOW = aHistogram.minBinHeight();
      double ZUP = aHistogram.maxBinHeight();
      aPAW_Manager.plotFunction2D(aRegion,*rep,
                                  XLOW,XUP,YLOW,YUP,ZLOW,ZUP,
                                  aNLEVEL,aModel);
    }

  } else {

    // Default is random points.

    int number;
    region_field_value(aRegion,"numberOfPlottedBins2D",number,aPrinter);
    if(number==0) region_set_automated(aRegion,aHistogram);
    std::string field;
    Lib::smanip::printf(field,64,"plotter.binsStyle[%d].",number);

    aRegion.setParameter(field+"modeling",SbModeling_points);

    aRegion.setParameter(field+"painting","UNIFORM");
    aRegion.setParameter(field+"color","0 0 0");

    aRegion.setParameter(field+"markerStyle","DOT");

    aRegion.setParameter("plotter.shapeAutomated","FALSE");
    aRegion.setParameter("plotter.shape","XY");

    aRegion.plot(aHistogram);

  }
}
