#include <AIDA/IPlotterRegion.h>
#include <AIDA/IAnnotation.h>
#include <AIDA/IBaseHistogram.h>

#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/mmanip.h>

#define SOLID  "0xFFFF"
#define DASHED "0x00FF"
#define DOTTED "0x0101"
#define DASH_DOTTED "0x1C47"

#define region_MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
inline void region_set_COL(
 AIDA::IPlotterRegion& aRegion
,const std::string& aField
,int aCOLOR
) 
//////////////////////////////////////////////////////////////////////////////
// 1 black
// 2 red
// 3 green
// 4 blue
// 5 yellow
// 6 magenta
// 7 cyan
// 8 white
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string scolor;

       if(aCOLOR==0) scolor = "0 0 0"; //black
  else if(aCOLOR==1) scolor = "0 0 0"; //black
  else if(aCOLOR==2) scolor = "1 0 0"; //red
  else if(aCOLOR==3) scolor = "0 1 0"; //green
  else if(aCOLOR==4) scolor = "0 0 1"; //blue
  else if(aCOLOR==5) scolor = "1 1 0"; //yellow
  else if(aCOLOR==6) scolor = "1 0 1"; //magenta
  else if(aCOLOR==7) scolor = "0 1 1"; //cyan
  else if(aCOLOR==8) scolor = "1 1 1"; //white
  else               scolor = "1 1 1"; //white

  aRegion.setParameter(aField+"color",scolor);  
}
//////////////////////////////////////////////////////////////////////////////
inline void region_set_DMOD(
 AIDA::IPlotterRegion& aRegion
,const std::string& aField
,int aDMOD
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aDMOD==2) {
    aRegion.setParameter(aField+"linePattern",DASHED);
  } else if(aDMOD==3) {
    aRegion.setParameter(aField+"linePattern",DOTTED);
  } else if(aDMOD==4) {
    aRegion.setParameter(aField+"linePattern",DASH_DOTTED);
  } else {
    aRegion.setParameter(aField+"linePattern",SOLID);
  }
}
//////////////////////////////////////////////////////////////////////////////
inline bool region_field_value(
 AIDA::IPlotterRegion& aRegion
,const std::string& aField
,int& aValue
,Slash::Core::IWriter& aPrinter
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string svalue = aRegion.parameterValue("plotter."+aField);
  unsigned int number;
  if(!Lib::smanip::touint(svalue,number)) {
    Lib::Out out(aPrinter);
    out << "plotter_field_value :" 
        << " for field " << Lib::sout(aField) 
        << ", can't convert " << Lib::sout(svalue) << "to long." << Lib::endl;
    aValue = 0;
    return false;
  }
  aValue = number;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
inline void region_set_MSCF(
 AIDA::IPlotterRegion& aRegion
,const std::string& aField
,double aMSCF
) 
//////////////////////////////////////////////////////////////////////////////
// Default aMSCF is 1 and match markers_9_9.
// FIXME : then no larger size today available.
// Note that with the below : 
//   MSCF = 0.56 -> 5
//   MSCF = 0.78 -> 7
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int sz = int(9.*aMSCF);
  aRegion.setParameter(aField+"markerSize",Lib::smanip::tostring(sz));
}
//////////////////////////////////////////////////////////////////////////////
inline void region_set_HWID(
 AIDA::IPlotterRegion& aRegion
,const std::string& aField
,double aHWID
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //  HWID  //  1   // Histogram line WIDth
  //From graflib/higz/ikernel/islwsc.F
  int HWID = region_MAXIMUM(int(aHWID/2.2),1);
  aRegion.setParameter(aField+"lineWidth",Lib::smanip::tostring(HWID));
}
//////////////////////////////////////////////////////////////////////////////
inline void region_set_automated(
 AIDA::IPlotterRegion& aRegion
,AIDA::IBaseHistogram& aHistogram
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : region_set_automated \"%s\"\n",histogram_name(aHistogram).c_str());

  AIDA::IAnnotation& annotation = aHistogram.annotation();

  bool automated = true;
  std::string svalue = annotation.value("PlottingValueAutomated");
  if(svalue!="") {
    if(!Lib::smanip::tobool(svalue,automated)) automated = true;
  }

  if(aHistogram.dimension()==1) {
    if(automated) {
      aRegion.setParameter("plotter.yAxisAutomated","TRUE");
    } else {
      aRegion.setParameter("plotter.yAxisAutomated","FALSE");
      std::string smn = annotation.value("PlottingValueMinimum");
      std::string smx = annotation.value("PlottingValueMaximum");
      aRegion.setParameter("plotter.yAxisMinimum",smn);
      aRegion.setParameter("plotter.yAxisMaximum",smx);
    }
  } else if(aHistogram.dimension()==2) {
    if(automated) {
      aRegion.setParameter("plotter.zAxisAutomated","TRUE");
    } else {
      aRegion.setParameter("plotter.zAxisAutomated","FALSE");
      std::string smn = annotation.value("PlottingValueMinimum");
      std::string smx = annotation.value("PlottingValueMaximum");
      aRegion.setParameter("plotter.zAxisMinimum",smn);
      aRegion.setParameter("plotter.zAxisMaximum",smx);
    }
  }
}

