extern "C" {
  void pafunc_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Lib/fmanip.h>
#include <Lib/smanip.h>

#include <BatchLab/Helpers/AIDA.h>

#include <OpenPAW/PAW_Manager.h>

#include "KuipHelpers.h"
#include "PlotHistogram2D.h"
#include "FindHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void pafunc_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  //std::string cmd_name = ku_name();
  std::string cmd_path = ku_path() ? ku_path() : "";
  if(session->verboseLevel()) {
    out << "pafunc :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(cmd_path=="/FUNCTION/PLOT") {

    std::string UFUNC = ku_gets();
    double XLOW = ku_getr();
    double XUP = ku_getr();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "pafunc :" 
          << " " << cmd_path
          << " UFUNC= " << Lib::sout(UFUNC) 
          << " XLOW= " << XLOW
          << " XUP= " << XUP
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string funcName = pawManager->generateFunctionName();

    char* path = ku_macro_path();
    std::string paths = path?path:"";

    // Managed because there is no retain mode for function.
    AIDA::IFunction* func = create_function_vector
      (funcName,paths,UFUNC,*session,*aida,*memoryTree,true);

    if(!func) {
      out << "pafunc :" 
          << " can't create function " << Lib::sout(UFUNC) 
          << Lib::endl;
      return;
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) {
        // If superposed, CERN/PAW ignores the title of the object.
        func->setTitle(""); 
      } else {
        pawManager->plotterNext(*plotter);
      }

      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      if(func->dimension()==1) {

        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P")) {
          pawManager->plotFunction1D(region,*func,XLOW,XUP,"P");
        } else {
          pawManager->plotFunction1D(region,*func,XLOW,XUP,"C");
        }

      } else {
        out << "pafunc :" 
            << " dimension mismatch."
            << " /FUNCTION/PLOT is for 1D functions"
            << " (" << func->dimension() << " found)."
            << Lib::endl;
      }

      delete plotter;
    }
    
  } else if(cmd_path=="/FUNCTION/FUN2") {

    std::string ID = ku_gets();
    std::string UFUNC = ku_gets();
    int NCX = ku_geti();
    double XMIN = ku_getr();
    double XMAX = ku_getr();
    int NCY = ku_geti();
    double YMIN = ku_getr();
    double YMAX = ku_getr();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "pafunc :" 
          << " " << cmd_path
          << " ID= " << Lib::sout(ID) 
          << " UFUNC= " << Lib::sout(UFUNC) 
          << " NCX= " << NCX
          << " XMIN= " << XMIN
          << " XMAX= " << XMAX
          << " NCY= " << NCY
          << " YMIN= " << YMIN
          << " YMAX= " << YMAX
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    if((NCX<=0)||(XMAX<=XMIN)) {
      out << "pafunc :" 
          << " bad booking values."
          << Lib::endl;
      return;
    }
    if((NCY<=0)||(YMAX<=YMIN)) {
      out << "pafunc :" 
          << " bad booking values."
          << Lib::endl;
      return;
    }

    if(memoryTree->find(ID)) {
      memoryTree->rm(ID); //ok.
      out << " *** WARNING in OpenPAW :"
          << " Already existing histogram replaced : ID="
          << ID 
          << Lib::endl;
    }
    AIDA::IHistogram2D* histogram = 
      create_histogram2D(*aida,*memoryTree,ID,UFUNC,
                         NCX,XMIN,XMAX,NCY,YMIN,YMAX);
    if(!histogram) {
      out << "pafunc :" 
            << " can't create histogram " << Lib::sout(ID) << Lib::endl;
      return;
    }

    double dx = (XMAX - XMIN)/NCX;
    double dy = (YMAX - YMIN)/NCY;

    std::string funcName = pawManager->generateFunctionName();

    char* path = ku_macro_path();
    std::string paths = path?path:"";

    AIDA::IFunction* func = create_function_vector
      (funcName,paths,UFUNC,*session,*aida,*memoryTree,false);

    if(!func) {
      out << "pafunc :" 
          << " can't create function " << Lib::sout(UFUNC) 
          << Lib::endl;
      return;
    }

    AIDA::IAnnotation& annotation = histogram->annotation();
    annotation.addItem("Codelet",func->codeletString());

    double epsilon = 0;

    std::vector<double> args(2);
    for(int ibin = 0;ibin<NCX;ibin++) {
      double x = XMIN + (ibin + 0.5 + epsilon) * dx;
      args[0] = x;
      for(int jbin = 0;jbin<NCY;jbin++) {
        double y = YMIN + (jbin + 0.5 + epsilon) * dy;
        args[1] = y;
        double value = func->value(args);
        histogram->fill(x,y,value);
      }
    }
    delete func;

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);
      
      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      bool empty = true;
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"SURF")) {
        region_plot_histogram2D
          (*session,region,*histogram,"SURF",printer,0,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"LEGO")) {
        region_plot_histogram2D
          (*session,region,*histogram,"LEGO",printer,0,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"TEXT")) {
        region_plot_histogram2D
          (*session,region,*histogram,"TEXT",printer,0,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"CONT")) {
        region_plot_histogram2D
          (*session,region,*histogram,"CONT_DMOD",printer,10,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"BOX")) {
        region_plot_histogram2D
          (*session,region,*histogram,"BOX",printer,0,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::hasUnkownOption(OPTS,CHOPT)) {
        region_plot_histogram2D
          (*session,region,*histogram,"",printer,0,*pawManager);
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty) {
        // Do nothing.
      }

      delete plotter;
    }

  } else if(cmd_path=="/FUNCTION/FUN1") {

    std::string ID = ku_gets();
    std::string UFUNC = ku_gets();
    int NCX = ku_geti();
    double XMIN = ku_getr();
    double XMAX = ku_getr();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "pafunc :" 
          << " " << cmd_path
          << " ID= " << Lib::sout(ID) 
          << " UFUNC= " << Lib::sout(UFUNC) 
          << " NCX= " << NCX
          << " XMIN= " << XMIN
          << " XMAX= " << XMAX
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    if((NCX<=0)||(XMAX<=XMIN)) {
      out << "pafunc :" 
          << " bad booking values."
          << Lib::endl;
      return;
    }

    if(memoryTree->find(ID)) {
      memoryTree->rm(ID); //ok.
      out << " *** WARNING in OpenPAW :"
          << " Already existing histogram replaced : ID="
          << ID 
          << Lib::endl;
    }

    AIDA::IHistogram1D* histogram = 
      create_histogram1D(*aida,*memoryTree,ID,UFUNC,NCX,XMIN,XMAX);
    if(!histogram) {
      out << "pafunc :" 
            << " can't create histogram " << Lib::sout(ID) << Lib::endl;
      return;
    }

    double dx = (XMAX - XMIN)/NCX;

    std::string funcName = pawManager->generateFunctionName();

    char* path = ku_macro_path();
    std::string paths = path?path:"";

    AIDA::IFunction* func = create_function_vector
      (funcName,paths,UFUNC,*session,*aida,*memoryTree,false);

    if(!func) {
      out << "pafunc :" 
          << " can't create function " << Lib::sout(UFUNC) 
          << Lib::endl;
      return;
    }

    AIDA::IAnnotation& annotation = histogram->annotation();
    annotation.addItem("Codelet",func->codeletString());

    //double epsilon = 0.001; // Trick to avoid zero.
    double epsilon = 0;

    std::vector<double> args(1);
    for(int ibin = 0;ibin<NCX;ibin++) {
      double x = XMIN + (ibin + 0.5 + epsilon) * dx;
      args[0] = x;
      double value = func->value(args);
      histogram->fill(x,value);
    }
    delete func;

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      bool empty = true;
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C")) {
        pawManager->plotHistogram1D(region,*histogram,"C",false); 
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"E")) {
        pawManager->plotHistogram1D(region,*histogram,"E",false); 
	empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty) {
        // Do nothing.
        //FIXME : ?
        //func->setTitle(UFUNC);      
        //pawManager->plotFunction1D(region,*func,XMIN,XMAX,"C");
      }

      delete plotter;
    }

  } else if(cmd_path=="/FUNCTION/POINTS") {

    int NPX = ku_geti();
    int NPY = ku_geti();
    int NPZ = ku_geti();
    pawManager->setFunctionPoints(NPX,NPY,NPZ);

  } else {
    out << "pafunc :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
