//
// Contain HEPVis and Inventor related code.
//


extern "C" {
  void pagpri_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Slash/UI/IUI.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>

#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/UI.h>
#include <OnX/Helpers/Inventor.h>

#include <BatchLab/Helpers/AIDA.h>
#include "ArrayHelpers.h"

#include <OpenPAW/PAW_Manager.h>
#include "KuipHelpers.h"

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/engines/SoCalculator.h>

#include <HEPVis/nodes/SoTextHershey.h>
#include <HEPVis/nodekits/SoPage.h>
#include <HEPVis/nodekits/SoAxis.h>
#include <HEPVis/nodekits/SoPlotterRegion.h>
#include <HEPVis/nodekits/SoPlotter.h>

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
void pagpri_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  //std::string cmd_name = ku_name();
  std::string cmd_path = ku_path() ? ku_path() : "";
  if(session->verboseLevel()) {
    out << "pagpri :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  //OPTs_type& opts = pawManager->opts();
  ATTs_type& atts = pawManager->atts();

  //double LWID = atts.value("LWID"); 
  //LWID = MAXIMUM(int(LWID/2.2),1);  //From graflib/higz/ikernel/islwsc.F

  if(cmd_path=="/GRAPHICS/PRIMITIVES/GRAPH") {

    int N = ku_geti();
    std::string X = ku_gets();
    std::string Y = ku_gets();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);
    if(session->verboseLevel()) {
      out << "pagpri :" 
          << " CHOPT " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    std::string XNAME;
    std::vector< std::pair<int,int> > xdims;
    if(!OpenPAW::PAW_Manager::parseName(X,XNAME,xdims,printer)) return;
    Lib::smanip::touppercase(XNAME);
    Slash::Data::IArray* xvec = find_vector(*memoryTree,XNAME);
    if(!xvec) {
      out << "pagpri :" 
          << " can't find vector " << Lib::sout(XNAME) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return;
    }
    std::vector< std::pair<unsigned int,unsigned int> > xcut;
    if(!vector_get_cut(*xvec,xdims,xcut)) {
      out << "pagpri :" 
          << " mismatch dimension in " << Lib::sout(XNAME) 
          << Lib::endl;
      return;
    }

    std::string YNAME;
    std::vector< std::pair<int,int> > ydims;
    if(!OpenPAW::PAW_Manager::parseName(Y,YNAME,ydims,printer)) return;
    Lib::smanip::touppercase(YNAME);
    Slash::Data::IArray* yvec = find_vector(*memoryTree,YNAME);
    if(!yvec) {
      out << "pagpri :" 
          << " can't find vector " << Lib::sout(YNAME) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return;
    }
    std::vector< std::pair<unsigned int,unsigned int> > ycut;
    if(!vector_get_cut(*yvec,ydims,ycut)) {
      out << "pagpri :" 
          << " mismatch dimension in " << Lib::sout(YNAME) 
          << Lib::endl;
      return;
    }

    //std::string name = VNAME + "_rep";
    int xsize = xvec->size();
    int ysize = yvec->size();
    int number = MINIMUM(xsize,N);
    number = MINIMUM(number,ysize);

    // create temporary not managed cloud :
    std::string name = "cloud2D_" + X + "_" + Y;
    AIDA::ICloud2D* cloud = 
      create_cloud2D(*aida,*memoryTree,path_not_managed()+"/"+name,"");
    if(!cloud) {
      out << "pagpri :" 
          << " can't create a cloud2D." << Lib::endl;
      return;
    }

    const std::vector<double>& xv = xvec->vector();
    const std::vector<double>& yv = yvec->vector();
    for(int index=0;index<number;index++) cloud->fill(xv[index],yv[index]);

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      //If A (axes) are required we go in the next region.
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"A")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C"))
        pawManager->plotCloud2D(region,*cloud,"C");
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"L"))
        pawManager->plotCloud2D(region,*cloud,"L");
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P"))
        pawManager->plotCloud2D(region,*cloud,"P");

      delete plotter;
    }

    // Plotter is in retain mode, we can now get rid of the object.
    delete cloud;

  } else if(cmd_path=="/GRAPHICS/PRIMITIVES/TEXT") {

    double X = ku_getr();
    double Y = ku_getr();
    std::string TEXT = ku_gets();
    double SIZE = ku_getr();
    double ANGLE = ku_getr();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    Slash::UI::IUI* ui = Lib_findUI(*session);
    if(!ui) return;
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
    SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
    SoPlotter* soPlotter = soPlotterRegion->getPlotter();

    SoSeparator* etc = soPlotter->getEtcSeparator();
    if(!etc) return;

    SoSeparator* sep = new SoSeparator;

    SoDrawStyle* soDrawStyle = new SoDrawStyle;
    soDrawStyle->style.setValue(SoDrawStyle::LINES);
    soDrawStyle->linePattern.setValue(0xFFFF);
    soDrawStyle->lineWidth.setValue(1);
    sep->addChild(soDrawStyle);

    SoMaterial* soMaterial = new SoMaterial;
    soMaterial->diffuseColor.setValue(SbColor(0,0,0));
    sep->addChild(soMaterial);

    //FIXME : works only if SoPlotter::shape XY
    //FIXME : avoid SoCalculator ?
   {SoCalculator* soCalculator = new SoCalculator;
    SoAxis* soAxis = soPlotter->getXAxis();
    soCalculator->a.connectFrom(&(soAxis->minimumValue));
    soCalculator->b.connectFrom(&(soAxis->maximumValue));
    soCalculator->c.connectFrom(&(soAxis->width));
    std::string s;
    Lib::smanip::printf(s,128,"oA=vec3f(c*(%g-a)/(b-a),0,0)",X);
    soCalculator->expression.setValue(s.c_str());

    SoTranslation* soTranslation = new SoTranslation;
    soTranslation->translation.connectFrom(&(soCalculator->oA));
    sep->addChild(soTranslation);}

   {SoCalculator* soCalculator = new SoCalculator;
    SoAxis* soAxis = soPlotter->getYAxis();
    soCalculator->a.connectFrom(&(soAxis->minimumValue));
    soCalculator->b.connectFrom(&(soAxis->maximumValue));
    soCalculator->c.connectFrom(&(soAxis->width));
    std::string s;
    Lib::smanip::printf(s,128,"oA=vec3f(0,c*(%g-a)/(b-a),0)",Y);
    soCalculator->expression.setValue(s.c_str());

    SoTranslation* soTranslation = new SoTranslation;
    soTranslation->translation.connectFrom(&(soCalculator->oA));
    sep->addChild(soTranslation);}

    ANGLE *= M_PI / 180;
    SoRotationXYZ* soRotation = new SoRotationXYZ;
    soRotation->axis.setValue(SoRotationXYZ::Z);
    soRotation->angle.setValue((float)ANGLE);
    sep->addChild(soRotation);

    //SIZE is in page coordinate YSIZ.

    SoTextHershey* soTextHershey = new SoTextHershey;
    soTextHershey->height.setValue(float(SIZE));
    soTextHershey->encoding.setValue("PAW");
    soTextHershey->string.setValue(TEXT.c_str());
    sep->addChild(soTextHershey);

    etc->addChild(sep);

  } else if(cmd_path=="/GRAPHICS/PRIMITIVES/ITX") {

    double X = ku_getr();
    double Y = ku_getr();
    std::string TEXT = ku_gets();

    Slash::UI::IUI* ui = Lib_findUI(*session);
    if(!ui) return;
    SoPage* soPage = ui_SoPage(*ui);
    if(!soPage) return;
    SoRegion* soRegion = soPage->currentRegion();
    if(!soRegion) return;
    if(!soRegion->isOfType(SoPlotterRegion::getClassTypeId())) return;
    SoPlotterRegion* soPlotterRegion = (SoPlotterRegion*)soRegion;
    SoPlotter* soPlotter = soPlotterRegion->getPlotter();

    SoSeparator* etc = soPlotter->getEtcSeparator();
    if(!etc) return;

    SoSeparator* sep = new SoSeparator;

    SoDrawStyle* soDrawStyle = new SoDrawStyle;
    soDrawStyle->style.setValue(SoDrawStyle::LINES);
    soDrawStyle->linePattern.setValue(0xFFFF);
    soDrawStyle->lineWidth.setValue(1);
    sep->addChild(soDrawStyle);

    SoMaterial* soMaterial = new SoMaterial;
    soMaterial->diffuseColor.setValue(SbColor(0,0,0));
    sep->addChild(soMaterial);

    //FIXME : works only if SoPlotter::shape XY
    //FIXME : avoid SoCalculator ?
   {SoCalculator* soCalculator = new SoCalculator;
    SoAxis* soAxis = soPlotter->getXAxis();
    soCalculator->a.connectFrom(&(soAxis->minimumValue));
    soCalculator->b.connectFrom(&(soAxis->maximumValue));
    soCalculator->c.connectFrom(&(soAxis->width));
    std::string s;
    Lib::smanip::printf(s,128,"oA=vec3f(c*(%g-a)/(b-a),0,0)",X);
    soCalculator->expression.setValue(s.c_str());

    SoTranslation* soTranslation = new SoTranslation;
    soTranslation->translation.connectFrom(&(soCalculator->oA));
    sep->addChild(soTranslation);}

   {SoCalculator* soCalculator = new SoCalculator;
    SoAxis* soAxis = soPlotter->getYAxis();
    soCalculator->a.connectFrom(&(soAxis->minimumValue));
    soCalculator->b.connectFrom(&(soAxis->maximumValue));
    soCalculator->c.connectFrom(&(soAxis->width));
    std::string s;
    Lib::smanip::printf(s,128,"oA=vec3f(0,c*(%g-a)/(b-a),0)",Y);
    soCalculator->expression.setValue(s.c_str());

    SoTranslation* soTranslation = new SoTranslation;
    soTranslation->translation.connectFrom(&(soCalculator->oA));
    sep->addChild(soTranslation);}

    double SIZE = atts.value("CHHE"); 

    //SIZE is in page coordinate YSIZ.

    SoTextHershey* soTextHershey = new SoTextHershey;
    soTextHershey->height.setValue(float(SIZE));
    soTextHershey->encoding.setValue("PAW");
    soTextHershey->string.setValue(TEXT.c_str());
    sep->addChild(soTextHershey);

    etc->addChild(sep);

  } else {
    out << "pagpri :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
