extern "C" {
  void panntu_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Lib/sjust.h>
#include <Lib/Time.h>

#include <BatchLab/Core/amanip.h>
#include <BatchLab/Core/T_amanip.h>

#include "KuipHelpers.h"
#include "FindHelpers.h"
#include "PlotHistogram2D.h"

#define NotFound (-1)

//#define SHOW_TIME

//////////////////////////////////////////////////////////////////////////////
void panntu_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "panntu :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  OPTs_type& opts = pawManager->opts();

  if(cmd_path=="/NTUPLE/PROJECT") {

    std::string IDH = ku_gets();
    std::string IDN = ku_gets();
    std::string UWFUNC = ku_gets();
    //int NEVENT = 
    ku_geti();
    //int IFIRST =
    ku_geti();
 
    std::string PATH;
    std::vector<std::string> vars;
    if(!pawManager->parseTupleName(IDN,PATH,vars)) return;

    AIDA::ITree* tree = 0;
    AIDA::ITuple* tuple = tuple_find(*session,PATH,tree);
    if(!tuple) {
      out << "panntu :" 
          << " can't find tuple " << Lib::sout(PATH) << "."
          << Lib::endl;
      return;
    }

    std::string sfilter;
    if(!pawManager->filterCuts(UWFUNC,sfilter)) {
      out << "panntu :" 
          << " problem treating " << Lib::sout(UWFUNC) 
          << Lib::endl;
      return;
    }
    if(sfilter=="1.") sfilter = "";

    if(vars.size()==1) {

      AIDA::IHistogram1D* histogram = histogram1D_find(*session,IDH);
      if(!histogram) {
        out << "panntu :" 
            << " can't find histogram " << Lib::sout(IDH) 
            << Lib::endl;
        return;
      }

      AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
      if(tupleFactory) {
        AIDA::IEvaluator* evalX = tupleFactory->createEvaluator(vars[0]);
        if(evalX) {
          AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
          if(filter) {
            tuple->project(*histogram,*evalX,*filter);
            delete filter;
          }
          delete evalX;
        }
        delete tupleFactory;
      }

    } else if(vars.size()==2) {
      AIDA::IHistogram2D* histogram = histogram2D_find(*session,IDH);
      if(!histogram) {
        out << "panntu :" 
            << " can't find histogram " << Lib::sout(IDH) 
            << Lib::endl;
        return;
      }

      AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree); 
      if(tupleFactory) {
        AIDA::IEvaluator* evalX = tupleFactory->createEvaluator(vars[1]);
        if(evalX) {
          AIDA::IEvaluator* evalY = tupleFactory->createEvaluator(vars[0]);
          if(evalY) {
            AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
            if(filter) {
              tuple->project(*histogram,*evalX,*evalY,*filter);
              delete filter;
            }
            delete evalY;
          }
          delete evalX;
        }
        delete tupleFactory;
      }
    }

  } else if(cmd_path=="/NTUPLE/PLOT") {

    std::string IDN = ku_gets();
    std::string UWFUNC = ku_gets();
    int NEVENT = ku_geti();
    int IFIRST = ku_geti();
    int NUPD = ku_geti();
    std::string OPTION = ku_gets();
    int IDH = ku_geti();

    if(session->verboseLevel()) {
      out << "panntu :" 
          << " " << cmd_path
          << " IDN= " << Lib::sout(IDN)
          << " UWFUNC= " << Lib::sout(UWFUNC)
          << " NEVENT= " << NEVENT
          << " IFIRST= " << IFIRST
          << " NUPD= " << NUPD
          << " OPTION= " << Lib::sout(OPTION)
          << " IDH= " << IDH
          << Lib::endl;
    }

    if(NEVENT<0) IDH = -NEVENT;
    if(IFIRST<0) IDH = -IFIRST;
    if(NUPD<0) IDH = -NUPD;

    Lib::smanip::touppercase(OPTION);
    std::vector<std::string> OPTS;
    kuip_getOptions("OPTION",OPTS);
    std::string CHOPT = OPTION;

    std::string PATH;
    std::vector<std::string> vars;
    if(!pawManager->parseTupleName(IDN,PATH,vars)) return;

    AIDA::ITree* tree = 0;
    AIDA::ITuple* tuple = tuple_find(*session,PATH,tree);
    if(!tuple) {
      out << "panntu :" 
          << " can't find tuple " << Lib::sout(PATH) << "."
          << Lib::endl;
      return;
    }

    std::string sfilter;
    if(!pawManager->filterCuts(UWFUNC,sfilter)) {
      out << "panntu :" 
          << " problem treating " << Lib::sout(UWFUNC) 
          << Lib::endl;
      return;
    }
    if(sfilter=="1.") sfilter = "";

    if(vars.size()==1) {

      AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree);
      AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
      AIDA::IEvaluator* evalX = tupleFactory->createEvaluator(vars[0]);
      delete tupleFactory;

      std::string ID = Lib::smanip::tostring(IDH);
      if(IDH==1000000) if(memoryTree->find(ID)) memoryTree->rm(ID); //ok.

#ifdef SHOW_TIME
      Lib::Time begin = Lib::Time::now();
#endif

      AIDA::IHistogram1D* rep = find_histogram1D(*memoryTree,ID);
      if(rep) {

        // Look if ID already plotted.
        // If so we have to create another histo.
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) {
          AIDA::IPlotter* plotter = create_plotter(*aida);
          if(plotter) {
            AIDA::IPlotterRegion& region = plotter->currentRegion();
            std::string s = 
              region.parameterValue("plotter.plottedObjectNames");
            if(s.find("\""+ID+"\"")!=std::string::npos) {
              int n;
              region_field_value(region,"numberOfPlottedObject",n,printer);
              ID = "tmp_" + ID + "_" + Lib::smanip::tostring(n);
              rep = copy_histogram1D(*aida,*memoryTree,ID,*rep);
            }
            delete plotter;
          }      
        }      

        rep->reset();

        tuple->project(*rep,*evalX,*filter);

      } else {

        // Use a temporary not managed cloud.
        // It permits to have only one pass on the tuple to get
        // min, max and get values to fill the histo.
        // The cost may be the memory consumption of a cloud...

        AIDA::ICloud1D* cloud = 
          create_cloud1D(*aida,*memoryTree,path_not_managed()+"/"+ID,"");
        if(!cloud){
          delete evalX;
          delete filter;
          return;
        }
        if(!BatchLab::T_amanip::project<AIDA::ICloud1D>
              (printer,*tuple,*cloud,*evalX,*filter)) {
          delete evalX;
          delete filter;
          return;
        }
        double xmin = cloud->lowerEdge();
        double xmax = cloud->upperEdge();
        if(xmax<xmin) {
          out << "panntu :" 
              << " xmin>xmax."
              << Lib::endl;
          delete evalX;
          delete filter;
          return;
        } else if(xmax==xmin) {
          xmax = xmin + 1;
        } else { // Tilt xmax in order that max value is in the histo :
          xmax = xmax + (xmax-xmin) * 0.01; //FIXME : try to avoid that.
        }

        std::string title = vars[0];
        rep = create_histogram1D(*aida,*memoryTree,ID,title,100,xmin,xmax);

        if(!rep) {
          out << "panntu :" 
              << " can't create histogram " << Lib::sout(ID) 
              << " in memory tree."
              << Lib::endl;
          delete cloud;
          delete evalX;
          delete filter;
          return;
        }

        cloud->fillHistogram(*rep);
        delete cloud;

      }

#ifdef SHOW_TIME
      Lib::Time end = Lib::Time::elapsed(begin);
      out << "panntu :" << " elapsed : " << end.seconds() << " s" << Lib::endl;
#endif

      if(session->verboseLevel()) {
        out << "panntu :" 
            << " projection : "
            << Lib::endl;
        BatchLab::amanip::hprint(*rep,"projection",printer);
      }

      delete evalX;
      delete filter;

      AIDA::IPlotter* plotter = create_plotter(*aida);
      if(plotter) {
        pawManager->setPlotterZONE(*plotter);
  
        if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
          pawManager->plotterNext(*plotter);
        AIDA::IPlotterRegion& region = plotter->currentRegion();
  
        pawManager->regionSetLayout(region);
  
        if(opts.value("NBAR")=="BAR") CHOPT += "B";

        bool empty = true;
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C")) {
          pawManager->plotHistogram1D(region,*rep,"C",false); 
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"L")) {
          pawManager->plotHistogram1D(region,*rep,"L",false); 
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"E")) {
          pawManager->plotHistogram1D(region,*rep,"E",false); 
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P")) {
          pawManager->plotHistogram1D(region,*rep,"P",false); 
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"*")) {
          pawManager->plotHistogram1D(region,*rep,"*",false); 
          empty = false;
        }

        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"B")) {
          pawManager->plotHistogram1D(region,*rep,"B",false); 
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty)
          pawManager->plotHistogram1D(region,*rep,"",false); 
   
        delete plotter;
      }

    } else if(vars.size()==2) {

      AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree);
      AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
      AIDA::IEvaluator* evalX = tupleFactory->createEvaluator(vars[1]);
      AIDA::IEvaluator* evalY = tupleFactory->createEvaluator(vars[0]);
      delete tupleFactory;

      std::string ID = Lib::smanip::tostring(IDH);
      if(IDH==1000000) if(memoryTree->find(ID)) memoryTree->rm(ID); //ok.

#ifdef SHOW_TIME
      Lib::Time begin = Lib::Time::now();
#endif

      AIDA::IHistogram2D* rep = find_histogram2D(*memoryTree,ID);
      if(rep) {

        // Look if ID already plotted.
        // If so we have to create another histo.
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) {
          AIDA::IPlotter* plotter = create_plotter(*aida);
          if(plotter) {
            AIDA::IPlotterRegion& region = plotter->currentRegion();
            std::string s = 
              region.parameterValue("plotter.plottedObjectNames");
            if(s.find("\""+ID+"\"")!=std::string::npos) {
              int n;
              region_field_value(region,"numberOfPlottedObject",n,printer);
              ID = "tmp_" + ID + "_" + Lib::smanip::tostring(n);
              rep = copy_histogram2D(*aida,*memoryTree,ID,*rep);
            }
            delete plotter;
          }      
        }      

        rep->reset();

        tuple->project(*rep,*evalX,*evalY,*filter);

      } else {

        // Use a temporary not managed cloud.
        // It permits to have only one pass on the tuple to get
        // min, max and get values to fill the histo.
        // The cost may be the memory consumption of a cloud...

        AIDA::ICloud2D* cloud = 
          create_cloud2D(*aida,*memoryTree,path_not_managed()+"/"+ID,"");
        if(!cloud){
          delete evalX;
          delete evalY;
          delete filter;
          return;
        }
        if(!BatchLab::T_amanip::project<AIDA::ICloud2D>
              (printer,*tuple,*cloud,*evalX,*evalY,*filter)) {
          delete evalX;
          delete evalY;
          delete filter;
          return;
        }

        double xmin = cloud->lowerEdgeX();
        double xmax = cloud->upperEdgeX();
        if(xmax<xmin) {
          out << "panntu :" 
              << " xmin>xmax."
              << Lib::endl;
          delete evalX;
          delete evalY;
          delete filter;
          return;
        } else if(xmax==xmin) {
          xmax = xmin + 1;
        } else { // Tilt xmax in order that max value is in the histo :
          xmax = xmax + (xmax-xmin) * 0.01; //FIXME : try to avoid that.
         }

        double ymin = cloud->lowerEdgeY();
        double ymax = cloud->upperEdgeY();
        if(ymax<ymin) {
          out << "panntu :" 
              << " ymin>ymax."
              << Lib::endl;
          delete evalX;
          delete evalY;
          delete filter;
          return;
        } else if(ymax==ymin) {
          ymax = ymin + 1;
        } else { // Tilt ymax in order that max value is in the histo :
          ymax = ymax + (ymax-ymin) * 0.01; //FIXME : try to avoid that.
        }
        std::string title = vars[0]+" VS. "+vars[1];
        rep = create_histogram2D(*aida,*memoryTree,ID,title,
                                 100,xmin,xmax,100,ymin,ymax);

        if(!rep) {
          out << "panntu :" 
              << " can't create histogram " << Lib::sout(ID) 
              << " in memory tree."
              << Lib::endl;
          delete cloud;
          delete evalX;
          delete evalY;
          delete filter;
          return;
        }

        cloud->fillHistogram(*rep);
        delete cloud;

      }

#ifdef SHOW_TIME
      Lib::Time end = Lib::Time::elapsed(begin);
      out << "panntu :" << " elapsed : " << end.seconds() << " s" << Lib::endl;
#endif

      delete evalX;
      delete evalY;
      delete filter;

      AIDA::IPlotter* plotter = create_plotter(*aida);
      if(plotter) {
        pawManager->setPlotterZONE(*plotter);
  
        if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
          pawManager->plotterNext(*plotter);
        AIDA::IPlotterRegion& region = plotter->currentRegion();
  
        pawManager->regionSetLayout(region);

        bool empty = true;
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"SURF")) {
          region_plot_histogram2D
            (*session,region,*rep,"SURF",printer,0,*pawManager);
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"LEGO")) {
          region_plot_histogram2D(*session,region,*rep,"LEGO",printer,0,*pawManager);
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"TEXT")) {
          region_plot_histogram2D(*session,region,*rep,"TEXT",printer,0,*pawManager);
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"CONT")) {
          region_plot_histogram2D(*session,region,*rep,"CONT_DMOD",printer,10,*pawManager);
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"BOX")) {
          region_plot_histogram2D(*session,region,*rep,"BOX",printer,0,*pawManager);
          empty = false;
        }
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty) {
          region_plot_histogram2D(*session,region,*rep,"",printer,0,*pawManager); 
        }

        delete plotter;
      }

    }

  } else if(cmd_path=="/NTUPLE/SCAN") {

    std::string IDN = ku_gets();
    std::string UWFUNC = ku_gets();
    int NEVENT = ku_geti();
    int IFIRST = ku_geti();
    std::string OPTION = ku_gets();
    std::string VARLIST = ku_gets();

    if(session->verboseLevel()) {
      out << "panntu :" 
          << " " << cmd_path
          << " IDN= " << Lib::sout(IDN)
          << " UWFUNC= " << Lib::sout(UWFUNC)
          << " NEVENT= " << NEVENT
          << " IFIRST= " << IFIRST
          << " OPTION= " << Lib::sout(OPTION)
          << " VARLIST = " << Lib::sout(VARLIST)
          << Lib::endl;
    }

    Lib::smanip::touppercase(OPTION);
    std::vector<std::string> OPTS;
    kuip_getOptions("OPTION",OPTS);
    std::string CHOPT = OPTION;

    AIDA::ITree* tree = 0;
    AIDA::ITuple* tuple = tuple_find(*session,IDN,tree);
    if(!tuple) {
      out << "panntu :" 
          << " can't find tuple " << Lib::sout(IDN) << "."
          << Lib::endl;
      return;
    }

    std::vector<std::string> vars;
    Lib::smanip::words(VARLIST,",",vars);
    unsigned int vari;
    unsigned int varn = vars.size();
    std::vector<int> varis(varn);
    for(vari=0;vari<varn;vari++) {
      int coli = tuple->findColumn(vars[vari]);
      if(coli==NotFound) {
        out << "panntu :" 
            << " variable " << Lib::sout(vars[vari]) 
            << " not a column of tuple " << Lib::sout(IDN) 
            << Lib::endl;
        return;
      }
      varis[vari] = coli;
    }

    std::string sfilter;
    if(!pawManager->filterCuts(UWFUNC,sfilter)) {
      out << "panntu :" 
          << " problem treating " << Lib::sout(UWFUNC) 
          << Lib::endl;
      return;
    }
    if(sfilter=="1.") sfilter = "";

    AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree);
    AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
    delete tupleFactory;
    if(!filter->initialize(*tuple)) {
      out << "panntu :"
          << " Filter initialization failed." 
          << Lib::endl;
      delete filter;
      return;
    }

    out << "+-------+";
    for(vari=0;vari<varn;vari++) out << "--------------+";
    out << Lib::endl;

    out << "| Event |";
    for(vari=0;vari<varn;vari++)
      out << Lib::sjust(tuple->columnName(varis[vari]),14,Lib::smanip::middle) << "|";
    out << Lib::endl;
 
    out << "+-------+";
    for(vari=0;vari<varn;vari++) out << "--------------+";
    out << Lib::endl;

    int eventn = 0; 

    int rowi = 1; 
    tuple->start();
    while(tuple->next()) {
      if(rowi>=IFIRST) {
        if((NEVENT!=99999999)&&(rowi>NEVENT)) break;
        if(filter->accept()) {
          out << "|" << Lib::sjust(rowi,7,Lib::smanip::middle) << "|";
          for(vari=0;vari<varn;vari++) {
            int coli = varis[vari];
            std::string type = tuple->columnType(coli);
            if(type=="float") {
              out << Lib::sjust(tuple->getFloat(coli),14,Lib::smanip::middle);
            } else if(type=="double") {
              out << Lib::sjust(tuple->getDouble(coli),14,Lib::smanip::middle);
            } else if(type=="char") {
              out << Lib::sjust(tuple->getChar(coli),14,Lib::smanip::middle);
            } else if(type=="short") {
              out << Lib::sjust(tuple->getShort(coli),14,Lib::smanip::middle);
            } else if(type=="int") {
              out << Lib::sjust(tuple->getInt(coli),14,Lib::smanip::middle);
	    } else if(type=="long") {
              out << Lib::sjust((Lib::int64)(tuple->getLong(coli)),14,Lib::smanip::middle);
            } else if(type=="boolean") {
             out << Lib::sjust(tuple->getBoolean(coli),14,Lib::smanip::middle);
            } else if(type=="string") {
              out << Lib::sjust(tuple->getString(coli),14,Lib::smanip::middle);
            } else if(type=="ITuple") {
              out << Lib::sjust(std::string("object"),14,Lib::smanip::middle);
            } else {
              out << Lib::sjust(std::string("none"),14,Lib::smanip::middle);
            }
            out << "|";
          }
          out << Lib::endl;
          eventn++;
        }
      }
      rowi++;
    }

    out << "+-------+";
    for(vari=0;vari<varn;vari++) out << "--------------+";
    out << Lib::endl;

    out << "==> " << eventn 
        << " events satisfied the imposed cuts" 
        << Lib::endl;

    delete filter;

  } else if(cmd_path=="/NTUPLE/LOOP") {

    std::string IDN = ku_gets();
    std::string UWFUNC = ku_gets();
    int NEVENT = ku_geti();
    int IFIRST = ku_geti();

    if(session->verboseLevel()) {
      out << "panntu :" 
          << " " << cmd_path
          << " IDN= " << Lib::sout(IDN)
          << " UWFUNC= " << Lib::sout(UWFUNC)
          << " NEVENT= " << NEVENT
          << " IFIRST= " << IFIRST
          << Lib::endl;
    }

    std::string mask;
    int ibit = 0;

    std::string::size_type pos = UWFUNC.find(">>");
    if(pos!=std::string::npos) {
      std::string smask = UWFUNC.substr(pos+2,UWFUNC.size()-(pos+2)); 
      std::string::size_type lpar = smask.rfind('(');
      if(lpar==std::string::npos) { //Syntax error.
        out << "panntu :" 
            << " Syntax error in " << Lib::sout(UWFUNC) 
            << Lib::endl;
        return;
      }
      mask = smask.substr(0,lpar);
      Lib::smanip::strip(mask);

      // Argument :
      std::string::size_type rpar = smask.find(')',lpar);
      if(rpar==std::string::npos) { //Syntax error
        out << "panntu :" 
            << " Syntax error in " << Lib::sout(UWFUNC) 
            << Lib::endl;
        return;
      }
      std::string sarg = smask.substr(lpar+1,rpar-lpar-1);
      if(!Lib::smanip::toint(sarg,ibit)) { //Syntax error.
        out << "panntu :" 
            << " Syntax error in " << Lib::sout(UWFUNC) 
            << Lib::endl;
        return;
      }
      if((ibit<1)||(ibit>32)) {
        out << "panntu :" 
            << " bad bit value " << ibit
            << ". Should be in [1,32]."
            << Lib::endl;
        return;
      }

      UWFUNC = UWFUNC.substr(0,pos);      
    }

    FILE* file = 0;
    if(mask!="") {
      OpenPAW::Unit u = pawManager->findUnit(mask);
      if(!u.isFILE()) {
        out << "panntu :" 
            << " mask file " << Lib::sout(mask) << " not opened."
            << Lib::endl;
        return;
      }
      file = u.fFILE;
    }

    AIDA::ITree* tree = 0;
    AIDA::ITuple* tuple = tuple_find(*session,IDN,tree);
    if(!tuple) {
      out << "panntu :" 
          << " can't find tuple " << Lib::sout(IDN) << "."
          << Lib::endl;
      return;
    }

    std::string sfilter;
    if(!pawManager->filterCuts(UWFUNC,sfilter)) {
      out << "panntu :" 
          << " problem treating " << Lib::sout(UWFUNC) 
          << Lib::endl;
      return;
    }
    if(sfilter=="1.") sfilter = "";

    AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree);
    AIDA::IFilter* filter = tupleFactory->createFilter(sfilter);
    delete tupleFactory;
    if(!filter->initialize(*tuple)) {
      out << "panntu :"
          << " Filter initialization failed." 
          << Lib::endl;
      delete filter;
      return;
    }

    int eventn = 0; 
    long end = 0L;
    if(file) {
      ::fseek(file,0L,SEEK_END);
      end = ::ftell(file);
      ::rewind(file);
    }

    int rowi = 1; 
    tuple->start();
    while(tuple->next()) {
      if(rowi>=IFIRST) {
        if((NEVENT!=99999999)&&(rowi>NEVENT)) break;
        bool value = filter->accept();
        if(value) eventn++;
        if(file) {    
          if(::ftell(file)==end) {
            unsigned int i = 0;
            char* buff = (char*)&i;
            if(value) i = (1 << (ibit-1));
            int nitem = ::fwrite(buff,4,1,file);
            if(nitem!=1) {
              out << "panntu :"
                  << " IO problem 1 : " << nitem
                  << Lib::endl;
              delete filter;
              return;
            }
            end = ::ftell(file);
          } else {
            unsigned int i = 0;
            char* buff = (char*)&i;
            int nitem = ::fread(buff,4,1,file);
            if(nitem!=1) {
              out << "panntu :"
                  << " IO problem 2 : " << nitem
                  << Lib::endl;
              delete filter;
              return;
            }
            unsigned int j = (1 << (ibit-1));
            if(value) i |= j;
            else      i &= ~j;
            if(::fseek(file,-4L, SEEK_CUR)){
              out << "panntu :"
                  << " IO problem 3." 
                  << Lib::endl;
              delete filter;
              return;
            }
            nitem = ::fwrite(buff,4,1,file);
            if(nitem!=1) {
              out << "panntu :"
                  << " IO problem 4." 
                  << Lib::endl;
              delete filter;
              return;
            }
	    ::fflush(file); //WIN32 : seems needed.
          }
        }
      }
      rowi++;
    }

    if(mask!="") {
      out << " bit " << ibit << ": "
          << Lib::sjust(eventn,9,Lib::smanip::right)
          << " " << UWFUNC
          << Lib::endl;
    }

    delete filter;

  } else {
    out << "panntu :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
