extern "C" {
  void pasigm_(void*);

  int sigma_C(const char*,void*); 
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Slash/Data/IArray.h>
#include <Slash/Store/ITree.h> //FIXME

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/Processor.h>
#include <Lib/Array.h>
#include <Lib/valuemanip.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/ArrayFactory.h> //FIXME : not AIDA.

#include "ArrayHelpers.h"
#include <OpenPAW/PAW_Manager.h>

static bool SIGMA(Slash::Core::ISession&,const std::string&);
static Lib::Array<double> IArray2Array(const Slash::Data::IArray&); 
static bool setIArray(Slash::Data::IArray&,
                      const std::vector<unsigned int>&, 
                      const std::vector<double>&); 

//////////////////////////////////////////////////////////////////////////////
int sigma_C(
 const char* aScript
,void* aData
) 
//////////////////////////////////////////////////////////////////////////////
// Called by kuip when doing $SIGMA(<exp>)
// It is used in PAW.c to initialize KUIP/SIGMA through a call to kusigm.
// It is the equivalent of CERNLIBG/pawlib/sigma/src/sigmae.F
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aData;
  //printf("debug : sigma_C : try to proceed \"%s\"...\n",aScript);
  SIGMA(*session,aScript); 
  //return 1; //Failure
  return 0; //Success
}
//////////////////////////////////////////////////////////////////////////////
void pasigm_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "pasigm :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  if(cmd_path=="/FORTRAN/SIGMA") {

    std::string EXPR;
    int argc = ku_npar();
    for(int index=0;index<argc;index++) EXPR += ku_gets();

    SIGMA(*session,EXPR); 

  } else {
    out << "pasigm :" 
        << " " << cmd_path << " : dummy." << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool SIGMA(
 Slash::Core::ISession& aSession
,const std::string& aScript
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aScript=="") return true;

  Slash::Core::IWriter& printer = aSession.printer();

  OpenPAW::PAW_Manager* pawManager = find_PAW(aSession);
  if(!pawManager) return false;

  AIDA::IAnalysisFactory* aida = find_aida(aSession);
  if(!aida) return false;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return false;

  //FIXME
  Slash::Store::ITree* baseTree = 
    Lib_Cast(*memoryTree,Slash::Store::ITree);
  if(!baseTree) return false;

  std::string::size_type eq = aScript.find('=');
  if(eq==std::string::npos) {
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " no assignement in " << Lib::sout(aScript) << Lib::endl;
    return false;
  }
  
  std::string assign = aScript.substr(0,eq);
  Lib::smanip::strip(assign);
  Lib::smanip::touppercase(assign);

  std::string script = aScript.substr(eq+1,aScript.size()-(eq+1));  
  Lib::smanip::tolowercase(script);
  Lib::smanip::replace(script,"#",","); //For ARRAY(x,y#z)

  Lib::Processor processor(script,printer);
  std::vector<std::string> names;
  if(!processor.findVariables(names)) {
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " can't find variables in " << Lib::sout(aScript) << Lib::endl;
    return false;
  }
  if(!processor.compile(names)) {
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " compilation of " << Lib::sout(aScript) << " failed." << Lib::endl;
    return false;
  }

  // Get variables :
  Lib::Parameters<std::string,Lib::Value>& variables = 
    pawManager->variables();

  std::vector<Lib::Value> vals;
  std::vector<unsigned int> orders;
  bool foundOrders = false;
  unsigned int number = names.size();
  for(unsigned int count=0;count<number;count++) {
    std::string name = names[count];
    Lib::smanip::touppercase(name);
    Slash::Data::IArray* vector = find_vector(*memoryTree,name);
    if(!vector) {
      if(!variables.isKey(name)) {  // Search in sSIGMA variables :
        Lib::Out out(printer);
        out << "SIGMA :" 
            << " can't find variable " << Lib::sout(name) << Lib::endl;
        return false;
      } else {
        vals.push_back(variables.value(name));
      }
    } else {
      if(!foundOrders) {
        orders = vector->orders();
        foundOrders = true;
      } else {
        if(vector->orders()!=orders) {
          Lib::Out out(printer);
          out << "SIGMA :" 
              << " orders mismatch."
              << Lib::endl;
          return false;
        }
      }
      Lib::Array<double> arr = IArray2Array(*vector);
      vals.push_back(Lib::Value(arr.orders(),arr.vector()));
    }
  }

 {Slash::Data::IArray* vassign = find_vector(*memoryTree,assign);
  //if(vassign) memoryTree->rm(assign);
  delete vassign;
  if(variables.isKey(assign)) variables.remove(assign);}

  Lib::Value var;
  std::string serror;
  if(!processor.execute(vals,var,serror)) {
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " execution of " << Lib::sout(aScript) << " failed." 
        << Lib::endl;
    out << serror << Lib::endl;
    return false;
  }
  if(serror==Lib_valuemanip_error_div_zero) {
    // See Lib::valuemanip::divide.
    // continue anyway. 
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " execution of " << Lib::sout(aScript) 
        << " has divide by zero errors, but continue anyway." 
        << Lib::endl;
  }

  if(var.type()==Slash::Core::IValue::DOUBLE) {

    variables.add(assign,"",var,Lib::Value(),Lib::Value()); 
  } else if(var.type()==Slash::Core::IValue::UNSIGNED_INT) {

    variables.add
      (assign,"",Lib::Value((double)var.get_unsigned_int()),
                 Lib::Value(),Lib::Value());    

  } else if(var.type()==Slash::Core::IValue::INT) {

    variables.add(assign,"",Lib::Value((double)var.get_int()),
                          Lib::Value(),Lib::Value());    

  } else if(var.type()==Slash::Core::IValue::ARRAY_DOUBLE) {

    std::vector<unsigned int> is;
    const std::vector<double>& stdv = var.get_array_double(is);

    BatchLab::ArrayFactory factory(printer,*baseTree); //FIXME
    Slash::Data::IArray* vassign = factory.create(assign,assign,is);
    if(!vassign) {
      Lib::Out out(printer);
      out << "SIGMA :" 
          << " can't create vector " << Lib::sout(assign) << Lib::endl;
      return false;
    } 
    if(!setIArray(*vassign,is,stdv)) {
      Lib::Out out(printer);
      out << "SIGMA :" 
          << " can't assign vector " << Lib::sout(assign) << Lib::endl;
      return false;
    }
  } else {
    Lib::Out out(printer);
    out << "SIGMA :" 
        << " result not a vector or a double." 
        << Lib::endl;
    return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
static Lib::Array<double> IArray2Array(
 const Slash::Data::IArray& aArray
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Array<double> vec(aArray.orders());
  std::vector<double>& stdvec = vec.vector();
  const std::vector<double>& astdv = aArray.vector();
  int sz = astdv.size();
  for(int index=0;index<sz;index++) stdvec[index] = astdv[index];
  return vec;
}
//////////////////////////////////////////////////////////////////////////////
static bool setIArray(
 Slash::Data::IArray& aArray
,const std::vector<unsigned int>& aIS
,const std::vector<double>& aVec
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aArray.orders()!=aIS) return false;
  std::vector<double>& astdv = aArray.vector();
  int sz = aVec.size();
  for(int index=0;index<sz;index++) astdv[index] = aVec[index];
  return true;
} 
