extern "C" {
  void pavect_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include "ArrayHelpers.h"

#include "KuipHelpers.h"
#include "FindHelpers.h"

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
void pavect_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  //std::string cmd_name = ku_name();
  std::string cmd_path = ku_path() ? ku_path() : "";
  if(session->verboseLevel()) {
    out << "pavect :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(cmd_path=="/VECTOR/DRAW") {

    std::string VNAME = ku_gets();
    std::string ID = ku_gets();
    std::string CHOPT = ku_gets();
    if(session->verboseLevel()) {
      out << "pavect :" 
          << " " << cmd_path
          << " VNAME= " << Lib::sout(VNAME) 
          << " ID= " << Lib::sout(ID) 
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string NAME;
    std::vector< std::pair<int,int> > dims;
    if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return;
    Lib::smanip::touppercase(NAME);
    Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
    if(!vector) {
      out << "pavect :" 
          << " can't find vector " << Lib::sout(NAME) << Lib::endl;
      return;
    } 
    std::vector< std::pair<unsigned int,unsigned int> > cut;
    if(!vector_get_cut(*vector,dims,cut)) {
      out << "pavect :" 
            << " mismatch dimension in " << Lib::sout(VNAME) 
          << Lib::endl;
      return;
    }

    const std::vector<double>& v = vector->vector();
    unsigned int vsize = v.size();

    int n = 0;
   {for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) n++;
    }}


    //FIXME : must be able to do a region.plot(Slash::Data::IArray).

    // create temporary not managed histo :
    std::string name = VNAME + "_rep";
    AIDA::IHistogram1D* rep = 
      create_histogram1D(*aida,*memoryTree,
                         path_not_managed()+"/"+name,VNAME,n,1,n+1);
    if(!rep) {
      out << "pavect :" 
            << " can't create histogram " << Lib::sout(name) 
          << " in memory tree."
          << Lib::endl;
      return;
    }

    int i = 0;
    for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        rep->fill(i+1.5,v[index]);
        i++;
      }
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      bool empty = true;
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C")) {
        pawManager->plotHistogram1D(region,*rep,"C",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"L")) {
        pawManager->plotHistogram1D(region,*rep,"L",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"E")) {
        pawManager->plotHistogram1D(region,*rep,"E",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P")) {
        pawManager->plotHistogram1D(region,*rep,"P",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"*")) {
        pawManager->plotHistogram1D(region,*rep,"*",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"B")) {
        pawManager->plotHistogram1D(region,*rep,"B",true); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty)
        pawManager->plotHistogram1D(region,*rep,"",true); 

      delete plotter;
    }

    // Plotter is in retain mode, we can now get rid of the rep.
    delete rep;

  } else if(cmd_path=="/VECTOR/PLOT") {

    std::string VNAME = ku_gets();
    std::string ID = ku_gets();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "pavect :" 
          << " " << cmd_path
          << " VNAME= " << Lib::sout(VNAME) 
          << " ID= " << Lib::sout(ID) 
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string NAME;
    std::vector< std::pair<int,int> > dims;
    if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return;
    Lib::smanip::touppercase(NAME);
    Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
    if(!vector) {
      out << "pavect :" 
          << " can't find vector " << Lib::sout(NAME) << Lib::endl;
      return;
    } 
    std::vector< std::pair<unsigned int,unsigned int> > cut;
    if(!vector_get_cut(*vector,dims,cut)) {
      out << "pavect :" 
            << " mismatch dimension in " << Lib::sout(VNAME) 
          << Lib::endl;
      return;
    }

    const std::vector<double>& v = vector->vector();
    unsigned int vsize = v.size();

    double xmn = 0;
    double xmx = 0;
    int n = 0;
    double value;
   {for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        value = v[index];
        if(n==0) {
          xmx = xmn = value;
        } else {
          xmn = MINIMUM(xmn,value);
          xmx = MAXIMUM(xmx,value);
        }
        n++;
      }
    }}

    if(n<=0) {
      out << "pavect :" 
            << " vector selection is empty." << Lib::sout(VNAME) 
          << Lib::endl;
      return;
    }

    if(xmx==xmn) {
      double value = xmn;
      xmn = value - 1;
      xmx = value + 1;
    } else { 
      double dx = 0.1 * (xmx - xmn);
      xmn -= dx;
      xmx += dx;
    }
     
    if(memoryTree->find(ID)) memoryTree->rm(ID); //ok.
    AIDA::IHistogram1D* rep = 
      create_histogram1D(*aida,*memoryTree,ID,VNAME,100,xmn,xmx);
    if(!rep) {
      out << "pavect :" 
            << " can't create histogram " << Lib::sout(ID) 
          << " in memory tree."
          << Lib::endl;
      return;
    }

    for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        rep->fill(v[index]);
      }
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      bool empty = true;
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C")) {
        pawManager->plotHistogram1D(region,*rep,"C",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"L")) {
        pawManager->plotHistogram1D(region,*rep,"L",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"E")) {
        pawManager->plotHistogram1D(region,*rep,"E",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P")) {
        pawManager->plotHistogram1D(region,*rep,"P",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"*")) {
        pawManager->plotHistogram1D(region,*rep,"*",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"B")) {
        pawManager->plotHistogram1D(region,*rep,"B",false); 
        empty = false;
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty)
        pawManager->plotHistogram1D(region,*rep,"",false); 
 
      delete plotter;
    }

  } else if(cmd_path=="/VECTOR/HFILL") {

    std::string VNAME = ku_gets();
    std::string ID = ku_gets();

    std::string NAME;
    std::vector< std::pair<int,int> > dims;
    if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return;
    Lib::smanip::touppercase(NAME);
    Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
    if(!vector) {
      out << "pavect :" 
          << " can't find vector " << Lib::sout(NAME) << Lib::endl;
      return;
    } 
    std::vector< std::pair<unsigned int,unsigned int> > cut;
    if(!vector_get_cut(*vector,dims,cut)) {
      out << "pavect :" 
            << " mismatch dimension in " << Lib::sout(VNAME) 
          << Lib::endl;
      return;
    }

    AIDA::IBaseHistogram* histogram = baseHistogram_find(*session,ID);
    if(!histogram) {
      out << "pavect :" 
          << " can't find histogram " << Lib::sout(ID) 
          << Lib::endl;
      return;
    }

    if(histogram->dimension()!=1) {
      out << "pavect :" 
          << " histogram of dimension " << histogram->dimension()
          << " not yet supported."
          << Lib::endl;
      return;
    }

    
    AIDA::IHistogram1D* histogram1D = Lib_Cast(*histogram,AIDA::IHistogram1D);

    const std::vector<double>& v = vector->vector();
    unsigned int vsize = v.size();

    for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        histogram1D->fill(v[index]);
      }
    }

  } else {
    out << "pavect :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
