extern "C" {
  void pazrz_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>

#include <BatchLab/Helpers/AIDA.h>

#include <OpenPAW/PAW_Manager.h>
#include "KuipHelpers.h"
#include "FindHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void pazrz_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "pazrz :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  if(cmd_path=="/ZEBRA/RZ/CDIR") {

    std::string CHPATH = ku_gets();
    std::string CHOPT = ku_gets();

    //Lib::smanip::touppercase(CHOPT);
    //std::vector<std::string> OPTS;
    //kuip_getOptions("CHOPT",OPTS);

    Units_type& units = pawManager->units();

    if(CHPATH=="") { //pwd
      AIDA::ITree* tree = pawManager->findCurrentTree();
      if(!tree) {
        out << " Not current tree."
            << Lib::endl;
        return;
      }
      std::string dir = tree->pwd();
      if(dir=="/") dir = "";
      if(pawManager->currentTree()=="MemoryTree") {
        out << " Current Working Directory = //PAWC" << dir
            << Lib::endl;
      } else {
        //Find unit : 
        int unit = units.key(OpenPAW::Unit(OpenPAW::Unit::TREE,
                                           pawManager->currentTree()));
        if(unit==units.keyNotFound()) {
          out << " Current tree unit not found."
              << Lib::endl;
          return;
        }
        out << " Current Working Directory = //LUN" << unit
            << dir
            << Lib::endl;
      }
      return;
    } 

    bool is_pawc;
    std::string path;
    if(!is_pawc_path(CHPATH,is_pawc,path)) {
      out << "pazrz :" 
          << " syntax error in " << Lib::sout(CHPATH) 
          << Lib::endl;
      return;
    }
    if(is_pawc) {
      AIDA::ITree* tree = find_memoryTree(*aida);
      if(!tree) {
        out << " No memory tree."
            << Lib::endl;
        return;
      }
      pawManager->setCurrentTree("MemoryTree");
      tree->cd(path);
      return;
    }

    bool is_unit;
    int unit;
    if(!is_unit_path(CHPATH,is_unit,path,unit)) {
      out << "pazrz :" 
          << " syntax error in " << Lib::sout(CHPATH) 
          << Lib::endl;
      return;
    }
    if(is_unit) {
      OpenPAW::Unit u = units.value(unit);
      AIDA::ITree* tree = Lib_findManager(*session,u.fString,AIDA::ITree);
      if(!tree) {
        out << "pazrz :" 
            << " tree with unit " << unit << " not found." << Lib::endl;
        return;
      }
      pawManager->setCurrentTree(u.fString);
      tree->cd(path);
      return;
    } 

    AIDA::ITree* tree = pawManager->findCurrentTree();
    if(!tree) {
      out << " No current tree."
          << Lib::endl;
      return;
    }
    //printf("debug : \"%s\"\n",tree->storeName().c_str());
    tree->cd(CHPATH);

  } else if(cmd_path=="/ZEBRA/RZ/MDIR") {

    std::string CHDIR = ku_gets();
    /*int NWKEY =*/ ku_geti();
    /*std::string CHFORM =*/ ku_gets();
    /*std::string CHTAGS =*/ ku_gets();
    AIDA::ITree* tree = pawManager->findCurrentTree();
    if(!tree) {
      out << " Not current tree."
          << Lib::endl;
      return;
    }
    tree->mkdir(CHDIR);

  } else if(cmd_path=="/ZEBRA/RZ/LDIR") {

    std::string CHPATH = ku_gets();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    bool recursive = OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"T");

    if(CHPATH=="//") { // List units :
      std::string s = "//PAWC";
      Lib::smanip::justify(s,18,Lib::smanip::left);
      out << " " << s
          << "MemoryTree"
          << Lib::endl; 

      const Lib::Parameters<int,OpenPAW::Unit>::List& lunits = 
        pawManager->units().list();
      Lib::Parameters<int,OpenPAW::Unit>::ConstIterator it;
      for(it=lunits.begin();it!=lunits.end();++it) {
        std::string s;
        Lib::smanip::printf(s,32,"//LUN%d",(*it).fKey);
        Lib::smanip::justify(s,18,Lib::smanip::left);
        out << " " << s
            << (*it).fValue.fString
            << Lib::endl; 
      }
    } else {
      AIDA::ITree* tree = pawManager->findCurrentTree();
      if(!tree) {
        out << " Not current tree."
            << Lib::endl;
        return;
      }
      tree->ls(CHPATH,recursive);
    }

  } else {
    out << "pazrz :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
