extern "C" {
  int vector_create_C(void*);
  int vector_delete_C(void*);
  int vector_read_C(void*);
  int vector_write_C(void*);
  int vector_input_C(void*);
  int vector_list_C(void*);
  int vector_operation_C(void*);
  int vector_print_C(void*);
  int vector_find_C(const char*,void*);
  int vector_value_C(const char*,double*,void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>

#include <Slash/Core/ISession.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/FortranOut.h>
#include <Lib/FileWriter.h>
#include <Lib/valuemanip.h>

#include <BatchLab/Helpers/AIDA.h>
#include <BatchLab/Core/ArrayFactory.h> //FIXME : not AIDA.

#include "ArrayHelpers.h"
#include <OpenPAW/PAW_Manager.h>

#include "KuipHelpers.h"

#define MAXIMUM(a,b) ((a)>(b)?a:b)
#define NotFound (-1)

//////////////////////////////////////////////////////////////////////////////
int vector_create_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
// From KUIP/kvect.c/kxvcreate
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_create_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;

  //FIXME
  Slash::Store::ITree* baseTree = 
    Lib_SafeCast(*memoryTree,Slash::Store::ITree);
  if(!baseTree) return -1;

  //if(cmd_path!="/VECTOR/CREATE") return;

  std::string VNAME = ku_gets();
  std::string TYPE = ku_gets();
  std::string VALUES = ku_gets();

  //printf("debug : \"%s\" \"%s\" \"%s\"\n",
  // VNAME.c_str(),TYPE.c_str(),VALUES.c_str());

  if( (TYPE!="I") && (TYPE!="i") && (TYPE!="R") && (TYPE!="r") ) {
    out << "vector_create_C :" 
        << " TYPE " << Lib::sout(TYPE) << " not supported." << Lib::endl;
    return -1;
  }

  //bool real = ((TYPE=="R") || (TYPE=="r") ? true : false);

  std::string NAME;
  std::vector< std::pair<int,int> > dims;
  if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
  Lib::smanip::touppercase(NAME);

  // Create :
  if(!dims.size()) {
    dims.push_back( std::pair<int,int>(0,0));
  }
  unsigned int dimension = dims.size();
  std::vector<unsigned int> orders(dimension);
  std::vector< std::pair<unsigned int,unsigned int> > cut;
  for(unsigned int index = 0;index<dimension;index++) {
    //FIXME : not clear if range are accepted in the CREATE command.
    //if(dims[index].first!=dims[index].second) {
    //  out << "vector_create_C :" 
    //            << " Bad format in " << Lib::sout(VNAME) 
    //      << " but continue anyway." << Lib::endl;
    //}
    orders[index] = dims[index].second+1;
    if(dims[index].second==NotFound) {
      out << "vector_create_C :" 
          << " have a NotFound max." << Lib::endl;
      return -1;
    }
    cut.push_back(std::pair<unsigned int,unsigned int>(0,dims[index].second));
  }
  BatchLab::ArrayFactory factory(printer,*baseTree); //FIXME

  if(memoryTree->find(NAME)) {
    memoryTree->rm(NAME); //ok.
    out << " *** VECTOR/CREATE " << NAME
        << ": existing vector " << NAME << " replaced"
        << Lib::endl;
  }

  Slash::Data::IArray* vector = factory.create(NAME,NAME,orders);
  if(!vector) {
    out << "vector_create_C :" 
        << " can't create vector " << Lib::sout(NAME) << "."
        << Lib::endl;
    return -1;
  }

  // Values :
  std::vector<double> values;
  if(!Lib::smanip::todoubles(VALUES,values,",")) {
    out << "vector_create_C :" 
        << " can't convert " << Lib::sout(VALUES) << " to doubles." 
        << Lib::endl;
  } else {
    vector->fill(values,&cut);
  }

  if(session->verboseLevel())
    print_vector(*vector,printer,std::string("vector ")+NAME+" :");

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_delete_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_delete_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;
 
  //if(cmd_path!="/VECTOR/DELETE") return;

  std::string VLIST = ku_gets();
  Lib::smanip::touppercase(VLIST);

  std::string::size_type star = VLIST.find('*');
  if(star==std::string::npos){
    std::string VNAME = VLIST;
    std::string NAME;
    std::vector< std::pair<int,int> > dims;
    if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
    Lib::smanip::touppercase(NAME);
    //Avoid warnings of a memoryTree->rm in case obj is not here.
    AIDA::IManagedObject* mobj = memoryTree->find(NAME);
    if(mobj) delete mobj;
  } else { //wildcard.
    std::vector<std::string> objs = memoryTree->listObjectNames();
    std::vector<std::string> typs = memoryTree->listObjectTypes();
    if(typs.size()!=objs.size()) {
      out << "vector_delete_C :" 
          << " problem retreiving directory infos."
          << Lib::endl;
      return -1;
    }
    unsigned int objn = objs.size();
    for(unsigned int index = 0;index<objn;index++) {
      std::string& name = objs[index];
      std::string& type = typs[index];
      //printf("debug : %s %s\n",name.c_str(),type.c_str());
      if(type=="Slash::Data::IArray") {
        if(Lib::smanip::match(name,VLIST)) {
          if(session->verboseLevel()) {
            out << "vector_delete_C :" 
                << Lib::sout(name) << " matches " << Lib::sout(VLIST) 
                << Lib::endl;
          }
          //Avoid warnings of a memoryTree->rm in case obj is not here.
          AIDA::IManagedObject* mobj = memoryTree->find(name);
          if(mobj) delete mobj;
        }
      }
    }
  }

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_read_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_read_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;


  //if(cmd_path!="/VECTOR/READ") return;

  std::string VLIST = ku_gets(); //FIXME : handle list of vectors.
  std::string FNAME = ku_gets();

  std::vector<double> values;
  if(!Lib::fmanip::read(FNAME,values)) {
    out << "vector_read_C :" 
        << " can't read file " << Lib::sout(FNAME) << Lib::endl;
    return -1;
  }

  if(session->verboseLevel()) {
    out << "vector_read_C :" 
        << " file " << Lib::sout(FNAME) << Lib::endl;
    for(unsigned int index=0;index<values.size();index++) {
      out << values[index] << Lib::endl;
    }
  }

  std::string VNAME = VLIST; //FIXME : handle list.
  std::string NAME;
  std::vector< std::pair<int,int> > dims;
  if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
  Lib::smanip::touppercase(NAME);
  Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
  if(!vector) {
    out << "vector_read_C :" 
        << " can't find vector " << Lib::sout(NAME) << Lib::endl;
    return -1;
  } 
  std::vector< std::pair<unsigned int,unsigned int> > cut;
  if(!vector_get_cut(*vector,dims,cut)) {
    out << "vector_read_C :" 
        << " mismatch dimension in " << Lib::sout(NAME) 
        << Lib::endl;
    return -1;
  }

  vector->fill(values,&cut);

  if(session->verboseLevel())
    print_vector(*vector,printer,std::string("vector ")+NAME+" :");

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_write_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_wite_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;


  //if(cmd_path!="/VECTOR/WRITE") return;

  std::string VLIST = ku_gets();
  std::string FNAME = ku_gets();
  std::string FORMAT = ku_gets();
  std::string CHOPT = ku_gets();

  if(session->verboseLevel()) {
    out << "vector_wite_C :" 
        << " FNAME = " << Lib::sout(FNAME) 
        << " FORMAT = " << Lib::sout(FORMAT) 
        << Lib::endl;
  }

  Lib::smanip::touppercase(CHOPT);
  std::vector<std::string> OPTS;
  kuip_getOptions("CHOPT",OPTS);
  if(session->verboseLevel()) {
    out << "vector_wite_C :" 
        << " CHOPT " << Lib::sout(CHOPT) 
        << Lib::endl;
  }

  std::string VNAME = VLIST; //FIXME : handle list.
  std::string NAME;
  std::vector< std::pair<int,int> > dims;
  if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
  Lib::smanip::touppercase(NAME);
  Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
  if(!vector) {
    out << "vector_write_C :" 
        << " can't find vector " << Lib::sout(NAME) 
        << " in tree " << Lib::sout(memoryTree->storeName()) 
        << Lib::endl;
    return -1;
  }
  std::vector< std::pair<unsigned int,unsigned int> > cut;
  if(!vector_get_cut(*vector,dims,cut)) {
    out << "vector_write_C :" 
        << " mismatch dimension in " << Lib::sout(VNAME) 
        << Lib::endl;
    return -1;
  }

  const std::vector<double>& v = vector->vector();

  std::string name = Lib::fmanip::name(FNAME);
  Lib::FileWriter fileWriter(name,printer);
  Lib::FortranOut f77out(fileWriter,"("+FORMAT+")");
  f77out << v << Lib::endl;

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_input_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_input_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;

  //if(cmd_path!="/VECTOR/INPUT") return;

  std::string VNAME = ku_gets();
  std::string VALUES = ku_gets();

  // Values :
  std::vector<double> values;
  if(!Lib::smanip::todoubles(VALUES,values,",")) {
    out << "vector_input_C :" 
        << " can't convert " << Lib::sout(VALUES) << " to doubles." 
        << Lib::endl;
    return -1;
  }
  if(session->verboseLevel()) {
    out << "vector_input_C :" 
        << " VALUES " << Lib::endl;
    for(unsigned int index=0;index<values.size();index++) {
      out << values[index] << Lib::endl;
    }
  }

  std::string NAME;
  std::vector< std::pair<int,int> > dims;
  if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
  Lib::smanip::touppercase(NAME);
  Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
  if(!vector) {
    out << "vector_input_C :" 
        << " can't find vector " << Lib::sout(NAME) << Lib::endl;
    return -1;
  } 
  std::vector< std::pair<unsigned int,unsigned int> > cut;
  if(!vector_get_cut(*vector,dims,cut)) {
    out << "vector_input_C :" 
        << " mismatch dimension in " << Lib::sout(NAME) 
        << Lib::endl;
    return -1;
  }

  vector->fill(values,&cut);

  if(session->verboseLevel())
    print_vector(*vector,printer,std::string("vector ")+NAME+" :");

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_operation_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_operation_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;

  //FIXME
  Slash::Store::ITree* baseTree = 
    Lib_SafeCast(*memoryTree,Slash::Store::ITree);
  if(!baseTree) return -1;

  BatchLab::ArrayFactory factory(printer,*baseTree);

  //cmd_path==/VECTOR/OPERATIONS/VBIAS
  //cmd_path==/VECTOR/OPERATIONS/VSCALE
  //cmd_path==/VECTOR/OPERATIONS/VADD
  //cmd_path==/VECTOR/OPERATIONS/VSUBTRACT
  //cmd_path==/VECTOR/OPERATIONS/VMULTIPLY
  //cmd_path==/VECTOR/OPERATIONS/VDIVIDE

  if( (cmd_name=="VBIAS") || (cmd_name=="VSCALE") )  {

    std::string VNAME1 = ku_gets();
    double BIAS = ku_getr();
    double SCALE = BIAS;
    std::string VNAME2 = ku_gets();

    std::string NAME1;
    std::vector< std::pair<int,int> > dims1;
    if(!OpenPAW::PAW_Manager::parseName(VNAME1,NAME1,dims1,printer)) return -1;
    Lib::smanip::touppercase(NAME1);
    Slash::Data::IArray* vec1 = find_vector(*memoryTree,NAME1);
    if(!vec1) {
      out << "vector_operation_C :" 
          << " can't find vector " << Lib::sout(NAME1) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return -1;
    }
    std::vector< std::pair<unsigned int,unsigned int> > cut1;
    if(!vector_get_cut(*vec1,dims1,cut1)) {
      out << "vector_operation_C :" 
          << " mismatch dimension in " << Lib::sout(VNAME1) 
          << Lib::endl;
      return -1;
    }

    std::string NAME2;
    std::vector< std::pair<int,int> > dims2;
    if(!OpenPAW::PAW_Manager::parseName(VNAME2,NAME2,dims2,printer)) return -1;
    Lib::smanip::touppercase(NAME2);

    if(cmd_name=="VBIAS") {
      if(NAME2==NAME1) { //operation on itself
        vec1->add(BIAS,&cut1);
      } else {
        if(memoryTree->find(NAME2)) memoryTree->rm(NAME2); //ok.
        Slash::Data::IArray* vector = factory.add(NAME2,*vec1,BIAS);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't add vector " << Lib::sout(NAME1) << "."
              << Lib::endl;
          return -1;
        }
      }

    } else if(cmd_name=="VSCALE") {  

      if(NAME2==NAME1) { //operation on itself
        vec1->multiply(SCALE);
      } else {
        if(memoryTree->find(NAME2)) memoryTree->rm(NAME2); //ok.
        Slash::Data::IArray* vector = factory.multiply(NAME2,*vec1,SCALE);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't scale vector " << Lib::sout(NAME1) << "."
              << Lib::endl;
          return -1;
        }
      }

    }
    

  } else if( (cmd_name=="VADD") || 
             (cmd_name=="VSUBTRACT") ||
             (cmd_name=="VMULTIPLY") ||
             (cmd_name=="VDIVIDE") )  {

    std::string VNAME1 = ku_gets();
    std::string VNAME2 = ku_gets();
    std::string VNAME3 = ku_gets();

    std::string NAME1;
    std::vector< std::pair<int,int> > dims1;
    if(!OpenPAW::PAW_Manager::parseName(VNAME1,NAME1,dims1,printer)) return -1;
    Lib::smanip::touppercase(NAME1);
    Slash::Data::IArray* vec1 = find_vector(*memoryTree,NAME1);
    if(!vec1) {
      out << "vector_operation_C :" 
          << " can't find vector " << Lib::sout(NAME1) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return -1;
    }
    std::vector< std::pair<unsigned int,unsigned int> > cut1;
    if(!vector_get_cut(*vec1,dims1,cut1)) {
      out << "vector_operation_C :" 
          << " mismatch dimension in " << Lib::sout(VNAME1) 
          << Lib::endl;
      return -1;
    }

    std::string NAME2;
    std::vector< std::pair<int,int> > dims2;
    if(!OpenPAW::PAW_Manager::parseName(VNAME2,NAME2,dims2,printer)) return -1;
    Lib::smanip::touppercase(NAME2);
    Slash::Data::IArray* vec2 = find_vector(*memoryTree,NAME2);
    if(!vec2) {
      out << "vector_operation_C :" 
          << " can't find vector " << Lib::sout(NAME2) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return -1;
    }
    std::vector< std::pair<unsigned int,unsigned int> > cut2;
    if(!vector_get_cut(*vec2,dims2,cut2)) {
      out << "vector_operation_C :" 
          << " mismatch dimension in " << Lib::sout(VNAME2) 
          << Lib::endl;
      return -1;
    }

    std::string NAME3;
    std::vector< std::pair<int,int> > dims3;
    if(!OpenPAW::PAW_Manager::parseName(VNAME3,NAME3,dims3,printer)) return -1;
    Lib::smanip::touppercase(NAME3);

    if(cmd_name=="VADD") {

      if(NAME3==NAME1) { //operation on itself
        // V1 + V2 = V1
        vec1->add(*vec2);
      } else if(NAME3==NAME2) { //operation on itself
        // V1 + V2 = V2
        vec2->add(*vec1);
      } else {
        if(memoryTree->find(NAME3)) memoryTree->rm(NAME3); //ok.
        Slash::Data::IArray* vector = factory.add(NAME3,*vec1,*vec2);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't add vector " 
              << Lib::sout(NAME1) << " with "
              << Lib::sout(NAME2) << "."
              << Lib::endl;
          return -1;
        }
      }

    } else if(cmd_name=="VSUBTRACT") {

      if(NAME3==NAME1) { //operation on itself
        // V1 - V2 = V1
        vec1->subtract(*vec2);
      } else if(NAME3==NAME2) { //operation on itself
        // V1 - V2 = V2
        vec2->multiply(-1);
        vec2->add(*vec1);
      } else {
        if(memoryTree->find(NAME3)) memoryTree->rm(NAME3); //ok.
        Slash::Data::IArray* vector = factory.subtract(NAME3,*vec1,*vec2);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't subtract vector " 
              << Lib::sout(NAME1) << " with "
              << Lib::sout(NAME2) << "."
              << Lib::endl;
          return -1;
        }
      }

    } else if(cmd_name=="VMULTIPLY") {

      if(NAME3==NAME1) { //operation on itself
        // V1 * V2 = V1
        vec1->multiply(*vec2);
      } else if(NAME3==NAME2) { //operation on itself
        // V1 * V2 = V2
        vec2->multiply(*vec1);
      } else {
        if(memoryTree->find(NAME3)) memoryTree->rm(NAME3); //ok.
        Slash::Data::IArray* vector = factory.multiply(NAME3,*vec1,*vec2);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't multiply vector " 
              << Lib::sout(NAME1) << " with "
              << Lib::sout(NAME2) << "."
              << Lib::endl;
          return -1;
        }
      }

    } else if(cmd_name=="VDIVIDE") {

      if(NAME3==NAME1) { //operation on itself
        // V1 / V2 = V1
        vec1->divide(*vec2);
      } else if(NAME3==NAME2) { //operation on itself
        // V1 / V2 = V2
        vec2->invert();
        vec2->multiply(*vec1);
      } else {
        if(memoryTree->find(NAME3)) memoryTree->rm(NAME3); //ok.
        Slash::Data::IArray* vector = factory.divide(NAME3,*vec1,*vec2);
        if(!vector) {
          out << "vector_operation_C :" 
              << " can't divide vector " 
              << Lib::sout(NAME1) << " with "
              << Lib::sout(NAME2) << "."
              << Lib::endl;
          return -1;
        }
      }

    }

  }

  return -1;
}
namespace OpenPAW {
class PrintRange {    
public:
  unsigned int begin;
  unsigned int end;
  double value;
public:
  void dump(Slash::Data::IArray& aArray,const std::string& aName,Lib::Out& aOut,const std::vector<int>& aDigits,bool aDENSE) {
    unsigned int dim = aArray.dimension();
    std::vector<unsigned int> is(dim);
    const std::vector<double>& v = aArray.vector();
    std::string s;

    if(aDENSE) { 

      if(begin==end) {
        // V(end) = <value>

        int index = end;
        aArray.indices(index,is);
        aOut << " " << aName;
        aOut << "("; 
        for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
          Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
          Lib::smanip::justify(s,aDigits[iaxis],Lib::smanip::right);
          if(iaxis) aOut << ",";
          aOut << s;
        }
        aOut << ") = " << v[index] << Lib::endl;


      } else {
        // V(begin)..(end) = <value>

        aOut << " " << aName;
       {int index = begin;
        aArray.indices(index,is);
        aOut << "("; 
        for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
          Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
          Lib::smanip::justify(s,aDigits[iaxis],Lib::smanip::right);
          if(iaxis) aOut << ",";
          aOut << s;
        }
        aOut << ")..";}

       {int index = end;
        aArray.indices(index,is);
        aOut << "("; 
        for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
          Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
          Lib::smanip::justify(s,aDigits[iaxis],Lib::smanip::right);
          if(iaxis) aOut << ",";
          aOut << s;
        }
        aOut << ") = " << v[index] << Lib::endl;}

      }

    } else {
      // V(begin) = ...
      // V(  end) = <value>

      if(end!=begin) {
        int index = begin;
        aOut << " " << aName << "("; 
        aArray.indices(index,is);
        for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
          Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
          Lib::smanip::justify(s,aDigits[iaxis],Lib::smanip::right);
          if(iaxis) aOut << ",";
          aOut << s;
        }
        aOut << ") = " << "..." << Lib::endl;
      }

     {int index = end;
      aOut << " " << aName << "("; 
      aArray.indices(index,is);
      for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
        Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
        Lib::smanip::justify(s,aDigits[iaxis],Lib::smanip::right);
        if(iaxis) aOut << ",";
        aOut << s;
      }
      aOut << ") = " << v[index] << Lib::endl;}

    }

  }
};
}
//////////////////////////////////////////////////////////////////////////////
int vector_print_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_print_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;

  //if(cmd_path!="/VECTOR/PRINT") return;


  std::string VNAME = ku_gets();
  int DENSE = ku_geti();

  std::string NAME;
  std::vector< std::pair<int,int> > dims;
  if(!OpenPAW::PAW_Manager::parseName(VNAME,NAME,dims,printer)) return -1;
  Lib::smanip::touppercase(NAME);
  Slash::Data::IArray* vector = find_vector(*memoryTree,NAME);
  if(!vector) {
    out << "vector_print_C :" 
        << " can't find vector " << Lib::sout(NAME) 
        << " in tree " << Lib::sout(memoryTree->storeName()) 
        << Lib::endl;
    return -1;
  }
  std::vector< std::pair<unsigned int,unsigned int> > cut;
  if(!vector_get_cut(*vector,dims,cut)) {
    out << "vector_print_C :" 
        << " mismatch dimension in " << Lib::sout(VNAME) 
        << Lib::endl;
    return -1;
  }

  unsigned int dim = vector->dimension();
  std::vector<unsigned int> is(dim);
  const std::vector<double>& v = vector->vector();
  unsigned int vsize = vector->size();

  // First pass to get number of digits for indices :
  std::vector<int> digits;
  digits.resize(dim,1);
  for(unsigned int index = 0;index<vsize;index++) {
    if(vector->accept(index,cut)) {
      vector->indices(index,is);
      for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
        int i = is[iaxis]+1;
        int ndigit = 1+(int)::log10((double)i);
        digits[iaxis] = MAXIMUM(digits[iaxis],ndigit);
      }
    }
  }

  if(DENSE!=0) {
    OpenPAW::PrintRange range;
    bool first = true;
    for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        if(first) {
          range.begin = index;
          range.end = index;
          range.value = v[index];
          first = false;
        } else {
          if(v[index]==range.value) {
            range.end = index;
            continue;
          } else { //Value changed :
            range.dump(*vector,NAME,out,digits,DENSE==2?true:false);
            range.begin = index;                    
            range.end = index;
            range.value = v[index];
          }
        }
      }
    }
    if(!first) range.dump(*vector,NAME,out,digits,DENSE==2?true:false);

  } else {

    std::string s;
    for(unsigned int index = 0;index<vsize;index++) {
      if(vector->accept(index,cut)) {
        // V(indices) = <value>
        out << " " << NAME << "("; 
        vector->indices(index,is);
        for(unsigned iaxis=0;iaxis<dim;iaxis++) { 
          Lib::smanip::printf(s,32,"%d",is[iaxis]+1);
          Lib::smanip::justify(s,digits[iaxis],Lib::smanip::right);
          if(iaxis) out << ",";
          out << s;
        }
        out << ") = " << v[index] << Lib::endl;
      }
    }

  }

  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_list_C(
 void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "vector_list_C :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return -1;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return -1;

  //if(cmd_path!="/VECTOR/LIST") return;


  std::vector<std::string> objs = memoryTree->listObjectNames();
  std::vector<std::string> typs = memoryTree->listObjectTypes();
  if(typs.size()!=objs.size()) {
    out << "vector_delete_C :" 
        << " problem retreiving directory infos."
        << Lib::endl;
    return -1;
  }
  unsigned int objn = objs.size();
  std::string nameh = " Vector Name                          ";
  out << " " << nameh << "Type Length    Dim-1    Dim-2    Dim-3" << Lib::endl;
  int vecn = 0;
  for(unsigned int index = 0;index<objn;index++) {
    std::string& name = objs[index];
    std::string& type = typs[index];
    //printf("debug : %s %s\n",name.c_str(),type.c_str());
    if(type=="Slash::Data::IArray") {
      Slash::Data::IArray* vector = find_vector(*memoryTree,name);
      if(vector) {

        std::string s;
        out << " ";

        s = name;
        Lib::smanip::justify(s,nameh.size(),Lib::smanip::left);
        out << s;

        s = "R";
        Lib::smanip::justify(s,4,Lib::smanip::right);
        out << s;
        out << " ";

        Lib::smanip::printf(s,32,"%d",vector->size());
        Lib::smanip::justify(s,6,Lib::smanip::right);
        out << s;
        out << "    ";

        if(vector->dimension()>=1) {
          Lib::smanip::printf(s,32,"%d",vector->orders()[0]);
          Lib::smanip::justify(s,5,Lib::smanip::right);
          out << s;
          out << "    ";
        }
        if(vector->dimension()>=2) {
          Lib::smanip::printf(s,32,"%d",vector->orders()[1]);
          Lib::smanip::justify(s,5,Lib::smanip::right);
          out << s;
          out << "    ";
        }
        if(vector->dimension()>=3) {
          Lib::smanip::printf(s,32,"%d",vector->orders()[2]);
          Lib::smanip::justify(s,5,Lib::smanip::right);
          out << s;
          out << "    ";
        }
        out << Lib::endl;
        vecn++;
      }
    }
  }
  out << " Total of " << vecn << " Vector(s)" << Lib::endl;
 
  return -1;
}
//////////////////////////////////////////////////////////////////////////////
int vector_find_C( 
 const char* aName
,void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return 0; //Failure

  Lib::Parameters<std::string,Lib::Value>& variables = 
    pawManager->variables();

  std::string name = aName;
  if(name=="LOOP") return 0; //Failure
  if(variables.isKey(name)) {
    return 1; //Success
  } else { //Search also in vectors.
    // We pass here when doing :
    //    VEC/CRE v(2) R 0. 0.
    //    x = v(2)
    AIDA::IAnalysisFactory* aida = find_aida(*session);
    if(!aida) return 0;
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(!memoryTree) return 0;
    Lib::smanip::touppercase(name);
    Slash::Data::IArray* vector = find_vector(*memoryTree,name);
    if(!vector) {
      Slash::Core::IWriter& printer = session->printer();
      Lib::Out out(printer);
      out << "vector_find_C :"
          << " " << Lib::sout(name) << " not found."
          << Lib::endl;
      return 0; //Failure
    }
    return 1; //Success
  }
}
//////////////////////////////////////////////////////////////////////////////
int vector_value_C( 
 const char* aName
,double* aValue
,void* aSession
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return 1; //Failure

  Lib::Parameters<std::string,Lib::Value>& variables = 
    pawManager->variables();

  // Look if aName is of the form <name>(<index>)
  std::string name;
  int aindex = 0;
  if(!Lib::smanip::parseArray(aName,name,aindex)) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " syntax error in " << Lib::sout(std::string(aName))
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  }

  if(variables.isKey(name)) {
    Lib::Value var = variables.value(name);
    if(var.type()==Slash::Core::IValue::ARRAY_DOUBLE) {
      if(aindex==NotFound) {
        Slash::Core::IWriter& printer = session->printer();
        Lib::Out out(printer);
        out << "vector_value_C :"
            << " vector variable without an index in " 
            << Lib::sout(std::string(aName))
            << Lib::endl;
        *aValue = 0;
        return 1; //Failure
      } 

      aindex--; //CERN/PAW = FORTRAN !
      std::vector<unsigned int> is;
      const std::vector<double>& stdv = var.get_array_double(is);
      if(aindex>=(int)stdv.size()) {
        Slash::Core::IWriter& printer = session->printer();
        Lib::Out out(printer);
        out << "vector_value_C :"
            << " bad index in " << Lib::sout(std::string(aName))
            << ". Size of vector is " << (unsigned int)stdv.size()
            << Lib::endl;
        *aValue = 0;
        return 1; //Failure
      }
      *aValue = stdv[aindex];
      return 0; //Success

    } 

    Slash::Core::IWriter& printer = session->printer();

    double d;
    if(!Lib::valuemanip::to_double(var,d)) { 
      Lib::Out out(printer);
      out << "vector_value_C :"
          << " can't convert Lib::Value to a double in " 
          << Lib::sout(std::string(aName))
          << Lib::endl;
      *aValue = 0;
      return 1; //Failure
    }
    *aValue = d;
    return 0; //Success
  } 

  //Search also in vectors.
  // We pass here when doing :
  //    VEC/CRE v(2) R 0. 0.
  //    x = v(2)
  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return 0;
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return 0;
  Lib::smanip::touppercase(name);
  Slash::Data::IArray* vector = find_vector(*memoryTree,name);
  if(!vector) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " " << Lib::sout(name) << " not found."
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  }
  if(aindex==NotFound) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " vector variable without an index in " 
        << Lib::sout(std::string(aName))
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  } 

  //FIXME : handle more dimensions.
  if(vector->dimension()!=1) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " vector dimension > 1 not treated yet in " 
        << Lib::sout(std::string(aName))
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  }
  aindex--; //CERN/PAW = FORTRAN !
  if(aindex>=(int)vector->size()) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " bad index in " << Lib::sout(std::string(aName))
        << ". Size of vector is " << vector->size()
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  }
  std::vector<unsigned int> is;
  is.push_back(aindex);
  double v;
  if(!vector->value(is,v)) {
    Slash::Core::IWriter& printer = session->printer();
    Lib::Out out(printer);
    out << "vector_value_C :"
        << " can't get value for IArray in " 
        << Lib::sout(std::string(aName))
        << Lib::endl;
    *aValue = 0;
    return 1; //Failure
  }
  *aValue = v;
  return 0; //Success
}
