#ifndef OpenPAW_PAW_Manager_h
#define OpenPAW_PAW_Manager_h

// Inheritance :
#include <Slash/Core/IManager.h>

#include <vector>
#include <Lib/Parameters.h>
#include <Lib/Value.h>

#include <OpenPAW/Unit.h>
#include <OpenPAW/ZONE.h>

namespace Slash {namespace Core {class ISession;}}
namespace Slash {namespace Core {class IWriter;}}

namespace AIDA {
 class ITree;
 class IPlotter;
 class IPlotterRegion;
 class IFunction;
 class ICloud2D;
 class IHistogram1D;
 class IDataPointSet;
}

namespace OpenPAW {

class ColorEntry {
public:
  ColorEntry();
  ColorEntry(double,double,double);
  ColorEntry(const ColorEntry&);
  ColorEntry& operator=(const ColorEntry&);
public:
  double fRed;
  double fGreen;
  double fBlue;
};

class Palette : public std::vector<int>  {
public:
  Palette();
  Palette(const Palette&);
  Palette(const std::vector<int>&);
  Palette& operator=(const Palette&);
};

class PAW_Manager : public virtual Slash::Core::IManager {
public: //Slash::Core::IManager
  virtual std::string name() const;
  virtual void* cast(const std::string&) const;
public:
  PAW_Manager(Slash::Core::ISession&);
  virtual ~PAW_Manager();
  Lib::Parameters<int,OpenPAW::Unit>& units();
  OpenPAW::Unit findUnit(const std::string&) const;
  Lib::Parameters<std::string,Lib::Value>& variables();
  Lib::Parameters<int,std::string>& cuts();
  Lib::Parameters<std::string,std::string>& opts();
  Lib::Parameters<std::string,double>& atts();
  void setUTIT(const std::string&);
  int paletteFromHCOL() const;
  void setContourMode(bool);
  void setCurrentTree(const std::string&);
  const std::string& currentTree() const;
  std::string generateFunctionName();
  void setFunctionPoints(int,int,int);
  int NPX() const;
  int NPY() const;
  int NPZ() const;
  void setZONE_K(int);
  int ZONE_K() const;
  void setDoZONE(bool);
  void setZONE(const ZONE&);
  void createColor(int);
  void setColor(int,double,double,double);
  void setPalette(int,const Palette&);
  
  void showATTs();
  void showOPTs();
  bool filterCuts(const std::string&,std::string&) const;
  bool parseTupleName(const std::string&,std::string&,
                      std::vector<std::string>&);
  int getInfosOptions(std::string&) const; 

  //AIDA :
  AIDA::ITree* findCurrentTree() const;
  void setPlotterZONE(AIDA::IPlotter&);
  bool plotterNext(AIDA::IPlotter&);
  void regionSet_GFON(AIDA::IPlotter&);

  // Region :
  void regionSetLayout(AIDA::IPlotterRegion&); 
  void regionSetColormap(AIDA::IPlotterRegion&,double,double,int); 
  void regionSet_VFON(AIDA::IPlotterRegion&);
  void regionSet_TFON(AIDA::IPlotterRegion&);
  void regionSet_LFON(AIDA::IPlotterRegion&);
  void regionSet_GFON(AIDA::IPlotterRegion&);

  // Plotting :
  void plotFunction1D(AIDA::IPlotterRegion&,AIDA::IFunction&,
                      double,double,const std::string&); 
  void plotFunction2D(AIDA::IPlotterRegion&,AIDA::IFunction&,
                      double,double,double,double,double,double,
                      int,const std::string&); 
  void plotCloud2D(AIDA::IPlotterRegion&,AIDA::ICloud2D&,const std::string&);
  void plotErrors2D(AIDA::IPlotterRegion&,AIDA::IDataPointSet&,int,double);
  void plotHistogram1D(AIDA::IPlotterRegion&,AIDA::IHistogram1D&,
                       const std::string&,bool); 
  void setHTYP(AIDA::IPlotterRegion&,int,int,double,double); 

  void addLegend(AIDA::IPlotterRegion&,
                 double,double,const std::string&,
                 const std::string&,int,bool); 
public:
  static bool parseName(const std::string&,
                        std::string&,std::vector< std::pair<int,int> >&,
                        Slash::Core::IWriter&);
  static bool isAnOption(const std::vector<std::string>&,
                         const std::string&,const std::string&); 
  static bool hasUnkownOption(const std::vector<std::string>&,
                              const std::string&); 
private:
  void regionInitialize(AIDA::IPlotterRegion&); 
  void getFont(const std::string&,std::string&,double&);
  std::string get_MTYP(int);
private:
  std::string fName;
  Slash::Core::ISession& fSession;
  Lib::Parameters<int,OpenPAW::Unit> fUnits;
  Lib::Parameters<std::string,Lib::Value> fVariables;
  Lib::Parameters<int,std::string> fCuts;
  Lib::Parameters<std::string,std::string> fOPTs;
  Lib::Parameters<std::string,double> fATTs;
  std::string fUTIT;
  bool fContourMode;
  std::string fCurrentTree;
  int fFunctionCounter;
  int fNPX;
  int fNPY;
  int fNPZ;
  int fZONE_K;
  bool fDoZONE;
  ZONE fZONE;
  std::vector<ColorEntry> fColorTable;
  std::vector<Palette> fPalettes;
};

}

typedef Lib::Parameters<std::string,double> ATTs_type;
typedef Lib::Parameters<std::string,std::string> OPTs_type;
typedef Lib::Parameters<int,OpenPAW::Unit> Units_type;

#define find_PAW(aSession)\
  Lib_findManager(aSession,"PAW_Manager",OpenPAW::PAW_Manager)

#define PALETTE_FROM_HCOL (-1)
#define PALETTE_HISTOGRAM_2D_PLOT_CONTOUR_OPT_0 51
#define PALETTE_HISTOGRAM_PLOT_OPT_COL 52

#endif
