extern "C" {
  void pafort_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
//#include <Slash/Data/IArray.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
//#include <Lib/FunctionManager.h>
#include <Lib/Value.h>

//#include <BatchLab/Core/ArrayFactory.h> //FIXME : not AIDA.
#include <BatchLab/Core/Function.h> //FIXME : not AIDA.
#include <BatchLab/Core/FunctionCatalog.h> //FIXME : not AIDA.

#include "FindHelpers.h"

#define MAXIMUM(a,b) ((a)>(b)?a:b)

//////////////////////////////////////////////////////////////////////////////
inline bool function_vector_get_value(
 const std::vector<Lib::Value>& aArgs
,const std::vector<Slash::Data::IArray*>& a_vcts
,Slash::Core::IWriter& aPrinter
,const std::string& aUFUNC
,Slash::Data::IFunction& aFunction
,int aNumber
,Lib::Value& aReturn
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int argn = aArgs.size();
  if(!argn) {
    Lib::Out out(aPrinter);
    out << "function_vector_get_value :" 
        << " in " << Lib::sout(aUFUNC) 
        << ", function without argument."
        << Lib::endl;
    return false;
  }

  std::vector<Slash::Core::IValue*> fargs;
  fargs.resize(argn,0);

  // Expect one INTEGER argument followed by some VECTORS.

  if(aArgs[0].type()!=Slash::Core::IValue::INT_STAR) {
    Lib::Out out(aPrinter);
    out << "function_vector_get_value :" 
        << " in " << Lib::sout(aUFUNC) 
        << ", expect a first integer argument."
        << Lib::endl;
    return false;
  }

  unsigned int vctsn = a_vcts.size();
  if(vctsn!=(argn-1)) {
    Lib::Out out(aPrinter);
    out << "function_vector_get_value :" 
        << " in " << Lib::sout(aUFUNC) 
        << ", mismatch number of VECTOR argument."
        << Lib::endl;
    return false;
  }

 {for(unsigned int argi=1;argi<argn;argi++) {
    if(aArgs[argi].type()!=Slash::Core::IValue::DOUBLE_STAR) {
      Lib::Out out(aPrinter);
      out << "function_vector_get_value :" 
          << " in " << Lib::sout(aUFUNC) 
          << ", expect a VECTOR for argument " << argi
          << Lib::endl;
      return false;
    }
  }}

  fargs[0] = new Lib::Value(&aNumber); //must be &.

 {for(unsigned int argi=1;argi<argn;argi++) {
    Slash::Data::IArray* a = a_vcts[argi-1];
    std::vector<double>& va = a->vector();
    if(!va.size()) {
      Lib::Out out(aPrinter);
      out << "function_vector_get_value :" 
          << " IArray as an empty std::vector<double>."
          << Lib::endl;
     {for(unsigned int argi=0;argi<fargs.size();argi++) delete fargs[argi];}
      return false;
    }
    double* p = &(va[0]); //FIXME : safe ?
    fargs[argi] = new Lib::Value(p);
  }}

  bool status = aFunction.value(fargs,aReturn);
  if(!status) {
    Lib::Out out(aPrinter);
    out << "function_vector_get_value :" 
        << " in " << Lib::sout(aUFUNC) 
        << " can't get value."
        << Lib::endl;
  } 

 {for(unsigned int argi=0;argi<argn;argi++) delete fargs[argi];}

  return status;
}

//////////////////////////////////////////////////////////////////////////////
void pafort_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "pafort :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(cmd_path=="/FORTRAN/FILE") {

    int LUN = ku_geti();
    std::string FNAME = ku_gets();
    std::string STATUS = ku_gets();

    Units_type& units = pawManager->units();

    if(LUN==0) {
      // If LUN is 0 the next free logical unit will be used.
      const Lib::Parameters<int,OpenPAW::Unit>::List& lunits = units.list();
      Lib::Parameters<int,OpenPAW::Unit>::ConstIterator it;
      for(it=lunits.begin();it!=lunits.end();++it) {
        LUN = MAXIMUM(LUN,(*it).fKey);
      }
      LUN++;
    }

    if(units.isKey(LUN)) {
      out << "pafort :" 
          << " LUN " << LUN << " already in used by OpenPAW." 
          << Lib::endl;
      return;
    }

    if(units.key
         (OpenPAW::Unit(OpenPAW::Unit::METAFILE,FNAME))!=units.keyNotFound()) {
      out << "pafort :" 
          << " " << Lib::sout(FNAME) << " already in used by OpenPAW." 
          << Lib::endl;
      return;
    }

    //OLD        Open existing file for reading.
    //APPEND     Open existing file and position at EOF.
    //NEW        Create new file; error if already existing.
    //UNKNOWN    Open existing or create new file.
    //DONTKNOW   Like UNKNOWN except on VMS opens highest cycle.

    std::string s = Lib::fmanip::name(FNAME);

    FILE* file = ::fopen(s.c_str(),"wb");
    if(!file) {
      out << "pafort :" 
          << " can't open " << Lib::sout(s) << " in wb mode." 
          << Lib::endl;
      return;
    }

    units.add(LUN,"",OpenPAW::Unit(OpenPAW::Unit::METAFILE,FNAME,file),
                     OpenPAW::Unit(),OpenPAW::Unit());

  } else if(cmd_path=="/FORTRAN/CLOSE") {

    int LUN = ku_geti();

    Units_type& units = pawManager->units();
    OpenPAW::Unit u = units.value(LUN);
    if(u!=units.valueNotFound()) {
      units.remove(LUN);
      if(u.isFILE()) {
        ::fclose(u.fFILE);
        if(u.fType==OpenPAW::Unit::METAFILE) {
          //FIXME : not really the right place to do that.
          AIDA::IPlotter* plotter = create_plotter(*aida);
          if(plotter) {
            plotter->writeToFile(u.fString,"PS");
            delete plotter;
          }   
        }
      } else {
        session->destroyManager(u.fString);
        pawManager->setCurrentTree("MemoryTree");
      }

      if(session->verboseLevel()) {
        out << "pafort :" 
            << " LUN " << LUN << " closed."
            << Lib::endl;
      }
      return;
    }

  } else if(cmd_path=="/FORTRAN/CALL") {
    std::string UROUT = ku_gets();

    // Extract name/parameters :
    std::string name;
    std::vector<std::string> sargs;
    if(!Lib::smanip::nameArgs(UROUT,name,sargs)) {
      out << "pafort :" 
          << " Syntax error in " << Lib::sout(UROUT) 
          << Lib::endl;
      return;
    }

    std::string sfunc = name;
    Lib::smanip::touppercase(sfunc);
    if(sfunc=="HFILL") {
      if(sargs.size()!=4) { 
        out << "pafort :" 
            << " For HFILL, 4 arguments are expected in " << Lib::sout(UROUT) 
            << ". Signature should be (I,R,R,R)"
            << Lib::endl;
        return;
      }
    /*int arg0;
      if(!Lib::smanip::toint(sargs[0],arg0)) { //Syntax error.
        out << "pafort :" 
            << " Syntax error in " << Lib::sout(UROUT) 
            << Lib::endl;
        return;
      }*/
      std::string ID = sargs[0];

      double x;
      if(!Lib::smanip::todouble(sargs[1],x)) { //Syntax error.
        out << "pafort :" 
            << " Syntax error for x in " << Lib::sout(UROUT) 
            << Lib::endl;
        return;
      }
      double y;
      if(!Lib::smanip::todouble(sargs[2],y)) { //Syntax error.
        out << "pafort :" 
            << " Syntax error for y in " << Lib::sout(UROUT) 
            << Lib::endl;
        return;
      }
      double w;
      if(!Lib::smanip::todouble(sargs[3],w)) { //Syntax error.
        out << "pafort :" 
            << " Syntax error for w in " << Lib::sout(UROUT) 
            << Lib::endl;
        return;
      }
      //FIXME : use BatchLabHBOOK ? But the below handle non-integer ID...
      //HFILL(arg1,arg2,arg3,arg4);
      AIDA::IBaseHistogram* histogram = baseHistogram_find(*session,ID);
      if(!histogram) {
        out << "pafort :" 
            << " can't find histogram " << Lib::sout(ID) 
            << Lib::endl;
        return;
      }
      if(histogram->dimension()==1) {
        AIDA::IHistogram1D* histogram1D = 
          Lib_Cast(*histogram,AIDA::IHistogram1D);
        histogram1D->fill(x,w);
      } else if(histogram->dimension()==2) {
        AIDA::IHistogram2D* histogram2D = 
          Lib_Cast(*histogram,AIDA::IHistogram2D);
        histogram2D->fill(x,y,w);
      } else {
        out << "pafort :" 
            << " histogram of dimension " << histogram->dimension()
            << " not yet supported."
            << Lib::endl;
        return;
      }
      return;
    }

    // On the fly compilation :
    if(sargs.size()!=1) { 
      out << "pafort :" 
          << " Syntax error in " << Lib::sout(UROUT) 
          << ". One argument expected."
          << Lib::endl;
      return;
    }

    std::string UFUNC = name;
    std::string NUMBER = sargs[0];
    int number;
    if(!Lib::smanip::toint(NUMBER,number)) { //Syntax error.
      out << "pafort :" 
          << " Syntax error in " << Lib::sout(UROUT) 
          << Lib::endl;
      return;
    }

    std::string funcName = pawManager->generateFunctionName();

    char* path = ku_macro_path();
    std::string paths = path?path:"";

    AIDA::IFunction* func = create_function_vector
      (funcName,paths,UFUNC,*session,*aida,*memoryTree,false);

    if(!func) {
      out << "pafort :" 
          << " can't get function " << Lib::sout(UFUNC) 
          << Lib::endl;
      return;
    }

    if(func->dimension()!=1) {
      out << "pafort :" 
          << " function " << Lib::sout(UFUNC)
          << " not of dimension one."
          << Lib::endl;
      delete func;
      return;
    }

    std::vector<double> fargs(1);
    fargs[0] = (double)number;
    func->value(fargs);

    delete func;

  } else {
    out << "pafort :" 
        << " " << cmd_path << " : dummy." << Lib::endl;
  }
}
