extern "C" {
  void paghpl_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>

#include <AIDA/IPlotterStyle.h>
#include <AIDA/IAxisStyle.h>
#include <AIDA/IDataPointSet.h>
#include <AIDA/IDataPoint.h>
#include <AIDA/IMeasurement.h>

#include <BatchLab/Helpers/AIDA.h>

#include <OpenPAW/PAW_Manager.h>

#include "KuipHelpers.h"
#include "RegionHelpers.h"
#include "ArrayHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void paghpl_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  //std::string cmd_name = ku_name();
  std::string cmd_path = ku_path() ? ku_path() : "";
  if(session->verboseLevel()) {
    out << "paghpl :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(cmd_path=="/GRAPHICS/HPLOT/ATITLE") {

    std::string XTIT = ku_gets();
    std::string YTIT = ku_gets();
    std::string ZTIT = ku_gets();
    int IALIGN = ku_geti();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "paghpl :" 
          << " /GRAPHICS/HPLOT/ATITLE "
          << " XTIT= " << Lib::sout(XTIT) 
          << " YTIT= " << Lib::sout(YTIT) 
          << " ZTIT= " << Lib::sout(ZTIT) 
          << " IALIGN= " << IALIGN
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      region.style().xAxisStyle().setLabel(XTIT);
      region.style().yAxisStyle().setLabel(YTIT);
      region.style().zAxisStyle().setLabel(ZTIT);

      delete plotter;
    }

  } else if(cmd_path=="/GRAPHICS/HPLOT/TICKS") {

    std::string CHOPT = ku_gets();
    /*double XVAL = */ ku_getr();
    /*double YVAL = */ ku_getr();

    if(session->verboseLevel()) {
      out << "paghpl :" 
          << " /GRAPHICS/HPLOT/TICKS "
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"")) {
        region.style().setParameter("rightAxisVisible","TRUE");
        region.style().setParameter("topAxisVisible","TRUE");
      }

      delete plotter;
    }

  } else if(cmd_path=="/GRAPHICS/HPLOT/NULL") {

    double XMIN = ku_getr();
    double XMAX = ku_getr();
    double YMIN = ku_getr();
    double YMAX = ku_getr();

    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "paghpl :" 
          << " /GRAPHICS/HPLOT/NULL "
          << " XMIN= " << Lib::sout(XMIN) 
          << " XMAX= " << Lib::sout(XMAX) 
          << " YMIN= " << Lib::sout(YMIN) 
          << " YMAX= " << Lib::sout(YMAX) 
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      AIDA::IPlotterRegion& region = plotter->currentRegion();
      region.clear();

      // Enforce plotter axes values :
      region.setParameter("plotter.xAxisAutomated","FALSE");
      region.setParameter("plotter.xAxisMinimum",Lib::smanip::tostring(XMIN));
      region.setParameter("plotter.xAxisMaximum",Lib::smanip::tostring(XMAX));

      region.setParameter("plotter.yAxisAutomated","FALSE");
      region.setParameter("plotter.yAxisMinimum",Lib::smanip::tostring(YMIN));
      region.setParameter("plotter.yAxisMaximum",Lib::smanip::tostring(YMAX));

      delete plotter;
    }

  } else if(cmd_path=="/GRAPHICS/HPLOT/ERRORS") {

    std::string X = ku_gets();
    std::string Y = ku_gets();
    std::string EX = ku_gets();
    std::string EY = ku_gets();
    int N = ku_geti();
    int ISYMB = ku_geti();
    double SSIZE = ku_getr();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "paghpl :" 
          << " /GRAPHICS/HPLOT/ERRORS "
          << " X= " << Lib::sout(X) 
          << " Y= " << Lib::sout(Y) 
          << " EX= " << Lib::sout(EX) 
          << " EX= " << Lib::sout(EY) 
          << " ISYMB= " << ISYMB
          << " SSIZE= " << SSIZE
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

#define GET_VEC(aS,aV,aVv) \
    Lib::smanip::touppercase(aS);\
    Slash::Data::IArray* aV = find_vector(*memoryTree,aS);\
    if(!aV) {\
      out << "paghpl :"\
          << " can't find vector " << Lib::sout(aS)\
          << " in MemoryTree."\
          << Lib::endl;\
      return;\
    }\
    if(aV->dimension()!=1) {\
      out << "paghpl :"\
          << " " << Lib::sout(aS) << " not a 1D vector."\
          << Lib::endl;\
      return;\
    }\
    if(N>int(aV->vector().size())) {\
      out << "paghpl :"\
          << " for " << Lib::sout(aS)\
          << " N=" << N << " > " << int(aV->vector().size())\
          << Lib::endl;\
      return;\
    }\
    const std::vector<double>& aVv = aV->vector();

    GET_VEC(X,xvec,xvecv)
    GET_VEC(Y,yvec,yvecv)
    GET_VEC(EX,exvec,exvecv)
    GET_VEC(EY,eyvec,eyvecv)

    // Create a temporary two dimensional IDataPointSet.
    AIDA::IDataPointSet* atmp = 
      create_dataPointSet(*aida,*memoryTree,
                          path_not_managed()+"/tmp","",
                          2);
    if(!atmp) {
      out << "pagpri :" 
          << " can't create a temporary AIDA object." 
          << Lib::endl;
      return;
    }

    for ( int i = 0; i<N; i++ ) {
      AIDA::IDataPoint* pt = atmp->addPoint();
     {AIDA::IMeasurement* ms = pt->coordinate(0);
      ms->setValue(xvecv[i]);
      double error = exvecv[i];
      ms->setErrorPlus(error);
      ms->setErrorMinus(error);}
     {AIDA::IMeasurement* ms = pt->coordinate(1);
      ms->setValue(yvecv[i]);
      double error = eyvecv[i];
      ms->setErrorPlus(error);
      ms->setErrorMinus(error);}
      //printf("debug : %d (%d) : %g %g : %g %g\n",
      //  i,N,xvecv[i],yvecv[i],exvecv[i],eyvecv[i]);      
    }


    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      pawManager->regionSetLayout(region);

      pawManager->plotErrors2D(region,*atmp,ISYMB,SSIZE);

      delete plotter;
    }

    // Plotter is in retain mode, we can now get rid of the object.
    delete atmp;

  } else if(cmd_path=="/GRAPHICS/HPLOT/KEY") {

    double X = ku_getr();
    double Y = ku_getr();
    int IATT = ku_geti();
    std::string TEXT = ku_gets();
    double DX = ku_getr();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "paghpl :" 
          << " /GRAPHICS/HPLOT/KEY "
          << " X= " << X 
          << " Y= " << Y 
          << " IATT= " << IATT
          << " TEXT= " << Lib::sout(TEXT) 
          << " DX= " << DX 
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      AIDA::IPlotterRegion& region = plotter->currentRegion();

      //FIXME pawManager->regionSetLayout(region);

      bool superpose = OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S");

      pawManager->addLegend(region,X,Y,TEXT,"M",IATT,superpose);

      delete plotter;
    }

  } else {
    out << "paghpl :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
