extern "C" {
  void pahio_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <BatchLab/Core/amanip.h>

#include "KuipHelpers.h"
#include "FindHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void pahio_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "pahio :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  if(cmd_path=="/HISTOGRAM/HIO/PRINT") {

    std::string ID = ku_gets();
    /*std::string CHOPT = */ku_gets();
 
    AIDA::IBaseHistogram* histo = baseHistogram_find(*session,ID);
    if(!histo) {
      out << "pahio :" 
          << " object " << Lib::sout(ID) 
          << " not an AIDA::IBaseHistogram."
          << Lib::endl;
      return;
    }

    BatchLab::amanip::hprint(*histo,ID,printer);

  } else if(cmd_path=="/HISTOGRAM/HIO/HRIN") {

    /*int ID =*/ ku_gets();
    /*int ICYCLE =*/ ku_geti();
    /*int IOFFSET =*/ ku_geti();

    if(pawManager->currentTree()=="MemoryTree") {
      out << "pahio :" 
          << " current tree is MemoryTree !"
          << Lib::endl;
      return;
    }

    AIDA::ITree* tree = pawManager->findCurrentTree();
    if(!tree) return;

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " HRIN on tree " << Lib::sout(tree->storeName()) 
          << Lib::endl;
    }

    std::string path = memoryTree->pwd();

    std::string stmp = "/tmp_memory";

    if(!tree->mount(stmp,*memoryTree,path)) {
      out << "pahio :" 
          << " can't mount path " << Lib::sout(path) 
          << " of MemoryTree on " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " path " << Lib::sout(path) 
          << " of MemoryTree mounted at " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
    }

    if(!tree->cp(tree->pwd(),stmp)) {
      out << "pahio :" 
          << " can't copy . "
          << " to " << Lib::sout(stmp)
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(!tree->unmount(stmp)) {
      out << "pahio :" 
          << " can't unmount path " << Lib::sout(stmp) 
          << " of tree " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " path " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << " unmounted."
          << Lib::endl;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " HRIN on tree " << Lib::sout(tree->storeName()) 
          << " done."
          << Lib::endl;
    }

  } else if(cmd_path=="/HISTOGRAM/HIO/HROUT") {

    std::string ID = ku_gets();
    std::string CHOPT = ku_gets();

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " " << cmd_path
          << " ID= " << Lib::sout(ID) 
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    if(pawManager->currentTree()=="MemoryTree") {
      out << "pahio :" 
          << " current tree is MemoryTree !"
          << Lib::endl;
      return;
    }

    AIDA::ITree* tree = pawManager->findCurrentTree();
    if(!tree) return;

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " HROUT on tree " << Lib::sout(tree->storeName()) 
          << Lib::endl;
    }

    std::string path = memoryTree->pwd();

    std::string stmp = "/tmp_memory";

    if(!tree->mount(stmp,*memoryTree,path)) {
      out << "pahio :" 
          << " can't mount path " << Lib::sout(path) 
          << " of MemoryTree on " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " path " << Lib::sout(path) 
          << " of MemoryTree mounted at " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
    }

    if(!tree->cp(stmp,tree->pwd())) {
      out << "pahio :" 
          << " can't copy " << Lib::sout(stmp) 
          << " to " << Lib::sout(".")
          << " of " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(!tree->unmount(stmp)) {
      out << "pahio :" 
          << " can't unmount path " << Lib::sout(stmp) 
          << " of tree " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " path " << Lib::sout(stmp) 
          << " of " << Lib::sout(tree->storeName()) 
          << " unmounted."
          << Lib::endl;
    }

    if(!tree->commit()) {
      out << "pahio :" 
          << " can't commit tree " << Lib::sout(tree->storeName()) 
          << Lib::endl;
      return;
    }

    if(session->verboseLevel()) {
      out << "pahio :" 
          << " HROUT on tree " << Lib::sout(tree->storeName()) 
          << " done."
          << Lib::endl;
    }

  } else {
    out << "pahio :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
