extern "C" {
  void pahist_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Lib/DLL.h>
#include <Lib/System.h>

#include "KuipHelpers.h"
#include "FindHelpers.h"
#include "PlotHistogram2D.h"

#define MINIMUM(a,b) ((a)<(b)?a:b)
#define MAXIMUM(a,b) ((a)>(b)?a:b)
#define NotFound (-1)

static bool region_setLimits(AIDA::IPlotterRegion&,AIDA::IHistogram1D&,const std::vector< std::pair<int,int> >&,Slash::Core::IWriter&);
static bool region_setLimits(AIDA::IPlotterRegion&,AIDA::IHistogram2D&,const std::vector< std::pair<int,int> >&,Slash::Core::IWriter&);

//////////////////////////////////////////////////////////////////////////////
void pahist_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  //std::string cmd_name = ku_name();
  std::string cmd_path = ku_path() ? ku_path() : "";
  if(session->verboseLevel()) {
    out << "pahist :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  OPTs_type& opts = pawManager->opts();
  ATTs_type& atts = pawManager->atts();

  if(cmd_path=="/HISTOGRAM/PLOT") {

    std::string NAME = ku_gets();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string ID;
    std::vector< std::pair<int,int> > dims;  
    if(!OpenPAW::PAW_Manager::parseName(NAME,ID,dims,printer)) return;

    AIDA::IBaseHistogram* histogram = baseHistogram_find(*session,ID);
    if(!histogram) {
      out << "pahist :" 
          << " can't find histogram " << Lib::sout(ID) 
          << Lib::endl;
      return;
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S") &&
         !OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"+") ){
        pawManager->plotterNext(*plotter);
      }
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"K")) {
        pawManager->setZONE_K(plotter->currentRegionNumber());
      }
      AIDA::IPlotterRegion* pregion = 0;
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"+")) {
        if(pawManager->ZONE_K()==NotFound) {         
          out << "pahist :" 
              << " option + : ZONE 'K' not set."
              << Lib::endl;
          delete plotter;
          return; 
        } else {
       	  pregion = plotter->region(pawManager->ZONE_K());
          if(!pregion) {
            out << "pahist :" 
                << " can't get region " << pawManager->ZONE_K()
                << Lib::endl;
            delete plotter;
            return; 
          }
        }
      } else {
        pregion = &(plotter->currentRegion());
      }
      AIDA::IPlotterRegion& region = *pregion;

      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"+")) {
        region.setParameter("plotter.superposeBins","TRUE");
      } else {
        region.setParameter("plotter.superposeBins","FALSE");
      }

      pawManager->regionSetLayout(region);

      if(histogram->dimension()==1) {
        AIDA::IHistogram1D* histogram1D = 
          Lib_Cast(*histogram,AIDA::IHistogram1D);
        if(histogram1D) {

          if(!region_setLimits(region,*histogram1D,dims,printer)) {
            delete plotter;
            return;
          }

          bool empty = true;
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"C")) {
            pawManager->plotHistogram1D(region,*histogram1D,"C",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"L")) {
            pawManager->plotHistogram1D(region,*histogram1D,"L",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"E")) {
            pawManager->plotHistogram1D(region,*histogram1D,"E",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"P")) {
            pawManager->plotHistogram1D(region,*histogram1D,"P",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"*")) {
            pawManager->plotHistogram1D(region,*histogram1D,"*",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"B")) {
            pawManager->plotHistogram1D(region,*histogram1D,"B",false); 
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty) {
            pawManager->plotHistogram1D(region,*histogram1D,"",false); 
          }
        } else {
          out << "pahist :" 
              << " cast problem."
              << Lib::endl;
        }

      } else if(histogram->dimension()==2) {
        AIDA::IHistogram2D* histogram2D = 
          Lib_Cast(*histogram,AIDA::IHistogram2D);
        if(histogram2D) {

          if(!region_setLimits(region,*histogram2D,dims,printer)) {
            delete plotter;
            return;
          }

          bool empty = true;
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"SURF")) {
            region_plot_histogram2D(*session,region,*histogram2D,"SURF",printer,0,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"LEGO")) {
            region_plot_histogram2D(*session,region,*histogram2D,"LEGO",printer,0,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"TEXT")) {
            region_plot_histogram2D(*session,region,*histogram2D,"TEXT",printer,0,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"CONT")) {
            region_plot_histogram2D(*session,region,*histogram2D,"CONT_DMOD",printer,10,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"BOX")) {
            region_plot_histogram2D(*session,region,*histogram2D,"BOX",printer,0,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"COL")) {
            region_plot_histogram2D(*session,region,*histogram2D,"COL",printer,0,*pawManager);
            empty = false;
          }
          if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"") && empty) {
            region_plot_histogram2D(*session,region,*histogram2D,"",printer,0,*pawManager); 
          }
        } else {
          out << "pahist :" 
              << " cast problem."
              << Lib::endl;
        }

      } else {
        out << "pahist :" 
            << " histo of dimension " << histogram->dimension()
            << " not treated."
            << Lib::endl;
      }

      delete plotter;
    }

  } else if(cmd_path=="/HISTOGRAM/FILE") {

    int LUN = ku_geti();
    std::string FNAME = ku_gets();
    /*int LRECL =*/ ku_geti();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    // In Unit : key=LUN, value=Unit.
    Units_type& units = pawManager->units();

    if(LUN==0) {
      // If LUN is 0 the next free logical unit will be used.
      const Lib::Parameters<int,OpenPAW::Unit>::List& lunits = units.list();
      Lib::Parameters<int,OpenPAW::Unit>::ConstIterator it;
      for(it=lunits.begin();it!=lunits.end();++it) {
        LUN = MAXIMUM(LUN,(*it).fKey);
      }
      LUN++;
    }

    if(units.isKey(LUN)) {
      out << "pahist :" 
          << " LUN " << LUN << " already in used by OpenPAW." 
          << Lib::endl;
      return;
    }

    std::string path,name,suffix;
    if(!Lib::smanip::pathNameSuffix(FNAME,path,name,suffix)) {
      out << "pahist :" 
          << " problem getting path,name,suffix in " << Lib::sout(FNAME)
          << Lib::endl;
      return;
    }
    std::string nameOnly = Lib::smanip::nosuffix(name);

    bool readOnly = true;
    bool createNew = false;
    if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"")) {
      // Existing file is opened (read mode only).      
    }
    if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"N")) {
      // A new file is opened.
      readOnly = false;
      createNew = true;
    }
    if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"U")) {
      // Existing file is opened to be modified.
      readOnly = false;
      createNew = false;
    }
    if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"D")) {
      // Reset lock.
    }

    std::string type;
    Lib::smanip::tolowercase(suffix);
    if( (suffix=="hbook") || (suffix=="rz") ) {
      // Look if BatchLabZebra DLL is around.
      bool zebraAvailable = false;
     {Lib::DLL dll("BatchLabZebra",printer,0,true);
      zebraAvailable = dll.isOpened();}
      if(!zebraAvailable) {
        out << "pahist :" 
            << " BatchLabZebra DLL not available"
            << " on this OpenPAW installation."
            << Lib::endl;
        out << "        " 
            << " We can't work on file " << Lib::sout(FNAME) << "."
            << Lib::endl;
        return;
      } 
      type = "ZEBRA";
    } else {
      type = suffix;
    }

    if(units.key
         (OpenPAW::Unit(OpenPAW::Unit::TREE,FNAME))!=units.keyNotFound()) {
      out << "pahist :" 
          << " " << Lib::sout(FNAME) << " already in used by OpenPAW." 
          << Lib::endl;
      return;
    }

    AIDA::ITreeFactory* treeFactory = aida->createTreeFactory();
    if(treeFactory) {
      std::string opts;
      Lib::smanip::printf(opts,64,"unit=%d",LUN);
      AIDA::ITree* tree = 
        treeFactory->create(FNAME,type,readOnly,createNew,opts);
      delete treeFactory;
      if(!tree) {
        out << "pahist :" 
            << " unable to create AIDA tree " << Lib::sout(FNAME)
            << Lib::endl;
      } else {
        units.add(LUN,"",OpenPAW::Unit(OpenPAW::Unit::TREE,FNAME),
                         OpenPAW::Unit(),OpenPAW::Unit());
        pawManager->setCurrentTree(FNAME);
        if( (type=="HIPPODRAW") || (type=="ASCII") ) {
          AIDA::ITupleFactory* tupleFactory = aida->createTupleFactory(*tree);
          if(tupleFactory) {
            tupleFactory->create(nameOnly,"","","");
            delete tupleFactory;
          }
        }

      }
    }

  } else if(cmd_path=="/HISTOGRAM/LIST") {

    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    AIDA::ITree* tree = pawManager->findCurrentTree();
    if(!tree) {
      out << " Not current tree."
          << Lib::endl;
      return;
    }
    tree->ls("",false);

  } else if(cmd_path=="/HISTOGRAM/DELETE") {

    std::string ID = ku_gets();

    // The H/DELETE of PAW admit an ID, 0 or * as argument.
    // OpenPAW extends that to the same logic as VECTOR, that is to say
    // an "HLIST". An HLIST being names, that may have wildcards,
    // separated by colon.

    std::string HLIST = ID;
    //Lib::smanip::touppercase(HLIST);
    if(HLIST=="0") HLIST = "*";

    std::string::size_type star = HLIST.find('*');
    if(star==std::string::npos){
      std::string NAME = HLIST;
      if(!memoryTree->rm(NAME)) { //ok.
        out << "pahist :" 
            << " unable to remove " << Lib::sout(NAME) 
            << Lib::endl;
      }
    } else { //wildcard.
      std::vector<std::string> objs = memoryTree->listObjectNames(".",true);
      std::vector<std::string> typs = memoryTree->listObjectTypes(".",true);
      if(typs.size()!=objs.size()) {
        out << "pahist :" 
            << " problem retreiving directory infos."
            << Lib::endl;
        return;
      }
      unsigned int objn = objs.size();
      for(unsigned int index = 0;index<objn;index++) {
        std::string& name = objs[index];
        std::string& type = typs[index];
        //printf("debug : %s %s\n",name.c_str(),type.c_str());
        if(type.find("IHistogram")!=std::string::npos) {
          if(Lib::smanip::match(name,HLIST)) {
            if(session->verboseLevel()) {
              out << "pahist :" 
                  << Lib::sout(name) << " matches " << Lib::sout(HLIST) 
                  << Lib::endl;
            }
            if(!memoryTree->rm(name)) { //ok.
              out << "pahist :" 
                  << " unable to remove " << Lib::sout(name) 
                  << " in tree " << memoryTree->storeName()
                  << Lib::endl;
            }
          }
        }
      }
    }

  } else if(cmd_path=="/HISTOGRAM/2D_PLOT/CONTOUR") {

    std::string ID = ku_gets();
    int NLEVEL = ku_geti();
    std::string CHOPT = ku_gets();
    if(session->verboseLevel()) {
      out << "pavect :" 
          << " " << cmd_path
          << " ID= " << Lib::sout(ID) 
          << " NLEVEL= " << NLEVEL
          << " CHOPT= " << Lib::sout(CHOPT) 
          << Lib::endl;
    }

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    //std::string PARAM = ku_gets(); //Vector

    std::string NAME;
    std::vector< std::pair<int,int> > dims;  
    if(!OpenPAW::PAW_Manager::parseName(ID,NAME,dims,printer)) return;

    AIDA::IHistogram2D* histogram = histogram2D_find(*session,NAME);
    if(!histogram) {
      out << "pahist :" 
          << " can't find histogram 2D " << Lib::sout(NAME) 
          << Lib::endl;
      return;
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);
  
      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      if(!region_setLimits(region,*histogram,dims,printer)) {
        delete plotter;
        return;
      }

      pawManager->regionSetLayout(region);
  
      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"3")) {
        //-3 The contour is drawn with filled colour levels.
        region_plot_histogram2D(*session,region,*histogram,
          "CONT_COL_FILLED",printer,NLEVEL,*pawManager);

      } else {
        // wire frame :
        //CHOPT 0 has priority over 1.
        if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"0")) {
          //-0 Use colour to distinguish contours.

          // In CERN/PAW The palette seems to be hard coded 
          // to one with 4 elements (1,2,3,4)
          double old_HCOL = atts.value("HCOL");
          std::string old_COLL = opts.value("COLL");
          atts.setValue("HCOL",0.01 * PALETTE_HISTOGRAM_2D_PLOT_CONTOUR_OPT_0);
          opts.setValue("COLL","COLV");
          region_plot_histogram2D(*session,region,*histogram,"CONT_COL",
                                   printer,NLEVEL,*pawManager);
          atts.setValue("HCOL",old_HCOL);
          opts.setValue("COLL",old_COLL);
        } else { //CHOPT 1,2
          //-1 Use line style to distinguish contours.
          //FIXME -2 Line style and colour are the same for all contours.
          region_plot_histogram2D(*session,region,*histogram,"CONT_DMOD",
                                   printer,NLEVEL,*pawManager);
        }

      }

      delete plotter;
    }

  } else if(cmd_path=="/HISTOGRAM/2D_PLOT/SURFACE") {

    std::string ID = ku_gets();
    /*double THETA =*/ ku_getr();
    /*double PHI =*/ ku_getr();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string NAME;
    std::vector< std::pair<int,int> > dims;  
    if(!OpenPAW::PAW_Manager::parseName(ID,NAME,dims,printer)) return;

    AIDA::IHistogram2D* histogram = histogram2D_find(*session,NAME);
    if(!histogram) {
      out << "pahist :" 
          << " can't find histogram 2D " << Lib::sout(NAME) 
          << Lib::endl;
      return;
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      if(!region_setLimits(region,*histogram,dims,printer)) {
        delete plotter;
        return;
      }

      pawManager->regionSetLayout(region);

      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,""))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"1"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"2"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"3"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"4"))

      if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"1")) {
        //CERN/PAW doc :
        //-1 Hidden surface algorithm is used and each cell is filled_
        //   with a colour corresponding to the Z value.
        region_plot_histogram2D(*session,region,*histogram,"SURF_COL",
                                printer,0,*pawManager);

      } else {
        region_plot_histogram2D(*session,region,*histogram,"SURF",
                                printer,0,*pawManager);
      }

      delete plotter;
    }

  } else if(cmd_path=="/HISTOGRAM/2D_PLOT/LEGO") {

    std::string ID = ku_gets();
    /*double THETA =*/ ku_getr();
    /*double PHI =*/ ku_getr();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    std::string NAME;
    std::vector< std::pair<int,int> > dims;  
    if(!OpenPAW::PAW_Manager::parseName(ID,NAME,dims,printer)) return;

    AIDA::IHistogram2D* histogram = histogram2D_find(*session,NAME);
    if(!histogram) {
      out << "pahist :" 
          << " can't find histogram 2D " << Lib::sout(NAME) 
          << Lib::endl;
      return;
    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      pawManager->setPlotterZONE(*plotter);

      if(!OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"S")) 
        pawManager->plotterNext(*plotter);
      AIDA::IPlotterRegion& region = plotter->currentRegion();

      if(!region_setLimits(region,*histogram,dims,printer)) {
        delete plotter;
        return;
      }

      pawManager->regionSetLayout(region);

      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,""))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"1"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"2"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"3"))
      //if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"4"))

      region_plot_histogram2D
        (*session,region,*histogram,"LEGO",printer,0,*pawManager);

      delete plotter;
    }

  } else {
    out << "pahist :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static bool region_setLimits(
 AIDA::IPlotterRegion& aRegion
,AIDA::IHistogram1D& aHistogram
,const std::vector< std::pair<int,int> >& aDims
,Slash::Core::IWriter& aPrinter
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aDims.size()) return true;
  if(aDims.size()!=1) {
    Lib::Out out(aPrinter);
    out << "region_setLimits :" 
        << " bad input dimension."
        << Lib::endl;
    return false;
  }
  int ixmn = aDims[0].first;
  int ixmx = aDims[0].second;
  int xbinn = aHistogram.axis().bins();
  if(ixmn==NotFound) ixmn = 0;
  if(ixmx==NotFound) ixmx = xbinn-1;
  if( (ixmn>ixmx) || (ixmn<0)||(ixmn>=xbinn) || (ixmx<0)||(ixmx>=xbinn) ) {
    Lib::Out out(aPrinter);
    out << "region_setLimits :" 
        << " bad binning "
        << " (xbins=" << xbinn 
        << ",ixmn=" << ixmn << ",ixmx=" << ixmx 
        << ")"
        << Lib::endl;
    return false;
  }
  double XLOW = aHistogram.axis().binLowerEdge(ixmn);
  double XUP = aHistogram.axis().binUpperEdge(ixmx);
  aRegion.setXLimits(XLOW,XUP);
  return true;
}
//////////////////////////////////////////////////////////////////////////////
static bool region_setLimits(
 AIDA::IPlotterRegion& aRegion
,AIDA::IHistogram2D& aHistogram
,const std::vector< std::pair<int,int> >& aDims
,Slash::Core::IWriter& aPrinter
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!aDims.size()) return true;
  if(aDims.size()!=2) {
    Lib::Out out(aPrinter);
    out << "region_setLimits :" 
        << " bad input dimension."
        << Lib::endl;
    return false;
  }
  int ixmn = aDims[0].first;
  int ixmx = aDims[0].second;
  int xbinn = aHistogram.xAxis().bins();
  if(ixmn==NotFound) ixmn = 0;
  if(ixmx==NotFound) ixmx = xbinn-1;
  int iymn = aDims[1].first;
  int iymx = aDims[1].second;
  int ybinn = aHistogram.yAxis().bins();
  if(iymn==NotFound) iymn = 0;
  if(iymx==NotFound) iymx = ybinn-1;
  if( (ixmn>ixmx) || (ixmn<0)||(ixmn>=xbinn) || (ixmx<0)||(ixmx>=xbinn) ||
      (iymn>iymx) || (iymn<0)||(iymn>=ybinn) || (iymx<0)||(iymx>=ybinn) ) {
    Lib::Out out(aPrinter);
    out << "region_setLimits :" 
        << " bad binning "
        << " (xbins=" << xbinn 
        << ",ixmn=" << ixmn << ",ixmx=" << ixmx 
        << ",ybins=" << ybinn 
        << ",iymn=" << iymn << ",iymx=" << iymx 
        << ")"
        << Lib::endl;
    return false;
  }
  double XLOW = aHistogram.xAxis().binLowerEdge(ixmn);
  double XUP = aHistogram.xAxis().binUpperEdge(ixmx);
  aRegion.setXLimits(XLOW,XUP);
  double YLOW = aHistogram.yAxis().binLowerEdge(iymn);
  double YUP = aHistogram.yAxis().binUpperEdge(iymx);
  aRegion.setYLimits(YLOW,YUP);
  return true;
}
