extern "C" {
  void papict_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/Manager.h>
#include <Lib/System.h>

#include <BatchLab/Helpers/AIDA.h>

#include <OpenPAW/PAW_Manager.h>

#include "KuipHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void papict_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "papict :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }
  
  OpenPAW::PAW_Manager* pawManager = find_PAW(*session);
  if(!pawManager) return;

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  ATTs_type& atts = pawManager->atts();

  if(cmd_path=="/PICTURE/IGSET") {

    std::string CHATT = ku_gets();
    double VALUE = ku_getr();
    Lib::smanip::touppercase(CHATT);

    if(CHATT=="SHOW") {  

      pawManager->showATTs();

    } else if(CHATT=="*") { 

      atts.setAllToDefault();

    } else if(!atts.isKey(CHATT)) {

      out << "papict :" 
          << " " << Lib::sout(CHATT) << " is not an attribute." 
          << Lib::endl;

    } else {

      if(VALUE==0) VALUE = atts.defaultValue(CHATT); //PAW convention.
      atts.setValue(CHATT,VALUE);
 
    }

  } else if(cmd_path=="/PICTURE/PRINT") {

    std::string FILE = ku_gets();
    /*int WIDTH =*/ ku_geti();  // used to scale .gif
    /*int HEIGHT =*/ ku_geti();
    std::string CHOPT = ku_gets();

    Lib::smanip::touppercase(CHOPT);
    std::vector<std::string> OPTS;
    kuip_getOptions("CHOPT",OPTS);

    bool sendToPrinter = false;
    if((FILE==" ")||(FILE=="HIGZPRINTER")) {
      FILE = "paw.ps";
      sendToPrinter = true; 
    }

    std::string stype;

    if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"PS")) {
      stype = "PS";
    } else if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"GIF")) {
      stype = "GIF";
    } else if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"JPEG")) {
      stype = "JPEG";
    } else if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"GL2PS")) {
      stype = "GL2PS";
    } else if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"GL2PDF")) {
      stype = "GL2PDF";
    } else if(OpenPAW::PAW_Manager::isAnOption(OPTS,CHOPT,"GL2SVG")) {
      stype = "GL2SVG";
    } else {

      std::string path,name,suffix;
      if(!Lib::smanip::pathNameSuffix(FILE,path,name,suffix)) return;
      Lib::smanip::touppercase(suffix);
      if(suffix=="EPS") {
        stype = "PS";
      } else if(suffix=="JPG") {
        stype = "JPEG";
      } else {
        stype = suffix;
      }

    }

    AIDA::IPlotter* plotter = create_plotter(*aida);
    if(plotter) {
      plotter->writeToFile(FILE,stype);
      delete plotter;
    }   

    if(sendToPrinter) {
      // If defined HIGZPRINTER should be of the form :
      //   csh> setenv HIGZPRINTER 'lp -dprinter_name paw.ps'
      //    sh> HIGZPRINTER='lp -dprinter_name paw.ps';export HIGZPRINTER
      std::string HIGZPRINTER;
      if(Lib::System::getenv("HIGZPRINTER",HIGZPRINTER)) {
        Lib::System::execute(HIGZPRINTER);
      }
    }

  } else {
    out << "papict :" 
        << " " << cmd_path << " : dummy." << Lib::endl;
  }
}
