extern "C" {
  void pnutil_(void*);
}

#include <kuip/ksys.h>
#include <kuip/kfor.h>
#include <kuip/kuser.h>
#include <kuip/klink.h>

#include <Slash/Core/ISession.h>
#include <Slash/Core/IWriter.h>

#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/smanip.h>
#include <Lib/fmanip.h>
#include <Lib/FileReader.h>
#include <Lib/Time.h>
#include <Lib/cstring.h>

#include <BatchLab/Helpers/AIDA.h>

#include <BatchLab/Core/amanip.h>

#include "FindHelpers.h"

//////////////////////////////////////////////////////////////////////////////
void pnutil_(
 void* aSession
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  Slash::Core::ISession* session = (Slash::Core::ISession*)aSession;

  Slash::Core::IWriter& printer = session->printer();
  Lib::Out out(printer);

  std::string cmd_path = ku_path() ? ku_path() : "";
  //std::string cmd_name = ku_name();
  if(session->verboseLevel()) {
    out << "pnutil :" 
        << " execute " << Lib::sout(cmd_path) 
        << Lib::endl;
  }

  AIDA::IAnalysisFactory* aida = find_aida(*session);
  if(!aida) return;

  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  //Lib::smanip smanip;

  if(cmd_path=="/NTUPLE/CREATE") {

    std::string IDN = ku_gets();
    std::string TITLE = ku_gets();
    /*int NVAR =*/ ku_geti();
    std::string CHRZPA = ku_gets();
    /*int NPRIME =*/ ku_geti();
    std::string VARLIST = ku_gets();
 
    if(memoryTree->find(IDN)) {
      memoryTree->rm(IDN); //ok.
      out << " *** WARNING in OpenPAW :"
          << " Already existing tuple replaced : ID="
          << IDN 
          << Lib::endl;
    }
    Lib::smanip::replace(VARLIST,","," ");

    AIDA::ITuple* tuple = create_tuple(*aida,*memoryTree,IDN,TITLE,VARLIST);
    if(!tuple) {
      out << "pnutil :" 
            << " can't create tuple " << Lib::sout(IDN) << Lib::endl;
      return;
    }

  } else if(cmd_path=="/NTUPLE/READ") {

    std::string IDN = ku_gets();
    std::string FNAME = ku_gets();
    std::string FORMAT = ku_gets();
    std::string OPT = ku_gets();
    /*int NEVENT =*/ ku_geti();
    std::string MATCH = ku_gets();
 
    AIDA::ITuple* tuple = find_tuple(*memoryTree,IDN);
    if(!tuple) {
      out << "pnutil :" 
          << " can't find tuple " << Lib::sout(IDN) 
          << " in tree " << Lib::sout(memoryTree->storeName()) 
          << Lib::endl;
      return;
    }

    Lib::FileReader reader(FNAME,printer);
    if(!reader.enabled()) {
      out << "pnutil :" 
          << " can't open file " << Lib::sout(FNAME) << Lib::endl;
      return;
    }

    std::vector<double> row(tuple->columns());

#define BUFSIZE 65536
    char buffer[BUFSIZE+1];

    Lib::Time begin = Lib::Time::now();

    unsigned int l;
    while(1) {
      if(!reader.sread(buffer,sizeof(buffer),l)) break; //EOF.
      Lib_strtods(buffer,row,true); //the last true is important.
      if(!tuple->fill(row)) {
        out << "pnutil :" 
            << " problem filling tuple from file" 
            << " " << Lib::sout(FNAME) << "."
            << Lib::endl;
        return;
      }
    }

    Lib::Time end = Lib::Time::elapsed(begin);

    out << " ==>   " << tuple->rows() << " events have been read" 
        << " in " << end.seconds() << " s (elapsed)."
        << Lib::endl;

  } else if(cmd_path=="/NTUPLE/PRINT") {

    std::string IDN = ku_gets();
 
    AIDA::ITree* tree = 0;
    AIDA::ITuple* tuple = tuple_find(*session,IDN,tree);
    if(!tuple) {
      out << "pnutil :" 
          << " object " << Lib::sout(IDN) 
          << " not an AIDA::ITuple."
          << Lib::endl;
      return;
    }

    BatchLab::amanip::hprint(*tuple,IDN,printer);

  } else {
    out << "pnutil :" 
        << " " << cmd_path << " : dummy." << Lib::endl;

  }
}
