//  Mai/2007 : G.Barrand : creation.

// this :
#include <Inventor/SDL/viewers/SoSDL_Viewer.h>

#include <Inventor/SbRotation.h>
#include <Inventor/SoSceneManager.h>
#include <Inventor/nodes/SoNode.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodekits/SoBaseKit.h>
#include <Inventor/sensors/SoNodeSensor.h>
#include <Inventor/actions/SoHandleEventAction.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/events/SoLocation2Event.h>

#include <Inventor/SDL/SbGL.h>

//////////////////////////////////////////////////////////////////////////////
SoSDL_Viewer::SoSDL_Viewer(
 SoSDL_Widget* aWidget
)
:fSceneManager(0)
,fSceneGraph(0)
,fUserSceneGraph(0)
,fCamera(0)
,fHeadLight(0)
,fDefaultDeltaRotation(1.0F * 3.1416F/180.0F) //1 degree
,fDefaultDeltaTranslation(1) 
,fCheckFloor(FALSE)
,fFloor(0)
,fJoy_rotate(true)
,fJoy_forback(true)
,fMove_what(ROTATE_RIGHT)
//NOTE : we must arrange to have key repeat 
//       at the same rythm than the gamepad.
,fMoveTimerInterval(30) //msec
,fMoveTimerID(0)
,fUnlockCB(0)
,fUnlockCB_Data(0)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fSceneManager = new SoSceneManager;

  aWidget->setViewer(this); //Used in the SoSDL::mainLoop.

  fCameraType = SoOrthographicCamera::getClassTypeId();

  fSceneGraph = new SoGroup;
  fSceneGraph->ref();

  //0
  SoLightModel* lightModel = new SoLightModel;
  lightModel->model.setValue(SoLightModel::PHONG);
  //lightModel->model.setValue(SoLightModel::BASE_COLOR);
  fSceneGraph->addChild(lightModel);

  //1
  fHeadLight = new SoDirectionalLight;
  fHeadLight->ref();
  fHeadLight->color.setValue(1,1,1);
  fHeadLight->direction.setValue(SbVec3f(1,-1,-10));
  fSceneGraph->addChild(fHeadLight);

  //2
  SoOrthographicCamera* camera = new SoOrthographicCamera();
  fSceneGraph->addChild(camera);
  setCamera(camera);

#define USER_SCENE_INDEX 3

  SDL_Surface* screen = SoSDL::screen();
  if(screen) {
    fSceneManager->setWindowSize(SbVec2s(screen->w,screen->h));
  }

  fSceneManager->setBackgroundColor(SbColor(0,0,0));

  fSceneManager->setSceneGraph(fSceneGraph);
}
//////////////////////////////////////////////////////////////////////////////
SoSDL_Viewer::~SoSDL_Viewer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fMoveTimerID) {
    SDL_RemoveTimer(fMoveTimerID);
  }
  if(fUserSceneGraph) {
    //fSceneGraph->removeChild(fUserSceneGraph);
    fUserSceneGraph->unref();
  }
  if(fCamera) fCamera->unref();
  if(fHeadLight) fHeadLight->unref();
  fSceneGraph->removeAllChildren();
  fSceneGraph->unref();
  delete fSceneManager;
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::setSceneGraph(
 SoNode* aScene
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aScene==fUserSceneGraph) return TRUE; //Done.

  if(fUserSceneGraph) {
    //fSceneGraph->removeChild(fUserSceneGraph);
    fUserSceneGraph->unref();
  }
  fUserSceneGraph = aScene;
  if(!fUserSceneGraph) return TRUE;

  fUserSceneGraph->ref();

  int number = fSceneGraph->getNumChildren();
  if(number==USER_SCENE_INDEX) {
    fSceneGraph->addChild(fUserSceneGraph);
  } else if(number==(USER_SCENE_INDEX+1)) {
    fSceneGraph->replaceChild(USER_SCENE_INDEX,fUserSceneGraph);
  } else {
    //error.
  }

  // Search for a camera in the user-supplied scenegraph.
  SbBool oldsearch = SoBaseKit::isSearchingChildren();
  SoBaseKit::setSearchingChildren(TRUE);
  SoSearchAction* searchAction = new SoSearchAction;
  searchAction->reset();
  searchAction->setType(SoCamera::getClassTypeId());
  searchAction->apply(fUserSceneGraph);
  SoBaseKit::setSearchingChildren(oldsearch);

  SoCamera* sceneCamera = 0;
  if(searchAction->getPath()) {
    SoFullPath* fullpath = (SoFullPath*)searchAction->getPath();
    SoNode* node = fullpath->getTail();
    if(node->isOfType(SoCamera::getClassTypeId())==FALSE) return FALSE;
    sceneCamera = (SoCamera*)node;
  }
  delete searchAction;

  // Make our own camera if none was available.
  /*
  if (!sceneCamera) {
    sceneCamera = (SoCamera*)fCameraType.createInstance();
    //PRIVATE(this)->deletecamera = TRUE;
  }
  */

  if(sceneCamera) setCamera(sceneCamera);

  //FIXME : Should notify the fSceneManager.
  return TRUE;
}
//////////////////////////////////////////////////////////////////////////////
SoNode* SoSDL_Viewer::getSceneGraph (
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fUserSceneGraph;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setCamera (
 SoCamera* aCamera
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aCamera==fCamera) return; //Done.
  if(fCamera) fCamera->unref();
  fCamera = aCamera;
  if(fCamera) fCamera->ref();
}
//////////////////////////////////////////////////////////////////////////////
SoCamera* SoSDL_Viewer::getCamera (
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCamera;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setCameraType (
 SoType aType
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCameraType = aType;
}
//////////////////////////////////////////////////////////////////////////////
SoType SoSDL_Viewer::getCameraType(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fCameraType;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setViewportRegion (
 const SbViewportRegion& aNewRegion
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fSceneManager->setViewportRegion(aNewRegion); 
}
//////////////////////////////////////////////////////////////////////////////
const SbViewportRegion& SoSDL_Viewer::getViewportRegion (
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fSceneManager->getViewportRegion(); 
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setBackgroundColor (
 const SbColor& aC
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  fSceneManager->setBackgroundColor(aC); 
}
//////////////////////////////////////////////////////////////////////////////
const SbColor& SoSDL_Viewer::getBackgroundColor (
) const 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{ 
  return fSceneManager->getBackgroundColor(); 
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setSize(
 const SbVec2s&
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //Must stay dummy since size is the full screen.
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::viewAll(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  if(!fCamera) return;
  // No shape under fSceneGraph, take directly fUserSceneGraph.
  if(!fUserSceneGraph) return;
  fCamera->viewAll(fUserSceneGraph,getViewportRegion());
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setViewing(
  SbBool
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::show(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  SoSDL::sendRender();
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setGLRenderAction(
 SoGLRenderAction*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
}
//////////////////////////////////////////////////////////////////////////////
SoGLRenderAction* SoSDL_Viewer::getGLRenderAction(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  return fSceneManager->getGLRenderAction();
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setTitle(
 const char*
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setHeadlight(
 SbBool aOnOff
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  fHeadLight->on.setValue(aOnOff);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::render(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
  if(!fSceneGraph) return;

  glEnable(GL_DEPTH_TEST); //FIXME : do it through the fSceneManager

  fSceneManager->render();
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setAutoClipping(
  SbBool
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setDecoration(
  SbBool
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{   
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::processEvent(
 SDL_Event& aEvent
,SbBool& aRender
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if( (aEvent.type==SDL_MOUSEBUTTONDOWN) || 
      (aEvent.type==SDL_MOUSEBUTTONUP) ) {
    
    // Build SoEvent.
    SoMouseButtonEvent event;
    
    // WARNING : for Coin, BUTTON2 is right button, BUTTON3 is middle button.
    switch(aEvent.button.button){
    case SDL_BUTTON_LEFT:
      event.setButton(SoMouseButtonEvent::BUTTON1);
      break;
    case SDL_BUTTON_RIGHT:
      event.setButton(SoMouseButtonEvent::BUTTON2);
      break;
    case SDL_BUTTON_MIDDLE:
      event.setButton(SoMouseButtonEvent::BUTTON3);
      break;
    default:
      event.setButton(SoMouseButtonEvent::ANY);
      break;
    }
    if(aEvent.button.state==SDL_PRESSED)   
      event.setState(SoButtonEvent::DOWN);
    else if(aEvent.button.state==SDL_RELEASED) 
      event.setState(SoButtonEvent::UP);
    else
      event.setState(SoButtonEvent::UNKNOWN);
      
    SDLMod state = SDL_GetModState();
    if((state & KMOD_LSHIFT) == KMOD_LSHIFT) event.setShiftDown (TRUE);
    if((state & KMOD_RSHIFT) == KMOD_RSHIFT) event.setShiftDown (TRUE);
    if((state & KMOD_LCTRL)  == KMOD_LCTRL)  event.setCtrlDown  (TRUE);
    if((state & KMOD_RCTRL)  == KMOD_RCTRL)  event.setCtrlDown  (TRUE);
    if((state & KMOD_LALT)   == KMOD_LALT)   event.setAltDown   (TRUE);  
    if((state & KMOD_RALT)   == KMOD_RALT)   event.setAltDown   (TRUE);  
    
    //SDL : origin is top-left with y down (like X11).
    //Inventor :  origin is bottom-left with y up (like OpenGL).
    const SbVec2s& win = getViewportRegion().getWindowSize();
    event.setPosition(SbVec2s(short(aEvent.button.x),
                              win[1] - short(aEvent.button.y)));
    
    SbBool status = fSceneManager->processEvent(&event);
    aRender = status;
    return status;

  } else if(aEvent.type==SDL_MOUSEMOTION) {
    
    // Build SoEvent.
    SoLocation2Event event;
    
    SDLMod state = SDL_GetModState();
    if((state & KMOD_LSHIFT) == KMOD_LSHIFT) event.setShiftDown (TRUE);
    if((state & KMOD_RSHIFT) == KMOD_RSHIFT) event.setShiftDown (TRUE);
    if((state & KMOD_LCTRL)  == KMOD_LCTRL)  event.setCtrlDown  (TRUE);
    if((state & KMOD_RCTRL)  == KMOD_RCTRL)  event.setCtrlDown  (TRUE);
    if((state & KMOD_LALT)   == KMOD_LALT)   event.setAltDown   (TRUE);  
    if((state & KMOD_RALT)   == KMOD_RALT)   event.setAltDown   (TRUE);  
    
    //SDL : origin is top-left with y down (like X11).
    //Inventor :  origin is bottom-left with y up (like OpenGL).
    const SbVec2s& win = getViewportRegion().getWindowSize();
    event.setPosition(SbVec2s(short(aEvent.motion.x),
                              win[1] - short(aEvent.motion.y)));
    
    SbBool status = fSceneManager->processEvent(&event);
    aRender = status;
    return status;

  } else if(aEvent.type==SDL_JOYBUTTONDOWN) {
#ifdef SDL_DEBUG_EVENT
    printf("debug : SDL_JOYBUTTON : which %d, button %d, state %d\n",aEvent.jbutton.which,aEvent.jbutton.button,aEvent.jbutton.state);        
#endif

     // on a Logitech Precision the button num = jbutton.button+1

     if(aEvent.jbutton.button==0) { 
       fJoy_rotate = fJoy_rotate?false:true;
       aRender = FALSE;
       return TRUE;
     }
     if(aEvent.jbutton.button==1) { 
       fJoy_forback = fJoy_forback?false:true;
       aRender = FALSE;
       return TRUE;
     }
     if((aEvent.jbutton.button==4)|| //shoot
        (aEvent.jbutton.button==5)) {
       return emulateLeftButtonDown(aRender);
     }

  } else if(aEvent.type==SDL_JOYBUTTONUP) {

     // on a Logitech Precision the button num = jbutton.button+1

     if((aEvent.jbutton.button==4)|| //shoot
        (aEvent.jbutton.button==5)) {
       return emulateLeftButtonUp(aRender);
     }

  } else if(aEvent.type==SDL_JOYAXISMOTION) {
#ifdef SDL_DEBUG_EVENT
    printf("debug : SDL_JOYAXISMOTION : which %d, axis %d, value %d\n",aEvent.jaxis.which,aEvent.jaxis.axis,aEvent.jaxis.value);        
#endif

    if(aEvent.jaxis.axis==0) {

      if((aEvent.jaxis.value>-8000)&&(aEvent.jaxis.value<8000)) {
        stopMoveTimer();
      } else {
        if(fMoveTimerID) {
          stopMoveTimer();
        } else {
          if(aEvent.jaxis.value>0) {
            fMove_what = (fJoy_rotate?
                            SoSDL_Viewer::ROTATE_RIGHT:
                            SoSDL_Viewer::TRANSLATE_RIGHT);
          } else {
            fMove_what = (fJoy_rotate?
                            SoSDL_Viewer::ROTATE_LEFT:
                            SoSDL_Viewer::TRANSLATE_LEFT);
          }
          startMoveTimer();
        }            
      }            

      aRender = FALSE;
      return TRUE;

    } else if(aEvent.jaxis.axis==1) {

      if((aEvent.jaxis.value>-8000)&&(aEvent.jaxis.value<8000)) {
        stopMoveTimer();
      } else {
        if(fMoveTimerID) {
          stopMoveTimer();
        } else {
          if(aEvent.jaxis.value>0) {
            fMove_what = (fJoy_forback?
                            SoSDL_Viewer::BACKWARD:
                            SoSDL_Viewer::DOWN);
          } else {
            fMove_what = (fJoy_forback?
                            SoSDL_Viewer::FORWARD:
                            SoSDL_Viewer::UP);
          }
          startMoveTimer();
        }            
      }            

      aRender = FALSE;
      return TRUE;
    }

  } else if(aEvent.type==SDL_KEYDOWN) {

#ifdef SDL_DEBUG_EVENT
     printf("debug : SDL_KEYDOWN : which %d, state %d\n",aEvent.key.which,aEvent.key.state);        
#endif
     if( (aEvent.key.keysym.sym==SDLK_RIGHT)||
         (aEvent.key.keysym.sym==SDLK_d) ){
       if(SoSDL::isShiftDown()) {
         updateMotion(SoSDL_Viewer::TRANSLATE_RIGHT);
       } else {
         updateMotion(SoSDL_Viewer::ROTATE_RIGHT);
       }
       aRender = TRUE;
       return TRUE;
     } else if( (aEvent.key.keysym.sym==SDLK_LEFT)||
                (aEvent.key.keysym.sym==SDLK_a) ){
       if(SoSDL::isShiftDown()) {
         updateMotion(SoSDL_Viewer::TRANSLATE_LEFT);
       } else {
         updateMotion(SoSDL_Viewer::ROTATE_LEFT);
       }
       aRender = TRUE;
       return TRUE;
     } else if( (aEvent.key.keysym.sym==SDLK_UP) ||
                (aEvent.key.keysym.sym==SDLK_w) ){
       if(SoSDL::isShiftDown()) {
         updateMotion(SoSDL_Viewer::UP);
       } else {
         updateMotion(SoSDL_Viewer::FORWARD);
       }
       aRender = TRUE;
       return TRUE;
     } else if( (aEvent.key.keysym.sym==SDLK_DOWN) ||
                (aEvent.key.keysym.sym==SDLK_s) ){
       if(SoSDL::isShiftDown()) {
         updateMotion(SoSDL_Viewer::DOWN);
       } else {
         updateMotion(SoSDL_Viewer::BACKWARD);
       }
       aRender = TRUE;
       return TRUE;
     }

  }

  aRender = FALSE;
  return FALSE;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::rotateCameraPositionAroundX(
 float aDeltaAngle
)
//////////////////////////////////////////////////////////////////////////////
// From CoinXt/SoGuiExaminerViewerP::rotXWheelMotion.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;

  SbVec3f dir;
  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);

  SbVec3f focalpoint = 
    fCamera->position.getValue() + fCamera->focalDistance.getValue() * dir;

  fCamera->orientation = 
    SbRotation(SbVec3f(-1,0,0),aDeltaAngle) * fCamera->orientation.getValue();

  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);

  SbVec3f pos = focalpoint - fCamera->focalDistance.getValue() * dir;
  setCameraPosition(pos);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::rotateCameraPositionAroundY(
 float aDeltaAngle
)
//////////////////////////////////////////////////////////////////////////////
// From CoinXt/SoGuiExaminerViewerP::rotYWheelMotion.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;

  SbVec3f dir;
  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);

  SbVec3f focalpoint = 
    fCamera->position.getValue() + fCamera->focalDistance.getValue() * dir;

  fCamera->orientation = 
    SbRotation(SbVec3f(0,1,0),aDeltaAngle) * fCamera->orientation.getValue();

  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);

  SbVec3f pos = focalpoint - fCamera->focalDistance.getValue() * dir;
  setCameraPosition(pos);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::rotateCameraAroundUp(
 float aDeltaAngle
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;
  SbVec3f up;
  fCamera->orientation.getValue().multVec(SbVec3f(0,1,0),up);
  fCamera->orientation = 
    SbRotation(up,aDeltaAngle) * fCamera->orientation.getValue();
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::translateCameraAlongDir(
 float aDelta
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;

  SbVec3f dir;
  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);

  SbVec3f pos = fCamera->position.getValue() + aDelta * dir;
  setCameraPosition(pos);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::translateCameraAlongSide(
 float aDelta
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;

  SbVec3f dir;
  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);
  SbVec3f up;
  fCamera->orientation.getValue().multVec(SbVec3f(0,1,0),up);

  SbVec3f side = dir.cross(up);

  SbVec3f pos = fCamera->position.getValue() + aDelta * side;
  setCameraPosition(pos);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::translateCameraAlongUp(
 float aDelta
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fCamera) return;

  SbVec3f dir;
  fCamera->orientation.getValue().multVec(SbVec3f(0,0,-1),dir);
  SbVec3f up;
  fCamera->orientation.getValue().multVec(SbVec3f(0,1,0),up);

  SbVec3f pos = fCamera->position.getValue() + aDelta * up;
  setCameraPosition(pos);
}
//////////////////////////////////////////////////////////////////////////////
float SoSDL_Viewer::getDefaultDeltaRotation(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDefaultDeltaRotation;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setDefaultDeltaRotation(
 float aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fDefaultDeltaRotation = aValue;
}
//////////////////////////////////////////////////////////////////////////////
float SoSDL_Viewer::getDefaultDeltaTranslation(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fDefaultDeltaTranslation;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setDefaultDeltaTranslation(
 float aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fDefaultDeltaTranslation = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setMoveTimerInterval(
 unsigned int aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fMoveTimerInterval = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::checkFloor(
 SbBool aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fCheckFloor = aValue;
}
//////////////////////////////////////////////////////////////////////////////
float SoSDL_Viewer::getFloor(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fFloor;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setFloor(
 float aValue
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fFloor = aValue;
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::setCameraPosition(
 const SbVec3f& aPos
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fCheckFloor==FALSE) {
    fCamera->position.setValue(aPos);
  } else {
    if(aPos[1]>fFloor) fCamera->position.setValue(aPos);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::updateMotion( 
 Move aWhat
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float deltaRot = getDefaultDeltaRotation();
  float deltaTrans = getDefaultDeltaTranslation();

  if(aWhat==ROTATE_RIGHT) {
    rotateCameraAroundUp(-deltaRot);
  } else if(aWhat==ROTATE_LEFT) {
    rotateCameraAroundUp(deltaRot);

  } else if(aWhat==TRANSLATE_RIGHT) {
    translateCameraAlongSide(deltaTrans);
  } else if(aWhat==TRANSLATE_LEFT) {
    translateCameraAlongSide(-deltaTrans);

  } else if(aWhat==UP) {
    translateCameraAlongUp(deltaTrans);
  } else if(aWhat==DOWN) {
    translateCameraAlongUp(-deltaTrans);

  } else if(aWhat==FORWARD) {
    translateCameraAlongDir(deltaTrans);
  } else if(aWhat==BACKWARD) {
    translateCameraAlongDir(-deltaTrans);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::startMoveTimer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fMoveTimerID) SDL_RemoveTimer(fMoveTimerID);
  fMoveTimerID = SDL_AddTimer(fMoveTimerInterval,moveTimerCB,this);
}
//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::stopMoveTimer(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fMoveTimerID) SDL_RemoveTimer(fMoveTimerID);
  fMoveTimerID = 0;
}

//////////////////////////////////////////////////////////////////////////////
Uint32 SoSDL_Viewer::moveTimerCB(
 Uint32 aInterval
,void* aTag
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSDL_Viewer* This = (SoSDL_Viewer*)aTag;
  This->updateMotion(This->fMove_what);
  //SoSDL::setViewerRender();
  SoSDL::sendRender();
  //return 0; //Stop the timer. (Returning aInterval would continue the timer).
  return aInterval;
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::emulateLeftButtonDown(
 SbBool& aRender
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SDL_Event event;
  event.type = SDL_MOUSEBUTTONDOWN;
  event.button.button = SDL_BUTTON_LEFT;
  event.button.state = SDL_PRESSED;
  const SbVec2s& win = getViewportRegion().getWindowSize();
  event.button.x = win[0]/2;
  event.button.y = win[1]/2;    
  return processEvent(event,aRender);
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::emulateLeftButtonUp(
 SbBool& aRender
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SDL_Event event;
  event.type = SDL_MOUSEBUTTONUP;
  event.button.button = SDL_BUTTON_LEFT;
  event.button.state = SDL_RELEASED;
  const SbVec2s& win = getViewportRegion().getWindowSize();
  event.button.x = win[0]/2;
  event.button.y = win[1]/2;    
  return processEvent(event,aRender);
}

//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::emulateMouseMotion(
 SbBool& aRender
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SDL_Event event;
  event.type = SDL_MOUSEMOTION;
  //FIXME event.motion.which = ;
  //FIXME event.motion.state = ;
  const SbVec2s& win = getViewportRegion().getWindowSize();
  event.motion.x = win[0]/2;
  event.motion.y = win[1]/2;    
  //FIXME event.motion.xrel = 0;
  //FIXME event.motion.yrel = 0;
  return processEvent(event,aRender);
}

//////////////////////////////////////////////////////////////////////////////
void SoSDL_Viewer::lock(
 const SbString& aPass
,SoSDL_UnlockCB aCB
,void* aData
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPass = aPass;
  fRead = 0;
  fUnlockCB = aCB;
  fUnlockCB_Data = aData;
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoSDL_Viewer::processEventLock(
 SDL_Event& aEvent
,SbBool& aRender
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aEvent.type==SDL_USEREVENT) return FALSE; //do not trap timers.

  aRender = FALSE;
  if(!fPass.getLength()) return FALSE;

  //printf("debug : pass event : 000\n");        
  switch (aEvent.type) {
  case SDL_KEYDOWN:{
    if(aEvent.key.keysym.sym==SDLK_RETURN) {
      if(fRead==fPass) { //unlock.
        fPass.makeEmpty();
        fRead.makeEmpty();
        if(fUnlockCB) {
          fUnlockCB(fUnlockCB_Data);
          aRender = TRUE;
        }
      } else {
        fRead.makeEmpty();
      }
    } else {
      if((aEvent.key.keysym.unicode & 0xFF80)==0) {
        char ch = aEvent.key.keysym.unicode & 0x7F;
        //printf("debug : pass event : char : uni %d : |%c| %d\n",
          //event.key.keysym.unicode,ch,ch);        
        fRead += ch;

      }
    }
  }break;
  default:
    break;			
  }

  return TRUE;
}
