//  Mai/2007 : G.Barrand : creation.

// The scene is a grey floor with a red cube over it.
// At startup, the camera is slightly on top of the floor
// looking at the cube.

#include <Inventor/SDL/SoSDL.h>
#include <Inventor/SDL/viewers/SoSDL_Viewer.h>

#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/nodes/SoTranslation.h>

#include <math.h>
#ifndef M_PI
#define M_PI       3.1415926535897931160E0
#define M_PI_2     1.5707963267948965580E0  
#endif

#define FM_PI      ((float)M_PI)
#define FM_PI_2    ((float)M_PI_2)

//Windows, Darwin : "main" becomes "SDL_main" through a cpp macro in SDL.h.

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
int main( 
 int aArgc
,char* aArgv[]
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{

  ///////////////////////////////////////////////
  // Init :
  ///////////////////////////////////////////////
  SoSDL_Widget* topWidget = SoSDL::init(640,480,SDL_FULLSCREEN);
  if(!topWidget) return 1;

  SDL_Surface* screen = SoSDL::screen();
  if(!screen) return 1;

  SoSeparator* sceneGraph = new SoSeparator;
  sceneGraph->ref();

  ///////////////////////////////////////////////
  // Camera (us) :
  ///////////////////////////////////////////////
  // Head light :
 {SoDirectionalLight* soLight = new SoDirectionalLight;
  soLight->color.setValue(1,1,1);
  // Same as Coin<Gui>/So<Gui>Viewer :
  soLight->direction.setValue(SbVec3f(1,-1,-10));
  sceneGraph->addChild(soLight);}

  SoPerspectiveCamera* soCamera = new SoPerspectiveCamera;
  soCamera->viewportMapping.setValue(SoCamera::ADJUST_CAMERA);
  float d = 100;
  soCamera->position.setValue(0,2,d); //us at y = 2.
  soCamera->focalDistance.setValue(d);
  soCamera->orientation.setValue(SbRotation(SbVec3f(0,1,0),0));
  soCamera->nearDistance.setValue(d*0.01F);
  soCamera->farDistance.setValue(100*d);
  soCamera->heightAngle.setValue(FM_PI/4.0F);
  sceneGraph->addChild(soCamera);

  // Light at ceil :
 {SoPointLight* soLight = new SoPointLight;
  soLight->location.setValue(0,100,0); //y = 100.
  sceneGraph->addChild(soLight);}

  SoLightModel* soLightModel = new SoLightModel;
  soLightModel->model.setValue(SoLightModel::PHONG);
  sceneGraph->addChild(soLightModel);

  ///////////////////////////////////////////////
  // Floor :
  ///////////////////////////////////////////////
 {SoSeparator* floor = new SoSeparator;
  sceneGraph->addChild(floor);
  
  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(SbColor(1,1,1));
  floor->addChild(material);

  SoCube* soCube = new SoCube;
  soCube->width.setValue(10*d);
  soCube->height.setValue(0.01F);
  soCube->depth.setValue(10*d);
  floor->addChild(soCube);}

  ///////////////////////////////////////////////
  // Scene :
  ///////////////////////////////////////////////
 {SoSeparator* scene = new SoSeparator;
  sceneGraph->addChild(scene);

  SoMaterial* material = new SoMaterial;
  material->diffuseColor.setValue(SbColor(1,0,0));
  scene->addChild(material);

  SoTranslation* trans = new SoTranslation;
  trans->translation.setValue(SbVec3f(0,10,0)); //y = 20
  scene->addChild(trans);

  SoCube* soCube = new SoCube;
  soCube->width.setValue(10.0F);
  soCube->height.setValue(10.0F);
  soCube->depth.setValue(10.0F);
  scene->addChild(soCube);}

  // Viewer :
  SoSDL_Viewer* viewer = new SoSDL_Viewer(topWidget);
  viewer->setSceneGraph(sceneGraph);
  viewer->setCamera(soCamera);
  viewer->setHeadlight(FALSE);
  viewer->setDefaultDeltaTranslation(1);
  viewer->show();
    
  SoSDL::show(topWidget);

  SoSDL::mainLoop();
    
  sceneGraph->unref();

  delete viewer;

  SoSDL::clear();  
	
  std::cout << "exiting..." << std::endl;

  return 0;
}
