//    This file is part of dvi2bitmap.
//    Copyright 1999--2002, Council for the Central Laboratory of the Research Councils
//    
//    This program is part of the Starlink Software Distribution: see
//    http://www.starlink.ac.uk 
//
//    dvi2bitmap is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    dvi2bitmap is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with dvi2bitmap; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    The General Public License is distributed along with this
//    program in the file LICENCE.
//
//    Author: Norman Gray <norman@astro.gla.ac.uk>
//    $Id: Util.cc,v 1.6 2005/07/06 10:25:48 barrand Exp $


#include <config.h>

#include <iostream>

#ifdef HAVE_CSTD_INCLUDE
#  include <cstdio>
#  include <cctype>
#  include <cstdlib> //LAL
#  if CCTYPE_IN_STD
using std::isspace;
using std::isxdigit;
#  endif
#else
#  include <stdio.h>
#  include <ctype.h>
#  include <stdlib.h> //LAL
#endif
#ifndef WIN32 //LAL : Laurent Garnier
#include <unistd.h>		// this is standard according to single-unix, 
				// how POSIXy is that?  How C++?
#else
#include <io.h>
#endif
				// how POSIXy is that?  How C++?
//#include <sys/wait.h>
//#include <map>

using STD::ostream;		// following are used several times
using STD::ends;
using STD::endl;
using STD::cerr;
using STD::strtoul;
using STD::strncpy;

#include "Util.h"
#include "stringstream.h"

/**
 * Various utility functions.
 */
namespace Util
{
    verbosities verbosity_ = normal;
}


/** Tokenise string at whitespace.
 *
 * @param str the string to be tokenised
 * @return a list containing the whitespace-separated tokens in the string
 */
string_list& Util::tokenise_string (string str)
{
    static string_list *l;

    if (verbosity_ > normal)
	cerr << "tokenise_string: string=<" << str << ">" << endl;

    if (l == 0)
	l = new string_list();
    else
	l->clear();

    unsigned int i=0;

    // skip leading whitespace
    while (i < str.length() && isspace(str[i]))
	i++;
    while (i < str.length())
    {
	unsigned int wstart = i;
	while (i < str.length() && !isspace(str[i]))
	    i++;
	string t = str.substr(wstart,i-wstart);
	if (verbosity_ > normal)
	    cerr << "tokenise:" << t << ":" << endl;
	l->push_back(t);
	while (i < str.length() && isspace(str[i]))
	    i++;
    }
    return *l;
}

//LAL : Laurent Garnier : Change to read Dvi color Files
/**
 * Get color from a dvi string
 * @param rgb the <code>BitmapColour</code> corresponding to the
 * <code>s</code> argument
 * @param s the dvi specification
 * @author Laurent Garnier
 */
static unsigned int sColorNumbers = 68;
static string sColorValue[68] = {"#D9FF4F","#FFFF00","#FFE628","#FFB528","#FFAE7A",
                                  "#FF804C","#FF8A80","#FF9400","#FF6321","#FF7D00",
                                  "#C20200","#FF3A21","#A60000","#AE0000","#B80000",
                                  "#FF0000","#FF0080","#FF00DE","#FF0A9C","#FF789E",
                                  "#FF5EFF","#FF00FF","#FF30FF","#FF2EFF","#A314FA",
                                  "#9700A8","#7302EB","#FF85FF","#E168FF","#AE5CFF",
                                  "#9933CC","#8C23FF","#8000FF","#351EFF","#4019FF",
                                  "#190CF5","#6E73FF","#616EC5","#59DEFF","#007091",
                                  "#0F75FF","#0080FF","#0000FF","#0FE3FF","#00FFFF",
                                  "#0AFFFF","#61FFE1","#26FFCC","#1EFAA3","#2EFFB3",
                                  "#26FFAB","#00FF80","#02FF7A","#4FFF80","#00FF00",
                                  "#00E100","#00C028","#80FF00","#8FFF42","#BDFF3D",
                                  "#009900","#8C0000","#4C0000","#660000","#DC9470",
                                  "#808080","#000000","#FFFFFF"};
static string sColorName[68] = {"GreenYellow","Yellow","Goldenrod","Dandelion","Apricot",
                                 "Peach","Melon","YellowOrange","Orange","BurntOrange",
                                 "Bittersweet","RedOrange","Mahogany","Maroon","BrickRed",
                                 "Red","OrangeRed","RubineRed","WildStrawberry","Salmon",
                                 "CarnationPink","Magenta","VioletRed","Rhodamine","Mulberry",
                                 "RedViolet","Fuchsia","Lavender","Thistle","Orchid",
                                 "DarkOrchid","Purple","Plum","Violet","RoyalPurple",
                                 "BlueViolet","Periwinkle","CadetBlue",
                                 "CornflowerBlue","MidnightBlue",
                                 "NavyBlue","RoyalBlue","Blue","Cerulean","Cyan",
                                 "ProcessBlue","SkyBlue","Turquoise","TealBlue","Aquamarine",
                                 "BlueGreen","Emerald","JungleGreen","SeaGreen","Green",
                                 "ForestGreen","PineGreen","LimeGreen","YellowGreen","SpringGreen",
                                 "OliveGreen","RawSienna","Sepia","Brown","Tan",
                                 "Gray","Black","White"};
bool Util::parseColor (Bitmap::BitmapColour& rgb, string color)
{
  string_list l = Util::tokenise_string (color);
  string_list::const_iterator s = l.begin();
  if (s == l.end()) return false;
  // read color type
  if (*s == "rgb") {                          // rgb type
    s++;
    if (s == l.end()) return false;

    // in latex, color are specify between 0-1
    char* t;
    double aValue;
    aValue = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (aValue>1))
      return false;

    if(aValue == 1) aValue = 0.999; // 1.0->255
    rgb.red   = static_cast<Byte>(aValue*256);
    s++;
    if (s == l.end())
      return false;

    aValue = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (aValue>1))
      return false;

    if(aValue == 1) aValue = 0.999; // 1.0->255
    rgb.green = static_cast<Byte>(aValue*256);
    s++;
    if (s == l.end())
      return false;

    aValue = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (aValue>1))
      return false;

    if(aValue == 1) aValue = 0.999; // 1.0->255
    rgb.blue  = static_cast<Byte>(aValue*256);

    return true;

  } else if (*s == "gray") {                      // gray
    s++;
    if (s == l.end())
      return false;
    char* t;
    double aValue;
    aValue = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (aValue>1))
      return false;
    
    if(aValue == 1) aValue = 0.999; // 1.0->255
    rgb.red   = static_cast<Byte>(aValue*256);
    rgb.green = static_cast<Byte>(aValue*256);
    rgb.blue  = static_cast<Byte>(aValue*256);

    return true;
 
  } else if ((*s == "cmy") || (*s == "cmyk")) { // cmyk or cmy
    s++;
    if (s == l.end())
      return false;
    
    char* t;
    double cyan,magenta,yellow,black;
    black =0;
    cyan = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (cyan>1))
      return false;
    
    if(cyan == 1) cyan = 0.999; // 1.0->255
    s++;
    if (s == l.end())
      return false;
    
    magenta = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (magenta>1))
      return false;
    
    if(magenta == 1) magenta = 0.999; // 1.0->255
    s++;
    if (s == l.end())
      return false;
    
    yellow = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (yellow>1))
      return false;
    
    if(yellow == 1) yellow = 0.999; // 1.0->255
    s++;
    if (s == l.end())
      return false;
    
    black = ::strtod(s->c_str(),&t);
    if( (t==s->c_str()) || (t!=(s->c_str()+s->size())) || (black>1))
      return false;
    
    if(black == 1) black = 0.999; // 1.0->255
    if ((cyan>=0) && (cyan<1) &&
        (magenta>=0) && (magenta<1) &&
        (yellow>=0) && (yellow<1) &&
        (black>=0) && (black<1)) {
      rgb.red   = static_cast<Byte>((1.0-(cyan*(1-black)+black))*256.0);
      rgb.green = static_cast<Byte>((1.0-(magenta*(1-black)+black))*256.0);
      rgb.blue  = static_cast<Byte>((1.0-(yellow*(1-black)+black))*256.0);
      
      return  true;
    }
  } else { //                                  read color
    if (string(*s).substr(0,2) == "##") { // add # in front of color in latex file
      // give 2 # in dvi file
      return Util::parseRGB (rgb,string(*s).substr(1).c_str());
    } else if (string(*s).substr(0,1) == "#") {
      return Util::parseRGB (rgb,color.c_str());
    } else {  // is it a reconise color name ?
      for (unsigned int a=0;a<sColorNumbers;a++) {
        if (sColorName[a].compare(string(*s)) ==0 ) {
          return Util::parseRGB (rgb,sColorValue[a].c_str());
          break;
        }
      }
    }
  }
  return false;
}
//LAL : end


/**
 * Parse an RGB specification.  This is either a sequence of three integers
 * separated by slashes (or in fact any non-number character), or else
 * a string of the form <code>#RRGGBB</code>.  Set the `rgb' structure to the
 * resulting numbers.  The integers must be in the range [0,255], and
 * may be specified in decimal, octal, or hex.
 *
 * @param rgb the <code>BitmapColour</code> corresponding to the
 * <code>s</code> argument
 * @param s the RGB specification
 *
 * @return true if the parse is successful.
*/
bool Util::parseRGB (Bitmap::BitmapColour& rgb, const char* s)
{
    const char *p = s;
    unsigned long val;

    while (*p != '\0' && isspace(*p))
	p++;
    
    if (*p == '#') {
	char buf[3];
	buf[2] = '\0';
	p++;
	strncpy(buf, p, 2);
	val = strtoul(buf, 0, 16);
	if (val > 255)		// can't happen, with two hex digits
	    return false;
	rgb.red = static_cast<Byte>(val);
	p += 2;
	
	strncpy(buf, p, 2);
	val = strtoul(buf, 0, 16);
	if (val > 255) return false;
	rgb.green = static_cast<Byte>(val);
	p += 2;
	
	strncpy(buf, p, 2);
	val = strtoul(buf, 0, 16);
	if (val > 255) return false;
	rgb.blue = static_cast<Byte>(val);
    } else {
	val = strtoul (s, const_cast<char**>(&p), 0);
	if (val > 255) return false;
	rgb.red = static_cast<Byte>(val);
	if (p == s)			// no digit
	    return false;
	if (*p == '\0')		// end of string
	    return false;
	s = p;
	while (!isxdigit(*s))
	{
	    if (*s == '\0') return false;
	    s++;
	}

	val = strtoul (s, const_cast<char**>(&p), 0);
	if (val > 255) return false;
	rgb.green = static_cast<Byte>(val);
	if (p == s)			// no digit
	    return false;
	if (*p == '\0')		// end of string
	    return false;
	s = p;
	while (!isxdigit(*s))
	{
	    if (*s == '\0') return false;
	    s++;
	}

	val = strtoul (s, const_cast<char**>(&p), 0);
	if (val > 255) return false;
	rgb.blue = static_cast<Byte>(val);
	if (p == s)			// no digit
	    return false;
    }

    return true;
}

/**
 * Convert a <code>string_list</code> to a null-terminated array of
 * character pointers.  The resulting array can conveniently be
 * deleted using {@link #delete_string_array}.
 *
 * @param l a string_list
 * @return a pointer to a null-terminated array of null-terminated
 * character arrays
 */
char** Util::string_list_to_array(string_list& l)
{
    int argc = l.size();
    char **sl = new char*[argc+1];
    int i = 0;
    for (string_list::const_iterator ci = l.begin();
	 ci != l.end();
	 ci++) {
	string s = *ci;
	sl[i] = new char[s.size()+1];
	STD::strcpy(sl[i], s.c_str());
	i++;
    }
    sl[i] = 0;
    return sl;
}

/**
 * Deletes the array of strings returned by {@link #string_list_to_array}.
 * 
 * @param sl a null-terminated array of strings
 */
void Util::delete_string_array(char** sl)
{
    for (int i=0; sl[i] != 0; i++)
	delete[] sl[i];
    delete[] sl;
}

/**
 * Sets the verbosity of the methods in this class
 *
 * @param level how verbose the class's methods should be
 */
void Util::verbosity (const verbosities level)
{
    verbosity_ = level;

    return;
}
