#!/bin/sh -f

# File produced by the obuild tool.

#////////////////////////////////////////////////
# Usage :
# ( UNIX> chmod a+x build_freetype2.sh )
#   UNIX> ./build_freetype2.sh
#
#////////////////////////////////////////////////

verbose_build=no
debug=no
optimize=yes
set_x=no
while test $# -ge 1 ; do
  case $1 in
    -h) 
        echo "Options :"
        echo "  -v : verbose."
        echo "  -d : build in debug mode."
        echo "  -g : build in debug mode."
        echo "  -x : execute script with set -x."
        echo ""
        exit ;;
    -v) verbose_build=yes ;;
    -x) set_x=yes ;;
    -d) debug=yes;optimize=no ;;
    -g) debug=yes;optimize=no ;;
    *) echo "unknown option : $1";exit ;;
   esac
  shift
done

if [ ${set_x} == yes ] ; then
  set -x
fi

if [ `uname` == Darwin ] ; then
  platform=Darwin
  cc=cc
  cxx=c++
  ar=ar
elif [ `uname` == Linux ] ; then
  platform=Linux
  cc=cc
  cxx=c++
  ar=ar
else
  platform=""
  cc=""
  cxx=""
  ar=""
fi

if [ "${platform}" == "" ] ; then
  echo "platform variable not defined."
  exit
fi

if [ "${cc}" == "" ] ; then
  echo "cc variable not defined."
  exit
fi
if [ "${cxx}" == "" ] ; then
  echo "cxx variable not defined."
  exit
fi

dir_bin=../${platform}_obuild
# Create the bin directory :
/bin/mkdir -p ${dir_bin}

cc_debug=""
cc_link_debug=""
cxx_debug=""
cxx_link_debug=""
if [ ${debug} == yes ] ; then
  cc_debug=-g
  cc_link_debug=-g
  cxx_debug=-g
  cxx_link_debug=-g
fi

cc_optimize=""
cc_link_optimize=""
cxx_optimize=""
cxx_link_optimize=""
if [ ${optimize} == yes ] ; then
  cc_optimize=-O
  cc_link_optimize=-O
  cxx_optimize=-O
  cxx_link_optimize=-O
fi

cc_macros=""
cxx_macros=""

cc_options=""
cxx_options=""


cc_includes="-I../include"

cxx_includes=""

/bin/rm -R -f ${dir_bin}/lib_freetype2
/bin/mkdir -p ${dir_bin}/lib_freetype2

# Compile freetype2 files :
if [ ${verbose_build} == yes ] ; then set -x;fi
if [ ${verbose_build} == yes ] ; then set +x;fi

# Create the shared library freetype2 :
objs=`find ${dir_bin}/lib_freetype2 -name '*.o' -print`
if [ `uname` == Darwin ] ; then
  /bin/rm -f ${dir_bin}/libfreetype2.dylib
if [ ${verbose_build} == yes ] ; then set -x;fi
  ${cxx} -dynamiclib -twolevel_namespace -undefined error -dynamic -single_module -o ${dir_bin}/libfreetype2.dylib  ${objs} 
if [ ${verbose_build} == yes ] ; then set +x;fi
else
  echo "Build shared lib : platform not supporter"
fi

# Cleanup :
/bin/rm -R -f ${dir_bin}/lib_freetype2
ofind_exe="${dir_bin}/ofind.exe"
/bin/rm -R -f ${ofind_exe}

