#!/bin/sh -f

# File produced by the obuild tool.

#////////////////////////////////////////////////
# Usage :
# ( UNIX> chmod a+x build_owrap.sh )
#   UNIX> ./build_owrap.sh
#
#////////////////////////////////////////////////

verbose_build=no
debug=no
optimize=yes
set_x=no
while test $# -ge 1 ; do
  case $1 in
    -h) 
        echo "Options :"
        echo "  -v : verbose."
        echo "  -d : build in debug mode."
        echo "  -g : build in debug mode."
        echo "  -x : execute script with set -x."
        echo ""
        exit ;;
    -v) verbose_build=yes ;;
    -x) set_x=yes ;;
    -d) debug=yes;optimize=no ;;
    -g) debug=yes;optimize=no ;;
    *) echo "unknown option : $1";exit ;;
   esac
  shift
done

if [ ${set_x} == yes ] ; then
  set -x
fi

if [ `uname` == Darwin ] ; then
  platform=Darwin
  cc=cc
  cxx=c++
  f77=g77
  ar=ar
elif [ `uname` == Linux ] ; then
  platform=Linux
  cc=cc
  cxx=c++
  f77=f77
  ar=ar
else
  platform=""
  cc=""
  cxx=""
  f77=""
  ar=""
fi

if [ "${platform}" == "" ] ; then
  echo "platform variable not defined."
  exit
fi

if [ "${cc}" == "" ] ; then
  echo "cc variable not defined."
  exit
fi
if [ "${cxx}" == "" ] ; then
  echo "cxx variable not defined."
  exit
fi
if [ "${f77}" == "" ] ; then
  echo "f77 variable not defined."
  exit
fi

dir_bin=../${platform}_obuild
# Create the bin directory :
/bin/mkdir -p ${dir_bin}

cc_debug=""
cc_link_debug=""
cxx_debug=""
cxx_link_debug=""
f77_debug=""
f77_link_debug=""
if [ ${debug} == yes ] ; then
  cc_debug=-g
  cc_link_debug=-g
  cxx_debug=-g
  cxx_link_debug=-g
  f77_debug=-g
  f77_link_debug=-g
fi

cc_optimize=""
cc_link_optimize=""
cxx_optimize=""
cxx_link_optimize=""
f77_optimize=""
f77_link_optimize=""
if [ ${optimize} == yes ] ; then
  cc_optimize=-O
  cc_link_optimize=-O
  cxx_optimize=-O
  cxx_link_optimize=-O
  f77_optimize=-O
  f77_link_optimize=-O
fi

if ! test -e './cpp/obuild_platform.cpp' ; then
  echo "Program source obuild_platform.cpp not found."
  exit
fi
obuild_platform_exe="${dir_bin}/obuild_platform.exe"
if ! test -e "${obuild_platform_exe}" ; then
  if [ "${cxx}" == "" ] ; then cxx=c++; fi
  ${cxx} -o "${obuild_platform_exe}" ./cpp/obuild_platform.cpp
  if ! test -e "${obuild_platform_exe}" ; then
    echo "Program obuild_platform.exe not found."
    exit
  fi
fi


cc_macros=""

cxx_macros=""

f77_macros=""

cc_options=""

cxx_options=""

f77_options=""

cc_link_options=""

cxx_link_options=""

f77_link_options=""


cc_includes=""

cxx_includes="-I../source"

f77_includes=""

/bin/rm -R -f ${dir_bin}/app_owrap
/bin/mkdir -p ${dir_bin}/app_owrap

# Compile owrap files :
if [ ${verbose_build} == yes ] ; then set -x;fi
${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/Out.o ../source/Lib/Out.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/sout.o ../source/Lib/sout.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/cmanip.o ../source/Lib/cmanip.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/System.o ../source/Lib/System.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/smanip.o ../source/Lib/smanip.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/fmanip.o ../source/Lib/fmanip.cxx
status=$?;if test $status != 0 ; then exit $status;fi

${cxx} -c ${cxx_debug} ${cxx_optimize} ${cxx_options} ${cxx_macros} ${cxx_includes} -o ${dir_bin}/app_owrap/owrap.o ../applications/owrap.cxx
status=$?;if test $status != 0 ; then exit $status;fi

if [ ${verbose_build} == yes ] ; then set +x;fi

linked_libraries=""

# Link owrap :
objs=`find ${dir_bin}/app_owrap -name '*.o' -print`
/bin/rm -f ${dir_bin}/owrap.exe
if [ ${verbose_build} == yes ] ; then set -x;fi
${cxx} ${cxx_link_debug} ${cxx_link_optimize} ${cxx_link_options} -o ${dir_bin}/owrap.exe ${objs} ${linked_libraries}
status=$?;if test $status != 0 ; then exit $status;fi
if [ ${verbose_build} == yes ] ; then set +x;fi

/bin/rm -R -f ${dir_bin}/app_owrap
obuild_find_exe="${dir_bin}/obuild_find.exe"
/bin/rm -R -f ${obuild_find_exe}

