#ifndef Lib_smanip_h
#define Lib_smanip_h

#include <string>
#include <vector>

#include <Lib/typedefs.h>

namespace Slash {namespace Core {class IWriter;}}

namespace Lib {

class smanip {
public:
  enum what { leading, trailing, both };
  static void strip(std::string&,what = both,char = ' ');
  enum side { left, right, middle };
  static bool justify(std::string&,unsigned int,side = left);
  static bool match(const std::string&,const std::string&);
  static std::vector<std::string> match(const std::vector<std::string>&,
					const std::string&);
  static bool getenv(std::string&);
  static bool replace(std::string&,const std::string&,const std::string&);
  static bool printf(std::string&,int,const char*,...);
  //static bool scanf(const std::string&,const char*,...);
  static bool toint(const std::string&,int&);
  static bool touint(const std::string&,unsigned int&);
  static bool touintx(const std::string&,unsigned int&);
  static bool topointer(const std::string&,void*&);
  static bool tobool(const std::string&,bool&);
  static bool tofloat(const std::string&,float&);
  static bool todouble(const std::string&,double&);
  static bool toint64(const std::string&,Lib::int64&);
  static bool toints(const std::string&,std::vector<int>&);
  static bool todoubles(const std::vector<std::string>&,std::vector<double>&);
  static bool todoubles(const std::string&,std::vector<double>&,const std::string& = " ");
  // On OSF1 tolower, toupper are CPP macros !
  static void tolowercase(std::string&);
  static void touppercase(std::string&);
  static bool togeometry(const std::string&,int&,int&,int&,int&);
  // Return value to be deleted with delete [].
  static char* tocstring(const std::string&);
  static bool polynomialDegree(const std::string&,int&);
  static int axtoi(const std::string&); 
  static bool pathNameSuffix(const std::string&,
                     std::string&,std::string&,std::string&);
  static bool isAbsolutePath(const std::string&);
  static bool pathWords(const std::string&,
                        std::vector<std::string>&,bool&,bool&,std::string&);
  static std::string suffix(const std::string&);
  static std::string nosuffix(const std::string&);

  static void lines(const std::string&,std::vector<std::string>&); //FIXME
  static std::vector<std::string> words(const std::string&,const std::string&,bool=false);
  static void words(const std::string&,const std::string&,std::vector<std::string>&,bool=false);
  static void dump(const std::string&,Slash::Core::IWriter&);

  static bool remove(std::vector<std::string>&,const std::string&);
  static bool removeLinesWith(std::vector<std::string>&,const std::string&);
  static void unique(std::vector<std::string>&);
  //static int lineIndex(const std::vector<std::string>&,const std::string&);
  static bool lineIndex(const std::vector<std::string>&,const std::string&,
                        unsigned int&);
  static bool belong(const std::vector<std::string>&,const std::string&);
  static void write(const std::vector<std::string>&,Slash::Core::IWriter&);
  static bool torgb(const std::string&,double&,double&,double&);
  static bool torgba(const std::string&,double&,double&,double&,double&);
  static bool torgbs(const std::string&,std::string&);
  static bool torgb(const std::string&,float&,float&,float&);
  static bool torgba(const std::string&,float&,float&,float&,float&);
  static void justifylines(const std::vector<std::string>&,std::vector<std::string>&);
  static void strip(std::vector<std::string>&,what = both,char = ' ');
  static bool replace(std::vector<std::string>&,const std::string&,const std::string&);
  static void sort(std::vector<std::string>&,bool = true);
  static void cmpsort(std::vector<std::string>&);
  static std::string tostring(const std::vector<std::string>&,const std::string&);
  static std::string tostring(unsigned char);
  static std::string tostring(char);
  static std::string tostring(unsigned short);
  static std::string tostring(short);
  static std::string tostring(unsigned int);
  static std::string tostring(int);
  static std::string tostring(Lib::uint64);
  static std::string tostring(Lib::int64);
  static std::string tostring(float);
  static std::string tostring(double);
  static std::string tostring(bool);
  static std::string tostring(const std::string&);
  static std::string tostring(const char*);
  static std::string tostring(const char*,unsigned int);
  static std::string tostring(unsigned int,const char*[]);
  static std::string tostringx(unsigned int);
  static std::string p2s(void*);
  static bool options(const std::string&,
              std::vector< std::pair<std::string,std::string> >&);
  static bool option(const std::string&,const std::string&,std::string&);
  static bool version(const std::string&,std::vector<unsigned int>&);
  static bool isVariable(const std::string&);
  static bool binRoot(const std::string&,std::string&);
  static bool nameArgs(const std::string&,std::string&,std::vector<std::string>&);
  static bool parseArray(const std::string&,std::string&,int&);
  static std::string int2coord(int);
  static bool coord2int(const std::string&,int&);
  static std::string toxml(const std::string&);
  static std::vector<std::string> tovector(int,char**);
  static void newArgcv(const std::vector<std::string>&,int&,char**&);
  static void delArgcv(int&,char**&);
  static bool beginWith(const std::string&,const std::string&,std::string&);
  static std::string removeVersions(const std::string&);
  static std::string dirname(const std::string&,unsigned int = 1);
  static std::string basename(const std::string&);

  template <class T>  
  static std::string tostring(const std::vector<T>& aVector,
                              const std::string& aSeparator = "\n" ) {
    unsigned int number = aVector.size();
    if(number<=0) return "";
    std::string result;
    number--;
    for(unsigned int index=0;index<number;index++) {
      result += tostring(aVector[index]);
      result += aSeparator;
    }
    result += tostring(aVector[number]);
    return result;
  }

};

}

#endif
