// this :
#include <HEPVis/nodekits/SoPlotter.h>

// Inventor :
#include <Inventor/errors/SoDebugError.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoPointSet.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTranslation.h>
#include <Inventor/nodes/SoRotationXYZ.h>
#include <Inventor/nodes/SoScale.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/actions/SoRayPickAction.h>
#include <Inventor/actions/SoGetBoundingBoxAction.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/actions/SoHandleEventAction.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/elements/SoModelMatrixElement.h>
#include <Inventor/elements/SoViewVolumeElement.h>

// HEPVis :
#include <HEPVis/SbString.h>
#include <HEPVis/SbMath.h>
#include <HEPVis/SbStyle.h>
#include <HEPVis/SbColors.h>
#include <HEPVis/SbPlottableThings.h>
#include <HEPVis/SbHatching.h>
#include <HEPVis/SbAxis.h>
#include <HEPVis/SbTessContour.h>
#include <HEPVis/ListContour.h>
#include <HEPVis/misc/SoStyleCache.h>
#include <HEPVis/misc/SoTools.h>
#include <HEPVis/nodes/SoLineStyle.h>
#include <HEPVis/nodes/SoTextStyle.h>
#include <HEPVis/nodes/SoTextHershey.h>
#include <HEPVis/nodes/SoMarkerSet.h>
#include <HEPVis/nodekits/SoAxis.h>
#include <HEPVis/nodes/SoGrid.h>
#include <HEPVis/nodes/SoStyle.h>
#include <HEPVis/actions/SoAlternateRepAction.h>

// Else :
//#define DEBUG
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>

static float TakeLog(float);
static float VerifyLog(float,float,float,SbBool);
static void BarChart(float,float,float&,float&);
static SbColor GetColor (float,int,float*,SbColor*);
static void ReverseList (int aNumber,SbVec3f*);
static void getHeight(int,const SbPList&,const SbPList&,int,float&,float&);
static float bins2DToFunc(const SbPlottableBins2D&,float,float);

#define LIST_SET(list,number,x,y,z) list[number].setValue((x),(y),(z))

#define NotFound (-1)
#define xAxisNode ((SoAxis*)xAxis.getValue())
#define yAxisNode ((SoAxis*)yAxis.getValue())
#define zAxisNode ((SoAxis*)zAxis.getValue())

#define DUMP_UPDATE_WHAT(aString) \
  if(verbose.getValue()==TRUE) SoDebugError::postInfo("SoPlotter","%lu : updateWhat : %s",(unsigned long)this,(aString));

#define Cast(aObject,aClass) (aClass*)(aObject).cast(#aClass)

// To keep picking infos.
class SoPickSeparator : public SoSeparator {
public:
  SoPickSeparator():fInfos(0){}
protected:
  virtual ~SoPickSeparator() {
    SbSTRDEL(fInfos);
  }
public:
  char* fInfos; 
};

SO_KIT_SOURCE(SoPlotter) 
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::initClass (
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SO_KIT_INIT_CLASS(SoPlotter,SoBaseKit,"BaseKit");
}
//////////////////////////////////////////////////////////////////////////////
SoPlotter::SoPlotter (
)
:fDefault__wallStyle(0)
,fDefault__gridStyle(0)
,fDefault__innerFrameStyle(0)
,fDefault__infosStyle(0)
,fDefault__titleStyle(0)

,fDefault__binsStyle(0)
,fDefault__pointsStyle(0)
,fDefault__functionStyle(0)
,fDefault__rightHatchStyle(0)
,fDefault__leftHatchStyle(0)
,fDefault__errorsStyle(0)
,fDefault__legendStyle(0)
,fStyleCache(0)
,fShape(-1)
,fScreenDataFrameUpperRightCorner(0,0,0)
,f_shape_Update(FALSE) //data
,f_dataScale_Update(FALSE) //data
,f_bins_Update(FALSE) //data
,f_points_Update(FALSE) //data
,f_function_Update(FALSE) //data
,f_wall_Update(FALSE)
,f_grid_Update(FALSE)
,f_innerFrame_Update(FALSE)
,f_title_Update(FALSE)
,f_xAxis_Update(FALSE)
,f_yAxis_Update(FALSE)
,f_zAxis_Update(FALSE)
,f_colorMapping_Update(FALSE)
,f_layout_Update(FALSE)
,f_ttfScale_Update(FALSE)

,fOld_width(0)
,fOld_height(0)
,fOld_depth(0)
,fOld_leftMargin(0)
,fOld_rightMargin(0)
,fOld_bottomMargin(0)
,fOld_topMargin(0)
,fOld_downMargin(0)
,fOld_upMargin(0)
,fOld_titleUp(FALSE)
,fOld_titleToAxis(0)
,fOld_titleHeight(0)
,fOld_titleAutomated(FALSE)
,fOld_superposeBins(FALSE)
,fOld_colorMapping("")
,fOld_xAxisAutomated(FALSE)
,fOld_xAxisMinimum(-1)
,fOld_xAxisMaximum(-1)
,fOld_xAxisLogScale(FALSE)
,fOld_yAxisAutomated(FALSE)
,fOld_yAxisMinimum(-1)
,fOld_yAxisMaximum(-1)
,fOld_yAxisLogScale(FALSE)
,fOld_zAxisAutomated(FALSE)
,fOld_zAxisMinimum(-1)
,fOld_zAxisMaximum(-1)
,fOld_zAxisLogScale(FALSE)
,fOld_topAxisVisible(FALSE)  
,fOld_rightAxisVisible(FALSE)  
,fOld_numberOfLevels(0)
,fOld_shapeAutomated(FALSE)
,fOld_shape(FALSE)
,fOld_ttfScale(0)  
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SO_KIT_CONSTRUCTOR(SoPlotter);

  SO_KIT_ADD_CATALOG_ENTRY(topSeparator,SoSeparator,FALSE,this,\0, FALSE);
  SO_KIT_ADD_CATALOG_ENTRY(layout,SoTransform,FALSE,topSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(titleSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(colorMappingSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(gridSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(wallSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);

  SO_KIT_ADD_CATALOG_ENTRY(xAxisSeparator,SoSeparator,FALSE,topSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(xAxisTransform,SoTransform,FALSE,xAxisSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(xAxis,SoAxis,FALSE,xAxisSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(yAxisSeparator,SoSeparator,FALSE,topSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(yAxisTransform,SoTransform,FALSE,yAxisSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(yAxis,SoAxis,FALSE,yAxisSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(zAxisSeparator,SoSeparator,FALSE,topSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(zAxisTransform,SoTransform,FALSE,zAxisSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(zAxis,SoAxis,FALSE,zAxisSeparator, \0,TRUE);

  SO_KIT_ADD_CATALOG_ENTRY(topAxisSeparator,SoSeparator,FALSE,topSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(rightAxisSeparator,SoSeparator,FALSE,topSeparator, \0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(etcSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);

  SO_KIT_ADD_CATALOG_ENTRY(dataSeparator,SoSeparator,FALSE,topSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(dataScale,SoScale,FALSE,dataSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(binsSeparator,SoSeparator,FALSE,dataSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(pointsSeparator,SoSeparator,FALSE,dataSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(functionSeparator,SoSeparator,FALSE,dataSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(errorsSeparator,SoSeparator,FALSE,dataSeparator,\0, TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(innerFrameSeparator,SoSeparator,FALSE,dataSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(etcDataSeparator,SoSeparator,FALSE,dataSeparator,\0,TRUE);
  // Styles :
  // must not be put under topSeparator because of alternate reps.
  SO_KIT_ADD_CATALOG_ENTRY(styleSeparator,SoSeparator,FALSE,this,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(wallStyle,SoStyle,FALSE,styleSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(gridStyle,SoStyle,FALSE,styleSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(innerFrameStyle,SoStyle,FALSE,styleSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(infosStyle,SoStyle,FALSE,styleSeparator,\0,TRUE);
  SO_KIT_ADD_CATALOG_ENTRY(titleStyle,SoTextStyle,FALSE,styleSeparator,\0,TRUE);
  
  SO_KIT_ADD_CATALOG_LIST_ENTRY(binsStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(pointsStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(functionStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(rightHatchStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(leftHatchStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(errorsStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_ADD_CATALOG_LIST_ENTRY(legendStyle,SoSeparator,FALSE,styleSeparator,\0,SoStyle,TRUE);
  SO_KIT_INIT_INSTANCE();

  SO_NODE_ADD_FIELD(alternateRep,(NULL));

  SO_NODE_ADD_FIELD(verbose,(FALSE));

  float xfac = 1.0F/20.0F; //0.05
  float yfac = 1.0F/20.0F; //0.05

  // Take PAW defaults :
  float XSIZ = 20 * xfac;    //1     //page width
  float YSIZ = 20 * yfac;    //1     //page height
  float XMGL = 2 * xfac;     //0.1   //left x margin (to data frame).
  float XMGR = 2 * xfac;     //0.1   //right y margin (to data frame).
  float YMGL = 2 * yfac;     //0.1   //low y margin (to data frame).
  float YMGU = 2 * yfac;     //0.1   //up y margin (to data frame).
/* Axes :
  float VSIZ = 0.28F * yfac; //0.014 //tick label character size.
  float XVAL = 0.4F * xfac;  //0.02 //x distance of y tick label to data frame.
  float YVAL = 0.4F * yfac;  //0.02 //y distance of x tick label to data frame.
  float XTIC = 0.3F * yfac;  //0.015 //y length of X axis ticks.
  float YTIC = 0.3F * xfac;  //0.015 //x length of Y axis ticks.
  float XLAB = 1.4F * xfac;  //0.07  //x distance of y title to data frame.
  float YLAB = 0.8F * yfac;  //0.04  //y distance of x title to data frame.
  float ASIZ = 0.28F * yfac; //0.014 // axis title (label) character size.
*/
  float YHTI = 1.2F * yfac;  //0.06  //y distance of title to x axis.
  float TSIZ = 0.28F * yfac; //0.014 //title character size

  float zfac = 1.0F/20.0F; //0.05
  float ZSIZ = 20 * zfac;    //1     //page height
  float ZMGD = 2 * zfac;     //0.1   //low y margin (to data frame).
  float ZMGU = 2 * zfac;     //0.1   //up y margin (to data frame).

  // Data area :
  //float wData = XSIZ-XMGL-XMGR;
  //float hData = YSIZ-YMGL-YMGU;
  //float dData = ZSIZ-ZMGD-ZMGU;

  SO_NODE_ADD_FIELD(width,(XSIZ));
  SO_NODE_ADD_FIELD(height,(YSIZ));
  SO_NODE_ADD_FIELD(leftMargin,(XMGL));
  SO_NODE_ADD_FIELD(rightMargin,(XMGR));
  SO_NODE_ADD_FIELD(bottomMargin,(YMGL));
  SO_NODE_ADD_FIELD(topMargin,(YMGU));

  SO_NODE_ADD_FIELD(depth,(ZSIZ));
  SO_NODE_ADD_FIELD(downMargin,(ZMGD));
  SO_NODE_ADD_FIELD(upMargin,(ZMGU));

  SO_NODE_ADD_FIELD(titleUp,(TRUE));

  SO_NODE_ADD_FIELD(titleToAxis,(YHTI));
  SO_NODE_ADD_FIELD(titleHeight,(TSIZ));
  SO_NODE_ADD_FIELD(titleAutomated,(TRUE));
  SO_NODE_ADD_FIELD(title,(""));
  SO_NODE_ADD_FIELD(superposeBins,(FALSE));
  SO_NODE_ADD_FIELD(colorMapping,("black 0 cyan 5 green 10 orange 15 red"));
  
  SO_NODE_ADD_FIELD(xAxisLogScale,(FALSE));
  SO_NODE_ADD_FIELD(yAxisLogScale,(FALSE));
  SO_NODE_ADD_FIELD(zAxisLogScale,(FALSE));
  
  SO_NODE_ADD_FIELD(xAxisAutomated,(TRUE));
  SO_NODE_ADD_FIELD(xAxisMinimum,(0));
  SO_NODE_ADD_FIELD(xAxisMaximum,(1));

  SO_NODE_ADD_FIELD(yAxisAutomated,(TRUE));
  SO_NODE_ADD_FIELD(yAxisMinimum,(0));
  SO_NODE_ADD_FIELD(yAxisMaximum,(1));

  SO_NODE_ADD_FIELD(zAxisAutomated,(TRUE));
  SO_NODE_ADD_FIELD(zAxisMinimum,(0));
  SO_NODE_ADD_FIELD(zAxisMaximum,(1));

  SO_NODE_ADD_FIELD(numberOfLevels,(10));

  SO_NODE_ADD_FIELD(topAxisVisible,(FALSE));
  SO_NODE_ADD_FIELD(rightAxisVisible,(FALSE));

  SO_NODE_ADD_FIELD(ttfScale,(1));

  SO_NODE_ADD_FIELD(shapeAutomated,(TRUE));

  SO_NODE_ADD_FIELD(frozen,(FALSE));

  // Output fields :  
  SO_NODE_ADD_FIELD(shape,(XY));
  SO_NODE_ADD_FIELD(infos,(""));
  SO_NODE_ADD_FIELD(legend,(""));
  SO_NODE_ADD_FIELD(numberOfPlottedObject,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedBins1D,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedBins2D,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedPoints2D,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedPoints3D,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedFunction1D,(0));
  SO_NODE_ADD_FIELD(numberOfPlottedFunction2D,(0));
  SO_NODE_ADD_FIELD(plottedObjectNames,(""));

  SO_NODE_DEFINE_ENUM_VALUE(Shape,XY);
  SO_NODE_DEFINE_ENUM_VALUE(Shape,XYZ);
  
  SO_NODE_SET_SF_ENUM_TYPE(shape,Shape);
  
  setPAW_AxesTextSize(XSIZ,YSIZ);

  fDefault__wallStyle = new SoStyle;
  fDefault__wallStyle->ref();
  fDefault__wallStyle->setFromString("color lightgrey\nmodeling boxes");
  setPart("wallStyle",fDefault__wallStyle->copy());

  fDefault__gridStyle = new SoStyle;
  fDefault__gridStyle->ref();
  fDefault__gridStyle->setFromString("color black\nlinePattern dashed");
  setPart("gridStyle",fDefault__gridStyle->copy());

  fDefault__innerFrameStyle = new SoStyle;
  fDefault__innerFrameStyle->ref();
  fDefault__innerFrameStyle->setFromString("color black\nlinePattern solid");
  setPart("innerFrameStyle",fDefault__innerFrameStyle->copy());

  fDefault__infosStyle = new SoStyle;
  fDefault__infosStyle->ref();
  fDefault__infosStyle->setFromString("color black\nfontName Hershey\nencoding PAW\nsmoothing FALSE");
  fDefault__infosStyle->options.setValue
    ("name entries mean rms fit_quality fit_ndf fit_parameters");
  setPart("infosStyle",fDefault__infosStyle->copy());

  fDefault__titleStyle = new SoTextStyle;
  fDefault__titleStyle->ref();
  fDefault__titleStyle->setFromString("color black\nfontName Hershey\nencoding PAW\nsmoothing FALSE");
  setPart("titleStyle",fDefault__titleStyle->copy());

  fDefault__binsStyle = new SoStyle;
  fDefault__binsStyle->ref();
  //Have encoding for legend box.
  fDefault__binsStyle->setFromString("color darkslategrey\nlinePattern solid\nareaStyle solid\nmarkerStyle dot\nmodeling boxes\nvisible true\nencoding PAW\nsmoothing FALSE");

  fDefault__pointsStyle = new SoStyle;
  fDefault__pointsStyle->ref();
  //Have encoding for legend box.
  fDefault__pointsStyle->setFromString("color darkslategrey\nlinePattern solid\nmarkerStyle dot\nmodeling points\nvisible true\nencoding PAW\nsmoothing FALSE");

  fDefault__functionStyle = new SoStyle;
  fDefault__functionStyle->ref();
  //Have encoding for legend box.
  fDefault__functionStyle->setFromString("color black\nlinePattern solid\nareaStyle solid\nmodeling lines\nvisible true\nencoding PAW\nsmoothing FALSE");

  fDefault__rightHatchStyle = new SoStyle;
  fDefault__rightHatchStyle->ref();
  fDefault__rightHatchStyle->setFromString("color black\nspacing .02");

  fDefault__leftHatchStyle = new SoStyle;
  fDefault__leftHatchStyle->ref();
  fDefault__leftHatchStyle->setFromString("color black\nspacing .02\nangle -0.7854");

  fDefault__errorsStyle = new SoStyle;
  fDefault__errorsStyle->ref();
  fDefault__errorsStyle->setFromString("color black\nvisible false\nmarkerStyle plus");

  fDefault__legendStyle = new SoStyle;
  fDefault__legendStyle->ref();
  fDefault__legendStyle->setFromString("color black");

  fStyleCache = new SoStyleCache;
  fStyleCache->ref();

  plottedObjectNames.deleteValues(0);

  // Set a fancy value on some field of SbStyle 
  // so that the first doAction start correctly :
  fOld_wallStyle.setTransparency(-1);
  fOld_gridStyle.setTransparency(-1);
  fOld_innerFrameStyle.setTransparency(-1);
  fOld_infosStyle.setTransparency(-1);
  fOld_titleStyle.setTransparency(-1);

  fOld_xAxis_lineStyle.setTransparency(-1);
  fOld_xAxis_ticksStyle.setTransparency(-1);
  fOld_xAxis_labelsStyle.setTransparency(-1);
  fOld_xAxis_titleStyle.setTransparency(-1);
  fOld_xAxis_magStyle.setTransparency(-1);

  fOld_yAxis_lineStyle.setTransparency(-1);
  fOld_yAxis_ticksStyle.setTransparency(-1);
  fOld_yAxis_labelsStyle.setTransparency(-1);
  fOld_yAxis_titleStyle.setTransparency(-1);
  fOld_yAxis_magStyle.setTransparency(-1);

  fOld_zAxis_lineStyle.setTransparency(-1);
  fOld_zAxis_ticksStyle.setTransparency(-1);
  fOld_zAxis_labelsStyle.setTransparency(-1);
  fOld_zAxis_titleStyle.setTransparency(-1);
  fOld_zAxis_magStyle.setTransparency(-1);
}
//////////////////////////////////////////////////////////////////////////////
SoPlotter::~SoPlotter (
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoMaterials %d\n",fStyleCache->getNumChildren());

  clearPlottableObjects();

  fDefault__wallStyle->unref();
  fDefault__gridStyle->unref();
  fDefault__innerFrameStyle->unref();
  fDefault__infosStyle->unref();
  fDefault__titleStyle->unref();
  
  fDefault__binsStyle->unref();
  fDefault__pointsStyle->unref();
  fDefault__functionStyle->unref();
  fDefault__rightHatchStyle->unref();
  fDefault__leftHatchStyle->unref();
  fDefault__errorsStyle->unref();
  fDefault__legendStyle->unref();

  fStyleCache->unref();
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
static int sDoCounter = 0;
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::doAction(
 SoAction* aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(verbose.getValue()==TRUE) {
    SoDebugError::postInfo("SoPlotter::doAction","%lu (%d).",
      (unsigned long)this,sDoCounter++);
  }

  updateChildren();

  // Update fScreenDataFrameUpperRightCorner.
  // It may be used by the SoPlotterRegion to attach the infos region.

  // aAction needs the SoModelMatrixElement and SoViewVolumeElement.
  if(aAction->isOfType(SoHandleEventAction::getClassTypeId())) return;
  if(aAction->isOfType(SoGetMatrixAction::getClassTypeId())) return;

  SO_ALTERNATEREP_DO_ACTION(aAction)

  if(fShape==XY) {
    float XSIZ = width.getValue();
    float XMGL = leftMargin.getValue();
    float XMGR = rightMargin.getValue();
    float wData = XSIZ-XMGL-XMGR;

    float YSIZ = height.getValue();
    float YMGL = bottomMargin.getValue();
    float YMGU = topMargin.getValue();
    float hData = YSIZ-YMGL-YMGU;
    if( (wData<=0) || (hData<=0) ) {
      SoDebugError::postInfo("SoPlotter::doAction","null data area.");
    } else {
      SoTransform* transform = (SoTransform*)layout.getValue();
      const SbVec3f& t = transform->translation.getValue();
      const SbMatrix& mat = SoModelMatrixElement::get(aAction->getState());
      const SbViewVolume& vv = SoViewVolumeElement::get(aAction->getState());

     {SbVec3f pos(wData,hData,0);
      //SbVec3f pos(wData+XMGR,hData+YMGU,0); //Should give screen 1 1
      //SbVec3f pos(-XMGL,-YMGL,0); //Should give screen 0 0
      pos += t;
      mat.multVecMatrix(pos,pos);
      vv.projectToScreen(pos,fScreenDataFrameUpperRightCorner);
      // The below gives same effect as the upper.
      //float x = (wData + XMGL)/XSIZ;
      //float y = (hData + YMGL)/YSIZ;
      //fScreenDataFrameUpperRightCorner.setValue(x,y,0);
      }

    }
  }
}
#define ACTION(aClass,aSuper,aAction,aMethod)\
void aClass::aMethod(aAction* action){\
  /*printf("debug : %s::%s\n",#aClass,#aMethod);*/\
  aClass::doAction(action);\
  aSuper::aMethod(action);\
}
ACTION(SoPlotter,SoBaseKit,SoCallbackAction,callback)
ACTION(SoPlotter,SoBaseKit,SoGLRenderAction,GLRender)
ACTION(SoPlotter,SoBaseKit,SoGetBoundingBoxAction,getBoundingBox)
ACTION(SoPlotter,SoBaseKit,SoGetMatrixAction,getMatrix)
ACTION(SoPlotter,SoBaseKit,SoHandleEventAction,handleEvent)
ACTION(SoPlotter,SoBaseKit,SoRayPickAction,rayPick)
ACTION(SoPlotter,SoBaseKit,SoPickAction,pick)

#define ACTION2(aClass,aSuper,aAction,aMethod)\
void aClass::aMethod(aAction* action){\
  /*printf("debug : %s::%s\n",#aClass,#aMethod);*/\
  aClass::updateChildren();\
  aSuper::aMethod(action);\
}
ACTION2(SoPlotter,SoBaseKit,SoSearchAction,search)
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::write(
 SoWriteAction* aAction
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoNode* altRep = alternateRep.getValue();
  if(altRep) {
    //NOTE : the below logic loose the fact
    //       that we had a SoPlotter. 
    //       We may also introduce a coworking SoAltPlotter
    //       that has no polyhedron field, and arrange
    //       that the SoAlternateRepAction swaps 
    //       from SoPlotter to SoAltPlotter.
    altRep->write(aAction);
  } else {
    updateChildren();

    // freeze for writing.
    // When read back, the created SoPlotter 
    // will have the frozen field to TRUE and
    // then the updateChildren method will not
    // demolish the read scene.
    frozen.setValue(TRUE);

    SoBaseKit::write(aAction);

    // release freeze :
    frozen.setValue(FALSE);
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SoTransform* SoPlotter::getLayout(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoTransform*)layout.getValue();
}
//////////////////////////////////////////////////////////////////////////////
SoScale* SoPlotter::getDataScale(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoScale*)dataScale.getValue();
}
//////////////////////////////////////////////////////////////////////////////
SoSeparator* SoPlotter::getEtcSeparator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoSeparator*)etcSeparator.getValue();
}
//////////////////////////////////////////////////////////////////////////////
SoSeparator* SoPlotter::getTopSeparator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoSeparator*)topSeparator.getValue();
}
//////////////////////////////////////////////////////////////////////////////
SoSeparator* SoPlotter::getEtcDataSeparator(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoSeparator*)etcDataSeparator.getValue();
}
//////////////////////////////////////////////////////////////////////////////
SoAxis* SoPlotter::getXAxis(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return xAxisNode;
}
//////////////////////////////////////////////////////////////////////////////
SoAxis* SoPlotter::getYAxis(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return yAxisNode;
}
//////////////////////////////////////////////////////////////////////////////
SoAxis* SoPlotter::getZAxis(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return zAxisNode;
}
//////////////////////////////////////////////////////////////////////////////
SoNodeKitListPart* SoPlotter::getLegendStyleList(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return (SoNodeKitListPart*)legendStyle.getValue();
}

SoStyle* SoPlotter::getWallStyle() const {
  return (SoStyle*)wallStyle.getValue();}
SoStyle* SoPlotter::getGridStyle() const {
  return (SoStyle*)gridStyle.getValue();}
SoStyle* SoPlotter::getInnerFrameStyle() const {
  return (SoStyle*)innerFrameStyle.getValue();}
SoStyle* SoPlotter::getInfosStyle() const {
  return (SoStyle*)infosStyle.getValue();}
SoTextStyle* SoPlotter::getTitleStyle() const {
  return (SoTextStyle*)titleStyle.getValue();}


#define SO_GET_STYLE(aMethod,aStyle,aDefault,aOld) \
SoStyle* SoPlotter::aMethod(int aIndex) {\
  SoGroup* group = getContainerNode(#aStyle);\
  if(aIndex<0) return 0;\
  int number = group->getNumChildren();\
  if(aIndex>=number) {\
    int index;\
    for(index=number;index<=aIndex;index++) {\
      SbString fmt(#aStyle);\
      fmt += "[%d]";\
      setPart(SbStringPrintF(32,fmt.getString(),index),aDefault->copy());\
    }\
    aOld.resize(aIndex+1);\
    for(index=number;index<=aIndex;index++) {\
      SoStyle* soStyle = (SoStyle*)group->getChild(index);\
      aOld[index] = soStyle->getSbStyle();\
    }\
  }\
  return (SoStyle*)group->getChild(aIndex);\
}

SO_GET_STYLE(getBinsStyle,binsStyle,fDefault__binsStyle,fOld_binsStyle)
SO_GET_STYLE(getPointsStyle,pointsStyle,fDefault__pointsStyle,fOld_pointsStyle)
SO_GET_STYLE(getFunctionStyle,functionStyle,fDefault__functionStyle,fOld_functionStyle)
SO_GET_STYLE(getRightHatchStyle,rightHatchStyle,fDefault__rightHatchStyle,fOld_rightHatchStyle)
SO_GET_STYLE(getLeftHatchStyle,leftHatchStyle,fDefault__leftHatchStyle,fOld_leftHatchStyle)
SO_GET_STYLE(getErrorsStyle,errorsStyle,fDefault__errorsStyle,fOld_errorsStyle)
SO_GET_STYLE(getLegendStyle,legendStyle,fDefault__legendStyle,fOld_legendStyle)

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#define SO_IS_STYLE_CHANGED(aMethod,aStyle,aOld) \
SbBool SoPlotter::aMethod(){\
  SoGroup* group = getContainerNode(#aStyle);\
  int number = group->getNumChildren();\
  SbBool changed = FALSE;\
  if(int(aOld.size())!=number) {\
    changed = TRUE;\
  } else {\
    for(int index=0;index<number;index++) {\
      SoStyle* soStyle = (SoStyle*)group->getChild(index);\
      if(aOld[index].isEqual(soStyle->getSbStyle())==FALSE) {\
        changed = TRUE;\
        break;\
      }\
    }\
  }\
  if(changed==TRUE) {\
    aOld.resize(number);\
    for(int index=0;index<number;index++) {\
      SoStyle* soStyle = (SoStyle*)group->getChild(index);\
      aOld[index] = soStyle->getSbStyle();\
    }\
    return TRUE;\
  } else {\
    return FALSE;\
  }\
}

SO_IS_STYLE_CHANGED(isBinsStyleChanged,binsStyle,fOld_binsStyle)
SO_IS_STYLE_CHANGED(isPointsStyleChanged,pointsStyle,fOld_pointsStyle)
SO_IS_STYLE_CHANGED(isFunctionStyleChanged,functionStyle,fOld_functionStyle)
SO_IS_STYLE_CHANGED(isRightHatchStyleChanged,rightHatchStyle,fOld_rightHatchStyle)
SO_IS_STYLE_CHANGED(isLeftHatchStyleChanged,leftHatchStyle,fOld_leftHatchStyle)
SO_IS_STYLE_CHANGED(isErrorsStyleChanged,errorsStyle,fOld_errorsStyle)
SO_IS_STYLE_CHANGED(isLegendStyleChanged,legendStyle,fOld_legendStyle)
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
const SbPList& SoPlotter::getPlottableObjects(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fPlottableObjects;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::clearPlottableObjects(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int objn = fPlottableObjects.getLength();
  for(int index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[index];
    if(object) delete object;
  }
  fPlottableObjects.truncate(0);

  numberOfPlottedObject.setValue(0);
  numberOfPlottedBins1D.setValue(0);
  numberOfPlottedBins2D.setValue(0);
  numberOfPlottedPoints2D.setValue(0);
  numberOfPlottedPoints3D.setValue(0);
  numberOfPlottedFunction1D.setValue(0);
  numberOfPlottedFunction2D.setValue(0);
  plottedObjectNames.deleteValues(0);

  // If a title had been given on some axis, it is probably
  // no more pertinent for further data.
 //{SbBool flag = xAxisNode->title.enableNotify(FALSE);
  xAxisNode->title.setValue("");
  //xAxisNode->enableNotify(flag);}
 //{SbBool flag = yAxisNode->title.enableNotify(FALSE);
  yAxisNode->title.setValue("");
  //yAxisNode->enableNotify(flag);}
 //{SbBool flag = zAxisNode->title.enableNotify(FALSE);
  zAxisNode->title.setValue("");
  //zAxisNode->enableNotify(flag);}

  // The same with axis log scale.
 //{SbBool flag = xAxisLogScale.enableNotify(FALSE);
  xAxisLogScale.setValue(FALSE);
  //xAxisLogScale.enableNotify(flag);}
 //{SbBool flag = yAxisLogScale.enableNotify(FALSE);
  yAxisLogScale.setValue(FALSE);
  //yAxisLogScale.enableNotify(flag);}
 //{SbBool flag = zAxisLogScale.enableNotify(FALSE);
  zAxisLogScale.setValue(FALSE);
  //zAxisLogScale.enableNotify(flag);}

  DUMP_UPDATE_WHAT("clearPlottableObjects");

  f_bins_Update = TRUE; 
  f_points_Update = TRUE;
  f_function_Update = TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setPlottableObject(
 SbPlottableObject* aPlottableObject
)
//////////////////////////////////////////////////////////////////////////////
// Given thing deleted by the plotter.
// Giving 0 clear the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  clearPlottableObjects();
  SbPList list; // An empty list.
  if(aPlottableObject) list.append(aPlottableObject);
  setPlottableObjects(list);
}
//////////////////////////////////////////////////////////////////////////////
SbBool SoPlotter::isEmpty(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return ( fPlottableObjects.getLength()<=0 ? TRUE : FALSE );
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::addPlottableObject(
 SbPlottableObject* aPlottableObject
)
//////////////////////////////////////////////////////////////////////////////
// Given thing is deleted by the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPList list(fPlottableObjects);
  list.append(aPlottableObject);
  setPlottableObjects(list);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::removePlottableObject(
 SbPlottableObject* aPlottableObject
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPList list(fPlottableObjects);
  int index = list.find(aPlottableObject);
  if(index==NotFound) return;
  list.remove(index);
  setPlottableObjects(list);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::removePlottableObjectsOfClass(
 const SbString& aClass
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPList list;
  int objn = fPlottableObjects.getLength();
  for(int index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[index];
    if(!object) continue; 
    if(!object->isValid()) {
      SoDebugError::postInfo
      ("SoPlotter::removePlottableObjectsOfClass","object \"%s\" not valid.",
       object->getName());
      delete object;
      continue; 
    }
    if(object->cast(aClass.getString())) {
      delete object;
    } else {
      list.append(object);
    }
  }
  setPlottableObjects(list);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setPlottableObjects (
 const SbPList& aPlottableObjects
)
//////////////////////////////////////////////////////////////////////////////
//  Main method to declare to the plotter a list of things to represent.
//  Given thing in the list is deleted by the plotter.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fPlottableObjects.copy(aPlottableObjects);

  int objn = fPlottableObjects.getLength();

 //{SbBool flag = numberOfPlottedObject.enableNotify(FALSE);
  numberOfPlottedObject.setValue(objn);
  //numberOfPlottedObject.enableNotify(flag);}

 //{SbBool flag = plottedObjectNames.enableNotify(FALSE);
  plottedObjectNames.deleteValues(0);
  //plottedObjectNames.enableNotify(flag);}

  SbPList binsList,pointsList,functionList;
  int nbins1D = 0;
  int nbins2D = 0;
  int npoints2D = 0;
  int npoints3D = 0;
  int nfunction1D = 0;  
  int nfunction2D = 0;  
  int index;
  for(index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[index];
    if(!object) continue; 
    if(!object->isValid()) {
      SoDebugError::postInfo
      ("SoPlotter::setPlottableObjects","object \"%s\" not valid.",
       object->getName());
      continue; 
    }
    if(Cast(*object,SbPlottableBins1D)) {
      binsList.append(object);
      nbins1D++;
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else if(Cast(*object,SbPlottableBins2D)) {
      binsList.append(object);
      nbins2D++;
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else if(Cast(*object,SbPlottablePoints2D)) {
      pointsList.append(object);
      npoints2D++;
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else if(Cast(*object,SbPlottablePoints3D)) {
      pointsList.append(object);
      npoints3D++;
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else if(Cast(*object,SbPlottableFunction1D)) {
      functionList.append(object);
      nfunction1D++;  
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else if(Cast(*object,SbPlottableFunction2D)) {
      functionList.append(object);
      nfunction2D++;  
      plottedObjectNames.set1Value
        (plottedObjectNames.getNum(),SbString(object->getName()));
    } else {
      SoDebugError::postInfo
        ("SoPlotter::setPlottableObjects","unkown plottable object type.");
    }
  }

  numberOfPlottedBins1D.setValue(nbins1D);
  numberOfPlottedBins2D.setValue(nbins2D);
  numberOfPlottedPoints2D.setValue(npoints2D);
  numberOfPlottedPoints3D.setValue(npoints3D);
  numberOfPlottedFunction1D.setValue(nfunction1D);
  numberOfPlottedFunction2D.setValue(nfunction2D);

  //FIXME : should be more precise (compare with what existed).
  DUMP_UPDATE_WHAT("setPlottableObjects");

  f_shape_Update = TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::raiseUpdateFlag(
 UpdateFlag aWhat
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aWhat==SHAPE)
    f_shape_Update = TRUE;
  else if(aWhat==DATA_SCALE)
    f_dataScale_Update = TRUE;
  else if(aWhat==BINS)
    f_bins_Update = TRUE;
  else if(aWhat==POINTS)
    f_points_Update = TRUE;
  else if(aWhat==FUNCTION)
    f_function_Update = TRUE;
  else if(aWhat==WALL)
    f_wall_Update = TRUE;
  else if(aWhat==GRID)
    f_grid_Update = TRUE;
  else if(aWhat==INNER_FRAME)
    f_innerFrame_Update = TRUE;
  else if(aWhat==TITLE)
    f_title_Update = TRUE;
  else if(aWhat==X_AXIS) 
    f_xAxis_Update = TRUE;
  else if(aWhat==Y_AXIS)
    f_yAxis_Update = TRUE;
  else if(aWhat==Z_AXIS)
    f_zAxis_Update = TRUE;
  else if(aWhat==COLOR_MAPPING)
    f_colorMapping_Update = TRUE;
  else if(aWhat==LAYOUT)
    f_layout_Update = TRUE;
  else if(aWhat==TTF_SCALE)
    f_ttfScale_Update = TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::raiseAllUpdateFlags(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  f_shape_Update = TRUE;
  f_dataScale_Update = TRUE;
  f_bins_Update = TRUE;
  f_points_Update = TRUE;
  f_function_Update = TRUE;
  f_wall_Update = TRUE;
  f_grid_Update = TRUE;
  f_innerFrame_Update = TRUE;
  f_title_Update = TRUE;
  f_xAxis_Update = TRUE;
  f_yAxis_Update = TRUE;
  f_zAxis_Update = TRUE;
  f_colorMapping_Update = TRUE;
  f_layout_Update = TRUE;
  f_ttfScale_Update = TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateWhat(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fOld_width!=width.getValue()) {
    fOld_width = width.getValue();
    f_xAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(rightAxisVisible.getValue()==TRUE) {
      f_yAxis_Update = TRUE; //FIXME : we should have a f_rightAxis_Update
    }    
  }    
  if(fOld_height!=height.getValue()) {
    fOld_height = height.getValue();
    f_yAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(topAxisVisible.getValue()==TRUE) {
      f_xAxis_Update = TRUE; //FIXME : we should have a f_topAxis_Update
    }    
  }    
  if(fOld_depth!=depth.getValue()) {
    fOld_depth = depth.getValue();
    f_zAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
  }    
  if(fOld_leftMargin!=leftMargin.getValue()) {
    fOld_leftMargin = leftMargin.getValue();
    f_xAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(rightAxisVisible.getValue()==TRUE) {
      f_yAxis_Update = TRUE; //FIXME : we should have a f_rightAxis_Update
    }    
  }    
  if(fOld_rightMargin!=rightMargin.getValue()) {
    fOld_rightMargin = rightMargin.getValue();
    f_xAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(rightAxisVisible.getValue()==TRUE) {
      f_yAxis_Update = TRUE; //FIXME : we should have a f_rightAxis_Update
    }    
  }    
  if(fOld_bottomMargin!=bottomMargin.getValue()) {
    fOld_bottomMargin = bottomMargin.getValue();
    f_yAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(topAxisVisible.getValue()==TRUE) {
      f_xAxis_Update = TRUE; //FIXME : we should have a f_topAxis_Update
    }    
  }    
  if(fOld_topMargin!=topMargin.getValue()) {
    fOld_topMargin = topMargin.getValue();
    f_yAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
    if(topAxisVisible.getValue()==TRUE) {
      f_xAxis_Update = TRUE; //FIXME : we should have a f_topAxis_Update
    }    
  }    
  if(fOld_downMargin!=downMargin.getValue()) {
    fOld_downMargin = downMargin.getValue();
    f_zAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
  }    
  if(fOld_upMargin!=upMargin.getValue()) {
    fOld_upMargin = upMargin.getValue();
    f_zAxis_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
  }    
  if(fOld_shapeAutomated!=shapeAutomated.getValue()) {
    DUMP_UPDATE_WHAT("shapeAutomated");
    fOld_shapeAutomated = shapeAutomated.getValue();
    f_shape_Update = TRUE; 
  }    
  if(fOld_shape!=shape.getValue()) {
    DUMP_UPDATE_WHAT("shape");
    fOld_shape = shape.getValue();
    f_shape_Update = TRUE;
  }    

  if(fOld_titleUp!=titleUp.getValue()) {
    DUMP_UPDATE_WHAT("titleUp");
    fOld_titleUp = titleUp.getValue();
    f_title_Update = TRUE;
  }    
  if(fOld_titleToAxis!=titleToAxis.getValue()) {
    DUMP_UPDATE_WHAT("titleToAxis");
    fOld_titleToAxis = titleToAxis.getValue();
    f_title_Update = TRUE;
  }    
  if(fOld_titleHeight!=titleHeight.getValue()) {
    DUMP_UPDATE_WHAT("titleHeight");
    fOld_titleHeight = titleHeight.getValue();
    f_title_Update = TRUE;
  }    
  if(fOld_titleAutomated!=titleAutomated.getValue()) {
    DUMP_UPDATE_WHAT("titleAutomated");
    fOld_titleAutomated = titleAutomated.getValue();
    f_title_Update = TRUE;
  }    
  if(fOld_superposeBins!=superposeBins.getValue()) {
    DUMP_UPDATE_WHAT("superposeBins");
    fOld_superposeBins = superposeBins.getValue();
    f_bins_Update = TRUE;
  }    
  if(fOld_colorMapping!=colorMapping.getValue()) {
    DUMP_UPDATE_WHAT("colorMapping");
    fOld_colorMapping = colorMapping.getValue();
    //FIXME f_xxx_Update = TRUE;
  }    

  if(fOld_xAxisAutomated!=xAxisAutomated.getValue()) {
    DUMP_UPDATE_WHAT("xAxisAutomated");
    fOld_xAxisAutomated = xAxisAutomated.getValue();
    f_xAxis_Update = TRUE;
  }    
  if(fOld_xAxisMinimum!=xAxisMinimum.getValue()) {
    DUMP_UPDATE_WHAT("xAxisMinimum");
    fOld_xAxisMinimum = xAxisMinimum.getValue();
    f_xAxis_Update = TRUE;
  }    
  if(fOld_xAxisMaximum!=xAxisMaximum.getValue()) {
    DUMP_UPDATE_WHAT("xAxisMaximum");
    fOld_xAxisMaximum = xAxisMaximum.getValue();
    f_xAxis_Update = TRUE;
  }    
  if(fOld_xAxisLogScale!=xAxisLogScale.getValue()) {
    DUMP_UPDATE_WHAT("xAxisLogScale");
    fOld_xAxisLogScale = xAxisLogScale.getValue();
    f_xAxis_Update = TRUE;
  }    

  if(fOld_yAxisAutomated!=yAxisAutomated.getValue()) {
    DUMP_UPDATE_WHAT("yAxisAutomated");
    fOld_yAxisAutomated = yAxisAutomated.getValue();
    f_yAxis_Update = TRUE;
  }    
  if(fOld_yAxisMinimum!=yAxisMinimum.getValue()) {
    DUMP_UPDATE_WHAT("yAxisMinimum");
    fOld_yAxisMinimum = yAxisMinimum.getValue();
    f_yAxis_Update = TRUE;
  }    
  if(fOld_yAxisMaximum!=yAxisMaximum.getValue()) {
    DUMP_UPDATE_WHAT("yAxisMaximum");
    fOld_yAxisMaximum = yAxisMaximum.getValue();
    f_yAxis_Update = TRUE;
  }    
  if(fOld_yAxisLogScale!=yAxisLogScale.getValue()) {
    DUMP_UPDATE_WHAT("yAxisLogScale");
    fOld_yAxisLogScale = yAxisLogScale.getValue();
    f_yAxis_Update = TRUE;
  }    

  if(fOld_zAxisAutomated!=zAxisAutomated.getValue()) {
    DUMP_UPDATE_WHAT("zAxisAutomated");
    fOld_zAxisAutomated = zAxisAutomated.getValue();
    f_zAxis_Update = TRUE;
  }    
  if(fOld_zAxisMinimum!=zAxisMinimum.getValue()) {
    DUMP_UPDATE_WHAT("zAxisMinimum");
    fOld_zAxisMinimum = zAxisMinimum.getValue();
    f_zAxis_Update = TRUE;
  }    
  if(fOld_zAxisMaximum!=zAxisMaximum.getValue()) {
    DUMP_UPDATE_WHAT("zAxisMaximum");
    fOld_zAxisMaximum = zAxisMaximum.getValue();
    f_zAxis_Update = TRUE;
  }    
  if(fOld_zAxisLogScale!=zAxisLogScale.getValue()) {
    DUMP_UPDATE_WHAT("zAxisLogScale");
    fOld_zAxisLogScale = zAxisLogScale.getValue();
    f_zAxis_Update = TRUE;
  }    

  if(fOld_topAxisVisible!=topAxisVisible.getValue()) {
    DUMP_UPDATE_WHAT("topAxisVisible");
    fOld_topAxisVisible = topAxisVisible.getValue();
    f_xAxis_Update = TRUE;
  }    
  if(fOld_rightAxisVisible!=rightAxisVisible.getValue()) {
    DUMP_UPDATE_WHAT("rightAxisVisible");
    fOld_rightAxisVisible = rightAxisVisible.getValue();
    f_yAxis_Update = TRUE;
  }    
  if(fOld_numberOfLevels!=numberOfLevels.getValue()) {
    DUMP_UPDATE_WHAT("numberOfLevels");
    fOld_numberOfLevels = numberOfLevels.getValue();
    f_function_Update = TRUE;
  }    

  // X axis :
 {SoLineStyle* soStyle = xAxisNode->getLineStyle();
  if(fOld_xAxis_lineStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("xAxis_lineStyle");
    fOld_xAxis_lineStyle = soStyle->getSbStyle();
    f_xAxis_Update = TRUE;
  }}
 {SoLineStyle* soStyle = xAxisNode->getTicksStyle();
  if(fOld_xAxis_ticksStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("xAxis_ticksStyle");
    fOld_xAxis_ticksStyle = soStyle->getSbStyle();
    f_xAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = xAxisNode->getLabelsStyle();
  if(fOld_xAxis_labelsStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("xAxis_labelsStyle");
    fOld_xAxis_labelsStyle = soStyle->getSbStyle();
    f_xAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = xAxisNode->getTitleStyle();
  if(fOld_xAxis_titleStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("xAxis_titleStyle");
    fOld_xAxis_titleStyle = soStyle->getSbStyle();
    f_xAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = xAxisNode->getMagStyle();
  if(fOld_xAxis_magStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("xAxis_magStyle");
    fOld_xAxis_magStyle = soStyle->getSbStyle();
    f_xAxis_Update = TRUE;
  }}
  if(fOld_xAxis_modeling!=xAxisNode->modeling.getValue()) {
    DUMP_UPDATE_WHAT("xAxis_modeling");
    fOld_xAxis_modeling = xAxisNode->modeling.getValue();
    f_xAxis_Update = TRUE;
  }

  // Y axis :
 {SoLineStyle* soStyle = yAxisNode->getLineStyle();
  if(fOld_yAxis_lineStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("yAxis_lineStyle");
    fOld_yAxis_lineStyle = soStyle->getSbStyle();
    f_yAxis_Update = TRUE;
  }}
 {SoLineStyle* soStyle = yAxisNode->getTicksStyle();
  if(fOld_yAxis_ticksStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("yAxis_ticksStyle");
    fOld_yAxis_ticksStyle = soStyle->getSbStyle();
    f_yAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = yAxisNode->getLabelsStyle();
  if(fOld_yAxis_labelsStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("yAxis_labelsStyle");
    fOld_yAxis_labelsStyle = soStyle->getSbStyle();
    f_yAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = yAxisNode->getTitleStyle();
  if(fOld_yAxis_titleStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("yAxis_titleStyle");
    fOld_yAxis_titleStyle = soStyle->getSbStyle();
    f_yAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = yAxisNode->getMagStyle();
  if(fOld_yAxis_magStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("yAxis_magStyle");
    fOld_yAxis_magStyle = soStyle->getSbStyle();
    f_yAxis_Update = TRUE;
  }}
  if(fOld_yAxis_modeling!=yAxisNode->modeling.getValue()) {
    DUMP_UPDATE_WHAT("yAxis_modeling");
    fOld_yAxis_modeling = yAxisNode->modeling.getValue();
    f_yAxis_Update = TRUE;
  }

  // Z axis :
 {SoLineStyle* soStyle = zAxisNode->getLineStyle();
  if(fOld_zAxis_lineStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("zAxis_lineStyle");
    fOld_zAxis_lineStyle = soStyle->getSbStyle();
    f_zAxis_Update = TRUE;
  }}
 {SoLineStyle* soStyle = zAxisNode->getTicksStyle();
  if(fOld_zAxis_ticksStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("zAxis_ticksStyle");
    fOld_zAxis_ticksStyle = soStyle->getSbStyle();
    f_zAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = zAxisNode->getLabelsStyle();
  if(fOld_zAxis_labelsStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("zAxis_labelsStyle");
    fOld_zAxis_labelsStyle = soStyle->getSbStyle();
    f_zAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = zAxisNode->getTitleStyle();
  if(fOld_zAxis_titleStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("zAxis_titleStyle");
    fOld_zAxis_titleStyle = soStyle->getSbStyle();
    f_zAxis_Update = TRUE;
  }}
 {SoTextStyle* soStyle = zAxisNode->getMagStyle();
  if(fOld_zAxis_magStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("zAxis_magStyle");
    fOld_zAxis_magStyle = soStyle->getSbStyle();
    f_zAxis_Update = TRUE;
  }}
  if(fOld_zAxis_modeling!=zAxisNode->modeling.getValue()) {
    DUMP_UPDATE_WHAT("zAxis_modeling");
    fOld_zAxis_modeling = zAxisNode->modeling.getValue();
    f_zAxis_Update = TRUE;
  }
  if(f_yAxis_Update==TRUE) {
    // In shape = XYZ z axis position depends of axis y width.
    f_zAxis_Update = TRUE;
  }

 {SoStyle* soStyle = getWallStyle();
  if(fOld_wallStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("wallStyle");
    fOld_wallStyle = soStyle->getSbStyle();
    f_wall_Update = TRUE;
  }}

 {SoStyle* soStyle = getGridStyle();
  if(fOld_gridStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("gridStyle");
    fOld_gridStyle = soStyle->getSbStyle();
    f_grid_Update = TRUE;
  }}

 {SoStyle* soStyle = getInnerFrameStyle();
  if(fOld_innerFrameStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("innerFrameStyle");
    fOld_innerFrameStyle = soStyle->getSbStyle();
    f_innerFrame_Update = TRUE;
  }}

 {SoTextStyle* soStyle = getTitleStyle();
  if(fOld_titleStyle.isEqual(soStyle->getSbStyle())==FALSE) {
    DUMP_UPDATE_WHAT("titleStyle");
    fOld_titleStyle = soStyle->getSbStyle();
    f_title_Update = TRUE;
  }}

  //FIXME f_colorMapping_Update = TRUE;

  // f_[bins,points,function]_Update also touched 
  // in [clear,set]PlottableObjects.
  if(isBinsStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("BinsStyleChanged");
    f_bins_Update = TRUE;
  }
  if(isRightHatchStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("RightHatchStyleChanged");
    f_bins_Update = TRUE;
  }
  if(isLeftHatchStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("LeftHatchStyleChanged");
    f_bins_Update = TRUE;
  }
  if(isErrorsStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("ErrorsStyleChanged");
    f_bins_Update = TRUE;
  }
  if(isPointsStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("PointsStyleChanged");
    f_points_Update = TRUE;
  }
  if(isFunctionStyleChanged()==TRUE) {
    DUMP_UPDATE_WHAT("FunctionStyleChanged");
    f_function_Update = TRUE;
  }

  if(fOld_ttfScale!=ttfScale.getValue()) {
    fOld_ttfScale = ttfScale.getValue();
    f_ttfScale_Update = TRUE;
   {SoTextStyle* style = (SoTextStyle*)titleStyle.getValue();
    if(SbStringIsTTF(style->fontName.getValue())==TRUE) {
      f_title_Update = TRUE;
    }}
    //FIXME : f_colorMapping_Update = TRUE;
  }

  if(f_shape_Update==TRUE) {
    f_bins_Update = TRUE; 
    f_points_Update = TRUE;
    f_function_Update = TRUE;
    f_wall_Update = TRUE;
    f_grid_Update = TRUE;
    f_title_Update = TRUE;
    f_innerFrame_Update = TRUE;
  }

  if( (f_bins_Update==TRUE) || 
      (f_points_Update==TRUE) ||
      (f_function_Update==TRUE)) {
    DUMP_UPDATE_WHAT("bins points function");
    f_xAxis_Update = TRUE;
    f_yAxis_Update = TRUE;
    f_zAxis_Update = TRUE;
    f_dataScale_Update = TRUE;
    f_layout_Update = TRUE;
  }

  if( (f_xAxis_Update==TRUE) || 
      (f_yAxis_Update==TRUE) ||
      (f_zAxis_Update==TRUE)) {
    DUMP_UPDATE_WHAT("xyzAxis");
    f_grid_Update = TRUE;
  }

  if(verbose.getValue()==TRUE) {
    if(f_shape_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update shape");
    if(f_xAxis_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update xAxis");
    if(f_yAxis_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update yAxis");
    if(f_zAxis_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update zAxis");
    if(f_wall_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update wall");
    if(f_grid_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update grid");
    if(f_innerFrame_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update innerFrame");
    if(f_title_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update title");
    if(f_dataScale_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update dataScale");
    if(f_bins_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update bins");
    if(f_points_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update points");
    if(f_function_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update function");
    if(f_layout_Update==TRUE) 
      SoDebugError::postInfo("SoPlotter::updateWhat","update layout");
  }
}

//FIXME : we should avoid ERASE_ALL (and f_*_Update = TRUE in 
// the updateChildren method).

// Will clear all sub scene graphs.
#define ERASE_ALL \
    f_bins_Update = TRUE;\
    f_points_Update = TRUE;\
    f_function_Update = TRUE;\
    f_wall_Update = TRUE;\
    f_grid_Update = TRUE;\
    f_innerFrame_Update = TRUE;\
    f_title_Update = TRUE;\
    f_colorMapping_Update = TRUE;\
    f_xAxis_Update = TRUE;\
    f_yAxis_Update = TRUE;\
    f_zAxis_Update = TRUE;\
    drawData = FALSE;\
    drawWall = FALSE;\
    drawGrid = FALSE;\
    drawInnerFrame = FALSE;\
    drawTitle = FALSE;\
    drawColorMapping = FALSE;\
    drawXaxis = FALSE;\
    drawYaxis = FALSE;\
    drawZaxis = FALSE;\
    drawInfos = FALSE;\
    drawLegend = FALSE;

//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateChildren(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(frozen.getValue()==TRUE) return;

#ifdef DEBUG
  printf("debug : SoPlotter::updateChildren : %ld\n",this);
#endif

  updateWhat();

  if(f_shape_Update==TRUE) {
    f_shape_Update = FALSE;
    updateShape();
  }

  if( (f_bins_Update==TRUE) || 
      (f_points_Update==TRUE) ||
      (f_function_Update==TRUE)) {
    updateSbAxes();
  }

  if(f_layout_Update==TRUE) {
    f_layout_Update = FALSE;
    updateLayout();
  }

  SbBool drawColorMapping = FALSE; //FIXME
  SoStyle* soInnerFrameStyle = (SoStyle*)innerFrameStyle.getValue();
  SbBool drawInnerFrame = soInnerFrameStyle->visible.getValue();
  SoStyle* soWallStyle = (SoStyle*)wallStyle.getValue();
  SbBool drawWall = soWallStyle->visible.getValue();
  SoStyle* soGridStyle = (SoStyle*)gridStyle.getValue();
  SbBool drawGrid = soGridStyle->visible.getValue();
  SoTextStyle* soTitleStyle = (SoTextStyle*)titleStyle.getValue();
  SbBool drawTitle = soTitleStyle->visible.getValue();
  SbBool drawData = TRUE;
  SbBool drawXaxis = TRUE;
  SbBool drawYaxis = TRUE;
  SbBool drawZaxis = TRUE;
  SbBool drawInfos = TRUE;
  SbBool drawLegend = TRUE;

  if(f_dataScale_Update==TRUE) {
    f_dataScale_Update = FALSE;
    float XSIZ = width.getValue();
    float XMGL = leftMargin.getValue();
    float XMGR = rightMargin.getValue();
    float wData = XSIZ-XMGL-XMGR;

    float YSIZ = height.getValue();
    float YMGL = bottomMargin.getValue();
    float YMGU = topMargin.getValue();
    float hData = YSIZ-YMGL-YMGU;
    SoScale* soDataScale = (SoScale*)dataScale.getValue();
    //FIXME : if doing the below we have problem with picking
    // of bins, innerFrame when passing to PAW style !
    //SbBool flag = soDataScale->scaleFactor.enableNotify(FALSE);
    if(fShape==XY) {
      soDataScale->scaleFactor.setValue(wData,hData,depth.getValue()*0.5F);
    } else if(fShape==XYZ) {
      float ZSIZ = depth.getValue();
      float ZMGD = downMargin.getValue();
      float ZMGU = upMargin.getValue();
      float dData = ZSIZ-ZMGD-ZMGU;
      soDataScale->scaleFactor.setValue(wData,hData,dData);
    }
    //soDataScale->scaleFactor.enableNotify(flag);
  }

  SbPList bins1DList,bins2DList;
  SbPList points2DList,points3DList;
  SbPList function1DList,function2DList;
  SbPList binsListSwMnMx;
  int objn = fPlottableObjects.getLength();
 {for(int index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[index];
    if(!object) continue; 
    if(!object->isValid()) {
      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo
        ("SoPlotter::updateChildren","object not valid.");
      }
      continue; 
    }
    if(void* h1 = Cast(*object,SbPlottableBins1D)) {
      bins1DList.append(h1);
      float bmin,bmax; // Oftenly used, save them.
      ((SbPlottableBins1D*)h1)->getBinsSumOfWeightsRange(bmin,bmax);
      binsListSwMnMx.append(new SbVec2f(bmin,bmax));
    } else if(void* h2 = Cast(*object,SbPlottableBins2D)) {
      bins2DList.append(h2);
    } else if(void* c2 = Cast(*object,SbPlottablePoints2D)) {
      points2DList.append(c2);
    } else if(void* c3 = Cast(*object,SbPlottablePoints3D)) {
      points3DList.append(c3);
    } else if(void* f = Cast(*object,SbPlottableFunction1D)) {
      function1DList.append(f);
    } else if(void* f = Cast(*object,SbPlottableFunction2D)) {
      function2DList.append(f);
    }
  }}

  int bins1DNumber = bins1DList.getLength();
  int bins2DNumber = bins2DList.getLength();
  int points2DNumber = points2DList.getLength();
  int points3DNumber = points3DList.getLength();
  int function1DNumber = function1DList.getLength();
  int function2DNumber = function2DList.getLength();
  if( (bins1DNumber==0) && (bins2DNumber==0) && 
      (points2DNumber==0) && (points3DNumber==0) && 
      (function1DNumber==0) && (function2DNumber==0) ) { 
    DUMP_UPDATE_WHAT("no plottable");
    // Will clear all sub scene graphs.
    ERASE_ALL
  }

#ifdef DEBUG
  printf("debug : SoPlotter::updateChildren : objn = %d ; bins1D = %d ; bins2D = %d ; points2D = %d ; points3D = %d ; function = %d\n",
  objn,bins1DNumber,bins2DNumber,points2DNumber,points3DNumber,functionNumber);
#endif

  SbBool superpose = superposeBins.getValue();
  if(superpose==TRUE) {
    // Check compatibility of bins :
    if( (bins1DNumber<=0) || (fShape!=XY) ) {
      superpose = FALSE;
    } else {
      SbPlottableBins1D* bins = (SbPlottableBins1D*)bins1DList[0];
      if(bins->getDimension()!=1) {
        superpose = FALSE;
      } else {
        int xnbin = bins->getAxisNumberOfBins();
        float xmn = bins->getAxisMinimum();
        float xmx = bins->getAxisMaximum();
        superpose = TRUE;
        for(int ibins=1;ibins<bins1DNumber;ibins++) {
          SbPlottableBins1D* binsloop = 
            (SbPlottableBins1D*)bins1DList[ibins];
          if( (xnbin!=binsloop->getAxisNumberOfBins()) ||
              (xmn!=binsloop->getAxisMinimum()) ||
              (xmx!=binsloop->getAxisMaximum()) ) {
            superpose = FALSE;
            break;
          }       
        }       
        if(superpose==TRUE) { //Compatible bins :
          if(yAxisAutomated.getValue()==TRUE) {
            // Correct Y axis if XY shape and superposing bins.
            // Get min/max
            float bmin,bmax;
            getHeight(bins1DNumber-1,bins1DList,binsListSwMnMx,0,bmin,bmax);
            bmin = bmax;            
            for(int ibin=1;ibin<xnbin;ibin++) {
              float mini,maxi;
              getHeight
                (bins1DNumber-1,bins1DList,binsListSwMnMx,ibin,mini,maxi);
              bmin = SbMinimum(bmin,maxi);
              bmax = SbMaximum(bmax,maxi);
            }
            f_yDataAxis.setMinimumValue(bmin);
            f_yDataAxis.setMaximumValue(bmax);
            f_yDataAxis.adjustAxis();
          }
        }
      }       
    }
  }

  float xmin =  f_xDataAxis.getMinimumValue();
  float xmax =  f_xDataAxis.getMaximumValue();
  SbBool xlog = f_xDataAxis.isLogScale();
  if(xlog==TRUE) {
    if((xmin<=0) || (xmax<=0) ) {
      f_xDataAxis.adjustAxis();
      xmin = f_xDataAxis.getMinimumValue();
      xmax = f_xDataAxis.getMaximumValue();
      // Now should have reasonable values.
    }
    if((xmin<=0) || (xmax<=0) ) {
      xlog = FALSE;
    } else {
      xmin = FLOG10(xmin);
      xmax = FLOG10(xmax);
    }
  }

  float ymin =  f_yDataAxis.getMinimumValue();
  float ymax =  f_yDataAxis.getMaximumValue();
  SbBool ylog = f_yDataAxis.isLogScale();
  if(ylog==TRUE) {
    if((ymin<=0) || (ymax<=0) ) {
      f_yDataAxis.adjustAxis();
      ymin = f_yDataAxis.getMinimumValue();
      ymax = f_yDataAxis.getMaximumValue();
      // Now should have reasonable values.
    }
    if((ymin<=0) || (ymax<=0) ) {
      ylog = FALSE;
    }else{
      ymin = FLOG10(ymin);
      ymax = FLOG10(ymax);
    }
  }

  float zmin =  f_zDataAxis.getMinimumValue();
  float zmax =  f_zDataAxis.getMaximumValue();
  SbBool zlog = f_zDataAxis.isLogScale();
  if(zlog==TRUE) {
    if((zmin<=0) || (zmax<=0) ) {
      f_zDataAxis.adjustAxis();
      zmin = f_zDataAxis.getMinimumValue();
      zmax = f_zDataAxis.getMaximumValue();
      // Now should have reasonable values.
    }
    if((zmin<=0) || (zmax<=0) ) {
      zlog = FALSE;
    }else{
      zmin = FLOG10(zmin);
      zmax = FLOG10(zmax);
    }
  }

 {float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;
  if(fShape==XY) {
    if( (xmin>=xmax) || (ymin>=ymax) ) {
      DUMP_UPDATE_WHAT("bad min/max axes");
      ERASE_ALL
    }
    if( (wData<=0) || (hData<=0) ) {
      DUMP_UPDATE_WHAT("null data area");
      ERASE_ALL
      SoDebugError::postInfo("SoPlotter::updateChildren","null data area.");
    }
  } else if(fShape==XYZ) {
    float ZSIZ = depth.getValue();
    float ZMGD = downMargin.getValue();
    float ZMGU = upMargin.getValue();
    float dData = ZSIZ-ZMGD-ZMGU;
    if( (xmin>=xmax) || (ymin>=ymax) || (zmin>=zmax) ) {
      DUMP_UPDATE_WHAT("bad min/max axes");
      ERASE_ALL
    }
    if( (wData<=0) || (hData<=0) || (dData<=0) ) {
      DUMP_UPDATE_WHAT("null data area");
      ERASE_ALL
      SoDebugError::postInfo("SoPlotter::updateChildren","null data area.");
    }
  }}

  float dx   = xmax - xmin;
  float dy   = ymax - ymin;
  float dz   = zmax - zmin;

  SbRepBox boxX(xmin,dx,xlog);
  SbRepBox boxY(ymin,dy,ylog);
  SbRepBox boxZ(zmin,dz,zlog);

  if((f_title_Update==TRUE) ||
     (f_bins_Update==TRUE) ||
     (f_points_Update==TRUE) ||
     (f_function_Update==TRUE) ) {
    if(titleAutomated.getValue()==TRUE) {
      SbString s = getTitle();
      if(title.getValue()!=s) {
        //SbBool flag = title.enableNotify(FALSE);//FIXME
        title.setValue(s); //Output
        //title.enableNotify(flag);//FIXME
      }
    }
  }

  if((f_bins_Update==TRUE) ||
     (f_points_Update==TRUE) ||
     (f_function_Update==TRUE) ) {

    // Infos, legend concern data only.
    // WARNING : infos field is an "output" field. 

    infos.deleteValues(0);
    if(drawInfos==TRUE ) {

      //SbBool flag = infos.enableNotify(FALSE);//FIXME
      SbString sinfos = getInfos();
      SbPList lines = SbStringGetLines(sinfos);
      int linen = lines.getLength();
      for(int index=0;index<linen;index++) {
        SbString* line = ((SbString*)lines[index]);
        infos.set1Value(index,*line);
        delete line;
      }
      //infos.enableNotify(flag);

    }

    if(drawLegend==FALSE) {
      legend.deleteValues(0);
    } else {
      updateLegend();
    }

    f_innerFrame_Update = TRUE;
  }

  if(fShape==XY) {

    float d = 1./4;

    float ZOFFSET = d/(bins1DNumber+bins2DNumber+points2DNumber+1);
    float zz = 0;

    if(f_bins_Update==TRUE) {
      f_bins_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XY : update bins",(unsigned long)this);
      }

      SoSeparator* binsNode = (SoSeparator*)binsSeparator.getValue();
      //SbBool flag = binsNode->enableNotify(FALSE);//FIXME
      binsNode->removeAllChildren();
      SoSeparator* errorsNode = (SoSeparator*)errorsSeparator.getValue();
      //SbBool flag2 = errorsNode->enableNotify(FALSE);//FIXME
      errorsNode->removeAllChildren();
      if(drawData==TRUE) {
        int ibins;
        for(ibins=0;ibins<bins1DNumber;ibins++) {
          SbPlottableBins1D* binsloop = (SbPlottableBins1D*)bins1DList[ibins];
          zz += ZOFFSET; // ibins = 0 back (PAW convention).
          updateBins1D_XY(*binsloop,ibins,bins1DList,binsListSwMnMx,superpose,
                          boxX,boxY,zz);        
        }
        for(ibins=0;ibins<bins2DNumber;ibins++) {
          SbPlottableBins2D* binsloop = (SbPlottableBins2D*)bins2DList[ibins];
          updateBins2D_XY(*binsloop,ibins,boxX,boxY,boxZ,zz,drawColorMapping);
        }
      }
      //binsNode->enableNotify(flag);//FIXME
      //errorsNode->enableNotify(flag2);//FIXME
    }

    if(f_points_Update==TRUE) {
      f_points_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XY : update points",(unsigned long)this);
      }

      SoSeparator* pointsNode = (SoSeparator*)pointsSeparator.getValue();
      //SbBool flag = pointsNode->enableNotify(FALSE);//FIXME
      pointsNode->removeAllChildren();
      if(drawData==TRUE) {
        for(int ipoints=0;ipoints<points2DNumber;ipoints++) {
          SbPlottablePoints2D* pointsloop = 
            (SbPlottablePoints2D*)points2DList[ipoints];
          //FIXME : handle zz.
          updatePoints2D_XY(*pointsloop,ipoints,boxX,boxY);
        }
      }
      //pointsNode->enableNotify(flag);//FIXME
    }

    if(f_function_Update==TRUE) {
      f_function_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XY : update function",(unsigned long)this);
      }

      SoSeparator* functionNode = (SoSeparator*)functionSeparator.getValue();
      //SbBool flag = functionNode->enableNotify(FALSE);//FIXME
      functionNode->removeAllChildren();
      if(drawData==TRUE) {
        //zz = 0; // Functions in front.
       {for(int ifunc=0;ifunc<function1DNumber;ifunc++) {
          SbPlottableFunction1D*  funcloop = 
            (SbPlottableFunction1D*)function1DList[ifunc];
          zz += ZOFFSET;
          updateFunction_XY(*funcloop,ifunc,boxX,boxY,boxZ,zz);
        }}
       {for(int ifunc=0;ifunc<function2DNumber;ifunc++) {
          SbPlottableFunction2D*  funcloop = 
            (SbPlottableFunction2D*)function2DList[ifunc];
          zz += ZOFFSET;
          updateFunction_XY(*funcloop,ifunc,boxX,boxY,boxZ,zz);
        }}
      }
      //functionNode->enableNotify(flag);//FIXME
    }    

  } else if(fShape==XYZ) {
    
    if(f_bins_Update==TRUE) {
      f_bins_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XYZ : update bins",(unsigned long)this);
      }

      SoSeparator* binsNode = (SoSeparator*)binsSeparator.getValue();
      //SbBool flag = binsNode->enableNotify(FALSE);//FIXME
      binsNode->removeAllChildren();
      if(drawData==TRUE) {
        for(int ibins=0;ibins<bins2DNumber;ibins++) {
          SbPlottableBins2D* binsloop = (SbPlottableBins2D*)bins2DList[ibins];
          updateBins2D_XYZ(*binsloop,ibins,boxX,boxY,boxZ);
        }
        //drawColorMapping = TRUE;
      }
      //binsNode->enableNotify(flag);//FIXME
    }

    if(f_points_Update==TRUE) {
      f_points_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XYZ : update points",(unsigned long)this);
      }

      SoSeparator* pointsNode = (SoSeparator*)pointsSeparator.getValue();
      //SbBool flag = pointsNode->enableNotify(FALSE);//FIXME
      pointsNode->removeAllChildren();
      if(drawData==TRUE) {
        for(int ipoints=0;ipoints<points3DNumber;ipoints++) {
          SbPlottablePoints3D* pointsloop = 
            (SbPlottablePoints3D*)points3DList[ipoints];
          updatePoints3D_XYZ(*pointsloop,ipoints,boxX,boxY,boxZ);
        }
      }
      //pointsNode->enableNotify(flag);//FIXME
    }

    if(f_function_Update==TRUE) {
      f_function_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : XYZ : update function",(unsigned long)this);
      }

      SoSeparator* functionNode = (SoSeparator*)functionSeparator.getValue();
      //SbBool flag = functionNode->enableNotify(FALSE);//FIXME
      functionNode->removeAllChildren();
      if(drawData==TRUE) {
        for(int ifunc=0;ifunc<function2DNumber;ifunc++) {
          SbPlottableFunction2D* funcloop = 
            (SbPlottableFunction2D*)function2DList[ifunc];
          updateFunction_XYZ(*funcloop,ifunc,boxX,boxY,boxZ);
        }
      }
      //functionNode->enableNotify(flag);//FIXME
    }

  }

  // Inner frame for XY, must be in front of grid :
  if(f_innerFrame_Update==TRUE) {
    f_innerFrame_Update = FALSE;

    if(verbose.getValue()==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateChildren",
      "%lu : update innerFrame : draw %d : shape %d",
      (unsigned long)this,drawInnerFrame,fShape);
    }

    SoSeparator* innerFrameNode = 
      (SoSeparator*)innerFrameSeparator.getValue();
    //SbBool flag = innerFrameNode->enableNotify(FALSE);//FIXME
    innerFrameNode->removeAllChildren();
    if(drawInnerFrame==TRUE) {
      if(fShape==XY) {
        updateInnerFrame_XY();
      } else {
        updateInnerFrame_XYZ();
      }
    }	
    //innerFrameNode->enableNotify(flag);//FIXME
  }

#ifdef DEBUG
  printf("SoPlotter::updateChildren : Creation of Axis in USG Shape : %d  X?:%d Y?:%d Z?:%d\n",fShape,f_xAxis_Update,f_yAxis_Update,f_zAxis_Update);
#endif

  if(f_ttfScale_Update) {
    f_ttfScale_Update = FALSE;
    xAxisNode->ttfScale.setValue(ttfScale.getValue());
    yAxisNode->ttfScale.setValue(ttfScale.getValue());
    zAxisNode->ttfScale.setValue(ttfScale.getValue());
  }

  if(fShape==XY) {
    if(f_xAxis_Update==TRUE) {
      f_xAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update X axis",(unsigned long)this);
      }

      if(drawXaxis==FALSE) {
        if(xAxisNode->width.getValue()!=0) xAxisNode->width.setValue(0);
        ((SoSeparator*)topAxisSeparator.getValue())->removeAllChildren();
      } else {
        updateXAxis2D();    
        updateAxis(xAxisNode,f_xDataAxis);
        ((SoSeparator*)topAxisSeparator.getValue())->removeAllChildren();
        if(topAxisVisible.getValue()==TRUE) updateTopAxis(); 
      }
    }
    if(f_yAxis_Update==TRUE) {
      f_yAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update Y axis",(unsigned long)this);
      }

      if(drawYaxis==FALSE) {
        if(yAxisNode->width.getValue()!=0) yAxisNode->width.setValue(0);
        ((SoSeparator*)rightAxisSeparator.getValue())->removeAllChildren();
      } else {
        updateYAxis2D();
        updateAxis(yAxisNode,f_yDataAxis);
        ((SoSeparator*)rightAxisSeparator.getValue())->removeAllChildren();
        if(rightAxisVisible.getValue()==TRUE) updateRightAxis(); 
      }
    }
    if(f_zAxis_Update==TRUE) {
      f_zAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update Z axis",(unsigned long)this);
      }

      if(drawZaxis==FALSE) {
        if(zAxisNode->width.getValue()!=0) zAxisNode->width.setValue(0);
      } else {
        updateZAxis2D();    
      }
    }
  } else if(fShape==XYZ) {
    ((SoSeparator*)topAxisSeparator.getValue())->removeAllChildren();
    ((SoSeparator*)rightAxisSeparator.getValue())->removeAllChildren();
    if(f_xAxis_Update==TRUE) {
      f_xAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update X axis",(unsigned long)this);
      }

      if(drawXaxis==FALSE) {
        if(xAxisNode->width.getValue()!=0) xAxisNode->width.setValue(0);
      } else {
        updateXAxis3D();    
        updateAxis(xAxisNode,f_xDataAxis);
      }
    }
    if(f_yAxis_Update==TRUE) {
      f_yAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update Y axis",(unsigned long)this);
      }

      if(drawYaxis==FALSE) {
        if(yAxisNode->width.getValue()!=0) yAxisNode->width.setValue(0);
      } else {
        updateYAxis3D();    
        updateAxis(yAxisNode,f_yDataAxis);
      }
    }
    if(f_zAxis_Update==TRUE) {
      f_zAxis_Update = FALSE;

      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo("SoPlotter::updateChildren",
        "%lu : update Z axis",(unsigned long)this);
      }

      if(drawZaxis==FALSE) {
        if(zAxisNode->width.getValue()!=0) zAxisNode->width.setValue(0);
      } else {
        updateZAxis3D();    
        updateAxis(zAxisNode,f_zDataAxis);
      }
    }
  }

  if(f_wall_Update==TRUE) {
    f_wall_Update = FALSE;

    if(verbose.getValue()==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateChildren",
      "%lu : update wall",(unsigned long)this);
    }

    SoSeparator* wallSeparatorNode = (SoSeparator*)wallSeparator.getValue();
    //SbBool flag = wallSeparatorNode->enableNotify(FALSE);//FIXME
    wallSeparatorNode->removeAllChildren();
    if(drawWall==TRUE) {
      if(fShape==XY) {
        updateWall2D();
      } else if(fShape==XYZ) {
        updateWall3D();
      }
    }
    //wallSeparatorNode->enableNotify(flag);//FIXME
  }

  if(f_grid_Update==TRUE) {
    f_grid_Update = FALSE;

    if(verbose.getValue()==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateChildren",
      "%lu : update grid",(unsigned long)this);
    }

    SoSeparator* gridSeparatorNode = (SoSeparator*)gridSeparator.getValue();
    //SbBool flag = gridSeparatorNode->enableNotify(FALSE);//FIXME
    gridSeparatorNode->removeAllChildren();
    if(drawGrid==TRUE) {
      if(fShape==XY) {
        updateGrid2D();
      } else if(fShape==XYZ) {
        updateGrid3D();
      }
    }
    //gridSeparatorNode->enableNotify(flag);//FIXME
  }

  if(f_title_Update==TRUE) {
    f_title_Update = FALSE;

    if(verbose.getValue()==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateChildren",
      "%lu : update title",(unsigned long)this);
    }

    SoSeparator* titleNode = (SoSeparator*)titleSeparator.getValue();
    //SbBool flag = titleNode->enableNotify(FALSE);//FIXME
    titleNode->removeAllChildren();
    if(drawTitle==TRUE) updateTitle();
    //titleNode->enableNotify(flag);//FIXME
  }
  
  if(f_colorMapping_Update==TRUE) {
    f_colorMapping_Update = FALSE;

    if(verbose.getValue()==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateChildren",
      "%lu : update colormapping",(unsigned long)this);
    }

    SoSeparator* colorMappingNode = 
      (SoSeparator*)colorMappingSeparator.getValue();
    //SbBool flag = colorMappingNode->enableNotify(FALSE);//FIXME
    colorMappingNode->removeAllChildren();
    if(drawColorMapping==TRUE) updateColorMapping();
    //colorMappingNode->enableNotify(flag);//FIXME
  }

 {for(int count=0;count<bins1DNumber;count++) {
    delete ((SbVec2f*)binsListSwMnMx[count]);
  }}      
}
//////////////////////////////////////////////////////////////////////////////
SoPlotter::Shape SoPlotter::getShape(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SbPlottableObject* major_bins = 0;
  SbPlottableObject* major_points = 0;
  SbPlottableObject* major_function = 0;

  int objn = fPlottableObjects.getLength();
  for(int index=0;index<objn;index++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[index];
    if(!object) continue; 
    if(!object->isValid()) {
      //SoDebugError::postInfo
      //("SoPlotter::getShape","object \"%s\" not valid.",
      // object->getName());
      continue; 
    }
    if(Cast(*object,SbPlottableBins1D)) {
      if(!major_bins) major_bins = object;
    } else if(Cast(*object,SbPlottableBins2D)) {
      if(!major_bins) major_bins = object;
    } else if(Cast(*object,SbPlottablePoints2D)) {
      if(!major_points) major_points = object;
    } else if(Cast(*object,SbPlottablePoints3D)) {
      if(!major_points) major_points = object;
    } else if(Cast(*object,SbPlottableFunction1D)) {
      if(!major_function) major_function = object;
    } else if(Cast(*object,SbPlottableFunction2D)) {
      if(!major_function) major_function = object;
    } else {
      SoDebugError::postInfo
        ("SoPlotter::getShape","unkown plottable object type.");
    }
  }

  Shape retShape = XY;

  if(shapeAutomated.getValue()==TRUE) {
    // Guess XY or XYZ shape :
    if(major_bins) { // major bins compells shape type.
      if(major_bins->getDimension()==1) {
        retShape = XY;
      } else if(major_bins->getDimension()==2) {
        retShape = XY; //lego is not the default.
      } else {
        retShape = XYZ;
      }
    } else if(major_points) { // major points compells shape type.
      if(major_points->getDimension()==1) {
        retShape = XY;
      } else if(major_points->getDimension()==2) {
        retShape = XY;
      } else {
        retShape = XYZ;
      }
    } else if(major_function) { // major function compell shape policy.
      if(major_function->getDimension()==1) {
        retShape = XY;
      } else {
        retShape = XYZ;
      }
    } else {
      retShape = XY; //Default.
    }
  } else {
    retShape = (Shape)shape.getValue();
  }
  return retShape;
}
//////////////////////////////////////////////////////////////////////////////
const SbVec3f& SoPlotter::getScreenDataFrameUpperRightCorner(
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fScreenDataFrameUpperRightCorner;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateShape(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fShape = getShape();
  if(shapeAutomated.getValue()==TRUE) {
    //SbBool flag = shape->enableNotify(FALSE);
    shape.setValue(fShape);
    //shape->enableNotify(flag);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateSbAxes(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef DEBUG
  printf("debug : SoPlotter::setAxes : %ld\n",this);
#endif
  SbPList binsList,pointsList,functionList;
  int objn = fPlottableObjects.getLength();
  for(int count=0;count<objn;count++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[count];
    if(!object) continue; 
    if(!object->isValid()) {
      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo
        ("SoPlotter::setAxes","object not valid.");
      }
      continue; 
    }
    if(Cast(*object,SbPlottableBins1D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottableBins2D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottablePoints2D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottablePoints3D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottableFunction1D)) {
      functionList.append(object);
    } else if(Cast(*object,SbPlottableFunction2D)) {
      functionList.append(object);
    }
  }
  int binsNumber = binsList.getLength();
  int pointsNumber = pointsList.getLength();
  int functionNumber = functionList.getLength();

  // Who fixes the axes ?
  SbPlottableObject* major_bins = 
    (SbPlottableObject*)(binsNumber>0?binsList[0]:0) ;
  SbPlottableObject* major_func = 
    (SbPlottableObject*)(functionNumber>0?functionList[0]:0);
  SbPlottableObject* major_points = 
    (SbPlottableObject*)(pointsNumber>0?pointsList[0]:0);
  
  f_xDataAxis.setMinimumValue(0);
  f_xDataAxis.setMaximumValue(0);
  f_xDataAxis.setLogScale(xAxisLogScale.getValue());

  f_yDataAxis.setMinimumValue(0);
  f_yDataAxis.setMaximumValue(0);
  f_yDataAxis.setLogScale(yAxisLogScale.getValue());

  f_zDataAxis.setMinimumValue(0);
  f_zDataAxis.setMaximumValue(0);
  f_zDataAxis.setLogScale(zAxisLogScale.getValue());

  if(xAxisAutomated.getValue()==FALSE) {
    f_xDataAxis.setMinimumValue(xAxisMinimum.getValue());
    f_xDataAxis.setMaximumValue(xAxisMaximum.getValue());
  }

  if(yAxisAutomated.getValue()==FALSE) {
    f_yDataAxis.setMinimumValue(yAxisMinimum.getValue());
    f_yDataAxis.setMaximumValue(yAxisMaximum.getValue());
  }

  if(zAxisAutomated.getValue()==FALSE) {
    f_zDataAxis.setMinimumValue(zAxisMinimum.getValue());
    f_zDataAxis.setMaximumValue(zAxisMaximum.getValue());
  }

  if(major_bins) { 
    
    if(major_bins->getDimension()==1) {
      if(xAxisAutomated.getValue()==TRUE) {
        SbPlottableBins1D* bins = Cast(*major_bins,SbPlottableBins1D);
        f_xDataAxis.setMinimumValue(bins->getAxisMinimum());
        f_xDataAxis.setMaximumValue(bins->getAxisMaximum());
      }
      
      if(yAxisAutomated.getValue()==TRUE) {
        SbPlottableBins1D* bins = Cast(*major_bins,SbPlottableBins1D);
        float mn,mx;
        bins->getBinsSumOfWeightsRange(mn,mx);
        //f_yDataAxis.setMinimumValue(mn);
        if(mn>=0) mn = 0; //PAW convention.
        f_yDataAxis.setMinimumValue(mn);
        f_yDataAxis.setMaximumValue(mx);
        f_yDataAxis.adjustAxis();
      }
      
    } else if(major_bins->getDimension()==2) {
      if(xAxisAutomated.getValue()==TRUE) {
        SbPlottableBins2D* bins = Cast(*major_bins,SbPlottableBins2D);
        f_xDataAxis.setMinimumValue(bins->getAxisMinimumX());
        f_xDataAxis.setMaximumValue(bins->getAxisMaximumX());
      }

      if(yAxisAutomated.getValue()==TRUE) {
        SbPlottableBins2D* bins = Cast(*major_bins,SbPlottableBins2D);
        f_yDataAxis.setMinimumValue(bins->getAxisMinimumY());
        f_yDataAxis.setMaximumValue(bins->getAxisMaximumY());
      }

      if(zAxisAutomated.getValue()==TRUE) {
        SbPlottableBins2D* bins = Cast(*major_bins,SbPlottableBins2D);
        float mn,mx;
        bins->getBinsSumOfWeightsRange(mn,mx);
        f_zDataAxis.setMinimumValue(mn);
        f_zDataAxis.setMaximumValue(mx);
        f_zDataAxis.adjustAxis();
      }
    } else if(major_bins->getDimension()==3) {
      //FIXME : should do something.
    } else {
      // Unusual case.
    }
    
  } else  if(major_points) { 
    
    if(major_points->getDimension()==2) {
      if(xAxisAutomated.getValue()==TRUE) {
        SbPlottablePoints2D* points = Cast(*major_points,SbPlottablePoints2D);
        f_xDataAxis.setMinimumValue(points->getAxisMinimumX());
        f_xDataAxis.setMaximumValue(points->getAxisMaximumX());
      }      
      if(yAxisAutomated.getValue()==TRUE) {
        SbPlottablePoints2D* points = Cast(*major_points,SbPlottablePoints2D);
        float ymn = points->getAxisMinimumY();
        float ymx = points->getAxisMaximumY();
        // For pawex22 ?
        //f_yDataAxis.setMinimumValue(ymn*1.1F);
        //f_yDataAxis.setMaximumValue(ymx*1.1F);
        f_yDataAxis.setMinimumValue(ymn);
        f_yDataAxis.setMaximumValue(ymx);
      }
      
    } else if(major_points->getDimension()==3) {
      if(xAxisAutomated.getValue()==TRUE) {
        SbPlottablePoints3D* points = Cast(*major_points,SbPlottablePoints3D);
        f_xDataAxis.setMinimumValue(points->getAxisMinimumX());
        f_xDataAxis.setMaximumValue(points->getAxisMaximumX());
      }      
      
      if(yAxisAutomated.getValue()==TRUE) {
        SbPlottablePoints3D* points = Cast(*major_points,SbPlottablePoints3D);
        f_yDataAxis.setMinimumValue(points->getAxisMinimumY());
        f_yDataAxis.setMaximumValue(points->getAxisMaximumY());
      }
      
      if(zAxisAutomated.getValue()==TRUE) {
        SbPlottablePoints3D* points = Cast(*major_points,SbPlottablePoints3D);
        f_zDataAxis.setMinimumValue(points->getAxisMinimumZ());
        f_zDataAxis.setMaximumValue(points->getAxisMaximumZ());
      }
      
    } else {
      // Unusual case.
    }
    
  } else  if(major_func) { 
    
    if(major_func->getDimension()==1) {

      SbPlottableFunction1D* func = Cast(*major_func,SbPlottableFunction1D);

      if(xAxisAutomated.getValue()==TRUE) {
        float xmn = func->getMinimumX();
        float xmx = func->getMaximumX();
        if(xmx<=xmn) {
          xmn = -1;
          xmx = 1;
        }
        f_xDataAxis.setMinimumValue(xmn);
        f_xDataAxis.setMaximumValue(xmx);
      }
      
      if(yAxisAutomated.getValue()==TRUE) {
        float xmn = f_xDataAxis.getMinimumValue();
        float xmx = f_xDataAxis.getMaximumValue();
        int nstp = func->getNumberOfStepsX();

        nstp = nstp <=0 ? 100 : nstp;
        float df = (xmx - xmn)/nstp;      
        SbBool problem = FALSE;
        float vmin;        
        if(!func->value(xmn,vmin)) problem = TRUE;
        float vmax = vmin;
        for(int ibin=0;ibin<=nstp;ibin++) { 
          float xx = xmn + ibin * df;
          float val;
          if(!func->value(xx,val)) problem = TRUE;
          vmax = SbMaximum(vmax,val);
          vmin = SbMinimum(vmin,val);
        }
        if(problem==TRUE) {
          SoDebugError::postInfo("SoPlotter::updateSbAxes",
            " problem when getting some function value.");
        }
        f_yDataAxis.setMinimumValue(vmin);
        f_yDataAxis.setMaximumValue(vmax);
        f_yDataAxis.adjustAxis();
      }
      
    } else if(major_func->getDimension()==2) {

      SbPlottableFunction2D* func = Cast(*major_func,SbPlottableFunction2D);

      if(xAxisAutomated.getValue()==TRUE) {
        float xmn = func->getMinimumX();
        float xmx = func->getMaximumX();
        if(xmx<=xmn) {
          xmn = -1;
          xmx = 1;
        }
        f_xDataAxis.setMinimumValue(xmn);
        f_xDataAxis.setMaximumValue(xmx);
      }

      if(yAxisAutomated.getValue()==TRUE) {
        float ymn =  func->getMinimumY();
        float ymx =  func->getMaximumY();
        if(ymx<=ymn) {
          ymn = -1;
          ymx = 1;
        }
        f_yDataAxis.setMinimumValue(ymn);
        f_yDataAxis.setMaximumValue(ymx);
      }
      
      if(zAxisAutomated.getValue()==TRUE) {

        float xmn = f_xDataAxis.getMinimumValue();
        float xmx = f_xDataAxis.getMaximumValue();
        int nx = func->getNumberOfStepsX();
        nx = nx <=0 ? 20 : nx;

        float ymn = f_yDataAxis.getMinimumValue();
        float ymx = f_yDataAxis.getMaximumValue();
        int ny = func->getNumberOfStepsY();
        ny = ny <=0 ? 20 : ny;

        float dfx = (xmx - xmn)/nx;
        float dfy = (ymx - ymn)/ny;
        
        SbBool problem = FALSE;
        float vmin;
        if(!func->value(xmn,ymn,vmin)) problem = TRUE;
        float vmax = vmin;
        for(int jbin=ny-1;jbin>=0;jbin--) { 
          for(int ibin=nx-1;ibin>=0;ibin--) { 
            float xx = xmn + ibin * dfx;
            float yy = ymn + jbin * dfy;
            float val;
            if(!func->value(xx,yy,val)) problem = TRUE;
            vmax = SbMaximum(vmax,val);
            vmin = SbMinimum(vmin,val);
          }
        }
        if(problem==TRUE) {
          SoDebugError::postInfo("SoPlotter::updateSbAxes",
            " problem when getting some function value.");
        }
        f_zDataAxis.setMinimumValue(vmin);
        f_zDataAxis.setMaximumValue(vmax);
        f_zDataAxis.adjustAxis();
      }
    } else if(major_func->getDimension()==3) {
      //FIXME : should do something.
    } else {
      // Unusual case.
    } 
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateLayout(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;

  SoTransform* transform = (SoTransform*)layout.getValue();
  //SbBool flag = transform->enableNotify(FALSE);
  SbMatrix matrix;
  matrix.makeIdentity();
  transform->setMatrix(matrix);

  if(fShape==XY) {
    // In rep primitives (SoLineSet, etc...), (0,0)
    // is the lower left corner of the data area square; 
    transform->translation.setValue(-XSIZ/2+XMGL,-YSIZ/2+YMGL,-0.5F);
  } else {
    float ZSIZ = depth.getValue();
    float ZMGD = downMargin.getValue();
    float ZMGU = upMargin.getValue();
    float dData = ZSIZ-ZMGD-ZMGU;

    // XYZ global transformation (to have a "lego" layout) :
    //  translate so that the center of the scene 
    //  is the center of the data area cube; 
    //  then rotate to have lego 3D layout.
    SoTranslation* center = new SoTranslation;
    center->ref();
    center->translation.setValue(-wData/2,-hData/2,-dData/2);

    SoScale* scale = new SoScale;
    scale->ref();
    scale->scaleFactor.setValue(0.6F,0.6F,0.6F);

    SoRotationXYZ* z45 = new SoRotationXYZ;
    z45->ref();
    z45->axis = SoRotationXYZ::Z;
    z45->angle = 45*FM_PI/180;

    SoRotationXYZ* x70 = new SoRotationXYZ;
    x70->ref();
    x70->axis = SoRotationXYZ::X;
    x70->angle = -70 * FM_PI/180;

    transform->combineLeft(x70);
    transform->combineLeft(z45);    
    transform->combineLeft(scale);
    transform->combineLeft(center); //Applied first.

    center->unref();
    scale->unref();
    z45->unref();
    x70->unref();
  }

  //transform->enableNotify(flag);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateXAxis2D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  //SbBool flag = xAxisNode->enableNotify(aFlag);
  xAxisNode->verbose.setValue(verbose.getValue());
  xAxisNode->tickUp.setValue(TRUE);
  xAxisNode->width.setValue(wData);

 {SoTextStyle* style = xAxisNode->getLabelsStyle(); 
  style->orientationX.setValue(SbVec3f(1,0,0));
  style->orientationY.setValue(SbVec3f(0,1,0));
  style->horizontalJustification.setValue(SoTextStyle::CENTER);
  style->verticalJustification.setValue(SoTextStyle::TOP);}

 {SoTextStyle* style = xAxisNode->getTitleStyle(); 
  style->orientationX.setValue(SbVec3f(1,0,0));
  style->orientationY.setValue(SbVec3f(0,1,0));
  if(xAxisNode->titleToAxisJustification.getValue()==SoAxis::LEFT) {
    style->horizontalJustification.setValue(SoTextStyle::LEFT);
  } else if(xAxisNode->titleToAxisJustification.getValue()==SoAxis::CENTER) {
    style->horizontalJustification.setValue(SoTextStyle::CENTER);
  } else if(xAxisNode->titleToAxisJustification.getValue()==SoAxis::RIGHT) {
    style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  }  
  style->verticalJustification.setValue(SoTextStyle::TOP);}

 {SoTextStyle* style = xAxisNode->getMagStyle(); 
  style->horizontalJustification.setValue(SoTextStyle::LEFT);
  style->verticalJustification.setValue(SoTextStyle::MIDDLE);}
  //xAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)xAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  //transf->enableNotify(flag);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateYAxis2D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;

  //SbBool flag = yAxisNode->enableNotify(aFlag);
  yAxisNode->verbose.setValue(verbose.getValue());
  yAxisNode->tickUp.setValue(TRUE);
  yAxisNode->width.setValue(hData);

 {SoTextStyle* style = yAxisNode->getLabelsStyle(); 
  style->orientationX.setValue(SbVec3f(0,1,0));
  style->orientationY.setValue(SbVec3f(1,0,0));
  style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  style->verticalJustification.setValue(SoTextStyle::MIDDLE);}

 {SoTextStyle* style = yAxisNode->getTitleStyle(); 
  style->rotated.setValue(TRUE);
  style->orientationX.setValue(SbVec3f(1, 0,0));
  style->orientationY.setValue(SbVec3f(0,-1,0));
  if(yAxisNode->titleToAxisJustification.getValue()==SoAxis::LEFT) {
    style->horizontalJustification.setValue(SoTextStyle::LEFT);
  } else if(yAxisNode->titleToAxisJustification.getValue()==SoAxis::CENTER) {
    style->horizontalJustification.setValue(SoTextStyle::CENTER);
  } else if(yAxisNode->titleToAxisJustification.getValue()==SoAxis::RIGHT) {
    style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  }  
  style->verticalJustification.setValue(SoTextStyle::BOTTOM);}

 {SoTextStyle* style = yAxisNode->getMagStyle(); 
  style->horizontalJustification.setValue(SoTextStyle::CENTER);
  style->verticalJustification.setValue(SoTextStyle::BOTTOM);}
  //yAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)yAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  SbMatrix roty;
  roty.setRotate(SbRotation(SbVec3f(0,1,0),FM_PI));
  transf->multLeft(roty);
  SbMatrix rotz;
  rotz.setRotate(SbRotation(SbVec3f(0,0,1),FM_PI_2));
  transf->multLeft(rotz); //Applied first.*/
  //transf->enableNotify(flag);
 }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateZAxis2D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //SbBool flag = zAxisNode->enableNotify(aFlag);
  zAxisNode->verbose.setValue(verbose.getValue());
  zAxisNode->width.setValue(0);
  //zAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)zAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  //transf->enableNotify(flag);
 }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateXAxis3D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  //SbBool flag = xAxisNode->enableNotify(aFlag);
  xAxisNode->verbose.setValue(verbose.getValue());
  xAxisNode->tickUp.setValue(FALSE);
  xAxisNode->width.setValue(wData);
 {SoTextStyle* style = xAxisNode->getLabelsStyle(); 
  style->fontName.setValue("Hershey"); //Enforce Hershey.
  style->orientationX.setValue(SbVec3f(1,0,0));
  style->orientationY.setValue(SbVec3f(0,1,0));
  style->horizontalJustification.setValue(SoTextStyle::CENTER);
  style->verticalJustification.setValue(SoTextStyle::TOP);}
 {SoTextStyle* style = xAxisNode->getTitleStyle(); 
  style->orientationX.setValue(SbVec3f(1,0,0));
  style->orientationY.setValue(SbVec3f(0,1,0));
  style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  style->verticalJustification.setValue(SoTextStyle::TOP);}
 //{SoTextStyle* style = xAxisNode->getMagStyle(); 
 // style->horizontalJustification.setValue(SoTextStyle::LEFT);
 // style->verticalJustification.setValue(SoTextStyle::MIDDLE);}
  //xAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)xAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  SbMatrix rotx;
  rotx.setRotate(SbRotation(SbVec3f(1,0,0),FM_PI_2));
  transf->multLeft(rotx);
  //transf->enableNotify(flag);
 }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateYAxis3D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{  
  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;

  //SbBool flag = yAxisNode->enableNotify(aFlag);
  yAxisNode->verbose.setValue(verbose.getValue());
  yAxisNode->tickUp.setValue(FALSE);
  yAxisNode->width.setValue(hData);
 {SoTextStyle* style = yAxisNode->getLabelsStyle(); 
  style->fontName.setValue("Hershey"); //Enforce Hershey.
  style->orientationX.setValue(SbVec3f(-1,0,0));
  style->orientationY.setValue(SbVec3f( 0,1,0));
  style->horizontalJustification.setValue(SoTextStyle::CENTER);
  style->verticalJustification.setValue(SoTextStyle::TOP);}
 {SoTextStyle* style = yAxisNode->getTitleStyle(); 
  style->orientationX.setValue(SbVec3f(-1,0,0));
  style->orientationY.setValue(SbVec3f( 0,1,0));
  style->horizontalJustification.setValue(SoTextStyle::LEFT);
  style->verticalJustification.setValue(SoTextStyle::TOP);}
 //{SoTextStyle* style = yAxisNode->getMagStyle(); 
 // style->horizontalJustification.setValue(SoTextStyle::CENTER);
 // style->verticalJustification.setValue(SoTextStyle::BOTTOM);}
  //yAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)yAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  SbMatrix roty;
  roty.setRotate(SbRotation(SbVec3f(0,1,0),FM_PI_2));
  transf->multLeft(roty);
  SbMatrix rotz;
  rotz.setRotate(SbRotation(SbVec3f(0,0,1),FM_PI_2));
  transf->multLeft(rotz);
  //transf->enableNotify(flag);
 }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateZAxis3D(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float ZSIZ = depth.getValue();
  float ZMGD = downMargin.getValue();
  float ZMGU = upMargin.getValue();
  float dData = ZSIZ-ZMGD-ZMGU;
  //SbBool flag = zAxisNode->enableNotify(aFlag);
  zAxisNode->verbose.setValue(verbose.getValue());
  zAxisNode->tickUp.setValue(FALSE);
  zAxisNode->width.setValue(dData);
 {SoTextStyle* style = zAxisNode->getLabelsStyle(); 
  style->fontName.setValue("Hershey"); //Enforce Hershey.
  style->orientationX.setValue(SbVec3f(0,1,0));
  style->orientationY.setValue(SbVec3f(1,0,0));
  style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  style->verticalJustification.setValue(SoTextStyle::MIDDLE);}
 {SoTextStyle* style = zAxisNode->getTitleStyle(); 
  style->orientationX.setValue(SbVec3f(0,1,0));
  style->orientationY.setValue(SbVec3f(1,0,0));
  style->horizontalJustification.setValue(SoTextStyle::RIGHT);
  style->verticalJustification.setValue(SoTextStyle::BOTTOM);}
 //{SoTextStyle* style = zAxisNode->getMagStyle(); 
 // style->horizontalJustification.setValue(SoTextStyle::CENTER);
 // style->verticalJustification.setValue(SoTextStyle::BOTTOM);}
  //zAxisNode->enableNotify(flag);

 {SoTransform* transf = (SoTransform*)zAxisTransform.getValue();
  //flag = transf->enableNotify(aFlag);
  SbMatrix matrix;
  matrix.makeIdentity();
  transf->setMatrix(matrix);
  SbMatrix trans;
  trans.setTranslate(SbVec3f(0,yAxisNode->width.getValue(),0));
  transf->multLeft(trans);
  SbMatrix rotz;
  rotz.setRotate(SbRotation(SbVec3f(0,0,1),-FM_PI_2));
  transf->multLeft(rotz);
  SbMatrix roty;
  roty.setRotate(SbRotation(SbVec3f(0,1,0),-FM_PI_2));
  transf->multLeft(roty); //Applied first.
  //transf->enableNotify(flag);
 }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateAxis (
 SoAxis* aAxis
,SbAxis& aAxisData
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
#ifdef DEBUG
  printf("debug : SoPlotter::updateAxis %ld : %g %g\n\n",
    aAxis,aAxisData.getMinimumValue(),aAxisData.getMaximumValue());
#endif

  //SbBool flag = aAxis->enableNotify(FALSE);

  aAxis->minimumValue.setValue(aAxisData.getMinimumValue());
  aAxis->maximumValue.setValue(aAxisData.getMaximumValue());
  aAxis->logScale.setValue(aAxisData.isLogScale());

  aAxis->updateChildren(); // So that the grid be correct.

  //aAxis->enableNotify(flag);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateLegend(
)
//////////////////////////////////////////////////////////////////////////////
// WARNING : legend field is an "output" field. 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //SbBool flag = legend.enableNotify(FALSE);//FIXME
  //SbBool flag2 = legendStyle.getValue()->enableNotify(FALSE);//FIXME
  legend.deleteValues(0);

  int index = 0;
  int binsi = 0;
  int pointsi = 0;
  int funci = 0;
  int number = fPlottableObjects.getLength();
  for(int count=0;count<number;count++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[count];
    if(!object) continue; 
    if(!object->isValid()) {
      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo
        ("SoPlotter::updateLegend","object not valid.");
      }
      continue; 
    }
    if(void* h1 = Cast(*object,SbPlottableBins1D)) {
      SbPlottableBins1D* bins = (SbPlottableBins1D*)h1;
      const char* leg = bins->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart(SbStringPrintF(32,"legendStyle[%d]",index),getBinsStyle(binsi));
      binsi++;
      index++;
    } else if(void* h2 = Cast(*object,SbPlottableBins2D)) {
      SbPlottableBins2D* bins = (SbPlottableBins2D*)h2;
      const char* leg = bins->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart(SbStringPrintF(32,"legendStyle[%d]",index),getBinsStyle(binsi));
      binsi++;
      index++;
    } else if(void* c2 = Cast(*object,SbPlottablePoints2D)) {
      SbPlottablePoints2D* points = (SbPlottablePoints2D*)c2;
      const char* leg = points->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart
        (SbStringPrintF(32,"legendStyle[%d]",index),getPointsStyle(pointsi));
      pointsi++;
      index++;
    } else if(void* c3 = Cast(*object,SbPlottablePoints3D)) {
      SbPlottablePoints3D* points = (SbPlottablePoints3D*)c3;
      const char* leg = points->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart
        (SbStringPrintF(32,"legendStyle[%d]",index),getPointsStyle(pointsi));
      pointsi++;
      index++;
    } else if(void* f = Cast(*object,SbPlottableFunction1D)) {
      SbPlottableFunction1D* function = (SbPlottableFunction1D*)f;
      const char* leg = function->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart
        (SbStringPrintF(32,"legendStyle[%d]",index),getFunctionStyle(funci));
      funci++;
      index++;
    } else if(void* f = Cast(*object,SbPlottableFunction2D)) {
      SbPlottableFunction2D* function = (SbPlottableFunction2D*)f;
      const char* leg = function->getLegend();
      if((leg==NULL)||(*leg=='\0')) continue;
      legend.set1Value(index,leg);
      setPart
        (SbStringPrintF(32,"legendStyle[%d]",index),getFunctionStyle(funci));
      funci++;
      index++;
    }
  }

  //legend.enableNotify(flag); //FIXME
  //legendStyle.getValue()->enableNotify(flag2); //FIXME

}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SbString SoPlotter::getTitle(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int count;
  SbPList binsList,pointsList,functionList;
  int objn = fPlottableObjects.getLength();
  for(count=0;count<objn;count++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[count];
    if(!object) continue; 
    if(!object->isValid()) {
      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo
        ("SoPlotter::getTitle","object not valid.");
      }
      continue; 
    }
    if(Cast(*object,SbPlottableBins1D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottableBins2D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottablePoints2D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottablePoints3D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottableFunction1D)) {
      functionList.append(object);
    } else if(Cast(*object,SbPlottableFunction2D)) {
      functionList.append(object);
    }
  }
  int binsNumber = binsList.getLength();
  int pointsNumber = pointsList.getLength();
  int functionNumber = functionList.getLength();

  if(!binsNumber && !pointsNumber && !functionNumber) return "";

  SbPlottableObject* major_bins = 
    (SbPlottableObject*)(binsNumber>0 ? binsList[0] : 0) ;
  SbPlottableObject* major_func =
    (SbPlottableObject*)(functionNumber>0 ? functionList[0] : 0);
  SbPlottableObject* major_points =
    (SbPlottableObject*)(pointsNumber>0 ? pointsList[0] : 0);

  if(major_bins) { 
    
    return major_bins->getLabel();
    
  } else  if(major_points) { 
    
    return major_points->getLabel();
    
  } else  if(major_func) { 
    
    return major_func->getLabel();

  } else {
  
    return "";

  }
}
//////////////////////////////////////////////////////////////////////////////
SbString SoPlotter::getInfos(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int count;
  SbPList binsList,pointsList,functionList;
  int objn = fPlottableObjects.getLength();
  for(count=0;count<objn;count++) {
    SbPlottableObject* object = (SbPlottableObject*)fPlottableObjects[count];
    if(!object) continue; 
    if(!object->isValid()) {
      if(verbose.getValue()==TRUE) {
        SoDebugError::postInfo
        ("SoPlotter::getInfos","object not valid.");
      }
      continue; 
    }
    if(Cast(*object,SbPlottableBins1D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottableBins2D)) {
      binsList.append(object);
    } else if(Cast(*object,SbPlottablePoints2D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottablePoints3D)) {
      pointsList.append(object);
    } else if(Cast(*object,SbPlottableFunction1D)) {
      functionList.append(object);
    } else if(Cast(*object,SbPlottableFunction2D)) {
      functionList.append(object);
    }
  }
  int binsNumber = binsList.getLength();
  int pointsNumber = pointsList.getLength();
  int functionNumber = functionList.getLength();

  if(!binsNumber && !pointsNumber && !functionNumber) return "";

  SbPlottableObject* major_bins = 
    (SbPlottableObject*)(binsNumber>0 ? binsList[0] : 0) ;
  SbPlottableObject* major_func =
    (SbPlottableObject*)(functionNumber>0 ? functionList[0] : 0);
  SbPlottableObject* major_points =
    (SbPlottableObject*)(pointsNumber>0 ? pointsList[0] : 0);

  SoStyle* soInfosStyle = (SoStyle*)infosStyle.getValue();
  const SbString& options = soInfosStyle->options.getValue();

  if(major_bins) { 
    
    return major_bins->getInfos(options.getString());

  } else  if(major_points) { 
    
    return major_points->getInfos(options.getString());

  } else  if(major_func) { 
    
    return major_func->getInfos(options.getString());

  } else {

    return SbString();

  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SoDrawStyle* SoPlotter::getLineStyle(
 const SbStyle& aStyle
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStyleCache->getLineStyle 
           (aStyle.getLinePattern(),(float)aStyle.getLineWidth());
}
//////////////////////////////////////////////////////////////////////////////
SoDrawStyle* SoPlotter::getLineStyle(
 SoStyle* aStyle
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return fStyleCache->getLineStyle
          (aStyle->linePattern.getValue(),(float)aStyle->lineWidth.getValue());
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
SbVec3f* SoPlotter::clipPoints2D(
 const std::vector<SbVec3f>& aPoints
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZ
,int& aNumber
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aNumber = 0;
  int number = aPoints.size();
  if(number<=0) return 0;

  SbVec3f* points = new SbVec3f[number];
  int ipoint = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  for(int index=0;index<number;index++) {
    float xx = aPoints[index][0];
    float yy = aPoints[index][1];

    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);

    if((xx>=0)&&(xx<=1)&&(yy>=0)&&(yy<=1)) {
      LIST_SET(points,ipoint,xx,yy,aZ);
      ipoint++;
    }
  }

  if(ipoint<=0) {
    aNumber = 0;
    delete [] points;
    return 0;
  } else {
    aNumber = ipoint;
    return points;
  }
}
//////////////////////////////////////////////////////////////////////////////
SbVec3f* SoPlotter::clipPoints3D(
 const std::vector<SbVec3f>& aPoints
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,int& aNumber
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aNumber = 0;
  int number = aPoints.size();
  if(number<=0) return 0;

  SbVec3f* points = new SbVec3f[number];
  int ipoint = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  for(int index=0;index<number;index++) {
    float xx = aPoints[index][0];
    float yy = aPoints[index][1];
    float zz = aPoints[index][2];

    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    zz = VerifyLog(zz,zmin,dz,zlog);

    if( (xx>=0)&&(xx<=1) && 
        (yy>=0)&&(yy<=1) &&
        (zz>=0)&&(zz<=1)
        ) {
      LIST_SET(points,ipoint,xx,yy,zz);
      ipoint++;
    }
  }

  if(ipoint<=0) {
    aNumber = 0;
    delete [] points;
    return 0;
  } else {
    aNumber = ipoint;
    return points;
  }
}
//////////////////////////////////////////////////////////////////////////////
SbVec3f* SoPlotter::clipPolyLine2D(
 const std::vector<SbVec3f>& aPoints
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,int& aNumber
)
//////////////////////////////////////////////////////////////////////////////
//  Clip line in aBoxX, aBoxY.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  int number = aPoints.size();
  SbVec3f* points = new SbVec3f[3 * number + 10];
  int ipoint = 0;

  float xprev = 0;
  float yprev = 0;
  for(int index=0;index<number;index++) {
    float xx = aPoints[index][0];
    float yy = aPoints[index][1];
    float zz = aPoints[index][2];
    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    if((xx>=0)&&(xx<=1) ) {
      if(yy>1) {
        if(index==0) {
          LIST_SET(points,ipoint,xx,1,zz);ipoint++;
        } else {
          if(yprev>1) {
            LIST_SET(points,ipoint,xx,1,zz);ipoint++;
          } else if(yprev<0) {
            float a = (yy - yprev)/(xx - xprev);
            float b = yy - a * xx;
            LIST_SET(points,ipoint,-b/a,0,zz);ipoint++;
            LIST_SET(points,ipoint,(1 - b)/a,1,zz);ipoint++;
            LIST_SET(points,ipoint,xx,1,zz);ipoint++;
          } else {
            float a = (yy - yprev)/(xx - xprev);
            float b = yy - a * xx;
            LIST_SET(points,ipoint,(1 - b)/a,1,zz);ipoint++;
            LIST_SET(points,ipoint,xx,1,zz);ipoint++;
          }
        }
      } else if (yy < 0) {
        if(index==0) {
          LIST_SET(points,ipoint,xx,0,zz);ipoint++;
        } else {
          if(yprev<0) {
            LIST_SET(points,ipoint,xx,0,zz);ipoint++;
          } else if(yprev>1) {
            float a = (yy - yprev)/(xx - xprev);
            float b = yy - a * xx;
            LIST_SET(points,ipoint,(1 - b)/a,1,zz);ipoint++;
            LIST_SET(points,ipoint,-b/a,0,zz);ipoint++;
            LIST_SET(points,ipoint,xx,0,zz);ipoint++;
          } else {
            float a = (yy - yprev)/(xx - xprev);
            float b = yy - a * xx;
            LIST_SET(points,ipoint,-b/a,0,zz);ipoint++;
            LIST_SET(points,ipoint,xx,0,zz);ipoint++;
           }
        }
      } else {
        if(index==0) {
          LIST_SET(points,ipoint,xx,yy,zz);ipoint++;
        } else if( (yprev>1) || (yprev<0) ) {
          // interpolate :
          float a = (yy - yprev)/(xx - xprev);
          float b = yy - a * xx;
          if(yprev>1) {
            LIST_SET(points,ipoint,(1 - b)/a,1,zz);ipoint++;
          } else {
            LIST_SET(points,ipoint,-b/a,0,zz);ipoint++;
          }
          LIST_SET(points,ipoint,xx,yy,zz);ipoint++;
        } else {
          LIST_SET(points,ipoint,xx,yy,zz);ipoint++;
        }
      }
    }
    xprev = xx;
    yprev = yy;
  }

  if(ipoint<=0) {
    aNumber = 0;
    delete [] points;
    return 0;
  } else {
    aNumber = ipoint;
    return points;
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateTopAxis(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;

  SoSeparator* sep = (SoSeparator*)topAxisSeparator.getValue();
  //SbBool flag = sep->enableNotify(FALSE);

  SoTransform* transf = new SoTransform;
  transf->translation.setValue(SbVec3f(0,hData,0));
  sep->addChild(transf);

  SoAxis* axis = (SoAxis*)xAxisNode->copy();
  SoTextStyle* style = axis->getLabelsStyle(); 
  style->visible.setValue(FALSE);
  axis->tickUp.setValue(FALSE);
  axis->title.setValue("");
  sep->addChild(axis);

  //sep->enableNotify(flag);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateRightAxis(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  SoSeparator* sep = (SoSeparator*)rightAxisSeparator.getValue();
  //SbBool flag = sep->enableNotify(FALSE);

  SoTransform* transf = new SoTransform;
  SbMatrix trans;
  trans.setTranslate(SbVec3f(wData,0,0));
  transf->multLeft(trans);
  SbMatrix roty;
  roty.setRotate(SbRotation(SbVec3f(0,1,0),FM_PI));
  transf->multLeft(roty);
  SbMatrix rotz;
  rotz.setRotate(SbRotation(SbVec3f(0,0,1),FM_PI_2));
  transf->multLeft(rotz); //Applied first.
  sep->addChild(transf);

  SoAxis* axis = (SoAxis*)yAxisNode->copy();
  SoTextStyle* style = axis->getLabelsStyle(); 
  style->visible.setValue(FALSE);
  axis->tickUp.setValue(FALSE);
  axis->title.setValue("");
  sep->addChild(axis);

  //sep->enableNotify(flag);
}
//////////////////////////////////////////////////////////////////////////////
static void buildWall(
 const SbVec3f& aVertex
,const SbVec3f& aX
,const SbVec3f& aY
,float aSx  
,float aSy
,SoSeparator* aWall
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aSx<=0) return;
  if(aSy<=0) return;

  SoGrid* grid = new SoGrid;

  // Fields :
 
  grid->position.setValue (aVertex);
  grid->X.setValue (aX);
  grid->Y.setValue (aY);
  grid->rows.setValue (1);
  grid->columns.setValue (1);
  grid->width.setValue (aSx);
  grid->height.setValue (aSy);
  
  aWall->addChild(grid);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateWall2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::updateWall2D\n");
 
  SoStyle* style = (SoStyle*)wallStyle.getValue();

  SoSeparator* sep = (SoSeparator*)wallSeparator.getValue();

  sep->addChild(fStyleCache->getMaterial
               (style->color.getValue(),
                style->transparency.getValue()));
      
  sep->addChild(fStyleCache->getFilled());
  
  buildWall(SbVec3f(0,0,-depth.getValue()/2),
            SbVec3f(1,0,0),SbVec3f(0,1,0),
            xAxisNode->width.getValue(),
            yAxisNode->width.getValue(),
            sep);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateWall3D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::updateWall3D\n");

  SoStyle* style = (SoStyle*)wallStyle.getValue();

  SoSeparator* sep = (SoSeparator*)wallSeparator.getValue();

  sep->addChild(fStyleCache->getMaterial
               (style->color.getValue(),
                style->transparency.getValue()));
      
  sep->addChild(fStyleCache->getFilled());
    
  // plan x-y at z = 0.
 {float ZSIZ = depth.getValue();
  float ZMGD = downMargin.getValue();
  float ZMGU = upMargin.getValue();
  float dData = ZSIZ-ZMGD-ZMGU;
  float woffset = dData * 0.01F; //FIXME : PLANE_OFFSET
  //float woffset = 0;
  buildWall(SbVec3f(0,0,-woffset),
            SbVec3f(1,0,0),
            SbVec3f(0,1,0),
            xAxisNode->width.getValue()+woffset,
            yAxisNode->width.getValue()+woffset,
            sep);}

  // plan z-y at x = 1.
 {float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;
  float woffset = wData * 0.01F;
  buildWall(SbVec3f(xAxisNode->width.getValue()+woffset,0,-woffset),
            SbVec3f(0,0,1),
            SbVec3f(0,1,0),
            zAxisNode->width.getValue()+woffset,
            yAxisNode->width.getValue()+woffset,
            sep);}

  // plan x-z at y = 1.
 {float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;
  float woffset = hData * 0.01F;
  buildWall(SbVec3f(0,yAxisNode->width.getValue()+woffset,-woffset),
            SbVec3f(1,0,0),
            SbVec3f(0,0,1),
            xAxisNode->width.getValue()+woffset,
            zAxisNode->width.getValue()+woffset,
            sep);}
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateGrid2D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int pos = 0;
  int number =
    xAxisNode->tickNumber.getValue() + yAxisNode->tickNumber.getValue();
  if(number<=0) return;

  SoStyle* style = (SoStyle*)gridStyle.getValue();

  SbBool draw_vertical = TRUE;
  SbBool draw_horizontal = TRUE;
  if(style->options.getValue()=="vertical") draw_horizontal = FALSE;
  if(style->options.getValue()=="horizontal") draw_vertical = FALSE;

  SoSeparator* sep = (SoSeparator*)gridSeparator.getValue();

  sep->addChild(fStyleCache->getMaterial
               (style->color.getValue(),
                style->transparency.getValue()));

  sep->addChild(getLineStyle(style));

  float gridOffset = depth.getValue()/2; //In from of data.
  SbVec3f* points = new SbVec3f[2 * number];
  if(draw_vertical==TRUE) {
    int number = xAxisNode->tickNumber.getValue();
    for(int count=0;count<number;count++) { 
      float xx = xAxisNode->coords[count];
      LIST_SET(points,pos,xx, 0,gridOffset);pos++;
      LIST_SET(points,pos,xx, yAxisNode->width.getValue(),gridOffset);pos++;
    }
  }
  if(draw_horizontal==TRUE) {
    int number = yAxisNode->tickNumber.getValue();
    for(int count=0;count<number;count++) {
      float yy = yAxisNode->coords[count];
      LIST_SET(points,pos,0,yy,gridOffset);pos++;
      LIST_SET(points,pos,xAxisNode->width.getValue(),yy,gridOffset);pos++;
    }
  }

  if(pos>0) {
    SoCoordinate3* coordinate3 = new SoCoordinate3;
    int32_t pointn = pos;
    coordinate3->point.setValues(0,pointn,points);
    sep->addChild(coordinate3);

    SoLineSet* lineSet = new SoLineSet;
    int segmentn = pointn/2;
    int32_t* vertices = new int32_t[segmentn];
    for (int count=0;count<segmentn;count++) vertices[count] = 2;
    lineSet->numVertices.setValues(0,segmentn,vertices);
    delete [] vertices;
    sep->addChild(lineSet);
  }
  delete [] points;

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateGrid3D(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int ntick = zAxisNode->tickNumber.getValue();
  if(ntick<=0) return;

  SoStyle* style = (SoStyle*)gridStyle.getValue();

  SoSeparator* sep = (SoSeparator*)gridSeparator.getValue();

  sep->addChild(fStyleCache->getMaterial
               (style->color.getValue(),
                style->transparency.getValue()));

  sep->addChild(getLineStyle(style));

  SbVec3f* points = new SbVec3f[4 * ntick];
  int pos = 0;
  for(int count=0;count<ntick;count++) {
    float xe = xAxisNode->width.getValue();
    float ye = yAxisNode->width.getValue();
    float zz = zAxisNode->coords[count];
    LIST_SET(points,pos,0 ,ye,zz);pos++;
    LIST_SET(points,pos,xe,ye,zz);pos++;
    LIST_SET(points,pos,xe,ye,zz);pos++;
    LIST_SET(points,pos,xe,0 ,zz);pos++;
  }
  if(pos>0) {
    SoCoordinate3* coordinate3 = new SoCoordinate3;
    int32_t pointn = pos;
    coordinate3->point.setValues(0,pointn,points);
    sep->addChild(coordinate3);

    SoLineSet* lineSet = new SoLineSet;
    int segmentn = pointn/2;
    int32_t* vertices = new int32_t[segmentn];
    for (int count=0;count<segmentn;count++) vertices[count] = 2;
    lineSet->numVertices.setValues(0,segmentn,vertices);
    delete [] vertices;
    sep->addChild(lineSet);
  }

  delete [] points;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateBins1D_XY(
 const SbPlottableBins1D& aBins
,int aIndex
,const SbPList& aBins1DList
,const SbPList& aBinsListSwMnMx
,SbBool aSuperpose
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::updateBins1D_XY : %d\n",aIndex);

  int xnbin = aBins.getAxisNumberOfBins();
  std::vector<SbRepBin1D> bins(xnbin);
  for(int ibin=0;ibin<xnbin;ibin++) { 
    float xx = aBins.getBinLowerEdge(ibin);
    float xe = aBins.getBinUpperEdge(ibin);
    float vmin,vmax;
    if(aSuperpose==TRUE) {
      getHeight(aIndex,aBins1DList,aBinsListSwMnMx,ibin,vmin,vmax);
    } else {
      vmin = 0;
      vmax = aBins.getBinSumOfWeights(ibin);
    }
    bins[ibin] = SbRepBin1D(xx,xe,vmin,vmax);
  }

  float zHatchOffset = 0.01F;
  float zErrorOffset = 2 * zHatchOffset;

  SoStyle* graphicStyle = getBinsStyle(aIndex);
  if(graphicStyle->visible.getValue()==TRUE) {

    const SbString& modeling = graphicStyle->modeling.getValue();
    SbPaintingPolicy painting = 
      (SbPaintingPolicy)graphicStyle->painting.getValue();

    SbBool oneNode = 
      (xnbin > graphicStyle->multiNodeLimit.getValue() ? TRUE : FALSE);

    SbBool barChart = FALSE;

    char s[128];
    sprintf(s,"Bin1D/0x%lx",(unsigned long)&aBins);

    SoSeparator* binsNode = (SoSeparator*)binsSeparator.getValue();

    if(modeling==SbModeling_top_lines){
      if(oneNode) {
        repBins1D_xy_top_lines_one(binsNode,graphicStyle,
                                bins,aBoxX,aBoxY,aZZ);
      } else {
        repBins1D_xy_top_lines(binsNode,graphicStyle,
                            painting,colorMapping.getValue(),
                            bins,aBoxX,aBoxY,aZZ,SbString(s));
      }
    } else if(modeling==SbModeling_points) {
      if(oneNode) {
        repBins1D_xy_points_one(binsNode,graphicStyle,
                             bins,aBoxX,aBoxY,aZZ);
      } else {
        repBins1D_xy_points(binsNode,graphicStyle,
                         painting,colorMapping.getValue(),
                         bins,aBoxX,aBoxY,aZZ,SbString(s));
      }
    } else if(modeling==SbModeling_boxes) {
      if(oneNode) {
        repBins1D_xy_boxes_one(binsNode,graphicStyle,
                            bins,aBoxX,aBoxY,aZZ);
      } else {
        repBins1D_xy_boxes(binsNode,graphicStyle,
                        painting,colorMapping.getValue(),
                        bins,aBoxX,aBoxY,aZZ,SbString(s));
      }
    } else if(modeling==SbModeling_wire_boxes) {
      if(oneNode) {
        repBins1D_xy_wire_boxes_one(binsNode,graphicStyle,barChart,
                                 bins,aBoxX,aBoxY,aZZ);
      } else {
        repBins1D_xy_wire_boxes(binsNode,graphicStyle,barChart,
                             painting,colorMapping.getValue(),
                             bins,aBoxX,aBoxY,aZZ,SbString(s));
      }
    } else if(modeling==SbModeling_bar_chart) {
      barChart = TRUE;
      if(oneNode) {
        repBins1D_xy_wire_boxes_one(binsNode,graphicStyle,barChart,
                                 bins,aBoxX,aBoxY,aZZ);
      } else {
        repBins1D_xy_wire_boxes(binsNode,graphicStyle,barChart,
                             painting,colorMapping.getValue(),
                             bins,aBoxX,aBoxY,aZZ,SbString(s));
      }
    } else if(modeling==SbModeling_lines){
      repBins1D_xy_lines_one(binsNode,graphicStyle,
                          bins,aBoxX,aBoxY,aZZ);
    } else if(modeling==SbModeling_curve){
      repBins1D_xy_curve_one(binsNode,graphicStyle,
                          bins,aBoxX,aBoxY,aZZ);
    }
  
    SbHatchingPolicy hatching = 
      (SbHatchingPolicy)graphicStyle->hatching.getValue();
    if(hatching!=SbHatchingNone) {
      // WARNING : must come AFTER repBins1D_xy.
      SoStyle* rightHatchGC = getRightHatchStyle(aIndex);
      SoStyle* leftHatchGC = getLeftHatchStyle(aIndex);
      if((hatching==SbHatchingRight)||((hatching==SbHatchingLeftAndRight))) {
        if(oneNode) {
          repHatch1D_xy_one(binsNode,rightHatchGC,barChart,
                         bins,aBoxX,aBoxY,aZZ+zHatchOffset);
        } else {
          repHatch1D_xy(binsNode,rightHatchGC,barChart,
                     bins,aBoxX,aBoxY,aZZ+zHatchOffset);
        }
      }
      if((hatching==SbHatchingLeft)||((hatching==SbHatchingLeftAndRight))) {
        if(oneNode) {
          repHatch1D_xy_one(binsNode,leftHatchGC,barChart,
                        bins,aBoxX,aBoxY,aZZ+zHatchOffset);
        } else {
          repHatch1D_xy(binsNode,leftHatchGC,barChart,
                     bins,aBoxX,aBoxY,aZZ+zHatchOffset);
        }
      }
    }

  }

 {// Errors :
  SoStyle* graphicStyle = getErrorsStyle(aIndex);
  if(graphicStyle->visible.getValue()==TRUE) { 
    std::vector<float> bars(xnbin);
    for(int ibin=0;ibin<xnbin;ibin++) bars[ibin] = aBins.getBinBar(ibin);

    SoSeparator* errorsNode = (SoSeparator*)errorsSeparator.getValue();
    SbMarkerStyle markerStyle = 
    (SbMarkerStyle)graphicStyle->markerStyle.getValue();
    if(markerStyle==SbMarkerPlus)
      repErrorsPlus_xy(errorsNode,graphicStyle,
                    bins,aBoxX,aBoxY,bars,aZZ+zErrorOffset);
    else 
      repErrorsI_xy(errorsNode,graphicStyle,
                 bins,aBoxX,aBoxY,bars,aZZ+zErrorOffset);
 }}
}

typedef struct {
  const SbPlottableFunction2D* fFunc2D;
  const SbPlottableBins2D* fBins2D;
  double fLimits[4];
  double fLimitsIn[4];
  SbBool fProblem;
} SbFunc;

static double bins2D_to_contour(double aX,double aY,void* aData) {
  SbFunc* func =(SbFunc*)aData;
  if( (aX<func->fLimitsIn[0])||
      (aX>func->fLimitsIn[1])||
      (aY<func->fLimitsIn[2])||
      (aY>func->fLimitsIn[3])
    ) return -FLT_MAX;
  return bins2DToFunc(*(func->fBins2D),(float)aX,(float)aY);
}
static double log_bins2D_to_contour(double aX,double aY,void* aData) {
  SbFunc* func =(SbFunc*)aData;
  if( (aX<func->fLimitsIn[0])||
      (aX>func->fLimitsIn[1])||
      (aY<func->fLimitsIn[2])||
      (aY>func->fLimitsIn[3])
    ) return -FLT_MAX;
  return TakeLog(bins2DToFunc(*(func->fBins2D),(float)aX,(float)aY));
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateBins2D_XY(
 const SbPlottableBins2D& aBins
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aZZ
,SbBool& drawColorMapping
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* binsNode = (SoSeparator*)binsSeparator.getValue();

  SoStyle* graphicStyle = getBinsStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  const SbString& modeling = graphicStyle->modeling.getValue();
  SbPaintingPolicy painting = 
    (SbPaintingPolicy)graphicStyle->painting.getValue();

  int xnbin = aBins.getAxisNumberOfBinsX();
  int ynbin = aBins.getAxisNumberOfBinsY();

  float bmin;
  float bmax;
  aBins.getBinsSumOfWeightsRange(bmin,bmax);

  // one node decision :
  int ibin,jbin;
  int count = 0;
  for(jbin=ynbin-1;jbin>=0;jbin--) {
    for(ibin=xnbin-1;ibin>=0;ibin--) { 
      if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;
      count++;
    }
  }
  SbBool oneNode = 
    (count > graphicStyle->multiNodeLimit.getValue()?TRUE:FALSE);

  if( (modeling==SbModeling_curve)  || (modeling==SbModeling_filled_curve) ){

    float zmin = aBoxZ.fPos;
    float dz = aBoxZ.fWidth;
    SbBool zlog = aBoxZ.fLog;

    float xmn = f_xDataAxis.getMinimumValue();
    float xmx = f_xDataAxis.getMaximumValue();
    float ymn = f_yDataAxis.getMinimumValue();
    float ymx = f_yDataAxis.getMaximumValue();

    CListContour listContour;
    //int nFir = 32;
    int nFir = 128;
    listContour.SetFirstGrid(nFir,nFir); //Default : 32,32 
    //int nSec = 256;
    int nSec = 512;
    listContour.SetSecondaryGrid(nSec,nSec); //Default : 256,256.

    double limits[4];
    // User limits :
    limits[0] = xmn;
    limits[1] = xmx;    
    limits[2] = ymn;    
    limits[3] = ymx;    

    SbFunc sbFunc;
    sbFunc.fFunc2D = 0;
    sbFunc.fProblem = FALSE;
    sbFunc.fBins2D = &aBins;
    sbFunc.fLimitsIn[0] = limits[0];
    sbFunc.fLimitsIn[1] = limits[1];
    sbFunc.fLimitsIn[2] = limits[2];
    sbFunc.fLimitsIn[3] = limits[3];

    // Extend the grid to have some borders in order to close contours :
    int n = nSec - 2 * 10;
    double dx = (limits[1]-limits[0]) /n;
    double dy = (limits[3]-limits[2]) /n;
    limits[0] = limits[0] - 10 * dx;
    limits[1] = limits[1] + 10 * dx;
    limits[2] = limits[2] - 10 * dy;
    limits[3] = limits[3] + 10 * dy;

    sbFunc.fLimits[0] = limits[0];
    sbFunc.fLimits[1] = limits[1];
    sbFunc.fLimits[2] = limits[2];
    sbFunc.fLimits[3] = limits[3];

    listContour.SetLimits(limits);

    int zn = numberOfLevels.getValue();
    std::vector<double> zs(zn+1);
    float zmax = zmin + dz;
    double zd = (zmax-zmin)/zn;
    for(int zi=0;zi<=zn;zi++) zs[zi] = zmin + zi * zd;
    listContour.SetPlanes(zs);

    if(zlog==TRUE)
      listContour.SetFieldFcn(log_bins2D_to_contour,(void*)&sbFunc);
    else
      listContour.SetFieldFcn(bins2D_to_contour,(void*)&sbFunc);

    listContour.CContour::Generate();
    if(!listContour.CompactStrips()) {
      SoDebugError::postInfo("SoPlotter::updateBins2D_XY",
       " CListContour::CompactStrips() : failure.");
      return;
    }
  
    char s[128];
    //sprintf(s,"Function/0x%lx",(unsigned long)aFunction.nativeObject());
    sprintf(s,"%s","");
    if(modeling==SbModeling_filled_curve) {
      repContour_xy_filled(binsNode,
                           graphicStyle,painting,colorMapping.getValue(),
                           listContour,aBoxX,aBoxY,aBoxZ,aZZ,SbString(s));
    } else {
      repContour_xy(binsNode,graphicStyle,painting,colorMapping.getValue(),
                    listContour,aBoxX,aBoxY,aBoxZ,aZZ,SbString(s));
    }

    //if(sbFunc.fProblem==TRUE) {
    //  SoDebugError::postInfo("SoPlotter::updateBins2D_XY",
    //    " problem when getting some function value.");
    //}

    return;
  }

  int number = 0;
 {for(jbin=ynbin-1;jbin>=0;jbin--) {
    for(ibin=xnbin-1;ibin>=0;ibin--) { 
      if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;
      number++;
  }}}

  int bini = 0;
  std::vector<SbRepBin2D> bins(number);
  for(jbin=ynbin-1;jbin>=0;jbin--) {
    for(ibin=xnbin-1;ibin>=0;ibin--) { 
      if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;

      float val = aBins.getBinSumOfWeights(ibin,jbin);

      float xx = aBins.getBinLowerEdgeX(ibin);
      float xe = aBins.getBinUpperEdgeX(ibin);
      float yy = aBins.getBinLowerEdgeY(jbin);
      float ye = aBins.getBinUpperEdgeY(jbin);
          
      bins[bini] = SbRepBin2D(xx,xe,yy,ye,val,ibin,jbin);
      bini++;
    }
  }
          
  //FIXME f_colorMapping_Update = TRUE;

  char s[128];
  sprintf(s,"Bin2D/0x%lx",(unsigned long)&aBins);
  if(modeling==SbModeling_solid) {
    drawColorMapping = TRUE;
    repBins2D_xy_solid_one(binsNode,
                           graphicStyle,painting,colorMapping.getValue(),
                           bins,aBoxX,aBoxY,aBoxZ);
  } else if(modeling==SbModeling_points) {
    repBins2D_xy_random_one(binsNode,
                            graphicStyle,
                            bins,aBoxX,aBoxY,bmin,bmax);
  } else if(modeling==SbModeling_boxes) {
    if(oneNode) {
      repBins2D_xy_box_one(binsNode,
                           graphicStyle,
                           bins,aBoxX,aBoxY,bmin,bmax);
    } else {
      repBins2D_xy_box(binsNode,
                       graphicStyle,
                       bins,aBoxX,aBoxY,bmin,bmax,
                       SbString(s));
    }
  } else if(modeling==SbModeling_wire_boxes) {
    if(oneNode) {
      repBins2D_xy_wire_box_one(binsNode,
                                graphicStyle,
                                bins,aBoxX,aBoxY,bmin,bmax);
    } else {
      repBins2D_xy_wire_box(binsNode,
                            graphicStyle,
                            bins,aBoxX,aBoxY,bmin,bmax,
                            SbString(s));
    }
  } else if(modeling==SbModeling_texts) {
    if(oneNode) {
      repBins2D_xy_text_one(binsNode,
                            graphicStyle,
                            bins,aBoxX,aBoxY);
    } else {
      repBins2D_xy_text(binsNode,
                        graphicStyle,
                        bins,aBoxX,aBoxY,
                        SbString(s));
     }
  }
}

static double function_to_contour(double aX,double aY,void* aData) {
  SbFunc* func =(SbFunc*)aData;
  if( (aX<func->fLimitsIn[0])||
      (aX>func->fLimitsIn[1])||
      (aY<func->fLimitsIn[2])||
      (aY>func->fLimitsIn[3])
    ) return -FLT_MAX;
  float value;
  if(!func->fFunc2D->value((float)aX,(float)aY,value)) func->fProblem = TRUE;
  return double(value);
}
static double log_function_to_contour(double aX,double aY,void* aData) {
  SbFunc* func =(SbFunc*)aData;
  if( (aX<func->fLimitsIn[0])||
      (aX>func->fLimitsIn[1])||
      (aY<func->fLimitsIn[2])||
      (aY>func->fLimitsIn[3])
    ) return -FLT_MAX;
  float value;
  if(!func->fFunc2D->value((float)aX,(float)aY,value)) func->fProblem = TRUE;
  return TakeLog(double(value));
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateFunction_XY(
 const SbPlottableFunction1D& aFunction
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* functionNode = (SoSeparator*)functionSeparator.getValue();

  SoStyle* graphicStyle = getFunctionStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  const SbString& modeling = graphicStyle->modeling.getValue();
  SbPaintingPolicy painting = 
    (SbPaintingPolicy)graphicStyle->painting.getValue();

  float xmn = aFunction.getMinimumX();
  float xmx = aFunction.getMaximumX();
  if(xmx<=xmn) {
    xmn = f_xDataAxis.getMinimumValue();
    xmx = f_xDataAxis.getMaximumValue();
  }
  int nstp = aFunction.getNumberOfStepsX();
  nstp = nstp <=0 ? 100 : nstp;
  
  float df = (xmx - xmn)/nstp;      
    
  SbBool problem = FALSE;
  std::vector<SbVec3f> points(nstp+1);
  for(int ibin=0;ibin<=nstp;ibin++) { 
    float xx = xmn + ibin * df;
    float val;
    if(!aFunction.value(xx,val)) problem = TRUE;
    points[ibin] = SbVec3f(xx,val,aZZ);
  }
  if(problem==TRUE) {
    SoDebugError::postInfo("SoPlotter::updateFunction_XY",
      " problem when getting some function value.");
  }
  
  char s[128];
  sprintf(s,"Function/0x%lx",(unsigned long)aFunction.nativeObject());

  if(modeling==SbModeling_points)  {

    int ptn;
    SbVec3f* pts = clipPoints2D(points,aBoxX,aBoxY,0,ptn);
    if(ptn) {
      SoSeparator* separator = new SoSeparator();
      separator->setName("sceneGraph");
      functionNode->addChild(separator);
      separator->addChild(fStyleCache->getPoints());
      separator->addChild(fStyleCache->getMaterial
        (graphicStyle->color.getValue(),
         graphicStyle->transparency.getValue()));
      SoTools::addPointsToNode(separator,ptn,pts,
        (SbMarkerStyle)graphicStyle->markerStyle.getValue(),
        graphicStyle->markerSize.getValue(),SbString(s));
      delete [] pts;
    }  

  } else { //Default.

    int ptn;
    SbVec3f* pts = clipPolyLine2D(points,aBoxX,aBoxY,ptn);
    if(ptn) {
      SoSeparator* separator = new SoSeparator();
      separator->setName("sceneGraph");
      functionNode->addChild(separator);
      separator->addChild(getLineStyle(graphicStyle));
      separator->addChild(fStyleCache->getNormalBindingOverall());
      separator->addChild(fStyleCache->getNormalZ());
      separator->addChild(fStyleCache->getMaterial
        (graphicStyle->color.getValue(),
         graphicStyle->transparency.getValue()));
      SoTools::addLinesToNode(separator,ptn,pts,SbString(s));
      delete [] pts;
    }  

  }

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateFunction_XY(
 const SbPlottableFunction2D& aFunction
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* functionNode = (SoSeparator*)functionSeparator.getValue();

  SoStyle* graphicStyle = getFunctionStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  const SbString& modeling = graphicStyle->modeling.getValue();
  SbPaintingPolicy painting = 
    (SbPaintingPolicy)graphicStyle->painting.getValue();

  if( (modeling==SbModeling_curve) ||
      (modeling==SbModeling_filled_curve) ) {

    float zmin = aBoxZ.fPos;
    float dz = aBoxZ.fWidth;
    SbBool zlog = aBoxZ.fLog;

    float xmn = aFunction.getMinimumX();
    float xmx = aFunction.getMaximumX();
    if(xmx<=xmn) {
      xmn = f_xDataAxis.getMinimumValue();
      xmx = f_xDataAxis.getMaximumValue();
    }
    float ymn = aFunction.getMinimumY();
    float ymx = aFunction.getMaximumY();
    if(ymx<=ymn) {
      ymn = f_yDataAxis.getMinimumValue();
      ymx = f_yDataAxis.getMaximumValue();
    }

    CListContour listContour;
    //int nFir = 32;
    int nFir = 128;
    listContour.SetFirstGrid(nFir,nFir); //Default : 32,32 
    //int nSec = 256;
    int nSec = 512;
    listContour.SetSecondaryGrid(nSec,nSec); //Default : 256,256.

    double limits[4];
    // User limits :
    limits[0] = xmn;
    limits[1] = xmx;    
    limits[2] = ymn;    
    limits[3] = ymx;    

    SbFunc sbFunc;
    sbFunc.fFunc2D = &aFunction;
    sbFunc.fProblem = FALSE;
    sbFunc.fBins2D = 0;
    sbFunc.fLimitsIn[0] = limits[0];
    sbFunc.fLimitsIn[1] = limits[1];
    sbFunc.fLimitsIn[2] = limits[2];
    sbFunc.fLimitsIn[3] = limits[3];

    // Extend the grid to have some borders in order to close contours :
    int n = nSec - 2 * 10;
    double dx = (limits[1]-limits[0]) /n;
    double dy = (limits[3]-limits[2]) /n;
    limits[0] = limits[0] - 10 * dx;
    limits[1] = limits[1] + 10 * dx;
    limits[2] = limits[2] - 10 * dy;
    limits[3] = limits[3] + 10 * dy;

    sbFunc.fLimits[0] = limits[0];
    sbFunc.fLimits[1] = limits[1];
    sbFunc.fLimits[2] = limits[2];
    sbFunc.fLimits[3] = limits[3];

    listContour.SetLimits(limits);

    int zn = numberOfLevels.getValue();
    std::vector<double> zs(zn+1);
    float zmax = zmin + dz;
    double zd = (zmax-zmin)/zn;
    for(int zi=0;zi<=zn;zi++) zs[zi] = zmin + zi * zd;
    listContour.SetPlanes(zs);

    if(zlog==TRUE) {
      listContour.SetFieldFcn(log_function_to_contour,(void*)&sbFunc);
    } else {
      listContour.SetFieldFcn(function_to_contour,(void*)&sbFunc);
    }

    listContour.CContour::Generate();
    if(!listContour.CompactStrips()) {
      SoDebugError::postInfo
        ("SoPlotter::updateFunction_XY",
         " CListContour::CompactStrips() : failure.");
    } else {

      char s[128];
      sprintf(s,"Function/0x%lx",(unsigned long)aFunction.nativeObject());
      if(modeling==SbModeling_filled_curve) {
        repContour_xy_filled(functionNode,
                             graphicStyle,painting,colorMapping.getValue(),
                             listContour,aBoxX,aBoxY,aBoxZ,aZZ,SbString(s));
      } else {
        repContour_xy(functionNode,graphicStyle,painting,
                      colorMapping.getValue(),
                      listContour,aBoxX,aBoxY,aBoxZ,aZZ,SbString(s));
      }
    }

    if(sbFunc.fProblem==TRUE) {
      SoDebugError::postInfo("SoPlotter::updateFunction_XY",
        " problem when getting some function value.");
    }

  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updatePoints2D_XY(
 const SbPlottablePoints2D& aPoints
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* pointsNode = (SoSeparator*)pointsSeparator.getValue();

  SoStyle* graphicStyle = getPointsStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  int npoint = aPoints.getNumberOfPoints ();
  SbBool oneNode = 
    (npoint > graphicStyle->multiNodeLimit.getValue() ? TRUE : FALSE);
  oneNode = TRUE;

  std::vector<SbVec3f> points(npoint);
  for(int count=0;count<npoint;count++) {
    float xx,yy;
    aPoints.getIthPoint (count,xx,yy);
    points[count] = SbVec3f(xx,yy,0); //FIXME : z ?
  }
  if(oneNode) {
    int ptn;
    SbVec3f* pts = clipPoints2D(points,aBoxX,aBoxY,0,ptn);
    if(ptn) {
      SoSeparator* separator = new SoSeparator();
      pointsNode->addChild(separator);
      const SbString& modeling = graphicStyle->modeling.getValue();
      if(modeling==SbModeling_lines){
        separator->addChild(getLineStyle(graphicStyle));
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addLinesToNode(separator,ptn,pts);
      } else if(modeling==SbModeling_segments) {
        separator->addChild(getLineStyle(graphicStyle));
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addSegmentsToNode(separator,ptn,pts);
      } else if(modeling==SbModeling_curve){
        separator->addChild(getLineStyle(graphicStyle));
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        //SoTools::addLagrangePolynomialCurveToNode(separator,ptn,pts);
        SoTools::addNurbsCurveToNode(separator,ptn,pts);
        //SoTools::addSplineCurveToNode(separator,ptn,pts);
      } else { //Default
        separator->addChild(fStyleCache->getPoints());
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addPointsToNode(separator,ptn,pts,
          (SbMarkerStyle)graphicStyle->markerStyle.getValue(),
          graphicStyle->markerSize.getValue());
      }
      delete [] pts;
    }
  } else {
    repPoints2D_xy(pointsNode,graphicStyle,points,aBoxX,aBoxY,0);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateBins2D_XYZ(
 const SbPlottableBins2D& aBins
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* binsNode = (SoSeparator*)binsSeparator.getValue();

  SoStyle* graphicStyle = getBinsStyle(aIndex);
  if(graphicStyle->visible.getValue()==FALSE) return;

  int xnbin = aBins.getAxisNumberOfBinsX();
  int ynbin = aBins.getAxisNumberOfBinsY();

  float bmin;
  float bmax;
  aBins.getBinsSumOfWeightsRange(bmin,bmax);

  // one node decision :
  int ibin,jbin;
  int count = 0;
  for(jbin=ynbin-1;jbin>=0;jbin--) {
    for(ibin=xnbin-1;ibin>=0;ibin--) { 
      if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;
      count++;
    }
  }
  SbBool oneNode = 
    (count > graphicStyle->multiNodeLimit.getValue()?TRUE:FALSE);

  int number = 0;
 {for(jbin=ynbin-1;jbin>=0;jbin--) {
    for(ibin=xnbin-1;ibin>=0;ibin--) { 
      if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;
      number++;
  }}}

  const SbString& modeling = graphicStyle->modeling.getValue();
  if(modeling==SbModeling_boxes){

    int bini = 0;
    std::vector<SbRepBin2D> bins(number);
    for(jbin=ynbin-1;jbin>=0;jbin--) {
      for(ibin=xnbin-1;ibin>=0;ibin--) { 
        if(aBins.getBinNumberOfEntries(ibin,jbin)<=0) continue;
  
        float val = aBins.getBinSumOfWeights(ibin,jbin);
  
        float xx = aBins.getBinLowerEdgeX(ibin);
        float xe = aBins.getBinUpperEdgeX(ibin);
        float yy = aBins.getBinLowerEdgeY(jbin);
        float ye = aBins.getBinUpperEdgeY(jbin);
                
        bins[bini] = SbRepBin2D(xx,xe,yy,ye,val,ibin,jbin);
        bini++;
      }
    }
            
    char s[128];
    sprintf(s,"Bin2D/0x%lx",(unsigned long)&aBins);
  
    if(oneNode) {
      repBins2D_xyz_boxes_one(binsNode,
                          graphicStyle,
                          bins,aBoxX,aBoxY,aBoxZ,bmin,bmax);
    } else {
      SbPaintingPolicy painting = 
        (SbPaintingPolicy)graphicStyle->painting.getValue();
      repBins2D_xyz_boxes(binsNode,
                graphicStyle,painting,colorMapping.getValue(),
                bins,aBoxX,aBoxY,aBoxZ,bmin,bmax,SbString(s));
    }

  } else if(modeling==SbModeling_curve){

    std::vector<SbRepTopFace2D> faces((xnbin-1)*(ynbin-1));
    int facei = 0;
    for(int jbin=0;jbin<(ynbin-1);jbin++) { 
      for(int ibin=0;ibin<(xnbin-1);ibin++) { 
    
        float xx_0 = aBins.getBinLowerEdgeX(ibin);
        float xe_0 = aBins.getBinUpperEdgeX(ibin);
        float xx_1 = aBins.getBinLowerEdgeX(ibin+1);
        float xe_1 = aBins.getBinUpperEdgeX(ibin+1);

        float yy_0 = aBins.getBinLowerEdgeY(jbin);
        float ye_0 = aBins.getBinUpperEdgeY(jbin);
        float yy_1 = aBins.getBinLowerEdgeY(jbin+1);
        float ye_1 = aBins.getBinUpperEdgeY(jbin+1);

        float xx = (xx_0+xe_0)/2;
        float xe = (xx_1+xe_1)/2;
        float yy = (yy_0+ye_0)/2;
        float ye = (yy_1+ye_1)/2;
 
        float val1 = aBins.getBinSumOfWeights(ibin,jbin);
        float val2 = aBins.getBinSumOfWeights(ibin+1,jbin);
        float val3 = aBins.getBinSumOfWeights(ibin+1,jbin+1);
        float val4 = aBins.getBinSumOfWeights(ibin,jbin+1);
              
        faces[facei] = SbRepTopFace2D(xx,xe,yy,ye,val1,val2,val3,val4); 
        facei++;
      }
    }

    SbPaintingPolicy painting = 
      (SbPaintingPolicy)graphicStyle->painting.getValue();
  
    SbBool oneNode = 
     ((int)faces.size() > graphicStyle->multiNodeLimit.getValue()?TRUE:FALSE);
  
    if(oneNode) {       
      repTopFaces2D_xyz_one(binsNode,graphicStyle,
                        faces,aBoxX,aBoxY,aBoxZ);
    } else {
      repTopFaces2D_xyz(binsNode,graphicStyle,
                    painting,colorMapping.getValue(),
                    faces,aBoxX,aBoxY,aBoxZ);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updatePoints3D_XYZ(
 const SbPlottablePoints3D& aPoints
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* pointsNode = (SoSeparator*)pointsSeparator.getValue();

  SoStyle* graphicStyle = getPointsStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  int npoint = aPoints.getNumberOfPoints ();
  SbBool oneNode = 
    (npoint > graphicStyle->multiNodeLimit.getValue()?TRUE:FALSE);
  oneNode = TRUE;

  std::vector<SbVec3f> points(npoint);
  for(int count=0;count<npoint;count++) {
    float x,y,z;
    aPoints.getIthPoint (count,x,y,z);
    points[count] = SbVec3f(x,y,z);
  }
  if(oneNode) {
    int ptn;
    SbVec3f* pts = clipPoints3D(points,aBoxX,aBoxY,aBoxZ,ptn);
    if(ptn) {
      SoSeparator* separator = new SoSeparator();
      pointsNode->addChild(separator);
      const SbString& modeling =  graphicStyle->modeling.getValue();
      if(modeling==SbModeling_lines){
        separator->addChild(getLineStyle(graphicStyle));
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addLinesToNode(separator,ptn,pts);
      } else if(modeling==SbModeling_segments) {
        separator->addChild(getLineStyle(graphicStyle));
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addSegmentsToNode(separator,ptn,pts);
      } else { //Default
        separator->addChild(fStyleCache->getPoints());
        separator->addChild(fStyleCache->getMaterial
          (graphicStyle->color.getValue(),
           graphicStyle->transparency.getValue()));
        SoTools::addPointsToNode(separator,ptn,pts,
          (SbMarkerStyle)graphicStyle->markerStyle.getValue(),
          graphicStyle->markerSize.getValue());
      }
      delete [] pts;
    }
  } else {
    repPoints3D_xyz(pointsNode,graphicStyle,points,aBoxX,aBoxY,aBoxZ);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateFunction_XYZ(
 const SbPlottableFunction2D& aFunction
,int aIndex
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* functionNode = (SoSeparator*)functionSeparator.getValue();

  SoStyle* graphicStyle = getFunctionStyle(aIndex);

  if(graphicStyle->visible.getValue()==FALSE) return;

  if(aFunction.getDimension()!=2) return;

  float xmn = aFunction.getMinimumX();
  float xmx = aFunction.getMaximumX();
  if(xmx<=xmn) {
    xmn = f_xDataAxis.getMinimumValue();
    xmx = f_xDataAxis.getMaximumValue();
  }
  int nx = aFunction.getNumberOfStepsX();
  nx = nx <=0 ? 20 : nx;

  float ymn = aFunction.getMinimumY();
  float ymx = aFunction.getMaximumY();
  if(ymx<=ymn) {
    ymn = f_yDataAxis.getMinimumValue();
    ymx = f_yDataAxis.getMaximumValue();
  }
  int ny = aFunction.getNumberOfStepsY();
  ny = ny <=0 ? 20 : ny;

  float dfx = (xmx - xmn)/nx;
  float dfy = (ymx - ymn)/ny;

  //printf("debug : nx %d ny %d\n",nx,ny);
  
  SbBool problem = FALSE;

  int facei = 0;
  std::vector<SbRepTopFace2D> faces(nx*ny);
  //  Take a slightly lowest delta value to avoid 
  // precision pbs at boundaries :
  dfx *= 0.99999F;
  dfy *= 0.99999F;
  float* vprev = new float[2*nx];
  for(int jbin=ny-1;jbin>=0;jbin--) { 
    float o1 = 0;
    float o4 = 0;
    for(int ibin=nx-1;ibin>=0;ibin--) { 
  
      float xx = xmn + ibin * dfx;
      float yy = ymn + jbin * dfy;
      float xe = xx + dfx;
      float ye = yy + dfy;
       
      // Values at the corner :    
      float val1;
      if(!aFunction.value(xx,yy,val1)) problem = TRUE;
      float val2;
      if(ibin==nx-1) {
        if(!aFunction.value(xe,yy,val2)) problem = TRUE;
      } else {
        val2 = o1;
      }
      float val3,val4;
      if(jbin==ny-1) {
        if(ibin==nx-1) {
          if(!aFunction.value(xe,ye,val3)) problem = TRUE;
        } else {
          val3 = o4;
        }
        if(!aFunction.value(xx,ye,val4)) problem = TRUE;
      } else {
        val3 = vprev[2*ibin+1];
        val4 = vprev[2*ibin];
      }
      //printf("debug : %d %d : %g %g %g %g\n",
      //   ibin,jbin,
      //   val1,val2,val3,val4);
      vprev[2*ibin] = val1;
      vprev[2*ibin+1] = val2;
      o4 = val4;
      o1 = val1;
            
      faces[facei] = SbRepTopFace2D(xx,xe,yy,ye,val1,val2,val3,val4); 
      facei++;
    }
  }
  delete [] vprev;

  if(problem==TRUE) {
    SoDebugError::postInfo("SoPlotter::updateFunction_XYZ",
      " problem when getting some function value.");
  }

  SbPaintingPolicy painting = 
    (SbPaintingPolicy)graphicStyle->painting.getValue();

  SbBool oneNode = 
   ((int)faces.size() > graphicStyle->multiNodeLimit.getValue()?TRUE:FALSE);

  if(oneNode) {       
    repTopFaces2D_xyz_one(functionNode,graphicStyle,
                      faces,aBoxX,aBoxY,aBoxZ);
  } else {
    repTopFaces2D_xyz(functionNode,graphicStyle,
                  painting,colorMapping.getValue(),
                  faces,aBoxX,aBoxY,aBoxZ);
  }

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

 // border X (Y=0)
 {SbVec3f* ptsx = new SbVec3f[nx+1+2];
  int posx = 0;
  SbBool problem = FALSE;
  for(int ibin=nx-1;ibin>=0;ibin--) { 

    float xx = xmn + ibin * dfx;
    float xe = xx + dfx;
    float yy = ymn;
       
    float val1;
    if(!aFunction.value(xx,yy,val1)) problem = TRUE;
        
    val1 = VerifyLog(val1,zmin,dz,zlog);
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
            
    if(val1<0) val1 = 0;
    if(val1>1) val1 = 1;

    if((xx>=0)&&(xx<=1)   &&
       (xe>=0)&&(xe<=1)   &&
       (yy>=0)&&(yy<=1) ) {
         if(ibin==nx-1) {
        LIST_SET(ptsx,posx,xe,yy,0);posx++;
        LIST_SET(ptsx,posx,xe,yy,val1);posx++;
        LIST_SET(ptsx,posx,xx,yy,val1);posx++;
      } else if(ibin==0) {
        LIST_SET(ptsx,posx,xx,yy,val1);posx++;
        LIST_SET(ptsx,posx,xx,yy,0);posx++;
      } else {
        LIST_SET(ptsx,posx,xx,yy,val1);posx++;
      }
    }
  }

  if(problem==TRUE) {
    SoDebugError::postInfo("SoPlotter::updateFunction_XYZ",
      " problem when getting some function value.");
  }

  SoSeparator* separator = new SoSeparator();
  separator->setName("sceneGraph");
  functionNode->addChild(separator);
  char s[128];
  sprintf(s,"Function/0x%lx",(unsigned long)aFunction.nativeObject());

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild
    (fStyleCache->getMaterial(SbColor(0.6F,0.2F,0.2F), //brown
                              graphicStyle->transparency.getValue()));
   SoTools::addPolygonToNode(separator,posx,ptsx,SbVec3f(0,-1,0),
                            SbString(s));
  delete [] ptsx;}

  // border Y (X=0) 
 {SbVec3f* ptsy = new SbVec3f[ny+1+2];
  int posy = 0;
  SbBool problem = FALSE;
  for(int jbin=ny-1;jbin>=0;jbin--) { 

    float xx = xmn;
    float yy = ymn + jbin * dfy;
    float ye = yy + dfy;
     
    float val1;
    if(!aFunction.value(xx,yy,val1)) problem = TRUE;
            
    val1 = VerifyLog(val1,zmin,dz,zlog);
    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
            
    if(val1<0) val1 = 0;
    if(val1>1) val1 = 1;

    if((xx>=0)&&(xx<=1)   &&
       (yy>=0)&&(yy<=1)   &&
       (ye>=0)&&(ye<=1)  ) {
      if(jbin==ny-1) {
        LIST_SET(ptsy,posy,xx,ye,0);posy++;
        LIST_SET(ptsy,posy,xx,ye,val1);posy++;
        LIST_SET(ptsy,posy,xx,yy,val1);posy++;
      } else if(jbin==0) {
        LIST_SET(ptsy,posy,xx,yy,val1);posy++;
        LIST_SET(ptsy,posy,xx,yy,0);posy++;
      } else {
        LIST_SET(ptsy,posy,xx,yy,val1);posy++;
      }
    }
  }

  if(problem==TRUE) {
    SoDebugError::postInfo("SoPlotter::updateFunction_XYZ",
      " problem when getting some function value.");
  }

  SoSeparator* separator = new SoSeparator();
  separator->setName("sceneGraph");
  functionNode->addChild(separator);

  ReverseList(posy,ptsy);

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild
    (fStyleCache->getMaterial(SbColor(0.6F,0.2F,0.2F), //brown
                              graphicStyle->transparency.getValue()));
   SoTools::addPolygonToNode(separator,posy,ptsy,SbVec3f(-1,0,0));
       
  delete [] ptsy;} //end borders
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateTitle(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fShape==XYZ) return; //FIXME

  SoSeparator* titleNode = (SoSeparator*)titleSeparator.getValue();

  SoTextStyle* graphicStyle = (SoTextStyle*)titleStyle.getValue();

  // Use the XY layout transform to position the title.
  // (Else we would have to compensate the 3D rotation
  //  in order to bring the title at the right position 
  //  without rotation).
  //titleNode->addChild(layout.getValue());

  SoMaterial* material = 
    fStyleCache->getMaterial(graphicStyle->color.getValue(),
                             graphicStyle->transparency.getValue());
  titleNode->addChild(material);

  float text_size = titleHeight.getValue() * graphicStyle->scale.getValue();

  SbString fontName = graphicStyle->fontName.getValue();
  if( (fontName.getLength() >=6) &&
      (fontName.getSubString(0,5)=="Text3/") ) {
    titleNode->addChild(fStyleCache->getFilled());
  } else if((fontName.getLength() >=6 ) && 
            (fontName.getSubString(0,5)=="Text2/")) {
  } else if(SbStringIsTTF(fontName)==TRUE) {
    text_size *= ttfScale.getValue();
  } else {  // Default is Hershey.
    titleNode->addChild(
      fStyleCache->getLineStyle(graphicStyle->linePattern.getValue(),
                       (float)graphicStyle->lineWidth.getValue()));
  }

  float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;

  float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;

  float xx = wData/2;
  float yy = hData + titleToAxis.getValue();
  SoTools::HorizontalJustification hJust = SoTools::CENTER;
  SoTools::VerticalJustification vJust = SoTools::BOTTOM;
  if(titleUp.getValue()==FALSE) {
    yy = -titleToAxis.getValue();
    vJust = SoTools::TOP;
  }
  SbVec3f vertex;
  vertex.setValue(xx,yy,0);

  //printf("debug : %lu : %g %g : text_size : %g : %s\n",
  //  this,xx,yy,text_size,title.getValue().getString());

  SbVec3f TX(1,0,0);
  SbVec3f TY(0,1,0);
  SoTools::addAlignedStringToNode(titleNode,
                                  fontName,
                                  graphicStyle->encoding.getValue(),
                                  graphicStyle->smoothing.getValue(),
                                  title.getValue(),vertex,TX,TY,
                                  text_size,
                                  hJust,vJust);

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateInnerFrame_XY(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : updateInnerFrame_XY\n");

  SoSeparator* innerFrameNode = (SoSeparator*)innerFrameSeparator.getValue();
  SoStyle* graphicStyle = (SoStyle*)innerFrameStyle.getValue();

  SbVec3f points[5];
  float z = 0.5F;

  LIST_SET(points,0,0,0,z);
  LIST_SET(points,1,1,0,z);
  LIST_SET(points,2,1,1,z);
  LIST_SET(points,3,0,1,z);
  LIST_SET(points,4,0,0,z);
  innerFrameNode->addChild(getLineStyle(graphicStyle));
  innerFrameNode->addChild(fStyleCache->getMaterial
    (graphicStyle->color.getValue(),
     graphicStyle->transparency.getValue()));

  SoTools::addLinesToNode(innerFrameNode,5,points);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateInnerFrame_XYZ(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* innerFrameNode = (SoSeparator*)innerFrameSeparator.getValue();

  SoStyle* graphicStyle = (SoStyle*)innerFrameStyle.getValue();

  innerFrameNode->addChild(getLineStyle(graphicStyle));
  innerFrameNode->addChild(fStyleCache->getMaterial
    (graphicStyle->color.getValue(),graphicStyle->transparency.getValue()));
  SbVec3f points[5];
  LIST_SET(points,0,0,0,0);
  LIST_SET(points,1,1,0,0);
  LIST_SET(points,2,1,1,0);
  LIST_SET(points,3,0,1,0);
  LIST_SET(points,4,0,0,0);
  SoTools::addLinesToNode(innerFrameNode,5,points);
  LIST_SET(points,0,0,0,1);
  LIST_SET(points,1,1,0,1);
  LIST_SET(points,2,1,1,1);
  LIST_SET(points,3,0,1,1);
  LIST_SET(points,4,0,0,1);
  SoTools::addLinesToNode(innerFrameNode,5,points);
  LIST_SET(points,0,0,0,0);
  LIST_SET(points,1,0,0,1);
  SoTools::addLinesToNode(innerFrameNode,2,points);
  LIST_SET(points,0,1,0,0);
  LIST_SET(points,1,1,0,1);
  SoTools::addLinesToNode(innerFrameNode,2,points);
  LIST_SET(points,0,1,1,0);
  LIST_SET(points,1,1,1,1);
  SoTools::addLinesToNode(innerFrameNode,2,points);
  LIST_SET(points,0,0,1,0);
  LIST_SET(points,1,0,1,1);
  SoTools::addLinesToNode(innerFrameNode,2,points);

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::updateColorMapping(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* colorMappingNode = 
    (SoSeparator*)colorMappingSeparator.getValue();

  SbBool showLowUp = FALSE;

  float* values = 0;
  SbColor* colors = 0;
  int valuen = SbStyle::getValueColorMapping
    (colorMapping.getValue(),values,colors);
  int colorn = (showLowUp==TRUE?valuen+1:valuen-1);

  float w  = 0.1F;
  float margin = 0.05F;
  float xx = 1.01F;
  float yy = 0;
  float h = colorn ? (1.F - margin * (colorn + 1))/colorn : 0;

  // Use the XY layout transform to position the title.
  // (Else we would have to compensate the 3D rotation
  //  in order to bring the title at the right position 
  //  without rotation).
  //colorMappingNode->addChild(layout.getValue());

  for(int count=0;count<colorn;count++)        {
    SbStyle graphicStyle;
    graphicStyle.setColor(colors[(showLowUp==TRUE?count:count+1)]);
    SbVec3f points[4];
    LIST_SET(points,0,xx      ,yy + margin     ,0);
    LIST_SET(points,1,xx + w  ,yy + margin     ,0);
    LIST_SET(points,2,xx + w  ,yy + margin + h ,0);
    LIST_SET(points,3,xx      ,yy + margin + h ,0);

    colorMappingNode->addChild(fStyleCache->getFilled());
    colorMappingNode->addChild(fStyleCache->getNormalBindingPerFace());
    colorMappingNode->addChild
      (fStyleCache->getMaterial(graphicStyle.getColor(),
                                graphicStyle.getTransparency()));

    SoTools::addPolygonToNode(colorMappingNode,4,points,SbVec3f(0,0,1)); 

    graphicStyle.setColor(SbColor_black);
    if(values) {
      char string[128];
      if(showLowUp==TRUE) {
        if(count==0) {
          sprintf(string,"<%g",values[0]);
        } else if(count==colorn-1) {
          sprintf(string,"%g<=",values[valuen-1]);
        } else {
          sprintf(string,"%g<= <%g",values[count-1],values[count]);
        }
      } else {
        sprintf(string,"%g<= <%g",values[count],values[count+1]);
      }
      int length = strlen(string);
      float text_size = SbMinimum(w / length,margin)  * 0.9F;
      SoMaterial* material = 
        fStyleCache->getMaterial(graphicStyle.getColor(),
                                 graphicStyle.getTransparency());
      colorMappingNode->addChild(material);

      SbString fontName = graphicStyle.getFontName();
      if( (fontName.getLength() >=6) &&
          (fontName.getSubString(0,5)=="Text3/") ) {
        colorMappingNode->addChild(fStyleCache->getFilled());
      } else if((fontName.getLength() >=6 ) && 
                (fontName.getSubString(0,5)=="Text2/")) {
      } else if(SbStringIsTTF(fontName)==TRUE) {
        text_size *= ttfScale.getValue();
      } else {  // Default is Hershey.
        colorMappingNode->addChild(getLineStyle(graphicStyle));
      }
      SoTools::addAlignedStringToNode(colorMappingNode,
                                      graphicStyle.getFontName(),
                                      graphicStyle.getEncoding(),
                                      graphicStyle.getSmoothing(),
                                      string,
                                      SbVec3f(xx + w/2,yy+0.01F,0),
                                      SbVec3f(1,0,0),
                                      SbVec3f(0,1,0),
                                      text_size,
                                      SoTools::CENTER,SoTools::BOTTOM);
    }
    yy += h + margin;
  }

  delete [] values;
  delete [] colors;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// reps xy ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_top_lines(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME SbPaintingPolicy painting = 
  //(SbPaintingPolicy)aStyle->painting.getValue();
  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float y0 = 0;
  y0 = VerifyLog(y0,ymin,dy,ylog);
  if(y0<0) y0 = 0;
  if(y0>1) y0 = 1;

  SoSeparator* separator = new SoSeparator();

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbBool empty = TRUE;
  SbVec3f points[5];
  int32_t coordIndex[6];
  float yp = 0;
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float y = aBins[index].fVal;
    
    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor(aBins[index].fVal,valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }
                          
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    y = VerifyLog(y,ymin,dy,ylog);

    // Clipping :
    if(xe<0) continue;
    if(xx>1) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(y<0) y = 0;
    if(y>1) y = 1;

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    aBins[index].fSeparator = sep;

    char s[128];
    sprintf(s,"%d",index);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    sep->addChild(getLineStyle(aStyle));

    int32_t pointn = 3;
    LIST_SET(points,0,xx,yp,aZZ);
    LIST_SET(points,1,xx,y,aZZ);
    LIST_SET(points,2,xe,y,aZZ);
    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = SO_END_LINE_INDEX;
    if(index==xnbin-1){
      pointn = 4;
      LIST_SET(points,3,xe,y0,aZZ);
      coordIndex[3] = icoord + 3;
      coordIndex[4] = SO_END_LINE_INDEX;
    }

    coordinate3->point.setValues(icoord,pointn,points);
    icoord += pointn;

    SoIndexedLineSet* lineSet = new SoIndexedLineSet;
    lineSet->setName(nodeName);
    lineSet->coordIndex.setValues(0,pointn+1,coordIndex);
    sep->addChild(lineSet);

    empty = FALSE;

    yp = y;
  }

  if(values) delete [] values;
  if(colors) delete [] colors;

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_top_lines_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(getLineStyle(aStyle));
          
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float y0 = 0;
  y0 = VerifyLog(y0,ymin,dy,ylog);
  if(y0<0) y0 = 0;
  if(y0>1) y0 = 1;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedLineSet* lineSet = new SoIndexedLineSet;
  separator->addChild(lineSet);
  int iindex = 0;

  int32_t coordIndex[6];
  SbVec3f points[5];
  SbBool empty = TRUE;
  float yp = 0;
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float y = aBins[index].fVal;

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    y = VerifyLog(y,ymin,dy,ylog);

    // Clipping :
    if(xe<0) continue;
    if(xx>1) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(y<0) y = 0;
    if(y>1) y = 1;

    int pointn = 3;
    LIST_SET(points,0,xx,yp,aZZ);
    LIST_SET(points,1,xx,y,aZZ);
    LIST_SET(points,2,xe,y,aZZ);
    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = SO_END_LINE_INDEX;
    if(index==xnbin-1){
      pointn = 4;
      LIST_SET(points,3,xe,y0,aZZ);
      coordIndex[3] = icoord + 3;
      coordIndex[4] = SO_END_LINE_INDEX;
    }

    coordinate3->point.setValues(icoord,pointn,points);
    icoord += pointn;

    lineSet->coordIndex.setValues(iindex,pointn+1,coordIndex);
    iindex += (pointn+1);

    empty = FALSE;
    yp = y;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_points(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
        
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getPoints());

  //SoCoordinate3* coordinate3 = new SoCoordinate3;
  //separator->addChild(coordinate3);
  //int icoord = 0;

  SbBool empty = TRUE;
  SbVec3f points[1];
  //int32_t coordIndex[2];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float x = (aBins[index].fXmin + aBins[index].fXmax)/2;
    float y = aBins[index].fVal;

    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor(aBins[index].fVal,valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }
                          
    x = VerifyLog(x,xmin,dx,xlog);
    y = VerifyLog(y,ymin,dy,ylog);

    // Clipping :
    if(x<0) continue;
    if(x>1) continue;
    if(y<0) continue;
    if(y>1) continue;

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    aBins[index].fSeparator = sep;

    char s[128];
    sprintf(s,"%d",index);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    LIST_SET(points,0,x,y,aZZ);

    SoTools::addPointsToNode(sep,1,points,
      (SbMarkerStyle)aStyle->markerStyle.getValue(),
      aStyle->markerSize.getValue(),nodeName);

    empty = FALSE;
  }

  if(values) delete [] values;
  if(colors) delete [] colors;

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_points_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(fStyleCache->getPoints());
          
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];
  SbBool empty = TRUE;
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float x = (aBins[index].fXmin + aBins[index].fXmax)/2;
    float y = aBins[index].fVal;

    x = VerifyLog(x,xmin,dx,xlog);
    y = VerifyLog(y,ymin,dy,ylog);

    // Clipping :
    if(x<0) continue;
    if(x>1) continue;
    if(y<0) continue;
    if(y>1) continue;

    LIST_SET(points,0,x,y,aZZ);

    SoTools::addPointsToNode(separator,1,points,
      (SbMarkerStyle)aStyle->markerStyle.getValue(),
      aStyle->markerSize.getValue());

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_boxes(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins1D_xy_boxes : ID \"%s\" zz %g\n",
    //aID.getString(),aZZ);

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
        
  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild(fStyleCache->getNormalZ());

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbBool empty = TRUE;
  SbVec3f points[5];
  int32_t coordIndex[6];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    float val = aBins[index].fVal;

    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor(val,valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    //FIXME if(ye<=yy) continue; //Else we shall have a tessellation error.

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    aBins[index].fSeparator = sep;

    char s[128];
    sprintf(s,"%d",index);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;
 
    SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
    indexedFaceSet->setName(nodeName);
    indexedFaceSet->coordIndex.setValues(0,6,coordIndex);
    sep->addChild(indexedFaceSet);

    empty = FALSE;
  }

  if(values) delete [] values;
  if(colors) delete [] colors;

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_boxes_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());          
  separator->addChild(fStyleCache->getNormalZ());

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
  separator->addChild(indexedFaceSet);
  int iindex = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbBool empty = TRUE;
  SbVec3f points[5];
  int32_t coordIndex[6];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    //FIXME if(ye<=yy) continue; //Else we shall have a tessellation error.

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;  

    indexedFaceSet->coordIndex.setValues(iindex,6,coordIndex);
    iindex += 6;

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_wire_boxes(
 SoSeparator* aParent
,SoStyle* aStyle
,SbBool aBarChart
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins1D_xy_wire_boxes : ID \"%s\" zz %g\n",
    //aID.getString(),aZZ);

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
        
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SoSeparator* separator = new SoSeparator();

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbBool empty = TRUE;
  int32_t coordIndex[6];
  SbVec3f points[5];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    float val = aBins[index].fVal;

    if(aBarChart==TRUE)
      BarChart(aStyle->barOffset.getValue(),aStyle->barWidth.getValue(),xx,xe);

    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor(val,valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }
                          
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    SoPickSeparator* sep = new SoPickSeparator; 
    sep->setName("sceneGraph");
    separator->addChild(sep);

    aBins[index].fSeparator = sep;

    char s[128];
    sprintf(s,"%d",index);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    sep->addChild(getLineStyle(aStyle));

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;
 
    SoIndexedLineSet* lineSet = new SoIndexedLineSet;
    lineSet->setName(nodeName);
    lineSet->coordIndex.setValues(0,6,coordIndex);
    sep->addChild(lineSet);

    empty = FALSE;
  }

  if(values) delete [] values;
  if(colors) delete [] colors;

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_wire_boxes_one(
 SoSeparator* aParent
,SoStyle* aStyle
,SbBool aBarChart
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(getLineStyle(aStyle));
          
  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedLineSet* lineSet = new SoIndexedLineSet;
  separator->addChild(lineSet);
  int iindex = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbBool empty = TRUE;
  int32_t coordIndex[6];
  SbVec3f points[5];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    if(aBarChart==TRUE)
      BarChart(aStyle->barOffset.getValue(),aStyle->barWidth.getValue(),xx,xe);

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;
 
    lineSet->coordIndex.setValues(iindex,6,coordIndex);
    iindex += 6;

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_curve_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(getLineStyle(aStyle));
          
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  int xnbin = aBins.size();
  SbVec3f* points = new SbVec3f[xnbin?xnbin:1];
  int pointi = 0;

  for(int index=0;index<xnbin;index++) { 
    float x = (aBins[index].fXmin + aBins[index].fXmax)/2;
    float y = aBins[index].fVal;

    x = VerifyLog(x,xmin,dx,xlog);
    y = VerifyLog(y,ymin,dy,ylog);

    //FIXME : clipping should be done on the nurbs.

    LIST_SET(points,pointi,x,y,aZZ);
    pointi++;

  }
  if(pointi<=1) {
    separator->unref();
  } else {
    SoTools::addNurbsCurveToNode(separator,pointi,points);
    aParent->addChild(separator);
  }

  delete [] points;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins1D_xy_lines_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
// Draw lines connecting top middle of bins.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_lines_one\n");
  int xnbin = aBins.size();
  std::vector<SbVec3f> points(xnbin);
  for(int index=0;index<xnbin;index++) { 
    float x = (aBins[index].fXmin + aBins[index].fXmax)/2;
    float y = aBins[index].fVal;
    points[index] = SbVec3f(x,y,aZZ);
  }
  int ptn;
  SbVec3f* pts = clipPolyLine2D(points,aBoxX,aBoxY,ptn);
  aParent->addChild(getLineStyle(aStyle));
  aParent->addChild(fStyleCache->getMaterial
              (aStyle->color.getValue(),
               aStyle->transparency.getValue()));
  SoTools::addLinesToNode(aParent,ptn,pts);
  delete [] pts;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repHatch1D_xy(
 SoSeparator* aParent
,SoStyle* aStyle
,SbBool aBarChart
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repHatch1D_xy : zz %g\n",aZZ);

  SoSeparator* separator = new SoSeparator;

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;
          
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbBool empty = TRUE;

  SbVec3f points[5];
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    if(aBarChart==TRUE)
      BarChart(aStyle->barOffset.getValue(),aStyle->barWidth.getValue(),xx,xe);

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    // Use the bin separator in order that 
    // picking a hatch picks also the bin.
    SoSeparator* binSep = aBins[index].fSeparator;
    if(binSep) {
      //We can have multiple hatching for a bins ; have a separator :
      SbHatching sbHatching;            
      sbHatching.setOffsetPoint(SbVec3f(0,0,aZZ));
      sbHatching.setAngle(aStyle->angle.getValue());
      sbHatching.setSpacing(aStyle->spacing.getValue()); 
      sbHatching.setStripWidth(aStyle->stripWidth.getValue()); 

      int res = sbHatching.checkPolyline(points,4);      
      if (res == TRUE) res = sbHatching.computePolyline(points,4);

      unsigned int numPoints = sbHatching.getNoPoints();
      unsigned int numVertices = sbHatching.getNoVertices();
      if((res==TRUE) && numPoints && numVertices) {

	SoSeparator* hatchSep = new SoSeparator(); 
	binSep->addChild(hatchSep);
	
        SbVec3f* xyz = new SbVec3f[numPoints];
        const std::vector<SbVec3f>& hatchPointsList = sbHatching.getPoints();
	for (unsigned int a=0;a<numPoints;a++) xyz[a] = hatchPointsList[a];
        coordinate3->point.setValues(icoord,numPoints,xyz);
        delete [] xyz;
	
	int32_t* coord = new int32_t[numVertices*3];
        for (unsigned int a=0;a<numVertices;a++) {
          int ia = 3 * a;
	  coord[ia] = icoord;
          icoord++;
	  coord[ia+1] = icoord;
          icoord++;
	  coord[ia+2] = SO_END_LINE_INDEX;
	}

	if(aStyle->stripWidth.getValue()==0) {
	  hatchSep->addChild(getLineStyle(aStyle));
          hatchSep->addChild(fStyleCache->getNormalBindingOverall());
          hatchSep->addChild(fStyleCache->getNormalZ());
    	  hatchSep->addChild(material);
          hatchSep->addChild(coordinate3);
	  SoIndexedLineSet* indexedLineSet = new SoIndexedLineSet;
	  hatchSep->addChild(indexedLineSet);
	  indexedLineSet->coordIndex.setValues(0,numVertices*3,coord);
	} else {
	  hatchSep->addChild(fStyleCache->getFilled());
          hatchSep->addChild(fStyleCache->getNormalBindingPerFace());
          hatchSep->addChild(fStyleCache->getNormalZ());
    	  hatchSep->addChild(material);
          hatchSep->addChild(coordinate3);
	  SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
	  hatchSep->addChild(indexedFaceSet);
	  indexedFaceSet->coordIndex.setValues(0,numVertices*3,coord);
	}
        delete [] coord;

        empty = FALSE;
      }
    }
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repHatch1D_xy_one(
 SoSeparator* aParent
,SoStyle* aStyle
,SbBool aBarChart
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator;

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  if(aStyle->stripWidth.getValue()!=0) {
    separator->addChild(fStyleCache->getFilled());
    separator->addChild(fStyleCache->getNormalBindingPerFace());          
    separator->addChild(fStyleCache->getNormalZ());
  } else {
    separator->addChild(getLineStyle(aStyle));
  }

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;
  
  SoIndexedLineSet* lineSet = 0;
  SoIndexedFaceSet* faceSet = 0;
  if(aStyle->stripWidth.getValue()==0) {
    lineSet = new SoIndexedLineSet;
    separator->addChild(lineSet);
  } else {
    faceSet = new SoIndexedFaceSet;
    separator->addChild(faceSet);
  }
  int iindex = 0;
    
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];
  int xnbin = aBins.size();
  SbBool empty = TRUE;

  for(int index=0;index<xnbin;index++) {
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fVmin;
    float ye = aBins[index].fVal;
    if(ye<yy) {
      yy = aBins[index].fVal;
      ye = aBins[index].fVmin;
    }

    if(aBarChart==TRUE)
      BarChart(aStyle->barOffset.getValue(),aStyle->barWidth.getValue(),xx,xe);

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);

    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    LIST_SET(points,0,xx,yy,aZZ);
    LIST_SET(points,1,xe,yy,aZZ);
    LIST_SET(points,2,xe,ye,aZZ);
    LIST_SET(points,3,xx,ye,aZZ);
    LIST_SET(points,4,xx,yy,aZZ);

    SbHatching sbHatching;            
    sbHatching.setOffsetPoint(SbVec3f(0,0,aZZ));
    sbHatching.setAngle(aStyle->angle.getValue());
    sbHatching.setSpacing(aStyle->spacing.getValue()); 
    sbHatching.setStripWidth(aStyle->stripWidth.getValue()); 
    
    int res = sbHatching.checkPolyline(points,4);      
    if (res == TRUE) res = sbHatching.computePolyline(points,4);

    unsigned int numPoints = sbHatching.getNoPoints();
    unsigned int numVertices = sbHatching.getNoVertices();
    if((res==TRUE) && numPoints && numVertices) {

      const std::vector<SbVec3f>& hatchPointsList = sbHatching.getPoints();

      SbVec3f* xyz = new SbVec3f[numPoints];
      for (unsigned int a=0;a<numPoints;a++) xyz[a] = hatchPointsList[a];
      coordinate3->point.setValues(icoord,numPoints,xyz);
      delete [] xyz;

      int32_t* coord = new int32_t[numVertices*3];
      for (unsigned int a=0;a<numVertices;a++) {
        int ia = 3 * a;
        coord[ia] = icoord;
        icoord++;
	coord[ia+1] = icoord;
        icoord++;
	coord[ia+2] = SO_END_LINE_INDEX;
      }

      if(aStyle->stripWidth.getValue()==0) {
	lineSet->coordIndex.setValues(iindex,numVertices*3,coord);
      } else {
	faceSet->coordIndex.setValues(iindex,numVertices*3,coord);
      }
      delete [] coord;

      iindex += numVertices*3;

      empty = FALSE;
    }
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_random_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aBmin
,float aBmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_random_one\n");

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  //  Draw for each bins a number of random 
  // points proportiannal to bins range.

  float range = aBmax - aBmin;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  int32_t icoord = 0;

  SbVec3f points[100];

  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    // If range is 0. ; then all bins that have entries 
    // have same values. Draw one point.

    int nmin = 1;
    int nmax = 50;
    int npt = range>0. ? (int)((val - aBmin)*(nmax-nmin)/range + nmin):1;
    if(npt>0) {
      int ipt = 0;                              
      float xdbin = xe - xx;
      float ydbin = ye - yy;
      for(int count=0;count<npt;count++) {
        float xxx = xx + xdbin * (((float)rand())/RAND_MAX);
        float yyy = yy + ydbin * (((float)rand())/RAND_MAX);
        xxx = VerifyLog(xxx ,xmin,dx  ,xlog);
        yyy = VerifyLog(yyy ,ymin,dy  ,ylog);
        if((xxx>=0)&&(xxx<=1)  &&
           (yyy>=0)&&(yyy<=1)  ) { 
          LIST_SET(points,ipt,xxx,yyy,0);
          ipt++;
        }                                  
      }
      if(ipt>0) {
        int32_t pointn = ipt;
        coordinate3->point.setValues(icoord,pointn,points);
        icoord += pointn;
      }
    }

  }//end for                      

  if(icoord<=0) {
    coordinate3->unref();
  } else {
    SoSeparator* separator = new SoSeparator();
    aParent->addChild(separator);

    separator->addChild(fStyleCache->getPoints());

    SoMaterial* material = 
      fStyleCache->getMaterial(aStyle->color.getValue(),
                               aStyle->transparency.getValue());
    separator->addChild(material);

    separator->addChild(coordinate3);

    SbMarkerStyle smark = (SbMarkerStyle)aStyle->markerStyle.getValue();
    if(smark==SbMarkerDot) {
      SoPointSet* pointSet = new SoPointSet;
      pointSet->numPoints.setValue(icoord);
      separator->addChild(pointSet);
    } else {
      HEPVis_SoMarkerSet* markerSet = new HEPVis_SoMarkerSet;
      int wmark = aStyle->markerSize.getValue();
      markerSet->markerIndex.setValue
        (HEPVis_SoMarkerSet::getMarkerIndex(smark,wmark));
      markerSet->numPoints.setValue(icoord);
      separator->addChild(markerSet);
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_box_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aBmin
,float aBmax
)
//////////////////////////////////////////////////////////////////////////////
// Draw box of size proportionnal to bin value.                  
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_box_one\n");

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float range = aBmax - aBmin;
            
  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  SoNormal* normal = new SoNormal;
  separator->addChild(normal);
  int inormal = 0;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
  separator->addChild(indexedFaceSet);
  int iindex = 0;

  SbVec3f line[4];
  int32_t coordIndex[5];
  SbVec3f zunit(0,0,1);
  for(unsigned int index=0;index<aBins.size();index++) {
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    float xsize,ysize;
    if(range>0) {                  
      // If val = bmax, the box maps the cell.
      xsize = (val - aBmin) * (xe - xx) / range;
      ysize = (val - aBmin) * (ye - yy) / range;
    } else {
      //If range is 0. ; then all bins that have 
      // entries have same values. Draw box xdbin * ydbin.
      xsize = xe - xx;
      ysize = ye - yy;
    }

    xx = xx + ((xe-xx) - xsize)/2;
    xe = xx + xsize;
    yy = yy + ((ye-yy) - ysize)/2;
    ye = yy + ysize;
                                  
    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                                  
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    line[0].setValue(xx,yy,0);
    line[1].setValue(xe,yy,0);
    line[2].setValue(xe,ye,0);
    line[3].setValue(xx,ye,0);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,4,line);
    icoord += 4;  

    normal->vector.setValues(inormal,1,&zunit);
    inormal++;  

    indexedFaceSet->coordIndex.setValues(iindex,5,coordIndex);
    iindex += 5;
  }

  if(icoord<=0) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
            
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_box(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aBmin
,float aBmax
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
// Draw box of size proportionnal to bin value.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_box\n");
  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild(fStyleCache->getNormalZ());

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbBool empty = TRUE;
  SbVec3f points[5];
  int32_t coordIndex[6];
  float range = aBmax - aBmin;            
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    float xsize,ysize;
    if(range>0) {                  
      // If val = bmax, the box maps the cell.
      xsize = (val - aBmin) * (xe - xx) / range;
      ysize = (val - aBmin) * (ye - yy) / range;
    } else {
      //If range is 0. ; then all bins that have 
      // entries have same values. Draw box xdbin * ydbin.
      xsize = xe - xx;
      ysize = ye - yy;
    }

    xx = xx + ((xe-xx) - xsize)/2;
    xe = xx + xsize;
    yy = yy + ((ye-yy) - ysize)/2;
    ye = yy + ysize;
                                  
    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                                  
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    char s[128];
    sprintf(s,"%d %d",aBins[index].fI,aBins[index].fJ);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(aStyle->color.getValue(),
                               aStyle->transparency.getValue());
    sep->addChild(material);

    LIST_SET(points,0,xx,yy,0);
    LIST_SET(points,1,xe,yy,0);
    LIST_SET(points,2,xe,ye,0);
    LIST_SET(points,3,xx,ye,0);
    LIST_SET(points,4,xx,yy,0);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;  

    SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
    indexedFaceSet->setName(nodeName);
    indexedFaceSet->coordIndex.setValues(0,6,coordIndex);
    sep->addChild(indexedFaceSet);

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_wire_box(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aBmin
,float aBmax
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
// Draw box of size proportionnal to bin value.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_wire_box\n");
  SoSeparator* separator = new SoSeparator();

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbBool empty = TRUE;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];                      
  int32_t coordIndex[6];
  float range = aBmax - aBmin;
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    float xsize,ysize;
    if(range>0) {                  
      // If val = bmax, the box maps the cell.
      xsize = (val - aBmin) * (xe - xx) / range;
      ysize = (val - aBmin) * (ye - yy) / range;
    } else {
      //If range is 0. ; then all bins that have 
      // entries have same values. Draw box xdbin * ydbin.
      xsize = xe - xx;
      ysize = ye - yy;
    }

    xx = xx + ((xe-xx) - xsize)/2;
    xe = xx + xsize;
    yy = yy + ((ye-yy) - ysize)/2;
    ye = yy + ysize;
                                  
    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                                  
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    char s[128];
    sprintf(s,"%d %d",aBins[index].fI,aBins[index].fJ);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(aStyle->color.getValue(),
                               aStyle->transparency.getValue());
    sep->addChild(material);

    sep->addChild(getLineStyle(aStyle));

    LIST_SET(points,0,xx,yy,0);
    LIST_SET(points,1,xe,yy,0);
    LIST_SET(points,2,xe,ye,0);
    LIST_SET(points,3,xx,ye,0);
    LIST_SET(points,4,xx,yy,0);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;

    SoIndexedLineSet* lineSet = new SoIndexedLineSet;
    lineSet->setName(nodeName);
    lineSet->coordIndex.setValues(0,6,coordIndex);
    sep->addChild(lineSet);

    empty = FALSE;
  }            
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_wire_box_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aBmin
,float aBmax
)
//////////////////////////////////////////////////////////////////////////////
// Draw wired box of size proportionnal to bin value.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_wire_box_one\n");
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(getLineStyle(aStyle));

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedLineSet* lineSet = new SoIndexedLineSet;
  separator->addChild(lineSet);
  int iindex = 0;

  SbBool empty = TRUE;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];                      
  int32_t coordIndex[6];
  float range = aBmax - aBmin;
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    float xsize,ysize;
    if(range>0) {                  
      // If val = bmax, the box maps the cell.
      xsize = (val - aBmin) * (xe - xx) / range;
      ysize = (val - aBmin) * (ye - yy) / range;
    } else {
      //If range is 0. ; then all bins that have 
      // entries have same values. Draw box xdbin * ydbin.
      xsize = xe - xx;
      ysize = ye - yy;
    }

    xx = xx + ((xe-xx) - xsize)/2;
    xe = xx + xsize;
    yy = yy + ((ye-yy) - ysize)/2;
    ye = yy + ysize;
                                  
    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                                  
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    LIST_SET(points,0,xx,yy,0);
    LIST_SET(points,1,xe,yy,0);
    LIST_SET(points,2,xe,ye,0);
    LIST_SET(points,3,xx,ye,0);
    LIST_SET(points,4,xx,yy,0);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = icoord + 4;
    coordIndex[5] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,5,points);
    icoord += 5;

    lineSet->coordIndex.setValues(iindex,6,coordIndex);
    iindex += 6;

    empty = FALSE;
  }            
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_solid_one(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
// Draw box of bin size colored according a color map.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_solid_one\n");

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
            
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  //float zmin = aBoxZ.fPos;
  //float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  if(zlog==TRUE) {
    if(valn==1) {
      values[0] = TakeLog(values[0]);
    } else if(valn>=2) {
      //CERN/PAW coloring :
      if(values[0]==0) values[0] = 10e-5F;
      float vmin = TakeLog(values[0]);
      float vmax = TakeLog(values[valn-1]);
      float dv = (vmax-vmin)/(valn-1);
      for(int count=0;count<valn;count++) {
        values[count] = vmin + dv * count;
      }
    }
  }

  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild(fStyleCache->getNormalZ());

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbBool empty = TRUE;
  SbVec3f points[4];
  int32_t coordIndex[5];                      
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;
    SbColor sbColor;
    if((aPainting==SbPaintingByValue)&&(valn>0)) {
      sbColor = GetColor((zlog==TRUE?TakeLog(val):val),valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }

    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                                  
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    separator->addChild(material);

    LIST_SET(points,0,xx ,yy ,0);
    LIST_SET(points,1,xe ,yy ,0);
    LIST_SET(points,2,xe ,ye ,0);
    LIST_SET(points,3,xx ,ye ,0);

    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = icoord + 2;
    coordIndex[3] = icoord + 3;
    coordIndex[4] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,4,points);
    icoord += 4;  

    SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
    indexedFaceSet->coordIndex.setValues(0,5,coordIndex);
    separator->addChild(indexedFaceSet);

    empty = FALSE;
  }
            
  if(values) delete [] values;
  if(colors) delete [] colors;

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_text_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
)
//////////////////////////////////////////////////////////////////////////////
//Draw box and text for number of entries.          
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_text_one\n");

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];
  int32_t coordIndex[6];
            
  SoSeparator* separator = new SoSeparator();

  separator->addChild(getLineStyle(aStyle));

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  SbBool drawWireBoxe = FALSE;

  SoCoordinate3* coordinate3 = 0;
  int icoord = 0;

  SoIndexedLineSet* lineSet = 0;
  int iindex = 0;
  if(drawWireBoxe==TRUE) {
    coordinate3 = new SoCoordinate3;
    separator->addChild(coordinate3);

    lineSet = new SoIndexedLineSet;
    separator->addChild(lineSet);
  }

  SbBool empty = TRUE;
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    int ival = (int)val;
    if(ival==0) continue;

    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    char sval[32];
    sprintf (sval,"%d",ival);
    SbString sbval(sval);
    int charn = sbval.getLength();
    if(charn<=0) continue;

    float dx = xe-xx;
    float dy = ye-yy;
                  
    if(drawWireBoxe==TRUE) {

      LIST_SET(points,0,xx,yy,0);
      LIST_SET(points,1,xe,yy,0);
      LIST_SET(points,2,xe,ye,0);
      LIST_SET(points,3,xx,ye,0);
      LIST_SET(points,4,xx,yy,0);

      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = icoord + 3;
      coordIndex[4] = icoord + 4;
      coordIndex[5] = SO_END_LINE_INDEX;

      coordinate3->point.setValues(icoord,5,points);
      icoord += 5;
 
      lineSet->coordIndex.setValues(iindex,6,coordIndex);
      iindex += 6;
    }

    float x = xx + 0.1F * dx;
    float y = yy + 0.1F * dy;
    float z = 0;
    float w = dx * 0.5F;
    float h = dy * 0.5F;

    SoSeparator* sepText = new SoSeparator;
    separator->addChild(sepText);

    SoTransform* transform = new SoTransform;
    transform->scaleFactor.setValue(SbVec3f(w/charn,h,1));
    transform->translation.setValue(x,y,z);
    sepText->addChild(transform);

    SoTextHershey* text = new SoTextHershey;
    text->string.setValue(sbval);
    sepText->addChild (text);

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }            
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xy_text(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//Draw box and text for number of entries.          
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_xy_text\n");

  SoSeparator* separator = new SoSeparator();

  SbBool drawWireBoxe = FALSE;

  SoCoordinate3* coordinate3 = 0;
  int icoord = 0;

  if(drawWireBoxe==TRUE) {
    coordinate3 = new SoCoordinate3;
    separator->addChild(coordinate3);
  }

  SbBool empty = TRUE;
  int32_t coordIndex[6];

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  SbVec3f points[5];
            
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float val = aBins[index].fVal;

    int ival = (int)val;
    if(ival==0) continue;

    xx = VerifyLog(xx ,xmin,dx  ,xlog);
    xe = VerifyLog(xe ,xmin,dx  ,xlog);
    yy = VerifyLog(yy ,ymin,dy  ,ylog);
    ye = VerifyLog(ye ,ymin,dy  ,ylog);
                
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    char sval[32];
    sprintf (sval,"%d",ival);
    SbString sbval(sval);
    int charn = sbval.getLength();
    if(charn<=0) continue;

    SoPickSeparator* sep = new SoPickSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    char s[128];
    sprintf(s,"%d %d",aBins[index].fI,aBins[index].fJ);
    sep->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)sep->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(aStyle->color.getValue(),
                               aStyle->transparency.getValue());
    sep->addChild(material);

    sep->addChild(getLineStyle(aStyle));

    float dx = xe-xx;
    float dy = ye-yy;
                  
    if(drawWireBoxe==TRUE) {
      LIST_SET(points,0,xx,yy,0);
      LIST_SET(points,1,xe,yy,0);
      LIST_SET(points,2,xe,ye,0);
      LIST_SET(points,3,xx,ye,0);
      LIST_SET(points,4,xx,yy,0);
      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = icoord + 3;
      coordIndex[4] = icoord + 4;
      coordIndex[5] = SO_END_LINE_INDEX;
  
      coordinate3->point.setValues(icoord,5,points);
      icoord += 5;
 
      SoIndexedLineSet* lineSet = new SoIndexedLineSet;
      lineSet->setName(nodeName);
      lineSet->coordIndex.setValues(0,6,coordIndex);
      sep->addChild(lineSet);
    }

    float x = xx + 0.1F * dx;
    float y = yy + 0.1F * dy;
    float z = 0;
    float w = dx * 0.5F;
    float h = dy * 0.5F;

    SoTransform* transform = new SoTransform;
    transform->scaleFactor.setValue(SbVec3f(w/charn,h,1));
    transform->translation.setValue(x,y,z);
    sep->addChild(transform);

    SoTextHershey* text = new SoTextHershey;
    text->setName(nodeName);
    text->string.setValue(sbval);
    sep->addChild (text);

    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }            
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repContour_xy(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,CListContour& aContour
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : repContour_xy\n");

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
            
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  if(zlog==TRUE) {
    if(valn==1) {
      values[0] = TakeLog(values[0]);
    } else if(valn>=2) {
      //CERN/PAW coloring :
      float vmin = TakeLog(values[0]);
      float vmax = TakeLog(values[valn-1]);
      float dv = (vmax-vmin)/(valn-1);
      for(int count=0;count<valn;count++) {
        values[count] = vmin + dv * count;
      }
    }
  }

  SoSeparator* separator = new SoSeparator();

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbBool empty = TRUE;

  for (unsigned int i=0;i<aContour.GetNPlanes();i++)  {
    CLineStripList* pStripList = aContour.GetLines(i);
    if(!pStripList) {
      SoDebugError::postInfo
      ("SoPlotter::repContour_xy","problem with contour.");
      separator->unref();
      if(values) delete [] values;
      if(colors) delete [] colors;
      return;
    }

    SoSeparator* sep = new SoSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    //If zlog true, zz is already in log.
    float zz = (float)aContour.GetPlane(i);
 
    SbColor sbColor;
    if(valn>0) {
      if(aPainting==SbPaintingByLevel) {
        sbColor = colors[(i % (valn-1))+1];
      } else { //SbPaintingByValue
        sbColor = GetColor(zz,valn,values,colors);
      } 
    } else {
      sbColor = aStyle->color.getValue();
    }

    zz = VerifyLog(zz,zmin,dz,FALSE);
    if(zz>1) continue;
    if(zz<0) continue;

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    sep->addChild(getLineStyle(aStyle));

    SoIndexedLineSet* lineSet = new SoIndexedLineSet;
    lineSet->setName(aID);
    sep->addChild(lineSet);
    int iindex = 0;

    for (CLineStripList::iterator pos=pStripList->begin(); 
         pos != pStripList->end() ; pos++) {
      CLineStrip* pStrip = (*pos);
      if(!pStrip) {
        SoDebugError::postInfo
        ("SoPlotter::repContour_xy","problem with contour.");
        separator->unref();
        if(values) delete [] values;
        if(colors) delete [] colors;
        return;
      }
      if (pStrip->empty()) continue;
    
      /*
      // putting point at start and end of strip
      // retreiving index
      unsigned int index=pStrip->front();
      double xb=aContour.GetXi(index);
      double yb=aContour.GetYi(index);
      // retreiving index
      //glColor4f(0,0,1,.8f);
      index=pStrip->back();
      double xe=aContour.GetXi(index);
      double ye=aContour.GetYi(index);
      */    

      SbVec3f* points = new SbVec3f[pStrip->size()];
      int pointi = 0;
      int32_t* coordIndex = new int32_t[pStrip->size()+1];
      for (CLineStrip::iterator pos2=pStrip->begin(); 
           pos2 != pStrip->end() ; pos2++) {
        // retreiving index
        unsigned int index=(*pos2);
        float xx = (float)aContour.GetXi(index);
        float yy = (float)aContour.GetYi(index);
        xx = VerifyLog(xx,xmin,dx,xlog);
        yy = VerifyLog(yy,ymin,dy,ylog);
        if( 
            (xx<0) || (xx>1) ||
            (yy<0) || (yy>1) 
          ) {
          // Throw away this strip :
        }
        LIST_SET(points,pointi,xx,yy,aZZ+zz);
        coordIndex[pointi] = icoord + pointi;
        pointi++;
      }
      coordIndex[pointi] = SO_END_LINE_INDEX;

      if(pointi) {
        coordinate3->point.setValues(icoord,pointi,points);
        icoord += pointi;
        lineSet->coordIndex.setValues(iindex,pointi+1,coordIndex);
        iindex += pointi+1;
        empty = FALSE;
      }

      delete [] points;
      delete [] coordIndex;

    }
  }

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }

  if(values) delete [] values;
  if(colors) delete [] colors;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repContour_xy_filled(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,CListContour& aContour
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aZZ
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : repContour_xy_filled\n");

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
            
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  if(zlog==TRUE) {
    if(valn==1) {
      values[0] = TakeLog(values[0]);
    } else if(valn>=2) {
      //CERN/PAW coloring :
      float vmin = TakeLog(values[0]);
      float vmax = TakeLog(values[valn-1]);
      float dv = (vmax-vmin)/(valn-1);
      for(int count=0;count<valn;count++) {
        values[count] = vmin + dv * count;
      }
    }
  }

  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
  separator->addChild(fStyleCache->getNormalZ());

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  {// Draw background :
    SoSeparator* sep = new SoSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    float zz = aZZ - 0.01F;

    SbColor sbColor;
    if(valn>0) {
      sbColor = colors[0];
    } else {
      sbColor = aStyle->color.getValue();
    }

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    float xx = xmin;
    float xe = xmin+dx; 
    float yy = ymin;
    float ye = ymin+dy; 

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
                          
    SbVec3f points[4];
    LIST_SET(points,0,xx,yy,zz);
    LIST_SET(points,1,xe,yy,zz);
    LIST_SET(points,2,xe,ye,zz);
    LIST_SET(points,3,xx,ye,zz);
    coordinate3->point.setValues(icoord,4,points);
  
    int32_t coordIndex[5];
    coordIndex[0] = icoord;
    coordIndex[1] = icoord+1;
    coordIndex[2] = icoord+2;
    coordIndex[3] = icoord+3;
    coordIndex[4] = SO_END_FACE_INDEX;
    icoord += 4;

    SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
    indexedFaceSet->setName(aID);
    indexedFaceSet->coordIndex.setValues(0,5,coordIndex);
    sep->addChild(indexedFaceSet);
  } // End background.

  SbBool empty = TRUE;
  for (unsigned int i=0;i<aContour.GetNPlanes();i++)  {

    CLineStripList* pStripList = aContour.GetLines(i);
    if(!pStripList) {
      SoDebugError::postInfo
      ("SoPlotter::repContour_xy_filled","problem with contour.");
      separator->unref();
      if(values) delete [] values;
      if(colors) delete [] colors;
      return;
    }

    SoSeparator* sep = new SoSeparator;
    sep->setName("sceneGraph");
    separator->addChild(sep);

    //If zlog true, zz is already in log.
    float zz = (float)aContour.GetPlane(i);

    SbColor sbColor;
    if(valn>0) {
      if(aPainting==SbPaintingByLevel) {
        sbColor = colors[(i % (valn-1))+1];
      } else {  //SbPaintingByValue
        sbColor = GetColor(zz,valn,values,colors);
      } 
    } else {
      sbColor = aStyle->color.getValue();
    }

    zz = VerifyLog(zz,zmin,dz,FALSE);
    if(zz>1) continue;
    if(zz<0) continue;

    std::vector< std::vector<SbVec3f> > contourVector;
    unsigned int nbContours = 0;
    contourVector.resize(pStripList->size());
    for (CLineStripList::iterator pos=pStripList->begin();
         pos != pStripList->end() ; pos++) {
      CLineStrip* pStrip = (*pos);
      if (pStrip->size() >2) {
        //contourVector[nbContours].resize(0);
        //contourVector[nbContours].clear();
        for (CLineStrip::iterator pos2=pStrip->begin();
             pos2 != pStrip->end();pos2++) {
          unsigned int index=(*pos2);
          float xx = (float)aContour.GetXi(index);
          float yy = (float)aContour.GetYi(index);
          xx = VerifyLog(xx,xmin,dx,xlog);
          yy = VerifyLog(yy,ymin,dy,ylog);
          contourVector[nbContours].push_back(SbVec3f(xx,yy,aZZ+zz));
        }
        nbContours++;
      }
    }
    
    SbTessContour tessContour;
    std::vector<SbTessTriangle> tris = 
      tessContour.getFilledArea(contourVector);

    if(tris.size()<=0) continue;

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    sep->addChild(material);

    SbVec3f* points = new SbVec3f[tris.size()*3];
    int pointi = 0;
    int32_t* coordIndex = new int32_t[tris.size()*4];
    int coordi = 0;
    for(unsigned itri=0;itri<tris.size();itri++) {
      SbTessTriangle& tri = tris[itri];

     {float xx = (float)tri.pointA[0];
      float yy = (float)tri.pointA[1];
      float zz = (float)tri.pointA[2];
      LIST_SET(points,pointi,xx,yy,zz);
      coordIndex[coordi] = icoord + pointi;
      coordi++;
      pointi++;}

     {float xx = (float)tri.pointB[0];
      float yy = (float)tri.pointB[1];
      float zz = (float)tri.pointB[2];
      LIST_SET(points,pointi,xx,yy,zz);
      coordIndex[coordi] = icoord + pointi;
      coordi++;
      pointi++;}

     {float xx = (float)tri.pointC[0];
      float yy = (float)tri.pointC[1];
      float zz = (float)tri.pointC[2];
      LIST_SET(points,pointi,xx,yy,zz);
      coordIndex[coordi] = icoord + pointi;
      coordi++;
      pointi++;}

      coordIndex[coordi] = SO_END_FACE_INDEX;
      coordi++;
    }

    coordinate3->point.setValues(icoord,pointi,points);
    icoord += pointi;

    SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
    indexedFaceSet->setName(aID);
    sep->addChild(indexedFaceSet);
    indexedFaceSet->coordIndex.setValues(0,coordi,coordIndex);

    delete [] points;
    delete [] coordIndex;

    empty = FALSE;

  }

  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }

  if(values) delete [] values;
  if(colors) delete [] colors;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repErrorsI_xy(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const std::vector<float>& aBars
,float aZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  separator->addChild(getLineStyle(aStyle));

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);
            
  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedLineSet* lineSet = new SoIndexedLineSet;
  separator->addChild(lineSet);
  int iindex = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  int32_t coordIndex[7];
  SbBool empty = TRUE;
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
          
    //Need all bins modeled for fitting.
    //if(binsloop->getBinNumberOfEntries(index)<=0) continue;
                  
    float val = aBins[index].fVal;
    float bar_height = aBars[index];
                          
    float bar_min = val - bar_height/2;
    float bar_max = val + bar_height/2;
                          
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    val = VerifyLog(val,ymin,dy,ylog);
          
    bar_min = VerifyLog(bar_min,ymin,dy,ylog);
    bar_max = VerifyLog(bar_max,ymin,dy,ylog);

    if(xe<0) continue;
    if(xx>1) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(val<0) val = 0;
    if(val>1) val = 1;

    float ex = (xe+xx)/2;
    //if( (ex >=0)&&(ex <=1) ) { //FIXME : have to clip

    float edx = 0.3F * (xe-xx);
    int ipt = 0;
    SbVec3f points[6];
    if(bar_min >1)  {
      // do nothing
    } else  if(bar_max <0)  {
      // do nothing
    } else  if(bar_min <0) { 
      if(bar_max >1) { 
        LIST_SET(points,ipt,ex,0,aZ);ipt++;
        LIST_SET(points,ipt,ex,1,aZ);ipt++;
      } else { 
        LIST_SET(points,ipt,ex,0,aZ);ipt++;
        LIST_SET(points,ipt,ex,bar_max,aZ);ipt++;
        LIST_SET(points,ipt,ex-edx,bar_max,aZ);ipt++;
        LIST_SET(points,ipt,ex+edx,bar_max,aZ);ipt++;
      }
    } else  if(bar_max >1) { 
      LIST_SET(points,ipt,ex-edx,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex+edx,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex,1,aZ);ipt++;
    } else {
      LIST_SET(points,ipt,ex-edx,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex+edx,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex    ,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex    ,bar_max,aZ);ipt++;
      LIST_SET(points,ipt,ex-edx,bar_max,aZ);ipt++;
      LIST_SET(points,ipt,ex+edx,bar_max,aZ);ipt++;
    }

    if(ipt<=0) continue;

    for(int i=0;i<ipt;i++) coordIndex[i] = icoord + i;
    coordIndex[ipt] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,ipt,points);
    icoord += ipt;

    lineSet->coordIndex.setValues(iindex,ipt+1,coordIndex);
    iindex += (ipt+1);

    empty = FALSE;
  }          
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repErrorsPlus_xy(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin1D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const std::vector<float>& aBars
,float aZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoSeparator* separator = new SoSeparator();

  separator->addChild(getLineStyle(aStyle));

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);
            
  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedLineSet* lineSet = new SoIndexedLineSet;
  separator->addChild(lineSet);
  int iindex = 0;

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  int32_t coordIndex[7];
  SbBool empty = TRUE;
  int xnbin = aBins.size();
  for(int index=0;index<xnbin;index++) { 
          
    //Need all bins modeled for fitting.
    //if(binsloop->getBinNumberOfEntries(index)<=0) continue;
                  
    float val = aBins[index].fVal;
    float bar_height = aBars[index];
                          
    float bar_min = val - bar_height/2;
    float bar_max = val + bar_height/2;
                          
    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    val = VerifyLog(val,ymin,dy,ylog);
          
    bar_min = VerifyLog(bar_min,ymin,dy,ylog);
    bar_max = VerifyLog(bar_max,ymin,dy,ylog);

    if(xe<0) continue;
    if(xx>1) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(val<0) val = 0;
    if(val>1) val = 1;

    float ex = (xe+xx)/2;
    //if( (ex >=0)&&(ex <=1) ) //FIXME : have to clip
    //float edx = 0.3F * (xe-xx);

    int ipt = 0;
    SbVec3f points[6];
    if(bar_min >1)  {
      // do nothing
    } else  if(bar_max <0)  {
      // do nothing
    } else  if(bar_min <0) { 
      if(bar_max >1) { 
        LIST_SET(points,ipt,ex,0,aZ);ipt++;
        LIST_SET(points,ipt,ex,1,aZ);ipt++;
      } else { 
        LIST_SET(points,ipt,ex,0,aZ);ipt++;
        LIST_SET(points,ipt,ex,bar_max,aZ);ipt++;
      }
    } else  if(bar_max>1) { 
      LIST_SET(points,ipt,ex,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex,1,aZ);ipt++;
    } else {
      LIST_SET(points,ipt,ex    ,bar_min,aZ);ipt++;
      LIST_SET(points,ipt,ex    ,bar_max,aZ);ipt++;
    }

    if(ipt<=0) continue;

    for(int i=0;i<ipt;i++) coordIndex[i] = icoord + i;
    coordIndex[ipt] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,ipt,points);
    icoord += ipt;

    lineSet->coordIndex.setValues(iindex,ipt+1,coordIndex);
    iindex += (ipt+1);

    // top of bin :
    ipt = 0;
    LIST_SET(points,ipt,xx,val,aZ);ipt++;
    LIST_SET(points,ipt,xe,val,aZ);ipt++;
    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 1;
    coordIndex[2] = SO_END_LINE_INDEX;

    coordinate3->point.setValues(icoord,ipt,points);
    icoord += ipt;
    lineSet->coordIndex.setValues(iindex,ipt+1,coordIndex);
    iindex += (ipt+1);

    empty = FALSE;
  }          
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repPoints2D_xy(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbVec3f>& aPoints
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,float aZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  int npoint = aPoints.size();

  SbVec3f points[1];

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  for(int index=0;index<npoint;index++) {
    float xx = aPoints[index][0];
    float yy = aPoints[index][1];

    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);

    if((xx>=0)&&(xx<=1)&&(yy>=0)&&(yy<=1)) {
      SoSeparator* separator = new SoSeparator();
      aParent->addChild(separator);
      LIST_SET(points,0,xx,yy,aZ);
      separator->addChild(fStyleCache->getPoints());
      separator->addChild(material);

      SoTools::addPointsToNode(separator,1,points,
        (SbMarkerStyle)aStyle->markerStyle.getValue(),
        aStyle->markerSize.getValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// reps xyz //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xyz_boxes(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aBmin
,float //aBmax
,const SbString& aID
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D\n");

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);
        
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float zz,ze;
    float val = aBins[index].fVal;
    zz = aBmin;
    ze = val;

    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor(val,valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }
                          
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
    zz = VerifyLog(zz,zmin,dz,zlog);
    ze = VerifyLog(ze,zmin,dz,zlog);
                          
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    if(zz>1) continue;
    if(ze<0) continue;
    if(zz<0) zz = 0;
    if(ze>1) ze = 1;
    if(yy>=ye) continue;
    if(xx>=xe) continue;
    if(zz>=ze) continue;

    SoPickSeparator* separator = new SoPickSeparator;
    separator->setName("sceneGraph");
    aParent->addChild(separator);

    char s[128];
    sprintf(s,"%d %d",aBins[index].fI,aBins[index].fJ);
    separator->fInfos = SbSTRDUP(s);

    sprintf(s,"/0x%lx",(unsigned long)separator->fInfos);
    SbString nodeName = aID;
    nodeName += SbString(s);

    SoMaterial* material = 
      fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
    separator->addChild(material);

    separator->addChild(fStyleCache->getFilled());
          
    float sx = xe-xx;
    float sy = ye-yy;
    float sz = ze-zz;
    //FIXME : something better (SoPolyhedron) ?
    SoTransform* transform = new SoTransform;
    transform->translation.setValue(xx+sx/2,yy+sy/2,sz/2);
    separator->addChild(transform);

    SoCube* cube = new SoCube;
    cube->setName(nodeName);
    cube->width.setValue(sx);
    cube->height.setValue(sy);
    cube->depth.setValue(sz);
    separator->addChild(cube);
  }
  
  if(values) delete [] values;
  if(colors) delete [] colors;

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repBins2D_xyz_boxes_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepBin2D>& aBins
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
,float aBmin
,float //aBmax
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repBins2D_one\n");

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                              aStyle->transparency.getValue());
  separator->addChild(material);

  SoNormal* normal = new SoNormal;
  separator->addChild(normal);
  int inormal = 0;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
  separator->addChild(indexedFaceSet);
  int iindex = 0;          

  SbVec3f line[8];
  SbVec3f normals[6];
  int32_t coordIndex[30];
  SbBool empty = TRUE;
  for(unsigned int index=0;index<aBins.size();index++) {

    float xx = aBins[index].fXmin;
    float xe = aBins[index].fXmax;
    float yy = aBins[index].fYmin;
    float ye = aBins[index].fYmax;
    float zz,ze;
   {float val = aBins[index].fVal;
    zz = aBmin;
    ze = val;}

    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
    zz = VerifyLog(zz,zmin,dz,zlog);
    ze = VerifyLog(ze,zmin,dz,zlog);
                          
    // Clipping :
    if(xx>1) continue;
    if(xe<0) continue;
    if(xx<0) xx = 0;
    if(xe>1) xe = 1;
    if(yy>1) continue;
    if(ye<0) continue;
    if(yy<0) yy = 0;
    if(ye>1) ye = 1;

    if(zz>1) continue;
    if(ze<0) continue;
    if(zz<0) zz = 0;
    if(ze>1) ze = 1;
    if(yy>=ye) continue;
    if(xx>=xe) continue;
    if(zz>=ze) continue;

    line[0].setValue(xx,yy,zz);
    line[1].setValue(xe,yy,zz);
    line[2].setValue(xx,ye,zz);
    line[3].setValue(xe,ye,zz);
    line[4].setValue(xx,yy,ze);
    line[5].setValue(xe,yy,ze);
    line[6].setValue(xx,ye,ze);
    line[7].setValue(xe,ye,ze);

    // z back :
    normals[0].setValue(0,0,-1);
    coordIndex[0] = icoord + 0;
    coordIndex[1] = icoord + 2;
    coordIndex[2] = icoord + 3;
    coordIndex[3] = icoord + 1;
    coordIndex[4] = SO_END_FACE_INDEX;
    // y front :
    normals[1].setValue(0,1,0);
    coordIndex[5] = icoord + 2;
    coordIndex[6] = icoord + 6;
    coordIndex[7] = icoord + 7;
    coordIndex[8] = icoord + 3;
    coordIndex[9] = SO_END_FACE_INDEX;
    // x front :
    normals[2].setValue(1,0,0);
    coordIndex[10] = icoord + 1;
    coordIndex[11] = icoord + 3;
    coordIndex[12] = icoord + 7;
    coordIndex[13] = icoord + 5;
    coordIndex[14] = SO_END_FACE_INDEX;
    // y back :
    normals[3].setValue(0,-1,0);
    coordIndex[15] = icoord + 0;
    coordIndex[16] = icoord + 1;
    coordIndex[17] = icoord + 5;
    coordIndex[18] = icoord + 4;
    coordIndex[19] = SO_END_FACE_INDEX;
    // x back :
    normals[4].setValue(-1,0,0);
    coordIndex[20] = icoord + 0;
    coordIndex[21] = icoord + 4;
    coordIndex[22] = icoord + 6;
    coordIndex[23] = icoord + 2;
    coordIndex[24] = SO_END_FACE_INDEX;
    // z front :
    normals[5].setValue(0,0,1);
    coordIndex[25] = icoord + 4;
    coordIndex[26] = icoord + 5;
    coordIndex[27] = icoord + 7;
    coordIndex[28] = icoord + 6;
    coordIndex[29] = SO_END_FACE_INDEX;

    coordinate3->point.setValues(icoord,8,line);
    icoord += 8;

    normal->vector.setValues(inormal,6,normals);
    inormal += 6;

    indexedFaceSet->coordIndex.setValues(iindex,30,coordIndex);
    iindex += 30;
    empty = FALSE;
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repPoints3D_xyz(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbVec3f>& aPoints
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  int npoint = aPoints.size();

  SbVec3f points[1];

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  for(int index=0;index<npoint;index++) {
    float xx = aPoints[index][0];
    float yy = aPoints[index][1];
    float zz = aPoints[index][2];

    xx = VerifyLog(xx,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    zz = VerifyLog(zz,zmin,dz,zlog);

    if( (xx>=0)&&(xx<=1) && 
        (yy>=0)&&(yy<=1) &&
        (zz>=0)&&(zz<=1)
        ) {
      SoSeparator* separator  = new SoSeparator();
      aParent->addChild(separator);
      LIST_SET(points,0,xx,yy,zz);
      separator->addChild(fStyleCache->getPoints());
      separator->addChild(material);
      SoTools::addPointsToNode(separator,1,points,
        (SbMarkerStyle)aStyle->markerStyle.getValue(),
        aStyle->markerSize.getValue());
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repTopFaces2D_xyz(
 SoSeparator* aParent
,SoStyle* aStyle
,SbPaintingPolicy aPainting
,const SbString& aColorMapping
,const std::vector<SbRepTopFace2D>& aTopFaces
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //printf("debug : SoPlotter::repTopFaces2D_xyz\n");

  float* values = 0;
  SbColor* colors = 0;
  int valn = SbStyle::getValueColorMapping(aColorMapping,values,colors);

  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  if(zlog==TRUE) {
    if(valn==1) {
      values[0] = TakeLog(values[0]);
    } else if(valn>=2) {
      //CERN/PAW coloring :
      if(values[0]==0) values[0] = 10e-5F;
      float vmin = TakeLog(values[0]);
      float vmax = TakeLog(values[valn-1]);
      float dv = (vmax-vmin)/(valn-1);
      for(int count=0;count<valn;count++) {
        values[count] = vmin + dv * count;
      }
    }
  }

  SbBool empty = TRUE;
        
  SoSeparator* separator = new SoSeparator();

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFaceIndexed());

  SoNormal* normal = new SoNormal;
  separator->addChild(normal);
  int inormal = 0;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SbVec3f line[3];
  int32_t coordIndex[4];
  int32_t normalIndex[2];
  int number = aTopFaces.size();
  for(int index=0;index<number;index++) {
    float xx = aTopFaces[index].fXmin;
    float xe = aTopFaces[index].fXmax;
    float yy = aTopFaces[index].fYmin;
    float ye = aTopFaces[index].fYmax;
    float val1 = aTopFaces[index].fVal1;
    float val2 = aTopFaces[index].fVal2;
    float val3 = aTopFaces[index].fVal3;
    float val4 = aTopFaces[index].fVal4;

    SbColor sbColor;
    if( (aPainting==SbPaintingByValue) && (valn>0)) {
      sbColor = GetColor((zlog==TRUE?TakeLog(val1):val1),valn,values,colors);
    } else {
      sbColor = aStyle->color.getValue();
    }

    val1 = VerifyLog(val1,zmin,dz,zlog);
    val2 = VerifyLog(val2,zmin,dz,zlog);
    val3 = VerifyLog(val3,zmin,dz,zlog);
    val4 = VerifyLog(val4,zmin,dz,zlog);
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
            
    if(val1<0) val1 = 0;
    if(val1>1) val1 = 1;

    if(val2<0) val2 = 0;
    if(val2>1) val2 = 1;

    if(val3<0) val3 = 0;
    if(val3>1) val3 = 1;

    if(val4<0) val4 = 0;
    if(val4>1) val4 = 1;

    if((xx>=0)&&(xx<=1)   &&
       (xe>=0)&&(xe<=1)   &&
       (yy>=0)&&(yy<=1)   &&
       (ye>=0)&&(ye<=1)  ) {

      SoPickSeparator* sep = new SoPickSeparator;
      sep->setName("sceneGraph");
      separator->addChild(sep);

     /*char s[128];
      sprintf(s,"%d %d",aBins[index].fI,aBins[index].fJ);
      separator->fInfos = SbSTRDUP(s);

      sprintf(s,"/0x%lx",(unsigned long)separator->fInfos);
      SbString nodeName = aID;
      nodeName += SbString(s);*/

      SoMaterial* material = 
        fStyleCache->getMaterial(sbColor,aStyle->transparency.getValue());
      sep->addChild(material);

      SbPlane plane1(SbVec3f(xx,yy,val1),
                     SbVec3f(xe,yy,val2),
                     SbVec3f(xx,ye,val4));
      SbVec3f norm1 = plane1.getNormal();

      line[0].setValue(xx,yy,val1);
      line[1].setValue(xe,yy,val2);
      line[2].setValue(xx,ye,val4);

      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = SO_END_FACE_INDEX;

      normalIndex[0] = inormal;

      coordinate3->point.setValues(icoord,3,line);
      icoord += 3;  

      normal->vector.setValues(inormal,1,&norm1);
      inormal++;  

      SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
      //indexedFaceSet->setName(nodeName);
      indexedFaceSet->coordIndex.setValues(0,4,coordIndex);
      sep->addChild(indexedFaceSet);

      //////////////////////////////////////
      SbPlane plane2(SbVec3f(xe,yy,val2),
                     SbVec3f(xe,ye,val3),
                     SbVec3f(xx,ye,val4));

      SbVec3f norm2 = plane2.getNormal();

      line[0].setValue(xe,yy,val2);
      line[1].setValue(xe,ye,val3);
      line[2].setValue(xx,ye,val4);

      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = SO_END_FACE_INDEX;

      normalIndex[1] = inormal;

      coordinate3->point.setValues(icoord,3,line);
      icoord += 3;  

      normal->vector.setValues(inormal,1,&norm2);
      inormal++;  

      indexedFaceSet->coordIndex.setValues(4,4,coordIndex);
      indexedFaceSet->normalIndex.setValues(0,2,normalIndex);

      empty = FALSE;
    }              
  }
  if(values) delete [] values;
  if(colors) delete [] colors;
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::repTopFaces2D_xyz_one(
 SoSeparator* aParent
,SoStyle* aStyle
,const std::vector<SbRepTopFace2D>& aTopFaces
,const SbRepBox& aBoxX
,const SbRepBox& aBoxY
,const SbRepBox& aBoxZ
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float xmin = aBoxX.fPos;
  float dx = aBoxX.fWidth;
  SbBool xlog = aBoxX.fLog;

  float ymin = aBoxY.fPos;
  float dy = aBoxY.fWidth;
  SbBool ylog = aBoxY.fLog;

  float zmin = aBoxZ.fPos;
  float dz = aBoxZ.fWidth;
  SbBool zlog = aBoxZ.fLog;

  SbBool empty = TRUE;
        
  SoSeparator* separator = new SoSeparator();

  SoMaterial* material = 
    fStyleCache->getMaterial(aStyle->color.getValue(),
                             aStyle->transparency.getValue());
  separator->addChild(material);

  separator->addChild(fStyleCache->getFilled());
  separator->addChild(fStyleCache->getNormalBindingPerFace());
        
  SoNormal* normal = new SoNormal;
  separator->addChild(normal);
  int inormal = 0;

  SoCoordinate3* coordinate3 = new SoCoordinate3;
  separator->addChild(coordinate3);
  int icoord = 0;

  SoIndexedFaceSet* indexedFaceSet = new SoIndexedFaceSet;
  separator->addChild(indexedFaceSet);
  int iindex = 0;

  SbVec3f line[3];
  int32_t coordIndex[4];
  int number = aTopFaces.size();
  for(int index=0;index<number;index++) {
    float xx = aTopFaces[index].fXmin;
    float xe = aTopFaces[index].fXmax;
    float yy = aTopFaces[index].fYmin;
    float ye = aTopFaces[index].fYmax;
    float val1 = aTopFaces[index].fVal1;
    float val2 = aTopFaces[index].fVal2;
    float val3 = aTopFaces[index].fVal3;
    float val4 = aTopFaces[index].fVal4;

    val1 = VerifyLog(val1,zmin,dz,zlog);
    val2 = VerifyLog(val2,zmin,dz,zlog);
    val3 = VerifyLog(val3,zmin,dz,zlog);
    val4 = VerifyLog(val4,zmin,dz,zlog);
    xx = VerifyLog(xx,xmin,dx,xlog);
    xe = VerifyLog(xe,xmin,dx,xlog);
    yy = VerifyLog(yy,ymin,dy,ylog);
    ye = VerifyLog(ye,ymin,dy,ylog);
            
    if(val1<0) val1 = 0;
    if(val1>1) val1 = 1;

    if(val2<0) val2 = 0;
    if(val2>1) val2 = 1;

    if(val3<0) val3 = 0;
    if(val3>1) val3 = 1;

    if(val4<0) val4 = 0;
    if(val4>1) val4 = 1;

    if((xx>=0)&&(xx<=1)   &&
       (xe>=0)&&(xe<=1)   &&
       (yy>=0)&&(yy<=1)   &&
       (ye>=0)&&(ye<=1)  ) {
      SbPlane plane1(SbVec3f(xx,yy,val1),
                     SbVec3f(xe,yy,val2),
                     SbVec3f(xx,ye,val4));
      SbVec3f norm1 = plane1.getNormal();

      line[0].setValue(xx,yy,val1);
      line[1].setValue(xe,yy,val2);
      line[2].setValue(xx,ye,val4);

      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = SO_END_FACE_INDEX;

      coordinate3->point.setValues(icoord,3,line);
      icoord += 3;  

      normal->vector.setValues(inormal,1,&norm1);
      inormal++;  

      indexedFaceSet->coordIndex.setValues(iindex,4,coordIndex);
      iindex += 4;

      SbPlane plane2(SbVec3f(xe,yy,val2),
                     SbVec3f(xe,ye,val3),
                     SbVec3f(xx,ye,val4));

      SbVec3f norm2 = plane2.getNormal();

      line[0].setValue(xe,yy,val2);
      line[1].setValue(xe,ye,val3);
      line[2].setValue(xx,ye,val4);

      coordIndex[0] = icoord + 0;
      coordIndex[1] = icoord + 1;
      coordIndex[2] = icoord + 2;
      coordIndex[3] = SO_END_FACE_INDEX;

      coordinate3->point.setValues(icoord,3,line);
      icoord += 3;  

      normal->vector.setValues(inormal,1,&norm2);
      inormal++;  

      indexedFaceSet->coordIndex.setValues(iindex,4,coordIndex);
      iindex += 4;

      empty = FALSE;
    }              
  }
  if(empty==TRUE) {
    separator->unref();
  } else {
    aParent->addChild(separator);
  }
}
//////////////////////////////////////////////////////////////////////////////
/// Style helpers ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::resetStyles(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float xfac = 1.0F/20.0F; //0.05
  float yfac = 1.0F/20.0F; //0.05

  // Take PAW defaults :
  float XSIZ = 20 * xfac;    //1     //page width
  float YSIZ = 20 * yfac;    //1     //page height
  float XMGL = 2 * xfac;     //0.1   //left x margin (to data frame).
  float XMGR = 2 * xfac;     //0.1   //right y margin (to data frame).
  float YMGL = 2 * yfac;     //0.1   //low y margin (to data frame).
  float YMGU = 2 * yfac;     //0.1   //up y margin (to data frame).
/* Axes :
  float VSIZ = 0.28F * yfac; //0.014 //tick label character size.
  float XVAL = 0.4F * xfac;  //0.02 //x distance of y tick label to data frame.
  float YVAL = 0.4F * yfac;  //0.02 //y distance of x tick label to data frame.
  float XTIC = 0.3F * yfac;  //0.015 //y length of X axis ticks.
  float YTIC = 0.3F * xfac;  //0.015 //x length of Y axis ticks.
  float XLAB = 1.4F * xfac;  //0.07  //x distance of y title to data frame.
  float YLAB = 0.8F * yfac;  //0.04  //y distance of x title to data frame.
  float ASIZ = 0.28F * yfac; //0.014 // axis title (label) character size.
*/
  float YHTI = 1.2F * yfac;  //0.06  //y distance of title to x axis.
  float TSIZ = 0.28F * yfac; //0.014 //title character size

  // Data area :
  //float wData = XSIZ-XMGL-XMGR;
  //float hData = YSIZ-YMGL-YMGU;

  width.setValue(XSIZ);
  height.setValue(YSIZ);
  leftMargin.setValue(XMGL);
  rightMargin.setValue(XMGR);
  bottomMargin.setValue(YMGL);
  topMargin.setValue(YMGU);
  depth.setValue(1);
  titleUp.setValue(TRUE);
  titleToAxis.setValue(YHTI);
  titleHeight.setValue(TSIZ);
  titleAutomated.setValue(TRUE);
  //title.setValue("");
  //superposeBins.setValue(FALSE);
  colorMapping.setValue("black 0 cyan 5 green 10 orange 15 red");
  
  // Is it part of the style ?
  //xAxisLogScale.setValue(FALSE);
  //yAxisLogScale.setValue(FALSE);
  //zAxisLogScale.setValue(FALSE);
  //xAxisAutomated.setValue(TRUE);
  //xAxisMinimum.setValue(0);
  //xAxisMaximum.setValue(1);
  //yAxisAutomated.setValue(TRUE);
  //yAxisMinimum.setValue(0);
  //yAxisMaximum.setValue(1);
  //zAxisAutomated.setValue(TRUE);
  //zAxisMinimum.setValue(0);
  //zAxisMaximum.setValue(1);

  numberOfLevels.setValue(10);
  topAxisVisible.setValue(FALSE);
  rightAxisVisible.setValue(FALSE);

  shapeAutomated.setValue(TRUE);

  setPAW_AxesTextSize(XSIZ,YSIZ);

  setPart("wallStyle",fDefault__wallStyle->copy());
  setPart("gridStyle",fDefault__gridStyle->copy());
  setPart("innerFrameStyle",fDefault__innerFrameStyle->copy());
  setPart("infosStyle",fDefault__infosStyle->copy());
  setPart("titleStyle",fDefault__titleStyle->copy());

  SoGroup* group = 0;
  group = getContainerNode("binsStyle");
  group->removeAllChildren();
  group = getContainerNode("pointsStyle");
  group->removeAllChildren();
  group = getContainerNode("functionStyle");
  group->removeAllChildren();
  group = getContainerNode("rightHatchStyle");
  group->removeAllChildren();
  group = getContainerNode("leftHatchStyle");
  group->removeAllChildren();
  group = getContainerNode("errorsStyle");
  group->removeAllChildren();
  group = getContainerNode("legendStyle");
  group->removeAllChildren();

  // Reset axes ; but not their title which are not automatic.
 {SbString s = xAxisNode->title.getValue();
  xAxisNode->reset();
  xAxisNode->title.setValue(s);}

 {SbString s = yAxisNode->title.getValue();
  yAxisNode->reset();
  yAxisNode->title.setValue(s);}

 {SbString s = zAxisNode->title.getValue();
  zAxisNode->reset();
  zAxisNode->title.setValue(s);}

  f_shape_Update = TRUE;
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesModeling(
 const SbString& aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->modeling.setValue(aValue);
  yAxisNode->modeling.setValue(aValue);
  zAxisNode->modeling.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesColor(
 const SbColor& aColor
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->getLineStyle()->color.setValue(aColor);
  xAxisNode->getTicksStyle()->color.setValue(aColor);
  xAxisNode->getLabelsStyle()->color.setValue(aColor);
  xAxisNode->getTitleStyle()->color.setValue(aColor);
  xAxisNode->getMagStyle()->color.setValue(aColor);

  yAxisNode->getLineStyle()->color.setValue(aColor);
  yAxisNode->getTicksStyle()->color.setValue(aColor);
  yAxisNode->getLabelsStyle()->color.setValue(aColor);
  yAxisNode->getTitleStyle()->color.setValue(aColor);
  yAxisNode->getMagStyle()->color.setValue(aColor);

  zAxisNode->getLineStyle()->color.setValue(aColor);
  zAxisNode->getTicksStyle()->color.setValue(aColor);
  zAxisNode->getLabelsStyle()->color.setValue(aColor);
  zAxisNode->getTitleStyle()->color.setValue(aColor);
  zAxisNode->getMagStyle()->color.setValue(aColor);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesTextScale(
 float aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->getLabelsStyle()->scale.setValue(aValue);
  xAxisNode->getTitleStyle()->scale.setValue(aValue);
  xAxisNode->getMagStyle()->scale.setValue(aValue);

  yAxisNode->getLabelsStyle()->scale.setValue(aValue);
  yAxisNode->getTitleStyle()->scale.setValue(aValue);
  yAxisNode->getMagStyle()->scale.setValue(aValue);

  zAxisNode->getLabelsStyle()->scale.setValue(aValue);
  zAxisNode->getTitleStyle()->scale.setValue(aValue);
  zAxisNode->getMagStyle()->scale.setValue(aValue);

}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesLinePattern(
 unsigned short aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->getLineStyle()->pattern.setValue(aValue);
  yAxisNode->getLineStyle()->pattern.setValue(aValue);
  zAxisNode->getLineStyle()->pattern.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesLineWidth(
 int aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->getLineStyle()->width.setValue(aValue);
  yAxisNode->getLineStyle()->width.setValue(aValue);
  zAxisNode->getLineStyle()->width.setValue(aValue);

  xAxisNode->getTicksStyle()->width.setValue(aValue);
  yAxisNode->getTicksStyle()->width.setValue(aValue);
  zAxisNode->getTicksStyle()->width.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesTickLength(
 float aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->tickLength.setValue(aValue);
  yAxisNode->tickLength.setValue(aValue);
  zAxisNode->tickLength.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesTitleHeight(
 float aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->titleHeight.setValue(aValue);
  yAxisNode->titleHeight.setValue(aValue);
  zAxisNode->titleHeight.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setAxesLabelHeight(
 float aValue
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  xAxisNode->labelHeight.setValue(aValue);
  yAxisNode->labelHeight.setValue(aValue);
  zAxisNode->labelHeight.setValue(aValue);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setFont(
 const SbString& aFont
,SbBool aSmoothing
,SbBool aHinting
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {SoTextStyle* style = getXAxis()->getLabelsStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}
 {SoTextStyle* style = getXAxis()->getTitleStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}

 {SoTextStyle* style = getYAxis()->getLabelsStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}
 {SoTextStyle* style = getYAxis()->getTitleStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}

 {SoTextStyle* style = getZAxis()->getLabelsStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}
 {SoTextStyle* style = getZAxis()->getTitleStyle();
  style->fontName.setValue(aFont);
  style->smoothing.setValue(aSmoothing);
  style->hinting.setValue(aHinting);}

  getTitleStyle()->fontName.setValue(aFont);
  getTitleStyle()->smoothing.setValue(aSmoothing);
  getTitleStyle()->hinting.setValue(aHinting);

  //plotter->get_infosStyle()->fontName.setValue(aFont);
  //plotter->get_infosStyle()->smoothing.setValue(aSmoothing);
  //plotter->get_infosStyle()->hinting.setValue(aHinting);
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::setPAW_AxesTextSize(
 float aXSIZ
,float aYSIZ
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float xfac = aXSIZ/20.0F;
  float yfac = aYSIZ/20.0F;

  // Take PAW defaults :
  float VSIZ = 0.28F * yfac; //0.014 //tick label character size.
  float XVAL = 0.4F * xfac;  //0.02 //x distance of y tick label to data frame.
  float YVAL = 0.4F * yfac;  //0.02 //y distance of x tick label to data frame.
  float XTIC = 0.3F * yfac;  //0.015 //y length of X axis ticks.
  float YTIC = 0.3F * xfac;  //0.015 //x length of Y axis ticks.
  float XLAB = 1.4F * xfac;  //0.07  //x distance of y title to data frame.
  float YLAB = 0.8F * yfac;  //0.04  //y distance of x title to data frame.
  float ASIZ = 0.28F * yfac; //0.014 // axis title (label) character size.
  //float YHTI = 1.2F * yfac;  //0.06  //y distance of title to x axis.
  //float TSIZ = 0.28F * yfac; //0.014 //title character size

  // Initialize axes :
  if(fShape==XY) {
    xAxisNode->tickLength.setValue(XTIC);
    xAxisNode->labelToAxis.setValue(YVAL);
    xAxisNode->labelHeight.setValue(VSIZ);
    xAxisNode->titleToAxis.setValue(YLAB);
    xAxisNode->titleHeight.setValue(ASIZ);

    yAxisNode->tickLength.setValue(YTIC);
    yAxisNode->labelToAxis.setValue(XVAL);
    yAxisNode->labelHeight.setValue(VSIZ);
    yAxisNode->titleToAxis.setValue(XLAB);
    yAxisNode->titleHeight.setValue(ASIZ);
  } else { //XYZ    
    xAxisNode->tickLength.setValue(XTIC);
    xAxisNode->labelToAxis.setValue(YVAL);
    xAxisNode->labelHeight.setValue(VSIZ);
    xAxisNode->titleToAxis.setValue(YLAB);
    xAxisNode->titleHeight.setValue(ASIZ);
  
    yAxisNode->tickLength.setValue(XTIC);
    yAxisNode->labelToAxis.setValue(YVAL);
    yAxisNode->labelHeight.setValue(VSIZ);
    yAxisNode->titleToAxis.setValue(YLAB);
    yAxisNode->titleHeight.setValue(ASIZ);
  
    zAxisNode->tickLength.setValue(YTIC);
    zAxisNode->labelToAxis.setValue(XVAL);
    zAxisNode->labelHeight.setValue(VSIZ);
    zAxisNode->titleToAxis.setValue(XLAB);
    zAxisNode->titleHeight.setValue(ASIZ);
  }  

}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
float TakeLog(
 float aVal  
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aVal<=0) {
    return -FLT_MAX;
  } else {
    return FLOG10(aVal);
  }
}
//////////////////////////////////////////////////////////////////////////////
float VerifyLog(
 float aVal  
,float aMin  
,float aDx   
,SbBool aLog  
)
//////////////////////////////////////////////////////////////////////////////
// Use FLT_MAX ; -HUGE_VAL does not work on some system (exa: ULTRIX).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 if(aLog==TRUE) {
    if(aVal>0.0F) {
      return (FLOG10(aVal) - aMin)/aDx;
    } else { // Return a negative large number :
      //return -FLT_MAX; 
      return -100;
    }
  } else {
    // Simple protection against value that could exceed a float :
    if(aVal>(aMin+100.F * aDx)) return 100; 
    if(aVal<aMin-100.F * aDx) return -100;
    // Rescale :
    return (aVal - aMin)/aDx;
  }
}
//////////////////////////////////////////////////////////////////////////////
SbColor GetColor(
 float aValue
,int aValuen
,float* aValues
,SbColor* aColors
)
//////////////////////////////////////////////////////////////////////////////
// There are aValuen (n) entries in aValues and (n+1) aColors
//  aColors[0] aValues[0] aColors[1] aValues[1]...
//                 aValues[n-2] aColors[n-1] aValues[n-1] aColors[n]
//      black 10 cyan 50 green 100 orange 200 blue 300 pink 400 red
// valuen = 6
// values[0] 10 
// values[1] 50 
// values[2] 100 
// values[3] 200 
// values[4] 300 
// values[5] 400 
//
// colors[0] black
// colors[1] cyan
// colors[2] green
// colors[3] orange
// colors[4] blue
// colors[5] pink
// colors[6] red
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aValuen<=0) return SbColor_black;
  if(aValue<aValues[0]) {
    return aColors[0];
  } else {
    for(int count=0;count<=(aValuen-2);count++) {
      if( (aValues[count]<=aValue) && (aValue<aValues[count+1]) ) {
        return aColors[count+1];
      }
    }
    return aColors[aValuen];
  }
}
//////////////////////////////////////////////////////////////////////////////
void ReverseList (
 int aNumber
,SbVec3f* aList 
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int end = aNumber - 1;
  int half = aNumber/2;
  SbVec3f tmp;
  int index;
  for(int count=0;count<half;count++) { 
    index = end - count;
    tmp = aList[count];
    aList[count] = aList[index];
    aList[index] = tmp;
  }
}
//////////////////////////////////////////////////////////////////////////////
void getHeight(
 int aBinsi
,const SbPList& aBinss
,const SbPList& aBinssSwMnMx //(bim,bmax) of each SbPlottableBins1D
,int aIndex                  //bin of the aBinsi SbPlottableBins1D
,float& aMin
,float& aMax
)
//////////////////////////////////////////////////////////////////////////////
// Get total height of a bin when superposing bins.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aMin = 0;
  aMax = ((SbPlottableBins1D*)aBinss[0])->getBinSumOfWeights(aIndex);
  for(int ibins=1;ibins<=aBinsi;ibins++) {
    float bmin = (*((SbVec2f*)aBinssSwMnMx[ibins]))[0];
    float delta = 
     ((SbPlottableBins1D*)aBinss[ibins])->getBinSumOfWeights(aIndex) - bmin;
    aMin = aMax; //previous max.
    aMax += delta;
  }   
}
//////////////////////////////////////////////////////////////////////////////
void BarChart(
 float aBarOffset
,float aBarWidth
,float& aBegin
,float& aEnd
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  float xe = (aEnd - aBegin)*aBarOffset;
  float xw = (aEnd - aBegin)*aBarWidth;
  aEnd = aBegin + xe + xw;
  aBegin = aBegin + xe;
}
//////////////////////////////////////////////////////////////////////////////
float bins2DToFunc(
 const SbPlottableBins2D& aBins
,float aX
,float aY
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  int xn = aBins.getAxisNumberOfBinsX();
  float xmn = aBins.getAxisMinimumX();
  float xmx = aBins.getAxisMaximumX();
  int yn = aBins.getAxisNumberOfBinsY();
  float ymn = aBins.getAxisMinimumY();
  float ymx = aBins.getAxisMaximumY();

  float dx = (xmx-xmn)/xn;
  float dy = (ymx-ymn)/yn;
  int ibin = (int)((aX-xmn)/dx);
  int jbin = (int)((aY-ymn)/dy);

  float xx_0 = aBins.getBinLowerEdgeX(ibin);
  //float xe_0 = aBins.getBinUpperEdgeX(ibin);
  float xx_1 = aBins.getBinLowerEdgeX(ibin+1);
  //float xe_1 = aBins.getBinUpperEdgeX(ibin+1);

  float yy_0 = aBins.getBinLowerEdgeY(jbin);
  //float ye_0 = aBins.getBinUpperEdgeY(jbin);
  float yy_1 = aBins.getBinLowerEdgeY(jbin+1);
  //float ye_1 = aBins.getBinUpperEdgeY(jbin+1);

  float val1 = aBins.getBinSumOfWeights(ibin,jbin);
  float val2 = aBins.getBinSumOfWeights(ibin+1,jbin);
  //float val3 = aBins.getBinSumOfWeights(ibin+1,jbin+1);
  float val4 = aBins.getBinSumOfWeights(ibin,jbin+1);

  // Interpolate :
  SbVec3f p1(xx_0,yy_0,val1);
  SbVec3f p2(xx_1,yy_0,val2);
  //SbVec3f p3(xx_1,yy_1,val3);
  SbVec3f p4(xx_0,yy_1,val4);

  //FIXME : case of (x,y) in (p2,p3,p4)

  SbPlane plane(p1,p2,p4);
  SbVec3f pt;
  SbLine line(SbVec3f(aX,aY,0),SbVec3f(aX,aY,10));
  plane.intersect(line,pt);

  return pt[2];
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::dataFrame2vp(
 const SbVec3f& aPos
,SbVec3f& aVP
) const
//////////////////////////////////////////////////////////////////////////////
// aPos is in data frame NDC coordinates.
// aVP is in viewport/screen coordinates (in [0,1]).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;
  if(XSIZ==0.0F) return;
  aVP[0] = (wData*aPos[0] + XMGL)/XSIZ;}

 {float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;
  if(YSIZ==0.0F) return;
  aVP[1] = (hData*aPos[1] + YMGL)/YSIZ;}

 {float ZSIZ = depth.getValue();
  float ZMGD = downMargin.getValue();
  float ZMGU = upMargin.getValue();
  float dData = ZSIZ-ZMGD-ZMGU;
  if(ZSIZ==0.0F) return;
  aVP[2] = (dData*aPos[2] + ZMGD)/ZSIZ;}
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::axis2vp(
 const SbVec3f& aPos
,SbVec3f& aVP
) const
//////////////////////////////////////////////////////////////////////////////
// aPos is in axes coordinates.
// aVP is in viewport/screen coordinates (in [0,1]).
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Assume that axes min,max,logScale are up to date.

  SbVec3f d; // In data area coodinate. In [0,1][0,1][0,1].
 {float mn = xAxisNode->minimumValue.getValue();
  float mx = xAxisNode->maximumValue.getValue();
  if(mx==mn) return;
  SbBool lg = xAxisNode->logScale.getValue();
  d[0] = VerifyLog(aPos[0],mn,mx-mn,lg);}

 {float mn = yAxisNode->minimumValue.getValue();
  float mx = yAxisNode->maximumValue.getValue();
  if(mx==mn) return;
  SbBool lg = yAxisNode->logScale.getValue();
  d[1] = VerifyLog(aPos[1],mn,mx-mn,lg);}

 {float mn = zAxisNode->minimumValue.getValue();
  float mx = zAxisNode->maximumValue.getValue();
  if(mx==mn) return;
  SbBool lg = zAxisNode->logScale.getValue();
  d[2] = VerifyLog(aPos[2],mn,mx-mn,lg);}

 {float XSIZ = width.getValue();
  float XMGL = leftMargin.getValue();
  float XMGR = rightMargin.getValue();
  float wData = XSIZ-XMGL-XMGR;
  if(XSIZ==0.0F) return;
  aVP[0] = (wData*d[0] + XMGL)/XSIZ;}

 {float YSIZ = height.getValue();
  float YMGL = bottomMargin.getValue();
  float YMGU = topMargin.getValue();
  float hData = YSIZ-YMGL-YMGU;
  if(YSIZ==0.0F) return;
  aVP[1] = (hData*d[1] + YMGL)/YSIZ;}

 {float ZSIZ = depth.getValue();
  float ZMGD = downMargin.getValue();
  float ZMGU = upMargin.getValue();
  float dData = ZSIZ-ZMGD-ZMGU;
  if(ZSIZ==0.0F) return;
  aVP[2] = (dData*d[2] + ZMGD)/ZSIZ;}
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::generateAlternateRep(SoAction*) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  alternateRep.setValue(topSeparator.getValue());
}
//////////////////////////////////////////////////////////////////////////////
void SoPlotter::clearAlternateRep(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  alternateRep.setValue(NULL);
}
