// this :
#include <obuild/Builder.h>

// Slash :
#include <Slash/Core/IFileGetter.h>

// Lib :
#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/sout.h>
#include <Lib/FileWriter.h>
#include <Lib/smanip.h>
#include <Lib/cmanip.h>
#include <Lib/fmanip.h>
#include <Lib/dirmanip.h>
#include <Lib/Element.h>
#include <Lib/ItemML.h>
#include <Lib/BaseML.h>
#include <Lib/vmanip.h>
#include <Lib/Arguments.h>
#include <Lib/System.h>

// this :
#include <obuild/Version.h>

// templates, embedded code :
#include <obuild/tools_obuild.h>
#include <obuild/obuild_find_cpp.h>
#include <obuild/obuild_platform_cpp.h>
#include <obuild/obuild_windef_c.h>
#include <obuild/obuild_subs_cpp.h>
#include <obuild/obuild_subf_cpp.h>

#include <obuild/build_find_sh.h>
#include <obuild/build_find_csh.h>
#include <obuild/build_find_bat.h>
#include <obuild/build_subs_bat.h>
#include <obuild/build_subs_sh.h>
#include <obuild/build_subf_bat.h>
#include <obuild/build_subf_sh.h>

#include <obuild/build_platform_sh.h>
#include <obuild/build_platform_csh.h>
#include <obuild/build_platform_bat.h>

#include <obuild/build_xxx_header_sh.h>
#include <obuild/build_xxx_header_bat.h>

#include <obuild/checkout_release_header_sh.h>
#include <obuild/checkout_release_header_bat.h>

#include <obuild/build_header_sh.h>
#include <obuild/build_header_bat.h>

#include <obuild/build_project_header_sh.h>
#include <obuild/build_project_header_bat.h>

#include <obuild/build_Darwin_app_sh.h>
#include <obuild/build_sys_sh.h>
#include <obuild/build_shlib_sh.h>

#include <obuild/run_header_sh.h>
#include <obuild/run_header_bat.h>

#include <obuild/build_release_header_sh.h>
#include <obuild/build_release_header_bat.h>

#include <obuild/setup_header_sh.h>
#include <obuild/setup_header_csh.h>
#include <obuild/setup_header_bat.h>

#include <obuild/setup_trailer_sh.h>
#include <obuild/setup_trailer_csh.h>
#include <obuild/setup_trailer_bat.h>

#include <obuild/broadcast_install_header_sh.h>

#include <obuild/check_foreigns_header_sh.h>
#include <obuild/check_app_sh.h>

#include <obuild/header_iss.h>
#include <obuild/trailer_iss.h>

#include <obuild/install_sh.h>

// include code of obuild_platform to get the code 
// of evaluate_expression used in obuild::Builder::build_on_platform.
#define main obuild_platform_main
#include "../templates/obuild_platform.cpp"

#define BUILD_ON_PLATFORM_FP \
   {bool build_yes;\
    if(!build_on_platform(platform,build_yes)) {\
      fp.disable();\
      Lib::fmanip::remove(sfile);\
      return false;\
    }\
    if(!build_yes) continue;}

#define BUILD_ON_PLATFORM \
   {bool build_yes;\
    if(!build_on_platform(platform,build_yes)) return false;\
    if(!build_yes) continue;}

namespace obuild {

class Visitor : public Slash::Core::IDirectoryVisitor {
public:
  virtual bool directory(const std::string& aString,bool& aProcess) {
    aProcess = true;
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(aString,p,n,s);
    if(n=="CVS") aProcess = false;
    if(n==".svn") aProcess = false;
    return true;
  }
  virtual bool file(const std::string& aString) {
    if(fBuilder.handled(aString)) {
      fFiles.push_back(aString);
      fPlatforms.push_back(fPlatform);
    }
    return true;
  }
public:
  Visitor(Builder& aBuilder,
          std::vector<std::string>& aFiles,
          std::vector<std::string>& aPlatforms,
          const std::string& aPlatform)
  :fBuilder(aBuilder)
  ,fFiles(aFiles)
  ,fPlatforms(aPlatforms)
  ,fPlatform(aPlatform)
  {}
  virtual ~Visitor() {};
private:
  Builder& fBuilder;
  std::vector<std::string>& fFiles;
  std::vector<std::string>& fPlatforms;
  std::string fPlatform;
};

class FileGetter : public virtual Slash::Core::IFileGetter {
public: //Slash::Core::IFileGetter:
  virtual bool fetch(const std::string& aFileName,
                     const Slash::Core::IFileGetter::Arguments& aArgs,
                     std::string& aNewName) {
    Lib::Arguments args(aArgs);
    std::string package;
    if(args.find("package",package)) {
      std::string version;
      args.find("version",version);

      std::string opath;
      if(!Lib::System::getenv("OBUILD_PATH",opath)) {
        Lib::Out out(fPrinter);
        out << "obuild::FileGetter::fetch :"
            << " environment variable OBUILD_PATH not defined."
            << Lib::endl
            << " OBUILD_PATH sould be a ';' separated list"
            << " of head directories containing packages."
            << Lib::endl;
        return false;
      }
      std::string sep = Lib::System::pathSeparator();
      std::vector<std::string> paths = Lib::smanip::words(opath,sep);
      
      std::string path,serr;
      if(!Lib::dirmanip::find(paths,package,version,path,serr)) {
        Lib::Out out(fPrinter);
        out << "obuild::FileGetter::fetch :"
            << " " << serr 
            << Lib::endl;
        return false;
      }
      if(path=="") {
        Lib::Out out(fPrinter);
        out << "obuild::FileGetter::fetch :"
            << Lib::endl
            << " file " << Lib::sout(aFileName)
            << Lib::endl
            << " not found by searching package " << Lib::sout(package)
            << " with version " << Lib::sout(version)
            << Lib::endl
            << " under the paths given by the environment" 
            << Lib::endl
            << " variable OBUILD_PATH having value :" 
            << Lib::endl
            << Lib::sout(opath)
            << Lib::endl;
        return false;
      }

      //printf("debug : fetch %s in package %s\n",
        //aFileName.c_str(),package.c_str());

      aNewName = path+"/"+aFileName;
    } else {
      aNewName = aFileName;
    }
    if(fQuery) {
      Lib::Out out(fPrinter);
      out << " reference :" 
          << " " << aNewName
          << Lib::endl;
    }
    return true;  
  }  
public:
  FileGetter(Slash::Core::IWriter& aPrinter)
  :fPrinter(aPrinter),fQuery(false){}
  virtual ~FileGetter(){}
  void setQuery(bool aValue) { fQuery = aValue;}
private:
  Slash::Core::IWriter& fPrinter;
  bool fQuery;
};

}

//////////////////////////////////////////////////////////////////////////////
obuild::Builder::Builder(
 Slash::Core::IWriter& aPrinter
)
:fPrinter(aPrinter)
,fTopML(0)
,fPackML(0)
,fProduced_setups(false)
,fProduced_platform_exe(false)
,fProduced_find_exe(false)
,fProduced_windef_exe(false)
,fProduced_subs_exe(false)
,fProduced_subf_exe(false)
,fProduced_Darwin_app(false)
,fProduced_build_sys_sh(false)
,fProduced_build_shlib_sh(false)
,fProduced_check_app_sh(false)
,fProduced_local_what("xxx")
,fProduced_local_platform_exe(false)
,fProduced_local_find_exe(false)
,fProduced_local_windef_exe(false)
,fProduced_local_subs_exe(false)
,fProduced_local_subf_exe(false)
,fProduced_local_LD_PATH(false)
,fProduce_local_exit_sh("exit")
,fProduced_local_pwd_count(0)
,fProduced_local_group_count(0)
,fProduced_local_group_begin(false)
,fProduced_local_if_count(0)
,fProduced_local_if_begin(false)
,fProduced_local_main_count(0)
,fProduced_local_main_begin(false)
,fProdWhat(build_sh)
,f_sh("")
,f_dir("sh")
,f_cmt("#")
,fComponent("")
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // We can produce bat file on a UNIX !
  f_sep = Lib::System::fileSeparator();
}
//////////////////////////////////////////////////////////////////////////////
/// build methods : //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::build(
 const std::string& aFile
,eWhat aProdWhat
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fProdWhat = aProdWhat;

  if(fProdWhat==obuild::Builder::build_sh) {
    //f_sh = ".sh";
    f_sh = ""; //UNIX has no extension for not sourced shell scripts.
    f_dir = "sh";
    f_cmt = "#";
    f_exe = ""; //UNIX has no extension for application.
  } else if(fProdWhat==obuild::Builder::build_bat) {
    f_sh = ".bat";
    f_dir = "DOS";
    f_cmt = "REM #";
    f_exe = ".exe";
  }
  // We can produce bat file on a UNIX !
  f_sep = Lib::System::fileSeparator();

  if(aFile=="") {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build :"
        << " no obuild file to work with."
        << Lib::endl;
    return false;
  }

  int verboseLevel = 0;
  Lib::ItemMLFactory factory;
  Lib::BaseML ml(factory,verboseLevel);
  ml.setTags(tags());
  FileGetter fg(fPrinter);
  ml.setFileGetter(&fg);

  if(!ml.loadFile(aFile,false,fPrinter)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build :"
        << " problem when loading file " << Lib::sout(aFile) 
        << Lib::endl;    
    return false;
  }

  Lib::ItemML* top = ml.topItem();
  if(!top) {
    // Empty file : do nothing.
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build :"
        << " file " << Lib::sout(aFile) << " is empty."
        << Lib::endl;
    return false;
  } 

  if(!top->hasEmptyAttributeValue(fPrinter)) return false;

  if(!handle_tools(*top)) return false;

  fTopML = top; //Need top because of templates and tools.

  // Reset global production flags (for build_pack_else) :
  fProduced_setups = false;
  fProduced_platform_exe = false;
  fProduced_find_exe = false;
  fProduced_windef_exe = false;
  fProduced_subs_exe = false;
  fProduced_subf_exe = false;
  fProduced_Darwin_app = false;
  fProduced_build_sys_sh = false;
  fProduced_build_shlib_sh = false;
  fProduced_check_app_sh = false;

  Lib::ItemML* projectML = 0;
 {const std::list<Lib::ItemML*>& list = top->children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    const std::string& tagName = (*it)->tagName();
    if(tagName=="project") {   
      projectML = *it;
      break;
    }
  }}
  if(projectML) {
    if(!out_build_project_sh(*projectML)) {
      std::string name;
      projectML->attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build :"
          << " production of script failure for project" 
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }
    if(!out_build_release_sh(*projectML)) {
      std::string name;
      projectML->attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build :"
          << " production of script failure for project" 
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }
    if(!out_checkout_release_sh(*projectML)) {
      std::string name;
      projectML->attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build :"
          << " production of checkout release script failure for project" 
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }
    if(!out_iss(*projectML)) {
      std::string name;
      projectML->attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build :"
          << " production of .iss failure for project" 
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }

  } else {

    fPackML = 0;
   {const std::list<Lib::ItemML*>& list = top->children();
    std::list<Lib::ItemML*>::const_iterator it;
    for(it=list.begin();it!=list.end();++it) {
      const std::string& tagName = (*it)->tagName();
      if(tagName=="package") {   
        fPackML = *it;
        break;
      }
    }}
    if(fPackML) { if(!build_pack(*fPackML)) return false; }
    fPackML = 0;

  }

  fTopML = 0;

  if(!build_pack_else()) return false;

  if(!Lib::dirmanip::removeIfEmpty("cpp")) return false;

  if(!Lib::dirmanip::removeIfEmpty(f_dir)) return false;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::build_pack(
 Lib::ItemML& aPackML //<package>
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string package;
  std::string version;
  if(!package_and_version(package,version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :"
        << " can't get package and version." 
        << Lib::endl;
    return false;
  }

 {std::string name;
  if(!aPackML.attributeValue("name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :"
        << " no attribute name on the <package> XML item."
        << Lib::endl;
    return false;
  }
  if(name!=package) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :"
        << " package name mismatch :"
        << " " << Lib::sout(name) << " found in the <package> XML item,"
        << " whilst your are under the " << Lib::sout(package) 
        << " directory."
        << Lib::endl;
    return false;
  }

  if(version!="") {
    std::string atb_version;
    if(!aPackML.attributeValue("version",atb_version)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack :"
          << " no attribute version on the <package> XML item."
          << Lib::endl;
      return false;
    }
    if(atb_version!=version) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack :"
          << " package version mismatch :"
          << " " << Lib::sout(atb_version) 
          << " found in the <package> XML item,"
          << " whilst your are under the " 
          << Lib::sout(version) << " directory."
          << Lib::endl;
      return false;
    }
  } else {
    std::string atb_version;
    if(aPackML.attributeValue("version",atb_version)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack :"
          << " attribute version on the <package> XML item"
          << " but we are not undera pack/version/obuild tree."
          << Lib::endl;
      return false;
    }
  }}

  // First remove existing materials :
  if(!cleanup()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :"
        << " problem removing some existing build files." 
        << Lib::endl;
    return false;
  }

  if(!check_name(package)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :"
        << " problem with " << Lib::sout(package)
        << ". Package name should have the syntax of a programming variable." 
        << Lib::endl;
    return false;
  }

  if(!check_env(aPackML)) return false;

  fPackage = package;
  fVersion = version; // could be empty.
  fCopyRight = "";
  fPublisher = "";
  fURL = "";

  if(!out_check_foreigns_sh(aPackML)) {
    cleanup();
    return false;
  }

  // Instrument "uses" with "package" so that <path from="xxx"/>
  // found in components works properly. Must be done first.
 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aPackML,"use",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);
    if(!handle_use(itemML)) { // Does not induce production of code.
      std::string name;
      itemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build :" 
          << " failure for " 
          << Lib::sout(itemML.tagName())
          << " " << Lib::sout(name)
          << Lib::endl;
      cleanup();
      return false;
    }    
  }

  // Other components :
  if(!doIt(aPackML)) {
    cleanup();
    return false;
  }

  // Produce global build script.
  // Should be done after the components
  // because of the <not_in_main_build>.
  if(!out_build_sh(aPackML)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :" 
        << " production of build" << f_sh << " failed." 
        << Lib::endl;
    cleanup();
    return false;
  }

  if(!out_run_sh(aPackML)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_pack :" 
        << " production of run" << f_sh << " failed." 
        << Lib::endl;
    cleanup();
    return false;
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::build_pack_else(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // If needed, produce setups, and per component coworking programs :
  if(fProduced_setups) {
    if(!check_directory(f_dir)) {
      cleanup();
      return false;
    }
    if(!out_install_sh()) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of install" << f_sh << " failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_platform_exe) {
    if(!check_directory("cpp")) {
      cleanup();
      return false;
    }
    if(!out_oxxx_cpp("./cpp/obuild_platform.cpp",
          obuild_platform_cpp_linen,obuild_platform_cpp_lines)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of obuild_platform.cpp failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_find_exe) {
    if(!check_directory("cpp")) {
      cleanup();
      return false;
    }
    if(!out_oxxx_cpp("./cpp/obuild_find.cpp",
          obuild_find_cpp_linen,obuild_find_cpp_lines)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of obuild_find.cpp failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_windef_exe) {
    if(!check_directory("cpp")) {
      cleanup();
      return false;
    }
    if(!out_oxxx_cpp("./cpp/obuild_windef.c",
          obuild_windef_c_linen,obuild_windef_c_lines)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of obuild_windef.c failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_subs_exe) {
    if(!check_directory("cpp")) {
      cleanup();
      return false;
    }
    if(!out_oxxx_cpp("./cpp/obuild_subs.cpp",
          obuild_subs_cpp_linen,obuild_subs_cpp_lines)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of obuild_subs.cpp failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }


  if(fProduced_subf_exe) {
    if(!check_directory("cpp")) {
      cleanup();
      return false;
    }
    if(!out_oxxx_cpp("./cpp/obuild_subf.cpp",
          obuild_subf_cpp_linen,obuild_subf_cpp_lines)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of obuild_subf.cpp failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_Darwin_app) {
    if(!check_directory(f_dir)) {
      cleanup();
      return false;
    }
    if(!out_build_Darwin_app_sh()) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of build_Darwin_app failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_build_sys_sh) {
    if(!check_directory(f_dir)) {
      cleanup();
      return false;
    }
    if(!out_build_sys_sh()) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of build_sys failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_build_shlib_sh) {
    if(!check_directory(f_dir)) {
      cleanup();
      return false;
    }
    if(!out_build_shlib_sh()) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of build_shlib failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  if(fProduced_check_app_sh) {
    if(!check_directory(f_dir)) {
      cleanup();
      return false;
    }
    if(!out_check_app_sh()) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::build_pack_else :"
          << " production of check_app failed." 
          << Lib::endl;
      cleanup();
      return false;
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::doIt(
 Lib::ItemML& aItemML //<package>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  const std::string& tagName = aItemML.tagName();
  if(tagName=="package") {   
  } else if(tagName=="use") {   
  } else if(tagName=="version") {   

    if(!handle_version(aItemML)) {
      std::string name;
      aItemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " failure for " 
          << Lib::sout(tagName)
          << "."
          << Lib::endl;
      return false;
    }

  } else if(tagName=="library") {   
    if(!handle_template(aItemML)) return false;
    if(!out_build_library_sh(aItemML)) {
      std::string name;
      aItemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of script failure for " 
          << Lib::sout(tagName)
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }

  } else if(tagName=="dll") {   
    if(!handle_template(aItemML)) return false;
    if(!out_build_dll_sh(aItemML)) {
      std::string name;
      aItemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of script failure for " 
          << Lib::sout(tagName)
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }

  } else if(tagName=="application") {   
    if(!handle_template(aItemML)) return false;
    if(!out_build_application_sh(aItemML)) {
      std::string name;
      aItemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of script failure for " 
          << Lib::sout(tagName)
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }

  } else if(tagName=="darwin_app") {   
    if(!handle_template(aItemML)) return false;
    if(!out_build_darwin_app_sh(aItemML)) {
      std::string name;
      aItemML.attributeValue("name",name);
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of script failure for " 
          << Lib::sout(tagName)
          << " " << Lib::sout(name)
          << Lib::endl;
      return false;
    }

  } else if(tagName=="setup") {   

    if(!out_setup_sh(aItemML)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of setup.sh failed." 
          << Lib::endl;
      return false;
    }

    if(!out_setup_csh(aItemML)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of setup.csh failed." 
          << Lib::endl;
      return false;
    }

    if(!out_setup_bat(aItemML)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of setup.bat failed." 
          << Lib::endl;
      return false;
    }

    if(!out_broadcast_install_sh(aItemML)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::doIt :" 
          << " production of broadcast_install" << f_sh << " failed." 
          << Lib::endl;
      return false;
    }

  }

  // Go down children :
  const std::list<Lib::ItemML*>& list = aItemML.children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if(!doIt(*(*it))) return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
/// Component producer methods. //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_library_sh(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {std::string platform;
  if(fProdWhat==obuild::Builder::build_sh) {
    platform = "UNIX";
  } else {
    platform = "Windows_NT";
  }
  bool doit;
  if(!done_on(aItemML,platform,doit)) return false;
  if(!doit) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true;
  }}

  std::vector<std::string> files;
  std::vector<std::string> platforms;
  if(!getFiles(aItemML,files,platforms)) return false;

  if(!files.size()) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true; // In fact nothing to do ! Ok.
  }

  ///////////////////////////////////////////////
  /// It seems that there is something to do ////
  /// for this component.                    ////
  ///////////////////////////////////////////////

  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_library_sh :"
        << " name attribute not found for library." 
        << Lib::endl;
    return false;
  }
  fComponent = name;

  std::string kind_UNIX = "shared";
  std::string kind_Windows_NT = "shared";
 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"kind",packs,verss,platforms,
                                   verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& platform = platforms[index]; //"platform"
    const std::string& value = vals[index];
    if(platform=="Windows_NT") {
      kind_Windows_NT = value;
    } else if(platform=="UNIX") {
      kind_UNIX = value;
    } else if(platform=="") {
      kind_Windows_NT = value;
      kind_UNIX = value;
    } else {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_library_sh :" 
          << " for library " << Lib::sout(name)
          << ", only platform=\"UNIX\" or \"Windows_NT\" supported"
          << " in <kind> property."
          << Lib::endl;
      return false;
    }
  }}

  //////////////////////
  /// Produce code : ///
  //////////////////////
  if(!check_directory(f_dir)) return false;
  std::string sfile = "."+f_sep+f_dir+f_sep+"build_lib_"+name+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_library_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  if(!out_build_xxx_core_sh(aItemML,out,"lib",name,files,platforms)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  // Specific to libraries :

  std::string lang;
  if(!guess_linker_language(aItemML,files,lang)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(lang=="java") {

    out << cmt() << " Create the archive library " << name << " :" 
        << Lib::endl;

    out_begin_pwd(out);

    std::string dir_bin_xxx = dir_bin(fPackage)+sep()+"lib_"+name;
    out_cd(out,quote(dir_bin_xxx));

    std::string slib = ".."+sep()+name+".jar";
    out_rm(out,slib);

    out_verbose_on(out);
    out << "jar cf " << slib << " ." << Lib::endl;
    out_status(out);
    out_verbose_off(out);

    out_end_pwd(out);
    out << Lib::endl;

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(aItemML,"copy",packs,verss,platforms,  //copy jar
                                     verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    bool done_something = false;
    for(unsigned int index=0;index<number;index++) {
      const std::string& value = vals[index];
      const std::string& platform = platforms[index];
      const std::string& verbatim = verbatims[index];
      if(value=="") continue;
      BUILD_ON_PLATFORM_FP
  
      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      std::string sto;
      if(verbatim=="yes") {
        sto = dir_bin(fPackage)+sep()+value;
      } else {
        sto = dir_bin(fPackage)+sep()+value+".jar";
      }

      out_rm(out,quote(sto));

      std::string sfrom = dir_bin(fPackage)+sep()+name+".jar";

      out_copy(out,quote(sfrom),quote(sto));

      done_something = true;

      if(has_test) out_end_platform(out,spif);
    }
    if(done_something) out << Lib::endl;}
  
  } else { //not java lang

  if(fProdWhat==obuild::Builder::build_sh) {

    //FIXME : be dynamic : someone may want archive on Darwin
    //        but shared on Linux !

    if(kind_UNIX=="shared") {
    } else if(kind_UNIX=="archive") {
    } else {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_library_sh :" 
          << " unknown kind " << Lib::sout(kind_UNIX) 
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    bool do_archive = false;
    if(kind_UNIX=="shared") {
      do_archive = aItemML.isProperty("build_archive");
    } else {
      do_archive = true;
    }

    if(kind_UNIX=="shared") {

      // Shared lib builder options coming from tools :
      if(!out_begin_tool(lang+"_shared_lib_builder",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

      // Shared lib builder options coming from the component :
      if(!out_options(aItemML,"c_shared_lib_builder_options",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }
      if(!out_options(aItemML,"cxx_shared_lib_builder_options",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }
      if(!out_options(aItemML,"f77_shared_lib_builder_options",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

      if(!out_linked_libs(aItemML,out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

      // Create the shared library :
      // Link the shared libs (at least needed on Windows_NT) :

      out << "# Create the shared library " << name << " :" << Lib::endl;

      out_cd_bin(out);

      out << "obuild_objs=`find ./lib_" << name << " -name '*.o' -print`" 
          << Lib::endl;

      std::string dot_so = ".${obuild_"+lang+"_shared_lib_builder_suffix}";

      std::string slib = 
        "${obuild_"+lang+"_shared_lib_builder_prefix}"+name+dot_so;

      out_if_element_stamp("lib_"+name,"."+sep()+slib,out);

      out_rm(out,"."+sep()+slib);

      out_verbose_on(out);
      out << " eval"
          << " ${obuild_" << lang << "_shared_lib_builder_command}" 
          << " ${obuild_" << lang << "_shared_lib_builder_debug_local}" 
          << " ${obuild_" << lang << "_shared_lib_builder_optimize_local}"
          << " ${obuild_" << lang << "_shared_lib_builder_options}"
      // Darwin : need dir_bin(fPackage) to have abs path in binary lib.
          << " -o " << esc_quote(quote(dir_bin(fPackage)+sep()+slib))
        //<< " -o " << quote(slib)
          << " ${obuild_objs}"
          << " ${obuild_linked_paths}"
          << " ${obuild_linked_libraries}"
          << Lib::endl;
      out_status(out);
      out_verbose_off(out);

     {std::string stamp = "."+sep()+"lib_"+name+sep()+"component_stamp";
      out << "touch " << stamp << Lib::endl;}
      out << Lib::endl;

      out << "fi" << Lib::endl; //element_stamp

      out_cd_obuild(out);
      out << Lib::endl;

      fProduced_build_shlib_sh = true;
    } 

    if(do_archive) {

      if(!out_begin_tool("archiver",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

      out << "# Create the archive library " << name << " :" << Lib::endl;

     {std::string s = "./lib_"+name;
      out_cd_bin(out);
      out << "obuild_objs=`find " << s << " -name '*.o' -print`" 
          << Lib::endl;

      std::string dot_a = ".${obuild_archiver_suffix}";
      // Darwin : need dir_bin(fPackage) to have abs path in binary lib.
      std::string slib = dir_bin(fPackage)+"/lib"+name+dot_a;
      out_rm(out,slib);

      out_verbose_on(out);
      out << " eval"
          << " ${obuild_archiver_command}" 
          << " ${obuild_archiver_options}"
          << " " << esc_quote(quote(slib))
          << " ${obuild_objs}"
          << Lib::endl;
      out_status(out);
      out_verbose_off(out);
      out << Lib::endl;

      out_cd_obuild(out);}
  
    }

  } else if(fProdWhat==obuild::Builder::build_bat) {

    if(kind_Windows_NT=="shared") {
    } else if(kind_Windows_NT=="archive") {
    } else {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_library_sh :" 
          << " unknown kind " << Lib::sout(kind_Windows_NT) 
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    bool do_archive = false;
    std::string dot_a = ".lib";
    if(kind_Windows_NT=="shared") {
      do_archive = aItemML.isProperty("build_archive");
      dot_a = ".arc"; //Can't be a .lib.
    } else {
      do_archive = true;
    }

    if(kind_Windows_NT=="shared") {

      bool defWay = !aItemML.isProperty("declspec_way");

      // Shared bat

      if(!out_begin_tool(lang+"_linker",out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

    /*if(!out_begin_tool("archiver",out)) { //heavy.
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }*/

      if(!out_linked_libs(aItemML,out)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

      if(!out_dll_bat(out,"lib",name,defWay)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

    } 

    if(do_archive) {

    /*if(!out_begin_tool("archiver",out)) { //heavy
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }*/

      out_begin_pwd(out);

      out_cd_bin(out);

      std::string sobs = ".\\lib_"+name+"\\tmp_objs";

      out_rm(out,sobs);

      out << "FOR %%f IN (.\\lib_" << name
          << "\\*.obj) DO ECHO %%f >> " << sobs
          << Lib::endl;
      out << Lib::endl;

      out << "REM # Create the archive library " << name << " :" << Lib::endl;
      std::string sarc = ".\\" + name + dot_a;
      out_rm(out,sarc);

      out_verbose_on(out);
    //out << "%obuild_archiver_command% %obuild_archiver_options%" //heavy
      out << "lib.exe /nologo /machine:ix86"
          << " /out:" << sarc 
          << " @" << sobs
          << Lib::endl;
      out_status(out);
      out_verbose_off(out);
      out << Lib::endl;
  
      out_end_pwd(out);
    } 

  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"copy",packs,verss,platforms,  //copy library
                                   verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = vals.size();
  bool done_something = false;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index];
    const std::string& verbatim = verbatims[index];
    if(value=="") continue;
    BUILD_ON_PLATFORM_FP

    if(fProdWhat==obuild::Builder::build_sh) {

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      std::string dot_ext;
      if(kind_UNIX=="shared") {
        dot_ext = ".${obuild_"+lang+"_shared_lib_builder_suffix}";
      } else {   
        dot_ext = ".${obuild_archiver_suffix}";
      }

      std::string sfrom = dir_bin(fPackage)+sep()+"lib"+name+dot_ext;
      std::string sto;
      if(verbatim=="yes") {
        sto = dir_bin(fPackage)+sep()+value;
      } else { //smart copy.
        sto = dir_bin(fPackage)+sep()+value+dot_ext;
      }
  
      out_rm(out,quote(sto));
      out_copy(out,quote(sfrom),quote(sto));
  
      if(has_test) out_end_platform(out,spif);

    } else if(fProdWhat==obuild::Builder::build_bat) {
 
      //FIXME : copy what ? .lib or .dll ?
      //out_rm(out,sto);
      //out_copy(out,sfrom,sto);

    }

    done_something = true;
  }
  if(done_something) out << Lib::endl;}

  out_build_xxx_end_sh(aItemML,out,"lib",name);

  } //end java lang


  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_dll_bat(
 Lib::Out& aOut
,const std::string& aWhat  //lib, dll for target binary directory
,const std::string& aName
,bool aDefWay //true
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  out_begin_pwd(aOut);

  out_cd_bin(aOut);

  std::string dir_bin_xxx = ".\\"+aWhat+"_"+aName;

  std::string sarc = dir_bin_xxx + "\\" + aName + ".arc";
  std::string sdef = dir_bin_xxx + "\\" + aName + ".def";
  std::string stmp = dir_bin_xxx + "\\tmp";
  std::string sobs = dir_bin_xxx + "\\tmp_objs";
  std::string sdll = ".\\" + aName + ".dll";

  std::string slib;
  std::string sexp;
  if(aWhat=="lib") {
    // Put the .lib in the binary directory since it is used to link with.
    slib = ".\\" + aName + ".lib";
    sexp = ".\\" + aName + ".exp";
  } else if(aWhat=="dll") {
    slib = dir_bin_xxx + "\\" + aName + ".lib";
    sexp = dir_bin_xxx + "\\" + aName + ".exp";
  } else {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_dll_bat :" 
        << " bad keyword " << Lib::sout(aWhat) 
        << " when producing dll bat code for " << Lib::sout(aName) << "."
        << Lib::endl;
    return false;
  }

  out_rm(aOut,sobs);

  aOut << "FOR %%f IN (" << dir_bin_xxx << "\\*.obj) DO ECHO %%f >> " << sobs
       << Lib::endl;
  aOut << Lib::endl;

  if(aDefWay) {

    aOut << "REM # Create the archive library " << aName << " :" << Lib::endl;
    out_verbose_on(aOut);
    //aOut << "%obuild_archiver_command% %obuild_archiver_options%" //heavy

    aOut << "lib.exe /nologo /machine:ix86"
         << " /out:" << sarc 
         << " @" << sobs
         << Lib::endl;
    out_status(aOut);
    out_verbose_off(aOut);
    aOut << Lib::endl;
  
    if(!fProduced_local_windef_exe) {
      std::string s = dir_pack(fPackage)+"\\obuild\\cpp\\obuild_windef.c";
      aOut << "REM # Build the obuild_windef.exe utility program :" 
           << Lib::endl;
      aOut << "cl.exe /nologo /DWIN32 /MD /O2 /W3" 
           << " /Feobuild_windef.exe" 
           << " " << quote(s) 
           << " 1> cl_exe_error"
           << Lib::endl;
      aOut << "@IF ERRORLEVEL 1 @TYPE cl_exe_error && @GOTO obuild_return" 
           << Lib::endl;
      aOut << "@IF NOT ERRORLEVEL 1 @DEL cl_exe_error" 
           << Lib::endl;
      out_rm(aOut,"obuild_windef.obj");
      aOut << Lib::endl;
  
      fProduced_local_windef_exe = true;
      fProduced_windef_exe = true;
    }

    aOut << "REM # Building the .def file :" << Lib::endl;
    out_verbose_on(aOut);
    // There is no /nologo on dumpbin.exe. Then, to keep
    // the command silent we redirect to dumpbin_error file.
    aOut << "dumpbin.exe /symbols"
         << " /out:" << stmp
         << " " << sarc 
         << " 1> dumpbin_exe_error " << sarc 
         << Lib::endl;
    aOut << "@IF ERRORLEVEL 1 @TYPE dumpbin_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "@IF NOT ERRORLEVEL 1 @DEL dumpbin_exe_error" 
         << Lib::endl;
    out_verbose_off(aOut);
  
    aOut << ".\\obuild_windef.exe " << aName 
         << " < " << stmp
         << " > " << sdef
         << Lib::endl;
  
    aOut << "REM # Building the linking library :" << Lib::endl;
    aOut << "REM # It produces a .exp file." << Lib::endl;
    out_verbose_on(aOut);
    aOut << "lib.exe /nologo /machine:ix86"
         << " /out:" << slib
         << " /def:" << sdef
         << " 1> lib_exe_error"
         << Lib::endl;
    aOut << "@IF ERRORLEVEL 1 @TYPE lib_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "@IF NOT ERRORLEVEL 1 @DEL lib_exe_error" 
         << Lib::endl;
    out_verbose_off(aOut);
    aOut << Lib::endl;

    aOut << "REM # Building the DLL :" << Lib::endl;
    out_rm(aOut,sdll);
    out_verbose_on(aOut);
    aOut << "link.exe /nologo /dll"
         << " /out:" << sdll
         << " " << sexp
         << " " << sarc
         << " %obuild_linked_paths%"
         << " %obuild_linked_libraries%"
         << Lib::endl;
    out_verbose_off(aOut);
    aOut << Lib::endl;
  
  } else {

    aOut << "REM # Building the DLL :" << Lib::endl;
    out_rm(aOut,sdll);
    out_verbose_on(aOut);
    aOut << "link.exe /nologo /dll"
         << " /out:" << sdll
         //<< " " << sexp
         << " @" << sobs
         << " %obuild_linked_paths%"
         << " %obuild_linked_libraries%"
         << " 1> link_exe_error"
         << Lib::endl;

    aOut << "@IF ERRORLEVEL 1 @TYPE link_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "@IF NOT ERRORLEVEL 1 @DEL link_exe_error" 
         << Lib::endl;

    out_verbose_off(aOut);
    aOut << Lib::endl;
  
  }

  out_end_pwd(aOut);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_dll_sh(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {std::string platform;
  if(fProdWhat==obuild::Builder::build_sh) {
    platform = "UNIX";
  } else {
    platform = "Windows_NT";
  }
  bool doit;
  if(!done_on(aItemML,platform,doit)) return false;
  if(!doit) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true;
  }}

  std::vector<std::string> files;
  std::vector<std::string> platforms;
  if(!getFiles(aItemML,files,platforms)) return false;

  if(!files.size()) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true; // In fact nothing to do ! Ok.
  }

  ///////////////////////////////////////////////
  /// It seems that there is something to do ////
  /// for this component.                    ////
  ///////////////////////////////////////////////

  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_dll_sh :"
        << " name attribute not found for dll." 
        << Lib::endl;
    return false;
  }
  fComponent = name;

  std::string lang;
  if(!guess_linker_language(aItemML,files,lang)) return false;

  //////////////////////
  /// Produce code : ///
  //////////////////////
  if(!check_directory(f_dir)) return false;
  std::string sfile = "."+f_sep+f_dir+f_sep+"build_dll_"+name+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_dll_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  if(!out_build_xxx_core_sh(aItemML,out,"dll",name,files,platforms)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  // Specific to dlls :

  if(!out_linked_libs(aItemML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(fProdWhat==obuild::Builder::build_sh) {

    // DLL builder options coming from the tools :
    if(!out_begin_tool(lang+"_dll_builder",out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    // DLL builder options coming from the component :
    if(!out_options(aItemML,"c_dll_builder_options",out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!out_options(aItemML,"cxx_dll_builder_options",out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!out_options(aItemML,"f77_dll_builder_options",out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    // Create the dll :
    out << "# Create the dll " << name << " :" << Lib::endl;

    out_cd_bin(out);

    out << "obuild_objs=`find ./dll_" << name << " -name '*.o' -print`" 
        << Lib::endl;

    std::string dll = "${obuild_"+lang+"_dll_builder_prefix}"+
      name+".${obuild_"+lang+"_dll_builder_suffix}";

    out_if_element_stamp("dll_"+name,"."+sep()+dll,out);

    out_rm(out,"."+sep()+dll);

    out_verbose_on(out);
    out << " eval"
        << " ${obuild_" << lang << "_dll_builder_command}" 
        << " ${obuild_" << lang << "_dll_builder_options}"
        //Darwin : need dir_bin
        << " -o " << esc_quote(quote(dir_bin(fPackage)+"/"+dll))
        << " ${obuild_objs}"
        << " ${obuild_linked_paths}"
        << " ${obuild_linked_libraries}"
        << Lib::endl;
    out_status(out);
    out_verbose_off(out);

   {std::string stamp = "."+sep()+"dll_"+name+sep()+"component_stamp";
    out << "touch " << stamp << Lib::endl;}
    out << Lib::endl;

    out << "fi" << Lib::endl; //element_stamp

    out_cd_obuild(out);
    out << Lib::endl;
  
  } else if(fProdWhat==obuild::Builder::build_bat) {

    bool defWay = !aItemML.isProperty("declspec_way");

    if(!out_dll_bat(out,"dll",name,defWay)) return false;

  }

  // Handle <save/>
 {Lib::ItemML* saveML = aItemML.findByTag("save");
  if(saveML) {

    Subs subs;
   {std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(*saveML,"subs_var",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      std::string from;
      if(!itemML.propertyValue("from",from)) continue;
      std::string to;
      if(!itemML.propertyValue("to",to)) continue;
      subs.push_back(std::pair<std::string,std::string>(from,to));
    }}

    if(lang!="cxx") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_dll_sh :" 
          << " <save> not yet handle with lang " << Lib::sout(lang)
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string save_file = name+".save";

    if(fProdWhat==obuild::Builder::build_sh) {

      if(!out_save_sh(out,save_file,subs)) return false;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      if(!out_save_bat(out,save_file,subs)) return false;

    }

  }}

  // Handle <python_module/>
  if(aItemML.isProperty("python_module")) {

    std::string pyname;
    aItemML.attributeValue("python_module","name",pyname);
    if(pyname!="") {

      if(fProdWhat==obuild::Builder::build_sh) {
  
        std::string dll = dir_bin(fPackage)+"/"+
          "${obuild_"+lang+"_dll_builder_prefix}"+
          name+".${obuild_"+lang+"_dll_builder_suffix}";
  
        std::string pym = dir_bin(fPackage)+"/"+pyname+".so";
  
        out_rm(out,quote(pym));
        out_move(out,quote(dll),quote(pym));

      } else if(fProdWhat==obuild::Builder::build_bat) {
        std::string dll = dir_bin(fPackage)+"\\" + name + ".dll";
        std::string pym = dir_bin(fPackage)+"\\" + pyname + ".pyd";
  
        out_rm(out,quote(pym));
        out_move(out,quote(dll),quote(pym));
  
      }
    }
  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"copy",packs,verss,platforms, //copy dll
                                   verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = vals.size();
  bool done_something = false;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index];
    const std::string& verbatim = verbatims[index];
    if(value=="") continue;
    BUILD_ON_PLATFORM_FP

    std::string spif = if_count();
    bool has_test = out_begin_platform(out,platform,spif);

    if(fProdWhat==obuild::Builder::build_sh) {

      std::string sfrom = dir_bin(fPackage)+"/"+
        "${obuild_"+lang+"_dll_builder_prefix}"+
        name+".${obuild_"+lang+"_dll_builder_suffix}";
  
      std::string sto;
      if(verbatim=="yes") {
        sto = dir_bin(fPackage)+sep()+value;
      } else { //smart copy.
        sto = dir_bin(fPackage)+sep()+
              "${obuild_"+lang+"_dll_builder_prefix}"+
              value+".${obuild_"+lang+"_dll_builder_suffix}";
      }

      out_rm(out,quote(sto));
      out_copy(out,quote(sfrom),quote(sto));

    } else if(fProdWhat==obuild::Builder::build_bat) {

      std::string sfrom = dir_bin(fPackage)+sep()+name+".dll";
  
      std::string sto;
      if(verbatim=="yes") {
        sto = dir_bin(fPackage)+sep()+value;
      } else { //smart copy.
        sto = dir_bin(fPackage)+sep()+value+".dll";
      }

      out_rm(out,quote(sto));
      out_copy(out,quote(sfrom),quote(sto));

    }

    if(has_test) out_end_platform(out,spif);

    done_something = true;
  }
  if(done_something) out << Lib::endl;}

  out_build_xxx_end_sh(aItemML,out,"dll",name);

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_application_sh(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 {std::string platform;
  if(fProdWhat==obuild::Builder::build_sh) {
    platform = "UNIX";
  } else {
    platform = "Windows_NT";
  }
  bool doit;
  if(!done_on(aItemML,platform,doit)) return false;
  if(!doit) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true;
  }}

  std::vector<std::string> files;
  std::vector<std::string> platforms;
  if(!getFiles(aItemML,files,platforms)) return false;

  if(!files.size()) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true; // In fact nothing to do ! Ok.
  }

  ///////////////////////////////////////////////
  /// It seems that there is something to do ////
  /// for this component.                    ////
  ///////////////////////////////////////////////

  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_application_sh :"
        << " name attribute not found for application." 
        << Lib::endl;
    return false;
  }
  fComponent = name;

  std::string lang;
  if(!guess_linker_language(aItemML,files,lang)) return false;

  //////////////////////
  /// Produce code : ///
  //////////////////////
  if(!check_directory(f_dir)) return false;
  std::string sfile = "."+f_sep+f_dir+f_sep+"build_app_"+name+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_application_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  if(!out_build_xxx_core_sh(aItemML,out,"app",name,files,platforms)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  // Linker options coming from the tools :
  if(lang=="java") {

    out << cmt() << " Create the archive library " << name << " :" 
        << Lib::endl;

    out_begin_pwd(out);

    std::string dir_bin_xxx = dir_bin(fPackage)+sep()+"app_"+name;
    out_cd(out,quote(dir_bin_xxx));

    std::string slib = ".."+sep()+name+".jar";
    out_rm(out,slib);

    out_verbose_on(out);
    out << "jar cf " << slib << " ." << Lib::endl;
    out_status(out);
    out_verbose_off(out);

    out_end_pwd(out);
    out << Lib::endl;

  } else { //end java lang

  if(!out_begin_tool(lang+"_linker",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  // Linker options coming from the component :
  if(!out_options(aItemML,"c_linker_options",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(!out_options(aItemML,"cxx_linker_options",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(!out_options(aItemML,"f77_linker_options",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(!out_options(aItemML,"objc_linker_options",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(!out_options(aItemML,"objcxx_linker_options",out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  // Specific to applications :

  if(!out_linked_libs(aItemML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(fProdWhat==obuild::Builder::build_sh) {

    // Link :
    out << "# Link " << name << " :" << Lib::endl;

    out_cd_bin(out);

    out << "obuild_objs=`find ./app_" << name << " -name '*.o' -print`" 
        << Lib::endl;

    out_if_element_stamp("app_"+name,"."+sep()+name+f_exe,out);

    out_rm(out,"."+sep()+name+f_exe);

    out_verbose_on(out);
    out << " eval"
        << " ${obuild_" << lang << "_linker_command}" 
        << " ${obuild_" << lang << "_linker_debug_local}" 
        << " ${obuild_" << lang << "_linker_optimize_local}"
        << " ${obuild_" << lang << "_linker_options}"
        << " -o " << esc_quote(quote(dir_bin(fPackage)+"/"+name+f_exe))
        << " ${obuild_objs}"
        << " ${obuild_linked_paths}"
        << " ${obuild_linked_libraries}"
        << Lib::endl;
    out_status(out);
    out_verbose_off(out);

   {std::string stamp = "."+sep()+"app_"+name+sep()+"component_stamp";
    out << "touch " << stamp << Lib::endl;}
    out << Lib::endl;

    out << "fi" << Lib::endl; //element_stamp

    out_cd_obuild(out);
    out << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    // Put obj names in a temporary file :
    out_begin_pwd(out);

    out_cd_bin(out);

    std::string dir_objs = ".\\app_" + name;
    std::string sobs = ".\\app_" + name + "\\tmp_objs";

    out_rm(out,sobs);

    out << "FOR %%f IN (" << dir_objs << "\\*.obj) DO ECHO %%f >> " << sobs
        << Lib::endl;

    // Link :
    out << "REM # Link " << name << " :" << Lib::endl;

    std::string sexe = ".\\" + name + f_exe;
    out_rm(out,sexe);

    out_verbose_on(out);
    out << "%obuild_" << lang << "_linker_command%" 
        << " %obuild_" << lang << "_linker_debug_local%" 
        << " %obuild_" << lang << "_linker_optimize_local%"
        << " %obuild_" << lang << "_linker_options%"
        << " /out:" << sexe
        << " @" << sobs
        << " %obuild_linked_paths%"
        << " %obuild_linked_libraries%"
        << Lib::endl;
    out_status(out);
    out_verbose_off(out);
    out << Lib::endl;

    out_end_pwd(out);
  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"copy",packs,verss,platforms, //copy application
                                   verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = vals.size();
  bool done_something = false;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index];
    const std::string& verbatim = verbatims[index];
    if(value=="") continue;
    BUILD_ON_PLATFORM_FP

    std::string spif = if_count();
    bool has_test = out_begin_platform(out,platform,spif);

    std::string sfrom = dir_bin(fPackage)+sep()+name+f_exe;

    std::string sto;
    if(verbatim=="yes") {
      sto = dir_bin(fPackage)+sep()+value;
    } else { //smart copy.
      sto = dir_bin(fPackage)+sep()+value+f_exe;
    }

    out_rm(out,quote(sto));
    out_copy(out,quote(sfrom),quote(sto));

    done_something = true;

    if(has_test) out_end_platform(out,spif);
  }
  if(done_something) out << Lib::endl;}


  // Handle <save/>
 {Lib::ItemML* saveML = aItemML.findByTag("save");
  if(saveML) {

    Subs subs;
   {std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(*saveML,"subs_var",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      std::string from;
      if(!itemML.propertyValue("from",from)) continue;
      std::string to;
      if(!itemML.propertyValue("to",to)) continue;
      subs.push_back(std::pair<std::string,std::string>(from,to));
    }}

    if(lang!="cxx") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_application_sh :" 
          << " <save> not yet handle with lang " << Lib::sout(lang)
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string save_file = name+".save";

    if(fProdWhat==obuild::Builder::build_sh) {

      if(!out_save_sh(out,save_file,subs)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

    } else if(fProdWhat==obuild::Builder::build_bat) {

      if(!out_save_bat(out,save_file,subs)) {
        fp.disable();
        Lib::fmanip::remove(sfile);
        return false;
      }

    }

  }}

  out_build_xxx_end_sh(aItemML,out,"app",name);

  } // end java

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_darwin_app_sh(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  if(!fTopML) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_darwin_app_sh :" 
        << " fTopML is null." 
        << Lib::endl;
    return false;
  }

 {std::string platform;
  if(fProdWhat==obuild::Builder::build_sh) {
    platform = "UNIX";
  } else {
    platform = "Windows_NT";
  }
  bool doit;
  if(!done_on(aItemML,platform,doit)) return false;
  if(!doit) {
    aItemML.setPropertyValue("not_in_main_build","");
    return true;
  }}

  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_darwin_app_sh :"
        << " name attribute not found for application." 
        << Lib::endl;
    return false;
  }
  fComponent = name;

  //////////////////////
  /// Produce code : ///
  //////////////////////
  if(!check_directory(f_dir)) return false;
  std::string sfile = "."+f_sep+f_dir+f_sep+"build_darwin_app_"+name+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_darwin_app_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("build_darwin_app_sh");

  out << "#!/bin/sh -f" << Lib::endl;
  out << Lib::endl;

  out_template(out,build_xxx_header_sh_linen,
                   build_xxx_header_sh_lines);

  if(!out_not_done_on(aItemML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }  

  out_pre_build(aItemML,out);

  if(!out_check_foreigns(aItemML,out)) return false;
  if(!out_check_env(aItemML,out)) return false;

  // Build variables (especially the ones in foreigns :
  if(!out_variables(*fPackML,out)) return false;

  fProduced_Darwin_app = true;

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,
                  "path",packs,verss,platforms,
                        verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }}

  std::string spif = if_count();
  bool has_test_Darwin = out_begin_platform(out,"Darwin",spif);

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"copy_dll",
                    packs,verss,platforms,verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  std::string dlls;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    std::string value = vals[index];
    if(!replace_at_at(value)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(dlls!="") dlls += " ";
    dlls += quote(value);
  }

  out << "./sh/build_Darwin_app " << name << " " << dlls << Lib::endl;
  out << Lib::endl;}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aItemML,"copy_dir",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string from;
    if(!itemML.propertyValue("from",from)) continue;

    std::string to;
    if(!itemML.propertyValue("to",to)) continue;

    if(!replace_at_at(from)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!replace_at_at(to)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    out_mkdir(out,quote(to));

    out << "if [ -d " << quote(from) << " ] ; then" << Lib::endl;

    out << "  /bin/cp -f -R " << quote(from) << " " << quote(to) << Lib::endl;
    out << "else" << Lib::endl;
    out << "  echo \"Directory " << from << " not found.\"" << Lib::endl;

    out << "fi" << Lib::endl;

    //CYGWIN : to rm warnings :
    //  find <dir> -name CVS -print0 | xargs -0 /bin/rm -R -f
    //but it doesn not work on OSF1.
    //CYGWIN, Darwin : 
    //  find <dir> -name CVS -delete
    //but it doesn not work on Linux and OSF1.

    out << "find " << quote(to) 
         << " -depth -name CVS -exec /bin/rm -R -f {} \\;" 
         << Lib::endl;

    out << "find " << quote(to) 
         << " -depth -name .svn -exec /bin/rm -R -f {} \\;" 
         << Lib::endl;

  }
  out << Lib::endl;}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aItemML,"copy_files",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string from;
    if(!itemML.propertyValue("from",from)) continue;

    std::string to;
    if(!itemML.propertyValue("to",to)) continue;

    std::string copy_what;
    if(!itemML.propertyValue("what",copy_what)) continue;

    if(!replace_at_at(from)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!replace_at_at(to)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    out_mkdir(out,quote(to));

    out << "if [ -d " << quote(from) << " ] ; then" << Lib::endl;

   {std::vector<std::string> words = Lib::smanip::words(copy_what," ");
    unsigned int wordn = words.size();
    for(unsigned int index=0;index<wordn;index++){
      out  << "  find" 
           << " " << quote(from) 
           << " -maxdepth 1"
           << " -name '" << words[index] << "'"
           << " -follow"
           << " -exec /bin/cp -f {} " << quote(to) 
           << " \\;" 
           << Lib::endl;
    }}


    out << "else" << Lib::endl;
    out << "  echo \"Directory " << from << " not found.\"" << Lib::endl;

    out << "fi" << Lib::endl;

    //CYGWIN : to rm warnings :
    //  find <dir> -name CVS -print0 | xargs -0 /bin/rm -R -f
    //but it doesn not work on OSF1.
    //CYGWIN, Darwin : 
    //  find <dir> -name CVS -delete
    //but it doesn not work on Linux and OSF1.

    out << "find " << quote(to) 
         << " -depth -name CVS -exec /bin/rm -R -f {} \\;" 
         << Lib::endl;

    out << "find " << quote(to) 
         << " -depth -name .svn -exec /bin/rm -R -f {} \\;" 
         << Lib::endl;

  }
  out << Lib::endl;}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aItemML,"copy_file",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string from;
    if(!itemML.propertyValue("from",from)) continue;

    std::string to;
    if(!itemML.propertyValue("to",to)) continue;

    if(!replace_at_at(from)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!replace_at_at(to)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

   {std::string to_dir = Lib::smanip::dirname(to);
    out_mkdir(out,quote(to_dir));}

    out_copy(out,quote(from),quote(to),"");

  }
  out << Lib::endl;}

  if(has_test_Darwin) out_end_platform(out,spif);

  // Cleanup :
  out_rm(out,quote(dir_bin(fPackage)+"/obuild_find"));
  out << Lib::endl;

  out_post_build(aItemML,out);

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_sh(
 Lib::ItemML& aItemML //top item
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!check_directory(f_dir)) return false;

  std::string sfile = "."+f_sep+f_dir+f_sep+"build"+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("build_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,build_header_sh_linen,build_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,build_header_bat_linen,build_header_bat_lines);
  }

  out_pre_build(aItemML,out);

  if(!out_check_foreigns(aItemML,out)) return false;
  if(!out_check_env(aItemML,out)) return false;

  // libraries :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"library",list);
  bool done_something = false;
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string yes_no = 
      (*it)->isProperty("not_in_main_build")?"no":"yes";

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::string name;
    if((*it)->attributeValue("","name",name)) {

      out_begin_group(out,group);
 
      out_begin_main(out,yes_no);
 
      if(fProdWhat==obuild::Builder::build_sh) {
        std::string s = "./sh/build_lib_"+name+f_sh;

        out << "  if [ -e " << s << " ] ; then" << Lib::endl;
        out << "    " << s << " ${obuild_args}" 
            << Lib::endl;
        out_status(out,4);
        out << "  fi" << Lib::endl;

      } else if(fProdWhat==obuild::Builder::build_bat) {
        std::string s = ".\\DOS\\build_lib_"+name+".bat";

        std::string sif = if_count();

        out << "  IF NOT EXIST " << s << "  GOTO end_if_build_" << sif 
            << Lib::endl;
        out << "    CALL " << s << " %obuild_args%" << Lib::endl;
        out_status(out,4);
        out << "    @ECHO OFF" << Lib::endl;
        out << ":end_if_build_" << sif << Lib::endl;

      }

      out_end_main(out);

      out_end_group(out);
 
      done_something = true;

    }
  }
  if(done_something) out << Lib::endl;}

  // dlls :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"dll",list);
  bool done_something = false;
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string yes_no = 
      (*it)->isProperty("not_in_main_build")?"no":"yes";

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::string name;
    if((*it)->attributeValue("","name",name)) {

      out_begin_group(out,group);
 
      out_begin_main(out,yes_no);

      if(fProdWhat==obuild::Builder::build_sh) {
        std::string s = "./sh/build_dll_"+name+f_sh;

        out << "  if [ -e " << s << " ] ; then" << Lib::endl;
        out << "    " << s << " ${obuild_args}" 
            << Lib::endl;
        out_status(out,4);
        out << "  fi" << Lib::endl;

      } else if(fProdWhat==obuild::Builder::build_bat) {
        std::string s = ".\\DOS\\build_dll_"+name+".bat";

        std::string sif = if_count();

        out << "  IF NOT EXIST " << s << "  GOTO end_if_build_" << sif 
            << Lib::endl;
        out << "    CALL " << s << " %obuild_args%" << Lib::endl;
        out_status(out,4);
        out << "    @ECHO OFF" << Lib::endl;
        out << ":end_if_build_" << sif << Lib::endl;

      }

      out_end_main(out);

      out_end_group(out);
 
      done_something = true;

    }
  }
  if(done_something) out << Lib::endl;}

  // applications :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"application",list);
  bool done_something = false;
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string yes_no = 
      (*it)->isProperty("not_in_main_build")?"no":"yes";

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::string name;
    if((*it)->attributeValue("","name",name)) {

      out_begin_group(out,group);
 
      out_begin_main(out,yes_no);

      if(fProdWhat==obuild::Builder::build_sh) {
        std::string s = "./sh/build_app_"+name+f_sh;

        out << "  if [ -e " << s << " ] ; then" << Lib::endl;
        out << "    " << s << " ${obuild_args}" 
            << Lib::endl;
        out_status(out,4);
        out << "  fi" << Lib::endl;

      } else if(fProdWhat==obuild::Builder::build_bat) {
        std::string s = ".\\DOS\\build_app_"+name+".bat";

        std::string sif = if_count();

        out << "  IF NOT EXIST " << s << "  GOTO end_if_build_" << sif 
            << Lib::endl;
        out << "    CALL " << s << " %obuild_args%" << Lib::endl;
        out_status(out,4);
        out << "    @ECHO OFF" << Lib::endl;
        out << ":end_if_build_" << sif << Lib::endl;

      }

      out_end_main(out);

      out_end_group(out);
 
      done_something = true;
    }
  }
  if(done_something) out << Lib::endl;}

  // darwin_apps :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"darwin_app",list);
  bool done_something = false;
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string yes_no = 
      (*it)->isProperty("not_in_main_build")?"no":"yes";

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::string name;
    if((*it)->attributeValue("","name",name)) {

      if(fProdWhat==obuild::Builder::build_sh) {
        out_begin_group(out,group);

        out_begin_main(out,yes_no);

        std::string s = "./sh/build_darwin_app_"+name+f_sh;

        out << "  if [ -e " << s << " ] ; then" << Lib::endl;
        out << "    " << s << " ${obuild_args}" 
            << Lib::endl;
        out_status(out,4);
        out << "  fi" << Lib::endl;

        out_end_main(out);

        out_end_group(out);

      } else if(fProdWhat==obuild::Builder::build_bat) {
        // Make no sens on Windows.
      }

      done_something = true;
    }
  }
  if(done_something) out << Lib::endl;}

  if(fProdWhat==obuild::Builder::build_sh) {
    // Install the setup scripts :
    std::string s = "./sh/install"+f_sh;
    out << "if [ -e " << s << " ] ; then" << Lib::endl;
    out << "  " << s << Lib::endl;
    out << "fi" << Lib::endl;
    out << Lib::endl;
  } 

  out_post_build(aItemML,out);

  if(fProdWhat==obuild::Builder::build_sh) {
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out << ":obuild_return" << Lib::endl;
    out << "ENDLOCAL" << Lib::endl;
    out << "@ECHO ON" << Lib::endl;
  }

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_load_library_path_sh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"load_library_path",packs,verss,platforms,
                  verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& verbatim = verbatims[index];
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    //printf("debug : ld_lib_path : 001 : %d : %s plat : %s\n",
      //index,value.c_str(),platform.c_str());

    std::string path;
    if(verbatim=="yes") {
      path = value;
      //replace_at_at(value);
      if(path=="") continue;
    } else {
      if(pack=="this") {
        path = dir_bin(fPackage);
      } else {    
        path = dir_bin(pack);
      } 
    }

    bool do_unix = true;
    bool do_win = true;
    if(platform=="") {     // Do UNIXes and Windows_NT.
    } else if(platform=="Windows_NT") {
      do_unix = false;
    } else {              // Assume some UNIX.
      do_win = false;
    }

    //FIXME : path = no_quote(path);

    if(fProdWhat==obuild::Builder::build_sh) {

      if(!do_unix) continue;

      //FIXME : template the below ?

      // Darwin : LD_LIBRARY_PATH needed only if loading with dlopen.
      // Darwin : It is not needed if using nsopen.

      std::string spif = if_count();

      // has_test_Darwin always true !
      bool has_test_Darwin = out_begin_platform(aOut,"Darwin",spif);

       {std::string spif2 = if_count();
        bool has_test = out_begin_platform(aOut,platform,spif2);

        // Darwin : DYLD_LIBRARY_PATH not needed to activate .dylib
        //          if building the .dylib with absolute paths !
        // Darwin : but DYLD_LIBRARY_PATH needed to load DLL with nsopen.
  
        aOut << "  if [  \"${DYLD_LIBRARY_PATH}\" = \"\" ] ; then" 
             << Lib::endl;
        aOut << "    DYLD_LIBRARY_PATH=" << quote(path)
             <<      Lib::endl;
        aOut << "    export DYLD_LIBRARY_PATH" << Lib::endl;
        aOut << "  else" << Lib::endl;
        std::string s = "${DYLD_LIBRARY_PATH}:"+path;
        aOut << "    DYLD_LIBRARY_PATH=" << quote(s) << Lib::endl;
        aOut << "    export DYLD_LIBRARY_PATH" << Lib::endl;
        aOut << "  fi" << Lib::endl;

        if(has_test) out_end_platform(aOut,spif2);}

      if(has_test_Darwin) if(!out_else_platform(aOut)) return false;

        std::string spif3 = if_count();
        // has_test_CYGWIN always true !
        bool has_test_CYGWIN = out_begin_platform(aOut,"CYGWIN",spif3);

         {std::string spif2 = if_count();
          bool has_test = out_begin_platform(aOut,platform,spif2);

          //FIXME : should put code checking that path is already in PATH.

          aOut << "if [  \"${PATH}\" = \"\" ] ; then" << Lib::endl;
          aOut << "  PATH=" << quote(path) << Lib::endl;
          aOut << "  export PATH" << Lib::endl;
          aOut << "else" << Lib::endl;
          std::string s = "${PATH}:"+path;
          aOut << "  PATH=" << quote(s)  << Lib::endl;
          aOut << "  export PATH" << Lib::endl;
          aOut << "fi" << Lib::endl;

          if(has_test) out_end_platform(aOut,spif2);}

        if(has_test_CYGWIN) if(!out_else_platform(aOut)) return false;

         {std::string spif2 = if_count();
          bool has_test = out_begin_platform(aOut,platform,spif2);

          aOut << "  if [  \"${LD_LIBRARY_PATH}\" = \"\" ] ; then" 
               << Lib::endl;
          aOut << "    LD_LIBRARY_PATH=" << quote(path) << Lib::endl;
          aOut << "    export LD_LIBRARY_PATH" << Lib::endl;
          aOut << "  else" << Lib::endl;
          std::string s = "${LD_LIBRARY_PATH}:"+path;
          aOut << "    LD_LIBRARY_PATH=" << quote(s) << Lib::endl;
          aOut << "    export LD_LIBRARY_PATH" << Lib::endl;
          aOut << "  fi" << Lib::endl;

          if(has_test) out_end_platform(aOut,spif2);}

        if(has_test_CYGWIN) out_end_platform(aOut,spif3);

      if(has_test_Darwin) out_end_platform(aOut,spif);
      aOut << Lib::endl;

      aEmpty = false;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      if(!do_win) continue;

      if(pack=="this") fProduced_local_LD_PATH = true;

      std::string sif = fPackage+"_"+if_count();

      //FIXME : should put code checking that path is already in PATH.

      aOut << "IF DEFINED PATH GOTO else_if_path_" << sif << Lib::endl;
      aOut << "  SET PATH=" << path << Lib::endl;
      aOut << "  GOTO end_if_path_" << sif << Lib::endl;
      aOut << ":else_if_path_" << sif << Lib::endl;
      aOut << "  SET PATH=%PATH%;" << path << Lib::endl;
      aOut << ":end_if_path_" << sif << Lib::endl;
      aOut << Lib::endl;

      aEmpty = false;
    }

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_load_library_path_csh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"load_library_path",
                packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& verbatim = verbatims[index];
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    std::string path;
    if(verbatim=="yes") {
      path = value;
      //replace_at_at(value);
      if(path=="") continue;
    } else {
      if(pack=="this") {
        path = dir_bin(fPackage);
      } else {    
        path = dir_bin(pack);
      } 
    }

    bool do_unix = true;
    bool do_win = true;
    if(platform=="") {     // Do UNIXes and Windows_NT.
    } else if(platform=="Windows_NT") {
      do_unix = false;
    } else {              // Assume some UNIX.
      do_win = false;
    }

    if(!do_unix) continue;

    //FIXME path = no_quote(path); 

    //FIXME : template the below ?

    // Darwin : LD_LIBRARY_PATH needed only if loading with dlopen.
    // Darwin : It is not needed if using nsopen.

    bool has_test_Darwin = out_begin_platform_csh(aOut,"Darwin");

     {bool has_test = out_begin_platform_csh(aOut,platform);

      aOut << "  if (  $?DYLD_LIBRARY_PATH ) then" << Lib::endl;
      std::string s = "${DYLD_LIBRARY_PATH}:"+path;
      aOut << "    setenv DYLD_LIBRARY_PATH " << quote(s) << Lib::endl;
      aOut << "  else" << Lib::endl;
      aOut << "    setenv DYLD_LIBRARY_PATH " << quote(path) << Lib::endl;
      aOut << "  endif" << Lib::endl;

      if(has_test) out_end_platform_csh(aOut);}

    if(has_test_Darwin) out_else_platform_csh(aOut);

      bool has_test_CYGWIN = out_begin_platform_csh(aOut,"CYGWIN");

       {bool has_test = out_begin_platform_csh(aOut,platform);
        aOut << "if (  $?PATH ) then" << Lib::endl;
        std::string s = "${PATH}:"+path;
        aOut << "  setenv PATH " << quote(s) << Lib::endl;
        aOut << "else" << Lib::endl;
        aOut << "  setenv PATH " << quote(path) << Lib::endl;
        aOut << "endif" << Lib::endl;
        aOut << Lib::endl;
        if(has_test) out_end_platform_csh(aOut);}

      if(has_test_CYGWIN) out_else_platform_csh(aOut);

       {bool has_test = out_begin_platform_csh(aOut,platform);

        aOut << "  if (  $?LD_LIBRARY_PATH ) then" << Lib::endl;
        std::string s = "${LD_LIBRARY_PATH}:"+path;
        aOut << "    setenv LD_LIBRARY_PATH " << quote(s) << Lib::endl;
        aOut << "  else" << Lib::endl;
        aOut << "    setenv LD_LIBRARY_PATH " << quote(path) << Lib::endl;
        aOut << "  endif" << Lib::endl;

        if(has_test) out_end_platform_csh(aOut);}

      if(has_test_CYGWIN) out_end_platform_csh(aOut);

    if(has_test_Darwin) out_end_platform_csh(aOut);
    aOut << Lib::endl;

    aEmpty = false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_run_path_sh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"run_path",packs,verss,platforms,
                  verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& verbatim = verbatims[index];
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"

    std::string path;
    if(verbatim=="yes") {
      path = value;
      //replace_at_at(value);
      if(path=="") continue;
    } else {
      if(pack=="this") {
        path = dir_bin(fPackage);
      } else {    
        path = dir_bin(pack);
      } 
    }

    bool do_unix = true;
    bool do_win = true;
    if(platform=="Windows_NT") {
      do_unix = false;
    } else if(platform=="UNIX") {
      do_win = false;
    } else if(platform=="") {
    } else {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_run_path_sh :" 
          << " for <load_library_path> "
          << ", only platform=\"UNIX\" or \"Windows_NT\""
          << " can be specified."
          << Lib::endl;
      return false;
    }

    //FIXME path = no_quote(path);

    if(fProdWhat==obuild::Builder::build_sh) {

      if(!do_unix) continue;

      //FIXME : template the below ?

      //FIXME : should put code checking that path is already in PATH.

      aOut << "if [  \"${PATH}\" = \"\" ] ; then" << Lib::endl;
      aOut << "  PATH=" << quote(path) << Lib::endl;
      aOut << "  export PATH" << Lib::endl;
      aOut << "else" << Lib::endl;
      std::string s = "${PATH}:"+path;
      aOut << "  PATH=" << quote(s)  << Lib::endl;
      aOut << "  export PATH" << Lib::endl;
      aOut << "fi" << Lib::endl;

      aEmpty = false;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      if(!do_win) continue;

      if((pack=="this")&&fProduced_local_LD_PATH) continue;

      std::string sif = fPackage+"_"+if_count();

      //FIXME : should put code checking that path is already in PATH.

      aOut << "IF DEFINED PATH GOTO else_if_path_" << sif << Lib::endl;
      aOut << "  SET PATH=" << path << Lib::endl;
      aOut << "  GOTO end_if_path_" << sif << Lib::endl;
      aOut << ":else_if_path_" << sif << Lib::endl;
      aOut << "  SET PATH=%PATH%;" << path << Lib::endl;
      aOut << ":end_if_path_" << sif << Lib::endl;
      aOut << Lib::endl;

      aEmpty = false;
    }

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_run_path_csh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"run_path",
                packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& verbatim = verbatims[index];
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"

    std::string path;
    if(verbatim=="yes") {
      path = value;
      //replace_at_at(value);
      if(path=="") continue;
    } else {
      if(pack=="this") {
        path = dir_bin(fPackage);
      } else {    
        path = dir_bin(pack);
      } 
    }

    bool do_unix = true;
    bool do_win = true;
    if(platform=="Windows_NT") {
      do_unix = false;
    } else if(platform=="UNIX") {
      do_win = false;
    } else if(platform=="") {
    } else {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_run_path_csh :" 
          << " for <load_library_path> "
          << ", only platform=\"UNIX\" or \"Windows_NT\""
          << " can be specified."
          << Lib::endl;
      return false;
    }

    if(!do_unix) continue;

    //FIXME path = no_quote(path);

    //FIXME : template the below ?

    //FIXME : should put code checking that path is already in PATH.

    aOut << "if (  $?PATH ) then" << Lib::endl;
    std::string s = "${PATH}:"+path;
    aOut << "  setenv PATH " << quote(s) << Lib::endl;
    aOut << "else" << Lib::endl;
    aOut << "  setenv PATH " << quote(path) << Lib::endl;
    aOut << "endif" << Lib::endl;
    aOut << Lib::endl;

    aEmpty = false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_path_append_sh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"path_append",packs,verss,platforms,
                  verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& variable = names[index];
    std::string path = vals[index];
    const std::string& platform = platforms[index];
    const std::string& pack = packs[index];
    BUILD_ON_PLATFORM

    if(!replace_at_at(path)) return false;

    if(path=="") continue;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);
      
    if(fProdWhat==obuild::Builder::build_sh) {

      path = tounix(path);
      path = no_quote(path);

      aOut << "  if [  \"${" << variable << "}\" = \"\" ] ; then" << Lib::endl;
      aOut << "    " << variable << "=" << quote(path) << Lib::endl;
      aOut << "    export " << variable << Lib::endl;
      aOut << "  else" << Lib::endl;
      std::string s = "${"+variable+"}:"+path;
      aOut << "    " << variable << "=" << quote(s) << Lib::endl;
      aOut << "    export " << variable << Lib::endl;
      aOut << "  fi" << Lib::endl;
      aOut << Lib::endl;

      aEmpty = false;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      path = towin(path);
      path = no_quote(path);

      std::string sif = fPackage+"_"+if_count();

      aOut << "IF DEFINED " << variable << " GOTO else_if_var_" << sif 
           << Lib::endl;
      aOut << "  SET " << variable << "=" << path << Lib::endl;
      aOut << "  GOTO end_if_var_" << sif << Lib::endl;
      aOut << ":else_if_var_" << sif << Lib::endl;
      aOut << "  SET " << variable << "=%" << variable << "%;" 
           << path << Lib::endl;
      aOut << ":end_if_var_" << sif << Lib::endl;
      aOut << Lib::endl;

      aEmpty = false;
    }

    if(has_test) out_end_platform(aOut,spif);

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_path_append_csh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"path_append",packs,verss,platforms,
                  verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& variable = names[index];
    std::string path = vals[index];
    const std::string& platform = platforms[index];
    const std::string& pack = packs[index];
    BUILD_ON_PLATFORM

    if(!replace_at_at(path)) return false;

    if(path=="") continue;

    path = tounix(path);
    path = no_quote(path);

    bool has_test = out_begin_platform_csh(aOut,platform);
  
    aOut << "  if (  $?" << variable << " ) then" << Lib::endl;
    std::string s = "${"+variable+"}:"+path;
    aOut << "    setenv " << variable << " " << quote(s) << Lib::endl;
    aOut << "  else" << Lib::endl;
    aOut << "    setenv " << variable << " " << quote(path) << Lib::endl;
    aOut << "  endif" << Lib::endl;

    aOut << Lib::endl;

    aEmpty = false;

    if(has_test) out_end_platform_csh(aOut);

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_setup_sh(
 Lib::ItemML& aSetupML //<setup>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  std::string package = fPackage;

  std::string sfile = "setup.sh";
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_setup_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("setup_sh");
  fProduce_local_exit_sh = "return";

  out_template(out,setup_header_sh_linen,setup_header_sh_lines);

  bool empty = true;

  // Build variables (especially the ones in foreigns :
  if(!out_variables(*fPackML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"path",
        packs,verss,platforms,verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"exec",packs,verss,platforms,
                                 verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  if(number) {
    out << "# Execute setup of used packages :" << Lib::endl;
    for(unsigned int index=0;index<number;index++) {
      const std::string& pack = packs[index];
      const std::string& platform = platforms[index];
      BUILD_ON_PLATFORM_FP

      if(pack=="this") continue;

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      std::string s = dir_pack(pack)+"/obuild/setup.sh";

      out << "if [ ! -e " << quote(s) << " ] ; then" << Lib::endl;
      out << "  echo \"setup.sh not found for package " << pack << ".\"" 
          << Lib::endl;
      out << "  return" << Lib::endl;
      out << "fi" << Lib::endl;
      out << ". " << quote(s) << Lib::endl;
      out << Lib::endl;

      if(has_test) out_end_platform(out,spif);

      empty = false;
    }
  }}

 {out << "# Set environment variables of this package :" << Lib::endl;
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"env",packs,verss,platforms,
                                 verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& name = names[index];
    const std::string& platform = platforms[index];
    std::string value = vals[index];
    BUILD_ON_PLATFORM_FP

    if(!replace_at_at(value)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string spif = if_count();
    bool has_test = out_begin_platform(out,platform,spif);

    out << name << "=" << quote(tounix(value)) << Lib::endl;
    out << "export " << name << Lib::endl;

    if(has_test) out_end_platform(out,spif);
    empty = false;
  }}
  out << Lib::endl;

  if(!out_path_append_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  
  if(!out_load_library_path_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_run_path_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  out_template(out,setup_trailer_sh_linen,setup_trailer_sh_lines);

  fp.disable();

  if(empty) {
    Lib::fmanip::remove(sfile);
  } else {
    fProduced_setups = true;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_setup_csh(
 Lib::ItemML& aSetupML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  std::string package = fPackage;

  std::string sfile = "setup.csh";
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_setup_csh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("setup_csh");

  out_template(out,setup_header_csh_linen,setup_header_csh_lines);

  bool empty = true;

  // Build variables (especially the ones in foreigns :
  if(!out_variables_csh(*fPackML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"path",
        packs,verss,platforms,verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_csh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"exec",packs,verss,platforms,
                                 verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_csh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  if(number) {
    out << "# Execute setup of used packages :" << Lib::endl;
    for(unsigned int index=0;index<number;index++) {
      const std::string& pack = packs[index];      
      const std::string& platform = platforms[index];
      BUILD_ON_PLATFORM_FP

      if(pack=="this") continue;

      bool has_test = out_begin_platform_csh(out,platform);

      std::string s = dir_pack(pack)+"/obuild/setup.csh";
      out << "source " << quote(s) << Lib::endl;

      if(has_test) out_end_platform_csh(out);

      out << Lib::endl;

      empty = false;
    }
  }}

 {out << "# Set environment variables of this package :" << Lib::endl;
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"env",packs,verss,platforms,
                                 verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& name = names[index];
    const std::string& platform = platforms[index];
    std::string value = vals[index];
    BUILD_ON_PLATFORM_FP

    if(!replace_at_at(value)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    value = tounix(value);
    value = no_quote(value); //FIXME ?

    bool has_test = out_begin_platform_csh(out,platform);

    out << "setenv " << name << " " << quote(value) << Lib::endl;

    if(has_test) out_end_platform_csh(out);

    empty = false;
  }}
  out << Lib::endl;

  if(!out_path_append_csh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_load_library_path_csh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_run_path_csh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  out_template(out,setup_trailer_csh_linen,setup_trailer_csh_lines);

  fp.disable();

  if(empty) {
     Lib::fmanip::remove(sfile);
  } else {
    fProduced_setups = true;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_setup_bat(
 Lib::ItemML& aSetupML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) return true;

  std::string package = fPackage;

  std::string sfile = "setup.bat";
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_setup_bat :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("setup_bat");

  out_template(out,setup_header_bat_linen,setup_header_bat_lines);

  bool empty = true;

  // Build variables (especially the ones in foreigns :
  if(!out_variables(*fPackML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"path",
        packs,verss,platforms,verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"exec",packs,verss,platforms,
                                 verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  if(number) {
    out << "REM # Execute setup of used packages :" << Lib::endl;
    for(unsigned int index=0;index<number;index++) {
      const std::string& pack = packs[index];      
      if(pack=="this") continue;
      std::string s = dir_pack(pack) + "\\obuild\\setup.bat";
      out << "IF NOT EXIST " << quote(s)
          << "  ECHO setup.bat not found for package " << pack << "."
          << "  && GOTO obuild_return" 
          << Lib::endl;
      out << "CALL " << quote(s) << Lib::endl;
      out << "@ECHO OFF" << Lib::endl;
      out << Lib::endl;
      empty = false;
    }
  }}

 {out << "REM # Set environment variables of this package :" << Lib::endl;
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aSetupML,"env",packs,verss,platforms,
                                 verbatims,names,ifs,vals)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& name = names[index];
    const std::string& platform = platforms[index];
    std::string value = vals[index];
    BUILD_ON_PLATFORM_FP

    if(!replace_at_at(value)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string spif = if_count();
    bool has_test = out_begin_platform(out,platform,spif);

    if(value=="") 
      out << "SET " << name << "=\"\"" << Lib::endl;
    else
      out << "SET " << name << "=" << towin(value) << Lib::endl;

    if(has_test) out_end_platform(out,spif);

    empty = false;
  }}
  out << Lib::endl;

  if(!out_path_append_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_load_library_path_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  if(!out_run_path_sh(aSetupML,out,empty)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  out_template(out,setup_trailer_bat_linen,setup_trailer_bat_lines);

  fp.disable();

  if(empty) {
     Lib::fmanip::remove(sfile);
  } else {
    fProduced_setups = true;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_Darwin_app_sh(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  std::string sfile = "./sh/build_Darwin_app";
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_build_Darwin_app_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  out_template(out,build_Darwin_app_sh_linen,build_Darwin_app_sh_lines);

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_install_sh(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) {
    // not needed with bat file ; they know thir own location.
    return true;
  }

  std::string sfile = "./sh/install"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_install_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  std::string s = Lib::smanip::tostring(install_sh_linen,install_sh_lines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  out << s;

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_sys_sh(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true; //FIXME

  std::string sfile = "./sh/build_sys"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_build_sys_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  std::string s = 
    Lib::smanip::tostring(build_sys_sh_linen,build_sys_sh_lines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  out << s;

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_shlib_sh(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true; //FIXME

  std::string sfile = "./sh/build_shlib"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_build_shlib_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  std::string s = 
    Lib::smanip::tostring(build_shlib_sh_linen,build_shlib_sh_lines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  out << s;

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_check_app_sh(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true; //FIXME

  std::string sfile = "./sh/check_app"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_check_app_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  std::string s = Lib::smanip::tostring(check_app_sh_linen,check_app_sh_lines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  out << s;

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_broadcast_install_sh(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) {
    // not needed with bat file ; they know thir own location.
    return true;
  }

  if(!fProduced_setups) return true; //Nothing to do.

  std::string package = fPackage;

  std::string sfile = "./sh/broadcast_install"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_broadcast_install_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("broadcast_install_sh");

  out_template(out,broadcast_install_header_sh_linen,
                   broadcast_install_header_sh_lines);

  bool empty = true;

  // Do the job for the package itself first.
  // This avoid to have to cd back to ourself at end 
  // and then avoid a "save current directory logic".

  std::string s = dir_pack(package)+"/obuild/sh/install"+f_sh;

  out << "if [ ! -e " << quote(s) << " ] ; then" << Lib::endl;
  out << "  echo \"install script not found for package " 
      << package << ".\"" 
      << Lib::endl;
  out << "  exit" << Lib::endl;
  out << "fi" << Lib::endl;
  out << "if [ ${obuild_verbose} = yes ] ; then" << Lib::endl;
  out << "  echo \"install setups of " << package << "\"..." << Lib::endl;
  out << "fi" << Lib::endl;
  out << "cd " << quote(dir_pack(package)+"/obuild") << Lib::endl;
  out << "./sh/install" << f_sh << Lib::endl;
  out << Lib::endl;
  empty = false;

  // Install used packages :
 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"exec",packs,verss,platforms,
                                 verbatims,names,ifs,vals,true)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  if(packs.size()) empty = false;
  if(!out_paths_sh(packs,verss,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }
  unsigned int number = packs.size();
  if(number) {
    out << "# install setup scripts of used packages :" << Lib::endl;
    for(unsigned int index=0;index<number;index++) {
      const std::string& pack = packs[index];
      if(pack=="this") continue;
      std::string s = dir_pack(pack)+"/obuild/sh/broadcast_install"+f_sh;
      out << "if [ ! -e " << quote(s) << " ] ; then" << Lib::endl;
      out << "  echo \"broadcast_install script not found for package " 
          << pack << ".\"" 
          << Lib::endl;
      out << "  exit" << Lib::endl;
      out << "fi" << Lib::endl;
      out << "cd " << quote(dir_pack(pack)+"/obuild") << Lib::endl;
      out << "./sh/broadcast_install" << f_sh << " ${obuild_args}" 
          << Lib::endl;
      out << Lib::endl;
      empty = false;
    }
  }}

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(empty) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_run_sh(
 Lib::ItemML& aItemML //top item
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sfile = "."+f_sep+f_dir+f_sep+"run"+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_run_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("run_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,run_header_sh_linen,run_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,run_header_bat_linen,run_header_bat_lines);
  }

  out_begin_pwd(out);

  if(fProdWhat==obuild::Builder::build_sh) {
    out_mkdir(out,"/tmp/obuild_run");
    out << "cd /tmp/obuild_run" << Lib::endl;
    out << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    //FIXME
  }


  bool done_something = false;

  // applications :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"application",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string name;
    if(!(*it)->attributeValue("","name",name)) continue;

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"run",
                      packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    if(number) out_begin_group(out,group);

    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index]; //"platform"
      std::string run = vals[index];
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);
   
      if(fProdWhat==obuild::Builder::build_sh) {

        if(run=="") {
          std::string s = dir_bin(fPackage)+sep()+name;

          out << "  if [ -e " << quote(s) << " ] ; then" << Lib::endl;
          out_verbose_on(out);
          out << "    " << quote(s) << Lib::endl;
          //out_status(out,4); //Not necessary globally fatal.
          out_verbose_off(out);
          out << "  fi" << Lib::endl;
        } else {
          if(!replace_at_at(run)) {
            fp.disable();
            Lib::fmanip::remove(sfile);
            return false;
          }

          out_verbose_on(out);
          out << "    " << run << Lib::endl;
          //out_status(out,4);
          out_verbose_off(out);
        }

      } else if(fProdWhat==obuild::Builder::build_bat) {

        if(run=="") {
          std::string s = dir_bin(fPackage)+sep()+name+".exe";

          std::string sif = if_count();

          out << "  IF NOT EXIST " << quote(s) 
              << "  GOTO end_if_run_" << sif 
              << Lib::endl;
          out_verbose_on(out);
          out << "    " << quote(s) << Lib::endl;
          out_status(out,4);
          out_verbose_off(out);
          out << ":end_if_run_" << sif << Lib::endl;

        } else {
          if(!replace_at_at(run)) {
            fp.disable();
            Lib::fmanip::remove(sfile);
            return false;
          }

          out_verbose_on(out);
          out << "    " << run << Lib::endl;
          out_status(out,4);
          out << "    @ECHO OFF" << Lib::endl;
          out_verbose_off(out);
        }

      }

      if(has_test) out_end_platform(out,spif);

      done_something = true;

    }
    if(number) out_end_group(out);

  }
  if(done_something) out << Lib::endl;}

  // darwin_app :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"darwin_app",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    std::string run;
    if(!(*it)->propertyValue("run",run)) continue;

    std::string group;
    if(!(*it)->propertyValue("group",group)) group = "main";

    std::string name;
    if((*it)->attributeValue("","name",name)) {
      if(fProdWhat==obuild::Builder::build_sh) {

        std::string spif = if_count();
        bool has_test_Darwin = out_begin_platform(out,"Darwin",spif);

        out_begin_group(out,group);

        if(run=="") {
          std::string s = dir_bin(fPackage)+sep()+name+".app";

          out << "  if [ -e " << quote(s) << " ] ; then" << Lib::endl;
          out_verbose_on(out);
          out << "    open " << quote(s) << Lib::endl;
          //out_status(out,4); //Not necessary globally fatal.
          out_verbose_off(out);
          out << "  fi" << Lib::endl;
        } else {
          if(!replace_at_at(run)) {
            fp.disable();
            Lib::fmanip::remove(sfile);
            return false;
          }

          out_verbose_on(out);
          out << "    " << run << Lib::endl;
          //out_status(out,4);
          out_verbose_off(out);
        }

        out_end_group(out);

        if(has_test_Darwin) out_end_platform(out,spif);

      } else if(fProdWhat==obuild::Builder::build_bat) {
        // Make no sens on Windows_NT.
      }
      done_something = true;
    }
  }
  if(done_something) out << Lib::endl;}

  out_end_pwd(out);

  if(fProdWhat==obuild::Builder::build_sh) {

    out_rmdir(out,"/tmp/obuild_run");

  } else if(fProdWhat==obuild::Builder::build_bat) {
    out << ":obuild_return" << Lib::endl;
    out << "ENDLOCAL" << Lib::endl;
    out << "@ECHO ON" << Lib::endl;
  }

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(!done_something) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_project_sh(
 Lib::ItemML& aProjectML //<project>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string pj_name;
  if(!aProjectML.attributeValue("name",pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_project_sh :"
        << " no attribute name on the <project> XML item."
        << Lib::endl;
    return false;
  }

  std::string pj_version;
  if(!aProjectML.attributeValue("version",pj_version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_project_sh :"
        << " no attribute version on the <project> XML item."
        << Lib::endl;
    return false;
  }

  if(!check_name(pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_project_sh :"
        << " problem with " << Lib::sout(pj_name)
        << ". Project name should have the syntax of a programming variable." 
        << Lib::endl;
    return false;
  }

  fPackage = pj_name;
  fVersion = pj_version;

  std::string sfile = "."+f_sep+f_dir+f_sep+"build"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_project_sh :"
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("build_project_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,build_project_header_sh_linen,
                     build_project_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,build_project_header_bat_linen,
                     build_project_header_bat_lines);
  }

  out_begin_pwd(out);

  out << Lib::endl;

  bool done_something = false;

 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"package",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    if((*it)->isProperty("not_in_main_build")) continue;

    std::string pack_name;
    (*it)->attributeValue("name",pack_name);
    std::string pack_version;
    (*it)->attributeValue("version",pack_version);

    if(!out_path_sh(pack_name,pack_version,out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    //printf("debug : %s : %s\n",pack_name.c_str(),pack_version.c_str());

    std::string dir_pack_obuild = var_val(path_variable(pack_name));
    dir_pack_obuild += sep();
    dir_pack_obuild += "obuild";

    dir_pack_obuild = quote(dir_pack_obuild);

    out << "cd " << dir_pack_obuild << Lib::endl;

    out_verbose_on(out);
    if(fProdWhat==obuild::Builder::build_sh) {
      out << "echo \"build " << pack_name << "/" << pack_version 
          << " ...\"" 
          << Lib::endl;
      out << "./sh/build  ${obuild_args}" << Lib::endl;
      out_status(out);
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out << "ECHO build " << pack_name << "/" << pack_version 
          << " ..." 
          << Lib::endl;
      out << "CALL DOS\\build %obuild_args%" << Lib::endl;
      out << "@ECHO OFF" << Lib::endl;
      //FIXME out_status(out); //Does not work. It stops systematically.
    }
    out_verbose_off(out);

    done_something = true;
 }}
  out << Lib::endl;

  out_end_pwd(out);

  if(fProdWhat==obuild::Builder::build_sh) {

  } else if(fProdWhat==obuild::Builder::build_bat) {

    out << ":obuild_return" << Lib::endl;
    out << "ENDLOCAL" << Lib::endl;
    out << "@ECHO ON" << Lib::endl;
  }

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(!done_something) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_release_sh(
 Lib::ItemML& aProjectML //<project>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string pj_name;
  if(!aProjectML.attributeValue("name",pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_release_sh :"
        << " no attribute name on the <project> XML item."
        << Lib::endl;
    return false;
  }

  std::string pj_version;
  if(!aProjectML.attributeValue("version",pj_version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_release_sh :"
        << " no attribute version on the <project> XML item."
        << Lib::endl;
    return false;
  }

  if(!check_name(pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_release_sh :"
        << " problem with " << Lib::sout(pj_name)
        << ". Project name should have the syntax of a programming variable." 
        << Lib::endl;
    return false;
  }

  fPackage = pj_name;
  fVersion = pj_version;

  std::string sfile = "."+f_sep+f_dir+f_sep+"build_release"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_release_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("build_release_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,build_release_header_sh_linen,
                     build_release_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,build_release_header_bat_linen,
                     build_release_header_bat_lines);
  }

  // Build variables (especially the ones in foreigns :
  if(!out_variables(aProjectML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  std::string dbin = var_val(dir_bin_variable());

  out_begin_pwd(out);

  out_cd_bin(out);
  out_rmdir(out,pj_name);
  out_mkdir(out,pj_name);
  out << "cd " << pj_name << Lib::endl;
  out_mkdir(out,pj_version);
  out << "cd " << pj_version << Lib::endl;
  out_mkdir(out,"lib");
  out_mkdir(out,"bin");
  out_mkdir(out,"Resources");
  out_var_pwd(out,"obuild_dir_project");

  //out_end_pwd(out);

  out << Lib::endl;

  bool done_something = false;

  std::string rel_dir = 
    quote(var_val("obuild_dir_project"));

  std::string rel_dir_lib = 
    quote(var_val("obuild_dir_project")+sep()+"lib"+sep()+".");

  std::string rel_dir_bin = 
    quote(var_val("obuild_dir_project")+sep()+"bin"+sep()+".");

  // save version of <package> in a Versions file :
 {std::string file = 
    quote(var_val("obuild_dir_project")+sep()+"Versions");

  out_rm(out,file);

 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"package",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML* releaseML = (*it)->findByTag("release");
    if(!releaseML) continue;

    std::string pack_name;
    (*it)->attributeValue("name",pack_name);
    std::string pack_version;
    (*it)->attributeValue("version",pack_version);

    //printf("debug : %s : %s\n",pack_name.c_str(),pack_version.c_str());

    std::string s = "package " + pack_name + " " + pack_version;

    if(fProdWhat==obuild::Builder::build_sh) {
      out << "echo \"" << s << "\" >> " << file << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out << "ECHO "   << s << "   >> " << file << Lib::endl;
    }

  }}


 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"foreign",list);
  Lib::ItemML::unique(list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML* releaseML = (*it)->findByTag("release");
    if(!releaseML) continue;

    std::string name;
    (*it)->attributeValue("name",name);

    std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"native_version",packs,verss,platforms,
                                    verbatims,names,ifs,vals)) return false;
    unsigned int number = vals.size();
    if(number<=0) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_release_sh :" 
          << " WARNING : foreign " << Lib::sout(name) 
          << " without <native_version>."
          << Lib::endl;
    }

    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index];
      BUILD_ON_PLATFORM_FP

      std::string s = "foreign " + name + " " + vals[index];

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      if(fProdWhat==obuild::Builder::build_sh) {
        out << "echo \"" << s << "\" >> " << file << Lib::endl;
      } else if(fProdWhat==obuild::Builder::build_bat) {
        out << "ECHO "   << s << "   >> " << file << Lib::endl;
      }

      if(has_test) out_end_platform(out,spif);

    }

  }}

  out << Lib::endl;}

  // handle <release> of <foreign> :
  // (See for example foreign/Python.obuild)
  //FIXME : the same foreign (for exa Python) can come 
  //        from different packages !
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"foreign",list);
  Lib::ItemML::unique(list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML* releaseML = (*it)->findByTag("release");
    if(!releaseML) continue;

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*releaseML,"copy_dll",
                      packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      std::string value = vals[index];
      const std::string& platform = platforms[index]; //"platform"
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      //replace_at_at(value);
      out_copy_check(out,quote(value),towin(rel_dir_bin));

      if(has_test) out_end_platform(out,spif);

      done_something = true;
    }}

   {std::string top_from;
    bool out_something;
    if(!out_release_sh(out,*releaseML,top_from,out_something)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(out_something) done_something = true;}

  }
  out << Lib::endl;}

  // handle <release> of <package> :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"package",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    Lib::ItemML* releaseML = 0;

    // Find first <release> in first level of children.
    // ( <foreign> can have also a <release> ).
   {const std::list<Lib::ItemML*>& list2 = (*it)->children();
    std::list<Lib::ItemML*>::const_iterator it2;
    for(it2=list2.begin();it2!=list2.end();++it2) {
      if((*it2)->tagName()=="release") {
        releaseML = *it2;
        break;
      }
    }}

    if(!releaseML) continue;

    std::string pack_name;
    (*it)->attributeValue("name",pack_name);
    std::string pack_version;
    (*it)->attributeValue("version",pack_version);

    if(!out_path_sh(pack_name,pack_version,out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    //printf("debug : <release> under %s : %s\n",
      //pack_name.c_str(),pack_version.c_str());

   {std::string top_from = var_val(path_variable(pack_name));
    top_from += sep();
    bool out_something;
    if(!out_release_sh(out,*releaseML,top_from,out_something)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(out_something) done_something = true;}

 }}
  out << Lib::endl;

  // libraries :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"library",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML& itemML = *(*it);

    if(!handle_template(itemML)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string name;
    if(!itemML.attributeValue("","name",name)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    Lib::ItemML* packML = get_parent_package_ML(itemML);
    if(!packML) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_release_sh :" 
          << " <" << itemML.tagName() << "> " << Lib::sout(name)
          << " has no parent <package>."
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

   {std::string platform;
    if(fProdWhat==obuild::Builder::build_sh) {
      platform = "UNIX";
    } else {
      platform = "Windows_NT";
    }
    bool doit;
    if(!done_on(itemML,platform,doit)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!doit) continue;}

    //printf("debug : %s : %s\n",itemML.tagName().c_str(),name.c_str());

    std::string pack_name;
    packML->attributeValue("name",pack_name);
    std::string pack_version;
    packML->attributeValue("version",pack_version);

    if(!out_path_sh(pack_name,pack_version,out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string from = var_val(path_variable(pack_name));
    from += sep();
    from += dbin;

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      out << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      out << "  echo \"WARNING : Directory " << from << " not found.\"" 
          << Lib::endl;
      out << "else" << Lib::endl;

      // Darwin :
      out_copy_check(out,quote(from+sep()+"lib"+name+".dylib"),rel_dir_lib);

      // Linux :
      out_copy_check(out,quote(from+sep()+"lib"+name+".so"),rel_dir_lib);

      // CYGWIN :
      out_copy_check(out,quote(from+sep()+"lib"+name+".dll.a"),rel_dir_lib);
      out_copy_check(out,quote(from+sep()+name+".dll"),rel_dir_bin);

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      out << "IF NOT EXIST " << quote(from)
          << "  ECHO WARNING : Directory " << from << " not found."
          << "  && GOTO end_if_release_" << sif 
          << Lib::endl;

      out_copy_check(out,quote(from+sep()+name+".lib"),towin(rel_dir_lib));

      out_copy_check(out,quote(from+sep()+name+".dll"),towin(rel_dir_bin));

    }

    if(fProdWhat==obuild::Builder::build_sh) {
      out << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out << ":end_if_release_" << sif << Lib::endl;
    }

    done_something = true;
  }}
  out << Lib::endl;

  // dlls :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"dll",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML& itemML = *(*it);

    if(!handle_template(itemML)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string name;
    if(!itemML.attributeValue("","name",name)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    Lib::ItemML* packML = get_parent_package_ML(itemML);
    if(!packML) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_release_sh :" 
          << " <" << itemML.tagName() << "> " << Lib::sout(name)
          << " has no parent <package>."
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

   {std::string platform;
    if(fProdWhat==obuild::Builder::build_sh) {
      platform = "UNIX";
    } else {
      platform = "Windows_NT";
    }
    bool doit;
    if(!done_on(itemML,platform,doit)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!doit) continue;}

    //printf("debug : %s : %s\n",itemML.tagName().c_str(),name.c_str());

    std::vector<std::string> copies;
    itemML.propertyValues("copy",copies);

    std::string pack_name;
    packML->attributeValue("name",pack_name);
    std::string pack_version;
    packML->attributeValue("version",pack_version);

    if(!out_path_sh(pack_name,pack_version,out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string pyname;
    itemML.attributeValue("python_module","name",pyname);

    std::string save_file;
    if(itemML.isProperty("save")) {
      save_file = name+".save";
    }

    std::string from = var_val(path_variable(pack_name));
    from += sep();
    from += dbin;

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      out << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      out << "  echo \"WARNING : Directory " << from << " not found.\"" 
          << Lib::endl;
      out << "else" << Lib::endl;

      // Darwin :
      out_copy_check(out,quote(from+sep()+name+".bundle"),rel_dir_bin);

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+scopy+".bundle"),rel_dir_bin);
      }}

      if(pyname!="") {
        out_copy_check(out,quote(from+sep()+pyname+".so"),rel_dir_bin);
      }

      out_copy_check(out,quote(from+sep()+"lib"+name+".jnilib"),rel_dir_bin);

      // Linux :
      out_copy_check(out,quote(from+sep()+"lib"+name+".so"),rel_dir_bin);

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+"lib"+scopy+".so"),rel_dir_bin);
      }}

      // CYGWIN :
      out_copy_check(out,quote(from+sep()+name+".dll"),rel_dir_bin);

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+scopy+".dll"),rel_dir_bin);
      }}

      if(save_file!="") {
        out_copy_check(out,quote(from+sep()+save_file),rel_dir_bin);
      }

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      out << "IF NOT EXIST " << quote(from)
          << "  ECHO WARNING : Directory " << from << " not found."
          << "  && GOTO end_if_release_" << sif 
          << Lib::endl;

      out_copy_check(out,quote(from+sep()+name+".lib"),towin(rel_dir_lib));

      out_copy_check(out,quote(from+sep()+name+".dll"),towin(rel_dir_bin));

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+scopy+".dll"),towin(rel_dir_bin));
      }}

      if(pyname!="") {
        out_copy_check(out,quote(from+sep()+pyname+".pyd"),towin(rel_dir_bin));
      }

      if(save_file!="") {
        out_copy_check(out,quote(from+sep()+save_file),towin(rel_dir_bin));
      }

    }

    if(fProdWhat==obuild::Builder::build_sh) {
      out << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out << ":end_if_release_" << sif << Lib::endl;
    }

    done_something = true;
  }}
  out << Lib::endl;

  // applications :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"application",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML& itemML = *(*it);

    if(!handle_template(itemML)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string name;
    if(!itemML.attributeValue("","name",name)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string save_file;
    if(itemML.isProperty("save")) {
      save_file = name+".save";
    }

    Lib::ItemML* packML = get_parent_package_ML(itemML);
    if(!packML) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_build_release_sh :" 
          << " <" << itemML.tagName() << "> " << Lib::sout(name)
          << " has no parent <package>."
          << Lib::endl;
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

   {std::string platform;
    if(fProdWhat==obuild::Builder::build_sh) {
      platform = "UNIX";
    } else {
      platform = "Windows_NT";
    }
    bool doit;
    if(!done_on(itemML,platform,doit)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(!doit) continue;}

    //printf("debug : %s : %s\n",itemML.tagName().c_str(),name.c_str());

    std::vector<std::string> copies;
    itemML.propertyValues("copy",copies);

    std::string pack_name;
    packML->attributeValue("name",pack_name);
    std::string pack_version;
    packML->attributeValue("version",pack_version);

    if(!out_path_sh(pack_name,pack_version,out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string from = var_val(path_variable(pack_name));
    from += sep();
    from += dbin;

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      out << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      out << "  echo \"WARNING : Directory " << from << " not found.\"" 
          << Lib::endl;
      out << "else" << Lib::endl;

      out_copy_check(out,quote(from+sep()+name),rel_dir_bin);

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+scopy),rel_dir_bin);
      }}

      if(save_file!="") {
        out_copy_check(out,quote(from+sep()+save_file),rel_dir_bin);
      }

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      out << "IF NOT EXIST " << quote(from)
          << "  ECHO WARNING : Directory " << from << " not found."
          << "  && GOTO end_if_release_" << sif 
          << Lib::endl;

      out_copy_check(out,quote(from+sep()+name+".exe"),towin(rel_dir_bin));

     {unsigned int number = copies.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& scopy = copies[index];
        out_copy_check(out,quote(from+sep()+scopy+".exe"),towin(rel_dir_bin));
      }}

      if(save_file!="") {
        out_copy_check(out,quote(from+sep()+save_file),towin(rel_dir_bin));
      }

    }

    if(fProdWhat==obuild::Builder::build_sh) {
      out << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out << ":end_if_release_" << sif << Lib::endl;
    }

    done_something = true;
  }}
  out << Lib::endl;

 {// Treat <release> directly under <project> (used in osc_snovis) :
  std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"release",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML& releaseML = *(*it);
    Lib::ItemML* packML = get_parent_package_ML(releaseML);
    if(packML) continue; //Not under <project>.

/*
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_build_release_sh :"
        << " do not treat <release> directly under <project>."
        << Lib::endl;

    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
*/

   {std::string top_from = var_val(path_variable(pj_name));
    top_from += sep();
    bool out_something;
    if(!out_release_sh(out,releaseML,top_from,out_something)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    if(out_something) done_something = true;}

  }}

  if(fProdWhat==obuild::Builder::build_sh) {
    out_begin_pwd(out);
    out_cd_obuild(out);
    std::string s = "./sh/build_sys"+f_sh;
    out << "if [ -e " << s << " ] ; then" << Lib::endl;
    out << "  " << s << " " << rel_dir << Lib::endl;
    out_status(out,2);
    out << "fi" << Lib::endl;
    out_end_pwd(out);
    out << Lib::endl;
    fProduced_build_sys_sh = true;
  }

 {// building zip file :
  std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"zip",list);
  std::vector<Lib::ItemML*>::iterator it;
  if(list.size()) {
    if(!out_platform_signature(out)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
  }
  for(it=list.begin();it!=list.end();++it) {
    Lib::ItemML& itemML = *(*it);

    std::string platform;
    itemML.attributeValue("platform",platform);
    BUILD_ON_PLATFORM_FP

    std::string spif = if_count();
    bool has_test = out_begin_platform(out,platform,spif);

    std::string sif = if_count();

    // Remove some dir from the install area :
   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(itemML,"rm_dir",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size(); 
    for(unsigned int index=0;index<number;index++) {
      const std::string& value = vals[index];
      if(value=="") continue;
      // value is relative to install area :
      std::string from = var_val("obuild_dir_project");
      out_rmdir(out,quote(from+sep()+value));
    }}

    // Remove some file from the install area :
   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(itemML,"rm_files",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& value = vals[index];
      if(value=="") continue;
      std::string remove_what = value;

      // from is relative to install area :
      std::string from = var_val("obuild_dir_project");
  
      if(fProdWhat==obuild::Builder::build_sh) {

       {std::vector<std::string> words = Lib::smanip::words(remove_what," ");
        unsigned int wordn = words.size();
        for(unsigned int index=0;index<wordn;index++){
          out << "find" 
              << " " << quote(from) 
              << " -maxdepth 1"
              << " -name '" << words[index] << "'"
              << " -follow"
              << " -exec /bin/rm -f {}"
              << " \\;" 
              << Lib::endl;
        }}

      } else if(fProdWhat==obuild::Builder::build_bat) {

        from = towin(from);

       {std::vector<std::string> words = Lib::smanip::words(remove_what," ");
        unsigned int wordn = words.size();
        for(unsigned int index=0;index<wordn;index++){
          out << "DEL " 
              << " " << quote(from) 
              << "\\" << words[index]
              << Lib::endl;
        }}
      }

    }}

    // zip :
    std::string zip_file;
    if(!itemML.propertyValue("file",zip_file)) {
      zip_file = "@@package@@-@@version@@-@@platform_signature@@.zip";
    }

    if(!replace_at_at(zip_file)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }

    std::string zip_what;
    itemML.propertyValue("what",zip_what);
    if(zip_what=="") zip_what = fPackage;
    
    out_cd_bin(out);

    out_rm(out,zip_file);

    if(fProdWhat==obuild::Builder::build_sh) {

      out << "zip -q -r " << zip_file
          << " " << zip_what
          << Lib::endl;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      // zip.exe taken from cygwin.
      //FIXME : should check that zip.exe exists.
      out << "zip -q -r " << zip_file
          << " " << zip_what
          << Lib::endl;

    }
    //out_status(out);
    out_cd_obuild(out);

    if(has_test) out_end_platform(out,spif);
  }}

  out_cd_bin(out);

  if(fProdWhat==obuild::Builder::build_sh) {

    out_rm(out,"obuild_platform");

    out_end_pwd(out);

  } else if(fProdWhat==obuild::Builder::build_bat) {

    out_rm(out,"obuild_platform.exe");

    out_end_pwd(out);

    out << ":obuild_return" << Lib::endl;
    out << "ENDLOCAL" << Lib::endl;
    out << "@ECHO ON" << Lib::endl;
  }

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(!done_something) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_release_sh(
 Lib::Out& aOut
,Lib::ItemML& aParentML //<project>, <package> or <foreign>
,const std::string& aFromTop //If not empty, must be ended with sep()
,bool& a_done_something
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  a_done_something = false;

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aParentML,"copy_file",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string platform;
    itemML.attributeValue("platform",platform);
    BUILD_ON_PLATFORM

    std::string opts;
    itemML.propertyValue("options",opts);

    std::string copy_from;
    if(!itemML.propertyValue("from",copy_from)) continue;
    bool from_absolute = false;
   {std::string s;
    itemML.attributeValue("from","absolute",s);
    if(s!=""){
      if(!Lib::smanip::tobool(s,from_absolute)) {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_file>, absolute attribute value " << Lib::sout(s) 
            << " is not a boolean."
            << Lib::endl;
        return false;
      }
    }}
    if(!replace_at_at(copy_from)) return false;
    std::string from;
    if(from_absolute) {
      from = copy_from;
    } else {
      if(aFromTop=="") {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_file>, not absolute required, but top dir is empty."
            << Lib::endl;
        return false;
      }
      from = aFromTop+copy_from;
    }

    std::string copy_to;
    if(!itemML.propertyValue("to",copy_to)) continue;
    std::string to = var_val("obuild_dir_project")+sep()+copy_to;
    std::string to_dir = Lib::smanip::dirname(to);

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "if [ ! -e " << quote(from) << " ] ; then" << Lib::endl;
      aOut << "  echo \"WARNING : File " << from << " not found.\"" 
           << Lib::endl;
      aOut << "else" << Lib::endl;

      out_mkdir(aOut,quote(to_dir));

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      aOut << "IF NOT EXIST " << quote(from)
           << "  ECHO WARNING : File " << from << " not found."
           << "  && GOTO end_if_release_" << sif 
           << Lib::endl;

      out_mkdir(aOut,quote(towin(to_dir)));

      to = towin(to);

    }

    out_copy(aOut,quote(from),quote(to),opts);

    if(fProdWhat==obuild::Builder::build_sh) {
      aOut << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      aOut << ":end_if_release_" << sif << Lib::endl;
    }

    if(has_test) out_end_platform(aOut,spif);

    a_done_something = true;
  }}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aParentML,"copy_dir",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string platform;
    itemML.attributeValue("platform",platform);
    BUILD_ON_PLATFORM

    // Assume a UNIX syntax (with / separator).
    std::string copy_from;
    if(!itemML.propertyValue("from",copy_from)) continue;
    bool from_absolute = false;
   {std::string s;
    itemML.attributeValue("from","absolute",s);
    if(s!=""){
      if(!Lib::smanip::tobool(s,from_absolute)) {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_dir>, absolute attribute value " << Lib::sout(s) 
            << " is not a boolean."
            << Lib::endl;
        return false;
      }
    }}
    if(!replace_at_at(copy_from)) return false;
    std::string from;
    if(from_absolute) {
      from = copy_from;
    } else {
      from = aFromTop+copy_from;
      if(aFromTop=="") {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_dir>, not absolute required, but top dir is empty."
            << Lib::endl;
        return false;
      }
    }

    std::string copy_to;
    if(!itemML.propertyValue("to",copy_to)) continue;

    if(!replace_at_at(copy_to)) return false;

    //printf("debug : <release> <copy_dir> %s %s\n",
      //copy_from.c_str(),copy_to.c_str());

     
    std::string to = var_val("obuild_dir_project")+sep()+copy_to;
  
    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      aOut << "  echo \"WARNING : Directory " << from << " not found.\"" 
           << Lib::endl;
      aOut << "else" << Lib::endl;

      out_mkdir(aOut,quote(to));

      aOut << "/bin/cp -f -R " << quote(from) << " " << quote(to) << Lib::endl;

      //CYGWIN : to rm warnings :
      //  find <dir> -name CVS -print0 | xargs -0 /bin/rm -R -f
      //but it doesn not work on OSF1.
      //CYGWIN, Darwin : 
      //  find <dir> -name CVS -delete
      //but it doesn not work on Linux and OSF1.

     {std::string spif2 = if_count();
      bool has_test_CYGWIN = out_begin_platform(aOut,"CYGWIN",spif2);

        aOut << "find " << quote(to)
             << " -name CVS -print0 | xargs -0 /bin/rm -R -f" 
             << Lib::endl;

        aOut << "find " << quote(to)
             << " -name .svn -print0 | xargs -0 /bin/rm -R -f" 
             << Lib::endl;

      if(has_test_CYGWIN) if(!out_else_platform(aOut)) return false;

        aOut << "find " << quote(to)
             << " -depth -name CVS -exec /bin/rm -R -f {} \\;" 
             << Lib::endl;

        aOut << "find " << quote(to)
            << " -depth -name .svn -exec /bin/rm -R -f {} \\;" 
            << Lib::endl;

      if(has_test_CYGWIN) out_end_platform(aOut,spif2);}

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      aOut << "IF NOT EXIST " << quote(from)
           << "  ECHO WARNING : Directory " << from << " not found."
           << "  && GOTO end_if_release_" << sif 
           << Lib::endl;

      // XCOPY needs the "from" basename...
      to += sep()+Lib::smanip::basename(copy_from);
      to = towin(to);

      out_mkdir(aOut,quote(to));

      out_xcopy(aOut,quote(from),quote(to));
    }

    if(fProdWhat==obuild::Builder::build_sh) {
      aOut << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      aOut << ":end_if_release_" << sif << Lib::endl;
    }

    if(has_test) out_end_platform(aOut,spif);

    a_done_something = true;
 }}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aParentML,"copy_files",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string platform;
    itemML.attributeValue("platform",platform);
    BUILD_ON_PLATFORM

    std::string copy_from;
    if(!itemML.propertyValue("from",copy_from)) continue;
    bool from_absolute = false;
   {std::string s;
    itemML.attributeValue("from","absolute",s);
    if(s!=""){
      if(!Lib::smanip::tobool(s,from_absolute)) {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_files>, absolute attribute value " << Lib::sout(s) 
            << " is not a boolean."
            << Lib::endl;
        return false;
      }
    }}
    if(!replace_at_at(copy_from)) return false;
    std::string from;
    if(from_absolute) {
      from = copy_from;
    } else {
      from = aFromTop+copy_from;
      if(aFromTop=="") {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::out_release_sh :" 
            << " in <copy_files>, not absolute required, but top dir is empty."
            << Lib::endl;
        return false;
      }
    }

    std::string copy_to;
    if(!itemML.propertyValue("to",copy_to)) continue;

    std::string copy_what;
    if(!itemML.propertyValue("what",copy_what)) continue;

    std::string to = var_val("obuild_dir_project")+sep()+copy_to;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      aOut << "  echo \"WARNING : Directory " << from << " not found.\"" 
           << Lib::endl;
      aOut << "else" << Lib::endl;

      out_mkdir(aOut,quote(to));

     {std::vector<std::string> words = Lib::smanip::words(copy_what," ");
      unsigned int wordn = words.size();
      for(unsigned int index=0;index<wordn;index++){
        aOut << "find" 
             << " " << quote(from) 
             << " -maxdepth 1"
             << " -name '" << words[index] << "'"
             << " -follow"
             << " -exec /bin/cp -f {} " << quote(to) 
             << " \\;" 
             << Lib::endl;
      }}

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      aOut << "IF NOT EXIST " << quote(from)
           << "  ECHO WARNING : Directory " << from << " not found."
           << "  && GOTO end_if_release_" << sif 
           << Lib::endl;

      out_mkdir(aOut,quote(to));

     {std::vector<std::string> words = Lib::smanip::words(copy_what," ");
      unsigned int wordn = words.size();
      for(unsigned int index=0;index<wordn;index++){
        out_xcopy(aOut,quote(from)+sep()+words[index],quote(to));
      }}

      //CVS directories not copied by the upper (hidden files).
    }

    if(fProdWhat==obuild::Builder::build_sh) {
      aOut << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      aOut << ":end_if_release_" << sif << Lib::endl;
    }

    if(has_test) out_end_platform(aOut,spif);

    a_done_something = true;
  }}

 {std::vector<Lib::ItemML*> items;
  Lib::ItemML::collectByTag(aParentML,"remove_files",items);
  unsigned int number = items.size();
  for(unsigned int index=0;index<number;index++) {
    Lib::ItemML& itemML = *(items[index]);

    std::string remove_from;
    if(!itemML.propertyValue("from",remove_from)) continue;

    std::string remove_what;
    if(!itemML.propertyValue("what",remove_what)) continue;

    std::string platform;
    itemML.attributeValue("platform",platform);
    BUILD_ON_PLATFORM

    std::string from = var_val("obuild_dir_project")+sep()+remove_from;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    std::string sif = if_count();

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "if [ ! -d " << quote(from) << " ] ; then" << Lib::endl;
      aOut << "  echo \"WARNING : Directory " << from << " not found.\"" 
           << Lib::endl;
      aOut << "else" << Lib::endl;

     {std::vector<std::string> words = Lib::smanip::words(remove_what," ");
      unsigned int wordn = words.size();
      for(unsigned int index=0;index<wordn;index++){
        aOut << "find" 
             << " " << quote(from) 
             << " -maxdepth 1"
             << " -name '" << words[index] << "'"
             << " -follow"
             << " -exec /bin/rm -f {}"
             << " \\;" 
             << Lib::endl;
      }}

    } else if(fProdWhat==obuild::Builder::build_bat) {

      from = towin(from);

      aOut << "IF NOT EXIST " << quote(from)
           << "  ECHO WARNING : Directory " << from << " not found."
           << "  && GOTO end_if_release_" << sif 
           << Lib::endl;

     {std::vector<std::string> words = Lib::smanip::words(remove_what," ");
      unsigned int wordn = words.size();
      for(unsigned int index=0;index<wordn;index++){
        aOut << "DEL " 
             << " " << quote(from) 
             << "\\" << words[index]
             << Lib::endl;
      }}

      //CVS directories not copied by the upper (hidden files).
    }

    if(fProdWhat==obuild::Builder::build_sh) {
      aOut << "fi" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      aOut << ":end_if_release_" << sif << Lib::endl;
    }

    if(has_test) out_end_platform(aOut,spif);

    a_done_something = true;
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_checkout_release_sh(
 Lib::ItemML& aProjectML //<project>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string pj_name;
  if(!aProjectML.attributeValue("name",pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_checkout_release_sh :"
        << " no attribute name on the <project> XML item."
        << Lib::endl;
    return false;
  }

  std::string pj_version;
  if(!aProjectML.attributeValue("version",pj_version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_checkout_release_sh :"
        << " no attribute version on the <project> XML item."
        << Lib::endl;
    return false;
  }

  if(!check_name(pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_checkout_release_sh :"
        << " problem with " << Lib::sout(pj_name)
        << ". Project name should have the syntax of a programming variable." 
        << Lib::endl;
    return false;
  }

  fPackage = pj_name;
  fVersion = pj_version;

  std::string sfile = "."+f_sep+f_dir+f_sep+"checkout_release"+f_sh;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_checkout_release_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("checkout_release_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,checkout_release_header_sh_linen,
                     checkout_release_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,checkout_release_header_bat_linen,
                     checkout_release_header_bat_lines);
  }

  bool done_something = false;

  out_begin_pwd(out);

  out << "cd .." << Lib::endl;
  out << "cd .." << Lib::endl;
  out << "cd .." << Lib::endl;
  out << Lib::endl;

 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aProjectML,"package",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string pack_name;
    (*it)->attributeValue("name",pack_name);
    std::string pack_version;
    (*it)->attributeValue("version",pack_version);

    std::string pack_rep;
    if(!(*it)->propertyValue("repository",pack_rep)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_checkout_release_sh :"
          << " WARNING : package " << Lib::sout(pack_name) 
          << " has no <repository>."
          << Lib::endl;      
    }

    Lib::smanip::replace(pack_rep,"@@user@@","anonymous");

    if(fProdWhat==obuild::Builder::build_sh) {
      out_mkdir(out,pack_name);
      out_cd(out,pack_name);
      out << "CVSROOT=" << pack_rep << Lib::endl;
      out << "export CVSROOT" << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      out_mkdir(out,pack_name);
      out_cd(out,pack_name);
      out << "set CVSROOT=" << pack_rep << Lib::endl;
    }
    out << "cvs co"
        << " -d " << pack_version 
        << " -r " << pack_version 
        << " " << pack_name
        << Lib::endl;
    out << "cd .." << Lib::endl;
    out << Lib::endl;

    done_something = true;
 }}
  out << Lib::endl;

  out_end_pwd(out);

  if(fProdWhat==obuild::Builder::build_sh) {

  } else if(fProdWhat==obuild::Builder::build_bat) {

    out << ":obuild_return" << Lib::endl;
    out << "ENDLOCAL" << Lib::endl;
    out << "@ECHO ON" << Lib::endl;
  }

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(!done_something) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_iss(
 Lib::ItemML& aProjectML //<project>
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat!=obuild::Builder::build_bat) return true;

  std::string pj_name;
  if(!aProjectML.attributeValue("name",pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_iss :"
        << " no attribute name on the <project> XML item."
        << Lib::endl;
    return false;
  }

  std::string pj_version;
  if(!aProjectML.attributeValue("version",pj_version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_iss :"
        << " no attribute version on the <project> XML item."
        << Lib::endl;
    return false;
  }

  if(!check_name(pj_name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_iss :"
        << " problem with " << Lib::sout(pj_name)
        << ". Project name should have the syntax of a programming variable." 
        << Lib::endl;
    return false;
  }

  fPackage = pj_name;
  fVersion = pj_version;
  aProjectML.propertyValue("copyright",fCopyRight);
  aProjectML.propertyValue("publisher",fPublisher);
  aProjectML.propertyValue("url",fURL);

  std::string sfile = "."+f_sep+f_dir+f_sep+"this.iss";

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_iss :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  reset_local_flags("iss");

  out_template(out,header_iss_linen,header_iss_lines);

  bool done_something = false;
  
  done_something = true;

  std::string drel = 
    ".."+sep()+"Windows_NT_obuild"+sep()+fPackage+sep()+fVersion;

  std::string dapp = "{app}";
  std::string dbin = "{app}"+sep()+"bin";
  std::string dlib = "{app}"+sep()+"lib";
  std::string dres = "{app}"+sep()+"Resources";

  out << "[Files]" << Lib::endl;
  out << "; Applications, DLLS, Python modules :" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"bin"+sep()+"*.exe") << ";"
      << "DestDir:" 
      << dbin << ";"
      << "Components:RTK" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"bin"+sep()+"*.dll") << ";"
      << "DestDir:" 
      << dbin << ";"
      << "Components:RTK" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"bin"+sep()+"*.pyd") << ";"
      << "DestDir:" 
      << dbin << ";"
      << "Components:RTK" << Lib::endl;

  out << "; Libraries :" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"lib"+sep()+"*.lib") << ";"
      << "DestDir:" 
      << dlib << ";"
      << "Components:RTK" << Lib::endl;

  out << "; Resources :" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"Resources"+sep()+"*.*") << ";"
      << "DestDir:" 
      << dres << ";"
      << "Components:RTK;Flags:recursesubdirs" << Lib::endl;

  // The below assumes that there are README, License files.
  out << "; Else :" << Lib::endl;
  out << "Source:" 
      << (drel+sep()+"README") << ";"
      << "DestDir:" 
      << dapp << ";"
      << "Components:RTK" << Lib::endl;

  out << "Source:" 
      << (drel+sep()+"License") << ";"
      << "DestDir:" 
      << dapp << ";"
      << "Components:RTK" << Lib::endl;

  out << "Source:" 
      << (drel+sep()+"*.bat") << ";"
      << "DestDir:" 
      << dapp << ";"
      << "Components:RTK" << Lib::endl;

  out << "Source:" 
      << (drel+sep()+"bin"+sep()+"*.save") << ";"
      << "DestDir:" 
      << dbin << ";"
      << "Components:RTK" << Lib::endl;

/*
Source: Release\v15r0\tools\requirements; DestDir: {app}\cmt\; Components: BAT
*/

  out << Lib::endl;
  out_template(out,trailer_iss_linen,trailer_iss_lines);

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(!done_something) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_oxxx_cpp(
 const std::string& aName
,unsigned int aIncn
,const char* aIncs[]
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sfile = aName;
  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::out_oxxx_cpp :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  std::string s = Lib::smanip::tostring(aIncn,aIncs);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  out << s;

  fp.disable();

  return true;
}
//////////////////////////////////////////////////////////////////////////////
/// Secondary producer methods used by the main ones. ////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_save_sh(
 Lib::Out& out
,const std::string& aName
,const Subs& aSubs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  /// Get top directory :
  out_begin_pwd(out);

  out_cd_bin(out);

  out << "cd ../../.." << Lib::endl;
  out << "obuild_dir_top=`pwd`" << Lib::endl;

  out_end_pwd(out);

  if(!fProduced_local_subf_exe) {

    out_template2(out,build_subf_sh_linen,build_subf_sh_lines);

    fProduced_local_subf_exe = true;
    fProduced_subf_exe = true;
  }

  out_begin_pwd(out);

  out_cd_bin(out);

  out_rm(out,aName);

  out << "echo \"obuild_package"
      << "=" << fPackage << "\"" << " >> "
      << aName << Lib::endl;
 
  out << "echo \"obuild_version"
      << "=" << fVersion << "\"" << " >> "
      << aName << Lib::endl;
 
#define SAVE_SH(aWhat,aValue) \
  out << "echo \"" << #aWhat \
      << "=${" << #aValue << "}\"" << " >> " \
      << aName << Lib::endl;
 
  SAVE_SH(obuild_cxx_command,obuild_cxx_command)
  SAVE_SH(obuild_cxx_debug,obuild_cxx_debug)
  SAVE_SH(obuild_cxx_optimize,obuild_cxx_optimize)
  SAVE_SH(obuild_cxx_options,obuild_cxx_options)
  SAVE_SH(obuild_cxx_macros,obuild_cxx_macros)
  SAVE_SH(obuild_cxx_dll_builder_options,obuild_cxx_dll_builder_options)

  SAVE_SH(obuild_cxx_includes,obuild_cxx_includes)
  SAVE_SH(obuild_linked_paths,obuild_linked_paths)
  SAVE_SH(obuild_linked_libraries,obuild_linked_libraries)

  out << "./obuild_subf" 
      << " " << aName 
      << " " << quote(var_val("obuild_dir_top"))
      << " @@top@@" 
      << Lib::endl;
  out << "unset obuild_dir_top" << Lib::endl;

 {Subs::const_iterator it;
  for(it=aSubs.begin();it!=aSubs.end();++it) {
    out << "./obuild_subf" 
        << " " << aName 
        << " " << quote(var_val((*it).first))
        << " '" << quote(var_val((*it).second)) << "'" 
        << Lib::endl;
  }}

  out_end_pwd(out);

  out << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_save_bat(
 Lib::Out& out
,const std::string& aName
,const Subs& aSubs
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) return true;

  /// Get top directory :
  out_begin_pwd(out);

  out_cd_bin(out);

  out << "cd ..\\..\\.." << Lib::endl;
  out << "FOR /F \"usebackq delims=;\" %%d IN (`cd`)"
      << " DO SET obuild_dir_top=%%d"
      << Lib::endl;

  out_end_pwd(out);

  if(!fProduced_local_subf_exe) {

    out_template2(out,build_subf_bat_linen,build_subf_bat_lines);

    fProduced_local_subf_exe = true;
    fProduced_subf_exe = true;
  }

  out_begin_pwd(out);

  out_cd_bin(out);

  out_rm(out,aName);

  out << "ECHO obuild_package"
      << "=" << fPackage << " >> "
      << aName << Lib::endl;
 
  out << "ECHO obuild_package"
      << "=" << fVersion << " >> "
      << aName << Lib::endl;
 
#define SAVE_BAT(aWhat,aValue) \
  out << "ECHO " << #aWhat \
      << "=%" << #aValue << "%" << " >> " \
      << aName << Lib::endl;

  SAVE_BAT(obuild_cxx_command,obuild_cxx_command)
  SAVE_BAT(obuild_cxx_debug,obuild_cxx_debug)
  SAVE_BAT(obuild_cxx_optimize,obuild_cxx_optimize)
  SAVE_BAT(obuild_cxx_options,obuild_cxx_options)
  SAVE_BAT(obuild_cxx_macros,obuild_cxx_macros)
  SAVE_BAT(obuild_cxx_dll_builder_options,obuild_cxx_dll_builder_options)

  SAVE_BAT(obuild_cxx_includes,obuild_cxx_includes)
  SAVE_BAT(obuild_linked_paths,obuild_linked_paths)
  SAVE_BAT(obuild_linked_libraries,obuild_linked_libraries)

  out << ".\\obuild_subf.exe" 
      << " " << aName
      << " " << quote(var_val("obuild_dir_top"))
      << " @@top@@"
      << Lib::endl;

 {Subs::const_iterator it;
  for(it=aSubs.begin();it!=aSubs.end();++it) {
    out << ".\\obuild_subf.exe" 
        << " " << aName
        << " " << quote(var_val((*it).first))
        << " " << var_val(var_val((*it).second))
        << Lib::endl;
  }}

  out_end_pwd(out);

  out << "SET obuild_dir_top=" << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_check_foreigns_sh(
 Lib::ItemML& aItemML //top item
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_bat) return true;

  if(!check_directory(f_dir)) return false;
  std::string sfile = "."+f_sep+f_dir+f_sep+"check_foreigns"+f_sh;

  Lib::FileWriter fp(sfile,fPrinter);
  if(!fp.enabled()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_check_foreigns_sh :" 
        << " can't open file " << Lib::sout(sfile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  out_template(out,check_foreigns_header_sh_linen,
                   check_foreigns_header_sh_lines);

  // Build variables (especially the ones in foreigns :
  if(!out_variables(aItemML,out)) {
    fp.disable();
    Lib::fmanip::remove(sfile);
    return false;
  }

  bool empty = true;

  // foreigns :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"foreign",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::string foreign_name;
    (*it)->attributeValue("name",foreign_name);

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"check_directory",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index]; //"platform"
      const std::string& name = names[index];
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      out << "if [ \"${obuild_check_what}\" = " 
          << quote(foreign_name) << " ] ; then" << Lib::endl;
      out << "  if [ ! -d " << quote(name) << " ] ; then" << Lib::endl;
      if(value!="") {
        out << "    echo " << quote(value) << Lib::endl;
      } else {
        out << "    echo 'obuild_check : directory " 
            << Lib::sout(value) << " not found.'" << Lib::endl;
      }
      out << "    exit 1" << Lib::endl; //FIXME : have an atb : warn,fatal.
      out << "  fi" << Lib::endl;
      out << "fi" << Lib::endl;

      empty = false;

      if(has_test) out_end_platform(out,spif);
    }}

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"check_file",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index]; //"platform"
      const std::string& name = names[index];
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      out << "if [ \"${obuild_check_what}\" = " 
          << quote(foreign_name) << " ] ; then" << Lib::endl;
      out << "  if [ ! -e " << quote(name) << " ] ; then" << Lib::endl;
      if(value!="") {
        out << "    echo " << quote(value) << Lib::endl;
      } else {
        out << "    echo 'obuild_check : directory " 
            << Lib::sout(value) << " not found.'" << Lib::endl;
      }
      out << "    exit 1" << Lib::endl; //FIXME : have an atb : warn,fatal.
      out << "  fi" << Lib::endl;
      out << "fi" << Lib::endl;

      empty = false;

      if(has_test) out_end_platform(out,spif);
    }}

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"check_application",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index]; //"platform"
      const std::string& name = names[index];
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      out << "if [ \"${obuild_check_what}\" = " 
          << quote(foreign_name) << " ] ; then" << Lib::endl;

     {std::string s = "./sh/check_app"+f_sh;
      out << "  if [ -e " << s << " ] ; then" << Lib::endl;
      out << "    " << s << " " << name << " '" << value << "'" << Lib::endl;
      out_status(out,4);
      out << "  fi" << Lib::endl;
      out << Lib::endl;}

      out << "fi" << Lib::endl;

      empty = false;

      if(has_test) out_end_platform(out,spif);

      fProduced_check_app_sh = true;
    }}

   {std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"check_env",
          packs,verss,platforms,verbatims,names,ifs,vals)) {
      fp.disable();
      Lib::fmanip::remove(sfile);
      return false;
    }
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& platform = platforms[index]; //"platform"
      const std::string& name = names[index];
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM_FP

      std::string spif = if_count();
      bool has_test = out_begin_platform(out,platform,spif);

      out << "if [ \"${obuild_check_what}\" = " 
          << quote(foreign_name) << " ] ; then" << Lib::endl;
      out << "  if [  \"${" << name << "}\" = \"\" ] ; then" << Lib::endl;
      if(value!="") {
        out << "    echo " << quote(value) << Lib::endl;
      } else {
        out << "    echo 'obuild_check : environment variable " 
            << Lib::sout(value) << " not defined.'" << Lib::endl;
      }
      out << "    exit 1" << Lib::endl; //FIXME : have an atb : warn,fatal.
      out << "  fi" << Lib::endl;
      out << "fi" << Lib::endl;

      empty = false;

      if(has_test) out_end_platform(out,spif);
    }}


  }
  out << Lib::endl;}

  fp.disable();

  Lib::fmanip::chmod(sfile,Lib::fmanip::USER,
                           Lib::fmanip::ADD,
                           Lib::fmanip::EXECUTE);

  if(empty) {
    Lib::fmanip::remove(sfile);
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_template(
 Lib::Out& out
,unsigned int aLinen 
,const char* aLines[]
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = Lib::smanip::tostring(aLinen,aLines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");
  if(fVersion=="") 
    Lib::smanip::replace(s," with version @@version@@","");
  else 
    Lib::smanip::replace(s,"@@version@@",fVersion);

  if(Lib::smanip::replace(s,"@@component@@",fComponent)) {
    if(fComponent=="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_template :" 
          << " some @@component@@ found but fComponent was empty."
          << Lib::endl;
    }
  }

  Lib::smanip::replace(s,"@@copyright@@",fCopyRight);
  Lib::smanip::replace(s,"@@publisher@@",fPublisher);
  Lib::smanip::replace(s,"@@url@@",fURL);

  if(!replace_at_at(s)) return false;

  out << s;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_template2(
 Lib::Out& aOut
,unsigned int aLinen 
,const char* aLines[]
,bool aExit //false
) 
//////////////////////////////////////////////////////////////////////////////
// Used to import build_[find,platform].[sh,csh,bat]
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = Lib::smanip::tostring(aLinen,aLines);
  Lib::smanip::replace(s,"@@double_quote@@","\"");
  Lib::smanip::replace(s,"@@back_slash@@","\\");

  if(aExit) Lib::smanip::replace(s,"@@exit@@",fProduce_local_exit_sh);

  if(!replace_at_at(s)) return false;

  aOut << s;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::reset_local_flags(
 const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fProduced_local_what = aWhat;
  fProduced_local_platform_exe = false;
  fProduced_local_find_exe = false;
  fProduced_local_windef_exe = false;
  fProduced_local_subs_exe = false;
  fProduced_local_subf_exe = false;
  fProduced_local_LD_PATH = false;
  fProduce_local_exit_sh = "exit";
  fProduced_local_pwd_count = 0;
  fProduced_local_group_count = 0;
  fProduced_local_group_begin = false;
  fProduced_local_if_count = 0;
  fProduced_local_if_begin = false;
  fProduced_local_main_count = 0;
  fProduced_local_main_begin = false;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::if_count(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"%d",fProduced_local_if_count);
  fProduced_local_if_count++;
  return s;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::check_name(
 const std::string& aName
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = aName.size();
  if(!number) return false;
  for(unsigned int index=0;index<number;index++) {
    char c = aName[index];
    if(Lib::cmanip::isAlphanumeric(c)) continue;
    if(Lib::cmanip::isDigit(c)) continue;
    if(c=='_') continue;
    return false;
  }
  if(Lib::cmanip::isDigit(aName[0])) return false;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::check_env(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"build_check_env",
        packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& platform = platforms[index]; //"platform"
    const std::string& name = names[index];
    const std::string& value = vals[index];

    if(!Lib::System::isenv(name)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::check_env :"
          << " Environment variable " << Lib::sout(name) << " not defined."
          << Lib::endl;
      return false;
    }
  }
  return true; 
}
//////////////////////////////////////////////////////////////////////////////
Lib::ItemML* obuild::Builder::get_parent_package_ML(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::ItemML* parent = &aItemML;
  while(parent) {
    std::string tag = parent->tagName();
    if(tag=="package") return parent;
    parent = parent->parent();    
  }
  return 0;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::replace_at_at(
 std::string& aString
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::smanip::replace(aString,"@@obuild_version@@",OBUILD_VERSION);
  Lib::smanip::replace(aString,"@@package@@",fPackage);
  Lib::smanip::replace(aString,"@@version@@",fVersion);

  if(Lib::smanip::replace(aString,"@@component@@",fComponent)) {
    if(fComponent=="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::replace_at_at :" 
          << " some @@component@@ found but fComponent was empty."
          << Lib::endl;
    }
  }

  Lib::smanip::replace(aString,"@@path_variable@@",path_variable(fPackage));

  if(aString.find("@@platform_variable@@")!=std::string::npos) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::replace_at_at :" 
        << " @@platform_variable@@ no more supported."
        << Lib::endl;
    return false;
  } 

  Lib::smanip::replace(aString,"@@dir_bin@@",dir_bin(fPackage));
  Lib::smanip::replace(aString,"@@dir_package@@",dir_pack(fPackage));

  Lib::smanip::replace
    (aString,"@@dir_bin_variable@@",dir_bin_variable());

  Lib::smanip::replace(aString,"@@path_separator@@",path_sep());

  // Used by <zip> :
  Lib::smanip::replace
    (aString,"@@platform_signature@@",var_val("obuild_platform_signature"));

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_xxx_core_sh(
 Lib::ItemML& aItemML
,Lib::Out& out
,const std::string& aWhat //app, lib, dll
,const std::string& aName
,const std::vector<std::string>& aFiles
,const std::vector<std::string>& aPlatforms
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  reset_local_flags("build_"+aWhat+"_"+aName+"_sh");

  if(fProdWhat==obuild::Builder::build_sh) {
    out << "#!/bin/sh -f" << Lib::endl;
    out << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out << "@ECHO OFF" << Lib::endl;
    out << "SETLOCAL" << Lib::endl;
    out << Lib::endl;
  }

  //out_begin_build(aItemML,out); // "platform" variable not yet defined.

  if(fProdWhat==obuild::Builder::build_sh) {
    out_template(out,build_xxx_header_sh_linen,
                      build_xxx_header_sh_lines);
  } else if(fProdWhat==obuild::Builder::build_bat) {
    out_template(out,build_xxx_header_bat_linen,
                      build_xxx_header_bat_lines);
  }

  if(!out_not_done_on(aItemML,out)) return false;

  if(!out_check_foreigns(aItemML,out)) return false;
  if(!out_check_env(aItemML,out)) return false;

  // Build variables (especially the ones in foreigns :
  if(!out_variables(*fPackML,out)) return false;

  // Build the tools (here compilers) infos.
  // Per component infos will be appended to them.
  if(!out_begin_tool(aFiles,out)) return false;

  // Compilation macros :
  if(!out_compilation_macros(aItemML,"c_macro",out)) return false;
  if(!out_compilation_macros(aItemML,"cxx_macro",out)) return false;
  if(!out_compilation_macros(aItemML,"f77_macro",out)) return false;
  if(!out_compilation_macros(aItemML,"objc_macro",out)) return false;
  if(!out_compilation_macros(aItemML,"objcxx_macro",out)) return false;
  

  // Compilation options :
  if(!out_options(aItemML,"c_options",out)) return false;
  if(!out_options(aItemML,"cxx_options",out)) return false;
  if(!out_options(aItemML,"f77_options",out)) return false;
  if(!out_options(aItemML,"objc_options",out)) return false;
  if(!out_options(aItemML,"objcxx_options",out)) return false;
  
/*
  // Link options :
  if(!out_options(aItemML,"c_linker_options",out)) return false;
  if(!out_options(aItemML,"cxx_linker_options",out)) return false;
  if(!out_options(aItemML,"f77_linker_options",out)) return false;
  if(!out_options(aItemML,"objc_linker_options",out)) return false;
  if(!out_options(aItemML,"objcxx_linker_options",out)) return false;
*/

  if(!out_path_of_used_packages(aItemML,out)) return false;
  
  if(!out_if_done(aItemML,out)) return false; //Need paths.

  // Includes :
  if(!out_incs(aItemML,"c_include",out)) return false;  
  if(!out_incs(aItemML,"cxx_include",out)) return false;
  if(!out_incs(aItemML,"f77_include",out)) return false;
  if(!out_incs(aItemML,"objc_include",out)) return false;
  if(!out_incs(aItemML,"objcxx_include",out)) return false;
  
  if(fProdWhat==obuild::Builder::build_sh) {

    // Remove existing things :
    std::string dir = dir_bin(fPackage)+"/"+aWhat+"_"+aName;
    out << "if [ ${obuild_dir_bin_o_cleanup} = yes ] ; then" << Lib::endl;
    out << "  /bin/rm -R -f " << quote(dir) << Lib::endl;
    out << "fi" << Lib::endl;
    out << Lib::endl;
    out_mkdir(out,quote(dir));
    out << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    std::string dir = dir_bin(fPackage)+"\\"+aWhat+"_"+aName;
    out_rmdir(out,quote(dir));
    out_mkdir(out,quote(dir));
    out << Lib::endl;
  }

  out_pre_build(aItemML,out);

  if(!out_commands(aItemML,out)) return false;

  std::string lang;
  if(!guess_linker_language(aItemML,aFiles,lang)) return false;

  if(fProdWhat==obuild::Builder::build_sh) {
    // Compilations :
    out << "# Compile " << aName << " files :" << Lib::endl;
    out << "if [ ${obuild_do_not_compile} = no ] ; then" << Lib::endl;
    out << Lib::endl;

    out_verbose_on(out);

    if(lang=="java") {

      bool empty;
      if(!out_linked_jars(aItemML,out,empty)) return false;

      // Compile all the files in one command :
      //bool has_test = out_begin_platform(out,platform);
  
      std::string jfiles;
      unsigned int number = aFiles.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& file = aFiles[index];
        if(jfiles!="") jfiles += " ";  
        jfiles += tounix(file);
      }

      std::string slnk = "-classpath "+quote("${obuild_linked_jars}");
      if(empty) slnk = "";

      std::string dir = dir_bin(fPackage)+sep()+aWhat+"_"+aName;

      //out << "eval"
      out << " ${obuild_java_command}"
          << " ${obuild_java_debug_local}"
          << " -d " << quote(dir) << " "
          << " " << slnk
          << " " << jfiles
          << Lib::endl;
      out_status(out);
      out << Lib::endl;

      //if(has_test) out_end_platform(out);

    } else {
      unsigned int number = aFiles.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& file = aFiles[index];
        const std::string& platform = aPlatforms[index]; //"platform"
        BUILD_ON_PLATFORM

        std::string spif = if_count();
        bool has_test = out_begin_platform(out,platform,spif);

        bool empty;
        out_compile_file(file,aWhat+"_"+aName,out,empty);
        if(empty) out_empty(out);
  
        if(has_test) out_end_platform(out,spif);
      }
    }

    out_verbose_off(out);
    out << Lib::endl;
    out << "fi #do_not_compile" << Lib::endl;
    out << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    if(lang=="java") {

      bool empty;
      if(!out_linked_jars(aItemML,out,empty)) return false;

      // Compile all the files in one command :
      //bool has_test = out_begin_platform(out,platform);
  
      std::string jfiles;
      unsigned int number = aFiles.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& file = aFiles[index];
        if(jfiles!="") jfiles += " ";  
        jfiles += towin(file);
      }

      std::string slnk = "-classpath %obuild_linked_jars%";
      if(empty) slnk = "";

      std::string dir = dir_bin(fPackage)+sep()+aWhat+"_"+aName;
      out << "%obuild_java_command%"
           << " %obuild_java_debug_local%"
           << " -d " << quote(dir) << " "
           << " " << slnk
           << " " << jfiles
           << Lib::endl;
      out_status(out);
      out << Lib::endl;

      //if(has_test) out_end_platform(out);

    } else {

      out_verbose_on(out);
      unsigned int number = aFiles.size();
      for(unsigned int index=0;index<number;index++) {
        const std::string& file = aFiles[index];
        const std::string& platform = aPlatforms[index]; //"platform"
        BUILD_ON_PLATFORM
  
        std::string spif = if_count();
        bool has_test = out_begin_platform(out,platform,spif);

        bool empty;
        out_compile_file(file,aWhat+"_"+aName,out,empty);
        if(empty) out_empty(out);
  
        if(has_test) out_end_platform(out,spif);
      }
      out_verbose_off(out);
      out << Lib::endl;

    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_build_xxx_end_sh(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,const std::string& aWhat
,const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Cleanup :
  if(fProdWhat==obuild::Builder::build_sh) {

    std::string dir = dir_bin(fPackage)+"/"+aWhat+"_"+aName;
    aOut << "# Some cleanup :" << Lib::endl;
    aOut << "if [ ${obuild_dir_bin_o_cleanup} = yes ] ; then" << Lib::endl;
    aOut << "  /bin/rm -R -f " << quote(dir) << Lib::endl;
    aOut << "fi" << Lib::endl;
    out_rm(aOut,quote(dir_bin(fPackage)+"/obuild_find"));
    aOut << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    std::string d_bin = dir_bin(fPackage);
    std::string sobs = d_bin+"\\"+aWhat+"_"+aName+"\\tmp_objs";
    aOut << "REM # Some cleanup :" << Lib::endl;
    out_rm(aOut,quote(sobs));

    if(aWhat=="app") {
      std::string silk = d_bin+"\\" + aName + ".ilk";
      std::string spdb = d_bin+"\\" + aName + ".pdb";
      std::string sobs = d_bin+"\\"+aWhat+"_" + aName + "\\tmp_objs";
      out_rm(aOut,quote(silk));
      out_rm(aOut,quote(spdb));
      out_rm(aOut,quote(sobs));
    }

    std::string sexp = d_bin+"\\" + aName + ".exp";
    out_rm(aOut,quote(sexp));

    aOut << Lib::endl;
  }

  out_post_build(aItemML,aOut);

  if(fProdWhat==obuild::Builder::build_sh) {
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << ":obuild_return" << Lib::endl;
    aOut << "ENDLOCAL" << Lib::endl;
    aOut << "@ECHO ON" << Lib::endl;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_append(
 Lib::Out& aOut
,const std::string& aWhat
,const std::string& aValue
,bool aFirst
,bool aDirSep
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aValue=="") return true;
  if(fProdWhat==obuild::Builder::build_sh) {
    std::string sp = aDirSep?":":" ";
    if(aFirst) {
      aOut << aWhat << "=" << quote(aValue) <<Lib::endl;
    } else {
      std::string s = "${"+aWhat+"}"+sp+aValue;
      aOut << aWhat << "=" << quote(s) << Lib::endl;
    }
  } else if(fProdWhat==obuild::Builder::build_bat) {
    std::string sp = aDirSep?";":" ";
    if(aFirst) {
      aOut << "SET " << aWhat << "=" << aValue << "" <<Lib::endl;
    } else {
      aOut << "SET " << aWhat << "=%" << aWhat << "%" << sp 
           << aValue << Lib::endl;
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_append_csh(
 Lib::Out& aOut
,const std::string& aWhat
,const std::string& aValue
,bool aFirst
,bool aDirSep
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aValue=="") return true;
  if(fProdWhat==obuild::Builder::build_bat) return true;

  std::string sp = aDirSep?":":" ";
  if(aFirst) {
    aOut << "set " << aWhat << "=" << quote(aValue) <<Lib::endl;
  } else {
    std::string s = "${"+aWhat+"}"+sp+aValue;
    aOut << "set " << aWhat << "=" << quote(s) << Lib::endl;
  }
  return true;
}
/*
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_build(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  //FIXME : should handle platform.
  //FIXME : can't handle platform since the shell platform variable
  //FIXME : is not yet defined !
  //FIXME : should stop things is someone attempt to have "platform".

  //if(fProdWhat==obuild::Builder::build_bat) return true;

  // pre_build scripts :

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"begin_build",
                packs,verss,platforms,verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  if(number) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_build :"
        << " <begin_build> no more supported."
        << Lib::endl;
    return false;
  }

  if(number) aOut << f_cmt << " begin_build scripts :" << Lib::endl;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    if(platform!="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_begin_build :"
          << " platform not authorized on <begin_build>."
          << Lib::endl;
      return false;
    }
    aOut << value << Lib::endl;
  }
  if(number) aOut << Lib::endl;}

  return true;
}
*/
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_pre_build(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"pre_build",
            packs,verss,platforms,verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  if(number) aOut << f_cmt << " pre_build scripts :" << Lib::endl;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    if(value=="") continue;

    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    aOut << value << Lib::endl;

    if(has_test) out_end_platform(aOut,spif);

  }
  if(number) aOut << Lib::endl;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_post_build(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"post_build",
            packs,verss,platforms,verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  if(number) aOut << f_cmt << " post_build scripts :" << Lib::endl;
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    if(value=="") continue;

    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    aOut << value << Lib::endl;

    if(has_test) out_end_platform(aOut,spif);

  }
  if(number) aOut << Lib::endl;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_path_sh(
 const std::string& aPack
,const std::string& aVers
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  packs.push_back(aPack);
  std::vector<std::string> verss;
  verss.push_back(aVers);
  return out_paths_sh(packs,verss,aOut);
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_paths_sh(
 const std::vector<std::string> aPacks
,const std::vector<std::string> aVerss
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = aPacks.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = aPacks[index];
    if(pack=="this") {
    } else if(pack=="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_paths_sh :"
          << " empty package name."
          << Lib::endl;
      return false;
    } else {
      const std::string& vers = aVerss[index];
      if(fProdWhat==obuild::Builder::build_sh) {

        if(!fProduced_local_find_exe) {
          out_template2(aOut,build_find_sh_linen,build_find_sh_lines,true);

          fProduced_local_find_exe = true;
          fProduced_find_exe = true;
        }

        aOut << "if [ \"${obuild_find_exe}\" = \"\" ] ; then" 
             << Lib::endl;
        aOut << "  echo \"Variable obuild_find_exe not defined.\"" 
             << Lib::endl;
        aOut << "  " << fProduce_local_exit_sh << Lib::endl;
        aOut << "fi"  << Lib::endl;

        aOut << path_variable(pack) << "=`\"${obuild_find_exe}\"" 
            << " " << pack << " '" << vers << "'" << "`" 
            << Lib::endl;
        aOut << "if [ \"${" << path_variable(pack) << "}\" = \"\" ] ; then" 
             << Lib::endl;
        aOut << "  echo \"Can't find " << pack << " package.\"" << Lib::endl;
        aOut << "  " << fProduce_local_exit_sh << Lib::endl;
        aOut << "fi"  << Lib::endl;

      } else if(fProdWhat==obuild::Builder::build_bat) {

        if(!fProduced_local_find_exe) {

          out_template2(aOut,build_find_bat_linen,build_find_bat_lines);

          fProduced_local_find_exe = true;
          fProduced_find_exe = true;
        }

        out_begin_pwd(aOut);

        out_cd_bin(aOut);

        aOut << "IF NOT EXIST obuild_find.exe"
             << "  ECHO Program obuild_find.exe not found."
             << "  && GOTO obuild_return" 
             << Lib::endl;

        aOut << "FOR /F \"usebackq delims=;\" %%j IN (`"
             << ".\\obuild_find.exe " 
             << " " << pack << " " << vers
             << "`)"
             << " DO SET " << path_variable(pack) << "=%%j" 
             << Lib::endl;

        out_end_pwd(aOut);

        aOut << "IF NOT DEFINED " << path_variable(pack)
             << "  ECHO Can't find " << pack << " package."
             << "  && GOTO obuild_return" 
             << Lib::endl;

      }
    }
  }
  if(number) aOut << Lib::endl;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_paths_csh(
 const std::vector<std::string> aPacks
,const std::vector<std::string> aVerss
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = aPacks.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = aPacks[index];
    if(pack=="this") {
    } else if(pack=="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_paths_sh :"
          << " empty pack name."
          << Lib::endl;
      return false;
    } else {
      const std::string& vers = aVerss[index];

      if(!fProduced_local_find_exe) {

        out_template2(aOut,build_find_csh_linen,build_find_csh_lines);

        fProduced_local_find_exe = true;
        fProduced_find_exe = true;
      }

      aOut << "if ( \"${obuild_find_exe}\" == \"\" ) then" << Lib::endl;
      aOut << "  echo \"Variable obuild_find_exe not defined.\"" << Lib::endl;
      aOut << "  exit" << Lib::endl;
      aOut << "endif"  << Lib::endl;

      aOut << "set " << path_variable(pack) << "=`\"${obuild_find_exe}\" " 
          << pack << " '" << vers << "'" << "`" 
          << Lib::endl;
    }
  }
  if(number) aOut << Lib::endl;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_path_of_used_packages(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> vals;

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
      "c_include",packs2,verss2,platforms2,
                   verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
        "cxx_include",packs2,verss2,platforms2,
                      verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
        "f77_include",packs2,verss2,platforms2,
                      verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
        "objc_include",packs2,verss2,platforms2,
                      verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
        "objcxx_include",packs2,verss2,platforms2,
                      verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
                  "lib",packs2,verss2,platforms2,
                        verbatims2,names2,ifs2,vals2)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

 {std::vector<std::string> packs2;
  std::vector<std::string> verss2;
  std::vector<std::string> platforms2;
  std::vector<std::string> verbatims2;
  std::vector<std::string> names2;
  std::vector<std::string> ifs2;
  std::vector<std::string> vals2;
  if(!getProperties(aItemML,
                  "path",packs2,verss2,platforms2,
                        verbatims2,names2,ifs2,vals2,true)) return false;
  Lib::append<std::string>(packs,packs2);  
  Lib::append<std::string>(verss,verss2);  
  Lib::append<std::string>(vals,vals2);  
  }

  if(!out_paths_sh(packs,verss,aOut)) return false;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_incs(
 Lib::ItemML& aItemML
,const std::string& aWhat
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,aWhat,packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;

  std::string variable = "obuild_"+aWhat+"s";

  bool done_something = false;

  bool first = true; //then init the variable too.

  unsigned int number = names.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& inc = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    const std::string& if_s = ifs[index]; //"if"
    BUILD_ON_PLATFORM

    std::string sif = if_count();
    if(if_s!="") if(!out_begin_if(aOut,if_s,sif)) return false;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    bool done_something_local = false;

    if(verbatims[index]=="yes") {
      if(inc!="") {
        if(!out_append(aOut,variable,inc,first)) return false;
        if(first) first = false;
        done_something_local = true;
      }
    } else {
      if(pack=="this") {
        if(inc!="") {
          if(fProdWhat==obuild::Builder::build_sh) {
            std::string sinc = "-I"+quote(tounix(inc));
            if(!out_append(aOut,variable,esc_quote(sinc),first)) 
              return false;
          } else if(fProdWhat==obuild::Builder::build_bat) {
            std::string sinc = "/I"+quote(towin(inc));
            if(!out_append(aOut,variable,sinc,first)) 
              return false;
          }
          if(first) first = false;
          done_something_local = true;
        }
      } else {
        if(inc=="") {
          if(fProdWhat==obuild::Builder::build_sh) {
            std::string sinc = "-I"+quote(dir_pack(pack));
            if(!out_append(aOut,variable,esc_quote(sinc),first)) 
              return false;
          } else if(fProdWhat==obuild::Builder::build_bat) {
            std::string sinc = "/I"+quote(dir_pack(pack));
            if(!out_append(aOut,variable,sinc,first)) 
              return false;
          }
          if(first) first = false;
          done_something_local = true;
        } else {
          if(fProdWhat==obuild::Builder::build_sh) {
            std::string sinc = "-I"+quote(dir_pack(pack)+"/"+tounix(inc));
            if(!out_append(aOut,variable,esc_quote(sinc),first)) 
              return false;
          } else if(fProdWhat==obuild::Builder::build_bat) {
            std::string sinc = "/I"+quote(dir_pack(pack)+"\\"+towin(inc));
            if(!out_append(aOut,variable,sinc,first)) 
              return false;
          }
          if(first) first = false;
          done_something_local = true;
        }
      }
    }

    if(!done_something_local) out_empty(aOut);
    else done_something = true;

    if(has_test) out_end_platform(aOut,spif);

    if(if_s!="") out_end_if(aOut,sif);
  }

  if(done_something) aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_platform(
 Lib::Out& aOut
,const std::string& aPlatform
,const std::string& a_sif
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPlatform=="") return false;

  if(fProdWhat==obuild::Builder::build_sh) {

    if(!fProduced_local_platform_exe) {
      out_template2(aOut,build_platform_sh_linen,
                          build_platform_sh_lines,true);

      fProduced_local_platform_exe = true;
      fProduced_platform_exe = true;
    }

    aOut << "if [ \"${obuild_platform_exe}\" = \"\" ] ; then" << Lib::endl;
    aOut << "  echo \"Variable obuild_platform_exe not defined.\"" 
         << Lib::endl;
    aOut << "  " << fProduce_local_exit_sh << Lib::endl;
    aOut << "fi"  << Lib::endl;

    aOut << "obuild_platform_result"
         << "=`\"${obuild_platform_exe}\""
         << " '" << aPlatform << "'`" 
         << Lib::endl;
    aOut << "if [ ${obuild_platform_result} = yes ] ; then" << Lib::endl;

    return true;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    if(!fProduced_local_platform_exe) {
      out_template2(aOut,build_platform_bat_linen,
                          build_platform_bat_lines);

      fProduced_local_platform_exe = true;
      fProduced_platform_exe = true;
    }


    out_begin_pwd(aOut);

    out_cd_bin(aOut);

    aOut << "IF NOT EXIST .\\obuild_platform.exe"
         << "  ECHO Program obuild_platform.exe not found."
         << "  && GOTO obuild_return" 
         << Lib::endl;

    aOut << "FOR /F \"usebackq\" %%j IN (`"
         << ".\\obuild_platform.exe " 
         << " \"" << aPlatform << "\""
         << "`)"
         << " DO SET obuild_platform_result=%%j" 
         << Lib::endl;

    out_end_pwd(aOut);

    aOut << "IF NOT %obuild_platform_result% == yes GOTO end_if_platform_" 
         << a_sif 
         << Lib::endl;

    return true;

  } else {

    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_else_platform(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "else" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    //aOut << ") ELSE (" << Lib::endl;
    Lib::Out out(fPrinter);
    out << "obuild::out_else_platform :"
        << " DOS not supported."
        << Lib::endl;
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_end_platform(
 Lib::Out& aOut
,const std::string& a_sif
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "fi" << Lib::endl;
    aOut << "unset obuild_platform_result" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << ":end_if_platform_" << a_sif << Lib::endl;
    aOut << "SET obuild_platform_result=" << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_platform_csh(
 Lib::Out& aOut
,const std::string& aPlatform
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPlatform=="") return false;

  if(fProdWhat==obuild::Builder::build_sh) {

    if(!fProduced_local_platform_exe) {
      out_template2(aOut,build_platform_csh_linen,
                          build_platform_csh_lines);

      fProduced_local_platform_exe = true;
      fProduced_platform_exe = true;
    }

    aOut << "if ( \"${obuild_platform_exe}\" == \"\" ) then" << Lib::endl;
    aOut << "  echo \"Variable obuild_platform_exe not defined.\"" 
         << Lib::endl;
    aOut << "  " << fProduce_local_exit_sh << Lib::endl;
    aOut << "endif"  << Lib::endl;

    aOut << "set obuild_platform_result" 
         << "=`\"${obuild_platform_exe}\""
         //<< " ${" << s << "}" 
         << " '" << aPlatform << "'`" 
         << Lib::endl;
    aOut << "if ( ${obuild_platform_result} == yes ) then" << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    return false;

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_else_platform_csh(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "else" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_end_platform_csh(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "endif" << Lib::endl;
    aOut << "unset obuild_platform_result" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_platform_signature(
 Lib::Out& aOut
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {

    if(!fProduced_local_platform_exe) {
      out_template2(aOut,build_platform_sh_linen,
                          build_platform_sh_lines,true);

      fProduced_local_platform_exe = true;
      fProduced_platform_exe = true;
    }

    aOut << "if [ \"${obuild_platform_exe}\" = \"\" ] ; then" << Lib::endl;
    aOut << "  echo \"Variable obuild_platform_exe not defined.\"" 
         << Lib::endl;
    aOut << "  " << fProduce_local_exit_sh << Lib::endl;
    aOut << "fi"  << Lib::endl;

    aOut << "obuild_platform_signature"
         << "=`\"${obuild_platform_exe}\"`"
         << Lib::endl;

    return true;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    if(!fProduced_local_platform_exe) {
      out_template2(aOut,build_platform_bat_linen,
                          build_platform_bat_lines);

      fProduced_local_platform_exe = true;
      fProduced_platform_exe = true;
    }


    out_begin_pwd(aOut);

    out_cd_bin(aOut);

    aOut << "IF NOT EXIST .\\obuild_platform.exe"
         << "  ECHO Program obuild_platform.exe not found."
         << "  && GOTO obuild_return" 
         << Lib::endl;

    aOut << "FOR /F \"usebackq\" %%j IN (`"
         << ".\\obuild_platform.exe`)" 
         << " DO SET obuild_platform_signature=%%j" 
         << Lib::endl;

    out_end_pwd(aOut);

    return true;

  } else {

    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_if(
 Lib::Out& aOut
,const std::string& aIf
,const std::string& a_sif
,bool a_csh //false
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  bool if_eq = true;
  std::vector<std::string> words = Lib::smanip::words(aIf,"!=");
  if(words.size()==2) {
    if_eq = false;
  } else {
    words = Lib::smanip::words(aIf,"=");
    if(words.size()!=2) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_begin_if :" 
          << " bad syntax in "  << Lib::sout(aIf)
          << Lib::endl;
      return false;
    }
    if_eq = true;
  }
  std::string var = words[0];
  if(var=="") {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_if :" 
        << " bad syntax in "  << Lib::sout(aIf)
        << ". Some variable expected."
        << Lib::endl;
    return false;
  }
  std::string value = words[1];
  if(value=="") {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_if :" 
        << " bad syntax in "  << Lib::sout(aIf)
        << ". Some value expected."
        << Lib::endl;
    return false;
  }

  if(fProduced_local_if_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_if :" 
        << " begin / end mismatch." 
        << Lib::endl;
    return false;
  }
  fProduced_local_if_begin = true;

  if(fProdWhat==obuild::Builder::build_sh) {
    if(a_csh) {
      if(if_eq) {
        aOut << "if ( $?" << var << " ) then" << Lib::endl;
        aOut << "if ( " << quote(var_val(var)) << " == " 
             << quote(value) << " ) then" 
             << Lib::endl;
      } else {
        aOut << "if ( $?" << var << " ) then" << Lib::endl;
        aOut << "if ( " << quote(var_val(var)) << " != " 
             << quote(value) << " ) then" 
             << Lib::endl;
      }
    } else {
      if(if_eq) {
        aOut << "if [ " << quote(var_val(var)) << " = " 
             << quote(value) << " ] ; then" 
             << Lib::endl;
      } else {
        aOut << "if [ " << quote(var_val(var)) << " != " 
             << quote(value) << " ] ; then" 
             << Lib::endl;
      }
    }
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF NOT DEFINED " << var
         << " GOTO end_if_if_" << fProduced_local_what << "_" << a_sif 
         << Lib::endl;
    if(if_eq) {
      aOut << "IF NOT " << var_val(var) << " == " << value
           << " GOTO end_if_if_" << fProduced_local_what << "_" << a_sif
           << Lib::endl;
    } else {
      aOut << "IF NOT " << var_val(var) << " != " << value
           << " GOTO end_if_if_" << fProduced_local_what << "_" << a_sif
           << Lib::endl;
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_end_if(
 Lib::Out& aOut
,const std::string& a_sif
,bool a_csh //false
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fProduced_local_if_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_end_if :" 
        << " begin / end mismatch." 
        << Lib::endl;
    //return false;
  }
  fProduced_local_if_begin = false;

  if(fProdWhat==obuild::Builder::build_sh) {
    if(a_csh) {
      aOut << "endif" << Lib::endl;
      aOut << "endif" << Lib::endl;
    } else {
      aOut << "fi" << Lib::endl;
    }
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << ":end_if_if_" << fProduced_local_what << "_" << a_sif << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_begin_group(
 Lib::Out& aOut
,const std::string& aGroup
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProduced_local_group_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_group :" 
        << " begin / end mismatch." 
        << Lib::endl;
    //return false;
  }
  fProduced_local_group_begin = true;

  std::string s;
  Lib::smanip::printf(s,32,"%d",fProduced_local_group_count);

  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "if [ \"${obuild_group}\" = " << aGroup << " ] ; then" 
         << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF NOT %obuild_group% == " << aGroup 
         << " GOTO end_if_group_" << s 
         << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_end_group(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fProduced_local_group_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_end_group :" 
        << " begin / end mismatch." 
        << Lib::endl;
    //return false;
  }
  fProduced_local_group_begin = false;

  std::string s;
  Lib::smanip::printf(s,32,"%d",fProduced_local_group_count);

  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "fi" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << ":end_if_group_" << s << Lib::endl;
  }

  fProduced_local_group_count++;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_begin_main(
 Lib::Out& aOut
,const std::string& aYesNo
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProduced_local_main_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_main :" 
        << " begin / end mismatch." 
        << Lib::endl;
    //return false;
  }
  fProduced_local_main_begin = true;

  std::string s;
  Lib::smanip::printf(s,32,"%d",fProduced_local_main_count);

  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "if [ \"${obuild_do_mains}\" = " << aYesNo << " ] ; then" 
         << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF NOT %obuild_do_mains% == " << aYesNo
         << " GOTO end_if_main_" << s 
         << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_end_main(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fProduced_local_main_begin) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_end_main :" 
        << " begin / end mismatch." 
        << Lib::endl;
    //return false;
  }
  fProduced_local_main_begin = false;

  std::string s;
  Lib::smanip::printf(s,32,"%d",fProduced_local_main_count);

  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "fi" << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << ":end_if_main_" << s << Lib::endl;
  }

  fProduced_local_main_count++;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_linked_libs(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"lib",packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;

  // Need to separate the load paths because 
  // of the "xxx-config --libs" programs.
  std::string vlkpaths = "obuild_linked_paths";
  std::string vlklibs = "obuild_linked_libraries";

  bool done_something = false;

  bool first = true; //then init the variable too.

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& lib = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    const std::string& if_s = ifs[index]; //"if"
    BUILD_ON_PLATFORM

    //printf("debug : %s %s\n",lib.c_str(),platform.c_str());

    std::string sif = if_count();
    if(if_s!="") if(!out_begin_if(aOut,if_s,sif)) return false;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    bool done_something_local = false;

    if(verbatims[index]=="yes") {
      if(lib!="") {
        if(!out_append(aOut,vlklibs,lib,first)) return false;
        if(first) first = false;
        done_something_local = true;
      }  
    } else {

      if(lib!="") {
        std::string path;
        if(pack=="this") {
          path = dir_bin(fPackage);
        } else {    
          path = dir_bin(pack);
        }

        if(fProdWhat==obuild::Builder::build_sh) {

         {std::string s = "-L"+esc_quote(quote(path));
          if(!out_append(aOut,vlkpaths,s,first)) return false;}
         {std::string s = " -l" + lib;
          if(!out_append(aOut,vlklibs,s,first)) return false;}
          if(first) first = false;

        } else if(fProdWhat==obuild::Builder::build_bat) {

         {std::string s = "/LIBPATH:"+quote(path);
          if(!out_append(aOut,vlkpaths,s,first)) return false;}
         {std::string s = lib+".lib";
          if(!out_append(aOut,vlklibs,s,first)) return false;}
          if(first) first = false;

        }

        done_something_local = true;
      }
    }

    if(!done_something_local) out_empty(aOut);
    else done_something = true;

    if(has_test) out_end_platform(aOut,spif);

    if(if_s!="") out_end_if(aOut,sif);

  }
  if(done_something) aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_linked_jars(
 Lib::ItemML& aItemML
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aEmpty = true;

  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"lib",packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;

  std::string variable = "obuild_linked_jars";

  bool done_something = false;

  bool first = true; //then init the variable too.

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& pack = packs[index];
    const std::string& lib = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    bool done_something_local = false;

    if(verbatims[index]=="yes") {
      if(lib!="") {
        if(!out_append(aOut,variable,lib,first,true)) return false;
        if(first) first = false;
        done_something_local = true;
      }  
    } else {

      if(lib!="") {
        std::string path;
        if(pack=="this") {
          path = dir_bin(fPackage);
        } else {    
          path = dir_bin(pack);
        }

        std::string s = path+sep()+lib+".jar";
        if(fProdWhat==obuild::Builder::build_sh) {
          if(!out_append(aOut,variable,quote(s),first,true)) return false;
          if(first) first = false;
        } else if(fProdWhat==obuild::Builder::build_bat) {
          if(!out_append(aOut,variable,quote(s),first,true)) return false;
          if(first) first = false;
        }

        done_something_local = true;
      }
    }

    if(!done_something_local) out_empty(aOut);
    else done_something = true;

    if(has_test) out_end_platform(aOut,spif);

  }
  if(done_something) aOut << Lib::endl;

  aEmpty = !done_something;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_status(
 Lib::Out& aOut
,unsigned int aIndent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<aIndent;index++) aOut << " ";
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "obuild_status=$?;if [ ${obuild_status} != 0 ] ;" 
         << " then exit ${obuild_status};fi" 
         <<Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "@IF ERRORLEVEL 1 @GOTO obuild_return" <<Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_verbose_on(
 Lib::Out& aOut
,unsigned int aIndent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<aIndent;index++) aOut << " ";
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "if [ ${obuild_verbose} = yes ] ; then set -x;fi" 
         << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "@IF %obuild_verbose% == yes @ECHO ON " 
         << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_verbose_off(
 Lib::Out& aOut
,unsigned int aIndent
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  for(unsigned int index=0;index<aIndent;index++) aOut << " ";
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "if [ ${obuild_verbose} = yes ] ; then set +x;fi" 
         << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "@IF %obuild_verbose% == yes @ECHO OFF " 
         << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_empty(
 Lib::Out& aOut
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "echo > /dev/null" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "@ECHO > NUL " << Lib::endl;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_var_pwd(
 Lib::Out& aOut
,const std::string& aVariable
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << aVariable << "=`pwd`" << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "FOR /F \"usebackq delims=;\" %%d IN (`cd`)"
         << " DO SET " << aVariable << "=%%d"
         << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_pwd(
 Lib::Out& aOut
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  Lib::smanip::printf(s,32,"obuild_pwd_%s_%d",
    fPackage.c_str(),fProduced_local_pwd_count);
  fProduced_local_pwd_count++;
  return out_var_pwd(aOut,s);
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_end_pwd(
 Lib::Out& aOut
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  fProduced_local_pwd_count--;
  std::string s;

  Lib::smanip::printf(s,32,"obuild_pwd_%s_%d",
    fPackage.c_str(),fProduced_local_pwd_count);

  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "cd \"${" << s << "}\"" << Lib::endl;
    aOut << "unset " << s << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "cd \"%" << s << "%\"" << Lib::endl;
    aOut << "SET " << s << "=" << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_cd(
 Lib::Out& aOut
,const std::string& aDir
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "cd " << aDir << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "CD " << aDir << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_rmdir(
 Lib::Out& aOut
,const std::string& aDir
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "/bin/rm -R -f " << aDir << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF EXIST " << aDir << " RMDIR /S /Q " << aDir << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_mkdir(
 Lib::Out& aOut
,const std::string& aDir
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "/bin/mkdir -p " << aDir << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF NOT EXIST " << aDir << " MKDIR " << aDir << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_rm(
 Lib::Out& aOut
,const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "/bin/rm -f " << aFile << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "IF EXIST " << aFile << " DEL " << aFile << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_copy(
 Lib::Out& aOut
,const std::string& aFrom
,const std::string& aTo
,const std::string& aOptions
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    if(aOptions!="") {
      aOut << "/bin/cp -f" 
           << " " << aOptions
           << " " << aFrom 
           << " " << aTo 
           << Lib::endl;
    } else {
      aOut << "/bin/cp -f" 
           << " " << aFrom 
           << " " << aTo 
           << Lib::endl;
    }
  } else if(fProdWhat==obuild::Builder::build_bat) {
    if(aOptions!="") {
      aOut << "COPY" 
           << " " << aOptions
           << " " << aFrom 
           << " " << aTo 
           << " 1> copy_exe_error"
           << Lib::endl;
    } else {
      aOut << "COPY" 
           << " " << aFrom 
           << " " << aTo 
           << " 1> copy_exe_error"
           << Lib::endl;
    }
    aOut << "@IF ERRORLEVEL 1 @TYPE copy_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "@IF NOT ERRORLEVEL 1 @DEL copy_exe_error" 
         << Lib::endl;

  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_copy_check(
 Lib::Out& aOut
,const std::string& aFrom
,const std::string& aTo
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "if [ -e " << aFrom << " ] ; then"
         << "  /bin/cp -f " << aFrom << " " << aTo << " ;"
         << "fi" 
         << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    std::string sif = if_count();
    aOut << "IF NOT EXIST " << aFrom << " GOTO end_if_copy_" << sif
         << Lib::endl;
    aOut << "  COPY " << aFrom << " " << aTo << " 1> copy_exe_error"
         << Lib::endl;
    aOut << "  @IF ERRORLEVEL 1 @TYPE copy_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "  @IF NOT ERRORLEVEL 1 @DEL copy_exe_error" 
         << Lib::endl;
    aOut << ":end_if_copy_" << sif << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_xcopy(
 Lib::Out& aOut
,const std::string& aFrom
,const std::string& aTo
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return false;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    //CVS directories not copied by the below (/E is for hidden files).
    aOut << "XCOPY /E /Q" 
         << " " << aFrom 
         << " " << aTo 
         << " 1> xcopy_exe_error"
         << Lib::endl;
    aOut << "@IF ERRORLEVEL 1 @TYPE xcopy_exe_error && @GOTO obuild_return" 
         << Lib::endl;
    aOut << "@IF NOT ERRORLEVEL 1 @DEL xcopy_exe_error" 
         << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_move(
 Lib::Out& aOut
,const std::string& aFrom
,const std::string& aTo
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    aOut << "/bin/mv " << aFrom << " " << aTo << Lib::endl;
  } else if(fProdWhat==obuild::Builder::build_bat) {
    aOut << "MOVE " << aFrom << " " << aTo << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_variables(
 Lib::ItemML& aPackML //<package> or <project>
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Get variables in the "foreign" :
  std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aPackML,"foreign",list);
  Lib::ItemML::unique(list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"variable",packs,verss,platforms,
                                    verbatims,names,ifs,vals)) return false;
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& name = names[index];
      const std::string& platform = platforms[index]; //"platform"
      const std::string& if_s = ifs[index]; //"if"
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM

      std::string sif = if_count();

      if(if_s!="") if(!out_begin_if(aOut,if_s,sif)) return false;
 
      std::string spif = if_count();
      bool has_test = out_begin_platform(aOut,platform,spif);

      bool first = true;
      if(!out_append(aOut,name,value,first)) return false;

      if(has_test) out_end_platform(aOut,spif);

      if(if_s!="") out_end_if(aOut,sif);
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_variables_csh(
 Lib::ItemML& aPackML //<package> or <project>
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Get variables in the "foreign" :
  std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aPackML,"foreign",list);
  Lib::ItemML::unique(list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(*(*it),"variable",packs,verss,platforms,
                                    verbatims,names,ifs,vals)) return false;
    unsigned int number = vals.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& name = names[index];
      const std::string& platform = platforms[index]; //"platform"
      const std::string& if_s = ifs[index]; //"if"
      const std::string& value = vals[index];
      BUILD_ON_PLATFORM

      std::string sif = if_count();

      if(if_s!="") if(!out_begin_if(aOut,if_s,sif,true)) return false;
 
      bool has_test = out_begin_platform_csh(aOut,platform);

      bool first = true;
      if(!out_append_csh(aOut,name,value,first)) return false;

      if(has_test) out_end_platform_csh(aOut);

      if(if_s!="") out_end_if(aOut,sif,true);
    }
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_tool(
 const std::vector<std::string>& aFiles
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> done;
  bool done_something = false;
  unsigned int number = aFiles.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& file = aFiles[index];
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(file,p,n,s);
    std::string tool;
    if(!parse_tool(s,tool)) return false;
    if(!Lib::smanip::belong(done,tool)) {
      out_begin_tool(tool,aOut);
      done.push_back(tool);
      done_something = true;
    }
  }
  if(done_something) aOut << Lib::endl;
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_begin_tool(
 const std::string& aTool
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopML) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_tool :" 
        << " fTopML is null." 
        << Lib::endl;
    return false;
  }
  Lib::ItemML* toolML = fTopML->findItem(aTool);
  if(!toolML) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_tool :"
        << " no <tool> XML item for " << Lib::sout(aTool) << "."
        << Lib::endl;
    return false;
  } 
  if(toolML->tagName()!="tool") {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_begin_tool :"
        << " An XML item for " << Lib::sout(aTool) 
        << " had been found but it is not a <tool> !"
        << Lib::endl;
    return false;
  } 

  if(!out_tool_item(*toolML,aTool,"command",true,aOut)) return false;
  if(!out_tool_item(*toolML,aTool,"optimize",false,aOut)) return false;
  if(!out_tool_item(*toolML,aTool,"debug",false,aOut)) return false;
  if(!out_tool_item(*toolML,aTool,"options",false,aOut)) return false;
  if(!out_tool_item(*toolML,aTool,"prefix",false,aOut)) return false;
  if(!out_tool_item(*toolML,aTool,"suffix",false,aOut)) return false;

  if(fProdWhat==obuild::Builder::build_sh) {

    aOut << "if [ ${obuild_debug} = yes ] ; then" << Lib::endl;
    aOut << "  obuild_" << aTool << "_debug_local="
         << "${obuild_" << aTool << "_debug}" << Lib::endl;
    aOut << "else" << Lib::endl;
    aOut << "  obuild_" << aTool << "_debug_local=" << Lib::endl;
    aOut << "fi" << Lib::endl;
    aOut << Lib::endl;

    aOut << "if [ ${obuild_optimize} = yes ] ; then" << Lib::endl;
    aOut << "  obuild_" << aTool << "_optimize_local=" 
         << "${obuild_" << aTool << "_optimize}" << Lib::endl;
    aOut << "else" << Lib::endl;
    aOut << "  obuild_" << aTool << "_optimize_local=" << Lib::endl;
    aOut << "fi" << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

   {std::string sif = fProduced_local_what+"_"+if_count();

    aOut << "IF NOT %obuild_debug% == yes GOTO else_if_db_" << sif 
         << Lib::endl;
    aOut << "  SET obuild_" << aTool << "_debug_local=" 
         << "%obuild_" << aTool << "_debug%" << Lib::endl;
    aOut << "  GOTO end_if_db_" << sif << Lib::endl;
    aOut << ":else_if_db_" << sif << Lib::endl;
    aOut << "  SET obuild_" << aTool << "_debug_local=" << Lib::endl;
    aOut << ":end_if_db_" << sif << Lib::endl;
    aOut << Lib::endl;}

   {std::string sif = fProduced_local_what+"_"+if_count();

    aOut << "IF NOT %obuild_optimize% == yes GOTO else_if_opt_" << sif 
         << Lib::endl;
    aOut << "  SET obuild_" << aTool << "_optimize_local=" 
         << "%obuild_" << aTool << "_optimize%" << Lib::endl;
    aOut << "  GOTO end_if_opt_" << sif << Lib::endl;
    aOut << ":else_if_opt_" << sif << Lib::endl;
    aOut << "  SET obuild_" << aTool << "_optimize_local=" << Lib::endl;
    aOut << ":end_if_opt_" << sif << Lib::endl;
    aOut << Lib::endl;}

  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_tool_item(
 Lib::ItemML& aToolML
,const std::string& aTool
,const std::string& aProperty
,bool aMendatory
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aToolML,aProperty,
        packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  bool done_something = false;

  bool first = true; //then init the variable too.

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& if_s = ifs[index]; //"if"
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    if(aMendatory&&(value=="")) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_tool_item :"
          << " for tool " << Lib::sout(aTool) 
          << " and platform " << Lib::sout(platform) 
          << ", no infos found for mendatory property " 
          << Lib::sout(aProperty)
          << "."
          << Lib::endl;
      return false;
    }

    if(value=="") continue; //do nothing.

    std::string sif = if_count();
    if(if_s!="") if(!out_begin_if(aOut,if_s,sif)) return false;
 
    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    std::string variable = "obuild_"+aTool+"_"+aProperty;

    if(!out_append(aOut,variable,value,first)) return false;
    //FIXME if(first) first = false; //NO appending. (Else "libHCL. so" !)

    //FIXME if(fProdWhat==obuild::Builder::build_sh) {
    //FIXME : do not produce infos related to Windows_NT.
    //FIXME } else if(fProdWhat==obuild::Builder::build_bat) {
    //FIXME : do not produce infos related to UNIXes.
    //FIXME }

    done_something = true;

    if(has_test) out_end_platform(aOut,spif);

    if(if_s!="") out_end_if(aOut,sif);
  }

  if(aMendatory) {
    if(fProdWhat==obuild::Builder::build_sh) {
      //FIXME : do not produce infos related to Windows_NT.

      aOut << "if [ \"${obuild_"+aTool+"_"+aProperty+"}\" = \"\" ] ; then" 
           << Lib::endl;
      aOut << "  echo \"Variable obuild_" << aTool << "_" << aProperty 
           << " not defined.\"" << Lib::endl; 
      aOut << "  exit" << Lib::endl;
      aOut << "fi" << Lib::endl;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      //FIXME : do not produce infos related to UNIXes.

      aOut << "IF \"%obuild_"+aTool+"_"+aProperty+"%\" == \"\"" 
           << "  ECHO Variable obuild_" << aTool << "_" << aProperty 
           << " not defined."
           << "  && GOTO obuild_return" 
           << Lib::endl;

    }
    done_something = true;
  }

  if(done_something) aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_compile_file(
 const std::string& aFile
,const std::string& aBinSubDir
,Lib::Out& aOut
,bool& aEmpty
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aEmpty = true;
  std::string p,n,s;
  Lib::smanip::pathNameSuffix(aFile,p,n,s);
  n = Lib::smanip::nosuffix(n);

  std::string obj_file = "."+sep()+aBinSubDir+sep()+n+s_obj();

  if(s=="c") {
    if(fProdWhat==obuild::Builder::build_sh) {

      // Some compiler (OSF1/cxx) produces extra things in the
      // current directory ; then we cd to the bin dir :
      out_cd_bin(aOut);

      out_if_component_stamp(aBinSubDir,aFile,aOut);

      aOut << " eval"
           << " ${obuild_c_command} -c"
           << " ${obuild_c_debug_local}"
           << " ${obuild_c_optimize_local}" 
           << " ${obuild_c_options}" 
           << " ${obuild_c_macros}" 
           << " ${obuild_c_includes}"
           << " -o " << obj_file << " "
           << esc_quote(quote(tounix(aFile)))
           << Lib::endl;
      out_status(aOut);

     {std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
      aOut << "touch " << stamp << Lib::endl;
      aOut << Lib::endl;}

      aOut << "fi" << Lib::endl; //component_stamp

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    } else if(fProdWhat==obuild::Builder::build_bat) {

      out_cd_bin(aOut);
      aOut << "%obuild_c_command% /c"
           << " %obuild_c_debug_local%" 
           << " %obuild_c_optimize_local%" 
           << " %obuild_c_options%" 
           << " %obuild_c_macros%" 
           << " %obuild_c_includes%"
           << " /Fo" << obj_file << " "
           << quote(towin(aFile))
           << " 1> cl_exe_error"
           << Lib::endl;

      aOut << "@IF ERRORLEVEL 1 @TYPE cl_exe_error && @GOTO obuild_return" 
           << Lib::endl;
      aOut << "@IF NOT ERRORLEVEL 1 @DEL cl_exe_error" 
           << Lib::endl;

      //out_status(aOut);
      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    }
  } else if( (s=="cxx")||
             (s=="cpp")||
             (s=="cc")||   //dvi2bitmap uses .cc for C++ suffix.
             (s=="c++") ) {
    if(fProdWhat==obuild::Builder::build_sh) {

      out_cd_bin(aOut);

      out_if_component_stamp(aBinSubDir,aFile,aOut);

      aOut << " eval"
           << " ${obuild_cxx_command} -c"
           << " ${obuild_cxx_debug_local}"
           << " ${obuild_cxx_optimize_local}"
           << " ${obuild_cxx_options}" 
           << " ${obuild_cxx_macros}"
           << " ${obuild_cxx_includes}"
           << " -o " << obj_file << " "
           << esc_quote(quote(tounix(aFile)))
           << Lib::endl;
      out_status(aOut);

     {std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
      aOut << "touch " << stamp << Lib::endl;
      aOut << Lib::endl;}

      aOut << "fi" << Lib::endl; //component_stamp

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    } else if(fProdWhat==obuild::Builder::build_bat) {

      out_cd_bin(aOut);
      aOut << "%obuild_cxx_command% /c"
           << " %obuild_cxx_debug_local%" 
           << " %obuild_cxx_optimize_local%" 
           << " %obuild_cxx_options%" 
           << " %obuild_cxx_macros%"
           << " %obuild_cxx_includes%"
           << " /Fo" << obj_file << " "
           << " /Tp" << quote(towin(aFile))
           << " 1> cl_exe_error"
           << Lib::endl;

      aOut << "@IF ERRORLEVEL 1 @TYPE cl_exe_error && @GOTO obuild_return" 
           << Lib::endl;
      aOut << "@IF NOT ERRORLEVEL 1 @DEL cl_exe_error" 
           << Lib::endl;

      //out_status(aOut);

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    }
  } else if(s=="f") {
    if(fProdWhat==obuild::Builder::build_sh) {

      out_cd_bin(aOut);

      out_if_component_stamp(aBinSubDir,aFile,aOut);

      aOut << " eval"
           << " ${obuild_f77_command} -c"
           << " ${obuild_f77_debug_local}"
           << " ${obuild_f77_optimize_local}"
           << " ${obuild_f77_options}" 
           << " ${obuild_f77_macros}"
           << " ${obuild_f77_includes}"
           << " -o " << obj_file << " "
           << esc_quote(quote(tounix(aFile)))
           << Lib::endl;
      out_status(aOut);

     {std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
      aOut << "touch " << stamp << Lib::endl;
      aOut << Lib::endl;}

      aOut << "fi" << Lib::endl; //component_stamp

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    } else if(fProdWhat==obuild::Builder::build_bat) {

      //FIXME : check the  below.
      // Options for DF98 fortran !

      out_cd_bin(aOut);
      aOut << "%obuild_f77_command% /compile_only"
           << " %obuild_f77_debug_local%" 
           << " %obuild_f77_optimize_local%" 
           << " %obuild_f77_options%" 
           << " %obuild_f77_macros%"
           << " %obuild_f77_includes%"
           << " /object:" << obj_file << " "
           << " " << quote(towin(aFile))
           << Lib::endl;
      out_status(aOut);
      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    }
  } else if(s=="m") {
    if(fProdWhat==obuild::Builder::build_sh) {

      out_cd_bin(aOut);

      out_if_component_stamp(aBinSubDir,aFile,aOut);

      aOut << " eval"
           << " ${obuild_objc_command} -c"
           << " ${obuild_objc_debug_local}"
           << " ${obuild_objc_optimize_local}"
           << " ${obuild_objc_options}" 
           << " ${obuild_objc_macros}"
           << " ${obuild_objc_includes}"
           << " -o " << obj_file << " "
           << esc_quote(quote(tounix(aFile)))
           << Lib::endl;
      out_status(aOut);

     {std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
      aOut << "touch " << stamp << Lib::endl;
      aOut << Lib::endl;}

      aOut << "fi" << Lib::endl; //component_stamp

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      //FIXME
    }
  } else if(s=="mm") {
    if(fProdWhat==obuild::Builder::build_sh) {

      out_cd_bin(aOut);

      out_if_component_stamp(aBinSubDir,aFile,aOut);

      aOut << " eval"
           << " ${obuild_objcxx_command} -c"
           << " ${obuild_objcxx_debug_local}"
           << " ${obuild_objcxx_optimize_local}"
           << " ${obuild_objcxx_options}" 
           << " ${obuild_objcxx_macros}"
           << " ${obuild_objcxx_includes}"
           << " -o " << obj_file << " "
           << esc_quote(quote(tounix(aFile)))
           << Lib::endl;
      out_status(aOut);

     {std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
      aOut << "touch " << stamp << Lib::endl;
      aOut << Lib::endl;}

      aOut << "fi" << Lib::endl; //component_stamp

      out_cd_obuild(aOut);
      aOut << Lib::endl;
      aEmpty = false;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      //FIXME
    }
  } else if( (s=="h") ||
             (s=="hh") ||
             (s=="ic") ||
             (s=="icc") ||
             (s=="yacc") || 
             (s=="in") ||   //configure .in files.
             (s=="cdf") ||  //KUIP cdf files.
             (s=="c~") ||
             (s=="cxx~") ){
  } else {
    Lib::Out out(fPrinter);
    out << "obuild::out_compile_file :" 
        << " warning : for file " << Lib::sout(aFile)
        << "; unknown suffix " 
        << Lib::sout(s) 
        << ". Ignored." 
        << Lib::endl;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_compilation_macros(
 Lib::ItemML& aItemML
,const std::string& aProperty
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Compilation macros :
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,aProperty,packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;

  std::string variable = "obuild_"+aProperty+"s";

  bool done_something = false;

  //bool first = true; //then init the variable too.
  bool first = false; //then append to variable.

  unsigned int number = names.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& name = names[index];
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    bool done_something_local = false;

    if(verbatims[index]=="yes") {
      if(value!="") {
        if(!out_append(aOut,variable,value,first)) return false;
        if(first) first = false;
        done_something_local = true;
      }
    } else {
      if(name!="") {
        if(value=="") {
          if(fProdWhat==obuild::Builder::build_sh) {
            if(!out_append(aOut,variable,"-D"+name,first)) return false;
          } else if(fProdWhat==obuild::Builder::build_bat) {
            if(!out_append(aOut,variable,"/D"+name,first)) return false;
          }
          if(first) first = false;
          done_something_local = true;
        } else {
          if(fProdWhat==obuild::Builder::build_sh) {
            if(!out_append(aOut,variable,"-D"+name+"="+value,first)) 
              return false;
          } else if(fProdWhat==obuild::Builder::build_bat) {
            if(!out_append(aOut,variable,"/D"+name+"="+value,first)) 
              return false;
          }
          if(first) first = false;
          done_something_local = true;
        }
      }
    }

    if(!done_something_local) out_empty(aOut);
    else done_something = true;

    if(has_test) out_end_platform(aOut,spif);
  }

  if(done_something) aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_options(
 Lib::ItemML& aItemML
,const std::string& aProperty
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Compilation macros :
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,aProperty,packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;

  std::string variable = "obuild_"+aProperty;

  bool done_something = false;

  //bool first = true; //then init the variable too.
  bool first = false; //then append to variable.

  unsigned int number = names.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    BUILD_ON_PLATFORM

    if(value=="") continue;

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    if(!out_append(aOut,variable,value,first)) return false;
    if(first) first = false;
    done_something = true;

    if(has_test) out_end_platform(aOut,spif);
  }

  if(done_something) aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_not_done_on(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_not_done_on :"
        << " name attribute not found for " 
        << Lib::sout(aItemML.tagName()) 
        << Lib::endl;
    return false;
  }

 {std::string s;
  if(aItemML.attributeValue("platform",s)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::out_not_done_on :"
        << " platform attribute not supported on tag " 
        << Lib::sout(aItemML.tagName()) 
        << Lib::endl;
    return false;
  }}

 {std::vector<std::string> vals;
  if(!aItemML.propertyValues("not_done_on",vals)) return false;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& platform = vals[index];
    BUILD_ON_PLATFORM

    std::string spif = if_count();
    bool has_test = out_begin_platform(aOut,platform,spif);

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "  if [ ${obuild_verbose} = yes ] ; then" << Lib::endl;
      aOut << "    echo '" << name << " not done on this platform'." 
           << Lib::endl;
      aOut << "  fi" << Lib::endl;
      aOut << "  exit" << Lib::endl;

    } else if(fProdWhat==obuild::Builder::build_bat) {

      aOut << "  IF %obuild_verbose% == yes ECHO" 
           << " " << name << " not done on this platform." << Lib::endl;
      aOut << "  GOTO obuild_return" << Lib::endl;

    }

    if(has_test) out_end_platform(aOut,spif);

  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_commands(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"command",
        packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    std::string value = vals[index];
    if(value=="") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::out_commands :"
          << " <command> without a value." 
          << Lib::endl;
      return false;
    }
 
    if(fProdWhat==obuild::Builder::build_bat) {
      value = towin(value);
    }

    aOut << value << Lib::endl;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_check_foreigns(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"check_foreign",
        packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    if(value=="") continue;
    if(fProdWhat==obuild::Builder::build_sh) {
      std::string s = "./sh/check_foreigns"+f_sh;
      aOut << "if [ -e " << s << " ] ; then" << Lib::endl;
      aOut << "  " << s << " " << value << Lib::endl;
      out_status(aOut,2);
      aOut << "fi" << Lib::endl;
      aOut << Lib::endl;
    } else if(fProdWhat==obuild::Builder::build_bat) {
      //FIXME : out << "CALL DOS/check_foreigns.bat" << Lib::endl;
    } 
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_check_env(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"check_env",
        packs,verss,platforms,verbatims,names,ifs,vals)) return false;

  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& platform = platforms[index]; //"platform"
    const std::string& name = names[index];
    const std::string& value = vals[index];

    if(fProdWhat==obuild::Builder::build_sh) {

      aOut << "  if [  \"${" << name << "}\" = \"\" ] ; then" << Lib::endl;
      if(value!="") {
        aOut << "    echo " << quote(value) << Lib::endl;
      } else {
        aOut << "    echo 'Environment variable " 
             << Lib::sout(value) << " not defined.'" << Lib::endl;
      }
      aOut << "    exit 1" << Lib::endl; //FIXME : have an atb : warn,fatal.
      aOut << "  fi" << Lib::endl;
      aOut << "fi" << Lib::endl;

    } else if(fProdWhat==obuild::Builder::build_bat) {
      //FIXME : NOT TESTED

      std::string sif = fPackage+"_"+if_count();

      aOut << "IF DEFINED " << name << " GOTO end_if_env_" << sif 
           << Lib::endl;
      if(value!="") {
        aOut << "    ECHO " << value << Lib::endl;
      } else {
        aOut << "    ECHO Environment variable " 
             << Lib::sout(value) << " not defined." << Lib::endl;
        aOut << "    GOTO obuild_return" <<Lib::endl;
      }
      aOut << ":end_if_env_" << sif << Lib::endl;
      aOut << Lib::endl;

    }
  }
  return true; 
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_if_done(
 Lib::ItemML& aItemML
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
// Look if binary directory of package exists :
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string build_if_done; //Used in UseAIDA.
  aItemML.propertyValue("build_if_done",build_if_done);
  if(build_if_done=="") return true;

  std::string dbin = dir_bin(build_if_done);

  if(fProdWhat==obuild::Builder::build_sh) {

    aOut << "if [ ! -d " << quote(dbin) << " ] ; then exit;fi" << Lib::endl;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    aOut << "IF NOT EXIST " << quote(dbin) 
         << "  GOTO obuild_return" << Lib::endl;

  }
  aOut << Lib::endl;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_cd_obuild(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aOut << "cd " << quote(dir_obuild(fPackage)) << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_cd_bin(
 Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aOut << "cd " << quote(dir_bin(fPackage)) << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_if_component_stamp(
 const std::string& aBinSubDir
,const std::string& aFile
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string stamp = "."+sep()+aBinSubDir+sep()+"component_stamp";
  aOut << "if [ ${obuild_check_time} = no -o " 
  //   << stamp << " -ot " << quote(tounix(aFile)) //does not work on OSF1.
  //obuild_file_is_older defined in templates/build_xxx_header.sh.
       << "`obuild_file_is_older " << stamp << " " << quote(tounix(aFile)) 
       << "` = yes ] ; then" 
       << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
void obuild::Builder::out_if_element_stamp(
 const std::string& aBinSubDir
,const std::string& aFile
,Lib::Out& aOut
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string stamp = "."+sep()+aBinSubDir+sep()+"element_stamp";
  aOut << "if [ ${obuild_check_time} = no -o " 
       << "`obuild_file_is_older " 
       << aFile
       << " " 
       << stamp
       << "` = yes ] ; then" 
       << Lib::endl;
}
//////////////////////////////////////////////////////////////////////////////
/// Default obuild file //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::out_default_obuild(
 Slash::Core::IWriter& aPrinter
,const std::string& aPackage
,const std::string& aVersion
,const std::string& aFile
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s;
  if(!default_obuild(aPackage,aVersion,s)) {
    Lib::Out out(aPrinter);
    out << "obuild::out_default_obuild :" 
        << " problem to get default obuild."
        << Lib::endl;    
    return false;
  }

  Lib::FileWriter fp(aFile,aPrinter);
  if(!fp.enabled()) {
    Lib::Out out(aPrinter);
    out << "obuild::out_default_obuild :" 
        << " can't open file " << Lib::sout(aFile) 
        << Lib::endl;
    return false;
  }
  Lib::Out out(fp);

  out << s;

  fp.disable();

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::default_obuild(
 const std::string& aPackage
,const std::string& aVersion
,std::string& aXML
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string package = aPackage;

  bool has_dir_source;
  Lib::dirmanip::isDirectory("../source",has_dir_source);
  bool has_dir_src;
  Lib::dirmanip::isDirectory("../src",has_dir_src);
  bool has_dir_include;
  Lib::dirmanip::isDirectory("../include",has_dir_include);
  bool has_dir_package;
  Lib::dirmanip::isDirectory("../"+package,has_dir_package);

  std::string sxml;
  sxml += "<obuild>\n";
  sxml += "\n";
  sxml += "  <package name=\"" + aPackage+ "\" version=\""+aVersion+"\">\n";
  sxml += "\n";

  std::string sdir = "${"+path_variable(package)+"}";
  std::string sincs;
 {// cc :
  sincs +="      <c_include>"+sdir+"</c_include>\n";
  if(has_dir_include) 
    sincs +="      <c_include>"+sdir+"/include</c_include>\n";
  if(has_dir_package) {
    sincs += "      <c_include>"+sdir+"/"+package+"</c_include>\n";
  }

  // cxx :
  sincs +="      <cxx_include>"+sdir+"</cxx_include>\n";
  if(has_dir_include) 
    sincs +="      <cxx_include>"+sdir+"/include</cxx_include>\n";
  if(has_dir_package) {
    sincs += "      <cxx_include>"+sdir+"/"+package+"</cxx_include>\n";
  }}

 {std::string s;
  Lib::smanip::printf(s,1024,"    <library name=\"%s\">\n",package.c_str());
  sxml += s;
  if(has_dir_source) sxml +="      <directory>../source</directory>\n";
  if(has_dir_src) sxml +="      <directory>../src</directory>\n";
  sxml += sincs;
  sxml += "    </library>\n";
  sxml += "\n";}

  // Get files in applications :

  bool have_exe = false;
  std::vector<std::string> files;
  if(Lib::dirmanip::entries("../applications",files,false)) {
    unsigned int number = files.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& file = files[index];

      std::string p,n,s;
      Lib::smanip::pathNameSuffix(file,p,n,s);
      n = Lib::smanip::nosuffix(n);
      if(s=="cxx") {
      } else if(s=="c") {
      } else {
        continue;
      }
  
     {std::string s;
      Lib::smanip::printf(s,1024,"    <application name=\"%s\">\n",
        n.c_str());
      sxml += s;
      Lib::smanip::printf(s,1024,"      <file>../applications/%s</file>\n",
        file.c_str());
      sxml += s;
      sxml += sincs;
      Lib::smanip::printf(s,1024,"      <lib>%s</lib>\n",package.c_str());
      sxml += s;
      sxml += "    </application>\n";
      sxml += "\n";}

      have_exe = true;
    }
  } 

  if(Lib::dirmanip::entries("../examples",files,false)) {
    unsigned int number = files.size();
    for(unsigned int index=0;index<number;index++) {
      const std::string& file = files[index];

      std::string p,n,s;
      Lib::smanip::pathNameSuffix(file,p,n,s);
      n = Lib::smanip::nosuffix(n);
      if(s=="cxx") {
      } else if(s=="c") {
      } else {
        continue;
      }
  
     {std::string s;
      Lib::smanip::printf(s,1024,"    <application name=\"%s\">\n",
        n.c_str());
      sxml += s;
      Lib::smanip::printf(s,1024,"      <file>../examples/%s</file>\n",
        file.c_str());
      sxml += s;
      sxml += sincs;
      Lib::smanip::printf(s,1024,"      <lib>%s</lib>\n",package.c_str());
      sxml += s;
      sxml += "    </application>\n";
      sxml += "\n";}

      have_exe = true;
    }
  }

  if(have_exe) {
    sxml += "    <setup>\n";
    sxml += "      <run_path/>\n";
    sxml += "      <load_library_path/>\n";
    sxml += "    </setup>\n";
    sxml += "\n";
  }

  sxml += "  </package>\n";
  sxml += "\n";
  sxml += "</obuild>\n";

  aXML = sxml;

  return true;
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
/// Code that does not produce code. /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
std::vector<std::string> obuild::Builder::tags(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::vector<std::string> tags;
  tags.push_back("obuild");
  tags.push_back("package");
  tags.push_back("version");
  tags.push_back("use");
  tags.push_back("library");
  tags.push_back("dll");
  tags.push_back("application");
  tags.push_back("template");
  tags.push_back("setup");
  tags.push_back("foreign");
  tags.push_back("reference");
  tags.push_back("tool");
  tags.push_back("dummy");
  tags.push_back("darwin_app");

  tags.push_back("project");
  tags.push_back("release");

  tags.push_back("copy_dir");
  tags.push_back("copy_file");
  tags.push_back("copy_files");
  tags.push_back("remove_files");
  tags.push_back("zip");
  tags.push_back("save");
  tags.push_back("subs_var");
  return tags;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::cmt(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "#";
  } else {
    return "REM #";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::sep(
) 
//////////////////////////////////////////////////////////////////////////////
// WARNING : use this method to build paths writen in the
//  in the .sh, .csh or .bat files.
// In particular do not use f_sep for that.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "/";
  } else {
    return "\\";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::path_sep(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return ":";
  } else {
    return ";";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::s_obj(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return ".o";
  } else {
    return ".obj";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::dir_bin_variable(
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "obuild_dir_bin";
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::path_variable(
 const std::string& aPack
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return "obuild_"+aPack+"_path";
  //return "obuild_path_"+aPack;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::dir_pack(
 const std::string& aPack
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "${"+path_variable(aPack)+"}";
  } else {
    return "%"+path_variable(aPack)+"%";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::var_val(
 const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "${"+aName+"}";
  } else {
    return "%"+aName+"%";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::dir_bin(
 const std::string& aPack
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "${"+path_variable(aPack)+"}/${"+dir_bin_variable()+"}";
  } else {
    return "%"+path_variable(aPack)+"%\\%"+dir_bin_variable()+"%";
  }
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::dir_obuild(
 const std::string& aPack
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {
    return "${"+path_variable(aPack)+"}/obuild";
  } else {
    return "%"+path_variable(aPack)+"%\\obuild";
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::handle_use(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Put package and version attributes on properties.
  std::string package;

  if(!aItemML.attributeValue("name",package)) return false;

  aItemML.addAttribute("package",package); //Needed to produce setups.

  std::string version;
  aItemML.attributeValue("version",version);

  const std::list<Lib::Element*> props = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;
  for(it=props.begin();it!=props.end();++it) {
    if((*it)->isAttribute("package")) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_use :"
          << " a property as already the package attribute."
          << Lib::endl;
      return false;
    } else {
      (*it)->addAttribute("package",package);
    }
    if((*it)->isAttribute("version")) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_use :"
          << " a property as already the version attribute."
          << Lib::endl;
      return false;
    } else {
      if(version!="") {
        (*it)->addAttribute("version",version);
      }
    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::handle_version(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string sfile;
  if(!aItemML.propertyValue("file",sfile)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_version :"
        << " no <file> property found."
        << Lib::endl;
    return false;
  }

  std::string file = ".."+f_sep+sfile;
  std::vector<std::string> lines;
  if(!Lib::fmanip::read(file,lines)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_version :"
        << " can't read " << Lib::sout(file)
        << Lib::endl;
    return false;
  }

  std::string var;
  if(!aItemML.propertyValue("cpp_macro",var)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_version :"
        << " no <cpp_macro> property found."
        << Lib::endl;
    return false;
  }

  bool found = false;
  unsigned int linen = lines.size();
  for(unsigned int linei=0;linei<linen;linei++) {
    const std::string& line = lines[linei];
    std::vector<std::string> words;
    Lib::smanip::words(line," ",words);
    unsigned int wordn = words.size();
    if(wordn!=3) continue;
    if(words[1]!=var) continue;
    std::string s = quote(fVersion);
    if(words[2]!=s) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_version :"
          << " version mismatch in file " << Lib::sout(file)
          << " for variable " << Lib::sout(var)
          << "."
          << Lib::endl;
      out << " path version is " << Lib::sout(fVersion)
          << " whilst " << Lib::sout(var)
          << " is " << Lib::sout(words[2])
          << "."
          << Lib::endl;
      return false;
    }
    found = true;
    break;
  }
  if(!found) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_version :"
        << " variable " << Lib::sout(var)
        << " not found in file " << Lib::sout(file)
        << "."
        << Lib::endl;
    return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::getFiles(
 Lib::ItemML& aItemML
,std::vector<std::string>& aFiles
,std::vector<std::string>& aPlatforms
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aFiles.clear();
  aPlatforms.clear();

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"file",packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& file = vals[index];
    const std::string& platform = platforms[index]; //"platform"
    if(handled(file)) {
      aFiles.push_back(file);
      aPlatforms.push_back(platform);
    }
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"files",packs,verss,platforms,
                                verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    const std::string& platform = platforms[index]; //"platform"

    std::string vp,vn,vs;
    Lib::smanip::pathNameSuffix(value,vp,vn,vs);
    std::string::size_type star = vn.find('*');
    if(star!=std::string::npos) { //Should be wildcard on the file name.
      std::vector<std::string> files2;
      Lib::smanip::getenv(vp); //For G4Examples_Core_T template.
      if(Lib::dirmanip::entries(vp,files2,false)) {
        Lib::smanip::cmpsort(files2);
        unsigned int number = files2.size();
        for(unsigned int index=0;index<number;index++) {
          const std::string& file = files2[index];
          std::string p,n,s;
          Lib::smanip::pathNameSuffix(file,p,n,s);
          if(Lib::smanip::match(n,vn)) {
            std::string f = vp+"/"+n;
            if(handled(f)) {
              aFiles.push_back(f);
              aPlatforms.push_back(platform);
            }
          }
        } 
      } else {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::getFiles :" 
            << " WARNING : can't get files for " << Lib::sout(value) 
            << Lib::endl;
        //return false;
      }
    } else {
      if(handled(value)) {
        aFiles.push_back(value);
        aPlatforms.push_back(platform);
      }
    }
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,"directory",packs,verss,platforms,
                                      verbatims,names,ifs,vals)) return false;
  unsigned int number = vals.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& directory = vals[index];
    const std::string& platform = platforms[index]; //"platform"

    Lib::Directory tree;
    tree.setPath(directory);
    if(!Lib::dirmanip::content(tree)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::getFiles :" 
          << "Can't get files of " << Lib::sout(directory) 
          << Lib::endl;
      return false;
    }
    std::vector<std::string> files;
    obuild::Visitor visitor(*this,files,aPlatforms,platform);
    if(!tree.visit(visitor)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::getFiles :" 
          << "Can't visit " << Lib::sout(directory) 
          << Lib::endl;
      return false;
    }
    Lib::smanip::cmpsort(files);
    Lib::append<std::string>(aFiles,files);  
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::getProperties(
 Lib::ItemML& aItemML
,const std::string& aProperty
,std::vector<std::string>& aPacks
,std::vector<std::string>& aVerss
,std::vector<std::string>& aPlatforms
,std::vector<std::string>& aVerbatims
,std::vector<std::string>& aNames
,std::vector<std::string>& aIfs
,std::vector<std::string>& aValues
,bool aCheckFromUse
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopML) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::getProperties :" 
        << " fTopML is null." 
        << Lib::endl;
    return false;
  }
  aPacks.clear();
  aVerss.clear();
  aPlatforms.clear();
  aVerbatims.clear();
  aNames.clear();
  aIfs.clear();
  aValues.clear();
  const std::list<Lib::Element*> props = aItemML.properties();
  std::list<Lib::Element*>::const_iterator it;
  for(it=props.begin();it!=props.end();++it) {
    if((*it)->name()!=aProperty) continue;

    std::string main_platform;
    bool has_platform = (*it)->attributeValue("platform",main_platform);

    std::string from;
    Lib::ItemML* fromItemML = 0;
    if((*it)->attributeValue("from",from)) {

      // from of the form : "itemML_name" or "itemML_tag itemML_name"
     {std::vector<std::string> words;
      Lib::smanip::words(from," ",words);
      unsigned int wordn = words.size();
      if(wordn==1) {
        fromItemML = fTopML->findItem(words[0]);
      } else if(wordn==2) {
        fromItemML = fTopML->findItemWithTag(words[0],words[1]);
      } else {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::getProperties :" 
            << " Bad syntax in the \"from\" attribute " << Lib::sout(from) 
            << ". One or two strings expected."
            << Lib::endl;
        return false;
      }}

      if(!fromItemML) {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::getProperties :" 
            << " Can't find XML item " << Lib::sout(from) 
            << Lib::endl;
        return false;
      }

      if(aCheckFromUse) {
        if(fromItemML->tagName()!="use") {
          Lib::Out out(fPrinter);
          out << "obuild::Builder::getProperties :" 
              << " XML item " << Lib::sout(from) << " is not a use one."
              << Lib::endl;
          return false;
        }
      }

    } else {
      // From the current itemML.
      fromItemML = &aItemML;
    }

    std::string what;
    if((*it)->attributeValue("copy",what)) { //copy="" from=""
      if( (what==aProperty) && (fromItemML==&aItemML) ) {
        // Can't copy from itself.
        Lib::Out out(fPrinter);
        out << "obuild::Builder::getProperties :" 
            << "Can't copy from itself."
            << Lib::endl;
        return false;
      }

      std::vector<std::string> packs;
      std::vector<std::string> verss;
      std::vector<std::string> platforms;
      std::vector<std::string> verbatims;
      std::vector<std::string> names;
      std::vector<std::string> ifs;
      std::vector<std::string> vals;
      //printf("debug : copy \"%s\" from \"%s\" : begin \n",
        //from.c_str(),what.c_str());
      if(!getProperties
        (*fromItemML,what,packs,verss,platforms,
                          verbatims,names,ifs,vals)) return false;
      //printf("debug : copy \"%s\" from \"%s\" : %d : end\n",
        //from.c_str(),what.c_str(),vals.size());
      if(!vals.size()) {
        if(fromItemML==&aItemML) {
          Lib::Out out(fPrinter);
          out << "obuild::Builder::getProperties :" 
              << " Can't copy."
              << " Property " << Lib::sout(what) << " not found" 
              << " in current XML item."
              << Lib::endl;
        } else {
          Lib::Out out(fPrinter);
          out << "obuild::Builder::getProperties :" 
              << " Can't copy."
              << " Property " << Lib::sout(what) << " not found" 
              << " in XML item " << Lib::sout(from)
              << Lib::endl;
        }
        return false;
      }

     {unsigned int number = packs.size();
      for(unsigned int index=0;index<number;index++) {

        const std::string& s = platforms[index];

        //printf("debug : name %s : val %s\n",
          //names[index].c_str(),vals[index].c_str());
        //printf("debug : xxx : main : %s : plat : %s\n",
          //main_platform.c_str(),s.c_str());

        if(s!="") {
          if(has_platform) {
            if(s==main_platform) {
              aPlatforms.push_back(s);
            } else {
              // Platform conflict. Ignore property :
              //printf("debug : platform conflict\n");
              //FIXME : case main_platform is "Darwin"
              //FIXME : and s is "UNIX"...
              continue;
            }
          } else {
            aPlatforms.push_back(s);
          }
        } else {
          aPlatforms.push_back(main_platform);
        }

        aPacks.push_back(packs[index]);  
        aVerss.push_back(verss[index]);  
        aVerbatims.push_back(verbatims[index]);  
        aNames.push_back(names[index]);  
        aIfs.push_back(ifs[index]);  
        aValues.push_back(vals[index]);  

      }}


    } else { //Not a copy.

      if(fromItemML==&aItemML) {

        // <lib package="Lib" version="v8r*">LibXML</lib>

        std::string package;
        if((*it)->attributeValue("package",package)) {
          std::string version;
          (*it)->attributeValue("version",version);
          aPacks.push_back(package);        
          aVerss.push_back(version);        
        } else {
          aPacks.push_back("this");        
          aVerss.push_back("");        
        }

      } else {

        // Example : <path from="AIDA"/>

        std::string package;
        if(fromItemML->attributeValue("package",package)) {
          std::string version;
          fromItemML->attributeValue("version",version);
          aPacks.push_back(package);        
          aVerss.push_back(version);        
        } else {
          aPacks.push_back("this");        
          aVerss.push_back("");        
        }

      }

     {std::string s;
      if((*it)->attributeValue("platform",s)) {
        if(has_platform) {
          if(s==main_platform) {
            aPlatforms.push_back(s);
          } else {
            //FIXME : case main_platform is "Darwin"
            //FIXME : and s is "UNIX"...
            Lib::Out out(fPrinter);
            out << "obuild::Builder::getProperties :" 
                << " platform conflict for property " 
                << Lib::sout(aProperty) << "."
                << Lib::endl;
            return false;
          }
        } else {
          aPlatforms.push_back(s);
        }
      } else {
        aPlatforms.push_back(main_platform);
      }}

     {std::string s;
      if(!(*it)->attributeValue("verbatim",s)) s = "no";
      bool b;
      if(!Lib::smanip::tobool(s,b)) {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::getProperties :" 
            << " verbatim attribute value " << Lib::sout(s) 
            << " is not a boolean."
            << Lib::endl;
        return false;
      }
      aVerbatims.push_back(s);}

     {std::string s;
      (*it)->attributeValue("name",s);
      aNames.push_back(s);}

     {std::string s;
      (*it)->attributeValue("if",s);
      aIfs.push_back(s);}

      const std::string& value = (*it)->value();
      aValues.push_back(value);        

      //printf("debug : for \"%s\" add value \"%s\"\n",
        //aProperty.c_str(),value.c_str());

    }
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::parse_tool(
 const std::string& aString
,std::string& aTool
) const
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aString=="c") {
    aTool = "c";
  } else if( (aString=="cxx") ||
             (aString=="cpp") ||
             (aString=="cc")  ||
             (aString=="c++") ) {
    // dvi2bitmap uses .cc for C++ file suffix !
    // SGI Inventor has .c++ for C++ file suffix !
    aTool = "cxx";
  } else if(aString=="f") {
    aTool = "f77";

  } else if( (aString=="objc") ||
             (aString=="m") ) {
    aTool = "objc";

  } else if( (aString=="objcxx") ||
             (aString=="mm") ) {
    aTool = "objcxx";

  } else if(aString=="C") {
    aTool = "c";
  } else if(aString=="C++") {
    aTool = "cxx";
  } else if(aString=="FORTRAN") {
    aTool = "f77";

  } else if(aString=="java") {
    aTool = "java";

  } else {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::parse_tool :" 
        << " unhandled tool " << Lib::sout(aString) 
        << Lib::endl;
    aTool = "";
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::handled(
 const std::string& aFile
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string p,n,s;
  Lib::smanip::pathNameSuffix(aFile,p,n,s);
  n = Lib::smanip::nosuffix(n);
  if(s=="c") {
    return true;
  } else if( (s=="cxx")||(s=="cpp")||(s=="cc") ) {
    return true;
  } else if(s=="f") {
    return true;
  } else if(s=="m") {
    return true;
  } else if(s=="mm") {
    return true;
  } else if(s=="java") {
    return true;
  } else if( (s=="h") ||
             (s=="hh") ||
             (s=="ic") ||
             (s=="icc") ||
             (s=="yacc") || 
             (s=="in") ||   //configure .in files.
             (s=="cdf") ||  //KUIP cdf files.
             (s=="c~") ||
             (s=="cxx~") ){
    return false;
  } else {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handled :" 
        << " unhandled extension " << Lib::sout(s) 
        << Lib::endl;
    return false;
  }
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::done_on(
 Lib::ItemML& aItemML
,const std::string& aWhat
,bool& aDo
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDo = true;

  if( (aWhat!="UNIX") && (aWhat!="Windows_NT") ){
    Lib::Out out(fPrinter);
    out << "obuild::Builder::do_on :"
        << " unknown keyword " << Lib::sout(aWhat)
        << Lib::endl;
    return false;
  }

 {std::string s;
  if(aItemML.attributeValue("platform",s)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::do_on :"
        << " platform attribute not supported on tag " 
        << Lib::sout(aItemML.tagName()) 
        << Lib::endl;
    return false;
  }}

 {std::vector<std::string> vals;
  if(!aItemML.propertyValues("build_on",vals)) return false;
  unsigned int number = vals.size();
  if(number>=2) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::do_on :"
        << " only one <build_on> supported."
        << Lib::endl;
    return false;
  }
  for(unsigned int index=0;index<number;index++) {
    const std::string& value = vals[index];
    std::vector<std::string> words = Lib::smanip::words(value," ");
    unsigned int wordn = words.size();
    if(wordn==1) {
      std::string platform = words[0];

      if(platform==aWhat) {
        aDo = true;
      } else if(platform!=aWhat) {
        aDo = false;
      } else {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::do_on_UNIX :"
            << " unknown keyword " << Lib::sout(platform)
            << " on <build_on> XML item."
            << Lib::endl;
        return false;
      }
    } else if(wordn==2) {
      if(words[0]!="not") {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::do_on_UNIX :"
            << " unknown keyword " << Lib::sout(words[0])
            << " on <build_on> XML item."
            << Lib::endl;
        return false;
      }

      std::string platform = words[1];

      if(platform==aWhat) {
        aDo = false;
      } else if(platform!=aWhat) {
        aDo = true;
      } else {
        Lib::Out out(fPrinter);
        out << "obuild::Builder::do_on_UNIX :"
            << " unknown keyword " << Lib::sout(platform)
            << " on <build_on> XML item."
            << Lib::endl;
        return false;
      }
    }
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::handle_template(
 Lib::ItemML& aItemML
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!fTopML) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_template :" 
        << " fTopML is null." 
        << Lib::endl;
    return false;
  }
  std::string name;
  if(!aItemML.attributeValue("","name",name)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_template :"
        << " name attribute not found." 
        << Lib::endl;
    return false;
  }
  std::string stemplate;
  if(aItemML.attributeValue("","template",stemplate)&&(stemplate!="")) {
    Lib::ItemML* templateItemML = fTopML->findItem(stemplate);
    if(!templateItemML) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_template :"
          << " template XML item " << Lib::sout(stemplate) << " not found." 
          << Lib::endl;
      return false;
    }
    if(templateItemML->tagName()!="template") {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_template :"
          << " XML item named " << Lib::sout(stemplate) 
          << " not a template. It is a " 
          << Lib::sout(templateItemML->tagName()) 
          << Lib::endl;
      return false;
    }

    // Before copying we get the <replace> items :
    std::vector<std::string> packs;
    std::vector<std::string> verss;
    std::vector<std::string> platforms;
    std::vector<std::string> verbatims;
    std::vector<std::string> names;
    std::vector<std::string> ifs;
    std::vector<std::string> vals;
    if(!getProperties(aItemML,"replace",
                    packs,verss,platforms,verbatims,names,ifs,vals)) {
      return false;
    }

    //aItemML.copy(*templateItemML,Lib::ItemML::ATTRIBUTES);

    aItemML.removeProperties("replace");
    // Do not remove existing properties (group, etc....
    aItemML.copy(*templateItemML,Lib::ItemML::PROPERTIES,false);

    aItemML.copy(*templateItemML,Lib::ItemML::CHILDREN);

    aItemML.replace("@@name@@",name);

   {unsigned int number = names.size();
    for(unsigned int index=0;index<number;index++) {
      aItemML.replace(names[index],vals[index]);
    }}

    // Have to disconnect the template atb :
    aItemML.removeAttribute("template");

    return true;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::guess_linker_language(
 Lib::ItemML& aItemML
,const std::vector<std::string>& aFiles
,std::string& aLang
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
 // linker no more supported as attribute :
 {std::string value;
  if(aItemML.attributeValue("linker",value)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::guess_linker_language :"
        << " linker no more supported as attribute." 
        << Lib::endl;
    return false;
  }}

 {std::vector<std::string> packs;
  std::vector<std::string> verss;
  std::vector<std::string> platforms;
  std::vector<std::string> verbatims;
  std::vector<std::string> names;
  std::vector<std::string> ifs;
  std::vector<std::string> vals;
  if(!getProperties(aItemML,
                  "linker",packs,verss,platforms,
                        verbatims,names,ifs,vals,true)) {
    return false;
  }
  if(vals.size()>=2) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::guess_linker_language :"
        << " only one linker property expected." 
        << Lib::endl;
    return false;
  }
  if(vals.size()==1) {
    if(!parse_tool(vals[0],aLang)) return false;
    return true;
  }}

  // Else, guess from files suffixes :

  if(!aFiles.size()) return false;

  if(aFiles.size()==1) {
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(aFiles[0],p,n,s);
    if(!parse_tool(s,aLang)) return false;
    return true;
  }

  // Look if all files are in same language :
 {aLang = "";
  bool sameLang = true;
  unsigned int number = aFiles.size();
  for(unsigned int index=0;index<number;index++) {
    const std::string& file = aFiles[index];
    std::string p,n,s;
    Lib::smanip::pathNameSuffix(file,p,n,s);
    std::string tool;
    if(!parse_tool(s,tool)) return false;
    if(aLang=="") {
      aLang = tool;
    } else if(tool!=aLang) {
      if( (tool=="c") && (aLang=="cxx") ) {        
      } else if( (tool=="cxx") && (aLang=="c") ) {        
        aLang = "cxx"; // Switch to C++.
      } else {
        sameLang = false;
        break;
      }
    }
  }
  if(sameLang) return true;}

  Lib::Out out(fPrinter);
  out << "obuild::Builder::guess_linker_language :"
      << " can't guess tool to link with."
      << Lib::endl;

  return false;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::check_directory(
 const std::string& aName
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!Lib::dirmanip::exists(aName)) {
    if(!Lib::dirmanip::create(aName)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::check_directory :" 
          << " can't create " << Lib::sout(aName) << " directory." 
          << Lib::endl;
      return false;
    }
  } else {
    bool is;
    Lib::dirmanip::isDirectory(aName,is);
    if(!is) {
      // Something exists but it is not a directory !
      Lib::Out out(fPrinter);
      out << "obuild::Builder::check_directory :" 
          << " some " << Lib::sout(aName) 
          << " exists which is not a directory. Please remove it." 
          << Lib::endl;
      return false;
    }
  }
  return true;        
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::package_and_version(
 std::string& aPackage
,std::string& aVersion
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Assumed to be in a obuild directory put under a package hierarchy.
  std::string dir;
  if(!Lib::dirmanip::directory(dir)) {
    aPackage = "";
    aVersion = "";
    return false;
  }
  //printf("debug : dir \"%s\"\n",dir.c_str());
  std::vector<std::string> words;
  bool absolute;
  bool winPath;
  std::string drive;
  // pathWords assume that Last item is a file name, then
  // returned words is a list without the last item.
  if(!Lib::smanip::pathWords(dir,words,absolute,winPath,drive)) {
    aPackage = "";
    aVersion = "";
    return false;
  }
  unsigned int wordn = words.size();
  //{for(unsigned int index=0;index<wordn;index++) {
  //  printf("debug :   item \"%s\"\n",words[index].c_str());
  //}}
  if(wordn<=0) { // one item only (obuild ?) in dir.
    aPackage = "";
    aVersion = "";
    return false;
  }
  // At least two items in dir :  xxx/obuild
  // Look if xxx is a version :
  std::vector<unsigned int> verss;
  if(!Lib::smanip::version(words[wordn-1],verss)) {
    // xxx not a version. Assume to be under : pack/obuild 
    // without an in between version directory
    aPackage = words[wordn-1]; //FIXME : check package syntax ?
    aVersion = "";
    return true;
  } else if(wordn>=2) { // At least three items in dir.
    // Assume to be under : pack/version/obuild
    aPackage = words[wordn-2]; //FIXME : check package syntax ?
    aVersion = words[wordn-1];
    return true;
  } else { // Only two items in dir ! version/obuild !
    aPackage = "";
    aVersion = "";
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::rm(
 const std::string& aDirectory
,const std::string& aPattern
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(!Lib::dirmanip::exists(aDirectory)) return true;
  if(!Lib::dirmanip::removeInDirectory(aDirectory,aPattern)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::rm :"
        << " problem removing " << Lib::sout(aPattern) 
        << " in directory " << Lib::sout(aDirectory)
        << "."
        << Lib::endl;
    return false;
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::cleanup(
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(fProdWhat==obuild::Builder::build_sh) {

    // sh :
    if(!rm("sh","build")) return false;
    if(!rm("sh","build_*")) return false;
    if(!rm("sh","install")) return false;
    if(!rm("sh","broadcast_install")) return false;
    if(!rm("sh","check_foreigns")) return false;
    if(!rm("sh","check_app")) return false;
    if(!rm("sh","run")) return false;
    if(!rm("sh","build_Darwin_app")) return false;
    if(!rm("sh","build_release")) return false;
    if(!rm("sh","checkout_release")) return false;
    if(!rm("cpp","obuild_find.cpp")) return false;
    if(!rm("cpp","obuild_platform.cpp")) return false;
    if(!rm(".","setup.sh")) return false;
    if(!rm(".","setup.csh")) return false;

    // Deprecated material :
    if(!rm(".","build.sh")) return false;
    if(!rm("sh","build_*.sh")) return false;
    if(!rm("sh","install.sh")) return false;
    if(!rm("sh","broadcast_install.sh")) return false;
    if(!rm("sh","check_foreigns.sh")) return false;
    if(!rm("sh","release")) return false;
    if(!rm(".","run")) return false;

    if(!rm(".","ofind.cpp")) return false;
    if(!rm(".","oplatform.cpp")) return false;
    if(!rm(".","install.sh")) return false;
    if(!rm("sh","install_setups.sh")) return false;

    if(!Lib::dirmanip::removeIfEmpty("cpp")) return false;
    if(!Lib::dirmanip::removeIfEmpty("sh")) return false;

  } else if(fProdWhat==obuild::Builder::build_bat) {

    // DOS :
    if(!rm("DOS","build.bat")) return false;
    if(!rm("DOS","build_*.bat")) return false;
    if(!rm("DOS","run.bat")) return false;
    if(!rm("DOS","build_release.bat")) return false;
    if(!rm("DOS","checkout_release.bat")) return false;
    if(!rm("DOS","this.iss")) return false;
    if(!rm("cpp","obuild_windef.c")) return false;
    if(!rm("cpp","obuild_subs.cpp")) return false;

    // Deprecated material :
    if(!rm(".","build.bat")) return false;
    if(!rm(".","run.bat")) return false;

    if(!Lib::dirmanip::removeIfEmpty("cpp")) return false;
    if(!Lib::dirmanip::removeIfEmpty("DOS")) return false;
  }

  return true;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::towin(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string fname = aString;

  // string may contain ${xxx}
  Lib::smanip::replace(fname,"${","%");
  Lib::smanip::replace(fname,"}","%");
    
  Lib::smanip::replace(fname,"/","\\");
  return fname;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::tounix(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string fname = aString;

  // string may contain %xxx%
  Lib::smanip::replace(fname,"%","${");
  Lib::smanip::replace(fname,"%","}");
    
  Lib::smanip::replace(fname,"\\","/");
  return fname;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::esc_quote(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = aString;
  Lib::smanip::replace(s,"\"","\\\"");
  return s;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::no_quote(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string s = aString;
  Lib::smanip::replace(s,"\"","");
  return s;
}
//////////////////////////////////////////////////////////////////////////////
std::string obuild::Builder::quote(
 const std::string& aString
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return std::string("\"")+aString+"\"";
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::build_on_platform(
 const std::string& aPlatform
,bool& aYes
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aPlatform=="") {
    aYes = true;
    return true;
  }

  bool result;
  if(!evaluate_expression("Windows_NT","*",aPlatform,result)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::build_on_platform :"
        << " can't guess Windows_NT or not from platform string " 
        << Lib::sout(aPlatform) << "."
        << Lib::endl;    
    aYes = true;
    return false;
  }
  if(fProdWhat==obuild::Builder::build_sh) {
    aYes = result?false:true;    
  } else {
    aYes = result;    
  }
  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::handle_tools(
 Lib::ItemML& aItemML //top item
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  unsigned int number = 0;
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"tool",list);
  number = list.size();}
  if(!number) {
    // Load default tools :
    Lib::Out out(fPrinter);

    //out << "obuild::Builder::handle_tools :"
    //    << " load default tools..."
    //    << Lib::endl;

    int verboseLevel = 0;
    Lib::ItemMLFactory factory;
    Lib::BaseML ml(factory,verboseLevel);
    std::vector<std::string> tags;
    tags.push_back("obuild");
    tags.push_back("tool");
    ml.setTags(tags);

    std::string s = Lib::smanip::tostring(tools_obuild_linen,
                                          tools_obuild_lines);
    Lib::smanip::replace(s,"@@double_quote@@","\"");
    Lib::smanip::replace(s,"@@back_slash@@","\\");

    if(!ml.loadString(s,fPrinter)) {
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_tools :"
          << " problem when loading XML string " << Lib::sout(s) 
          << Lib::endl;    
      return false;
    }

    Lib::ItemML* top = ml.topItem();
    if(!top) {
      // Empty file : do nothing.
      Lib::Out out(fPrinter);
      out << "obuild::Builder::handle_tools :"
          << " XML string is empty."
          << Lib::endl;
      return false;
    } 
    if(!top->hasEmptyAttributeValue(fPrinter)) return false;

    // Reconnect to aItemML :
    top->setParent(&aItemML);
    ml.empty(); //Disconnect from ml.
    aItemML.addChild(top);
  }

  // Ask again :
 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(aItemML,"tool",list);
  if(!list.size()) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::handle_tools :"
        << " no tool XML item found."
        << Lib::endl;
    return false;
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::findProjectPackages(
 Slash::Core::IWriter& aPrinter
,const std::string& aFile
,std::vector<std::string>& aReps
,std::vector<std::string>& aPVs
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aReps.clear();
  aPVs.clear();

  if(aFile=="") {
    Lib::Out out(aPrinter);
    out << "obuild::Builder::findProjectPackages :"
        << " no obuild file to work with."
        << Lib::endl;
    return false;
  }

  int verboseLevel = 0;
  Lib::ItemMLFactory factory;
  Lib::BaseML ml(factory,verboseLevel);
  ml.setTags(tags());
  FileGetter fg(aPrinter);
  ml.setFileGetter(&fg);

  if(!ml.loadFile(aFile,false,aPrinter)) {
    Lib::Out out(aPrinter);
    out << "obuild::Builder::findProjectPackages :"
        << " problem when loading file " << Lib::sout(aFile) 
        << Lib::endl;    
    return false;
  }

  Lib::ItemML* top = ml.topItem();
  if(!top) {
    // Empty file : do nothing.
    Lib::Out out(aPrinter);
    out << "obuild::Builder::findProjectPackages :"
        << " file " << Lib::sout(aFile) << " is empty."
        << Lib::endl;
    return false;
  } 

  if(!top->hasEmptyAttributeValue(aPrinter)) return false;

  Lib::ItemML* projectML = 0;
 {const std::list<Lib::ItemML*>& list = top->children();
  std::list<Lib::ItemML*>::const_iterator it;
  for(it=list.begin();it!=list.end();++it) {
    const std::string& tagName = (*it)->tagName();
    if(tagName=="project") {   
      projectML = *it;
      break;
    }
  }
  if(!projectML) {
    Lib::Out out(aPrinter);
    out << "obuild::Builder::findProjectPackages :"
        << " file " << Lib::sout(aFile) << " has no <project>."
        << Lib::endl;
    return false;
  }}


 {std::vector<Lib::ItemML*> list;
  Lib::ItemML::collectByTag(*projectML,"package",list);
  std::vector<Lib::ItemML*>::iterator it;
  for(it=list.begin();it!=list.end();++it) {

    if((*it)->isProperty("not_in_main_build")) continue;

    std::string pack_name;
    (*it)->attributeValue("name",pack_name);
    std::string pack_version;
    (*it)->attributeValue("version",pack_version);

    std::string pack_rep;
    if(!(*it)->propertyValue("repository",pack_rep)) {
      Lib::Out out(aPrinter);
      out << "obuild::Builder::findProjectPackages :"
          << " WARNING : package " << Lib::sout(pack_name) 
          << " has no <repository>."
          << Lib::endl;      
    }

    aReps.push_back(pack_rep);
    aPVs.push_back(pack_name+"/"+pack_version);
    //printf("debug : %s : %s\n",pack_name.c_str(),pack_version.c_str());
  }}

  return true;
}
//////////////////////////////////////////////////////////////////////////////
/// query methods : //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::query(
 const std::string& aFile
,const std::string& aWhat
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if(aFile=="") {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::query :"
        << " no obuild file to work with."
        << Lib::endl;
    return false;
  }

  std::string package;
  std::string version;
  if(!package_and_version(package,version)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::query :"
        << " can't get package and version." 
        << Lib::endl;
    return false;
  }

  if((aWhat=="")||(aWhat=="file")) {
    Lib::Out out(fPrinter);
    out << " file :" 
        << " " << aFile
        << Lib::endl;
  }

  int verboseLevel = 0;
  Lib::ItemMLFactory factory;
  Lib::BaseML ml(factory,verboseLevel);
  ml.setTags(tags());
  FileGetter fg(fPrinter);
  ml.setFileGetter(&fg);

  if((aWhat=="")||(aWhat=="refs")||(aWhat=="references")) {
    fg.setQuery(true);
  }

  if(!ml.loadFile(aFile,false,fPrinter)) {
    Lib::Out out(fPrinter);
    out << "obuild::Builder::query :"
        << " problem when loading file " << Lib::sout(aFile) 
        << Lib::endl;    
    return false;
  }

  Lib::ItemML* top = ml.topItem();
  if(!top) {
    // Empty file : do nothing.
    Lib::Out out(fPrinter);
    out << "obuild::Builder::query :"
        << " file " << Lib::sout(aFile) << " is empty."
        << Lib::endl;
    return false;
  } 

  if(!top->hasEmptyAttributeValue(fPrinter)) return false;

  if(!handle_tools(*top)) return false;

  fTopML = top;

  bool status = queryByTags(*top,aWhat);

  fTopML = 0;

  return status;
}
//////////////////////////////////////////////////////////////////////////////
bool obuild::Builder::queryByTags(
 Lib::ItemML& aItemML
,const std::string& aWhat
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  if((aWhat=="")||(aWhat=="packs")||(aWhat=="packages")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"package",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);

      std::string name;
      itemML.attributeValue("name",name);
      std::string vers;
      itemML.attributeValue("version",vers);
  
      Lib::Out out(fPrinter);
      out << " package :" 
          << " " << name
          << " " << vers
          << Lib::endl;
    }
  }

  if((aWhat=="")||(aWhat=="foreigns")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"foreign",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);

      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " foreign :" 
          << " " << name
          << Lib::endl;
    }
  }

  if((aWhat=="")||(aWhat=="uses")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"use",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
  
      std::string name;
      itemML.attributeValue("name",name);
      std::string vers;
      itemML.attributeValue("version",vers);
  
      Lib::Out out(fPrinter);
      out << " use :" 
          << " " << name
          << " " << vers
          << Lib::endl;
    }
  }

  if((aWhat=="")||(aWhat=="libs")||(aWhat=="libraries")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"library",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      if(!handle_template(itemML)) return false;

      bool notInMain = itemML.isProperty("not_in_main_build");
  
      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " library :" 
          << " " << name
          << (notInMain?" (not in main build)":"")
          << Lib::endl;
    }
  }
  
  if((aWhat=="")||(aWhat=="dlls")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"dll",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      if(!handle_template(itemML)) return false;

      bool notInMain = itemML.isProperty("not_in_main_build");
  
      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " dll :" 
          << " " << name
          << (notInMain?" (not in main build)":"")
          << Lib::endl;
    }
  }
  
  if((aWhat=="")||(aWhat=="apps")||(aWhat=="applications")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"application",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      if(!handle_template(itemML)) return false;

      bool notInMain = itemML.isProperty("not_in_main_build");

      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " application :" 
          << " " << name
          << (notInMain?" (not in main build)":"")
          << Lib::endl;
    }
  }
  
  if((aWhat=="")||(aWhat=="darwin_apps")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"darwin_app",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);
      if(!handle_template(itemML)) return false;

      bool notInMain = itemML.isProperty("not_in_main_build");

      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " darwin_app :" 
          << " " << name
          << (notInMain?" (not in main build)":"")
          << Lib::endl;
    }
  }
  
  if((aWhat=="")||(aWhat=="tools")) {
    std::vector<Lib::ItemML*> items;
    Lib::ItemML::collectByTag(aItemML,"tool",items);
    unsigned int number = items.size();
    for(unsigned int index=0;index<number;index++) {
      Lib::ItemML& itemML = *(items[index]);

      std::string name;
      itemML.attributeValue("name",name);
  
      Lib::Out out(fPrinter);
      out << " tool :" 
          << " " << name
          << Lib::endl;
    }
  }

  return true;
}
